(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     98509,       1901]
NotebookOptionsPosition[     97967,       1878]
NotebookOutlinePosition[     98384,       1895]
CellTagsIndexPosition[     98341,       1892]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Construct Worksheet for Aerodynamics", "Section",
 CellChangeTimes->{{3.4903860186376*^9, 3.4903860388856*^9}, {
  3.494751686681371*^9, 3.4947517063433714`*^9}, {3.4947517563773713`*^9, 
  3.494751789530371*^9}, {3.4957933644365654`*^9, 3.495793369255565*^9}}],

Cell["\<\
Richard Fearn, December 13, 2010\
\>", "Text",
 CellChangeTimes->{{3.5012417852898035`*^9, 3.5012418108468037`*^9}}],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellChangeTimes->{{3.49829539478961*^9, 3.4982954004796095`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Interpretation", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Alpha]D", "=", "5."}], ",", 
      RowBox[{"filename", "=", "\"\<e205.dat\>\""}], ",", 
      RowBox[{"id", "=", "4412"}], ",", 
      RowBox[{"nFile", "=", "\"\<data\>\""}], ",", 
      RowBox[{"nEq", "=", "50"}], ",", 
      RowBox[{"layout", "=", "HalfCosineSpacing"}], ",", 
      RowBox[{"model", "=", "PMVortexLinearNeumann"}], ",", 
      RowBox[{"reset", "=", "False"}], ",", 
      RowBox[{"source", "=", "1"}], ",", 
      RowBox[{"viz", "=", "sp"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Panel", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Grid", "[", 
       RowBox[{
        RowBox[{"{", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Style", "[", 
             RowBox[{
             "\"\<Worksheet for Airfoil Aerodynamics\>\"", ",", 
              "\"\<Section\>\""}], "]"}], ",", "SpanFromLeft"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"\"\<\>\"", ",", "SpanFromLeft"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Style", "[", 
             RowBox[{"\"\<Input:\>\"", ",", "\"\<Subsection\>\""}], "]"}], 
            ",", "SpanFromLeft", ",", "\"\<angle of attack (deg)\>\"", ",", 
            "SpanFromLeft", ",", "SpanFromLeft", ",", 
            RowBox[{"InputField", "[", 
             RowBox[{
              RowBox[{"Dynamic", "[", "\[Alpha]D", "]"}], ",", "Number", ",", 
              RowBox[{"FieldSize", "\[Rule]", "3"}]}], "]"}]}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"\"\<\>\"", ",", "\"\<source of airfoil\>\"", ",", 
            RowBox[{"SetterBar", "[", 
             RowBox[{
              RowBox[{"Dynamic", "[", "source", "]"}], ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"1", "\[Rule]", "\"\<UIUC database\>\""}], ",", 
                RowBox[{"2", "\[Rule]", "\"\<file in Airfoil Data\>\""}], ",", 
                RowBox[{"3", "\[Rule]", "\"\<NACA 4-Digit\>\""}]}], "}"}]}], 
             "]"}], ",", "SpanFromLeft"}], "}"}], ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"\"\<\>\"", ",", "\"\<file name\>\"", ",", 
            RowBox[{"InputField", "[", 
             RowBox[{
              RowBox[{"Dynamic", "[", "filename", "]"}], ",", "String", ",", 
              RowBox[{"Enabled", "\[Rule]", 
               RowBox[{"Dynamic", "[", 
                RowBox[{"source", "\[NotEqual]", "3"}], "]"}]}], ",", 
              RowBox[{"FieldSize", "\[Rule]", "10"}]}], "]"}], ",", 
            "SpanFromLeft", ",", "\"\<\>\"", ",", "\"\<4-digit id\>\"", ",", 
            RowBox[{"InputField", "[", 
             RowBox[{
              RowBox[{"Dynamic", "[", 
               RowBox[{"id", ",", 
                RowBox[{
                 RowBox[{"If", "[", 
                  RowBox[{
                   RowBox[{"0", "\[LessEqual]", "#", "\[LessEqual]", "9999"}],
                    ",", 
                   RowBox[{"id", "=", "#"}], ",", 
                   RowBox[{"id", "=", 
                    RowBox[{"FromDigits", "[", 
                    RowBox[{"Take", "[", 
                    RowBox[{
                    RowBox[{"IntegerDigits", "[", "#", "]"}], ",", "4"}], 
                    "]"}], "]"}]}]}], "]"}], "&"}]}], "]"}], ",", 
              RowBox[{"ContinuousAction", "\[Rule]", "True"}], ",", 
              RowBox[{"Enabled", "\[Rule]", 
               RowBox[{"Dynamic", "[", 
                RowBox[{"source", "\[Equal]", "3"}], "]"}]}], ",", 
              RowBox[{"FieldSize", "\[Rule]", "3"}]}], "]"}]}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"\"\<\>\"", ",", "\"\<discretization:\>\"", ",", 
            RowBox[{"InputField", "[", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"Dynamic", "[", "nFile", "]"}], ",", 
              RowBox[{"Enabled", "\[Rule]", 
               RowBox[{"Dynamic", "[", 
                RowBox[{"source", "\[NotEqual]", "3"}], "]"}]}], ",", 
              RowBox[{"FieldSize", "\[Rule]", "3"}]}], "]"}], ",", "\"\<\>\"",
             ",", "\"\<\>\"", ",", "\"\<\>\"", ",", 
            RowBox[{"InputField", "[", 
             RowBox[{
              RowBox[{"Dynamic", "[", "nEq", "]"}], ",", 
              RowBox[{"Enabled", "\[Rule]", 
               RowBox[{"Dynamic", "[", 
                RowBox[{"source", "\[Equal]", "3"}], "]"}]}], ",", 
              RowBox[{"FieldSize", "\[Rule]", "3"}]}], "]"}]}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"\"\<\>\"", ",", "\"\<layout\>\"", ",", 
            RowBox[{"SetterBar", "[", 
             RowBox[{
              RowBox[{"Dynamic", "[", "layout", "]"}], ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{
                "ConstantSpacing", "\[Rule]", "\"\<ConstantSpacing\>\""}], 
                ",", 
                RowBox[{"CosineSpacing", "\[Rule]", "\"\<CosineSpacing\>\""}],
                 ",", 
                RowBox[{
                "HalfCosineSpacing", "\[Rule]", 
                 "\"\<HalfCosineSpacing\>\""}]}], "}"}], ",", 
              RowBox[{"Enabled", "\[Rule]", 
               RowBox[{"Dynamic", "[", 
                RowBox[{
                 RowBox[{"source", "\[Equal]", "3"}], " ", "||", " ", 
                 RowBox[{"nFile", "\[NotEqual]", "\"\<data\>\""}]}], 
                "]"}]}]}], "]"}], ",", "SpanFromLeft"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"\"\<\>\"", ",", "\"\<model: PM\>\"", ",", 
            RowBox[{"SetterBar", "[", 
             RowBox[{
              RowBox[{"Dynamic", "[", "model", "]"}], ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{
                "PMVortexLinearNeumann", "\[Rule]", 
                 "\"\<VortexLinearNeumann\>\""}], ",", 
                RowBox[{
                "PMDoubletQuadraticDirichlet", "\[Rule]", 
                 "\"\<DoubletQuadraticDirichlet\>\""}], ",", 
                RowBox[{
                "PMDoubletConstantDirichlet", "\[Rule]", 
                 "\"\<DoubletConstantDirichlet\>\""}]}], "}"}]}], "]"}], ",", 
            "SpanFromLeft"}], "}"}], ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Style", "[", 
             RowBox[{"\"\<Output:\>\"", ",", "\"\<Subsection\>\""}], "]"}], 
            ",", "SpanFromLeft"}], "}"}], ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"\"\<\>\"", ",", "\"\<flow visualization\>\"", ",", 
            RowBox[{"SetterBar", "[", 
             RowBox[{
              RowBox[{"Dynamic", "[", "viz", "]"}], ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"sp", "\[Rule]", "\"\<stream plot\>\""}], ",", 
                RowBox[{"slp", "\[Rule]", "\"\<streamline plot\>\""}], ",", 
                RowBox[{"sdp", "\[Rule]", 
                 RowBox[{"Row", "[", 
                  RowBox[{"{", 
                   RowBox[{"\"\<stream-\>\"", ",", 
                    FormBox[
                    SubscriptBox["C", "p"],
                    TraditionalForm], ",", "\"\<plot\>\""}], "}"}], "]"}]}]}],
                "}"}]}], "]"}], ",", "SpanFromLeft"}], "}"}], ",", 
          RowBox[{"(*", " ", 
           RowBox[{"4", " ", "columns"}], " ", "*)"}], "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"\"\<\>\"", ",", 
            RowBox[{"Row", "[", 
             RowBox[{"{", 
              RowBox[{
              "\"\<Aerodynamic properties are presented as graph of surface\>\
\"", " ", ",", Cell[TextData[Cell[BoxData[
                FormBox[
                 SubscriptBox["C", "p"], TraditionalForm]]]]], " ", ",", 
               "\"\<with numerical values for\>\"", ",", " ", Cell[TextData[
               Cell[BoxData[
                FormBox[
                 SubscriptBox["C", "\[ScriptL]"], TraditionalForm]]]]], " ", 
               ",", "\"\<and\>\"", ",", " ", Cell[TextData[Cell[BoxData[
                FormBox[
                 SubscriptBox["C", "m"], TraditionalForm]]]]], ",", 
               "\"\<.\>\""}], "}"}], "]"}], ",", "SpanFromLeft"}], "}"}], ",",
           "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"\"\<\>\"", ",", 
            RowBox[{"Button", "[", 
             RowBox[{"\"\<reset\>\"", ",", 
              RowBox[{
               RowBox[{"\[Alpha]D", "=", "5."}], ";", 
               RowBox[{"filename", "=", "\"\<e205.dat\>\""}], ";", 
               RowBox[{"id", "=", "4412"}], ";", 
               RowBox[{"nEq", "=", "50"}], ";", 
               RowBox[{"nFile", "=", "\"\<data\>\""}], ";", 
               RowBox[{"layout", "=", "HalfCosineSpacing"}], ";", 
               RowBox[{"model", "=", "PMVortexLinearNeumann"}], ";", 
               RowBox[{"reset", "\[Rule]", "False"}], ";", 
               RowBox[{"source", "=", "1"}], ";", 
               RowBox[{"viz", "=", "sp"}], ";"}], ",", 
              RowBox[{"Appearance", "\[Rule]", "\"\<Palette\>\""}]}], "]"}]}],
            "}"}]}], "\[IndentingNewLine]", "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"Alignment", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
            "Left", ",", "Right", ",", "Left", ",", "Center", ",", "Right", 
             ",", "Right", ",", "Left"}], "}"}], ",", "Center", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{
              RowBox[{"{", 
               RowBox[{"1", ",", "1"}], "}"}], "\[Rule]", 
              RowBox[{"{", 
               RowBox[{"Center", ",", "Center"}], "}"}]}], ",", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{
                RowBox[{"-", "2"}], ",", "2"}], "}"}], "\[Rule]", 
              RowBox[{"{", 
               RowBox[{"Left", ",", "Center"}], "}"}]}], ",", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"3", ",", "3"}], "}"}], "\[Rule]", 
              RowBox[{"{", 
               RowBox[{"Right", ",", "Center"}], "}"}]}], ",", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"3", ",", "6"}], "}"}], "\[Rule]", 
              RowBox[{"{", 
               RowBox[{"Left", ",", "Center"}], "}"}]}]}], "}"}]}], "}"}]}], 
        ",", "\[IndentingNewLine]", 
        RowBox[{"Dividers", "\[Rule]", "None"}], ",", "\[IndentingNewLine]", 
        RowBox[{"ItemSize", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
            ".1", ",", "10", ",", "7", ",", "4", ",", "4", ",", "5", ",", 
             "15"}], "}"}], ",", "Automatic"}], "}"}]}]}], 
       "\[IndentingNewLine]", "]"}], ",", 
      RowBox[{"(*", " ", 
       RowBox[{"close", " ", "grid"}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"ImageSize", "\[Rule]", "All"}]}], "]"}], ",", 
    RowBox[{"(*", " ", 
     RowBox[{"close", " ", "panel"}], " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{"(*", 
     RowBox[{
      RowBox[{
      "**", "**", "**", "**", "**", "**", "**", "**", "**", "**", "**", "**", 
       "**", "**", "**", "**", "**", "**", "**", "**", "**", "**", "**", "**",
        "**", "**", "**", "**", "**"}], "*", 
      RowBox[{"**", "**", "**", "**", "**"}]}], "******)"}], 
    "\[IndentingNewLine]", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"xMin", "=", 
         RowBox[{"-", "0.5"}]}], ",", 
        RowBox[{"xMax", "=", "1.5"}], ",", 
        RowBox[{"zMin", "=", 
         RowBox[{"-", "0.5"}]}], ",", 
        RowBox[{"zMax", "=", "0.5"}], ",", 
        RowBox[{"\[CapitalDelta]", "=", "0.07"}], ",", "\[IndentingNewLine]", 
        "a", ",", "aAll", ",", "aW", ",", "b", ",", "bAll", ",", "c", ",", 
        "cAll", ",", "cl", ",", "cm", ",", "colorFunction", ",", "cp", ",", 
        "cpCMin", ",", "cpS", ",", "gAirfoil", ",", "gSurfaceCp", ",", "gViz",
         ",", "icA", ",", "ic\[Psi]A", ",", "icB", ",", "ic\[Psi]B", ",", 
        "icC", ",", "ic\[Psi]C", ",", "nP", ",", "rAll", ",", "rC", ",", "rF",
         ",", "rP", ",", "rPC", ",", "rTE", ",", "tuples", ",", "V", ",", 
        "VMagC", ",", "V\[Infinity]", ",", "xC", ",", "xCenP", ",", "xW", ",",
         "x0", ",", "z0", ",", "\[Alpha]", ",", "\[Psi]", ",", 
        "\[Psi]\[Infinity]"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"\[Alpha]", ",", "rF", ",", "V\[Infinity]"}], "}"}], "=", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Alpha]D", "*", "Degree"}], ",", 
          RowBox[{"{", 
           RowBox[{"x", ",", "z"}], "}"}], ",", 
          RowBox[{"UniformFlow", "[", 
           RowBox[{"1", ",", "\[Alpha]"}], "]"}]}], "}"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"rP", "=", 
        RowBox[{"Which", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"source", "\[Equal]", "1"}], " ", "&&", " ", 
           RowBox[{"nFile", "\[Equal]", "\"\<data\>\""}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"AirfoilFromDataFile", "[", 
           RowBox[{
           "\"\<http://www.ae.uiuc.edu/m-selig/ads/coord/\>\"", "<>", 
            "filename"}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{"source", "\[Equal]", "1"}], ",", "\[IndentingNewLine]", 
          RowBox[{"AirfoilSurfacePoints", "[", 
           RowBox[{
            RowBox[{"FitAirfoil", "[", 
             RowBox[{"AirfoilFromDataFile", "[", 
              RowBox[{
              "\"\<http://www.ae.uiuc.edu/m-selig/ads/coord/\>\"", "<>", 
               "filename"}], "]"}], "]"}], ",", 
            RowBox[{"NDiscretizeUnitSegment", "[", 
             RowBox[{"nFile", ",", 
              RowBox[{"Layout", "\[Rule]", "layout"}]}], "]"}]}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"source", "\[Equal]", "2"}], " ", "&&", " ", 
           RowBox[{"nFile", "\[Equal]", "\"\<data\>\""}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"AirfoilFromDataFile", "[", 
           RowBox[{
           "\"\<Aerodynamics\\\\Airfoil Data\\\\\>\"", "<>", "filename"}], 
           "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{"source", "\[Equal]", "2"}], ",", "\[IndentingNewLine]", 
          RowBox[{"AirfoilSurfacePoints", "[", 
           RowBox[{
            RowBox[{"FitAirfoil", "[", 
             RowBox[{"AirfoilFromDataFile", "[", 
              RowBox[{
              "\"\<Aerodynamics\\\\Airfoil Data\\\\\>\"", "<>", "filename"}], 
              "]"}], "]"}], ",", 
            RowBox[{"NDiscretizeUnitSegment", "[", 
             RowBox[{"nFile", ",", 
              RowBox[{"Layout", "\[Rule]", "layout"}]}], "]"}]}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"source", "\[Equal]", "3"}], ",", "\[IndentingNewLine]", 
          RowBox[{"AirfoilSurfacePoints", "[", 
           RowBox[{"NACA4DigitAirfoil", "[", 
            RowBox[{"id", ",", 
             RowBox[{"NDiscretizeUnitSegment", "[", 
              RowBox[{"nEq", ",", 
               RowBox[{"Layout", "\[Rule]", "layout"}]}], "]"}]}], "]"}], 
           "]"}]}], "\[IndentingNewLine]", "]"}]}], ";", 
       RowBox[{"(*", " ", 
        RowBox[{"close", " ", "Which"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"nP", "=", 
        RowBox[{
         RowBox[{"NumberOfVectors", "[", "rP", "]"}], "-", "1"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"cl", ",", "cm"}], "}"}], "=", 
        RowBox[{"Which", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"model", "===", "PMVortexLinearNeumann"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{"{", 
             RowBox[{"a", ",", "b", ",", "rC", ",", "VMagC"}], "}"}], "=", 
            RowBox[{"model", "[", 
             RowBox[{"rP", ",", "\[Alpha]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"ForceAndMoment", "[", 
            RowBox[{
            "\"\<Vortex\>\"", ",", "a", ",", "b", ",", "rP", ",", "\[Alpha]", 
             ",", "rC"}], "]"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"model", "===", "PMDoubletQuadraticDirichlet"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"rTE", "=", 
            RowBox[{"Mean", "[", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"First", "[", "rP", "]"}], ",", 
               RowBox[{"Last", "[", "rP", "]"}]}], "}"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"rPC", "=", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{"rP", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"1", "\[Rule]", "rTE"}], ",", 
                RowBox[{
                 RowBox[{"-", "1"}], "\[Rule]", "rTE"}]}], "}"}]}], "]"}]}], 
           ";", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
             "a", ",", "b", ",", "c", ",", "aW", ",", "rC", ",", "VMagC"}], 
             "}"}], "=", 
            RowBox[{"PMDoubletQuadraticDirichlet", "[", 
             RowBox[{"rPC", ",", "\[Alpha]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"xW", "=", 
            RowBox[{"XWake", "/.", 
             RowBox[{"Options", "[", 
              RowBox[{"PMDoubletQuadraticDirichlet", ",", "XWake"}], 
              "]"}]}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"rAll", "=", 
            RowBox[{"Append", "[", 
             RowBox[{"rP", ",", 
              RowBox[{"{", 
               RowBox[{"xW", ",", "0."}], "}"}]}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"aAll", ",", "bAll", ",", "cAll"}], "}"}], "=", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Append", "[", 
               RowBox[{"a", ",", "aW"}], "]"}], ",", 
              RowBox[{"Append", "[", 
               RowBox[{"b", ",", "0.0"}], "]"}], ",", 
              RowBox[{"Append", "[", 
               RowBox[{"c", ",", "0.0"}], "]"}]}], "}"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"ForceAndMoment", "[", 
            RowBox[{
            "\"\<Doublet\>\"", ",", "a", ",", "b", ",", "c", ",", "rPC", ",", 
             "\[Alpha]", ",", "rC"}], "]"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"model", "===", "PMDoubletConstantDirichlet"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"rTE", "=", 
            RowBox[{"Mean", "[", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"First", "[", "rP", "]"}], ",", 
               RowBox[{"Last", "[", "rP", "]"}]}], "}"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"rPC", "=", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{"rP", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"1", "\[Rule]", "rTE"}], ",", 
                RowBox[{
                 RowBox[{"-", "1"}], "\[Rule]", "rTE"}]}], "}"}]}], "]"}]}], 
           ";", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"a", ",", "aW", ",", "rC", ",", "VMagC"}], "}"}], "=", 
            RowBox[{"PMDoubletConstantDirichlet", "[", 
             RowBox[{"rPC", ",", "\[Alpha]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"xW", "=", 
            RowBox[{"XWake", "/.", 
             RowBox[{"Options", "[", 
              RowBox[{"PMDoubletQuadraticDirichlet", ",", "XWake"}], 
              "]"}]}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"rAll", "=", 
            RowBox[{"Append", "[", 
             RowBox[{"rP", ",", 
              RowBox[{"{", 
               RowBox[{"xW", ",", "0."}], "}"}]}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"aAll", "=", 
            RowBox[{"Append", "[", 
             RowBox[{"a", ",", "aW"}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"ForceAndMoment", "[", 
            RowBox[{
            "\"\<Doublet\>\"", ",", "a", ",", "rPC", ",", "\[Alpha]", ",", 
             "rC"}], "]"}]}]}], "\[IndentingNewLine]", "]"}]}], ";", 
       RowBox[{"(*", " ", 
        RowBox[{"close", " ", "Which"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"xC", "=", 
        RowBox[{
         RowBox[{"Components", "[", "rC", "]"}], "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"cpS", "=", 
        RowBox[{"1", "-", 
         RowBox[{"VMagC", "^", "2"}]}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"xCenP", "=", 
        RowBox[{
         RowBox[{"-", "cm"}], "/", "cl"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"gSurfaceCp", "=", 
        RowBox[{"PlotSurfacePressureCoefficient", "[", 
         RowBox[{"xC", ",", "cpS", ",", "\[IndentingNewLine]", 
          RowBox[{"Epilog", "\[Rule]", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Inset", "[", 
              RowBox[{
               RowBox[{"Panel", "[", 
                RowBox[{"Grid", "[", "\[IndentingNewLine]", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{"Row", "[", 
                    RowBox[{"{", 
                    RowBox[{"\"\<\[Alpha] = \>\"", ",", 
                    RowBox[{"NumberForm", "[", 
                    RowBox[{
                    RowBox[{"\[Alpha]", "/", "Degree"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"3", ",", "2"}], "}"}]}], "]"}], ",", 
                    "\"\<\[Degree]\>\""}], "}"}], "]"}], "}"}], ",", 
                   "\[IndentingNewLine]", 
                   RowBox[{"{", 
                    RowBox[{"Row", "[", 
                    RowBox[{"{", 
                    RowBox[{"\"\<\!\(\*SubscriptBox[
StyleBox[\"C\",
FontSlant->\"Italic\"], \"\[ScriptL]\"]\) = \>\"", ",", 
                    RowBox[{"NumberForm", "[", 
                    RowBox[{"cl", ",", 
                    RowBox[{"{", 
                    RowBox[{"3", ",", "2"}], "}"}]}], "]"}]}], "}"}], "]"}], 
                    "}"}], ",", "\[IndentingNewLine]", 
                   RowBox[{"{", 
                    RowBox[{"Row", "[", 
                    RowBox[{"{", 
                    RowBox[{"\"\<\!\(\*
StyleBox[SubscriptBox[\"C\", \"m\"],
FontSlant->\"Italic\"]\) = \>\"", ",", 
                    RowBox[{"NumberForm", "[", 
                    RowBox[{"cm", ",", 
                    RowBox[{"{", 
                    RowBox[{"3", ",", "3"}], "}"}]}], "]"}]}], "}"}], "]"}], 
                    "}"}], ",", "\[IndentingNewLine]", 
                   RowBox[{"{", 
                    RowBox[{"Row", "[", 
                    RowBox[{"{", 
                    RowBox[{"\"\<\!\(\*SubscriptBox[
StyleBox[\"x\",
FontSlant->\"Italic\"], \"cp\"]\) = \>\"", ",", " ", 
                    RowBox[{"NumberForm", "[", 
                    RowBox[{"xCenP", ",", 
                    RowBox[{"{", 
                    RowBox[{"3", ",", "3"}], "}"}]}], "]"}]}], "}"}], "]"}], 
                    "}"}]}], "}"}], "\[IndentingNewLine]", "]"}], "]"}], ",", 
               RowBox[{"{", 
                RowBox[{"Right", ",", "Top"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"Right", ",", "Top"}], "}"}]}], "]"}], ",", 
             RowBox[{"Inset", "[", 
              RowBox[{
               RowBox[{"PlotAirfoil", "[", 
                RowBox[{"rP", ",", 
                 RowBox[{"Epilog", "\[Rule]", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"EdgeForm", "[", "Black", "]"}], ",", "Yellow", 
                    ",", 
                    RowBox[{"Disk", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"xCenP", ",", "0"}], "}"}], ",", ".012"}], 
                    "]"}]}], "}"}]}], ",", 
                 RowBox[{"Frame", "\[Rule]", "False"}], ",", 
                 RowBox[{"PlotRange", "\[Rule]", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}], ",", "Automatic"}], 
                   "}"}]}]}], "]"}], ",", 
               RowBox[{"{", 
                RowBox[{"0", ",", "0"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"0", ",", "0"}], "}"}], ",", "1"}], "]"}]}], 
            "}"}]}]}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"If", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"viz", "===", "slp"}], "\[IndentingNewLine]", ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"\[Psi]\[Infinity]", "=", 
           RowBox[{"UniformFlowPsi", "[", 
            RowBox[{"rF", ",", "\[Alpha]"}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"Which", "[", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"model", "===", "PMVortexLinearNeumann"}], 
            "\[IndentingNewLine]", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{
              RowBox[{"{", 
               RowBox[{"ic\[Psi]A", ",", "ic\[Psi]B"}], "}"}], "=", 
              RowBox[{"ICPsiVortexLinear", "[", 
               RowBox[{"rF", ",", "rP"}], "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"\[Psi]", "=", 
              RowBox[{"\[Psi]\[Infinity]", "+", 
               RowBox[{"ic\[Psi]A", ".", "a"}], "+", 
               RowBox[{"ic\[Psi]B", ".", "b"}]}]}]}], "\[IndentingNewLine]", 
            ",", "\[IndentingNewLine]", 
            RowBox[{"model", "===", "PMDoubletQuadraticDirichlet"}], 
            "\[IndentingNewLine]", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{
              RowBox[{"{", 
               RowBox[{"ic\[Psi]A", ",", "ic\[Psi]B", ",", "ic\[Psi]C"}], 
               "}"}], "=", 
              RowBox[{"ICPsiDoubletQuadratic", "[", 
               RowBox[{"rF", ",", "rAll"}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"\[Psi]", "=", 
              RowBox[{"\[Psi]\[Infinity]", "+", 
               RowBox[{"ic\[Psi]A", ".", "aAll"}], "+", 
               RowBox[{"ic\[Psi]B", ".", "bAll"}], "+", 
               RowBox[{"ic\[Psi]C", ".", "cAll"}]}]}]}], 
            "\[IndentingNewLine]", ",", "\[IndentingNewLine]", 
            RowBox[{"model", "===", "PMDoubletConstantDirichlet"}], 
            "\[IndentingNewLine]", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"ic\[Psi]A", "=", 
              RowBox[{"ICPsiDoubletLinear", "[", 
               RowBox[{"rF", ",", "rAll"}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"\[Psi]", "=", 
              RowBox[{"\[Psi]\[Infinity]", "+", 
               RowBox[{"ic\[Psi]A", ".", "aAll"}]}]}]}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"gViz", "=", 
           RowBox[{"ContourPlot", "[", 
            RowBox[{"\[Psi]", ",", 
             RowBox[{"{", 
              RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
             RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
             RowBox[{"Contours", "\[Rule]", "16"}], ",", 
             RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
             RowBox[{"FrameLabel", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
             RowBox[{"MaxRecursion", "\[Rule]", "3"}], ",", 
             RowBox[{"PerformanceGoal", "\[Rule]", "\"\<Speed\>\""}], ",", 
             RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}]}], 
         "\[IndentingNewLine]", ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"V\[Infinity]", "=", 
           RowBox[{"UniformFlow", "[", 
            RowBox[{"1", ",", "\[Alpha]"}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"z0", "=", 
           RowBox[{"Range", "[", 
            RowBox[{"zMin", ",", "zMax", ",", "\[CapitalDelta]"}], "]"}]}], 
          ";", "\[IndentingNewLine]", 
          RowBox[{"x0", "=", 
           RowBox[{"Table", "[", 
            RowBox[{
             RowBox[{"xMin", "+", "\[CapitalDelta]"}], ",", 
             RowBox[{"{", 
              RowBox[{"Length", "[", "z0", "]"}], "}"}]}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"tuples", "=", 
           RowBox[{"Transpose", "[", 
            RowBox[{"{", 
             RowBox[{"x0", ",", "z0"}], "}"}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"Which", "[", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"model", "===", "PMVortexLinearNeumann"}], 
            "\[IndentingNewLine]", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{
              RowBox[{"{", 
               RowBox[{"icA", ",", "icB"}], "}"}], "=", 
              RowBox[{"ICVortexLinear", "[", 
               RowBox[{"rF", ",", "rP"}], "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"V", "=", 
              RowBox[{"V\[Infinity]", "+", 
               RowBox[{"icA", ".", "a"}], "+", 
               RowBox[{"icB", ".", "b"}]}]}]}], "\[IndentingNewLine]", ",", 
            "\[IndentingNewLine]", 
            RowBox[{"model", "===", "PMDoubletQuadraticDirichlet"}], 
            "\[IndentingNewLine]", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{
              RowBox[{"{", 
               RowBox[{"icA", ",", "icB", ",", "icC"}], "}"}], "=", 
              RowBox[{"ICDoubletQuadratic", "[", 
               RowBox[{"rF", ",", "rAll"}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"V", "=", 
              RowBox[{"V\[Infinity]", "+", 
               RowBox[{"icA", ".", "aAll"}], "+", 
               RowBox[{"icB", ".", "bAll"}], "+", 
               RowBox[{"icC", ".", "cAll"}]}]}]}], "\[IndentingNewLine]", ",",
             "\[IndentingNewLine]", 
            RowBox[{"model", "===", "PMDoubletConstantDirichlet"}], 
            "\[IndentingNewLine]", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"icA", "=", 
              RowBox[{"ICDoubletConstant", "[", 
               RowBox[{"rF", ",", "rAll"}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"V", "=", 
              RowBox[{"V\[Infinity]", "+", 
               RowBox[{"icA", ".", "aAll"}]}]}]}]}], "\[IndentingNewLine]", 
           "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"Which", "[", 
           RowBox[{
            RowBox[{"viz", "===", "sp"}], "\[IndentingNewLine]", ",", 
            "\[IndentingNewLine]", 
            RowBox[{"gViz", "=", 
             RowBox[{"StreamPlot", "[", 
              RowBox[{"V", ",", 
               RowBox[{"{", 
                RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
               RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
               RowBox[{"FrameLabel", "\[Rule]", 
                RowBox[{"{", 
                 RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
               RowBox[{"StreamPoints", "\[Rule]", 
                RowBox[{"{", "tuples", "}"}]}], ",", 
               RowBox[{"RotateLabel", "\[Rule]", "False"}], ",", 
               RowBox[{"StreamScale", "\[Rule]", 
                RowBox[{"{", 
                 RowBox[{"Full", ",", "All", ",", "0.015"}], "}"}]}]}], 
              "]"}]}], "\[IndentingNewLine]", ",", "\[IndentingNewLine]", 
            RowBox[{"viz", "===", "sdp"}], "\[IndentingNewLine]", ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"cpCMin", "=", 
              RowBox[{"Min", "[", "cpS", "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"colorFunction", "[", 
               RowBox[{"c_", ",", "cMin_"}], "]"}], ":=", 
              RowBox[{"Hue", "[", 
               RowBox[{
                RowBox[{"If", "[", 
                 RowBox[{
                  RowBox[{"c", "\[GreaterEqual]", "0"}], ",", ".0483667", ",",
                   "0.633338"}], "]"}], ",", 
                RowBox[{"If", "[", 
                 RowBox[{
                  RowBox[{"c", "\[GreaterEqual]", "0"}], ",", "c", ",", 
                  RowBox[{"c", "/", "cMin"}]}], "]"}], ",", 
                RowBox[{"Which", "[", 
                 RowBox[{
                  RowBox[{"cMin", "\[LessEqual]", "c", "<", 
                   RowBox[{"-", "1"}]}], ",", "0.6472", ",", 
                  RowBox[{
                   RowBox[{"-", "1"}], "\[LessEqual]", "c", "<", "0"}], ",", 
                  RowBox[{"1", "+", 
                   RowBox[{".3528", "c"}]}], ",", 
                  RowBox[{"c", "\[GreaterEqual]", "0"}], ",", "1"}], "]"}]}], 
               "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"cp", "=", 
              RowBox[{"1", "-", 
               RowBox[{"V", ".", "V"}]}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"gViz", "=", 
              RowBox[{"StreamDensityPlot", "[", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"V", ",", "cp"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
                RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
                RowBox[{"ColorFunction", "\[Rule]", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"colorFunction", "[", 
                    RowBox[{"#", ",", "cpCMin"}], "]"}], "&"}], ")"}]}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"ColorFunctionScaling", "\[Rule]", "False"}], ",", 
                RowBox[{"FrameLabel", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
                RowBox[{"MaxRecursion", "\[Rule]", "6"}], ",", 
                RowBox[{"RotateLabel", "\[Rule]", "False"}], ",", 
                RowBox[{"StreamPoints", "\[Rule]", 
                 RowBox[{"{", "tuples", "}"}]}], ",", 
                RowBox[{"StreamScale", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{"Full", ",", "All", ",", "0.015"}], "}"}]}]}], 
               "]"}]}]}]}], "\[IndentingNewLine]", "]"}], ";"}]}], 
        "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{"close", " ", "If"}], " ", "*)"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"gAirfoil", "=", 
        RowBox[{"PlotAirfoil", "[", "rP", "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"graphs", "=", 
        RowBox[{"GraphicsColumn", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Show", "[", 
            RowBox[{"gViz", ",", "gAirfoil"}], "]"}], ",", "gSurfaceCp"}], 
          "}"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"CreateDocument", "[", 
        RowBox[{"{", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Which", "[", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"source", "\[Equal]", "1"}], ",", "\[IndentingNewLine]", 
            RowBox[{"TextCell", "[", 
             RowBox[{
              RowBox[{
              "\"\<Potential Flow Over the Airfoil Described by the UIUC File \
\>\"", "<>", "filename"}], ",", "\"\<Section\>\""}], "]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"source", "\[Equal]", "2"}], ",", "\[IndentingNewLine]", 
            RowBox[{"TextCell", "[", 
             RowBox[{
              RowBox[{
              "\"\<Potential Flow Over the Airfoil Described by the Data File \
\>\"", "<>", "filename"}], ",", "\"\<Section\>\""}], "]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"source", "\[Equal]", "3"}], ",", "\[IndentingNewLine]", 
            RowBox[{"TextCell", "[", 
             RowBox[{
              RowBox[{"\"\<Potential Flow Over a NACA-\>\"", "<>", 
               RowBox[{"ToString", "[", "id", "]"}], "<>", 
               "\"\< Airfoil\>\""}], ",", "\"\<Section\>\""}], "]"}]}], "]"}],
           ",", "\[IndentingNewLine]", 
          RowBox[{"TextCell", "[", 
           RowBox[{"\"\<Model: \>\"", "<>", 
            RowBox[{"ToString", "[", "model", "]"}]}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"source", "\[Equal]", "1"}], " ", "||", " ", 
               RowBox[{"source", "\[Equal]", "2"}]}], " ", ")"}], " ", "&&", 
             " ", 
             RowBox[{"nFile", "\[Equal]", "\"\<data\>\""}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"TextCell", "[", 
             RowBox[{"\"\<Discretization: \>\"", "<>", 
              RowBox[{"ToString", "[", "nP", "]"}], "<>", 
              "\"\< panels from data file.\>\""}], "]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"TextCell", "[", 
             RowBox[{"\"\<Discretization: \>\"", "<>", 
              RowBox[{"ToString", "[", "nP", "]"}], "<>", 
              "\"\< panels in \>\"", "<>", 
              RowBox[{"ToString", "[", "layout", "]"}], "<>", 
              "\"\< layout\>\""}], "]"}]}], "]"}], ",", "\[IndentingNewLine]",
           "graphs"}], "}"}], "]"}], ";"}]}], "\[IndentingNewLine]", "]"}]}], 
   RowBox[{"(*", " ", 
    RowBox[{"close", " ", "module"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{
     RowBox[{
     "**", "**", "**", "**", "**", "**", "**", "**", "**", "**", "**", "**", "**",
       "**", "**", "**", "**", "**", "**", "**", "**", "**", "**", "**", "**",
       "**", "**", "**", "**", "**", "**", "**", "**"}], "*", "**"}], 
    "******)"}], "\[IndentingNewLine]", "]"}], 
  RowBox[{"(*", " ", 
   RowBox[{"close", " ", "Interpretation"}], " ", "*)"}]}]], "Input",
 CellChangeTimes->{{3.4903861261036*^9, 3.4903861335126*^9}, {
   3.4903862515436*^9, 3.4903862543266*^9}, {3.4903862858836*^9, 
   3.4903865283156*^9}, {3.4903865633456*^9, 3.4903865637906*^9}, {
   3.4903866195666*^9, 3.4903867092826*^9}, {3.4903867530346003`*^9, 
   3.4903868726976*^9}, {3.4903869063386*^9, 3.4903871011556*^9}, {
   3.4903871591626*^9, 3.4903872173526*^9}, {3.4903872485355997`*^9, 
   3.4903872927676*^9}, 3.4903874201096*^9, {3.4906353220267563`*^9, 
   3.4906353323071566`*^9}, 3.4906353733039565`*^9, {3.4906356256807566`*^9, 
   3.4906358874019566`*^9}, {3.4906359451063566`*^9, 
   3.4906360212187567`*^9}, {3.4906361066599565`*^9, 
   3.4906361416351566`*^9}, {3.4906361813193564`*^9, 3.490636183082156*^9}, {
   3.4907175430712805`*^9, 3.490717558452881*^9}, {3.4907175892940807`*^9, 
   3.490717673300081*^9}, {3.4907177070116806`*^9, 3.490717798786481*^9}, {
   3.4907178641504807`*^9, 3.4907179047728806`*^9}, {3.4907185035008807`*^9, 
   3.490718609424881*^9}, {3.4907186926820807`*^9, 3.490718698688081*^9}, {
   3.4907226165812807`*^9, 3.490722672335681*^9}, {3.4907238396632805`*^9, 
   3.490723871190881*^9}, {3.490723935540881*^9, 3.490723940532881*^9}, {
   3.4907239977536807`*^9, 3.490724079310481*^9}, {3.4907243491436806`*^9, 
   3.4907243568500805`*^9}, {3.490724417019281*^9, 3.4907244920240808`*^9}, {
   3.4908073520174603`*^9, 3.49080738208646*^9}, {3.49080800630346*^9, 
   3.4908080401414604`*^9}, {3.4908080770834603`*^9, 
   3.4908081062014604`*^9}, {3.4908081436624603`*^9, 
   3.4908083068124604`*^9}, {3.4908083378474607`*^9, 
   3.4908083741174603`*^9}, {3.4908084891534605`*^9, 
   3.4908085144004602`*^9}, {3.4908085516204605`*^9, 
   3.4908086692874603`*^9}, {3.4908087186554604`*^9, 
   3.4908087440244603`*^9}, {3.4908102657664604`*^9, 
   3.4908102935384607`*^9}, {3.4908103468244605`*^9, 
   3.4908104300684605`*^9}, {3.49081046403946*^9, 3.4908106372274604`*^9}, {
   3.4908107736064606`*^9, 3.49081078467846*^9}, 3.49081082473046*^9, {
   3.49081086119246*^9, 3.4908111119374604`*^9}, {3.4908111449154606`*^9, 
   3.4908111454654603`*^9}, {3.4908111759304605`*^9, 
   3.4908112788574605`*^9}, {3.4908114692024603`*^9, 
   3.4908115000234604`*^9}, {3.4908115769584603`*^9, 
   3.4908115863014603`*^9}, {3.49081167798046*^9, 3.4908116827384605`*^9}, {
   3.4908117225754604`*^9, 3.4908118827864604`*^9}, {3.4908144243014603`*^9, 
   3.4908145525074606`*^9}, {3.4908145969244604`*^9, 
   3.4908147239784603`*^9}, {3.490815586606*^9, 3.490815687376*^9}, {
   3.490815719512*^9, 3.4908157553789997`*^9}, {3.490815817683*^9, 
   3.4908159132130003`*^9}, {3.490815998796*^9, 3.490816045264*^9}, {
   3.490816103367*^9, 3.490816106852*^9}, {3.4908161519300003`*^9, 
   3.4908161560889997`*^9}, {3.490816217399*^9, 3.490816219366*^9}, {
   3.490816426292*^9, 3.4908164568450003`*^9}, {3.490816526645*^9, 
   3.490816543543*^9}, {3.490816622796*^9, 3.490816641487*^9}, {
   3.490816787488*^9, 3.490816832528*^9}, {3.490816895394*^9, 
   3.490816941473*^9}, 3.490817326671*^9, {3.4908177158594*^9, 
   3.4908177394323997`*^9}, {3.4908177919284*^9, 3.4908178050634003`*^9}, {
   3.4908179079344*^9, 3.4908179133094*^9}, {3.4908179498574*^9, 
   3.4908180354294*^9}, {3.4908181024544*^9, 3.4908181079234*^9}, {
   3.4908181703934*^9, 3.4908182087074003`*^9}, {3.4908182690254*^9, 
   3.4908182866233997`*^9}, {3.4908183404044*^9, 3.4908183545794*^9}, {
   3.4908183952553997`*^9, 3.4908184100313997`*^9}, {3.4908184591434*^9, 
   3.4908184799154*^9}, {3.4908185286364*^9, 3.4908185411373997`*^9}, {
   3.4908185920424*^9, 3.4908186173004*^9}, {3.4908186724664*^9, 
   3.4908186780424*^9}, {3.4908187300834*^9, 3.4908187349484*^9}, {
   3.4908188271314*^9, 3.4908188595134*^9}, 3.4908189010734*^9, {
   3.4908189434414*^9, 3.4908189599653997`*^9}, 3.4908190536154003`*^9, {
   3.4908190889504004`*^9, 3.4908190944533997`*^9}, {3.4908191336284*^9, 
   3.4908192169184*^9}, {3.4908192582743998`*^9, 3.4908192594694*^9}, {
   3.4908193116854*^9, 3.4908193745334*^9}, {3.4908194798484*^9, 
   3.4908195301314*^9}, {3.4908195720154*^9, 3.4908196054474*^9}, 
   3.4908197038504*^9, 3.4908197551354*^9, {3.4908197924764*^9, 
   3.4908197959114*^9}, {3.4908198548424*^9, 3.4908199964094*^9}, {
   3.4915564793412886`*^9, 3.491556543792289*^9}, 3.491560251067289*^9, {
   3.4915603420112886`*^9, 3.491560363318289*^9}, {3.491560401831289*^9, 
   3.4915604604142885`*^9}, {3.491562028982289*^9, 3.4915620411972885`*^9}, {
   3.4915621585962887`*^9, 3.4915621595632887`*^9}, {3.4915621922342887`*^9, 
   3.491562212455289*^9}, {3.4915639453552885`*^9, 3.491563953061289*^9}, {
   3.491564027917289*^9, 3.491564036988289*^9}, {3.491564137922289*^9, 
   3.491564175075289*^9}, {3.491564383295289*^9, 3.4915644115182886`*^9}, {
   3.491564480090289*^9, 3.4915645122612886`*^9}, {3.491564543450289*^9, 
   3.491564574542289*^9}, {3.491564680581289*^9, 3.4915647334992886`*^9}, {
   3.491564905394289*^9, 3.4915649200112886`*^9}, 3.4915649505862885`*^9, {
   3.4915651310862885`*^9, 3.491565175401289*^9}, {3.491565213507289*^9, 
   3.4915652604382887`*^9}, {3.491565303465289*^9, 3.4915653148742886`*^9}, {
   3.491565376272289*^9, 3.491565521160289*^9}, {3.491565569692289*^9, 
   3.4915656070432887`*^9}, {3.4915657467632885`*^9, 3.491565762258289*^9}, {
   3.491566116278289*^9, 3.491566206132289*^9}, 3.491566241901289*^9, {
   3.4915662990302887`*^9, 3.491566315292289*^9}, {3.491566392217289*^9, 
   3.4915664108122888`*^9}, {3.4916404707070074`*^9, 3.491640516170007*^9}, {
   3.4916405599430075`*^9, 3.4916405925580072`*^9}, {3.4917268911614*^9, 
   3.4917269101274*^9}, {3.4917287279694*^9, 3.4917287356563997`*^9}, {
   3.4917288901974*^9, 3.4917289883704*^9}, {3.4917301388794003`*^9, 
   3.4917301556274*^9}, {3.4917307096904*^9, 3.4917307171854*^9}, {
   3.4917307472784*^9, 3.4917308669374*^9}, {3.4917310781484003`*^9, 
   3.4917310855164003`*^9}, {3.4917311669463997`*^9, 
   3.4917311758803997`*^9}, {3.4917312060434*^9, 3.4917312214374*^9}, {
   3.4917312613924*^9, 3.4917312729974003`*^9}, {3.4917314235544*^9, 
   3.4917314365464*^9}, 3.4917316275224*^9, {3.4917316951384*^9, 
   3.4917317070204*^9}, {3.4921628085187645`*^9, 3.492162984944764*^9}, {
   3.492163017364764*^9, 3.4921630612427645`*^9}, {3.4921631188777647`*^9, 
   3.492163192109764*^9}, {3.4921632443867645`*^9, 3.4921633052687645`*^9}, {
   3.492163528168764*^9, 3.4921635581227646`*^9}, {3.4921636316587644`*^9, 
   3.492163758194764*^9}, {3.4921638029507647`*^9, 3.492163834022764*^9}, 
   3.4921638658587646`*^9, {3.492164012977764*^9, 3.4921640417767644`*^9}, 
   3.4921640805337644`*^9, {3.492164132333764*^9, 3.4921641474747643`*^9}, {
   3.4921642238887644`*^9, 3.492164271208764*^9}, {3.4921643018487644`*^9, 
   3.4921643930017643`*^9}, 3.4921644277187643`*^9, {3.4921645888657646`*^9, 
   3.4921646214367647`*^9}, {3.4921646550727644`*^9, 
   3.4921646780317645`*^9}, {3.4921647249347644`*^9, 3.492164725965764*^9}, {
   3.4921647705287647`*^9, 3.4921647904927645`*^9}, 3.4921648319957647`*^9, {
   3.4921648696197643`*^9, 3.4921649316387644`*^9}, {3.4921650130547647`*^9, 
   3.4921650192577643`*^9}, {3.4921650507927647`*^9, 3.4921650558687644`*^9}, 
   3.4921651263147645`*^9, {3.492165189899764*^9, 3.4921652221277647`*^9}, 
   3.492165258432764*^9, {3.4921652977857647`*^9, 3.4921653787437644`*^9}, 
   3.4921654846867647`*^9, {3.4921655450997643`*^9, 3.4921656861717644`*^9}, {
   3.4921657178897643`*^9, 3.4921658651057644`*^9}, {3.492166041932764*^9, 
   3.4921660542577643`*^9}, {3.492180074704*^9, 3.492180077731*^9}, {
   3.4921801173450003`*^9, 3.492180126975*^9}, {3.4921802412320004`*^9, 
   3.4921802427860003`*^9}, {3.492180367008*^9, 3.492180370579*^9}, {
   3.492180567012*^9, 3.492180576953*^9}, {3.492180607502*^9, 
   3.4921806087799997`*^9}, {3.492180659551*^9, 3.4921806633059998`*^9}, {
   3.4921807015810003`*^9, 3.4921807110690002`*^9}, {3.4922469563934717`*^9, 
   3.4922469607014713`*^9}, {3.4922479983264713`*^9, 
   3.4922480429544716`*^9}, {3.492248111019471*^9, 3.4922481666214714`*^9}, {
   3.492248209835471*^9, 3.4922483200844717`*^9}, {3.4922483540284715`*^9, 
   3.4922484358444715`*^9}, {3.4922484732264714`*^9, 3.4922484963044715`*^9}, 
   3.4924223049248*^9, {3.4929384867677703`*^9, 3.4929385010357704`*^9}, {
   3.4929385548897705`*^9, 3.4929385795117702`*^9}, 3.4929395658547707`*^9, {
   3.4929396401697702`*^9, 3.4929396624057703`*^9}, 3.4929397397197704`*^9, {
   3.49293981099477*^9, 3.4929398136227703`*^9}, {3.4929399009467707`*^9, 
   3.4929399323007703`*^9}, {3.4929399674577703`*^9, 
   3.4929400053057704`*^9}, {3.4929400587147703`*^9, 3.49294007030177*^9}, {
   3.4929401087737703`*^9, 3.4929401397477703`*^9}, {3.4929405906547704`*^9, 
   3.4929406446597705`*^9}, {3.4929408761487703`*^9, 
   3.4929409021737704`*^9}, {3.49294098940777*^9, 3.4929410297837706`*^9}, {
   3.4929410786467705`*^9, 3.4929410814387703`*^9}, {3.4929411146367702`*^9, 
   3.4929411692457705`*^9}, {3.4929414543327703`*^9, 3.49294164826377*^9}, {
   3.4929416791757703`*^9, 3.4929417384727707`*^9}, {3.4929417718577704`*^9, 
   3.4929417850997705`*^9}, {3.4929419095337706`*^9, 3.49294192927077*^9}, {
   3.4929419699187703`*^9, 3.49294204267177*^9}, {3.4929421757657704`*^9, 
   3.4929421886587706`*^9}, {3.4929422795497704`*^9, 
   3.4929422847647705`*^9}, {3.49294249964577*^9, 3.49294251234177*^9}, {
   3.4929425448407707`*^9, 3.4929425467667704`*^9}, {3.4929425771607704`*^9, 
   3.4929425781777706`*^9}, {3.49345451865627*^9, 3.49345454129527*^9}, {
   3.49345469708327*^9, 3.49345471104627*^9}, {3.49345474288927*^9, 
   3.49345475227727*^9}, {3.4934548458932695`*^9, 3.49345508865127*^9}, {
   3.49345537944427*^9, 3.4934555372802696`*^9}, 3.49345557367927*^9, 
   3.4934556192462697`*^9, {3.4934556909712696`*^9, 3.49345572809227*^9}, {
   3.49345576030427*^9, 3.4934557695682697`*^9}, {3.49345585229527*^9, 
   3.4934558604242697`*^9}, {3.49345591353127*^9, 3.4934559269662695`*^9}, {
   3.4934559614892697`*^9, 3.4934560864762697`*^9}, {3.49345626307327*^9, 
   3.49345645138927*^9}, {3.4934564864712696`*^9, 3.49345648796027*^9}, {
   3.49345651979027*^9, 3.4934566486622696`*^9}, {3.49345670038227*^9, 
   3.49345676859027*^9}, {3.49345680033727*^9, 3.49345682932227*^9}, 
   3.49345688628827*^9, {3.49345701644427*^9, 3.49345708827627*^9}, {
   3.49345726435627*^9, 3.4934573273902698`*^9}, {3.49345748572927*^9, 
   3.49345763532627*^9}, {3.4934576733602695`*^9, 3.49345771554627*^9}, {
   3.4934577462462697`*^9, 3.49345781305227*^9}, {3.4934578490632696`*^9, 
   3.49345795463227*^9}, {3.49345813911627*^9, 3.4934581646732697`*^9}, 
   3.4934583735302696`*^9, {3.4934584289502697`*^9, 3.49345850661227*^9}, {
   3.4934585512402697`*^9, 3.49345861177427*^9}, {3.49345866797927*^9, 
   3.49345870075827*^9}, {3.49345877898127*^9, 3.4934587922472696`*^9}, {
   3.49345885299527*^9, 3.49345888473727*^9}, {3.49345891780027*^9, 
   3.49345905458627*^9}, {3.49345909318427*^9, 3.4934592545292697`*^9}, {
   3.49345935145527*^9, 3.49345937438927*^9}, {3.49345945277127*^9, 
   3.49345954702027*^9}, {3.49345957763527*^9, 3.49345964057127*^9}, 
   3.4940610259194*^9, {3.4940611998444*^9, 3.4940612184374*^9}, {
   3.4940616344104*^9, 3.4940616836113997`*^9}, 3.4940618561324*^9, {
   3.4940619518714*^9, 3.4940620401334*^9}, {3.4941457332022953`*^9, 
   3.494145741927295*^9}, {3.494150996175295*^9, 3.494151027486295*^9}, {
   3.494151075852295*^9, 3.494151113600295*^9}, {3.4941511561622953`*^9, 
   3.494151159199295*^9}, {3.4941511904522953`*^9, 3.494151263389295*^9}, {
   3.4941512935352955`*^9, 3.4941514168722954`*^9}, {3.494151462455295*^9, 
   3.4941514634432955`*^9}, {3.494151601657295*^9, 3.4941516028732953`*^9}, {
   3.4945780213475657`*^9, 3.494578027874566*^9}, {3.494578061121566*^9, 
   3.4945780664485655`*^9}, 3.494578125028566*^9, {3.4945785355335655`*^9, 
   3.4945785360245657`*^9}, {3.4945786328065658`*^9, 
   3.4945786532805657`*^9}, {3.4945787819225655`*^9, 
   3.4945787862415657`*^9}, {3.4945788639205656`*^9, 3.494578864534566*^9}, {
   3.4945817791107655`*^9, 3.494581787721966*^9}, {3.4945818486087656`*^9, 
   3.494581864957566*^9}, {3.494581910696766*^9, 3.494581970647566*^9}, {
   3.494582002190766*^9, 3.4945820471031656`*^9}, {3.494582135742366*^9, 
   3.4945821399387655`*^9}, {3.494582185069566*^9, 3.494582228952366*^9}, {
   3.4945823651403656`*^9, 3.4945824208479657`*^9}, {3.4945824522975655`*^9, 
   3.4945824561663656`*^9}, {3.494582587019166*^9, 3.494582784920766*^9}, {
   3.494582821658766*^9, 3.4945829218263655`*^9}, {3.4945831006179657`*^9, 
   3.494583153673566*^9}, {3.494583296397966*^9, 3.494583403601166*^9}, {
   3.494583436314366*^9, 3.4945834557675657`*^9}, {3.494583580598766*^9, 
   3.494583594482766*^9}, {3.494583640830366*^9, 3.4945836489423656`*^9}, {
   3.494583702325566*^9, 3.4945837415595655`*^9}, {3.4945837720419655`*^9, 
   3.4945838456583657`*^9}, {3.4945838894787655`*^9, 3.494583940724766*^9}, {
   3.494584034012766*^9, 3.4945840602519655`*^9}, {3.494584138563966*^9, 
   3.494584236578766*^9}, {3.4945843962027655`*^9, 3.494584454967966*^9}, {
   3.494584494186366*^9, 3.494584511065566*^9}, {3.4945845420939655`*^9, 
   3.494584576039566*^9}, {3.4945847171727657`*^9, 3.4945847254563656`*^9}, 
   3.4945847897751656`*^9, {3.494584880379966*^9, 3.494584882283166*^9}, {
   3.4945849173207655`*^9, 3.494584918241166*^9}, {3.4945849885347657`*^9, 
   3.4945851426003656`*^9}, {3.4946645730683002`*^9, 3.4946645896133003`*^9}, 
   3.4946648406673*^9, {3.4946650121993*^9, 3.4946651003633003`*^9}, 
   3.4946654039383*^9, {3.4946655031883*^9, 3.4946655430003*^9}, 
   3.4946655762703*^9, 3.4946657523623*^9, {3.4946657937373*^9, 
   3.4946658890013*^9}, {3.4946659297613*^9, 3.4946660225622997`*^9}, {
   3.4946660669193*^9, 3.4946663305123*^9}, 3.4946666101633*^9, 
   3.4946669726153*^9, {3.4946670285213003`*^9, 3.4946670879493*^9}, {
   3.4946671286483*^9, 3.4946671701953*^9}, {3.4946672460413*^9, 
   3.4946672494753*^9}, {3.4946673632113*^9, 3.4946673722653*^9}, 
   3.4946674714643*^9, {3.4946675996493*^9, 3.4946676402303*^9}, 
   3.4946679244653*^9, 3.4946680021883*^9, {3.4946680431503*^9, 
   3.4946680479512997`*^9}, 3.4946681255762997`*^9, {3.4946682553563004`*^9, 
   3.4946682890283003`*^9}, {3.4946684599723*^9, 3.4946684867473*^9}, {
   3.4946685949632998`*^9, 3.4946686086673*^9}, {3.4946686866763*^9, 
   3.4946687680613003`*^9}, {3.4946688258813*^9, 3.4946688899603*^9}, 
   3.4946689441723003`*^9, {3.4946689771293*^9, 3.4946689854363003`*^9}, {
   3.4946690682763*^9, 3.4946690852643003`*^9}, {3.4946693171713*^9, 
   3.4946694154753*^9}, {3.4946694496133003`*^9, 3.4946695281863003`*^9}, {
   3.4946695892753*^9, 3.4946696076053*^9}, 3.4946697038723*^9, 
   3.4946697505863*^9, 3.4946698130723*^9, {3.4946699203633003`*^9, 
   3.4946699491752996`*^9}, {3.4946700499653*^9, 3.4946700511723003`*^9}, {
   3.4946701189593*^9, 3.4946701676723003`*^9}, {3.4946702235483*^9, 
   3.4946702411733*^9}, {3.4946703269793*^9, 3.4946703618782997`*^9}, {
   3.4946704811453*^9, 3.4946704831433*^9}, 3.4946705409702997`*^9, {
   3.4946705885203*^9, 3.4946706193843*^9}, {3.4946707219463*^9, 
   3.4946707709442997`*^9}, {3.4946708106713*^9, 3.4946708479823*^9}, {
   3.4946708803623*^9, 3.4946708969803*^9}, {3.4946711746603003`*^9, 
   3.4946711935593*^9}, {3.4946714015523*^9, 3.4946714049653*^9}, {
   3.4947503144963713`*^9, 3.494750325604371*^9}, {3.4947503646543713`*^9, 
   3.494750403347371*^9}, {3.494750497382371*^9, 3.494750502889371*^9}, {
   3.494750641398371*^9, 3.4947506529073715`*^9}, {3.4947507850023713`*^9, 
   3.4947508128763714`*^9}, {3.4947510113953714`*^9, 
   3.4947510396073713`*^9}, {3.494751099795371*^9, 3.494751100880371*^9}, 
   3.494751898273371*^9, {3.4947519648103714`*^9, 3.494752130853371*^9}, {
   3.494752162995371*^9, 3.4947521955883713`*^9}, {3.4947523122773714`*^9, 
   3.494752313854371*^9}, {3.4947524727653713`*^9, 3.494752473462371*^9}, {
   3.4947527283553715`*^9, 3.494752739882371*^9}, {3.494752950629371*^9, 
   3.494752961078371*^9}, {3.494753035830371*^9, 3.494753040740371*^9}, {
   3.494753120767371*^9, 3.494753146059371*^9}, {3.494753255061371*^9, 
   3.494753283889371*^9}, {3.494753337584371*^9, 3.494753338794371*^9}, {
   3.494753527304371*^9, 3.494753556941371*^9}, {3.494753631585371*^9, 
   3.4947536700883713`*^9}, {3.4947537271803713`*^9, 
   3.4947537634283714`*^9}, {3.4947539758363714`*^9, 
   3.4947540055343714`*^9}, {3.494754116087371*^9, 3.494754377832371*^9}, 
   3.494754667745371*^9, {3.4947547093903713`*^9, 3.4947547837583714`*^9}, {
   3.494754837599371*^9, 3.4947548403253713`*^9}, {3.494754871361371*^9, 
   3.4947548973053713`*^9}, {3.494754929181371*^9, 3.4947549514823713`*^9}, {
   3.494754994866371*^9, 3.4947550361543713`*^9}, {3.4947551719823713`*^9, 
   3.4947551962993712`*^9}, {3.4947552877203712`*^9, 
   3.4947552892703714`*^9}, {3.4947553328383713`*^9, 3.4947553677633715`*^9}, 
   3.494755408945371*^9, {3.4947554407893715`*^9, 3.494755457198371*^9}, {
   3.494755610342371*^9, 3.494755611760371*^9}, 3.4947556666473713`*^9, {
   3.4947558242633715`*^9, 3.4947558337183714`*^9}, {3.494755918406371*^9, 
   3.494755918886371*^9}, {3.494755960135371*^9, 3.494756014405371*^9}, {
   3.494756054478371*^9, 3.494756087339371*^9}, {3.495443687664482*^9, 
   3.495443717334482*^9}, {3.4954437747924824`*^9, 3.4954437801344824`*^9}, {
   3.4954439042924824`*^9, 3.4954439101794825`*^9}, {3.495443962378482*^9, 
   3.495443967140482*^9}, {3.4954441343374825`*^9, 3.495444148091482*^9}, {
   3.4954442083334823`*^9, 3.495444223111482*^9}, {3.4954443085244823`*^9, 
   3.4954443271554823`*^9}, {3.495444456395482*^9, 3.495444465684482*^9}, {
   3.495444565032482*^9, 3.495444596526482*^9}, {3.495445014104482*^9, 
   3.495445037954482*^9}, {3.4954451238494825`*^9, 3.4954451312604823`*^9}, {
   3.4954453925164824`*^9, 3.4954454109504824`*^9}, {3.495445487869482*^9, 
   3.4954454936684823`*^9}, 3.495445670750482*^9, {3.4954457649584823`*^9, 
   3.4954457930104823`*^9}, {3.495445982935482*^9, 3.4954461040834823`*^9}, {
   3.4954461345964823`*^9, 3.4954461890234823`*^9}, {3.495446221788482*^9, 
   3.495446410986482*^9}, {3.495446492817482*^9, 3.495446601308482*^9}, {
   3.495446759822482*^9, 3.495446769906482*^9}, {3.495446815956482*^9, 
   3.4954468476134825`*^9}, {3.495446893934482*^9, 3.495446956498482*^9}, {
   3.4954470388594823`*^9, 3.4954471079764824`*^9}, 3.4954471574644823`*^9, {
   3.4954473288854823`*^9, 3.4954473788374825`*^9}, {3.495447409467482*^9, 
   3.495447475819482*^9}, {3.4954475204834824`*^9, 3.4954475743914824`*^9}, {
   3.495447668966482*^9, 3.495447692121482*^9}, {3.495448039968482*^9, 
   3.495448091789482*^9}, {3.495448138482482*^9, 3.495448147327482*^9}, {
   3.495448234006482*^9, 3.495448316659482*^9}, {3.495448348959482*^9, 
   3.4954484303134823`*^9}, {3.495448483164482*^9, 3.495448697279482*^9}, {
   3.495448773402482*^9, 3.4954487789814825`*^9}, {3.4954488404414825`*^9, 
   3.495448850434482*^9}, {3.4954489649784822`*^9, 3.4954489767784824`*^9}, {
   3.495449006921482*^9, 3.495449007519482*^9}, {3.495787942062065*^9, 
   3.495788020555065*^9}, {3.495788082141065*^9, 3.4957881509780655`*^9}, {
   3.4957881837210655`*^9, 3.495788193352065*^9}, 3.495788272133065*^9, {
   3.495788348403065*^9, 3.495788359134065*^9}, 3.495788433137065*^9, {
   3.495788520397065*^9, 3.4957885647280655`*^9}, 3.495788807140065*^9, {
   3.495788915142065*^9, 3.4957889264710655`*^9}, {3.4957889905660653`*^9, 
   3.495789037484065*^9}, {3.495789092866065*^9, 3.4957891050700655`*^9}, {
   3.4957891700720654`*^9, 3.4957892742210655`*^9}, {3.495789306561065*^9, 
   3.495789326616065*^9}, {3.4957893640300655`*^9, 3.495789505413065*^9}, {
   3.4957895372640653`*^9, 3.495789697552065*^9}, 3.495790687937065*^9, 
   3.4957936028725653`*^9, {3.495793643085565*^9, 3.495793644751565*^9}, 
   3.4957937399245653`*^9, 3.495793837834565*^9, {3.495793935381565*^9, 
   3.495793939027565*^9}, {3.495794031235565*^9, 3.495794031897565*^9}, {
   3.4957941282385654`*^9, 3.4957941289975653`*^9}, 3.495794195854565*^9, 
   3.495794296564565*^9, {3.495794337176565*^9, 3.495794337413565*^9}, 
   3.495794375305565*^9, 3.4957944057865653`*^9, {3.495794452288565*^9, 
   3.4957944743445654`*^9}, {3.495794878751565*^9, 3.495794885958565*^9}, 
   3.495794922206565*^9, {3.4957949626615653`*^9, 3.4957949649365654`*^9}, {
   3.495795015083565*^9, 3.4957950476255655`*^9}, {3.4957951732675653`*^9, 
   3.4957951860305653`*^9}, {3.495873908737378*^9, 3.4958739095553784`*^9}, {
   3.4958740362253785`*^9, 3.4958740368403783`*^9}, {3.4958740736543784`*^9, 
   3.495874124440378*^9}, {3.4958741587013783`*^9, 3.495874164626378*^9}, {
   3.4958742176933784`*^9, 3.4958742248133783`*^9}, {3.4958744040413785`*^9, 
   3.4958744111823783`*^9}, {3.4958744423063784`*^9, 
   3.4958744782813787`*^9}, {3.4958745181863785`*^9, 
   3.4958745840683784`*^9}, {3.495874634501378*^9, 3.4958746658143787`*^9}, {
   3.495874733192378*^9, 3.495874734249378*^9}, {3.4958747816663785`*^9, 
   3.495874810671378*^9}, {3.4958748561323786`*^9, 3.4958748915663786`*^9}, {
   3.4958749353843784`*^9, 3.495874997619378*^9}, {3.4958750381433783`*^9, 
   3.4958750394183784`*^9}, {3.4958751360693784`*^9, 
   3.4958751542833786`*^9}, {3.4958752120563784`*^9, 
   3.4958752235643787`*^9}, {3.4958754270473785`*^9, 3.495875428079378*^9}, {
   3.4958754898093786`*^9, 3.4958756687723784`*^9}, {3.4958757058703785`*^9, 
   3.4958757316293783`*^9}, {3.4958758149123783`*^9, 
   3.4958758187803783`*^9}, {3.4958758527923784`*^9, 
   3.4958759044723787`*^9}, {3.495876047815378*^9, 3.4958760860493784`*^9}, {
   3.4958761680903783`*^9, 3.4958762513523784`*^9}, {3.4958763079883785`*^9, 
   3.4958764156583786`*^9}, {3.4958764724083786`*^9, 3.4958765356413784`*^9}, 
   3.4958766506793785`*^9, {3.4958767646493783`*^9, 3.495876765593378*^9}, {
   3.4958775216613784`*^9, 3.4958775408903785`*^9}, 3.4958776034573784`*^9, {
   3.4958776418403783`*^9, 3.495877670605378*^9}, {3.4958778241363783`*^9, 
   3.4958779290493784`*^9}, {3.4958780482943783`*^9, 
   3.4958780580643787`*^9}, {3.4958780907383785`*^9, 
   3.4958781474113784`*^9}, {3.4958783221983786`*^9, 3.495878404499378*^9}, {
   3.4958785973663783`*^9, 3.4958786369113784`*^9}, {3.4958786779843783`*^9, 
   3.4958786804813786`*^9}, 3.495878730190378*^9, {3.4958788042243786`*^9, 
   3.4958788049063787`*^9}, {3.4958788739153786`*^9, 3.4958789532203784`*^9}, 
   3.4958790272553787`*^9, {3.4958790732173786`*^9, 3.4958790844203787`*^9}, {
   3.4958791341443787`*^9, 3.4958791630893784`*^9}, 3.4958793015893784`*^9, 
   3.4958793824653783`*^9, {3.4958794351333785`*^9, 3.4958794374973783`*^9}, {
   3.4958794764643784`*^9, 3.4958795630483785`*^9}, {3.4958797163183784`*^9, 
   3.4958797308013783`*^9}, {3.4958797752133784`*^9, 3.4958799821443787`*^9}, 
   3.4958807669193783`*^9, 3.4958807971993785`*^9, {3.495880845546378*^9, 
   3.4958808475103784`*^9}, {3.4958809849373784`*^9, 3.495881122112378*^9}, {
   3.4960467866017737`*^9, 3.4960468528407736`*^9}, {3.4960468852207737`*^9, 
   3.496046910896774*^9}, {3.496046945724774*^9, 3.496046952721774*^9}, {
   3.4960469956567736`*^9, 3.4960470106677737`*^9}, {3.496047053751774*^9, 
   3.496047127962774*^9}, {3.4960471701777735`*^9, 3.4960471909567738`*^9}, {
   3.4960475975427737`*^9, 3.496047617304774*^9}, {3.4960476681777735`*^9, 
   3.496047726376774*^9}, {3.4960484300167737`*^9, 3.496048432028774*^9}, {
   3.4965746964741697`*^9, 3.4965747247441697`*^9}, {3.4965747819841695`*^9, 
   3.49657484530917*^9}, {3.4965750093591695`*^9, 3.49657505889917*^9}, 
   3.4965751730991697`*^9, {3.4965759378641696`*^9, 3.4965759592441697`*^9}, {
   3.4965760047491693`*^9, 3.4965760696191697`*^9}, {3.4965761135791693`*^9, 
   3.4965761194491696`*^9}, {3.4965761869041696`*^9, 
   3.4965762091291695`*^9}, {3.49657624876917*^9, 3.4965762492941694`*^9}, {
   3.4965765201191697`*^9, 3.4965765214941697`*^9}, {3.4965766726241693`*^9, 
   3.49657668776917*^9}, {3.4965768085541697`*^9, 3.4965768725541697`*^9}, {
   3.4965770905341697`*^9, 3.4965771103391695`*^9}, {3.4965772613341694`*^9, 
   3.4965772707641697`*^9}, 3.4965774107601695`*^9, {3.4965774451941695`*^9, 
   3.4965774537931695`*^9}, {3.4965774938501697`*^9, 
   3.4965774943621693`*^9}, {3.49657757914417*^9, 3.4965776012071695`*^9}, {
   3.4965777351811695`*^9, 3.4965778206691694`*^9}, 3.4965827488651695`*^9, {
   3.49658284130717*^9, 3.49658284999317*^9}, {3.4965830469001694`*^9, 
   3.4965830487011695`*^9}, {3.49658333132117*^9, 3.4965833408691697`*^9}, {
   3.4965864343031693`*^9, 3.49658645067017*^9}, 3.4965879459041696`*^9, {
   3.4965880083901696`*^9, 3.49658802667517*^9}, {3.4965880673931694`*^9, 
   3.49658806859717*^9}, {3.4965880989051695`*^9, 3.4965881887001696`*^9}, {
   3.4965885126011696`*^9, 3.4965885644851694`*^9}, {3.4966547527715645`*^9, 
   3.4966547672639647`*^9}, {3.496655586391364*^9, 3.4966555869997644`*^9}, {
   3.4966556757637644`*^9, 3.4966557121117644`*^9}, {3.4966557665869646`*^9, 
   3.4966557685993643`*^9}, {3.4966558085197644`*^9, 
   3.4966558096429644`*^9}, {3.4966558918237643`*^9, 3.4966558924477644`*^9}, 
   3.4966559562205644`*^9, {3.4966560057037644`*^9, 3.4966560077005644`*^9}, {
   3.4966560439705644`*^9, 3.4966560449065647`*^9}, {3.496656104311364*^9, 
   3.4966561122829647`*^9}, {3.4966561530457644`*^9, 3.4966561543405647`*^9}, 
   3.4966633026817646`*^9, {3.4966689133153644`*^9, 3.496668923954564*^9}, 
   3.4967386291573763`*^9, {3.4967387212285757`*^9, 3.4967387752669764`*^9}, {
   3.496738845264176*^9, 3.496738848399776*^9}, {3.4967388949501762`*^9, 
   3.496739029390976*^9}, {3.496739073008576*^9, 3.4967390785309763`*^9}, {
   3.496739147155376*^9, 3.496739170180976*^9}, {3.496739212456976*^9, 
   3.496739214469376*^9}, {3.4967392654813757`*^9, 3.496739282688176*^9}, 
   3.496739313778976*^9, {3.496739363698976*^9, 3.496739395491776*^9}, 
   3.496739429827376*^9, {3.496740609842576*^9, 3.4967406113869762`*^9}, {
   3.4967406671257763`*^9, 3.496740676142576*^9}, 3.4982176309642873`*^9, {
   3.4983240311896095`*^9, 3.4983240320056095`*^9}, {3.4983252073456097`*^9, 
   3.4983252078856096`*^9}, {3.4983252784046097`*^9, 
   3.4983252798236094`*^9}, {3.49832542013361*^9, 3.4983254218146095`*^9}, {
   3.499086880102579*^9, 3.4990869089025793`*^9}, {3.499087001003579*^9, 
   3.4990870090725794`*^9}, {3.499087056910579*^9, 3.4990870775605793`*^9}, {
   3.4990871581985793`*^9, 3.499087160260579*^9}, {3.4990872021905794`*^9, 
   3.4990872039355793`*^9}, 3.499087237491579*^9, 3.499087451997579*^9, {
   3.4990874880795794`*^9, 3.4990874893505793`*^9}, {3.499087522997579*^9, 
   3.499087528524579*^9}, {3.4990876331065793`*^9, 3.499087657522579*^9}, {
   3.4990877541575794`*^9, 3.4990877551065793`*^9}, 3.4990934462349997`*^9, {
   3.499093476921*^9, 3.49909350108*^9}, {3.499093541167*^9, 
   3.4990935693450003`*^9}, {3.499093609144*^9, 3.499093639509*^9}, {
   3.499093696727*^9, 3.499093712341*^9}, 3.499093917337*^9, {
   3.499093999741*^9, 3.499094001512*^9}, {3.4990940525480003`*^9, 
   3.499094068046*^9}, {3.499094111227*^9, 3.4990941484449997`*^9}, {
   3.499094206837*^9, 3.499094339447*^9}, {3.499094381141*^9, 
   3.4990944122869997`*^9}, {3.4990944430360003`*^9, 3.499094473939*^9}, {
   3.499094606424*^9, 3.499094610585*^9}, 3.499094702627*^9, 
   3.4990947802200003`*^9, 3.4990948114849997`*^9, {3.499094853135*^9, 
   3.499094853401*^9}, {3.499094933141*^9, 3.4990949465959997`*^9}, {
   3.4990950060220003`*^9, 3.499095028506*^9}, {3.499095063199*^9, 
   3.499095064361*^9}, {3.499095095841*^9, 3.499095107877*^9}, {
   3.499095156934*^9, 3.499095178737*^9}, {3.499095212026*^9, 
   3.499095213109*^9}, {3.499095252131*^9, 3.499095252733*^9}, {
   3.499095283731*^9, 3.499095305653*^9}, 3.499095349946*^9, {
   3.499095434719*^9, 3.499095463724*^9}, {3.499095506657*^9, 
   3.4990955244040003`*^9}, {3.499095566337*^9, 3.499095583837*^9}, {
   3.4990958634309998`*^9, 3.4990958713199997`*^9}, {3.499095959434*^9, 
   3.4990959914519997`*^9}, {3.4990960307209997`*^9, 3.49909605298*^9}, {
   3.499096137742*^9, 3.499096146158*^9}, 3.4990961789300003`*^9, 
   3.499096235433*^9, {3.499096453936*^9, 3.499096461741*^9}, {
   3.499096544549*^9, 3.4990965935559998`*^9}, 3.4990966394519997`*^9, {
   3.4990967600480003`*^9, 3.4990967635950003`*^9}, {3.499096794255*^9, 
   3.49909680965*^9}, {3.4990968645360003`*^9, 3.499096870962*^9}, {
   3.499097007981*^9, 3.499097015858*^9}, 3.499097104738*^9, {
   3.499102395844*^9, 3.499102402583*^9}, {3.499102627043*^9, 
   3.4991026278199997`*^9}, {3.499102823734*^9, 3.4991028727869997`*^9}, 
   3.499102932543*^9, {3.49910297363*^9, 3.499103038104*^9}, {
   3.499103119538*^9, 3.4991031203269997`*^9}, 3.4994196592868443`*^9, {
   3.4994199800428443`*^9, 3.499420062422844*^9}, {3.499420131035844*^9, 
   3.499420140834844*^9}, {3.4994203952378445`*^9, 3.4994204008328443`*^9}, 
   3.4994204647598443`*^9, 3.4994204956268444`*^9, {3.499420543953844*^9, 
   3.4994205461418443`*^9}, {3.499420620347844*^9, 3.499420631550844*^9}, 
   3.4994206659178443`*^9, {3.499420710040844*^9, 3.499420710330844*^9}, 
   3.499420790632844*^9, 3.499420849630844*^9, {3.499420946030844*^9, 
   3.499420952738844*^9}, {3.499421068314844*^9, 3.499421076925844*^9}, {
   3.4994212823518443`*^9, 3.499421305533844*^9}, {3.4994226480348444`*^9, 
   3.4994227737188444`*^9}, {3.499422809014844*^9, 3.499422830216844*^9}, {
   3.4994228798318443`*^9, 3.4994229264288445`*^9}, {3.4994231946318445`*^9, 
   3.4994231959438443`*^9}, {3.499423281710844*^9, 3.4994232845348444`*^9}, {
   3.4994233927258444`*^9, 3.499423422339844*^9}, {3.4994235091458445`*^9, 
   3.4994235462268443`*^9}, {3.4994235887448444`*^9, 3.4994237065158443`*^9}, 
   3.499423768533844*^9, {3.499423851526844*^9, 3.4994238549428444`*^9}, {
   3.4994238865228443`*^9, 3.4994238875208445`*^9}, {3.4994239932258444`*^9, 
   3.4994239971578445`*^9}, 3.499424047052844*^9, {3.4994244968458443`*^9, 
   3.499424510650844*^9}, {3.4994247036788445`*^9, 3.499424732024844*^9}, {
   3.499424770033844*^9, 3.4994248470018444`*^9}, 3.499425054599844*^9, {
   3.499425089507844*^9, 3.499425090620844*^9}, {3.4994252334708443`*^9, 
   3.499425236382844*^9}, {3.4994252964508443`*^9, 3.499425300451844*^9}, {
   3.499425423714844*^9, 3.499425444894844*^9}, {3.4994255135728445`*^9, 
   3.499425626613844*^9}, {3.499425671120844*^9, 3.4994256724118443`*^9}, {
   3.499425722905844*^9, 3.499425730511844*^9}, {3.499426066716844*^9, 
   3.499426288551844*^9}, {3.4994267675628443`*^9, 3.4994267676948442`*^9}, {
   3.499426876632844*^9, 3.4994268768558445`*^9}, {3.499428334295844*^9, 
   3.499428348051844*^9}, 3.4994283943258443`*^9, {3.499428450820844*^9, 
   3.499428453309844*^9}, {3.499428593540844*^9, 3.4994285965398445`*^9}, {
   3.4994286936218443`*^9, 3.499428702734844*^9}, {3.499428756533844*^9, 
   3.499429085316844*^9}, {3.4994291639308443`*^9, 3.499429170396844*^9}, {
   3.499429228243844*^9, 3.499429300061844*^9}, {3.499429344526844*^9, 
   3.4994293579218445`*^9}, {3.499430257025844*^9, 3.499430274823844*^9}, {
   3.4994303081388445`*^9, 3.4994303925208445`*^9}, 3.4994305004498444`*^9, {
   3.499430712137844*^9, 3.499430733748844*^9}, {3.4995061050165977`*^9, 
   3.499506105599598*^9}, {3.4995061625175977`*^9, 3.4995061631815977`*^9}, {
   3.4995062708195972`*^9, 3.4995062759795976`*^9}, {3.499506310408598*^9, 
   3.4995063364595976`*^9}, {3.500812539642795*^9, 3.50081255327148*^9}, {
   3.5009972994834003`*^9, 3.5009973309486*^9}, {3.5009976174894*^9, 
   3.5009976239634*^9}, {3.500997681699*^9, 3.5009977023845997`*^9}, {
   3.5009998584762*^9, 3.5009998590690002`*^9}, {3.5009999388006*^9, 
   3.5009999473962*^9}, {3.5010000698036003`*^9, 3.5010000762152*^9}, {
   3.5010001228904*^9, 3.5010001236548*^9}, {3.5010002086904*^9, 
   3.5010002152736*^9}, {3.5010003932696*^9, 3.5010004386656*^9}, {
   3.501000508616*^9, 3.5010005328428*^9}, {3.5010005700643997`*^9, 
   3.5010005717804003`*^9}, {3.5010006290168*^9, 3.5010006776576*^9}, {
   3.5010007166264*^9, 3.5010007713512*^9}, {3.5010008311772003`*^9, 
   3.5010008422844*^9}, {3.5010010420736*^9, 3.501001278008*^9}, {
   3.50100137309*^9, 3.5010013744316*^9}, {3.5010014545064*^9, 
   3.5010016121132*^9}, {3.5010018151316*^9, 3.5010018972188*^9}, 
   3.5010019389955997`*^9, {3.5014130459407997`*^9, 3.5014130615095997`*^9}}],

Cell[BoxData[
 TagBox[
  DynamicModuleBox[{$CellContext`\[Alpha]D$$ = 5., $CellContext`filename$$ = 
   "e205.dat", $CellContext`id$$ = 4412, $CellContext`nFile$$ = 
   "data", $CellContext`nEq$$ = 50, $CellContext`layout$$ = 
   Aerodynamics`AirfoilGeometry`HalfCosineSpacing, $CellContext`model$$ = 
   Aerodynamics`PanelMethods`PMVortexLinearNeumann, $CellContext`reset$$ = 
   False, $CellContext`source$$ = 1, $CellContext`viz$$ = $CellContext`sp}, 
   InterpretationBox[
    PanelBox[
     TagBox[GridBox[{
        {
         StyleBox["\<\"Worksheet for Airfoil Aerodynamics\"\>", "Section",
          StripOnInput->False], "\[SpanFromLeft]", "\[SpanFromLeft]", 
         "\[SpanFromLeft]", "\[SpanFromLeft]", "\[SpanFromLeft]", 
         "\[SpanFromLeft]"},
        {"\<\"\"\>", "\[SpanFromLeft]", "\[SpanFromLeft]", "\[SpanFromLeft]", 
         "\[SpanFromLeft]", "\[SpanFromLeft]", "\[SpanFromLeft]"},
        {
         StyleBox["\<\"Input:\"\>", "Subsection",
          StripOnInput->False], 
         "\[SpanFromLeft]", "\<\"angle of attack (deg)\"\>", 
         "\[SpanFromLeft]", "\[SpanFromLeft]", 
         InputFieldBox[Dynamic[$CellContext`\[Alpha]D$$], Number,
          FieldSize->3], ""},
        {"\<\"\"\>", "\<\"source of airfoil\"\>", 
         InterpretationBox[
          StyleBox[GridBox[{
             {
              
              SetterBox[Dynamic[$CellContext`source$$], {
               1}, "\<\"UIUC database\"\>"], 
              
              SetterBox[Dynamic[$CellContext`source$$], {
               2}, "\<\"file in Airfoil Data\"\>"], 
              
              SetterBox[Dynamic[$CellContext`source$$], {
               3}, "\<\"NACA 4-Digit\"\>"]}
            },
            BaselinePosition->{1, 1},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.]}, 
                Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}, "RowsIndexed" -> {}}],
           Deployed->True],
          SetterBar[
           Dynamic[$CellContext`source$$], {
           1 -> "UIUC database", 2 -> "file in Airfoil Data", 3 -> 
            "NACA 4-Digit"}]], "\[SpanFromLeft]", "\[SpanFromLeft]", 
         "\[SpanFromLeft]", "\[SpanFromLeft]"},
        {"\<\"\"\>", "\<\"file name\"\>", 
         InputFieldBox[Dynamic[$CellContext`filename$$], String,
          Enabled->Dynamic[$CellContext`source$$ != 3],
          FieldSize->10], "\[SpanFromLeft]", "\<\"\"\>", "\<\"4-digit id\"\>", 
         InputFieldBox[
          Dynamic[$CellContext`id$$, If[
           0 <= # <= 9999, $CellContext`id$$ = #, $CellContext`id$$ = 
            FromDigits[
              Take[
               IntegerDigits[#], 4]]]& ],
          ContinuousAction->True,
          Enabled->Dynamic[$CellContext`source$$ == 3],
          FieldSize->3]},
        {"\<\"\"\>", "\<\"discretization:\"\>", 
         InputFieldBox[Dynamic[$CellContext`nFile$$],
          Enabled->Dynamic[$CellContext`source$$ != 3],
          FieldSize->3], "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", 
         InputFieldBox[Dynamic[$CellContext`nEq$$],
          Enabled->Dynamic[$CellContext`source$$ == 3],
          FieldSize->3]},
        {"\<\"\"\>", "\<\"layout\"\>", 
         InterpretationBox[
          StyleBox[GridBox[{
             {
              
              SetterBox[Dynamic[$CellContext`layout$$], {
               Aerodynamics`AirfoilGeometry`ConstantSpacing}, \
"\<\"ConstantSpacing\"\>",
               Enabled->Dynamic[
                 
                 Or[$CellContext`source$$ == 3, $CellContext`nFile$$ != 
                  "data"]]], 
              
              SetterBox[Dynamic[$CellContext`layout$$], {
               Aerodynamics`AirfoilGeometry`CosineSpacing}, \
"\<\"CosineSpacing\"\>",
               Enabled->Dynamic[
                 
                 Or[$CellContext`source$$ == 3, $CellContext`nFile$$ != 
                  "data"]]], 
              SetterBox[Dynamic[$CellContext`layout$$], {
               Aerodynamics`AirfoilGeometry`HalfCosineSpacing}, \
"\<\"HalfCosineSpacing\"\>",
               Enabled->Dynamic[
                 
                 Or[$CellContext`source$$ == 3, $CellContext`nFile$$ != 
                  "data"]]]}
            },
            BaselinePosition->{1, 1},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.]}, 
                Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}, "RowsIndexed" -> {}}],
           Deployed->True],
          SetterBar[
           Dynamic[$CellContext`layout$$], {
           Aerodynamics`AirfoilGeometry`ConstantSpacing -> "ConstantSpacing", 
            Aerodynamics`AirfoilGeometry`CosineSpacing -> "CosineSpacing", 
            Aerodynamics`AirfoilGeometry`HalfCosineSpacing -> 
            "HalfCosineSpacing"}, Enabled -> Dynamic[
             
             Or[$CellContext`source$$ == 3, $CellContext`nFile$$ != 
              "data"]]]], "\[SpanFromLeft]", "\[SpanFromLeft]", 
         "\[SpanFromLeft]", "\[SpanFromLeft]"},
        {"\<\"\"\>", "\<\"model: PM\"\>", 
         InterpretationBox[
          StyleBox[GridBox[{
             {
              
              SetterBox[Dynamic[$CellContext`model$$], {
               Aerodynamics`PanelMethods`PMVortexLinearNeumann}, \
"\<\"VortexLinearNeumann\"\>"], 
              
              SetterBox[Dynamic[$CellContext`model$$], {
               Aerodynamics`PanelMethods`PMDoubletQuadraticDirichlet}, \
"\<\"DoubletQuadraticDirichlet\"\>"], 
              
              SetterBox[Dynamic[$CellContext`model$$], {
               Aerodynamics`PanelMethods`PMDoubletConstantDirichlet}, \
"\<\"DoubletConstantDirichlet\"\>"]}
            },
            BaselinePosition->{1, 1},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.]}, 
                Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}, "RowsIndexed" -> {}}],
           Deployed->True],
          SetterBar[
           Dynamic[$CellContext`model$$], {
           Aerodynamics`PanelMethods`PMVortexLinearNeumann -> 
            "VortexLinearNeumann", 
            Aerodynamics`PanelMethods`PMDoubletQuadraticDirichlet -> 
            "DoubletQuadraticDirichlet", 
            Aerodynamics`PanelMethods`PMDoubletConstantDirichlet -> 
            "DoubletConstantDirichlet"}]], "\[SpanFromLeft]", 
         "\[SpanFromLeft]", "\[SpanFromLeft]", "\[SpanFromLeft]"},
        {
         StyleBox["\<\"Output:\"\>", "Subsection",
          StripOnInput->False], "\[SpanFromLeft]", "\[SpanFromLeft]", 
         "\[SpanFromLeft]", "\[SpanFromLeft]", "\[SpanFromLeft]", 
         "\[SpanFromLeft]"},
        {"\<\"\"\>", "\<\"flow visualization\"\>", 
         InterpretationBox[
          StyleBox[GridBox[{
             {
              
              SetterBox[
               Dynamic[$CellContext`viz$$], {$CellContext`sp}, "\<\"stream \
plot\"\>"], 
              
              SetterBox[
               Dynamic[$CellContext`viz$$], {$CellContext`slp}, \
"\<\"streamline plot\"\>"], 
              SetterBox[Dynamic[$CellContext`viz$$], {$CellContext`sdp}, 
               TemplateBox[{"\"stream-\"",SubscriptBox["C", "p"],"\"plot\""},
                "Row",
                
                DisplayFunction->(
                 RowBox[{#, "\[InvisibleSpace]", #2, 
                   "\[InvisibleSpace]", #3}]& ),
                InterpretationFunction->(RowBox[{"Row", "[", 
                   RowBox[{"{", 
                    RowBox[{#, ",", #2, ",", #3}], "}"}], "]"}]& )]]}
            },
            BaselinePosition->{1, 1},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.]}, 
                Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}, "RowsIndexed" -> {}}],
           Deployed->True],
          SetterBar[
           
           Dynamic[$CellContext`viz$$], {$CellContext`sp -> 
            "stream plot", $CellContext`slp -> 
            "streamline plot", $CellContext`sdp -> Row[{"stream-", 
               Subscript[C, $CellContext`p], "plot"}]}]], "\[SpanFromLeft]", 
         "\[SpanFromLeft]", "\[SpanFromLeft]", "\[SpanFromLeft]"},
        {"\<\"\"\>", 
         TemplateBox[{
          "\"Aerodynamic properties are presented as graph of surface\"",
           InterpretationBox[
            Cell[
             BoxData[
              Cell[
               BoxData[
                FormBox[
                 SubscriptBox["C", "p"], TraditionalForm]]]]], 
            TextCell[
             RawBoxes[
              Cell[
               BoxData[
                FormBox[
                 SubscriptBox["C", "p"], TraditionalForm]]]]]],
           "\"with numerical values for\"",InterpretationBox[
            Cell[
             BoxData[
              Cell[
               BoxData[
                FormBox[
                 SubscriptBox["C", "\[ScriptL]"], TraditionalForm]]]]], 
            TextCell[
             RawBoxes[
              Cell[
               BoxData[
                FormBox[
                 SubscriptBox["C", "\[ScriptL]"], TraditionalForm]]]]]],
           "\"and\"",InterpretationBox[
            Cell[
             BoxData[
              Cell[
               BoxData[
                FormBox[
                 SubscriptBox["C", "m"], TraditionalForm]]]]], 
            TextCell[
             RawBoxes[
              Cell[
               BoxData[
                FormBox[
                 SubscriptBox["C", "m"], TraditionalForm]]]]]],"\".\""},
          "Row",
          
          DisplayFunction->(
           RowBox[{#, "\[InvisibleSpace]", #2, "\[InvisibleSpace]", #3, 
             "\[InvisibleSpace]", #4, "\[InvisibleSpace]", #5, 
             "\[InvisibleSpace]", #6, "\[InvisibleSpace]", #7}]& ),
          InterpretationFunction->(RowBox[{"Row", "[", 
             RowBox[{"{", 
               
               RowBox[{#, ",", #2, ",", #3, ",", #4, ",", #5, ",", #6, 
                 ",", #7}], "}"}], "]"}]& )], "\[SpanFromLeft]", 
         "\[SpanFromLeft]", "\[SpanFromLeft]", "\[SpanFromLeft]", 
         "\[SpanFromLeft]"},
        {"\<\"\"\>", 
         ButtonBox["\<\"reset\"\>",
          Appearance->"Palette",
          
          ButtonFunction:>($CellContext`\[Alpha]D$$ = 
            5.; $CellContext`filename$$ = "e205.dat"; $CellContext`id$$ = 
            4412; $CellContext`nEq$$ = 50; $CellContext`nFile$$ = 
            "data"; $CellContext`layout$$ = 
            Aerodynamics`AirfoilGeometry`HalfCosineSpacing; \
$CellContext`model$$ = 
            Aerodynamics`PanelMethods`PMVortexLinearNeumann; \
$CellContext`reset$$ -> False; $CellContext`source$$ = 
            1; $CellContext`viz$$ = $CellContext`sp; Null),
          Evaluator->Automatic,
          Method->"Preemptive"], "", "", "", "", ""}
       },
       AutoDelete->False,
       GridBoxAlignment->{
        "Columns" -> {Left, Right, Left, Center, Right, Right, Left}, 
         "Rows" -> {{Center}}, 
         "ItemsIndexed" -> {{1, 1} -> {Center, Center}, {-2, 2} -> {
            Left, Center}, {3, 3} -> {Right, Center}, {3, 6} -> {
            Left, Center}}},
       GridBoxDividers->{"Columns" -> {{None}}, "Rows" -> {{None}}},
       GridBoxItemSize->{
        "Columns" -> {0.1, 10, 7, 4, 4, 5, 15}, "Rows" -> {{Automatic}}}],
      "Grid"],
     ImageSize->All],
    Module[{$CellContext`xMin$ = -0.5, $CellContext`xMax$ = 
      1.5, $CellContext`zMin$ = -0.5, $CellContext`zMax$ = 
      0.5, $CellContext`\[CapitalDelta]$ = 
      0.07, $CellContext`a$, $CellContext`aAll$, $CellContext`aW$, \
$CellContext`b$, $CellContext`bAll$, $CellContext`c$, $CellContext`cAll$, \
$CellContext`cl$, $CellContext`cm$, $CellContext`colorFunction$, \
$CellContext`cp$, $CellContext`cpCMin$, $CellContext`cpS$, \
$CellContext`gAirfoil$, $CellContext`gSurfaceCp$, $CellContext`gViz$, \
$CellContext`icA$, $CellContext`ic\[Psi]A$, $CellContext`icB$, \
$CellContext`ic\[Psi]B$, $CellContext`icC$, $CellContext`ic\[Psi]C$, \
$CellContext`nP$, $CellContext`rAll$, $CellContext`rC$, $CellContext`rF$, \
$CellContext`rP$, $CellContext`rPC$, $CellContext`rTE$, $CellContext`tuples$, \
$CellContext`V$, $CellContext`VMagC$, $CellContext`V\[Infinity]$, \
$CellContext`xC$, $CellContext`xCenP$, $CellContext`xW$, $CellContext`x0$, \
$CellContext`z0$, $CellContext`\[Alpha]$, $CellContext`\[Psi]$, $CellContext`\
\[Psi]\[Infinity]$}, {$CellContext`\[Alpha]$, $CellContext`rF$, \
$CellContext`V\[Infinity]$} = {$CellContext`\[Alpha]D$$ 
        Degree, {$CellContext`x, $CellContext`z}, 
        Aerodynamics`InfluenceCoefficients`UniformFlow[
        1, $CellContext`\[Alpha]$]}; $CellContext`rP$ = Which[
        And[$CellContext`source$$ == 1, $CellContext`nFile$$ == "data"], 
        Aerodynamics`AirfoilGeometry`AirfoilFromDataFile[
         StringJoin[
         "http://www.ae.uiuc.edu/m-selig/ads/coord/", \
$CellContext`filename$$]], $CellContext`source$$ == 1, 
        Aerodynamics`AirfoilGeometry`AirfoilSurfacePoints[
         Aerodynamics`AirfoilGeometry`FitAirfoil[
          Aerodynamics`AirfoilGeometry`AirfoilFromDataFile[
           StringJoin[
           "http://www.ae.uiuc.edu/m-selig/ads/coord/", \
$CellContext`filename$$]]], 
         Aerodynamics`AirfoilGeometry`NDiscretizeUnitSegment[$CellContext`\
nFile$$, Aerodynamics`AirfoilGeometry`Layout -> $CellContext`layout$$]], 
        And[$CellContext`source$$ == 2, $CellContext`nFile$$ == "data"], 
        Aerodynamics`AirfoilGeometry`AirfoilFromDataFile[
         StringJoin[
         "Aerodynamics\\Airfoil Data\\", $CellContext`filename$$]], \
$CellContext`source$$ == 2, 
        Aerodynamics`AirfoilGeometry`AirfoilSurfacePoints[
         Aerodynamics`AirfoilGeometry`FitAirfoil[
          Aerodynamics`AirfoilGeometry`AirfoilFromDataFile[
           StringJoin[
           "Aerodynamics\\Airfoil Data\\", $CellContext`filename$$]]], 
         Aerodynamics`AirfoilGeometry`NDiscretizeUnitSegment[$CellContext`\
nFile$$, Aerodynamics`AirfoilGeometry`Layout -> $CellContext`layout$$]], \
$CellContext`source$$ == 3, 
        Aerodynamics`AirfoilGeometry`AirfoilSurfacePoints[
         Aerodynamics`AirfoilGeometry`NACA4DigitAirfoil[$CellContext`id$$, 
          
          Aerodynamics`AirfoilGeometry`NDiscretizeUnitSegment[$CellContext`\
nEq$$, Aerodynamics`AirfoilGeometry`Layout -> $CellContext`layout$$]]]]; \
$CellContext`nP$ = 
      Aerodynamics`CartesianVectors`NumberOfVectors[$CellContext`rP$] - 
       1; {$CellContext`cl$, $CellContext`cm$} = 
      Which[$CellContext`model$$ === 
        Aerodynamics`PanelMethods`PMVortexLinearNeumann, {$CellContext`a$, \
$CellContext`b$, $CellContext`rC$, $CellContext`VMagC$} = \
$CellContext`model$$[$CellContext`rP$, $CellContext`\[Alpha]$]; 
        Aerodynamics`PanelMethods`ForceAndMoment[
         "Vortex", $CellContext`a$, $CellContext`b$, $CellContext`rP$, \
$CellContext`\[Alpha]$, $CellContext`rC$], $CellContext`model$$ === 
        Aerodynamics`PanelMethods`PMDoubletQuadraticDirichlet, \
$CellContext`rTE$ = Mean[{
            First[$CellContext`rP$], 
            Last[$CellContext`rP$]}]; $CellContext`rPC$ = 
         ReplacePart[$CellContext`rP$, {
           1 -> $CellContext`rTE$, -1 -> $CellContext`rTE$}]; \
{$CellContext`a$, $CellContext`b$, $CellContext`c$, $CellContext`aW$, \
$CellContext`rC$, $CellContext`VMagC$} = 
         Aerodynamics`PanelMethods`PMDoubletQuadraticDirichlet[$CellContext`\
rPC$, $CellContext`\[Alpha]$]; $CellContext`xW$ = 
         ReplaceAll[Aerodynamics`PanelMethods`XWake, 
           Options[
           Aerodynamics`PanelMethods`PMDoubletQuadraticDirichlet, 
            Aerodynamics`PanelMethods`XWake]]; $CellContext`rAll$ = 
         Append[$CellContext`rP$, {$CellContext`xW$, 
            0.}]; {$CellContext`aAll$, $CellContext`bAll$, \
$CellContext`cAll$} = {
           Append[$CellContext`a$, $CellContext`aW$], 
           Append[$CellContext`b$, 0.], 
           Append[$CellContext`c$, 0.]}; 
        Aerodynamics`PanelMethods`ForceAndMoment[
         "Doublet", $CellContext`a$, $CellContext`b$, $CellContext`c$, \
$CellContext`rPC$, $CellContext`\[Alpha]$, $CellContext`rC$], \
$CellContext`model$$ === 
        Aerodynamics`PanelMethods`PMDoubletConstantDirichlet, \
$CellContext`rTE$ = Mean[{
            First[$CellContext`rP$], 
            Last[$CellContext`rP$]}]; $CellContext`rPC$ = 
         ReplacePart[$CellContext`rP$, {
           1 -> $CellContext`rTE$, -1 -> $CellContext`rTE$}]; \
{$CellContext`a$, $CellContext`aW$, $CellContext`rC$, $CellContext`VMagC$} = 
         Aerodynamics`PanelMethods`PMDoubletConstantDirichlet[$CellContext`\
rPC$, $CellContext`\[Alpha]$]; $CellContext`xW$ = 
         ReplaceAll[Aerodynamics`PanelMethods`XWake, 
           Options[
           Aerodynamics`PanelMethods`PMDoubletQuadraticDirichlet, 
            Aerodynamics`PanelMethods`XWake]]; $CellContext`rAll$ = 
         Append[$CellContext`rP$, {$CellContext`xW$, 0.}]; $CellContext`aAll$ = 
         Append[$CellContext`a$, $CellContext`aW$]; 
        Aerodynamics`PanelMethods`ForceAndMoment[
         "Doublet", $CellContext`a$, $CellContext`rPC$, \
$CellContext`\[Alpha]$, $CellContext`rC$]]; $CellContext`xC$ = Part[
        Aerodynamics`CartesianVectors`Components[$CellContext`rC$], 
        1]; $CellContext`cpS$ = 
      1 - $CellContext`VMagC$^2; $CellContext`xCenP$ = \
(-$CellContext`cm$)/$CellContext`cl$; $CellContext`gSurfaceCp$ = 
      Aerodynamics`Graphics`PlotSurfacePressureCoefficient[$CellContext`xC$, \
$CellContext`cpS$, Epilog -> {
          Inset[
           Panel[
            Grid[{{
               Row[{"\[Alpha] = ", 
                 NumberForm[$CellContext`\[Alpha]$/Degree, {3, 2}], 
                 "\[Degree]"}]}, {
               
               Row[{"\!\(\*SubscriptBox[\n StyleBox[\"C\",\n\
FontSlant->\"Italic\"], \"\[ScriptL]\"]\) = ", 
                 NumberForm[$CellContext`cl$, {3, 2}]}]}, {
               
               Row[{"\!\(\*\nStyleBox[SubscriptBox[\"C\", \"m\"],\n\
FontSlant->\"Italic\"]\) = ", 
                 NumberForm[$CellContext`cm$, {3, 3}]}]}, {
               
               Row[{"\!\(\*SubscriptBox[\n StyleBox[\"x\",\n\
FontSlant->\"Italic\"], \"cp\"]\) = ", 
                 NumberForm[$CellContext`xCenP$, {3, 3}]}]}}]], {
           Right, Top}, {Right, Top}], 
          Inset[
           Aerodynamics`Graphics`PlotAirfoil[$CellContext`rP$, Epilog -> {
              EdgeForm[Black], Yellow, 
              Disk[{$CellContext`xCenP$, 0}, 0.012]}, Frame -> False, 
            PlotRange -> {{0, 1}, Automatic}], {0, 0}, {0, 0}, 1]}]; 
     If[$CellContext`viz$$ === $CellContext`slp, $CellContext`\[Psi]\
\[Infinity]$ = 
        Aerodynamics`InfluenceCoefficients`UniformFlowPsi[$CellContext`rF$, \
$CellContext`\[Alpha]$]; 
       Which[$CellContext`model$$ === 
         Aerodynamics`PanelMethods`PMVortexLinearNeumann, {$CellContext`ic\
\[Psi]A$, $CellContext`ic\[Psi]B$} = 
          Aerodynamics`InfluenceCoefficients`ICPsiVortexLinear[$CellContext`\
rF$, $CellContext`rP$]; $CellContext`\[Psi]$ = \
$CellContext`\[Psi]\[Infinity]$ + 
           Dot[$CellContext`ic\[Psi]A$, $CellContext`a$] + 
           Dot[$CellContext`ic\[Psi]B$, $CellContext`b$], \
$CellContext`model$$ === 
         Aerodynamics`PanelMethods`PMDoubletQuadraticDirichlet, \
{$CellContext`ic\[Psi]A$, $CellContext`ic\[Psi]B$, $CellContext`ic\[Psi]C$} = 
          Aerodynamics`InfluenceCoefficients`ICPsiDoubletQuadratic[$\
CellContext`rF$, $CellContext`rAll$]; $CellContext`\[Psi]$ = $CellContext`\
\[Psi]\[Infinity]$ + Dot[$CellContext`ic\[Psi]A$, $CellContext`aAll$] + 
           Dot[$CellContext`ic\[Psi]B$, $CellContext`bAll$] + 
           Dot[$CellContext`ic\[Psi]C$, $CellContext`cAll$], \
$CellContext`model$$ === 
         Aerodynamics`PanelMethods`PMDoubletConstantDirichlet, \
$CellContext`ic\[Psi]A$ = 
          Aerodynamics`InfluenceCoefficients`ICPsiDoubletLinear[$CellContext`\
rF$, $CellContext`rAll$]; $CellContext`\[Psi]$ = $CellContext`\[Psi]\
\[Infinity]$ + 
           Dot[$CellContext`ic\[Psi]A$, $CellContext`aAll$]]; \
$CellContext`gViz$ = 
        ContourPlot[$CellContext`\[Psi]$, {$CellContext`x, \
$CellContext`xMin$, $CellContext`xMax$}, {$CellContext`z, $CellContext`zMin$, \
$CellContext`zMax$}, AspectRatio -> Automatic, Contours -> 16, ContourShading -> 
          None, FrameLabel -> {"x", "z"}, MaxRecursion -> 3, PerformanceGoal -> 
          "Speed", RotateLabel -> False], $CellContext`V\[Infinity]$ = 
        Aerodynamics`InfluenceCoefficients`UniformFlow[
         1, $CellContext`\[Alpha]$]; $CellContext`z0$ = 
        Range[$CellContext`zMin$, $CellContext`zMax$, $CellContext`\
\[CapitalDelta]$]; $CellContext`x0$ = 
        Table[$CellContext`xMin$ + $CellContext`\[CapitalDelta]$, {
           Length[$CellContext`z0$]}]; $CellContext`tuples$ = 
        Transpose[{$CellContext`x0$, $CellContext`z0$}]; 
       Which[$CellContext`model$$ === 
         Aerodynamics`PanelMethods`PMVortexLinearNeumann, {$CellContext`icA$, \
$CellContext`icB$} = 
          Aerodynamics`InfluenceCoefficients`ICVortexLinear[$CellContext`rF$, \
$CellContext`rP$]; $CellContext`V$ = $CellContext`V\[Infinity]$ + 
           Dot[$CellContext`icA$, $CellContext`a$] + 
           Dot[$CellContext`icB$, $CellContext`b$], $CellContext`model$$ === 
         Aerodynamics`PanelMethods`PMDoubletQuadraticDirichlet, \
{$CellContext`icA$, $CellContext`icB$, $CellContext`icC$} = 
          Aerodynamics`InfluenceCoefficients`ICDoubletQuadratic[$CellContext`\
rF$, $CellContext`rAll$]; $CellContext`V$ = $CellContext`V\[Infinity]$ + 
           Dot[$CellContext`icA$, $CellContext`aAll$] + 
           Dot[$CellContext`icB$, $CellContext`bAll$] + 
           Dot[$CellContext`icC$, $CellContext`cAll$], $CellContext`model$$ === 
         Aerodynamics`PanelMethods`PMDoubletConstantDirichlet, \
$CellContext`icA$ = 
          Aerodynamics`InfluenceCoefficients`ICDoubletConstant[$CellContext`\
rF$, $CellContext`rAll$]; $CellContext`V$ = $CellContext`V\[Infinity]$ + 
           Dot[$CellContext`icA$, $CellContext`aAll$]]; 
       Which[$CellContext`viz$$ === $CellContext`sp, $CellContext`gViz$ = 
         StreamPlot[$CellContext`V$, {$CellContext`x, $CellContext`xMin$, \
$CellContext`xMax$}, {$CellContext`z, $CellContext`zMin$, $CellContext`zMax$},
            AspectRatio -> Automatic, FrameLabel -> {"x", "z"}, 
           StreamPoints -> {$CellContext`tuples$}, RotateLabel -> False, 
           StreamScale -> {
            Full, All, 
             0.015}], $CellContext`viz$$ === $CellContext`sdp, \
$CellContext`cpCMin$ = Min[$CellContext`cpS$]; $CellContext`colorFunction$[
            Pattern[$CellContext`c, 
             Blank[]], 
            Pattern[$CellContext`cMin, 
             Blank[]]] := Hue[
            If[$CellContext`c >= 0, 0.0483667, 0.633338], 
            
            If[$CellContext`c >= 
             0, $CellContext`c, $CellContext`c/$CellContext`cMin], 
            Which[
             
             Inequality[$CellContext`cMin, LessEqual, $CellContext`c, 
              Less, -1], 0.6472, 
             Inequality[-1, LessEqual, $CellContext`c, Less, 0], 1 + 
             0.3528 $CellContext`c, $CellContext`c >= 0, 
             1]]; $CellContext`cp$ = 
          1 - Dot[$CellContext`V$, $CellContext`V$]; $CellContext`gViz$ = 
          StreamDensityPlot[{$CellContext`V$, $CellContext`cp$}, \
{$CellContext`x, $CellContext`xMin$, $CellContext`xMax$}, {$CellContext`z, \
$CellContext`zMin$, $CellContext`zMax$}, AspectRatio -> Automatic, 
            ColorFunction -> ($CellContext`colorFunction$[#, \
$CellContext`cpCMin$]& ), ColorFunctionScaling -> False, 
            FrameLabel -> {"x", "z"}, MaxRecursion -> 6, RotateLabel -> False,
             StreamPoints -> {$CellContext`tuples$}, 
            StreamScale -> {Full, All, 0.015}]]; 
       Null]; $CellContext`gAirfoil$ = 
      Aerodynamics`Graphics`PlotAirfoil[$CellContext`rP$]; \
$CellContext`graphs = GraphicsColumn[{
         Show[$CellContext`gViz$, $CellContext`gAirfoil$], \
$CellContext`gSurfaceCp$}]; CreateDocument[{
        Which[$CellContext`source$$ == 1, 
         TextCell[
          StringJoin[
          "Potential Flow Over the Airfoil Described by the UIUC File ", \
$CellContext`filename$$], "Section"], $CellContext`source$$ == 2, 
         TextCell[
          StringJoin[
          "Potential Flow Over the Airfoil Described by the Data File ", \
$CellContext`filename$$], "Section"], $CellContext`source$$ == 3, 
         TextCell[
          StringJoin["Potential Flow Over a NACA-", 
           ToString[$CellContext`id$$], " Airfoil"], "Section"]], 
        TextCell[
         StringJoin["Model: ", 
          ToString[$CellContext`model$$]]], 
        If[
         And[
          
          Or[$CellContext`source$$ == 1, $CellContext`source$$ == 
           2], $CellContext`nFile$$ == "data"], 
         TextCell[
          StringJoin["Discretization: ", 
           ToString[$CellContext`nP$], " panels from data file."]], 
         TextCell[
          StringJoin["Discretization: ", 
           ToString[$CellContext`nP$], " panels in ", 
           ToString[$CellContext`layout$$], 
           " layout"]]], $CellContext`graphs}]; Null]],
   DynamicModuleValues:>{}],
  Setting[#, {0}]& ]], "Output",
 CellChangeTimes->{3.5014131000727997`*^9}]
}, Open  ]]
}, Open  ]]
},
WindowSize->{711, 907},
WindowMargins->{{15, Automatic}, {Automatic, -7}},
ShowSelection->True,
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->"Default.nb",
PrivateNotebookOptions -> {"NotebookAuthor" -> ""}
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 267, 3, 71, "Section"],
Cell[849, 27, 126, 3, 29, "Text"],
Cell[978, 32, 124, 2, 31, "Input"],
Cell[CellGroupData[{
Cell[1127, 38, 71003, 1292, 3874, "Input"],
Cell[72133, 1332, 25806, 542, 313, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
