(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[      9787,        336]
NotebookOptionsPosition[      7006,        251]
NotebookOutlinePosition[      8177,        282]
CellTagsIndexPosition[      8134,        279]
WindowTitle->AirfoilGeometry - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "GuideColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[CellGroupData[{

Cell["AirfoilGeometry", "GuideTitle",
 CellID->942062912],

Cell["\<\
This package provides functions for discretizing airfoil geometry in \
preparation for using computational models to compute flow over the airfoil.\
\>", "GuideAbstract",
 CellID->2001916300],

Cell[TextData[{
 Cell[TextData[ButtonBox["AirfoilFromDataFile",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/AirfoilFromDataFile"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " imports airfoil data and casts it in form used in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"]
}], "GuideText",
 CellID->203374175],

Cell[TextData[{
 Cell[TextData[ButtonBox["AirfoilInteriorQ",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/AirfoilInteriorQ"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " is a point interior to a discretized airfoil?"
}], "GuideText",
 CellID->240184747],

Cell[TextData[{
 Cell[TextData[ButtonBox["AirfoilSurfacePoints",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/AirfoilSurfacePoints"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " collection of vectors locating points on airfoil surface"
}], "GuideText",
 CellID->612614724],

Cell[TextData[{
 Cell[TextData[ButtonBox["FitAirfoil",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/FitAirfoil"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " fits a B-spline to points specifying an airfoil"
}], "GuideText",
 CellID->31839155],

Cell[TextData[{
 Cell[TextData[ButtonBox["NACA4DigitAirfoil",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/NACA4DigitAirfoil"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " thickness and camber properties of NACA four-digit airfoil"
}], "GuideText",
 CellID->16749258],

Cell[TextData[{
 Cell[TextData[ButtonBox["NDiscretizeUnitSegment",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/NDiscretizeUnitSegment"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values used for discretization of airfoil"
}], "GuideText",
 CellID->56389064],

Cell[TextData[{
 Cell[TextData[ButtonBox["PanelAngles",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PanelAngles"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " angles of panels describing discretized airfoil with ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis"
}], "GuideText",
 CellID->5001441],

Cell[TextData[{
 Cell[TextData[ButtonBox["PanelLengths",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PanelLengths"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " lengths of panels describing discretized airfoil"
}], "GuideText",
 CellID->146539965],

Cell[TextData[{
 Cell[TextData[ButtonBox["PanelNormals",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PanelNormals"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " collection of unit normal vectors to panels describing discretized airfoil"
}], "GuideText",
 CellID->43477983],

Cell[TextData[{
 Cell[TextData[ButtonBox["PanelPoints",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PanelPoints"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " collection of vectors locating points on panels describing discretized \
airfoil"
}], "GuideText",
 CellID->396586081],

Cell[TextData[{
 Cell[TextData[ButtonBox["ThickAirfoilQ",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ThickAirfoilQ"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " does airfoil have thickness?"
}], "GuideText",
 CellID->312758],

Cell[TextData[{
 Cell[TextData[ButtonBox["TrailingEdgeAngle",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/TrailingEdgeAngle"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " angle between trailing-edge panels"
}], "GuideText",
 CellID->166696838],

Cell[TextData[{
 Cell[TextData[ButtonBox["XToTheta",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/XToTheta"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " converts ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values to ",
 Cell[BoxData[
  StyleBox["\[Theta]", "TR"]], "InlineFormula"],
 " values using van de Vooren transformation"
}], "GuideText",
 CellID->95899216]
}, Open  ]],

Cell[CellGroupData[{

Cell["TUTORIALS", "GuideTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["1.1 Introduction To Airfoil Geometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToAirfoilGeometry"]], \
"GuideTutorial",
 CellID->530691785],

Cell[TextData[ButtonBox["1.2 Airfoils From Data Files",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromDataFiles"]], "GuideTutorial",
 CellID->189364675],

Cell[TextData[ButtonBox["1.3 Airfoils From Thickness And Camber Properties",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromThicknessAndCamberProperties"]], \
"GuideTutorial",
 CellID->806871991],

Cell[TextData[ButtonBox["1.4 Airfoils From Conformal Mapping",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromConformalMapping"]], \
"GuideTutorial",
 CellID->86999773],

Cell[TextData[ButtonBox["1.5 Panel Geometry",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/PanelGeometry"]], "GuideTutorial",
 CellID->858207573],

Cell[TextData[ButtonBox["Overview",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]], "GuideTutorial",
 CellID->64701022]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "GuideMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["Aerodynamics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"]], "GuideMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"AirfoilGeometry - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 9, 57, 52.5156000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Guide", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "None", "summary" -> 
    "This package provides functions for discretizing airfoil geometry in \
preparation for using computational models to compute flow over the airfoil.",
     "synonyms" -> {}, "title" -> "AirfoilGeometry", "type" -> "Guide", "uri" -> 
    "Aerodynamics/guide/AirfoilGeometry"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[608, 21, 75, 1, 70, "GuideColorBar"],
Cell[686, 24, 191, 5, 70, "LinkTrail"],
Cell[CellGroupData[{
Cell[902, 33, 57, 1, 70, "GuideTitle",
 CellID->942062912],
Cell[962, 36, 201, 4, 70, "GuideAbstract",
 CellID->2001916300],
Cell[1166, 42, 362, 11, 70, "GuideText",
 CellID->203374175],
Cell[1531, 55, 301, 9, 70, "GuideText",
 CellID->240184747],
Cell[1835, 66, 320, 9, 70, "GuideText",
 CellID->612614724],
Cell[2158, 77, 287, 8, 70, "GuideText",
 CellID->31839155],
Cell[2448, 87, 315, 9, 70, "GuideText",
 CellID->16749258],
Cell[2766, 98, 379, 12, 70, "GuideText",
 CellID->56389064],
Cell[3148, 112, 361, 11, 70, "GuideText",
 CellID->5001441],
Cell[3512, 125, 293, 8, 70, "GuideText",
 CellID->146539965],
Cell[3808, 135, 318, 8, 70, "GuideText",
 CellID->43477983],
Cell[4129, 145, 323, 9, 70, "GuideText",
 CellID->396586081],
Cell[4455, 156, 272, 8, 70, "GuideText",
 CellID->312758],
Cell[4730, 166, 292, 9, 70, "GuideText",
 CellID->166696838],
Cell[5025, 177, 429, 14, 70, "GuideText",
 CellID->95899216]
}, Open  ]],
Cell[CellGroupData[{
Cell[5491, 196, 62, 1, 70, "GuideTutorialsSection",
 CellID->415694126],
Cell[5556, 199, 202, 5, 70, "GuideTutorial",
 CellID->530691785],
Cell[5761, 206, 184, 4, 70, "GuideTutorial",
 CellID->189364675],
Cell[5948, 212, 226, 5, 70, "GuideTutorial",
 CellID->806871991],
Cell[6177, 219, 199, 5, 70, "GuideTutorial",
 CellID->86999773],
Cell[6379, 226, 163, 3, 70, "GuideTutorial",
 CellID->858207573],
Cell[6545, 231, 162, 4, 70, "GuideTutorial",
 CellID->64701022]
}, Open  ]],
Cell[CellGroupData[{
Cell[6744, 240, 62, 1, 70, "GuideMoreAboutSection",
 CellID->23220180],
Cell[6809, 243, 155, 3, 70, "GuideMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[6979, 249, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

