(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     11618,        404]
NotebookOptionsPosition[      8024,        294]
NotebookOutlinePosition[      9284,        326]
CellTagsIndexPosition[      9241,        323]
WindowTitle->CartesianVectors - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "GuideColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[CellGroupData[{

Cell["CartesianVectors", "GuideTitle",
 CellID->942062912],

Cell[TextData[{
 "CartesianVectors is the name of the data type defined to describe \
collections of vectors in the  ",
 StyleBox["Aerodynamics ",
  FontSlant->"Italic"],
 "software.  You should be able to use the numerical models for airfoil \
aerodynamics without being familiar with this package."
}], "GuideAbstract",
 CellID->2001916300],

Cell["\<\
The primary reasons for introducing the data type CartesianVectors are \
convenience of pattern matching for input arguments for functions defined in \
the other packages, and for simplifying vector algebra in function \
definitions. An understanding of the data type will be helpful for reading or \
altering function definitions in other packages.  \
\>", "GuideAbstract",
 CellID->767589011],

Cell[TextData[{
 "Most properties of the data type are defined by overloading ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions and are obvious extensions of the arithmetic and list operation \
used by ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ".  The appendix, A1 Data Type CartesianVectors, includes examples of using \
several overloaded ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions."
}], "GuideAbstract",
 CellID->247703532],

Cell[CellGroupData[{

Cell["Constructor and Selector", "GuideFunctionsSubsection",
 CellID->64787135],

Cell[TextData[{
 Cell[TextData[ButtonBox["MakeCartesianVectors",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/MakeCartesianVectors"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " constructs data type CartesianVectors from list of components"
}], "GuideText",
 CellID->305513954],

Cell[TextData[{
 Cell[TextData[ButtonBox["Components",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/Components"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " selects the list of components from the data type CartesianVectors"
}], "GuideText",
 CellID->78280888],

Cell["\t", "GuideDelimiterSubsection",
 CellID->130491264]
}, Open  ]],

Cell[CellGroupData[{

Cell["Exported Functions, General", "GuideFunctionsSubsection",
 CellID->792128480],

Cell[TextData[{
 Cell[TextData[ButtonBox["MultiplyByList",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/MultiplyByList"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " multiplies collection of vectors by a list"
}], "GuideText",
 CellID->638282084],

Cell[TextData[{
 Cell[TextData[ButtonBox["NumberOfVectors",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/NumberOfVectors"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " counts number of vectors in collection"
}], "GuideText",
 CellID->177633201],

Cell[TextData[{
 Cell[TextData[ButtonBox["ReplicateVector",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ReplicateVector"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " makes collection of vectors consisting of copies of input vector"
}], "GuideText",
 CellID->108018996],

Cell["\t", "GuideDelimiterSubsection",
 CellID->669805398]
}, Open  ]],

Cell[CellGroupData[{

Cell["Exported Functions, 3-D Vectors", "GuideFunctionsSubsection",
 CellID->280113763],

Cell[TextData[{
 Cell[TextData[ButtonBox["ThreeDimQ",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ThreeDimQ"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " tests collection of vectors for being three dimensional"
}], "GuideText",
 CellID->422546481],

Cell["\t", "GuideDelimiterSubsection",
 CellID->150899699]
}, Open  ]],

Cell[CellGroupData[{

Cell["Exported Functions, 2-D Vectors", "GuideFunctionsSubsection",
 CellID->431151480],

Cell[TextData[{
 Cell[TextData[ButtonBox["ComplexToCartesianVectors",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ComplexToCartesianVectors"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " converts list of complex numbers to collection of vectors"
}], "GuideText",
 CellID->564851738],

Cell[TextData[{
 Cell[TextData[ButtonBox["ExtendXyToXyz",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ExtendXyToXyz"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " extends 2-D vectors to 3-D vectors by padding ",
 Cell[BoxData[
  StyleBox["z", "TI"]], "InlineFormula"]
}], "GuideText",
 CellID->434658853],

Cell[TextData[{
 Cell[TextData[ButtonBox["ExtendXzToXyz",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ExtendXzToXyz"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " extends 2-D vectors to 3-D vectors by padding ",
 Cell[BoxData[
  StyleBox["y", "TI"]], "InlineFormula"]
}], "GuideText",
 CellID->66601704],

Cell[TextData[{
 Cell[TextData[ButtonBox["MirrorXAxis",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/MirrorXAxis"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " constructs mirror image through ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]], "InlineMath"],
 "-axis of collection of vectors"
}], "GuideText",
 CellID->201642374],

Cell[TextData[{
 Cell[TextData[ButtonBox["TwoDimQ",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/TwoDimQ"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " tests collection of vectors for being two dimensional"
}], "GuideText",
 CellID->22123132],

Cell["\t", "GuideDelimiterSubsection",
 CellID->21302215]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Overloaded ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Functions with Separate Symbol Pages"
}], "GuideFunctionsSubsection",
 CellID->964251101],

Cell[TextData[{
 Cell[TextData[ButtonBox["Dot",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/Dot"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " ",
 "element-by-element dot products of two collections of vectors"
}], "GuideText",
 CellID->949222565],

Cell[TextData[{
 Cell[TextData[ButtonBox["Cross",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/Cross"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " ",
 "element-by-element cross products of two collections of vectors"
}], "GuideText",
 CellID->158758426]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TUTORIALS", "GuideTutorialsSection",
 CellID->40747645],

Cell[TextData[ButtonBox["A1 Data Type CartesianVectors",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/CartesianVectors"]], "GuideTutorial",
 CellID->137824362],

Cell[TextData[ButtonBox["Overview",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]], "GuideTutorial",
 CellID->247482623]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "GuideMoreAboutSection",
 CellID->483393488],

Cell[TextData[ButtonBox["Aerodynamics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"]], "GuideMoreAbout",
 CellID->422375765]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"CartesianVectors - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 9, 57, 55.3392000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Guide", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "None", "summary" -> 
    "CartesianVectors is the name of the data type defined to describe \
collections of vectors in the Aerodynamics software. You should be able to \
use the numerical models for airfoil aerodynamics without being familiar with \
this package.", "synonyms" -> {}, "title" -> "CartesianVectors", "type" -> 
    "Guide", "uri" -> "Aerodynamics/guide/CartesianVectors"}, "LinkTrails" -> 
  "", "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[609, 21, 75, 1, 70, "GuideColorBar"],
Cell[687, 24, 191, 5, 70, "LinkTrail"],
Cell[CellGroupData[{
Cell[903, 33, 58, 1, 70, "GuideTitle",
 CellID->942062912],
Cell[964, 36, 342, 8, 70, "GuideAbstract",
 CellID->2001916300],
Cell[1309, 46, 404, 7, 70, "GuideAbstract",
 CellID->767589011],
Cell[1716, 55, 474, 14, 70, "GuideAbstract",
 CellID->247703532],
Cell[CellGroupData[{
Cell[2215, 73, 79, 1, 70, "GuideFunctionsSubsection",
 CellID->64787135],
Cell[2297, 76, 325, 9, 70, "GuideText",
 CellID->305513954],
Cell[2625, 87, 306, 8, 70, "GuideText",
 CellID->78280888],
Cell[2934, 97, 58, 1, 70, "GuideDelimiterSubsection",
 CellID->130491264]
}, Open  ]],
Cell[CellGroupData[{
Cell[3029, 103, 83, 1, 70, "GuideFunctionsSubsection",
 CellID->792128480],
Cell[3115, 106, 294, 9, 70, "GuideText",
 CellID->638282084],
Cell[3412, 117, 292, 9, 70, "GuideText",
 CellID->177633201],
Cell[3707, 128, 318, 9, 70, "GuideText",
 CellID->108018996],
Cell[4028, 139, 58, 1, 70, "GuideDelimiterSubsection",
 CellID->669805398]
}, Open  ]],
Cell[CellGroupData[{
Cell[4123, 145, 87, 1, 70, "GuideFunctionsSubsection",
 CellID->280113763],
Cell[4213, 148, 294, 8, 70, "GuideText",
 CellID->422546481],
Cell[4510, 158, 58, 1, 70, "GuideDelimiterSubsection",
 CellID->150899699]
}, Open  ]],
Cell[CellGroupData[{
Cell[4605, 164, 87, 1, 70, "GuideFunctionsSubsection",
 CellID->431151480],
Cell[4695, 167, 331, 9, 70, "GuideText",
 CellID->564851738],
Cell[5029, 178, 350, 10, 70, "GuideText",
 CellID->434658853],
Cell[5382, 190, 349, 10, 70, "GuideText",
 CellID->66601704],
Cell[5734, 202, 374, 11, 70, "GuideText",
 CellID->201642374],
Cell[6111, 215, 287, 8, 70, "GuideText",
 CellID->22123132],
Cell[6401, 225, 57, 1, 70, "GuideDelimiterSubsection",
 CellID->21302215]
}, Open  ]],
Cell[CellGroupData[{
Cell[6495, 231, 173, 6, 70, "GuideFunctionsSubsection",
 CellID->964251101],
Cell[6671, 239, 293, 9, 70, "GuideText",
 CellID->949222565],
Cell[6967, 250, 299, 9, 70, "GuideText",
 CellID->158758426]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[7315, 265, 61, 1, 70, "GuideTutorialsSection",
 CellID->40747645],
Cell[7379, 268, 180, 4, 70, "GuideTutorial",
 CellID->137824362],
Cell[7562, 274, 163, 4, 70, "GuideTutorial",
 CellID->247482623]
}, Open  ]],
Cell[CellGroupData[{
Cell[7762, 283, 63, 1, 70, "GuideMoreAboutSection",
 CellID->483393488],
Cell[7828, 286, 154, 3, 70, "GuideMoreAbout",
 CellID->422375765]
}, Open  ]],
Cell[7997, 292, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

