(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[      8086,        281]
NotebookOptionsPosition[      5513,        204]
NotebookOutlinePosition[      6794,        237]
CellTagsIndexPosition[      6751,        234]
WindowTitle->InfluenceCoefficients - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "GuideColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[CellGroupData[{

Cell["InfluenceCoefficients", "GuideTitle",
 CellID->942062912],

Cell["\<\
This package contains functions for the commonly used influence coefficients \
for two-dimensional potential flow.  For each singularity, functions for \
velocity, velocity-potential and  stream-function influence coefficients are \
defined.\
\>", "GuideAbstract",
 CellID->2001916300],

Cell[TextData[{
 Cell[TextData[ButtonBox["DisplacementVectors",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/DisplacementVectors"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " displacement vectors from singularities to field points "
}], "GuideText",
 CellID->461542648],

Cell[TextData[{
 Cell[TextData[ButtonBox["UniformFlow",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/UniformFlow"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " velocity, velocity potential and stream function for uniform flow"
}], "GuideText",
 CellID->411125827],

Cell[TextData[{
 Cell[TextData[ButtonBox["ICSourcePoint",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICSourcePoint"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " point sources"
}], "GuideText",
 CellID->133394187],

Cell[TextData[{
 Cell[TextData[ButtonBox["ICVortexPoint",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICVortexPoint"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " point vortices"
}], "GuideText",
 CellID->254624606],

Cell[TextData[{
 Cell[TextData[ButtonBox["ICDoubletPoint",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletPoint"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " point doublets"
}], "GuideText",
 CellID->18652149],

Cell[TextData[{
 Cell[TextData[ButtonBox["ICSourceConstant",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICSourceConstant"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " source panels of constant strength"
}], "GuideText",
 CellID->13136292],

Cell[TextData[{
 Cell[TextData[ButtonBox["ICVortexConstant",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICVortexConstant"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " vortex panels of constant strength"
}], "GuideText",
 CellID->175155773],

Cell[TextData[{
 Cell[TextData[ButtonBox["ICDoubletConstant",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletConstant"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " doublet panels of constant strength"
}], "GuideText",
 CellID->353611842],

Cell[TextData[{
 Cell[TextData[ButtonBox["ICSourceLinear",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICSourceLinear"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " source panels of linearly varying strength"
}], "GuideText",
 CellID->383210361],

Cell[TextData[{
 Cell[TextData[ButtonBox["ICVortexLinear",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICVortexLinear"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " vortex panels of linearly varying strength"
}], "GuideText",
 CellID->632449638],

Cell[TextData[{
 Cell[TextData[ButtonBox["ICDoubletLinear",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletLinear"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " doublet panels of linearly varying strength"
}], "GuideText",
 CellID->126639774],

Cell[TextData[{
 Cell[TextData[ButtonBox["ICDoubletQuadratic",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletQuadratic"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " doublet panels of quadratically varying strength"
}], "GuideText",
 CellID->1463276848]
}, Open  ]],

Cell[CellGroupData[{

Cell["TUTORIALS", "GuideTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["2.3 IsolatedPanel",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/IsolatedPanel"]], "GuideTutorial",
 CellID->806871991],

Cell[TextData[ButtonBox["Overview",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]], "GuideTutorial",
 CellID->1885805579]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "GuideMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["Aerodynamics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"]], "GuideMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"InfluenceCoefficients - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 9, 57, 59.7384000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Guide", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "None", "summary" -> 
    "This package contains functions for the commonly used influence \
coefficients for two-dimensional potential flow. For each singularity, \
functions for velocity, velocity-potential and stream-function influence \
coefficients are defined.", "synonyms" -> {}, "title" -> 
    "InfluenceCoefficients", "type" -> "Guide", "uri" -> 
    "Aerodynamics/guide/InfluenceCoefficients"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[614, 21, 75, 1, 70, "GuideColorBar"],
Cell[692, 24, 191, 5, 70, "LinkTrail"],
Cell[CellGroupData[{
Cell[908, 33, 63, 1, 70, "GuideTitle",
 CellID->942062912],
Cell[974, 36, 295, 6, 70, "GuideAbstract",
 CellID->2001916300],
Cell[1272, 44, 318, 9, 70, "GuideText",
 CellID->461542648],
Cell[1593, 55, 308, 8, 70, "GuideText",
 CellID->411125827],
Cell[1904, 65, 260, 8, 70, "GuideText",
 CellID->133394187],
Cell[2167, 75, 261, 8, 70, "GuideText",
 CellID->254624606],
Cell[2431, 85, 265, 9, 70, "GuideText",
 CellID->18652149],
Cell[2699, 96, 289, 9, 70, "GuideText",
 CellID->13136292],
Cell[2991, 107, 290, 9, 70, "GuideText",
 CellID->175155773],
Cell[3284, 118, 293, 9, 70, "GuideText",
 CellID->353611842],
Cell[3580, 129, 294, 9, 70, "GuideText",
 CellID->383210361],
Cell[3877, 140, 294, 9, 70, "GuideText",
 CellID->632449638],
Cell[4174, 151, 297, 9, 70, "GuideText",
 CellID->126639774],
Cell[4474, 162, 309, 9, 70, "GuideText",
 CellID->1463276848]
}, Open  ]],
Cell[CellGroupData[{
Cell[4820, 176, 62, 1, 70, "GuideTutorialsSection",
 CellID->415694126],
Cell[4885, 179, 162, 3, 70, "GuideTutorial",
 CellID->806871991],
Cell[5050, 184, 164, 4, 70, "GuideTutorial",
 CellID->1885805579]
}, Open  ]],
Cell[CellGroupData[{
Cell[5251, 193, 62, 1, 70, "GuideMoreAboutSection",
 CellID->23220180],
Cell[5316, 196, 155, 3, 70, "GuideMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[5486, 202, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

