(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     10499,        323]
NotebookOptionsPosition[      7895,        244]
NotebookOutlinePosition[      9443,        285]
CellTagsIndexPosition[      9400,        282]
WindowTitle->Aerodynamics - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Overview\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/Aerodynamics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/Aerodynamics\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/Aerodynamics"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
Aerodynamics.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/Aerodynamics", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["Aerodynamics", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData["Aerodynamics"], "InlineFormula"],
     "\[LineSeparator]is a collection of five packages providing functions to \
implement numerical models for two-dimensional steady incompressible \
potential flow over an airfoil using panel methods."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->501836004],

Cell[TextData[{
 "There are five packages included in the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " software."
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], "InlineFormula"],
 " defines a data type for specifying collections of vectors in Cartesian \
coordinates.  This data type facilitates vector operations involving \
collections of vectors and provides convenient pattern matching for input \
arguments in defined functions used in the following packages."
}], "Notes",
 CellID->87114273],

Cell[TextData[{
 Cell[BoxData[
  ButtonBox["AirfoilGeometry",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilGeometry"]], "InlineFormula"],
 " provides functions for specifying airfoil shape, discretizing the airfoil \
and computing geometric properties of the discretized airfoil."
}], "Notes",
 CellID->155588193],

Cell[TextData[{
 Cell[BoxData[
  ButtonBox["InfluenceCoefficients",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/InfluenceCoefficients"]], 
  "InlineFormula"],
 " provides a collection of functions for velocity, velocity-potential and \
stream-function influence coefficients for commonly used two-dimensional \
potential-flow point and line singularities."
}], "Notes",
 CellID->214136407],

Cell[TextData[{
 ButtonBox["PanelMethods",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PanelMethods"],
 " provides functions for the implementation of panel methods."
}], "Notes",
 CellID->36831175],

Cell[TextData[{
 ButtonBox["Graphics",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/Graphics"],
 " provides functions for plotting some results such as airfoil shape and \
surface pressure coefficient in conventional formats used by the aerodynamics \
community."
}], "Notes",
 CellID->46191181],

Cell["New in 1.1", "Notes",
 CellID->529540534]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["Overview",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]], "Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["Aerodynamics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"]], "MoreAbout",
 CellID->1665078683]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Aerodynamics - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 9, 58, 5.2608000}", "context" -> 
    "Aerodynamics`Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Package Paclet Symbol", "language" -> "en", "paclet" -> 
    "Aerodynamics Package", "status" -> "None", "summary" -> 
    "Aerodynamics is a collection of five packages providing functions to \
implement numerical models for two-dimensional steady incompressible \
potential flow over an airfoil using panel methods.", "synonyms" -> {}, 
    "title" -> "Aerodynamics", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/Aerodynamics"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[605, 21, 76, 1, 70, "SymbolColorBar"],
Cell[684, 24, 191, 5, 70, "LinkTrail"],
Cell[878, 31, 2607, 60, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3510, 95, 55, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[3568, 98, 465, 11, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4070, 114, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->501836004],
Cell[4653, 133, 158, 6, 70, "Notes",
 CellID->1067943069],
Cell[4814, 141, 482, 10, 70, "Notes",
 CellID->87114273],
Cell[5299, 153, 337, 8, 70, "Notes",
 CellID->155588193],
Cell[5639, 163, 407, 10, 70, "Notes",
 CellID->214136407],
Cell[6049, 175, 214, 6, 70, "Notes",
 CellID->36831175],
Cell[6266, 183, 310, 8, 70, "Notes",
 CellID->46191181],
Cell[6579, 193, 47, 1, 70, "Notes",
 CellID->529540534]
}, Closed]],
Cell[CellGroupData[{
Cell[6663, 199, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[7085, 213, 159, 4, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[CellGroupData[{
Cell[7281, 222, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[7703, 236, 150, 3, 70, "MoreAbout",
 CellID->1665078683]
}, Open  ]],
Cell[7868, 242, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

