(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[      9854,        292]
NotebookOptionsPosition[      7373,        218]
NotebookOutlinePosition[      8898,        258]
CellTagsIndexPosition[      8855,        255]
WindowTitle->AirfoilGeometry - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"1.1 Introduction to Airfoil Geometry\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToAirfoilGeometry"], \
"\<\"1.2 Airfoils from Data Files\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AirfoilsFromDataFiles"], "\<\"1.3 \
Airfoils from Thickness and Camber Properties\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/\
AirfoilsFromThicknessAndCamberProperties"], "\<\"1.4 Airfoils from Conformal \
Mapping\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AirfoilsFromConformalMapping"], "\<\"1.5 \
Panel Geometry\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/PanelGeometry"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"AirfoilGeometry\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/AirfoilGeometry"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/AirfoilGeometry\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/AirfoilGeometry"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
AirfoilGeometry.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/AirfoilGeometry", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["AirfoilGeometry", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData["AirfoilGeometry"], "InlineFormula"],
     "\[LineSeparator]is a package providing functions to specify airfoil \
shape, discretize the airfoil and compute geometric properties of the \
discretized airfoil."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->322609720],

Cell["New in 1.1", "Notes",
 CellID->529540534]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["1.1 Introduction to Airfoil Geometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToAirfoilGeometry"]], "Tutorials",
 CellID->341631938],

Cell[TextData[ButtonBox["1.2 Airfoils from Data Files",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromDataFiles"]], "Tutorials",
 CellID->187059247],

Cell[TextData[ButtonBox["1.3 Airfoils from Thickness and Camber Properties",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromThicknessAndCamberProperties"]], \
"Tutorials",
 CellID->284174586],

Cell[TextData[ButtonBox["1.4 Airfoils from Conformal Mapping",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromConformalMapping"]], "Tutorials",
 CellID->47973038],

Cell[TextData[ButtonBox["1.5 Panel Geometry",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/PanelGeometry"]], "Tutorials",
 CellID->23098279]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["AirfoilGeometry",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/AirfoilGeometry"]], "MoreAbout",
 CellID->1665078683]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"AirfoilGeometry - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 9, 58, 13.9812000}", "context" -> 
    "Aerodynamics`AirfoilGeometry`", "keywords" -> {}, "index" -> True, 
    "label" -> "Aerodynamics Package Paclet Symbol", "language" -> "en", 
    "paclet" -> "Aerodynamics Package", "status" -> "None", "summary" -> 
    "AirfoilGeometry is a package providing functions to specify airfoil \
shape, discretize the airfoil and compute geometric properties of the \
discretized airfoil.", "synonyms" -> {}, "title" -> "AirfoilGeometry", "type" -> 
    "Symbol", "uri" -> "Aerodynamics/ref/AirfoilGeometry"}, "LinkTrails" -> 
  "", "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[608, 21, 76, 1, 70, "SymbolColorBar"],
Cell[687, 24, 191, 5, 70, "LinkTrail"],
Cell[881, 31, 3228, 70, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4134, 105, 58, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4195, 108, 434, 11, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4666, 124, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->322609720],
Cell[5249, 143, 47, 1, 70, "Notes",
 CellID->529540534]
}, Closed]],
Cell[CellGroupData[{
Cell[5333, 149, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[5755, 163, 196, 4, 70, "Tutorials",
 CellID->341631938],
Cell[5954, 169, 180, 4, 70, "Tutorials",
 CellID->187059247],
Cell[6137, 175, 222, 5, 70, "Tutorials",
 CellID->284174586],
Cell[6362, 182, 193, 4, 70, "Tutorials",
 CellID->47973038],
Cell[6558, 188, 158, 3, 70, "Tutorials",
 CellID->23098279]
}, Open  ]],
Cell[CellGroupData[{
Cell[6753, 196, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[7175, 210, 156, 3, 70, "MoreAbout",
 CellID->1665078683]
}, Open  ]],
Cell[7346, 216, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

