(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     73050,       1881]
NotebookOptionsPosition[     66803,       1685]
NotebookOutlinePosition[     69123,       1740]
CellTagsIndexPosition[     69037,       1735]
WindowTitle->AirfoilSurfacePoints - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"1.2 AirfoilsFromDataFiles\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AirfoilsFromDataFiles"], "\<\"1.3 \
Airfoils From Thickness And Camber Properties\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/\
AirfoilsFromThicknessAndCamberProperties"], "\<\"1.4 Airfoils From Conformal \
Mapping\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AirfoilsFromConformalMapping"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"AirfoilFromDataFile\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/AirfoilFromDataFile"], "\<\"FitAirfoil\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/FitAirfoil"], "\<\"NACA4DigitAirfoil\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/NACA4DigitAirfoil"], \
"\<\"NDiscretizeUnitSegment\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/NDiscretizeUnitSegment"], \
"\<\"PlotAirfoil\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PlotAirfoil"], "\<\"XToTheta\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/ref/XToTheta"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"AirfoilGeometry\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/AirfoilGeometry"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/ref/AirfoilSurfacePoints\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/AirfoilSurfacePoints"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
AirfoilSurfacePoints.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/AirfoilSurfacePoints", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["AirfoilSurfacePoints", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     "AirfoilSurfacePoints[spline, ",
     Cell[BoxData[
      StyleBox["x", "TI"]], "InlineFormula"],
     "]\[LineSeparator]returns a collection of vectors locating endpoints for \
panels discretizing an airfoil specified by a B-spline.  Locations of points \
on the surface of the airfoil are determined from the list of ",
     Cell[BoxData[
      StyleBox["x", "TI"]], "InlineFormula"],
     "-values. "
    }]]},
   {"", Cell[TextData[{
     "AirfoilSurfacePoints[properties]\[LineSeparator]returns a collection of \
vectors locating endpoints for panels discretizing an airfoil specified by a \
list of thickness and camber properties.  This nested list of properties \
includes ",
     Cell[BoxData[
      StyleBox["x", "TI"]], "InlineFormula"],
     "-values for discretization."
    }]]},
   {"", Cell["\<\
AirfoilSurfacePoints[t, tau, theta]\[LineSeparator]returns a collection of \
vectors locating endpoints for panels discretizing a van de Vooren airfoil of \
thickness parameter t and trailing-edge angle tau corresponding to a list of \
theta values in the range 0 to Pi.\
\>"]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, 
   "Rows" -> {None, None, {None}}, "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->621105191],

Cell[TextData[{
 "Collections of vectors are of the data type ",
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/guide/CartesianVectors"]], 
  "InlineFormula"],
 "."
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 "AirfoilSurfacePoints is used to rediscretize an airfoil specified by data \
files to a user-specified discretization.  The function AirfoilFromDataFile \
is used to import the file and arrange the data as a collection of vectors \
using the conventions of ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  The function FitAirfoil generates a B-spline function fitting the data.  \
AirfoilSurfacePoints uses the B-spline function to compute a collection of \
vectors locating points on the surface of the airfoil at a list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values.  The primary source for airfoil data is from the data base \
maintained by Michael Selig at University of Illinois (",
 ButtonBox["UIUC Airfoil Data Site)",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.ae.uiuc.edu/m-selig/ads.html"], None}],
 "."
}], "Notes",
 CellID->213740052],

Cell[TextData[{
 "For airfoils specified by equations for thickness and camber properties, \
AirfoilSurfacePoints computes a collection of vectors locating points on the \
surface of the airfoil from these properties at a list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values.  The example used in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " for this type of specification is the family of NACA four-digit airfoils."
}], "Notes",
 CellID->610349747],

Cell["\<\
The van de Vooren airfoil is used as an example of an airfoil specified by \
conformal mapping.  The last definition of AirfoilSurfacePoints is specific \
to this family of symmetric airfoils. \
\>", "Notes",
 CellID->447409252],

Cell["New in 1.0   |   Last modified in 1.2", "Notes",
 CellID->529540534]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->33526749],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->383676243],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->251463159],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->492977788],

Cell["\<\
Here are the names of the data files in the folder Airfoil Data.  Repetitive \
parts have been stripped away from path names.\
\>", "ExampleText",
 CellID->411631528],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"StringTake", "[", 
     RowBox[{"#", ",", 
      RowBox[{
       RowBox[{
        RowBox[{"StringPosition", "[", 
         RowBox[{"#", ",", "\"\<Airfoil Data\\\\\>\""}], "]"}], "[", 
        RowBox[{"[", 
         RowBox[{"1", ",", "2"}], "]"}], "]"}], "-", 
       RowBox[{"StringLength", "[", "#", "]"}]}]}], "]"}], "&"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"FileNames", "[", 
    RowBox[{"\"\<Aerodynamics\\\\Airfoil Data\\\\*\>\"", ",", "$Path"}], 
    "]"}]}], "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->161538214],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"clarky.dat\"\>", ",", "\<\"e205.dat\"\>", 
   ",", "\<\"e374.dat\"\>", ",", "\<\"l7769.dat\"\>", 
   ",", "\<\"naca23012.dat\"\>", ",", "\<\"naca4412.dat\"\>", 
   ",", "\<\"naca4412m.dat\"\>", ",", "\<\"naca642415.dat\"\>", 
   ",", "\<\"naca747a315.dat\"\>", ",", "\<\"s1223.dat\"\>", 
   ",", "\<\"sd7003.dat\"\>", ",", "\<\"usa35b.dat\"\>", 
   ",", "\<\"clarky.dat\"\>", ",", "\<\"e205.dat\"\>", 
   ",", "\<\"e374.dat\"\>", ",", "\<\"l7769.dat\"\>", 
   ",", "\<\"naca23012.dat\"\>", ",", "\<\"naca4412.dat\"\>", 
   ",", "\<\"naca642415.dat\"\>", ",", "\<\"naca747a315.dat\"\>", 
   ",", "\<\"s1223.dat\"\>", ",", "\<\"sd7003.dat\"\>", 
   ",", "\<\"usa35b.dat\"\>"}], "}"}]], "Output",
 ImageSize->{564, 67},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->25324894]
}, Open  ]],

Cell[TextData[{
 "Choose the Eppler 205 airfoil for this example.  Use AirfoilFromDataFile to \
read the data and arrange it in the form used in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  Use FitAirfoil to fit a B-spline to the data, and the resulting spline \
function to compute a collection of vectors locating points on the surface of \
the airfoil.  Plot both the data and the spline fit to the data."
}], "ExampleText",
 CellID->193925074],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<Aerodynamics\\\\Airfoil Data\\\\e205.dat\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"spline", "=", 
   RowBox[{"FitAirfoil", "[", "rD", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gD", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gSpline", "=", 
   RowBox[{"ParametricPlot", "[", 
    RowBox[{
     RowBox[{"spline", "[", "p", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"p", ",", "0", ",", "1"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gD", ",", "gSpline"}], "]"}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->435709841],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3f+PFPUdx/Fr+pf037hEueMOFAk9D4vYWmrhZAtt1Wo0MagRalsIQrrx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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->532113269]
}, Open  ]],

Cell["\<\
The following discretization is based on the spline fit to the data.  A plot \
of the user-specified discretization is shown below. \
\>", "ExampleText",
 CellID->105478441],

Cell[CellGroupData[{

Cell["PlotAirfoil[AirfoilSurfacePoints[spline,Range[0.,1.,0.05]]]", "Input",
 CellLabel->"In[4]:=",
 CellID->78161919],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3c+LG+cdx3GV/iX9NxZKDyGHkENIIeRgfFmDsC/NJbm0F/uSXFoQyWUX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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->342558728]
}, Open  ]],

Cell["\<\
Next, consider the NACA 4415 airfoil defined by equations for thickness and \
camber properties that are computed by the function NACA4DigitAirfoil.  Note \
that functions can be nested.\
\>", "ExampleText",
 CellID->656638239],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotAirfoil", "[", 
  RowBox[{"AirfoilSurfacePoints", "[", 
   RowBox[{"NACA4DigitAirfoil", "[", 
    RowBox[{"4415", ",", 
     RowBox[{"Range", "[", 
      RowBox[{"0.", ",", "1.", ",", "0.1"}], "]"}]}], "]"}], "]"}], 
  "]"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->52317584],

Cell[BoxData[
 GraphicsBox[
  {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxTTMoPSmViYGAQBWIQvWGdZJn3v/f2zdsO14lMidxf8Y37fO6eN/aPvkpJ
dMwu358tL+tzsfGlPd/ZKQcaE1r3t36+U1Do+8ze1cOo7JfQhP0hKUt3F8s9
ti+JXuB7+ePU/VNzzDpfPr9vv0te5v2GN7P3z5oJAjftv6Z2Pzy6YsF+cdvJ
e6YUXLaf+evSikOai/cb+Ym0FTafsk9Ykfl3su7S/Vc/bhNYfmm3/akjV5fM
XrtkPwMaiHesUepJ2G4fxX08cYLPZnump3JrD64/Yd+TXP7l06ed9oGal1de
/HrJvmHDnHaRE3vsYfaXHS+2cDm/x37ryRRrbp4H9k1uG85nPdllL8J0cNUn
98f2uy8z+ZzesN3e8+b76Oj6Z/Y5iv6Tnydstj8pLPY8ceNL+4j5z9p6JNfY
Ky3uPaV99429VuPt1zfezbNfofHZ5RbDB1j42QMAAsKt8g==
    "]]},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2000000000000004`, 
      FormBox["1.2000000000000004`", TraditionalForm]}, {1.4000000000000001`, 
      FormBox["1.4000000000000001`", TraditionalForm]}, {1.6, 
      FormBox["1.6`", TraditionalForm]}}, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}, None, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}},
  ImageSize->{504., 180.},
  PlotRange->{{-0.05, 1.05}, {-0.1, 0.201}},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->12481083]
}, Open  ]],

Cell[TextData[{
 "Finally, consider an airfoil defined by a mapping function.  The example \
used is a member of  the van de Vooren family of symmetric airfoils.  The \
list of ",
 Cell[BoxData[
  StyleBox["\[Theta]", "TR"]], "InlineFormula"],
 "-values locates points on a circle that are mapped to the airfoil using the \
van de Vooren transformation.  The range of ",
 Cell[BoxData[
  StyleBox["\[Theta]", "TR"]], "InlineFormula"],
 " is ",
 Cell[BoxData["0"], "InlineFormula"],
 " to ",
 Cell[BoxData[
  StyleBox["\[Pi]", "TR"]], "InlineFormula"],
 " to generate points on the upper surface of the airfoil, and mirror \
symmetry is used to generate points on the lower surface.  The thickness \
parameter, ",
 Cell[BoxData[
  StyleBox["t", "TI"]], "InlineFormula"],
 ", and the trailing-edge angle, ",
 Cell[BoxData[
  StyleBox["\[Tau]", "TR"]], "InlineFormula"],
 ", are parameters in the transformation.  Plot a member of this family of \
airfoils."
}], "ExampleText",
 CellID->179618537],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotAirfoil", "[", 
  RowBox[{"AirfoilSurfacePoints", "[", 
   RowBox[{"0.12", ",", 
    RowBox[{"9.81", " ", "Degree"}], ",", 
    RowBox[{"N", "[", 
     RowBox[{"Range", "[", 
      RowBox[{"0", ",", "\[Pi]", ",", 
       RowBox[{"\[Pi]", "/", "30"}]}], "]"}], "]"}]}], "]"}], "]"}]], "Input",
 CellLabel->"In[6]:=",
 CellID->4154308],

Cell[BoxData[
 GraphicsBox[
  {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdk3tI01EUx39YqSMTlyVIDxRkE4lEG+Rj42tYGFKoOdCMSHvYRoI9MCTC
ZqHlonxg2mw+Uucjc5KlFKlHCu3hjE0Mn5EsxJmzRZrEMnsQ/dG5cLkXLvfe
c77fz9f/aOaBEy6CICh+zz/r3/EZ/3YhN1dNV1gdiM63Lz77KaPZid6JyEIH
9nwRJOMzSlLNG8cyRA50RamXUxpPkkn5eilW+QmeK7J2vTSLkm+YqmTn5mFO
UjRVl2oo7eHaEavajmx9fXn66FVaPCsKTZfPwX5n98qZrUWkFbt3+DpmIXg7
xfaXpRRQOd1ZnG+DPGlSETCoow06z7wCtxnoFyRfo92qSRPUdthyfhom+xtR
h7SWru8qv6cY+4Bt2/2f3Oqup5R1mcLtHVaYEzzSyoIbyaLp31yincJCxaDy
iKqZPJqTh7L63qG/CasfJbbQ8lth4KNmHFWS4e+S9/ep4VSX1ho3gm8Rc4Wd
zlbKWLrsEhsxDHOPyrlfZySfC69CS2IsuLgclxP4wEhhJlVwwg8T9nq2t22U
GykwRrHJOvACZl+RonpnK8m0BkOu63O4K1VasaGFUp9656gzu2ErPbhFXdBM
8bV1uY2WDnhl++2TzjdQat8hrfiaEaouqc1rso78pK6+Q011GHWGd48dr6Gp
QYo0hpWhprK36FKIjqLWz1bqLVdwuufumlCfYhLy4h8PTRxDaqItPCU6l4T/
Ro2cnYPdB3sf7H+w+sDqB+sPrH8wfcD0A9MXTH8wf8D8A/MXzH8wPsD4AeML
jD8wPsH4BeMbjH+wfIDlByxfYPkDyyd4fn8BKknaug==
    "]]},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2000000000000004`, 
      FormBox["1.2000000000000004`", TraditionalForm]}, {1.4000000000000001`, 
      FormBox["1.4000000000000001`", TraditionalForm]}, {1.6, 
      FormBox["1.6`", TraditionalForm]}}, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}, None, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}},
  ImageSize->{504., 180.},
  PlotRange->{{-0.05, 1.05}, {-0.1, 0.201}},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->210625658]
}, Open  ]],

Cell[TextData[{
 "Although the list of ",
 Cell[BoxData[
  StyleBox["\[Theta]", "TR"]], "InlineFormula"],
 "-values can be used for discretization, a function XToTheta is defined to \
provide user-specified discretization consistent with the other sources of \
information for airfoils.  This function uses the van de Vooren \
transformation to provide an interpolating function for ",
 Cell[BoxData[
  StyleBox["\[Theta]", "TR"]], "InlineFormula"],
 " as a function of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "."
}], "ExampleText",
 CellID->88818872],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotAirfoil", "[", 
  RowBox[{"AirfoilSurfacePoints", "[", 
   RowBox[{"0.12", ",", 
    RowBox[{"9.81", " ", "Degree"}], ",", 
    RowBox[{"XToTheta", "[", 
     RowBox[{"0.12", ",", 
      RowBox[{"9.81", " ", "Degree"}], ",", 
      RowBox[{"Range", "[", 
       RowBox[{"0.", ",", "1.", ",", "0.1"}], "]"}]}], "]"}]}], "]"}], 
  "]"}]], "Input",
 CellLabel->"In[7]:=",
 CellID->261648493],

Cell[BoxData[
 GraphicsBox[
  {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxTTMoPSmViYGAQBWIQDQEf7GGsPXXmh86ceWNvE3jrcplc6/5j7e6TZs58
ac9h4iNw5tfU/fu6PySnpT2z32V/7s33aQv2h8RmGxkbP7bXS6lpW7hr6X47
SRagKQ/sN19drccntGp/w9SY2TNn3rRf5+Uq1mm1dn9J4BMTY+PL9n/eVDTz
bly3fwfn/QUzZ56033D+xISrS9bu37G68OfMmTvtT3C53FkYvGI/AwpYYIMm
b4+m3x7NfHs0++3R3GeP5n57NP/Zo/nfHi187NHDDwCAqpfN
    "]]},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2000000000000004`, 
      FormBox["1.2000000000000004`", TraditionalForm]}, {1.4000000000000001`, 
      FormBox["1.4000000000000001`", TraditionalForm]}, {1.6, 
      FormBox["1.6`", TraditionalForm]}}, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}, None, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}},
  ImageSize->{504., 180.},
  PlotRange->{{-0.05, 1.05}, {-0.1, 0.201}},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->126894391]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Scope",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->1293636265],

Cell[TextData[{
 "A list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values controls the layout of the discretized airfoil used in numerical \
models.  The function NDiscretizeUnitSegment generates such a list and \
provides some flexibility for layout.  This is illustrated using the NACA \
3415 airfoil."
}], "ExampleText",
 CellID->148587983],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotAirfoil", "[", 
  RowBox[{"AirfoilSurfacePoints", "[", 
   RowBox[{"NACA4DigitAirfoil", "[", 
    RowBox[{"3415", ",", 
     RowBox[{"NDiscretizeUnitSegment", "[", 
      RowBox[{"10", ",", 
       RowBox[{"Layout", "\[Rule]", "ConstantSpacing"}]}], "]"}]}], "]"}], 
   "]"}], "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->562629873],

Cell[BoxData[
 GraphicsBox[
  {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxTTMoPSmViYGAQBWIQzf2tImz7v/f276WO99itjdz/osBBufbAG/u3Ulsq
mDWb9i9m3tu6u/2lfZVch05I/oT931QNOVcFP7NncKmdsvrM9P1vV5Y9XKj8
2N57vdS5AqH5+1dy8i1/9/a+/ZYv+65N/7ho/6yZIHDTXuR1mXHd8aX7NW8m
nXRKvGz/5ulisTe/l+9X0OaUivU8Zb8t9U3cR4mV+7fqXiwqddttv8NtDe81
/uX7GdBAPWvSgb1vttuLOEV/snHYZC/CoSV57eUJ+0S5E+nTBXfY2/cyz1Vm
vWwfE7mjMCZllz3M/mcsdfwv7HfZ6/yuWMPB+cDeI9Pv1q70HfbVe963HHF6
bF+58xfv3eyt9jeWzT6sVPnMvsWmcK5nxUb7ifpla8tXv7T3VQhV3+2x0n6T
RmSZzM039jYLdr6oujXb/lfL4ZAlDB9g4WcPAJfCo1Y=
    "]]},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2000000000000004`, 
      FormBox["1.2000000000000004`", TraditionalForm]}, {1.4000000000000001`, 
      FormBox["1.4000000000000001`", TraditionalForm]}, {1.6, 
      FormBox["1.6`", TraditionalForm]}}, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}, None, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}},
  ImageSize->{504., 180.},
  PlotRange->{{-0.05, 1.05}, {-0.1, 0.201}},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->24604840]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotAirfoil", "[", 
  RowBox[{"AirfoilSurfacePoints", "[", 
   RowBox[{"NACA4DigitAirfoil", "[", 
    RowBox[{"3415", ",", 
     RowBox[{"NDiscretizeUnitSegment", "[", 
      RowBox[{"10", ",", 
       RowBox[{"Layout", "\[Rule]", "CosineSpacing"}]}], "]"}]}], "]"}], 
   "]"}], "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->15423508],

Cell[BoxData[
 GraphicsBox[
  {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxTTMoPSmViYGAQBWIQzf2tImz7v/f276WO99itjdzfHjyF6a/Re3uJiH3O
H9Zl71/Yx9J7+ukb+5ctjgu21zXuj5BLt2oNfWnfbb1+ccyjCfuvXghj/XX/
iX0Q38m7FUqz96/g5Fv+7u19+01f9l2b/nHRfmOm5xPKTa/Zq38R/tjFvXx/
VPWJUA3e0/ZikVNO7xNbuf8Jp1y4rNwu+zNabpNnXVq2f9MZzzyvdXPti2VF
OP7qzN3PgAbyJmtrfs6fYm/jtWV5sPUi+zsBSmdUj22z53msFbjk3kZ7I+mk
Gd7bTtrP/HZffa73DvtakZfmM5mv2Wd9+pb+csYue+3fFWs4OB/Yu2f63dqV
vsP+srLjtFrmp/YdM6zDXBI327PY8J2aW/nSXuvY9u1ab1bas6dN5lvz9419
l7vH57L9s+wrGTcICNq8t1f7uvNn8d4G+18th0OWMHyAhZ89AA6qoTI=
    "]]},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2000000000000004`, 
      FormBox["1.2000000000000004`", TraditionalForm]}, {1.4000000000000001`, 
      FormBox["1.4000000000000001`", TraditionalForm]}, {1.6, 
      FormBox["1.6`", TraditionalForm]}}, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}, None, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}},
  ImageSize->{504., 180.},
  PlotRange->{{-0.05, 1.05}, {-0.1, 0.201}},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->689854535]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotAirfoil", "[", 
  RowBox[{"AirfoilSurfacePoints", "[", 
   RowBox[{"NACA4DigitAirfoil", "[", 
    RowBox[{"3415", ",", 
     RowBox[{"NDiscretizeUnitSegment", "[", 
      RowBox[{"10", ",", 
       RowBox[{"Layout", "\[Rule]", "HalfCosineSpacing"}]}], "]"}]}], "]"}], 
   "]"}], "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->466370447],

Cell[BoxData[
 GraphicsBox[
  {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxTTMoPSmViYGAQBWIQzfWtImz7v/f2zdLHe+zWRu4/JNA2J+fdK3uuLTcv
Xynr2q9+Xkz3LeczexPRvT6pZTP2T2q6y9KQ89C+8ve7Hww5C/efzn62Ljj+
lv3bLws8HlYt3d9UvVtH9MMl+51Bq9s9ZFfsb0+a/YdR46Q9T4/CYxXxlftX
cUee4Jyy196zqWmefs3y/Z9+nW2MZ1hjn7P+QFjapUX7YxtcJj0VnGDf1H5a
8dyXqfsZ0IDbxZbDrzyb7O8kirEorZtpr5e6XPSe7DL7IlPvFYYPV9q36V5J
/mK5y/61vZUE189N9pY/lqg57jluf+b41nm827bbe011tyzsvWR/bfP8xnsB
u+wParnYK6Tcsheb25TxQ3yXfd78u+cDWx/aO4g5XvXV3G7f6b5ztrL+M/ti
zrub39zZaN90dOn6Xr7XQPfpWn/uXmL/q+VwyBKGD/Y3IeFnDwD/26OP
    "]]},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2000000000000004`, 
      FormBox["1.2000000000000004`", TraditionalForm]}, {1.4000000000000001`, 
      FormBox["1.4000000000000001`", TraditionalForm]}, {1.6, 
      FormBox["1.6`", TraditionalForm]}}, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}, None, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}},
  ImageSize->{504., 180.},
  PlotRange->{{-0.05, 1.05}, {-0.1, 0.201}},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->482981440]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Possible Issues",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->1305812373],

Cell["\<\
For some airfoils defined by data files, numerical problems may be \
encountered for fine discretization.  The airfoils used as examples in this \
documentation (see the folder Airfoil Data) were tested using the three \
values of the option Layout for several discretizations up to 500 panels \
without encountering problems.  Please let me know at rlf@ufl.edu if you \
encounter numerical problems re-discretizing airfoils specified by data files.\
\>", "ExampleText",
 CellID->373231908]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[{
 Cell[BoxData[
  StyleBox[
   ButtonBox["AirfoilFromDataFile",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/AirfoilFromDataFile"],
   FontFamily->"Verdana"]], "InlineFormula"],
 "\[NonBreakingSpace]",
 StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", 
  "InlineSeparator"],
 " ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["FitAirfoil",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/FitAirfoil"],
   FontFamily->"Verdana"]], "InlineFormula"],
 "\[NonBreakingSpace]",
 StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", 
  "InlineSeparator"],
 " ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["NACA4DigitAirfoil",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/NACA4DigitAirfoil"],
   FontFamily->"Verdana"]], "InlineFormula"],
 "\[NonBreakingSpace]",
 StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", 
  "InlineSeparator"],
 " ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["NDiscretizeUnitSegment",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/NDiscretizeUnitSegment"],
   FontFamily->"Verdana"]], "InlineFormula"],
 "\[NonBreakingSpace]",
 StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", 
  "InlineSeparator"],
 " ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["PlotAirfoil",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/PlotAirfoil"],
   FontFamily->"Verdana"]], "InlineFormula"],
 "\[NonBreakingSpace]",
 StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", 
  "InlineSeparator"],
 " ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["XToTheta",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/XToTheta"],
   FontFamily->"Verdana"]], "InlineFormula"]
}], "SeeAlso",
 CellID->12912236]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["1.2 AirfoilsFromDataFiles",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromDataFiles"]], "Tutorials",
 CellID->341631938],

Cell[TextData[ButtonBox["1.3 Airfoils From Thickness And Camber Properties",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromThicknessAndCamberProperties"]], \
"Tutorials",
 CellID->333185244],

Cell[TextData[ButtonBox["1.4 Airfoils From Conformal Mapping",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromConformalMapping"]], "Tutorials",
 CellID->113142828]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["AirfoilGeometry",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/AirfoilGeometry"]], "MoreAbout",
 CellID->202782803]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["RELATED LINKS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "RelatedLinksSection",
 CellID->1584193535],

Cell[TextData[ButtonBox["UIUC Airfoil Data Site",
 BaseStyle->"Hyperlink",
 ButtonData->{
   URL["http://www.ae.uiuc.edu/m-selig/ads.html"], None}]], "RelatedLinks",
 CellID->1038487239]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"AirfoilSurfacePoints - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 9, 58, 21.6096000}", "context" -> 
    "Aerodynamics`AirfoilGeometry`", "keywords" -> {}, "index" -> True, 
    "label" -> "Aerodynamics Package Paclet Symbol", "language" -> "en", 
    "paclet" -> "Aerodynamics Package", "status" -> "None", "summary" -> 
    "AirfoilSurfacePoints[spline, x] returns a collection of vectors locating \
endpoints for panels discretizing an airfoil specified by a B-spline. \
Locations of points on the surface of the airfoil are determined from the \
list of x-values. AirfoilSurfacePoints[properties] returns a collection of \
vectors locating endpoints for panels discretizing an airfoil specified by a \
list of thickness and camber properties. This nested list of properties \
includes x-values for discretization.AirfoilSurfacePoints[t, tau, theta] \
returns a collection of vectors locating endpoints for panels discretizing a \
van de Vooren airfoil of thickness parameter t and trailing-edge angle tau \
corresponding to a list of theta values in the range 0 to Pi.", 
    "synonyms" -> {}, "title" -> "AirfoilSurfacePoints", "type" -> "Symbol", 
    "uri" -> "Aerodynamics/ref/AirfoilSurfacePoints"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[8886, 240, 460, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->33526749]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 68895, 1728}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[613, 21, 76, 1, 70, "SymbolColorBar"],
Cell[692, 24, 191, 5, 70, "LinkTrail"],
Cell[886, 31, 3963, 88, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4874, 123, 63, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4940, 126, 1306, 31, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6283, 162, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->621105191],
Cell[6866, 181, 256, 9, 70, "Notes",
 CellID->1067943069],
Cell[7125, 192, 918, 20, 70, "Notes",
 CellID->213740052],
Cell[8046, 214, 485, 11, 70, "Notes",
 CellID->610349747],
Cell[8534, 227, 238, 5, 70, "Notes",
 CellID->447409252],
Cell[8775, 234, 74, 1, 70, "Notes",
 CellID->529540534]
}, Closed]],
Cell[CellGroupData[{
Cell[8886, 240, 460, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->33526749],
Cell[CellGroupData[{
Cell[9371, 257, 148, 5, 70, "ExampleSection",
 CellID->383676243],
Cell[9522, 264, 132, 6, 70, "ExampleText",
 CellID->251463159],
Cell[9657, 272, 100, 3, 70, "Input",
 CellID->492977788],
Cell[9760, 277, 176, 4, 70, "ExampleText",
 CellID->411631528],
Cell[CellGroupData[{
Cell[9961, 285, 607, 18, 70, "Input",
 CellID->161538214],
Cell[10571, 305, 867, 18, 88, "Output",
 CellID->25324894]
}, Open  ]],
Cell[11453, 326, 460, 9, 70, "ExampleText",
 CellID->193925074],
Cell[CellGroupData[{
Cell[11938, 339, 840, 27, 101, "Input",
 CellID->435709841],
Cell[12781, 368, 5843, 103, 201, "Output",
 CellID->532113269]
}, Open  ]],
Cell[18639, 474, 183, 4, 70, "ExampleText",
 CellID->105478441],
Cell[CellGroupData[{
Cell[18847, 482, 118, 2, 28, "Input",
 CellID->78161919],
Cell[18968, 486, 4835, 87, 201, "Output",
 CellID->342558728]
}, Open  ]],
Cell[23818, 576, 237, 5, 70, "ExampleText",
 CellID->656638239],
Cell[CellGroupData[{
Cell[24080, 585, 305, 9, 70, "Input",
 CellID->52317584],
Cell[24388, 596, 5428, 124, 201, "Output",
 CellID->12481083]
}, Open  ]],
Cell[29831, 723, 994, 26, 70, "ExampleText",
 CellID->179618537],
Cell[CellGroupData[{
Cell[30850, 753, 361, 10, 70, "Input",
 CellID->4154308],
Cell[31214, 765, 5791, 130, 201, "Output",
 CellID->210625658]
}, Open  ]],
Cell[37020, 898, 572, 15, 70, "ExampleText",
 CellID->88818872],
Cell[CellGroupData[{
Cell[37617, 917, 416, 12, 70, "Input",
 CellID->261648493],
Cell[38036, 931, 5246, 121, 201, "Output",
 CellID->126894391]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[43331, 1058, 221, 7, 70, "ExampleSection",
 CellID->1293636265],
Cell[43555, 1067, 367, 9, 70, "ExampleText",
 CellID->148587983],
Cell[CellGroupData[{
Cell[43947, 1080, 364, 10, 70, "Input",
 CellID->562629873],
Cell[44314, 1092, 5424, 124, 201, "Output",
 CellID->24604840]
}, Open  ]],
Cell[CellGroupData[{
Cell[49775, 1221, 361, 10, 70, "Input",
 CellID->15423508],
Cell[50139, 1233, 5437, 124, 201, "Output",
 CellID->689854535]
}, Open  ]],
Cell[CellGroupData[{
Cell[55613, 1362, 366, 10, 70, "Input",
 CellID->466370447],
Cell[55982, 1374, 5437, 124, 201, "Output",
 CellID->482981440]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[61468, 1504, 231, 7, 70, "ExampleSection",
 CellID->1305812373],
Cell[61702, 1513, 500, 8, 70, "ExampleText",
 CellID->373231908]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[62251, 1527, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[62671, 1541, 1765, 58, 70, "SeeAlso",
 CellID->12912236]
}, Open  ]],
Cell[CellGroupData[{
Cell[64473, 1604, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[64895, 1618, 177, 4, 70, "Tutorials",
 CellID->341631938],
Cell[65075, 1624, 222, 5, 70, "Tutorials",
 CellID->333185244],
Cell[65300, 1631, 194, 4, 70, "Tutorials",
 CellID->113142828]
}, Open  ]],
Cell[CellGroupData[{
Cell[65531, 1640, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[65953, 1654, 155, 3, 70, "MoreAbout",
 CellID->202782803]
}, Open  ]],
Cell[CellGroupData[{
Cell[66145, 1662, 427, 12, 70, "RelatedLinksSection",
 CellID->1584193535],
Cell[66575, 1676, 186, 4, 70, "RelatedLinks",
 CellID->1038487239]
}, Open  ]],
Cell[66776, 1683, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

