(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     15606,        502]
NotebookOptionsPosition[     11929,        384]
NotebookOutlinePosition[     13663,        431]
CellTagsIndexPosition[     13578,        426]
WindowTitle->ComplexToCartesianVectors - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"A1 CartesianVectors\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/CartesianVectors"], "\<\"1.4 Airfoils \
from Conformal Mapping\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AirfoilsFromConformalMapping"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"AirfoilSurfacePoints\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/AirfoilSurfacePoints"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"CartesianVectors\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/CartesianVectors"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/ref/ComplexToCartesianVectors\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/ComplexToCartesianVectors"]}, Visible -> 
            False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
ComplexToCartesianVectors.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/ComplexToCartesianVectors", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["ComplexToCartesianVectors", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ComplexToCartesianVectors", "[", 
       StyleBox["c", "TI"], "]"}]], "InlineFormula"],
     "\[LineSeparator]converts a list of complex numbers to a collection of \
two-dimensional vectors of the data type ",
     Cell[BoxData[
      ButtonBox["CartesianVectors",
       BaseStyle->"Link",
       ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], 
      "InlineFormula"],
     "."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->221661774],

Cell["\<\
One source of information about airfoil geometry is complex-variable \
transformations that map a circle to an airfoil shape.  The function \
ComplexToCartesianVectors converts a list of complex numbers into a \
collection of two-dimensional vectors of the data type CartesianVectors.\
\>", "Notes",
 CellID->1067943069],

Cell["New in 1.0   |   Last modified in 1.1", "Notes",
 CellID->529540534]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->313792616],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->447047460],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->325550546],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->775518656],

Cell["Construct a list of complex numbers.", "ExampleText",
 CellID->75855286],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"zList", "=", 
  RowBox[{"RandomComplex", "[", 
   RowBox[{
    RowBox[{"1", "+", "I"}], ",", "5"}], "]"}]}]], "Input",
 CellLabel->"In[2]:=",
 CellID->414255905],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"0.2987432940746757`", "\[VeryThinSpace]", "+", 
    RowBox[{"0.8154219724105003`", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{"0.5416597983316869`", "\[VeryThinSpace]", "+", 
    RowBox[{"0.8433201449772807`", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{"0.6026801552983161`", "\[VeryThinSpace]", "+", 
    RowBox[{"0.52755883490029`", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{"0.38392804836704264`", "\[VeryThinSpace]", "+", 
    RowBox[{"0.9229044024516502`", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{"0.13118342116321613`", "\[VeryThinSpace]", "+", 
    RowBox[{"0.8626325364009024`", " ", "\[ImaginaryI]"}]}]}], 
  "}"}]], "Output",
 ImageSize->{442, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->17753360]
}, Open  ]],

Cell["\<\
Convert the list of complex numbers to a collection of two-dimensional \
vectors of the data type CartesianVectors.\
\>", "ExampleText",
 CellID->48320982],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"r", "=", 
  RowBox[{"ComplexToCartesianVectors", "[", "zList", "]"}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->515085932],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "0.2987432940746757`", ",", "0.5416597983316869`", ",", 
     "0.6026801552983161`", ",", "0.38392804836704264`", ",", 
     "0.13118342116321613`"}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
    "0.8154219724105003`", ",", "0.8433201449772807`", ",", 
     "0.52755883490029`", ",", "0.9229044024516502`", ",", 
     "0.8626325364009024`"}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{0.2987432940746757, 
    0.5416597983316869, 0.6026801552983161, 0.38392804836704264`, 
    0.13118342116321613`}, {0.8154219724105003, 0.8433201449772807, 
    0.52755883490029, 0.9229044024516502, 0.8626325364009024}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{343, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->610357556]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Head", "[", "r", "]"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->373451540],

Cell[BoxData["CartesianVectors"], "Output",
 ImageSize->{116, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->84360204]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[Cell[BoxData[
 StyleBox[
  ButtonBox["AirfoilSurfacePoints",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilSurfacePoints"],
  FontFamily->"Verdana"]], "InlineFormula"]], "SeeAlso",
 CellID->929782353]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[{
 ButtonBox["A1",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/CartesianVectors"],
 ButtonBox[" ",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/CartesianVectors"],
 ButtonBox["CartesianVectors",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/CartesianVectors"]
}], "Tutorials",
 CellID->341631938],

Cell[TextData[ButtonBox["1.4 Airfoils from Conformal Mapping",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromConformalMapping"]], "Tutorials",
 CellID->680602796]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["CartesianVectors",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/CartesianVectors"]], "MoreAbout",
 CellID->139415997]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"ComplexToCartesianVectors - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 9, 58, 28.6920000}", "context" -> 
    "Aerodynamics`CartesianVectors`", "keywords" -> {}, "index" -> True, 
    "label" -> "Aerodynamics Package Paclet Symbol", "language" -> "en", 
    "paclet" -> "Aerodynamics Package", "status" -> "None", "summary" -> 
    "ComplexToCartesianVectors[c] converts a list of complex numbers to a \
collection of two-dimensional vectors of the data type CartesianVectors.", 
    "synonyms" -> {}, "title" -> "ComplexToCartesianVectors", "type" -> 
    "Symbol", "uri" -> "Aerodynamics/ref/ComplexToCartesianVectors"}, 
  "LinkTrails" -> "", "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[5972, 171, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->313792616]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 13435, 419}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[618, 21, 76, 1, 70, "SymbolColorBar"],
Cell[697, 24, 191, 5, 70, "LinkTrail"],
Cell[891, 31, 3289, 77, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4205, 112, 68, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4276, 115, 632, 18, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4945, 138, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->221661774],
Cell[5528, 157, 330, 6, 70, "Notes",
 CellID->1067943069],
Cell[5861, 165, 74, 1, 70, "Notes",
 CellID->529540534]
}, Closed]],
Cell[CellGroupData[{
Cell[5972, 171, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->313792616],
Cell[CellGroupData[{
Cell[6458, 188, 148, 5, 70, "ExampleSection",
 CellID->447047460],
Cell[6609, 195, 132, 6, 70, "ExampleText",
 CellID->325550546],
Cell[6744, 203, 100, 3, 70, "Input",
 CellID->775518656],
Cell[6847, 208, 78, 1, 70, "ExampleText",
 CellID->75855286],
Cell[CellGroupData[{
Cell[6950, 213, 185, 6, 70, "Input",
 CellID->414255905],
Cell[7138, 221, 821, 18, 54, "Output",
 CellID->17753360]
}, Open  ]],
Cell[7974, 242, 165, 4, 70, "ExampleText",
 CellID->48320982],
Cell[CellGroupData[{
Cell[8164, 250, 148, 4, 70, "Input",
 CellID->515085932],
Cell[8315, 256, 899, 23, 54, "Output",
 CellID->610357556]
}, Open  ]],
Cell[CellGroupData[{
Cell[9251, 284, 100, 3, 70, "Input",
 CellID->373451540],
Cell[9354, 289, 173, 5, 36, "Output",
 CellID->84360204]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[9588, 301, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[10008, 315, 237, 6, 70, "SeeAlso",
 CellID->929782353]
}, Open  ]],
Cell[CellGroupData[{
Cell[10282, 326, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[10704, 340, 370, 11, 70, "Tutorials",
 CellID->341631938],
Cell[11077, 353, 194, 4, 70, "Tutorials",
 CellID->680602796]
}, Open  ]],
Cell[CellGroupData[{
Cell[11308, 362, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[11730, 376, 157, 3, 70, "MoreAbout",
 CellID->139415997]
}, Open  ]],
Cell[11902, 382, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

