(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     19309,        599]
NotebookOptionsPosition[     15751,        485]
NotebookOutlinePosition[     17471,        532]
CellTagsIndexPosition[     17386,        527]
WindowTitle->DisplacementVectors - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"2.1 Introduction to Influence Coefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"InfluenceCoefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/InfluenceCoefficients"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/DisplacementVectors\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/DisplacementVectors"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
DisplacementVectors.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/DisplacementVectors", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["DisplacementVectors", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"DisplacementVectors", "[", 
       RowBox[{
        SubscriptBox[
         StyleBox["r", "TI"], 
         StyleBox["a", "TI"]], ",", 
        SubscriptBox[
         StyleBox["r", "TI"], 
         StyleBox["b", "TI"]]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes the displacement vectors between all pairs of \
points for the two collections of vectors, ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["a", "TI"]]], "InlineFormula"],
     " and ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["b", "TI"]]], "InlineFormula"],
     "."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->83451093],

Cell[TextData[{
 "Collections of vectors are of the data type ",
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], "InlineFormula"],
 "."
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 "The collections of vectors ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["a", "TI"]]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["b", "TI"]]], "InlineFormula"],
 " have components that are simple lists.  If there are ",
 Cell[BoxData[
  StyleBox["m", "TI"]], "InlineFormula"],
 " vectors in the collection ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["a", "TI"]]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  StyleBox["n", "TI"]], "InlineFormula"],
 " vectors in the collection ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["b", "TI"]]], "InlineFormula"],
 ", the result will be a collection of ",
 Cell[BoxData[
  StyleBox[
   RowBox[{"m", " ", "n"}], "TI"]], "InlineFormula"],
 " vectors whose components are represented as matrices with ",
 Cell[BoxData[
  StyleBox["m", "TI"]], "InlineFormula"],
 " rows and ",
 Cell[BoxData[
  StyleBox["n", "TI"]], "InlineFormula"],
 " columns."
}], "Notes",
 CellID->41491542],

Cell["\<\
Computation of displacement vectors is common to all functions for \
potential-flow influence coefficients.\
\>", "Notes",
 CellID->37883947],

Cell["New in 1.0", "Notes",
 CellID->529540534]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->100873724],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->120716317],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->630439552],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->873638675],

Cell["Specify two collections of two-dimensional vectors.", "ExampleText",
 CellID->109948455],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rA", "=", 
  RowBox[{"MakeCartesianVectors", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Array", "[", 
      RowBox[{"xA", ",", "3"}], "]"}], ",", 
     RowBox[{"Array", "[", 
      RowBox[{"zA", ",", "3"}], "]"}]}], "}"}], "]"}]}]], "Input",
 CellLabel->"In[2]:=",
 CellID->75690307],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"xA", "[", "1", "]"}], ",", 
     RowBox[{"xA", "[", "2", "]"}], ",", 
     RowBox[{"xA", "[", "3", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"zA", "[", "1", "]"}], ",", 
     RowBox[{"zA", "[", "2", "]"}], ",", 
     RowBox[{"zA", "[", "3", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`xA[1], 
     $CellContext`xA[2], 
     $CellContext`xA[3]}, {
     $CellContext`zA[1], 
     $CellContext`zA[2], 
     $CellContext`zA[3]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{295, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->37785859]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rB", "=", 
  RowBox[{"MakeCartesianVectors", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Array", "[", 
      RowBox[{"xB", ",", "4"}], "]"}], ",", 
     RowBox[{"Array", "[", 
      RowBox[{"zB", ",", "4"}], "]"}]}], "}"}], "]"}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->627517166],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"xB", "[", "1", "]"}], ",", 
     RowBox[{"xB", "[", "2", "]"}], ",", 
     RowBox[{"xB", "[", "3", "]"}], ",", 
     RowBox[{"xB", "[", "4", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"zB", "[", "1", "]"}], ",", 
     RowBox[{"zB", "[", "2", "]"}], ",", 
     RowBox[{"zB", "[", "3", "]"}], ",", 
     RowBox[{"zB", "[", "4", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`xB[1], 
     $CellContext`xB[2], 
     $CellContext`xB[3], 
     $CellContext`xB[4]}, {
     $CellContext`zB[1], 
     $CellContext`zB[2], 
     $CellContext`zB[3], 
     $CellContext`zB[4]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{387, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->630054672]
}, Open  ]],

Cell["Compute displacement vectors.", "ExampleText",
 CellID->74055380],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"DisplacementVectors", "[", 
    RowBox[{"rA", ",", "rB"}], "]"}], ")"}], "//", "MatrixForm"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->578584902],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnT1y40YQhenySXwNHsOB460NWJuZIbN1tiGZMVSoUJkYKmTIAzDYkKFC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    "], {{0, 86}, {384, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{384, 86},
  PlotRange->{{0, 384}, {0, 86}}]], "Output",
 ImageSize->{388, 86},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->358594059]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["2.1 Introduction to Influence Coefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"]], \
"Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["InfluenceCoefficients",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/InfluenceCoefficients"]], "MoreAbout",
 CellID->200997734]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"DisplacementVectors - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 9, 58, 46.9440000}", "context" -> 
    "Aerodynamics`InfluenceCoefficients`", "keywords" -> {}, "index" -> True, 
    "label" -> "Aerodynamics Package Paclet Symbol", "language" -> "en", 
    "paclet" -> "Aerodynamics Package", "status" -> "None", "summary" -> 
    "DisplacementVectors[r_a, r_b] computes the displacement vectors between \
all pairs of points for the two collections of vectors, r_a and r_b.", 
    "synonyms" -> {}, "title" -> "DisplacementVectors", "type" -> "Symbol", 
    "uri" -> "Aerodynamics/ref/DisplacementVectors"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[6764, 217, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->100873724]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 17243, 520}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[612, 21, 76, 1, 70, "SymbolColorBar"],
Cell[691, 24, 191, 5, 70, "LinkTrail"],
Cell[885, 31, 2742, 64, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3652, 99, 62, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[3717, 102, 868, 28, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4622, 135, 579, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->83451093],
Cell[5204, 154, 251, 8, 70, "Notes",
 CellID->1067943069],
Cell[5458, 164, 1065, 39, 70, "Notes",
 CellID->41491542],
Cell[6526, 205, 151, 4, 70, "Notes",
 CellID->37883947],
Cell[6680, 211, 47, 1, 70, "Notes",
 CellID->529540534]
}, Closed]],
Cell[CellGroupData[{
Cell[6764, 217, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->100873724],
Cell[CellGroupData[{
Cell[7250, 234, 148, 5, 70, "ExampleSection",
 CellID->120716317],
Cell[7401, 241, 132, 6, 70, "ExampleText",
 CellID->630439552],
Cell[7536, 249, 100, 3, 70, "Input",
 CellID->873638675],
Cell[7639, 254, 94, 1, 70, "ExampleText",
 CellID->109948455],
Cell[CellGroupData[{
Cell[7758, 259, 314, 10, 70, "Input",
 CellID->75690307],
Cell[8075, 271, 775, 26, 36, "Output",
 CellID->37785859]
}, Open  ]],
Cell[CellGroupData[{
Cell[8887, 302, 315, 10, 70, "Input",
 CellID->627517166],
Cell[9205, 314, 912, 30, 36, "Output",
 CellID->630054672]
}, Open  ]],
Cell[10132, 347, 71, 1, 70, "ExampleText",
 CellID->74055380],
Cell[CellGroupData[{
Cell[10228, 352, 199, 6, 28, "Input",
 CellID->578584902],
Cell[10430, 360, 3960, 72, 107, "Output",
 CellID->358594059]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[14451, 439, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[14873, 453, 210, 5, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[CellGroupData[{
Cell[15120, 463, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[15542, 477, 167, 3, 70, "MoreAbout",
 CellID->200997734]
}, Open  ]],
Cell[15724, 483, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

