(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     38480,       1248]
NotebookOptionsPosition[     31936,       1032]
NotebookOutlinePosition[     33943,       1083]
CellTagsIndexPosition[     33857,       1078]
WindowTitle->ForceAndMoment - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"3.6 Aerodynamic Coefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicCoefficients"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PMVortexLinearNeumann\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PMVortexLinearNeumann"], \
"\<\"PMDoubletConstantDirichlet\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PMDoubletConstantDirichlet"], \
"\<\"PMDoubletQuadraticDirichlet\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PMDoubletQuadraticDirichlet"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PanelMethods\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/PanelMethods"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/ForceAndMoment\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/ForceAndMoment"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
ForceAndMoment.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/ForceAndMoment", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["ForceAndMoment", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ForceAndMoment", "[", 
       RowBox[{
        StyleBox["singularity", "TI"], ",", 
        StyleBox["strength", "TI"], ",", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"], ",", 
        StyleBox["alpha", "TI"], ",", 
        StyleBox[
         SubscriptBox["r", "c"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes lift and pitching-moment coefficients given \
the name of the singularity, values for the strength parameters, angle(s) of \
attack, locations of panel endpoints, and locations of collocation points."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ForceAndMoment", "[", 
       RowBox[{
        StyleBox["singularity", "TI"], ",", 
        StyleBox["strength", "TI"], ",", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"], ",", 
        StyleBox["alpha", "TI"], ",", 
        StyleBox[
         SubscriptBox["r", "c"], "TI"], 
        StyleBox[",", "TI"], 
        StyleBox["True", "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]includes intermediate results of panel contributions to \
lift and pitching-moment coefficients for single angle of attack."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {None, {None}}, 
   "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->97092080],

Cell[TextData[{
 "Collections of vectors are of the data type ",
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], "InlineFormula"],
 "."
}], "Notes",
 CellID->1067943069],

Cell["Singularity name may be \"Doublet\" or \"Vortex\".", "Notes",
 CellID->43177974],

Cell["\<\
There may be multiple lists of strength parameters, depending on the \
distribution of singularity density.\
\>", "Notes",
 CellID->331145015],

Cell[TextData[{
 "If a single value of ",
 Cell[BoxData[
  StyleBox["\[Alpha]", "TR"]], "InlineFormula"],
 " is input, the function returns the list ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{
    SubscriptBox[
     StyleBox["c", "TI"], "\[ScriptL]"], ",", 
    SubscriptBox[
     StyleBox["c", "TI"], 
     StyleBox["m", "TI"]]}], "}"}]], "InlineFormula"],
 ", where ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["c", "TI"], "\[ScriptL]"]], "InlineFormula"],
 " denotes lift coefficient and ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["c", "TI"], 
   StyleBox["m", "TI"]]], "InlineFormula"],
 ", the pitching-moment coefficient about the origin of the coordinate \
system.  If True is appended to the input parameters, the function output \
includes intermediate results",
 Cell[BoxData[","]],
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{
    SubscriptBox[
     StyleBox["c", "TI"], 
     StyleBox["\[ScriptL]", "TI"]], ",", 
    SubscriptBox[
     StyleBox["c", "TI"], 
     StyleBox["m", "TI"]], ",", 
    SubscriptBox[
     StyleBox[
      RowBox[{"\[CapitalDelta]", 
       StyleBox["c", "TI"]}]], 
     StyleBox["\[ScriptL]", "TI"]], ",", 
    SubscriptBox[
     StyleBox[
      RowBox[{"\[CapitalDelta]", 
       StyleBox["c", "TI"]}]], 
     StyleBox["m", "TI"]]}], "}"}]], "InlineFormula"],
 " where ",
 Cell[BoxData[
  StyleBox["\[CapitalDelta]", "TR"]], "InlineFormula"],
 " preceding the symbols denotes panel contributions to the coefficient."
}], "Notes",
 CellID->186262702],

Cell[TextData[{
 "If a list of angles of attack is input, a straight line is fit to the \
resulting lift and moment coefficients, and the function returns the list ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{
    SubscriptBox[
     StyleBox["c", "TI"], 
     StyleBox["l\[Alpha]", "TI"]], ",", 
    SubscriptBox[
     StyleBox["\[Alpha]", "TR"], 
     StyleBox["0", "TR"]], ",", 
    SubscriptBox[
     StyleBox["x", "TI"], 
     StyleBox["ac", "TI"]], ",", 
    SubscriptBox[
     StyleBox["c", "TI"], 
     StyleBox["m0", "TI"]]}], "}"}]], "InlineFormula"],
 ", where ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["c", "TI"], 
   StyleBox["l\[Alpha]", "TI"]]], "InlineFormula"],
 " is the slope of the lift curve, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["\[Alpha]", "TR"], 
   StyleBox["0", "TR"]]], "InlineFormula"],
 " is the angle of attack at zero lift, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["x", "TI"], 
   StyleBox["ac", "TI"]]], "InlineFormula"],
 " locates the aerodynamic center and ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["c", "TI"], 
   StyleBox["m0", "TI"]]], "InlineFormula"],
 " is the moment coefficient at zero lift (couple).  In the range of ",
 Cell[BoxData[
  StyleBox["\[Alpha]", "TR"]], "InlineFormula"],
 " where ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["c", "TI"], 
   StyleBox["l", "TI"]]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["c", "TI"], 
   StyleBox["m", "TI"]]], "InlineFormula"],
 " vary linearly, the lift and moment coefficients can be written as ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["c", "TI"], 
    StyleBox["l", "TI"]], "=", 
   RowBox[{
    SubscriptBox[
     StyleBox["c", "TI"], 
     StyleBox["l\[Alpha]", "TI"]], 
    RowBox[{"(", 
     RowBox[{
      StyleBox["\[Alpha]", "TR"], "-", 
      SubscriptBox[
       StyleBox["\[Alpha]", "TR"], 
       StyleBox["0", "TR"]]}], ")"}]}]}]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["c", "TI"], 
    StyleBox["m", "TI"]], "=", 
   RowBox[{
    SubscriptBox[
     StyleBox["c", "TI"], 
     StyleBox["m0", "TI"]], "-", 
    RowBox[{
     SubscriptBox[
      StyleBox["c", "TI"], 
      StyleBox["l", "TI"]], " ", 
     SubscriptBox[
      StyleBox["x", "TI"], 
      StyleBox["ac", "TI"]]}]}]}]], "InlineFormula"],
 "."
}], "Notes",
 CellID->351441413],

Cell["New in 1.1   |   Last modified in 1.2", "Notes",
 CellID->118789608]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->803683892],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->234108375],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->218104157],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->326492779],

Cell[TextData[{
 "Use AirfoilFromDataFile to import data from the file for the Eppler-205 \
airfoil, located in the folder Airfoil Data, and rearrange the data to the \
form used in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  Also, display the number of panels specifying the discretized airfoil. "
}], "ExampleText",
 CellID->158695459],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<Aerodynamics\\\\Airfoil Data\\\\e205.dat\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"nP", "=", 
  RowBox[{"Length", "[", 
   RowBox[{"PanelLengths", "[", "rD", "]"}], "]"}]}]}], "Input",
 CellLabel->"In[2]:=",
 CellID->317289154],

Cell[BoxData["60"], "Output",
 ImageSize->{18, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->414264162]
}, Open  ]],

Cell["Plot the discretized airfoil specified by the data.", "ExampleText",
 CellID->235318411],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotAirfoil", "[", "rD", "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->109742180],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3c+LG+cdx3GV/iX9NxZKD8aHkINxwfhgfNkFkVyai31JLu2lvdggkosN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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->430539507]
}, Open  ]],

Cell["\<\
For a specified airfoil and angle of attack, PMVortexLinearNeumann returns \
strength parameters for the vortex panels, locations of collocation points \
and surface speeds at collocation points.  The function ForceAndMoment \
computes lift and pitching moment coefficients for the airfoil.  The \
pitching-moment coefficient is about the leading edge of the airfoil.\
\>", "ExampleText",
 CellID->40832064],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Alpha]", "=", 
   RowBox[{"3.", "Degree"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "rC", ",", "vMagC"}], "}"}], "=", 
   RowBox[{"PMVortexLinearNeumann", "[", 
    RowBox[{"rD", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl", ",", "cm"}], "}"}], "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Vortex\>\"", ",", "a", ",", "b", ",", "rD", ",", "\[Alpha]", ",", 
    "rC"}], "]"}]}]}], "Input",
 CellLabel->"In[4]:=",
 CellID->462013216],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.6359229334317504`", ",", 
   RowBox[{"-", "0.21167313112806962`"}]}], "}"}]], "Output",
 ImageSize->{149, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->426703140]
}, Open  ]],

Cell[TextData[{
 "If a list of angles of attack is input to the model function, lists of \
strength parameters are output, and ForceAndMoment computes aerodynamic \
characteristics of the airfoil in terms of slope of lift curve, angle of \
attack at zero lift, aerodynamic center and pitching moment at zero lift.  \
These computations are based on fits of straight lines to the lift and \
pitching moment coefficients, ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["c", "TI"], 
    StyleBox["l", "TI"]], "=", 
   RowBox[{
    SubscriptBox[
     StyleBox["c", "TI"], 
     StyleBox["l\[Alpha]", "TI"]], 
    RowBox[{"(", 
     RowBox[{
      StyleBox["\[Alpha]", "TR"], "-", 
      SubscriptBox[
       StyleBox["\[Alpha]", "TR"], 
       StyleBox["0", "TR"]]}], ")"}]}]}]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["c", "TI"], 
    StyleBox["m", "TI"]], "=", 
   RowBox[{
    SubscriptBox[
     StyleBox["c", "TI"], 
     StyleBox["m0", "TI"]], "-", 
    RowBox[{
     SubscriptBox[
      StyleBox["x", "TI"], 
      StyleBox["ac", "TI"]], " ", 
     SubscriptBox[
      StyleBox["c", "TI"], 
      StyleBox["l", "TI"]]}]}]}]], "InlineFormula"],
 "."
}], "ExampleText",
 CellID->39200997],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Alpha]L", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "4."}], ",", "0.", ",", "4."}], "}"}], "Degree"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"aL", ",", "bL", ",", "rC", ",", "vMagC"}], "}"}], "=", 
   RowBox[{"PMVortexLinearNeumann", "[", 
    RowBox[{"rD", ",", "\[Alpha]L"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl\[Alpha]", ",", "\[Alpha]0", ",", "xac", ",", "cm0"}], "}"}], 
  "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Vortex\>\"", ",", "aL", ",", "bL", ",", "rD", ",", "\[Alpha]L", ",", 
    "rC"}], "]"}]}]}], "Input",
 CellLabel->"In[5]:=",
 CellID->110914806],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"6.787617175665635`", ",", 
   RowBox[{"-", "0.04129970570828633`"}], ",", "0.25974578404344517`", ",", 
   RowBox[{"-", "0.046417359166541286`"}]}], "}"}]], "Output",
 ImageSize->{298, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->591652800]
}, Open  ]],

Cell["Display the angle of attack at zero lift in degrees.", "ExampleText",
 CellID->42722081],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Alpha]0", "/", "Degree"}]], "Input",
 CellLabel->"In[6]:=",
 CellID->595815551],

Cell[BoxData[
 RowBox[{"-", "2.366298832217161`"}]], "Output",
 ImageSize->{54, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->76581020]
}, Open  ]],

Cell[TextData[{
 "Compute the lift and moment coefficients at ",
 Cell[BoxData[
  StyleBox["\[Alpha]", "TR"]], "InlineFormula"],
 " using the linear characteristics computed from the list of three angles of \
attack."
}], "ExampleText",
 CellID->195026435],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"cl\[Alpha]", 
    RowBox[{"(", 
     RowBox[{"\[Alpha]", "-", "\[Alpha]0"}], " ", ")"}]}], ",", 
   RowBox[{"cm0", "-", 
    RowBox[{"xac", " ", "cl\[Alpha]", 
     RowBox[{"(", 
      RowBox[{"\[Alpha]", "-", "\[Alpha]0"}], " ", ")"}]}]}]}], 
  "}"}]], "Input",
 CellLabel->"In[7]:=",
 CellID->4822966],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.6357253960563514`", ",", 
   RowBox[{"-", "0.211544350601528`"}]}], "}"}]], "Output",
 ImageSize->{149, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->521897331]
}, Open  ]],

Cell["\<\
These values can be compared to those above for a single angle of attack.\
\>", "ExampleText",
 CellID->84929584]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Scope",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->1293636265],

Cell["\<\
Repeat the computation of the basic example for the model \
PMDoubletConstantDirichlet for the same input.\
\>", "ExampleText",
 CellID->794166201],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "aW", ",", "rC", ",", "vMagC"}], "}"}], "=", 
   RowBox[{"PMDoubletConstantDirichlet", "[", 
    RowBox[{"rD", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"cl", ",", "cm"}], "}"}], "=", 
   RowBox[{"ForceAndMoment", "[", 
    RowBox[{
    "\"\<Doublet\>\"", ",", "a", ",", "rD", ",", "\[Alpha]", ",", "rC"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{"cl", ",", "cm"}], "}"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->443450959],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.6481580536096048`", ",", 
   RowBox[{"-", "0.2171192731411692`"}]}], "}"}]], "Output",
 ImageSize->{149, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->137950503]
}, Open  ]],

Cell["For the list of angles of attack.", "ExampleText",
 CellID->62876428],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"aL", ",", "aW", ",", "rC", ",", "vMagC"}], "}"}], "=", 
   RowBox[{"PMDoubletConstantDirichlet", "[", 
    RowBox[{"rD", ",", "\[Alpha]L"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl\[Alpha]", ",", "\[Alpha]0", ",", "xac", ",", "cm0"}], "}"}], 
  "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Doublet\>\"", ",", "aL", ",", "rD", ",", "\[Alpha]L", ",", "rC"}], 
   "]"}]}]}], "Input",
 CellLabel->"In[2]:=",
 CellID->38785546],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"6.721155160182575`", ",", 
   RowBox[{"-", "0.044045836708306335`"}], ",", "0.2569757757791636`", ",", 
   RowBox[{"-", "0.05047908533008495`"}]}], "}"}]], "Output",
 ImageSize->{298, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->7754684]
}, Open  ]],

Cell["\<\
Repeat the computation of the basic for the model PMDoubletQuadraticDirichlet \
for the same input.\
\>", "ExampleText",
 CellID->781746724],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c", ",", "aW", ",", "rC", ",", "vMagC"}], 
    "}"}], "=", 
   RowBox[{"PMDoubletQuadraticDirichlet", "[", 
    RowBox[{"rD", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"cl", ",", "cm"}], "}"}], "=", 
   RowBox[{"ForceAndMoment", "[", 
    RowBox[{
    "\"\<Doublet\>\"", ",", "a", ",", "b", ",", "c", ",", "rD", ",", 
     "\[Alpha]", ",", "rC"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{"cl", ",", "cm"}], "}"}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->161130905],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.6359972898605237`", ",", 
   RowBox[{"-", "0.21880037336843816`"}]}], "}"}]], "Output",
 ImageSize->{135, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->24234104]
}, Open  ]],

Cell["For the list of angles of attack.", "ExampleText",
 CellID->9097697],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"aL", ",", "bL", ",", "cL", ",", "aW", ",", "rC", ",", "vMagC"}], 
    "}"}], "=", 
   RowBox[{"PMDoubletQuadraticDirichlet", "[", 
    RowBox[{"rD", ",", "\[Alpha]L"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl\[Alpha]", ",", "\[Alpha]0", ",", "xac", ",", "cm0"}], "}"}], 
  "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Doublet\>\"", ",", "aL", ",", "bL", ",", "cL", ",", "rD", ",", 
    "\[Alpha]L", ",", "rC"}], "]"}]}]}], "Input",
 CellLabel->"In[4]:=",
 CellID->85011794],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"6.7882366327168135`", ",", 
   RowBox[{"-", "0.041302109302870205`"}], ",", "0.2601955124583561`", ",", 
   RowBox[{"-", "0.05324000902023526`"}]}], "}"}]], "Output",
 ImageSize->{284, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->63521236]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Generalizations & Extensions",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->1020263627],

Cell["\<\
If the contributions to lift and moment coefficient of each panel are needed, \
these intermediate results can be included in the output.\
\>", "ExampleText",
 CellID->845325350],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"cl", ",", "cm", ",", 
     RowBox[{"{", 
      RowBox[{"\[CapitalDelta]cl", ",", "\[CapitalDelta]cm"}], "}"}]}], "}"}],
    "=", 
   RowBox[{"ForceAndMoment", "[", 
    RowBox[{
    "\"\<Vortex\>\"", ",", "a", ",", "b", ",", "rD", ",", "\[Alpha]", ",", 
     "rC", ",", "True"}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->33224848],

Cell["\<\
Panel contributions are simple lists of length equal to the number of panels.\
\>", "ExampleText",
 CellID->177558706],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Length", "/@", 
   RowBox[{"{", 
    RowBox[{"\[CapitalDelta]cl", ",", "\[CapitalDelta]cm"}], "}"}]}], 
  "\[Equal]", 
  RowBox[{"{", 
   RowBox[{"nP", ",", "nP"}], "}"}]}]], "Input",
 CellLabel->"In[2]:=",
 CellID->286306963],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->4968380]
}, Open  ]],

Cell["\<\
Airfoil coefficients are obtained by summing contributions of panels.\
\>", "ExampleText",
 CellID->215360752],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl", ",", "cm"}], "}"}], "\[Equal]", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Plus", "@@", "\[CapitalDelta]cl"}], ",", 
    RowBox[{"Plus", "@@", "\[CapitalDelta]cm"}]}], "}"}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->652262456],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->276516004]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[{
 Cell[BoxData[
  StyleBox[
   ButtonBox["PMVortexLinearNeumann",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/PMVortexLinearNeumann"],
   FontFamily->"Verdana"]], "InlineFormula"],
 " . ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["PMDoubletConstantDirichlet",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/PMDoubletConstantDirichlet"],
   FontFamily->"Verdana"]], "InlineFormula"],
 " . ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["PMDoubletQuadraticDirichlet",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/PMDoubletQuadraticDirichlet"],
   FontFamily->"Verdana"]], "InlineFormula"]
}], "SeeAlso",
 CellID->402133314]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["3.6 Aerodynamic Coefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicCoefficients"]], "Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["PanelMethods",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/PanelMethods"]], "MoreAbout",
 CellID->126385335]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"ForceAndMoment - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 9, 59, 10.1100000}", "context" -> 
    "Aerodynamics`PanelMethods`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Package Paclet Symbol", "language" -> "en", "paclet" -> 
    "Aerodynamics Package", "status" -> "None", "summary" -> 
    "ForceAndMoment[singularity, strength, r_p, alpha, r_c] computes lift and \
pitching-moment coefficients given the name of the singularity, values for \
the strength parameters, angle(s) of attack, locations of panel endpoints, \
and locations of collocation points.ForceAndMoment[singularity, strength, \
r_p, alpha, r_c, True] includes intermediate results of panel contributions \
to lift and pitching-moment coefficients for single angle of attack.", 
    "synonyms" -> {}, "title" -> "ForceAndMoment", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/ForceAndMoment"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[10772, 339, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->803683892]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 33713, 1071}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[607, 21, 76, 1, 70, "SymbolColorBar"],
Cell[686, 24, 191, 5, 70, "LinkTrail"],
Cell[880, 31, 3334, 76, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4239, 111, 57, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4299, 114, 1388, 36, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5724, 155, 579, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->97092080],
Cell[6306, 174, 251, 8, 70, "Notes",
 CellID->1067943069],
Cell[6560, 184, 86, 1, 70, "Notes",
 CellID->43177974],
Cell[6649, 187, 152, 4, 70, "Notes",
 CellID->331145015],
Cell[6804, 193, 1491, 50, 70, "Notes",
 CellID->186262702],
Cell[8298, 245, 2360, 86, 70, "Notes",
 CellID->351441413],
Cell[10661, 333, 74, 1, 70, "Notes",
 CellID->118789608]
}, Closed]],
Cell[CellGroupData[{
Cell[10772, 339, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->803683892],
Cell[CellGroupData[{
Cell[11258, 356, 148, 5, 70, "ExampleSection",
 CellID->234108375],
Cell[11409, 363, 132, 6, 70, "ExampleText",
 CellID->218104157],
Cell[11544, 371, 100, 3, 70, "Input",
 CellID->326492779],
Cell[11647, 376, 351, 8, 70, "ExampleText",
 CellID->158695459],
Cell[CellGroupData[{
Cell[12023, 388, 344, 11, 70, "Input",
 CellID->317289154],
Cell[12370, 401, 159, 5, 36, "Output",
 CellID->414264162]
}, Open  ]],
Cell[12544, 409, 94, 1, 70, "ExampleText",
 CellID->235318411],
Cell[CellGroupData[{
Cell[12663, 414, 108, 3, 28, "Input",
 CellID->109742180],
Cell[12774, 419, 5001, 89, 201, "Output",
 CellID->430539507]
}, Open  ]],
Cell[17790, 511, 417, 7, 70, "ExampleText",
 CellID->40832064],
Cell[CellGroupData[{
Cell[18232, 522, 603, 18, 70, "Input",
 CellID->462013216],
Cell[18838, 542, 260, 8, 36, "Output",
 CellID->426703140]
}, Open  ]],
Cell[19113, 553, 1245, 41, 70, "ExampleText",
 CellID->39200997],
Cell[CellGroupData[{
Cell[20383, 598, 740, 23, 70, "Input",
 CellID->110914806],
Cell[21126, 623, 337, 9, 36, "Output",
 CellID->591652800]
}, Open  ]],
Cell[21478, 635, 94, 1, 70, "ExampleText",
 CellID->42722081],
Cell[CellGroupData[{
Cell[21597, 640, 105, 3, 70, "Input",
 CellID->595815551],
Cell[21705, 645, 191, 6, 36, "Output",
 CellID->76581020]
}, Open  ]],
Cell[21911, 654, 256, 7, 70, "ExampleText",
 CellID->195026435],
Cell[CellGroupData[{
Cell[22192, 665, 355, 12, 70, "Input",
 CellID->4822966],
Cell[22550, 679, 258, 8, 36, "Output",
 CellID->521897331]
}, Open  ]],
Cell[22823, 690, 123, 3, 70, "ExampleText",
 CellID->84929584]
}, Open  ]],
Cell[CellGroupData[{
Cell[22983, 698, 221, 7, 70, "ExampleSection",
 CellID->1293636265],
Cell[23207, 707, 157, 4, 70, "ExampleText",
 CellID->794166201],
Cell[CellGroupData[{
Cell[23389, 715, 598, 18, 70, "Input",
 CellID->443450959],
Cell[23990, 735, 259, 8, 36, "Output",
 CellID->137950503]
}, Open  ]],
Cell[24264, 746, 75, 1, 70, "ExampleText",
 CellID->62876428],
Cell[CellGroupData[{
Cell[24364, 751, 544, 16, 70, "Input",
 CellID->38785546],
Cell[24911, 769, 334, 9, 36, "Output",
 CellID->7754684]
}, Open  ]],
Cell[25260, 781, 150, 4, 70, "ExampleText",
 CellID->781746724],
Cell[CellGroupData[{
Cell[25435, 789, 645, 19, 70, "Input",
 CellID->161130905],
Cell[26083, 810, 259, 8, 36, "Output",
 CellID->24234104]
}, Open  ]],
Cell[26357, 821, 74, 1, 70, "ExampleText",
 CellID->9097697],
Cell[CellGroupData[{
Cell[26456, 826, 595, 17, 70, "Input",
 CellID->85011794],
Cell[27054, 845, 336, 9, 36, "Output",
 CellID->63521236]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[27439, 860, 244, 7, 70, "ExampleSection",
 CellID->1020263627],
Cell[27686, 869, 188, 4, 70, "ExampleText",
 CellID->845325350],
Cell[27877, 875, 412, 13, 70, "Input",
 CellID->33224848],
Cell[28292, 890, 128, 3, 70, "ExampleText",
 CellID->177558706],
Cell[CellGroupData[{
Cell[28445, 897, 261, 9, 70, "Input",
 CellID->286306963],
Cell[28709, 908, 159, 5, 36, "Output",
 CellID->4968380]
}, Open  ]],
Cell[28883, 916, 120, 3, 70, "ExampleText",
 CellID->215360752],
Cell[CellGroupData[{
Cell[29028, 923, 285, 9, 70, "Input",
 CellID->652262456],
Cell[29316, 934, 161, 5, 36, "Output",
 CellID->276516004]
}, Open  ]]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[29538, 946, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[29958, 960, 687, 22, 70, "SeeAlso",
 CellID->402133314]
}, Open  ]],
Cell[CellGroupData[{
Cell[30682, 987, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[31104, 1001, 182, 4, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[CellGroupData[{
Cell[31323, 1010, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[31745, 1024, 149, 3, 70, "MoreAbout",
 CellID->126385335]
}, Open  ]],
Cell[31909, 1030, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

