(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    258426,       5524]
NotebookOptionsPosition[    249514,       5234]
NotebookOutlinePosition[    251658,       5287]
CellTagsIndexPosition[    251571,       5282]
WindowTitle->ICDoubletConstant - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"2.1 Introduction to Influence Coefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"], "\
\<\"2.3 Isolated Panel\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IsolatedPanel"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"ICDoubletPoint\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICDoubletPoint"], "\<\"ICDoubletLinear\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICDoubletLinear"], \
"\<\"ICDoubletQuadratic\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICDoubletQuadratic"], "\<\"ICSourceConstant\"\
\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICSourceConstant"], \
"\<\"ICVortexConstant\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICVortexConstant"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"InfluenceCoefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/InfluenceCoefficients"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/ICDoubletConstant\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/ICDoubletConstant"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
ICDoubletConstant.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/ICDoubletConstant", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["ICDoubletConstant", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICDoubletConstant", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes velocity influence coefficients at field \
points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     " due to connected doublet panels of constant strength whose endpoints \
are located by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICPhiDoubletConstant", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes velocity-potential influence coefficients at \
field points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     ", due to connected doublet panels of constant strength whose endpoints \
are located by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICPsiDoubletConstant", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]",
     "computes stream-function influence coefficients at field points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     ", due to connected doublet panels of constant strength whose endpoints \
are located by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, 
   "Rows" -> {None, None, {None}}, "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->261953622],

Cell[TextData[{
 "Input arguments are collections of vectors of the data type ",
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], "InlineFormula"],
 ".",
 "    ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation may be used for single vectors."
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 "The doublet density for a doublet panel of constant strength is written as ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["\[Mu]", "TR"], 
    RowBox[{"(", 
     StyleBox["x", "TI"], ")"}]}], "=", 
   StyleBox["a", "TI"]}]], "InlineFormula"],
 ", where ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " is the strength parameter for the panel.  For collections of panels, \
strength parameters are written in a list ordered according to the clockwise \
numbering convention for panels."
}], "Notes",
 CellID->476702286],

Cell[TextData[{
 "Output for velocity influence coefficients is a collection of vectors of \
the data type CartesianVectors.  Influence coefficients for velocity \
potential and stream function are lists.  For the special case of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation being used to specify a single field point, ",
 "output is simplified as illustrated in the examples",
 "."
}], "Notes",
 CellID->30234373],

Cell[TextData[{
 "The velocity, velocity potential and stream function for a panel are \
written as products of a geometric influence coefficient and the strength \
parameter.\t\t\t\t",
 "\t\t\t\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["V", "TI"], "=", 
   RowBox[{
    StyleBox["a", "TI"], " ", 
    StyleBox["ic", "TI"]}]}]], "InlineFormula"],
 "\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Phi]", "TR"], "=", 
   RowBox[{
    StyleBox["a", "TI"], " ", 
    StyleBox["ic\[Phi]", "TI"]}]}]], "InlineFormula"],
 "\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Psi]", "TR"], "=", 
   RowBox[{
    StyleBox["a", "TI"], " ", 
    StyleBox["ic\[Psi]", "TI"]}]}]], "InlineFormula"]
}], "Notes",
 CellID->86946217],

Cell["New in 1.0   |   Last modified in 1.2", "Notes",
 CellID->529540534]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->361088721],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->541600658],

Cell[TextData[{
 "Load ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->513831186],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->66474307],

Cell[TextData[{
 "This example demonstrates some properties of a single panel.  Algebraic \
equations for influence coefficients are simplest in a local coordinate \
system where the panel extends from the origin along the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 " axis to ",
 Cell[BoxData[
  StyleBox["\[Delta]", "TR"]], "InlineFormula"],
 ", the length of the panel."
}], "ExampleText",
 CellID->659456463],

Cell[TextData[{
 "Specify an arbitrary field point and the two panel endpoints.  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation is used for a single vector."
}], "ExampleText",
 CellID->441523031],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"rF", ",", "rP"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"x", ",", "z"}], "}"}], ",", 
     RowBox[{"MakeCartesianVectors", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "\[Delta]"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}]}], 
  ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->64953589],

Cell["Compute the velocity influence coefficient.", "ExampleText",
 CellID->326501090],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"icu", ",", "icw"}], "}"}], "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"ICDoubletConstant", "[", 
     RowBox[{"rF", ",", "rP"}], "]"}], ",", 
    RowBox[{"\[Delta]", ">", "0"}]}], "]"}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->560964283],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"z", " ", "\[Delta]", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "2"}], " ", "x"}], "+", "\[Delta]"}], ")"}]}], 
     RowBox[{"2", " ", "\[Pi]", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"], "-", 
        RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
        SuperscriptBox["\[Delta]", "2"]}], ")"}]}]]}], ",", 
   FractionBox[
    RowBox[{"\[Delta]", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["x", "2"]}], "+", 
       SuperscriptBox["z", "2"], "+", 
       RowBox[{"x", " ", "\[Delta]"}]}], ")"}]}], 
    RowBox[{"2", " ", "\[Pi]", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"], "-", 
       RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
       SuperscriptBox["\[Delta]", "2"]}], ")"}]}]]}], "}"}]], "Output",
 ImageSize->{436, 41},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->506532777]
}, Open  ]],

Cell["Compute the velocity-potential influence coefficient.", "ExampleText",
 CellID->44302699],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ic\[Phi]", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"ICPhiDoubletConstant", "[", 
     RowBox[{"rF", ",", "rP"}], "]"}], ",", 
    RowBox[{"\[Delta]", ">", "0"}]}], "]"}]}]], "Input",
 CellLabel->"In[4]:=",
 CellID->575890672],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"ArcTan", "[", 
    RowBox[{"x", ",", "z"}], "]"}], "-", 
   RowBox[{"ArcTan", "[", 
    RowBox[{
     RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}]}], 
  RowBox[{"2", " ", "\[Pi]"}]]], "Output",
 ImageSize->{204, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->163445906]
}, Open  ]],

Cell[TextData[{
 "Display the algebraic equations for the velocity-potential influence \
coefficient, replacing ArcTan function by named angles (See ",
 ButtonBox["2.1 Introduction to AirfoilGeometry",
  BaseStyle->"Hyperlink",
  ButtonData->{
    FrontEnd`FileName[{
      ParentDirectory[
       ParentDirectory[]], "Tutorials"}, 
     "IntroductionToInfluenceCoefficients.nb", CharacterEncoding -> 
     "WindowsANSI"], "geom"}],
 " for sketch defining angles).  Although the substitution of named angles is \
made to simplify the appearance of equations, note that the original \
equations are maintained for subsequent computations."
}], "ExampleText",
 CellID->120984796],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ic\[Phi]", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
     "\[Theta]2"}]}], "}"}]}]], "Input",
 CellLabel->"In[5]:=",
 CellID->23616962],

Cell[BoxData[
 FractionBox[
  RowBox[{"\[Theta]1", "-", "\[Theta]2"}], 
  RowBox[{"2", " ", "\[Pi]"}]]], "Output",
 ImageSize->{50, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->105038943]
}, Open  ]],

Cell["Compute the stream-function influence coefficient.", "ExampleText",
 CellID->472419597],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ic\[Psi]", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"ICPsiDoubletConstant", "[", 
     RowBox[{"rF", ",", "rP"}], "]"}], ",", 
    RowBox[{"\[Delta]", ">", "0"}]}], "]"}]}]], "Input",
 CellLabel->"In[6]:=",
 CellID->8664355],

Cell[BoxData[
 FractionBox[
  RowBox[{"Log", "[", 
   FractionBox[
    RowBox[{
     SuperscriptBox["x", "2"], "+", 
     SuperscriptBox["z", "2"], "-", 
     RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
     SuperscriptBox["\[Delta]", "2"]}], 
    RowBox[{
     SuperscriptBox["x", "2"], "+", 
     SuperscriptBox["z", "2"]}]], "]"}], 
  RowBox[{"4", " ", "\[Pi]"}]]], "Output",
 ImageSize->{115, 46},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->315545623]
}, Open  ]],

Cell["\<\
Write the velocity components, velocity potential and stream function as \
products of the strength parameter and the geometric influence coefficient.\
\>", "ExampleText",
 CellID->9427774],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"u", ",", "w", ",", "\[Phi]", ",", "\[Psi]"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", " ", "icu"}], ",", 
     RowBox[{"a", " ", "icw"}], ",", 
     RowBox[{"a", " ", "ic\[Phi]"}], ",", 
     RowBox[{"a", " ", "ic\[Psi]"}]}], "}"}]}], ";"}]], "Input",
 CellLabel->"In[7]:=",
 CellID->258291294],

Cell[TextData[{
 "Verify that the relations between velocity, velocity potential and stream \
function are satisfied.\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["u", "TI"], "=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Phi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["x", "TI"]}]], "=", 
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Psi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["z", "TI"]}]]}]}]], "InlineFormula"],
 "\t\t\t",
 Cell[BoxData[
  RowBox[{"w", "=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Phi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["z", "TI"]}]], "=", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{"\[PartialD]", 
       StyleBox["\[Psi]", "TR"]}], 
      RowBox[{"\[PartialD]", 
       StyleBox["x", "TI"]}]]}]}]}]], "InlineFormula"],
 "\t\t\t\t"
}], "ExampleText",
 CellID->78226744],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"u", "\[Equal]", 
     RowBox[{"D", "[", 
      RowBox[{"\[Phi]", ",", "x"}], "]"}]}], ",", 
    RowBox[{"u", "\[Equal]", 
     RowBox[{"D", "[", 
      RowBox[{"\[Psi]", ",", "z"}], "]"}]}]}], "}"}], "//", 
  "Simplify"}]], "Input",
 CellLabel->"In[8]:=",
 CellID->13857323],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->28233041]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"w", "\[Equal]", 
     RowBox[{"D", "[", 
      RowBox[{"\[Phi]", ",", "z"}], "]"}]}], ",", 
    RowBox[{"w", "\[Equal]", 
     RowBox[{"-", 
      RowBox[{"D", "[", 
       RowBox[{"\[Psi]", ",", "x"}], "]"}]}]}]}], "}"}], "//", 
  "Simplify"}]], "Input",
 CellLabel->"In[9]:=",
 CellID->692553449],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->350091554]
}, Open  ]],

Cell[TextData[{
 "Flow visualization provides plausibility check on the computed flow field.  \
Set ",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Delta]", "TR"], "=", "1"}]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  RowBox[{
   StyleBox["a", "TI"], " ", "=", " ", "1"}]], "InlineFormula"],
 "  to provide unit integrated panel strength, ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", 
     StyleBox["\[Delta]", "TR"]], 
    RowBox[{"\[Mu]", 
     RowBox[{"(", 
      StyleBox["x", "TI"], ")"}], 
     RowBox[{"\[DifferentialD]", 
      StyleBox["x", "TI"]}]}]}], "=", "1"}]], "InlineFormula"],
 ".    Circular regions near the edges of the panel are excluded from the \
velocity plot to avoid showing vectors of very large magnitude.  Also, \
nonlinear scaling is used for magnitudes of vectors to enhance visibility in \
the far field."
}], "ExampleText",
 CellID->78531558],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xMin", ",", "xMax", ",", "zMin", ",", "zMax"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "2", ",", 
     RowBox[{"-", "1.2"}], ",", "1.2"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"uPlot", ",", "wPlot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"u", ",", "w"}], "}"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", "\[Rule]", "1"}], ",", 
      RowBox[{"\[Delta]", "\[Rule]", "1"}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Phi]Plot", ",", "\[Psi]Plot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\[Phi]", ",", "\[Psi]"}], "}"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", "\[Rule]", "1"}], ",", 
      RowBox[{"\[Delta]", "\[Rule]", "1"}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gV", "=", 
   RowBox[{"VectorPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"uPlot", ",", "wPlot"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"RegionFunction", "\[Rule]", 
      RowBox[{"Function", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"x", ",", "z"}], "}"}], ",", 
        RowBox[{
         RowBox[{"0.01", "<", 
          RowBox[{
           RowBox[{"x", "^", "2"}], "+", 
           RowBox[{"z", "^", "2"}]}]}], " ", "&&", " ", 
         RowBox[{"0.01", "<", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"x", "-", "1"}], ")"}], "^", "2"}], "+", 
           RowBox[{"z", "^", "2"}]}]}]}]}], "]"}]}], ",", 
     RowBox[{"VectorColorFunction", "->", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"RGBColor", "[", 
         RowBox[{"0", ",", "1", ",", "0"}], "]"}], "&"}], ")"}]}], ",", 
     RowBox[{"VectorPoints", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"18", ",", "18"}], "}"}]}], ",", 
     RowBox[{"VectorScale", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Large", ",", "Automatic", ",", 
        RowBox[{
         RowBox[{"Sqrt", "[", "#5", "]"}], "&"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Phi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Phi]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "13"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Red"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Psi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "13"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Blue"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gPanel", "=", 
   RowBox[{"Graphics", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"GrayLevel", "[", "0", "]"}], ",", 
      RowBox[{"AbsoluteThickness", "[", "3", "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"Transpose", "[", 
        RowBox[{"Components", "[", 
         RowBox[{"rP", "/.", 
          RowBox[{"\[Delta]", "\[Rule]", "1.0"}]}], "]"}], "]"}], "]"}]}], 
     "}"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gV", ",", "g\[Phi]", ",", "g\[Psi]", ",", "gPanel", ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
   RowBox[{"FrameTicks", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.0"}], ",", "2.0", ",", "0.25"}], "]"}], "]"}], ",", 
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.2"}], ",", "1.2", ",", "0.2"}], "]"}], "]"}], ",", 
      "None", ",", "None"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", " ", "72"}]}], ",", 
   RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellLabel->"In[10]:=",
 CellID->1746043248],

Cell[BoxData[
 GraphicsBox[{{{}, {
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.008757479559774926, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0292847608124003`, -1.1954445927625155`}, \
{-0.9707152391875996, -1.2045554072374844`}}]}}, 
      {Arrowheads[{{0.009207562759918983, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0300317531201706`, -1.0505140900952905`}, \
{-0.9699682468798294, -1.067132968728239}}]}}, 
      {Arrowheads[{{0.009672561889281542, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0301733316798953`, -0.9049557519942684}, \
{-0.9698266683201047, -0.9303383656527907}}]}}, 
      {Arrowheads[{{0.010142411369821677`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0294355199651162`, -0.7588162722205656}, \
{-0.9705644800348837, -0.7941249042500229}}]}}, 
      {Arrowheads[{{0.010601410454064824`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0275035542268465`, -0.6122566452830468}, \
{-0.9724964457731534, -0.6583315900110711}}]}}, 
      {Arrowheads[{{0.011027071558958663`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0240732087329942`, -0.46560292007593257`}, \
{-0.9759267912670059, -0.5226323740417148}}]}}, 
      {Arrowheads[{{0.011390304415636945`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0189509663535345`, -0.319374432145047}, \
{-0.9810490336464656, -0.38650792079612983`}}]}}, 
      {Arrowheads[{{0.011658373599048815`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0121924370725626`, -0.17424175030783912`}, \
{-0.9878075629274373, -0.24928766145686715`}}]}}, 
      {Arrowheads[{{0.011801480861586295`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0042156532544741`, -0.030872924176476788`}, \
{-0.9957843467455257, -0.1103035464117589}}]}}, 
      {Arrowheads[{{0.011801480861586298`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9957843467455259, 
         0.11030354641175852`}, {-1.0042156532544741`, 
         0.030872924176476392`}}]}}, 
      {Arrowheads[{{0.011658373599048812`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9878075629274374, 
         0.24928766145686676`}, {-1.0121924370725626`, 
         0.17424175030783873`}}]}}, 
      {Arrowheads[{{0.011390304415636945`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9810490336464656, 
         0.38650792079612944`}, {-1.0189509663535345`, 
         0.3193744321450466}}]}}, 
      {Arrowheads[{{0.011027071558958679`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9759267912670058, 
         0.5226323740417145}, {-1.0240732087329942`, 0.4656029200759322}}]}}, 
      {Arrowheads[{{0.010601410454064823`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9724964457731535, 
         0.6583315900110708}, {-1.0275035542268465`, 0.6122566452830464}}]}}, 
      {Arrowheads[{{0.010142411369821663`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9705644800348838, 
         0.7941249042500225}, {-1.0294355199651162`, 0.7588162722205651}}]}}, 
      {Arrowheads[{{0.009672561889281542, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9698266683201047, 
         0.9303383656527902}, {-1.0301733316798953`, 0.9049557519942678}}]}}, 
      {Arrowheads[{{0.009207562759918983, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9699682468798294, 
         1.0671329687282385`}, {-1.0300317531201706`, 1.05051409009529}}]}}, 
      {Arrowheads[{{0.008757479559774909, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9707152391875997, 
         1.204555407237484}, {-1.0292847608124003`, 1.195444592762515}}]}}, 
      {Arrowheads[{{0.009378057649365223, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.855260525540082, -1.1993833517758685`}, \
{-0.7917982979893297, -1.2006166482241316`}}]}}, 
      {Arrowheads[{{0.009939135637952038, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8568593732459084, -1.0542972383464162`}, \
{-0.7901994502835031, -1.0633498204771132`}}]}}, 
      {Arrowheads[{{0.010536127696573524`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8579393687370748, -0.9083024517049932}, \
{-0.7891194547923369, -0.9269916659420656}}]}}, 
      {Arrowheads[{{0.01116046987803137, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8581343297764338, -0.7613376362105022}, \
{-0.7889244937529779, -0.7916035402600862}}]}}, 
      {Arrowheads[{{0.011794938313547856`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8569510113441722, -0.6134696755842207}, \
{-0.7901078121852395, -0.6571185597098974}}]}}, 
      {Arrowheads[{{0.01240945952606158, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8538023337205408, -0.4650107932079543}, \
{-0.793256489808871, -0.5232245009096932}}]}}, 
      {Arrowheads[{{0.012957745607494619`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8481475980188165, -0.3166521463626772}, \
{-0.7989112255105951, -0.38923020657849966`}}]}}, 
      {Arrowheads[{{0.0133790157522867, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8397883226400458, -0.16950759579004993`}, \
{-0.8072705008893659, -0.25402181597465634`}}]}}, 
      {Arrowheads[{{0.013610467207734915`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8292352151334533, -0.02488263704994343}, \
{-0.8178236083959584, -0.11629383353829226`}}]}}, 
      {Arrowheads[{{0.013610467207734915`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8178236083959584, 
         0.11629383353829187`}, {-0.8292352151334533, 
         0.024882637049943035`}}]}}, 
      {Arrowheads[{{0.01337901575228671, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8072705008893659, 
         0.254021815974656}, {-0.8397883226400458, 0.1695075957900495}}]}}, 
      {Arrowheads[{{0.012957745607494615`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7989112255105952, 
         0.38923020657849927`}, {-0.8481475980188165, 
         0.31665214636267675`}}]}}, 
      {Arrowheads[{{0.012409459526061567`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.793256489808871, 
         0.5232245009096927}, {-0.8538023337205407, 0.46501079320795385`}}]}}, 
      {Arrowheads[{{0.011794938313547856`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7901078121852395, 
         0.657118559709897}, {-0.8569510113441722, 0.6134696755842203}}]}}, 
      {Arrowheads[{{0.011160469878031378`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7889244937529779, 
         0.7916035402600858}, {-0.8581343297764338, 0.7613376362105018}}]}}, 
      {Arrowheads[{{0.010536127696573524`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7891194547923369, 
         0.9269916659420653}, {-0.8579393687370748, 0.9083024517049929}}]}}, 
      {Arrowheads[{{0.009939135637952038, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7901994502835032, 
         1.0633498204771128`}, {-0.8568593732459086, 1.0542972383464158`}}]}}, 
      {Arrowheads[{{0.00937805764936524, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7917982979893295, 
         1.200616648224131}, {-0.855260525540082, 1.1993833517758679`}}]}}, 
      {Arrowheads[{{0.010028582045836449`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6806880835238145, -1.2045718209665592`}, \
{-0.613429563535009, -1.1954281790334407`}}]}}, 
      {Arrowheads[{{0.010723839177871548`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6833408829250238, -1.059650470936451}, \
{-0.6107767641337996, -1.0579965878870783`}}]}}, 
      {Arrowheads[{{0.011488462175840704`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6857203362016645, -0.9135394319676216}, \
{-0.6083973108571589, -0.9217546856794375}}]}}, 
      {Arrowheads[{{0.012321479593959411`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6874169563504872, -0.7659843725279379}, \
{-0.6067006907083362, -0.7869568039426507}}]}}, 
      {Arrowheads[{{0.01321166464844315, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6877654327105881, -0.6168002507598085}, \
{-0.6063522143482353, -0.6537879845343095}}]}}, 
      {Arrowheads[{{0.014127437282837505`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6857701450723328, -0.4660602105901082}, \
{-0.6083475019864906, -0.5221750835275392}}]}}, 
      {Arrowheads[{{0.015001920477789359`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6801689145901553, -0.3144542330153651}, \
{-0.6139487324686681, -0.3914281199258117}}]}}, 
      {Arrowheads[{{0.015720715902700525`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6699194436367744, -0.16372483867098367`}, \
{-0.624198203422049, -0.25980457309372257`}}]}}, 
      {Arrowheads[{{0.016136752994528315`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6553001714264801, -0.016603884624773864`}, \
{-0.6388174756323433, -0.12457258596346182`}}]}}, 
      {Arrowheads[{{0.016136752994528315`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6388174756323433, 
         0.12457258596346144`}, {-0.6553001714264801, 
         0.01660388462477346}}]}}, 
      {Arrowheads[{{0.01572071590270053, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.624198203422049, 
         0.25980457309372224`}, {-0.6699194436367742, 
         0.16372483867098325`}}]}}, 
      {Arrowheads[{{0.01500192047778937, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6139487324686681, 
         0.3914281199258114}, {-0.6801689145901553, 0.3144542330153647}}]}}, 
      {Arrowheads[{{0.014127437282837527`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6083475019864906, 
         0.5221750835275388}, {-0.6857701450723329, 0.46606021059010777`}}]}}, 
      {Arrowheads[{{0.013211664648443169`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6063522143482353, 
         0.6537879845343091}, {-0.6877654327105882, 0.6168002507598079}}]}}, 
      {Arrowheads[{{0.012321479593959411`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6067006907083362, 
         0.7869568039426502}, {-0.6874169563504872, 0.7659843725279375}}]}}, 
      {Arrowheads[{{0.011488462175840704`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6083973108571589, 
         0.9217546856794371}, {-0.6857203362016645, 0.9135394319676211}}]}}, 
      {Arrowheads[{{0.010723839177871548`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6107767641337996, 
         1.057996587887078}, {-0.6833408829250238, 1.0596504709364507`}}]}}, 
      {Arrowheads[{{0.010028582045836464`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6134295635350089, 
         1.1954281790334402`}, {-0.6806880835238145, 1.2045718209665588`}}]}}, 
      {Arrowheads[{{0.010686841592073983`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5050229242208859, -1.2110567474129683`}, \
{-0.43615354636734915`, -1.1889432525870318`}}]}}, 
      {Arrowheads[{{0.01153588055756898, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5088040073459758, -1.0668012326683816`}, \
{-0.43237246324225925`, -1.0508458261551479`}}]}}, 
      {Arrowheads[{{0.012502257682518812`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5127489356390823, -0.9211981232504092}, \
{-0.4284275349491527, -0.9140959943966497}}]}}, 
      {Arrowheads[{{0.01360355016055833, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.516544910205147, -0.7737528564958358}, \
{-0.4246315603830881, -0.7791883199747528}}]}}, 
      {Arrowheads[{{0.0148533176118672, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5195336530200181, -0.6238460536133018}, \
{-0.421642817568217, -0.6467421816808162}}]}}, 
      {Arrowheads[{{0.016246603848198992`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5204060805549432, -0.47085494182591875`}, \
{-0.4207703900332919, -0.5173803522917287}}]}}, 
      {Arrowheads[{{0.01772249532044765, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5167777864460723, -0.31468316440533306`}, \
{-0.4243986841421628, -0.39119918853584384`}}]}}, 
      {Arrowheads[{{0.01909020570483918, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5052261872655951, -0.15723034109493597`}, \
{-0.43595028332263996`, -0.2662990706697703}}]}}, 
      {Arrowheads[{{0.01997075769135485, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.48380675289395847`, -0.004308718763198563}, \
{-0.4573697176942766, -0.13686775182503713`}}]}}, 
      {Arrowheads[{{0.019970757691354847`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.4573697176942767, 
         0.13686775182503674`}, {-0.48380675289395836`, 
         0.0043087187631981605`}}]}}, 
      {Arrowheads[{{0.01909020570483917, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.43595028332264, 
         0.26629907066976993`}, {-0.505226187265595, 
         0.15723034109493556`}}]}}, 
      {Arrowheads[{{0.01772249532044766, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.4243986841421628, 
         0.39119918853584346`}, {-0.5167777864460723, 
         0.31468316440533256`}}]}}, 
      {Arrowheads[{{0.016246603848198978`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.4207703900332919, 
         0.5173803522917283}, {-0.5204060805549431, 0.4708549418259183}}]}}, 
      {Arrowheads[{{0.0148533176118672, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.421642817568217, 
         0.6467421816808158}, {-0.5195336530200181, 0.6238460536133014}}]}}, 
      {Arrowheads[{{0.013603550160558338`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.424631560383088, 
         0.7791883199747524}, {-0.516544910205147, 0.7737528564958354}}]}}, 
      {Arrowheads[{{0.012502257682518827`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.4284275349491527, 
         0.9140959943966493}, {-0.5127489356390824, 0.9211981232504087}}]}}, 
      {Arrowheads[{{0.011535880557568996`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.43237246324225925`, 
         1.0508458261551474`}, {-0.5088040073459759, 1.0668012326683811`}}]}}, 
      {Arrowheads[{{0.010686841592073983`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.43615354636734915`, 
         1.1889432525870312`}, {-0.5050229242208859, 1.2110567474129676`}}]}}, 
      {Arrowheads[{{0.011319996057258395`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.327601598160624, -1.218611921694494}, \
{-0.26063369595702274`, -1.181388078305506}}]}}, 
      {Arrowheads[{{0.01233150372043959, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.33231104737175454`, -1.0756411995084052`}, \
{-0.25592424674589215`, -1.0420058593151242`}}]}}, 
      {Arrowheads[{{0.013519431694777389`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3377358341454298, -0.9314574135953002}, \
{-0.250499459972217, -0.9038367040517588}}]}}, 
      {Arrowheads[{{0.01493258633801389, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.34385088545849796`, -0.7854348509591862}, \
{-0.24438440865914882`, -0.7675063255114023}}]}}, 
      {Arrowheads[{{0.016638922272212447`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.35041228615552894`, -0.6365759983178482}, \
{-0.2378230079621178, -0.6340122369762697}}]}}, 
      {Arrowheads[{{0.018729192987741367`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.35656358383148956`, -0.4832584418281626}, \
{-0.23167171028615716`, -0.5049768522894849}}]}}, 
      {Arrowheads[{{0.0212932493854948, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.35966343323796923`, -0.32300050870974406`}, \
{-0.2285718608796775, -0.3828818442314328}}]}}, 
      {Arrowheads[{{0.024247296647982416`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3521887644790814, -0.1537891787788364}, \
{-0.23604652963856537`, -0.26974023298586985`}}]}}, 
      {Arrowheads[{{0.026697831192861544`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3199567182871483, 
         0.015988603364738466`}, {-0.2682785758304984, \
-0.15716507395297413`}}]}}, 
      {Arrowheads[{{0.026697831192861548`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.2682785758304986, 
         0.1571650739529738}, {-0.31995671828714817`, \
-0.015988603364738882`}}]}}, 
      {Arrowheads[{{0.024247296647982437`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.2360465296385654, 
         0.2697402329858696}, {-0.3521887644790814, 0.1537891787788359}}]}}, 
      {Arrowheads[{{0.021293249385494806`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.2285718608796775, 
         0.38288184423143246`}, {-0.35966343323796923`, 
         0.3230005087097436}}]}}, 
      {Arrowheads[{{0.018729192987741367`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.23167171028615716`, 
         0.5049768522894845}, {-0.35656358383148956`, 0.4832584418281622}}]}}, 
      {Arrowheads[{{0.016638922272212458`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.23782300796211778`, 
         0.6340122369762694}, {-0.350412286155529, 0.6365759983178477}}]}}, 
      {Arrowheads[{{0.014932586338013894`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.2443844086591488, 
         0.7675063255114019}, {-0.34385088545849796`, 0.7854348509591857}}]}}, 
      {Arrowheads[{{0.013519431694777397`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.25049945997221695`, 
         0.9038367040517583}, {-0.3377358341454298, 0.9314574135952998}}]}}, 
      {Arrowheads[{{0.01233150372043959, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.25592424674589215`, 
         1.0420058593151238`}, {-0.33231104737175454`, 
         1.0756411995084048`}}]}}, 
      {Arrowheads[{{0.011319996057258402`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.26063369595702274`, 
         1.1813880783055055`}, {-0.32760159816062406`, 
         1.2186119216944935`}}]}}, 
      {Arrowheads[{{0.011887593592644272`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.14780733058802156`, -1.2266232688386096`}, \
{-0.08748678705903692, -1.17337673116139}}]}}, 
      {Arrowheads[{{0.013052212367797389`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.15287182628651808`, -1.0854750730477827`}, \
{-0.08242229136054041, -1.0321719857757465`}}]}}, 
      {Arrowheads[{{0.01445170004907857, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.15908561841389984`, -0.943623318278592}, \
{-0.07620849923315866, -0.8916707993684668}}]}}, 
      {Arrowheads[{{0.01616913553089657, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.16675566078163923`, -0.8006067928917622}, \
{-0.06853845686541925, -0.7523343835788263}}]}}, 
      {Arrowheads[{{0.018341178210477076`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.17629174720465554`, -0.6556284134772696}, \
{-0.059002370442402946`, -0.6149598218168483}}]}}, 
      {Arrowheads[{{0.021217219252625052`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.1882573356788691, -0.5071509271858639}, \
{-0.04703678196818939, -0.4810843669317836}}]}}, 
      {Arrowheads[{{0.025314106954168615`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.20330395193299788`, -0.35158154324662866`}, \
{-0.0319901657140606, -0.3543008096945482}}]}}, 
      {Arrowheads[{{0.03184289065040897, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.21994412830189045`, -0.17788218445724527`}, \
{-0.015349989345168041`, -0.245647227307461}}]}}, 
      {Arrowheads[{{0.04264483907234297, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.19955083418471042`, 
         0.04823724197591278}, {-0.03574328346234806, \
-0.18941371256414846`}}]}}, 
      {Arrowheads[{{0.04264483907234301, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.03574328346234833, 
         0.1894137125641484}, {-0.19955083418471015`, \
-0.0482372419759135}}]}}, 
      {Arrowheads[{{0.03184289065040898, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.015349989345168, 
         0.24564722730746075`}, {-0.21994412830189047`, 
         0.17788218445724474`}}]}}, 
      {Arrowheads[{{0.025314106954168615`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.03199016571406059, 
         0.35430080969454786`}, {-0.20330395193299788`, 
         0.3515815432466282}}]}}, 
      {Arrowheads[{{0.021217219252625066`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.04703678196818935, 
         0.48108436693178325`}, {-0.18825733567886915`, 
         0.5071509271858634}}]}}, 
      {Arrowheads[{{0.018341178210477076`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.05900237044240293, 
         0.6149598218168479}, {-0.17629174720465557`, 0.6556284134772692}}]}}, 
      {Arrowheads[{{0.01616913553089658, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.06853845686541922, 
         0.7523343835788259}, {-0.16675566078163928`, 0.8006067928917617}}]}}, 
      {Arrowheads[{{0.014451700049078572`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.07620849923315864, 
         0.8916707993684665}, {-0.15908561841389984`, 0.9436233182785917}}]}}, 
      {Arrowheads[{{0.013052212367797396`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.08242229136054038, 
         1.0321719857757463`}, {-0.1528718262865181, 1.0854750730477825`}}]}}, 
      {Arrowheads[{{0.011887593592644274`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.0874867870590369, 
         1.1733767311613899`}, {-0.14780733058802156`, 
         1.2266232688386094`}}]}}, 
      {Arrowheads[{{0.012348989019976758`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.03467339342273591, -1.234106937153656}, {
         0.08297366540079386, -1.1658930628463438`}}]}}, 
      {Arrowheads[{{0.013637896700657104`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.030121679135397997`, -1.0949666001301526`}, {
         0.08752537968813177, -1.0226804586933769`}}]}}, 
      {Arrowheads[{{0.01520422431323159, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.024355805709462795`, -0.9558502589065597}, {
         0.09329125311406697, -0.8794438587404992}}]}}, 
      {Arrowheads[{{0.017149016881836055`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.016974362728579453`, -0.8166796057595832}, {
         0.10067269609495032`, -0.7362615707110054}}]}}, 
      {Arrowheads[{{0.019638762292407706`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.007400069285722573, -0.6773978691971717}, {
         0.1102469895378072, -0.5931903660969463}}]}}, 
      {Arrowheads[{{0.022983539176766755`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.005286385727639217, -0.5381601411418938}, {
         0.12293344455116899`, -0.4500751529757536}}]}}, 
      {Arrowheads[{{0.027872703268567535`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.022850451479291267`, -0.4001305876520876}, {
         0.14049751030282104`, -0.3057517652890892}}]}}, 
      {Arrowheads[{{0.03630211737329321, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.049442839235132985`, -0.26982755840113404`}, {
         0.16708989805866276`, -0.15370185336357223`}}]}}}, 
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.03630211737329324, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.1670898980586628, 
         0.15370185336357178`}, {-0.049442839235133054`, 
         0.2698275584011337}}]}}, 
      {Arrowheads[{{0.027872703268567556`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.1404975103028211, 
         0.3057517652890888}, {-0.022850451479291337`, 
         0.40013058765208726`}}]}}, 
      {Arrowheads[{{0.022983539176766758`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.12293344455116904`, 
         0.4500751529757533}, {-0.005286385727639273, 0.5381601411418934}}]}}, 
      {Arrowheads[{{0.01963876229240771, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.11024698953780723`, 0.5931903660969459}, {
         0.007400069285722538, 0.6773978691971713}}]}}, 
      {Arrowheads[{{0.01714901688183606, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.10067269609495035`, 0.7362615707110048}, {
         0.016974362728579426`, 0.8166796057595826}}]}}, 
      {Arrowheads[{{0.015204224313231596`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.093291253114067, 0.8794438587404988}, {
         0.02435580570946278, 0.9558502589065593}}]}}, 
      {Arrowheads[{{0.013637896700657133`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.08752537968813179, 1.0226804586933762`}, {
         0.030121679135397976`, 1.0949666001301521`}}]}}, 
      {Arrowheads[{{0.012348989019976785`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.08297366540079387, 1.1658930628463433`}, {
         0.034673393422735906`, 1.2341069371536557`}}]}}, 
      {Arrowheads[{{0.01267200792064229, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.21963695998977964`, -1.2399240464533328`}, {
         0.25095127530433836`, -1.160075953546667}}]}}, 
      {Arrowheads[{{0.014044095568926383`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.21648795085046235`, -1.102472410371767}, {
         0.25410028444365573`, -1.0151746484517625`}}]}}, 
      {Arrowheads[{{0.015713768587182292`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.21246379191679837`, -0.9656752996190408}, {
         0.25812444337731966`, -0.8696188180280181}}]}}, 
      {Arrowheads[{{0.01777666558666945, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.20732548150218333`, -0.8297334185502494}, {
         0.26326275379193476`, -0.7232077579203391}}]}}, 
      {Arrowheads[{{0.020368162482886854`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.20087293735525857`, -0.6950141571985943}, {
         0.26971529793885946`, -0.5755740780955236}}]}}, 
      {Arrowheads[{{0.023680580653072644`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.1932211857133614, -0.5623247705746172}, {
         0.27736704958075664`, -0.4259105235430303}}]}}, 
      {Arrowheads[{{0.027966359368554843`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.18579408302915487`, -0.4336078995516175}, {
         0.2847941522649632, -0.27227445338955936`}}]}}, 
      {Arrowheads[{{0.03335274736091598, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.1849157847006836, -0.3127701536661972}, {
         0.28567245059343443`, -0.11075925809850905`}}]}}, 
      {Arrowheads[{{0.038485375977068144`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.20949440331546673`, -0.19824904398673726`}, {
         0.2610938319786513, 0.05707257339850158}}]}}, 
      {Arrowheads[{{0.03848537597706815, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.26109383197865116`, -0.057072573398502024`}, {
         0.20949440331546687`, 0.19824904398673693`}}]}}, 
      {Arrowheads[{{0.03335274736091601, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.28567245059343443`, 0.11075925809850856`}, {
         0.1849157847006836, 0.3127701536661969}}]}}, 
      {Arrowheads[{{0.027966359368554856`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.2847941522649632, 0.2722744533895589}, {
         0.18579408302915484`, 0.43360789955161716`}}]}}, 
      {Arrowheads[{{0.023680580653072658`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.27736704958075664`, 0.42591052354302983`}, {
         0.1932211857133614, 0.5623247705746168}}]}}, 
      {Arrowheads[{{0.020368162482886875`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.2697152979388595, 0.5755740780955232}, {
         0.20087293735525855`, 0.6950141571985939}}]}}, 
      {Arrowheads[{{0.017776665586669477`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.2632627537919347, 0.7232077579203385}, {
         0.20732548150218333`, 0.8297334185502491}}]}}, 
      {Arrowheads[{{0.015713768587182278`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.25812444337731966`, 0.8696188180280178}, {
         0.21246379191679834`, 0.9656752996190403}}]}}, 
      {Arrowheads[{{0.014044095568926381`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.2541002844436557, 1.015174648451762}, {
         0.21648795085046235`, 1.1024724103717665`}}]}}, 
      {Arrowheads[{{0.012672007920642296`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.2509512753043384, 1.1600759535466667`}, {
         0.2196369599897796, 1.2399240464533325`}}]}}, 
      {Arrowheads[{{0.012837443888257518`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4063385653448983, -1.2431041255504744`}, {
         0.41719084641980797`, -1.1568958744495255`}}]}}, 
      {Arrowheads[{{0.014249727624220065`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4052165744197831, -1.106600636749776}, {
         0.41831283734492325`, -1.0110464220737534`}}]}}, 
      {Arrowheads[{{0.01596461030278986, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.40378430405671245`, -0.9710816467740162}, {
         0.4197451077079939, -0.8642124708730426}}]}}, 
      {Arrowheads[{{0.01806548109955965, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.40198000121789784`, -0.8368194838728339}, {
         0.42154941054680845`, -0.7161216925977546}}]}}, 
      {Arrowheads[{{0.020646634420394227`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3998139120321679, -0.704136591850719}, {
         0.4237154997325384, -0.566451643443399}}]}}, 
      {Arrowheads[{{0.023779869217497602`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.39756325006964255`, -0.5733302117030541}, {
         0.42596616169506374`, -0.41490508241459334`}}]}}, 
      {Arrowheads[{{0.027405376742041413`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.39623784092196435`, -0.4443771590151003}, {
         0.42729157084274194`, -0.2615051939260766}}]}}, 
      {Arrowheads[{{0.03106820856889267, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.39819169078244, -0.31602557002020487`}, {
         0.4253377209822663, -0.10750384174450141`}}]}}, 
      {Arrowheads[{{0.03359921691897553, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.40604209905985, -0.18415018846067976`}, {
         0.41748731270485623`, 0.042973717872444084`}}]}}, 
      {Arrowheads[{{0.03359921691897553, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.41748731270485623`, -0.04297371787244447}, {
         0.40604209905985006`, 0.18415018846067938`}}]}}, 
      {Arrowheads[{{0.03106820856889267, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4253377209822663, 0.10750384174450102`}, {
         0.39819169078244, 0.3160255700202045}}]}}, 
      {Arrowheads[{{0.02740537674204142, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4272915708427419, 0.26150519392607613`}, {
         0.39623784092196435`, 0.4443771590150999}}]}}, 
      {Arrowheads[{{0.02377986921749762, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.42596616169506374`, 0.4149050824145929}, {
         0.39756325006964255`, 0.5733302117030538}}]}}, 
      {Arrowheads[{{0.020646634420394227`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4237154997325384, 0.5664516434433986}, {
         0.3998139120321679, 0.7041365918507185}}]}}, 
      {Arrowheads[{{0.018065481099559667`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.42154941054680845`, 0.7161216925977542}, {
         0.40198000121789784`, 0.8368194838728336}}]}}, 
      {Arrowheads[{{0.015964610302789842`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.41974510770799384`, 0.8642124708730423}, {
         0.4037843040567124, 0.9710816467740158}}]}}, 
      {Arrowheads[{{0.014249727624220094`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4183128373449232, 1.011046422073753}, {
         0.4052165744197831, 1.1066006367497758`}}]}}, 
      {Arrowheads[{{0.012837443888257518`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.41719084641980797`, 1.156895874449525}, {
         0.4063385653448983, 1.243104125550474}}]}}, 
      {Arrowheads[{{0.012837443888257518`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5936614346551021, -1.2431041255504744`}, {
         0.5828091535801925, -1.1568958744495255`}}]}}, 
      {Arrowheads[{{0.014249727624220066`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5947834255802175, -1.106600636749776}, {
         0.5816871626550773, -1.0110464220737534`}}]}}, 
      {Arrowheads[{{0.015964610302789846`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.596215695943288, -0.9710816467740162}, {
         0.5802548922920066, -0.8642124708730428}}]}}, 
      {Arrowheads[{{0.018065481099559653`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5980199987821027, -0.8368194838728339}, {
         0.5784505894531919, -0.7161216925977546}}]}}, 
      {Arrowheads[{{0.02064663442039423, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6001860879678326, -0.704136591850719}, {
         0.5762845002674619, -0.566451643443399}}]}}, 
      {Arrowheads[{{0.023779869217497616`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.602436749930358, -0.5733302117030541}, {
         0.5740338383049367, -0.4149050824145933}}]}}, 
      {Arrowheads[{{0.0274053767420414, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6037621590780361, -0.4443771590151002}, {
         0.5727084291572584, -0.26150519392607663`}}]}}, 
      {Arrowheads[{{0.03106820856889265, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6018083092175606, -0.3160255700202048}, {
         0.5746622790177341, -0.10750384174450146`}}]}}, 
      {Arrowheads[{{0.03359921691897553, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5939579009401504, -0.18415018846067976`}, {
         0.5825126872951442, 0.042973717872444084`}}]}}, 
      {Arrowheads[{{0.03359921691897553, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5825126872951442, -0.04297371787244447}, {
         0.5939579009401504, 0.18415018846067938`}}]}}, 
      {Arrowheads[{{0.03106820856889267, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.574662279017734, 0.10750384174450102`}, {
         0.6018083092175605, 0.3160255700202045}}]}}, 
      {Arrowheads[{{0.027405376742041423`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5727084291572585, 0.2615051939260762}, {
         0.6037621590780362, 0.44437715901509994`}}]}}, 
      {Arrowheads[{{0.023779869217497623`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5740338383049366, 0.4149050824145929}, {
         0.6024367499303579, 0.5733302117030538}}]}}, 
      {Arrowheads[{{0.02064663442039423, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.576284500267462, 0.5664516434433986}, {
         0.6001860879678327, 0.7041365918507185}}]}}, 
      {Arrowheads[{{0.018065481099559653`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5784505894531919, 0.7161216925977542}, {
         0.5980199987821027, 0.8368194838728334}}]}}, 
      {Arrowheads[{{0.015964610302789846`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5802548922920066, 0.8642124708730423}, {
         0.596215695943288, 0.9710816467740158}}]}}, 
      {Arrowheads[{{0.014249727624220096`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5816871626550771, 1.011046422073753}, {
         0.5947834255802174, 1.1066006367497758`}}]}}, 
      {Arrowheads[{{0.012837443888257518`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5828091535801925, 1.156895874449525}, {
         0.5936614346551021, 1.243104125550474}}]}}, 
      {Arrowheads[{{0.012672007920642294`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7803630400102208, -1.2399240464533328`}, {
         0.7490487246956621, -1.160075953546667}}]}}, 
      {Arrowheads[{{0.014044095568926383`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7835120491495382, -1.102472410371767}, {
         0.7458997155563448, -1.0151746484517625`}}]}}, 
      {Arrowheads[{{0.01571376858718228, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7875362080832021, -0.9656752996190408}, {
         0.7418755566226808, -0.8696188180280182}}]}}, 
      {Arrowheads[{{0.017776665586669467`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7926745184978172, -0.8297334185502494}, {
         0.7367372462080658, -0.723207757920339}}]}}, 
      {Arrowheads[{{0.02036816248288685, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.799127062644742, -0.6950141571985943}, {
         0.7302847020611409, -0.5755740780955237}}]}}, 
      {Arrowheads[{{0.023680580653072658`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8067788142866391, -0.5623247705746172}, {
         0.7226329504192437, -0.4259105235430303}}]}}, 
      {Arrowheads[{{0.027966359368554843`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8142059169708457, -0.43360789955161744`}, {
         0.7152058477350371, -0.2722744533895594}}]}}, 
      {Arrowheads[{{0.033352747360916, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.815084215299317, -0.3127701536661972}, {
         0.7143275494065658, -0.11075925809850908`}}]}}, 
      {Arrowheads[{{0.03848537597706819, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7905055966845338, -0.1982490439867374}, {
         0.738906168021349, 0.05707257339850172}}]}}, 
      {Arrowheads[{{0.03848537597706819, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7389061680213492, -0.057072573398502136`}, {
         0.7905055966845338, 0.19824904398673704`}}]}}, 
      {Arrowheads[{{0.03335274736091603, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.714327549406566, 0.11075925809850856`}, {
         0.8150842152993171, 0.3127701536661969}}]}}, 
      {Arrowheads[{{0.027966359368554863`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7152058477350371, 0.27227445338955897`}, {
         0.8142059169708458, 0.43360789955161705`}}]}}, 
      {Arrowheads[{{0.02368058065307266, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7226329504192437, 0.4259105235430299}, {
         0.8067788142866391, 0.5623247705746168}}]}}, 
      {Arrowheads[{{0.020368162482886868`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7302847020611409, 0.5755740780955233}, {
         0.799127062644742, 0.6950141571985939}}]}}, 
      {Arrowheads[{{0.017776665586669474`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7367372462080657, 0.7232077579203386}, {
         0.7926745184978172, 0.8297334185502491}}]}}, 
      {Arrowheads[{{0.01571376858718228, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7418755566226808, 0.8696188180280178}, {
         0.7875362080832021, 0.9656752996190403}}]}}, 
      {Arrowheads[{{0.014044095568926383`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7458997155563447, 1.015174648451762}, {
         0.7835120491495381, 1.1024724103717665`}}]}}, 
      {Arrowheads[{{0.0126720079206423, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7490487246956621, 1.1600759535466667`}, {
         0.7803630400102209, 1.2399240464533325`}}]}}, 
      {Arrowheads[{{0.01234898901997676, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9653266065772645, -1.234106937153656}, {
         0.9170263345992066, -1.1658930628463438`}}]}}, 
      {Arrowheads[{{0.013637896700657133`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9698783208646026, -1.0949666001301526`}, {
         0.9124746203118688, -1.0226804586933766`}}]}}, 
      {Arrowheads[{{0.015204224313231596`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9756441942905377, -0.9558502589065597}, {
         0.9067087468859335, -0.8794438587404992}}]}}, 
      {Arrowheads[{{0.017149016881836045`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9830256372714211, -0.8166796057595831}, {
         0.8993273039050502, -0.7362615707110054}}]}}, 
      {Arrowheads[{{0.0196387622924077, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.992599930714278, -0.6773978691971717}, {
         0.8897530104621933, -0.5931903660969464}}]}}, 
      {Arrowheads[{{0.022983539176766744`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0052863857276397`, -0.5381601411418937}, {
         0.8770665554488314, -0.4500751529757538}}]}}, 
      {Arrowheads[{{0.027872703268567518`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0228504514792918`, -0.4001305876520875}, {
         0.8595024896971795, -0.30575176528908937`}}]}}, 
      {Arrowheads[{{0.03630211737329323, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0494428392351336`, -0.2698275584011338}, {
         0.8329101019413375, -0.15370185336357245`}}]}}}, 
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.03630211737329326, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8329101019413375, 0.153701853363572}, {
         1.0494428392351338`, 0.2698275584011335}}]}}, 
      {Arrowheads[{{0.027872703268567563`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8595024896971794, 0.30575176528908893`}, {
         1.022850451479292, 0.4001305876520871}}]}}, 
      {Arrowheads[{{0.022983539176766755`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8770665554488314, 0.45007515297575335`}, {
         1.0052863857276397`, 0.5381601411418934}}]}}, 
      {Arrowheads[{{0.01963876229240772, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8897530104621932, 0.5931903660969459}, {
         0.9925999307142781, 0.6773978691971712}}]}}, 
      {Arrowheads[{{0.017149016881836045`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8993273039050501, 0.736261570711005}, {0.983025637271421,
          0.8166796057595826}}]}}, 
      {Arrowheads[{{0.015204224313231606`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9067087468859335, 0.8794438587404988}, {
         0.9756441942905378, 0.9558502589065593}}]}}, 
      {Arrowheads[{{0.013637896700657133`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9124746203118687, 1.0226804586933764`}, {
         0.9698783208646025, 1.0949666001301523`}}]}}, 
      {Arrowheads[{{0.01234898901997676, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9170263345992066, 1.1658930628463435`}, {
         0.9653266065772645, 1.2341069371536557`}}]}}, 
      {Arrowheads[{{0.011887593592644272`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.147807330588022, -1.2266232688386096`}, {
         1.0874867870590375`, -1.17337673116139}}]}}, 
      {Arrowheads[{{0.013052212367797375`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1528718262865185`, -1.0854750730477827`}, {
         1.0824222913605408`, -1.0321719857757468`}}]}}, 
      {Arrowheads[{{0.014451700049078555`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1590856184139002`, -0.943623318278592}, {
         1.076208499233159, -0.8916707993684669}}]}}, 
      {Arrowheads[{{0.016169135530896585`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1667556607816396`, -0.8006067928917622}, {
         1.0685384568654195`, -0.7523343835788264}}]}}, 
      {Arrowheads[{{0.018341178210477056`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.176291747204656, -0.6556284134772696}, {
         1.0590023704424034`, -0.6149598218168484}}]}}, 
      {Arrowheads[{{0.02121721925262504, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1882573356788695`, -0.5071509271858637}, {
         1.0470367819681898`, -0.4810843669317837}}]}}, 
      {Arrowheads[{{0.02531410695416859, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2033039519329982`, -0.3515815432466286}, {
         1.031990165714061, -0.3543008096945483}}]}}, 
      {Arrowheads[{{0.03184289065040897, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2199441283018908`, -0.17788218445724513`}, {
         1.0153499893451685`, -0.24564722730746114`}}]}}, 
      {Arrowheads[{{0.04264483907234298, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1995508341847108`, 0.048237241975912906`}, {
         1.0357432834623486`, -0.18941371256414857`}}]}}, 
      {Arrowheads[{{0.042644839072343024`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0357432834623488`, 0.18941371256414852`}, {
         1.1995508341847105`, -0.0482372419759136}}]}}, 
      {Arrowheads[{{0.03184289065040901, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0153499893451685`, 0.24564722730746089`}, {
         1.219944128301891, 0.1778821844572446}}]}}, 
      {Arrowheads[{{0.025314106954168622`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0319901657140609`, 0.3543008096945479}, {
         1.2033039519329982`, 0.3515815432466281}}]}}, 
      {Arrowheads[{{0.021217219252625073`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0470367819681898`, 0.4810843669317833}, {
         1.1882573356788697`, 0.5071509271858634}}]}}, 
      {Arrowheads[{{0.018341178210477087`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0590023704424034`, 0.6149598218168479}, {
         1.1762917472046561`, 0.6556284134772692}}]}}, 
      {Arrowheads[{{0.016169135530896585`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0685384568654197`, 0.7523343835788259}, {
         1.1667556607816398`, 0.8006067928917616}}]}}, 
      {Arrowheads[{{0.014451700049078555`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.076208499233159, 0.8916707993684665}, {
         1.1590856184139002`, 0.9436233182785916}}]}}, 
      {Arrowheads[{{0.013052212367797375`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0824222913605408`, 1.0321719857757463`}, {
         1.1528718262865185`, 1.0854750730477822`}}]}}, 
      {Arrowheads[{{0.011887593592644272`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0874867870590372`, 1.1733767311613899`}, {
         1.1478073305880219`, 1.2266232688386094`}}]}}, 
      {Arrowheads[{{0.01131999605725841, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3276015981606244`, -1.218611921694494}, {
         1.260633695957023, -1.181388078305506}}]}}, 
      {Arrowheads[{{0.012331503720439582`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.332311047371755, -1.0756411995084052`}, {
         1.2559242467458926`, -1.0420058593151242`}}]}}, 
      {Arrowheads[{{0.01351943169477738, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3377358341454302`, -0.9314574135953002}, {
         1.2504994599722175`, -0.9038367040517588}}]}}, 
      {Arrowheads[{{0.014932586338013894`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3438508854584983`, -0.7854348509591862}, {
         1.2443844086591491`, -0.7675063255114023}}]}}, 
      {Arrowheads[{{0.01663892227221243, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3504122861555294`, -0.6365759983178482}, {
         1.2378230079621184`, -0.6340122369762698}}]}}, 
      {Arrowheads[{{0.018729192987741363`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.35656358383149, -0.4832584418281625}, {
         1.2316717102861576`, -0.5049768522894849}}]}}, 
      {Arrowheads[{{0.021293249385494803`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3596634332379696`, -0.323000508709744}, {
         1.2285718608796778`, -0.3828818442314328}}]}}, 
      {Arrowheads[{{0.024247296647982412`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3521887644790818`, -0.15378917877883638`}, {
         1.2360465296385659`, -0.2697402329858699}}]}}, 
      {Arrowheads[{{0.026697831192861524`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3199567182871488`, 0.015988603364738382`}, {
         1.268278575830499, -0.15716507395297408`}}]}}, 
      {Arrowheads[{{0.026697831192861517`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.268278575830499, 0.1571650739529737}, {
         1.3199567182871486`, -0.0159886033647388}}]}}, 
      {Arrowheads[{{0.024247296647982405`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2360465296385659`, 0.2697402329858696}, {
         1.3521887644790815`, 0.1537891787788359}}]}}, 
      {Arrowheads[{{0.021293249385494813`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.228571860879678, 0.3828818442314325}, {
         1.3596634332379698`, 0.32300050870974356`}}]}}, 
      {Arrowheads[{{0.018729192987741363`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2316717102861576`, 0.5049768522894845}, {
         1.35656358383149, 0.48325844182816213`}}]}}, 
      {Arrowheads[{{0.01663892227221246, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2378230079621182`, 0.6340122369762694}, {
         1.3504122861555294`, 0.6365759983178477}}]}}, 
      {Arrowheads[{{0.01493258633801386, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2443844086591493`, 0.7675063255114019}, {
         1.3438508854584983`, 0.7854348509591856}}]}}, 
      {Arrowheads[{{0.013519431694777375`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2504994599722175`, 0.9038367040517583}, {
         1.3377358341454302`, 0.9314574135952997}}]}}, 
      {Arrowheads[{{0.012331503720439582`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2559242467458926`, 1.0420058593151238`}, {
         1.332311047371755, 1.0756411995084048`}}]}}, 
      {Arrowheads[{{0.01131999605725841, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2606336959570232`, 1.1813880783055055`}, {
         1.3276015981606246`, 1.2186119216944935`}}]}}, 
      {Arrowheads[{{0.010686841592073983`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5050229242208863`, -1.2110567474129683`}, {
         1.4361535463673496`, -1.1889432525870318`}}]}}, 
      {Arrowheads[{{0.011535880557568972`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5088040073459763`, -1.0668012326683816`}, {
         1.4323724632422599`, -1.0508458261551479`}}]}}, 
      {Arrowheads[{{0.012502257682518803`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5127489356390829`, -0.9211981232504092}, {
         1.4284275349491533`, -0.9140959943966498}}]}}, 
      {Arrowheads[{{0.013603550160558338`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5165449102051474`, -0.7737528564958357}, {
         1.4246315603830884`, -0.7791883199747527}}]}}, 
      {Arrowheads[{{0.0148533176118672, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5195336530200185`, -0.6238460536133018}, {
         1.4216428175682174`, -0.6467421816808162}}]}}, 
      {Arrowheads[{{0.016246603848199002`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5204060805549435`, -0.4708549418259187}, {
         1.4207703900332922`, -0.5173803522917287}}]}}, 
      {Arrowheads[{{0.017722495320447635`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5167777864460728`, -0.314683164405333}, {
         1.4243986841421634`, -0.39119918853584384`}}]}}, 
      {Arrowheads[{{0.019090205704839176`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5052261872655954`, -0.157230341094936}, {
         1.4359502833226403`, -0.2662990706697703}}]}}, 
      {Arrowheads[{{0.019970757691354837`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.483806752893959, -0.004308718763198605}, {
         1.457369717694277, -0.13686775182503708`}}]}}, 
      {Arrowheads[{{0.019970757691354837`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4573697176942773`, 0.13686775182503672`}, {
         1.483806752893959, 0.004308718763198188}}]}}, 
      {Arrowheads[{{0.019090205704839165`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4359502833226405`, 0.26629907066976993`}, {
         1.5052261872655954`, 0.15723034109493556`}}]}}, 
      {Arrowheads[{{0.017722495320447642`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4243986841421632`, 0.39119918853584346`}, {
         1.5167777864460725`, 0.31468316440533256`}}]}}, 
      {Arrowheads[{{0.01624660384819901, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4207703900332924`, 0.5173803522917284}, {
         1.5204060805549438`, 0.47085494182591825`}}]}}, 
      {Arrowheads[{{0.014853317611867204`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4216428175682174`, 0.6467421816808158}, {
         1.5195336530200185`, 0.6238460536133013}}]}}, 
      {Arrowheads[{{0.013603550160558338`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4246315603830886`, 0.7791883199747524}, {
         1.5165449102051476`, 0.7737528564958353}}]}}, 
      {Arrowheads[{{0.012502257682518803`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.428427534949153, 0.9140959943966493}, {
         1.5127489356390826`, 0.9211981232504087}}]}}, 
      {Arrowheads[{{0.011535880557568972`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4323724632422596`, 1.0508458261551474`}, {
         1.508804007345976, 1.0668012326683811`}}]}}, 
      {Arrowheads[{{0.010686841592073983`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4361535463673496`, 1.1889432525870312`}, {
         1.5050229242208863`, 1.2110567474129676`}}]}}, 
      {Arrowheads[{{0.010028582045836464`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.680688083523815, -1.2045718209665592`}, {
         1.6134295635350093`, -1.1954281790334407`}}]}}, 
      {Arrowheads[{{0.010723839177871548`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6833408829250243`, -1.059650470936451}, {
         1.6107767641338, -1.0579965878870783`}}]}}, 
      {Arrowheads[{{0.011488462175840704`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6857203362016648`, -0.9135394319676214}, {
         1.6083973108571592`, -0.9217546856794374}}]}}, 
      {Arrowheads[{{0.012321479593959411`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6874169563504875`, -0.7659843725279378}, {
         1.6067006907083365`, -0.7869568039426506}}]}}, 
      {Arrowheads[{{0.01321166464844315, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6877654327105887`, -0.6168002507598085}, {
         1.6063522143482358`, -0.6537879845343095}}]}}, 
      {Arrowheads[{{0.014127437282837505`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6857701450723332`, -0.4660602105901082}, {
         1.608347501986491, -0.5221750835275392}}]}}, 
      {Arrowheads[{{0.015001920477789359`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6801689145901557`, -0.3144542330153651}, {
         1.6139487324686685`, -0.3914281199258117}}]}}, 
      {Arrowheads[{{0.01572071590270053, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6699194436367748`, -0.1637248386709837}, {
         1.6241982034220495`, -0.2598045730937226}}]}}, 
      {Arrowheads[{{0.016136752994528308`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6553001714264806`, -0.01660388462477389}, {
         1.6388174756323437`, -0.1245725859634618}}]}}, 
      {Arrowheads[{{0.016136752994528308`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6388174756323437`, 0.12457258596346141`}, {
         1.6553001714264806`, 0.016603884624773496`}}]}}, 
      {Arrowheads[{{0.01572071590270051, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6241982034220495`, 0.2598045730937222}, {
         1.6699194436367746`, 0.16372483867098328`}}]}}, 
      {Arrowheads[{{0.015001920477789364`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6139487324686685`, 0.39142811992581134`}, {
         1.6801689145901557`, 0.3144542330153647}}]}}, 
      {Arrowheads[{{0.014127437282837513`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.608347501986491, 0.5221750835275388}, {
         1.6857701450723332`, 0.46606021059010777`}}]}}, 
      {Arrowheads[{{0.013211664648443153`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6063522143482356`, 0.6537879845343091}, {
         1.6877654327105884`, 0.6168002507598079}}]}}, 
      {Arrowheads[{{0.012321479593959416`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6067006907083368`, 0.7869568039426502}, {
         1.6874169563504877`, 0.7659843725279374}}]}}, 
      {Arrowheads[{{0.011488462175840704`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6083973108571594`, 0.9217546856794371}, {
         1.685720336201665, 0.913539431967621}}]}}, 
      {Arrowheads[{{0.010723839177871548`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6107767641338, 1.057996587887078}, {1.6833408829250243`, 
         1.0596504709364507`}}]}}, 
      {Arrowheads[{{0.010028582045836464`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6134295635350093`, 1.1954281790334402`}, {
         1.680688083523815, 1.2045718209665588`}}]}}, 
      {Arrowheads[{{0.009378057649365256, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8552605255400825`, -1.1993833517758685`}, {
         1.7917982979893299`, -1.2006166482241316`}}]}}, 
      {Arrowheads[{{0.009939135637952025, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8568593732459089`, -1.0542972383464162`}, {
         1.7901994502835037`, -1.0633498204771132`}}]}}, 
      {Arrowheads[{{0.010536127696573524`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.857939368737075, -0.9083024517049932}, {
         1.7891194547923372`, -0.9269916659420656}}]}}, 
      {Arrowheads[{{0.01116046987803137, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8581343297764341`, -0.7613376362105022}, {
         1.7889244937529782`, -0.7916035402600862}}]}}, 
      {Arrowheads[{{0.011794938313547856`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8569510113441725`, -0.6134696755842207}, {
         1.7901078121852398`, -0.6571185597098974}}]}}, 
      {Arrowheads[{{0.012409459526061593`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8538023337205412`, -0.4650107932079543}, {
         1.7932564898088712`, -0.5232245009096932}}]}}, 
      {Arrowheads[{{0.012957745607494608`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.848147598018817, -0.3166521463626772}, {
         1.7989112255105957`, -0.38923020657849966`}}]}}, 
      {Arrowheads[{{0.0133790157522867, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8397883226400462`, -0.16950759579004993`}, {
         1.8072705008893664`, -0.25402181597465634`}}]}}, 
      {Arrowheads[{{0.013610467207734908`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8292352151334539`, -0.024882637049943444`}, {
         1.817823608395959, -0.11629383353829223`}}]}}, 
      {Arrowheads[{{0.013610467207734912`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.817823608395959, 0.11629383353829185`}, {
         1.8292352151334539`, 0.02488263704994305}}]}}, 
      {Arrowheads[{{0.0133790157522867, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8072705008893664`, 0.25402181597465595`}, {
         1.8397883226400462`, 0.16950759579004954`}}]}}, 
      {Arrowheads[{{0.012957745607494608`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.7989112255105957`, 0.38923020657849927`}, {
         1.848147598018817, 0.3166521463626768}}]}}, 
      {Arrowheads[{{0.012409459526061564`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.7932564898088714`, 0.5232245009096927}, {
         1.8538023337205412`, 0.4650107932079539}}]}}, 
      {Arrowheads[{{0.011794938313547856`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.79010781218524, 0.657118559709897}, {1.8569510113441727`,
          0.6134696755842203}}]}}, 
      {Arrowheads[{{0.011160469878031407`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.7889244937529782`, 0.7916035402600858}, {
         1.8581343297764343`, 0.7613376362105018}}]}}, 
      {Arrowheads[{{0.01053612769657353, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.7891194547923375`, 0.9269916659420653}, {
         1.8579393687370753`, 0.9083024517049928}}]}}, 
      {Arrowheads[{{0.009939135637952025, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.7901994502835037`, 1.0633498204771128`}, {
         1.8568593732459089`, 1.0542972383464158`}}]}}, 
      {Arrowheads[{{0.009378057649365256, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.79179829798933, 1.200616648224131}, {1.8552605255400827`,
          1.1993833517758679`}}]}}, 
      {Arrowheads[{{0.008757479559774909, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.0292847608124007`, -1.1954445927625155`}, {
         1.9707152391876002`, -1.2045554072374844`}}]}}, 
      {Arrowheads[{{0.009207562759918951, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.030031753120171, -1.0505140900952905`}, {
         1.9699682468798299`, -1.067132968728239}}]}}, 
      {Arrowheads[{{0.009672561889281511, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.0301733316798956`, -0.9049557519942684}, {
         1.9698266683201051`, -0.9303383656527907}}]}}, 
      {Arrowheads[{{0.010142411369821663`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.0294355199651166`, -0.7588162722205656}, {
         1.9705644800348843`, -0.7941249042500229}}]}}, 
      {Arrowheads[{{0.010601410454064812`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.027503554226847, -0.6122566452830468}, {
         1.972496445773154, -0.6583315900110711}}]}}, 
      {Arrowheads[{{0.011027071558958653`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.0240732087329945`, -0.46560292007593257`}, {
         1.9759267912670062`, -0.5226323740417148}}]}}, 
      {Arrowheads[{{0.011390304415636921`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.0189509663535348`, -0.319374432145047}, {
         1.9810490336464661`, -0.38650792079612983`}}]}}, 
      {Arrowheads[{{0.011658373599048814`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.0121924370725632`, -0.17424175030783914`}, {
         1.9878075629274379`, -0.24928766145686712`}}]}}, 
      {Arrowheads[{{0.011801480861586293`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.004215653254475, -0.0308729241764768}, {
         1.9957843467455263`, -0.11030354641175888`}}]}}, 
      {Arrowheads[{{0.011801480861586285`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9957843467455263`, 0.11030354641175849`}, {
         2.0042156532544744`, 0.030872924176476406`}}]}}, 
      {Arrowheads[{{0.011658373599048802`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9878075629274379`, 0.24928766145686676`}, {
         2.012192437072563, 0.17424175030783873`}}]}}, 
      {Arrowheads[{{0.011390304415636921`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9810490336464661`, 0.38650792079612944`}, {
         2.0189509663535348`, 0.3193744321450466}}]}}, 
      {Arrowheads[{{0.0110270715589587, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9759267912670062`, 0.5226323740417145}, {
         2.024073208732995, 0.4656029200759322}}]}}, 
      {Arrowheads[{{0.010601410454064823`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.972496445773154, 0.6583315900110708}, {2.027503554226847,
          0.6122566452830464}}]}}, 
      {Arrowheads[{{0.010142411369821663`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9705644800348843`, 0.7941249042500225}, {
         2.0294355199651166`, 0.7588162722205651}}]}}, 
      {Arrowheads[{{0.009672561889281572, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9698266683201051`, 0.9303383656527903}, {
         2.030173331679896, 0.9049557519942679}}]}}, 
      {Arrowheads[{{0.009207562759919014, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9699682468798299`, 1.0671329687282385`}, {
         2.0300317531201713`, 1.05051409009529}}]}}, 
      {Arrowheads[{{0.008757479559774909, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9707152391876002`, 1.204555407237484}, {
         2.0292847608124007`, 1.195444592762515}}]}}}}}, 
   GraphicsComplexBox[CompressedData["
1:eJx0vXl8U8X3/x9Q3HAFXFEsgoC+RVAUENEJ4IILi4IoyFIXCiqbIBVQ9FIL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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1Ftoj3Ecx/HvyJKwMTOHzWEph5LkVJILNw5z2rCUUsiV/DdsSzmVQ8kx
MzFnwkjYZiQJmyTMlCTmQpIoGy4oYuX15OK99/P5fX+/7/P7/Z/n2dDlJUWp
tIjI9qesf0QWxg+ImIDMvhHlcnpWRB++bqz70IgRvSIaXO/uF7EHvcx7Ij9F
KjOimdfxulw9sRaT9Cjn2dbOHxix3fU21A6KqNejythBVOKJft/we0jEV+5i
TjqK1RZhIeboV2dsmXqB+51Bk+v6wRF/3aNFzw6+Lj8wXpljTL6Q7FM+z81c
w8V8g28i3zmfcYX9v+Fp+uRbu5QLMap3RJrxnmg19wc+YbXeGXkRueiEiRiJ
P8Y/O+dPNOItWtCq/yL15ZiBjShBnv4XsiNeO3s1P7B+Cz+V9/Bz9bV8V17J
zfqM5XPyVK5UX8x1cgHftD6HD8vDuEm9G++SO/pE3LP+MZfKr3iH+nfeJH/g
S9Zf4xXyHb6hfpaL5UNcb/0yniKneL36Tp4pb+Dj1k/nMfJCvqg+mfPl0Xze
+t+eZVc5XV6lPoSz5Szeb/079T+eYxsXqL/gr/JDPmL9AX4pn+RM9Vv8Xr7M
W61vct6P+j3nqerTvDfzMBATMAK79Tnq96/BXjSiAUvN/5k8U3Na+DPeJuP6
1niODbji+T/CbXwxnus9GYm73qsefMy7sg/7UYYTOI5q9dN8n0/xPe5wnxd6
VHnnGuVa52iXm/VpS56FPFetNDm7+63hKnuYxYW+iyIswGZ73IQ2PX45ezt/
wTW1WtQl6LlEvyL3XICrxgqdd41+E/VLcSlKkm/Y77PX/HLzK1CGlH10todO
yNB7uzzcHscl33////9P/gGHk6z9
          "]]},
        "0.42000000000000004`"],
       Annotation[#, 0.42000000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1FmIT2EYx/FnMCTrjG0Y2z+ErBdcyDqU3ZAlN8YyXIkZ+54tS9YoWaNk
7JMw9omQLdMoyTYiSUaWXJAkyufk4jvf/+993uc97znnnZPKLxxbkBYRWf7M
bhqRgc7NIrogvXHEgpYRi7AQaQ0iCtSrcCHfNWcuV5XncDVeYt5SLEZ1/fOM
pxvP5K7m10pFtMuI6Ob3fXNKsiMu4Dwu4SJGqJcla+gfJediNEqzIkYmdTxM
9qX+s3VEvrWn4618uVVE98yI1+YObuLGXG8In25kn3qKuRoPsqf1egbyBh5k
zQrzcuVyvaP5ldxLrdx+ThsrsudinuA6LYynkmeltrB+xAu1HLWUnjweg472
Ecbr4KW53/EBs/TUbRGRjTT0RAf8Nl7ZPOIHbqIC5Xhp/fHq0zAEy1GA5tY/
6n6eewZ7+bb+lfxQ3sKP1OfydXkGl1mnOx+R+/JO9Yl8Vh7OF/U35t1yG76l
XpM3yX8aRtzQ/4AL5ae8Tv0br5Df8Qn9ZzhfLuUL6od5gryLz+mfyr3l2bxY
fSMPlZfxAf2Duas8jo+r9+KU3JmL9P/yXmrI6fJM9VbcUM7k7frfqP92Bj7z
cPXH/FW+w3v07+An8kGup36Z38qneI3+W+73vfUecV/1HGclF83QA+2x2Tr7
PP9j2IqbKEGe+T+Sd2pOOVeiIhm37jHvsQTF3v89XMUn49nOSQdcd6Zq835n
ZVvyf4X5yDJvr9optUPySX6sr4/x587aMPsuMzaUn8kDneMcZDj3A4ytdc0c
Xm0fJ9zTKj7OH615xX2PtOc3+kfxVXmqNfrji7F+/JePcBGmoNQermGG61da
Y3LyPdA7ifOSbwDuqM/0jNraZye/ZzX9/235B+Rxowg=
          "]]},
        "0.35000000000000003`"],
       Annotation[#, 0.35000000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1GtsjmcYwPFLiEgcSyl1qn1QCYLYh6Eogpgso8qnUoc5rdN2sqGtQ6mI
YyxZ5tD0w+LciBCERtDSlRASwaaVLIugCVt8mGSUmv2e7MO//173dbjv93nu
9x20uCinsE1ETPHnn54RywZErMByXE2L+KpbxNu+ES14g/fdI76R+6C2gFt5
i7rNKBJ/FK/if7mQC6yvHRixDkX+L+EylKJdL/W9I9rydv5a/k9+ibny83DE
jJnyh/hzPsxFPDs9ophzeAGfsj6fUwdFjHHOsZiY4gxmjfd/FsZhkj3OWDuN
YT7LQWtb+6jtEVHBE/ipWc9QIleKMlxXv4HXJ3PU7bfXbrNm2WOtM1SKF3EJ
1mGW+HtegzT1q5PzYjcOoqN5l9VkcjpemFOjrgEncbZfxFHkWW/qH9GMOwjn
es15zn6Wa7ELR3EAO7yvwWpGow++QDay1N/1Tp5mRNRxubnVPusfnvEF7iJf
xffFe3mfOfX8l/gez5B/yS3i33mP/pTUiB7mDeCV8u24vfiN/GH9Q8UZ4s/4
mHwODxdP5Ur9JTxdvI3XJPeKx4rz+Yz+HzlX/DOfk7/Ei8Sn+Lj+J1wmfsUV
8g+5UHyTr+h/z9vFHXzeOvlP+CdxTz6vfwafFs/jH+Sz+JB4BN/Wv4Qvi4v5
rvxOviXewNf07+ffxEe4r/wqz78U07AQc9Bozh3vsQm1eI1mtOjPlP80eadI
R2cUWH/u/f+NRnRyVz7yEPO/dE/ykOHeZfMjue+S56wnQ00/TLE2GXPcrRW8
EkuxV7yc89Xu0V8lHp/c9+Tuo968cnd7Ezai2Z15jvZyBXhg/xrrF5EvPsBD
rT3S15h8X82sMT+Xi5PvurVse01QV239of1O8K/8WK4Jw/VfSL7neka6M6vl
jomn6bskP50bnOEGfsG38nVm1OIq2oiLrQdvU5+SzNHfnUdxq/nv7N+a/G5h
qWe1Vd0C3iHfVV2FeAueJc/VfuVp//8O/gc5V8UW
          "]]},
        "0.28`"],
       Annotation[#, 0.28, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1Hlo13Ucx/G3mOeM/VJn29ycIzKQUswgxD8sMPEPD5wd1syNeTDnMfpD
0DU3LKemMUHdPKYu80BEChMRW6iIhKIgFrllEW0mHqHZLofg8fjiH88993q/
P8f3+Hx/+SXlBct7RcSH/twZiYyIypcjcl6KmDci4pbcNjyic3DEk+yIIrVi
3FW/od6l/lT9jbyIMehv7lhemRmxCgPkNXJBKqJC/phr5IHqD6yRxuvkT9Sr
9It4o5xSf6jfg01ysfpj+z22Xzqn8IprXKQ+d1jEEt5s3C/644ZElOZElGEx
qvGR9XrnR8zha8YPNLaFB3AzN6jvQpp8Xd5j/b1oxB+uazc3YBd+108Z18bp
3MpHPYNM6+/x/y3PZy9XqDXwav6GP+dGruRi117D++VFvB7r8L78Ja9FXpZn
x9VodW3zXUer+3yLr6lN4jz1Qp6B15J3ZE6aXjP/j39QZq9BuRFZeOpZjOdR
6FG/KbfjNFpwCc32KNAvwntYhaXItv5+7+M356Sez5pfyRfkr/iyfjk3ySV8
0TpjeJ88kTfrf8DfyVP5uPlDeZucz2f0+/F6+dHQiJ/M/5mXyb/yF/r3uEL+
mw+Zf5SL5VP8g34jz5a38Pfmz+MJ8hJeoV/DU+SVvNP8yfy6PIsP6r/NefJo
/tb8bmepj9xbLtXP5cFyir82/0/9HmfgDk/Vv8L/yue4zvxavio38Iv6J/gv
+TBXm38m+b6sd5kn6k9ydqYhE2/iVWywznbP/wA24jSOodD49uSd4hJuoiWp
W/eA93gMR7z/8ziJ2+pZzskoNCXfAO9wVnpxlT0eOEuf8Wjjso0fhnf138FM
4+fzguTsolYuSerGtrnGOnm5XM/lvIWX8bbk2+StvJQv4gIy7DXec5iufjC5
Zz7kOrb7vupRh6tqLxizw9id6JJz5G7O5XYezp2cxR2czYvdQz/fYl/0Qcp9
pCOSd+GZ1RpTakyh8WX27rJXNzrwSL8z+b3zPhYkPV7I1fbv8P8Gcz81t0m+
L9/17v7jKvker9Wfo/+jfDvj+W/qM5Yy1OM=
          "]]},
        "0.21000000000000002`"],
       Annotation[#, 0.21000000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1W1MllUcx/EDYqlhqYiADzfQREPXi7KnWTk3IQIjQU0UMwNfNBNIy0pn
IFaTXLOtmdZ60TJz2JrOmnNKU1dmomxlJlD5AChmISq0kWum9rnmix/f6/94
/uc6577ILH9pZlVcCGG9PwdHhdA3MoRnYiE0J4dQnR5C/pAQDvGfHRbC93gO
f8AC/kbsZB/G83gEp/PXqGtR/wffP/qNZacMDSELX04N4caIEMZ5vonjsVVu
XEoIaZkGwVfkxOO1jBD64SnxCfL6e74HEzAbX5M316zz6Fdr91qvFf/Gs2pq
5RSZpzOajwLF0RizzOJfpX4+1slL1LOWvZrK9Lsk7zJ1Uw9docX8N6M+o+0B
4zGOlvFXqE8wfyUOsK8+fW7Hddg/zX6tWWGtSvkPJHnGB7FY7c9mLcJjWKa+
wTxP20Ob2nbq0mcY31+YhMk0nNK920J9J+o/gXL0q8NyeU+JLcJCLOL/SZ9i
PKX/VDm/qNvOt4M2y4vx1cqtoXY55dHa5p2EzXxTMMZfioU0znzX1QwSa8Ee
OkeL9b1jTAipdMMe78csusrfye6lfdRKR6nFGsXiz1EOraAllKb/Z97HCXdg
Ix5QvwoPs9/BJvEqbGCXYaM+9+Kn7Mn4nvhs3M7Ow6/VJ+EGdgbuF78N17L/
HR7CN9E9xwr2cVwj3o0r2W24Vf2XuJC9B78S/wRnst/HHeoX4CPsF3G5+NuY
y34dP1I/DSeyi/Bz8Ycwxs7Gzer7nE8CO579gvhoHMq+C99Vf1L8qnP/E/PE
f8Qu9rf4gfr1eIz9MSaK78LT7HqsUb/ffjv0a8LJ4lPcuemUQvfRWKrTZ5P3
v4XW0T7aSaXye6MzpaPUSa2RX98tznEnfeH8D9JuusCf6p5kUYP7NRA/dFcC
Vlvjiru0FLPlpclPpmJ528TrqYn/Yb5meXnmP8T3BBbo8by8fFyIb8p5nD8b
M/jq7bdAbabnfDxiT8vlNeKruEx8vp57xUtxLnsqlmApzaNH9au1n8dwDU7C
JdFdxiF+4+PVDtYvMfrtqj1OubRVbY41L5k1F6ui74D3XYmL7PVuvjt9NwZH
36joe0XP8qfzD/Q8gC5SF5Xwx/ir9e12Xm/pPYdvL3uBddvldNBpaqMz1C7v
gvU68LoZ/6Me59YiNkPtSP3OiM1R34YleFn8pOfV+hfKSZXzW/Tt1ud37BI/
gW+IPyk+Qny2uu9G3fp/8T/qke8x
          "]]},
        "0.14`"],
       Annotation[#, 0.14, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1GtsjmcYwPGbotoilCnr6pARsnUbkm18EDPTLYs4++CwZWrDthjRbURI
NocRmURQsg/Lts4WDG9lE6f3pUwqOptTK5vRolSboqVSLab7PR/+/T/XdV/X
dd9Pn7vtmzt/4ietQggr/cjpHMKRzBAq0kM4zFe4iM8/FcIffFNcwtV8iv+W
P8214r/4Fp/hy/LnuU58jstQimm9QtiXEcJ+XFczQ1zJmV1CmGDvqmgPtEYS
2mCy/PLe9lD3wLxquSau4Vn6cxE3L4GHam71CaGZ+5o5U+/Xeh+Jn+hJeiaE
Fm7DefoW4oi+eerWq0vpHsIK8ZCnQ0jzPIhTeTAXyaf0DKE9kpEu/4r8WI5x
IWbLz8Eme1ztEcJG/kCcsBZHvdpr8nPl3o/ObM/Nau7L5fNPctudtZfaLLyk
bnJXNc67BflYKJ6k7je9T7zjGb0tvFd83PyRekrFN9RUoSDak5ebV8Ez1Vd4
3yFcKjecs+Sn8hj0d47HZqRYK+M6XMNcM1OzQsjAf84xmPuhUb5SXI84LuAk
yuwx3vo7GIVF+Ag9zP8huh++0yY+rH8JF4tXc4n1eXxA/B6fMCebvxMP43XW
J/FOcQ7v0Z/OG8S9OWG9La8SN3UL4aD+4/yx+Cx/Yb2WF4vLeav+HfyueB8X
Wv+WJ4jX8y79M/hV8YecZ30FvyH+nLfof52fE4/jH62/zFnigfy9/vu+WZK4
lXi29UzuLO7Ea/VftN7o+93kHOt/co24iDdGd5hPi7/hNOu/8iXxz7xMf8L7
XjGvhIdZH+5ev43uGIRn8ZU5+X7/BViDOGKYqr4++qY4iUpciPLmFviOMWzz
/Y9hL6rkM9yTfjjgbrXnze5Kizu01B53eD5nql3geaKanfwLTukdKl8ot9v5
y8XF+i/zLnFM/jV/1yMQc+bdGO0ds/W8gBexwzvmmnVI/Thnb9A/nuPiWfJJ
njuZ0xFrrb1ldp7ngXoHoIPnJeYWO/MJrFbzOx/DUXSL/v7Naq2uq+d1nud4
n1XqcrmDs6UhFc32uuc8tX73d/lLNR31PI6+LS8TL8UUfcnq26EtGvVVqa9W
d4MvyTXI/cP/4iKmO+Mivdn2b2fW85zMn8l9irFm9vRuj5z5IW7rP6tvjHyz
uAkPUBPdJ/k3o3sobsA9XJU/Kj9a/m70vwl1KJdPyJ9zrv8Bp+n8rw==
          "]]},
        "0.07`"],
       Annotation[#, 0.07, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV0nloj3EcB/APOTbnZhczYyhJjjlmMQr5Q8pVskKk5EiUsk2UkiNRcuYP
x+ZMZiQxNkkiSpLjDwmZa44Z2cyVvH5/vH7vPt/P8/k+v+d5vnmLVs1a2Soi
uvgZzBCGMox8hjOCssyIaakRJ+RJjjO6d0QhIylgFKUpER9yIz5Sz2c+ccr1
X+U3GmjkCzPteahHxGEOUs4RHneLaMmK+Mlqmmhgnf3/yX3ZEe27R1Tbt5/M
5K29Lpq9wSmqciIqKLb+pFdEHXf4SyPF7lEla9hKBXvZ7B59/behZDCFIgpd
fzcj4mWfiFq5PvE+0iKeee4LsoP+AXlfvV3uts91Wa++Jyfrv5PN6qdym/nO
6RFd7ZctF+uHbK3+rn/E/AB1jnqkPKo/TQ5UT5D7za+Rk9Qb5Wr9pbJAPVdW
mt8pZ6gPynP6l+R89Wl5zPxzWar+KDfoP5DL1TflFfMtcpO6jeet1c+Vu9Sp
8rz5yfKMepbcoV8oD6sHydvmF8hq9Qp5V3+LvKVeK6+Z3yMfqstllv4y77+E
icxjOo/tc8d3fEINjdTRbL6f/rDEN1VnymSWWH/l+39JnCOSnJU/sr/9pzon
xeQ4Q0XykV6+fOEeC+Vi3ugtk0uZ7TukuyaDNIp7RsxhpXs0ORPNfKeFH5x2
Hsf5vuMZQxFjqbP/L/3f/OQvf6h0fRUt3sFZucq+V63XcJkrVPPafHKeM0YS
pZ6nhLWU8Ua/nXfQnrYkk8Rb6x1lJzrQhc68s54iU+lKGt14bz0j8S5JpztZ
1FvPlj3pQS9y+GD9P9mIo6s=
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1H1MlWUYx/GbtSRacyoeAUGSQ5KmciimSKSZ0cabmlbSkkkSQWTL3Mq1
LGcJhtoM8W1IL2TNzIYNzWzammspbM2V6VpLM105a1PczGVqap/njy/f87vu
+7ru53nOc8ipWzj7+aQQwnp/YoNDmJQZQhEmYnxqCPVZIQxRL5Tvwd3IU39c
faB6gZxAPnLVH1W/VX28PA5jMVNtBpLVp2BlWgjFw0LovD2EAfIheWN2CPfZ
Mxn3IsOsEs637/xIZ3CuWXHErBVZuzyI9bfqH2N9g3n/qh2U4/I5fbmcricN
A/Vl6ztvT4G+TGtvODeLl3OK9aUck08MD+E3nETC3lfVU9WX8FB+jUc67z+f
mzJCeBYdQ0LYaf8253+Cy+muBz+ob5eTzW/g/fo2q/VbK5OTckIo589j7kP/
aBSYN8f+nc7ZhVLX/xAW6H0OzT63YAX6zGnlN1Ghr8c1Pcg/ue5uc3dgsVmL
+G+cxSuewTVe76wB+vfoiXMMf+jbpX8/tqLbM+tCtfrRESGcQi+uoh/V7qWb
92IFurAOzc7Ice350TNDGUpQZH+fez3h+9nHS8z9yPX94r56OMX6Jj4kr+K1
5nzNZ+TvuNT6ab4o/8yt+m8b6vs1L4Prrd9QT5Iv8Hv689Qz5UL+wPp0Hi1P
5Q36X+Rp8uu8yHojT5Cf4E/1r+GZcifvsL6ba+RtvEX/cV4s/8VLrX/PTfI3
/KX+f3i5fJP73Wd9BLfJg/gz/aW8XZ7Fq60X8bvyXXxA/zzeIy/gPust/K38
Mn+lv50Py+/zMOvPeP4v4QHMxQwcMafX93gUe9GPU7ioP249EX2ncoxvQYP6
Sd//ORxBsnflCueaX+E9qUamd6iEf4zeWf7VGbVcj9+tLfP8/+QzOGZemf4D
3rVKrsJ0HLZ3tXd3FVbihnfkOip8LscXPu9Go1mX7F1j1tvoca9PegfmYx6q
1Gq5kT/GVly3v4Yv8BZuwmnnX5Gvok0+6Lrq9PXyC85biOPOO4Za91WDFj1Z
fqfNPFdu0ztK7/368ngqD7dWxWPkcq5EBdqj+zPrLSSi/6fuI59TI0e/QXsu
eW4b7bvGm+RC9U55M9r1rUOxnjQ9kzidJzivw97guivVstXyXPudmKx2M7r0
p/A79hWb+WH0HNBh3iw9s/Ew4nof4SlmzuE75Md4FFfzNPU6HivP53H8FJep
N0T/p+WnOcGNXK7+P61c4jI=
          "]]},
        RowBox[{"-", "0.07`"}]],
       Annotation[#, -0.07, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1W1MlWUYwPHbXpnUlDggkAanRQplhR2JEAOkJJoaWWMTtFotQT6UTszl
cutNTVqLU5aOlJfKtUrNtCgsV06Hw5kiVGbZ1HStlrZ86UOuZb9nffjzf67r
uq/rvp/nnOcQf+SJmY8PCyEk/WlKDyGRHUJ+Tggf5IbQkhbC9a5Huc7CR673
Z4ZwsesB7s0K4TNUjQrhQl4Id/I8M9pGh9DMSU7h2NUhZCATf4wMYci6bzAO
U826G9X4zax+ex66JoQf8B2+x0G0otC8G1CA8bgRbfK3cgITUIyJSMr3m78H
CWd9PVrHk8y/4Aw11tyD65wpH2Xyl6DdOUqsy3Puaepxns5r9K/GPvOutK7T
uknWtYon8xn3X8b7xWnqb6lXiF8Sr9NX6bqd1+JNDMjHrHvI7IfxIArtVW3d
VHRb04WJzlYgP0f9LvlOuQ4k5MfJz5avkk+Pe/48Vm6WXAPqkW6P9Z7rSO52
psn2LbeujLvEI+TfUb+CO6J7ki9VL43OLU6N+tRTuDA6S7Qvcu1zm3Wn3Xcx
V9prCu5ABcqxKrpfvfeb8wBmImHuGrnVOO9zKDE3bl4eivTcgnPyCfkcueyo
JpeLP+WL5JebMcaex3wff8Zx7LDXV9FzxQEstr7EGZ/k2zlVbmtGCMN5C3/B
21Gr95x7aDIv0zPcFM1zvZFPcJozL+W91j7NX/P86FmoL+Ai78tKvIgV9lqO
ZSgVN1tzzPqF9v/dmWfIhatC2O3e73Ndi3vRZ+4U/ta8D63bjEV6WvgMnnLf
//Br5l2qt8f6OMdwXM8WvV9iPTbYuxN18kNjQjiKPpzHKdTZfwP3Yhk68Sqe
t0ees45HOqpRiuLovJ7XT57RNl5i7tvOdshz2cwp6m/wXvHK6N03Zzv/It7D
Veon+Kz4IK/QnxrzDpmXxY+q/ysfxKd5nf58+RzxBO5Sn8ZjxeW8Sv9CrhQ/
w/PV53JCPIvf1/8yzxC380b1j7lB/C536/+RF4l/5aXq+7hJvIM/1f8XPye+
yP1uUx/Nr4hH8Cb9VfyeuJZb1Yt5rbiAd+mfwz3iZt6t/gLvFC/mz/UneUDc
wRnqjZ5/CypQj+kYNKfP5ziEXpzCUZyN3hf1m6LPVBzjy/GY/BGf/0kM4jLf
lb/5WvNrfE/qkBO923xA7WY+bI9GXiL/rGffw5/gsFk1evt9zxq4zTuYxKC1
w8yrl9vqujH6fTNrbvT7mv3//4//AJoh40s=
          "]]},
        RowBox[{"-", "0.14`"}]],
       Annotation[#, -0.14, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1Hloz3Ecx/HPnCU5ttnPNnbYYrmPhvkDuctVMmzhDzGhKIS5r4TkjAw7
3MdIErmSiNIIE2mOFaGQ3Pf1+OWP557f9+f1+bw/3+/n+/0tfcKMEdNjQggj
/clsFkLDuBDSOQMtsDIlhMu4gvPJIVTFhnCOH/I8eSGWJIawDEvxqlEIcfJY
NEZ+QghHrc3jOqkh3OXaXMnFkRB2ob76kXqv+fuwH4+ahrCHd6Msup88x7wM
86uS1Mi0Xwb6ux4t/5gWQj/XB+w3iucbG8mFnMv5+uRhu3ue6zmLeAd26rED
9zzTZ/2/YBbeY57n+cmbra/lnk7plcZxeNo4hBPWXcQ+lOtVglzjlc1DqMZV
fMcb5OpfzmewEiXYhOX2SHXPbRGLAchBtvnXmjgLz3WWC/Xd474fOIfjXFe+
lSvUq3mDPhf4ufo695E/4w/q+7zK+nrxzlu/CE+Q/zb+V/6Od1mfaTxR3olL
5YO5pbonb7F+JvdWL+EZ8kncRT2GD1u/joeqi/io/CTnqw9wmfVVPFv9khfK
b/Jk9SU+bf0nXqaO8bxn5cm8Xt2Aj1nfhw+ph/MaeTbvVGfxFevH8in1FL4m
X8GX1XP4nPUb+Za6mOPlBc5/FnohD0NwR5+r3mMlzuANqvHB+jR5u+g7Vcdx
HUw0/sT7f407qO1b+cbp+g/yneQi0TeUw7dl7fmhPQq40Pg0nooH5kfkTVFh
/XLvJMH1ZN9sASLpxnx/t3wbKzhfdtD1Wi5Xr+YjvIbH6TvTex0f/Zb5hfFf
9ihT70YeDut5CAdRLW9gXonxUvxQp6h/cip/5e/44ey+Refad4EzWIj5WIxF
eOwZLkR/v9zFuXz0nK2MZ6G58RSk4ovxLfpm6RXvOdvyW73bRH/n1v2Sb5O3
UyfIO/E7eUfuIf8j3y7vrL7hfitQ0/ig6D7mD+BmPJBrGO/Pyeq+nMT9OMZ4
iTMpRXfr/+jfjYvV2ZzsvLtya3t05t/R3wUXyTtwkrwjZ8qHuZc2rody68j/
/6X/AECZyVk=
          "]]},
        RowBox[{"-", "0.21000000000000002`"}]],
       Annotation[#, -0.21000000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1Htol2UUwPEjaia51JlzqW2OmIK3QtSmoDGdBV4zBpuYkLqZmnhFkcRq
SrjNvMwUvItouX4bETGdEirKhlPKS2UisXkhxJwpFpKYiJ8X//ju+55znnPe
533f57ecWYveX9gmIgr8qciIOJkZcQLLu0bUZkesk1uAl1+NSEMn9BGv6xEx
xJo7PSP+wm3kyFdkRXTtFtGofk/ub/ST32LWJrXevSL69o7IxUbxcDMedomo
Uu9rXZO+M/hfbqt9bMMg8VT1gfyV+D3Xw/RN4V7mdXG/DWbl6m91v7vJPa3N
El+WX8+/cSW38JfczNdxDU1mVZk7xryxyMc4FGCk2W+Z10PPZ9ZmcHpORI1c
s+sUt3CatR/zCWvmJ++Rx5i72Nx8PqA+T/6wuA6D0yO+sfdK77PYPSq9jyIu
5wqUqf1jxhpei2q988yYj7mY5PkazFklX2wP1XLTuAjTrW8y9xPzdvA5nMXv
7tlfzwCs0N+ZH3OVnrbJvszK5nTcsOfv9RzHAaTsaQ8K5S+95pnRgEdoRaHZ
Ka7HWuzBZpT5jlmeY0ByLjAOeRhqfWP3iD/6RBzllebut98rnuM7fkF9K59L
ziBvNOdH/lPcxPnqN/mB+DJ/ob/jKxEvmZfBM9WfyD9Vv8+79L8un6n+Ju9V
H8+54lG8Rf8Sflv8KS9UL+Eh4iKu1r+eJ4q3c436DzxN/DXv03+Vl4lv8Sr1
n3iO+CQf1v8vfy4Oz3tUvSdvEKdxrf58PiSezOXqQ3mnuB+f1j+d68RzuVF9
DZ8SL+dj+jfxefFu7qZe6v0vxWgUYwIumtPgO15CPVrRggf6s9UHJt9UnM7t
MVu+2fe/g4to56z8l/wPMP9d56QQmc5QHl9QG8TDsVKugMfiSvIu9PyMEWbm
YXXyu+ISZ7UUH/hOMzDbdYpncZnzeEp/ibg0qeEjzEGV2rfOUTFXcw1SeMce
jum96p6nrfvQPWfiDfERs+qT37m9HUx+68k3F3dw/SJ+1V9u3i9Zz/8XPgPd
tMKb
          "]]},
        RowBox[{"-", "0.28`"}]],
       Annotation[#, -0.28, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1Gloz3EcwPHPhnIzzMwxDHuAEkuuOR645Yk5VyQ3Ze7kPmLEZuSBW0KE
5JhMYszmSo4HCFHkCqXxgCTx+ufBe+/f5/P5fr7H7/v7r+2UeaPykyIi058+
rSLWNI7ozat5RFpEctuI4Xy1WcS+RhHlrSP+NI1ox+syItI9r+VmXGzMohYR
C7EARXJDm0cMQR/koCd6oxcqzfu9YcSqdOthBepZdyV/VP+AzebehP6es+Tr
qi/n1ubuJ9cXHcR15DcaV4B3cu9RYf7a8qGehBvicnyz5iT1L1xHvjayUd9Z
u3EDXpRiLzzRuAJrzXKe2ZiDbc6Z5Pz35YMfcDI3xTfP13i3d/XWuF1cJp5u
3TOpETXt7yzX4jx722q9CVzIpdbKN3YeRolzMRqPzVNk3UKMEd+2xiB+Zo/n
zNtc33leZp4lXN/4X7xdPtlzifEZnILXes7ovYLDONEyYj9y5R+591eowE98
Rq79n+CLWI/9KMZa766VM3REQwxED2QbX+l8L9o4Dy817yH7eupMp7mG+k6+
Ky7gIvNc5rfi2zxA/Q1XiR/zBv01m7gj86XyZPXf8n/Uv/Je/Znyaepd+ID6
MG4vzuEd+udzP/Eqnqs+lbuKx/Jx/Vt4hHgXn1Q/x+PFR/mg/me8UPyel6vf
4+niMr6g/zuvEf/lUvV05y4U1+VT+gfwMfFI3qSezXvEWVyuP49LxDO5Un0d
Xxcv5kv6t/F98T5upD7N+1+AvhiH4Xhongr3+AgX8RmvUKU/Q71T4k7FKVwd
U+Rfuv9PeIhqvpUfid+V+Qf7TnKR5hvqwQ/UOnN3LJVb4b1f4Ez5En5ursF6
7/jOxvH4xDeNJ8YXJ34vGCs+YvxhLNZ/K3HfuJn4bYiL7W1G4///g/4BJZep
rg==
          "]]},
        RowBox[{"-", "0.35000000000000003`"}]],
       Annotation[#, -0.35000000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1FtI1nccx/FvB5DoIrVyubTmqouCooK62aKLrdMmWcoaI7NSiyAP1Tps
LDpQBtFypktXFiOioSujk8WoKLSNkk0HsY62RXRyZK6LeRERvR66ePt+Pr/v
7/t9fs///8OswvLcsj4RMcafuvSIKVg5PKLmnYiOlIjeYRErfK7AeLmL9yIH
pSMj5vF/9vTginqJ3koz8gdH7OaFvCsj4llyRJW8SP6OC7gErWas5AJ9i5CP
xfK3er4xezcvkRdbX4KlKJWv6CvjQuwxbylX2tvte2rkIrmai/l7LtX3q/M9
cc6nGG1257sR9/CHnglq/1qrRR3m44V9z62f9F25cpkZ1b6jBut8Xos/1S/b
95XPG7AeVWnOZH+bWor5qUjGIEx1lnJnmmXf0KyIZfaeGRExW76nJ1fthnl5
/Lc8y/475iTZ0zQ0YpQZx3k0b7Tnkb2PkZbo5RkYkRrxv76+9rTzU9xNPCNz
+mU6C3r9hnGciR7rnXIXmtGBFrR7LnPUF+BDrEIRUszf7wy/v+f98Dn9X/Il
eQu3qi/nU/IXfNmcMfyDPJl3qGfzEXk6/6x/IO+Sh/FZ9ddDIjbJL/i0/otc
LLfx1+oPebV8kw/qP8yfyye4Ub2WP5F38k/6P+OJciGXqG/kaXI5V+n/gEfJ
s/mA+gROk7N4n/5uz/qVe/AycRfVh1gfoJ7E2/Rft96jfp+nq//GD+RfEndZ
fwVflau5v/ox/kv+kdfrP+t33zavlSerT/W+P8IgjEUGNptT6fnXYyua0YBc
+7sS7xQt6ERHYt3ceu+xAYe8//M4jn+sJ7snmTjl/rXxtcQddpYMd2Y7p/M2
Hs437f9Y3213LNvadXs/5VvyTPd2rj3F5uRwkzzJeS7Ys9lau3Nv4VK5BGvM
avS7N1lrTrx7+4/iGN63noU8c2rT3/4/egNFjbSJ
          "]]},
        RowBox[{"-", "0.42000000000000004`"}]],
       Annotation[#, -0.42000000000000004`, "Tooltip"]& ], {}, {}}}], 
   GraphicsComplexBox[CompressedData["
1:eJx0vXl8jNf7/z+W0oVaaqk39aaqlGqpVpVyxlJqeVdbuqAtXexKi1ZLW4NS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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0b1OE1AYh/G3E2yWFmFgYrNEqwxOtHAHEhwIAYNoLQwkAjOgwCAUh9pA
GOES+LTeAOIMujpoSUiAkqC9AH4Nw5PnnPP+3/OR0/125uX7REQsoPEgYiUZ
kemMWOYySugxX+f/6ql0xLu2iGp7RJEP+SuOsK+eU9/j56mIbx1yGJGb50Us
4APOZb7zCY7RkPnBZX5kjwpn+AsPc85+edSc87eJ/tfu9cR4nLP8GDcPI6bc
tWjthfEgdmU39G7iqf0mnJPlN7yEj6jJzMnOotW8BdNIog1r1kv4J1fQW7H2
jAvcyzv23saR+pBczl3m3KOP+5HHb7VbmVU9o3o+8RivcV1twDv/mNeb7+NL
vsYVLtQ/y+0bH2DduIRXclv6dq11eXNVbq/5J3ym/hOnSDj3F0/Kp9P3f30H
GP5Mbw==
          "]]},
        "0.396`"],
       Annotation[#, 0.396, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0s1L1FEUh/EjqOG7lo4ZtihDozb+A6a5ScpFBZarRnpxkeAIQaZQttBN
MZK6yG0JBdGu2pRoK4Nw0q0JopvetM2MbqKgz+Di4bnne889986POXY9dXmw
ICLm0FcTsXcwYrg+YpePHIr4mYiosP7BlVwlq0Q1ahCyXj7HXdjW9wsZfMEy
VrGCJ9URtTzFdfynKqLV2ffufaN+h7c4bM6Q7IR3/NXToOcf19ubcTbB0/xR
vogFPFSXybtrI3J1fgd28SE/HxvOF3Exvpq9qW+Nt/iAbMWdGT1pnsQZWRuS
5naau8VneZNPOnONn9qfRbu6A6+tz/MFPLBfon+MS3mIs96T4rvqS+4eVt/D
fP6c+o78tnoAhbJ17xmVXbE3wle5XF6GHXtNvk+P7JX7RmUjyCKHl2i0/4KP
8mf9N8y9iYv6wzvS1gWc0ZOyHsR9dbG7bvFvdT9XqCe4nMd5Tv9zPMNps07h
uPybuom/c5+zyfx96h3u4WZ5Cx77vo+w5E1L/Cn/zc1O+i3pxP7/8D+z/FxJ

          "]]},
        "0.33`"],
       Annotation[#, 0.33, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0slL1GEYwPGnIlAItEVTWqBLQdKtiKig0Cgbm2Ky9WDQQahDUdDyB1hJ
CXVIp+jUsUN3rTyV2rQbVKC2WThq0anUcavPjw5fvu+zvb/nZWbV8dOZU3Mi
ohOTCyN2LY7YictlEZfQhAn5Q6URq5dGHOZcScRRXiM+wh/UnyAl7uLH6CiP
6OF27uZNiyJemFvv7g14LvduScQzfs/tam+cH+rv5Uf8kvvVXnMfv+Jt7hnV
u9kdn8Uj8l94iIeRx4D6fH197hhAPzr1TKvNoMnsVu/K6tuhrwiDeop5t1ot
xtWm9Jz3vgKf48nEeg7gID7KpfWWOM+69y8axBmz1+Xq0Yo27JGvw0kzWXGj
u+6Lz4q3m3uLXlTITZlvUc85P5DrwCfMFc/DAvND9s3jqr5rKJhJqS3Dcpzx
3v2c0luHlc4rcDfZBWmUIpO8N3l3sqdd1vkNs9yKCvff5Eq+o96CrD1uoQ1b
fOM2b1S/wVXiY85ruYFr7XXPXjN8wT3TfJFn+Zd8tW/X4Kf+fb77g2vk85wW
Dye/K0axVzzCXerfzH/HV9S7b5Cr5Yv0NOu5gjH7jPNvHuM/fML+OXsVxE/L
///f/wE9q3yT
          "]]},
        "0.264`"],
       Annotation[#, 0.264, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0rtL1XEYx/GnoaWCPJ6jpZs31BBcJCGIpKAhB0nzci6DulQOuqlZkdRU
mTUEVuo/YGsQBXoObXkDL4MOUo1dBnUNpdeh4c3nuXy+z/P9fflVDY50Dp+I
iHn8SEYMI5+KyJ6LeHI2oiD+UBrRWB5xARl5rtgvahnFkv5aSUTa2VW6KP+I
PApoSkTc4GvHgplD5ryneVrAErZ5qp2vQg2eyv86e5nelLeK//AlaB9N45k7
TCEjruZ7Lr7P+wApu2blV9Tf0TY6R2vooRm19IW8jmadL+Ofxkt0oROf3LGX
9zPtlt9CC/+22hYmMGBXP0Lvt2/fU2vgmTb7HiYwinGM4aF+zrykOIUSlKLH
eydoL83qL4tXUMlfgWY7rpu7hU38tOsu7y/6DW/Vrsnf0A3+TcyIz7tXBSqx
rtZh9mtMmvcIxzhCu9orjIvHcIB9XFWbwo4dF73TLr1jzm0sF2t2rtB6+Vd6
Un6Jv7/4Nviidsru0ziDx3pp3/U9+f9/+wfrAVgf
          "]]},
        "0.198`"],
       Annotation[#, 0.198, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl009Mz3Ecx/F3HIyh0C82zK8krbYUkz9t5GA2Bwdjc2C1XxYLZTqS4erg
YDrQ1K8yRmE2yq+QjtQvNxvWcHJwsLlkYzy+c3ju+X29Pu/P5/P9fVelmY6D
7QURkUdxUURtKqKsJGI9HsvdcorruFfXhwvLIrJ8f3nEA3TJv+TqwogheRP/
kXP2rULbioix4ohxvMQLPDFXaeaQ58O45fybWGv+rv4elshLUYin5p9hxnqT
+Tw3c043iTacQtpd61Bu/bT8gTt4A//gCj7LH/kcV/FPruZOnk3mnduyMqLG
84i7R7FItxCdOG/uuDtakMH35LvpevzuMrOl2Of90zyBV9hs5jVPosQZKXTh
UnKWfRv1a+Rt5lZzPYfv2uA9/nrOmmlBkVygv2LfZZzUzdi7x8w0N3KeT+in
eLf8hnfxW27Vf7Uvw4XOGnTWF/kz+j236wZ4LrmT98p9PGv9Np+Re/mb9U+6
I85ZrOvR1epy7hjHWPIuvsE7Mw/5hvVh7uZp3RQO2HsdDWZ3YhhD2K+7hh2e
t+MOBpPfqLuKSnvfu2uBexvlVrnO2f1mBpBFOvnemG+m3kxF8jeACd083VZd
kzwqb5Gfc7k8wr+dVWP9qHwMj3RzuirdRaQ9lya/KfX/f+YfGgxwwg==
          "]]},
        "0.132`"],
       Annotation[#, 0.132, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV0slLl1EYxfGrVmomqT9Nm7UkkOYwtMk2zZORjQsXCZWRlGlpE402bbJF
i3II0sqiAVpVBA3opk2DVtDwB1TQQGVQ0PB5F4fvc85zn3vvO+RVbFuxNS6E
8Id6BoawPBbCC5yeEcJtdbf6QmYIGfyq7BBiGJ8VQhwVpIfQqDcEc3FJWgin
zBxQH6Sh8jxcKm+UH1IfpmHyUbhMflp+RD1clmP/Oue9sXetulT/tfodvaV6
vZPOXznIXaiMcsytxkJ5tjrLXI11J/jF5rNkmbLtsg6K8bP5TFykn44xvlrv
Mg3kS/g0XKifihn8Vr2LlMLP4gfgcWcco2T1TFl/XOCe82me2Ve42Uw7JeqV
umcSNpg5Qvs9dzw/HKdZ/9j6YvwVnWu/cnPzZHNpDnVRJ62Tt9Jke3yydoN3
uJE20T7ZDvsNoFRKoe1UTSXODxhH/6ytwi00Q97L/6QfVCnbRMXyD/xHek8V
svU0Rf7IOV3qBvfvxKN4V3aHitytgaa6ayGNcMcWnCT7a/YeP4TOyQZjhWct
s8ctdb7+DXvcjN6PPX/z48xd5QtwopkJNJ7Srb9Ok81fww5rrtBec73mxkTf
lM/H7/weeYd1u/EbP1p+ST8Pv/K75O369fiFz42+uf5I/MzXyc/rt0fvWF3l
3J3YJDuvX6tuxRpss6YlOtNsM4513+Tof8ezWGn2HCbRC+82EV9iX/0e7Mc/
jzx2Yx/5s4j8E0zAp3gm+lb2L3JmM99CD6J/Tn4fy53Tpm6St9Ja+zyUr8EE
+g+g+oBk
          "]]},
        "0.066`"],
       Annotation[#, 0.066, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], 
         LineBox[{13, 2538, 3963, 3869, 5658, 2839, 63, 88, 113, 138, 163, 
          188, 213, 238, 263, 3519, 288, 3522, 312, 3525, 336, 3529, 361, 386,
           411, 436, 461, 486, 2581, 4005, 3951, 5714, 2906, 536, 561, 2584, 
          4008, 3955, 5719, 2913, 611}]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV0ltoz2Ecx/EH5WxHtta2/44yOdYkhytKm1J2SNncbRcmxw1FGRFGuBhW
ymEjZNgw5+LC2SUXszlsFBeYTWohu/H6Xbx7P9/P9/t7fs/v+f9zqjaWbRgW
QhjEk/gQViSH8JTnJoVwxvqx9ZqUEB5x5aQQKjAvMYQaWdPEEHrMxCeE8J7j
+AOPMvOOJ6i7eTy/5ZHyLh6nfsNj+TROIdN+GZhu5igaka7e4b0x9FufNJeA
FjQj5oyZOGc9MzWEs7xSnWv+l/nz6iz1RZ6lf4Er1JP1B/UvqbPVl3m2fitX
qqfo/9ZvU+eo69UF+CNrlyXhmPMl83HOksewyuwMc0PWHXq56l3qMWZGo9c3
p8vX8jo8tJ7jvQ/4TLRPdB/cgmZMs0+VParxWq/afb/iQnmDfReizz4/UIfN
OCJbEp3V+i/qsRONsmUYbo8R2C3bgyZZCeJk8dgn248TsnJ0ene+s+ShRj4Q
/aayLhQ7x3P5CzxDod5L/sfzfdcQL8Ahzx2OzoYC/an47Pkv0d37L92wTyt3
8FdZCZei21yR57u4mK+YSbTveuf6Zu472mSbZFe5T93OReprHBftjVr1df6h
38HF6puchNuoU9/ifv07vFR9l1MwINvifSmye+o03Eeqeqv8p/427nXGj+hB
ibN+4u3yNHMx86WyRt+/N7pjZMnyuEx+NLpz6wZky/O5PPqPyQ9YH0RO9J3u
pIhXR78FOu1fK1vkDIuxEN2y5Z7LsP4PUxqJHQ==
          "]]},
        RowBox[{"-", "0.066`"}]],
       Annotation[#, -0.066, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0ztszWEYx/GnUQYDrUu7CGmkmlQvcc5pS7Vp4tLEpQYRNdCEtIPrIamG
qS2R0MGKGEjpBQPVy0QNQjHRYmBoQtSEWFhUff4xfPN9n9/7PO/7/pNzig5l
d5/IiYgJpAojRvMiVi+J6FocUVwQMay+sjyigFO8RlaCMfVVdSGnuVd2C324
jfvO6MyP6Lfu4iH1A6SdG+7plj0xe45z1Bn5sP0NnKs+L39u/5FsgqtkL7gI
1e6rQY/eBjzTs40X6rls7pX6JWrdPajvDgZwD3dRn3wXj+AhxjCKBvkZc2fx
1T2N6hm+7uxmPNYzjk3YjN/21rvzF3eaqeJZ1Mr+8E0zB7FI/Vddj1zvm4de
edrMAusm98znXXxBdszeO+dv1T/FW/gtH5VPJnerXyfv4Dd8RD6TvBmDzml0
3wB/UfdxVt3PdfpXLovYyKu41Gyb2Wl9LXwDPUsjLiGjZ4f37MR2/FR/0PcR
P6wP6ylOfg94L9tj9hoqrGvsVaNSX4f6gHU5l2Eusbyd98vXcilmUSI/xfnO
7LB3Gu34jm+oc/5FZLxnUm+rb5jiNjOtKJc/VZdxJSpw0lwWn/EJ68x3Y6/Z
ZkzLVpg5zvvU49Zz5lJ6Wpw5kvf/f/EPGWVujQ==
          "]]},
        RowBox[{"-", "0.132`"}]],
       Annotation[#, -0.132, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0ktLlVEUxvHlxAQHec3ykvQBQhMiTzfMCyjUoIiwU2oEBVodyUmDpJlC
UF8g8QYODB1Wg0Sh0vJybOIlg8SJjpPSiYX0kwZ/nrWe/ey19355T9ztvJbK
iIhhbOVHPMaHgojbRRG9hyM+qifyIiqPREznRLTqZ+i53IgambO00HoBrtub
0O+pV2V61ZkyF835xJvGS96MGS/oZzrLS5jdru7Anhl/8Qfn+Y94KWQiC4fQ
hSf2lRdGHEcZNp1XwXvlrBvIdu4W7xR+4xcG+FUyu+qnZuzQbvqMd8faMXUx
inAUzWYkcRNt1tO8JC2R73f/DHcMvMNbvMEZ2W1zm9z9y8H70Cc7R0vds01m
Sr+ob6ApuU6s639gUj3hjDX1d3xDWi5hXzW+Whvi1fPG9Mvq9/bM6R8efC+M
8Jr0XepG+sCdNnhr6gFaS/tpHR2kl+i+N702bxSz+iX+vrPm9Wks4Ceuyo3z
r3jLZZyUvcW/wHuO085c9v579q/Q+7RFrhWDckNIIkdfLd+DFpnN/P//3z92
MWEA
          "]]},
        RowBox[{"-", "0.198`"}]],
       Annotation[#, -0.198, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0r1L1lEYh/FbKKEa7E0dnIQEGwwsNJOwQnsM8UF9pKZajAIza0nB1iCl
IKIioj8g0nypTdFIHWuoJSV8KbEGjTJzUMnUjzhcXOd7n3Pf5/zgl9twM3Uj
LSJe4mdmxOW9EXPcwPPckxGR2h9Rj7J9EbVcgwfqCRQfjLiVHXGci9Ckr4Sv
6O06EHFdPiHn8wu1w7yqPsnPzGnmY3KJddK5u/KI3O7sKPdyH15jh/2duKP+
1N11eKjvPOdsnclyFs+9s8KbWpwtlx9zsdpV6yK+Zu+C80P6Qq3NnRvybU6T
V9QL3XEUu81v5zL7p5CQ92DWuR/4jnT5G89gCl8xjXFz+vW2um+AOzBgxmmc
Qbm+Cvz3plVv3OBlXucVfqv/gzn/5Pe8xkfMKECX/ETu5G68wkkz36DAd+S5
s9F6zIxLcr58kZPyIwzpHcQn94zyRx7hbO/JwiF35GFMrVrvZ+7V946r5GG+
h/vYJZ+zN2mdaz1jXap3Qp5GUm2Kv6j/5Xp5iRewiJT8hyvt/+I6+TcnzDiL
hYztf3MTdR9nyw==
          "]]},
        RowBox[{"-", "0.264`"}]],
       Annotation[#, -0.264, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0jFI1GEYx/GncwgdOtPz3LSETtRBZ6eKMsGwvMQlg+6ItKBzCCuovTb1
ukGb03JJy8H2shMEraEhR6MSHNQltYL6SMOX7/P+3ud93nv/3Mn8SLZwJCIm
0ZGKmK6NuMDdeFgdkUhHPOIK/puMqKmLqMWo9eXjrL6HWf2XrF9wL7/klZqI
dWce6L0iu8/9XOWOSmzZa6qPyMoK7rip7nTuBl+1/mHuJppljbIMn+BPzvWp
s5g3Zw6vcNeZs7KMeYPuauZrXG9GGgn7SesnfIwf84Jzb/Aa+3r20GXGlHXO
fot6g8/p/8rn+Tqftn8GRW+cwEe/6Quv8bienLvHOM8H8n2sHn6bw++Ed86+
x1v1bT3L6l/qUe/7zQn9/eoK/mzOiJ4pLvAzPip/ykPWRR7mOjNSKOMDlnBL
XrK/aOaOt+2iqG4we4Lz9n6atYeLeqdlPTzDZW/5I1/mO1xAm5mtaFFvmzXs
W+Q4j+fyb/wdA/JTZpRk7an//61/DcNWzQ==
          "]]},
        RowBox[{"-", "0.33`"}]],
       Annotation[#, -0.33, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0TkspVEUwPEzCck0YnmWRKemsdQjYykUEtGIghghE8FIzGvGzqOyFMQa
kuktBYVKMZR6S2WS8cQ0lplQ+70o/vnfc+455977fWXdQ63fPkTEGF5yIxby
IspLIub5XLzIFeIlfhWXJCIG8iM2iiMGeZXXsYZj+2f8UBjxiDp9n9GpbqBI
PW4yZ6j5zbfyv6zTfMp/uM78E+SgQ9zGnwoiapE28xZ/9X4xt8G6ixu5Hq/m
J93zq1yTONvMLGzq3UK1eT3iKu7lOczi3rwRvT+Qwgye5aZ4GhOYxJNcgfoE
xsVjGM30OHPH/G0cqWmRq3J+n3wl16Aad/Y+ek/S/ncMo9ldkuatcIW9Zf6v
9oWf+B+ekdabUrdvfYAZ62m0Z/6Bvl25Uu8+VLdn/dNdLu1f4SLzHcTX3K++
OPH+r98AX4tNVw==
          "]]},
        RowBox[{"-", "0.396`"}]],
       Annotation[#, -0.396, "Tooltip"]& ], {}, {}}}], 
   {GrayLevel[0], AbsoluteThickness[3], LineBox[{{0, 0}, {1., 0}}]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.75, 
      FormBox[
       RowBox[{"-", "0.75`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.25, 
      FormBox[
       RowBox[{"-", "0.25`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.25, 
      FormBox["0.25`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.75, 
      FormBox["0.75`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}, {1.25, 
      FormBox["1.25`", TraditionalForm]}, {1.5, 
      FormBox["1.5`", TraditionalForm]}, {1.75, 
      FormBox["1.75`", TraditionalForm]}, {2., 
      FormBox["2.`", TraditionalForm]}}, {{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2, 
      FormBox["1.2`", TraditionalForm]}}, None, None},
  ImageSize->504,
  Method->{"TransparentPolygonMesh" -> True},
  PlotRange->{{-1.1920937271229854`, 
   2.1920937271229857`}, {-1.3920937271229854`, 1.3920937271229854`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 421},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[10]=",
 CellID->172570705]
}, Open  ]],

Cell["\<\
Arrows for velocity field are green, equipotential lines are red, streamlines \
are blue and the panel is black.  Note orthogonality of equipotential lines \
and streamlines, and that streamlines are everywhere parallel to the velocity \
field.\
\>", "ExampleText",
 CellID->680409942],

Cell["\<\
Potential-flow singularities may have discontinuities in velocity, velocity \
potential or stream function across the panel.  Such discontinuities are \
called jump conditions.\
\>", "ExampleText",
 CellID->62898067],

Cell["\<\
Compute the properties of tangent-component velocity across the panel.\
\>", "ExampleText",
 CellID->673890120],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"icuMinus", "=", 
   RowBox[{"Limit", "[", 
    RowBox[{"icu", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", ">", "0"}], ",", 
        RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
       "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"icuPlus", "=", 
   RowBox[{"Limit", "[", 
    RowBox[{"icu", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", ">", "0"}], ",", 
        RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
       "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", 
      RowBox[{"-", "1"}]}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"icuJump", "=", 
   RowBox[{"icuPlus", "-", "icuMinus"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"uJump", "=", 
  RowBox[{"a", " ", "icuJump"}]}]}], "Input",
 CellLabel->"In[11]:=",
 CellID->98488054],

Cell[BoxData["0"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->224494157]
}, Open  ]],

Cell["\<\
Tangent-component velocity is continuous across the constant-strength doublet \
panel.\
\>", "ExampleText",
 CellID->153014540],

Cell["\<\
Compute the properties of the normal-component velocity across the the panel.\
\>", "ExampleText",
 CellID->134318740],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"icwMinus", "=", 
   RowBox[{"Limit", "[", 
    RowBox[{"icw", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", ">", "0"}], ",", 
        RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
       "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"icwPlus", "=", 
   RowBox[{"Limit", "[", 
    RowBox[{"icw", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", ">", "0"}], ",", 
        RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
       "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", 
      RowBox[{"-", "1"}]}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"icwJump", "=", 
   RowBox[{"icwPlus", "-", "icwMinus"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"wJump", "=", 
  RowBox[{"a", " ", "icwJump"}]}]}], "Input",
 CellLabel->"In[12]:=",
 CellID->326115239],

Cell[BoxData["0"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->23245551]
}, Open  ]],

Cell["\<\
Normal-component velocity is continuous across the constant-strength doublet \
panel.\
\>", "ExampleText",
 CellID->531709516],

Cell["\<\
Compute the properties of the velocity potential across the panel.\
\>", "ExampleText",
 CellID->211831706],

Cell[BoxData[
 RowBox[{
  RowBox[{"ic\[Phi]Jump", "=", 
   RowBox[{
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Phi]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
        "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", 
       RowBox[{"-", "1"}]}]}], "]"}], "-", 
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Phi]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
        "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}], ";"}]], "Input",
 CellLabel->"In[13]:=",
 CellID->189498322],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Phi]Jump", "=", 
  RowBox[{"a", " ", "ic\[Phi]Jump"}]}]], "Input",
 CellLabel->"In[14]:=",
 CellID->48296031],

Cell[BoxData[
 RowBox[{"-", "a"}]], "Output",
 ImageSize->{19, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[14]=",
 CellID->612304285]
}, Open  ]],

Cell["\<\
There is a jump in velocity potential across the panel consistent with the \
jump in tangent-component velocity.\
\>", "ExampleText",
 CellID->56731244],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uJump", "\[Equal]", 
  RowBox[{"D", "[", 
   RowBox[{"\[Phi]Jump", ",", "x"}], "]"}]}]], "Input",
 CellLabel->"In[15]:=",
 CellID->2069797],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[15]=",
 CellID->85110783]
}, Open  ]],

Cell["\<\
Compute the properties of the stream function across the panel.\
\>", "ExampleText",
 CellID->403496840],

Cell[BoxData[
 RowBox[{
  RowBox[{"ic\[Psi]Jump", "=", 
   RowBox[{
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Psi]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
        "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", 
       RowBox[{"-", "1"}]}]}], "]"}], "-", 
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Psi]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
        "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}], ";"}]], "Input",
 CellLabel->"In[16]:=",
 CellID->102020424],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Psi]Jump", "=", 
  RowBox[{"a", " ", "ic\[Psi]Jump"}]}]], "Input",
 CellLabel->"In[17]:=",
 CellID->442886645],

Cell[BoxData["0"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[17]=",
 CellID->23297039]
}, Open  ]],

Cell["Stream function is continuous across the panel.", "ExampleText",
 CellID->325279064]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Scope",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->1293636265],

Cell[TextData[{
 "The general application of these influence coefficients is in the \
construction of the panel method ",
 Cell[BoxData[
  ButtonBox["PMDoubletConstantDirichlet",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/PMDoubletConstantDirichlet"]], 
  "InlineFormula"],
 ".  As part of the interior Dirichlet boundary conditions, \
velocity-potential influence coefficients are computed at each mid-panel \
collocation point due to the doublet panels representing the discretized \
airfoil.  Because there is a jump in velocity potential across a doublet \
panel, collocation points are displaced slightly to the interior of the \
discretized airfoil.  The airfoil used for this example is the NACA-4412 \
discretized to six panels."
}], "ExampleText",
 CellID->246788676],

Cell["\<\
Compute collections of vectors locating panel endpoints, collocation points \
and outward unit normals to panels.  A suffix is added to symbol names to \
distinguish this usage for an airfoil from usage in the basic example for an \
isolated panel.  Each of these collections of vectors is of the data type \
CartesianVectors.\
\>", "ExampleText",
 CellID->121431475],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rPA", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"NACA4DigitAirfoil", "[", 
     RowBox[{"4412", ",", 
      RowBox[{"NDiscretizeUnitSegment", "[", "3", "]"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"uvNA", "=", 
   RowBox[{"PanelNormals", "[", "rPA", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rCA", "=", 
   RowBox[{
    RowBox[{"PanelPoints", "[", "rPA", "]"}], "-", 
    RowBox[{
     RowBox[{"10.", "^", 
      RowBox[{"-", "10"}]}], "*", "uvNA"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Head", "/@", 
  RowBox[{"{", 
   RowBox[{"rPA", ",", "rCA", ",", "unA"}], "}"}]}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->27822125],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"CartesianVectors", ",", "CartesianVectors", ",", "Symbol"}], 
  "}"}]], "Output",
 ImageSize->{306, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->330622408]
}, Open  ]],

Cell["\<\
These are all two-dimensional vectors.  The seven vectors locating panel \
endpoints describe six connected panels specifying the discretized airfoil.  \
Components of vectors are arranged in simple lists.\
\>", "ExampleText",
 CellID->363116584],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Dimensions", "/@", 
  RowBox[{"{", 
   RowBox[{"rPA", ",", "rCA", ",", "uvNA"}], "}"}]}]], "Input",
 CellLabel->"In[2]:=",
 CellID->131208994],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", "7"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "6"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "6"}], "}"}]}], "}"}]], "Output",
 ImageSize->{157, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->954714706]
}, Open  ]],

Cell["\<\
Compute the velocity-potential influence coefficients.  Note that the result \
is a list whose components can be arranged in matrix form with six rows and \
six columns.\
\>", "ExampleText",
 CellID->128983040],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ic\[Phi]A", "=", 
   RowBox[{"ICPhiDoubletConstant", "[", 
    RowBox[{"rCA", ",", "rPA"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Head", "[", "ic\[Phi]A", "]"}], ",", 
   RowBox[{"Dimensions", "[", "ic\[Phi]A", "]"}]}], "}"}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->689942839],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"List", ",", 
   RowBox[{"{", 
    RowBox[{"6", ",", "6"}], "}"}]}], "}"}]], "Output",
 ImageSize->{96, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->417167147]
}, Open  ]],

Cell["\<\
Here is the matrix of interior velocity-potential influence coefficients.\
\>", "ExampleText",
 CellID->257626546],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ic\[Phi]A", "//", "MatrixForm"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->101065266],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0.4999999998102939`", 
      RowBox[{"-", "0.0022273345344517134`"}], "0.006105731350625923`", 
      "0.03146025767449068`", "0.03732853944227429`", "0.4249754370132271`"},
     {
      RowBox[{"-", "0.0022557893706444516`"}], "0.49999999980664717`", 
      "0.010925568281378605`", "0.0894129018986817`", "0.33115493672656665`", 
      "0.06997122788997975`"},
     {"0.0008605550399303361`", "0.010779339859687318`", 
      "0.4999999998105143`", "0.39657375171809556`", "0.0676592759179671`", 
      "0.023650317195764485`"},
     {"0.008463109646500324`", "0.04431754465421936`", "0.39153718221408`", 
      "0.49999999981585164`", "0.03893155493049978`", "0.016272032508912168`"},
     {"0.05467218370684849`", "0.32704823438780467`", "0.06414083607523152`", 
      "0.03946641953990921`", "0.499999999812063`", "0.013881836712484186`"},
     {"0.4238189915990201`", "0.022182543764145336`", "0.01125199927170436`", 
      "0.026501227638181496`", "0.013886127354568269`", "0.49999999981197596`"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 ImageSize->{479, 87},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]//MatrixForm=",
 CellID->528798639]
}, Open  ]],

Cell[TextData[{
 "More details for formulating the model is given in the ",
 ButtonBox["implementation",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/Implementation#577981106"],
 " section of the chapter on panel methods."
}], "ExampleText",
 CellID->198004268]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Properties & Relations",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->2123667759],

Cell["\<\
There is a relation between doublet density and circulation density for \
two-dimensional line singularities.\
\>", "ExampleText",
 CellID->342911820],

Cell[TextData[{
 "\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["\[Gamma]", "TR"], 
    RowBox[{"(", 
     StyleBox["x", "TI"], ")"}]}], "=", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"\[DifferentialD]", 
      StyleBox["\[Mu]", "TR"]}], 
     RowBox[{"\[DifferentialD]", 
      StyleBox["x", "TI"]}]]}]}]], "InlineFormula"],
 "\t\t\t\t"
}], "ExampleText",
 CellID->315517505],

Cell["\<\
The equivalent circulation density for a doublet panel of constant strength \
is zero.\
\>", "ExampleText",
 CellID->218711756],

Cell[TextData[{
 "A constant-strength doublet panel is equivalent to contra-rotating point \
vortices of strength ",
 Cell[BoxData[
  RowBox[{"\[MinusPlus]", 
   StyleBox["a", "TI"]}]], "InlineFormula"],
 " at panel endpoints."
}], "ExampleText",
 CellID->26256628],

Cell["\<\
Compute the velocity field due to the pair of contra-rotating point vortices \
at panel endpoints.\
\>", "ExampleText",
 CellID->12887027],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[CapitalGamma]", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "a"}], ",", "a"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"VVortex", "=", 
   RowBox[{
    RowBox[{"ICVortexPoint", "[", 
     RowBox[{"rF", ",", "rP"}], "]"}], ".", "\[CapitalGamma]"}]}], 
  ";"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->345799449],

Cell[TextData[{
 "The velocity components for a constant-strength doublet panel were computed \
earlier and named ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{
    StyleBox["u", "TI"], ",", 
    StyleBox["w", "TI"]}], "}"}]], "InlineFormula"],
 ".  Test the two velocity fields for equality."
}], "ExampleText",
 CellID->373792664],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"VDoublet", "=", 
   RowBox[{"{", 
    RowBox[{"u", ",", "w"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"VVortex", "\[Equal]", "VDoublet"}], "//", "Simplify"}]}], "Input",
 CellLabel->"In[2]:=",
 CellID->561478990],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->109510433]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[{
 StyleBox[ButtonBox["ICDoubletPoint",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletPoint"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICDoubletLinear",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletLinear"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICDoubletQuadratic",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletQuadratic"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICSourceConstant",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICSourceConstant"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICVortexConstant",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICVortexConstant"],
  FontFamily->"Verdana"],
 " "
}], "SeeAlso",
 CellID->745892494]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["2.1 Introduction to Influence Coefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"]], \
"Tutorials",
 CellID->290096468],

Cell[TextData[ButtonBox["2.3 Isolated Panel",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/IsolatedPanel"]], "Tutorials",
 CellID->112343462]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["InfluenceCoefficients",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/InfluenceCoefficients"]], "MoreAbout",
 CellID->509755738]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"ICDoubletConstant - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 9, 59, 22.3872000}", "context" -> 
    "Aerodynamics`InfluenceCoefficients`", "keywords" -> {}, "index" -> True, 
    "label" -> "Aerodynamics Package Paclet Symbol", "language" -> "en", 
    "paclet" -> "Aerodynamics Package", "status" -> "None", "summary" -> 
    "ICDoubletConstant[r_f, r_p] computes velocity influence coefficients at \
field points r_f due to connected doublet panels of constant strength whose \
endpoints are located by r_p.ICPhiDoubletConstant[r_f, r_p] computes \
velocity-potential influence coefficients at field points r_f, due to \
connected doublet panels of constant strength whose endpoints are located by \
r_p.ICPsiDoubletConstant[r_f, r_p] computes stream-function influence \
coefficients at field points r_f, due to connected doublet panels of constant \
strength whose endpoints are located by r_p.", "synonyms" -> {}, "title" -> 
    "ICDoubletConstant", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/ICDoubletConstant"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[9756, 295, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->361088721]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 251428, 5275}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[610, 21, 76, 1, 70, "SymbolColorBar"],
Cell[689, 24, 191, 5, 70, "LinkTrail"],
Cell[883, 31, 3688, 84, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4596, 119, 60, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4659, 122, 2276, 71, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6972, 198, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->261953622],
Cell[7555, 217, 371, 12, 70, "Notes",
 CellID->1067943069],
Cell[7929, 231, 557, 16, 70, "Notes",
 CellID->476702286],
Cell[8489, 249, 433, 10, 70, "Notes",
 CellID->30234373],
Cell[8925, 261, 717, 26, 70, "Notes",
 CellID->86946217],
Cell[9645, 289, 74, 1, 70, "Notes",
 CellID->529540534]
}, Closed]],
Cell[CellGroupData[{
Cell[9756, 295, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->361088721],
Cell[CellGroupData[{
Cell[10242, 312, 148, 5, 70, "ExampleSection",
 CellID->541600658],
Cell[10393, 319, 128, 6, 70, "ExampleText",
 CellID->513831186],
Cell[10524, 327, 99, 3, 70, "Input",
 CellID->66474307],
Cell[10626, 332, 429, 11, 70, "ExampleText",
 CellID->659456463],
Cell[11058, 345, 213, 6, 70, "ExampleText",
 CellID->441523031],
Cell[11274, 353, 495, 18, 70, "Input",
 CellID->64953589],
Cell[11772, 373, 86, 1, 70, "ExampleText",
 CellID->326501090],
Cell[CellGroupData[{
Cell[11883, 378, 308, 10, 70, "Input",
 CellID->560964283],
Cell[12194, 390, 1394, 44, 62, "Output",
 CellID->506532777]
}, Open  ]],
Cell[13603, 437, 95, 1, 70, "ExampleText",
 CellID->44302699],
Cell[CellGroupData[{
Cell[13723, 442, 267, 8, 70, "Input",
 CellID->575890672],
Cell[13993, 452, 380, 13, 54, "Output",
 CellID->163445906]
}, Open  ]],
Cell[14388, 468, 677, 15, 70, "ExampleText",
 CellID->120984796],
Cell[CellGroupData[{
Cell[15090, 487, 391, 13, 70, "Input",
 CellID->23616962],
Cell[15484, 502, 244, 8, 54, "Output",
 CellID->105038943]
}, Open  ]],
Cell[15743, 513, 93, 1, 70, "ExampleText",
 CellID->472419597],
Cell[CellGroupData[{
Cell[15861, 518, 265, 8, 70, "Input",
 CellID->8664355],
Cell[16129, 528, 513, 17, 67, "Output",
 CellID->315545623]
}, Open  ]],
Cell[16657, 548, 199, 4, 70, "ExampleText",
 CellID->9427774],
Cell[16859, 554, 380, 12, 70, "Input",
 CellID->258291294],
Cell[17242, 568, 947, 34, 70, "ExampleText",
 CellID->78226744],
Cell[CellGroupData[{
Cell[18214, 606, 339, 12, 70, "Input",
 CellID->13857323],
Cell[18556, 620, 208, 7, 36, "Output",
 CellID->28233041]
}, Open  ]],
Cell[CellGroupData[{
Cell[18801, 632, 363, 13, 70, "Input",
 CellID->692553449],
Cell[19167, 647, 209, 7, 36, "Output",
 CellID->350091554]
}, Open  ]],
Cell[19391, 657, 913, 26, 70, "ExampleText",
 CellID->78531558],
Cell[CellGroupData[{
Cell[20329, 687, 4741, 136, 70, "Input",
 CellID->1746043248],
Cell[25073, 825, 205514, 3747, 70, "Output",
 CellID->172570705]
}, Open  ]],
Cell[230602, 4575, 295, 6, 70, "ExampleText",
 CellID->680409942],
Cell[230900, 4583, 226, 5, 70, "ExampleText",
 CellID->62898067],
Cell[231129, 4590, 121, 3, 70, "ExampleText",
 CellID->673890120],
Cell[CellGroupData[{
Cell[231275, 4597, 1134, 33, 70, "Input",
 CellID->98488054],
Cell[232412, 4632, 159, 5, 36, "Output",
 CellID->224494157]
}, Open  ]],
Cell[232586, 4640, 137, 4, 70, "ExampleText",
 CellID->153014540],
Cell[232726, 4646, 128, 3, 70, "ExampleText",
 CellID->134318740],
Cell[CellGroupData[{
Cell[232879, 4653, 1135, 33, 70, "Input",
 CellID->326115239],
Cell[234017, 4688, 158, 5, 36, "Output",
 CellID->23245551]
}, Open  ]],
Cell[234190, 4696, 136, 4, 70, "ExampleText",
 CellID->531709516],
Cell[234329, 4702, 117, 3, 70, "ExampleText",
 CellID->211831706],
Cell[234449, 4707, 921, 26, 70, "Input",
 CellID->189498322],
Cell[CellGroupData[{
Cell[235395, 4737, 135, 4, 70, "Input",
 CellID->48296031],
Cell[235533, 4743, 176, 6, 36, "Output",
 CellID->612304285]
}, Open  ]],
Cell[235724, 4752, 162, 4, 70, "ExampleText",
 CellID->56731244],
Cell[CellGroupData[{
Cell[235911, 4760, 163, 5, 70, "Input",
 CellID->2069797],
Cell[236077, 4767, 161, 5, 36, "Output",
 CellID->85110783]
}, Open  ]],
Cell[236253, 4775, 114, 3, 70, "ExampleText",
 CellID->403496840],
Cell[236370, 4780, 921, 26, 70, "Input",
 CellID->102020424],
Cell[CellGroupData[{
Cell[237316, 4810, 136, 4, 70, "Input",
 CellID->442886645],
Cell[237455, 4816, 158, 5, 36, "Output",
 CellID->23297039]
}, Open  ]],
Cell[237628, 4824, 90, 1, 70, "ExampleText",
 CellID->325279064]
}, Open  ]],
Cell[CellGroupData[{
Cell[237755, 4830, 221, 7, 70, "ExampleSection",
 CellID->1293636265],
Cell[237979, 4839, 795, 16, 70, "ExampleText",
 CellID->246788676],
Cell[238777, 4857, 377, 7, 70, "ExampleText",
 CellID->121431475],
Cell[CellGroupData[{
Cell[239179, 4868, 737, 22, 70, "Input",
 CellID->27822125],
Cell[239919, 4892, 252, 8, 36, "Output",
 CellID->330622408]
}, Open  ]],
Cell[240186, 4903, 256, 5, 70, "ExampleText",
 CellID->363116584],
Cell[CellGroupData[{
Cell[240467, 4912, 166, 5, 70, "Input",
 CellID->131208994],
Cell[240636, 4919, 361, 13, 36, "Output",
 CellID->954714706]
}, Open  ]],
Cell[241012, 4935, 220, 5, 70, "ExampleText",
 CellID->128983040],
Cell[CellGroupData[{
Cell[241257, 4944, 356, 10, 70, "Input",
 CellID->689942839],
Cell[241616, 4956, 255, 9, 36, "Output",
 CellID->417167147]
}, Open  ]],
Cell[241886, 4968, 124, 3, 70, "ExampleText",
 CellID->257626546],
Cell[CellGroupData[{
Cell[242035, 4975, 110, 3, 70, "Input",
 CellID->101065266],
Cell[242148, 4980, 1705, 36, 122, "Output",
 CellID->528798639]
}, Open  ]],
Cell[243868, 5019, 281, 7, 70, "ExampleText",
 CellID->198004268]
}, Closed]],
Cell[CellGroupData[{
Cell[244186, 5031, 238, 7, 70, "ExampleSection",
 CellID->2123667759],
Cell[244427, 5040, 160, 4, 70, "ExampleText",
 CellID->342911820],
Cell[244590, 5046, 394, 16, 70, "ExampleText",
 CellID->315517505],
Cell[244987, 5064, 137, 4, 70, "ExampleText",
 CellID->218711756],
Cell[245127, 5070, 265, 8, 70, "ExampleText",
 CellID->26256628],
Cell[245395, 5080, 148, 4, 70, "ExampleText",
 CellID->12887027],
Cell[245546, 5086, 381, 13, 70, "Input",
 CellID->345799449],
Cell[245930, 5101, 331, 10, 70, "ExampleText",
 CellID->373792664],
Cell[CellGroupData[{
Cell[246286, 5115, 269, 8, 70, "Input",
 CellID->561478990],
Cell[246558, 5125, 161, 5, 36, "Output",
 CellID->109510433]
}, Open  ]]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[246780, 5137, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[247200, 5151, 815, 27, 70, "SeeAlso",
 CellID->745892494]
}, Open  ]],
Cell[CellGroupData[{
Cell[248052, 5183, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[248474, 5197, 210, 5, 70, "Tutorials",
 CellID->290096468],
Cell[248687, 5204, 159, 3, 70, "Tutorials",
 CellID->112343462]
}, Open  ]],
Cell[CellGroupData[{
Cell[248883, 5212, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[249305, 5226, 167, 3, 70, "MoreAbout",
 CellID->509755738]
}, Open  ]],
Cell[249487, 5232, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

