(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    282155,       6089]
NotebookOptionsPosition[    273677,       5815]
NotebookOutlinePosition[    275833,       5868]
CellTagsIndexPosition[    275746,       5863]
WindowTitle->ICDoubletLinear - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"2.1 Introduction To Influence Coefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"], "\
\<\"2.3 Isolated Panel\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IsolatedPanel"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"ICDoubletPoint\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICDoubletPoint"], "\<\"ICDoubletConstant\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICDoubletConstant"], \
"\<\"ICDoubletQuadratic\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICDoubletQuadratic"], \
"\<\"ICSourceLinear\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICSourceLinear"], "\<\"ICVortexLinear\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/ref/ICVortexLinear"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"InfluenceCoefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/InfluenceCoefficients"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/ICDoubletLinear\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/ICDoubletLinear"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
ICDoubletLinear.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/ICDoubletLinear", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["ICDoubletLinear", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICDoubletLinear", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        SubscriptBox[
         StyleBox["r", "TI"], 
         StyleBox["p", "TI"]]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes velocity influence coefficients at field \
points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     " due to connected doublet panels of linearly varying strength whose end \
points are located by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICPhiDoubletLinear", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes velocity-potential influence coefficients at \
field points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     " due to connected doublet panels of linearly varying strength whose end \
points are located by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICPsiDoubletLinear", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]",
     "computes stream-function influence coefficients at field points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     " due to connected doublet panels of linearly varying strength whose end \
points are located by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, 
   "Rows" -> {None, None, {None}}, "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->438187347],

Cell[TextData[{
 "Input arguments are collections of vectors of the data type ",
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], "InlineFormula"],
 ".  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation may be used for single vectors."
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 "The doublet density for the panel of linearly varying strength is written \
as ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"\[Mu]", 
    RowBox[{"(", 
     StyleBox["x", "TI"], ")"}]}], "=", 
   RowBox[{
    StyleBox["a", "TI"], "+", 
    RowBox[{
     StyleBox["b", "TI"], " ", 
     StyleBox["x", "TI"]}]}]}]], "InlineFormula"],
 ", where ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 " is the distance from the beginning of the panel, constants ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 " are the strength parameters for the panel.  For collections of panels, the \
strength parameters are written in two lists, one for the constant term and \
one for the linear term in the equation for doublet density.  Within each \
list strength parameters are ordered according to the clockwise numbering \
convention for panels. "
}], "Notes",
 CellID->390622451],

Cell[TextData[{
 "Results for velocity influence coefficients are two collections of vectors, \
one associated with the ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " parameters and the other, with the ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 " parameters.  Influence coefficients for velocity potential and stream \
function are two lists of scalar influence coefficients.  For the special \
case of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation being used to specify a single field point, output is simplified \
as illustrated in the examples."
}], "Notes",
 CellID->433990591],

Cell[TextData[{
 "The velocity, velocity potential and stream function for a panel are \
written as the sum of two terms, each term being the product of a geometric \
influence coefficient and the appropriate strength parameter.\t\t\t\t\t\t\t\t\
\t\t\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["V", "TI"], "=", 
   RowBox[{
    RowBox[{
     StyleBox["a", "TI"], " ", 
     StyleBox["ica", "TI"]}], "+", 
    RowBox[{
     StyleBox["b", "TI"], " ", 
     StyleBox["icb", "TI"]}]}]}]], "InlineFormula"],
 "\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Phi]", "TR"], "=", 
   RowBox[{
    RowBox[{
     StyleBox["a", "TI"], " ", 
     StyleBox["ic\[Phi]a", "TI"]}], "+", 
    RowBox[{
     StyleBox["b", "TI"], " ", 
     StyleBox["ic\[Phi]b", "TI"]}]}]}]], "InlineFormula"],
 "\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Psi]", "TR"], "=", 
   RowBox[{
    RowBox[{
     StyleBox["a", "TI"], " ", 
     StyleBox["ic\[Psi]a", "TI"]}], "+", " ", 
    RowBox[{
     StyleBox["b", "TI"], " ", 
     StyleBox["ic\[Psi]b", "TI"]}]}]}]], "InlineFormula"]
}], "Notes",
 CellID->213924258],

Cell["New in 1.0   |   Last modified in 1.2", "Notes",
 CellID->529540534]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->7993419],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->446874384],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->513831186],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->66474307],

Cell[TextData[{
 "This example demonstrates some properties of a single panel.  Algebraic \
equations for influence coefficients are simplest in a local coordinate \
system where the panel extends from the origin along the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis to ",
 Cell[BoxData[
  StyleBox["\[Delta]", "TR"]], "InlineFormula"],
 ", the length of the panel."
}], "ExampleText",
 CellID->659456463],

Cell[TextData[{
 "Specify an arbitrary field point and panel endpoints.  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation is used for a single vector."
}], "ExampleText",
 CellID->42111717],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"rF", ",", "rP"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"x", ",", "z"}], "}"}], ",", 
     RowBox[{"MakeCartesianVectors", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "\[Delta]"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}]}], 
  ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->64953589],

Cell[TextData[{
 "Compute the velocity influence coefficients.  There are separate influence \
coefficients for the constant and linear terms in the function describing \
doublet distribution.  Each influence coefficient is a single vector written \
in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation."
}], "ExampleText",
 CellID->326501090],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"icua", ",", "icwa"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"icub", ",", "icwb"}], "}"}]}], "}"}], "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"ICDoubletLinear", "[", 
      RowBox[{"rF", ",", "rP"}], "]"}], ",", 
     RowBox[{"\[Delta]", ">", "0"}]}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->560964283],

Cell[TextData[{
 "Display the algebraic equations for components of the velocity influence \
coefficient associated with strength parameter ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 "."
}], "ExampleText",
 CellID->120984796],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"icua", ",", "icwa"}], "}"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->108853583],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"z", " ", "\[Delta]", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "2"}], " ", "x"}], "+", "\[Delta]"}], ")"}]}], 
     RowBox[{"2", " ", "\[Pi]", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"], "-", 
        RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
        SuperscriptBox["\[Delta]", "2"]}], ")"}]}]]}], ",", 
   FractionBox[
    RowBox[{"\[Delta]", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["x", "2"]}], "+", 
       SuperscriptBox["z", "2"], "+", 
       RowBox[{"x", " ", "\[Delta]"}]}], ")"}]}], 
    RowBox[{"2", " ", "\[Pi]", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"], "-", 
       RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
       SuperscriptBox["\[Delta]", "2"]}], ")"}]}]]}], "}"}]], "Output",
 ImageSize->{436, 41},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->132083116]
}, Open  ]],

Cell[TextData[{
 "Display the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-component of the velocity influence coefficient associated with the linear \
term."
}], "ExampleText",
 CellID->217164756],

Cell[CellGroupData[{

Cell[BoxData["icub"], "Input",
 CellLabel->"In[5]:=",
 CellID->176773044],

Cell[BoxData[
 FractionBox[
  RowBox[{
   FractionBox[
    RowBox[{"z", " ", "\[Delta]"}], 
    RowBox[{
     SuperscriptBox["x", "2"], "+", 
     SuperscriptBox["z", "2"], "-", 
     RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
     SuperscriptBox["\[Delta]", "2"]}]], "+", 
   RowBox[{"ArcTan", "[", 
    RowBox[{"x", ",", "z"}], "]"}], "-", 
   RowBox[{"ArcTan", "[", 
    RowBox[{
     RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}]}], 
  RowBox[{"2", " ", "\[Pi]"}]]], "Output",
 ImageSize->{288, 42},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->612390350]
}, Open  ]],

Cell[TextData[{
 "Display the algebraic equations for components of the velocity influence \
coefficient associated with strength parameter ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 " replacing ArcTan function by named angles (See ",
 ButtonBox["2.1 Introduction to InfluenceCoefficients",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"],
 " for sketch defining angles).  Although the substitution of named angles is \
made to simplify the appearance of equations, note that the original \
equations are maintained for subsequent computations."
}], "ExampleText",
 CellID->364097714],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"icub", ",", "icwb"}], "}"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
     "\[Theta]2"}]}], "}"}]}]], "Input",
 CellLabel->"In[6]:=",
 CellID->25568170],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     FractionBox[
      RowBox[{"z", " ", "\[Delta]"}], 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"], "-", 
       RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
       SuperscriptBox["\[Delta]", "2"]}]], "+", "\[Theta]1", "-", 
     "\[Theta]2"}], 
    RowBox[{"2", " ", "\[Pi]"}]], ",", 
   FractionBox[
    RowBox[{
     FractionBox[
      RowBox[{"\[Delta]", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "x"}], "+", "\[Delta]"}], ")"}]}], 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"], "-", 
       RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
       SuperscriptBox["\[Delta]", "2"]}]], "-", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"Log", "[", 
       FractionBox[
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"], "-", 
         RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
         SuperscriptBox["\[Delta]", "2"]}], 
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"]}]], "]"}]}]}], 
    RowBox[{"2", " ", "\[Pi]"}]]}], "}"}]], "Output",
 ImageSize->{366, 46},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->65959356]
}, Open  ]],

Cell["\<\
Compute the velocity-potential influence coefficient and display in terms of \
named angles.\
\>", "ExampleText",
 CellID->44302699],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Phi]a", ",", "ic\[Phi]b"}], "}"}], "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"ICPhiDoubletLinear", "[", 
      RowBox[{"rF", ",", "rP"}], "]"}], ",", 
     RowBox[{"\[Delta]", ">", "0"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"ic\[Phi]a", ",", "ic\[Phi]b"}], "}"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
     "\[Theta]2"}]}], "}"}]}]}], "Input",
 CellLabel->"In[7]:=",
 CellID->575890672],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"\[Theta]1", "-", "\[Theta]2"}], 
    RowBox[{"2", " ", "\[Pi]"}]], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{
       RowBox[{"-", "2"}], " ", "x", " ", "\[Theta]1"}], "+", 
      RowBox[{"2", " ", "x", " ", "\[Theta]2"}], "+", 
      RowBox[{"z", " ", 
       RowBox[{"Log", "[", 
        FractionBox[
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["z", "2"], "-", 
          RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
          SuperscriptBox["\[Delta]", "2"]}], 
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["z", "2"]}]], "]"}]}]}], 
     RowBox[{"4", " ", "\[Pi]"}]]}]}], "}"}]], "Output",
 ImageSize->{305, 46},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->79019023]
}, Open  ]],

Cell["Compute stream-function influence coefficients.", "ExampleText",
 CellID->22197235],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Psi]a", ",", "ic\[Psi]b"}], "}"}], "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"ICPsiDoubletLinear", "[", 
      RowBox[{"rF", ",", "rP"}], "]"}], ",", 
     RowBox[{"\[Delta]", ">", "0"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"ic\[Psi]a", ",", "ic\[Psi]b"}], "}"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
     "\[Theta]2"}]}], "}"}]}]}], "Input",
 CellLabel->"In[8]:=",
 CellID->8664355],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"Log", "[", 
     FractionBox[
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"], "-", 
       RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
       SuperscriptBox["\[Delta]", "2"]}], 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"]}]], "]"}], 
    RowBox[{"4", " ", "\[Pi]"}]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"2", " ", "\[Delta]"}], "+", 
     RowBox[{"2", " ", "z", " ", "\[Theta]1"}], "-", 
     RowBox[{"2", " ", "z", " ", "\[Theta]2"}], "+", 
     RowBox[{"x", " ", 
      RowBox[{"Log", "[", 
       FractionBox[
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"], "-", 
         RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
         SuperscriptBox["\[Delta]", "2"]}], 
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"]}]], "]"}]}]}], 
    RowBox[{"4", " ", "\[Pi]"}]]}], "}"}]], "Output",
 ImageSize->{384, 46},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->30632950]
}, Open  ]],

Cell["\<\
Write the velocity components, velocity potential and stream function in \
terms of geometric influence coefficients and strength parameters.\
\>", "ExampleText",
 CellID->9427774],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"u", ",", "w"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"a", " ", "icua"}], "+", 
      RowBox[{"b", " ", "icub"}]}], ",", 
     RowBox[{
      RowBox[{"a", " ", "icwa"}], "+", 
      RowBox[{"b", " ", "icwb"}]}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Phi]", ",", "\[Psi]"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"a", " ", "ic\[Phi]a"}], "+", 
      RowBox[{"b", " ", "ic\[Phi]b"}]}], ",", 
     RowBox[{
      RowBox[{"a", " ", "ic\[Psi]a"}], "+", 
      RowBox[{"b", " ", "ic\[Psi]b"}]}]}], "}"}]}], ";"}]}], "Input",
 CellLabel->"In[9]:=",
 CellID->258291294],

Cell[TextData[{
 "Verify that the relations between velocity, velocity potential and stream \
function are satisfied.\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["u", "TI"], "=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Phi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["x", "TI"]}]], "=", 
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Psi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["z", "TI"]}]]}]}]], "InlineFormula"],
 "\t\t\t",
 Cell[BoxData[
  RowBox[{"w", "=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Phi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["z", "TI"]}]], "=", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{"\[PartialD]", 
       StyleBox["\[Psi]", "TR"]}], 
      RowBox[{"\[PartialD]", 
       StyleBox["x", "TI"]}]]}]}]}]], "InlineFormula"],
 "\t\t\t\t"
}], "ExampleText",
 CellID->78226744],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"u", "\[Equal]", 
     RowBox[{"D", "[", 
      RowBox[{"\[Phi]", ",", "x"}], "]"}]}], ",", 
    RowBox[{"u", "\[Equal]", 
     RowBox[{"D", "[", 
      RowBox[{"\[Psi]", ",", "z"}], "]"}]}]}], "}"}], "//", 
  "Simplify"}]], "Input",
 CellLabel->"In[10]:=",
 CellID->13857323],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[10]=",
 CellID->4037215]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"w", "\[Equal]", 
     RowBox[{"D", "[", 
      RowBox[{"\[Phi]", ",", "z"}], "]"}]}], ",", 
    RowBox[{"w", "\[Equal]", 
     RowBox[{"-", 
      RowBox[{"D", "[", 
       RowBox[{"\[Psi]", ",", "x"}], "]"}]}]}]}], "}"}], "//", 
  "Simplify"}]], "Input",
 CellLabel->"In[11]:=",
 CellID->499128776],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->266724237]
}, Open  ]],

Cell[TextData[{
 "Flow visualization provides a plausibility check on the computed flow \
field.  Since the contribution of the ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " parameter is illustrated in ICDoubletConstant, set ",
 Cell[BoxData[
  RowBox[{
   StyleBox["a", "TI"], "=", "0"}]], "InlineFormula"],
 ".  Set ",
 Cell[BoxData[
  RowBox[{
   StyleBox["b", "TI"], "=", "2"}]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Delta]", "TR"], "=", "1"}]], "InlineFormula"],
 " to provide unit integrated panel strength,  ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", 
     StyleBox["\[Delta]", "TR"]], 
    RowBox[{"\[Mu]", 
     RowBox[{"(", 
      StyleBox["x", "TI"], ")"}], 
     RowBox[{"\[DifferentialD]", 
      StyleBox["x", "TI"]}]}]}], "=", "1"}]], "InlineFormula"],
 ".  Circular regions near the edges of the panel are excluded from the \
vector plot to avoid showing vectors of very large magnitude.  Also, \
nonlinear scaling is used for magnitudes of vectors to enhance visibility in \
the far field."
}], "ExampleText",
 CellID->78531558],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xMin", ",", "xMax", ",", "zMin", ",", "zMax"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "2", ",", 
     RowBox[{"-", "1.2"}], ",", "1.2"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"uPlot", ",", "wPlot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"u", ",", "w"}], "}"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", "\[Rule]", "0"}], ",", 
      RowBox[{"b", "\[Rule]", "2"}], ",", 
      RowBox[{"\[Delta]", "\[Rule]", "1"}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Phi]Plot", ",", "\[Psi]Plot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\[Phi]", ",", "\[Psi]"}], "}"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", "\[Rule]", "0"}], ",", 
      RowBox[{"b", "\[Rule]", "2"}], ",", 
      RowBox[{"\[Delta]", "\[Rule]", "1"}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gV", "=", 
   RowBox[{"VectorPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"uPlot", ",", "wPlot"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"RegionFunction", "\[Rule]", 
      RowBox[{"Function", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"x", ",", "z"}], "}"}], ",", 
        RowBox[{
         RowBox[{"0.01", "<", 
          RowBox[{
           RowBox[{"x", "^", "2"}], "+", 
           RowBox[{"z", "^", "2"}]}]}], " ", "&&", " ", 
         RowBox[{"0.01", "<", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"x", "-", "1"}], ")"}], "^", "2"}], "+", 
           RowBox[{"z", "^", "2"}]}]}]}]}], "]"}]}], ",", 
     RowBox[{"VectorColorFunction", "->", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"RGBColor", "[", 
         RowBox[{"0", ",", "1", ",", "0"}], "]"}], "&"}], ")"}]}], ",", 
     RowBox[{"VectorPoints", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"20", ",", "20"}], "}"}]}], ",", 
     RowBox[{"VectorScale", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Large", ",", "Automatic", ",", 
        RowBox[{
         RowBox[{"Sqrt", "[", "#5", "]"}], "&"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Phi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Phi]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "13"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Red"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Psi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "7"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Blue"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gPanel", "=", 
   RowBox[{"Graphics", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"GrayLevel", "[", "0", "]"}], ",", 
      RowBox[{"AbsoluteThickness", "[", "3", "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"Transpose", "[", 
        RowBox[{"Components", "[", 
         RowBox[{"rP", "/.", 
          RowBox[{"\[Delta]", "\[Rule]", "1.0"}]}], "]"}], "]"}], "]"}]}], 
     "}"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gV", ",", "g\[Phi]", ",", "g\[Psi]", ",", "gPanel", ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
   RowBox[{"FrameTicks", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.0"}], ",", "2.0", ",", "0.25"}], "]"}], "]"}], ",", 
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.2"}], ",", "1.2", ",", "0.2"}], "]"}], "]"}], ",", 
      "None", ",", "None"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", " ", "72"}]}], ",", 
   RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellLabel->"In[12]:=",
 CellID->1746043248],

Cell[BoxData[
 GraphicsBox[{{{}, {
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.006209537189352544, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0201883631071063`, -1.194166631539747}, \
{-0.9798116368928937, -1.205833368460253}}]}}, 
      {Arrowheads[{{0.0064499081621183655`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.020248849441319, -1.0655335965219181`}, \
{-0.979751150558681, -1.0818348245307134`}}]}}, 
      {Arrowheads[{{0.0066916174427485925`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0199575117881956`, -0.9366667903785253}, \
{-0.9800424882118044, -0.958070051726738}}]}}, 
      {Arrowheads[{{0.006930194439394226, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0192194274468718`, -0.8076116333642532}, \
{-0.9807805725531282, -0.8344936297936418}}]}}, 
      {Arrowheads[{{0.007159746761162853, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.017938658130796, -0.6784489994625541}, \
{-0.982061341869204, -0.7110246847479725}}]}}, 
      {Arrowheads[{{0.0073728588721011295`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0160323335781904`, -0.5493023839403731}, \
{-0.9839676664218097, -0.5875397213227852}}]}}, 
      {Arrowheads[{{0.0075607250558988185`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0134514083451296`, -0.42033947298451807`}, \
{-0.9865485916548704, -0.4638710533312719}}]}}, 
      {Arrowheads[{{0.007713682836018593, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.010205831764648, -0.2917626457670297}, \
{-0.9897941682353519, -0.3398163016013917}}]}}, 
      {Arrowheads[{{0.007822273137870617, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0063870018742078`, -0.16378370492692715`}, \
{-0.9936129981257923, -0.21516366349412588`}}]}}, 
      {Arrowheads[{{0.007878765547764658, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0021756850033943`, -0.03658359004174511}, \
{-0.9978243149966056, -0.0897321994319395}}]}}, 
      {Arrowheads[{{0.007878765547764658, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9978243149966057, 
         0.08973219943193911}, {-1.0021756850033943`, 
         0.03658359004174472}}]}}, 
      {Arrowheads[{{0.007822273137870626, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9936129981257922, 
         0.21516366349412552`}, {-1.0063870018742078`, 
         0.16378370492692676`}}]}}, 
      {Arrowheads[{{0.007713682836018581, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.989794168235352, 
         0.33981630160139137`}, {-1.0102058317646478`, 
         0.2917626457670293}}]}}, 
      {Arrowheads[{{0.0075607250558988185`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9865485916548704, 
         0.46387105333127143`}, {-1.0134514083451296`, 
         0.4203394729845176}}]}}, 
      {Arrowheads[{{0.00737285887210114, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9839676664218096, 
         0.5875397213227848}, {-1.0160323335781904`, 0.5493023839403727}}]}}, 
      {Arrowheads[{{0.0071597467611628765`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9820613418692039, 
         0.7110246847479721}, {-1.017938658130796, 0.6784489994625535}}]}}, 
      {Arrowheads[{{0.0069301944393942655`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9807805725531281, 
         0.8344936297936413}, {-1.019219427446872, 0.8076116333642527}}]}}, 
      {Arrowheads[{{0.006691617442748607, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9800424882118043, 
         0.9580700517267375}, {-1.0199575117881956`, 0.9366667903785248}}]}}, 
      {Arrowheads[{{0.006449908162118335, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.979751150558681, 
         1.081834824530713}, {-1.0202488494413189`, 1.0655335965219177`}}]}}, 
      {Arrowheads[{{0.006209537189352544, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9798116368928937, 
         1.2058333684602525`}, {-1.0201883631071063`, 
         1.1941666315397466`}}]}}, 
      {Arrowheads[{{0.0066027572394272305`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.864093089396974, -1.1960209169744063`}, \
{-0.8201174369188154, -1.2039790830255936`}}]}}, 
      {Arrowheads[{{0.0068947408463120585`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8645147160975233, -1.0671842287364073`}, \
{-0.8196958102182661, -1.0801841923162243`}}]}}, 
      {Arrowheads[{{0.007193912362684988, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8645756941299532, -0.93799903272502}, \
{-0.8196348321858362, -0.9567378093802431}}]}}, 
      {Arrowheads[{{0.007495420300340212, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8641432321172526, -0.8084924890810741}, \
{-0.8200672941985367, -0.8336127740768209}}]}}, 
      {Arrowheads[{{0.007792219511738833, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8630679706230078, -0.6787381146620035}, \
{-0.8211425556927816, -0.710735569548523}}]}}, 
      {Arrowheads[{{0.008074577814281383, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8611998958587146, -0.5488737001347803}, \
{-0.8230106304570748, -0.5879684051283781}}]}}, 
      {Arrowheads[{{0.00832981660693647, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8584187013623118, -0.4191155108059475}, \
{-0.8257918249534777, -0.46509501550984245`}}]}}, 
      {Arrowheads[{{0.008542693274016078, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8546800772810874, -0.28975744515016766`}, \
{-0.829530449034702, -0.3418215022182538}}]}}, 
      {Arrowheads[{{0.008696956637017384, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8500691862865702, -0.16113949469584177`}, \
{-0.8341413400292192, -0.21780787372521127`}}]}}, 
      {Arrowheads[{{0.008778342423963517, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8448356661741304, -0.033576079364867534`}, \
{-0.8393748601416589, -0.09273971010881707}}]}}, 
      {Arrowheads[{{0.008778342423963519, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.839374860141659, 
         0.0927397101088167}, {-0.8448356661741304, 0.03357607936486713}}]}}, 
      {Arrowheads[{{0.008696956637017384, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8341413400292191, 
         0.21780787372521088`}, {-0.8500691862865701, 
         0.16113949469584138`}}]}}, 
      {Arrowheads[{{0.008542693274016078, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.829530449034702, 
         0.3418215022182534}, {-0.8546800772810874, 0.28975744515016727`}}]}}, 
      {Arrowheads[{{0.008329816606936475, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8257918249534776, 
         0.46509501550984206`}, {-0.8584187013623117, 
         0.41911551080594706`}}]}}, 
      {Arrowheads[{{0.008074577814281371, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8230106304570748, 
         0.5879684051283776}, {-0.8611998958587146, 0.5488737001347799}}]}}, 
      {Arrowheads[{{0.00779221951173883, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8211425556927816, 
         0.7107355695485227}, {-0.8630679706230077, 0.6787381146620031}}]}}, 
      {Arrowheads[{{0.007495420300340203, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8200672941985367, 
         0.8336127740768204}, {-0.8641432321172526, 0.8084924890810737}}]}}, 
      {Arrowheads[{{0.007193912362684988, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8196348321858362, 
         0.9567378093802428}, {-0.8645756941299532, 0.9379990327250197}}]}}, 
      {Arrowheads[{{0.0068947408463120585`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8196958102182661, 
         1.0801841923162239`}, {-0.8645147160975233, 1.0671842287364068`}}]}}, 
      {Arrowheads[{{0.0066027572394272305`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8201174369188153, 
         1.2039790830255932`}, {-0.8640930893969739, 1.1960209169744058`}}]}}, 
      {Arrowheads[{{0.007024097678072949, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7079338513024956, -1.1984974053348667`}, \
{-0.6604872013290831, -1.2015025946651332`}}]}}, 
      {Arrowheads[{{0.007379059095269465, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7088345940272469, -1.0695294509186741`}, \
{-0.6595864586043318, -1.0778389701339575`}}]}}, 
      {Arrowheads[{{0.007750612448889058, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7094049897180137, -0.9400727362166478}, \
{-0.6590160629135651, -0.9546641058886156}}]}}, 
      {Arrowheads[{{0.008134378312579496, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7094722548956349, -0.8101141570651982}, \
{-0.6589487977359438, -0.8319911060926968}}]}}, 
      {Arrowheads[{{0.008522934866888482, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7088192655938985, -0.6796922194455092}, \
{-0.6596017870376802, -0.7097814647650174}}]}}, 
      {Arrowheads[{{0.0089045409396684, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.707194332093414, -0.5489315441003723}, \
{-0.6612267205381649, -0.5879105611627861}}]}}, 
      {Arrowheads[{{0.009261735147815166, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7043453457967027, -0.41808442282671426`}, \
{-0.6640757068348759, -0.46612610348907557`}}]}}, 
      {Arrowheads[{{0.00957051628883467, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7000941930713115, -0.28756326325251064`}, \
{-0.6683268595602674, -0.3440156841159109}}]}}, 
      {Arrowheads[{{0.009801712459242586, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6944557955668686, -0.157924697289401}, \
{-0.6739652570647102, -0.22102267113165208`}}]}}, 
      {Arrowheads[{{0.009926596242413783, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6877593024447263, -0.029752402546118266`}, \
{-0.6806617501868526, -0.09656338692756633}}]}}, 
      {Arrowheads[{{0.009926596242413783, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6806617501868525, 
         0.09656338692756594}, {-0.6877593024447262, 
         0.029752402546117877`}}]}}, 
      {Arrowheads[{{0.009801712459242586, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6739652570647101, 
         0.22102267113165167`}, {-0.6944557955668685, 0.1579246972894006}}]}}, 
      {Arrowheads[{{0.00957051628883467, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6683268595602674, 
         0.3440156841159105}, {-0.7000941930713114, 0.2875632632525102}}]}}, 
      {Arrowheads[{{0.009261735147815163, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.664075706834876, 
         0.46612610348907524`}, {-0.7043453457967027, 0.4180844228267139}}]}}, 
      {Arrowheads[{{0.0089045409396684, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6612267205381648, 
         0.5879105611627856}, {-0.7071943320934139, 0.5489315441003718}}]}}, 
      {Arrowheads[{{0.008522934866888482, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6596017870376802, 
         0.709781464765017}, {-0.7088192655938985, 0.6796922194455088}}]}}, 
      {Arrowheads[{{0.00813437831257948, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6589487977359438, 
         0.8319911060926963}, {-0.7094722548956348, 0.8101141570651977}}]}}, 
      {Arrowheads[{{0.007750612448889075, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6590160629135651, 
         0.954664105888615}, {-0.7094049897180138, 0.9400727362166472}}]}}, 
      {Arrowheads[{{0.007379059095269465, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6595864586043318, 
         1.077838970133957}, {-0.7088345940272469, 1.0695294509186737`}}]}}, 
      {Arrowheads[{{0.007024097678072949, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6604872013290831, 
         1.2015025946651328`}, {-0.7079338513024956, 1.1984974053348663`}}]}}, 
      {Arrowheads[{{0.007468815278331544, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5515339711715602, -1.2017067409855184`}, \
{-0.5010976077758079, -1.1982932590144815`}}]}}, 
      {Arrowheads[{{0.007899082321228287, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5530312670697292, -1.0727451898643374`}, \
{-0.499600311877639, -1.074623231188294}}]}}, 
      {Arrowheads[{{0.00836014723131202, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5542909775799272, -0.9431442768906783}, \
{-0.4983406013674409, -0.951592565214585}}]}}, 
      {Arrowheads[{{0.008849855348372742, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5551122109821712, -0.812822029045839}, \
{-0.4975193679651969, -0.829283234112056}}]}}, 
      {Arrowheads[{{0.009362619905572394, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5552106555171177, -0.6817361835940701}, \
{-0.4974209234302505, -0.7077375006164566}}]}}, 
      {Arrowheads[{{0.009887056295300874, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5542048249064202, -0.5499345709176462}, \
{-0.49842675404094794`, -0.5869075343455122}}]}}, 
      {Arrowheads[{{0.010402240394502755`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5516266755519976, -0.41763856111100933`}, \
{-0.5010049033953705, -0.46657196520478056`}}]}}, 
      {Arrowheads[{{0.010872720322720542`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5470015358216896, -0.2853592954900919}, \
{-0.5056300431256785, -0.34621965187832954`}}]}}, 
      {Arrowheads[{{0.01124515072872773, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.540070419937007, -0.1539906418888946}, \
{-0.5125611590103611, -0.22495672653215842`}}]}}, 
      {Arrowheads[{{0.011455391140315865`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5311676591929493, -0.024695517577483853`}, \
{-0.521463919754419, -0.10162027189620075`}}]}}, 
      {Arrowheads[{{0.011455391140315869`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5214639197544189, 
         0.10162027189620038`}, {-0.5311676591929492, 
         0.02469551757748345}}]}}, 
      {Arrowheads[{{0.011245150728727732`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5125611590103611, 
         0.22495672653215806`}, {-0.5400704199370069, 0.1539906418888942}}]}}, 
      {Arrowheads[{{0.010872720322720542`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5056300431256785, 
         0.34621965187832915`}, {-0.5470015358216896, 
         0.28535929549009154`}}]}}, 
      {Arrowheads[{{0.010402240394502747`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5010049033953705, 
         0.46657196520478017`}, {-0.5516266755519975, 0.4176385611110089}}]}}, 
      {Arrowheads[{{0.009887056295300893, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.4984267540409479, 
         0.5869075343455117}, {-0.5542048249064203, 0.5499345709176456}}]}}, 
      {Arrowheads[{{0.009362619905572403, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.49742092343025046`, 
         0.7077375006164561}, {-0.5552106555171177, 0.6817361835940696}}]}}, 
      {Arrowheads[{{0.008849855348372767, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.49751936796519686`, 
         0.8292832341120555}, {-0.5551122109821713, 0.8128220290458386}}]}}, 
      {Arrowheads[{{0.008360147231312011, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.49834060136744096`, 
         0.9515925652145846}, {-0.5542909775799272, 0.9431442768906778}}]}}, 
      {Arrowheads[{{0.007899082321228261, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.49960031187763904`, 
         1.0746232311882937`}, {-0.553031267069729, 1.0727451898643372`}}]}}, 
      {Arrowheads[{{0.00746881527833156, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5010976077758079, 
         1.198293259014481}, {-0.5515339711715603, 1.201706740985518}}]}}, 
      {Arrowheads[{{0.007928197145176013, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.39463544359654174`, -1.2057167929570232`}, \
{-0.34220666166661595`, -1.194283207042977}}]}}, 
      {Arrowheads[{{0.008445765940047189, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.396812410410258, -1.0769803718192845`}, \
{-0.34002969485289963`, -1.0703880492333468`}}]}}, 
      {Arrowheads[{{0.009014072699253745, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.39892616536005676`, -0.9474786109181}, \
{-0.3379159399031008, -0.9472582311871632}}]}}, 
      {Arrowheads[{{0.009635996596874053, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.4007828350251315, -0.8170366178904284}, \
{-0.33605927023802606`, -0.8250686452674665}}]}}, 
      {Arrowheads[{{0.010312149381297556`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.4020687431827613, -0.6854783036834208}, \
{-0.33477336208039626`, -0.7039953805271058}}]}}, 
      {Arrowheads[{{0.011038144509955107`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.40229400019799877`, -0.5526720632261605}, \
{-0.33454810506515886`, -0.5841700420369977}}]}}, 
      {Arrowheads[{{0.011798567650538333`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.4007242568154232, -0.4186362778335959}, \
{-0.3361178484477344, -0.465574248482194}}]}}, 
      {Arrowheads[{{0.012553734045741123`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3963446264710129, -0.2837710358515521}, \
{-0.3404974787921448, -0.34780791151686935`}}]}}, 
      {Arrowheads[{{0.013215050336264463`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3880566107657218, -0.14929258535332202`}, \
{-0.3487854944974359, -0.22965478306773102`}}]}}, 
      {Arrowheads[{{0.01362554032261534, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.37561829828504145`, -0.017611662599969415`}, \
{-0.3612238069781162, -0.1087041268737152}}]}}, 
      {Arrowheads[{{0.013625540322615336`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.36122380697811624`, 
         0.10870412687371481`}, {-0.37561829828504134`, 
         0.017611662599969012`}}]}}, 
      {Arrowheads[{{0.013215050336264465`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.34878549449743596`, 
         0.22965478306773066`}, {-0.38805661076572173`, 
         0.1492925853533216}}]}}, 
      {Arrowheads[{{0.01255373404574113, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3404974787921448, 
         0.347807911516869}, {-0.3963446264710128, 0.28377103585155167`}}]}}, 
      {Arrowheads[{{0.011798567650538336`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.33611784844773446`, 
         0.4655742484821936}, {-0.40072425681542323`, 
         0.41863627783359547`}}]}}, 
      {Arrowheads[{{0.011038144509955121`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.33454810506515886`, 
         0.5841700420369974}, {-0.4022940001979988, 0.55267206322616}}]}}, 
      {Arrowheads[{{0.010312149381297556`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3347733620803963, 
         0.7039953805271054}, {-0.40206874318276137`, 0.6854783036834204}}]}}, 
      {Arrowheads[{{0.009635996596874046, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3360592702380261, 
         0.8250686452674661}, {-0.4007828350251315, 0.8170366178904279}}]}}, 
      {Arrowheads[{{0.009014072699253745, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3379159399031008, 
         0.9472582311871628}, {-0.39892616536005676`, 0.9474786109180996}}]}}, 
      {Arrowheads[{{0.008445765940047194, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.34002969485289963`, 
         1.0703880492333466`}, {-0.39681241041025805`, 1.076980371819284}}]}}, 
      {Arrowheads[{{0.007928197145176013, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3422066616666159, 
         1.1942832070429763`}, {-0.3946354435965417, 1.2057167929570225`}}]}}, 
      {Arrowheads[{{0.008388862764077245, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.23690218026397636`, -1.210501265729053}, \
{-0.18415045131497074`, -1.189498734270947}}]}}, 
      {Arrowheads[{{0.0090033930764029, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.2397560695999547, -1.0822860595285548`}, \
{-0.1812965619789924, -1.0650823615240768`}}]}}, 
      {Arrowheads[{{0.009694279881611024, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.24280053459830675`, -0.9532583388931946}, \
{-0.17825209698064037`, -0.9414785032120688}}]}}, 
      {Arrowheads[{{0.010472759280188716`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.2459058426953086, -0.8231523662535374}, \
{-0.17514678888363852`, -0.8189528969043575}}]}}, 
      {Arrowheads[{{0.011351564830560894`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.24881685062872205`, -0.6916367742934044}, \
{-0.17223578095022507`, -0.6978369099171222}}]}}, 
      {Arrowheads[{{0.012344721147342341`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.2510633112891845, -0.558318714884417}, \
{-0.1699893202897626, -0.5785233903787415}}]}}, 
      {Arrowheads[{{0.013465546766194575`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.2517960508638141, -0.42278098074136805`}, \
{-0.169256580715133, -0.46142954557442184`}}]}}, 
      {Arrowheads[{{0.014715631329738096`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.24946205475539443`, -0.28473961793326513`}, \
{-0.1715905768235527, -0.34683932943515633`}}]}}, 
      {Arrowheads[{{0.016030876489407486`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.2412040615891119, -0.14472884988527446`}, \
{-0.17984856998983523`, -0.23421851853577863`}}]}}, 
      {Arrowheads[{{0.01707166962033203, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.22305832528701458`, -0.0067597331209871825`}, \
{-0.19799430629193254`, -0.11955605635269742`}}]}}, 
      {Arrowheads[{{0.017071669620332037`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.19799430629193265`, 
         0.11955605635269707`}, {-0.22305832528701447`, 
         0.006759733120986752}}]}}, 
      {Arrowheads[{{0.016030876489407486`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.17984856998983523`, 
         0.23421851853577821`}, {-0.2412040615891119, 
         0.14472884988527404`}}]}}, 
      {Arrowheads[{{0.014715631329738096`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.17159057682355272`, 
         0.346839329435156}, {-0.2494620547553944, 0.28473961793326474`}}]}}, 
      {Arrowheads[{{0.013465546766194575`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.169256580715133, 
         0.46142954557442145`}, {-0.2517960508638141, 
         0.42278098074136766`}}]}}, 
      {Arrowheads[{{0.012344721147342351`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.16998932028976257`, 
         0.5785233903787409}, {-0.2510633112891846, 0.5583187148844164}}]}}, 
      {Arrowheads[{{0.011351564830560894`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.17223578095022507`, 
         0.6978369099171218}, {-0.24881685062872205`, 0.691636774293404}}]}}, 
      {Arrowheads[{{0.010472759280188721`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.1751467888836385, 
         0.8189528969043571}, {-0.24590584269530863`, 0.8231523662535369}}]}}, 
      {Arrowheads[{{0.009694279881611026, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.17825209698064037`, 
         0.9414785032120683}, {-0.24280053459830675`, 0.9532583388931941}}]}}, 
      {Arrowheads[{{0.0090033930764029, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.1812965619789924, 
         1.0650823615240763`}, {-0.2397560695999547, 1.0822860595285544`}}]}}, 
      {Arrowheads[{{0.00838886276407724, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.18415045131497076`, 
         1.1894987342709464`}, {-0.23690218026397636`, 
         1.2105012657290524`}}]}}, 
      {Arrowheads[{{0.008832803557625236, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.0779558418417565, -1.2158809691644397`}, \
{-0.02730731605298012, -1.1841190308355605`}}]}}, 
      {Arrowheads[{{0.00954930503117039, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.08134130378358614, -1.0885205446341533`}, \
{-0.023921854111150467`, -1.058847876418478}}]}}, 
      {Arrowheads[{{0.010371953510029005`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.08520951575496176, -0.9604348898517724}, \
{-0.020053642139774855`, -0.9343019522534909}}]}}, 
      {Arrowheads[{{0.011322784025458048`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.0895508490577128, -0.8313127039588478}, \
{-0.015712308837023826`, -0.8107925591990472}}]}}, 
      {Arrowheads[{{0.012430944159108715`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.09427208082631874, -0.7007234536617023}, \
{-0.010991077068417877`, -0.6887502305488243}}]}}, 
      {Arrowheads[{{0.013737892203528586`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.09912190894916359, -0.5680770845532911}, \
{-0.0061412489455730315`, -0.5687650207098672}}]}}, 
      {Arrowheads[{{0.015309714700302476`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.10355527761389617`, -0.432557966394598}, \
{-0.001707880280840443, -0.45165255992119185`}}]}}, 
      {Arrowheads[{{0.017269733227738348`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.10643313765300214`, -0.29296196582782535`}, {
         0.0011699797582655214`, -0.3386169815405961}}]}}, 
      {Arrowheads[{{0.019898435879758265`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.10498769017238166`, -0.1471237348706093}, \
{-0.00027546772235494815`, -0.23182363355044372`}}]}}}, 
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.019898435879758276`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.0002754677223549759, 
         0.23182363355044341`}, {-0.10498769017238163`, 
         0.14712373487060884`}}]}}, 
      {Arrowheads[{{0.017269733227738348`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.0011699797582655075`, 
         0.3386169815405958}, {-0.10643313765300212`, 
         0.29296196582782497`}}]}}, 
      {Arrowheads[{{0.01530971470030247, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.0017078802808404708`, 
         0.4516525599211915}, {-0.10355527761389614`, 
         0.43255796639459754`}}]}}, 
      {Arrowheads[{{0.013737892203528586`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.006141248945573018, 
         0.5687650207098668}, {-0.09912190894916359, 0.5680770845532905}}]}}, 
      {Arrowheads[{{0.012430944159108719`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.010991077068417836`, 
         0.6887502305488239}, {-0.09427208082631877, 0.7007234536617017}}]}}, 
      {Arrowheads[{{0.011322784025458055`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.0157123088370238, 
         0.8107925591990468}, {-0.08955084905771282, 0.8313127039588474}}]}}, 
      {Arrowheads[{{0.010371953510029001`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.020053642139774862`, 
         0.9343019522534904}, {-0.08520951575496175, 0.960434889851772}}]}}, 
      {Arrowheads[{{0.009549305031170397, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.02392185411115045, 
         1.0588478764184779`}, {-0.08134130378358617, 1.088520544634153}}]}}, 
      {Arrowheads[{{0.008832803557625243, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.027307316052980087`, 
         1.1841190308355598`}, {-0.07795584184175652, 1.215880969164439}}]}}, 
      {Arrowheads[{{0.009238393871238818, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.0825469328777362, -1.2214811398614729`}, {
         0.1279793829117377, -1.1785188601385272`}}]}}, 
      {Arrowheads[{{0.010055342125620223`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.07895265314386844, -1.095265249698555}, {
         0.13157366264560544`, -1.0521031713540765`}}]}}, 
      {Arrowheads[{{0.011009848557020455`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.07462491724069481, -0.9685712617615788}, {
         0.13590139854877908`, -0.9261655803436846}}]}}, 
      {Arrowheads[{{0.012135253444882704`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.06943191147942859, -0.8411200216057295}, {
         0.1410944043100453, -0.8009852415521653}}]}}, 
      {Arrowheads[{{0.013476587038247518`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.0632574910292589, -0.7125005542974477}, {
         0.147268824760215, -0.6769731299130789}}]}}, 
      {Arrowheads[{{0.015098501521674083`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.056040045182861445`, -0.5821288125944961}, {
         0.15448627060661246`, -0.5547132926686622}}]}}, 
      {Arrowheads[{{0.017104124482305572`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.04781850087423345, -0.44921991718798715`}, {
         0.16270781491524045`, -0.4349906091278028}}]}}, 
      {Arrowheads[{{0.019686323668208962`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.038707100588728754`, -0.3128213016949107}, {
         0.17181921520074514`, -0.3187576456735108}}]}}, 
      {Arrowheads[{{0.02328420956258649, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.028391289883791165`, -0.17215718393817425`}, {
         0.18213502590568273`, -0.20679018448287878`}}]}}, 
      {Arrowheads[{{0.02895654985869146, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.012988670638769947`, -0.03016819919269049}, {
         0.19753764515070393`, -0.09614759028099412}}]}}, 
      {Arrowheads[{{0.02895654985869149, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.19753764515070404`, 0.09614759028099376}, {
         0.012988670638769864`, 0.030168199192690065`}}]}}, 
      {Arrowheads[{{0.023284209562586505`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.18213502590568276`, 0.20679018448287848`}, {
         0.028391289883791124`, 0.17215718393817378`}}]}}, 
      {Arrowheads[{{0.01968632366820897, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.17181921520074517`, 0.31875764567351045`}, {
         0.03870710058872871, 0.3128213016949103}}]}}, 
      {Arrowheads[{{0.017104124482305576`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.16270781491524045`, 0.4349906091278024}, {
         0.047818500874233436`, 0.4492199171879867}}]}}, 
      {Arrowheads[{{0.015098501521674081`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.15448627060661246`, 0.5547132926686618}, {
         0.056040045182861424`, 0.5821288125944956}}]}}, 
      {Arrowheads[{{0.013476587038247525`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.14726882476021502`, 0.6769731299130785}, {
         0.06325749102925887, 0.7125005542974473}}]}}, 
      {Arrowheads[{{0.012135253444882711`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.14109440431004533`, 0.800985241552165}, {
         0.06943191147942857, 0.8411200216057292}}]}}, 
      {Arrowheads[{{0.011009848557020455`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.13590139854877908`, 0.926165580343684}, {
         0.07462491724069481, 0.9685712617615783}}]}}, 
      {Arrowheads[{{0.010055342125620228`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.13157366264560547`, 1.052103171354076}, {
         0.07895265314386844, 1.0952652496985547`}}]}}, 
      {Arrowheads[{{0.009238393871238825, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.12797938291173772`, 1.1785188601385266`}, {
         0.08254693287773616, 1.2214811398614722`}}]}}, 
      {Arrowheads[{{0.009582169940053478, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.24480544744515437`, -1.2267348786587833`}, {
         0.28151034202853004`, -1.1732651213412164`}}]}}, 
      {Arrowheads[{{0.010490541424068977`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.24148653936528372`, -1.1018043354578013`}, {
         0.2848292501084007, -1.0455640855948305`}}]}}, 
      {Arrowheads[{{0.011567020999966348`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.23731904432008236`, -0.9767739014611861}, {
         0.2889967451536021, -0.9179629406440771}}]}}, 
      {Arrowheads[{{0.012855728837804777`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.23205654178743693`, -0.8514745801801398}, {
         0.29425924768624745`, -0.7906306829777552}}]}}, 
      {Arrowheads[{{0.014414687927071549`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.22540115886342718`, -0.7256255618818942}, {
         0.30091463061025725`, -0.6638481223286326}}]}}, 
      {Arrowheads[{{0.016320551476388688`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.21702734103784793`, -0.5987941387448344}, {
         0.3092884484358364, -0.5380479665183239}}]}}, 
      {Arrowheads[{{0.018674981872061333`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.20664682606164173`, -0.47040179178567953`}, {
         0.31966896341204265`, -0.4138087345301103}}]}}, 
      {Arrowheads[{{0.021610678808665018`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.1941194256491614, -0.33992156358297365`}, {
         0.33219636382452294`, -0.2916573837854478}}]}}, 
      {Arrowheads[{{0.025271513926771107`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.17957508093650595`, -0.20758967704783526`}, {
         0.3467407085371784, -0.17135769137321777`}}]}}, 
      {Arrowheads[{{0.02962915583088031, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.16367902647924581`, -0.07573372443276724}, {
         0.36263676299443864`, -0.05058206504091736}}]}}, 
      {Arrowheads[{{0.029629155830880317`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.36263676299443864`, 0.05058206504091696}, {
         0.16367902647924576`, 0.07573372443276687}}]}}, 
      {Arrowheads[{{0.02527151392677112, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3467407085371785, 0.17135769137321738`}, {
         0.17957508093650593`, 0.20758967704783488`}}]}}, 
      {Arrowheads[{{0.021610678808665035`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.332196363824523, 0.2916573837854474}, {
         0.19411942564916138`, 0.33992156358297326`}}]}}, 
      {Arrowheads[{{0.018674981872061343`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3196689634120427, 0.41380873453010997`}, {
         0.2066468260616417, 0.47040179178567915`}}]}}, 
      {Arrowheads[{{0.0163205514763887, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.30928844843583647`, 0.5380479665183234}, {
         0.2170273410378479, 0.598794138744834}}]}}, 
      {Arrowheads[{{0.014414687927071554`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.30091463061025725`, 0.663848122328632}, {
         0.22540115886342715`, 0.7256255618818936}}]}}, 
      {Arrowheads[{{0.012855728837804777`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.29425924768624745`, 0.7906306829777547}, {
         0.23205654178743693`, 0.8514745801801393}}]}}, 
      {Arrowheads[{{0.011567020999966354`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.2889967451536021, 0.9179629406440767}, {
         0.23731904432008233`, 0.9767739014611857}}]}}, 
      {Arrowheads[{{0.01049054142406898, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.2848292501084007, 1.0455640855948298`}, {
         0.2414865393652837, 1.1018043354578007`}}]}}, 
      {Arrowheads[{{0.009582169940053478, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.28151034202853004`, 1.1732651213412162`}, {
         0.24480544744515437`, 1.226734878658783}}]}}, 
      {Arrowheads[{{0.009840903959018236, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4087643164058752, -1.230953481318371}, {
         0.43334094675201973`, -1.169046518681629}}]}}, 
      {Arrowheads[{{0.010823846235434776`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.406286674642464, -1.107206112242004}, {
         0.4358185885154309, -1.0401623088106273`}}]}}, 
      {Arrowheads[{{0.012002606599705977`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4030566313509435, -0.9837834300204983}, {
         0.4390486318069514, -0.910953412084765}}]}}, 
      {Arrowheads[{{0.013432503822042129`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3987834263164952, -0.8606824479400906}, {
         0.44332183684139964`, -0.7814228152178042}}]}}, 
      {Arrowheads[{{0.015185598661954607`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.39306004327249844`, -0.7378348801711017}, {
         0.4490452198853965, -0.651638804039425}}]}}, 
      {Arrowheads[{{0.017351645400594782`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3853449503837965, -0.6150380373179901}, {
         0.45676031277409845`, -0.5218040679451682}}]}}, 
      {Arrowheads[{{0.02003012257802061, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.37500526764943265`, -0.49185001143298723`}, {
         0.46709999550846226`, -0.3923605148828027}}]}}, 
      {Arrowheads[{{0.02329801162659684, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.361533595947964, -0.36750013970376844`}, {
         0.4805716672099309, -0.2640788076646531}}]}}, 
      {Arrowheads[{{0.027129187109826043`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3450684829696842, -0.24100553864093302`}, {
         0.49703678018821074`, -0.13794182978012}}]}}, 
      {Arrowheads[{{0.03126800835254949, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3270399285869346, -0.11172544742397292`}, {
         0.5150653345709603, -0.014590342049711678`}}]}}, 
      {Arrowheads[{{0.03126800835254951, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5150653345709604, 0.014590342049711304`}, {
         0.32703992858693454`, 0.11172544742397253`}}]}}, 
      {Arrowheads[{{0.02712918710982605, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4970367801882108, 0.13794182978011965`}, {
         0.3450684829696841, 0.2410055386409326}}]}}, 
      {Arrowheads[{{0.023298011626596855`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.48057166720993094`, 0.26407880766465264`}, {
         0.36153359594796397`, 0.367500139703768}}]}}, 
      {Arrowheads[{{0.02003012257802061, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.46709999550846226`, 0.3923605148828023}, {
         0.3750052676494326, 0.4918500114329868}}]}}, 
      {Arrowheads[{{0.017351645400594796`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.45676031277409845`, 0.5218040679451678}, {
         0.3853449503837965, 0.6150380373179898}}]}}, 
      {Arrowheads[{{0.015185598661954612`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4490452198853965, 0.6516388040394245}, {
         0.3930600432724984, 0.7378348801711012}}]}}, 
      {Arrowheads[{{0.013432503822042145`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.44332183684139964`, 0.7814228152178038}, {
         0.3987834263164952, 0.8606824479400903}}]}}, 
      {Arrowheads[{{0.012002606599705977`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4390486318069514, 0.9109534120847645}, {
         0.4030566313509435, 0.9837834300204978}}]}}, 
      {Arrowheads[{{0.010823846235434776`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4358185885154309, 1.040162308810627}, {0.406286674642464,
          1.1072061122420038`}}]}}, 
      {Arrowheads[{{0.009840903959018236, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.43334094675201973`, 1.1690465186816286`}, {
         0.4087643164058752, 1.2309534813183705`}}]}}, 
      {Arrowheads[{{0.00999380117903193, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5740425661543467, -1.2334633524851293`}, {
         0.5838521706877586, -1.1665366475148704`}}]}}, 
      {Arrowheads[{{0.011026363121178129`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5729172661621205, -1.1105090386843657`}, {
         0.5849774706799848, -1.0368593823682661`}}]}}, 
      {Arrowheads[{{0.012276997649468161`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5713740833514799, -0.9882200247276461}, {
         0.5865206534906253, -0.9065168173776171}}]}}, 
      {Arrowheads[{{0.013813553151133176`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5691923310115958, -0.866771143493025}, {
         0.5887024058305095, -0.77533411966487}}]}}, 
      {Arrowheads[{{0.015728455783587496`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5660008055323743, -0.7463663980110855}, {
         0.591893931309731, -0.6431072861994411}}]}}, 
      {Arrowheads[{{0.018143537390147232`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.561161423287686, -0.6271897442868368}, {
         0.5967333135544193, -0.5096523609763215}}]}}, 
      {Arrowheads[{{0.021203337140008206`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5535844524803266, -0.5092294445190879}, {
         0.6043102843617787, -0.374981081796702}}]}}, 
      {Arrowheads[{{0.025026881445306973`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5415595673551752, -0.3917862292098543}, {
         0.6163351694869301, -0.23979271815856718`}}]}}, 
      {Arrowheads[{{0.02955838443932291, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5231484628912296, -0.2724960057102825}, {
         0.6347462739508756, -0.10645136271077053`}}]}}, 
      {Arrowheads[{{0.034316693390931345`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.49843676160404826`, -0.14685482880132927`}, {
         0.659457975238057, 0.020539039327644684`}}]}}, 
      {Arrowheads[{{0.03431669339093137, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6594579752380572, -0.02053903932764506}, {
         0.49843676160404815`, 0.14685482880132888`}}]}}, 
      {Arrowheads[{{0.02955838443932293, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6347462739508757, 0.10645136271077013`}, {
         0.5231484628912296, 0.2724960057102821}}]}}, 
      {Arrowheads[{{0.025026881445307, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6163351694869302, 0.23979271815856676`}, {
         0.5415595673551751, 0.39178622920985395`}}]}}, 
      {Arrowheads[{{0.021203337140008226`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6043102843617787, 0.37498108179670153`}, {
         0.5535844524803265, 0.5092294445190876}}]}}, 
      {Arrowheads[{{0.018143537390147253`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5967333135544193, 0.5096523609763209}, {
         0.5611614232876859, 0.6271897442868364}}]}}, 
      {Arrowheads[{{0.015728455783587492`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.591893931309731, 0.6431072861994407}, {
         0.5660008055323744, 0.7463663980110851}}]}}, 
      {Arrowheads[{{0.013813553151133176`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5887024058305095, 0.7753341196648695}, {
         0.5691923310115958, 0.8667711434930245}}]}}, 
      {Arrowheads[{{0.012276997649468165`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5865206534906254, 0.9065168173776167}, {
         0.5713740833514799, 0.9882200247276457}}]}}, 
      {Arrowheads[{{0.011026363121178129`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5849774706799848, 1.0368593823682655`}, {
         0.5729172661621205, 1.110509038684365}}]}}, 
      {Arrowheads[{{0.00999380117903193, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5838521706877586, 1.1665366475148702`}, {
         0.5740425661543467, 1.233463352485129}}]}}, 
      {Arrowheads[{{0.010025190696762058`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7399439842139818, -1.2337850281841602`}, {
         0.733740226312334, -1.1662149718158397`}}]}}, 
      {Arrowheads[{{0.011074460446568268`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7404607172935431, -1.1109871579528647`}, {
         0.7332234932327729, -1.0363812630997669`}}]}}, 
      {Arrowheads[{{0.012353948421732458`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7410715131971796, -0.988962019510102}, {
         0.7326126973291364, -0.9057748225951613}}]}}, 
      {Arrowheads[{{0.013942801639904533`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7417593692681924, -0.8679807667665995}, {
         0.7319248412581235, -0.7741244963912954}}]}}, 
      {Arrowheads[{{0.015957521841883177`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7424340376128318, -0.7484497901409938}, {
         0.7312501729134842, -0.6410238940695329}}]}}, 
      {Arrowheads[{{0.018573201480788046`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7427931989451237, -0.6309938908071386}, {
         0.7308910115811922, -0.5058482144560197}}]}}, 
      {Arrowheads[{{0.022053591923049967`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7419245254218889, -0.51656549886543}, {
         0.731759685104427, -0.36764502745035993`}}]}}, 
      {Arrowheads[{{0.026767368567933863`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7371762934424471, -0.406374650131089}, {
         0.736507917083869, -0.2252042972373325}}]}}, 
      {Arrowheads[{{0.033024797482977396`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7215335454552583, -0.30018238387938384`}, {
         0.7521506650710577, -0.07876498454166919}}]}}, 
      {Arrowheads[{{0.040114687338496144`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6836522325126781, -0.18805956101374455`}, {
         0.7900319780136378, 0.06174377154005993}}]}}, 
      {Arrowheads[{{0.04011468733849616, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7900319780136379, -0.06174377154006033}, {
         0.6836522325126779, 0.18805956101374416`}}]}}, 
      {Arrowheads[{{0.03302479748297742, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7521506650710578, 0.07876498454166875}, {
         0.7215335454552583, 0.3001823838793835}}]}}, 
      {Arrowheads[{{0.026767368567933877`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.736507917083869, 0.22520429723733207`}, {
         0.7371762934424471, 0.40637465013108864`}}]}}, 
      {Arrowheads[{{0.02205359192304999, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.731759685104427, 0.36764502745035943`}, {
         0.7419245254218889, 0.5165654988654297}}]}}, 
      {Arrowheads[{{0.01857320148078808, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7308910115811922, 0.5058482144560191}, {
         0.7427931989451237, 0.6309938908071383}}]}}, 
      {Arrowheads[{{0.015957521841883173`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7312501729134842, 0.6410238940695324}, {
         0.7424340376128317, 0.7484497901409933}}]}}, 
      {Arrowheads[{{0.013942801639904552`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7319248412581235, 0.774124496391295}, {
         0.7417593692681925, 0.8679807667665992}}]}}, 
      {Arrowheads[{{0.012353948421732458`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7326126973291364, 0.9057748225951608}, {
         0.7410715131971796, 0.9889620195101015}}]}}, 
      {Arrowheads[{{0.011074460446568268`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7332234932327728, 1.0363812630997664`}, {
         0.740460717293543, 1.1109871579528643`}}]}}, 
      {Arrowheads[{{0.010025190696762058`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7337402263123342, 1.1662149718158392`}, {
         0.7399439842139819, 1.2337850281841598`}}]}}, 
      {Arrowheads[{{0.009927938763075048, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.905578888342633, -1.2318005664681642`}, {
         0.8838947958678934, -1.1681994335318355`}}]}}, 
      {Arrowheads[{{0.01095521429839864, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9076707682366335, -1.1084294550232479`}, {
         0.881802915973893, -1.0389389660293835`}}]}}, 
      {Arrowheads[{{0.012208982274020423`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9103721109068451, -0.9856133279887097}, {
         0.8791015733036813, -0.9091235141165537}}]}}, 
      {Arrowheads[{{0.01377171911449401, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9139176154856199, -0.8635287959185947}, {
         0.8755560687249067, -0.7785764672393004}}]}}, 
      {Arrowheads[{{0.015772316839099708`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9186503812448382, -0.742456763713899}, {
         0.8708233029656882, -0.6470169204966276}}]}}, 
      {Arrowheads[{{0.018426085003670576`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9250720779145631, -0.6229023295704179}, {
         0.8644016062959634, -0.5139397756927404}}]}}, 
      {Arrowheads[{{0.022126935964062955`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9338831283937556, -0.5059397141111025}, {
         0.8555905558167709, -0.3782708122046874}}]}}, 
      {Arrowheads[{{0.02769061071592536, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9457632294007989, -0.3943891724238091}, {
         0.8437104548097276, -0.23718977494461235`}}]}}, 
      {Arrowheads[{{0.0370753426415944, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9583293034398082, -0.2976341993427827}, {
         0.8311443807707184, -0.08131316907827034}}]}}, 
      {Arrowheads[{{0.054207514290864414`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9281136875309813, -0.24354441836209967`}, {
         0.8613599966795452, 0.11722862888841507`}}]}}, 
      {Arrowheads[{{0.05420751429086448, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8613599966795455, -0.11722862888841576`}, {
         0.928113687530981, 0.24354441836209959`}}]}}, 
      {Arrowheads[{{0.03707534264159443, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8311443807707184, 0.08131316907826985}, {
         0.9583293034398082, 0.2976341993427824}}]}}, 
      {Arrowheads[{{0.027690610715925396`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8437104548097275, 0.2371897749446119}, {
         0.9457632294007989, 0.39438917242380883`}}]}}, 
      {Arrowheads[{{0.022126935964062965`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8555905558167709, 0.37827081220468695`}, {
         0.9338831283937556, 0.5059397141111022}}]}}, 
      {Arrowheads[{{0.018426085003670576`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8644016062959634, 0.51393977569274}, {0.9250720779145631,
          0.6229023295704175}}]}}, 
      {Arrowheads[{{0.015772316839099694`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8708233029656882, 0.6470169204966272}, {
         0.9186503812448382, 0.7424567637138985}}]}}, 
      {Arrowheads[{{0.01377171911449403, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8755560687249067, 0.7785764672392999}, {0.91391761548562,
          0.8635287959185942}}]}}, 
      {Arrowheads[{{0.012208982274020438`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8791015733036813, 0.9091235141165531}, {
         0.9103721109068451, 0.9856133279887093}}]}}, 
      {Arrowheads[{{0.01095521429839864, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.881802915973893, 1.0389389660293833`}, {
         0.9076707682366335, 1.1084294550232476`}}]}}, 
      {Arrowheads[{{0.009927938763075045, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8838947958678935, 1.1681994335318353`}, {
         0.905578888342633, 1.231800566468164}}]}}, 
      {Arrowheads[{{0.009706630010235561, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0700865163490996`, -1.2278277848163182`}, {
         1.0351766415456376`, -1.1721722151836818`}}]}}, 
      {Arrowheads[{{0.010672942252903878`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0733175868247007`, -1.1032931345057886`}, {
         1.0319455710700365`, -1.044075286546843}}]}}, 
      {Arrowheads[{{0.011843337631809336`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0774560817350332`, -0.9788351724529442}, {
         1.027807076159704, -0.9159016696523192}}]}}, 
      {Arrowheads[{{0.013290055207293065`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0828528117481107`, -0.8543621060729145}, {
         1.0224103461466265`, -0.7877431570849804}}]}}, 
      {Arrowheads[{{0.015126213333990822`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.090044283044854, -0.7296753548405223}, {
         1.0152188748498832`, -0.6597983293700044}}]}}, 
      {Arrowheads[{{0.017542594628484842`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0998973520728355`, -0.6043437234141541}, {
         1.0053658058219015`, -0.5324983818490042}}]}}, 
      {Arrowheads[{{0.02089643416288163, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1139324332232685`, -0.47736378052001066`}, {
         0.9913307246714687, -0.4068467457957792}}]}}, 
      {Arrowheads[{{0.025968507395782485`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1351604038954353`, -0.3459936317943119}, {
         0.970102753999302, -0.28558531557410954`}}]}}, 
      {Arrowheads[{{0.03494423556345348, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1704339374885309`, -0.19984296061644924`}, {
         0.9348292204062062, -0.17910440780460377`}}]}}}, 
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.03494423556345358, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.934829220406206, 0.17910440780460352`}, {
         1.1704339374885313`, 0.19984296061644877`}}]}}, 
      {Arrowheads[{{0.025968507395782495`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9701027539993019, 0.28558531557410916`}, {
         1.1351604038954353`, 0.3459936317943115}}]}}, 
      {Arrowheads[{{0.02089643416288167, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9913307246714685, 0.40684674579577884`}, {
         1.1139324332232687`, 0.4773637805200103}}]}}, 
      {Arrowheads[{{0.017542594628484856`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0053658058219015`, 0.5324983818490038}, {
         1.0998973520728355`, 0.6043437234141538}}]}}, 
      {Arrowheads[{{0.015126213333990822`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0152188748498832`, 0.6597983293700039}, {
         1.090044283044854, 0.7296753548405218}}]}}, 
      {Arrowheads[{{0.013290055207293065`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0224103461466265`, 0.78774315708498}, {
         1.0828528117481107`, 0.8543621060729141}}]}}, 
      {Arrowheads[{{0.011843337631809349`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.027807076159704, 0.9159016696523187}, {
         1.0774560817350332`, 0.9788351724529437}}]}}, 
      {Arrowheads[{{0.010672942252903878`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0319455710700365`, 1.0440752865468426`}, {
         1.0733175868247007`, 1.1032931345057881`}}]}}, 
      {Arrowheads[{{0.009706630010235579, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0351766415456376`, 1.1721722151836813`}, {
         1.0700865163490998`, 1.2278277848163177`}}]}}, 
      {Arrowheads[{{0.009378428497519763, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2328748682849635`, -1.22253587585988}, {
         1.1881777632939843`, -1.1774641241401196`}}]}}, 
      {Arrowheads[{{0.010252124715533682`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2365948401392377`, -1.0965792498128522`}, {
         1.18445779143971, -1.0507891712397792`}}]}}, 
      {Arrowheads[{{0.011292636624290196`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2411905849041047`, -0.9701762216186663}, {
         1.179862046674843, -0.924560620486597}}]}}, 
      {Arrowheads[{{0.01255090186880351, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.246902549884967, -0.8429811890482283}, {
         1.1741500816939807`, -0.7991240741096667}}]}}, 
      {Arrowheads[{{0.014100729473368408`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2540151047008008`, -0.7143805953998587}, {
         1.1670375268781472`, -0.6750930888106681}}]}}, 
      {Arrowheads[{{0.016052364028573414`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2627901106413246`, -0.5832404799630176}, {
         1.1582625209376234`, -0.5536016253001407}}]}}, 
      {Arrowheads[{{0.018572747515751453`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2731594073168675`, -0.4473662813306919}, {
         1.1478932242620805`, -0.436844244985098}}]}}, 
      {Arrowheads[{{0.0218966629501227, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2834119934997557`, -0.3024161623628748}, {
         1.137640638079192, -0.32916278500554674`}}]}}, 
      {Arrowheads[{{0.026175883077771377`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2848017552220563`, -0.14120049284568054`}, {
         1.1362508763568915`, -0.23774687557537244`}}]}}, 
      {Arrowheads[{{0.030331685052010355`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2488927136254189`, 0.03205057966016614}, {
         1.1721599179535291`, -0.15836636913385074`}}]}}, 
      {Arrowheads[{{0.030331685052010365`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1721599179535291`, 0.15836636913385044`}, {
         1.2488927136254186`, -0.032050579660166625`}}]}}, 
      {Arrowheads[{{0.0261758830777714, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1362508763568915`, 0.23774687557537222`}, {
         1.2848017552220563`, 0.14120049284568004`}}]}}, 
      {Arrowheads[{{0.0218966629501227, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1376406380791921`, 0.3291627850055464}, {
         1.2834119934997559`, 0.30241616236287433`}}]}}, 
      {Arrowheads[{{0.01857274751575148, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1478932242620803`, 0.43684424498509766`}, {
         1.2731594073168675`, 0.44736628133069145`}}]}}, 
      {Arrowheads[{{0.016052364028573442`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1582625209376232`, 0.5536016253001402}, {
         1.2627901106413246`, 0.5832404799630171}}]}}, 
      {Arrowheads[{{0.01410072947336844, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.167037526878147, 0.6750930888106675}, {
         1.2540151047008008`, 0.7143805953998581}}]}}, 
      {Arrowheads[{{0.01255090186880351, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1741500816939807`, 0.7991240741096662}, {
         1.246902549884967, 0.8429811890482278}}]}}, 
      {Arrowheads[{{0.011292636624290196`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.179862046674843, 0.9245606204865965}, {
         1.2411905849041047`, 0.9701762216186658}}]}}, 
      {Arrowheads[{{0.010252124715533682`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.18445779143971, 1.050789171239779}, {1.2365948401392377`,
          1.096579249812852}}]}}, 
      {Arrowheads[{{0.009378428497519738, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1881777632939843`, 1.1774641241401194`}, {
         1.2328748682849633`, 1.2225358758598799`}}]}}, 
      {Arrowheads[{{0.008970146511791585, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3937543429857884`, -1.216725759819141}, {
         1.3430877622773703`, -1.1832742401808591`}}]}}, 
      {Arrowheads[{{0.009732726263076695, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3973389730637031`, -1.089452033076731}, {
         1.3395031321994553`, -1.0579163879759006`}}]}}, 
      {Arrowheads[{{0.01061909673868824, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4015154647787311`, -0.9613765921283777}, {
         1.3353266404844273`, -0.9333602499768856}}]}}, 
      {Arrowheads[{{0.01165696924604875, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4062990848282475`, -0.8320756709729739}, {
         1.330543020434911, -0.8100295921849211}}]}}, 
      {Arrowheads[{{0.012879619155350465`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4115700028300084`, -0.7009011455947451}, {
         1.32527210243315, -0.6885725386157816}}]}}, 
      {Arrowheads[{{0.0143225811386093, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4168670571137438`, -0.5668865905744764}, {
         1.3199750481494148`, -0.5699555146886819}}]}}, 
      {Arrowheads[{{0.016008900097416727`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4209199520773192`, -0.4287252121265322}, {
         1.3159221531858392`, -0.45548531418925764`}}]}}, 
      {Arrowheads[{{0.017900707281195003`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.420729512857102, -0.2852332547675926}, {
         1.3161125924060562`, -0.34634569260082887`}}]}}, 
      {Arrowheads[{{0.019775518401874966`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4106552985766825`, -0.13755934662220537`}, {
         1.326186806686476, -0.24138802179884766`}}]}}, 
      {Arrowheads[{{0.02106159496150808, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3854088938376699`, 0.006064474276765666}, {
         1.3514332114254883`, -0.13238026375045026`}}]}}, 
      {Arrowheads[{{0.021061594961508088`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3514332114254886`, 0.13238026375044992`}, {
         1.3854088938376699`, -0.0060644742767660825`}}]}}, 
      {Arrowheads[{{0.01977551840187498, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.326186806686476, 0.24138802179884733`}, {
         1.4106552985766825`, 0.13755934662220493`}}]}}, 
      {Arrowheads[{{0.01790070728119501, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3161125924060564`, 0.34634569260082854`}, {
         1.4207295128571023`, 0.28523325476759215`}}]}}, 
      {Arrowheads[{{0.01600890009741673, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3159221531858392`, 0.4554853141892573}, {
         1.4209199520773192`, 0.4287252121265318}}]}}, 
      {Arrowheads[{{0.0143225811386093, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3199750481494146`, 0.5699555146886814}, {
         1.4168670571137436`, 0.566886590574476}}]}}, 
      {Arrowheads[{{0.012879619155350465`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.32527210243315, 0.688572538615781}, {1.4115700028300084`,
          0.7009011455947446}}]}}, 
      {Arrowheads[{{0.01165696924604875, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.330543020434911, 0.8100295921849207}, {
         1.4062990848282475`, 0.8320756709729734}}]}}, 
      {Arrowheads[{{0.01061909673868824, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3353266404844273`, 0.9333602499768852}, {
         1.4015154647787311`, 0.9613765921283772}}]}}, 
      {Arrowheads[{{0.009732726263076695, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3395031321994553`, 1.0579163879759002`}, {
         1.3973389730637031`, 1.0894520330767306`}}]}}, 
      {Arrowheads[{{0.008970146511791585, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.34308776227737, 1.1832742401808585`}, {
         1.3937543429857882`, 1.2167257598191403`}}]}}, 
      {Arrowheads[{{0.008512421345355098, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.552901693526351, -1.2110936569439972`}, {
         1.499729885421018, -1.1889063430560025`}}]}}, 
      {Arrowheads[{{0.009159637493586786, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.555929694386899, -1.0828432671443036`}, {
         1.49670188456047, -1.064525153908328}}]}}, 
      {Arrowheads[{{0.009890859708390695, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5591886008784785`, -0.9536762010805279}, {
         1.4934429780688905`, -0.9410606410247354}}]}}, 
      {Arrowheads[{{0.010716154219958335`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5625145268617953`, -0.8232517369585725}, {
         1.4901170520855735`, -0.8188535261993225}}]}}, 
      {Arrowheads[{{0.011642102822538445`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.565550711157693, -0.6911442502265628}, {
         1.487080867789676, -0.6983294339839637}}]}}, 
      {Arrowheads[{{0.0126650341472139, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5675991841803376`, -0.5569000389691942}, {
         1.4850323947670316`, -0.5799420662939642}}]}}, 
      {Arrowheads[{{0.01375717973187388, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5674239168988047`, -0.4202497266869581}, {
         1.4852076620485644`, -0.4639607996288318}}]}}, 
      {Arrowheads[{{0.01484272285025455, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5631557227883546`, -0.28164378412727664`}, {
         1.4894758561590142`, -0.3499351632411448}}]}}, 
      {Arrowheads[{{0.015770848346309086`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.55281018843681, -0.14314265146293775`}, {
         1.499821390510559, -0.23580471695811528`}}]}}, 
      {Arrowheads[{{0.01632484135913546, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5361067354359443`, -0.008786085703504372}, {
         1.5165248435114245`, -0.11752970377018024`}}]}}, 
      {Arrowheads[{{0.016324841359135454`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5165248435114247`, 0.11752970377017985`}, {
         1.5361067354359443`, 0.00878608570350397}}]}}, 
      {Arrowheads[{{0.015770848346309096`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.499821390510559, 0.23580471695811495`}, {
         1.55281018843681, 0.1431426514629373}}]}}, 
      {Arrowheads[{{0.014842722850254562`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4894758561590145`, 0.3499351632411445}, {
         1.5631557227883548`, 0.2816437841272762}}]}}, 
      {Arrowheads[{{0.013757179731873887`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4852076620485644`, 0.46396079962883147`}, {
         1.5674239168988047`, 0.42024972668695765`}}]}}, 
      {Arrowheads[{{0.0126650341472139, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4850323947670314`, 0.5799420662939637}, {
         1.5675991841803374`, 0.5569000389691937}}]}}, 
      {Arrowheads[{{0.011642102822538447`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.487080867789676, 0.6983294339839634}, {1.565550711157693,
          0.6911442502265623}}]}}, 
      {Arrowheads[{{0.010716154219958335`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4901170520855738`, 0.8188535261993221}, {
         1.5625145268617955`, 0.823251736958572}}]}}, 
      {Arrowheads[{{0.009890859708390695, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4934429780688905`, 0.941060641024735}, {
         1.5591886008784785`, 0.9536762010805274}}]}}, 
      {Arrowheads[{{0.009159637493586786, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.49670188456047, 1.0645251539083276`}, {1.555929694386899,
          1.0828432671443031`}}]}}, 
      {Arrowheads[{{0.008512421345355098, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.499729885421018, 1.1889063430560023`}, {
         1.552901693526351, 1.211093656943997}}]}}, 
      {Arrowheads[{{0.008033744891342804, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.7107063121832629`, -1.2060946115822726`}, {
         1.6577147404483166`, -1.1939053884177275`}}]}}, 
      {Arrowheads[{{0.008572394463255762, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.7129987421758817`, -1.0772693113435305`}, {
         1.655422310455698, -1.0700991097091008`}}]}}, 
      {Arrowheads[{{0.009163481913706478, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.7152206265653378`, -0.9475967082442175}, {
         1.6532004260662418`, -0.9471401338610458}}]}}, 
      {Arrowheads[{{0.009806854587415344, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.7171348622815528`, -0.816875671615871}, {
         1.6512861903500269`, -0.825229591542024}}]}}, 
      {Arrowheads[{{0.010496636881355952`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.718351070727048, -0.6849247371572721}, {
         1.6500699819045317`, -0.7045489470532545}}]}}, 
      {Arrowheads[{{0.011216616959266247`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.7182745746167134`, -0.5516718288528424}, {
         1.6501464780148662`, -0.5851702764103158}}]}}, 
      {Arrowheads[{{0.011933736828034518`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.7161001016731487`, -0.41732454125800383`}, {
         1.652320950958431, -0.466885985057786}}]}}, 
      {Arrowheads[{{0.012591485389049485`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.7109533385072595`, -0.2826141814725099}, {
         1.6574677141243201`, -0.3489647658959115}}]}}, 
      {Arrowheads[{{0.01310938732380734, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.702311970350022, -0.14896988689892665`}, {
         1.6661090822815574`, -0.22997748152212633`}}]}}, 
      {Arrowheads[{{0.013398913811954509`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.690651722321292, -0.01827327741356373}, {
         1.6777693303102879`, -0.10804251206012087`}}]}}, 
      {Arrowheads[{{0.013398913811954512`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6777693303102879`, 0.1080425120601205}, {
         1.690651722321292, 0.018273277413563334`}}]}}, 
      {Arrowheads[{{0.013109387323807349`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6661090822815576`, 0.229977481522126}, {
         1.7023119703500222`, 0.14896988689892626`}}]}}, 
      {Arrowheads[{{0.012591485389049493`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6574677141243201`, 0.3489647658959112}, {
         1.7109533385072595`, 0.2826141814725095}}]}}, 
      {Arrowheads[{{0.011933736828034528`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.652320950958431, 0.46688598505778567`}, {
         1.7161001016731487`, 0.4173245412580034}}]}}, 
      {Arrowheads[{{0.011216616959266247`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6501464780148662`, 0.5851702764103155}, {
         1.7182745746167134`, 0.551671828852842}}]}}, 
      {Arrowheads[{{0.010496636881355952`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6500699819045317`, 0.7045489470532541}, {
         1.718351070727048, 0.6849247371572716}}]}}, 
      {Arrowheads[{{0.009806854587415344, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6512861903500269`, 0.8252295915420236}, {
         1.7171348622815528`, 0.8168756716158706}}]}}, 
      {Arrowheads[{{0.009163481913706478, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6532004260662418`, 0.9471401338610453}, {
         1.7152206265653378`, 0.947596708244217}}]}}, 
      {Arrowheads[{{0.008572394463255795, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.655422310455698, 1.0700991097091006`}, {
         1.7129987421758819`, 1.0772693113435303`}}]}}, 
      {Arrowheads[{{0.008033744891342811, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6577147404483168`, 1.1939053884177269`}, {
         1.710706312183263, 1.2060946115822722`}}]}}, 
      {Arrowheads[{{0.0075566675651038044`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8676053177669676`, -1.2019325444988582`}, {
         1.8166052085488227`, -1.1980674555011415`}}]}}, 
      {Arrowheads[{{0.007999751645560542, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8691658213621694`, -1.0728748506252384`}, {
         1.815044704953621, -1.074493570427393}}]}}, 
      {Arrowheads[{{0.008472771195877664, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.870463496001956, -0.9431284759412981}, {
         1.8137470303138346`, -0.951608366163965}}]}}, 
      {Arrowheads[{{0.008971027761402843, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8712680188810078`, -0.8126121697795445}, {
         1.8129425074347827`, -0.8294930933783505}}]}}, 
      {Arrowheads[{{0.009484897904580266, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.871259022803514, -0.681306165489044}, {
         1.8129515035122763`, -0.7081675187214826}}]}}, 
      {Arrowheads[{{0.009997615303772772, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8700297496119096`, -0.5493174045881467}, {
         1.8141807767038807`, -0.5875247006750116}}]}}, 
      {Arrowheads[{{0.010483155402453344`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8671369014215533`, -0.4169650167568063}, {
         1.817073624894237, -0.46724550955898364`}}]}}, 
      {Arrowheads[{{0.010905632161814841`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8622295813158158`, -0.2848521728151507}, {
         1.8219809449999744`, -0.3467267745532707}}]}}, 
      {Arrowheads[{{0.011222518784465256`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.855260721617096, -0.15384579482360608`}, {
         1.8289498046986943`, -0.22510157359744695`}}]}}, 
      {Arrowheads[{{0.011393539913104343`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8466934894709317`, -0.024873983090214033`}, {
         1.8375170368448588`, -0.10144180638347057`}}]}}, 
      {Arrowheads[{{0.011393539913104348`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8375170368448588`, 0.1014418063834702}, {
         1.8466934894709317`, 0.024873983090213637`}}]}}, 
      {Arrowheads[{{0.011222518784465256`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8289498046986943`, 0.22510157359744656`}, {
         1.855260721617096, 0.1538457948236057}}]}}, 
      {Arrowheads[{{0.01090563216181485, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8219809449999744`, 0.3467267745532704}, {
         1.8622295813158158`, 0.2848521728151503}}]}}, 
      {Arrowheads[{{0.01048315540245335, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.817073624894237, 0.46724550955898325`}, {
         1.8671369014215533`, 0.41696501675680586`}}]}}, 
      {Arrowheads[{{0.009997615303772772, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8141807767038807`, 0.5875247006750112}, {
         1.8700297496119096`, 0.5493174045881463}}]}}, 
      {Arrowheads[{{0.009484897904580266, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8129515035122763`, 0.7081675187214822}, {
         1.871259022803514, 0.6813061654890435}}]}}, 
      {Arrowheads[{{0.008971027761402843, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8129425074347825`, 0.82949309337835}, {
         1.8712680188810076`, 0.812612169779544}}]}}, 
      {Arrowheads[{{0.008472771195877665, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8137470303138343`, 0.9516083661639647}, {
         1.8704634960019557`, 0.9431284759412977}}]}}, 
      {Arrowheads[{{0.007999751645560544, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.815044704953621, 1.0744935704273928`}, {
         1.8691658213621694`, 1.072874850625238}}]}}, 
      {Arrowheads[{{0.0075566675651038044`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8166052085488227`, 1.1980674555011412`}, {
         1.8676053177669676`, 1.201932544498858}}]}}, 
      {Arrowheads[{{0.007096696213581382, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.0239772439984414`, -1.1986265160371996`}, {
         1.9760227560015595`, -1.2013734839628005`}}]}}, 
      {Arrowheads[{{0.007459265816896999, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.024906968802244, -1.0695756933598959`}, {
         1.9750930311977568`, -1.077792727692736}}]}}, 
      {Arrowheads[{{0.007836901911849539, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.0254807024292094`, -0.9400113727444223}, {
         1.9745192975707915`, -0.9547254693608411}}]}}, 
      {Arrowheads[{{0.008223633817715055, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.0255107540879536`, -0.8099293205706246}, {
         1.9744892459120473`, -0.8321759425872703}}]}}, 
      {Arrowheads[{{0.008610035643238784, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.0247684794103202`, -0.6793893995902811}, {
         1.9752315205896807`, -0.7100842846202454}}]}}, 
      {Arrowheads[{{0.008982410542028583, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.0230049513598574`, -0.5485509643308779}, {
         1.9769950486401433`, -0.5882911409322803}}]}}, 
      {Arrowheads[{{0.009322456928223834, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.019998951022939, -0.41770490075541805`}, {
         1.9800010489770619`, -0.4665056255603718}}]}}, 
      {Arrowheads[{{0.009608059968809904, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.015636068717507, -0.28728036049274186`}, {
         1.9843639312824937`, -0.34429858687567966`}}]}}, 
      {Arrowheads[{{0.009815862264952764, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.0100025047043992`, -0.15779667171678324`}, {
         1.9899974952956014`, -0.22115069670426973`}}]}}, 
      {Arrowheads[{{0.009925692555407139, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.003447143566553, -0.02974483655680784}, {
         1.996552856433448, -0.09657095291687676}}]}}, 
      {Arrowheads[{{0.009925692555407139, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.996552856433448, 0.09657095291687637}, {
         2.003447143566553, 0.029744836556807458`}}]}}, 
      {Arrowheads[{{0.009815862264952794, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9899974952956014`, 0.2211506967042694}, {
         2.0100025047043997`, 0.15779667171678285`}}]}}, 
      {Arrowheads[{{0.009608059968809937, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9843639312824937`, 0.3442985868756792}, {
         2.0156360687175074`, 0.2872803604927414}}]}}, 
      {Arrowheads[{{0.009322456928223883, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9800010489770619`, 0.46650562556037145`}, {
         2.0199989510229392`, 0.41770490075541766`}}]}}, 
      {Arrowheads[{{0.00898241054202857, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9769950486401435`, 0.58829114093228}, {
         2.0230049513598574`, 0.5485509643308775}}]}}, 
      {Arrowheads[{{0.008610035643238792, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9752315205896807`, 0.7100842846202451}, {
         2.0247684794103202`, 0.6793893995902807}}]}}, 
      {Arrowheads[{{0.008223633817714964, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9744892459120476`, 0.8321759425872699}, {
         2.025510754087953, 0.8099293205706242}}]}}, 
      {Arrowheads[{{0.007836901911849539, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9745192975707915`, 0.9547254693608407}, {
         2.0254807024292094`, 0.9400113727444218}}]}}, 
      {Arrowheads[{{0.007459265816896966, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.975093031197757, 1.0777927276927353`}, {
         2.024906968802244, 1.0695756933598952`}}]}}, 
      {Arrowheads[{{0.007096696213581481, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9760227560015593`, 1.2013734839627999`}, {
         2.023977243998442, 1.198626516037199}}]}}}}}, 
   GraphicsComplexBox[CompressedData["
1:eJxknXd8FMX7x48iSBEBRQQpUaqCUgUE5DkBwa+ooIB0DFURkI4gAgcSuggo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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl0z9I1GEcx/Gv1yCWend2llf+4SCjU2loqCGoQaOGhhIitHIwsDDIIdOU
BoMaGhxyCHFxCNKhoaAhIY0gGiqCKDJosbqsRdKgjOjf62h4834+3+f7/J7f
73nuct197WdLIuIZJjdH/EpHLG+M+IIF4981EWNcklFLRYzruSeXYao6Yhrz
VRGP1F9zs74ufS/WR8zJC57zDu8xrrfEugn+KBfwAQ/0rbEugRactP60+ilc
Mte/KWKEe63b5n3K7dXq+XnjJjTjpblX2KdeYb6NKzmFJGox79nt6itcQL93
eZP1LriDp5jDqP3/crl9v/JWzuJWre9BbYP9uA2ldRE75QyPyT+Rl5c4hZx6
q7ydZ+QDfEjezYflPu6VL/Kk3M0n5CNcJ1/lQfk6P5Zv82X5BnfJs3xNfsIX
5IRxEm/rI77hM5rUC+ZXMI91an/4jPpe44OowQ40Ypf6FN/FBJ7jIa6oj/Ao
OnAOPcUe53XTee5Bznk1og4tyGPJOXeY68Sq8z/K9Wrt3FD8vTnbFVS572pk
UKovzWs5yWX8w/2u4jsWN7g3zFo/7P7v8xAfV/ukfxEJtQr7VxbvEikksV/9
vL4B9GMYQ8iaO8bLvqfg99JpPOiZA5ixZlpti1o+/f+/8g8IwHcy
          "]]},
        "0.44399999999999995`"],
       Annotation[#, 0.44399999999999995`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl00lsjVEYh/EjhohWlUvnVlFBxKKohQQJi0osDBExtJJWDEVKK4akhkYt
LKTRhUW7sBBFY6GJhepANySIjUQlpW20FUGH28EUQ/xuLJ483/895z3n3PN9
d17xsW2lE0IIP3BgVghFaSHsQzHasTsxhO70EHrQhY1JIeTjwcwQ3vI7dOKQ
uQcxZP4ujM0JIX12CC3m1Xruj4TQZ14/3uOkvXr5E//izzwxI4Tfntcky/ba
q3dRSgiFeufLC7DO2BCv5eFY1rfSvnlYge9qyzkXdfoK5ELcNv8OGlDsfJXO
Vsn19mtEHZ6iBdWxM6eGcBNnUI0q5Km/0JduvQ7eav0o9+GEsTfm9KMRz9GG
K/b4y3HOM8ILOQV3/dYepM8N4RVvwJRMe8gRrpF/YrE8wDOQrb5eXsZNcj5v
llfzFrmUS+QKvi4XcYG8nTPky3xKvsZP5HtcJd/gQrmVr8rP+LQ8wXMCOrNC
GMdHLFHvMx5FB6ap/eHD6ms9b0IycpGDVeq3+D5q8RLtuKR+ga9gJ8qxPzYn
9k7c5xpku68cZGApFmPAPRcYm+a9f3P/U3lHbK56inc7XS3CCZzKUzmZ4zmR
4ziJU/msdz1JfwWfxzn0WieNR80ZQ4a149UGvLNBfMFkvS2+6WZk+T4z0aWn
3FpHjZfxYT6CD+hHCaKx78vYEDfzMPfYoxtR31STWr15D3nQ2HEus9dFY3vc
y7j9vmIUDWojHEWN8z3S9xiteK2vLfY/NvZJX2/k/3/7H8hSmww=
          "]]},
        "0.37`"],
       Annotation[#, 0.37, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1GtozXEYwPGfYmZlZmO2M3OsKUKiXGtkQkIYFjaXMXOpmdzLCxMvFIni
xWRLbmVHGd7IXTG5SxlWc13mFiPFEvI5efHd93me3+95/r/z/52znCWrZ1a0
CyFk+DM/NYS5kRCKeR6ndwthekoIlVkhlKq9TguhRL1MvJCX8iLOsC8Tq+WN
6SFU8DL1z3rnYoraVCSjC1JQ2kM/1tu7snsIPdW6dQ0hm9vU6z3zJ5fxEc/9
wdfNbLK/nl/wDX7ON/kVr7PnJReY957zeSSyzS3IDKEIk1COUixX3+fc1ai0
fye38/zKeO7cx8R1OIB6XMBuPVV6j2ITdmMbhqrf0Rdxhgae4cyt/AZrrT2x
pxl1uI1L2OUZfzjJ+b9yH+6BWE+fD5FoCI94HDpke4Y8lffK29BX/omTEVXP
lw/kc/KJPE0+iqfLV/EK+WaukZdwsXwWZ8l38Ab5fr4hP8Xb5Id5vvwi75Hf
4o3yIO6Mxl4hfEcL+qm/sd6KBnRS+80r1UeLJyMdg5GLYerH+SyqcA9XsV19
C+/CHKzB0vie+J14n3mIel+5yEKCu+vPn7znYmtDMMBdDeVO8e80t7mPy74L
VzBCPhyHxIN4jL6cDDP0bI3fP36h1noMCXovcS2Xu98Yr+ITPNt3NDhvtXiW
uIYLudnMmLs8iVpcc46DZoxW7+1ZUbSqf8E3+//KX8d7zM0U54nPx38X4ixE
0N7sTOdKslZmbRkK1T7yA36ICjTJZ3hX98VX8dTMK/wh/hmc4564Ue0uJ5rV
In6Ht3hm/lMUO1cRFuvJd7735jVbHytOtH5G3wReYN4TbkBHnFYfz0Xqj/md
vhdp//+3/APq9KXx
          "]]},
        "0.296`"],
       Annotation[#, 0.296, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1Gloz3EcwPHviqKQYdifTaZQ4s8OI3IVmmNn7cA2R7G5j01R1CjHGooH
5La5j9UkciZWyBM5yoOtDIukyTxQKF7/PHjv/f0c3+P3+/7+G7p0Xf7auBBC
1J+8PiHMiYQwD3PRr18Is3qHsHhgCEtQjqvx+tTykYsic9r1zESFeiWODgjh
GIrUS9QLuZg3qd0wfyNXowqlagPsU2h+zaAQ1uj70DeEr+IiZPUPIW6wc3EP
/NHTk99xG96jNHZG+303fmT+EuPHfDIhhCl6Z1jvnFqxvWvlrqk9FDfyRrVe
cls4jn85S7X1IsZD8Uw+yiOx0/wsbkIuPpk/jTMxWC030fNiNlZhKZbLH7Tm
MWzXv5eDM22PxfZpMG7EETTjNurMOWxuPTajDjVIk39mXqKzv+YcZ+vgNmxQ
e6PnPRrxFHdRa48/3N17/sbDuD8ueaetSBwSwgueji5J9hDH8wHxTwwXf+Ge
SJafJh7FN8Uzeb54AmeLV/MK8RY+Li7nBeJ8joh3cZX4ED8WX+Ma8WleKL7D
+8VPuFr8N9mdi99yJ9oxQr5NvQOv0U3uN1fITzbOQgKiSEG6/FluwmE8xwPs
kN/GtSjEeiyL9cTuxPuchGTvKwURdHV3Y+WimOpdp3GK7yyDk9xDs2+gQz5V
/Dn2LbrHHzjqrjq5Xa1A7bveTvww7ozto3e6OJOjPJ7H8gnrHcdEcycgExlq
6fhorVJxOcrQKi7mFi7hRZ7hrL4G1ONq7Lfgzq/wRb7P5/keX+A8/Wf0ncYp
XJbfY+/daBW3YKB1b8ln6d3qmcqMr9tvpd/wKrxSf4kC65WZl6YnHakYjwwU
xN4hj0MUlZ5zod4xxh+sO5r32euidYusma8/x3rZqJO/IF8onyvfam4L5kf+
/9/6BzWXrR0=
          "]]},
        "0.22199999999999998`"],
       Annotation[#, 0.22199999999999998`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1F1szXccx/Gf7KI2W5TW1jK2miolvZis7Va0W4/jWGjpAyraTqkjHloP
Lb0gqSUSIiNYWmxDPYVEqcgEyS5GgoiwxBKmG90iSywxXFiySra9/tnF+7z/
n+/v+3s4////nKyG5oqmASGEVh+lQ0OYOjyEEhRjcHoIhakhxDJCmIZSHBji
Wt/FEer8i/ECtMpxucv4DL4kJ/iBsSKslz+VjxifbZ2jPIen2yeOofZKQ7me
GXIZ/2bex6jVV4fdb4WwB18ipp5UO2OdTnkv9iGhvkb9nPrX8jc4gDL1+Jsh
nFfvd5aXGCTf5178jGo92VyXFsIYruexvC76Hnr78BDDnXPZsBC6nfM0TqEH
Z1Bl/fnm3TSvhu/pu4v16sfV5qltcL3WGhs509pn1dM5Tc7jC/IkrpVTzH0D
beYF/tu8FufMdP0urqvncQ4+15/gHpTjd2sUcz5GGCvPdDbEsRyL0Ki+y5r7
sUn/Vv5X/6Yo2+ew79uNTlzBBWw3p8PcLrRiO9rxvvp18zKc/w6XOdsT7sNq
Yz/q+RXduIZL2GaPlzzQvfuTR/MwnHjbM0HGOyHc5hK8MtIecirvkP9CtvyY
X8dI9WI5l7+VYzxTLuBZ8gpeKrfxV3Id18hzOFPewuvk3XxZPsXt8kFeIF/k
L+Sr3CL/M8o7JN/l53iEsep9xp/gDlLU+jmpXuQ6gXTkIQuT1I9wDzpwA99h
s/pG3oZqNKMh6omeifv5EXLcrxJO8dziHEOJ+zzG+57l3r/HozmbR3Etn+aF
3M1NnlOb57kIDfgMS9ClvpgfR79h6xV4HoXIj/4b7DUe54yN43rneY591puv
tiB673HP2GX99/l7/omf6XmKXrlfTvU7HYJceyS9cz8Ymxv9RlCFvXI1PzD2
EF3RHs5+iGu40r5/4DVnfRXLrJlEuzkV6jei72qvZnusxko0YRWq1OutWah3
ub4CXhHdI7WF+MT8R9hpnx04Gf2uzZuqtkHvLPscV6tQO8aVHLNmKeaZn6tn
AsYjad2JPN2cpa5vuW6Mvqu+xfxCXsIHrRO3TiL6X8SH9qrW8wG36GnQUyVX
Ylz0n6Wew5udb7L+Ikyxf696b9r//+f/AcejzCg=
          "]]},
        "0.148`"],
       Annotation[#, 0.148, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1G1MlWUcx/Gr3GyLNgVU8BHZinKTFyCinAmHhzFLHmoNTBQXEIeUBFSa
2kBRoUSrpS9w63FqqzYx9UVNLd/QBGo9WDZ9IS/AWuaWLdeypZb2uV98z+/+
/a7//39d577vczKbOp5uvy+E0Odj0dQQ8tNDWILFKMBSLJsVQhGNoxAlKI5U
voJW4AlUoRKl8qnTQkhGZUoI5XwFHTd/GWrU1GJfWgiv4oosjjrZ0eQQ9ssO
oFbP57PV0+esfWytiSbQHM0wd7o9VujtUvdstEdqCBOokHXLGmSf6kuobXXd
TNfT36JzYJs5n1n/wH4fYqP1TWhDB9ox1x6NalusJ9Ap24w15u+dHsLfspv4
C2Nm/4sRa3fon/oa0Mtv4UvslTIzhHs0md6lw/Ih6+doYkYIf8jm0V3YjT70
4o75t3ELX9p/FJP0PGrtMSzAUmeN6c9Uk+E+NNH5dMQ5h/ET3ufXqkvSm2V9
ob5y/oa+GlqLOLKcr4AexoDv0DjP/UAN+tGFnXNCOEu/wgBO4Ahy5fNpNlKx
HDG0yG/NDWGy6wv0OsbxiPxn2Q1cwoMZIfxHO+XD1l/jT9M6/j3aww/SIf4A
3cz30XT+KbqKX0vf4rfSBN9KK/l8WsGX0U/4BbSIn0OT+Gt4mL9JX8cUeQ5/
P43z3yGNH6MfYZrnkInf3a8H6G3a736doSM4hgn8iA73eNz9vI4qz+ECTfPs
R2mOtR41+/AiDmEg8vJTntEXOIhjOIzu6B3Xd9cz3UO7Pc836X4062nQux7l
eAbVmCVfrL4Iv6ivpiewHLusZdFsjDpbBk1Hp33+MfMetsofkk1GvfpcM045
fzZN4VPpST6ddvGb9GxBvb422op28/KcZQlyUIhYdEb1zdafR6H+xuhdpt/6
rgX0G/o1ZuOa2qHo+9prkuurGHT9K30Dl51vDGfse1F+mv6g73sUowRxlKEU
SXo2mHdW7QvR/4TeDj3n+Y3R70jWJKty9h2yNlmjbAPdzldGv/PoPtKrOOh3
NYDj5rb4jymTvaRupZ5B2WpZPdbgce9Mnd7VWGfeNnXF6lfxhXQCe83qxxG9
T+qJyfbwr+BlHJJXy/PlfXwvduNdeak8T76T78EOvCOPyXPl2/ludOFteV70
H+6c/wPBJNeR
          "]]},
        "0.074`"],
       Annotation[#, 0.074, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV01tojnEcB/D/LqacbdiLMeZQLkaTUatxwbhwSFEoimwOwxwmLjbmvIPz
YY4z25zZbI6j3DgPoSZKLigpF1NKjmXyeS4++77f3/P/P8+z53nf1AUrp6+I
CyFk+HOmWwg3e4XQxDVucJ3T5lfkVS7TSAOnzOtkPRe4xEVqzc/J89FeznKG
GvPaaB8no051xPxFYggPkkN4KZ9zqqe93O0ewn3eOnZPVlm7XrboG2QGo3mm
j5IDE0KIyad6L5nE7z4h/OEHv/hJpfPkxUJYyjiyGU+i9f1kCn3oSzKpzvtX
tlGWFMIOyvlm3tjDs2E8F6mhwvnnp4SQzwxKKWRj3xDuyCdU0EAtI8z7yzQS
mUgmC81/9wsh3ucW2cp7Bpt/NPvKG9r3d3+ywPyh4zv1W3K2XiWL9Uvyrr5P
rta3ypg+Tc7S58pj+jqZq+fJyfooOUkfJ6/rQ+UYPVl20D8zSP8ud9HFPF2P
k2P1FyTp7+Q5unv+A/jSO4R20buRpZ7XbfmIOj7wihWe73vPs5Up0XuXSd7t
Y5nuWLE15ayhmoqomzd5N/c4RB01FLlGkX1tvlclssj37KjcS4498+xdQjYz
mUpv8wzrx/DJ+qmygYlscmyITOOxe0uRMQpc55dz/mOdeUezeOZYP8I5mtx/
mkzQE2WjHpOF+ip71jLHvuUyj3znG+leRpNOFpnRPVqf4/gisuyfL1/6P+fJ
Lo4Ns2Y4281KKWGCeWdr612zk1wW/YZd67VeL7uSQLPeTXa2vtm6+9Ezl0ut
P+E3W8UxKjlOnvkReZQKDnOIJeYH5EH2sp99LDbfLfewg13sZJF5mSxnO6WU
sNB8q9zGJrawmVzzYrmRIjawnrW+V/8BWOS2yA==
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1G1M1lUcxvFTU4G2mligWYKmzumdU8k2W/aimi9A5pRN3XSzolpOMcNp
hpsP02QKhi1o6lT0hW5l4QxFk+ZDpaHxIGJLzKk5i4S20gC1GdjnvPje131d
5/c75/zP/2FY/uK8dx8KIVT4+aV/CJcHUlzC5dQQugaFcFHeyl/Ez3Fc3i6/
IL/At+A8WuXX5U3yZv4cmmKf/Fd5vbye/xFn4xqyVpyS92DXgBDKB4ewm458
IoQRKOXPqDmLujiHubYZL30qhG/01JrnKL7GcePHUCO/h0/UrdZfTtfQ7/SW
+l+st8p4x+OuAyWy9bIvZRnWXM8f44vpAbyuppF/g35sjgnWyMJCaxZgr74c
fTOQpC4Z/fCC+pY0Z4Pp+g7zR3AIbbIZsj3qavgO87Sj2j6/QruxeebrNNaF
x+wjT8+L9G15PmaqmfSkDOPwPLJQYD/zjS/EHL1L6WIUqU/Xv9+aqbQ/n6A1
/Hg6h+9jjUewTF+PnjsoNF+6bAhOyxN0BFarn0KrkIsb5phMn8MgY7n2MhOv
4h3MQ768zJxbsCKeO/1P/Yro4zOQHsI+VOAkarBBT7neSizBBqyM1yw/rS/N
/pvp1HiO9AoWGTuv5ir24RSOoDg+G7SvM/6TZtIB2Pu05xFpmSE00JcQhliD
f5SW8p14hm+jKRgsn8yPotX8yzSbn0hz+Pn0TX4Z3crPpbP4aTSdX0vf4zfT
E/zndCW/g87mD9MS/ntayN/P8GzxP9G/cB3D5VeMd6AZfWR36VvySf5PQSoS
yMB4+W5ahXLUoRar5EW0GHkowGuxJt4T5/kZXsEB7Mft+C65N9n+j6MJz26m
+7konrNsj3txw309gZMYw49VM5r2xnc8PjPqsvROwFxjw/UnjD2L0ZjIj6Fl
xjdjGIYiE53WX0ArzTdL7y46O35b9HTJ62k3XUc/jHhGKl3LNGt+QR/wl/R0
m6evd7kf7smSaAqSMcpYijlvynvRE99rPhkPO68kWug6Npr/ffv8AMvxrb5q
YwdxKGK9JeqO0o3m2GTdLXo+it8evla+3PhWWS5fJs+mU5GDImMl5m2wp8b4
DYzfWGs00Z16jsdroy2yH6xXhzPxLMx7Vc01NPCN6JU1Yo05W2iF3k+tt45f
G/dgnd/Vt+E33MQf2C6/RW/jb3TiH+yQ36F30Y1/4znGfcn/B128A7c=
          "]]},
        RowBox[{"-", "0.074`"}]],
       Annotation[#, -0.074, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1H1sTmcYx/GbmmGbFn10qizWjGqohD+Emo1IdBoSbEu2ZEtGohNChQjC
FonOTDAtfeJllrWmSklk1Fun42klmy4l9KFU4iUkXuoPggwLnxN/fH3P77qv
6z53zzmeATPmT5vXIYSwOPqnVwi93w0hA6N6hFAcC+GyWlHfEL7FO+rdkYpr
1q5jm/WnaSH8geOuk/rqeC0muv6JC3ix3h8zQ0iYbcApnEYj1qhnp6vZY53e
7T1DaDf3Qk4gqeciEhlm0Yi/1Tfo/U1vi7O2WU/yVY7brxyD7HlOX5m+Sn0P
7Pk718s7Ocd60npc3i1vNdNi75v2uIEtcrBni55yPVV6cs0MRqtau54HuIv7
uIdL6iX+1h8Qd7/p5sp5lTzN9UvrU/mC+7Tae5c9N0dZfaWes1wqbzJTYa2M
N8rN6t9bb+Kf5VL1C+Yvu+d5vsKbnXcTVuj7Dhv1rNfbaGaZXM/PcNK9/0KT
uX/M1an9KdfhePR81evVa9UPy7U4hIT6EfX96hm9fSuIYQomo0p9v759qMEB
569wnri5jtZT8J/z7NI3y/VC5DvjGbUxPBpvuR7LXflD7sbZ5j/qE0Km/cZy
X37DO+iEPHsMRTNS1Vv9nQvV52GJuZjaPnukcaqcywflYfyFnKK/KxZF35uZ
xyh2vphaFhrUczk7eqb6J3ANCnHDHvk8HBnWCp3tU4zHLHyFb9TX2bMcS/Wv
4uf6l0bZfXY4dzXKUI+DWG2m1OwvWIDVWI489QZz6dH3wJOc7Q63Ya61s3qu
ohoJ1KLEPZ5yJ+/hLvfnHtiZ5VtF+nueP4/BS+TJb/fz/9b1QwyQb3EX9FHP
lwfyAfljLpBH8CdyEc+QF3Fc/pI/kydzTF7J8+X1fELezcvlbfy5fIjXyKe4
WH7WP4Q35fPcjmt4X73N+h00I0XtCc9UH+l6AtKQi34Ypv4r16AUp3EUK9SX
cAmmYg6+jnqid+J5VmEcsryzAp6I8Z7x/9Hviuc+2Pedw0M4m3N4IA/iD6Jv
SV9n/ffkHd7XfS7ybmejIvo95Uou1DPOPUbpz8donFDbo38vHjnPdvzrm6nm
S/Y+hib5TPSbajZp5kn0O8iP+YX157id+fo3/RWtW+hP
          "]]},
        RowBox[{"-", "0.148`"}]],
       Annotation[#, -0.148, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1FlsjlkYwPFnzFiCRLW1jWVU7FK0F03UFumF0Ywl1kjsybhgopYLpSVm
iJDUUqGTMNwM0WgmMVrUUlX7EltLXBhEwgzBJDJG0lbM/N64+H//8zznOc85
73m/78tYVDB12RcRMc7H2a4RlzpGnOIzOI3iryN+6BSxlouwBiWpEaO6R4zG
CIxELspTIv5Mi3iB19as58d8TP1N697ydX7DN3ieNfVowOQuEfMxDyvFU50j
0iNq9KjSoxKL5QdYNxD9nK0/8tW1UjeBcztHjNG7g9oWaINV1jebf48C50uX
646L8oO4D9ZZm8cVyMczPXI5C53N5XeLmIZx+B5zsEC+RM89KFS/kZvUFyax
ffY7Szl24RwqsdmaUmt/wXJsRhEy5S9al+b5bifP4mwv+RGWmruj5g+U4wKO
Y5M9PvCX7uIV9+QU/Noj4iHSvnHHPBKfkClu1zNiq/E79BY/59boKp8r7sdH
xWN5vDibvxUv5oXiVVwmns3Txd9xungDLxNv4xrxYS4S7+UZ4ireIq7jAnFj
L+9MXM9v8BQZ8o/Mv8RttJD7lxfJ5xjnoQMGoQeGyh/gCpTiMqpRLL+aN2EK
lmBuUuO+fnOfD91tL++rvXFb5LnfLN+5TPnBPJSzuS9f9T5zvLfh4iHi/9zv
sKQGlcZV+Nt4v/f2lt/rVatno9omdLQ+BRnmp4tHqfnHOMs4OJsPmT+Ik3rV
O9cJPoVqfNDvK++2JVY4x3LUyqVyGtJxxHM1+x08kW/ip/yRc7gTv7bPeeOT
xkft/zuOWXNG/wb7neb7/LMzlOG48UG5E1ytbov1ZcnvVY/zetRhln1n4i/0
8bx16tbquwbbkv+H5Dfu2Up5F7ZjJ3agv/pr6u/r8wD1uKd3A1+Vv8d3k+88
3+G7fEW+2Nn2OccU59mp92TewUXyk4yb1UzkC85wy7q9akuSZ5cvVFPDjag1
fw5X1PzkPHvU/ci7Uz////0P2OXGiQ==
          "]]},
        RowBox[{"-", "0.22199999999999998`"}]],
       Annotation[#, -0.22199999999999998`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1FtMz2Ecx/GHqYtsDv9KKMe5cFFkaXI+FmZExQybcrjAKscyLtqkYmLZ
yma4wFab2VIUFzHMpkzJTZtsbrAaXZg5deHwai7e//fz+X6f5/k9/+dX/2m7
inOKhoUQJvr4PTaEyPgQYhGHA4kh7MeniSG8RCf+JITwF0H/XHwI1SiLDWGA
+7gfhfpFWI01yEImosaFcD6iZ24hTpm7zP7lvJwv6GVylbySK3kVHzH3jHGW
8Vk+Jh9FqT1LsNG+z509m9t5Ex9Xz+UOOYc3Iw+PxoQwLS6E075LBcpRhUrc
RK196zDoLG/kX9zLTdb9QLPnH3KOu9xt3itke8Zie29Rm+VeZiMVPXrH9FKM
kzFg3U85ekIIMfIIOQoLrV2EDXr11hSo7UI+9mA30p1j0JzX3OI83fxVPcP3
/MbzPPsgN3Ex3+Eue2XwA76PbeZWD70DpGEuVnnmevdxXf8J4sz/wPHcwS/Q
aPyRk7mdW9CKR9ZNd440+6baa6Tchkv2bOVJej3W3DaucXc7JoewD9koRwlO
JjkbP0MNbuEaUtQTeSZGYQXSUaD+bVIIw4w7uQ+9mKr+Tu0zXiNqinfHReqP
9avku5wnX+YTcj23ydVcKJdxrLyOc+WtXCcf5nx5L6+W53CWvIQb5Rk8X07g
aPk9pshf+Axi1JPl38YL0IGI3MM3MNr7TEK/v4vh/J3L3dc9foIGvEUX9rvb
XvfZhzXutpMj3vdTTtY7YU4FinEFNUNZvdn7eYiLaMBVlHpGqXWDQ/+LXOp9
1fI57LRmu7V7sAw5WIt49VTz56MZ86z57hwZvEL+pN9ufMWZRhgv1Yviq3JH
5P9vzD8K0KaK
          "]]},
        RowBox[{"-", "0.296`"}]],
       Annotation[#, -0.296, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl011MjmEcx/GrN9YBUxGKXpSxMdq827xuMkkjBw4cZOZAG8XIxoGphQOz
ystGnHRQzWya8jLLJjoQY7NiZMIcRJJMWbNRPs8cfPe9f9f1///v+7nu587c
XVpYEhVC+I2XKSEMJ4bQyUPcxeMTQvjC0UkhfOYY7uMobkQDfk0KoWlqCGU4
jEOo0Z8xOYR03LW/PjWESrVb7J3kAh5R89qsN5goT0AhitQO6+lX931KCAMY
U3uez2GuZ5qVHEIWxiFPz2k9TXo2uz6AUsTa28ZbccKsTbxG3WqcNSfbnPfu
/QE9GHWPj/yX33G9nmK1ozyoPoXbeYm5uXofRWZM8xyYhVT7Y1hmL9uMZr2z
I88ttzmHTntdKHd9xLNeMacB1biPm6hUWzU9hFqUoBJHMc/6Q30JZj7jjeb0
cjeK7T1X8xYNaEMLKtxjmKP8ls+cGjln1M0I4RUS0kPo4BX4g3ly/Exn6XoQ
afInjkOy9eVyFt+QV/EGOYdz5T1cJB/kC/IOLpTzOFE+zvvkM9wq1/NR+RJv
l5v5lPyA98sjad6f/IK/ogfp1rvt9+IZgrUh3mV9set1mIA5SMF861f5GqrQ
jjs4Zr2MK1CAvdgZqXFe153nK2d7m1tw0dm2cqb3GM8ZfI9jve9Y/6dBecx7
+8Fxcp3z7fCO+qz9jHxHkW8KGfaSzE3ntbySB9Qt1ZcT6VH/FE9Q656LzL/M
jWoeW1uoZgHyrUVbi0EUFplV439YjXwzy+Vv6m+Z288tXGrvdOL/7/sftUmX
sw==
          "]]},
        RowBox[{"-", "0.37`"}]],
       Annotation[#, -0.37, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl019ozXEYx/HnnKM2+6NtZ//tbChH0XLhwsSFC5S0sJULFxQXU5uNpfxr
aooLF5uiKMkuKClhUVqh3Lggm5qacjMaQ61MMTsNr5OLd+/zeb7f7/P9/Z5f
Z/mBnrbuRESMY1VFxK7KiJ1YqI74Jh/nBK/kLGrqImrz1EZsLI+o9jusp7kK
lWhIR8zXR+QwiY/4gBn7Es4lkcJAlb56DPKo9e/WX/Msj/EbFNVElKAYOzxX
KZZg2pnPuOp8oTt/cwHP8RnnLurRx4O8WH1evdidJfn71YaWRrSo9XqnHBfZ
U4xe9xzF/bKIZvd0WT+MTvSgG+32J+1N4a9+ifwM0Oy+Cs+Uzr+PO37aO4df
2KBnq3ON7m1CBu3mtIzP2tvpzCl5TD7N5c58xT1r03xD7yEk9UnhT/67cCCr
73XPWoDHKMQTjOCytWF+iHF97/CAd9vbGNGBVvTjGE42RDzi5xjAbVzDGvU6
zqIEm7EO+9VnM2aAl5jCBBrV31v/glGkmsyCu9SfWj8nP+A2+QqfkG/yiHyB
O+U+Lpe38255D1+Sj/A++SBvldfyFnkT35VX8Hq5ihfJk8jIM3weheqr5Zzf
LXiBMnmch1BqxvX4lP+m/IP7zWuYn+EW3uEVDpnthHlOYZvZvvUNs77XQvr/
f+kfKsF2Hw==
          "]]},
        RowBox[{"-", "0.44399999999999995`"}]],
       Annotation[#, -0.44399999999999995`, "Tooltip"]& ], {}, {}}}], 
   GraphicsComplexBox[CompressedData["
1:eJx0nXeAjNcX94fonQgSJXpvUaPEGaJFL0kIgkSJHiVqhFFDInr0srqIsFos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    "], {{{}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0slLFVAYhvGvTEpITYPEHNKtLaIMQiWXLswCdVVQQrsKbJkKBW0bIG1f
K6nbwuEOZSUoapiVei/4D5QNBlG20MBK+l1aPDzf8J7DuUP9xatdvTsiIoH1
4oiRsogfPMoHSiOe7404qK5CJS7bpTmDFDb132UmUKevx519EXfxxy5VHvGb
l0sixszGkcQjd2c4jRSy9mdlT8pe4jX9e/MPWM2fk//Mn/ARX+13Ii1bwH3O
ZWVy+Okt8zinXuYb8tcx620zGJAd0nc4u2F/ms/LvnTPGfUk9+gvYMX+ENeh
zPyefdZsE3PuasRx3HffKy6XGZTJ2dc6s0ef0Rdx6KdkdquvyfehH1XmzbzL
vIkLuYVb+Yiztzirz+G2+rD3F6mf2LWqK9RT7pjGO/0SEnYn+DEv8lsck5vn
o/yaG7mBh2Xe5H8j/RIW1A/yn0u9Hyv2zWZf1E28oG9Td3vLM7m/Zk95myf4
Fyft5uTGudO7RvOfQT/GV+yr5WpRgxe+qw6ZJN+UScic0rfjIb6Zb5X8/3/+
AySfaLk=
          "]]},
        "0.2`"],
       Annotation[#, 0.2, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl1GtszmccxvG7e4F1bWmRSMz5tGVxTKwIy5Q5M6cYQdJ2jkkRhxFGlBCM
aiU2lKQHQ2fiGCEIYom1zlQN2xBeIJIlW4K2Tvvc8eLb639dv+t3/5/n6f95
WmXPGT07IYQw1p9bySEUpYZQgmKk1Q+hLCmEuq7roQ7G6dzDgbQQ/qaPzfdg
CX+fn4wnOMQ/pk/N9mEZ/5TPRKpzsmga3Ydf0di9Durl6v1jVpMSwjP+EJqY
z5StaxDCD1iPY/LVus/ls9FCZw5talZLu9O5tJ18Hm3OB2em0/n8Z/IFcY9P
kPegF523UNbVbBGtI8+TnXevbrLFsny9AmzCFflEr3sSzuh+rXuajqSZsgt2
LqIHlttNp7n0rM6POn/Yz9K7Q7fw5+Qt8Ru28nflU8z/ooV8e/ka+z+59yi+
Rj6aTtMpNxvjuoKOpR3odPlDnSHuu8HeYJpHu5hd0ulMh8o2yrY684brSnSV
d0O+fBhfQIfTR85q78wO+MR8e3wGZLW4an4NGVirv8V5111/qrdD755OO3ut
+RN8G/oB34pu4/80b8tftnMFvbDSOT3pCtpMb7NelV41mvLH+Y9p63gOftdt
yJ+nFShHlfwmvndGS69pCcrt/4ez5on6vWUf0Wq+Bi8xgX8Tn1N60vwUVjkn
k09w32z61jy4fkebmH2JJPcpkV03X8YvRYrdZEyVP5DfdF2FTrql5jtRiI58
Zfw/IE+vi/4r+WvkowC35JPi5xCfWT7RTjKfgiTMlNenM+KzEp9LZ51GJT9R
vlv/mPw4OsV7orFsHC1GCWbp5+ConWyahZM+r3L7n7u+oFOBSj4nfte9/8M4
iH/js2I2Anvtr9Q5Yncwv58u58vk3+o11M+mjegvKMMgvX46/ZGBAfgqfm56
e+ydi++BDtT7mV7ld9Hx5h/a/4YmxvPcq198P/Q7nRKdDL4vvuCL+T60N3K9
p+H2bmNBnNl5JRsWf4foUFpLi+Tz4/eW3xm/07Ihrqvj/5O+cOZzlKa+/y39
H8lqwsc=
          "]]},
        "0.1`"],
       Annotation[#, 0.1, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV0TtMk1EYBuBjoosUWhw0JF7iZTK4WAcBkwItgmCAGI22KggScFEXXDVe
ING4aMREoGiMg5fBxEUxogu6OBjBxURQGBi8AV4Hxfj8w5vn/95zzt/Tdm37
8V3HFoUQXslQLIQviRCqC0P4zDS/8jXnmOEsazjPMX7ndn5jLX9wnL9Yx5/c
wd98wwU28i+b+I9vGSsOIcsC5ljIKcbZwiK2MsGPfLAshE/skHPe0SO9ct93
KGGnfjQewnN5Id26FTyqv8U/Re4gI56fym7PtfbsYV30HpnWr/N562WDrNSt
kpf6WZ/zTDZ7Tsoj5w5be8gOZnXLfdYm9zytK+UZXtX3yRJ7Fku5flA/4XlS
ks7krd+Wk7LF/EH/XvL2Vdgf1ydkytxpPiJdMmDeyn6WsdTZU/bFnC2QpXLR
2oLMufuo1Osaov9IcvqdTDHFSo45Py4N3vfYXM9hzvg9mnnHvrtyT7rtq9K9
s1YdfSeOsEVuOlfsPifsSZufmDOctKdCzlq74h19skY/bH01K631WEuxl1XR
f6HP8Lw5zQus4UZ9Ey+ZG3mZzUzq9/GaeS/7mWWZfj8HzDkO8gDLozszbz7I
IbZG99S38br5EG+wndv0/wFbhXJW
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV01tMz3EYx/EvYjNKSaxLzMYcNuQwc9yQCzalgzncZOXUGG1omklMuKLk
nE05TGUuYg5XRmykjUUpW4VCcs3Ultfv4t3n+/k8z/N9fvv9f03M2Zu+Z0gI
Ic6frtEhpCaEsAarcSY2hJlxITwdG8IM2q/eiUp+Fj9tTAhT0S3L0n9Rfya9
QBeqL0AL/xGX40OokGc4z5d/oM24JD8v3+CcIn/r7nm0XJYmK6Pp9JsdO+3q
oNf0NOqZjh2y1/S97A2dgldo4F/SXPUX9B3fQLfxOWh1T4Xsmey4HYvtWIol
aMQAej3bL/zECj15spt0Il9Nk8w+dMc4Ol+tSDaPnsBx1PMjMMy+rXoemXuM
JyiRxav10d/4jDvyGFmB8zDaTtvwHT2IlR2lxdjsvnX6++35h2TZIgzVUy2v
wkbPlo21erfy5+gc/iydTZtQ7pxCW9Q/ooyfyx+iaXwhTaeTZJOR5ZyNTOQh
F5vko+zdTfOxC33mHnieemQ5n6Zn0Ox8ipYi2UyC3rGIj94jEpGqJzPahUr3
X0fQ+8nMoN+tlbagC50YqXaT3tNXh1o0uWOvbA/yMWDuqvwKVql1eGc19Lkd
tbRUfhLbo28XJfJ6M2lqdXrHYwKSkKt+l/6m+8zsRwF69f/V3yc/Yr6OH2f3
V/4+/43+Uf9CD/N31ROj35JvwwHZDVkPqqNvDYPuXa9WTG/x7c4H9VXpGcKn
8YGm06G0Wh5DN0TfHc2gw6O75IfMfY7+R3HMfcv45dFOz7SCrvQsq1DFF/K3
zKzma2gR321uS/RuzN5254/oWfn/0ZCauw==
          "]]},
        RowBox[{"-", "0.1`"}]],
       Annotation[#, -0.1, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl00lsjGEcgPG3Jw50qgQdSsUutgtOdbScNA2ppVQkEiGdLtSNA6KlY2pJ
9OTkJMJBdZoy05CmOChJRWMJLhJL6W6pROv3xeHJ8/7X7/3yzSw4UFWayAkh
fERtfghDuSHU8CAneIAruRpVGJkaQjwvhEP8JBZChfN+3NK3TTwqf9v5jN70
lBBK5LJcL26RH1O/yw3iDvnt6juQltsstxVbsFMuMy2EpViGJViB5chiFVZj
JdZiDTpQaO6pvSk7YnbmRHcSd6NJLk/uhDsUcJt4Nv9Se4Er4rj4pPpCc53R
+4mniw9yPrdiNJrlGcjiZvQunMZZd2hAPXrt3GBHD1/WM8vcTHxV68MnfMFn
TDb7Qe8kfs/vcN05qXYBjciI59tTa8c8PspFXM5p36ANuXpOmY1xtfxc9Rou
5EqeE31HLuAEx7nL/R6amRCP45JzXfSdoxzK7b0oH+M8NMudFu9xbuJdnOLd
fByd9tXxc33PsNG5WP2td3iDYvFrfoXFnr8IFbiht0zfT/kf2Ce3Fz1yKbWX
znXR79L8mPgPkngk7sJv52PqJc5JvQ/co5HP4xy6o98GxvEXRd6xjO951n20
Y0Juvd5me9bxVR6WL+UhHsUINol7zV7zrHHPqYx28ne1fnzDIAYwbOdh9SPo
N5fh1qgXj53voCW6B9rRl/v/v/gP5qaNjQ==
          "]]},
        RowBox[{"-", "0.2`"}]],
       Annotation[#, -0.2, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl08tvjGEUgPG3d63OME1oIyWZou20jbCx9CeQtBtrLdPO0Km6FDNtXZuQ
uLNwa8JGMO4blMSU1G3NxtItLJAisfT7YvHkOee857yXbzLJDbnuwbIQwlsc
SIRwPxbCPr7H+7l5XghzGkLojYdQw31cyzEuWY9zfn4IBbypD2G1/gTa5Sm0
4Y69OvgJWnFbPmr2Fk9hER7hOpZav8HL+CVacE2+Xf8OdMk7cVi8yjlt7tKO
VhTcJcV57uQx7uBRLrPHGTMlPs3TfJYvcy9vRF9U94aM/sc8Yfav9R7ndKNJ
fIgnkNZTZ30Tz+URTnLJ3C/xN8T1xbDEXMbeWaQxgH4cwxC2IodtGEZS/wtv
PCU+iRXizXxVfZC34Lt9f6BZ7SiX9DzFT3HW2WvFleJ10ax8VvwbfzCMZ+rT
WODOTWjEQiSi34kbeFLfO/Gk+aK4FnV4L++3nkYNDsrXc7W1Hh6PvjeXoyJ6
h3qLe74SB+x0/xEU1ZbLv1o/Lj+B5/LF6nlxAbuwB7tRpX4XY+K9GEdK/tl5
X/ABWfOfeIA/coZn9Nz0jpXiIj+QHzH7kC/yJZzDBZxHo3qFvkqUoxpVqLdf
zh5D0bdz59fRb4014k50YVbPFF+J/f8//QNgqW0I
          "]]},
        RowBox[{"-", "0.30000000000000004`"}]],
       Annotation[#, -0.30000000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0klIVWEYxvG3RQNkTlQELSI0jcC0yCI01AbQJIVo0UpKXZdtrGzjxoZN
m6YrBdp0FcV05TKNrEiT2uVUlgNE0UIC2/Y7tPjznOf/fu93Loe7s/Hi6Qtr
IqIfA5sifmRF/MQvjOnF2RHjuREVycxzpdwmG3AvMyJtVsI9dL5YtnFPuR2e
r3A39Zf6Lbmbe8ENYl5/x3+Vefwd7i6m9HP8Z3levs+I2MMfwYOciBTu442d
T8l9znyUhfhiJ4Upv20aM3hkb5/Zcc8ncAz1ziy747LZpP4BE7jB73XfWv66
52eed9lt1bv0Ef0VRlHKD/EHZY++YL7BHQV6iu/EHNdo1oym5Dtxh5PvhG69
C7ftTDp3iWvBdvvT+luzGdnP9SGfH0++P2r4b35/tTyjtyfvdmY9hvFbvybb
kMZV7Lc/5+wsOs0Pub+F26qXy82yTG6Ro3LZzhIW0at/974FdJs/xhMcMCtF
rfv+yJOyim+Vf/VVrGDEfhF/lq/QX+sDciMy0MEfNT8l18k6Wc6XoUdP4zmW
vH8x5///9B/uJGXY
          "]]},
        RowBox[{"-", "0.4`"}]],
       Annotation[#, -0.4, "Tooltip"]& ], {}, {}}}], 
   {GrayLevel[0], AbsoluteThickness[3], LineBox[{{0, 0}, {1., 0}}]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.75, 
      FormBox[
       RowBox[{"-", "0.75`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.25, 
      FormBox[
       RowBox[{"-", "0.25`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.25, 
      FormBox["0.25`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.75, 
      FormBox["0.75`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}, {1.25, 
      FormBox["1.25`", TraditionalForm]}, {1.5, 
      FormBox["1.5`", TraditionalForm]}, {1.75, 
      FormBox["1.75`", TraditionalForm]}, {2., 
      FormBox["2.`", TraditionalForm]}}, {{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2, 
      FormBox["1.2`", TraditionalForm]}}, None, None},
  ImageMargins->0.,
  ImageSize->504,
  Method->{"TransparentPolygonMesh" -> True},
  PlotRange->{{-1.1920937271229854`, 
   2.1920937271229857`}, {-1.3920937271229854`, 1.3920937271229854`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 421},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->52203327]
}, Open  ]],

Cell["\<\
Arrows for velocity field are green, equipotential lines are red, streamlines \
are blue and panel is black.  Note orthogonality of equipotential lines and \
streamlines, and streamlines parallel to velocity field.\
\>", "ExampleText",
 CellID->680409942],

Cell["\<\
Potential-flow singularities may have discontinuities in velocity, velocity \
potential and stream function across the panel.  Such discontinuities are \
called jump conditions.\
\>", "ExampleText",
 CellID->62898067],

Cell["\<\
Compute the jump conditions for the tangent-component velocity across panel.\
\>", "ExampleText",
 CellID->673890120],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icuaMinus", ",", "icubMinus"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{"icua", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}], ",", 
     RowBox[{"Limit", "[", 
      RowBox[{"icub", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icuaPlus", ",", "icubPlus"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{"icua", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", 
        RowBox[{"-", "1"}]}]}], "]"}], ",", 
     RowBox[{"Limit", "[", 
      RowBox[{"icub", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", 
        RowBox[{"-", "1"}]}]}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icuaJump", ",", "icubJump"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"icuaPlus", "-", "icuaMinus"}], ",", 
     RowBox[{"icubPlus", "-", "icubMinus"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"uJump", "=", 
  RowBox[{
   RowBox[{"a", " ", "icuaJump"}], "+", 
   RowBox[{"b", " ", "icubJump"}]}]}]}], "Input",
 CellLabel->"In[13]:=",
 CellID->98488054],

Cell[BoxData[
 RowBox[{"-", "b"}]], "Output",
 ImageSize->{19, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[13]=",
 CellID->9683319]
}, Open  ]],

Cell["\<\
There is a jump in tangent-component velocity across the doublet panel equal \
to the negative of the slope of the doublet-density function. \
\>", "ExampleText",
 CellID->546465426],

Cell["\<\
Compute the jump conditions for the normal-component velocity across panel.\
\>", "ExampleText",
 CellID->134318740],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icwaMinus", ",", "icwbMinus"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{"icwa", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}], ",", 
     RowBox[{"Limit", "[", 
      RowBox[{"icwb", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icwaPlus", ",", "icwbPlus"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{"icwa", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", 
        RowBox[{"-", "1"}]}]}], "]"}], ",", 
     RowBox[{"Limit", "[", 
      RowBox[{"icwb", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", 
        RowBox[{"-", "1"}]}]}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icwaJump", ",", "icwbJump"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"icwaPlus", "-", "icwaMinus"}], ",", 
     RowBox[{"icwbPlus", "-", "icwbMinus"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"wJump", "=", 
  RowBox[{
   RowBox[{"a", " ", "icwaJump"}], "+", 
   RowBox[{"b", " ", "icwbJump"}]}]}]}], "Input",
 CellLabel->"In[14]:=",
 CellID->326115239],

Cell[BoxData["0"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[14]=",
 CellID->81489890]
}, Open  ]],

Cell["\<\
Normal-component velocity is continuous across the doublet panel.\
\>", "ExampleText",
 CellID->531709516],

Cell["\<\
Compute the jump conditions for the velocity potential across panel.\
\>", "ExampleText",
 CellID->211831706],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Phi]aJump", ",", "ic\[Phi]bJump"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Phi]a", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", 
         RowBox[{"-", "1"}]}]}], "]"}], "-", 
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Phi]a", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], ",", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Phi]b", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", 
         RowBox[{"-", "1"}]}]}], "]"}], "-", 
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Phi]b", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[Phi]Jump", "=", 
  RowBox[{
   RowBox[{"a", " ", "ic\[Phi]aJump"}], "+", 
   RowBox[{"b", " ", "ic\[Phi]bJump"}]}]}]}], "Input",
 CellLabel->"In[15]:=",
 CellID->189498322],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "a"}], "-", 
  RowBox[{"b", " ", "x"}]}]], "Output",
 ImageSize->{49, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[15]=",
 CellID->3889469]
}, Open  ]],

Cell["\<\
There is a jump in velocity potential across the panel that it is consistent \
with that of the tangent-component velocity.\
\>", "ExampleText",
 CellID->56731244],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{"\[Phi]Jump", ",", "x"}], "]"}]], "Input",
 CellLabel->"In[16]:=",
 CellID->114820653],

Cell[BoxData[
 RowBox[{"-", "b"}]], "Output",
 ImageSize->{19, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[16]=",
 CellID->654690]
}, Open  ]],

Cell["\<\
Compute the jump conditions for the stream function across the panel.\
\>", "ExampleText",
 CellID->403496840],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Psi]aJump", ",", "ic\[Psi]bJump"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Psi]a", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", 
         RowBox[{"-", "1"}]}]}], "]"}], "-", 
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Psi]a", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], ",", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Psi]b", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", 
         RowBox[{"-", "1"}]}]}], "]"}], "-", 
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Psi]b", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[Psi]Jump", "=", 
  RowBox[{
   RowBox[{"a", " ", "ic\[Psi]aJump"}], "+", 
   RowBox[{"b", " ", "ic\[Psi]bJump"}]}]}]}], "Input",
 CellLabel->"In[17]:=",
 CellID->102020424],

Cell[BoxData["0"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[17]=",
 CellID->498728711]
}, Open  ]],

Cell["Stream function is continuous across the panel.", "ExampleText",
 CellID->325279064]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Scope",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->1293636265],

Cell[TextData[{
 "The general application of influence coefficients is in the construction of \
a panel method.  The current version of ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " does not include any panel methods based on doublet panels of linearly \
varying strength.  See the ",
 ButtonBox["corresponding section",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletQuadratic#1293636265"],
 " of ICDoubletQuadratic for an example of a general application of influence \
coefficients for doublet panels of quadratically varying strength."
}], "ExampleText",
 CellID->246788676]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Properties & Relations",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->2123667759],

Cell["\<\
There is a relation between doublet density and circulation density for \
two-dimensional line singularities.\
\>", "ExampleText",
 CellID->342911820],

Cell[TextData[{
 "\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["\[Gamma]", "TR"], 
    RowBox[{"(", 
     StyleBox["x", "TI"], ")"}]}], "=", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"\[DifferentialD]", 
      StyleBox["\[Mu]", "TR"]}], 
     RowBox[{"\[DifferentialD]", 
      StyleBox["x", "TI"]}]]}]}]], "InlineFormula"],
 "\t\t\t\t"
}], "ExampleText",
 CellID->315517505],

Cell["\<\
Write the equivalent circulation density for a doublet panel of linearly \
varying strength.\
\>", "ExampleText",
 CellID->218711756],

Cell[TextData[{
 "\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["\[Gamma]", "TR"], 
    RowBox[{"(", 
     StyleBox["x", "TI"], ")"}]}], "=", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{
      FractionBox["\[DifferentialD]", 
       RowBox[{"\[DifferentialD]", 
        StyleBox["x", "TI"]}]], 
      RowBox[{"(", 
       RowBox[{
        StyleBox["a", "TI"], "+", 
        StyleBox["bx", "TI"]}], ")"}]}]}], "=", 
    RowBox[{"-", 
     StyleBox["b", "TI"]}]}]}]], "InlineFormula"],
 "\t\t\t\t"
}], "ExampleText",
 CellID->902684560],

Cell[TextData[{
 "A doublet panel of linearly varying strength is equivalent to the \
superposition of a constant-strength vortex panel of strength ",
 Cell[BoxData[
  RowBox[{"-", 
   StyleBox["b", "TI"]}]], "InlineFormula"],
 " and point vortices of strength ",
 Cell[BoxData[
  StyleBox[
   RowBox[{"-", "a"}], "TI"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  RowBox[{
   StyleBox["a", "TI"], "+", 
   RowBox[{
    StyleBox["b", "TI"], " ", 
    StyleBox["\[Delta]", "TR"]}]}]], "InlineFormula"],
 " at panel endpoints."
}], "ExampleText",
 CellID->26256628],

Cell["\<\
Compute the velocity field due to the constant-strength vortex panel and two \
point vortices at panel endpoints.\
\>", "ExampleText",
 CellID->612343172],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[CapitalGamma]", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "a"}], ",", 
     RowBox[{"a", "+", 
      RowBox[{"b", " ", "\[Delta]"}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"ICVortexPoint", "[", 
  RowBox[{"rF", ",", "rP"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"VVortexPoints", "=", 
   RowBox[{
    RowBox[{"ICVortexPoint", "[", 
     RowBox[{"rF", ",", "rP"}], "]"}], ".", "\[CapitalGamma]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"VVortexPanel", "=", 
   RowBox[{
    RowBox[{"-", "b"}], " ", 
    RowBox[{"ICVortexConstant", "[", 
     RowBox[{"rF", ",", "rP"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"VVortex", "=", 
   RowBox[{"VVortexPoints", "+", "VVortexPanel"}]}], ";"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->345799449],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     FractionBox["z", 
      RowBox[{"2", " ", "\[Pi]", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"]}], ")"}]}]], ",", 
     FractionBox["z", 
      RowBox[{"2", " ", "\[Pi]", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["z", "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}], ")"}]}]]}], "}"}], 
   "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["x", 
       RowBox[{"2", " ", "\[Pi]", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["z", "2"]}], ")"}]}]]}], ",", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{"x", "-", "\[Delta]"}], 
       RowBox[{"2", " ", "\[Pi]", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["z", "2"], "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}], ")"}]}]]}]}], 
    "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
    Rational[1, 2] 
     Pi^(-1) $CellContext`z/($CellContext`x^2 + $CellContext`z^2), 
     Rational[1, 2] 
     Pi^(-1) $CellContext`z/($CellContext`z^2 + ($CellContext`x - \
$CellContext`\[Delta])^2)}, {
    Rational[-1, 2] 
     Pi^(-1) $CellContext`x/($CellContext`x^2 + $CellContext`z^2), 
     Rational[-1, 2] 
     Pi^(-1) ($CellContext`z^2 + ($CellContext`x - \
$CellContext`\[Delta])^2)^(-1) ($CellContext`x - $CellContext`\[Delta])}], 
   "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{456, 39},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->12562379]
}, Open  ]],

Cell[TextData[{
 "The velocity components for a doublet panel of linearly varying strength \
were computed earlier and named ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{
    StyleBox["u", "TI"], ",", 
    StyleBox["w", "TI"]}], "}"}]], "InlineFormula"],
 ".  Test the two velocity fields for equality."
}], "ExampleText",
 CellID->373792664],

Cell[BoxData[
 RowBox[{
  RowBox[{"VDoublet", "=", 
   RowBox[{"{", 
    RowBox[{"u", ",", "w"}], "}"}]}], ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->252617228],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"VVortex", "\[Equal]", "VDoublet"}], ",", 
   RowBox[{"Assumptions", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Element", "[", 
       RowBox[{"x", ",", "Reals"}], "]"}], ",", " ", 
      RowBox[{"Element", "[", 
       RowBox[{"z", ",", "Reals"}], "]"}], ",", 
      RowBox[{"\[Delta]", ">", "0"}]}], "}"}]}]}], "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->549191169],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->455951779]
}, Open  ]],

Cell["\<\
Look again at the jump conditions for tangent-component velocity component \
across the doublet panel.  Note that the jump is equal to the value of the \
equivalent circulation density on the panel.\
\>", "ExampleText",
 CellID->156863266]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[{
 StyleBox[ButtonBox["ICDoubletPoint",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletPoint"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICDoubletConstant",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletConstant"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICDoubletQuadratic",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletQuadratic"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICSourceLinear",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICSourceLinear"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICVortexLinear",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICVortexLinear"],
  FontFamily->"Verdana"],
 " "
}], "SeeAlso",
 CellID->745892494]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["2.1 Introduction To Influence Coefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"]], \
"Tutorials",
 CellID->20842831],

Cell[TextData[ButtonBox["2.3 Isolated Panel",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/IsolatedPanel"]], "Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["InfluenceCoefficients",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/InfluenceCoefficients"]], "MoreAbout",
 CellID->364391176]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"ICDoubletLinear - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 9, 59, 28.1904000}", "context" -> 
    "Aerodynamics`InfluenceCoefficients`", "keywords" -> {}, "index" -> True, 
    "label" -> "Aerodynamics Package Paclet Symbol", "language" -> "en", 
    "paclet" -> "Aerodynamics Package", "status" -> "None", "summary" -> 
    "ICDoubletLinear[r_f, r_p] computes velocity influence coefficients at \
field points r_f due to connected doublet panels of linearly varying strength \
whose end points are located by r_p.ICPhiDoubletLinear[r_f, r_p] computes \
velocity-potential influence coefficients at field points r_f due to \
connected doublet panels of linearly varying strength whose end points are \
located by r_p.ICPsiDoubletLinear[r_f, r_p] computes stream-function \
influence coefficients at field points r_f due to connected doublet panels of \
linearly varying strength whose end points are located by r_p.", 
    "synonyms" -> {}, "title" -> "ICDoubletLinear", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/ICDoubletLinear"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[10761, 324, 459, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->7993419]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 275604, 5856}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[608, 21, 76, 1, 70, "SymbolColorBar"],
Cell[687, 24, 191, 5, 70, "LinkTrail"],
Cell[881, 31, 3674, 83, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4580, 118, 58, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4641, 121, 2321, 72, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6999, 198, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->438187347],
Cell[7582, 217, 364, 11, 70, "Notes",
 CellID->1067943069],
Cell[7949, 230, 971, 28, 70, "Notes",
 CellID->390622451],
Cell[8923, 260, 633, 16, 70, "Notes",
 CellID->433990591],
Cell[9559, 278, 1088, 38, 70, "Notes",
 CellID->213924258],
Cell[10650, 318, 74, 1, 70, "Notes",
 CellID->529540534]
}, Closed]],
Cell[CellGroupData[{
Cell[10761, 324, 459, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->7993419],
Cell[CellGroupData[{
Cell[11245, 341, 148, 5, 70, "ExampleSection",
 CellID->446874384],
Cell[11396, 348, 132, 6, 70, "ExampleText",
 CellID->513831186],
Cell[11531, 356, 99, 3, 70, "Input",
 CellID->66474307],
Cell[11633, 361, 429, 11, 70, "ExampleText",
 CellID->659456463],
Cell[12065, 374, 204, 6, 70, "ExampleText",
 CellID->42111717],
Cell[12272, 382, 495, 18, 70, "Input",
 CellID->64953589],
Cell[12770, 402, 357, 9, 70, "ExampleText",
 CellID->326501090],
Cell[13130, 413, 445, 15, 70, "Input",
 CellID->560964283],
Cell[13578, 430, 243, 7, 70, "ExampleText",
 CellID->120984796],
Cell[CellGroupData[{
Cell[13846, 441, 121, 4, 70, "Input",
 CellID->108853583],
Cell[13970, 447, 1394, 44, 62, "Output",
 CellID->132083116]
}, Open  ]],
Cell[15379, 494, 215, 7, 70, "ExampleText",
 CellID->217164756],
Cell[CellGroupData[{
Cell[15619, 505, 73, 2, 70, "Input",
 CellID->176773044],
Cell[15695, 509, 620, 20, 63, "Output",
 CellID->612390350]
}, Open  ]],
Cell[16330, 532, 661, 14, 70, "ExampleText",
 CellID->364097714],
Cell[CellGroupData[{
Cell[17016, 550, 437, 15, 70, "Input",
 CellID->25568170],
Cell[17456, 567, 1362, 43, 67, "Output",
 CellID->65959356]
}, Open  ]],
Cell[18833, 613, 142, 4, 70, "ExampleText",
 CellID->44302699],
Cell[CellGroupData[{
Cell[19000, 621, 754, 24, 70, "Input",
 CellID->575890672],
Cell[19757, 647, 900, 28, 67, "Output",
 CellID->79019023]
}, Open  ]],
Cell[20672, 678, 89, 1, 70, "ExampleText",
 CellID->22197235],
Cell[CellGroupData[{
Cell[20786, 683, 752, 24, 70, "Input",
 CellID->8664355],
Cell[21541, 709, 1171, 36, 67, "Output",
 CellID->30632950]
}, Open  ]],
Cell[22727, 748, 190, 4, 70, "ExampleText",
 CellID->9427774],
Cell[22920, 754, 750, 26, 70, "Input",
 CellID->258291294],
Cell[23673, 782, 947, 34, 70, "ExampleText",
 CellID->78226744],
Cell[CellGroupData[{
Cell[24645, 820, 340, 12, 70, "Input",
 CellID->13857323],
Cell[24988, 834, 208, 7, 36, "Output",
 CellID->4037215]
}, Open  ]],
Cell[CellGroupData[{
Cell[25233, 846, 364, 13, 70, "Input",
 CellID->499128776],
Cell[25600, 861, 210, 7, 36, "Output",
 CellID->266724237]
}, Open  ]],
Cell[25825, 871, 1134, 33, 70, "ExampleText",
 CellID->78531558],
Cell[CellGroupData[{
Cell[26984, 908, 4826, 138, 70, "Input",
 CellID->1746043248],
Cell[31813, 1048, 220035, 4020, 70, "Output",
 CellID->52203327]
}, Open  ]],
Cell[251863, 5071, 265, 5, 70, "ExampleText",
 CellID->680409942],
Cell[252131, 5078, 227, 5, 70, "ExampleText",
 CellID->62898067],
Cell[252361, 5085, 127, 3, 70, "ExampleText",
 CellID->673890120],
Cell[CellGroupData[{
Cell[252513, 5092, 2392, 71, 70, "Input",
 CellID->98488054],
Cell[254908, 5165, 174, 6, 36, "Output",
 CellID->9683319]
}, Open  ]],
Cell[255097, 5174, 192, 4, 70, "ExampleText",
 CellID->546465426],
Cell[255292, 5180, 126, 3, 70, "ExampleText",
 CellID->134318740],
Cell[CellGroupData[{
Cell[255443, 5187, 2393, 71, 70, "Input",
 CellID->326115239],
Cell[257839, 5260, 158, 5, 36, "Output",
 CellID->81489890]
}, Open  ]],
Cell[258012, 5268, 116, 3, 70, "ExampleText",
 CellID->531709516],
Cell[258131, 5273, 119, 3, 70, "ExampleText",
 CellID->211831706],
Cell[CellGroupData[{
Cell[258275, 5280, 2082, 57, 70, "Input",
 CellID->189498322],
Cell[260360, 5339, 220, 8, 36, "Output",
 CellID->3889469]
}, Open  ]],
Cell[260595, 5350, 173, 4, 70, "ExampleText",
 CellID->56731244],
Cell[CellGroupData[{
Cell[260793, 5358, 130, 4, 70, "Input",
 CellID->114820653],
Cell[260926, 5364, 173, 6, 36, "Output",
 CellID->654690]
}, Open  ]],
Cell[261114, 5373, 120, 3, 70, "ExampleText",
 CellID->403496840],
Cell[CellGroupData[{
Cell[261259, 5380, 2082, 57, 70, "Input",
 CellID->102020424],
Cell[263344, 5439, 159, 5, 36, "Output",
 CellID->498728711]
}, Open  ]],
Cell[263518, 5447, 90, 1, 70, "ExampleText",
 CellID->325279064]
}, Open  ]],
Cell[CellGroupData[{
Cell[263645, 5453, 221, 7, 70, "ExampleSection",
 CellID->1293636265],
Cell[263869, 5462, 610, 13, 70, "ExampleText",
 CellID->246788676]
}, Closed]],
Cell[CellGroupData[{
Cell[264516, 5480, 238, 7, 70, "ExampleSection",
 CellID->2123667759],
Cell[264757, 5489, 160, 4, 70, "ExampleText",
 CellID->342911820],
Cell[264920, 5495, 398, 16, 70, "ExampleText",
 CellID->315517505],
Cell[265321, 5513, 143, 4, 70, "ExampleText",
 CellID->218711756],
Cell[265467, 5519, 550, 22, 70, "ExampleText",
 CellID->902684560],
Cell[266020, 5543, 565, 19, 70, "ExampleText",
 CellID->26256628],
Cell[266588, 5564, 164, 4, 70, "ExampleText",
 CellID->612343172],
Cell[CellGroupData[{
Cell[266777, 5572, 859, 27, 70, "Input",
 CellID->345799449],
Cell[267639, 5601, 1830, 58, 60, "Output",
 CellID->12562379]
}, Open  ]],
Cell[269484, 5662, 342, 10, 70, "ExampleText",
 CellID->373792664],
Cell[269829, 5674, 166, 6, 70, "Input",
 CellID->252617228],
Cell[CellGroupData[{
Cell[270020, 5684, 451, 13, 70, "Input",
 CellID->549191169],
Cell[270474, 5699, 161, 5, 36, "Output",
 CellID->455951779]
}, Open  ]],
Cell[270650, 5707, 249, 5, 70, "ExampleText",
 CellID->156863266]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[270948, 5718, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[271368, 5732, 811, 27, 70, "SeeAlso",
 CellID->745892494]
}, Open  ]],
Cell[CellGroupData[{
Cell[272216, 5764, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[272638, 5778, 209, 5, 70, "Tutorials",
 CellID->20842831],
Cell[272850, 5785, 159, 3, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[CellGroupData[{
Cell[273046, 5793, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[273468, 5807, 167, 3, 70, "MoreAbout",
 CellID->364391176]
}, Open  ]],
Cell[273650, 5813, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

