(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    187039,       3819]
NotebookOptionsPosition[    181627,       3649]
NotebookOutlinePosition[    183748,       3702]
CellTagsIndexPosition[    183661,       3697]
WindowTitle->ICDoubletPoint - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"2.1 Introduction to Influence Coefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"], "\
\<\"2.2 Point Singularities\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/PointSingularities"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"ICDoubletConstant\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICDoubletConstant"], \
"\<\"ICDoubletLinear\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICDoubletLinear"], \
"\<\"ICDoubletQuadratic\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICDoubletQuadratic"], "\<\"ICVortexPoint\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICVortexPoint"], "\<\"ICSourcePoint\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/ref/ICSourcePoint"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"InfluenceCoefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/InfluenceCoefficients"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/ICDoubletPoint\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/ICDoubletPoint"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
ICDoubletPoint.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/ICDoubletPoint", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["ICDoubletPoint", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICDoubletPoint", "[", 
       RowBox[{
        SubscriptBox[
         StyleBox["r", "TI"], 
         StyleBox["f", "TI"]], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["u", "d"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes velocity influence coefficients at field \
points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     ", due to doublets at ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     ". Orientation of doublets is specified by unit vectors ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["u", "TI"], 
       StyleBox["d", "TI"]]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICPhiDoubletPoint", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["u", "d"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes velocity-potential influence coefficients at \
field points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     ", due to doublets at ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     ".  Orientation of doublets is specified by unit vectors ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["u", "TI"], 
       StyleBox["d", "TI"]]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICPsiDoubletPoint", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["u", "d"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]",
     "computes stream-function influence coefficients at field points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     ", due to doublets at ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     ".  Orientation of doublets is specified by unit vectors ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["u", "TI"], 
       StyleBox["D", "TI"]]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[""]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, 
   "Rows" -> {None, None, None, {None}}, "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->592567457],

Cell[TextData[{
 "Input arguments are collections of vectors of the data type ",
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], "InlineFormula"],
 ".  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation may be used for single vectors."
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 "The strength of a doublet point is denoted by ",
 Cell[BoxData[
  StyleBox["\[Mu]", "TR"]], "InlineFormula"],
 ".   For a collection of doublets, strength parameters are represented as a \
list. "
}], "Notes",
 CellID->492519275],

Cell[TextData[{
 "Output for velocity influence coefficients is a collection of vectors of \
the data type CartesianVectors.  Influence coefficients for velocity \
potential and stream function are lists.  For the special case of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation being used to specify a single field point, ",
 "output is simplified as illustrated in the examples",
 "."
}], "Notes",
 CellID->30234373],

Cell[TextData[{
 "The velocity, velocity potential and stream function for a point doublet \
are written as products of a geometric influence coefficient and the strength \
parameter.\t\t\t\t\t\t\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["V", "TI"], "=", 
   RowBox[{"\[Mu]", " ", 
    StyleBox["ic", "TI"]}]}]], "InlineFormula"],
 "\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Phi]", "TR"], "=", 
   RowBox[{"\[Mu]", " ", 
    StyleBox["ic\[Phi]", "TI"]}]}]], "InlineFormula"],
 "\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Psi]", "TR"], "=", 
   RowBox[{"\[Mu]", " ", 
    StyleBox["ic\[Psi]", "TI"]}]}]], "InlineFormula"]
}], "Notes",
 CellID->35104674],

Cell["New in 1.0   |   Last modified in 1.2", "Notes",
 CellID->529540534]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->281944934],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->317889308],

Cell[TextData[{
 "Load ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->513831186],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->267685788],

Cell[TextData[{
 "This example demonstrates some properties of a single point doublet located \
at the origin and oriented in the direction of the positive ",
 Cell[BoxData[
  StyleBox["z", "TI"]], "InlineFormula"],
 "-axis.  Specify an arbitrary field point, the location of the doublet and \
the orientation of the doublet.  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation is used for single vectors."
}], "ExampleText",
 CellID->659456463],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"rF", ",", "rP", ",", "uvD"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"x", ",", "z"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->64953589],

Cell["Compute velocity influence coefficient.", "ExampleText",
 CellID->326501090],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"icu", ",", "icw"}], "}"}], "=", 
  RowBox[{"ICDoubletPoint", "[", 
   RowBox[{"rF", ",", "rP", ",", "uvD"}], "]"}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->223030288],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"x", " ", "z"}], 
    RowBox[{"\[Pi]", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"]}], ")"}], "2"]}]], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["x", "2"], "-", 
      SuperscriptBox["z", "2"]}], 
     RowBox[{"2", " ", "\[Pi]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"]}], ")"}], "2"]}]]}]}], "}"}]], "Output",
 ImageSize->{194, 41},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->88035154]
}, Open  ]],

Cell["Compute velocity-potential influence coefficient.", "ExampleText",
 CellID->44302699],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ic\[Phi]", "=", 
  RowBox[{"ICPhiDoubletPoint", "[", 
   RowBox[{"rF", ",", "rP", ",", "uvD"}], "]"}]}]], "Input",
 CellLabel->"In[4]:=",
 CellID->575890672],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox["z", 
   RowBox[{"2", " ", "\[Pi]", " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["x", "2"], "+", 
      SuperscriptBox["z", "2"]}], ")"}]}]]}]], "Output",
 ImageSize->{90, 35},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->106116437]
}, Open  ]],

Cell["Compute stream-function influence coefficient.", "ExampleText",
 CellID->472419597],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ic\[Psi]", "=", 
  RowBox[{"ICPsiDoubletPoint", "[", 
   RowBox[{"rF", ",", "rP", ",", "uvD"}], "]"}]}]], "Input",
 CellLabel->"In[5]:=",
 CellID->8664355],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox["x", 
   RowBox[{"2", " ", "\[Pi]", " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["x", "2"], "+", 
      SuperscriptBox["z", "2"]}], ")"}]}]]}]], "Output",
 ImageSize->{90, 35},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->134063245]
}, Open  ]],

Cell["\<\
Write the velocity components, velocity potential and stream function as the \
product of the strength parameter and the geometric influence coefficients.\
\>", "ExampleText",
 CellID->9427774],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"u", ",", "w", ",", "\[Phi]", ",", "\[Psi]"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Mu]", " ", "icu"}], ",", 
     RowBox[{"\[Mu]", " ", "icw"}], ",", 
     RowBox[{"\[Mu]", " ", "ic\[Phi]"}], ",", 
     RowBox[{"\[Mu]", " ", "ic\[Psi]"}]}], "}"}]}], ";"}]], "Input",
 CellLabel->"In[6]:=",
 CellID->136424127],

Cell[TextData[{
 "Verify that the relations between velocity, velocity potential and stream \
function are satisfied.\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["u", "TI"], "=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Phi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["x", "TI"]}]], "=", 
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Psi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["z", "TI"]}]]}]}]], "InlineFormula"],
 "\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["w", "TI"], "=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Phi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["z", "TI"]}]], "=", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{"\[PartialD]", 
       StyleBox["\[Psi]", "TR"]}], 
      RowBox[{"\[PartialD]", 
       StyleBox["x", "TI"]}]]}]}]}]], "InlineFormula"],
 "\t\t\t\t\t"
}], "ExampleText",
 CellID->78226744],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"u", "\[Equal]", 
    RowBox[{"D", "[", 
     RowBox[{"\[Phi]", ",", "x"}], "]"}]}], ",", 
   RowBox[{"u", "\[Equal]", 
    RowBox[{"D", "[", 
     RowBox[{"\[Psi]", ",", "z"}], "]"}]}]}], "}"}]], "Input",
 CellLabel->"In[7]:=",
 CellID->13857323],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->612803658]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"w", "\[Equal]", 
     RowBox[{"D", "[", 
      RowBox[{"\[Phi]", ",", "z"}], "]"}]}], ",", 
    RowBox[{"w", "\[Equal]", 
     RowBox[{"-", 
      RowBox[{"D", "[", 
       RowBox[{"\[Psi]", ",", "x"}], "]"}]}]}]}], "}"}], "//", 
  "Simplify"}]], "Input",
 CellLabel->"In[8]:=",
 CellID->692553449],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->216960717]
}, Open  ]],

Cell[TextData[{
 "Flow visualization is shown below for a point doublet of unit strength \
located at the origin and oriented along the positive ",
 Cell[BoxData[
  StyleBox["z", "TI"]], "InlineFormula"],
 "-axis.  A circular region near the origin is excluded from the vector plot \
to avoid showing vectors of large magnitude.  Also, nonlinear scaling is used \
for magnitudes of vectors to enhance visibility in the far field."
}], "ExampleText",
 CellID->78531558],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xMin", ",", "xMax", ",", "zMin", ",", "zMax"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "1.5"}], ",", "1.5", ",", 
     RowBox[{"-", "1.2"}], ",", "1.2"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"uPlot", ",", "wPlot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"u", ",", "w"}], "}"}], "/.", 
    RowBox[{"\[Mu]", "\[Rule]", "1"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Phi]Plot", ",", "\[Psi]Plot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\[Phi]", ",", "\[Psi]"}], "}"}], "/.", 
    RowBox[{"\[Mu]", "\[Rule]", "1"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gV", "=", 
   RowBox[{"VectorPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"uPlot", ",", "wPlot"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"RegionFunction", "\[Rule]", 
      RowBox[{"Function", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"x", ",", "z"}], "}"}], ",", 
        RowBox[{"0.025", "<", 
         RowBox[{
          RowBox[{"x", "^", "2"}], "+", 
          RowBox[{"z", "^", "2"}]}]}]}], "]"}]}], ",", 
     RowBox[{"VectorColorFunction", "->", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"RGBColor", "[", 
         RowBox[{"0", ",", "1", ",", "0"}], "]"}], "&"}], ")"}]}], ",", 
     RowBox[{"VectorPoints", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"14", ",", "14"}], "}"}]}], ",", 
     RowBox[{"VectorScale", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Large", ",", "Automatic", ",", 
        RowBox[{
         RowBox[{"Sqrt", "[", "#5", "]"}], "&"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Phi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Phi]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "13"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Red"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Psi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "7"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Blue"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gV", ",", "g\[Phi]", ",", "g\[Psi]", ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
   RowBox[{"FrameTicks", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.5"}], ",", "1.5", ",", "0.25"}], "]"}], "]"}], ",", 
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.2"}], ",", "1.2", ",", "0.2"}], "]"}], "]"}], ",", 
      "None", ",", "None"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", " ", "72"}]}], ",", 
   RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellLabel->"In[9]:=",
 CellID->1746043248],

Cell[BoxData[
 GraphicsBox[{{{}, {
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.004366318576526445, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.5144160395589483`, -1.1967563910992367`}, \
{-1.485583960441052, -1.2032436089007634`}}]}}, 
      {Arrowheads[{{0.004630470224779108, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.5145487046362593`, -1.00956259005028}, \
{-1.4854512953637407`, -1.0212066407189506`}}]}}, 
      {Arrowheads[{{0.0048914973968353604`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.5140321540472603`, -0.8219871411956355}, \
{-1.4859678459527397`, -0.8395513203428259}}]}}, 
      {Arrowheads[{{0.005135410733324209, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.5126293194204556`, -0.6342149757896187}, \
{-1.4873706805795446`, -0.6580927165180734}}]}}, 
      {Arrowheads[{{0.005344349256071994, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.5101674221035934`, -0.44658061940529026`}, \
{-1.4898325778964066`, -0.47649630367163265`}}]}}, 
      {Arrowheads[{{0.005498695594094025, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.5066444133588992`, -0.25954112120662304`}, \
{-1.4933555866411006`, -0.2943050326395306}}]}}, 
      {Arrowheads[{{0.0055810583626771425`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.502315826644669, -0.0735628570242084}, \
{-1.4976841733553312`, -0.11105252759117609`}}]}}, 
      {Arrowheads[{{0.005581058362677139, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.497684173355331, 
         0.1110525275911762}, {-1.5023158266446688`, 0.07356285702420853}}]}}, 
      {Arrowheads[{{0.005498695594094025, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.4933555866411008`, 
         0.29430503263953073`}, {-1.5066444133588994`, 
         0.25954112120662315`}}]}}, 
      {Arrowheads[{{0.005344349256071987, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.4898325778964066`, 
         0.47649630367163276`}, {-1.5101674221035934`, 
         0.4465806194052904}}]}}, 
      {Arrowheads[{{0.005135410733324209, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.4873706805795444`, 
         0.6580927165180736}, {-1.5126293194204554`, 0.6342149757896188}}]}}, 
      {Arrowheads[{{0.004891497396835368, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.4859678459527397`, 
         0.839551320342826}, {-1.5140321540472603`, 0.8219871411956355}}]}}, 
      {Arrowheads[{{0.004630470224779108, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.4854512953637407`, 
         1.0212066407189506`}, {-1.5145487046362593`, 1.00956259005028}}]}}, 
      {Arrowheads[{{0.004366318576526445, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.4855839604410517`, 
         1.2032436089007632`}, {-1.514416039558948, 1.1967563910992365`}}]}}, 
      {Arrowheads[{{0.004801879850049445, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2854557020356758`, -1.1990894749247596`}, \
{-1.2530058364258627`, -1.2009105250752403`}}]}}, 
      {Arrowheads[{{0.005160194681349427, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2862679068913443`, -1.011551259410986}, \
{-1.252193631570194, -1.0192179713582448`}}]}}, 
      {Arrowheads[{{0.0055291524839288205`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.286379180787388, -0.8232819571375655}, \
{-1.2520823576741502`, -0.8382565044008958}}]}}, 
      {Arrowheads[{{0.005889049624893626, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2853461053826807`, -0.6344283694277313}, \
{-1.2531154330788579`, -0.6578793228799609}}]}}, 
      {Arrowheads[{{0.006210412001103272, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2827308674601505`, -0.4454303897874951}, \
{-1.2557306710013878`, -0.4776465332894278}}]}}, 
      {Arrowheads[{{0.00645638739139603, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2783319069009562`, -0.25705915447776695`}, \
{-1.260129631560582, -0.2967869993683867}}]}}, 
      {Arrowheads[{{0.006590866124748174, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2724580233047167`, -0.07023767524290088}, \
{-1.2660035151568216`, -0.11437770937248358`}}]}}, 
      {Arrowheads[{{0.006590866124748174, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2660035151568216`, 
         0.11437770937248372`}, {-1.2724580233047167`, 
         0.07023767524290102}}]}}, 
      {Arrowheads[{{0.00645638739139603, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.260129631560582, 
         0.2967869993683868}, {-1.2783319069009562`, 
         0.25705915447776706`}}]}}, 
      {Arrowheads[{{0.006210412001103272, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2557306710013878`, 
         0.4776465332894279}, {-1.2827308674601505`, 0.4454303897874952}}]}}, 
      {Arrowheads[{{0.005889049624893626, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2531154330788576`, 
         0.657879322879961}, {-1.2853461053826805`, 0.6344283694277314}}]}}, 
      {Arrowheads[{{0.0055291524839288205`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2520823576741502`, 
         0.838256504400896}, {-1.286379180787388, 0.8232819571375657}}]}}, 
      {Arrowheads[{{0.005160194681349427, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.252193631570194, 
         1.0192179713582448`}, {-1.2862679068913443`, 1.011551259410986}}]}}, 
      {Arrowheads[{{0.004801879850049445, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2530058364258625`, 
         1.2009105250752403`}, {-1.2854557020356756`, 
         1.1990894749247596`}}]}}, 
      {Arrowheads[{{0.00528525618723926, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.056162505505021, -1.202568153124471}, \
{-1.0207605714180559`, -1.1974318468755287`}}]}}, 
      {Arrowheads[{{0.005774950673955711, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0580001200959672`, -1.0149454906963669`}, \
{-1.0189229568271092`, -1.015823740072864}}]}}, 
      {Arrowheads[{{0.006306904610538194, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0592847067133524`, -0.8260840179125726}, \
{-1.0176383702097243`, -0.8354544436258889}}]}}, 
      {Arrowheads[{{0.006857642339136383, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0592814098990337`, -0.6359008856562541}, \
{-1.017641667024043, -0.656406806651438}}]}}, 
      {Arrowheads[{{0.0073806537019540635`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0570015500871535`, -0.4448009510431145}, \
{-1.0199215268359232`, -0.4782759720338084}}]}}, 
      {Arrowheads[{{0.007804065832493457, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0516119558967751`, -0.2540194332233726}, \
{-1.0253111210263015`, -0.299826720622781}}]}}, 
      {Arrowheads[{{0.008045058342766335, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0432637701050542`, -0.0655085718304047}, \
{-1.0336593068180224`, -0.11910681278497975`}}]}}, 
      {Arrowheads[{{0.008045058342766335, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0336593068180224`, 
         0.11910681278497989`}, {-1.0432637701050542`, 
         0.06550857183040484}}]}}, 
      {Arrowheads[{{0.007804065832493464, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0253111210263015`, 
         0.29982672062278115`}, {-1.0516119558967751`, 
         0.2540194332233727}}]}}, 
      {Arrowheads[{{0.0073806537019540635`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0199215268359232`, 
         0.4782759720338085}, {-1.0570015500871535`, 0.4448009510431146}}]}}, 
      {Arrowheads[{{0.006857642339136354, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.017641667024043, 
         0.6564068066514381}, {-1.0592814098990335`, 0.6359008856562542}}]}}, 
      {Arrowheads[{{0.006306904610538194, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0176383702097243`, 
         0.8354544436258889}, {-1.0592847067133524`, 0.8260840179125726}}]}}, 
      {Arrowheads[{{0.005774950673955711, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0189229568271094`, 
         1.0158237400728638`}, {-1.0580001200959674`, 
         1.0149454906963666`}}]}}, 
      {Arrowheads[{{0.00528525618723926, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0207605714180557`, 
         1.197431846875529}, {-1.0561625055050208`, 1.2025681531244712`}}]}}, 
      {Arrowheads[{{0.0057984202171441285`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8258718712122951, -1.2073866962763906`}, \
{-0.7895127441723198, -1.1926133037236095`}}]}}, 
      {Arrowheads[{{0.0064645522854474505`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8290089640556493, -1.0203054409282308`}, \
{-0.7863756513289657, -1.0104637898409996`}}]}}, 
      {Arrowheads[{{0.007238761659638784, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8321799167799403, -0.8314591594379696}, \
{-0.7832046986046747, -0.8300793021004919}}]}}, 
      {Arrowheads[{{0.008108877356406254, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8344649525874971, -0.6401300010524285}, \
{-0.780919662797118, -0.6521776912552637}}]}}, 
      {Arrowheads[{{0.009016209982890475, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8339800385184757, -0.4460474773016123}, \
{-0.7814045768661393, -0.47702944577531053`}}]}}, 
      {Arrowheads[{{0.009823109296600746, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8280899149416313, -0.25067329902245894`}, \
{-0.7872947004429837, -0.3031728548236947}}]}}, 
      {Arrowheads[{{0.01031727032086186, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8155701171193332, -0.058292436603141595`}, \
{-0.7998144982652817, -0.12632294801224286`}}]}}, 
      {Arrowheads[{{0.010317270320861861`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7998144982652817, 
         0.126322948012243}, {-0.8155701171193334, 0.05829243660314174}}]}}, 
      {Arrowheads[{{0.009823109296600746, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7872947004429837, 
         0.3031728548236948}, {-0.8280899149416313, 0.25067329902245905`}}]}}, 
      {Arrowheads[{{0.009016209982890475, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7814045768661393, 
         0.4770294457753107}, {-0.8339800385184757, 0.4460474773016125}}]}}, 
      {Arrowheads[{{0.008108877356406254, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7809196627971179, 
         0.6521776912552638}, {-0.834464952587497, 0.6401300010524286}}]}}, 
      {Arrowheads[{{0.007238761659638784, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7832046986046747, 
         0.830079302100492}, {-0.8321799167799403, 0.8314591594379697}}]}}, 
      {Arrowheads[{{0.0064645522854474505`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7863756513289657, 
         1.0104637898409998`}, {-0.8290089640556493, 1.020305440928231}}]}}, 
      {Arrowheads[{{0.0057984202171441285`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7895127441723199, 
         1.1926133037236093`}, {-0.8258718712122952, 1.2073866962763904`}}]}}, 
      {Arrowheads[{{0.0062993219151477094`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5935728288330493, -1.2133133977772435`}, \
{-0.560273325013104, -1.1866866022227565`}}]}}, 
      {Arrowheads[{{0.007182008438656124, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5978022562591387, -1.027826708161696}, \
{-0.5560438975870144, -1.0029425226075346`}}]}}, 
      {Arrowheads[{{0.008292529360913821, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.603218887901165, -0.840571724750507}, \
{-0.5506272659449883, -0.8209667367879545}}]}}, 
      {Arrowheads[{{0.009682671628867115, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6094817470366278, -0.649851580831028}, \
{-0.5443644068095254, -0.6424561114766643}}]}}, 
      {Arrowheads[{{0.011352428298968771`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.614404779776342, -0.45310507839647673`}, \
{-0.5394413740698112, -0.46997184468044617`}}]}}, 
      {Arrowheads[{{0.013106524099836455`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6115303079457859, -0.24917961338653827`}, \
{-0.5423158459003674, -0.3046665404596154}}]}}, 
      {Arrowheads[{{0.014355611131809004`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.592081291927851, -0.04615092761815429}, \
{-0.5617648619183022, -0.13846445699723015`}}]}}, 
      {Arrowheads[{{0.014355611131809009`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5617648619183022, 
         0.1384644569972303}, {-0.592081291927851, 0.04615092761815443}}]}}, 
      {Arrowheads[{{0.013106524099836455`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5423158459003674, 
         0.3046665404596155}, {-0.6115303079457859, 0.24917961338653838`}}]}}, 
      {Arrowheads[{{0.01135242829896877, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5394413740698113, 
         0.4699718446804463}, {-0.6144047797763421, 0.4531050783964769}}]}}, 
      {Arrowheads[{{0.009682671628867117, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5443644068095255, 
         0.6424561114766643}, {-0.6094817470366279, 0.6498515808310281}}]}}, 
      {Arrowheads[{{0.008292529360913821, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5506272659449883, 
         0.8209667367879545}, {-0.603218887901165, 0.840571724750507}}]}}, 
      {Arrowheads[{{0.007182008438656107, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5560438975870146, 
         1.0029425226075346`}, {-0.5978022562591389, 1.0278267081616959`}}]}}, 
      {Arrowheads[{{0.0062993219151477094`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.560273325013104, 
         1.1866866022227565`}, {-0.5935728288330493, 1.2133133977772435`}}]}}, 
      {Arrowheads[{{0.006715695780463971, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.35825845199699197`, -1.2192354601827944`}, \
{-0.33404924031069977`, -1.1807645398172057`}}]}}, 
      {Arrowheads[{{0.007818497774238311, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3623159127636506, -1.036334082028082}, \
{-0.32999177954404113`, -0.9944351487411487}}]}}, 
      {Arrowheads[{{0.00931936010190327, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.36854802002730963`, -0.8529767865270823}, \
{-0.3237596722803821, -0.8085616750113791}}]}}, 
      {Arrowheads[{{0.011442083616757567`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.37839032870494793`, -0.6676064577563057}, \
{-0.31391736360274375`, -0.6247012345513864}}]}}, 
      {Arrowheads[{{0.014538201758969095`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3933858461538458, -0.47531446153846146`}, \
{-0.298921846153846, -0.44776246153846144`}}]}}, 
      {Arrowheads[{{0.018920713831614627`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.40862335090928825`, -0.26286743835310233`}, \
{-0.2836843413984035, -0.2909787154930513}}]}}, 
      {Arrowheads[{{0.023412197497480403`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3856050984595565, -0.023596761208579606`}, \
{-0.3067025938481353, -0.16101862340680484`}}]}}, 
      {Arrowheads[{{0.0234121974974804, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.30670259384813525`, 
         0.16101862340680495`}, {-0.3856050984595565, 
         0.023596761208579772`}}]}}, 
      {Arrowheads[{{0.018920713831614627`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.2836843413984035, 
         0.29097871549305143`}, {-0.40862335090928825`, 
         0.26286743835310244`}}]}}, 
      {Arrowheads[{{0.014538201758969095`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.298921846153846, 
         0.44776246153846155`}, {-0.3933858461538458, 0.4753144615384616}}]}}, 
      {Arrowheads[{{0.011442083616757559`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3139173636027438, 
         0.6247012345513866}, {-0.37839032870494793`, 0.6676064577563059}}]}}, 
      {Arrowheads[{{0.00931936010190327, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3237596722803821, 
         0.8085616750113792}, {-0.36854802002730963`, 0.8529767865270824}}]}}, 
      {Arrowheads[{{0.007818497774238311, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.32999177954404113`, 
         0.9944351487411487}, {-0.3623159127636506, 1.036334082028082}}]}}, 
      {Arrowheads[{{0.006715695780463971, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.33404924031069977`, 
         1.1807645398172055`}, {-0.35825845199699197`, 
         1.2192354601827942`}}]}}, 
      {Arrowheads[{{0.006957431393153585, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.11987106868543349`, -1.223113862294024}, \
{-0.11089816208379669`, -1.1768861377059758`}}]}}, 
      {Arrowheads[{{0.008207519126650457, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.12161681560588662`, -1.0424521940729106`}, \
{-0.10915241516334355`, -0.98831703669632}}]}}, 
      {Arrowheads[{{0.009999983686220891, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.12460721565720123`, -0.863330133398278}, \
{-0.10616201511202894`, -0.7982083281401835}}]}}, 
      {Arrowheads[{{0.012778398848067047`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.1303518084362273, -0.686725630175895}, \
{-0.10041742233300287`, -0.6055820621317972}}]}}, 
      {Arrowheads[{{0.017630159252579426`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.1434616806829387, -0.5141829589728184}, \
{-0.08730755008629147, -0.40889396410410456`}}]}}, 
      {Arrowheads[{{0.027958080305709808`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.18256713700500635`, -0.3435457441966317}, \
{-0.048202093764223825`, -0.21030040964952196`}}]}}}, 
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.027958080305709804`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.048202093764223866`, 
         0.21030040964952207`}, {-0.18256713700500632`, 
         0.3435457441966318}}]}}, 
      {Arrowheads[{{0.01763015925257943, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.08730755008629149, 
         0.4088939641041046}, {-0.14346168068293869`, 0.5141829589728185}}]}}, 
      {Arrowheads[{{0.012778398848067047`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.10041742233300288`, 
         0.6055820621317972}, {-0.1303518084362273, 0.686725630175895}}]}}, 
      {Arrowheads[{{0.009999983686220891, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.10616201511202894`, 
         0.7982083281401835}, {-0.12460721565720123`, 0.863330133398278}}]}}, 
      {Arrowheads[{{0.00820751912665049, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.10915241516334355`, 
         0.98831703669632}, {-0.12161681560588662`, 1.0424521940729108`}}]}}, 
      {Arrowheads[{{0.006957431393153585, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.11089816208379669`, 
         1.1768861377059758`}, {-0.11987106868543349`, 1.223113862294024}}]}}, 
      {Arrowheads[{{0.006957431393153554, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.11987106868543411`, -1.223113862294024}, {
         0.11089816208379727`, -1.176886137705976}}]}}, 
      {Arrowheads[{{0.008207519126650461, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.12161681560588727`, -1.0424521940729106`}, {
         0.10915241516334412`, -0.98831703669632}}]}}, 
      {Arrowheads[{{0.009999983686220893, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.12460721565720188`, -0.8633301333982779}, {
         0.10616201511202951`, -0.7982083281401834}}]}}, 
      {Arrowheads[{{0.012778398848067054`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.13035180843622798`, -0.6867256301758949}, {
         0.10041742233300341`, -0.6055820621317971}}]}}, 
      {Arrowheads[{{0.017630159252579426`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.14346168068293944`, -0.5141829589728183}, {
         0.08730755008629196, -0.40889396410410456`}}]}}, 
      {Arrowheads[{{0.02795808030570979, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.18256713700500715`, -0.3435457441966314}, {
         0.04820209376422423, -0.21030040964952224`}}]}}}, 
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.027958080305709787`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.04820209376422427, 0.21030040964952235`}, {
         0.18256713700500712`, 0.34354574419663153`}}]}}, 
      {Arrowheads[{{0.01763015925257942, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.08730755008629196, 0.4088939641041047}, {
         0.14346168068293944`, 0.5141829589728184}}]}}, 
      {Arrowheads[{{0.012778398848067052`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.10041742233300342`, 0.6055820621317973}, {
         0.13035180843622796`, 0.6867256301758952}}]}}, 
      {Arrowheads[{{0.009999983686220893, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.10616201511202951`, 0.7982083281401836}, {
         0.12460721565720188`, 0.8633301333982781}}]}}, 
      {Arrowheads[{{0.008207519126650492, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.10915241516334412`, 0.98831703669632}, {
         0.12161681560588727`, 1.0424521940729108`}}]}}, 
      {Arrowheads[{{0.006957431393153554, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.11089816208379728`, 1.1768861377059758`}, {
         0.11987106868543412`, 1.2231138622940239`}}]}}, 
      {Arrowheads[{{0.0067156957804639745`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3582584519969926, -1.2192354601827944`}, {
         0.3340492403107003, -1.1807645398172057`}}]}}, 
      {Arrowheads[{{0.007818497774238308, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.36231591276365127`, -1.036334082028082}, {
         0.3299917795440417, -0.9944351487411488}}]}}, 
      {Arrowheads[{{0.009319360101903255, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.36854802002731024`, -0.8529767865270823}, {
         0.3237596722803827, -0.8085616750113792}}]}}, 
      {Arrowheads[{{0.011442083616757557`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3783903287049486, -0.6676064577563057}, {
         0.3139173636027444, -0.6247012345513865}}]}}, 
      {Arrowheads[{{0.014538201758969095`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3933858461538464, -0.47531446153846135`}, {
         0.29892184615384654`, -0.44776246153846155`}}]}}, 
      {Arrowheads[{{0.018920713831614602`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.40862335090928875`, -0.2628674383531022}, {
         0.2836843413984042, -0.29097871549305143`}}]}}, 
      {Arrowheads[{{0.02341219749748037, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3856050984595569, -0.02359676120857969}, {
         0.30670259384813603`, -0.16101862340680478`}}]}}, 
      {Arrowheads[{{0.023412197497480376`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.306702593848136, 0.1610186234068049}, {
         0.38560509845955704`, 0.02359676120857984}}]}}, 
      {Arrowheads[{{0.0189207138316146, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.2836843413984042, 0.2909787154930515}, {
         0.40862335090928875`, 0.2628674383531024}}]}}, 
      {Arrowheads[{{0.014538201758969095`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.29892184615384654`, 0.4477624615384616}, {
         0.3933858461538464, 0.47531446153846146`}}]}}, 
      {Arrowheads[{{0.011442083616757557`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.31391736360274436`, 0.6247012345513866}, {
         0.37839032870494854`, 0.6676064577563058}}]}}, 
      {Arrowheads[{{0.009319360101903262, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3237596722803827, 0.8085616750113792}, {
         0.3685480200273103, 0.8529767865270823}}]}}, 
      {Arrowheads[{{0.007818497774238322, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3299917795440417, 0.9944351487411487}, {
         0.36231591276365127`, 1.036334082028082}}]}}, 
      {Arrowheads[{{0.0067156957804639745`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3340492403107004, 1.1807645398172055`}, {
         0.35825845199699263`, 1.2192354601827942`}}]}}, 
      {Arrowheads[{{0.0062993219151477094`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.59357282883305, -1.2133133977772435`}, {
         0.5602733250131047, -1.1866866022227565`}}]}}, 
      {Arrowheads[{{0.007182008438656107, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5978022562591395, -1.027826708161696}, {
         0.5560438975870152, -1.0029425226075348`}}]}}, 
      {Arrowheads[{{0.008292529360913814, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6032188879011656, -0.8405717247505069}, {
         0.550627265944989, -0.8209667367879545}}]}}, 
      {Arrowheads[{{0.009682671628867115, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6094817470366285, -0.6498515808310279}, {
         0.544364406809526, -0.6424561114766641}}]}}, 
      {Arrowheads[{{0.011352428298968759`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6144047797763427, -0.45310507839647673`}, {
         0.539441374069812, -0.4699718446804462}}]}}, 
      {Arrowheads[{{0.013106524099836455`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6115303079457866, -0.24917961338653827`}, {
         0.5423158459003681, -0.3046665404596154}}]}}, 
      {Arrowheads[{{0.014355611131808992`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5920812919278516, -0.04615092761815433}, {
         0.561764861918303, -0.13846445699723015`}}]}}, 
      {Arrowheads[{{0.014355611131808988`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.561764861918303, 0.13846445699723026`}, {
         0.5920812919278516, 0.04615092761815448}}]}}, 
      {Arrowheads[{{0.013106524099836443`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5423158459003681, 0.3046665404596155}, {
         0.6115303079457864, 0.24917961338653838`}}]}}, 
      {Arrowheads[{{0.011352428298968759`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.539441374069812, 0.4699718446804463}, {
         0.6144047797763427, 0.4531050783964768}}]}}, 
      {Arrowheads[{{0.009682671628867115, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5443644068095261, 0.6424561114766644}, {
         0.6094817470366286, 0.6498515808310281}}]}}, 
      {Arrowheads[{{0.008292529360913821, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.550627265944989, 0.8209667367879546}, {
         0.6032188879011656, 0.8405717247505071}}]}}, 
      {Arrowheads[{{0.007182008438656107, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5560438975870151, 1.0029425226075346`}, {
         0.5978022562591394, 1.0278267081616959`}}]}}, 
      {Arrowheads[{{0.0062993219151477094`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5602733250131047, 1.1866866022227565`}, {
         0.59357282883305, 1.2133133977772435`}}]}}, 
      {Arrowheads[{{0.0057984202171441285`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8258718712122958, -1.2073866962763906`}, {
         0.7895127441723204, -1.1926133037236095`}}]}}, 
      {Arrowheads[{{0.006464552285447435, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8290089640556498, -1.0203054409282308`}, {
         0.7863756513289664, -1.0104637898409996`}}]}}, 
      {Arrowheads[{{0.007238761659638784, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.832179916779941, -0.8314591594379696}, {
         0.7832046986046753, -0.8300793021004919}}]}}, 
      {Arrowheads[{{0.008108877356406257, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8344649525874978, -0.6401300010524285}, {
         0.7809196627971187, -0.6521776912552638}}]}}, 
      {Arrowheads[{{0.00901620998289048, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8339800385184764, -0.4460474773016123}, {
         0.78140457686614, -0.4770294457753106}}]}}, 
      {Arrowheads[{{0.009823109296600725, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8280899149416319, -0.25067329902245894`}, {
         0.7872947004429844, -0.3031728548236947}}]}}, 
      {Arrowheads[{{0.010317270320861856`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8155701171193339, -0.058292436603141616`}, {
         0.7998144982652824, -0.12632294801224286`}}]}}, 
      {Arrowheads[{{0.010317270320861851`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7998144982652824, 0.12632294801224297`}, {
         0.8155701171193339, 0.05829243660314177}}]}}, 
      {Arrowheads[{{0.009823109296600734, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7872947004429843, 0.3031728548236948}, {
         0.8280899149416319, 0.25067329902245905`}}]}}, 
      {Arrowheads[{{0.00901620998289048, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.78140457686614, 0.4770294457753107}, {0.8339800385184764,
          0.4460474773016124}}]}}, 
      {Arrowheads[{{0.008108877356406257, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7809196627971186, 0.6521776912552638}, {
         0.8344649525874976, 0.6401300010524285}}]}}, 
      {Arrowheads[{{0.007238761659638784, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7832046986046753, 0.830079302100492}, {0.832179916779941,
          0.8314591594379697}}]}}, 
      {Arrowheads[{{0.006464552285447435, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7863756513289665, 1.0104637898409998`}, {
         0.8290089640556499, 1.020305440928231}}]}}, 
      {Arrowheads[{{0.0057984202171441285`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7895127441723205, 1.1926133037236093`}, {
         0.8258718712122959, 1.2073866962763904`}}]}}, 
      {Arrowheads[{{0.00528525618723926, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0561625055050217`, -1.202568153124471}, {
         1.0207605714180565`, -1.1974318468755287`}}]}}, 
      {Arrowheads[{{0.005774950673955711, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0580001200959679`, -1.0149454906963669`}, {
         1.0189229568271099`, -1.015823740072864}}]}}, 
      {Arrowheads[{{0.006306904610538194, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.059284706713353, -0.8260840179125725}, {
         1.017638370209725, -0.8354544436258888}}]}}, 
      {Arrowheads[{{0.006857642339136383, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0592814098990344`, -0.6359008856562541}, {
         1.0176416670240436`, -0.656406806651438}}]}}, 
      {Arrowheads[{{0.007380653701954039, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0570015500871541`, -0.4448009510431145}, {
         1.019921526835924, -0.4782759720338084}}]}}, 
      {Arrowheads[{{0.007804065832493464, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0516119558967758`, -0.2540194332233726}, {
         1.0253111210263022`, -0.29982672062278104`}}]}}, 
      {Arrowheads[{{0.008045058342766332, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0432637701050549`, -0.06550857183040472}, {
         1.033659306818023, -0.11910681278497974`}}]}}, 
      {Arrowheads[{{0.008045058342766332, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.033659306818023, 0.11910681278497988`}, {
         1.0432637701050549`, 0.06550857183040486}}]}}, 
      {Arrowheads[{{0.007804065832493464, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0253111210263022`, 0.29982672062278115`}, {
         1.0516119558967758`, 0.2540194332233727}}]}}, 
      {Arrowheads[{{0.007380653701954039, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0199215268359239`, 0.4782759720338085}, {
         1.057001550087154, 0.4448009510431146}}]}}, 
      {Arrowheads[{{0.006857642339136354, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0176416670240436`, 0.6564068066514381}, {
         1.0592814098990342`, 0.6359008856562542}}]}}, 
      {Arrowheads[{{0.006306904610538194, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.017638370209725, 0.8354544436258889}, {1.059284706713353,
          0.8260840179125726}}]}}, 
      {Arrowheads[{{0.005774950673955711, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.01892295682711, 1.0158237400728638`}, {1.058000120095968,
          1.0149454906963666`}}]}}, 
      {Arrowheads[{{0.00528525618723926, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0207605714180563`, 1.197431846875529}, {
         1.0561625055050214`, 1.2025681531244712`}}]}}, 
      {Arrowheads[{{0.004801879850049445, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2854557020356765`, -1.1990894749247596`}, {
         1.2530058364258634`, -1.2009105250752403`}}]}}, 
      {Arrowheads[{{0.005160194681349427, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.286267906891345, -1.011551259410986}, {
         1.2521936315701947`, -1.0192179713582448`}}]}}, 
      {Arrowheads[{{0.0055291524839288205`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2863791807873888`, -0.8232819571375655}, {
         1.2520823576741509`, -0.8382565044008958}}]}}, 
      {Arrowheads[{{0.005889049624893626, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2853461053826813`, -0.6344283694277313}, {
         1.2531154330788585`, -0.6578793228799609}}]}}, 
      {Arrowheads[{{0.006210412001103251, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2827308674601512`, -0.4454303897874951}, {
         1.2557306710013887`, -0.4776465332894278}}]}}, 
      {Arrowheads[{{0.00645638739139603, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2783319069009569`, -0.25705915447776695`}, {
         1.2601296315605828`, -0.2967869993683867}}]}}, 
      {Arrowheads[{{0.00659086612474817, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2724580233047174`, -0.0702376752429009}, {
         1.2660035151568223`, -0.11437770937248357`}}]}}, 
      {Arrowheads[{{0.00659086612474817, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2660035151568223`, 0.1143777093724837}, {
         1.2724580233047174`, 0.07023767524290103}}]}}, 
      {Arrowheads[{{0.00645638739139603, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2601296315605828`, 0.2967869993683868}, {
         1.2783319069009569`, 0.25705915447776706`}}]}}, 
      {Arrowheads[{{0.006210412001103272, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2557306710013885`, 0.4776465332894279}, {
         1.2827308674601512`, 0.4454303897874952}}]}}, 
      {Arrowheads[{{0.005889049624893626, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2531154330788583`, 0.657879322879961}, {
         1.2853461053826811`, 0.6344283694277314}}]}}, 
      {Arrowheads[{{0.0055291524839288205`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2520823576741509`, 0.838256504400896}, {
         1.2863791807873888`, 0.8232819571375657}}]}}, 
      {Arrowheads[{{0.005160194681349427, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2521936315701947`, 1.0192179713582448`}, {
         1.286267906891345, 1.011551259410986}}]}}, 
      {Arrowheads[{{0.004801879850049445, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2530058364258632`, 1.2009105250752403`}, {
         1.2854557020356763`, 1.1990894749247596`}}]}}, 
      {Arrowheads[{{0.004366318576526445, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.514416039558949, -1.1967563910992367`}, {
         1.4855839604410526`, -1.2032436089007634`}}]}}, 
      {Arrowheads[{{0.004630470224779108, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.51454870463626, -1.00956259005028}, {
         1.4854512953637413`, -1.0212066407189506`}}]}}, 
      {Arrowheads[{{0.004891497396835368, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.514032154047261, -0.8219871411956355}, {
         1.4859678459527403`, -0.839551320342826}}]}}, 
      {Arrowheads[{{0.005135410733324209, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5126293194204563`, -0.6342149757896187}, {
         1.4873706805795452`, -0.6580927165180734}}]}}, 
      {Arrowheads[{{0.005344349256071994, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5101674221035941`, -0.44658061940529026`}, {
         1.4898325778964072`, -0.47649630367163265`}}]}}, 
      {Arrowheads[{{0.005498695594094005, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5066444133588999`, -0.25954112120662304`}, {
         1.4933555866411015`, -0.29430503263953056`}}]}}, 
      {Arrowheads[{{0.005581058362677135, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5023158266446697`, -0.0735628570242084}, {
         1.4976841733553319`, -0.11105252759117604`}}]}}, 
      {Arrowheads[{{0.005581058362677139, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4976841733553317`, 0.1110525275911762}, {
         1.5023158266446694`, 0.07356285702420853}}]}}, 
      {Arrowheads[{{0.005498695594093998, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4933555866411015`, 0.2943050326395307}, {
         1.5066444133588999`, 0.2595411212066232}}]}}, 
      {Arrowheads[{{0.005344349256071987, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4898325778964072`, 0.47649630367163276`}, {
         1.5101674221035941`, 0.4465806194052904}}]}}, 
      {Arrowheads[{{0.005135410733324209, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.487370680579545, 0.6580927165180736}, {1.512629319420456,
          0.6342149757896188}}]}}, 
      {Arrowheads[{{0.004891497396835368, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4859678459527403`, 0.839551320342826}, {
         1.514032154047261, 0.8219871411956355}}]}}, 
      {Arrowheads[{{0.004630470224779108, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4854512953637413`, 1.0212066407189506`}, {
         1.51454870463626, 1.00956259005028}}]}}, 
      {Arrowheads[{{0.004366318576526445, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4855839604410523`, 1.2032436089007632`}, {
         1.5144160395589488`, 1.1967563910992365`}}]}}}}}, 
   GraphicsComplexBox[CompressedData["
1:eJx0nXd8W9XduE3Zu+y9994QKOVQyiq07FJ2KavsVXYLCGhpKVAChL1MCTTM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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl01toD2AYx/HHYU4hK3Pa2PY3ikKJhJC5kCg57IJc2ckctrFxh2UOy8aG
G7aYi20KFxJuceFOmkmtkDWHOWQ32kE78Fkuvv3e93mfw+/5HzL3l+wsHhUR
vXg6I6I6PaIGl1AxO2LpnIib9BaeeT+QFrE7M2IXylIihsXLaYn8A6kR3fiF
DRkR6/FXfsrMiJ90Ou2fFdGH4/qeQKXzjnkRZ2hrImK0GVPEp6JQ7wIkOU9W
+xV18yOazesSb6K17p3iHbir/j6acQcteKU2lZ+3cq/LTVaXcG83M5P+ULdS
Xs1c3p3P0Qd2ecHv+KyIR3Q73YKXvK2iRXbPxSnnMdiHXr1T1C1xXitvHd2E
JvVN3pKcG723oNWMQfsmxJfTZd7G+aw6eNwo9pE28HQL9biNRrzjb0D+JTn1
cjrt0kAvu3/wNglj7Zsnno9+9OGvnGr73lC7hdbTcnllWMzrdfcJar/wOpF2
0dX8rMEn58/oxCa12Si2wzE9q8y9Kn6BXqNF6Nevwcx6vJRT6XM6i+9qvqEL
fXKuil1DLa6gDkNqDnk/iCJ085XD424s5GUBsvDevBV65DgnjfRT249e3OP/
Mp1G07zPxTykumfSDKQj2X2zecU8Zst/qN8e/veiVf82vMag9992HqC58hdg
kJ8BPOevAA3yhry3iw3T0/pV4CR69D1FL4r3yH1j7nmeL6AKpWrO0bNY5Dtu
G/mPOP8Z2Qt5Yrl4rM9WfZ7QbbSR5vv9FqBU70J6lB5RcxhJasYm/v+n/wFD
lqAe
          "]]},
        "0.42000000000000004`"],
       Annotation[#, 0.42000000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1Gloz3EcwPHPGDaeuc2uvzlL5ImcJUeGkgiTqyZHrs3xQHKVOxFz5drM
eCAPEM+EHM8co9RYIscwodDGf7bxWh689/5+v5/P9/P9fL+//ksUFk1fnRIR
ef60z4rI7BbxITMi3TgNJV0i6ntEHOajORF7e0bcy464iyq5U3hr94jJvIUr
e0WUyT+LHPGf8utQj5/W1qj50vov81e81rypa0Qn8wb7a+QkeUNGRLV4uXrz
zU/wXD7Dr/QyJi9iLNrkRuRzax7UO2KInIR5H1SqfUHPpUgVq7BW4W5jjUdh
hHMHcmfxOrG5xq2wGYWJiGUYavxAXj5PxTX72/J9vmzfDmfswx7U6jfXPaq8
RTWeoaP7ZFnLwCN3uqj/Qj2e50V8iV/bt1HsDdfKf8dbzFN4vbo13Gj/e15n
npTz17wZSSSsn7NWjunO3KnuNN7FM/i5eJF9xdgv/wCOecM6+b2sfXGXr/iM
DnK/8Wjv+UVetnkHOU+Nn6BW7BPueZOlvBqrUGH+Uc/n+LS66d5mjvwCDBc/
q14ZSvHW2ih93XH3u+hvbQDS7X2rRhr3Mx/s3PnOH8TzeFjLeWInxU6hybwR
y5xV5D45auUiG5XyE/yYu/q2P7xFZ/7OXfi4Hsf7vtfUG8fXeQIv4YNql+AQ
Nlnbo/Zwe9LsGcbp/Euf9WjAbyTRzvoD92rLD3kTF+hvt/3b3HkrrqpfqUax
2A4u4hniqe7QGq1wW49Jvc/WexY/4gzORE+slL+w5d15lxoLjHvrMw/90Bd9
UCZWiu1qTjSfjEnIx2W9XMFzNUr0/oL/yGvAEmcsxhS9LeU8/TRb/4vwJkfk
Nxk34qoat3yfm6h3txMtv32ukVPOy/W2AnOcWYA71mbJnY2ZuOFuE1reyhnj
eWTu//9D/wDbYbZJ
          "]]},
        "0.35000000000000003`"],
       Annotation[#, 0.35000000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1Ftsznccx/FvZeGGODzVgzI9TVutZInzYYnOoiTaUp2ua4WKxI3TrpwT
tVgE28h6IGbBbFy5EFEWIllvFlqEC5FtcWMUcWg9S4xMvP5x8X4+v9/3+/l9
f9/v//88T8GqDXXrMyLiEx+rsiK+zY9ood/RfzMjNudGrLSfmhexgk4ZG1Ew
JqImO2KwWDUtHRdRgq4PI85jRFFEusB59OMlBlDo3DejI1rHR+xEbmFEDlY7
OwnL1KqgF3i7UMf7Rg91YkuxBL/x1NNdzpc4W4pDeu3EWv0dpuvodH028bbL
dyT3iOfilZpf6SNHj7nIRh7G4JJzS9x7mdbRjfQH3hmewUzMMu81tbbmeB5q
TcN9+S32lebeRnvk51of1N+BBPcN0e/3dL87BlvvVudz9RuwDP/Yn5B7QI/Y
j9P3dLVnYLgeMtyx3rmXZkrjNt8Cd/S66zr67B9hJO8D3hF0jrMPxar45mOB
/oeqMQwD4mmM5rsnnkUrk2cjlimfQr34H2q9lcsUv28/KrkDm8Sv0at4qnaz
uRvkf9TL3+acZ5aU/QdqVll/ho95upP3TIvtr1uf9HxO4Sh+xgmzVRZHzMZM
95bTlHia/0vrDGxHi2e0BlOtr/JV0Wqcdb4bV3DWuTP42j170eeZTtbfU3pL
z3dxG3/hTwzX71jz9JjtNN8Nulqvz+R6xHuxRWwbtqOPfwcdpN4d2mjfzLsc
b3j/R47emuSy6aeexWuxRvkvsJD/P/oaz8VfIMX3iD+TzuXvT74n8vW4Zd1v
rj3Jbyn5DSIf5bxptSporTM1OKLve2Y4o7e1PL/L7XV+H9qS7yFtp9Xq1mAR
Wp2ppTtpnx6e8DxGvtqvxAfUaFHrAm0Qv0g79ZJyRwcdRXeJt2IlX5d8jXg1
jovd9C6OeY/HUS5WgZ94JtK3/G20PfmP0H8RFoodlS8VK8Ny+2bMt26iH2EC
ijGZr4SuccdE73qxXspoLT3HX2SeX81wyWy/0Ms0Xfj+/+4dR6S7PQ==
          "]]},
        "0.28`"],
       Annotation[#, 0.28, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1F1M1mUYx/Hbai5qITFKSN7kxS3ACLXNmToPfGmLZbxoSY0aBoJtuXkg
JXNOVzMVURZvBzhn5eSAF1mbulpkNF3ktBCwo9oqUsEWKC6nM+jzXwff5/e/
r/u6ftd138//eeZXbCt+b1YIodhHR2YIK+eGsDgphPVPhbDcc+f8ELrRhaKn
Q6hNl0vfp38nhLCTllh/QEvp7YwQpjCJ04kh3KLzngkhGWesq/WYUJdifZtu
tT4rvlavNLF19JReL9FeupnnuJybKJoXwvd0AD/gH/UzqYZPC2FLSgg1qMag
2kr6E62i2XwXYJOaMryON/EGyuXk8V2IHDkfiT1LNzr/DP9c8+chB89hITrV
FPLtoi/Tk2aYFp/BA3znPP/Sx9zjcn6P00tyRuTniw/T38QbzH0Y9XqNiZ9Q
1yKvDa343PoT2owmbDdXvPtoTZaDJrSgGXcRwyeVXwpq9Ljg/qppo9pjvBro
ERyO7ozPOXN9iw/t9dGvcQ39qOW11Z28JncjvuFTTt9CGfapGTTz7/LK5Fda
j9PLan7EpJwJ3LNfIZ6EGPHnxQow7CzlatLE4uxdsb5v3t3u86r9v/RbxP86
3S+2F73O1cGvz1n70YvCLO8OBpxnCa3i8TbqPAdswpS7jleb43mpvGV0FY6r
/1S8Hcf4dtBL+mTKzzNbv+f86N20HoreL/Pkmy3BvL9E90IvmrtNvBXr5SWK
vRq9q3gFN3AN1zGmZhwD0XuMP+SW8d9Ah/SP0yPdeWNpPJ7EA/M+pMfD+DXq
iy+wQ34n7cYuHqdoux49dJaaUfP8iXr9jto/RLfwfkTOeetYZ52DR/msMsts
upKuwCiG5I1gGLH87qp/gr7LPzf6reAzPin2U5Ec/R6tD/IplJOJYv1LUIoC
+fH2P3aOffhZ76sYwYtql+FL/pW+k69oFR20dwWbPb+DCjTiCPb4jtui99dc
U3zP0Ts0Tp852G+ObH0XIANZyMR9v+kie6fVl7qPXHe8gdbx2hndrfhB/2X1
OIBys/SIrZGzlO+0Pi/IW4LVYjXmvsWz1nrF3P//Q/8DgNnPdQ==
          "]]},
        "0.21000000000000002`"],
       Annotation[#, 0.21000000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1FtM12Ucx/EvedNqrqGZh/JUNs0DSuIBodEy7QLLQ+L+KAQhTJuaW00u
SmmUE+lGi7SlzZkmLK10HqbO0pnYypVOA1ddmMg8DLCDrVo32euZF28+z/f7
fE/P8/D7D69aOe/lrIio8WfXIxFj+0eMHxhR3C9i5/CIj1ExLGLWAxF5Q8Ww
mzFOXCX/s/yT+LsejujGTfQge1BEH3w+IKJc3Uv3R/RlX6EV7L38uWrso5Xs
Tv5+9q/Rxez9/JPsH6DV7Ov8A+wPxCG+X/U4SH+jg/j2mGm1edbgdZSYq44W
qfGJ/CfpQr6nzDodT+NPNTeIqeafwZ6JZ/AP/3I5c+UssdckZmk650MRGb41
7mcle+KDEaOt30AdxpijnvZyd4X2CrBY/ITBEdP4H6c17Fxanfxmz8VmPd9H
ifUC/Gj9E1rVaVTzVHoLtXbg3JCIsyhUb5o6BZiCfExFO86JbxHbjC1m/gAn
5FSq1cvc7fxtWOL+rut3A9ewLd2n2N+x2f4mNKFdzqd8YeYT+t5mn6IXaZma
5TjNXqRHK+3U/xfrDppJcfbbsVOPHcizdwt/4EX7I/Xo0vseWsS+l1awh9Dz
8v71tnXpbOK7zZCnRi27AfVo0edL853EPhSP8J74Jr0RrUn/w3jN+rZ3zdBb
NNsdP2Y9RVw+LcJ2+R/xb8U2dc+o/x1OIsdMM+RdsF5qhhzzXEIp/w2+DB3D
Poo+4vrimPVV2k178ChGYhS+dbYz+DrdH4rRpuchd3CfeodpBy7jtBkP0pfc
yWf0uHvIiGnl38C3Ee/gb/P/hQ7zfWiey7QTVzCCvV9Mlv/nn/VZoF9vdild
iItqDjXrMLzr/E14D+tTbfENdB1W6Lscx/U+TI9gV3pDsd+L26jWWdrD7kaZ
3ovwtnUj5niL2fgi/Y6I/UpsG/8PyDbjXLFr1XsrndUsVeY6JjaffbeYu8SP
kvef77QknYO/BMvMM9nZs8TMZz+Peajhn6XGHrWKaaPzl/M10DK6nk7nf5OW
sutphhak71/OTX2eULMQBai1vwpd/M/pP1Xc7jQfrfU2qzDRukVuXvre07dP
q9hX5bzKHs9uZk9IvwPsHPoCu8P+K+m3pP+d3+P/AcVV3HM=
          "]]},
        "0.14`"],
       Annotation[#, 0.14, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV01tsznccx/FfMR0mseiYGdquaTdhF+36FMuKZRc8isYWNaMHh+J5enS4
WLJlMpu7LSEY4rAs2IhqWoes1dYzh3WyXRCyEIQtEoQLmzpuxOt/8e7n9z1/
f7/+n6wF9bPq0kII5f5czQhhQWYIC1GFxViEH7NCWEYTWIoaJPET/8phIdxU
t/iNEGJDQ8hnT38lhFt8jXKWY9yoEMZjAm7zf8E3e0gIxeyJeF/dPvWT6Vdi
n4itoWvxNVrN+VTfR2pr5c2SF2MX4TFfHd+HfMvUtcud69xGP6Yf2eV/Odv0
2Y4p5k1Fpdh+dRX0S336yGtkV7PfZafU/4JOHEMX1vCnyxuo/iWc4juJl9XF
XwvhO7Et/JtxUZ+MV0MYjDL+1hEhzBvubpiIQruWvO5t6TS6h/6m12l0I6E+
me2e6tv1L6IX5TTIvUAb6QE04Q971epZgyRSaivsU45+7vwi0lGrXw02yFmP
8/rUq+8a6Z6YKf9Pux4Qa0K2mR32LtV/h1gh+1fnI2J/yW+z19/0GhL6FtIq
OQNoJS2g+bgV9aFz9J6k/jb7nH5n0GrXz8z4HKuwFquxR26H3VJoRjwnhA/Q
bed8usgblUffhfMze5TRf+ggb/+mc0zeOFqMneq/59+K7fqe1v93pHAeZ5Fn
pzF2HIsb7NH0LfyM9sgnXmz3XOf3ou/AG5TxdbtDszuU0P18R8Sy+NexZzgf
0yuXfVisl//BZfM3il2ih/gORt+Ee8zD/OjbxVE1Fd6zEv/a+R56UKZuDjL1
a5MzKvo2aIs5q6LvV04DrnqHDjv1Vd9J79v5AXowQm6a+Em1gfZGL+zyXmPN
3k3j3utb/d622zd0upqkOQkUIoZ3UIC5chrVHFd7Av3NH4AX1I2xaxpNZ/fF
f/Z5gio7HTe/wdwgnifvqd9oPTvFn9Bvg7fahI04rG+OnAdqH+I+uuQ9onG9
jjp3qm2nmfKmRr9/5yRfj74j+e7SUrMm22NS9JZmrNb7Dn+1vAlqSvlW8M2k
K2kRXwOdwa6l9ajDXvsM0XMoWsypUn9dnwz2NVpiTpEZMSyRX40r/HH+Ar58
lPPNxw96PQcQLtBG
          "]]},
        "0.07`"],
       Annotation[#, 0.07, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV00dPVVEYBdBDnCjVRtMHDyxxYMFeYhk4s6CJJv4AnTlA7Cj23hITnWhi
YhCwIlZs2AFFwYKVIlJEYmKMcWDixOhisNjc7+177rmHR/bSFYvyYkIICX5k
J4fwNSuEbjrp4gtZ5u2yg1ba+EzUvEV+opFmmsg0/yA/8pb3vCPDvEG+4SWv
eUXEvF6+4Bl1PGew+VNZSzVPqGGQ+cxICLOYzgzqhrqfetJ93mdICLH0Jp44
LplPzg5hChOYxETSzDPTre/eqCxI81z9Wqp5Qg3levvlAfZxiIMs1t8j97KT
3ewiVX+73EFapn2TTkWq58kog8kgwkX9rbrbGOg6hWQmWr+fHEACSST2fK5f
Nsi++KFTIa9x3frHMkI4QSFHOEyZfqOzaqOSEcP8bXmb4l3kUfKdx25ZwF/n
sUQuIxINYaVczmz9/jKdC9ZpoJ5i6//xnH/U0sseYxhpb79c/+Y7P/lBsv5J
+y3im06pLOGK/Xf5vJsOvtDZ8z76m53NFlpdt9PGcPe2yE800kwTA/Tneod5
zCGX+UzTL7TGJgrYyAaO66+Ra1nNetYxVX+lXEUe+aygv37UuWSR6/0XsoBj
5uOc43jGMJYc+pmP8vzRjCSHMfyzfozvXZXfg+yr99j3vIoHPOIhSeb35H3u
cJdKEs1vydtUcJMbJJhfk9e5zFWuEG9eLi9xgYuUEWd+Tp7nNGc5Q6x5iSyl
iGJO9fyPeqf/9nKq4A==
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1G1M1WUYx/EbLXGAKJpgWcA5oDPE+cJcQaKZyKsSxamV60EsG7UBCijR
1M2ZVi5amjhjbc62fGCKQKUy5yyCNhF0unLThtZqbXR8eKFrtUZ9/i++/O7r
+b6v8x+xypqK6qQQwlp/yiaH8DbW5YTwBrJiIWSilK8Kr/OtxQS+8XiW700c
zArhFvsL+unDIexBgp2GcUjB3swQUmmrnNH0Y/Yo2kx38zXlhfCkXqv0X4kV
+Cc3hL8xl78SH8i7oeZDepNuU9ts1hZ2h/qttJS9mW5kz1Kzi/2z3D/1SWAY
76pbL6dezjQ5q1HDPs6ujWrFB9VcxDT3mI64nJei/Yj38w/gAr7h2/5ICCfp
HvUPTOFHnXMeX8Oj+uFxLNJ3Bi2lxXQDfXkqdV6v1wacMus0ujHarPf4dmIH
rsuvk3uN1kf1chrN6PTGWn1qUI1v1VW60xqkem8KTvJffSyEn7BYfYH6Vfwr
MdP5lHia82/6taj7nXbznUax9wypKzfnkNh89hnnM2JLolyxE2Z+6VwiNpb/
aXqAHYt2h4Sc2xjGPdxFgZyQ7TvAD+wR/IWDZve70yUcxZT8EDKw0L2raC2m
6llJV+Bfu95Et6PGnnbTXbgsP4dOR7c+Q7iKNv2bzfkETWjFPnR5Q6c7f4WE
ux2jbcg3Ky/67t1zCX8aLacT6RyaRT+Xd867z9vLMr5z7GXOuWIV7B6xDPu9
b/ZSsXv0u+i3QqE7vSD2ImY5l7l3TF0c34v3og9V6t7CHf0a2HXuVI+B6JvX
b6NzA27aR5+3JMftlZboMx/L+SuiXakvjH539MgpoEX29hQOR98mPUKXRjvT
r1fv2eaeV/eg+n56xL4OI19skO+Q8zPy69UtoA2008wutKMDJ6J96TOgfoya
sUhGtTte5DvrPEjnytnE185upFv0es49nsdr5hWiSM4c2ijnspqd8q7Qd9hP
8BeLv883jv0jfwpNQyrmif3hjf95d6Aj9CMzkpz3q/lazmfR/zK+V8x8NZqL
En3T5bTwr4m+P6zjW4hFepbiV7PS1f9Cx9MyvttqJmISMjAZD+EOcpCLbMQR
w138D+IWzcQ=
          "]]},
        RowBox[{"-", "0.07`"}]],
       Annotation[#, -0.07, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1Gtsz1ccx/EjcU1EPKle1KVjEs8QQVzqr6KXiESQbO63FkHIECKbKJbF
bYsRijbZxFxaW11Twaw00RYPtMvSGr1IWs3MrSVpCMXrFw/e/Zzv55zv5fx+
v/5TlqydsaZTCCHHn7iEEFYnhfBkUAiraK++IQxBe1wIR3nxiSGU8W/iQ3wI
w3jD8deAEEpxA2+SQ3iLAnn5OIr6PiHM6RfCLTn/qLGBV0lr+cvUWo674o39
Q7hNX+lXwGujpc6O44/FdPOV8ivpeXqbdrKfRXvSqfQ6/zLep+itZ3d5r/Xd
bL0PP6JI3Roz1uMKvhwcQn9Um2c3/Rlr5O+gG/HOHWfRxUhyx7V0BWLO96bx
KFTnHu7gmPrt+nSgHJnm6mKOofS1OEaf0Im0kF8RqZnL6XE6no7mj8E192pR
bxr/P3rV81kW+TSHlx3lmeOK+Btnq6J35L6z1a/j5/HirPsgAfG47g5/4hpG
fxHCKLTL+V+tp0hRI0nuIn0Saa3730cNgvsVqlmEavNUiKtonr1y69P8ApqP
Rrkx/SZFd0W1Z1qFYdHMch7p34iv9f8KJWbosP8RHzBNTpp6q/lp1pvU+5e/
TlxD76MW89QagjQ9J2OhnEVYjKHqZ/K2yl0qzsY2+RfMm0tbB3rP9l7S72kb
3cN/T3eJ39Hd9ADvWzlj9Oms577om1Z7LrJ5YxFzr1S61bkH3mGaeBIeWt/g
1dFf1GmgTeJ6OsV+I70pfkR/s99Mm8VNNMP+Y1ombqFZ4kxk6JNojiPmSIi+
dZpEM/nJNF/clxbQfjSLf9B7PYQiPXaoN4W3EqfFl8WF9LDzp+h28WR7K3BC
XCI+SfPsH6PbxDF7OUhX81deLm+ieL/4qZ7d0QNdcUBeN5rn3CXnDkffaPTu
sddeh+c8Su4s8UzMwE7+G/4I/iY5P0R9eS94ubSBl+W7+I6f4R1v4T23l26d
zm+N7kgf6jvSMztr/ZJ3jhbjufUzdIt+T8QlmOM7TBY/1r8FlXr/JLeCNomb
sUG99Siznu/8ArRat+Fv50Y4f8b/zu8oQjH+QIP5JuiRivG4hIvRfPLqUYcH
SDXThEGff48/AbMS7J4=
          "]]},
        RowBox[{"-", "0.14`"}]],
       Annotation[#, -0.14, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1GloD3Acx/GfczzyAGNmzjlHIWGGGaHmvluZHElWozDMWdMKMzahHJsz
5iolrAwPkKuY3YralPMBGlHu1z8P3v/P7/v9fc/f/9+/+5JVs1Y2CSEM9xHT
MYS8TiHswquuITTgdGwIz7qEUI5o92k9QuhAr8aF0JKOZadgNJIxBuPQFn3b
h5CgRn9kdwhhfEwIC/n6YZgeA/kHYKIeOe5bq1faM4RW9GX3EL50C+ErGpEb
7d5dlbtBcgYjSY3d8mr5XrmroaP56uht9nOazB6DJD238B0VX6jWAvnv9HiP
Ir4WdB9/Ov8H52K+reJHySswdyEO8m3mS+Rbiv3sK+wDNE/uXjEFzq/lF9I3
dKZ6M/DdDt8wWN5iTBCbKybV7pMxBfl8I8w61dtNwxTMwHQ8lNNMTHOU2ytL
3wS+c3L60HRM1icVj/EIN/i2d7aP2XLoe1plpmpUIFP/Stop8k7qxdJ8/ZvR
IfqM0jcJIxGvzyxcVfOi+zLxl2hTnPFbSBQzAtvl99YnT7/d2IUG1GOL31CV
mGpUYq/cGrqHbnNXQLfSOr5aRKlTo2eRXrWR78B9Na3EZnPcp5voGrNmyVuL
u3bKphuxHvlyNtAv4haJX8f+TOvlNOCWef8iOF+mTWmqmHrndHFlke+DfY3+
9FaH7BqlXqM+G5w/0wfsg87Xvfk5fart+gKliI8PIQ7ldtlJC5CpTg7Nwg+7
zaaLECN/JV2OZPFtaDRK1HmChzgeeX9xv/TrJ3e82e45T6LNzdGXNrIPm/em
XRL5UvhKaAnfH/zGSTx3X4eP9swQs4C+Vb/C2+9gL5OTgRW4Y8/FdAlWy9ko
rlR8Nn1Cn2KnnAr6DLHmPyz2CA7hrLuj9Aydb+4eYubQeZiLNHXaYbXzQvVT
0MW+ceilVq55j8k/jmKcV+cUPY0TuMA+ST+K+xT5X0BHvqGx///X/gFdssK3

          "]]},
        RowBox[{"-", "0.21000000000000002`"}]],
       Annotation[#, -0.21000000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1Hloz3Ecx/H32I+ao6WwzQ7bkHO2uf6T+zZnKcrYhD8wVyFHSG721whF
/lKO8Yeb/EORIynFzNEcTTFC/nD84fHNH8+93p/3+/15H9/v97fCqpqZK9Ii
otyflfkR7XMjxnSPKOgS8TknIp+248vAX/E/6NA1opouQjr/aPm58qrl59G2
fCksFV+CedkRjWIp93p3i2hxfkbT5TbwN7Jf4DR7eVbELfbFoohPnSOO8GWq
f4yOE7uTFzGK9i42C/tMQcQ53NPvEU6hc4+Ijhiu/mK6HFlyKulM/HJ3Ld2G
ZYURtXQ3HsvPpcW4ok4jnuKyu5dQp99d+3y1xzr2N9rG+QMy8BHNmGK+G+b9
av6b9phv37V85/le2+0NXrLb6PeK/V38jXNb9ZpopfNGdTahi/4T7F8kd6lY
Ie3El4k14qsxSf5Wug1d+SfKL5bXIr8HzebLwguzNGC7vD323oe9KBYrQonc
v2oNpDPUmI4+Zjoup6d9ByWziJXScpShkP+6Pe8m7wB71H6tx2H77mXX0W/m
mJp8U/IPOe/m/6HOc72a6R3xj3S9Pi10Hf1Mh3hPQ1GOwRiEnXJ/69dNvRzU
63UOQ9U841mOcHckLrg/R7/NfKPFx+Ch+R6h2swP7LWAVmEhhqm73zylak90
ZwJ2yJkkdo3mm/uQ/g/Ea+Xep3XOefyZmC5/GipQK/+9GabptcscP815kO8d
31tU8O/kb2If4J+afO/mv4Qnag0wR3+k+B+K1+i3hb2Kb7J7A9ml6OdcYo4G
2pdWoop/bPIbxia575zL9C/FUf2OIGWnDfz14rOS70iPkXSDc7O89cl3abfW
8rLEWtFsmk7fJt+G3Cb6Rc5GdgV/a8/3apJLr9CX9BUCack7kpOy22l6Nvnd
4qp5b3sWBezuKFFrrv7jkx3ZvfhOmreverP4Z6NMLMPzOJHMhBykyW2Fmvz/
/8P+Ab0grmU=
          "]]},
        RowBox[{"-", "0.28`"}]],
       Annotation[#, -0.28, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1ElszVEUx/FjakxPg76+0tbUIhYqVhamGKqmhNawMYVEQmoITQm6MM9D
ggWLmlkUMUaCmEJQi1pISGqIpGKBCAtD2giffyx+/d577jm/c+5977XvopUV
K1pFRKE/23pFHMqL2IoH8XW/iNM9Is5Q/0xE14KI7pRDnXpGnCqMaBFP2//B
0/ZZ+M7ZavXf1b3Jj3hrv9L+Pr9qvIlVeBu/pu2d36K0/A56f6AmKlb7g98j
68d0mP/13hGNSf/iiBvYD/OpITdiB+6nyr4RG7GKfhdFlON8yqhdhotppPwU
5tA5Ph3wBdbJadZnoLoBVGrOoc5SfSKy6Yg5P5n5mtkrnV3GZXgDl2Kj85fm
/uIu+TTIuq07HOP5G7P0+IW19s3Jfe3bU5vkLuJLxNqZbbn77uZXiXtwBTY7
P6HuJB3he5SumiVLzyuYL15QmHyYEUVyC7HUzCnee3ms4vFUPGXWLjTX+Rx6
r99aOetpHZ23L+dZh2fN04Q14sO96wXrEXgRK8RG86zn9YxK9BpCnZw1qe+I
g+13yPtsvR3rve8xc4/kMYpG00zxWUl/8S3usQG3YppnLuUk/ubO4HosM/Nl
n80G64nW4+WXUp1Zh/AvobLEK/ku8v+ud4PYc7rHd7PYfWwtfxifNpgrP00P
xDOYR+3E692vLT7DO84OmPMunnB+nL7xLkn6ip3HCWZMeevS5I2xDDvjFMzG
iTiZJtFu+bv4bVO7xxw7rWt4XuI527ra/OPcbw2Ox+k0jSq87Ss1P93nBy3i
tZBm8NuHM/XJ8e5pyqWH8rtjN3rCv9hdimic9Vg1Y+ibnCp9N+q7idZ4o2r6
KDZfzwU0j+rNsj35LepRIG8qzxbMFt8rfx8VJ98/qrVu4X3drA/MWavfWfEz
9Dfz///NP7LMomQ=
          "]]},
        RowBox[{"-", "0.35000000000000003`"}]],
       Annotation[#, -0.35000000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl00lszVEUx/FjpapeDdHqU9UqWhJJDWFjKQQbqbDBRiwMVUOEDvQVrSlp
UU2QKCUxrQxNY2ajrGzs2loihKpQkbamz4vFN797xnvO/b9XtHFHeeWIiBjG
25yI63kR1/BgYsQN+mJqRBee45149rSIsahi1+CgvGrakB/RiI+FER9QI7en
KGJPbkSJeCnOqGtFrvhPdS3OaydFnKJr6Gma5c4EnhdHjKF5kyMyxUajle+x
uh7+Yf0Xii3CG3YvSvhWJyO+4Ttm8f2gA5gnVsJ+qUcpnYlKlPL/LpCbnles
F91otksTBvhr7XFM7nEk7TPP/POxAKXsC+LL5a1jr6DraRktp+1il/CFXU37
aQ3NotvNVYmvdjvgfeqQwl/3HLLnfudajPcu4zCkpo/9Bf04ydeMuXocUXMU
g3JS5r6I4/ZooydovX5H7ZmiVd5sHyaLJbEyPTcKpngj+wzxzaQjzBU4qP8h
3HQ+rMcN2kYL078H/rHY7u4KZDlPV/sLLe7tcF8//041j9i7aCWemvsZ7uMJ
HqJP7v30e9DzcnPUzTDne3cV0wE9u5z/qD9j1g5v0G3WzOkRnbSIJvHKXEdo
E7Z66xTdjZ96rqIbkKN2G92ExfKz6ARc1SeDvqY35Qy657f7ZqtdQsvEEt7q
sxnPmvUjvWP2u7iFe+jEZrn55u9Sl6SfcFvuafagPRIYpX4L/1ZkOI9Eg3OL
HS+rr6BXaD1/CnPM2c7OUXvOXrnpb2S+peZZhoSZs7E3/X3Tvyf3hbpGPQ/w
16Eq/////B+YLpkA
          "]]},
        RowBox[{"-", "0.42000000000000004`"}]],
       Annotation[#, -0.42000000000000004`, "Tooltip"]& ], {}, {}}}], 
   GraphicsComplexBox[CompressedData["
1:eJx0vXtc1EX7/79paZmlaXdpZZllWllpaWWnS8vSTnYys7Q0LTuXdmtlam0H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    "], {{{}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl01toz2Ecx/FnK8I2Noe2pRxyLGpjTkPRRgixf1aktloKNxsbaqXU2AU2
3LiwuZliY5PTNm1josx5OWzMqZRSywVLaUPx+uXi3ed5Pt/P9/s8v9//959a
VBIrjgshDCFpXAj7R4fwGfPGhvA8KYT2lBA6sCA5hBT1At4K63djQniPtsQQ
PtBOmkdjeIs+3JF9iVR9L2gGhqLZ9r/oIJaavQyLkY0l+GHWNv1z3OG+nh3O
66K76G7sRJbafGTxO6N7mHWcttNqWoVm9RYcw0l+nXPP4braPTOO8rrpKl4X
PSVXjUVmxnj5qOVvp1uRLb+BbsJXM9bRL+56Hwesy/FZ/hA9iNPyFbQS/fxG
2oDl5tfbX0IdcnmX6YooT0+YnWt9nr+abnGnm2hFW/SMuMWfJdtBZ3hn681M
o+lIRY3sGWxU32tfijIU6C1Cr9pr9KAPb/BXNk4ty7mB5pvZE82176WvsEBu
IfY4uwQFZvZ7/if4yR/U9xEr1Qrlk+hVmSu4bT9dppAXQ739ZnqB5tH5Mt/c
IcF5M3nPzOnGd95DfY/xCKPUHvBH0rl0IlrREv0mvLPyyWassW7mraWJ+mpp
Aq2hI2VG4LszB6JzkamWgW714dH3qjcTb/Aak3gN9CKK9ZbK7kMZcnjpdJjM
CbOqkWafijjZae4TTz/J5XjWcrlAq+Tire/yn6pfk7+B65jBb1RvQgUO47f3
3CX7hx6xr8QEs8djsp4p0Teib8CsRN4s6yb52Un//9//AGZBi7I=
          "]]},
        "0.36`"],
       Annotation[#, 0.36, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl1FtoiGEYwPEXOWWzA9nBjRE5JbGxrIyIOedQQnPc7IIYJQpj0mxyiM0N
NySnGQpForW15dRyuneBHXJjmePk8Htz8d//ed7neZ/3/b5vW9bG7cu29Qgh
JPkxbFAIXxNDGJkUwruEEEq5S756YAg/5Gu4VL4Dd5Cn/zYn6FvFj/ienqf8
kSdwhvVnnMkHU0MoxwHsVv+CiWZ08Tj1ZH258hRORBKmygfy+uQQfrvDX/Ef
LJIvxISUECYjG9/NmcI5WCsudk4hF3GFtSP4Y8Zy+/9ylbwSH/Q06VnHF9Wm
mJuDWnmCegFnyudxBTe4a6G+xvhuOMXaJHGl+Ciy9dXHOTzW/hZulc8W1+s9
Ln/AJ7iBK/k5t6AZJ93lGHLMXOr5V+Cs9dW8Ern6F/BifDS3gFs9TxN2iffg
nf4y3osa/eV8GB3xufgK8sy/LL+G87iFOkyP/TybP5v/Fd8x1/174kX8XnyB
R/FIvaNwT898M0eL14mfiM/xYy629pxTeTCy7Ltr/zRxo/iSvmJ5iefezKet
VSPBnhK1AXxKflz/JvVjXMQ1fAbfxNV8GlVYjhXohS3o43lTzdgqTuHe8nZ7
2jDT3HRn5HMaz+A33IRm9MVb9+rDI+yt5eF8Td9V7DSjTP2WtSVqL32Hbnmm
9ZvW+jsrjV9gCPrJb3C6egaui4fqr+MN8R2pr4/fXi05vuP4N8ED8Abj5a/j
mfEszpe/4v3yffF9O78dneJPOGTWHM89x1075O1oi7+P1mbhkd73GGMtxG/u
Lg/lG8Xd9maZ/yt+V/Wf3Kl+X31u/BtO/P9/4x/rlJNr
          "]]},
        "0.24`"],
       Annotation[#, 0.24, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl1Ftsz3cYx/Fvp8oFZjrdFoxtJTYtmRBqI+3FzCFVWmqikmlLtBmykJBO
D1QjIpnTsnEhGmZxSMxxyy4cokbX1UTirN3S6LCsbcZFhaB7fePi3ffv+TzP
9/n9/v+2v3cKl+cuSwghlPkxIDmEpt4hDHk1hL96hTChXwi31b+9FkIhT+eP
ZM2u7/JeM7/LitQNXMyN/L38D16kvsSL+XSfECbZf4qX2P+n7IzrVnv2mS+R
HeRr5q/jKr4wU2dmhHPneKmZe7JB6vvcaT5fPhfL1NnOzEAOjuhlmz/BHe5R
b6bTzJeyr/TLUCAr0tvUN4Tu3BM9kIRn5p7jifND1cNQ4Uw51qDWrrmyz7BE
XYJSzHEmH2vsPq4+gZP4RvahM33dq5/rX2UXMF92Vj3c9VG9Y3gTP6mz9A7r
DVQPwFuoint5jHwsytXfqit4B4/mCh7LLfp38b5drfwJn/F5Tumfxi+ow1ls
MN/AjTiPr+P3gjGeYabvOw875fM4H+PMT+Ns/GP3p9xqdx1Wul6FFvPlXIbt
5qu4Ov7u5Af4B0ywf596P3bjMA5hovnK+Hnx0P5OPLC7HlN8jgSkYbe5LjO1
3E29i5N4LyfyHs5wdjyOmKtUJ3N/pGCLfI7PmY9qdQ3WI93Zc3oFnq/e9UWk
qEfr5ch2uV7gTK59BZzHxbIiNOqnyUYiHfvV3zm/AzuRIRuPZHmG72CpnYnx
7yJmeB3T1E/t2swr7e7FB509JFuhvmimnS9wG3f4Xq6gVF2iLuFS3m5+Gya7
HoLh7jeYp8humg/qofF3ab7Q/ud4hqnq//RvoMt1atzleac61yR7xbl0WY7e
IvPb9BJcp8lmcDaK5VvlwfUH8o+dGcFbZC/0NnMX37Fvvb018b2gP8pcM1er
73BWfC/x27iNkepbvE7/Bmeqb/IbuI509TVeq5/CV/HEM1x2n0rZJW7DQvf+
HBs9x2NukC2I74X4f4kaeQ9nU+1Lit+xbK2sjau4nRPlD+J7QX0vvhv4Pr/g
d53r4tnqPKyO703+2X1yeRZmohlNOC5vwUNzmZ73EWfx8mjP3c6D7ezgVHUb
/6v/ozOZ6lb1QP2/+b34vx+fRf+A/qT4XfZ++Y7/H9ID2o8=
          "]]},
        "0.12`"],
       Annotation[#, 0.12, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], 
         LineBox[{13, 38, 63, 88, 113, 138, 163, 188, 213, 807, 238, 2841, 
          809, 1345, 263, 2649, 1283, 811, 1871, 1192, 1706, 286, 1887, 1293, 
          1890, 813, 1874, 1194, 1710, 307, 1896, 1296, 1897, 814, 1875, 1195,
           1713, 328, 1902, 1302, 1905, 815, 1197, 2652, 351, 1358, 817, 2844,
           376, 819, 401, 426, 451, 476, 501, 526, 551, 576, 601}]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl1UlsllUUh/ELlQRBKeBCgy3SmoBgRAgJqFQr3SCWEEChLQsUVIodGUwE
qlFmErAJCVqDUkCRQhxAo4IJKINicMNQB4ZghyhhBrUgUGv8nbh4eO75n3Nv
71fe92vO9OqJVZ1SSkv903J7Snm9U3oMjyI/M6WPbkvpWK+UjuMXVJo5i753
pHSGm/U/xBnz59R/W2+07z2U9EhpmryYp/NszMJYvS/MXba+HnswR/aB81+N
OfMnrOfIT/JcHtVTjdlYZGYxFuq9jtew0UyJbArK1RVxVxQ5txitztyn18L7
uU72uD13Oq9dtsvsbkyUHdGbwCP0HkI2+qJAtk1vp7kd6CdbaO/nPFw+AkvV
deplvJOH8Q5ezK36i7iXvV/KMrkHDlkfwS404ihWmTvFzTiIkziGh92hzO++
EvXquTwLeeaf51Jc8nOm8VRswAqsxGnzq7kWb5tfy+twQV7Ha5Dv/GvqG9iE
jvj9xGc3f4X/QpvzL/FF9HH/u9EVWdhkLvFmvpXruRtv4S7cwCPtfwTd1VvV
g50zBEPjfvJx7vCV3gPqPIzEZNlRvSL+Vm8/OquH6Y2X1Vs/7ayp/m8n8Qz1
C5hvbh5exrN6z2Cr+bXqd/AuNsjWY4GzalCI9Wba7a+1rtardOb78fniM8gL
/K7KZVV6FVweM7HGOP0czkU2tjsriz/lGn4F8+LnYT7uivconk00ISueV+7D
9+An78pu9XF+0s8fg7HqQrxh73Pqr63/0P+Gl9nT5E57rJt5L/fn5fJTZjLc
fbB6jb0z7B2vPwEPyjrzm/LSeBesn8IQ+VC8Jc+Id4dv4V/jPs4sxED9Ts49
IfsTA9SD1Pdxrfl7+V97fo73Hjnqgfq5vEo/mzv0f9S733mDkCX7R9YoG61+
wvqme7XjOlbad4MzzV3jFeqrvJy7ya6aP2jvFFkJirFEr40PyIt4MibFd4j8
PO+Tt9mXYX+N7DfZAv6dO7g1niVu4hY0o8D8S+73CW/Dx/jB/u18xWy13mfO
vaj+Xn5ZdoAvxDMhr4q9fE79nfx8PON8Ns6SV+hv4dPqvfGsqcsiU+9RN/Dm
+F51l/K4Y9xX3WBPo6wsvtP4xXiH4t2UH+aZ6kNcGs9ffCbkxrOL/N7//z34
D6Rr604=
          "]]},
        RowBox[{"-", "0.12`"}]],
       Annotation[#, -0.12, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl1GtozWEcwPFHYS47Gzub0mjMZcnkFbltxQyRLCFhZdRMaZPLzDVlNSY1
l7m1GbMpTFG2FqWEMpeRkpLs1cosydYivPD558V339/t+f2fc3bOmbilbHXp
oBBCqj9jUkJoioWQxte5DenYi3g8hD38Gr8TQxgsbxS34j6ykkOYihZnO5JC
uMm3cQu71SdEZ8WdeIUBM2/V3yBbL2e0O4gXcBrncpxzohq2jAphnLl5nlsk
TkAykjBDvdbMeey3t0atgk/zZHsmoUAvmDsprkaRfrF7bOYmvUdqc83flsfE
yzlDvoKr+Imzl8095U08WS1PfEKcKZ6IuHOV5kfxHW5Va4leJ5/xrBrxwyjm
93ySP3EXnuMjPmCOvdu9zh2ol+/iMsw3v5WL8c1zN3MhGlCFanSbr+FTuGD+
EtehV/08n0Wu/QPyX7iOv/iNheb7+DL323+Ja937u9pdcYLXdY/Hc3P0/vA0
veloN7/S3mxxkfiFuI47+LhagfmZvIoznWvlHHkubpjbJn/mWd18Tr8WMWfX
6DWpNyNLLcf9D5iplB/kq/oNqLZnLdZhCIYizflSTuUE53rMf8Fw87Pkw3i9
3gieLR/JbfrXos8/GpGhf1R9iXgpDtmXYuYwb9BrF7/zffijly6+pVbvnlfQ
gJ1qjXwNV1EiLzH72c4u5Ee7kab+K/p+2PUNcXmWO6VytZ1J/DP67uj1IlE+
RT/Gi53Piz777rQUy9DvWX045mxF9J6Yy+cj8h71r9gnL0e+vRXReXRG/3v5
Y88olz/kH/KX6n+dvRh9LjAW4/FAf6N+G+fZuQiFsf+/Jf8Av6GVmQ==
          "]]},
        RowBox[{"-", "0.24`"}]],
       Annotation[#, -0.24, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl01toz3EYx/HvbLKNHbGiLGuKrbCNrJRStpXSFMppTBOJ2sgOxbUxqZnD
TDtcOF45zC6wLSYX1JTzmMOuXI0UTVLI65uLd5/f8zyf5/k+v+//98+rqVtX
mxBCmMBgWgjXskK4jqrMEPrFm+lWbMFODE0LYUF2CAWYj3s8bzBreggjdBH+
pIdQIv5Lf+MF30s8xXM8w4Q5WzNC2O+sOtRiDe85tQ7MEOdgJsrMzI57qefb
4Qbt1XsT69Xuq53W0ybfRi86u0/tFe9xuTFaLveIjmIEpfr2yO1Dl/gArcVy
/hq6C1/MqKZV6EEzWvCJv5WeQDt/B+3EuPxZegorzJ8Q/8QFVMT7oCv5v8V+
sy/Ffvt2yp2hQ97jATboHVAfRJ9aIf8tmhvvSK2TtwtzxT20G5Xq9eIGNGKH
3h/u+J3ae4ziIz4gUW0yljo/iRaZ+RazxaO0GdVmLOMdN6OU7rXDNmzHRuTK
3eHbFHfj7cWQuFD+s10akWFeYfymnFFA08Xf5Yd5MtATczQLmRHxQ967nvvj
fAxiAL/kFyKd5zHNoO3OXO38bs9p8U7ivSHVOXn0ivpVtOIkkuM78k2hjbQJ
Daiz02EzDyE3fjd4jRIUI4l/t3MS6SSagBQkY47cbXQgxazz8XxaTpv0Nns+
iiNowTFUqNXrbcBBlMmtirvwJ6gV2bUYi/HVbk/khzFPbQm9TNfqq8RY/L3F
Od45VTwV+dn//8v/AGu3hZc=
          "]]},
        RowBox[{"-", "0.36`"}]],
       Annotation[#, -0.36, "Tooltip"]& ], {}, {}}}]},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1.5, 
      FormBox[
       RowBox[{"-", "1.5`"}], TraditionalForm]}, {-1.25, 
      FormBox[
       RowBox[{"-", "1.25`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.75, 
      FormBox[
       RowBox[{"-", "0.75`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.25, 
      FormBox[
       RowBox[{"-", "0.25`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.25, 
      FormBox["0.25`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.75, 
      FormBox["0.75`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}, {1.25, 
      FormBox["1.25`", TraditionalForm]}, {1.5, 
      FormBox["1.5`", TraditionalForm]}}, {{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2, 
      FormBox["1.2`", TraditionalForm]}}, None, None},
  ImageMargins->0.,
  ImageSize->504,
  Method->{"TransparentPolygonMesh" -> True},
  PlotRange->{{-1.6920937271229854`, 
   1.6920937271229854`}, {-1.3920937271229854`, 1.3920937271229854`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 421},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->65939803]
}, Open  ]],

Cell["\<\
Arrows for velocity field are green, equipotential lines are red, and \
streamlines are blue.  Note orthogonality of equipotential lines and \
streamlines, streamlines everywhere parallel to the velocity field.\
\>", "ExampleText",
 CellID->680409942]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Applications",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->258228157],

Cell[TextData[{
 "The influence coefficients for a point doublet are used to formulate \
potential flow over a circular cylinder in the tutorial ",
 ButtonBox["2.2.1 Uniform Flow Over a Circular Cylinder Without Circulation",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/PointSingularities#100587181"],
 "."
}], "ExampleText",
 CellID->159200508]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[{
 StyleBox[ButtonBox["ICDoubletConstant",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletConstant"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICDoubletLinear",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletLinear"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICDoubletQuadratic",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletQuadratic"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICVortexPoint",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICVortexPoint"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICSourcePoint",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICSourcePoint"],
  FontFamily->"Verdana"],
 " "
}], "SeeAlso",
 CellID->745892494]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["2.1 Introduction to Influence Coefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"]], \
"Tutorials",
 CellID->379763995],

Cell[TextData[ButtonBox["2.2 Point Singularities",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/PointSingularities"]], "Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["InfluenceCoefficients",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/InfluenceCoefficients"]], "MoreAbout",
 CellID->295956244]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"ICDoubletPoint - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 9, 59, 32.7456000}", "context" -> 
    "Aerodynamics`InfluenceCoefficients`", "keywords" -> {}, "index" -> True, 
    "label" -> "Aerodynamics Package Paclet Symbol", "language" -> "en", 
    "paclet" -> "Aerodynamics Package", "status" -> "None", "summary" -> 
    "ICDoubletPoint[r_f, r_p, u_d] computes velocity influence coefficients \
at field points r_f, due to doublets at r_p. Orientation of doublets is \
specified by unit vectors u_d.ICPhiDoubletPoint[r_f, r_p, u_d] computes \
velocity-potential influence coefficients at field points r_f, due to \
doublets at r_p. Orientation of doublets is specified by unit vectors \
u_d.ICPsiDoubletPoint[r_f, r_p, u_d] computes stream-function influence \
coefficients at field points r_f, due to doublets at r_p. Orientation of \
doublets is specified by unit vectors u_D.", "synonyms" -> {}, "title" -> 
    "ICDoubletPoint", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/ICDoubletPoint"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[9964, 300, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->281944934]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 183518, 3690}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[607, 21, 76, 1, 70, "SymbolColorBar"],
Cell[686, 24, 191, 5, 70, "LinkTrail"],
Cell[880, 31, 3678, 83, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4583, 118, 57, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4643, 121, 2867, 91, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[7547, 217, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->592567457],
Cell[8130, 236, 364, 11, 70, "Notes",
 CellID->1067943069],
Cell[8497, 249, 247, 7, 70, "Notes",
 CellID->492519275],
Cell[8747, 258, 433, 10, 70, "Notes",
 CellID->30234373],
Cell[9183, 270, 667, 22, 70, "Notes",
 CellID->35104674],
Cell[9853, 294, 74, 1, 70, "Notes",
 CellID->529540534]
}, Closed]],
Cell[CellGroupData[{
Cell[9964, 300, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->281944934],
Cell[CellGroupData[{
Cell[10450, 317, 148, 5, 70, "ExampleSection",
 CellID->317889308],
Cell[10601, 324, 128, 6, 70, "ExampleText",
 CellID->513831186],
Cell[10732, 332, 100, 3, 70, "Input",
 CellID->267685788],
Cell[10835, 337, 458, 11, 70, "ExampleText",
 CellID->659456463],
Cell[11296, 350, 390, 14, 70, "Input",
 CellID->64953589],
Cell[11689, 366, 82, 1, 70, "ExampleText",
 CellID->326501090],
Cell[CellGroupData[{
Cell[11796, 371, 222, 7, 70, "Input",
 CellID->223030288],
Cell[12021, 380, 732, 26, 62, "Output",
 CellID->88035154]
}, Open  ]],
Cell[12768, 409, 91, 1, 70, "ExampleText",
 CellID->44302699],
Cell[CellGroupData[{
Cell[12884, 414, 181, 5, 70, "Input",
 CellID->575890672],
Cell[13068, 421, 341, 12, 56, "Output",
 CellID->106116437]
}, Open  ]],
Cell[13424, 436, 89, 1, 70, "ExampleText",
 CellID->472419597],
Cell[CellGroupData[{
Cell[13538, 441, 179, 5, 70, "Input",
 CellID->8664355],
Cell[13720, 448, 341, 12, 56, "Output",
 CellID->134063245]
}, Open  ]],
Cell[14076, 463, 203, 4, 70, "ExampleText",
 CellID->9427774],
Cell[14282, 469, 396, 12, 70, "Input",
 CellID->136424127],
Cell[14681, 483, 967, 35, 70, "ExampleText",
 CellID->78226744],
Cell[CellGroupData[{
Cell[15673, 522, 298, 10, 70, "Input",
 CellID->13857323],
Cell[15974, 534, 209, 7, 36, "Output",
 CellID->612803658]
}, Open  ]],
Cell[CellGroupData[{
Cell[16220, 546, 363, 13, 70, "Input",
 CellID->692553449],
Cell[16586, 561, 209, 7, 36, "Output",
 CellID->216960717]
}, Open  ]],
Cell[16810, 571, 468, 9, 70, "ExampleText",
 CellID->78531558],
Cell[CellGroupData[{
Cell[17303, 584, 3845, 108, 70, "Input",
 CellID->1746043248],
Cell[21151, 694, 156780, 2822, 70, "Output",
 CellID->65939803]
}, Open  ]],
Cell[177946, 3519, 261, 5, 70, "ExampleText",
 CellID->680409942]
}, Open  ]],
Cell[CellGroupData[{
Cell[178244, 3529, 227, 7, 70, "ExampleSection",
 CellID->258228157],
Cell[178474, 3538, 366, 8, 70, "ExampleText",
 CellID->159200508]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[178889, 3552, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[179309, 3566, 809, 27, 70, "SeeAlso",
 CellID->745892494]
}, Open  ]],
Cell[CellGroupData[{
Cell[180155, 3598, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[180577, 3612, 210, 5, 70, "Tutorials",
 CellID->379763995],
Cell[180790, 3619, 169, 3, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[CellGroupData[{
Cell[180996, 3627, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[181418, 3641, 167, 3, 70, "MoreAbout",
 CellID->295956244]
}, Open  ]],
Cell[181600, 3647, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

