(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    244568,       5175]
NotebookOptionsPosition[    237110,       4935]
NotebookOutlinePosition[    239244,       4988]
CellTagsIndexPosition[    239157,       4983]
WindowTitle->ICSourceConstant - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"2.1 Introduction to InfluenceCoefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"], "\
\<\"2.3 Isolated Panel\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IsolatedPanel"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"ICSourcePoint\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICSourcePoint"], "\<\"ICSourceLinear\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICSourceLinear"], "\<\"ICDoubletConstant\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICDoubletConstant"], \
"\<\"ICVortexConstant\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICVortexConstant"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"InfluenceCoefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/InfluenceCoefficients"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/ICSourceConstant\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/ICSourceConstant"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
ICSourceConstant.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/ICSourceConstant", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["ICSourceConstant", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICSourceConstant", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes velocity influence coefficients at field \
points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     ", due to connected source panels of constant strength whose endpoints \
are located by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICPhiSourceConstant", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]",
     "computes velocity-potential influence coefficients at field points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     " due to connected source panels of constant strength whose endpoints \
are located by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICPsiVortexConstant", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes stream-function influence coefficients at \
field points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     " due to connected vortex panels of constant strength whose endpoints \
are located by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, 
   "Rows" -> {None, None, {None}}, "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->285780094],

Cell[TextData[{
 "Input arguments are collections of vectors of the data type ",
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], "InlineFormula"],
 ".  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation may be used for single vectors."
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 "The source density for a source panel of constant strength is written as ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["\[Sigma]", "TR"], 
    RowBox[{"(", 
     StyleBox["x", "TI"], 
     StyleBox[")", "TI"]}]}], 
   StyleBox["=", "TI"], 
   StyleBox["a", "TI"]}]], "InlineFormula"],
 ", where ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " is the strength parameter for the panel.  ",
 "For collections of panels, strength parameters are written in a list \
ordered according to the clockwise numbering convention for panels."
}], "Notes",
 CellID->1016887069],

Cell[TextData[{
 "Output for velocity influence coefficients is a collection of vectors of \
the data type CartesianVectors.  Influence coefficients for velocity \
potential and stream function are lists.  For the special case of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation being used to specify a single field point, ",
 "output is simplified as illustrated in the examples",
 "."
}], "Notes",
 CellID->30234373],

Cell[TextData[{
 "The velocity, velocity potential and stream function for a panel are \
written as products of a geometric influence coefficient and the strength \
parameter.\t\t\t\t\t\t\t\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["V", "TI"], "=", 
   RowBox[{
    StyleBox["a", "TI"], " ", 
    StyleBox["ic", "TI"]}]}]], "InlineFormula"],
 "\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Phi]", "TR"], "=", 
   RowBox[{
    StyleBox["a", "TI"], " ", 
    StyleBox["ic\[Phi]", "TI"]}]}]], "InlineFormula"],
 "\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Psi]", "TR"], "=", 
   RowBox[{
    StyleBox["a", "TI"], " ", 
    StyleBox["ic\[Psi]", "TI"]}]}]], "InlineFormula"]
}], "Notes",
 CellID->223280793],

Cell["New in 1.0   |   Last modified in 1.2", "Notes",
 CellID->529540534]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->158617825],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->78764249],

Cell[TextData[{
 "Load ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->513831186],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->267685788],

Cell[TextData[{
 "This example demonstrates some properties of a single panel.  Algebraic \
equations for influence coefficients are simplest in a local coordinate \
system where a panel extends from the origin along the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis to ",
 Cell[BoxData[
  StyleBox["\[Delta]", "TR"]], "InlineFormula"],
 ", the length of the panel."
}], "ExampleText",
 CellID->659456463],

Cell[TextData[{
 "Specify an arbitrary field point and panel endpoints.  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation is used for a single vector."
}], "ExampleText",
 CellID->42111717],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"rF", ",", "rP"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"x", ",", "z"}], "}"}], ",", 
     RowBox[{"MakeCartesianVectors", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "\[Delta]"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}]}], 
  ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->64953589],

Cell["Compute the velocity influence coefficients.", "ExampleText",
 CellID->326501090],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"icu", ",", "icw"}], "}"}], "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"ICSourceConstant", "[", 
     RowBox[{"rF", ",", "rP"}], "]"}], ",", 
    RowBox[{"\[Delta]", ">", "0"}]}], "]"}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->560964283],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"Log", "[", 
      FractionBox[
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"], "-", 
        RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
        SuperscriptBox["\[Delta]", "2"]}], 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"]}]], "]"}], 
     RowBox[{"4", " ", "\[Pi]"}]]}], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"-", 
      RowBox[{"ArcTan", "[", 
       RowBox[{"x", ",", "z"}], "]"}]}], "+", 
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}]}], 
    RowBox[{"2", " ", "\[Pi]"}]]}], "}"}]], "Output",
 ImageSize->{355, 46},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->427323328]
}, Open  ]],

Cell[TextData[{
 "Display the algebraic equations for components of the velocity influence \
coefficient, replacing ArcTan function by named angles (See ",
 ButtonBox["2.1 Introduction to InfluenceCoefficients",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"],
 " for sketch defining angles).  Although the substitution of named angles is \
made to simplify the appearance of equations, note that the original \
equations are maintained for subsequent computations."
}], "ExampleText",
 CellID->334847722],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"icu", ",", "icw"}], "}"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
     "\[Theta]2"}]}], "}"}]}]], "Input",
 CellLabel->"In[4]:=",
 CellID->176773044],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"Log", "[", 
      FractionBox[
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"], "-", 
        RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
        SuperscriptBox["\[Delta]", "2"]}], 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"]}]], "]"}], 
     RowBox[{"4", " ", "\[Pi]"}]]}], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"-", "\[Theta]1"}], "+", "\[Theta]2"}], 
    RowBox[{"2", " ", "\[Pi]"}]]}], "}"}]], "Output",
 ImageSize->{201, 46},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->718515614]
}, Open  ]],

Cell["\<\
Compute the velocity-potential influence coefficient and display in terms of \
named angles.\
\>", "ExampleText",
 CellID->44302699],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ic\[Phi]", "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"ICPhiSourceConstant", "[", 
      RowBox[{"rF", ",", "rP"}], "]"}], ",", 
     RowBox[{"\[Delta]", ">", "0"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ic\[Phi]", "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"ArcTan", "[", 
       RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
     RowBox[{
      RowBox[{"ArcTan", "[", 
       RowBox[{
        RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
      "\[Theta]2"}]}], "}"}]}], "//", "Simplify"}]}], "Input",
 CellLabel->"In[5]:=",
 CellID->575890672],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"2", " ", "z", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "\[Theta]1"}], "+", "\[Theta]2"}], ")"}]}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", 
     RowBox[{
      SuperscriptBox["x", "2"], "+", 
      SuperscriptBox["z", "2"]}], "]"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "x"}], "+", "\[Delta]"}], ")"}], " ", 
    RowBox[{"Log", "[", 
     RowBox[{
      SuperscriptBox["x", "2"], "+", 
      SuperscriptBox["z", "2"], "-", 
      RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
      SuperscriptBox["\[Delta]", "2"]}], "]"}]}]}], 
  RowBox[{"4", " ", "\[Pi]"}]]], "Output",
 ImageSize->{394, 37},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->440902438]
}, Open  ]],

Cell["\<\
Compute and display the stream-function influence coefficient.\
\>", "ExampleText",
 CellID->472419597],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ic\[Psi]", "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"ICPsiSourceConstant", "[", 
      RowBox[{"rF", ",", "rP"}], "]"}], ",", 
     RowBox[{"\[Delta]", ">", "0"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"ic\[Psi]", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
     "\[Theta]2"}]}], "}"}]}]}], "Input",
 CellLabel->"In[6]:=",
 CellID->8664355],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{
    RowBox[{
     RowBox[{"-", "2"}], " ", "x", " ", "\[Theta]1"}], "+", 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{"x", "-", "\[Delta]"}], ")"}], " ", "\[Theta]2"}], "+", 
    RowBox[{"z", " ", 
     RowBox[{"Log", "[", 
      FractionBox[
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"], "-", 
        RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
        SuperscriptBox["\[Delta]", "2"]}], 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"]}]], "]"}]}]}], 
   RowBox[{"4", " ", "\[Pi]"}]]}]], "Output",
 ImageSize->{269, 46},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->423449330]
}, Open  ]],

Cell["\<\
Write the velocity components, velocity potential and stream function as the \
product of the strength parameter and the geometric influence coefficients.\
\>", "ExampleText",
 CellID->9427774],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"u", ",", "w", ",", "\[Phi]", ",", "\[Psi]"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", " ", "icu"}], ",", 
     RowBox[{"a", " ", "icw"}], ",", 
     RowBox[{"a", " ", "ic\[Phi]"}], ",", 
     RowBox[{"a", " ", "ic\[Psi]"}]}], "}"}]}], ";"}]], "Input",
 CellLabel->"In[7]:=",
 CellID->258291294],

Cell[TextData[{
 "Verify that the relations between velocity, velocity potential and stream \
function are satisfied.\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["u", "TI"], "=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Phi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["x", "TI"]}]], "=", 
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Psi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["z", "TI"]}]]}]}]], "InlineFormula"],
 "\t\t\t",
 Cell[BoxData[
  RowBox[{"w", "=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Phi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["z", "TI"]}]], "=", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{"\[PartialD]", 
       StyleBox["\[Psi]", "TR"]}], 
      RowBox[{"\[PartialD]", 
       StyleBox["x", "TI"]}]]}]}]}]], "InlineFormula"],
 "\t\t\t\t"
}], "ExampleText",
 CellID->78226744],

Cell[TextData[{
 "To complete the next evaluation, ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  StyleBox["z", "TI"]], "InlineFormula"],
 " need to be declared as real variables."
}], "ExampleText",
 CellID->421446271],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "==", 
      RowBox[{"D", "[", 
       RowBox[{"\[Phi]", ",", "x"}], "]"}]}], ",", 
     RowBox[{"u", "==", 
      RowBox[{"D", "[", 
       RowBox[{"\[Psi]", ",", "z"}], "]"}]}]}], "}"}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"x", ",", "z"}], "}"}], "\[Element]", "Reals"}]}], 
  "]"}]], "Input",
 CellLabel->"In[8]:=",
 CellID->481371799],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->1267074]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"w", "\[Equal]", 
     RowBox[{"D", "[", 
      RowBox[{"\[Phi]", ",", "z"}], "]"}]}], ",", 
    RowBox[{"w", "\[Equal]", 
     RowBox[{"-", 
      RowBox[{"D", "[", 
       RowBox[{"\[Psi]", ",", "x"}], "]"}]}]}]}], "}"}], "//", 
  "Simplify"}]], "Input",
 CellLabel->"In[9]:=",
 CellID->692553449],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->107032324]
}, Open  ]],

Cell[TextData[{
 "Flow visualization provides plausibility check on the computed flow field.  \
Set ",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Delta]", "TR"], "=", "1"}]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  RowBox[{
   StyleBox["a", "TI"], " ", "=", " ", "1"}]], "InlineFormula"],
 "  to provide unit integrated panel strength, ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", 
     StyleBox["\[Delta]", "TR"]], 
    RowBox[{"\[Sigma]", 
     RowBox[{"(", 
      StyleBox["x", "TI"], ")"}], 
     RowBox[{"\[DifferentialD]", 
      StyleBox["x", "TI"]}]}]}], "=", "1"}]], "InlineFormula"],
 ".   ",
 "Circular regions near the edges of the panel have been excluded from the \
vector plot to avoid showing vectors of very large magnitude."
}], "ExampleText",
 CellID->78531558],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xMin", ",", "xMax", ",", "zMin", ",", "zMax"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "2", ",", 
     RowBox[{"-", "1.2"}], ",", "1.2"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"uPlot", ",", "wPlot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"u", ",", "w"}], "}"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", "\[Rule]", "1"}], ",", 
      RowBox[{"\[Delta]", "\[Rule]", "1"}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Phi]Plot", ",", "\[Psi]Plot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\[Phi]", ",", "\[Psi]"}], "}"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", "\[Rule]", "1"}], ",", 
      RowBox[{"\[Delta]", "\[Rule]", "1"}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gV", "=", 
   RowBox[{"VectorPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"uPlot", ",", "wPlot"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"RegionFunction", "\[Rule]", 
      RowBox[{"Function", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"x", ",", "z"}], "}"}], ",", 
        RowBox[{
         RowBox[{"0.005", "<", 
          RowBox[{
           RowBox[{"x", "^", "2"}], "+", 
           RowBox[{"z", "^", "2"}]}]}], " ", "&&", " ", 
         RowBox[{"0.005", "<", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"x", "-", "1"}], ")"}], "^", "2"}], "+", 
           RowBox[{"z", "^", "2"}]}]}]}]}], "]"}]}], ",", 
     RowBox[{"VectorColorFunction", "->", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"RGBColor", "[", 
         RowBox[{"0", ",", "1", ",", "0"}], "]"}], "&"}], ")"}]}], ",", 
     RowBox[{"VectorPoints", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"18", ",", "18"}], "}"}]}], ",", 
     RowBox[{"VectorScale", "\[Rule]", "Medium"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Phi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Phi]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "7"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Red"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Psi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "13"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Blue"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gPanel", "=", 
   RowBox[{"Graphics", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"GrayLevel", "[", "0", "]"}], ",", 
      RowBox[{"AbsoluteThickness", "[", "3", "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"Transpose", "[", 
        RowBox[{"Components", "[", 
         RowBox[{"rP", "/.", 
          RowBox[{"\[Delta]", "\[Rule]", "1.0"}]}], "]"}], "]"}], "]"}]}], 
     "}"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gV", ",", "g\[Phi]", ",", "g\[Psi]", ",", "gPanel", ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
   RowBox[{"FrameTicks", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.0"}], ",", "2.0", ",", "0.25"}], "]"}], "]"}], ",", 
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.2"}], ",", "1.2", ",", "0.2"}], "]"}], "]"}], ",", 
      "None", ",", "None"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", " ", "72"}]}], ",", 
   RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellLabel->"In[10]:=",
 CellID->1746043248],

Cell[BoxData[
 GraphicsBox[{{{}, 
    {Hue[0.67, 0.6, 0.6], 
     {Arrowheads[{{0.009116798619914617, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9807338717514044, -1.1838697719654656`}, \
{-1.0192661282485957`, -1.2161302280345343`}}]}}, 
     {Arrowheads[{{0.009569687754266346, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9788199149169332, -1.0431054477318142`}, \
{-1.0211800850830668`, -1.0745416110917152`}}]}}, 
     {Arrowheads[{{0.01003260438058273, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9767770061484123, -0.9026378443014492}, \
{-1.0232229938515875`, -0.9326562733456096}}]}}, 
     {Arrowheads[{{0.010494535920320246`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9746523699480947, -0.7625382505618518}, \
{-1.0253476300519053`, -0.7904029259087367}}]}}, 
     {Arrowheads[{{0.010939521084517023`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9725254688599791, -0.6228758215402782}, \
{-1.0274745311400209`, -0.6477124137538397}}]}}, 
     {Arrowheads[{{0.011346123674529836`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9705141070200185, -0.4837023461393182}, \
{-1.0294858929799815`, -0.5045329479783293}}]}}, 
     {Arrowheads[{{0.011688143720156748`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9687725946320472, -0.34502979430757774`}, \
{-1.0312274053679529`, -0.3608525586335991}}]}}, 
     {Arrowheads[{{0.011937480299842693`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9674747188902506, -0.20680544893832}, \
{-1.0325252811097494`, -0.21672396282638628`}}]}}, 
     {Arrowheads[{{0.01206948107950648, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9667780233212557, -0.06889697276363918}, \
{-1.0332219766787443`, -0.0722794978245965}}]}}, 
     {Arrowheads[{{0.012069481079506476`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9667780233212557, 
        0.06889697276363882}, {-1.0332219766787443`, 0.0722794978245961}}]}}, 
     {Arrowheads[{{0.011937480299842693`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9674747188902506, 
        0.20680544893831962`}, {-1.0325252811097494`, 0.2167239628263859}}]}}, 
     {Arrowheads[{{0.011688143720156748`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9687725946320472, 
        0.34502979430757735`}, {-1.0312274053679529`, 0.3608525586335987}}]}}, 
     {Arrowheads[{{0.011346123674529841`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9705141070200185, 
        0.4837023461393178}, {-1.0294858929799815`, 0.5045329479783289}}]}}, 
     {Arrowheads[{{0.010939521084517023`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9725254688599791, 
        0.6228758215402778}, {-1.0274745311400209`, 0.6477124137538393}}]}}, 
     {Arrowheads[{{0.010494535920320246`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9746523699480947, 
        0.7625382505618513}, {-1.0253476300519053`, 0.7904029259087363}}]}}, 
     {Arrowheads[{{0.01003260438058273, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9767770061484123, 
        0.9026378443014489}, {-1.0232229938515875`, 0.9326562733456093}}]}}, 
     {Arrowheads[{{0.009569687754266346, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9788199149169332, 
        1.0431054477318138`}, {-1.0211800850830668`, 1.0745416110917148`}}]}}, 
     {Arrowheads[{{0.009116798619914617, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9807338717514044, 
        1.1838697719654652`}, {-1.0192661282485957`, 1.2161302280345339`}}]}}, 
     {Arrowheads[{{0.0097764739834501, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8040469254264953, -1.181386143062818}, \
{-0.8430118981029164, -1.2186138569371818`}}]}}, 
     {Arrowheads[{{0.010342744314054715`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.801790582402836, -1.0403841856980935`}, \
{-0.8452682411265758, -1.0772628731254361`}}]}}, 
     {Arrowheads[{{0.010937677956452644`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7992987754129584, -0.899712904175814}, \
{-0.8477600481164532, -0.935581213471245}}]}}, 
     {Arrowheads[{{0.011550188090361797`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7966059841827857, -0.7594847871806495}, \
{-0.8504528393466261, -0.793456389289939}}]}}, 
     {Arrowheads[{{0.01216112908340845, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7937938880921884, -0.6198271343117402}, \
{-0.8532649354372233, -0.6507611009823778}}]}}, 
     {Arrowheads[{{0.012740596380756662`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7910124698751128, -0.48086287631464036`}, \
{-0.8560463536542989, -0.5073724178030071}}]}}, 
     {Arrowheads[{{0.013246569582145843`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7884945676311019, -0.3426707593544241}, \
{-0.8585642558983099, -0.3632115935867528}}]}}, 
     {Arrowheads[{{0.013627849871941577`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7865430632708874, -0.20522498643722126`}, \
{-0.8605157602585243, -0.218304425327485}}]}}, 
     {Arrowheads[{{0.013834467068875216`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7854663260855013, -0.06833838779139521}, \
{-0.8615924974439104, -0.07283808279684048}}]}}, 
     {Arrowheads[{{0.013834467068875216`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7854663260855013, 
        0.06833838779139484}, {-0.8615924974439104, 0.07283808279684006}}]}}, 
     {Arrowheads[{{0.0136278498719416, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7865430632708874, 
        0.20522498643722087`}, {-0.8605157602585244, 
        0.21830442532748462`}}]}}, 
     {Arrowheads[{{0.013246569582145843`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7884945676311019, 
        0.3426707593544237}, {-0.8585642558983099, 0.3632115935867524}}]}}, 
     {Arrowheads[{{0.012740596380756657`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7910124698751128, 
        0.48086287631464}, {-0.8560463536542989, 0.5073724178030067}}]}}, 
     {Arrowheads[{{0.01216112908340845, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7937938880921884, 
        0.6198271343117397}, {-0.8532649354372233, 0.6507611009823774}}]}}, 
     {Arrowheads[{{0.011550188090361797`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7966059841827857, 
        0.759484787180649}, {-0.8504528393466261, 0.7934563892899386}}]}}, 
     {Arrowheads[{{0.010937677956452644`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7992987754129584, 
        0.8997129041758135}, {-0.8477600481164532, 0.9355812134712446}}]}}, 
     {Arrowheads[{{0.010342744314054715`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.801790582402836, 
        1.040384185698093}, {-0.8452682411265758, 1.0772628731254357`}}]}}, 
     {Arrowheads[{{0.009776473983450126, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8040469254264953, 
        1.1813861430628176`}, {-0.8430118981029164, 1.2186138569371816`}}]}}, 
     {Arrowheads[{{0.010477306125975746`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6277414377817475, -1.1785359331371488`}, \
{-0.6663762092770759, -1.221464066862851}}]}}, 
     {Arrowheads[{{0.011183628626149165`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.625138516318523, -1.0371535747662564`}, \
{-0.6689791307403005, -1.080493484057273}}]}}, 
     {Arrowheads[{{0.011949495453044802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6221500641505607, -0.8961012470138565}, \
{-0.6719675829082626, -0.9391928706332024}}]}}, 
     {Arrowheads[{{0.012768289697852271`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6187690551976331, -0.7555398540556062}, \
{-0.6753485918611902, -0.7974013224149822}}]}}, 
     {Arrowheads[{{0.013622335338254642`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6150435971751111, -0.615681833577597}, \
{-0.6790740498837122, -0.654906401716521}}]}}, 
     {Arrowheads[{{0.014475369768919458`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6111259556165298, -0.4767823051369137}, \
{-0.6829916914422935, -0.5114529889807338}}]}}, 
     {Arrowheads[{{0.015263437675282373`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6073372076773217, -0.33908820015721075`}, \
{-0.6867804393815017, -0.3667941527839661}}]}}, 
     {Arrowheads[{{0.015890646781373582`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6042084973358147, -0.20270973761578687`}, \
{-0.6899091497230088, -0.2208196741489194}}]}}, 
     {Arrowheads[{{0.01624493513422706, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6023976663336901, -0.06742454185633001}, \
{-0.6917199807251334, -0.07375192873190567}}]}}, 
     {Arrowheads[{{0.01624493513422706, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6023976663336901, 
        0.06742454185632964}, {-0.6917199807251334, 0.07375192873190525}}]}}, 
     {Arrowheads[{{0.015890646781373582`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6042084973358147, 
        0.20270973761578648`}, {-0.6899091497230088, 0.220819674148919}}]}}, 
     {Arrowheads[{{0.015263437675282373`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6073372076773217, 
        0.33908820015721036`}, {-0.6867804393815017, 0.3667941527839657}}]}}, 
     {Arrowheads[{{0.014475369768919472`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6111259556165297, 
        0.47678230513691333`}, {-0.6829916914422935, 0.5114529889807333}}]}}, 
     {Arrowheads[{{0.013622335338254642`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6150435971751111, 
        0.6156818335775965}, {-0.6790740498837122, 0.6549064017165206}}]}}, 
     {Arrowheads[{{0.012768289697852282`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6187690551976331, 
        0.7555398540556058}, {-0.6753485918611902, 0.7974013224149819}}]}}, 
     {Arrowheads[{{0.011949495453044802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6221500641505607, 
        0.8961012470138561}, {-0.6719675829082626, 0.939192870633202}}]}}, 
     {Arrowheads[{{0.011183628626149165`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.625138516318523, 
        1.037153574766256}, {-0.6689791307403005, 1.0804934840572726`}}]}}, 
     {Arrowheads[{{0.01047730612597576, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6277414377817475, 
        1.1785359331371483`}, {-0.666376209277076, 1.2214640668628505`}}]}}, 
     {Arrowheads[{{0.011199963345734225`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.45199034966053925`, -1.175362977900796}, \
{-0.4891861209276958, -1.224637022099204}}]}}, 
     {Arrowheads[{{0.012072512361463282`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4490964672384976, -1.033422350108421}, \
{-0.49208000334973745`, -1.0842247087151085`}}]}}, 
     {Arrowheads[{{0.013051586754250093`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4456318649600362, -0.8917404906758423}, \
{-0.49554460562819885`, -0.9435536269712167}}]}}, 
     {Arrowheads[{{0.014144384224217989`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4415037498304014, -0.7505125291145733}, \
{-0.49967272075783364`, -0.8024286473560153}}]}}, 
     {Arrowheads[{{0.015348449856734211`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4366504558557032, -0.6100412537619427}, \
{-0.5045260147325319, -0.6605469815321754}}]}}, 
     {Arrowheads[{{0.01663799156292185, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4311153562503576, -0.4707788430506923}, \
{-0.5100611143378775, -0.5174564510669551}}]}}, 
     {Arrowheads[{{0.017936440162617952`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.425207147559441, -0.33333548645830935`}, \
{-0.5159693230287941, -0.3725468664828675}}]}}, 
     {Arrowheads[{{0.01907414277996193, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.41976679747917284`, -0.1983162046793514}, \
{-0.5214096731090622, -0.22521320708535486`}}]}}, 
     {Arrowheads[{{0.0197727696342968, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.41630829113565243`, -0.0657378637744822}, \
{-0.5248681794525827, -0.07543860681375349}}]}}, 
     {Arrowheads[{{0.0197727696342968, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.41630829113565243`, 
        0.06573786377448183}, {-0.5248681794525827, 0.07543860681375308}}]}}, 
     {Arrowheads[{{0.019074142779961962`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4197667974791728, 
        0.19831620467935102`}, {-0.5214096731090623, 
        0.22521320708535447`}}]}}, 
     {Arrowheads[{{0.01793644016261798, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.42520714755944095`, 
        0.33333548645830896`}, {-0.5159693230287942, 
        0.37254686648286706`}}]}}, 
     {Arrowheads[{{0.016637991562921853`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4311153562503576, 
        0.4707788430506919}, {-0.5100611143378775, 0.5174564510669548}}]}}, 
     {Arrowheads[{{0.015348449856734196`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4366504558557032, 
        0.6100412537619423}, {-0.5045260147325318, 0.660546981532175}}]}}, 
     {Arrowheads[{{0.014144384224217989`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4415037498304014, 
        0.7505125291145729}, {-0.49967272075783364`, 0.8024286473560148}}]}}, 
     {Arrowheads[{{0.013051586754250093`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4456318649600362, 
        0.8917404906758418}, {-0.49554460562819885`, 0.9435536269712163}}]}}, 
     {Arrowheads[{{0.012072512361463311`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4490964672384976, 
        1.0334223501084205`}, {-0.49208000334973745`, 
        1.0842247087151082`}}]}}, 
     {Arrowheads[{{0.011199963345734225`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.45199034966053925`, 
        1.1753629779007955`}, {-0.4891861209276958, 1.2246370220992036`}}]}}, 
     {Arrowheads[{{0.011912824053351102`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2769808901006054, -1.1719937029687708`}, \
{-0.3112544040170414, -1.228006297031229}}]}}, 
     {Arrowheads[{{0.01297064522774105, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.27394540027974335`, -1.0293099497731446`}, \
{-0.31428989383790334`, -1.0883371090503848`}}]}}, 
     {Arrowheads[{{0.014198637077553295`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.27015835485833006`, -0.8867058067067692}, \
{-0.3180769392593167, -0.9485883109402897}}]}}, 
     {Arrowheads[{{0.01563161768674601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2654013659875012, -0.744353715163706}, \
{-0.32283392813014555`, -0.8085874613068825}}]}}, 
     {Arrowheads[{{0.017308930966610743`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.25940254153050746`, -0.6025728171064615}, \
{-0.3288327525871393, -0.6680154181876566}}]}}, 
     {Arrowheads[{{0.019265812323821692`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2518642667677338, -0.46195937889901734`}, \
{-0.336371027349913, -0.52627591521863}}]}}, 
     {Arrowheads[{{0.021497164399658767`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.242624229607832, -0.323634918740149}, \
{-0.3456110645098147, -0.3822474342010279}}]}}, 
     {Arrowheads[{{0.0238257974448833, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2323151984543792, -0.1895702053836755}, \
{-0.35592009566326754`, -0.23395920638103077`}}]}}, 
     {Arrowheads[{{0.025567923001979954`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.224188508149462, -0.061887105362916216`}, \
{-0.36404678596818474`, -0.07928936522531947}}]}}, 
     {Arrowheads[{{0.025567923001979975`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.22418850814946195`, 
        0.06188710536291589}, {-0.3640467859681848, 0.07928936522531901}}]}}, 
     {Arrowheads[{{0.0238257974448833, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.23231519845437917`, 
        0.18957020538367514`}, {-0.35592009566326754`, 
        0.23395920638103035`}}]}}, 
     {Arrowheads[{{0.02149716439965878, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.24262422960783198`, 
        0.3236349187401486}, {-0.34561106450981477`, 
        0.38224743420102747`}}]}}, 
     {Arrowheads[{{0.019265812323821702`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.25186426676773377`, 
        0.46195937889901695`}, {-0.336371027349913, 0.5262759152186297}}]}}, 
     {Arrowheads[{{0.01730893096661073, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.25940254153050746`, 
        0.602572817106461}, {-0.3288327525871393, 0.6680154181876561}}]}}, 
     {Arrowheads[{{0.01563161768674601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2654013659875012, 
        0.7443537151637055}, {-0.32283392813014555`, 0.8085874613068821}}]}}, 
     {Arrowheads[{{0.014198637077553295`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.27015835485833006`, 
        0.8867058067067688}, {-0.3180769392593167, 0.9485883109402893}}]}}, 
     {Arrowheads[{{0.012970645227741053`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.27394540027974335`, 
        1.0293099497731442`}, {-0.3142898938379034, 1.0883371090503844`}}]}}, 
     {Arrowheads[{{0.011912824053351102`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.27698089010060534`, 
        1.1719937029687704`}, {-0.31125440401704135`, 
        1.2280062970312287`}}]}}, 
     {Arrowheads[{{0.012572289032024985`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.10286886640235525`, -1.16865860643912}, \
{-0.13242525124470325`, -1.2313413935608801`}}]}}, 
     {Arrowheads[{{0.013818604383719655`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.0999518562430109, -1.025098049253899}, \
{-0.13534226140404762`, -1.0925490095696302`}}]}}, 
     {Arrowheads[{{0.01530879780135258, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.09618119847300861, -0.8813226496434575}, \
{-0.1391129191740499, -0.9539714680036014}}]}}, 
     {Arrowheads[{{0.017116898560857404`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.09122496683432636, -0.7373876325651155}, \
{-0.14406915081273214`, -0.815553543905473}}]}}, 
     {Arrowheads[{{0.019351722288825795`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.08458181304088924, -0.59344446591864}, \
{-0.15071230460616925`, -0.677143769375478}}]}}, 
     {Arrowheads[{{0.022183664080880638`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.07545659418490722, -0.4498663135175107}, \
{-0.15983752346215127`, -0.5383689806001367}}]}}, 
     {Arrowheads[{{0.025900636613884826`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.06249876026141976, -0.30761415079728877`}, \
{-0.17279535738563873`, -0.39826820214388803`}}]}}, 
     {Arrowheads[{{0.030997853497116958`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.04331785477784088, -0.16964427604576146`}, \
{-0.1919762628692176, -0.2538851357189448}}]}}, 
     {Arrowheads[{{0.03754378372508139, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.016746676591492704`, -0.04764781363693983}, \
{-0.2185474410555658, -0.09352865695129585}}]}}, 
     {Arrowheads[{{0.0375437837250814, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.01674667659149265, 
        0.047647813636939526`}, {-0.21854744105556587`, 
        0.09352865695129538}}]}}, 
     {Arrowheads[{{0.03099785349711698, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.04331785477784081, 
        0.1696442760457611}, {-0.19197626286921765`, 0.2538851357189444}}]}}, 
     {Arrowheads[{{0.02590063661388485, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.06249876026141972, 
        0.3076141507972884}, {-0.17279535738563878`, 0.3982682021438877}}]}}, 
     {Arrowheads[{{0.022183664080880655`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.07545659418490719, 
        0.4498663135175103}, {-0.15983752346215127`, 0.5383689806001364}}]}}, 
     {Arrowheads[{{0.019351722288825802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.08458181304088921, 
        0.5934444659186395}, {-0.15071230460616927`, 0.6771437693754776}}]}}, 
     {Arrowheads[{{0.017116898560857404`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.09122496683432635, 
        0.7373876325651151}, {-0.14406915081273214`, 0.8155535439054725}}]}}, 
     {Arrowheads[{{0.015308797801352584`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.0961811984730086, 
        0.8813226496434571}, {-0.1391129191740499, 0.953971468003601}}]}}, 
     {Arrowheads[{{0.013818604383719655`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.09995185624301088, 
        1.0250980492538986`}, {-0.1353422614040476, 1.0925490095696297`}}]}}, 
     {Arrowheads[{{0.012572289032024982`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.10286886640235525`, 
        1.1686586064391196`}, {-0.13242525124470322`, 
        1.2313413935608797`}}]}}, 
     {Arrowheads[{{0.01312761790541763, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.07028607935322899, -1.1656823597169808`}, {
        0.04736097947030078, -1.2343176402830192`}}]}}, 
     {Arrowheads[{{0.014542990019468325`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.07274129182057976, -1.0212351469461067`}, {
        0.044905767002950014`, -1.096411911877423}}]}}, 
     {Arrowheads[{{0.016272107256669006`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.0759968065403877, -0.8762173814295938}, {
        0.04165025228314206, -0.9590767362174651}}]}}, 
     {Arrowheads[{{0.01842700829757588, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.08040620629958611, -0.7304975373000477}, {
        0.03724085252394366, -0.8224436391705408}}]}}, 
     {Arrowheads[{{0.021183581531526, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.08652876449451788, -0.5839016937607147}, {
        0.0311182943290119, -0.6866865415334032}}]}}, 
     {Arrowheads[{{0.02484493757949048, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.09530166230825723, -0.43616702544173586`}, {
        0.022345396515272543`, -0.5520682686759115}}]}}, 
     {Arrowheads[{{0.030017486788541166`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.10846384031919165`, -0.28675647859228914`}, {
        0.009183218504338127, -0.4191258743488877}}]}}, 
     {Arrowheads[{{0.03825714390929146, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{
        0.1299106071249561, -0.13388960786310047`}, {-0.012263548301426355`, \
-0.2896398039016058}}]}}, 
     {Arrowheads[{{0.0557576247082935, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.17077032920291607`, 
        0.034691933857151275`}, {-0.053123270379386286`, \
-0.17586840444538696`}}]}}, 
     {Arrowheads[{{0.05575762470829358, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{
        0.17077032920291624`, -0.0346919338571518}, {-0.05312327037938645, 
        0.1758684044453867}}]}}, 
     {Arrowheads[{{0.038257143909291504`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.12991060712495622`, 
        0.13388960786310003`}, {-0.012263548301426438`, 
        0.2896398039016055}}]}}, 
     {Arrowheads[{{0.030017486788541194`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.10846384031919168`, 0.2867564785922887}, {
        0.009183218504338085, 0.4191258743488874}}]}}, 
     {Arrowheads[{{0.0248449375794905, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.09530166230825726, 0.43616702544173547`}, {
        0.022345396515272516`, 0.5520682686759112}}]}}, 
     {Arrowheads[{{0.02118358153152602, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.08652876449451789, 0.5839016937607142}, {
        0.03111829432901188, 0.6866865415334029}}]}}, 
     {Arrowheads[{{0.01842700829757588, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.08040620629958611, 0.7304975373000473}, {
        0.03724085252394365, 0.8224436391705403}}]}}, 
     {Arrowheads[{{0.016272107256669006`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.07599680654038772, 0.8762173814295934}, {
        0.04165025228314205, 0.9590767362174647}}]}}, 
     {Arrowheads[{{0.014542990019468325`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.07274129182057976, 1.0212351469461063`}, {
        0.044905767002950014`, 1.0964119118774225`}}]}}, 
     {Arrowheads[{{0.01312761790541763, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.07028607935322899, 1.1656823597169803`}, {
        0.04736097947030078, 1.2343176402830187`}}]}}, 
     {Arrowheads[{{0.013529796440564321`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.24257710334739277`, -1.1634283164953787`}, {
        0.22801113194672523`, -1.2365716835046212`}}]}}, 
     {Arrowheads[{{0.015071701293711243`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2442224795541291, -1.0182548990235811`}, {
        0.22636575573998893`, -1.0993921597999483`}}]}}, 
     {Arrowheads[{{0.016979080664775204`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.24643595345234368`, -0.8721963280379421}, {
        0.22415228184177435`, -0.9630977896091168}}]}}, 
     {Arrowheads[{{0.01938754787014612, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2494747724971715, -0.7249520932131704}, {
        0.22111346279694652`, -0.8279890832574179}}]}}, 
     {Arrowheads[{{0.022503892202001684`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.25372673577531346`, -0.5760728545321517}, {
        0.21686149951880457`, -0.6945153807619663}}]}}, 
     {Arrowheads[{{0.026656977789932944`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2597549925736203, -0.424839216587763}, {
        0.21083324272049775`, -0.5633960775298844}}]}}, 
     {Arrowheads[{{0.03239168694035223, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2682565151000138, -0.26997370156017286`}, {
        0.2023317201941042, -0.4359086513810039}}]}}, 
     {Arrowheads[{{0.04058780375872303, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.279456630241509, -0.10898599215659382`}, {
        0.191131605052609, -0.31454341960811244`}}]}}, 
     {Arrowheads[{{0.05203750106773288, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.29007140791395924`, 0.06196091506845902}, {
        0.18051682738015878`, -0.2031373856566947}}]}}, 
     {Arrowheads[{{0.05203750106773292, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.29007140791395924`, -0.06196091506845952}, {
        0.18051682738015878`, 0.20313738565669442`}}]}}, 
     {Arrowheads[{{0.040587803758723055`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2794566302415091, 0.10898599215659337`}, {
        0.19113160505260895`, 0.3145434196081121}}]}}, 
     {Arrowheads[{{0.03239168694035226, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.26825651510001386`, 0.2699737015601725}, {
        0.2023317201941042, 0.43590865138100365`}}]}}, 
     {Arrowheads[{{0.02665697778993295, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2597549925736203, 0.42483921658776264`}, {
        0.21083324272049775`, 0.5633960775298841}}]}}, 
     {Arrowheads[{{0.022503892202001708`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.25372673577531346`, 0.5760728545321513}, {
        0.21686149951880454`, 0.694515380761966}}]}}, 
     {Arrowheads[{{0.01938754787014612, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.24947477249717154`, 0.72495209321317}, {
        0.22111346279694652`, 0.8279890832574175}}]}}, 
     {Arrowheads[{{0.016979080664775242`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.24643595345234368`, 0.8721963280379417}, {
        0.22415228184177435`, 0.9630977896091166}}]}}, 
     {Arrowheads[{{0.015071701293711243`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2442224795541291, 1.0182548990235807`}, {
        0.22636575573998893`, 1.0993921597999479`}}]}}, 
     {Arrowheads[{{0.013529796440564321`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.24257710334739277`, 1.1634283164953783`}, {
        0.22801113194672523`, 1.2365716835046208`}}]}}, 
     {Arrowheads[{{0.01374099754993978, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4142645887464283, -1.1622106908760546`}, {
        0.40926482301827805`, -1.2377893091239456`}}]}}, 
     {Arrowheads[{{0.015350155243130659`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.41484416434413124`, -1.016628806658473}, {
        0.40868524742057505`, -1.1010182521650567`}}]}}, 
     {Arrowheads[{{0.01735114455407311, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.41562844033926905`, -0.8699814753065289}, {
        0.40790097142543724`, -0.96531264234053}}]}}, 
     {Arrowheads[{{0.019888137372837372`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4167088557354353, -0.7218798286880315}, {
        0.40682055602927103`, -0.8310613477825571}}]}}, 
     {Arrowheads[{{0.023170576259998205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.41821461983240005`, -0.5717596670330394}, {
        0.4053147919323063, -0.6988285682610786}}]}}, 
     {Arrowheads[{{0.027496482743281682`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4203015184883747, -0.4188162627262188}, {
        0.40322789327633163`, -0.5694190313914287}}]}}, 
     {Arrowheads[{{0.03325362540608419, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4230585005155866, -0.26198853491654384`}, {
        0.4004709112491197, -0.4438938180246331}}]}}, 
     {Arrowheads[{{0.04080882460299786, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.42619529480784274`, -0.10022005844338494`}, {
        0.3973341169568635, -0.3233093533213213}}]}}, 
     {Arrowheads[{{0.05010844719022014, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.42855347429657387`, 0.06649264461105947}, {
        0.3949759374681325, -0.20766911519929515`}}]}}, 
     {Arrowheads[{{0.05010844719022016, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.42855347429657387`, -0.06649264461105991}, {
        0.3949759374681324, 0.20766911519929482`}}]}}, 
     {Arrowheads[{{0.04080882460299789, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.42619529480784274`, 0.10022005844338447`}, {
        0.3973341169568635, 0.32330935332132105`}}]}}, 
     {Arrowheads[{{0.03325362540608419, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4230585005155866, 0.2619885349165434}, {
        0.4004709112491197, 0.4438938180246327}}]}}, 
     {Arrowheads[{{0.027496482743281668`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4203015184883747, 0.4188162627262184}, {
        0.40322789327633163`, 0.5694190313914282}}]}}, 
     {Arrowheads[{{0.023170576259998205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.41821461983240005`, 0.5717596670330389}, {
        0.40531479193230624`, 0.6988285682610782}}]}}, 
     {Arrowheads[{{0.019888137372837393`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4167088557354353, 0.7218798286880309}, {0.406820556029271,
         0.8310613477825567}}]}}, 
     {Arrowheads[{{0.01735114455407311, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.41562844033926905`, 0.8699814753065285}, {
        0.40790097142543724`, 0.9653126423405296}}]}}, 
     {Arrowheads[{{0.015350155243130659`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.41484416434413124`, 1.0166288066584726`}, {
        0.408685247420575, 1.1010182521650562`}}]}}, 
     {Arrowheads[{{0.01374099754993978, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4142645887464283, 1.162210690876054}, {
        0.40926482301827805`, 1.2377893091239451`}}]}}, 
     {Arrowheads[{{0.01374099754993978, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5857354112535721, -1.1622106908760546`}, {
        0.5907351769817224, -1.2377893091239456`}}]}}, 
     {Arrowheads[{{0.015350155243130659`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5851558356558691, -1.016628806658473}, {
        0.5913147525794253, -1.1010182521650567`}}]}}, 
     {Arrowheads[{{0.01735114455407311, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5843715596607314, -0.8699814753065289}, {
        0.5920990285745632, -0.96531264234053}}]}}, 
     {Arrowheads[{{0.019888137372837355`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5832911442645651, -0.7218798286880315}, {
        0.5931794439707294, -0.831061347782557}}]}}, 
     {Arrowheads[{{0.023170576259998184`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5817853801676004, -0.5717596670330394}, {
        0.5946852080676942, -0.6988285682610785}}]}}, 
     {Arrowheads[{{0.027496482743281685`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5796984815116257, -0.4188162627262188}, {
        0.5967721067236689, -0.5694190313914287}}]}}, 
     {Arrowheads[{{0.033253625406084196`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5769414994844138, -0.26198853491654384`}, {
        0.5995290887508807, -0.4438938180246331}}]}}, 
     {Arrowheads[{{0.04080882460299786, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5738047051921576, -0.10022005844338494`}, {
        0.6026658830431371, -0.3233093533213213}}]}}, 
     {Arrowheads[{{0.05010844719022013, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5714465257034265, 0.06649264461105944}, {
        0.605024062531868, -0.20766911519929512`}}]}}, 
     {Arrowheads[{{0.05010844719022016, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5714465257034265, -0.06649264461105991}, {
        0.605024062531868, 0.20766911519929482`}}]}}, 
     {Arrowheads[{{0.040808824602997874`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5738047051921576, 0.1002200584433845}, {
        0.6026658830431371, 0.323309353321321}}]}}, 
     {Arrowheads[{{0.03325362540608419, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5769414994844138, 0.2619885349165434}, {
        0.5995290887508807, 0.4438938180246326}}]}}, 
     {Arrowheads[{{0.0274964827432817, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5796984815116257, 0.41881626272621836`}, {
        0.5967721067236689, 0.5694190313914284}}]}}, 
     {Arrowheads[{{0.023170576259998205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5817853801676004, 0.5717596670330389}, {
        0.5946852080676942, 0.6988285682610782}}]}}, 
     {Arrowheads[{{0.019888137372837372`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5832911442645651, 0.7218798286880309}, {
        0.5931794439707294, 0.8310613477825566}}]}}, 
     {Arrowheads[{{0.01735114455407311, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5843715596607314, 0.8699814753065285}, {
        0.5920990285745632, 0.9653126423405296}}]}}, 
     {Arrowheads[{{0.015350155243130659`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5851558356558691, 1.0166288066584726`}, {
        0.5913147525794253, 1.1010182521650562`}}]}}, 
     {Arrowheads[{{0.01374099754993978, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5857354112535722, 1.162210690876054}, {0.5907351769817225,
         1.2377893091239451`}}]}}, 
     {Arrowheads[{{0.013529796440564321`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7574228966526076, -1.1634283164953787`}, {
        0.7719888680532752, -1.2365716835046212`}}]}}, 
     {Arrowheads[{{0.015071701293711243`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7557775204458713, -1.0182548990235811`}, {
        0.7736342442600115, -1.0993921597999483`}}]}}, 
     {Arrowheads[{{0.016979080664775204`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7535640465476567, -0.8721963280379421}, {
        0.7758477181582261, -0.9630977896091168}}]}}, 
     {Arrowheads[{{0.01938754787014612, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7505252275028289, -0.7249520932131704}, {
        0.7788865372030539, -0.8279890832574179}}]}}, 
     {Arrowheads[{{0.022503892202001687`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.746273264224687, -0.5760728545321517}, {
        0.7831385004811959, -0.6945153807619663}}]}}, 
     {Arrowheads[{{0.026656977789932913`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7402450074263802, -0.4248392165877631}, {
        0.7891667572795028, -0.5633960775298843}}]}}, 
     {Arrowheads[{{0.032391686940352236`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7317434848999866, -0.2699737015601729}, {
        0.7976682798058964, -0.43590865138100393`}}]}}, 
     {Arrowheads[{{0.040587803758723034`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7205433697584913, -0.10898599215659387`}, {
        0.8088683949473915, -0.31454341960811244`}}]}}, 
     {Arrowheads[{{0.052037501067732896`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7099285920860411, 0.06196091506845902}, {
        0.8194831726198418, -0.2031373856566947}}]}}, 
     {Arrowheads[{{0.05203750106773293, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7099285920860411, -0.06196091506845949}, {
        0.8194831726198418, 0.2031373856566944}}]}}, 
     {Arrowheads[{{0.040587803758723055`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7205433697584913, 0.10898599215659341`}, {
        0.8088683949473916, 0.31454341960811205`}}]}}, 
     {Arrowheads[{{0.03239168694035225, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7317434848999865, 0.2699737015601725}, {
        0.7976682798058963, 0.43590865138100354`}}]}}, 
     {Arrowheads[{{0.02665697778993295, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7402450074263801, 0.42483921658776264`}, {
        0.7891667572795027, 0.5633960775298841}}]}}, 
     {Arrowheads[{{0.022503892202001694`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.746273264224687, 0.5760728545321513}, {0.783138500481196, 
        0.6945153807619658}}]}}, 
     {Arrowheads[{{0.019387547870146144`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7505252275028289, 0.72495209321317}, {0.778886537203054, 
        0.8279890832574176}}]}}, 
     {Arrowheads[{{0.016979080664775204`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7535640465476567, 0.8721963280379417}, {
        0.7758477181582261, 0.9630977896091164}}]}}, 
     {Arrowheads[{{0.015071701293711243`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7557775204458713, 1.0182548990235807`}, {
        0.7736342442600115, 1.0993921597999479`}}]}}, 
     {Arrowheads[{{0.013529796440564323`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7574228966526076, 1.1634283164953783`}, {
        0.7719888680532753, 1.2365716835046208`}}]}}, 
     {Arrowheads[{{0.01312761790541763, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9297139206467715, -1.1656823597169808`}, {
        0.9526390205296997, -1.2343176402830192`}}]}}, 
     {Arrowheads[{{0.014542990019468325`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9272587081794207, -1.0212351469461067`}, {
        0.9550942329970504, -1.096411911877423}}]}}, 
     {Arrowheads[{{0.016272107256669006`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9240031934596128, -0.8762173814295938}, {
        0.9583497477168584, -0.9590767362174651}}]}}, 
     {Arrowheads[{{0.01842700829757584, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9195937937004144, -0.7304975373000479}, {
        0.9627591474760568, -0.8224436391705406}}]}}, 
     {Arrowheads[{{0.021183581531526007`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9134712355054826, -0.5839016937607148}, {
        0.9688817056709886, -0.6866865415334034}}]}}, 
     {Arrowheads[{{0.02484493757949047, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9046983376917432, -0.4361670254417359}, {
        0.9776546034847279, -0.5520682686759115}}]}}, 
     {Arrowheads[{{0.030017486788541166`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8915361596808088, -0.28675647859228914`}, {
        0.9908167814956623, -0.4191258743488877}}]}}, 
     {Arrowheads[{{0.03825714390929144, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8700893928750443, -0.13388960786310056`}, {
        1.0122635483014268`, -0.28963980390160565`}}]}}, 
     {Arrowheads[{{0.055757624708293534`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8292296707970842, 0.03469193385715108}, {
        1.0531232703793871`, -0.17586840444538676`}}]}}, 
     {Arrowheads[{{0.05575762470829358, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8292296707970841, -0.03469193385715162}, {
        1.0531232703793871`, 0.17586840444538648`}}]}}, 
     {Arrowheads[{{0.038257143909291476`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8700893928750443, 0.13388960786310017`}, {
        1.012263548301427, 0.2896398039016053}}]}}, 
     {Arrowheads[{{0.030017486788541194`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8915361596808088, 0.28675647859228875`}, {
        0.9908167814956625, 0.4191258743488873}}]}}, 
     {Arrowheads[{{0.02484493757949048, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9046983376917432, 0.4361670254417355}, {
        0.9776546034847281, 0.5520682686759111}}]}}, 
     {Arrowheads[{{0.021183581531526007`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9134712355054826, 0.5839016937607142}, {
        0.9688817056709886, 0.6866865415334028}}]}}, 
     {Arrowheads[{{0.01842700829757587, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9195937937004144, 0.7304975373000474}, {0.962759147476057,
         0.8224436391705403}}]}}, 
     {Arrowheads[{{0.016272107256669013`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9240031934596128, 0.8762173814295934}, {
        0.9583497477168585, 0.9590767362174647}}]}}, 
     {Arrowheads[{{0.014542990019468325`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9272587081794207, 1.0212351469461063`}, {
        0.9550942329970504, 1.0964119118774225`}}]}}, 
     {Arrowheads[{{0.01312761790541763, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9297139206467715, 1.1656823597169803`}, {
        0.9526390205296997, 1.2343176402830187`}}]}}, 
     {Arrowheads[{{0.012572289032024979`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1028688664023556`, -1.16865860643912}, {
        1.1324252512447035`, -1.2313413935608801`}}]}}, 
     {Arrowheads[{{0.013818604383719657`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0999518562430113`, -1.025098049253899}, {
        1.135342261404048, -1.0925490095696302`}}]}}, 
     {Arrowheads[{{0.01530879780135259, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.096181198473009, -0.8813226496434575}, {
        1.1391129191740503`, -0.9539714680036014}}]}}, 
     {Arrowheads[{{0.0171168985608574, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0912249668343268`, -0.7373876325651155}, {
        1.1440691508127325`, -0.815553543905473}}]}}, 
     {Arrowheads[{{0.019351722288825802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0845818130408897`, -0.59344446591864}, {
        1.1507123046061698`, -0.677143769375478}}]}}, 
     {Arrowheads[{{0.02218366408088065, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0754565941849077`, -0.4498663135175107}, {
        1.1598375234621519`, -0.5383689806001367}}]}}, 
     {Arrowheads[{{0.025900636613884812`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0624987602614202`, -0.3076141507972888}, {
        1.1727953573856391`, -0.39826820214388803`}}]}}, 
     {Arrowheads[{{0.030997853497116933`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0433178547778412`, -0.16964427604576154`}, {
        1.1919762628692179`, -0.2538851357189447}}]}}, 
     {Arrowheads[{{0.03754378372508135, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0167466765914932`, -0.047647813636939894`}, {
        1.2185474410555661`, -0.09352865695129578}}]}}, 
     {Arrowheads[{{0.03754378372508142, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.016746676591493, 0.04764781363693961}, {
        1.2185474410555663`, 0.0935286569512953}}]}}, 
     {Arrowheads[{{0.03099785349711697, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0433178547778412`, 0.16964427604576116`}, {
        1.191976262869218, 0.25388513571894433`}}]}}, 
     {Arrowheads[{{0.025900636613884847`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0624987602614202`, 0.30761415079728843`}, {
        1.1727953573856393`, 0.39826820214388764`}}]}}, 
     {Arrowheads[{{0.022183664080880645`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0754565941849077`, 0.44986631351751033`}, {
        1.1598375234621519`, 0.5383689806001363}}]}}, 
     {Arrowheads[{{0.019351722288825802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0845818130408895`, 0.5934444659186395}, {
        1.1507123046061696`, 0.6771437693754776}}]}}, 
     {Arrowheads[{{0.0171168985608574, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0912249668343268`, 0.7373876325651151}, {
        1.1440691508127325`, 0.8155535439054725}}]}}, 
     {Arrowheads[{{0.01530879780135259, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.096181198473009, 0.8813226496434571}, {
        1.1391129191740503`, 0.953971468003601}}]}}, 
     {Arrowheads[{{0.013818604383719657`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0999518562430113`, 1.0250980492538986`}, {
        1.135342261404048, 1.0925490095696297`}}]}}, 
     {Arrowheads[{{0.012572289032024979`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1028688664023556`, 1.1686586064391196`}, {
        1.1324252512447035`, 1.2313413935608797`}}]}}, 
     {Arrowheads[{{0.011912824053351105`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.276980890100606, -1.1719937029687708`}, {
        1.311254404017042, -1.228006297031229}}]}}, 
     {Arrowheads[{{0.012970645227741053`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.273945400279744, -1.0293099497731446`}, {
        1.314289893837904, -1.0883371090503848`}}]}}, 
     {Arrowheads[{{0.014198637077553282`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2701583548583304`, -0.8867058067067692}, {
        1.318076939259317, -0.9485883109402896}}]}}, 
     {Arrowheads[{{0.01563161768674601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2654013659875016`, -0.744353715163706}, {
        1.322833928130146, -0.8085874613068825}}]}}, 
     {Arrowheads[{{0.01730893096661073, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.259402541530508, -0.6025728171064615}, {
        1.3288327525871397`, -0.6680154181876565}}]}}, 
     {Arrowheads[{{0.019265812323821695`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2518642667677342`, -0.46195937889901734`}, {
        1.3363710273499134`, -0.52627591521863}}]}}, 
     {Arrowheads[{{0.021497164399658764`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2426242296078325`, -0.323634918740149}, {
        1.3456110645098152`, -0.38224743420102786`}}]}}, 
     {Arrowheads[{{0.023825797444883293`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2323151984543796`, -0.18957020538367553`}, {
        1.355920095663268, -0.23395920638103074`}}]}}, 
     {Arrowheads[{{0.025567923001979954`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2241885081494626`, -0.06188710536291625}, {
        1.3640467859681853`, -0.07928936522531943}}]}}, 
     {Arrowheads[{{0.025567923001979954`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2241885081494626`, 0.061887105362915903`}, {
        1.3640467859681853`, 0.079289365225319}}]}}, 
     {Arrowheads[{{0.023825797444883293`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2323151984543796`, 0.18957020538367517`}, {
        1.355920095663268, 0.23395920638103032`}}]}}, 
     {Arrowheads[{{0.021497164399658757`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2426242296078325`, 0.3236349187401486}, {
        1.3456110645098152`, 0.3822474342010274}}]}}, 
     {Arrowheads[{{0.019265812323821702`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2518642667677342`, 0.46195937889901695`}, {
        1.3363710273499134`, 0.5262759152186297}}]}}, 
     {Arrowheads[{{0.01730893096661073, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.259402541530508, 0.602572817106461}, {1.3288327525871397`,
         0.6680154181876561}}]}}, 
     {Arrowheads[{{0.01563161768674601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2654013659875016`, 0.7443537151637055}, {
        1.322833928130146, 0.8085874613068821}}]}}, 
     {Arrowheads[{{0.014198637077553282`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2701583548583304`, 0.8867058067067688}, {
        1.318076939259317, 0.9485883109402892}}]}}, 
     {Arrowheads[{{0.012970645227741032`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.273945400279744, 1.0293099497731442`}, {
        1.3142898938379037`, 1.0883371090503844`}}]}}, 
     {Arrowheads[{{0.011912824053351105`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.276980890100606, 1.1719937029687704`}, {1.311254404017042,
         1.2280062970312287`}}]}}, 
     {Arrowheads[{{0.011199963345734237`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4519903496605397`, -1.175362977900796}, {
        1.4891861209276964`, -1.224637022099204}}]}}, 
     {Arrowheads[{{0.012072512361463282`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.449096467238498, -1.033422350108421}, {
        1.492080003349738, -1.0842247087151085`}}]}}, 
     {Arrowheads[{{0.013051586754250076`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4456318649600366`, -0.8917404906758423}, {
        1.4955446056281991`, -0.9435536269712167}}]}}, 
     {Arrowheads[{{0.014144384224217989`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.441503749830402, -0.7505125291145733}, {
        1.4996727207578342`, -0.8024286473560153}}]}}, 
     {Arrowheads[{{0.015348449856734177`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4366504558557036`, -0.6100412537619427}, {
        1.5045260147325321`, -0.6605469815321753}}]}}, 
     {Arrowheads[{{0.01663799156292185, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.431115356250358, -0.4707788430506923}, {
        1.510061114337878, -0.5174564510669551}}]}}, 
     {Arrowheads[{{0.01793644016261795, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4252071475594414`, -0.33333548645830935`}, {
        1.5159693230287945`, -0.37254686648286744`}}]}}, 
     {Arrowheads[{{0.019074142779961924`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4197667974791732`, -0.1983162046793514}, {
        1.5214096731090625`, -0.22521320708535486`}}]}}, 
     {Arrowheads[{{0.019772769634296777`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4163082911356528`, -0.06573786377448221}, {
        1.524868179452583, -0.07543860681375349}}]}}, 
     {Arrowheads[{{0.01977276963429677, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4163082911356528`, 0.06573786377448185}, {
        1.524868179452583, 0.07543860681375306}}]}}, 
     {Arrowheads[{{0.019074142779961955`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4197667974791732`, 0.19831620467935104`}, {
        1.5214096731090627`, 0.22521320708535444`}}]}}, 
     {Arrowheads[{{0.01793644016261795, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4252071475594414`, 0.333335486458309}, {
        1.5159693230287945`, 0.3725468664828671}}]}}, 
     {Arrowheads[{{0.01663799156292184, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.431115356250358, 0.47077884305069195`}, {
        1.510061114337878, 0.5174564510669547}}]}}, 
     {Arrowheads[{{0.015348449856734177`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4366504558557036`, 0.6100412537619423}, {
        1.5045260147325321`, 0.6605469815321748}}]}}, 
     {Arrowheads[{{0.014144384224217989`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4415037498304017`, 0.7505125291145729}, {
        1.499672720757834, 0.8024286473560148}}]}}, 
     {Arrowheads[{{0.013051586754250076`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4456318649600366`, 0.8917404906758418}, {
        1.4955446056281991`, 0.9435536269712163}}]}}, 
     {Arrowheads[{{0.012072512361463311`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.449096467238498, 1.0334223501084205`}, {1.492080003349738,
         1.0842247087151082`}}]}}, 
     {Arrowheads[{{0.011199963345734237`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4519903496605397`, 1.1753629779007955`}, {
        1.4891861209276964`, 1.2246370220992036`}}]}}, 
     {Arrowheads[{{0.010477306125975746`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6277414377817478`, -1.1785359331371488`}, {
        1.6663762092770762`, -1.221464066862851}}]}}, 
     {Arrowheads[{{0.01118362862614915, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6251385163185235`, -1.0371535747662564`}, {
        1.6689791307403008`, -1.080493484057273}}]}}, 
     {Arrowheads[{{0.011949495453044771`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6221500641505613`, -0.8961012470138565}, {
        1.671967582908263, -0.9391928706332024}}]}}, 
     {Arrowheads[{{0.012768289697852282`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6187690551976337`, -0.7555398540556062}, {
        1.6753485918611908`, -0.7974013224149823}}]}}, 
     {Arrowheads[{{0.013622335338254642`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6150435971751116`, -0.615681833577597}, {
        1.6790740498837127`, -0.654906401716521}}]}}, 
     {Arrowheads[{{0.014475369768919449`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6111259556165303`, -0.4767823051369137}, {
        1.682991691442294, -0.5114529889807337}}]}}, 
     {Arrowheads[{{0.015263437675282373`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6073372076773222`, -0.33908820015721075`}, {
        1.6867804393815022`, -0.3667941527839661}}]}}, 
     {Arrowheads[{{0.015890646781373564`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.604208497335815, -0.20270973761578687`}, {
        1.689909149723009, -0.2208196741489194}}]}}, 
     {Arrowheads[{{0.016244935134227038`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6023976663336905`, -0.06742454185633001}, {
        1.6917199807251337`, -0.07375192873190567}}]}}, 
     {Arrowheads[{{0.016244935134227038`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6023976663336905`, 0.06742454185632964}, {
        1.6917199807251337`, 0.07375192873190527}}]}}, 
     {Arrowheads[{{0.01589064678137356, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.604208497335815, 0.20270973761578648`}, {
        1.689909149723009, 0.22081967414891898`}}]}}, 
     {Arrowheads[{{0.015263437675282373`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6073372076773222`, 0.33908820015721036`}, {
        1.6867804393815022`, 0.3667941527839657}}]}}, 
     {Arrowheads[{{0.014475369768919454`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6111259556165303`, 0.47678230513691333`}, {
        1.682991691442294, 0.5114529889807333}}]}}, 
     {Arrowheads[{{0.013622335338254642`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6150435971751116`, 0.6156818335775965}, {
        1.6790740498837127`, 0.6549064017165206}}]}}, 
     {Arrowheads[{{0.012768289697852282`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6187690551976337`, 0.7555398540556058}, {
        1.6753485918611908`, 0.7974013224149819}}]}}, 
     {Arrowheads[{{0.011949495453044802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.622150064150561, 0.8961012470138561}, {1.671967582908263, 
        0.939192870633202}}]}}, 
     {Arrowheads[{{0.01118362862614915, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6251385163185235`, 1.037153574766256}, {
        1.6689791307403008`, 1.0804934840572726`}}]}}, 
     {Arrowheads[{{0.010477306125975746`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6277414377817478`, 1.1785359331371483`}, {
        1.6663762092770762`, 1.2214640668628505`}}]}}, 
     {Arrowheads[{{0.0097764739834501, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8040469254264957`, -1.181386143062818}, {
        1.8430118981029169`, -1.2186138569371818`}}]}}, 
     {Arrowheads[{{0.010342744314054698`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8017905824028364`, -1.0403841856980935`}, {
        1.8452682411265762`, -1.0772628731254361`}}]}}, 
     {Arrowheads[{{0.01093767795645266, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7992987754129588`, -0.899712904175814}, {
        1.8477600481164538`, -0.935581213471245}}]}}, 
     {Arrowheads[{{0.011550188090361782`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7966059841827862`, -0.7594847871806495}, {
        1.8504528393466264`, -0.793456389289939}}]}}, 
     {Arrowheads[{{0.01216112908340845, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7937938880921889`, -0.6198271343117402}, {
        1.8532649354372237`, -0.6507611009823778}}]}}, 
     {Arrowheads[{{0.012740596380756662`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7910124698751133`, -0.48086287631464036`}, {
        1.8560463536542995`, -0.5073724178030071}}]}}, 
     {Arrowheads[{{0.013246569582145828`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7884945676311024`, -0.3426707593544241}, {
        1.8585642558983102`, -0.3632115935867528}}]}}, 
     {Arrowheads[{{0.013627849871941577`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7865430632708879`, -0.20522498643722129`}, {
        1.8605157602585247`, -0.218304425327485}}]}}, 
     {Arrowheads[{{0.013834467068875216`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7854663260855017`, -0.06833838779139521}, {
        1.8615924974439109`, -0.07283808279684047}}]}}, 
     {Arrowheads[{{0.013834467068875216`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7854663260855017`, 0.06833838779139484}, {
        1.8615924974439109`, 0.07283808279684006}}]}}, 
     {Arrowheads[{{0.013627849871941575`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7865430632708879`, 0.2052249864372209}, {
        1.8605157602585247`, 0.2183044253274846}}]}}, 
     {Arrowheads[{{0.013246569582145828`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7884945676311024`, 0.3426707593544237}, {
        1.8585642558983102`, 0.3632115935867524}}]}}, 
     {Arrowheads[{{0.012740596380756657`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7910124698751133`, 0.48086287631464}, {
        1.8560463536542995`, 0.5073724178030067}}]}}, 
     {Arrowheads[{{0.01216112908340845, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7937938880921889`, 0.6198271343117397}, {
        1.8532649354372237`, 0.6507611009823774}}]}}, 
     {Arrowheads[{{0.011550188090361782`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7966059841827862`, 0.759484787180649}, {
        1.8504528393466264`, 0.7934563892899386}}]}}, 
     {Arrowheads[{{0.01093767795645266, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7992987754129588`, 0.8997129041758135}, {
        1.8477600481164538`, 0.9355812134712446}}]}}, 
     {Arrowheads[{{0.010342744314054698`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8017905824028364`, 1.040384185698093}, {
        1.8452682411265762`, 1.0772628731254357`}}]}}, 
     {Arrowheads[{{0.0097764739834501, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8040469254264957`, 1.1813861430628176`}, {
        1.8430118981029169`, 1.2186138569371814`}}]}}, 
     {Arrowheads[{{0.009116798619914539, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.980733871751405, -1.1838697719654656`}, {
        2.0192661282485957`, -1.2161302280345343`}}]}}, 
     {Arrowheads[{{0.009569687754266311, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9788199149169337`, -1.0431054477318142`}, {
        2.021180085083067, -1.0745416110917152`}}]}}, 
     {Arrowheads[{{0.010032604380582781`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9767770061484127`, -0.9026378443014493}, {
        2.023222993851588, -0.9326562733456097}}]}}, 
     {Arrowheads[{{0.010494535920320246`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9746523699480951`, -0.7625382505618518}, {
        2.0253476300519058`, -0.7904029259087367}}]}}, 
     {Arrowheads[{{0.010939521084516988`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9725254688599796`, -0.6228758215402782}, {
        2.027474531140021, -0.6477124137538397}}]}}, 
     {Arrowheads[{{0.0113461236745298, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.970514107020019, -0.4837023461393182}, {
        2.0294858929799817`, -0.5045329479783293}}]}}, 
     {Arrowheads[{{0.011688143720156648`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9687725946320478`, -0.34502979430757774`}, {
        2.031227405367953, -0.36085255863359905`}}]}}, 
     {Arrowheads[{{0.011937480299842693`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.967474718890251, -0.20680544893831998`}, {
        2.03252528110975, -0.21672396282638628`}}]}}, 
     {Arrowheads[{{0.01206948107950648, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9667780233212562`, -0.0688969727636392}, {
        2.0332219766787447`, -0.0722794978245965}}]}}, 
     {Arrowheads[{{0.012069481079506476`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9667780233212562`, 0.0688969727636388}, {
        2.0332219766787447`, 0.07227949782459608}}]}}, 
     {Arrowheads[{{0.011937480299842693`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.967474718890251, 0.20680544893831962`}, {2.03252528110975,
         0.2167239628263859}}]}}, 
     {Arrowheads[{{0.011688143720156648`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9687725946320478`, 0.34502979430757735`}, {
        2.031227405367953, 0.36085255863359866`}}]}}, 
     {Arrowheads[{{0.011346123674529803`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.970514107020019, 0.4837023461393178}, {
        2.0294858929799817`, 0.5045329479783289}}]}}, 
     {Arrowheads[{{0.010939521084516988`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9725254688599796`, 0.6228758215402779}, {
        2.027474531140021, 0.6477124137538394}}]}}, 
     {Arrowheads[{{0.010494535920320246`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9746523699480951`, 0.7625382505618513}, {
        2.0253476300519058`, 0.7904029259087363}}]}}, 
     {Arrowheads[{{0.010032604380582781`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9767770061484127`, 0.9026378443014489}, {
        2.023222993851588, 0.9326562733456093}}]}}, 
     {Arrowheads[{{0.009569687754266311, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9788199149169337`, 1.0431054477318138`}, {
        2.021180085083067, 1.0745416110917148`}}]}}, 
     {Arrowheads[{{0.009116798619914539, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.980733871751405, 1.1838697719654652`}, {
        2.0192661282485957`, 1.2161302280345339`}}]}}}}, 
   GraphicsComplexBox[CompressedData["
1:eJx0nXl8U8X398MiLiyCuLEoCorI4hdkEQE5RUWQVQQUEURFUEAB2VGEIgrI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    "], {{{}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], 
         LineBox[{3319, 4027, 3318, 4333, 4332, 4721, 4720, 3998, 3660, 5049, 
          3659, 4028, 3323, 4979, 3666, 3999, 3668, 5050, 3667, 4031, 4032, 
          3675, 5052, 3674, 3459, 3329, 4983, 3676, 3531, 4514, 3532, 4515, 
          4731, 4730, 4035, 3338, 4991, 3689, 4004, 3462, 3339, 4992, 3690, 
          3537, 4527, 3538, 4236, 3700, 4271, 4040, 4041, 3708, 3353}], 
         LineBox[{3425, 5024, 3813, 4021, 3495, 4379, 4378, 4270, 3636, 4217, 
          4218, 3823, 5068, 3822, 4065, 4066, 3834, 4862, 3833, 3501, 3435, 
          4699, 3645, 4461, 4462, 3840, 5071, 3839, 4069, 3443, 5038, 3845, 
          4024, 3504, 3444, 5039, 3846, 3649, 3853, 5073, 3852, 3506, 3448, 
          5043, 3854, 3654, 4869, 4870, 4868, 3855, 3507, 3454}], 
         LineBox[{3716, 5093, 3717, 4098, 4097, 4337, 4336, 4205, 4204, 4203, 
          3542, 3699, 5057, 3698, 4093, 4094, 3912, 4999, 3348, 4090, 3337, 
          4388, 4387, 4386, 3534, 3682, 5054, 3681, 4085, 3903, 5078, 3904, 
          4083, 4082, 3328, 4982, 3900, 3673, 3899, 4981, 3327, 4078, 3322, 
          4978, 3895, 3665, 4899, 4900, 4334, 4335, 3321, 4077, 4179}], 
         LineBox[{3805, 5066, 3806, 4139, 4138, 4844, 3804, 4295, 5092, 3633, 
          4683, 3634, 3943, 5025, 3426, 4142, 4143, 3949, 4951, 3948, 4952, 
          4146, 4857, 4858, 4690, 3640, 4689, 3641, 3956, 5034, 3436, 4151, 
          3441, 5036, 3961, 4154, 4153, 3962, 5083, 3963, 3841, 3969, 5100, 
          3968, 4158, 3970, 5085, 3971, 3847, 4963, 4964, 4960, 4962, 4961, 
          4162, 4163}]},
        "0.22999999999999998`"],
       Annotation[#, 0.22999999999999998`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1WtM12UYxvFHgTSlUqjVi7L+VGqbLZaglJBnm6bYCbUFiOAB8YQlICqa
lWKHlZu9sEwFO61VptVsrtnB1NQNdctOaic1NMXUDmYJ1edZLy6+z3Xd9/38
zn8SpbPundkuhPCSP893DeFqejgthEcuCWEO5l8awguy7fw1WCWrptHyVfwO
eTeskc2lu+W1+KJsNV1L16WHsFhfAk9cFkI2ZVj3wRY6SW2pITTq3aWvAXfj
bfLj+nLwNJ2h22mO7B/9Z62T9NXyedYljn0HzuOT5b9bd8A6fpB1ufpgXMh3
lP9lnYZL+GHWlep34lI+Xd5qPVt2Ez/SugfehT0xdAnhCdf5JO1xvt/pb8Lv
MV/P03oS1in6HrNHX/4eeVa8bsqmzmprzB82d4R+pJ/oKL0sX2GuxfqMfU7i
WTyFa+XbzTfgBVk/WS7lRfLJZpPitfJDaCgNlKfJO8lH8qMoP9bic5VfKb+P
v58+td5p/62YoZbABGbg9diEezGb+lAW5VBf+oG+dZ0P2ne1PQ7ofZd/jwpl
a2QHZbfQcL2ZOALP0R9xXt8Ufa/qa1HbxL8fn53sNdmpeC/pT2riz+GbahX0
On+ez6U22sO34nq1SnojvjfxHaEke+7j2+Pbag/RW/G94Y9RM9XI3onPKb5n
WOneDDBb6DwD/y9u1LNZvRO1k12M7fE41qlN0zOdyqmCptIC+RQcaq9JOAQn
43z5RBzMl+IgLMN58hKcQEU0noqpVj7QO1SNGx3zccccwPenYn48FVEhzXLu
/e3XUc8GvspMrr48WiQ7JFuI/fgF8Zvk6/CCmbl4gK/BHPUqPMFXx29SfTZ+
E+8P9lafiT/HY8Z7qV6BX/NTMVO9WVZuvU5W5jx6yYqw1DlOoBLqqedL2qXn
C+yhZ3+8H+ZWyQri74lsNK6lB8yMoxvdlxtouGwsP4YK6Cqz3eTD5CvNb+ZT
KV3WGS/HDdgBUzAFk/EiXI9/26PN7Hlsjb8d2Cg/jb/xv+Cv+Jxshf0zrbNk
velWOsY3x+vCZ+hm6160j/ZShuwpWmr+Wcesx+Xxe5Dtju8Svy0+I+wer5O+
UtuEV2A9VanVx99nXIafy3aYnc5/bHYGvsJPw8V8qvo2fiL/ET8J1/FlOJ9P
Ud/KF/Nb4vuEjXxRfCfit6P+CT+W/4Afhw38o/IC6zH0WfzmnePO+JzivfO7
vUXPhzQqvjvxm5Utit+gc17Z9f//Q/8BJM/mMQ==
          "]]},
        "0.184`"],
       Annotation[#, 0.184, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1Fdsj1EYx/FTRJB0EVvELCVG0Yo9YodqUDNKhFKjTbiw90jFjBER68Kq
Ulu4NCIuam+JESrmjV2xP29cfPs9z3N+zznvv2/7rz8hb0huTAihsx/ZcSEs
rBzCYizC5fgQ4mJDOJQYQiEO4oa6gGN5WZUQcs0ckFspvwrLsALLcUW/utwJ
+ZM4jrvqOWYump3N+epqyFfPVT+0Piv3iM/xWuesQbGzGuqtk8uXO6guQCO9
9Xqr9V5Y3zJzB7exx9w1mRT9LTJbZQrVm9Vt8EHmI1KRhvYR+j/5D3flbuiO
LuoKzivvjD7qvuiHnvpV9WP1B6szsEOvL2omhFALWe7s5v6x3J3r6l0w0wRN
kYSG5sfb324/w2wDmevRmeqJ+gM5HYPQ1N5zMy/QEclyHbgZZ8vukclyRgu5
ibxX/cZeW+R4viHqAv2hnCIzzcww6+HIxFu5T3ht/ZHTnZ3K08yO0is0O5rb
mZ1pdoz1LE5Tp2KG/WFmMjE0ekazwzlXfyR3Uo/g0RiFPP0S8y/x2V6B83L0
Dkfvwv5gZGC6+3/xb3yR+4o0vMM+M5Oiv1WU2h+IQZhq5ju/l9kvk23/mXtK
1MXqlzxFZjJaWveLniv6neFhdLb6Eb+P3hc/Nnubn6rv8DMeZzYLba3bIN3s
bhTbv66+yg/M3cd5dW3UQU1Us1eLz3EiV+R4rsAJfIzLchmO4Rguw0Vc6s4f
7vnC3/A1+j+1l+gdVEam+7b5jGkycepWvAkpcq3xynqX/AZn7eT1XEluHWeY
3Wi2iUw5vXrc3EwzJOOJer7cJZl5/Fe+Px6oB/A9+6fkzuA0akTvUj+RV8j0
ts4z14uXqxP0j8rdjN47F+EI4tXfZcrzArke1qX4hiT1Sv3GXF/2mvWB6F06
d6nPswT11PuxDwui76bo3cb//677BxM2rQk=
          "]]},
        "0.138`"],
       Annotation[#, 0.138, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl09tPz3Ecx/GPubBEySGHEMPoqJgNkU7kMJuaMcYyp0v+BIe7Elsox8xM
xrjAnZCcSVw5TOWQCBdOldPm9PjOxXPPz/v1eb8/n+/v+62xazeXbuoVQviB
+oQQJmLXwBC2xoVQMSiEpP4hXJK18EVuwGWMVDfyVVzBdVzDKPlNvoUbuIPb
GC1v4nu4i1Z1MyfzDvdsc1+b9SRZqrrCOo17DwhhcXwI6daZyMB2vVN4KiYj
G1mI0Vtp74DnP4SDeOOcTGdORlW0Rn9907jGzFJnHzYzQFbLJ80c4VP8U890
czMwEznqIA/289UF2C/Li86Ux2GBbCEWoUg+TDZEf4m6NHpX6qvqBm7kR+4f
w/fVTdzMd7nW7GJkqbOxyuxqtFk/w1O0oiWyM1boHW5/GF7LOnBe/o7fR+8B
b9GJ5/I1+udZF2MuMtz5Cu3oUXdjY/QbuBBdOGHmC2+Ifjvn4zPq5J94vTyP
5+Ajjss/8Dp5unNzrV9wp/f+ktOQI5uNWSjTn+I7LOfDejZ4zid6lliXINE5
Q7HM/jh9JTxEPRjz7RdjvP4UZ6UhFdeQrLfAXiHyUebcIq7Xm2Svn/lYrJTn
yuPlidH35KF8jvtxH47hWPTFWfwx9xe/Inue33xMXzd/VXdxD9fp3Sf/5r7v
OOOu06iRVUd/y3oeqI/qe2TmMR6iQ75XVh19A3PdmCCrRIXZKvke7Ea7bIus
3HkjrMuxU34h4f//9j/ClIx7
          "]]},
        "0.092`"],
       Annotation[#, 0.092, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl081TzXEUx/Fvm7JA3RgzVmWGOyOlhRYGS4yyiDAMK/0BNWPDzmMsQldG
DBbKjKdkFMpTbixQmelJ3QqJMeOywQxrXt+xeM/nnPM533N+93t/vyW19TV1
OSGEGTTOD6GlMIQLOI83+SGsmBfCdCKEd8hgCpMoUT+zIISUM50FIdThNE7h
Lpr4y/FH/BtZZ74iq/aNfkcq9qELQ3at5gX1v/Lb9rejQ/0OctQ75Ffte2nv
YT1H8MiZanqS34ij4jL947SPl6Vn5cdoA45jCx7zTtCtKOfvoJfoWrPXodOu
Lrse0O4Y4z7uYZHeHvqcvkCxuAiL9W9wdiMKPEsCVfYU66mhN+kt3MAyvaPO
jGEYQ/IRWooy7Ha20pwrzlXRpc7s9XybxbU0GWdgF39KPo1hMy7yt6m14q05
K1GOPeaN0zb1EX0T4klkkJS3m9UR7x3b9Uya95rXj1cYxAAO8PfjOq5hk97L
SJvxhN+Lp1goHzOjm6bjfdE+OogBVMb/wjM14JC4RO8o3Un7aWHcTxM0z29a
L35oz0G6Sq1HLS2vED+Ld2R+t1qv+JyZLchFHtaoFfEqoodP4lnsM+sX/Ykf
6m3x/UIrms2by88gxfusZ0Jcjy/x3ec3q5WJP9oxi/eYwQeUqs/Rm6RN+qrj
75DnxvuN30l8H/L/f3P/ANKcigI=
          "]]},
        "0.046`"],
       Annotation[#, 0.046, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl07lPlFEUhvGr1goyJhqdwVBrjFoJiX+A0WpMaHBJiCsuaCGNFoosamQZ
GAZwSVwAC43GCG4szTBgjRQuiSZamEGhNK7E343Fk+e+55x7v/t9MBW19emT
S0IIeRSXh3CqNISnGCoLoX9FCL+sf6MbC4kQ5sx0WX/lLP9BDt/kHp7nXl7E
X1xzxj1nLaiflt/gLd7jHUpQ6dx0SQjbeDdX8XX7JvQe8Bjf53Heot+HYTOj
zkyrjfGIvFW9BfWet8P8+ZUhXECdXpt6o3UTLmJe7araF/u3+wCveaM8y1f0
L6NdPmbuknUrNshJOcFrzI14xn4e5tU8o17HR+P3ifdST/Fj9ZfuNCE/53F+
wbPqu9w94exBc+vVB7icp/RfcYGnY1YvoAEp+S5P8STncQZJ+Q5Pct6+Quzz
Or6lvpZvc5nn7fTcpPURHMYh7HGfzzxq7hPXyAe5iDn0yzdxA9XyKp7xPZrj
+6HcejNnnN+FE3Kv3G2dxT57snIOH/V6uIhOfJAz3IG95jrNd8Rz5ON67dZt
2CSn5HNmzmLaesh9H3nPpfrLkPNuD+UB9WdmBuN35+/x/4T75AP8Az/jndSf
xL8jMqX/fwf/AMt3ffQ=
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl0jlPlAEQh/Eh0QTQ3X13l+UUFU/4CBZqYRTiVVAoqIVivAoX5fAGSzk+
ALBepUejiaiolUcpKlE7vOLRmqiJlP6IxZNn5j+zs+8eTV3d7cWyiJjC31RE
bxLxGE9wKx0xwXPyPp7FB+zKRHzkPKpw296CbMRCrLNbzhs5nY/IIIUar9lh
dzsemC3T18iX8wq8VTfxa/NXKOYi2tydkg/oB3FUPWLnnPvncQEd8k5cku/h
3bgoH8QARuXHvO6HfBQvvPeY/Dl/lc/IVtt5ww36eqzS1/JKruOdZq34rW7j
WnmirkYBS/Ul9zah0u0K3PPsM/N3kdGncX3+O5Nf5Rxf45T8Cmf1JU74j5uX
1Z/wU/2ZO9xZr+7kL/pqXqRfjGn9S3zDIf2E57hrfkC/H43qLbwZ982q7DTI
8ryEW/Be3czDdp7aGeJnPMLfccR82Py05z2Ds9gnf2hnL5/S96MPQ/YO20/4
uN/xl7rV5zpp1oNerDF7J1/L9ciq6zjHk25udXMbCrJyWSUqsMGsIO9Wn0AR
JTsHZV246X3u6MfsPdLf0I8n///b/wCWqlt5
          "]]},
        RowBox[{"-", "0.046`"}]],
       Annotation[#, -0.046, "Tooltip"]& ], {}, {}}}], 
   GraphicsComplexBox[CompressedData["
1:eJx0nXdAU8kX72Pvvay9rH3ta13bYO/Yu1ixiwURRUUjgth17V3sXVFRUdGJ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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV00lIlVEchvGjYGCUlpoNCiVuFFKC0lBrY5DlxmigjUFJLaxUilIqh4VW
lC7UArWoiEJo0KgbWAQiaSZoC5U2BhZtEiyCMLCw4efi5TnP+Q989+O7aWVV
eyujQgg/5G5iCFOrQriZFMLWtSHc4Ll4E2vc/VsWQqdzl2THh9CGEfVx9wuy
2Xn36hCKpSU5hC28Xc9znFDvsKNTWtw9cTfq7rdc5YMpITRjpr2N2KP+Vm1e
spzz7SyQS/Zm8yY9vTiifs3Oej5gRwOe4Wel1rlO0u08hw/09+ufkyh3b3g0
Ji23U/8zuyPSw6PwuPoJKZcKOSl1MuMdRfQMOk/b27A4y7sSQhjzjH/i/Dbz
f3EMl+rbtvge1O+ZLXAeMtMnrZ7lvpmn0iGj0i9FZiP6L/hN3XjF/mas5J04
zevxCD+FjfZsxyJejK/Vj+J+fhBreTrm8004aX8yZvFYrDb/fmUIifwz5qkv
4Ab+HavMv8IlfASH1Xvx1zrfDZabr8CP/DxuVL+OX/llLDO/Dyf4YXypvhPf
8DwsNR+Dj3gCrlffiC/4ejxgfnZFCLf5PD5Wn8Y2Po4l5m9hDX+ICepD2MT7
cJf5RjzOW/GOejUe4uVYaD4Hd/BCjFEvxT28BAvMp2Auz8B29TjM5FGYY/6d
bymeT+G8/jlM5V8wm0cwmg/gRfPd+NO30YUZ5o/hB34aZ/W34AxvwDSeuvgN
ef+ZOM6/rfH/kWH5JJMSq/8/uMeQzA==
          "]]},
        "0.42600000000000005`"],
       Annotation[#, 0.42600000000000005`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV01toznEcx/GvU3bhNNucpjkkFmYXCIs5hdxQsxRTjheU4wU5RAwjYmTO
NqdcOBXDjGjkLIUhMYeNopxKCDm+dvF53r/P7/P9/P7/5+n3dJgyJ2d2nYhI
8nEyKeJWakQp7m0RUUIDm0bMbBnxulFE/cYRx2TZrSOO40DMlpckRxzhb+oe
xUX8Ybwgr9Er5vtZ5zlnAo2nPnoHzZyz/8xMXWcvNHfAXrm9Knt7+D38decW
4ybvU0iZurnOeGDmD22R9dYpwl06O2mD9VW9jbhWZw111Ruqd0vnJ62WZeoV
4FadIlpufVlvBa7UyadOej31Lul8o9OyevaSZSlUY77CfnfrDKqW37FXg/9S
Iv7SGd/vFT9I3rb2O1APyqb9nntanmY9wEyG+c7WH5pE/JY15MfhZ3pDuz3r
lff5SI+pXquIH5hjvrp5xGbPfoRFzqzAAv42Vsvv6ldRludcwTIqdN5B/RO0
jW7TRRpuvlRvsf4hXOu8dTiL347P+aU4iZ+L+c7pj8P5kXhZPhXH8GNxCd8R
+/GZWOn8FOzOJ+B8/YeJEc1rfzvsK/+F7fhPOFv/PDbgb+A1+XH80SZiH07X
n4kv+EXYTb4F3/MFOFk/B+/zE7FcPgyv8n0xr/aO4xE+EdPk3fAsn4a5+u+a
uYv8dzwqf46b+Hs4qvbO4gL+MCbKr+BKvgxH6OfjDL4QS+TzcQI/HQfr98Ih
/GCsL8/D0fwozNJvg735LrhZ3hjT+cBe+nfcyyb8E/xu/gum8m8wgy/FOnwF
rtI/hF/9B3ZgF/1p+JSfh+/Mr8e3/DJsz6e6N6f8/ul4j//g7v2ka/SSKinB
/H/qpaTs
          "]]},
        "0.35500000000000004`"],
       Annotation[#, 0.35500000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwN0nloz3Ecx/EPc0QbGxs2IYkxS6uxI8dGm1az1RzF1hwbURh/OOcotESZ
K0eOsKYcy1XI0XJFch85x8hRrhI5Qnj88fw9v6/P+/N+f77fX5/uZbNGVjQJ
IST5eRsZQouoEPZ0CGE3KjqGMBOvrTe3fqh9CNnxIRzm3bEh7MIgudyeyZiE
gW1DKON99kzgTLmUM3g811q/0jmEvbzVGVswQG2s2kPnbJQ3YD1SrI+xft/6
X1RbW4MR1vJxw9pvzPMea8074l2uy5vlFM9Z9mRjEIZgMJKtp3E6+qOnMwZw
pf48tXo+4IyDqEMfsxahEpFyg3qh56U45D95Jd/wPa/5DHpbP29Oa7Uaz634
nHxaLcdZk+NCmILFasU8DuV4qj6fK/CtjXfhIpTZl8VpqPVtcZ1C6IpIpCAR
JfaP5lJkIh/DsNPZ0fricdf8ZP6DHvig57fZLTyP5c94g23OeOn/+IgHaGrO
Dy6yv7FdCOt8633eaHY9V8lXuVH9pv4nyHTWBT6OavNq9B/GJlzFWeTaf1Tf
Qv21vNK8VTxD3swN8mKeIM/iZeYM5Fw5j8+pl/FIeQxXyt05Q+7Hd8yP5b5y
S56j/15MCDHyC05X/8Vd5U88U/8pbiZf5kvqdfwjwT3nqfqn8zN5ASepb+D3
chVP1F/Et+XxfFI9hy/K6VysP4L3y9HcRT2JT8hdeJT+d9Eh7JC/8wH1Bl4r
3+IC/dt5rryPo9Uv8HL5OA/Xv4ynydW8U30Ol8hTOVt/Kg+VszlCvZgL5QLO
1B/P/eVevE49khPlf+5tqv5rHCU/4u/2f+EE+Q0ny0c5yPW8Qn8tf3X3t3Av
/eX8WJ7N7+xfzW/lJdxNTnBvjvn/E/mW/MHd+4lLeI47aGn/fxnGqco=
          "]]},
        "0.28400000000000003`"],
       Annotation[#, 0.28400000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV03tsjXccx/EfjUtSHJeaFpVZoonLUo6sSmKuVRaX4xJUXBth02k3xSYb
KenG/qCZIEo3kRNBbC7RFhNp2kaChNlEXIs/CEq2TLTC3F7nj/d5P5/v9/f9
Pb/n5Hl65xdNK2wRQhjv53iXEL7uFsKoSAhFPJK/4tFpaljmugBL8QU+x3D1
u11DWMmrUIzeKSE0qN3GvXYhJLUP4ZcPQqjAMHsuNNegvlvehXJkqeepX1ev
tsevzhJVm6k2iGfxdrUYD5An8RRMRtT6XB6PsRiHnMSc+kVnKOKIMxVyR76g
dh717vUCP7h/KYaayUatWjOq3K/Rusc4on8Yv6NGfULiuXk9D7Rnuvop/IE3
1p/kE6hGpn6JdeOcO+462f9RJ58xn6NWZn1b12v12nBInEdtgTXr+Dfrt6R6
fmxEHBV43QGdQ9jTI4QmPuTMD3mn/A/n66+xrgSLsQIF2Ge/2fbNx033W83L
8dz6XI5hkbN8yp8gbt8Uc+lIRiYyMMf66TwX2fgMoxJns3/EXCr+sn9//h8f
odHMK3u3cj2L/8V9lLvHPf/9E1xFC/s0c8z6O56lzDNd4a32PsOl8jm+o3/R
/A1ku1ctV2Kz/faaP4xtOIfTiffD+qPmvjUf5x/tt4kL5O18S/6O58uFXGKf
YTxWzuUa/UU8VZ7Ba+QPeYj8MV+2fxfuJ7fmYvN/d/LeyXc5S/8lp8tP+Uvz
JzlJPsv1+oe4ubtvgJeYX8a35W+4r/7P/Fgu5QXmY/ynPI+r9cdwnZzFeeZb
8gE5wj31+3KV3JOnmX/U0bcoN/FB/Vu8Rb7EE83v4pXyfo7o1/J6uZJzzJfw
UnkzV+gX8xx5CY8wH+WR8ghuqZ/Hk+SJnG0+lQfLfbhMP5kz5Le+jaj5C9xO
vsZN1v/HafJ97i8f5XeJ74k3mI/zM3kH9zGfz9flIn5k/U/8QP6ee8lp3ptj
/v8MviQ3evdeoB4NuIzW1r8HBty+dw==
          "]]},
        "0.21300000000000002`"],
       Annotation[#, 0.21300000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV1HtsTnccx/FfFyRjq9voQ93XR1CW2GaNu6hIti4pJbTZJkpqmZlLF9UR
KWncaYiaIWR5IiwVUUGW0AltEbd2c9nmUpeVGTLZslo2DfE6f7z7Pp/zvZxz
+pzn6T1jXs7cpBDCfH/udQqhEYu6hFCMIizGV6jsGEL8jRAup4aQxnf1/YY3
HR9Xv/9aCHeic/yD3Mf5kW1DKEgJYQTP4uHOD8MtfQu4p5753IsLea9r3FC7
bccxuYfze5zrzkejOeevqV/j8HoIWzqHUI537N+q75R7+5qv6LmKmLkj5q7q
vyTX8Tr9a7EG6eZW6682t4rr9NSjvbmp6rnIw3nnas1OcdwVXRBDX/MD7E9H
fyyx4wOu4Ur1gxhk1zn7q52bpNbAEzgHEzEZQ/WMteu040xO19Mcs5tP4gQq
5Ff5mJ1VqPX8peZquGd0HbXhXBPVsNM9xznBmfq2cTna6V/is1iGIqzFCixM
dl+ufd49ZHGl502yc7d7b8Wfqq8xX4YncilP07uU99m5wf2VYyUS2IFmM80d
QthlRxNX2Pk7b5X/5Hz1Yn0lmIkFmI3d9k21Nz/6rF1rIc/BP/rHczamR+8X
v4uEvR3NdUNrvIU48vTn8EfIwPsYE92b/cnmUvCj/QP4GXrjoZn/7W7heAo/
QSO+cY3b/k+PcAXBnqecrb/Bs5R5pku8ye4qLpXPcIP6BfO/IiP6LPkQ1tv3
rfn92IwzOIpM/QfMLTKf4BX2reLZcjlflxfzJ/IXXGLPUM6Ux/Nx9ek8QZ7E
xdH7we/JA7ne/g7cX27JheZ/ah9CW/kWD1H/j7vJj/lz89/zK3ItV6tX8NOu
3jEuMP8Z35CLuJ/6Rv5DLuVp5rO5Tv6Yj6iP5ZPyEM6N3jPeKydzqno/Piyn
8kTzD9qFsF1u4u/Ur/MG+SJnmd/GX8p7OFn9BC+TD/E48yU8S17PO9QLOU8u
4FHmB/NoeRQnqefyh3IWZ5hP4bflNC5Tb81x+bnvyWDzZ7mN/DM36f+LY3Jj
9PsgH+AXvitVvNx8gv+Wt3Ca+Xz+RZ7LD/Sv5nvR7wl3l2PR74n/f5wvyg+9
e/+iGjdRj5b6XwIme9eO
          "]]},
        "0.14200000000000002`"],
       Annotation[#, 0.14200000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV1HtoVnUcx/HfahbKHtc23XSPiOLAmQlKa4lp4mQmC6ZblqMSh2g0L3nB
S0lsMbyiQ2miJsZ8kLQtdTbNEAkzcYrNuaXOS6ZieCkUrSmWFPo6f7x9n8/v
9/1+z/k9nrP+0+aWfpQUQljsn20ZISTFQhjeO4SNmSHkpYZQZ+17+VJKCOd7
hnABfXqE8Es8hDh3yL35mpzNQ/SUZIXwFkrRoe+MmjZuMidLzXozM/lb+bT1
dvurrfW01m5OD26zls5X5QzONXeMeQO5gE/qa1HTzLvM6a6m2owYfyMfs37C
/k/cIKdYPx7V4z6Oq01zxhP8lxx3XamuCvtdf4e9fovnuZFXWR9hxkp+jZ+1
/gySMQ5voBAj7Y3nF/k5HsxnneElLvPsudaedHcODvJQHoaZ9kvkCm7A15in
brK1VvecxBmeI8Xz7jQvlT+2f6BXCAexG/uwN7Iz7+B65Jk9Aq9ijvqtzprj
OsEF7rGZa5Fq9lK/axUWYTWWYaGeCe590jMUcaPZwb23e4Yu/IH9VfprcFeu
5ilqP+UGM9d6hlosRwJb8FjP4/QQvjSjk+vNvMEb5Ttcbn+JukpMwzxUYLt5
75hbHr2H7rWQZ+Fv9YVcjKnONJJfRsLcdH1xdMUQ5KBMfQm/i3yMx+jo2cyP
6cvEafMH8b/oh9t6/on+312/zXdxHZvc44rf6Q+cwRM8QLH6y85S40ztvN7s
Q1wtN/Nl+z/rP4989zrMTVhjXp3+XfgczTiIAvV79C3Wn+Bl5q3gCrmWL8qf
8PvybK40ZzgXyIX8g/2pPEEu5SVyX35FHsyt5qdxrpzM8/W3pfmu5N84z/4j
jst/8kz9BzhJPspH7Nfzg2zvGE/X/yFfkhfxQPvr+JZczVP0F/Mp+T3eb38M
/yjn8eToPeMdcoyzo2+G98nZPFH/zRdC+ELu5J32L/JauYWL9G/mBfJXHLN/
mKvkJh6rv5JnyGt4i/35XCZP51H6h/Lr8igO0bfIb8pFnK8/k4fJA7jGflfO
kf+Lvuvo7w93k89xp/p7nCVf50HyHv7ft3KIP9Of4PvyBh6gv5w75Dl8U/1K
/l1eyn3kLO9No98/h1vk297hhziCX9GKZPVPARwU1AQ=
          "]]},
        "0.07100000000000001`"],
       Annotation[#, 0.07100000000000001, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV1G1M1WUYx/G77IVNJB4EBTKTo0CmIfKgTW3S3Bzp0dTEaQxm5iJ1YcMs
57Ahak5ZLZqZy7kzh4pLcFhkkU9rUVOjWtPU1Bc4TV0OFzbL6bLPefHle373
dV/X/3+f3Yfhr9TMeeOhEEKVP0cSQigYHMI4PIOxyMdh66N5DJ7C0xiFb6zn
ch5GIAcj0Wk9myMYhuF4El9bH8pPIBOPIwtfWR/CGUjDYKTjkPVUHoQkpCAZ
X1rvSg3hVFYI3/N36E0L4RamDwohivNqM7jD3mL+TS7hIkzAWXk8tw90Xj4j
F3IBNmSEsBHrsB4N+MKcZrXdqMY+7EVNYgjlj4XQbc9cTjFvgPfZY14iv63e
McQZsR8HcSBu83ZzCwrTQ3gWJVhu/6fOEfE5xqWesY2bkGj2audfi5XYhAbU
6pnp2Se8Qxm3mf0gJYRd3uER77BE/T39jbgp13OFvWt4n5lbvEMT1iOG7bir
564ZO8zo4xYzr/BW+U+uUl9lXx0WoQbV2GXePHOrcNazankp/rJ/KkdR6UwT
eRxi5ibry0R/jEYE8+1/kRegGNPwXPzdzE/Ql4afzM/jfzEM1/T8Y/bDPr/E
N9GDjz3jku/pOn7Ff7iNqP0XnKXRmX7hD8zu5Hq5iy+on4zfHRR71tH4fcFm
83bq/wwfoit+h1Fqf6u+t/THuMG8DVwtN/E5+R1+WV7GdeaM51J5Kh9Wr+SZ
8mxeJQ/lInkUd5ufxLlyP16h/+fkEAbKF7lQ/Q5nyjf4df0dHORv+bh6C9/O
dMd4sf7X+Ly8knPU3+c/5Hqu0B/lH+WF/Ln6FD4mF3J5/J4lucdygpyhnsMH
5Qyepf+q+idyH+9RP8db5FNcpn8bvyk3c4L6UV4rt/Pz+uv4VXkzb1dfwfPl
xTxJfz5PlifxA/vL+QW5jIvlNB4rZ3Oj/v4cke/5neTr/4EflU9zn/29nC73
cJ7cyvf9Vjr5Xf0xviV/xNn6q/iMvJyv2r+RL8urOUtOd2/afP+R+P8r+Zo7
/DeO43d0o5/9/wOX+sQf
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV1HtsjmcYx/Fb90dRVYd2elg2beIUiUWVLGPRRViXFAtaxzaUqLG01c4h
U9IWW9rGprIhTN6Y0ToGkYgtNBIzs44Iw9gfFmbLOs1KRjMZPs8f337v33Pd
1/Xc79PnfTNLyqeVdQshLPEnrlcIf78cwgO0IzkphIL+IcxAu1p5cggzrb/J
CKGQG+Ueru9KC6FIzuBKfV14gnS5Q707vrSeZ0+amSW8AMv1V2CR9WkzF/Jn
cn/799j/gZzFq83K0Bc/IIRMeSA67emHr6yX2fea+nJuNaeCH6mlYl90JnmL
uU1Id63FtZWuVclDo/nWQ3gVT+J30OKe+zHfegG+sz6P7zFRXx1qkSQP5jX2
VKNGrkWpa4uxD2W9PUfna1Ofxn0TQ+jpXnudNZFXqp9IDeEkDuIYjkR21q+5
GdlmvoHRWGb/jhTPxjrGue6xlZuQaPZqz2ktqvAJ6lCpZ7J7X3CGPD5s9rN+
Iex2hpecYZH6x/ob0S7X8Fx7P+IWMxucoQnrEcN2dOnpMmOnGZ3cbOZd/lz+
i4vVV9hXjfkoQyl2mzfD3GJcd69Kfh//2D+B81HkM73JIxEzt4++NMRjOLJQ
aP9UnoUcTMJb0dnMT9CXjJ+i/y0/wau4r+ex2d2sp3M77uAL9/jVc/oDV/A/
HiLf/ls+S6PPdJk/NfsU18jn+Jb6D/p/Rk70HvNR1EffC/0HsBnncBK59h/S
96H+GNeZt4FL5Sa+Ia/iOfJSrjZnDOfKE/hb9SKeLL/HK+RXeJQ8jNvMT+LB
chyX67/UN4Re8m3OVv+X0+Q/eYn+E/w8PYSz3KrezA/lHVyifzHflKt4kPom
/l2u4bn68/lHeTYfVx/PZ+RsLojesz7eYzlBTlUfxMfkVJ6i/576NrmT96rf
4Ab5Iufp38oV8h5OUD/Na+Wj/Lb+al4o1/N29XIulEt4rP4RPE4ey8/sL+B3
5TzOkZP5dTmTG/XHc5b8n+/JCP3nubt8lTvtf8Ap8h0eIh/ip74rp3id/hh3
yFs4U38xX4t+p/ie/Rv5t+h3h9PlFO/NYf+PLL4o3/cOP0IrfkEb4ux/AXdn
zdc=
          "]]},
        RowBox[{"-", "0.07100000000000001`"}]],
       Annotation[#, -0.07100000000000001, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV021MlWUcx/GLXiiBgQoKxCxPomtuuIj1InPBfBjNQ7lQlyGM1DDBeZjK
cA4Lnwckiha4XDtz1nRNN20tp02dxlyp1Vqp5cOmy9BkOQ+NdLB0fc6LL9/7
d/0fzn0f7hNZ3FARSwkh1PrTnx7C5uwQElwxNoQ3kZkXQgaWOy/NCuHr/BBK
eIu8FTNdH3U2g/8aEcKA2V36Z8mpmSFcs+NJvs7ZzrPQam5QX5fr2fpWyDEM
Oet2FnWWaWas6xyMwRxnffY/0nPXrj7E0IC5asfdQwW32dOOha7zzFXyPXPD
8Vhvrr1NvAbvqJ0wV8MdZraj3vWz5uo4YWYUMnJCeMbcB2ZaUKA+ERPQoK/T
3HPqI9WWuO5w/m7yPngbPsR2nB0TQr59tWrpetOwLCOE9XId96rfxh3MdBZV
P4NvkYp6PRGOq5eod3EnRjzledzjOqzCVmzASv3l7us7n13Gh5Lf3+gQ9nrm
J9z3EvUt5tvRJ7dwpd61fMDOtlz7sRFx7MZDMw/t2GNHgvfb+Qd/JN/lavVG
fc2owQosxV775tpbjUs+ayUvw3390zmKKs/0Mr+AuL2Z5nIxDJMRwXz9b/Bb
KMYsTEvem/1p5rLwg/2T+AHGodfMv3YH1xXchxv42Gdc8z3dxs/4D/2I6r/i
Wdo900/cYfcxbpF7+Ir69+Yvojj5DvFhtNr3qfkvsAM9OIoS/QfNrTYf5w32
beKlcidflpv4bbmOm+15iUvk6fyNehWXy3O4Uc7nF+Xn+YL9GTxRTuGY+R9H
ecfkq1ykPsC58h1+z/xX/PjpEE7zKfX93C9/wouS7yj/Jq/iAvVt/KfcwpXm
o3xeXsBfqr/KJ+Uinpd8z0aG8JmcJueoF/AROYdfN39LvVtO8Ofql7lNPsdl
5ru4Qd7HaeoneJ18mEvNN/NiuZV3q8d4vryIp5ov5FfkqfxI/zx+TS7jYjmL
p8jjud38MI7Ig34nhebP8nD5F07o/5uz5Rs8ST7IQ34rx/h983G+J+/k8ear
+Ve5nm/p38w35TWcJ2d7bw75f0T4nNzrHf4Hp/A7LiBF//+C98sS
          "]]},
        RowBox[{"-", "0.14200000000000002`"}]],
       Annotation[#, -0.14200000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV03toz3scx/G3P6bGNmzH77fjkgnpOITlD5c2KYeaS7mTdYzMpdxFmksu
f4yOjhPnuKQlNDSF1EknpKTczsl1jD/ILkzk52DLZeKxP557fl+f9+X7/e73
+/Wct3zysjYRUeBPqn3E7UREWoeIW9yW73DmjxEZuOf6Pu7iAarRyXn5DxFN
Zve4HpsTsUR+mRHR7KxOTz3moARz8dF5A2fZX8oLsBAtzvfZMdGObLUXznK4
kSc5+8AJ+T03owm5+j/zF3zC19Y96O58p+fo4DkOu15kfpWckjsiI+m9sEHv
RrxzlkQ/1z+jrmtELRKukxiJX+y5YE+l62M4jgZ5mvMCu2v0F/JzZzOdNfJ0
fsEz+CXP4nWdzWdGbM+N2IUtqMBeNGchO+KAXSmu9EzPeLfcyMXqq/WV4Vcs
QSkO2TfZ3mLcd58VvBBv9I/iIsz2bEN5ICrszTKXRBp+Qh6m6p/A05GP0RjR
+mz2p5vLxk37+3ATuqHezAe7v2FS62ePJ9jjHo/9jxtwC1/wFkX6a7zLDu/0
H++0+xxvki9zjfpV8/eQ717n+RTK7Tto/gR+x2X8jUL9VeZWma/gzfZt5VJ5
F1fLa3imvIjL7BnChfIo/kd9No+TJ/JquQsPlvvyDfszubccvNT8v50i2smP
eJD6e07Kz3mB+bP8tUvEJb6oXslv5f1cYn4+P5BXci/137hO3sSzzBfxdXkG
n1Ev4AvyIJ5ivqVjxBE5XU6o9+LTcoLHm69V/0tO8VH1at4uX+Mx5v/kZfJh
Tlc/z+vlUzzSfBnPlct5r/pSniqX8DDz/Xm4PIxb9E/hsfIYzpezeYDcg3eY
T+M8+aPfb3/zV7itfIdT+l9xjvyE+8hV/Mlv5RxvMF/Br+U/uIf5Yr4rL+Za
/dv4qbyWc+Uc35uTPo88vibX+w7/j4t4iBsI/d8BkFi+/g==
          "]]},
        RowBox[{"-", "0.21300000000000002`"}]],
       Annotation[#, -0.21300000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV0nloz3Ecx/H3GH5sw8bm3hzLEWH5wxFSTOZuckSO3OUWfyCJPxAhx1Ja
cuYopOSOpNzkHorMnWMYRSY8/PHcc6/38fl+vtu32cQ5hbOTIiLXj5QGETez
Im5hLMbhbWpEWUrE3syIb7zRTHmdiP3yPtSWH5l7jBJUqxXxhPuaKeUU+Tmn
8gvup36iUUQBp6lNVJuEyahw/htOV5/BMzELSe7wlcsxF/PwUa0qTrjDSVSq
GzHNXZJ4Op9Ra8yV60UkoxKqogqy1atzDSTQxPNSeLY7dXf2E/drqlZLrRm3
UVuo19teH+ySd/9/17SIPZzg8+qtPPuUucF+H4IBGISB2G5mjDvd05/DU/Gp
ZkQvLsBou525PYq9V1r9iCwkozVyMMz8QB6OPPRGNxQ5O2EvHdecn8vf0Qgv
7Xxz9h8MxTs8xSbPeOz9XuEWfuEzCsyXZESs9je4wWudfZyXyhe4RP+S/bvI
86zTfAgrnbfN/j6swwUcQw/zB+zNs1/My5y3nCfL6/m+vIBHytN4kXM6cQ+5
F5/UH8395UE8X27AHeWWfNX5qdxC/pvum7F/navLj7iDfjlnya95iv2j/Lth
xDk+q7+HP8tbebz9SfxAnsvN9dfwC3kpj7JfwFfkEXxEvzufkTtwof2K2hE7
5YScqd+cD8uZPMB+qf4WuYx36d/nVfJlzre/mWfJOzihf5oXy4e4p/1FPEFe
yUX6M3mYPJ672G/LXeUuXGG+kPvK+Zwnp3M7OZtX20/mHPmHb72t/YtcRb7N
Zebfc4b8lHPlA/zTN32cl9gv5g/yBs62P4bvyNO51PwKfiYv5Hpyhu/moP9H
Dl+WX/qGv+AsHuIq/uIf+7ejYg==
          "]]},
        RowBox[{"-", "0.28400000000000003`"}]],
       Annotation[#, -0.28400000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV1HlMz3Ecx/GP0UJKkcrVxPzDhvmHzTWEshzZHGFsyW2OlY2Y8QfG3Mxm
/YExE7lmoZr7LHPN+YcjsRkqxOSY5tEfz56/1+fzfr1/v+q7X0r2kgmLm4QQ
kv2ojgphe/sQBrcNoVnrEKYmhDAN01Hj7nC7EOp4h5kvZobiiLOvfL9jCN94
GGa7/87ZnINZKDTXhl/b9Qav0NJ7/DC3x3m9vcfMHMcvZw/sq+dTcgZfkMdw
rM483flYgNAqhDNmIrjAngYz5+RiTPG6TC+LE/VyzedhOVqZLzUTwwf0IuND
uCxfwhzzl/Tmcie9lebzsQoJ5q+auYYonWHOUjEc153dwGdUN+52Fo0L9tTI
XbxOx2ik6WbwWIxBBFrgbUwIFVyOUWY2+WwjeXxjj8c17uA6+8rt/c4dUKVX
5/xf4ww+4CV2+rwvEkN4h3v4jVqkmX/WJoSNfs+7vDna34xXy1f4mfub+o/Q
x/uUcBHW27dP/wi24ArOYqD5o3pL9Qt4jX1reZa8lR/LuTxJnsMr7OnLA+Uh
fN59FqfLGbxMTuLecne+Y38Ud5Ub4kJYqF/BzeXn3Mv9N24nv+cc/dP8t0MI
F7nM/SGulffyDP1sfiIv4RT3m/itvJon66fxbXkin3Q/gEvlXpyp/yfWsyNH
yvHuU/iEHM+j9Svd75Zr+KD7x7xBvsWp+rt4kbyfI92X8Eq5iAfpr+CZ8nre
434hT5BncD/9Htxf7sd/zGfyCDmV+8ix3FPuzBv1m3Ky/NMz3UP/GjeTH3CN
+Y8cJ7/kbvJRrvf8FXO+fgF/krdxZ/2p/FCey5Xm1/ErOY8T5DjPTaH/RzLf
kquSfG+gDE9xBw34D4WRq9c=
          "]]},
        RowBox[{"-", "0.35500000000000004`"}]],
       Annotation[#, -0.35500000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV0k1MzwEcx/FvSigPUUnJVma2TJoLWx42Y8NcQhmH2rAcmJhsMTMOMRsO
mC0dYqZlKw/DqCaxSVww4iJP48CWcGArJS+Hz96/9/fpt/r/8rfsWleVEBE/
ZcrUiHRpyYz4lR6xLTviN17lV+RvakSd2oBaiQzKOT6ifl0/cXzEd/s/pF8y
J0UMmXk7PeIP3jBzU4Y919tLMn+Lp2DD/zvqFRIZEed5qnqb/gSMrIgEGXE3
x90kM+/cTcS7Zkbjc56MlW50qGXYu+TOWLWdauOwkWeq39fPxiaeol6tP9vz
XlzvHaVSJrf5Z3fv4BK+TIonRnTifbknvfodODQtYq65N/wx7zH30M15aq88
P/eeIs+t2Cy13l/nb2qU49IpN2SR2aYpEbvcqceDEyIO4RZ+Al/wPVjGK7HG
nfm4iC/F2/obcRVfg7t5Fs7js7Db/RTM58OTI7bbf4Jj+Gss1P+BGfwTbrV/
DQdz/K+xXf8ifuNnsdz+ZnzJqzBP/xh+4Adwg/2V+IiX4hX9YmzjhVhifyDN
b86Tebp+HrbwdFxt/73+ad6HF/Rf4BHehcvtn8IdvAGT9VtxH2/GxfZrsILX
4hn97biWl+MC+wW4kC/AAfMluIIvxyI+CefwXDxqfxTO4L98iwX2H2Aif4p9
5r9gGn+DM3kT/vYt3ML99uvxKz+JufY34TO+Dd+bP4y9vBozeZrv5rLfYwZ2
8Y++tX5plx7plmH5BzYYjgk=
          "]]},
        RowBox[{"-", "0.42600000000000005`"}]],
       Annotation[#, -0.42600000000000005`, "Tooltip"]& ], {}, {}}}], 
   {GrayLevel[0], AbsoluteThickness[3], LineBox[{{0, 0}, {1., 0}}]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.75, 
      FormBox[
       RowBox[{"-", "0.75`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.25, 
      FormBox[
       RowBox[{"-", "0.25`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.25, 
      FormBox["0.25`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.75, 
      FormBox["0.75`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}, {1.25, 
      FormBox["1.25`", TraditionalForm]}, {1.5, 
      FormBox["1.5`", TraditionalForm]}, {1.75, 
      FormBox["1.75`", TraditionalForm]}, {2., 
      FormBox["2.`", TraditionalForm]}}, {{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2, 
      FormBox["1.2`", TraditionalForm]}}, None, None},
  ImageMargins->0.,
  ImageSize->504,
  Method->{"TransparentPolygonMesh" -> True},
  PlotRange->{{-1.1536749816983884`, 
   2.1536749816983884`}, {-1.3536749816983884`, 1.3536749816983884`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 419},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[10]=",
 CellID->157949271]
}, Open  ]],

Cell[TextData[{
 "Arrows for velocity field are green, equipotential lines are red, \
streamlines are blue and panel is black.  Note orthogonality of equipotential \
lines and streamlines, streamlines parallel to velocity field and outward \
flow associated with positive ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ".  The thickened streamline along negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis suggests a branch cut for the stream function.  Also, note that the \
velocity field shows an abrupt change in normal-component velocity as the \
panel is crossed."
}], "ExampleText",
 CellID->680409942],

Cell["\<\
Potential-flow singularities may have discontinuities in velocity, velocity \
potential or stream function across the panel.  Such discontinuities are \
called jump conditions.\
\>", "ExampleText",
 CellID->62898067],

Cell["\<\
Compute the properties of tangent-component velocity across the panel.\
\>", "ExampleText",
 CellID->673890120],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"icuMinus", "=", 
   RowBox[{"Limit", "[", 
    RowBox[{"icu", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", ">", "0"}], ",", 
        RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
       "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"icuPlus", "=", 
   RowBox[{"Limit", "[", 
    RowBox[{"icu", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", ">", "0"}], ",", 
        RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
       "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", 
      RowBox[{"-", "1"}]}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"icuJump", "=", 
   RowBox[{"icuPlus", "-", "icuMinus"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"uJump", "=", 
  RowBox[{"a", " ", "icuJump"}]}]}], "Input",
 CellLabel->"In[11]:=",
 CellID->98488054],

Cell[BoxData["0"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->71193903]
}, Open  ]],

Cell["\<\
Tangent-component velocity is continuous across the source panel.\
\>", "ExampleText",
 CellID->9527100],

Cell["\<\
Compute the properties of the normal-component velocity across the the panel.\
\>", "ExampleText",
 CellID->134318740],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"icwMinus", "=", 
   RowBox[{"Limit", "[", 
    RowBox[{"icw", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", ">", "0"}], ",", 
        RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
       "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"icwPlus", "=", 
   RowBox[{"Limit", "[", 
    RowBox[{"icw", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", ">", "0"}], ",", 
        RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
       "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", 
      RowBox[{"-", "1"}]}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"icwJump", "=", 
   RowBox[{"icwPlus", "-", "icwMinus"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"wJump", "=", 
  RowBox[{"a", " ", "icwJump"}]}]}], "Input",
 CellLabel->"In[12]:=",
 CellID->326115239],

Cell[BoxData["a"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->63567666]
}, Open  ]],

Cell[TextData[{
 "There is a jump in the normal-component velocity across the source panel \
equal to the value of the source density function, which is the constant ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 "."
}], "ExampleText",
 CellID->78220692],

Cell["\<\
Compute the properties of the velocity potential across the panel.\
\>", "ExampleText",
 CellID->211831706],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ic\[Phi]Jump", "=", 
   RowBox[{
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Phi]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
        "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", 
       RowBox[{"-", "1"}]}]}], "]"}], "-", 
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Phi]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
        "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[Phi]Jump", "=", 
  RowBox[{"a", " ", "ic\[Phi]Jump"}]}]}], "Input",
 CellLabel->"In[13]:=",
 CellID->189498322],

Cell[BoxData["0"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[13]=",
 CellID->852249739]
}, Open  ]],

Cell["Velocity potential is continuous across the source panel.", \
"ExampleText",
 CellID->56731244],

Cell["\<\
Compute the properties of the stream function across the panel.\
\>", "ExampleText",
 CellID->403496840],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ic\[Psi]Jump", "=", 
   RowBox[{
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Psi]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
        "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", 
       RowBox[{"-", "1"}]}]}], "]"}], "-", 
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Psi]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
        "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[Psi]Jump", "=", 
  RowBox[{"a", " ", "ic\[Psi]Jump"}]}]}], "Input",
 CellLabel->"In[14]:=",
 CellID->102020424],

Cell[BoxData[
 RowBox[{"a", " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", "x"}], "+", "\[Delta]"}], ")"}]}]], "Output",
 ImageSize->{63, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[14]=",
 CellID->261168276]
}, Open  ]],

Cell["\<\
There is a jump in the stream function consistent with the jump in \
normal-component velocity.\
\>", "ExampleText",
 CellID->325279064],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"wJump", "\[Equal]", 
  RowBox[{"-", 
   RowBox[{"D", "[", 
    RowBox[{"\[Psi]Jump", ",", "x"}], "]"}]}]}]], "Input",
 CellLabel->"In[15]:=",
 CellID->114614373],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[15]=",
 CellID->126065777]
}, Open  ]],

Cell[TextData[{
 "Look at the properties of the stream function as the negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis is crossed in a counterclockwise direction."
}], "ExampleText",
 CellID->635547329],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ic\[Psi]BranchJump", "=", 
   RowBox[{
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Psi]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"x", "<", "0"}]}], "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", 
       RowBox[{"-", "1"}]}]}], "]"}], "-", 
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Psi]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"x", "<", "0"}]}], "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[Psi]BranchJump", "=", 
  RowBox[{"a", " ", "ic\[Psi]BranchJump"}]}]}], "Input",
 CellLabel->"In[16]:=",
 CellID->81019929],

Cell[BoxData[
 RowBox[{"a", " ", "\[Delta]"}]], "Output",
 ImageSize->{21, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[16]=",
 CellID->34864227]
}, Open  ]],

Cell[TextData[{
 "There is a branch cut along the negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis for the stream function.  This abrupt change in value of the stream \
function as the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis is crossed causes the thickening of the streamline unless special \
care is taken with the options of ContourPlot."
}], "ExampleText",
 CellID->78970066]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Scope",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->1293636265],

Cell[TextData[{
 "The general application of influence coefficients is in the construction of \
a panel method.  The current version of ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " does not include any panel methods based on source panels of constant \
strength.  See the ",
 ButtonBox["scope section of ICDoubletConstant",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletConstant#1293636265"],
 " for an example of a general application of influence coefficients for \
doublet panels of constant strength."
}], "ExampleText",
 CellID->246788676]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[{
 StyleBox[ButtonBox["ICSourcePoint",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICSourcePoint"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICSourceLinear",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICSourceLinear"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICDoubletConstant",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletConstant"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICVortexConstant",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICVortexConstant"],
  FontFamily->"Verdana"],
 " "
}], "SeeAlso",
 CellID->745892494]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["2.1 Introduction to InfluenceCoefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"]], \
"Tutorials",
 CellID->543144427],

Cell[TextData[ButtonBox["2.3 Isolated Panel",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/IsolatedPanel"]], "Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["InfluenceCoefficients",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/InfluenceCoefficients"]], "MoreAbout",
 CellID->503989898]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"ICSourceConstant - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 0, 46.0344000}", "context" -> 
    "Aerodynamics`InfluenceCoefficients`", "keywords" -> {}, "index" -> True, 
    "label" -> "Aerodynamics Package Paclet Symbol", "language" -> "en", 
    "paclet" -> "Aerodynamics Package", "status" -> "None", "summary" -> 
    "ICSourceConstant[r_f, r_p] computes velocity influence coefficients at \
field points r_f, due to connected source panels of constant strength whose \
endpoints are located by r_p.ICPhiSourceConstant[r_f, r_p] computes \
velocity-potential influence coefficients at field points r_f due to \
connected source panels of constant strength whose endpoints are located by \
r_p.ICPsiVortexConstant[r_f, r_p] computes stream-function influence \
coefficients at field points r_f due to connected vortex panels of constant \
strength whose endpoints are located by r_p.", "synonyms" -> {}, "title" -> 
    "ICSourceConstant", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/ICSourceConstant"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[9670, 293, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->158617825]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 239014, 4976}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[609, 21, 76, 1, 70, "SymbolColorBar"],
Cell[688, 24, 191, 5, 70, "LinkTrail"],
Cell[882, 31, 3575, 82, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4482, 117, 59, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4544, 120, 2269, 71, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6850, 196, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->285780094],
Cell[7433, 215, 364, 11, 70, "Notes",
 CellID->1067943069],
Cell[7800, 228, 604, 18, 70, "Notes",
 CellID->1016887069],
Cell[8407, 248, 433, 10, 70, "Notes",
 CellID->30234373],
Cell[8843, 260, 713, 25, 70, "Notes",
 CellID->223280793],
Cell[9559, 287, 74, 1, 70, "Notes",
 CellID->529540534]
}, Closed]],
Cell[CellGroupData[{
Cell[9670, 293, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->158617825],
Cell[CellGroupData[{
Cell[10156, 310, 147, 5, 70, "ExampleSection",
 CellID->78764249],
Cell[10306, 317, 128, 6, 70, "ExampleText",
 CellID->513831186],
Cell[10437, 325, 100, 3, 70, "Input",
 CellID->267685788],
Cell[10540, 330, 427, 11, 70, "ExampleText",
 CellID->659456463],
Cell[10970, 343, 204, 6, 70, "ExampleText",
 CellID->42111717],
Cell[11177, 351, 495, 18, 70, "Input",
 CellID->64953589],
Cell[11675, 371, 87, 1, 70, "ExampleText",
 CellID->326501090],
Cell[CellGroupData[{
Cell[11787, 376, 307, 10, 70, "Input",
 CellID->560964283],
Cell[12097, 388, 873, 29, 67, "Output",
 CellID->427323328]
}, Open  ]],
Cell[12985, 420, 564, 11, 70, "ExampleText",
 CellID->334847722],
Cell[CellGroupData[{
Cell[13574, 435, 436, 15, 70, "Input",
 CellID->176773044],
Cell[14013, 452, 726, 24, 67, "Output",
 CellID->718515614]
}, Open  ]],
Cell[14754, 479, 142, 4, 70, "ExampleText",
 CellID->44302699],
Cell[CellGroupData[{
Cell[14921, 487, 682, 21, 70, "Input",
 CellID->575890672],
Cell[15606, 510, 817, 27, 58, "Output",
 CellID->440902438]
}, Open  ]],
Cell[16438, 540, 113, 3, 70, "ExampleText",
 CellID->472419597],
Cell[CellGroupData[{
Cell[16576, 547, 639, 20, 70, "Input",
 CellID->8664355],
Cell[17218, 569, 788, 25, 67, "Output",
 CellID->423449330]
}, Open  ]],
Cell[18021, 597, 203, 4, 70, "ExampleText",
 CellID->9427774],
Cell[18227, 603, 380, 12, 70, "Input",
 CellID->258291294],
Cell[18610, 617, 947, 34, 70, "ExampleText",
 CellID->78226744],
Cell[19560, 653, 259, 9, 70, "ExampleText",
 CellID->421446271],
Cell[CellGroupData[{
Cell[19844, 666, 451, 16, 70, "Input",
 CellID->481371799],
Cell[20298, 684, 207, 7, 36, "Output",
 CellID->1267074]
}, Open  ]],
Cell[CellGroupData[{
Cell[20542, 696, 363, 13, 70, "Input",
 CellID->692553449],
Cell[20908, 711, 209, 7, 36, "Output",
 CellID->107032324]
}, Open  ]],
Cell[21132, 721, 821, 25, 70, "ExampleText",
 CellID->78531558],
Cell[CellGroupData[{
Cell[21978, 750, 4607, 132, 70, "Input",
 CellID->1746043248],
Cell[26588, 884, 197614, 3603, 70, "Output",
 CellID->157949271]
}, Open  ]],
Cell[224217, 4490, 648, 14, 70, "ExampleText",
 CellID->680409942],
Cell[224868, 4506, 226, 5, 70, "ExampleText",
 CellID->62898067],
Cell[225097, 4513, 121, 3, 70, "ExampleText",
 CellID->673890120],
Cell[CellGroupData[{
Cell[225243, 4520, 1134, 33, 70, "Input",
 CellID->98488054],
Cell[226380, 4555, 158, 5, 36, "Output",
 CellID->71193903]
}, Open  ]],
Cell[226553, 4563, 114, 3, 70, "ExampleText",
 CellID->9527100],
Cell[226670, 4568, 128, 3, 70, "ExampleText",
 CellID->134318740],
Cell[CellGroupData[{
Cell[226823, 4575, 1135, 33, 70, "Input",
 CellID->326115239],
Cell[227961, 4610, 158, 5, 36, "Output",
 CellID->63567666]
}, Open  ]],
Cell[228134, 4618, 268, 7, 70, "ExampleText",
 CellID->78220692],
Cell[228405, 4627, 117, 3, 70, "ExampleText",
 CellID->211831706],
Cell[CellGroupData[{
Cell[228547, 4634, 1019, 29, 70, "Input",
 CellID->189498322],
Cell[229569, 4665, 159, 5, 36, "Output",
 CellID->852249739]
}, Open  ]],
Cell[229743, 4673, 101, 2, 70, "ExampleText",
 CellID->56731244],
Cell[229847, 4677, 114, 3, 70, "ExampleText",
 CellID->403496840],
Cell[CellGroupData[{
Cell[229986, 4684, 1019, 29, 70, "Input",
 CellID->102020424],
Cell[231008, 4715, 255, 9, 36, "Output",
 CellID->261168276]
}, Open  ]],
Cell[231278, 4727, 146, 4, 70, "ExampleText",
 CellID->325279064],
Cell[CellGroupData[{
Cell[231449, 4735, 185, 6, 70, "Input",
 CellID->114614373],
Cell[231637, 4743, 162, 5, 36, "Output",
 CellID->126065777]
}, Open  ]],
Cell[231814, 4751, 231, 6, 70, "ExampleText",
 CellID->635547329],
Cell[CellGroupData[{
Cell[232070, 4761, 940, 27, 70, "Input",
 CellID->81019929],
Cell[233013, 4790, 187, 6, 36, "Output",
 CellID->34864227]
}, Open  ]],
Cell[233215, 4799, 434, 11, 70, "ExampleText",
 CellID->78970066]
}, Open  ]],
Cell[CellGroupData[{
Cell[233686, 4815, 221, 7, 70, "ExampleSection",
 CellID->1293636265],
Cell[233910, 4824, 578, 13, 70, "ExampleText",
 CellID->246788676]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[234537, 4843, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[234957, 4857, 655, 22, 70, "SeeAlso",
 CellID->745892494]
}, Open  ]],
Cell[CellGroupData[{
Cell[235649, 4884, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[236071, 4898, 209, 5, 70, "Tutorials",
 CellID->543144427],
Cell[236283, 4905, 159, 3, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[CellGroupData[{
Cell[236479, 4913, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[236901, 4927, 167, 3, 70, "MoreAbout",
 CellID->503989898]
}, Open  ]],
Cell[237083, 4933, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

