(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    278584,       5963]
NotebookOptionsPosition[    270768,       5711]
NotebookOutlinePosition[    272917,       5764]
CellTagsIndexPosition[    272830,       5759]
WindowTitle->ICSourceLinear - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"2.1 Introduction to Influence Coefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"], "\
\<\"2.3 Isolated Panel\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IsolatedPanel"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"ICSourcePoint\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICSourcePoint"], "\<\"ICSourceConstant\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICSourceConstant"], "\<\"ICDoubletLinear\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICDoubletLinear"], "\<\"ICVortexLinear\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/ref/ICVortexLinear"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"InfluenceCoefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/InfluenceCoefficients"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/ICSourceLinear\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/ICSourceLinear"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
ICSourceLinear.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/ICSourceLinear", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["ICSourceLinear", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICSourceLinear", "[", 
       RowBox[{
        SubscriptBox[
         StyleBox["r", "TI"], 
         StyleBox["f", "TI"]], ",", 
        SubscriptBox[
         StyleBox["r", "TI"], 
         StyleBox["p", "TI"]]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes velocity influence coefficients at field \
points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     ", due to connected source panels of linearly varying strength whose \
endpoints are located by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICPhiSourceLinear", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes velocity-potential influence coefficients at \
field points ",
     Cell[BoxData[
      RowBox[{
       SubscriptBox[
        StyleBox["r", "TI"], 
        StyleBox["f", "TI"]], ","}]], "InlineFormula"],
     " due to connected source panels of linearly varying strength whose \
endpoints are located by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICPsiSourceLinear", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]",
     "computes stream-function influence coefficients at field points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     ", due to connected source panels of linearly varying strength whose \
endpoints are located by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, 
   "Rows" -> {None, None, {None}}, "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->105283275],

Cell[TextData[{
 "Input arguments are collections of vectors of the data type ",
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], "InlineFormula"],
 ".  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation may be used for single vectors."
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 "The source density for the source panel of linearly varying strength is \
written as ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["\[Sigma]", "TR"], 
    RowBox[{"(", 
     StyleBox["x", "TI"], ")"}]}], "=", 
   RowBox[{
    StyleBox["a", "TI"], "+", 
    RowBox[{
     StyleBox["b", "TI"], " ", 
     StyleBox["x", "TI"]}]}]}]], "InlineFormula"],
 ", where ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 " is the distance from the beginning of the panel, constants ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 " are the strength parameters for the panel.   For collections of panels, \
the strength parameters are written in two lists, one for the constant term \
and one for the linear term in the equation for source density.  Within each \
list strength parameters are ordered according to the clockwise numbering \
convention for panels."
}], "Notes",
 CellID->78062312],

Cell[TextData[{
 "Results for velocity influence coefficients are two collections of vectors, \
one associated with the ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " parameters and the other, with the ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 " parameters.  Influence coefficients for velocity potential and stream \
function are two lists of scalar influence coefficients.  For the special \
case of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation being used to specify a single field point, output is simplified \
as illustrated in the examples."
}], "Notes",
 CellID->433990591],

Cell[TextData[{
 "The velocity, velocity potential and stream function for a panel are \
written as the sum of two terms, each term being the product of a geometric \
influence coefficient and the appropriate strength parameter.\t\t\t\t\t\t\t\t\
\t\t\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["V", "TI"], "=", 
   RowBox[{
    RowBox[{
     StyleBox["a", "TI"], " ", 
     StyleBox["ica", "TI"]}], "+", 
    RowBox[{
     StyleBox["b", "TI"], " ", 
     StyleBox["icb", "TI"]}]}]}]], "InlineFormula"],
 "\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Phi]", "TR"], "=", 
   RowBox[{
    RowBox[{
     StyleBox["a", "TI"], " ", 
     StyleBox["ic\[Phi]a", "TI"]}], "+", 
    RowBox[{
     StyleBox["b", "TI"], " ", 
     StyleBox["ic\[Phi]b", "TI"]}]}]}]], "InlineFormula"],
 "\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Psi]", "TR"], "=", 
   RowBox[{
    RowBox[{
     StyleBox["a", "TI"], " ", 
     StyleBox["ic\[Psi]a", "TI"]}], "+", " ", 
    RowBox[{
     StyleBox["b", "TI"], " ", 
     StyleBox["ic\[Psi]b", "TI"]}]}]}]], "InlineFormula"]
}], "Notes",
 CellID->213924258],

Cell["New in 1.0   |   Last modified in 1.2", "Notes",
 CellID->529540534]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->190734705],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->5012110],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->513831186],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->66474307],

Cell[TextData[{
 "This example demonstrates some properties of a single panel.  Algebraic \
equations for influence coefficients are simplest in a local coordinate \
system where the panel extends from the origin along the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 " axis to ",
 Cell[BoxData[
  StyleBox["\[Delta]", "TR"]], "InlineFormula"],
 ", the length of the panel."
}], "ExampleText",
 CellID->659456463],

Cell[TextData[{
 "Specify arbitrary field point and panel endpoints.  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation is used for a single vector.an "
}], "ExampleText",
 CellID->42111717],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"rF", ",", "rP"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"x", ",", "z"}], "}"}], ",", 
     RowBox[{"MakeCartesianVectors", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "\[Delta]"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}]}], 
  ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->64953589],

Cell[TextData[{
 "Compute the velocity influence coefficients.  There are separate influence \
coefficients for the constant and linear terms in the function describing \
source distribution.  Each influence coefficient is a single vector written \
in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation."
}], "ExampleText",
 CellID->326501090],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"icua", ",", "icwa"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"icub", ",", "icwb"}], "}"}]}], "}"}], "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"ICSourceLinear", "[", 
      RowBox[{"rF", ",", "rP"}], "]"}], ",", 
     RowBox[{"\[Delta]", ">", "0"}]}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->560964283],

Cell[TextData[{
 "Display the ",
 Cell[BoxData[
  StyleBox["z", "TI"]], "InlineFormula"],
 "-component of the velocity influence coefficient associated with the \
constant term."
}], "ExampleText",
 CellID->217164756],

Cell[CellGroupData[{

Cell[BoxData["icwa"], "Input",
 CellLabel->"In[4]:=",
 CellID->319896790],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"ArcTan", "[", 
     RowBox[{"x", ",", "z"}], "]"}]}], "+", 
   RowBox[{"ArcTan", "[", 
    RowBox[{
     RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}]}], 
  RowBox[{"2", " ", "\[Pi]"}]]], "Output",
 ImageSize->{212, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->152403999]
}, Open  ]],

Cell[TextData[{
 "Display the algebraic equations for components of the velocity influence \
coefficient associated with strength parameter ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ", where the ArcTan function is replaced by named angles (See ",
 ButtonBox["2.1 Introduction to InfluenceCoefficients",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"],
 " for sketch defining angles).  Although the substitution of named angles is \
made to simplify the appearance of equations, note that the original \
equations are maintained for subsequent computations."
}], "ExampleText",
 CellID->120984796],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"icua", ",", "icwa"}], "}"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
     "\[Theta]2"}]}], "}"}]}]], "Input",
 CellLabel->"In[5]:=",
 CellID->176773044],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"Log", "[", 
      FractionBox[
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"], "-", 
        RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
        SuperscriptBox["\[Delta]", "2"]}], 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"]}]], "]"}], 
     RowBox[{"4", " ", "\[Pi]"}]]}], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"-", "\[Theta]1"}], "+", "\[Theta]2"}], 
    RowBox[{"2", " ", "\[Pi]"}]]}], "}"}]], "Output",
 ImageSize->{201, 46},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->355656643]
}, Open  ]],

Cell[TextData[{
 "Display the algebraic equations for components of the velocity influence \
coefficient associated with strength parameter ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 "."
}], "ExampleText",
 CellID->118713566],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"icub", ",", "icwb"}], "}"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
     "\[Theta]2"}]}], "}"}]}]], "Input",
 CellLabel->"In[6]:=",
 CellID->25568170],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{"2", " ", "\[Delta]"}], "+", 
      RowBox[{"2", " ", "z", " ", "\[Theta]1"}], "-", 
      RowBox[{"2", " ", "z", " ", "\[Theta]2"}], "+", 
      RowBox[{"x", " ", 
       RowBox[{"Log", "[", 
        FractionBox[
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["z", "2"], "-", 
          RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
          SuperscriptBox["\[Delta]", "2"]}], 
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["z", "2"]}]], "]"}]}]}], 
     RowBox[{"4", " ", "\[Pi]"}]]}], ",", 
   FractionBox[
    RowBox[{
     RowBox[{
      RowBox[{"-", "2"}], " ", "x", " ", "\[Theta]1"}], "+", 
     RowBox[{"2", " ", "x", " ", "\[Theta]2"}], "+", 
     RowBox[{"z", " ", 
      RowBox[{"Log", "[", 
       FractionBox[
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"], "-", 
         RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
         SuperscriptBox["\[Delta]", "2"]}], 
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"]}]], "]"}]}]}], 
    RowBox[{"4", " ", "\[Pi]"}]]}], "}"}]], "Output",
 ImageSize->{504, 46},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->6664491]
}, Open  ]],

Cell["\<\
Compute the velocity-potential influence coefficient and display in terms of \
named angles.\
\>", "ExampleText",
 CellID->44302699],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Phi]a", ",", "ic\[Phi]b"}], "}"}], "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"ICPhiSourceLinear", "[", 
      RowBox[{"rF", ",", "rP"}], "]"}], ",", 
     RowBox[{"\[Delta]", ">", "0"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"ic\[Phi]a", ",", "ic\[Phi]b"}], "}"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
     "\[Theta]2"}]}], "}"}]}]}], "Input",
 CellLabel->"In[7]:=",
 CellID->575890672],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{
      RowBox[{"-", "2"}], " ", "z", " ", "\[Theta]1"}], "+", 
     RowBox[{"2", " ", "z", " ", "\[Theta]2"}], "+", 
     RowBox[{"x", " ", 
      RowBox[{"Log", "[", 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"]}], "]"}]}], "-", 
     RowBox[{"x", " ", 
      RowBox[{"Log", "[", 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"], "-", 
        RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
        SuperscriptBox["\[Delta]", "2"]}], "]"}]}], "+", 
     RowBox[{"\[Delta]", " ", 
      RowBox[{"Log", "[", 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"], "-", 
        RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
        SuperscriptBox["\[Delta]", "2"]}], "]"}]}]}], 
    RowBox[{"4", " ", "\[Pi]"}]], ",", 
   RowBox[{
    FractionBox["1", 
     RowBox[{"8", " ", "\[Pi]"}]], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "2"}], " ", "x", " ", "\[Delta]"}], "+", 
      RowBox[{"4", " ", "x", " ", "z", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "\[Theta]1"}], "+", "\[Theta]2"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["x", "2"], "-", 
         SuperscriptBox["z", "2"]}], ")"}], " ", 
       RowBox[{"Log", "[", 
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"]}], "]"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          SuperscriptBox["x", "2"]}], "+", 
         SuperscriptBox["z", "2"], "+", 
         SuperscriptBox["\[Delta]", "2"]}], ")"}], " ", 
       RowBox[{"Log", "[", 
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"], "-", 
         RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
         SuperscriptBox["\[Delta]", "2"]}], "]"}]}]}], ")"}]}]}], 
  "}"}]], "Output",
 ImageSize->{584, 69},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->37289698]
}, Open  ]],

Cell["Compute stream-function influence coefficients.", "ExampleText",
 CellID->22197235],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Psi]a", ",", "ic\[Psi]b"}], "}"}], "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"ICPsiSourceLinear", "[", 
      RowBox[{"rF", ",", "rP"}], "]"}], ",", 
     RowBox[{"\[Delta]", ">", "0"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"ic\[Psi]a", ",", "ic\[Psi]b"}], "}"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
     "\[Theta]2"}]}], "}"}]}]}], "Input",
 CellLabel->"In[8]:=",
 CellID->8664355],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{
       RowBox[{"-", "2"}], " ", "x", " ", "\[Theta]1"}], "+", 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "\[Delta]"}], ")"}], " ", "\[Theta]2"}], "+", 
      RowBox[{"z", " ", 
       RowBox[{"Log", "[", 
        FractionBox[
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["z", "2"], "-", 
          RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
          SuperscriptBox["\[Delta]", "2"]}], 
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["z", "2"]}]], "]"}]}]}], 
     RowBox[{"4", " ", "\[Pi]"}]]}], ",", 
   FractionBox[
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["x", "2"], "-", 
        SuperscriptBox["z", "2"]}], ")"}], " ", "\[Theta]1"}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox["x", "2"]}], "+", 
        SuperscriptBox["z", "2"], "+", 
        SuperscriptBox["\[Delta]", "2"]}], ")"}], " ", "\[Theta]2"}], "-", 
     RowBox[{"z", " ", 
      RowBox[{"(", 
       RowBox[{"\[Delta]", "+", 
        RowBox[{"x", " ", 
         RowBox[{"Log", "[", 
          FractionBox[
           RowBox[{
            SuperscriptBox["x", "2"], "+", 
            SuperscriptBox["z", "2"], "-", 
            RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
            SuperscriptBox["\[Delta]", "2"]}], 
           RowBox[{
            SuperscriptBox["x", "2"], "+", 
            SuperscriptBox["z", "2"]}]], "]"}]}]}], ")"}]}]}], 
    RowBox[{"4", " ", "\[Pi]"}]]}], "}"}]], "Output",
 ImageSize->{383, 94},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->268350184]
}, Open  ]],

Cell["\<\
Write the velocity components, velocity potential and stream function in \
terms of geometric influence coefficients and strength parameters.\
\>", "ExampleText",
 CellID->9427774],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"u", ",", "w"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"a", " ", "icua"}], "+", 
      RowBox[{"b", " ", "icub"}]}], ",", 
     RowBox[{
      RowBox[{"a", " ", "icwa"}], "+", 
      RowBox[{"b", " ", "icwb"}]}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Phi]", ",", "\[Psi]"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"a", " ", "ic\[Phi]a"}], "+", 
      RowBox[{"b", " ", "ic\[Phi]b"}]}], ",", 
     RowBox[{
      RowBox[{"a", " ", "ic\[Psi]a"}], "+", 
      RowBox[{"b", " ", "ic\[Psi]b"}]}]}], "}"}]}], ";"}]}], "Input",
 CellLabel->"In[9]:=",
 CellID->258291294],

Cell[TextData[{
 "Verify that the relations between velocity, velocity potential and stream \
function are satisfied.\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["u", "TI"], "=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Phi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["x", "TI"]}]], "=", 
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Psi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["z", "TI"]}]]}]}]], "InlineFormula"],
 "\t\t\t",
 Cell[BoxData[
  RowBox[{"w", "=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Phi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["z", "TI"]}]], "=", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{"\[PartialD]", 
       StyleBox["\[Psi]", "TR"]}], 
      RowBox[{"\[PartialD]", 
       StyleBox["x", "TI"]}]]}]}]}]], "InlineFormula"],
 "\t\t\t\t"
}], "ExampleText",
 CellID->78226744],

Cell[TextData[{
 "The variables ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  StyleBox["z", "TI"]], "InlineFormula"],
 " need to be declared as real in the following tests."
}], "ExampleText",
 CellID->421446271],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "\[Equal]", 
      RowBox[{"D", "[", 
       RowBox[{"\[Phi]", ",", "x"}], "]"}]}], ",", 
     RowBox[{"u", "\[Equal]", 
      RowBox[{"D", "[", 
       RowBox[{"\[Psi]", ",", "z"}], "]"}]}]}], "}"}], ",", 
   RowBox[{"Assumptions", "\[Rule]", 
    RowBox[{
     RowBox[{"Element", "[", 
      RowBox[{"x", ",", "Reals"}], "]"}], " ", "&&", " ", 
     RowBox[{"Element", "[", 
      RowBox[{"z", ",", "Reals"}], "]"}]}]}]}], "]"}]], "Input",
 CellLabel->"In[10]:=",
 CellID->13857323],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[10]=",
 CellID->292255161]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"w", "\[Equal]", 
      RowBox[{"D", "[", 
       RowBox[{"\[Phi]", ",", "z"}], "]"}]}], ",", 
     RowBox[{"w", "\[Equal]", 
      RowBox[{"-", 
       RowBox[{"D", "[", 
        RowBox[{"\[Psi]", ",", "x"}], "]"}]}]}]}], "}"}], ",", 
   RowBox[{"Assumptions", "\[Rule]", 
    RowBox[{
     RowBox[{"Element", "[", 
      RowBox[{"x", ",", "Reals"}], "]"}], " ", "&&", " ", 
     RowBox[{"Element", "[", 
      RowBox[{"z", ",", "Reals"}], "]"}]}]}]}], "]"}]], "Input",
 CellLabel->"In[11]:=",
 CellID->499128776],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->537757821]
}, Open  ]],

Cell[TextData[{
 "Flow visualization provides a plausibility check on the computed flow \
field.  Since the contribution of the ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " parameter is illustrated in ICSourceConstant, set ",
 Cell[BoxData[
  RowBox[{
   StyleBox["a", "TI"], "=", "0"}]], "InlineFormula"],
 ".  Set ",
 Cell[BoxData[
  RowBox[{
   StyleBox["b", "TI"], "=", "2"}]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Delta]", "TR"], "=", "1"}]], "InlineFormula"],
 " to provide unit integrated panel strength,  ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", 
     StyleBox["\[Delta]", "TR"]], 
    RowBox[{"\[Sigma]", 
     RowBox[{"(", 
      StyleBox["x", "TI"], ")"}], 
     RowBox[{"\[DifferentialD]", 
      StyleBox["x", "TI"]}]}]}], "=", "1"}]], "InlineFormula"],
 ".  Circular regions near the edges of the panel are excluded from the \
vector plot to avoid showing vectors of very large magnitude."
}], "ExampleText",
 CellID->78531558],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xMin", ",", "xMax", ",", "zMin", ",", "zMax"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "2", ",", 
     RowBox[{"-", "1.2"}], ",", "1.2"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"uPlot", ",", "wPlot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"u", ",", "w"}], "}"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", "\[Rule]", "0"}], ",", 
      RowBox[{"b", "\[Rule]", "2"}], ",", 
      RowBox[{"\[Delta]", "\[Rule]", "1"}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Phi]Plot", ",", "\[Psi]Plot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\[Phi]", ",", "\[Psi]"}], "}"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", "\[Rule]", "0"}], ",", 
      RowBox[{"b", "\[Rule]", "2"}], ",", 
      RowBox[{"\[Delta]", "\[Rule]", "1"}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gV", "=", 
   RowBox[{"VectorPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"uPlot", ",", "wPlot"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"RegionFunction", "\[Rule]", 
      RowBox[{"Function", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"x", ",", "z"}], "}"}], ",", 
        RowBox[{
         RowBox[{"0.01", "<", 
          RowBox[{
           RowBox[{"x", "^", "2"}], "+", 
           RowBox[{"z", "^", "2"}]}]}], " ", "&&", " ", 
         RowBox[{"0.01", "<", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"x", "-", "1"}], ")"}], "^", "2"}], "+", 
           RowBox[{"z", "^", "2"}]}]}]}]}], "]"}]}], ",", 
     RowBox[{"VectorColorFunction", "->", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"RGBColor", "[", 
         RowBox[{"0", ",", "1", ",", "0"}], "]"}], "&"}], ")"}]}], ",", 
     RowBox[{"VectorPoints", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"20", ",", "20"}], "}"}]}], ",", 
     RowBox[{"VectorScale", "\[Rule]", "Medium"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Phi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Phi]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "7"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Red"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Psi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "13"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Blue"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gPanel", "=", 
   RowBox[{"Graphics", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"GrayLevel", "[", "0", "]"}], ",", 
      RowBox[{"AbsoluteThickness", "[", "3", "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"Transpose", "[", 
        RowBox[{"Components", "[", 
         RowBox[{"rP", "/.", 
          RowBox[{"\[Delta]", "\[Rule]", "1.0"}]}], "]"}], "]"}], "]"}]}], 
     "}"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gV", ",", "g\[Phi]", ",", "g\[Psi]", ",", "gPanel", ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
   RowBox[{"FrameTicks", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.0"}], ",", "2.0", ",", "0.25"}], "]"}], "]"}], ",", 
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.2"}], ",", "1.2", ",", "0.2"}], "]"}], "]"}], ",", 
      "None", ",", "None"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", " ", "72"}]}], ",", 
   RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellLabel->"In[12]:=",
 CellID->1746043248],

Cell[BoxData[
 GraphicsBox[{{{}, {
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.005832347130626463, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9870827825011577, -1.190432306930871}, \
{-1.0129172174988423`, -1.2095676930691286`}}]}}, 
      {Arrowheads[{{0.0060524304575283495`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9861055252609248, -1.0644534017627367`}, \
{-1.0138944747390752`, -1.0829150192898946`}}]}}, 
      {Arrowheads[{{0.006272151335081758, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9850964597381763, -0.9386094782156412}, \
{-1.0149035402618238`, -0.956127363889622}}]}}, 
      {Arrowheads[{{0.00648720772305301, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9840768324710009, -0.8129205205640996}, \
{-1.015923167528999, -0.8291847425937953}}]}}, 
      {Arrowheads[{{0.006692164383243755, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9830759850234515, -0.6874036890295363}, \
{-1.0169240149765486`, -0.7020699951809903}}]}}, 
      {Arrowheads[{{0.006880477305597652, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9821316747909672, -0.5620700219048621}, \
{-1.0178683252090328`, -0.5747720833582962}}]}}, 
      {Arrowheads[{{0.007044720770665323, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9812889526508116, -0.43692039642060676`}, \
{-1.0187110473491885`, -0.4472901298951831}}]}}, 
      {Arrowheads[{{0.0071771014325310006`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9805969035537518, -0.3119414413270435}, \
{-1.0194030964462482`, -0.319637506041378}}]}}, 
      {Arrowheads[{{0.007270292899646266, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9801029320612091, -0.18710270322418682`}, \
{-1.0198970679387909`, -0.19184466519686622`}}]}}, 
      {Arrowheads[{{0.007318500711098857, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9798452145987611, -0.06235672886386332}, \
{-1.020154785401239, -0.06395906060982129}}]}}, 
      {Arrowheads[{{0.007318500711098857, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9798452145987611, 
         0.062356728863862926`}, {-1.020154785401239, 0.0639590606098209}}]}}, 
      {Arrowheads[{{0.007270292899646267, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9801029320612091, 
         0.1871027032241864}, {-1.0198970679387909`, 
         0.19184466519686585`}}]}}, 
      {Arrowheads[{{0.0071771014325310006`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9805969035537518, 
         0.3119414413270431}, {-1.0194030964462482`, 0.3196375060413776}}]}}, 
      {Arrowheads[{{0.007044720770665323, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9812889526508116, 
         0.4369203964206064}, {-1.0187110473491885`, 
         0.44729012989518274`}}]}}, 
      {Arrowheads[{{0.006880477305597652, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9821316747909672, 
         0.5620700219048617}, {-1.0178683252090328`, 0.5747720833582958}}]}}, 
      {Arrowheads[{{0.0066921643832437, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9830759850234516, 
         0.6874036890295359}, {-1.0169240149765484`, 0.7020699951809899}}]}}, 
      {Arrowheads[{{0.0064872077230530645`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9840768324710008, 
         0.8129205205640991}, {-1.0159231675289992`, 0.8291847425937948}}]}}, 
      {Arrowheads[{{0.006272151335081758, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9850964597381763, 
         0.9386094782156408}, {-1.0149035402618238`, 0.9561273638896216}}]}}, 
      {Arrowheads[{{0.0060524304575283495`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9861055252609248, 
         1.0644534017627363`}, {-1.0138944747390752`, 
         1.0829150192898942`}}]}}, 
      {Arrowheads[{{0.005832347130626463, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9870827825011577, 
         1.1904323069308707`}, {-1.0129172174988423`, 
         1.2095676930691281`}}]}}, 
      {Arrowheads[{{0.006206248173771628, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8288862450476128, -1.189144343626524}, \
{-0.8553242812681765, -1.2108556563734756`}}]}}, 
      {Arrowheads[{{0.006473986083373216, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8277424189649016, -1.063097168977209}, \
{-0.8564681073508877, -1.0842712520754223`}}]}}, 
      {Arrowheads[{{0.006746016118932338, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8265349962223457, -0.9372066613836282}, \
{-0.8576755300934435, -0.9575301807216351}}]}}, 
      {Arrowheads[{{0.007017359038666378, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8252858398762478, -0.8115043128123168}, \
{-0.8589246864395415, -0.8306009503455781}}]}}, 
      {Arrowheads[{{0.007281202910676056, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8240290091210648, -0.6860207732029983}, \
{-0.8601815171947246, -0.7034529110075284}}]}}, 
      {Arrowheads[{{0.007528700129312999, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8228128867435277, -0.5607811372865296}, \
{-0.8613976395722618, -0.5760609679766288}}]}}, 
      {Arrowheads[{{0.007749041056843492, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8217004833559914, -0.43579809931573354`}, \
{-0.862510042959798, -0.44841242700005635`}}]}}, 
      {Arrowheads[{{0.007930042898494031, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8207661165033555, -0.3110636056976785}, \
{-0.863444409812434, -0.32051534167074297`}}]}}, 
      {Arrowheads[{{0.008059478677421765, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8200867485414304, -0.18654122924841257`}, \
{-0.864123777774359, -0.19240613917264046`}}]}}, 
      {Arrowheads[{{0.008127140917060639, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8197279406884549, -0.06216327079617147}, \
{-0.8644825856273345, -0.06415251867751313}}]}}, 
      {Arrowheads[{{0.00812714091706064, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8197279406884549, 
         0.062163270796171066`}, {-0.8644825856273345, 
         0.06415251867751276}}]}}, 
      {Arrowheads[{{0.008059478677421765, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8200867485414304, 
         0.18654122924841218`}, {-0.864123777774359, 0.1924061391726401}}]}}, 
      {Arrowheads[{{0.007930042898494034, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8207661165033554, 
         0.3110636056976781}, {-0.8634444098124339, 0.32051534167074264`}}]}}, 
      {Arrowheads[{{0.007749041056843494, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8217004833559913, 
         0.43579809931573316`}, {-0.8625100429597979, 0.448412427000056}}]}}, 
      {Arrowheads[{{0.007528700129312999, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8228128867435276, 
         0.5607811372865291}, {-0.8613976395722617, 0.5760609679766283}}]}}, 
      {Arrowheads[{{0.007281202910676083, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8240290091210648, 
         0.6860207732029978}, {-0.8601815171947247, 0.703452911007528}}]}}, 
      {Arrowheads[{{0.007017359038666378, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8252858398762478, 
         0.8115043128123164}, {-0.8589246864395415, 0.8306009503455777}}]}}, 
      {Arrowheads[{{0.006746016118932338, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8265349962223457, 
         0.9372066613836277}, {-0.8576755300934435, 0.9575301807216346}}]}}, 
      {Arrowheads[{{0.006473986083373201, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8277424189649016, 
         1.0630971689772086`}, {-0.8564681073508876, 1.0842712520754219`}}]}}, 
      {Arrowheads[{{0.006206248173771654, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8288862450476128, 
         1.1891443436265237`}, {-0.8553242812681765, 1.2108556563734754`}}]}}, 
      {Arrowheads[{{0.006609637979218297, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.670810674831191, -1.1876589043452088`}, \
{-0.6976103778003877, -1.2123410956547913`}}]}}, 
      {Arrowheads[{{0.0069362387021059384`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6694818411171742, -1.0614968851369009`}, \
{-0.6989392115144045, -1.085871535915731}}]}}, 
      {Arrowheads[{{0.007274898430320748, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6680428388179864, -0.9355096857596317}, \
{-0.7003782138135924, -0.9592271563456315}}]}}, 
      {Arrowheads[{{0.0076204419236468425`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6665117228680384, -0.8097445037067076}, \
{-0.7019093297635404, -0.8323607594511874}}]}}, 
      {Arrowheads[{{0.007964925433469158, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6649235297652557, -0.68425273266929}, \
{-0.703497522866323, -0.7052209515412365}}]}}, 
      {Arrowheads[{{0.00829689724012411, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6633361585862217, -0.5590843762946671}, \
{-0.7050848940453571, -0.5777577289684912}}]}}, 
      {Arrowheads[{{0.008600872015148477, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6618350936166834, -0.4342777003046109}, \
{-0.7065859590148953, -0.449932826011179}}]}}, 
      {Arrowheads[{{0.008857543275236931, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6605332560294991, -0.30984331173033325`}, \
{-0.7078877966020796, -0.3217356356380882}}]}}, 
      {Arrowheads[{{0.009045554088858494, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6595602626649099, -0.18574498713044998`}, \
{-0.708860789966669, -0.19320238129060305`}}]}}, 
      {Arrowheads[{{0.009145498238515728, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6590365061899229, -0.06188588630194985}, \
{-0.7093845464416558, -0.06442990317173475}}]}}, 
      {Arrowheads[{{0.009145498238515728, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6590365061899229, 
         0.06188588630194945}, {-0.7093845464416558, 0.06442990317173437}}]}}, 
      {Arrowheads[{{0.009045554088858493, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6595602626649099, 
         0.1857449871304496}, {-0.708860789966669, 0.19320238129060263`}}]}}, 
      {Arrowheads[{{0.008857543275236931, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6605332560294991, 
         0.30984331173033286`}, {-0.7078877966020796, 0.3217356356380878}}]}}, 
      {Arrowheads[{{0.008600872015148496, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6618350936166834, 
         0.4342777003046105}, {-0.7065859590148954, 0.4499328260111786}}]}}, 
      {Arrowheads[{{0.008296897240124104, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6633361585862217, 
         0.5590843762946667}, {-0.7050848940453571, 0.5777577289684906}}]}}, 
      {Arrowheads[{{0.007964925433469158, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6649235297652557, 
         0.6842527326692895}, {-0.703497522866323, 0.7052209515412361}}]}}, 
      {Arrowheads[{{0.007620441923646855, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6665117228680384, 
         0.8097445037067071}, {-0.7019093297635404, 0.832360759451187}}]}}, 
      {Arrowheads[{{0.007274898430320719, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6680428388179864, 
         0.9355096857596313}, {-0.7003782138135923, 0.959227156345631}}]}}, 
      {Arrowheads[{{0.006936238702105911, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6694818411171742, 
         1.0614968851369004`}, {-0.6989392115144045, 1.0858715359157303`}}]}}, 
      {Arrowheads[{{0.00660963797921827, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.670810674831191, 
         1.1876589043452084`}, {-0.6976103778003877, 1.2123410956547906`}}]}}, 
      {Arrowheads[{{0.007039283303892629, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5129196396952502, -1.185966198722545}, \
{-0.5397119392521178, -1.214033801277455}}]}}, 
      {Arrowheads[{{0.00743757170467205, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5113972223005709, -1.0596257498937696`}, \
{-0.5412343566467971, -1.087742671158862}}]}}, 
      {Arrowheads[{{0.007860197031827642, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.509700226248136, -0.9334674023966455}, \
{-0.5429313526992321, -0.9612694397086178}}]}}, 
      {Arrowheads[{{0.008303002732570253, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.507834464931372, -0.8075573258038666}, \
{-0.5447971140159962, -0.8345479373540283}}]}}, 
      {Arrowheads[{{0.00875811577628384, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.505826241373035, -0.6819758176195838}, \
{-0.5468053375743331, -0.7074978665909428}}]}}, 
      {Arrowheads[{{0.009212223675285664, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.503734385151495, -0.5568135423643095}, \
{-0.5488971937958732, -0.5800285628988489}}]}}, 
      {Arrowheads[{{0.009644502647382069, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5016650238156719, -0.43215973756816206`}, \
{-0.5509665551316962, -0.45205078874762783`}}]}}, 
      {Arrowheads[{{0.01002487650108978, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.49978438049334684`, -0.3080764410101723}, \
{-0.5528471984540213, -0.3235025063582492}}]}}, 
      {Arrowheads[{{0.010314658967867657`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.4983161141168159, -0.18455462187041918`}, \
{-0.5543154648305522, -0.19439274655063388`}}]}}, 
      {Arrowheads[{{0.010473334637000216`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.49749974778048905`, -0.06146341728037487}, \
{-0.5551318311668791, -0.06485237219330973}}]}}, 
      {Arrowheads[{{0.010473334637000184`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.4974997477804891, 
         0.06146341728037451}, {-0.555131831166879, 0.06485237219330932}}]}}, 
      {Arrowheads[{{0.010314658967867684`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.49831611411681587`, 
         0.18455462187041877`}, {-0.5543154648305523, 
         0.19439274655063346`}}]}}, 
      {Arrowheads[{{0.010024876501089778`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.49978438049334684`, 
         0.30807644101017195`}, {-0.5528471984540213, 0.3235025063582488}}]}}, 
      {Arrowheads[{{0.009644502647382045, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5016650238156719, 
         0.43215973756816173`}, {-0.5509665551316961, 
         0.45205078874762744`}}]}}, 
      {Arrowheads[{{0.009212223675285664, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.503734385151495, 
         0.556813542364309}, {-0.5488971937958732, 0.5800285628988484}}]}}, 
      {Arrowheads[{{0.008758115776283857, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.505826241373035, 
         0.6819758176195834}, {-0.5468053375743333, 0.7074978665909424}}]}}, 
      {Arrowheads[{{0.008303002732570253, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.507834464931372, 
         0.8075573258038662}, {-0.5447971140159962, 0.8345479373540279}}]}}, 
      {Arrowheads[{{0.00786019703182766, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.509700226248136, 
         0.933467402396645}, {-0.5429313526992322, 0.9612694397086173}}]}}, 
      {Arrowheads[{{0.007437571704672066, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5113972223005709, 
         1.0596257498937691`}, {-0.5412343566467972, 1.0877426711588616`}}]}}, 
      {Arrowheads[{{0.007039283303892629, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5129196396952503, 
         1.1859661987225445`}, {-0.5397119392521179, 1.2140338012774545`}}]}}, 
      {Arrowheads[{{0.007488184865313858, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3552938036361641, -1.184074618196049}, \
{-0.3815483016269936, -1.2159253818039508`}}]}}, 
      {Arrowheads[{{0.007971730703272832, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3535891774854786, -1.0574748273952699`}, \
{-0.38325292777767905`, -1.0898935936573617`}}]}}, 
      {Arrowheads[{{0.00849790733152536, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.35162792022984235`, -0.9310421310830761}, \
{-0.3852141850333152, -0.9636947110221872}}]}}, 
      {Arrowheads[{{0.009065937037439629, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.34939044793686513`, -0.8048606882189961}, \
{-0.3874516573262925, -0.8372445749388988}}]}}, 
      {Arrowheads[{{0.009671068739029813, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.346875430828018, -0.6790442378305463}, \
{-0.38996667443513966`, -0.7104294463799803}}]}}, 
      {Arrowheads[{{0.010301669531373096`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3441183053161762, -0.5537404109507061}, \
{-0.3927237999469814, -0.5831016943124521}}]}}, 
      {Arrowheads[{{0.01093448574250738, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3412216801546229, -0.42912753590599406`}, \
{-0.39562042510853473`, -0.45508299040979583`}}]}}, 
      {Arrowheads[{{0.011527423936015167`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.33840036385642946`, -0.30538966516233085`}, \
{-0.3984417414067282, -0.32618928220609067`}}]}}, 
      {Arrowheads[{{0.012011406894346611`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3360291853284775, -0.18263966028032397`}, \
{-0.4008129199346802, -0.19630770814072906`}}]}}, 
      {Arrowheads[{{0.01229284689478522, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3346254915482716, -0.06075859970182132}, \
{-0.40221661371488604`, -0.06555718977186328}}]}}, 
      {Arrowheads[{{0.012292846894785207`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.33462549154827165`, 
         0.060758599701820945`}, {-0.40221661371488604`, 
         0.06555718977186288}}]}}, 
      {Arrowheads[{{0.0120114068943466, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3360291853284775, 
         0.18263966028032358`}, {-0.40081291993468016`, 
         0.19630770814072868`}}]}}, 
      {Arrowheads[{{0.011527423936015167`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3384003638564294, 
         0.30538966516233046`}, {-0.39844174140672817`, 
         0.3261892822060903}}]}}, 
      {Arrowheads[{{0.01093448574250738, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3412216801546229, 
         0.4291275359059937}, {-0.39562042510853473`, 
         0.45508299040979544`}}]}}, 
      {Arrowheads[{{0.010301669531373096`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3441183053161762, 
         0.5537404109507058}, {-0.3927237999469814, 0.5831016943124517}}]}}, 
      {Arrowheads[{{0.009671068739029818, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.34687543082801797`, 
         0.6790442378305459}, {-0.38996667443513966`, 0.7104294463799798}}]}}, 
      {Arrowheads[{{0.009065937037439636, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3493904479368651, 
         0.8048606882189957}, {-0.3874516573262925, 0.8372445749388984}}]}}, 
      {Arrowheads[{{0.00849790733152536, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.35162792022984235`, 
         0.9310421310830757}, {-0.3852141850333152, 0.9636947110221867}}]}}, 
      {Arrowheads[{{0.007971730703272832, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3535891774854786, 
         1.0574748273952694`}, {-0.38325292777767905`, 
         1.0898935936573613`}}]}}, 
      {Arrowheads[{{0.007488184865313871, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.35529380363616403`, 
         1.1840746181960486`}, {-0.38154830162699366`, 
         1.2159253818039504`}}]}}, 
      {Arrowheads[{{0.00794438218689736, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.19802691970332084`, -1.182022591432196}, \
{-0.22302571187562628`, -1.2179774085678037`}}]}}, 
      {Arrowheads[{{0.008525747479929397, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.19618436145843685`, -1.0550706254611282`}, \
{-0.22486827012051028`, -1.0922977955915036`}}]}}, 
      {Arrowheads[{{0.009175137603544022, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.19399282139814852`, -0.9282342187092962}, \
{-0.2270598101807986, -0.9665026233959669}}]}}, 
      {Arrowheads[{{0.009898828209760315, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.19139162807920188`, -0.8016054340801667}, \
{-0.22966100349974525`, -0.8404998290777284}}]}}, 
      {Arrowheads[{{0.010700806242011914`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.1883243210464887, -0.6753230769056715}, \
{-0.23272831053245843`, -0.714150607304855}}]}}, 
      {Arrowheads[{{0.01157970983656983, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.1847569005958172, -0.5495925476771647}, \
{-0.23629573098312992`, -0.5872495575859937}}]}}, 
      {Arrowheads[{{0.012522566715066399`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.18071658916482486`, -0.4247104436574177}, \
{-0.24033604241412226`, -0.4595000826583722}}]}}, 
      {Arrowheads[{{0.013490870058439075`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.1763743369801651, -0.30108633042668886`}, \
{-0.24467829459878201`, -0.3304926169417326}}]}}, 
      {Arrowheads[{{0.014387468424857415`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.17222240390728416`, -0.1792158499785372}, \
{-0.24883022767166296`, -0.19973151844251583`}}]}}, 
      {Arrowheads[{{0.014991625909092891`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.16938077979026867`, -0.05937799442225772}, \
{-0.25167185178867846`, -0.06693779505142688}}]}}, 
      {Arrowheads[{{0.014991625909092891`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.16938077979026867`, 
         0.05937799442225736}, {-0.25167185178867846`, 
         0.06693779505142647}}]}}, 
      {Arrowheads[{{0.014387468424857412`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.17222240390728416`, 
         0.17921584997853682`}, {-0.24883022767166296`, 
         0.1997315184425154}}]}}, 
      {Arrowheads[{{0.013490870058439075`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.1763743369801651, 
         0.3010863304266885}, {-0.24467829459878201`, 0.3304926169417322}}]}}, 
      {Arrowheads[{{0.012522566715066403`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.18071658916482483`, 
         0.42471044365741734`}, {-0.2403360424141223, 0.4595000826583718}}]}}, 
      {Arrowheads[{{0.01157970983656983, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.1847569005958172, 
         0.5495925476771643}, {-0.23629573098312992`, 0.5872495575859933}}]}}, 
      {Arrowheads[{{0.010700806242011921`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.18832432104648866`, 
         0.6753230769056711}, {-0.23272831053245846`, 0.7141506073048546}}]}}, 
      {Arrowheads[{{0.009898828209760322, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.19139162807920185`, 
         0.8016054340801663}, {-0.22966100349974528`, 0.8404998290777279}}]}}, 
      {Arrowheads[{{0.00917513760354403, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.19399282139814847`, 
         0.9282342187092958}, {-0.22705981018079863`, 0.9665026233959665}}]}}, 
      {Arrowheads[{{0.008525747479929393, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.19618436145843685`, 
         1.0550706254611277`}, {-0.22486827012051025`, 
         1.0922977955915032`}}]}}, 
      {Arrowheads[{{0.007944382186897372, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.1980269197033208, 
         1.1820225914321956`}, {-0.22302571187562634`, 
         1.2179774085678032`}}]}}, 
      {Arrowheads[{{0.008390185008325167, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.041214183867355654`, -1.1798907759733095`}, \
{-0.06404897402738097, -1.2201092240266904`}}]}}, 
      {Arrowheads[{{0.009078248555941046, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.039323230099259476`, -1.052496297479609}, \
{-0.06593992779547714, -1.0948721235730228`}}]}}, 
      {Arrowheads[{{0.009866709049115414, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.03699863946633253, -0.9251171629886852}, \
{-0.06826451842840409, -0.9696196791165781}}]}}, 
      {Arrowheads[{{0.010773356935809516`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.03412746279815001, -0.7978307627675277}, \
{-0.0711356950965866, -0.8442745003903673}}]}}, 
      {Arrowheads[{{0.011818421782417406`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.03057267736799651, -0.6707700259550595}, \
{-0.0746904805267401, -0.7187036582554672}}]}}, 
      {Arrowheads[{{0.013024095866276258`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.02617701565992229, -0.5441582937402849}, \
{-0.07908614223481433, -0.5926838115228734}}]}}, 
      {Arrowheads[{{0.014413251091104938`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.020779305107986742`, -0.41836740041560166`}, \
{-0.08448385278674987, -0.4658431259001883}}]}}, 
      {Arrowheads[{{0.016005795324081196`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.01426105693596641, -0.2940237763747436}, \
{-0.0910021009587702, -0.3375551709936778}}]}}, 
      {Arrowheads[{{0.01780230422200126, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.006689763727994014, -0.17224694174464047`}, \
{-0.0985733941667426, -0.20670042667641259`}}]}}}, 
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.01780230422200127, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.006689763727993979, 
         0.1722469417446401}, {-0.09857339416674263, 
         0.20670042667641217`}}]}}, 
      {Arrowheads[{{0.016005795324081203`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.014261056935966396`, 
         0.2940237763747432}, {-0.09100210095877022, 0.3375551709936775}}]}}, 
      {Arrowheads[{{0.014413251091104935`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.02077930510798672, 
         0.4183674004156013}, {-0.08448385278674989, 
         0.46584312590018784`}}]}}, 
      {Arrowheads[{{0.013024095866276265`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.026177015659922254`, 
         0.5441582937402845}, {-0.07908614223481436, 0.592683811522873}}]}}, 
      {Arrowheads[{{0.011818421782417411`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.030572677367996494`, 
         0.6707700259550591}, {-0.07469048052674013, 0.7187036582554668}}]}}, 
      {Arrowheads[{{0.010773356935809513`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.03412746279815003, 
         0.7978307627675273}, {-0.07113569509658658, 0.8442745003903669}}]}}, 
      {Arrowheads[{{0.009866709049115423, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.036998639466332486`, 
         0.9251171629886847}, {-0.06826451842840413, 0.9696196791165776}}]}}, 
      {Arrowheads[{{0.009078248555941046, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.039323230099259476`, 
         1.0524962974796086`}, {-0.06593992779547714, 
         1.0948721235730223`}}]}}, 
      {Arrowheads[{{0.00839018500832516, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.04121418386735569, 
         1.179890775973309}, {-0.06404897402738093, 1.22010922402669}}]}}, 
      {Arrowheads[{{0.008802376264060243, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.11506858956264442`, -1.1778093962208378`}, {
         0.09545772622682948, -1.2221906037791623`}}]}}, 
      {Arrowheads[{{0.009599122356071514, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.1168679826781776, -1.0499088424460268`}, {
         0.09365833311129629, -1.0974595786066046`}}]}}, 
      {Arrowheads[{{0.01053363507836228, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.1191498238948175, -0.92187296297545}, {
         0.0913764918946564, -0.9728638791298134}}]}}, 
      {Arrowheads[{{0.01163920907154882, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.12207600349713729`, -0.7937323492648866}, {
         0.0884503122923366, -0.8483729138930084}}]}}, 
      {Arrowheads[{{0.012959134364095133`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.12586795598794384`, -0.6655624365084081}, {
         0.08465835980153005, -0.7239112477021185}}]}}, 
      {Arrowheads[{{0.0145510463971458, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.13082704590566652`, -0.5375202640478566}, {
         0.07969926988380739, -0.5993218412153017}}]}}, 
      {Arrowheads[{{0.016495972663418818`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.13736180988675353`, -0.4099068264886822}, {
         0.07316450590272036, -0.47430369982710774`}}]}}, 
      {Arrowheads[{{0.01892245515485331, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.14603645705447282`, -0.2832712157943033}, {
         0.06448985873500107, -0.34830773157411815`}}]}}, 
      {Arrowheads[{{0.02208847287830911, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.15772213425955608`, -0.15858260590985376`}, {
         0.0528041815299178, -0.22036476251119927`}}]}}, 
      {Arrowheads[{{0.026742429303635833`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.174309629923787, -0.037368709754197385`}, {
         0.03621668586568687, -0.08894707971948722}}]}}, 
      {Arrowheads[{{0.026742429303635865`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.17430962992378712`, 0.03736870975419701}, {
         0.03621668586568677, 0.08894707971948682}}]}}, 
      {Arrowheads[{{0.02208847287830912, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.15772213425955614`, 0.1585826059098534}, {
         0.05280418152991775, 0.22036476251119885`}}]}}, 
      {Arrowheads[{{0.018922455154853315`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.14603645705447285`, 0.2832712157943029}, {
         0.06448985873500104, 0.34830773157411776`}}]}}, 
      {Arrowheads[{{0.01649597266341881, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.13736180988675353`, 0.4099068264886818}, {
         0.07316450590272036, 0.4743036998271073}}]}}, 
      {Arrowheads[{{0.014551046397145804`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.13082704590566652`, 0.5375202640478561}, {
         0.07969926988380736, 0.5993218412153013}}]}}, 
      {Arrowheads[{{0.012959134364095152`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.12586795598794384`, 0.6655624365084076}, {
         0.08465835980153004, 0.7239112477021181}}]}}, 
      {Arrowheads[{{0.011639209071548827`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.12207600349713732`, 0.7937323492648861}, {
         0.08845031229233657, 0.848372913893008}}]}}, 
      {Arrowheads[{{0.010533635078362281`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.11914982389481751`, 0.9218729629754495}, {
         0.09137649189465638, 0.972863879129813}}]}}, 
      {Arrowheads[{{0.009599122356071516, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.11686798267817762`, 1.0499088424460263`}, {
         0.09365833311129629, 1.0974595786066041`}}]}}, 
      {Arrowheads[{{0.008802376264060241, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.1150685895626444, 1.1778093962208374`}, {
         0.09545772622682948, 1.2221906037791619`}}]}}, 
      {Arrowheads[{{0.00915381640199049, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.27079261856697123`, -1.175953866027958}, {
         0.25552317090671317`, -1.2240461339720419`}}]}}, 
      {Arrowheads[{{0.01005137873149673, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.27231970065824423`, -1.0475401952023262`}, {
         0.25399608881544017`, -1.0998282258503056`}}]}}, 
      {Arrowheads[{{0.011125171169617538`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.2743111042081475, -0.918806446943582}, {
         0.2520046852655369, -0.9759303951616812}}]}}, 
      {Arrowheads[{{0.012426287855545086`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.2769530256501976, -0.7897054240660123}, {
         0.2493627638234868, -0.8523998390918828}}]}}, 
      {Arrowheads[{{0.014024958414283724`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.2805198457644425, -0.6602008323353127}, {
         0.24579594370924193`, -0.729272851875214}}]}}, 
      {Arrowheads[{{0.016019498626009047`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.28541720375256774`, -0.5302910208787379}, {
         0.24089858572111664`, -0.6065510843844204}}]}}, 
      {Arrowheads[{{0.018551505501626698`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.2922414538211356, -0.4000523102393346}, {
         0.23407433565254882`, -0.48415821607645526`}}]}}, 
      {Arrowheads[{{0.02183527650579552, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3018549694899198, -0.2696997875101413}, {
         0.22446081998376463`, -0.3618791598582802}}]}}, 
      {Arrowheads[{{0.026219587123045885`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.31546655418217445`, -0.13961432052756337`}, {
         0.21084923529150995`, -0.23933304789348966`}}]}}, 
      {Arrowheads[{{0.03228642573359437, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3346315126780996, -0.010149144064949295`}, {
         0.1916842767955848, -0.11616664540873531`}}]}}, 
      {Arrowheads[{{0.0322864257335944, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.33463151267809965`, 0.010149144064948906`}, {
         0.19168427679558472`, 0.11616664540873492`}}]}}, 
      {Arrowheads[{{0.026219587123045906`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3154665541821745, 0.13961432052756295`}, {
         0.2108492352915099, 0.2393330478934893}}]}}, 
      {Arrowheads[{{0.021835276505795535`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.30185496948991986`, 0.2696997875101409}, {
         0.22446081998376458`, 0.3618791598582798}}]}}, 
      {Arrowheads[{{0.018551505501626715`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.29224145382113564`, 0.4000523102393342}, {
         0.2340743356525488, 0.48415821607645493`}}]}}, 
      {Arrowheads[{{0.016019498626009054`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.2854172037525678, 0.5302910208787375}, {
         0.2408985857211166, 0.60655108438442}}]}}, 
      {Arrowheads[{{0.014024958414283724`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.2805198457644425, 0.6602008323353122}, {
         0.2457959437092419, 0.7292728518752135}}]}}, 
      {Arrowheads[{{0.012426287855545112`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.27695302565019764`, 0.7897054240660117}, {
         0.24936276382348677`, 0.8523998390918823}}]}}, 
      {Arrowheads[{{0.011125171169617538`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.2743111042081475, 0.9188064469435816}, {
         0.2520046852655369, 0.9759303951616808}}]}}, 
      {Arrowheads[{{0.010051378731496727`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.27231970065824423`, 1.0475401952023258`}, {
         0.2539960888154402, 1.0998282258503052`}}]}}, 
      {Arrowheads[{{0.009153816401990493, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.27079261856697123`, 1.1759538660279576`}, {
         0.2555231709067131, 1.2240461339720414`}}]}}, 
      {Arrowheads[{{0.00941652695439679, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.426003034787061, -1.174523379173948}, {
         0.41610222837083394`, -1.225476620826052}}]}}, 
      {Arrowheads[{{0.010395370936828727`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.42706488611099847`, -1.0456711914297918`}, {
         0.41504037704689645`, -1.1016972296228396`}}]}}, 
      {Arrowheads[{{0.011584642304693391`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.42848452890062944`, -0.9163166870340761}, {
         0.4136207342572655, -0.9784201550711872}}]}}, 
      {Arrowheads[{{0.013053197241524914`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.43042763798038614`, -0.7863193722370242}, {
         0.4116776251775088, -0.8557858909208708}}]}}, 
      {Arrowheads[{{0.01489916508866094, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4331553463453619, -0.6554968983976643}, {
         0.40894991681253307`, -0.7339767858128623}}]}}, 
      {Arrowheads[{{0.017263677362387837`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.43708469670194605`, -0.523622499215867}, {
         0.40502056645594886`, -0.6132196060472913}}]}}, 
      {Arrowheads[{{0.02034927062267109, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.442880049178341, -0.39044187214976833`}, {
         0.39922521397955396`, -0.4937686541660215}}]}}, 
      {Arrowheads[{{0.024439701273845628`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.45156428391252335`, -0.2557373582797002}, {
         0.3905409792453716, -0.3758415890887213}}]}}, 
      {Arrowheads[{{0.029907240058843293`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4645709658195712, -0.11946979298671553`}, {
         0.3775342973383237, -0.25947757543433747`}}]}}, 
      {Arrowheads[{{0.03717191494905455, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.48353517985009614`, 0.018033388189613614`}, {
         0.3585700833077987, -0.14434917766329822`}}]}}, 
      {Arrowheads[{{0.03717191494905457, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.48353517985009625`, -0.018033388189614044`}, {
         0.35857008330779866`, 0.14434917766329786`}}]}}, 
      {Arrowheads[{{0.029907240058843317`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.46457096581957125`, 0.11946979298671513`}, {
         0.37753429733832367`, 0.25947757543433714`}}]}}, 
      {Arrowheads[{{0.024439701273845642`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.45156428391252335`, 0.2557373582796998}, {
         0.3905409792453715, 0.37584158908872095`}}]}}, 
      {Arrowheads[{{0.020349270622671105`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.442880049178341, 0.39044187214976794`}, {
         0.3992252139795539, 0.49376865416602117`}}]}}, 
      {Arrowheads[{{0.01726367736238784, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4370846967019461, 0.5236224992158666}, {
         0.40502056645594886`, 0.6132196060472909}}]}}, 
      {Arrowheads[{{0.014899165088660946`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4331553463453619, 0.6554968983976639}, {
         0.408949916812533, 0.7339767858128619}}]}}, 
      {Arrowheads[{{0.013053197241524917`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4304276379803862, 0.7863193722370238}, {
         0.4116776251775088, 0.8557858909208703}}]}}, 
      {Arrowheads[{{0.011584642304693391`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.42848452890062944`, 0.9163166870340756}, {
         0.4136207342572655, 0.9784201550711867}}]}}, 
      {Arrowheads[{{0.010395370936828763`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.42706488611099847`, 1.0456711914297914`}, {
         0.4150403770468965, 1.1016972296228393`}}]}}, 
      {Arrowheads[{{0.00941652695439679, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.426003034787061, 1.1745233791739476`}, {
         0.41610222837083394`, 1.2254766208260515`}}]}}, 
      {Arrowheads[{{0.009565930323572492, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5808335571359245, -1.1737026576699174`}, {
         0.5770611797061809, -1.2262973423300827`}}]}}, 
      {Arrowheads[{{0.010594915105533382`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.581269631177571, -1.0445757882962035`}, {
         0.5766251056645342, -1.1027926327564284`}}]}}, 
      {Arrowheads[{{0.01185805798421102, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5818704539192889, -0.9148171133798706}, {
         0.5760242829228164, -0.9799197287253927}}]}}, 
      {Arrowheads[{{0.013438837820147872`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.582726277079714, -0.7842067905822186}, {
         0.5751684597623912, -0.8578984725756763}}]}}, 
      {Arrowheads[{{0.015460867364983227`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5839938873907239, -0.652424642571941}, {
         0.5739008494513814, -0.7370490416385856}}]}}, 
      {Arrowheads[{{0.01811021140952181, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5859580948601761, -0.5190018489082063}, {
         0.5719366419819292, -0.6178402563549519}}]}}, 
      {Arrowheads[{{0.021667162918267813`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5891567551029365, -0.38326703298510606`}, {
         0.5687379817391689, -0.5009434933306839}}]}}, 
      {Arrowheads[{{0.02653810830123727, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5946211220438984, -0.2443462390360564}, {
         0.563273614798207, -0.38723270833236506`}}]}}, 
      {Arrowheads[{{0.03323866342155365, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6042337571765054, -0.10142270924611342`}, {
         0.5536609796656, -0.27752465917493957`}}]}}, 
      {Arrowheads[{{0.042212834944246366`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.620821937655021, 0.04538892243145749}, {
         0.5370727991870844, -0.17170471190514208`}}]}}, 
      {Arrowheads[{{0.0422128349442464, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.620821937655021, -0.04538892243145792}, {
         0.5370727991870843, 0.17170471190514175`}}]}}, 
      {Arrowheads[{{0.03323866342155368, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6042337571765054, 0.10142270924611298`}, {
         0.5536609796655999, 0.2775246591749393}}]}}, 
      {Arrowheads[{{0.026538108301237297`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5946211220438984, 0.24434623903605596`}, {
         0.5632736147982069, 0.3872327083323648}}]}}, 
      {Arrowheads[{{0.021667162918267837`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5891567551029365, 0.3832670329851056}, {
         0.5687379817391688, 0.5009434933306836}}]}}, 
      {Arrowheads[{{0.01811021140952183, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5859580948601761, 0.5190018489082059}, {
         0.5719366419819292, 0.6178402563549515}}]}}, 
      {Arrowheads[{{0.015460867364983227`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5839938873907239, 0.6524246425719405}, {
         0.5739008494513814, 0.7370490416385852}}]}}, 
      {Arrowheads[{{0.01343883782014787, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.582726277079714, 0.7842067905822182}, {
         0.5751684597623913, 0.8578984725756759}}]}}, 
      {Arrowheads[{{0.01185805798421102, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5818704539192889, 0.9148171133798702}, {
         0.5760242829228164, 0.9799197287253922}}]}}, 
      {Arrowheads[{{0.010594915105533382`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5812696311775711, 1.044575788296203}, {
         0.5766251056645343, 1.102792632756428}}]}}, 
      {Arrowheads[{{0.009565930323572492, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5808335571359244, 1.173702657669917}, {
         0.5770611797061808, 1.2262973423300823`}}]}}, 
      {Arrowheads[{{0.009585574334429407, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.735497803185295, -1.1736151830187262`}, {
         0.7381864073410209, -1.2263848169812739`}}]}}, 
      {Arrowheads[{{0.010624553818448469`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7352281373671895, -1.044446124794016}, {
         0.7384560731591263, -1.1029222962586156`}}]}}, 
      {Arrowheads[{{0.01190493806883662, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7348784135303701, -0.9146157378820281}, {
         0.7388057969959458, -0.980121104223235}}]}}, 
      {Arrowheads[{{0.013517179629487482`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7344205935369161, -0.7838763755782723}, {
         0.7392636169893998, -0.8582288875796228}}]}}, 
      {Arrowheads[{{0.015600358484109221`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7338212763616014, -0.6518465537623754}, {
         0.7398629341647144, -0.7376271304481512}}]}}, 
      {Arrowheads[{{0.018376988806976354`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7330567563620769, -0.5179134248239262}, {
         0.7406274541642391, -0.6189286804392322}}]}}, 
      {Arrowheads[{{0.02221776087653521, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7321810235566429, -0.3810479915144842}, {
         0.7415031869696731, -0.5031625348013057}}]}}, 
      {Arrowheads[{{0.027756405929004024`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7315966423023693, -0.23946939765345576`}, {
         0.7420875682239466, -0.3921095497149657}}]}}, 
      {Arrowheads[{{0.036022721449077796`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7330456119587327, -0.09026317587805516}, {
         0.7406385985675832, -0.2886841925429979}}]}}, 
      {Arrowheads[{{0.048132514236645246`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7421795017634862, 0.06939391399863501}, {
         0.7315047087628298, -0.1957097034723196}}]}}, 
      {Arrowheads[{{0.048132514236645316`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7421795017634862, -0.06939391399863556}, {
         0.7315047087628297, 0.1957097034723194}}]}}, 
      {Arrowheads[{{0.03602272144907783, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7330456119587327, 0.09026317587805469}, {
         0.7406385985675832, 0.2886841925429976}}]}}, 
      {Arrowheads[{{0.02775640592900405, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7315966423023693, 0.23946939765345532`}, {
         0.7420875682239465, 0.3921095497149654}}]}}, 
      {Arrowheads[{{0.02221776087653523, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7321810235566429, 0.38104799151448376`}, {
         0.7415031869696731, 0.5031625348013054}}]}}, 
      {Arrowheads[{{0.018376988806976375`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7330567563620769, 0.5179134248239257}, {
         0.7406274541642391, 0.6189286804392318}}]}}, 
      {Arrowheads[{{0.015600358484109221`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7338212763616014, 0.651846553762375}, {
         0.7398629341647144, 0.7376271304481508}}]}}, 
      {Arrowheads[{{0.013517179629487482`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7344205935369161, 0.7838763755782718}, {
         0.7392636169893998, 0.8582288875796223}}]}}, 
      {Arrowheads[{{0.01190493806883662, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7348784135303701, 0.9146157378820277}, {
         0.7388057969959458, 0.9801211042232345}}]}}, 
      {Arrowheads[{{0.010624553818448469`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7352281373671895, 1.0444461247940156`}, {
         0.7384560731591263, 1.1029222962586152`}}]}}, 
      {Arrowheads[{{0.009585574334429407, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.735497803185295, 1.1736151830187258`}, {
         0.7381864073410209, 1.2263848169812734`}}]}}, 
      {Arrowheads[{{0.009471236451657365, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8902524231497411, -1.1742841649842188`}, {
         0.8992212610607854, -1.225715835015781}}]}}, 
      {Arrowheads[{{0.01047659025108699, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8893092616151068, -1.0453241156345503`}, {
         0.9001644225954196, -1.1020443054180813`}}]}}, 
      {Arrowheads[{{0.011710497567857288`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8880529002990541, -0.9157924976531945}, {
         0.9014207839114724, -0.9789443444520688}}]}}, 
      {Arrowheads[{{0.013258147025265083`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8863368836939076, -0.7854901032167266}, {
         0.9031368005166189, -0.8566151599411683}}]}}, 
      {Arrowheads[{{0.015252649417393726`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8839236496166873, -0.6541131323717606}, {
         0.9055500345938392, -0.735360551838766}}]}}, 
      {Arrowheads[{{0.017915152185791726`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.880411067053701, -0.5211685302073933}, {
         0.9090626171568256, -0.615673575055765}}]}}, 
      {Arrowheads[{{0.021645608726291482`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.875083366171223, -0.3857774887590257}, {
         0.9143903180393037, -0.49843303755676416`}}]}}, 
      {Arrowheads[{{0.027261725681116195`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8665979873254185, -0.24612074343873985`}, {
         0.9228756968851081, -0.3854582039296816}}]}}, 
      {Arrowheads[{{0.036760391471791144`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8524782331108276, -0.09739117230146721}, {
         0.936995451099699, -0.2815561961195858}}]}}, 
      {Arrowheads[{{0.05575762470829352, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8329416331100434, 0.07754516900107722}, {
         0.9565320511004831, -0.20386095847476182`}}]}}, 
      {Arrowheads[{{0.055757624708293596`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8329416331100434, -0.07754516900107783}, {
         0.9565320511004831, 0.20386095847476166`}}]}}, 
      {Arrowheads[{{0.036760391471791186`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8524782331108275, 0.09739117230146672}, {
         0.936995451099699, 0.28155619611958554`}}]}}, 
      {Arrowheads[{{0.027261725681116213`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8665979873254184, 0.24612074343873946`}, {
         0.9228756968851081, 0.3854582039296813}}]}}, 
      {Arrowheads[{{0.021645608726291485`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.875083366171223, 0.3857774887590253}, {
         0.9143903180393036, 0.4984330375567638}}]}}, 
      {Arrowheads[{{0.017915152185791747`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.880411067053701, 0.5211685302073928}, {
         0.9090626171568256, 0.6156735750557647}}]}}, 
      {Arrowheads[{{0.015252649417393726`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8839236496166873, 0.6541131323717602}, {
         0.9055500345938392, 0.7353605518387656}}]}}, 
      {Arrowheads[{{0.013258147025265083`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8863368836939076, 0.7854901032167262}, {
         0.9031368005166189, 0.8566151599411679}}]}}, 
      {Arrowheads[{{0.01171049756785729, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8880529002990541, 0.915792497653194}, {
         0.9014207839114725, 0.9789443444520683}}]}}, 
      {Arrowheads[{{0.010476590251087032`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8893092616151068, 1.0453241156345499`}, {
         0.9001644225954197, 1.102044305418081}}]}}, 
      {Arrowheads[{{0.009471236451657365, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8902524231497411, 1.1742841649842184`}, {
         0.8992212610607854, 1.2257158350157806`}}]}}, 
      {Arrowheads[{{0.00923284850957926, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0453400122602163`, -1.1756201522424499`}, {
         1.0599231456345208`, -1.22437984775755}}]}}, 
      {Arrowheads[{{0.010165145654038659`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0438631064948791`, -1.047075316928247}, {
         1.061400051399858, -1.1002931041243846`}}]}}, 
      {Arrowheads[{{0.011294651277663631`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.041917150081641, -0.9181409520898199}, {
         1.0633460078130963`, -0.9765958900154433}}]}}, 
      {Arrowheads[{{0.012688663947837972`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0392918007855796`, -0.7887252565556279}, {
         1.0659713571091578`, -0.8533800066022671}}]}}, 
      {Arrowheads[{{0.014448986064514146`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0356483174051292`, -0.6587166119040722}, {
         1.069614840489608, -0.7307570723064545}}]}}, 
      {Arrowheads[{{0.01673850409958772, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0304116120835038`, -0.5279913048233081}, {
         1.074851545811233, -0.6088508004398502}}]}}, 
      {Arrowheads[{{0.0198391983454114, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0225308064523475`, -0.396456928794779}, {
         1.0827323514423894`, -0.48775359752101094`}}]}}, 
      {Arrowheads[{{0.024299476814383568`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0098604825795623`, -0.2642537095983036}, {
         1.0954026753151749`, -0.3673252377701179}}]}}, 
      {Arrowheads[{{0.0313959980966581, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9871546978582484, -0.13290122681766708`}, {
         1.1181084600364888`, -0.246046141603386}}]}}}, 
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.03139599809665814, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9871546978582483, 0.1329012268176667}, {
         1.118108460036489, 0.24604614160338556`}}]}}, 
      {Arrowheads[{{0.024299476814383596`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0098604825795623`, 0.2642537095983032}, {
         1.095402675315175, 0.3673252377701175}}]}}, 
      {Arrowheads[{{0.019839198345411414`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0225308064523475`, 0.3964569287947786}, {
         1.0827323514423897`, 0.4877535975210105}}]}}, 
      {Arrowheads[{{0.01673850409958774, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0304116120835038`, 0.5279913048233077}, {
         1.0748515458112333`, 0.6088508004398497}}]}}, 
      {Arrowheads[{{0.014448986064514164`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0356483174051292`, 0.6587166119040716}, {
         1.069614840489608, 0.730757072306454}}]}}, 
      {Arrowheads[{{0.012688663947837972`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0392918007855796`, 0.7887252565556274}, {
         1.0659713571091578`, 0.8533800066022666}}]}}, 
      {Arrowheads[{{0.011294651277663631`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.041917150081641, 0.9181409520898195}, {
         1.0633460078130963`, 0.9765958900154429}}]}}, 
      {Arrowheads[{{0.010165145654038659`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0438631064948791`, 1.0470753169282465`}, {
         1.061400051399858, 1.1002931041243842`}}]}}, 
      {Arrowheads[{{0.00923284850957926, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0453400122602163`, 1.1756201522424494`}, {
         1.0599231456345208`, 1.2243798477575496`}}]}}, 
      {Arrowheads[{{0.008892896153838184, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2009336312006167`, -1.177445239417315}, {
         1.2201190003783313`, -1.2225547605826852`}}]}}, 
      {Arrowheads[{{0.009724057314885109, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1991264192736475`, -1.0494288853321436`}, {
         1.2219262123053, -1.0979395357204877`}}]}}, 
      {Arrowheads[{{0.010709964817197227`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1968030076879979`, -0.9212344733092054}, {
         1.2242496238909497`, -0.973502368796058}}]}}, 
      {Arrowheads[{{0.011893633454494668`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.193765436363216, -0.7928813053644713}, {
         1.227287195215732, -0.8492239577934237}}]}}, 
      {Arrowheads[{{0.013333546223632073`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1897216771669041`, -0.6644441018965855}, {
         1.2313309544120437`, -0.7250295823139412}}]}}, 
      {Arrowheads[{{0.015109155934006888`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1842368129960952`, -0.5361258775544678}, {
         1.2368158185828526`, -0.6007162277086905}}]}}, 
      {Arrowheads[{{0.017323942402916338`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1766734362602962`, -0.4084340023626033}, {
         1.2443791953186518`, -0.47577652395318654`}}]}}, 
      {Arrowheads[{{0.02008465806227291, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1662058107195052`, -0.28262382159604577`}, {
         1.2548468208594425`, -0.3489551257723757}}]}}, 
      {Arrowheads[{{0.02334428484481492, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.152473191128361, -0.1617348981554929}, {
         1.2685794404505868`, -0.21721247026556012`}}]}}, 
      {Arrowheads[{{0.02619102823288703, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1393391561977777`, -0.05119251286843322}, {
         1.2817134753811703`, -0.0751232766052514}}]}}, 
      {Arrowheads[{{0.026191028232887027`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1393391561977777`, 0.05119251286843289}, {
         1.2817134753811703`, 0.07512327660525094}}]}}, 
      {Arrowheads[{{0.023344284844814947`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.152473191128361, 0.16173489815549258`}, {
         1.268579440450587, 0.2172124702655597}}]}}, 
      {Arrowheads[{{0.020084658062272946`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1662058107195052`, 0.2826238215960454}, {
         1.2548468208594428`, 0.34895512577237536`}}]}}, 
      {Arrowheads[{{0.017323942402916345`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.176673436260296, 0.4084340023626029}, {
         1.2443791953186516`, 0.4757765239531862}}]}}, 
      {Arrowheads[{{0.015109155934006888`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1842368129960952`, 0.5361258775544674}, {
         1.2368158185828526`, 0.60071622770869}}]}}, 
      {Arrowheads[{{0.01333354622363208, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1897216771669041`, 0.6644441018965851}, {
         1.2313309544120439`, 0.7250295823139407}}]}}, 
      {Arrowheads[{{0.011893633454494668`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.193765436363216, 0.7928813053644709}, {1.227287195215732,
          0.8492239577934233}}]}}, 
      {Arrowheads[{{0.010709964817197227`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1968030076879979`, 0.9212344733092049}, {
         1.2242496238909497`, 0.9735023687960576}}]}}, 
      {Arrowheads[{{0.009724057314885143, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1991264192736475`, 1.0494288853321432`}, {
         1.2219262123053, 1.0979395357204875`}}]}}, 
      {Arrowheads[{{0.008892896153838184, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2009336312006165`, 1.1774452394173145`}, {
         1.220119000378331, 1.2225547605826848`}}]}}, 
      {Arrowheads[{{0.008481405753401998, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3571075389945173`, -1.179544362697465}, {
         1.3797345662686409`, -1.2204556373025346`}}]}}, 
      {Arrowheads[{{0.009198216701380624, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3551764602354601`, -1.0520676424614475`}, {
         1.3816656450276983`, -1.0953007785911841`}}]}}, 
      {Arrowheads[{{0.010025926888739106`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3527744083712951`, -0.9245923580943564}, {
         1.3840676968918635`, -0.9701444840109068}}]}}, 
      {Arrowheads[{{0.01098546636286434, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3497647175182848`, -0.7972060982009198}, {
         1.3870773877448739`, -0.844899164956975}}]}}, 
      {Arrowheads[{{0.01209905854115949, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3459778302002265`, -0.6700732659969468}, {
         1.3908642750629316`, -0.7194004182135797}}]}}, 
      {Arrowheads[{{0.013384820889978446`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3412257655667403`, -0.5434950401167346}, {
         1.395616339696418, -0.5933470651464238}}]}}, 
      {Arrowheads[{{0.014841348159138842`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3353689959955803`, -0.4180065351579811}, {
         1.401473109267578, -0.46620399115780886`}}]}}, 
      {Arrowheads[{{0.01640847670888384, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3285269331811278`, -0.2944903554292496}, {
         1.4083151720820306`, -0.3370885919391718}}]}}, 
      {Arrowheads[{{0.01788615821487304, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3215801726737058`, -0.1741089316456346}, {
         1.4152619325894529`, -0.2048384367754184}}]}}, 
      {Arrowheads[{{0.018854946538373926`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3167719975869077`, -0.057421913530695284`}, {
         1.420070107676251, -0.06889387594298932}}]}}, 
      {Arrowheads[{{0.018854946538373923`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3167719975869077`, 0.05742191353069493}, {
         1.420070107676251, 0.0688938759429889}}]}}, 
      {Arrowheads[{{0.017886158214873, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3215801726737058`, 0.17410893164563424`}, {
         1.4152619325894527`, 0.20483843677541802`}}]}}, 
      {Arrowheads[{{0.016408476708883867`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3285269331811278`, 0.2944903554292493}, {
         1.4083151720820308`, 0.3370885919391714}}]}}, 
      {Arrowheads[{{0.014841348159138838`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3353689959955803`, 0.4180065351579807}, {
         1.401473109267578, 0.4662039911578084}}]}}, 
      {Arrowheads[{{0.013384820889978475`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3412257655667401`, 0.5434950401167341}, {
         1.395616339696418, 0.5933470651464233}}]}}, 
      {Arrowheads[{{0.012099058541159505`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3459778302002265`, 0.6700732659969464}, {
         1.3908642750629316`, 0.7194004182135794}}]}}, 
      {Arrowheads[{{0.010985466362864354`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3497647175182848`, 0.7972060982009194}, {
         1.3870773877448739`, 0.8448991649569747}}]}}, 
      {Arrowheads[{{0.010025926888739125`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3527744083712951`, 0.9245923580943559}, {
         1.3840676968918635`, 0.9701444840109065}}]}}, 
      {Arrowheads[{{0.009198216701380624, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3551764602354601`, 1.052067642461447}, {
         1.3816656450276983`, 1.0953007785911837`}}]}}, 
      {Arrowheads[{{0.008481405753401998, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3571075389945173`, 1.1795443626974647`}, {
         1.3797345662686409`, 1.2204556373025341`}}]}}, 
      {Arrowheads[{{0.008029516420398373, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.513843848607481, -1.1817187770348463`}, {
         1.538787730339888, -1.2182812229651534`}}]}}, 
      {Arrowheads[{{0.008632388500394769, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5119518965027254`, -1.0547175276667926`}, {
         1.5406796824446438`, -1.0926508933858392`}}]}}, 
      {Arrowheads[{{0.009308361368073429, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5096834885077843`, -0.9278352196103847}, {
         1.5429480904395847`, -0.9669016224948787}}]}}, 
      {Arrowheads[{{0.010063325248143124`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5069695860034862`, -0.8011780746271657}, {
         1.5456619929438826`, -0.8409271885307292}}]}}, 
      {Arrowheads[{{0.010898370452419875`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5037505175204904`, -0.6749114506426608}, {
         1.5488810614268789`, -0.7145622335678659}}]}}, 
      {Arrowheads[{{0.011803683976666373`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.50000826591871, -0.5492827902993247}, {
         1.552623313028659, -0.5875593149638337}}]}}, 
      {Arrowheads[{{0.012747611594505374`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.495833092075998, -0.42463517840133336`}, {
         1.556798486871371, -0.45957534791445653`}}]}}, 
      {Arrowheads[{{0.013660250109202, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4915367494227187`, -0.30137099902014036`}, {
         1.5610948295246503`, -0.33020794834828104`}}]}}, 
      {Arrowheads[{{0.014418887286838059`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4877748434075775`, -0.1797843412845667}, {
         1.5648567355397918`, -0.19916302713648631`}}]}}, 
      {Arrowheads[{{0.014861811736278863`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4854996425432467`, -0.05971601672576189}, {
         1.5671319364041223`, -0.06659977274792271}}]}}, 
      {Arrowheads[{{0.01486181173627886, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4854996425432467`, 0.05971601672576152}, {
         1.5671319364041223`, 0.06659977274792231}}]}}, 
      {Arrowheads[{{0.014418887286838059`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4877748434075775`, 0.17978434128456633`}, {
         1.5648567355397918`, 0.19916302713648593`}}]}}, 
      {Arrowheads[{{0.013660250109202, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4915367494227187`, 0.30137099902014003`}, {
         1.5610948295246503`, 0.3302079483482807}}]}}, 
      {Arrowheads[{{0.012747611594505374`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.495833092075998, 0.42463517840133297`}, {
         1.556798486871371, 0.45957534791445614`}}]}}, 
      {Arrowheads[{{0.011803683976666373`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.50000826591871, 0.5492827902993243}, {1.552623313028659, 
         0.5875593149638333}}]}}, 
      {Arrowheads[{{0.010898370452419875`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5037505175204904`, 0.6749114506426603}, {
         1.5488810614268789`, 0.7145622335678654}}]}}, 
      {Arrowheads[{{0.010063325248143152`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5069695860034862`, 0.8011780746271653}, {
         1.5456619929438828`, 0.8409271885307288}}]}}, 
      {Arrowheads[{{0.009308361368073429, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5096834885077843`, 0.9278352196103843}, {
         1.5429480904395847`, 0.9669016224948782}}]}}, 
      {Arrowheads[{{0.008632388500394769, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5119518965027254`, 1.0547175276667922`}, {
         1.5406796824446438`, 1.0926508933858388`}}]}}, 
      {Arrowheads[{{0.008029516420398373, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.513843848607481, 1.181718777034846}, {1.538787730339888, 
         1.2182812229651532`}}]}}, 
      {Arrowheads[{{0.0075642286772638285`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6710639036038477`, -1.183819649944015}, {
         1.6973571490277317`, -1.2161803500559847`}}]}}, 
      {Arrowheads[{{0.008062872078903572, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6693136457862887`, -1.0571944314192165`}, {
         1.6991074068452907`, -1.0901739896334153`}}]}}, 
      {Arrowheads[{{0.008605961716456785, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6672907034224491`, -0.9307456781553556}, {
         1.7011303492091303`, -0.9639911639499076}}]}}, 
      {Arrowheads[{{0.009191389106786005, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6649755859127247`, -0.8045675306434816}, {
         1.7034454667188552`, -0.8375377325144133}}]}}, 
      {Arrowheads[{{0.009811438055205065, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.662373526644597, -0.6787872076302136}, {
         1.7060475259869827`, -0.710686476580313}}]}}, 
      {Arrowheads[{{0.010449121743499962`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6595387831375144`, -0.5535653330143853}, {
         1.708882269494065, -0.5832767722487732}}]}}, 
      {Arrowheads[{{0.011073551308869042`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6566092470459104`, -0.4290804655244806}, {
         1.711811805585669, -0.4551300607913093}}]}}, 
      {Arrowheads[{{0.011636063610459723`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6538416957893314`, -0.3054818258109481}, {
         1.7145793568422485`, -0.3260971215574733}}]}}, 
      {Arrowheads[{{0.012071669552375616`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.651615781596175, -0.18279955162811645`}, {
         1.7168052710354045`, -0.19614781679293658`}}]}}, 
      {Arrowheads[{{0.012312273747450551`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6503556049618882`, -0.06084016705536821}, {
         1.7180654476696917`, -0.06547562241831639}}]}}, 
      {Arrowheads[{{0.01231227374745055, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6503556049618882`, 0.060840167055367836`}, {
         1.7180654476696917`, 0.06547562241831599}}]}}, 
      {Arrowheads[{{0.012071669552375616`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.651615781596175, 0.18279955162811606`}, {
         1.7168052710354045`, 0.1961478167929362}}]}}, 
      {Arrowheads[{{0.011636063610459723`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6538416957893312`, 0.3054818258109478}, {
         1.7145793568422483`, 0.32609712155747295`}}]}}, 
      {Arrowheads[{{0.011073551308869048`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6566092470459104`, 0.4290804655244802}, {
         1.711811805585669, 0.45513006079130897`}}]}}, 
      {Arrowheads[{{0.010449121743499996`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6595387831375144`, 0.5535653330143848}, {
         1.7088822694940653`, 0.5832767722487727}}]}}, 
      {Arrowheads[{{0.009811438055205096, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.662373526644597, 0.6787872076302132}, {1.706047525986983,
          0.7106864765803126}}]}}, 
      {Arrowheads[{{0.009191389106786005, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6649755859127247`, 0.8045675306434812}, {
         1.7034454667188552`, 0.8375377325144129}}]}}, 
      {Arrowheads[{{0.008605961716456785, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6672907034224491`, 0.9307456781553551}, {
         1.7011303492091303`, 0.9639911639499071}}]}}, 
      {Arrowheads[{{0.0080628720789036, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6693136457862887`, 1.057194431419216}, {
         1.699107406845291, 1.0901739896334148`}}]}}, 
      {Arrowheads[{{0.0075642286772638285`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6710639036038477`, 1.1838196499440146`}, {
         1.6973571490277317`, 1.2161803500559842`}}]}}, 
      {Arrowheads[{{0.007105765729122507, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.828664230594129, -1.1857561761064597`}, {
         1.8555462957216613`, -1.21424382389354}}]}}, 
      {Arrowheads[{{0.007514390773930752, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8271044475834914`, -1.0594046977115559`}, {
         1.8571060787322986`, -1.0879637233410757`}}]}}, 
      {Arrowheads[{{0.007947644178409844, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.82536207226951, -0.9332446302766508}, {
         1.8588484540462804`, -0.9614922118286123}}]}}, 
      {Arrowheads[{{0.008400220273752677, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8234455493755037`, -0.8073474202435148}, {
         1.8607649769402865`, -0.8347578429143803}}]}}, 
      {Arrowheads[{{0.008862390520394026, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8213876957625448`, -0.6817983401474195}, {
         1.8628228305532455`, -0.7076753440631072}}]}}, 
      {Arrowheads[{{0.009318372512754209, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8192586499889485`, -0.5566895494801384}, {
         1.864951876326842, -0.5801525557830199}}]}}, 
      {Arrowheads[{{0.009745004988461748, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8171786638291034`, -0.4321032100190081}, {
         1.867031862486687, -0.45210731629678175`}}]}}, 
      {Arrowheads[{{0.010111826059493307`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8153228575967693`, -0.30808172782387566`}, {
         1.8688876687190208`, -0.3234972195445458}}]}}, 
      {Arrowheads[{{0.01038413525194003, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8139051714556025`, -0.18458924301843047`}, {
         1.8703053548601878`, -0.19435812540262257`}}]}}, 
      {Arrowheads[{{0.010530054264729312`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.813131528003706, -0.061482305285287545`}, {
         1.8710789983120841`, -0.06483348418839706}}]}}, 
      {Arrowheads[{{0.010530054264729312`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.813131528003706, 0.06148230528528717}, {
         1.8710789983120841`, 0.06483348418839666}}]}}, 
      {Arrowheads[{{0.01038413525194003, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8139051714556025`, 0.18458924301843008`}, {
         1.8703053548601878`, 0.19435812540262215`}}]}}, 
      {Arrowheads[{{0.010111826059493307`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8153228575967693`, 0.3080817278238753}, {
         1.8688876687190208`, 0.3234972195445454}}]}}, 
      {Arrowheads[{{0.009745004988461748, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8171786638291034`, 0.4321032100190077}, {
         1.867031862486687, 0.45210731629678136`}}]}}, 
      {Arrowheads[{{0.009318372512754199, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8192586499889485`, 0.5566895494801379}, {
         1.864951876326842, 0.5801525557830194}}]}}, 
      {Arrowheads[{{0.008862390520394026, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8213876957625448`, 0.681798340147419}, {
         1.8628228305532455`, 0.7076753440631067}}]}}, 
      {Arrowheads[{{0.008400220273752677, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8234455493755037`, 0.8073474202435144}, {
         1.8607649769402865`, 0.8347578429143798}}]}}, 
      {Arrowheads[{{0.007947644178409844, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.82536207226951, 0.9332446302766504}, {
         1.8588484540462804`, 0.9614922118286119}}]}}, 
      {Arrowheads[{{0.007514390773930752, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8271044475834917`, 1.0594046977115554`}, {
         1.8571060787322988`, 1.0879637233410753`}}]}}, 
      {Arrowheads[{{0.007105765729122507, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.828664230594129, 1.1857561761064592`}, {
         1.8555462957216613`, 1.2142438238935396`}}]}}, 
      {Arrowheads[{{0.006667360476695942, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9865430463498734`, -1.1874863807594942`}, {
         2.0134569536501274`, -1.2125136192405057`}}]}}, 
      {Arrowheads[{{0.007001073435265354, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9851851074397413`, -1.0613210336899779`}, {
         2.0148148925602594`, -1.0860473873626537`}}]}}, 
      {Arrowheads[{{0.007346589267291139, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.983713472801786, -0.9353378217644712}, {
         2.016286527198215, -0.9593990203407922}}]}}, 
      {Arrowheads[{{0.007697986317420504, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9821487449642603`, -0.8095862750818046}, {
         2.0178512550357404`, -0.8325189880760905}}]}}, 
      {Arrowheads[{{0.008046355378701676, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.980530245491361, -0.684119055772271}, {
         2.01946975450864, -0.7053546284382557}}]}}, 
      {Arrowheads[{{0.008379259286396039, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9789214574863652`, -0.5589848945373914}, {
         2.021078542513636, -0.5778572107257669}}]}}, 
      {Arrowheads[{{0.008680656128542437, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9774128908179813`, -0.43421686427130585`}, {
         2.0225871091820196`, -0.44999366204448404`}}]}}, 
      {Arrowheads[{{0.008931755667815924, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9761185280179987`, -0.30981672731594}, {
         2.023881471982002, -0.3217622200524815}}]}}, 
      {Arrowheads[{{0.009113226996403953, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9751619375292397`, -0.18573951621086984`}, {
         2.0248380624707614`, -0.19320785221018316`}}]}}, 
      {Arrowheads[{{0.00920871027400897, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9746515163182177`, -0.061886335803872615`}, {
         2.025348483681783, -0.06442945366981198}}]}}, 
      {Arrowheads[{{0.00920871027400897, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9746515163182177`, 0.06188633580387223}, {
         2.025348483681783, 0.06442945366981159}}]}}, 
      {Arrowheads[{{0.009113226996403953, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9751619375292397`, 0.18573951621086948`}, {
         2.0248380624707614`, 0.19320785221018277`}}]}}, 
      {Arrowheads[{{0.008931755667815924, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9761185280179987`, 0.3098167273159396}, {
         2.023881471982002, 0.3217622200524811}}]}}, 
      {Arrowheads[{{0.008680656128542437, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9774128908179813`, 0.43421686427130546`}, {
         2.0225871091820196`, 0.44999366204448366`}}]}}, 
      {Arrowheads[{{0.008379259286396039, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9789214574863652`, 0.558984894537391}, {
         2.021078542513636, 0.5778572107257665}}]}}, 
      {Arrowheads[{{0.008046355378701676, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.980530245491361, 0.6841190557722705}, {2.01946975450864, 
         0.7053546284382552}}]}}, 
      {Arrowheads[{{0.007697986317420504, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9821487449642603`, 0.8095862750818041}, {
         2.0178512550357404`, 0.83251898807609}}]}}, 
      {Arrowheads[{{0.007346589267291139, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.983713472801786, 0.9353378217644708}, {2.016286527198215,
          0.9593990203407917}}]}}, 
      {Arrowheads[{{0.007001073435265354, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9851851074397413`, 1.0613210336899774`}, {
         2.0148148925602594`, 1.0860473873626533`}}]}}, 
      {Arrowheads[{{0.006667360476695942, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9865430463498734`, 1.1874863807594938`}, {
         2.0134569536501274`, 1.2125136192405053`}}]}}}}}, 
   GraphicsComplexBox[CompressedData["
1:eJx0nXdck9cX/+OoW+vee9XxddWtrQerto4OtVZt3XvXrXXHVXfrqnvgnq3g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    "], {{{}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], 
         LineBox[{3160, 3522, 3811, 3810, 4032, 3986, 3987, 4263, 4262, 3346, 
          4031, 3507, 4455, 4457, 4456, 4133, 4132, 3337, 4252, 3145, 3257, 
          3777, 3499, 4647, 3144, 3808, 4447, 4243, 4244, 4241, 3329, 4242, 
          3485, 4713, 3486, 4127, 4126, 4236, 4235, 3130, 3251, 3479, 4442, 
          3480, 4443, 3328, 3472, 4634, 3128, 3770, 3769, 3471, 4633, 3127, 
          3802, 3464, 4710, 3465, 4123, 4122, 4229, 4228, 4435}], 
         LineBox[{3245, 4700, 3629, 3793, 3794, 4121, 4556, 3630, 3967, 4212, 
          4072, 3637, 4071, 3636, 4560, 3293, 3953, 3952}], 
         LineBox[{3246, 3292, 4202, 4201, 3432, 3625, 4731, 3624, 3922, 3737, 
          4743, 3738, 3618, 3919, 3240, 4695, 3732, 3617, 3731, 4694, 3239, 
          3917, 3234, 3288, 4199, 4198, 3429, 3605, 4727, 3604, 4383, 3425, 
          4380, 4382, 4381, 4543, 3910, 4624, 4625, 4623, 3715, 3907, 3906, 
          3216, 4189, 4188, 4187, 3417, 4530, 4531, 4529, 3583, 4008, 3409, 
          4361, 4362, 4062, 4061, 4518, 3897, 3898, 3574, 4724, 3573}], 
         LineBox[{3477, 4758, 3478, 3470, 3663, 3766, 3930, 3642, 3926, 3964, 
          3963, 3469, 4441, 4096, 4097, 3845, 3929, 3662}]},
        "0.162`"],
       Annotation[#, 0.162, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1VlQllUcx/FjWtIoslhaipa5lS0KJC6IM6ItFhihBOikhSVkBeVMpC1T
F0EIlqIpKrY4lQnaVGYXrWqlNZlZFzZTVm5pXWRhTUkZ0OdMF1+/z/93fuc8
z/vIC0PKqgoru4UQNvvnyeQQ0lCUEsItaEgMYRn+7G0tKYS/uI7PoB6nzUu5
nRv4XzyFv83L+B9+mjuwIu4zL489buQudKLEvYrRIR/k/idc/4S8PiH8zGfp
dEe++YLUELL0epgnxPvr16MOXeabdZp0CvgcnUJ+zDyTE8w9MUkvTTZLtsHn
K+JB5hz5udan8BBzsfxF64nxGV0/Livll2R/eK45rtv4d5xCkt58vgPNnqen
3npO4Lm6l9mf5+wUvVSUyU7r3s7t3E92P79pz3Zsw4X2vsU7ucr+XTwUV8qu
wmiUOXMPf4JxGI/dsnIMc+Zw3Oce3e2v4h58hazRvQ7pHsYPOIojWCU/yb/i
F7ThN6yWt9n/sHMnOuOk59hr3mt+BF06QecmLsBXshokyAp0i2Ul6HT9gP0d
nGqthFP4PPTF22h37hp7b9Tr43ofvjY3IU+WZJ5n3wDd23gg78FunLG2UW+W
3gDX+5GGxeaB3IkF9lzES2SD+Rv9TSgyX2wutz6ch+Fb+Zc8Eg9ZH8Fd2Cqf
E99lfAfuW+GzZXA6PsNB61swW+dynW7eecAbsvHWq9xjHFfyF7wf+5Cpe0Lv
OJbgXutbOEP+Ix/DYtwjb+V0+VE+ggdxt7yFx8gP8yFUY6F8tOxWz/O6Z8hy
r7tkmTwWV+M1+UzdVi7V26A/Cs0oMbfE75j1oea1qJU1cYFsdeyZ18TvE1ai
xtzIM6wv5/XmFdwPDXjCXM/TrdfFM81LOQXXyda53wxZjTk59pFvXiufFn+G
+QDmefdzMdvnOd/nqJfl6gX9QlkvWSJ64xVkx98D8f1az9Gr1c80P8qfY7qz
rsc0ezu5WnYtXxMzTJKf4kXyqZyLKciSH+NK+WTOiV2MkR/gCnk2T8QEjJR/
yvPlGZweu/gQu1Aq34GP9Ko998d8PP4cYJG5RacVm9HX2naM8nny7bk0no8R
eA7P4gb5O9iqd6f9r/ICPps3xd8t9k61vtP5O/Cy7BJZrux78wvmheaN/IH5
eR5snmz9u/j/a64wN/P75nXxu2nOtn7QvMpcbn6G3zW/h0bXK+N71qnn/tbH
ut4Wv8OerTb5/79N/wEFCeNQ
          "]]},
        "0.108`"],
       Annotation[#, 0.108, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1Hloz3Ecx/GvazIzc4U/ZJfZjI1sKZojhkyusdUcseEfQyJ/UIpEckyT
Y+4zhlxhK9dcueU+cswfNrnL/IMpj2/+eO75fb0/r8/n+2m//RZXOHfcnAZB
EKT5sS4mCGIxuVUQnIkKgilcIq9HHD7KZ80/cWTLIGiOvGh7WgfBQPMoeShv
1t2CTWjUIgjKeJh5kl653BgJZp+dU2n+haPtLXDWV8/f8Q2H9SabLbVvEsfo
nDQrlk9xK6Q4pxsuOGcc2oV34jSdInsu6lxCupxvXqUbL6dyd1wxm4ZO9s3k
TL1i+xLkeKxxj0d6j/EQ6fY+4Vf8mjP4FWfyG67mT9yXP3I//szv+Q8P5t/8
F/XY4/w67/sZYu1AmPmBuyxHhNwU43UnIA+rzAe5Z71ePiqtR9hfwU34ifUS
dJA7osie6WhhbaF9UfwL23XGym0838dV3WuowhVcxm/z3XpdPCchEdO9s6N5
B9ySb2KG8xM4Hjfw3J5dyHV+e72/qMcBs3TrPdEdaeiB/ebdOBVdkYJk7DMf
6XPYGX6+zlrnjBx5RPjZeC4zW8tF7hSrX8hxvEN/uM42HqOzUqc1VmC0vNU8
2/qGsCcvM28a/q1hmFxqPsD6ah4sLzEf7+xoZ+dySy7HIWTpReos0mnGiznT
LANLwvv73eRgBLLtrcV8vRr+adaff3AW1/E7/sB9+D3Xogal3vWCX+IZell/
zk/5DqfwLU7m23yX53nPPc6QeyPR81WeGH4XcM1dr5v11CuQa+X53rMAR/Xa
WTvCbbki/N7ojdI7jzPybPksR+IEkuUca+dwTJ4lH+cIHESiPMRaBaZ6xzRs
dH5Da/FcGP5OPXfRy9Z56z7V2GvW2WyA2WlM0lsb8/9/1j+JfqIq
          "]]},
        "0.054`"],
       Annotation[#, 0.054, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl009Mz3Ecx/FPxYb+/1kZLr+rySGTbCYZbWzkXzaEdLEVxcGw1bI5SMmB
sHGQf/Nv/dsw+mf+X6qfyUX+bOagzUlTyZ/N4zeH556f9+v9Z99aRSpqNlXH
hRC+4VFyCPvTQ6hBNT6nhLAlKYT2tBA60IZsM72ozgihh/v1NyNX3aeegyN2
j+IwXqqf6Jfhuf0XWIx81OlH9Wt5iAdl83mAoxhCs94t39Hg/m2+ibs4qb7D
79CORnUHt6ELTepObrE/4c4kxlHo/k/+yuf1RvkCF/M6+XoM+tZ65KeGcIKX
u1Xr1qSZOi5UN8Ry/VO8Sh0nL9XfZj9R/UaexCvM1Oslex/jFI7nInmLmTXq
GeoddsvtdvuWHsyUZXOZfI+817sPs+Q5vFNeIe/3TpStdeece5Peu/QSeLWs
WTaMe+buIw1b9bfbfeA9zVyxudNmJrw7YxmmIwElZsewzMxxM1EMmPsuy3Aj
E1kYk733M31AK1bqT8mLuMXuW9kX70u8lEf1CvgTCvQPuruEP6oXYUQ/j8/K
hu2MeF/khfxaL5fP6L2SNfnORkS842WVviWOq3iebC7K3c+T7ZVF7Pa58Tj2
+8MhuzlmfvFvTOEv/mC22U4zOdzFpe5kmd3IC9wrd69bL1O22/tH7O8N47Hf
tfy6nVS+wRvsXOMU9RVO5qtcIm/lJPXl2N8PV9nfh0o8VD9z+ykOpP//P/0H
eYaESA==
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl00tIlFEYxvGjZW0aLcFKuli7IEhCyUAod2JWkK0zUCd0kZbghS5WI9Su
dboxqlVlQkF235VmECJSmy5OF8pJR7tBF6h+hxYP//d53vec853vm1nf0FbX
mhNCeESHCkMYS4TwMJ+nEfWRZSGM4lFchTeXhrAah3HfkhBu4xr+Fq7FO1gv
v4cl/F1ch3+tT9rzDzbhyoIQnsgPqDc7d781xbIklvPN8kn9oG5R59pnAS2M
55groVb5LB7ELG6gjZThP9GN2KcZ2iSfdPY29bR6O2aww35D2OmswejxGj6g
+3SaT3qGM/jNs/Wa/4rv7VVrjy/qd+qUvIKfUFerC2jAunxM0Iz15/V24EW8
RBdojn9qjxRVmp/nd9P3+I6sW0G/+Sq9IfN7+V18rvpXfFYcpDr5Tj5H/ROv
4hXaI6/lL6t/YBG/nLa6W5bvdk6vvbuwIt4fU3xnfIf6i9SLKY/aZaVm+uw1
rXcOM9iPaWw3c9jMMNWbq8RjsoH4LSntjun4DtRT+Jbe0Gd5i/l5bDR/Xb8r
3gEb+Ky8WX8OX5l/TS+oRv8lTsl79I/Tc+vOyvLccUz/MT2TbYm/I3Mf+RP6
J6knfjc6RSPycXNl5prMfYhrE///E/8A8t95GQ==
          "]]},
        RowBox[{"-", "0.054`"}]],
       Annotation[#, -0.054, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl0rtL1WEYB/CnJSLKNKrBqYaCLoM1JURaYJeli3+AQUOBl3NSk0AhldJq
sqDLUBrmbdUUKnDRTMyh0oMedUgIB62h7N7m50fDh+/7Pu/7POc9R3ddTJem
1kXEINObI27kRUzJm3IlJ6JiU8T63IhHWyIOqD+U++VbOckEd91tY9zd8oTa
Eb0pfWmuMOt8hixFFHOU4xxj+9aIbVygR+91PY18Nu+SeSvysvwim9Wb6HVv
h/t/1f8xyD37Idkp++VzBvikr1pPDT/t53xmif55eUJ+V/tBJRvdqXX/m/U1
WWjWV+sqZpmh2G9QJ7PJLBaYp4Ba8zbouSoP2R+kyZxcc59Zn9TbKU/JLpmn
3iNP23fLPnpp1rNqxhNvbOcxT+lI/ka0uNNKijT17g/Ls87OcYZSzlNtdr2z
F7ykwf6VXNRT5q0dPqedP2rd8pfskr/lkjtZ6zmGyPBRLWNuoXWV91dSwWH7
fXr2spP3fOAd00yxrPe23jvJd+AWreQ7G2fMeZs3vUl+S/Z46wOZkWPyNTXs
tr8vRxlJ6vomcv7/H68BeFZyIg==
          "]]},
        RowBox[{"-", "0.108`"}]],
       Annotation[#, -0.108, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl0jlPVXEQxuGxAgKyXD4BjVa2JigEZMelMRBJlLBaKUvY5AJXWTskNBhJ
ANHaFmP0A7igpUbUCgIUbKGl4jmh+OWd5Z2Zf05OSWff/d5LEfEKywURQzjA
Yiria17EofgKrmLrckRWYUQ2evRyaB+9WRRRhhsY4LlL7+E2MvI7dIdvkH/J
3kG1crUh+t7eEfVnaBI343p+RIW8Et/M9aOBvxH1mDA3h267ZmkNfzWmxR1q
R/wz4kPaLp8STyY+s2O0mtahFv3yUvoPf9Fgz3dzw8hzPxfbPCfqx0irr3nj
G6xgFfNuLOAlRvVPk+9IP/K3i9tQLs7or9Pf6Farpy10Fnv8j+k+zbg5gXf6
xWZ+0Q96ae+Ypp/wGS/kVXrPeSrprSRWm9KbxAZG5TN03L4xrPK+5Uslb5Wn
sevmQ55HaMU1/Df3gHahA53JPnNn9I/eD7pmx0/6JPkW4qf2fMGm/DXWUxf/
1DkRL1VM
          "]]},
        RowBox[{"-", "0.162`"}]],
       Annotation[#, -0.162, "Tooltip"]& ], {}, {}}}], 
   GraphicsComplexBox[CompressedData["
1:eJx0nXdcVMf399deYvdrjyXGxGjssbdBo8beYywxsXcsqFjRFQFrbIm9EbtR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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV01toznEcx/HvHCKGjQ3bGCNncYFGXEgjLsScSaO5kBwvpDnWsiWT0yyH
xIw5bOYwJUkOSchI4mIkIYckh5Bj4vVcfJ73//18f5/v/9l/z5NTuHzKsqSI
yPZS0DGiJD0iMzPiTFpEBp7FLCz0/gI5xU9LRbuIGjxg9j05onebiDTXQztH
DJOtnSLSea0zB/GHM8X6R/jfLhHVOJ/30qtyfVi227kX9zr/0fkcsxTXfe3r
JyV2pvJ9zuzDT85ssGMH/2XnTpzHu+ttS+yTEjvLsNz5185nm7Vy3Vo62Vls
vtbeZL5LKmQyz5c3zq+zL7yXJP8yIppiE+kpVZ7XYSmTLfLS+Vqcbu9Re6fh
MZyBr9x3Np7gM/E4zsI6rNepwbNYi+ewzv2XuPcsfggbOtiDJTobZaeskpNy
UDa19fc5c9tzaMSb7ncZL/E7ODSxH0/zI1jnsy7HCr4OV+vvxiq+GS/qT8dN
vBD76I/HNXwkVuq3wPk8DZfoD8KlPCexX/9D+4ip/Ddm6j/HcfwhlutX4kBe
h/P0b+IIfhEr9UuxGy/HNvpF2J4vxlL9XPyZFZGHU/QLsLl5fmK/fld8b94f
Qz8Fn/GmWKR/PzXiBn+KY/W/4wP+Dkv1z+MFfh2/mp/AWr4fF+svxB18JQ43
34YHeDEW6U/EjXwOvjEfjav4ECzQ/5cSMZcn8wHmvXERz0js139tPol/wcfm
jTiGN2C+/h7sy6sx2/wKDuP1WKC/HrN4Gd41X4Ft+QLM0x+M33yPR2Gq+QxM
Mp+Q2K+fjm/Ne+BV85b4hP/xLHP1b+E1/gibmX/Ee/wF5un397v77P+Riz/4
Fb+fBtkj9VItjfIf8WGk/w==
          "]]},
        "0.42600000000000005`"],
       Annotation[#, 0.42600000000000005`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV02tsTwccxvEfw15QtHXXarswQZDYUnZBpO4WsYYXspSQkOmMFyK1IKSp
IojSaKIoUbetjGxSjcvcyVAZXpTJEMNEMJdgSNjnvHj+3/P8nvP8zv+c8//n
TJ2dP6tRRAzysbpdRBn1Tok42iYip1PEMVzaNqKUDjqOzIhDuKF9RCWN7hAx
iprq9KJaWZbeAczG9a0ifnW8zXEj+XR7ptEvZtVmYbaEz3CcSQPtqtD5Etfh
budtMX/dIqK7c6c4t8Zss9l/ZsX8dv5dRsQOXO47LaMy3b52PHHOR3qVsjSd
DbhYZxGtdfxGrzyZ6SyibJ0s+kcvS2+FLEVvJbbEYnsXJs+Db8IvxabYjbrS
DM+vkC6aD+Dr8TM8ZV9n+/ol39+OHa61k3bRoI7um27LZ8jTdPaY/0x76Xuz
u7I/nTNCdg1H4vPkOcrH2Pe3az6mE3SV6umkax53Lxeolm5TA5W3jEjVv+He
m+Elu5+kR1zh3+Jg+07YXU/D5Qewr+vsxrrkmdtRRnPpJ9pEpXbe0T1rRwOe
tvMwHuR/x0/s3It7+K1YY88sLOfn4zz9dbiFX4Z1+uOxlJ+CH+uPwB/4z7FK
vxlO5tPxO/3eOJPPTvbrP0yLyOdfY0f9mziMv4xr9KuwF1+Dk/RP4wC+Dqv0
S7ALvwZb6BdhKl+IJfq5+KpzRB5+rV+ATeTjkv36GfhA3gPfy1vhX3xjLNK/
mOod8ddxqPwF/sHfxxL9/VjLH8dn8p34I1+JhfrTcTU/B/vLV+FGfjEW6X+F
xfxEvCsfjHP5flig/651xDd8c76nvBt+y3dI9uvfkY/ln+JVeQMO4c/hOP0K
7M5XY6b8CH7K78MC/QXYiV+O5+WzMYWfinn6ffC5/8YX2Fo+AUM+Mtmv3wbv
yXPwN/mHeI1/413k6p/Bo/wV/ED+CC/wtzBPv4ff7r/eRy6+5I/435yjCtpH
1dRA/wPVFLvW
          "]]},
        "0.35500000000000004`"],
       Annotation[#, 0.35500000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV02tszXccx/FfiyYu02pd2mlDBaEuQVNWEaQ6QkmpPVikBCGCqESkE2zS
Nc0usaLayaol7QOXEtVsdKITBHGNy4NDBEVdo2YTu5DZ9joP3n2fz+/7/3zP
/99zTvri1QVFMSGEXH8GfBjCnOQQCpCPuvgQZnNLzxBm8i45j2v5Z2d7XR/3
QQg7+oTwPapQbZbrmljnlfJ2VGCKsxy87xbCULMlvUJosqPOjn+clcq9vc52
zXiMxUcYhyTnYzgTo7DNe4zmGv1h/K2cwd9wpbMdrm+P7vS+X6IEZWbprnnm
/Av5c2xEirNktDlPc19n9ee5dnPvEL5DOQ7iABpxz7xjWgitfBernT3Tu+/1
Cs8wKcX9en3Y7iY8NSty/oSrXTvdLMn5T0hEKv5y3pkDrriPN9H/E14g3r44
1HT3/DwBychAOm7bG7G/FRPwGpvse84zPMdDr1/iFG7iMk57j5N2X8IRPEAE
Fd4jIfpcqSF04mt2v0oK4Yb8jifad8rey/jY/CiPdM8HuNnOEju2YC32Rz8f
lNnZpnvOjgifsfM4H5PP8xg7D/FBuY4b7FnFFfJ6/ky/knfLX3Gz/lwukxfy
IP2pvE7O5lr9TrxATuQV+sN5pdwvul//RaLvufw3J+vf41z5Om/Vr+UMuYHn
65/hcXIz1+qXcpq8lbvqF3OCvJxL9bP4z76+7zxbv5A7mOdH9+v35efmQ/hf
8+58R47hYv0rPXxG8m2eYv6Gr8pPuVT/Rz4in+TfzffwPvkHXq6/lMvlNTzW
fDPvlDdxsX4el8if8iPzibxWHs2F+u8T/AbkLvJQ84G8TO4T3a/fZj5L/o1v
mkd4snyB8/WreLBcz6nmLZwpN3Kh/gZOkb/mi+ZF3E1exDn6I/i13/B4jjf/
hP+Tp0X36yfxY7k//2Iex7fktz6LLP2zfEK+wbHm7XxJbuUc/SG+u7/6PLL4
D7nF7+gCqtCIekTwP1h+vz4=
          "]]},
        "0.28400000000000003`"],
       Annotation[#, 0.28400000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV03lsz3ccx/FPXWFoFVPauklrKxkpIQ1VpY7QCQuhloqIINsQKeGPRhrq
KKqORKg6QhThD7HGMUEQZxzbfkQccSwitjlSbKSOx/ePZ599vd/f9/t7/jpP
mzPul7gQwmR/mjYPoSophG2oxK6EEMa3DaGxenVyCCdbh7BDbYzads7nY2oj
eKs8jIcjDx0cX9gmhKnI+DqEPXJ9e6bIn5qF8I3/K5xjHbIdPwgf1NPVy9XW
YoOdWeq16qvlMqzCevUq561Lda3cyzGr1TK4jHvyJvXuXCp34W7oigTX0T66
PiQjFSlopP67mXrtQ/iDb2KIWi6ykYPBGIrh7ifecSP4aXT/aiMxLOohD/3c
x/J2IaxAKVZhJSa5njTPIB0Tzafyn+5vjN4PyMM0FGB3fAjFvAyzsBBz8cju
D2bjzOZyC16BJphgf6JnlIJ3cmP+zFedo9Z91uE54u1piK3O0YezkIQe6IQ7
zhGz+wGy8BrF9jzjkc7xyP//4DRu4QrOOMcpuy/jCB4ihgrnSPCc7nlfDfi6
3S9aecbyex5o32l7r2Co/q/c03Xv5xo7l9ixFvNRHX1rWGrnY7Pn7YjxWTuP
81H5Ave28yAfkHfwPnt+4gp5ES8wv4Gr5FKuMT+Ol8qF3C36nnmh3J8rzTfg
H+VEnmX+W54td4j2m3/eMoSx8n+cZP4+58o3uNx8JfeQ9/EU82e5n1zDleZL
OFUu56/MF3GCPJNLzGfy2xTfIn9vvoDr6edH+80n8zP9NP6o35zvyoGLzF9N
9I7kOzxEv5avyU+5xPxhPiKf4lf6e3ivvJlnmp/Oa+R53Fe/jLfIxVxkfhQv
kSfyE/2BPF/+jgvM17Xw/ctN5HT9rjxDbhPtN/9Yf7T8km/pxzhbvsj55jdy
d3knp+if4D7yIS4wv5jbysv5kv7P3FSeyjnmM/i13+YAjtcfz5+i32u033xL
/kvuyL/pN+Tb8v/eRab5c3xSvslx+n/zZfkB55hP8+3+631k8hv5hN/RRWzE
IexEDF8AmfPDbw==
          "]]},
        "0.21300000000000002`"],
       Annotation[#, 0.21300000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV1H1szWcYxvFHNFGiVUNbrZeYKcHQlGwrEbpJtmzeutWWgy0YtVjUuq3b
/miapmPLMmK6rSghGG3tEC+dDClB5i2bJY6lexGsRGpjZsykoz6/P65+f9d9
P9f9PL/nnNMBc0sKF3cIIVT4U5wewgJ6OyuEUb1CKMVcnK/2OnVJCaFBbaRa
U88QRmA931l9nn4nHE5rMkJYTeu6hdBoXXufEPbiHGvm0uNyW+WSrJ3Nt3cN
YRM/TH21zITMEGpwImaqZ9BM60pwCS2moda+hXXmvqx3x4x1fI76NrVBWMvf
Vi/S/xsH2m+Zcy2l5ebXWNfmbF/iNGumU3+5F/VfoiKaotYqW+j5c8/VtIqq
5JOjd6dO9LEZ4/Fn3GLtV9EdWBePzh7dA/8NxdU+U7ts5g7PzdjHubarHZM9
SmM9/4j5eAbL8D0qkM9zvph7iZtZzzfQdipRb4nu3synZL7gG1JD2Gfdbtnd
OFo/U6/FO/fFnfoDrM+J7ofPxlesTcezzvVCb3dBk2gOxWizTDl+SAupjEro
otn37BlkC7AbfkTJVGTvNGfMojvR9wQf4Gl73PIubdRKKeYk0Vp75GI+pdMQ
6k/N9kiYfZ7y6SaVm3MVn7XHRc9/0CE6R6fosD2azD5Je+gCJWilPVK986/u
oiP+YPb1Hu6d/w/HmXfI3FP0tP5eHO7c9dhoZoUZy6mUttFaqjLzkuwxMxJ4
xMxvcR//HY4y82ts4DdgnTmLcCX/Pr4rvwrX80uxUX46VvGv4kD5Z7CMfwJr
5TviLD4NF8oPxTeizzeaL9/6SAhT+X8xXf43LODP4Ar5WhzC1+FM+SM4hm/E
WvlKzOZXYGf5dzCVL8ZK+Ty8ne03i1PkY9hBf3I0X743XtXPwf/1u+IvfHt3
s+RP42G+GSfq38Lv+StYKb8L9/BN+Jf+FtzK12Cx/Dz8lF+Co/U/wTV8eTRf
/jms4Gfg7/rjsJQfiTH5tjTfeT6ZH6z/KM7ne0Xz5S/pP8/fwHP6CRzPH8fJ
8tX4GL8Rs/T3Yy4fx5j8B5jBL8MT+m9iF/41nCA/DG/6zT2JKfqFeJ+fFM2X
744tfD88oJ+EP/F3fRZ58kfxYPT/AoP+NTzJn4/+j8rn+O7+6fPIw3/4/X5H
x6ma4rSREvQQQkjfSA==
          "]]},
        "0.14200000000000002`"],
       Annotation[#, 0.14200000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV03tszXccxvHvSGaLVs0ULRUtVbYEqYWISXVqLqvrXHYsFrGlRakUzfZH
I43rajohCLXoMpdqiNA14hKEZUZjkzi1ziWrySKo+8wWttnr98dz3uf5fb7P
8zm/8zsnffbCycUvhRAWeXmeEMKMTiF80DmE6TSN+iaHEIuuUZb3GWkh9MGv
O4YwPzWEBVREvVyb5sy86H1iCHNxtfyXSSFUO1vFP9b/vjNTqKfzCboycDLf
A9P5dPxcpncXeczCe3ITnGnBzXq6O1OpMw038XdcH2de4Vqund1cH45dcY55
vtl8umJegVNdL8YFNNhnHckn2v0u5tFYSuLHYBFdinbxP2MjDdP5C76NTdhP
R7V9pfwS2hnds1xhtJ8O80V2nXd2O7+N6tqFUODe5tJM+phmUZN7WRPdH6XS
RlpPtc4f0nNQ/0EcaGdnfTe6uV/cb97Djkx6yKfidGeT8aLO91J8z5RHsyhG
38iU4XIqpFIqpmbdf9v5gnKpnY5V1CZ6dnYnudcUesK/jP9igx2PfP5ndIsS
9LSmrXYMwCGUTFnUnZrsiOu+RkPoAZXpuYmj7Gj2/g6doEY6RyftOK77LNXR
rxSndXYkuucrvotW+KPuu6+HcIH/C4fqO6H3HL1j/i2+6XPvwXqdS3WspRLa
TVtpmc7rst/piOMpnYfxEP899te5F2v57VijZx6u4z/FJfLr8St+BdbLT8Rl
/EzMkB+BpfwgrJJvhR/ySVgo3xfnRM836pe/1SGE8fyfmCx/FXP5n7BSvgqz
+BqcIX8K3+LrsUq+HFP5SnxFfjEm8gVYLp+Nf3QNIQfHyccwmOdH/fJd8KZ5
Jj43b4uX+f9e0yXfgCf5Jhxu/gjP879jufwBrOOP433zHbiL34wF8rPxC34h
DjSvwC18WdQvPxqX8lPxN/OhWML3w5j8s/Z+83wbvrd5On7Cd4z65a+bj+Xv
YaN5HIfxZzBffgP25KsxxfwIDuD3YUz+M+zEr8QfzIvwVf4jzJF/Ax/4zw3G
BPNJ+A+fF/XLt8cbfBoeNW+Nl/innkW2/Gk8xl/AF87fxrP8NczhM/12WzyP
bHzMH/E/OkMbaB9VU5z+B8PJ01w=
          "]]},
        "0.07100000000000001`"],
       Annotation[#, 0.07100000000000001, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV03tolXUcx/HfppuX3dSd3XRiypQUUqYoS2RqgmGWpoUsUSRr3iCTEvtj
yLASvAxRotwShdK2qUjO0/KCikneMIWOMdLp5rRhanmrlHmZvZ4/3nufz+/7
fL/f5+x5zsD5H85cmhRCeN+ftvQQcvJDyEUfxJCNq857cW9kIAuZaHWexuno
jp7ogRbnqdwNXZCCrrjiPImT0ZlnKT/ny86f8jN04Akeo9l5fW4IO1GLOuT0
DyEPubikPs1101GCl9HXeSH64VZGCH/E3IvP7dyG60iRr/FF/QezQoj3DeEQ
33T9j3bsRxyN+AGjnX+VE8J286v5rN6t/DUaMkMo9z0WYQ7mYx6azF6rXoUC
fIGNqHd9o1177dzLI83Oje6t0D3zHvUBdhbhrlzAs1wb41/NfK0ghBmYhHko
wzd6KvhTLMByfIAWsx/Z2YkJyDBjNVLxlt2Zvk8+/pFT+CmfseOe++/ADaSZ
k4xqO0ZwCWIYgv5osiNhdjNKcAcV5rTzZDtafL6JI7iA0zhqx2GzT6Ehei+Q
wAY70qPn4n+RxL+YfTs7hPPyQx5r3hFzT2Oi+j4e5r7rOG7mSjPWYxm+QzVW
mdmq97gZCT5m5n5ulH/m4Wbu4np5K9eas5g3yCv4Y/0beYv8Gcf1T+dV8hwe
qP8VXi6P5proHefZciYv0P8iL4yebzRf/40+Ibwh/8sx/Zd4gnyOq/TX8BC5
lt/Rf4xHyXGu0V/JBXIVd9P/EafL5Vypv5gf9AuhlF/XX8bP5anRfP153C4X
8WP1nvy7/Ky3WfrP8FG5icer3+Oz8nWu1P89N8iH+W/1b3mH/CWX63+X18lL
eaT6Gt4sV0Tz9b/KK+W3+ar6WF4mv8Rl+jt6eeflVHmw+gv8npwdzdffqj5F
/osvqCd4nHyCp+rfxIPkbZyvfoBHyLu5TP8nnCN/zifVl3B3eS6X6h/Kd/zm
xnCa+pv8RJ4UzdefxW1yIR9UT+bf5P88i2L9P/Eh+Tx3uv5PPiU3c6lc5N29
5XkV8335gN/RCWzCbmxDAv8DiG/QqA==
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV031olWUch/HfKIvUmXqcO+l6WUvKKJmSDZNlCUVppU3jGNNRNnujGnO5
dMlaZlitMQlzKxRSM0ki5xAtVEwyXZbQMUZlvhGimy9Ns2JaRp/nj2vX+Z77
/v7us/ucp3B2VdlLORFR7c/4oRGp3IiH8yMewmQ8NSTilbyIWqT7RywZFvG8
98rlU9dEnEYXTqIb871fYF+XbjdOYN7VEXN1qpFn/kx7Xva6tyCihl+Vi3Sa
zF4gnzOnVm+BXg+flXuQ1r3Rvnftm29fvfUGfg0VZszCRfsWJbP5db7A53mk
XrNeJumgzN5B10ZM4yl4FAPlqTwdt9q/1P43dRejxOtveSx38J38K1+mc5DX
mPm++1uOFtzlvUKffTVazNsrr+AP0DYgojId8QzK8SQq0OnMt6w3JneN99CM
dfZvMn+DMzfwaPeQZ95R9zeMP7N+nTOL8Luc5sfsTfEPZj7oDqZiIiqQwUc6
dckdYQ5q8AIOmf23My9hAvqbsRh9UObsXP9PPv6QL+d/uMMZPT5/L46jrzk5
aHHGKC5BCiNQgE5nZM0+gBKcQZ05x/g+ZxzyugvbsB97sN0ZW83ejTYcRBZN
zujnf/7ZXQR/Z/bJVMQ++S8eZ942c/fgHusbeaTP/Qm3m7nQjHdQhY/RggYz
D+vuNCPLO8zczJvkr/l2Mz/ldfIKXmvOs9wkz+O5+s38obyI2/Uf4Qa5nG/Q
v5dr5Du4VT/4cTmX5+jfzE8n328yX//4YM+kfJ5T+r/wBPl7btRv5RHyWp6h
v4PHyO3cql/PabmRr9Cv5n5yJdfrF/O54RGlPFk/w//Jk5L5+kP5mFzEF6xf
xT/J/w4yS7+Dt8udfLf1Ht4r/8b1+p9zm7yVT1tfxWvkZVyp/wS/Lb/Io60v
4eVyXTJf/35eKE/jI9bHcZV8G2f0ewf6zct95JusX8+z5cHJfP3D1h+QT/F+
61keL+/iScmzzoXySs63voVHyes5o1/LQ+Q3+Bvrz/GV8kwu1b+Fz3jmxnJf
61P4ojwxma8/gI/Kw/kL6zn8o/yn76JY/yv+Ut7Hl+w/wbvlA1wqF/ntdvu+
ivmsvMVztAtLsR4rkcX/k8PZ8A==
          "]]},
        RowBox[{"-", "0.07100000000000001`"}]],
       Annotation[#, -0.07100000000000001, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV03tolXUcx/HfaEbqtHY2N52ZuUSFks20ITEtoch5ybxwNE1im5KmbFNn
cZS1ahElon+EjkhQSyUpL0O8oGKlbVJNPIpoolNibd6X1+ZdX88f773P5/n+
Pt/nOTvn9Ckpn1CWEkKo9CclLYSKbiGMyg6hCCMxLjOEKa6ldXHIfHFOCO+4
Vvp0CKd7hNCEU5joWldnUp2p12tAMUow2ez6syHEeYFdlXjP6xuuTeN0vc56
1XZPl2fb3aj3IR/if+1vxkK9mHM1zhU79648FRmufeFaiWvlOuecne11K8/i
TPuznWmxqxXlqIgwa/cM5ZywZxE+9rpN7yPO1uujd9vZKnsTvAj/m7fjFj5x
rp97N/Aas7UoQI38Cj905gFy7Bpg1yBnX0Y+hmAwxmIMLuj8zOu7eu6sEH6T
U3qFsD96Fs+2DBvMtnmWzc5t5nx7M83Peh89+CfzXrq5uBL1eJKzMT7s/iM9
yziMwHTEsVonwZ9iBuZjDk7bfcs972M4OttRg1SMd+807y8L1+Qn+C4fdI+2
7v4/aEFHewJWusdALkAMfdETx9wjafdJFOAyEvY08xvRd8zrc9iDI2jAXvfY
bXc9tuAUkljqHp285xP+F48yQvjT7gvcKN/kofbtsbcBrzm3lQd47vVcF323
7fgaZfgBK1FtZ5Pur3YkeZ+d23mbvJ9fsvNH3iB/x+vs+YCXypU8T38Zfyt/
xnX6Y7lansq99V/n+fJgrtV/FPObk9PkGfr9eGb0+Ub79VvMR8vXOab/Nw+X
/+Il+rXcV17Hk/X38SC5jmv1qzhbXsId9Cu4k1zKVfp5fLVnCIU8Sj/OD+Si
aL9+N26Wc/m2+VN8XL6Xbpf+Qd4rH+Nh5m38h/wPV+lv4i3ybr5kvoa/l7/h
Uv33+St5Luebf8kr5ES0X/9NXixP4DPmQ7lMfpHj+u3P+M7LqfIL5s9xsZwe
7ddvMn9LvshHzJP8qnyAi/SX8/PyKs4y38ED5Y0c11/IGfLn/Hv0W+Un5Wlc
qN+fL0e/b+5o/jbfkUdE+/W78Fk5h3eaBz4q3/BZ5On/wrvkRr7vfCvXyye5
UM713T3v88rj/+QdfkcHsBwbsQpJPAbfyNk3
          "]]},
        RowBox[{"-", "0.14200000000000002`"}]],
       Annotation[#, -0.14200000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV02tM1XUcx/FfXqZcSg6CF8ocuWoLRMsx5ohyqxU1b+CoQekMyltD8/KA
MabV1so5fdCKtVy1QqcPnLKmNmXquE5l6+CcygJZIWZK2IWLJQ16/R+8eZ/P
//v/fH/n/M8hs3xL8eYHQgir/FmWFkLDzBBWTwuhlMvwBsaTQqjOCGG5+Vtm
Z10r53Nc7FoTvyM3cgua0TM7hK3pIWxDmXt+kUs55cEQEpNDuOie9qiHdUh1
fZrrV72+hvXYgI2Iud7Nm51RGTma2zX8iC7fkLeb9fIO7ovucf13rpL7+S4G
IryPIR7GIO5hBEOuL+SnUTcjhIM4gDw522efjywsQA6+esheZ0yZ4xy+gyLX
i6PnhJVYgVWI2fOjz/CyM5ZjCd5ECb62p4p3oSJ6ZtiELs9jyLMbRQES7fgQ
E7HSZ0ryvtLxpzyB/+U2ZwzM8nnQh6n2jPPnzsj2OhcxzEMGLjsjbncnctGP
Knt6+QVndHn9K06jA61ocMYpO1twFD8hjj3OSPAMrvpOxqaHcMHu37hdHuQ8
+07b24rn3HeMn/S+D3C9ndV2fIJKfBe9Z+y0s1v3nB1xPmPncf5ebuQsOw/x
QflLrrNnHe+Rt/N7+nv5C/l9rtdfyjvlUn5U/3neKj/DtfpjqSG8LifJFfqP
89vyrGi/fp/5q/JfHNO/xgXyRd6tX8vz5Dp+Tf8ML5TruVa/hmfIu3mS/hZO
kMu5Rj+H/3g4hHx+Rb+E/5MLo/36adwrZ/I98yl8Rb4fs0u/jRvky/ys+QCf
l3/mGv0jfFQ+xXfMv+Fv5U+5XH8Nfyy/ywvMP+LP5Kpov/6LXC0X8XXzPK6U
n+IS/ZEUv3l5ovyY+RxeK6dE+/W7zV+Sb3OHeZwXy01cqL+P58r7Od38BGfL
h7lEfwenyh9ws/l6niyXcb7+E9zvf24RTzVfxv/IS6L9+sncI8/mk+bjvoNL
8t+co3+Wf5DbedT9N7lF7uR8OdNv95bvK4fvyif8HzVhHw5jP+L4HwBjztc=

          "]]},
        RowBox[{"-", "0.21300000000000002`"}]],
       Annotation[#, -0.21300000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV02tolnUcxvGfGm0eatNt6Twxj3lgiSJDZiYUMg/TcqxQqHRT9yI8H5hj
igZiJvoicoS+8FTki9ARW7qNGZrKyheP5VlX6DIPDe0wsDzg8HO/uJ7vff1+
93X97+c0pHxFyfIuETHDS2fPiIa+EY10jN7PiGjC4uyI47iAb8aFONdsQ/+I
br0iTpgtNvswuSfZU5p5C1aYL0r2lPFSRHfz8a9ETKBOHQt1lFMZFZj9nhtx
3r0X6Be6RBfptvk1vE5XaZneVlyiow1X8TdxNS4z+1TfAGe1m1WarcG1tI7y
zDtwg3klrqdczzbSvMgzTKfsnIhC8yycjDPNZtEovaPpVWpxzhicTx2u99nv
p9l6V2ExDhsUMYKG0xxd691bRXU659hvdt/H1JXe5nuYZ9PffBd8hGc81/1+
EQ/pFqX5LDpx18sRY11PpEwaSrnJ5+e9pJx1hSZSO1XqacM3nXHd9W1qpHN0
mpqc0aDzFB2ma5Si7c5I994uDYx4lhXxo+67eJbvwAJ9jXpP0xT3HcGRnvtL
rNVZpeMTWkoHkmemjTpbZb/XkcJmnXX4LX8Cx+j8Gr/id+NBPUtwO78aV8jv
wC/4TVgrPws38vNwkPwbuJIfjzXyz/pEvMf34Mvlh+Mivm/SL3/Lfgb/D2bK
X8bX+Z9wm3wNDuUP4rvyzTiOr8Ua+WrM4bdhN/nlmM6XYbV8Pv41wG8Kp8uX
4lO+KOmXz8I2Pg//s38RL/KPe+uSP4NN/HmcbH8fW/gbWC3/DR7mG/BP+724
n/8My+Q/wK38R/ia/Rb8nK9M+uXfwir+HfzVvgCX8qOxVP5hZkQJ35UfYj8Q
F/AZSb98q/00/h6es0/hJP4kFsnvxMH8Hsy2r8ex/CEslV+DvfnN+IN9Bb7A
z8dC+RHY7n8zAdPsi/F/fmrSL98Tf+P74Xf2nb6Dn/l/MV/+OB7lz+IT9/+B
p/grWMjn+e3e8X3l4wO+3v/oJO2kQ7SHUvQcR53B1g==
          "]]},
        RowBox[{"-", "0.28400000000000003`"}]],
       Annotation[#, -0.28400000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV02tsjnccxvGfczBlbZU5xMSZEhKN1FbGqkSJTIm9qDIhWOtUNNIVycQp
k2WHRibOhBdCIw5BkDkFfVGyOGRjmWYnBHGqOoTE535xPd/n+v9+13XfT577
7jpjwcT5DSJiuI+69IjSjIg+NLpdRB6NohfO+znrS2N9f9opIh+LWkcUmk+l
Vq0iGn4QkWmnCS7vEFFvZxK9pBV8U+f9zSfz0/g3WIhFNJUGmt35KKKWqnXO
1n8Z5+A7u9/aSdEx2F4WNW4b0bxzRCMcyk+zU+fepmOJzNeyxVRCGXI5djrg
Oj0tZUbyI6jYfr1cCZbJLbJfmmT8pi72c+30wO/lUuW+sZfnrAKfudfl+BxX
yvZ0P73oR/NUTKP05Ex+mM7hSa+OT7Et5uBiu+tSIjZjvrMtuJW20UL6TG+u
vXFmn+N4XOp8CZVRLT/Czh++/0vH6QqdpxOue6x9xDnaT79TDa13vWbu+7rf
/TYt4pLf+j9W808xS99xveeTe7V3AHu49i6s0rlMxxoqph30M1XovCV7WkcN
ntR5CA/yv2IfnXtwN/8L7tQzE9fzi3C+/He4kV+BVfJjsYKfgp3kc3ABPxAr
5d+mep745vxX8t1wBp+R9Mv/bT6Gf4yt5W/gJ/xlXCtfiV35nThJ/iQO4Kuw
Ur4c0/m12FB+HjZLnjEsT553fNQxIhtHyxfgGz4v6ZdPxVq+C74wb4LX+Fcf
6pK/gCf433Co+QO8yP+F5fL7cD9/DO+Zb8Xt/A84Xb4QV/Nzsb/5KvyJL0v6
5UfiMn4C3jbPwmK+NxbI17WJ+IJvwH9s3hGL+JSkX/6WeS5/F6+Y1+AQ/gzm
yW/AzvwmTDM/jH35vVggX4pt+JV41nwWNuK/xGz57njfezUIm5rnYz0/LOmX
b4F/8u3wiPk7/8FV/glmyp/Co3w1vrb/D57jb2J28v56dv/zf2XiQ/6w9/QM
baC9tIlq6D0+ibGh
          "]]},
        RowBox[{"-", "0.35500000000000004`"}]],
       Annotation[#, -0.35500000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV01toznEcx/Hf5DDs2cwMM3uGrdAwIeZMGpmSHkIWc5hDzi7W2hYhluxC
LNmFnLILaclIGhlJXMwhcTFikUzIIYS24vVcfPbu/fv+Pt//ntPQtTti2xNC
CJn+JCaFMKZ/CH/6hbBiUAh/sRiXY77zJPO9vIOvkk7ZxyPOx5v3wccDQngi
j2RLisXpIUSzQvjn7iR3Engi74KTeYH00ztoTzdnpe51x2qe7nyaeQa+tq9N
XslOexPdybanB85wpyf24r1wFp8pWXqH7UlyVmZvBGt41Pkc81z8bl+lfd+w
ClPcGWJPMhbGXxP25qlYaccG/Y2yTtZLqcx3ryPD+yHj7JkgA3XGYwbmR0Jo
1y1y9wMuwMpkzzEv4FmYLVEZJjky1XkunvXcBv9n+UDvkWyW03JMquxoTQvh
5uAQWrDJcy7jJX4LR3g95/EcP4Fn7FmHh/hO3KZ/GI/z3digPx+r+FLM1J+G
23k+1up39A1hCU/kq/WH4RqeHt+v/8Z8Hv+CyfrPcAq/j9X6tTiEn8HF+k04
mjdgrX4FpvFqTNDfit15CVbo5+HnTO8bztWP4V9eGN+vn4ptPIo/zbviU/47
1S79u3idP8HJ5h/xHn+FFfoX8CK/hu3mJ/EUP4Il+sV4gG/CUeb78Sgvi+/X
n43lfCG+MJ+Am/lwjOn/6BPCIh54tvkgXMkj8f36reZz+Ht8aN6CE3kzFurX
4GBeh33NG3Ekr8eY/i5M4XvwtnkpduHLsEA/Bz/4/o3FbuZF+ItPj+/X74kv
eX+8Yt7pM3jEv2Ke/g28yh/gH/ff4h3+HAt41Pf2nc8rDz/xRr+JZqmReqmT
FvkPzTiTPw==
          "]]},
        RowBox[{"-", "0.42600000000000005`"}]],
       Annotation[#, -0.42600000000000005`, "Tooltip"]& ], {}, {}}}], 
   {GrayLevel[0], AbsoluteThickness[3], LineBox[{{0, 0}, {1., 0}}]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.75, 
      FormBox[
       RowBox[{"-", "0.75`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.25, 
      FormBox[
       RowBox[{"-", "0.25`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.25, 
      FormBox["0.25`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.75, 
      FormBox["0.75`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}, {1.25, 
      FormBox["1.25`", TraditionalForm]}, {1.5, 
      FormBox["1.5`", TraditionalForm]}, {1.75, 
      FormBox["1.75`", TraditionalForm]}, {2., 
      FormBox["2.`", TraditionalForm]}}, {{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2, 
      FormBox["1.2`", TraditionalForm]}}, None, None},
  ImageMargins->0.,
  ImageSize->504,
  Method->{"TransparentPolygonMesh" -> True},
  PlotRange->{{-1.1536749816983884`, 
   2.1536749816983884`}, {-1.3536749816983884`, 1.3536749816983884`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 419},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->64041075]
}, Open  ]],

Cell[TextData[{
 "Arrows for velocity field are green, equipotential lines are red, \
streamlines are blue and panel is black.  Note orthogonality of equipotential \
lines and streamlines, streamlines parallel to velocity field and outward \
flow associated with positive ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 ".  The thickened streamline along negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis suggests a branch cut for stream function.  Also, note that the \
velocity field shows an abrupt change in normal-component velocity as the \
panel is crossed."
}], "ExampleText",
 CellID->680409942],

Cell["\<\
Potential-flow singularities may have discontinuities in velocity, velocity \
potential and stream function across the panel.  Such discontinuities are \
called jump conditions.\
\>", "ExampleText",
 CellID->62898067],

Cell["\<\
Compute the jump conditions for the tangent-component velocity across panel.\
\>", "ExampleText",
 CellID->673890120],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icuaMinus", ",", "icubMinus"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{"icua", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}], ",", 
     RowBox[{"Limit", "[", 
      RowBox[{"icub", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icuaPlus", ",", "icubPlus"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{"icua", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", 
        RowBox[{"-", "1"}]}]}], "]"}], ",", 
     RowBox[{"Limit", "[", 
      RowBox[{"icub", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", 
        RowBox[{"-", "1"}]}]}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icuaJump", ",", "icubJump"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"icuaPlus", "-", "icuaMinus"}], ",", 
     RowBox[{"icubPlus", "-", "icubMinus"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"uJump", "=", 
  RowBox[{
   RowBox[{"a", " ", "icuaJump"}], "+", 
   RowBox[{"b", " ", "icubJump"}]}]}]}], "Input",
 CellLabel->"In[13]:=",
 CellID->98488054],

Cell[BoxData["0"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[13]=",
 CellID->237515591]
}, Open  ]],

Cell["Tangent-component velocity is continuous across the panel.", \
"ExampleText",
 CellID->9527100],

Cell["\<\
Compute the jump conditions for the normal-component velocity across panel.\
\>", "ExampleText",
 CellID->134318740],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icwaMinus", ",", "icwbMinus"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{"icwa", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}], ",", 
     RowBox[{"Limit", "[", 
      RowBox[{"icwb", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icwaPlus", ",", "icwbPlus"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{"icwa", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", 
        RowBox[{"-", "1"}]}]}], "]"}], ",", 
     RowBox[{"Limit", "[", 
      RowBox[{"icwb", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", 
        RowBox[{"-", "1"}]}]}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icwaJump", ",", "icwbJump"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"icwaPlus", "-", "icwaMinus"}], ",", 
     RowBox[{"icwbPlus", "-", "icwbMinus"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"wJump", "=", 
  RowBox[{
   RowBox[{"a", " ", "icwaJump"}], "+", 
   RowBox[{"b", " ", "icwbJump"}]}]}]}], "Input",
 CellLabel->"In[14]:=",
 CellID->326115239],

Cell[BoxData[
 RowBox[{"a", "+", 
  RowBox[{"b", " ", "x"}]}]], "Output",
 ImageSize->{41, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[14]=",
 CellID->416578630]
}, Open  ]],

Cell["\<\
At a given location on the panel, there is a jump in the normal-component \
velocity across the panel equal to the value of the source density at that \
point.\
\>", "ExampleText",
 CellID->105928607],

Cell["\<\
Compute the jump conditions for the velocity potential across panel.\
\>", "ExampleText",
 CellID->211831706],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Phi]aJump", ",", "ic\[Phi]bJump"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Phi]a", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", 
         RowBox[{"-", "1"}]}]}], "]"}], "-", 
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Phi]a", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], ",", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Phi]b", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", 
         RowBox[{"-", "1"}]}]}], "]"}], "-", 
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Phi]b", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[Phi]Jump", "=", 
  RowBox[{
   RowBox[{"a", " ", "ic\[Phi]aJump"}], "+", 
   RowBox[{"b", " ", "ic\[Phi]bJump"}]}]}]}], "Input",
 CellLabel->"In[15]:=",
 CellID->189498322],

Cell[BoxData["0"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[15]=",
 CellID->105008248]
}, Open  ]],

Cell["Velocity potential is continuous across the panel.", "ExampleText",
 CellID->56731244],

Cell["\<\
Compute the jump conditions for the stream function across the panel.\
\>", "ExampleText",
 CellID->403496840],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Psi]aJump", ",", "ic\[Psi]bJump"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Psi]a", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", 
         RowBox[{"-", "1"}]}]}], "]"}], "-", 
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Psi]a", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], ",", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Psi]b", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", 
         RowBox[{"-", "1"}]}]}], "]"}], "-", 
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Psi]b", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[Psi]Jump", "=", 
  RowBox[{
   RowBox[{"a", " ", "ic\[Psi]aJump"}], "+", 
   RowBox[{"b", " ", "ic\[Psi]bJump"}]}]}]}], "Input",
 CellLabel->"In[16]:=",
 CellID->102020424],

Cell[BoxData[
 RowBox[{
  RowBox[{"a", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "x"}], "+", "\[Delta]"}], ")"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", "b", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["x", "2"]}], "+", 
     SuperscriptBox["\[Delta]", "2"]}], ")"}]}]}]], "Output",
 ImageSize->{162, 32},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[16]=",
 CellID->97171684]
}, Open  ]],

Cell["\<\
There is a jump in stream function across the panel consistent with the jump \
in normal-component velocity.\
\>", "ExampleText",
 CellID->344923329],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"wJump", "==", 
  RowBox[{"-", 
   RowBox[{"D", "[", 
    RowBox[{"\[Psi]Jump", ",", "x"}], "]"}]}]}]], "Input",
 CellLabel->"In[17]:=",
 CellID->114820653],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[17]=",
 CellID->54832686]
}, Open  ]],

Cell[TextData[{
 "Look at the properties of the stream function as the negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis is crossed in a counterclockwise direction."
}], "ExampleText",
 CellID->635547329],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Psi]aBranchJump", ",", "ic\[Psi]bBranchJump"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Psi]a", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"x", "<", "0"}]}], "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", 
         RowBox[{"-", "1"}]}]}], "]"}], "-", 
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Psi]a", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"x", "<", "0"}]}], "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], ",", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Psi]b", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"x", "<", "0"}]}], "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", 
         RowBox[{"-", "1"}]}]}], "]"}], "-", 
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Psi]b", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"x", "<", "0"}]}], "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[Psi]BranchJump", "=", 
  RowBox[{
   RowBox[{"a", " ", "ic\[Psi]aBranchJump"}], "+", 
   RowBox[{"b", " ", "ic\[Psi]bBranchJump"}]}]}]}], "Input",
 CellLabel->"In[18]:=",
 CellID->81019929],

Cell[BoxData[
 RowBox[{
  RowBox[{"a", " ", "\[Delta]"}], "+", 
  FractionBox[
   RowBox[{"b", " ", 
    SuperscriptBox["\[Delta]", "2"]}], "2"]}]], "Output",
 ImageSize->{62, 35},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[18]=",
 CellID->6075130]
}, Open  ]],

Cell[TextData[{
 "There is a branch cut along the negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis for the stream function.  This abrupt change in value of the stream \
function as the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis is crossed causes the thickening of the streamline unless special \
care is taken with the options of ContourPlot."
}], "ExampleText",
 CellID->78970066]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Scope",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->1293636265],

Cell[TextData[{
 "The general application of influence coefficients is in the construction of \
a panel method.  The current version of ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " does not include any panel methods based on source panels of linearly \
varying strength.  See the ",
 ButtonBox["scope section of ICVortexLinear",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICVortexLinear#1293636265"],
 " for an example of a general application of influence coefficients for \
vortex panels of linearly varying strength."
}], "ExampleText",
 CellID->246788676]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[{
 StyleBox[ButtonBox["ICSourcePoint",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICSourcePoint"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICSourceConstant",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICSourceConstant"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICDoubletLinear",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletLinear"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICVortexLinear",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICVortexLinear"],
  FontFamily->"Verdana"],
 " "
}], "SeeAlso",
 CellID->745892494]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["2.1 Introduction to Influence Coefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"]], \
"Tutorials",
 CellID->38102921],

Cell[TextData[ButtonBox["2.3 Isolated Panel",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/IsolatedPanel"]], "Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["InfluenceCoefficients",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/InfluenceCoefficients"]], "MoreAbout",
 CellID->179478344]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"ICSourceLinear - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 0, 51.3852000}", "context" -> 
    "Aerodynamics`InfluenceCoefficients`", "keywords" -> {}, "index" -> True, 
    "label" -> "Aerodynamics Package Paclet Symbol", "language" -> "en", 
    "paclet" -> "Aerodynamics Package", "status" -> "None", "summary" -> 
    "ICSourceLinear[r_f, r_p] computes velocity influence coefficients at \
field points r_f, due to connected source panels of linearly varying strength \
whose endpoints are located by r_p.ICPhiSourceLinear[r_f, r_p] computes \
velocity-potential influence coefficients at field points r_f, due to \
connected source panels of linearly varying strength whose endpoints are \
located by r_p.ICPsiSourceLinear[r_f, r_p] computes stream-function influence \
coefficients at field points r_f, due to connected source panels of linearly \
varying strength whose endpoints are located by r_p.", "synonyms" -> {}, 
    "title" -> "ICSourceLinear", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/ICSourceLinear"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[10714, 325, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->190734705]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 272686, 5752}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[607, 21, 76, 1, 70, "SymbolColorBar"],
Cell[686, 24, 191, 5, 70, "LinkTrail"],
Cell[880, 31, 3562, 81, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4467, 116, 57, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4527, 119, 2360, 74, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6924, 198, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->105283275],
Cell[7507, 217, 364, 11, 70, "Notes",
 CellID->1067943069],
Cell[7874, 230, 999, 29, 70, "Notes",
 CellID->78062312],
Cell[8876, 261, 633, 16, 70, "Notes",
 CellID->433990591],
Cell[9512, 279, 1088, 38, 70, "Notes",
 CellID->213924258],
Cell[10603, 319, 74, 1, 70, "Notes",
 CellID->529540534]
}, Closed]],
Cell[CellGroupData[{
Cell[10714, 325, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->190734705],
Cell[CellGroupData[{
Cell[11200, 342, 146, 5, 70, "ExampleSection",
 CellID->5012110],
Cell[11349, 349, 132, 6, 70, "ExampleText",
 CellID->513831186],
Cell[11484, 357, 99, 3, 70, "Input",
 CellID->66474307],
Cell[11586, 362, 429, 11, 70, "ExampleText",
 CellID->659456463],
Cell[12018, 375, 204, 6, 70, "ExampleText",
 CellID->42111717],
Cell[12225, 383, 495, 18, 70, "Input",
 CellID->64953589],
Cell[12723, 403, 356, 9, 70, "ExampleText",
 CellID->326501090],
Cell[13082, 414, 444, 15, 70, "Input",
 CellID->560964283],
Cell[13529, 431, 217, 7, 70, "ExampleText",
 CellID->217164756],
Cell[CellGroupData[{
Cell[13771, 442, 73, 2, 70, "Input",
 CellID->319896790],
Cell[13847, 446, 401, 14, 54, "Output",
 CellID->152403999]
}, Open  ]],
Cell[14263, 463, 674, 14, 70, "ExampleText",
 CellID->120984796],
Cell[CellGroupData[{
Cell[14962, 481, 438, 15, 70, "Input",
 CellID->176773044],
Cell[15403, 498, 726, 24, 67, "Output",
 CellID->355656643]
}, Open  ]],
Cell[16144, 525, 243, 7, 70, "ExampleText",
 CellID->118713566],
Cell[CellGroupData[{
Cell[16412, 536, 437, 15, 70, "Input",
 CellID->25568170],
Cell[16852, 553, 1397, 42, 67, "Output",
 CellID->6664491]
}, Open  ]],
Cell[18264, 598, 142, 4, 70, "ExampleText",
 CellID->44302699],
Cell[CellGroupData[{
Cell[18431, 606, 753, 24, 70, "Input",
 CellID->575890672],
Cell[19187, 632, 2157, 66, 90, "Output",
 CellID->37289698]
}, Open  ]],
Cell[21359, 701, 89, 1, 70, "ExampleText",
 CellID->22197235],
Cell[CellGroupData[{
Cell[21473, 706, 751, 24, 70, "Input",
 CellID->8664355],
Cell[22227, 732, 1823, 56, 115, "Output",
 CellID->268350184]
}, Open  ]],
Cell[24065, 791, 190, 4, 70, "ExampleText",
 CellID->9427774],
Cell[24258, 797, 750, 26, 70, "Input",
 CellID->258291294],
Cell[25011, 825, 947, 34, 70, "ExampleText",
 CellID->78226744],
Cell[25961, 861, 253, 9, 70, "ExampleText",
 CellID->421446271],
Cell[CellGroupData[{
Cell[26239, 874, 584, 18, 70, "Input",
 CellID->13857323],
Cell[26826, 894, 210, 7, 36, "Output",
 CellID->292255161]
}, Open  ]],
Cell[CellGroupData[{
Cell[27073, 906, 609, 19, 70, "Input",
 CellID->499128776],
Cell[27685, 927, 210, 7, 36, "Output",
 CellID->537757821]
}, Open  ]],
Cell[27910, 937, 1033, 31, 70, "ExampleText",
 CellID->78531558],
Cell[CellGroupData[{
Cell[28968, 972, 4691, 134, 70, "Input",
 CellID->1746043248],
Cell[33662, 1108, 218285, 3987, 70, "Output",
 CellID->64041075]
}, Open  ]],
Cell[251962, 5098, 644, 14, 70, "ExampleText",
 CellID->680409942],
Cell[252609, 5114, 227, 5, 70, "ExampleText",
 CellID->62898067],
Cell[252839, 5121, 127, 3, 70, "ExampleText",
 CellID->673890120],
Cell[CellGroupData[{
Cell[252991, 5128, 2392, 71, 70, "Input",
 CellID->98488054],
Cell[255386, 5201, 159, 5, 36, "Output",
 CellID->237515591]
}, Open  ]],
Cell[255560, 5209, 101, 2, 70, "ExampleText",
 CellID->9527100],
Cell[255664, 5213, 126, 3, 70, "ExampleText",
 CellID->134318740],
Cell[CellGroupData[{
Cell[255815, 5220, 2393, 71, 70, "Input",
 CellID->326115239],
Cell[258211, 5293, 204, 7, 36, "Output",
 CellID->416578630]
}, Open  ]],
Cell[258430, 5303, 210, 5, 70, "ExampleText",
 CellID->105928607],
Cell[258643, 5310, 119, 3, 70, "ExampleText",
 CellID->211831706],
Cell[CellGroupData[{
Cell[258787, 5317, 2082, 57, 70, "Input",
 CellID->189498322],
Cell[260872, 5376, 159, 5, 36, "Output",
 CellID->105008248]
}, Open  ]],
Cell[261046, 5384, 92, 1, 70, "ExampleText",
 CellID->56731244],
Cell[261141, 5387, 120, 3, 70, "ExampleText",
 CellID->403496840],
Cell[CellGroupData[{
Cell[261286, 5394, 2082, 57, 70, "Input",
 CellID->102020424],
Cell[263371, 5453, 468, 17, 53, "Output",
 CellID->97171684]
}, Open  ]],
Cell[263854, 5473, 159, 4, 70, "ExampleText",
 CellID->344923329],
Cell[CellGroupData[{
Cell[264038, 5481, 179, 6, 70, "Input",
 CellID->114820653],
Cell[264220, 5489, 161, 5, 36, "Output",
 CellID->54832686]
}, Open  ]],
Cell[264396, 5497, 231, 6, 70, "ExampleText",
 CellID->635547329],
Cell[CellGroupData[{
Cell[264652, 5507, 1911, 53, 70, "Input",
 CellID->81019929],
Cell[266566, 5562, 287, 10, 56, "Output",
 CellID->6075130]
}, Open  ]],
Cell[266868, 5575, 434, 11, 70, "ExampleText",
 CellID->78970066]
}, Open  ]],
Cell[CellGroupData[{
Cell[267339, 5591, 221, 7, 70, "ExampleSection",
 CellID->1293636265],
Cell[267563, 5600, 587, 13, 70, "ExampleText",
 CellID->246788676]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[268199, 5619, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[268619, 5633, 651, 22, 70, "SeeAlso",
 CellID->745892494]
}, Open  ]],
Cell[CellGroupData[{
Cell[269307, 5660, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[269729, 5674, 209, 5, 70, "Tutorials",
 CellID->38102921],
Cell[269941, 5681, 159, 3, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[CellGroupData[{
Cell[270137, 5689, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[270559, 5703, 167, 3, 70, "MoreAbout",
 CellID->179478344]
}, Open  ]],
Cell[270741, 5709, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

