(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    195901,       3988]
NotebookOptionsPosition[    190344,       3809]
NotebookOutlinePosition[    192279,       3859]
CellTagsIndexPosition[    192192,       3854]
WindowTitle->ICSourcePoint - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"2.1 Introduction to Influence Coefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"], "\
\<\"2.2 Point Singularities\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/PointSingularities"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"ICSourceConstant\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICSourceConstant"], "\<\"ICSourceLinear\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICSourceLinear"], "\<\"ICVortexPoint\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICVortexPoint"], "\<\"ICDoubletPoint\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/ref/ICDoubletPoint"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"InfluenceCoefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/InfluenceCoefficients"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/ICSourcePoint\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/ICSourcePoint"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
ICSourcePoint.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/ICSourcePoint", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["ICSourcePoint", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICSourcePoint", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes velocity influence coefficients at field \
points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     ", due to point sources at ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICPhiSourcePoint", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes velocity-potential influence coefficients at \
field points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     ", due to point sources at ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICPsiSourcePoint", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]",
     "computes stream-function influence coefficients at field points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     ", due to point sources at ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, 
   "Rows" -> {None, None, {None}}, "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->158200725],

Cell[TextData[{
 "Input arguments are collections of vectors of the data type ",
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], "InlineFormula"],
 ".  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation may be used for single vectors."
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 "The strength of a point source is denoted by ",
 Cell[BoxData[
  StyleBox["\[Sigma]", "TR"]], "InlineFormula"],
 ".  For a collection of sources, strength parameters are represented as a \
list. "
}], "Notes",
 CellID->206410608],

Cell[TextData[{
 "Output for velocity influence coefficients is a collection of vectors of \
the data type CartesianVectors.  Influence coefficients for velocity \
potential and stream function are lists.  For the special case of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation being used to specify a single field point, ",
 "output is simplified as illustrated in the examples",
 "."
}], "Notes",
 CellID->30234373],

Cell[TextData[{
 "The velocity, velocity potential and stream function for a point source are \
written as products of a geometric influence coefficient and the strength \
parameter.\t\t\t\t\t\t\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["V", "TI"], "=", 
   RowBox[{"\[Sigma]", " ", 
    StyleBox["ic", "TI"]}]}]], "InlineFormula"],
 "\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Phi]", "TR"], "=", 
   RowBox[{"\[Sigma]", " ", 
    StyleBox["ic\[Phi]", "TI"]}]}]], "InlineFormula"],
 "\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Psi]", "TR"], "=", 
   RowBox[{"\[Sigma]", " ", 
    StyleBox["ic\[Psi]", "TI"]}]}]], "InlineFormula"]
}], "Notes",
 CellID->333352942],

Cell["New in 1.0   |   Last modified in 1.2", "Notes",
 CellID->529540534]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->407763394],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->113239048],

Cell[TextData[{
 "Load ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->513831186],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->267685788],

Cell[TextData[{
 "This example demonstrates some properties of a single point source located \
at the origin.  Specify an arbitrary field point and the location of the \
singularity.  Single vectors may be entered in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation."
}], "ExampleText",
 CellID->659456463],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"rF", ",", "rP"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"x", ",", "z"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->64953589],

Cell["\<\
Compute the algebraic expressions for components of the velocity influence \
coefficient.\
\>", "ExampleText",
 CellID->326501090],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"icu", ",", "icw"}], "}"}], "=", 
  RowBox[{"ICSourcePoint", "[", 
   RowBox[{"rF", ",", "rP"}], "]"}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->560964283],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["x", 
    RowBox[{"2", " ", "\[Pi]", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"]}], ")"}]}]], ",", 
   FractionBox["z", 
    RowBox[{"2", " ", "\[Pi]", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"]}], ")"}]}]]}], "}"}]], "Output",
 ImageSize->{184, 35},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->23414792]
}, Open  ]],

Cell["Compute velocity-potential influence coefficient.", "ExampleText",
 CellID->44302699],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ic\[Phi]", "=", 
  RowBox[{"ICPhiSourcePoint", "[", 
   RowBox[{"rF", ",", "rP"}], "]"}]}]], "Input",
 CellLabel->"In[4]:=",
 CellID->575890672],

Cell[BoxData[
 FractionBox[
  RowBox[{"Log", "[", 
   RowBox[{
    SuperscriptBox["x", "2"], "+", 
    SuperscriptBox["z", "2"]}], "]"}], 
  RowBox[{"4", " ", "\[Pi]"}]]], "Output",
 ImageSize->{83, 37},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->106111450]
}, Open  ]],

Cell["Computer stream-function influence coefficient.", "ExampleText",
 CellID->472419597],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ic\[Psi]", "=", 
  RowBox[{"ICPsiSourcePoint", "[", 
   RowBox[{"rF", ",", "rP"}], "]"}]}]], "Input",
 CellLabel->"In[5]:=",
 CellID->8664355],

Cell[BoxData[
 FractionBox[
  RowBox[{"ArcTan", "[", 
   RowBox[{"x", ",", "z"}], "]"}], 
  RowBox[{"2", " ", "\[Pi]"}]]], "Output",
 ImageSize->{90, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->157697989]
}, Open  ]],

Cell[TextData[{
 "Replace the ArcTan function with a named angle (see sketch at beginning of \
tutorial  ",
 ButtonBox["2.1 Introduction to InfluenceCoefficients",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"],
 ")."
}], "ExampleText",
 CellID->53987023],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ic\[Psi]", "/.", 
  RowBox[{
   RowBox[{"ArcTan", "[", 
    RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}]}]], "Input",
 CellLabel->"In[6]:=",
 CellID->30542685],

Cell[BoxData[
 FractionBox["\[Theta]1", 
  RowBox[{"2", " ", "\[Pi]"}]]], "Output",
 ImageSize->{26, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->392975652]
}, Open  ]],

Cell["\<\
Write the velocity components, velocity potential and stream function as the \
product of the strength parameter and the geometric influence coefficients.\
\>", "ExampleText",
 CellID->9427774],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"u", ",", "w", ",", "\[Phi]", ",", "\[Psi]"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Sigma]", " ", "icu"}], ",", 
     RowBox[{"\[Sigma]", " ", "icw"}], ",", 
     RowBox[{"\[Sigma]", " ", "ic\[Phi]"}], ",", 
     RowBox[{"\[Sigma]", " ", "ic\[Psi]"}]}], "}"}]}], ";"}]], "Input",
 CellLabel->"In[7]:=",
 CellID->258291294],

Cell[TextData[{
 "Verify that the relations between velocity, velocity potential and stream \
function are satisfied.\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["u", "TI"], "=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Phi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["x", "TI"]}]], "=", 
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Psi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["z", "TI"]}]]}]}]], "InlineFormula"],
 "\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["w", "TI"], "=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Phi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["z", "TI"]}]], "=", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{"\[PartialD]", 
       StyleBox["\[Psi]", "TR"]}], 
      RowBox[{"\[PartialD]", 
       StyleBox["x", "TI"]}]]}]}]}]], "InlineFormula"],
 "\t\t\t\t\t"
}], "ExampleText",
 CellID->78226744],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"u", "\[Equal]", 
    RowBox[{"D", "[", 
     RowBox[{"\[Phi]", ",", "x"}], "]"}]}], ",", 
   RowBox[{"u", "\[Equal]", 
    RowBox[{"D", "[", 
     RowBox[{"\[Psi]", ",", "z"}], "]"}]}]}], "}"}]], "Input",
 CellLabel->"In[8]:=",
 CellID->13857323],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->394611595]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"w", "\[Equal]", 
    RowBox[{"D", "[", 
     RowBox[{"\[Phi]", ",", "z"}], "]"}]}], ",", 
   RowBox[{"w", "\[Equal]", 
    RowBox[{"-", 
     RowBox[{"D", "[", 
      RowBox[{"\[Psi]", ",", "x"}], "]"}]}]}]}], "}"}]], "Input",
 CellLabel->"In[9]:=",
 CellID->692553449],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->433802873]
}, Open  ]],

Cell["\<\
Flow visualization is shown below for a point source of unit strength located \
at the origin.  A circular region near the singularity is excluded from the \
vector plot to avoid showing vectors of large magnitude.\
\>", "ExampleText",
 CellID->78531558],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xMin", ",", "xMax", ",", "zMin", ",", "zMax"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "1.5"}], ",", "1.5", ",", 
     RowBox[{"-", "1.2"}], ",", "1.2"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"uPlot", ",", "wPlot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"u", ",", "w"}], "}"}], "/.", 
    RowBox[{"\[Sigma]", "\[Rule]", "1"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Phi]Plot", ",", "\[Psi]Plot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\[Phi]", ",", "\[Psi]"}], "}"}], "/.", 
    RowBox[{"\[Sigma]", "\[Rule]", "1"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gV", "=", 
   RowBox[{"VectorPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"uPlot", ",", "wPlot"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"RegionFunction", "\[Rule]", 
      RowBox[{"Function", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"x", ",", "z"}], "}"}], ",", 
        RowBox[{"0.03", "<", 
         RowBox[{
          RowBox[{"x", "^", "2"}], "+", 
          RowBox[{"z", "^", "2"}]}]}]}], "]"}]}], ",", 
     RowBox[{"VectorColorFunction", "->", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"RGBColor", "[", 
         RowBox[{"0", ",", "1", ",", "0"}], "]"}], "&"}], ")"}]}], ",", 
     RowBox[{"VectorPoints", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"14", ",", "14"}], "}"}]}], ",", 
     RowBox[{"VectorScale", "\[Rule]", "Medium"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Phi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Phi]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "7"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Red"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Psi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "13"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Blue"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gV", ",", "g\[Phi]", ",", "g\[Psi]", ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
   RowBox[{"FrameTicks", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.5"}], ",", "1.5", ",", "0.25"}], "]"}], "]"}], ",", 
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.2"}], ",", "1.2", ",", "0.2"}], "]"}], "]"}], ",", 
      "None", ",", "None"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", " ", "72"}]}], ",", 
   RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellLabel->"In[10]:=",
 CellID->1746043248],

Cell[BoxData[
 GraphicsBox[{{{}, {
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.004289048054484118, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.4907692307692308`, -1.1926153846153846`}, \
{-1.5092307692307694`, -1.2073846153846153`}}]}}, 
      {Arrowheads[{{0.004548525024194409, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.4896185684142185`, -1.0083571847727018`}, \
{-1.5103814315857815`, -1.0224120459965291`}}]}}, 
      {Arrowheads[{{0.004804932811407701, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.488415142184387, -0.8243530018251989}, \
{-1.5115848578156132`, -0.8371854597132626}}]}}, 
      {Arrowheads[{{0.005044529625747898, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.4872309842283888`, -0.640653347052229}, \
{-1.512769015771611, -0.6516543452554632}}]}}, 
      {Arrowheads[{{0.005249770573881166, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.486170810810811, -0.4572833264033263}, \
{-1.5138291891891893`, -0.46579359667359654`}}]}}, 
      {Arrowheads[{{0.005401385452458525, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.4853604943923095`, -0.27422039896473394`}, \
{-1.5146395056076907`, -0.27962575488141966`}}]}}, 
      {Arrowheads[{{0.005482290651234566, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.484918651261495, -0.09137960930839961}, \
{-1.515081348738505, -0.09323577530698485}}]}}, 
      {Arrowheads[{{0.005482290651234566, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.484918651261495, 
         0.09137960930839975}, {-1.515081348738505, 0.09323577530698499}}]}}, 
      {Arrowheads[{{0.005401385452458525, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.4853604943923095`, 
         0.27422039896473405`}, {-1.5146395056076907`, 
         0.2796257548814198}}]}}, 
      {Arrowheads[{{0.005249770573881166, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.486170810810811, 
         0.4572833264033264}, {-1.5138291891891893`, 
         0.46579359667359665`}}]}}, 
      {Arrowheads[{{0.005044529625747898, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.4872309842283888`, 
         0.6406533470522291}, {-1.512769015771611, 0.6516543452554633}}]}}, 
      {Arrowheads[{{0.004804932811407701, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.488415142184387, 
         0.824353001825199}, {-1.5115848578156132`, 0.8371854597132627}}]}}, 
      {Arrowheads[{{0.004548525024194409, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.4896185684142185`, 
         1.0083571847727018`}, {-1.5103814315857815`, 
         1.0224120459965291`}}]}}, 
      {Arrowheads[{{0.004289048054484118, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.4907692307692308`, 
         1.1926153846153846`}, {-1.5092307692307694`, 
         1.2073846153846153`}}]}}, 
      {Arrowheads[{{0.0047169012218768284`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.259784094418411, -1.191068598359225}, \
{-1.2786774440431274`, -1.208931401640775}}]}}, 
      {Arrowheads[{{0.005068874973481217, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2583216783216782`, -1.0066573426573426`}, \
{-1.28013986013986, -1.024111888111888}}]}}, 
      {Arrowheads[{{0.0054313033482332115`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.256705890730422, -0.8225711284780944}, \
{-1.2817556477311163`, -0.8389673330603671}}]}}, 
      {Arrowheads[{{0.005784831407447224, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2550223155683906`, -0.6389204515620898}, \
{-1.2834392228931477`, -0.6533872407456025}}]}}, 
      {Arrowheads[{{0.006100506649715703, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2534293093767546`, -0.45579247613700163`}, \
{-1.2850322290847838`, -0.4672844469399213}}]}}, 
      {Arrowheads[{{0.006342129025796517, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2521528266621356`, -0.27319698036264767`}, \
{-1.2863087117994028`, -0.280649173483506}}]}}, 
      {Arrowheads[{{0.006474227895712769, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.251433991854906, -0.09101338122581128}, \
{-1.2870275466066323`, -0.0936020033895732}}]}}, 
      {Arrowheads[{{0.006474227895712769, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.251433991854906, 
         0.09101338122581142}, {-1.2870275466066323`, 
         0.09360200338957333}}]}}, 
      {Arrowheads[{{0.006342129025796517, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2521528266621356`, 
         0.2731969803626478}, {-1.2863087117994028`, 0.2806491734835061}}]}}, 
      {Arrowheads[{{0.006100506649715703, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2534293093767546`, 
         0.45579247613700175`}, {-1.2850322290847838`, 
         0.46728444693992144`}}]}}, 
      {Arrowheads[{{0.005784831407447224, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2550223155683906`, 
         0.6389204515620899}, {-1.2834392228931477`, 0.6533872407456026}}]}}, 
      {Arrowheads[{{0.0054313033482332115`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.256705890730422, 
         0.8225711284780945}, {-1.2817556477311163`, 0.8389673330603672}}]}}, 
      {Arrowheads[{{0.005068874973481217, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2583216783216782`, 
         1.0066573426573426`}, {-1.28013986013986, 1.024111888111888}}]}}, 
      {Arrowheads[{{0.0047169012218768284`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.259784094418411, 
         1.191068598359225}, {-1.2786774440431274`, 1.208931401640775}}]}}, 
      {Arrowheads[{{0.0051917232721398155`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0290980366641183`, -1.1891799534785366`}, \
{-1.0478250402589586`, -1.2108200465214631`}}]}}, 
      {Arrowheads[{{0.00567275165995242, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0272825432583068`, -1.0044540422970112`}, \
{-1.0496405336647698`, -1.0263151884722195`}}]}}, 
      {Arrowheads[{{0.006195291634254835, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.025128205128205, -0.8201025641025641}, \
{-1.0517948717948715`, -0.8414358974358974}}]}}, 
      {Arrowheads[{{0.006736282984742615, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0226979215160061`, -0.6363453733877371}, \
{-1.0542251554070705`, -0.6559623189199549}}]}}, 
      {Arrowheads[{{0.007250038641562075, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0202017446471054`, -0.4534229976209357}, \
{-1.0567213322759714`, -0.4696539254559872}}]}}, 
      {Arrowheads[{{0.007665957668748538, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0180466007022022`, -0.2714790935205872}, \
{-1.0588764762208744`, -0.2823670603255664}}]}}, 
      {Arrowheads[{{0.007902685346588587, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.016766291033807, -0.09037922586967166}, \
{-1.0601567858892698`, -0.09423615874571281}}]}}, 
      {Arrowheads[{{0.007902685346588587, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.016766291033807, 
         0.09037922586967179}, {-1.0601567858892698`, 
         0.09423615874571294}}]}}, 
      {Arrowheads[{{0.007665957668748538, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0180466007022022`, 
         0.2714790935205873}, {-1.0588764762208744`, 0.2823670603255665}}]}}, 
      {Arrowheads[{{0.007250038641562075, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0202017446471054`, 
         0.4534229976209358}, {-1.0567213322759714`, 
         0.46965392545598733`}}]}}, 
      {Arrowheads[{{0.006736282984742615, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0226979215160061`, 
         0.6363453733877372}, {-1.0542251554070705`, 0.655962318919955}}]}}, 
      {Arrowheads[{{0.006195291634254835, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.025128205128205, 
         0.8201025641025642}, {-1.0517948717948715`, 0.8414358974358975}}]}}, 
      {Arrowheads[{{0.00567275165995242, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0272825432583068`, 
         1.0044540422970112`}, {-1.0496405336647698`, 
         1.0263151884722195`}}]}}, 
      {Arrowheads[{{0.0051917232721398155`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0290980366641183`, 
         1.1891799534785366`}, {-1.0478250402589586`, 
         1.2108200465214631`}}]}}, 
      {Arrowheads[{{0.005695805863805785, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7989267184838575, -1.1869768388903028`}, \
{-0.8164578969007575, -1.213023161109697}}]}}, 
      {Arrowheads[{{0.006350149426125166, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7967970213905042, -1.001687684033777}, \
{-0.8185875939941106, -1.0290815467354535`}}]}}, 
      {Arrowheads[{{0.007110657655641629, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7940310621548223, -0.8167176639306746}, \
{-0.8213535532297926, -0.8448207976077868}}]}}, 
      {Arrowheads[{{0.007965374958327644, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7905494505494504, -0.6324395604395604}, \
{-0.8248351648351646, -0.6598681318681318}}]}}, 
      {Arrowheads[{{0.008856650564581726, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7864984615384614, -0.44942769230769225`}, \
{-0.8288861538461536, -0.47364923076923066`}}]}}, 
      {Arrowheads[{{0.009649270221388083, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7825352587514748, -0.2682978030005056}, \
{-0.8328493566331402, -0.285548350845648}}]}}, 
      {Arrowheads[{{0.010134686102653232`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7799404946383188, -0.08913605653009353}, \
{-0.8354441207462961, -0.09547932808529094}}]}}, 
      {Arrowheads[{{0.010134686102653232`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7799404946383188, 
         0.08913605653009365}, {-0.8354441207462961, 0.09547932808529107}}]}}, 
      {Arrowheads[{{0.009649270221388083, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7825352587514748, 
         0.2682978030005057}, {-0.8328493566331402, 0.2855483508456481}}]}}, 
      {Arrowheads[{{0.008856650564581726, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7864984615384614, 
         0.44942769230769236`}, {-0.8288861538461536, 
         0.47364923076923077`}}]}}, 
      {Arrowheads[{{0.007965374958327644, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7905494505494504, 
         0.6324395604395605}, {-0.8248351648351646, 0.6598681318681319}}]}}, 
      {Arrowheads[{{0.007110657655641629, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7940310621548223, 
         0.8167176639306747}, {-0.8213535532297926, 0.8448207976077869}}]}}, 
      {Arrowheads[{{0.006350149426125166, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7967970213905042, 
         1.001687684033777}, {-0.8185875939941106, 1.0290815467354535`}}]}}, 
      {Arrowheads[{{0.005695805863805785, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7989267184838575, 
         1.1869768388903028`}, {-0.8164578969007575, 1.213023161109697}}]}}, 
      {Arrowheads[{{0.006187843129446413, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5695334704346418, -1.1846296185040552`}, \
{-0.5843126834115117, -1.2153703814959447`}}]}}, 
      {Arrowheads[{{0.007054908793579613, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5673174541194843, -0.9984787192502929}, \
{-0.586528699726669, -1.0322905115189378`}}]}}, 
      {Arrowheads[{{0.008145776882472604, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5641172466263562, -0.8123288351419533}, \
{-0.5897289072197971, -0.8492096263965082}}]}}, 
      {Arrowheads[{{0.009511317872054843, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5594638860075337, -0.626599552328438}, \
{-0.5943822678386196, -0.6657081399792542}}]}}, 
      {Arrowheads[{{0.011151524941658724`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5529230769230766, -0.44233846153846146`}, \
{-0.6009230769230767, -0.48073846153846145`}}]}}, 
      {Arrowheads[{{0.012874578596638549`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.544933480044013, -0.2615680704211263}, \
{-0.6089126738021403, -0.2922780834250274}}]}}, 
      {Arrowheads[{{0.01410156059771489, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5385455418216727, -0.08616728669146759}, \
{-0.6153006120244807, -0.09844809792391687}}]}}, 
      {Arrowheads[{{0.01410156059771489, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5385455418216727, 
         0.08616728669146773}, {-0.6153006120244807, 0.09844809792391702}}]}}, 
      {Arrowheads[{{0.012874578596638549`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.544933480044013, 
         0.26156807042112634`}, {-0.6089126738021403, 0.2922780834250275}}]}}, 
      {Arrowheads[{{0.011151524941658717`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5529230769230766, 
         0.44233846153846157`}, {-0.6009230769230767, 0.4807384615384615}}]}}, 
      {Arrowheads[{{0.009511317872054843, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5594638860075337, 
         0.6265995523284381}, {-0.5943822678386196, 0.6657081399792543}}]}}, 
      {Arrowheads[{{0.008145776882472604, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5641172466263562, 
         0.8123288351419534}, {-0.5897289072197971, 0.8492096263965083}}]}}, 
      {Arrowheads[{{0.007054908793579613, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5673174541194843, 
         0.9984787192502929}, {-0.586528699726669, 1.0322905115189378`}}]}}, 
      {Arrowheads[{{0.006187843129446413, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5695334704346418, 
         1.1846296185040552`}, {-0.5843126834115117, 1.2153703814959447`}}]}}, 
      {Arrowheads[{{0.006596848447238295, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.34111458360690156`, -1.1825305565039261`}, \
{-0.35119310870079024`, -1.2174694434960736`}}]}}, 
      {Arrowheads[{{0.007680134209140241, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.33932367493681675`, -0.9953494464813298}, \
{-0.35298401737087504`, -1.035419784287901}}]}}, 
      {Arrowheads[{{0.009154435851060492, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3364497041420116, -0.8074792899408284}, \
{-0.3558579881656802, -0.8540591715976331}}]}}, 
      {Arrowheads[{{0.011239593633760895`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3315255012579093, -0.6188476023480979}, \
{-0.3607821910497824, -0.6734600899595944}}]}}, 
      {Arrowheads[{{0.014280919927654349`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3225378461538459, -0.4300504615384615}, \
{-0.3697698461538458, -0.4930264615384614}}]}}, 
      {Arrowheads[{{0.01858587490276452, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3061538461538459, -0.24492307692307685`}, \
{-0.38615384615384585`, -0.3089230769230768}}]}}, 
      {Arrowheads[{{0.022997873006245586`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.28490903287583763`, -0.07597574210022337}, \
{-0.40739865943185416`, -0.10863964251516109`}}]}}, 
      {Arrowheads[{{0.02299787300624558, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.28490903287583763`, 
         0.07597574210022348}, {-0.4073986594318541, 
         0.10863964251516126`}}]}}, 
      {Arrowheads[{{0.018585874902764504`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.30615384615384594`, 
         0.24492307692307697`}, {-0.3861538461538458, 0.3089230769230769}}]}}, 
      {Arrowheads[{{0.014280919927654349`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3225378461538459, 
         0.4300504615384616}, {-0.3697698461538458, 0.4930264615384615}}]}}, 
      {Arrowheads[{{0.011239593633760895`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3315255012579093, 
         0.618847602348098}, {-0.3607821910497824, 0.6734600899595945}}]}}, 
      {Arrowheads[{{0.009154435851060492, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3364497041420116, 
         0.8074792899408285}, {-0.3558579881656802, 0.8540591715976332}}]}}, 
      {Arrowheads[{{0.007680134209140241, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.33932367493681675`, 
         0.9953494464813298}, {-0.35298401737087504`, 1.035419784287901}}]}}, 
      {Arrowheads[{{0.006596848447238295, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.34111458360690156`, 
         1.1825305565039261`}, {-0.35119310870079024`, 
         1.2174694434960736`}}]}}, 
      {Arrowheads[{{0.006834306076848681, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.11358175719480197`, -1.1812502748259435`}, \
{-0.1171874735744282, -1.2187497251740562`}}]}}, 
      {Arrowheads[{{0.008062271070084457, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.11287569136625711`, -0.9933060840230651}, \
{-0.11789353940297306`, -1.0374631467461657`}}]}}, 
      {Arrowheads[{{0.009823014473758215, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.11166016421126158`, -0.8039531823210854}, \
{-0.11910906655796859`, -0.857585279217376}}]}}, 
      {Arrowheads[{{0.012552260161081737`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.10930303318436789`, -0.6120969858324617}, \
{-0.12146619758486228`, -0.6802107064752305}}]}}, 
      {Arrowheads[{{0.017318159203737377`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.10380814479637983`, -0.4152325791855203}, \
{-0.12696108597285036`, -0.5078443438914026}}]}}, 
      {Arrowheads[{{0.027463307553181465`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.08627218934911221, -0.20705325443786976`}, \
{-0.14449704142011796`, -0.3467928994082839}}]}}}, 
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.02746330755318145, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.08627218934911224, 
         0.2070532544378699}, {-0.14449704142011793`, 
         0.34679289940828395`}}]}}, 
      {Arrowheads[{{0.017318159203737346`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.10380814479637984`, 
         0.4152325791855205}, {-0.12696108597285033`, 0.5078443438914026}}]}}, 
      {Arrowheads[{{0.012552260161081737`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.10930303318436789`, 
         0.6120969858324619}, {-0.12146619758486228`, 0.6802107064752306}}]}}, 
      {Arrowheads[{{0.009823014473758215, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.11166016421126158`, 
         0.8039531823210855}, {-0.11910906655796859`, 0.8575852792173762}}]}}, 
      {Arrowheads[{{0.008062271070084457, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.11287569136625711`, 
         0.9933060840230651}, {-0.11789353940297306`, 
         1.0374631467461657`}}]}}, 
      {Arrowheads[{{0.006834306076848681, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.11358175719480197`, 
         1.1812502748259435`}, {-0.1171874735744282, 1.2187497251740562`}}]}}, 
      {Arrowheads[{{0.006834306076848681, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.11358175719480258`, -1.1812502748259435`}, {
         0.11718747357442882`, -1.2187497251740562`}}]}}, 
      {Arrowheads[{{0.008062271070084396, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.11287569136625772`, -0.9933060840230652}, {
         0.11789353940297369`, -1.0374631467461655`}}]}}, 
      {Arrowheads[{{0.009823014473758194, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.11166016421126217`, -0.8039531823210854}, {
         0.11910906655796921`, -0.8575852792173759}}]}}, 
      {Arrowheads[{{0.012552260161081739`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.10930303318436847`, -0.6120969858324617}, {
         0.12146619758486292`, -0.6802107064752305}}]}}, 
      {Arrowheads[{{0.017318159203737353`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.1038081447963804, -0.41523257918552037`}, {
         0.126961085972851, -0.5078443438914025}}]}}, 
      {Arrowheads[{{0.027463307553181448`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.08627218934911271, -0.20705325443786987`}, {
         0.14449704142011868`, -0.3467928994082838}}]}}}, 
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.027463307553181427`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.08627218934911274, 0.20705325443787}, {
         0.14449704142011865`, 0.34679289940828384`}}]}}, 
      {Arrowheads[{{0.017318159203737353`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.1038081447963804, 0.4152325791855205}, {
         0.126961085972851, 0.5078443438914026}}]}}, 
      {Arrowheads[{{0.012552260161081739`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.10930303318436847`, 0.6120969858324619}, {
         0.12146619758486292`, 0.6802107064752306}}]}}, 
      {Arrowheads[{{0.009823014473758194, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.11166016421126217`, 0.8039531823210855}, {
         0.11910906655796921`, 0.857585279217376}}]}}, 
      {Arrowheads[{{0.008062271070084396, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.11287569136625772`, 0.9933060840230652}, {
         0.11789353940297369`, 1.0374631467461655`}}]}}, 
      {Arrowheads[{{0.006834306076848681, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.11358175719480258`, 1.1812502748259435`}, {
         0.11718747357442882`, 1.2187497251740562`}}]}}, 
      {Arrowheads[{{0.006596848447238295, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3411145836069021, -1.1825305565039261`}, {
         0.3511931087007908, -1.2174694434960736`}}]}}, 
      {Arrowheads[{{0.007680134209140241, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3393236749368173, -0.9953494464813298}, {
         0.3529840173708756, -1.035419784287901}}]}}, 
      {Arrowheads[{{0.009154435851060476, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.33644970414201214`, -0.8074792899408284}, {
         0.3558579881656808, -0.854059171597633}}]}}, 
      {Arrowheads[{{0.011239593633760883`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3315255012579099, -0.6188476023480979}, {
         0.36078219104978304`, -0.6734600899595943}}]}}, 
      {Arrowheads[{{0.014280919927654344`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3225378461538465, -0.43005046153846155`}, {
         0.36976984615384645`, -0.49302646153846136`}}]}}, 
      {Arrowheads[{{0.01858587490276449, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.30615384615384655`, -0.2449230769230769}, {
         0.3861538461538464, -0.30892307692307674`}}]}}, 
      {Arrowheads[{{0.02299787300624554, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.2849090328758383, -0.07597574210022341}, {
         0.4073986594318546, -0.10863964251516105`}}]}}, 
      {Arrowheads[{{0.022997873006245544`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.28490903287583835`, 0.07597574210022354}, {
         0.40739865943185466`, 0.1086396425151612}}]}}, 
      {Arrowheads[{{0.01858587490276449, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.30615384615384655`, 0.24492307692307702`}, {
         0.3861538461538464, 0.30892307692307686`}}]}}, 
      {Arrowheads[{{0.014280919927654344`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3225378461538465, 0.43005046153846166`}, {
         0.36976984615384645`, 0.49302646153846147`}}]}}, 
      {Arrowheads[{{0.011239593633760883`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3315255012579099, 0.618847602348098}, {
         0.36078219104978304`, 0.6734600899595944}}]}}, 
      {Arrowheads[{{0.009154435851060473, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3364497041420122, 0.8074792899408285}, {
         0.3558579881656808, 0.8540591715976331}}]}}, 
      {Arrowheads[{{0.007680134209140241, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3393236749368173, 0.9953494464813298}, {
         0.3529840173708756, 1.035419784287901}}]}}, 
      {Arrowheads[{{0.006596848447238295, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3411145836069021, 1.1825305565039261`}, {
         0.3511931087007908, 1.2174694434960736`}}]}}, 
      {Arrowheads[{{0.006187843129446413, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5695334704346423, -1.1846296185040552`}, {
         0.5843126834115122, -1.2153703814959447`}}]}}, 
      {Arrowheads[{{0.007054908793579613, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.567317454119485, -0.9984787192502929}, {
         0.5865286997266697, -1.0322905115189378`}}]}}, 
      {Arrowheads[{{0.008145776882472588, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5641172466263569, -0.8123288351419533}, {
         0.5897289072197978, -0.8492096263965081}}]}}, 
      {Arrowheads[{{0.00951131787205483, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5594638860075344, -0.626599552328438}, {
         0.5943822678386202, -0.665708139979254}}]}}, 
      {Arrowheads[{{0.011151524941658695`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5529230769230774, -0.4423384615384615}, {
         0.6009230769230773, -0.4807384615384614}}]}}, 
      {Arrowheads[{{0.012874578596638543`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5449334800440137, -0.2615680704211263}, {
         0.608912673802141, -0.29227808342502737`}}]}}, 
      {Arrowheads[{{0.01410156059771487, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5385455418216734, -0.0861672866914676}, {
         0.6153006120244813, -0.09844809792391686}}]}}, 
      {Arrowheads[{{0.01410156059771487, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5385455418216734, 0.08616728669146773}, {
         0.6153006120244813, 0.09844809792391701}}]}}, 
      {Arrowheads[{{0.012874578596638543`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5449334800440137, 0.2615680704211264}, {
         0.608912673802141, 0.2922780834250275}}]}}, 
      {Arrowheads[{{0.011151524941658695`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5529230769230774, 0.4423384615384616}, {
         0.6009230769230773, 0.4807384615384615}}]}}, 
      {Arrowheads[{{0.009511317872054843, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5594638860075344, 0.6265995523284381}, {
         0.5943822678386202, 0.6657081399792543}}]}}, 
      {Arrowheads[{{0.008145776882472604, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5641172466263569, 0.8123288351419534}, {
         0.5897289072197978, 0.8492096263965083}}]}}, 
      {Arrowheads[{{0.007054908793579613, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.567317454119485, 0.9984787192502929}, {
         0.5865286997266697, 1.0322905115189378`}}]}}, 
      {Arrowheads[{{0.006187843129446413, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5695334704346423, 1.1846296185040552`}, {
         0.5843126834115122, 1.2153703814959447`}}]}}, 
      {Arrowheads[{{0.005695805863805785, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7989267184838582, -1.1869768388903028`}, {
         0.8164578969007582, -1.213023161109697}}]}}, 
      {Arrowheads[{{0.006350149426125166, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7967970213905049, -1.001687684033777}, {
         0.8185875939941113, -1.0290815467354535`}}]}}, 
      {Arrowheads[{{0.0071106576556416, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.794031062154823, -0.8167176639306746}, {
         0.8213535532297932, -0.8448207976077867}}]}}, 
      {Arrowheads[{{0.007965374958327644, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.790549450549451, -0.6324395604395604}, {
         0.8248351648351653, -0.6598681318681318}}]}}, 
      {Arrowheads[{{0.008856650564581726, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.786498461538462, -0.44942769230769225`}, {
         0.8288861538461543, -0.47364923076923066`}}]}}, 
      {Arrowheads[{{0.009649270221388064, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7825352587514754, -0.26829780300050565`}, {
         0.8328493566331407, -0.28554835084564806`}}]}}, 
      {Arrowheads[{{0.010134686102653232`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7799404946383195, -0.08913605653009353}, {
         0.8354441207462968, -0.09547932808529093}}]}}, 
      {Arrowheads[{{0.010134686102653232`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7799404946383195, 0.08913605653009367}, {
         0.8354441207462968, 0.09547932808529108}}]}}, 
      {Arrowheads[{{0.009649270221388064, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7825352587514754, 0.26829780300050576`}, {
         0.8328493566331407, 0.28554835084564817`}}]}}, 
      {Arrowheads[{{0.008856650564581726, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.786498461538462, 0.44942769230769236`}, {
         0.8288861538461543, 0.47364923076923077`}}]}}, 
      {Arrowheads[{{0.007965374958327644, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.790549450549451, 0.6324395604395605}, {
         0.8248351648351653, 0.6598681318681319}}]}}, 
      {Arrowheads[{{0.0071106576556416, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.794031062154823, 0.8167176639306747}, {
         0.8213535532297932, 0.8448207976077868}}]}}, 
      {Arrowheads[{{0.006350149426125166, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7967970213905049, 1.001687684033777}, {
         0.8185875939941113, 1.0290815467354535`}}]}}, 
      {Arrowheads[{{0.005695805863805785, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7989267184838582, 1.1869768388903028`}, {
         0.8164578969007582, 1.213023161109697}}]}}, 
      {Arrowheads[{{0.005191723272139785, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.029098036664119, -1.1891799534785368`}, {
         1.0478250402589593`, -1.2108200465214631`}}]}}, 
      {Arrowheads[{{0.00567275165995242, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0272825432583075`, -1.0044540422970112`}, {
         1.0496405336647705`, -1.0263151884722195`}}]}}, 
      {Arrowheads[{{0.006195291634254835, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0251282051282056`, -0.8201025641025641}, {
         1.0517948717948722`, -0.8414358974358974}}]}}, 
      {Arrowheads[{{0.006736282984742615, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0226979215160068`, -0.6363453733877371}, {
         1.0542251554070712`, -0.6559623189199549}}]}}, 
      {Arrowheads[{{0.007250038641562069, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0202017446471061`, -0.4534229976209357}, {
         1.056721332275972, -0.46965392545598716`}}]}}, 
      {Arrowheads[{{0.007665957668748538, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0180466007022029`, -0.2714790935205872}, {
         1.058876476220875, -0.2823670603255664}}]}}, 
      {Arrowheads[{{0.007902685346588587, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0167662910338076`, -0.09037922586967166}, {
         1.0601567858892704`, -0.09423615874571281}}]}}, 
      {Arrowheads[{{0.007902685346588587, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0167662910338076`, 0.0903792258696718}, {
         1.0601567858892704`, 0.09423615874571295}}]}}, 
      {Arrowheads[{{0.007665957668748538, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0180466007022029`, 0.2714790935205873}, {
         1.058876476220875, 0.2823670603255665}}]}}, 
      {Arrowheads[{{0.007250038641562075, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0202017446471061`, 0.4534229976209358}, {
         1.056721332275972, 0.46965392545598733`}}]}}, 
      {Arrowheads[{{0.006736282984742615, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0226979215160068`, 0.6363453733877372}, {
         1.0542251554070712`, 0.655962318919955}}]}}, 
      {Arrowheads[{{0.006195291634254835, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0251282051282056`, 0.8201025641025642}, {
         1.0517948717948722`, 0.8414358974358975}}]}}, 
      {Arrowheads[{{0.00567275165995242, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0272825432583075`, 1.0044540422970112`}, {
         1.0496405336647705`, 1.0263151884722195`}}]}}, 
      {Arrowheads[{{0.005191723272139785, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.029098036664119, 1.1891799534785368`}, {
         1.0478250402589593`, 1.2108200465214631`}}]}}, 
      {Arrowheads[{{0.0047169012218768284`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2597840944184118`, -1.191068598359225}, {
         1.278677444043128, -1.208931401640775}}]}}, 
      {Arrowheads[{{0.005068874973481217, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2583216783216788`, -1.0066573426573426`}, {
         1.2801398601398606`, -1.024111888111888}}]}}, 
      {Arrowheads[{{0.005431303348233201, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2567058907304227`, -0.8225711284780944}, {
         1.281755647731117, -0.838967333060367}}]}}, 
      {Arrowheads[{{0.005784831407447224, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2550223155683913`, -0.6389204515620898}, {
         1.2834392228931484`, -0.6533872407456025}}]}}, 
      {Arrowheads[{{0.006100506649715703, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2534293093767552`, -0.45579247613700163`}, {
         1.2850322290847844`, -0.4672844469399213}}]}}, 
      {Arrowheads[{{0.006342129025796517, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2521528266621362`, -0.27319698036264767`}, {
         1.2863087117994034`, -0.280649173483506}}]}}, 
      {Arrowheads[{{0.006474227895712769, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2514339918549067`, -0.09101338122581128}, {
         1.287027546606633, -0.09360200338957318}}]}}, 
      {Arrowheads[{{0.006474227895712769, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2514339918549067`, 0.09101338122581142}, {
         1.287027546606633, 0.09360200338957333}}]}}, 
      {Arrowheads[{{0.006342129025796517, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2521528266621362`, 0.2731969803626478}, {
         1.2863087117994034`, 0.2806491734835061}}]}}, 
      {Arrowheads[{{0.006100506649715703, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2534293093767552`, 0.45579247613700175`}, {
         1.2850322290847844`, 0.46728444693992144`}}]}}, 
      {Arrowheads[{{0.005784831407447224, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2550223155683913`, 0.6389204515620899}, {
         1.2834392228931484`, 0.6533872407456026}}]}}, 
      {Arrowheads[{{0.005431303348233201, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2567058907304227`, 0.8225711284780945}, {
         1.281755647731117, 0.8389673330603671}}]}}, 
      {Arrowheads[{{0.005068874973481217, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2583216783216788`, 1.0066573426573426`}, {
         1.2801398601398606`, 1.024111888111888}}]}}, 
      {Arrowheads[{{0.0047169012218768284`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2597840944184118`, 1.191068598359225}, {
         1.278677444043128, 1.208931401640775}}]}}, 
      {Arrowheads[{{0.004289048054484118, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4907692307692315`, -1.1926153846153846`}, {
         1.50923076923077, -1.2073846153846153`}}]}}, 
      {Arrowheads[{{0.004548525024194409, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4896185684142191`, -1.0083571847727018`}, {
         1.5103814315857822`, -1.0224120459965291`}}]}}, 
      {Arrowheads[{{0.004804932811407701, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4884151421843876`, -0.8243530018251989}, {
         1.511584857815614, -0.8371854597132626}}]}}, 
      {Arrowheads[{{0.005044529625747898, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4872309842283895`, -0.640653347052229}, {
         1.5127690157716116`, -0.6516543452554632}}]}}, 
      {Arrowheads[{{0.005249770573881166, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4861708108108116`, -0.4572833264033263}, {
         1.51382918918919, -0.46579359667359654`}}]}}, 
      {Arrowheads[{{0.005401385452458525, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4853604943923102`, -0.274220398964734}, {
         1.5146395056076913`, -0.2796257548814197}}]}}, 
      {Arrowheads[{{0.005482290651234566, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4849186512614956`, -0.09137960930839961}, {
         1.5150813487385058`, -0.09323577530698485}}]}}, 
      {Arrowheads[{{0.005482290651234566, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4849186512614956`, 0.09137960930839975}, {
         1.5150813487385058`, 0.09323577530698499}}]}}, 
      {Arrowheads[{{0.005401385452458525, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4853604943923102`, 0.27422039896473405`}, {
         1.5146395056076913`, 0.2796257548814198}}]}}, 
      {Arrowheads[{{0.005249770573881166, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4861708108108116`, 0.4572833264033264}, {
         1.51382918918919, 0.46579359667359665`}}]}}, 
      {Arrowheads[{{0.005044529625747898, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4872309842283895`, 0.6406533470522291}, {
         1.5127690157716116`, 0.6516543452554633}}]}}, 
      {Arrowheads[{{0.004804932811407701, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4884151421843876`, 0.824353001825199}, {
         1.511584857815614, 0.8371854597132627}}]}}, 
      {Arrowheads[{{0.004548525024194409, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4896185684142191`, 1.0083571847727018`}, {
         1.5103814315857822`, 1.0224120459965291`}}]}}, 
      {Arrowheads[{{0.004289048054484118, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4907692307692315`, 1.1926153846153846`}, {
         1.50923076923077, 1.2073846153846153`}}]}}}}}, 
   GraphicsComplexBox[CompressedData["
1:eJx0nXtAlEXU/8nM7K6ZmWY3M83MrMzKbqfSSkuttHuWpl3N7G6ZlZiZmVaW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    "], {{{}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV0j1I1HEcx/HfDUJQllq0JJmUOUVi6eADZQ0K1eIaBjq4pZk62NADlA5q
YU9ehZlnnVnnIg4aWSGC4GC2qIMFBl3aUugc+voPb97/z/fh97v/3eU3NNc2
xUIIH1C5N4SKfSGUI54TwjP04wWe406mnB3CbY7zxz0hlJmd4hX9hP1yDGFZ
nlQv1Z/gJXlQ/YbdHrsd3MtH1PNQbO4hP0JfdKfZx1zIT7Hl+QkXmTse9fBN
bdTZJ9SSvIHpLJ8LV51/wMy6mf183V2Nas3cgmvYpVdgt4vvmunke5yndhh9
zhtzVgrvcdl+vb0e9dfyMBKoVf+HtP2/fN7MRa7iC3yO/3AJV/EpPsunOc0F
XMpHuYSP8XfO4SLO4pOczcv8yz0rfMvdN5HvORa9n8/VLrehFQfVN/mK+jzG
vdsXtWn7dfKCPG5v3cwG0tit/zt6X14zl+Km6PvET6zhkt0ZvNJL2h/iEc7g
eWfG+Uf0/+F3cj+n5Go7c3LC7DAWnRXT+8qBH+CtuTPmPqNXXpXv87bczW/k
Cs+fMOCMl1iw/19u4EE5aWYEh+RcVOrNurfGb1qNHSqLbP0=
          "]], LineBox[CompressedData["
1:eJwV0V9ojXEcx/HvsiaLzcbtDsMVzjC7IY1cuMDNNn92WNG2s50Ltras/Jc/
N1jYysSFHReuTUrZSY4oduOGlKIIUYy2C5Qar+fi3ef7+fw+v+/zPD217b3N
PSURUUBrRcTmyohJ2jg3IjU/oga3qiLymJF18C3VETuwCzvxQz+D3eYMWjE4
L6JN9tO8l05jSPaOH6al6LKrGzlssfuXzjY6ptNuvqnTQe/y2+V/zKOyHJ8z
Z2k3uvCXv+MdP9n1GR9Rrnsf48m7uj8lW8D30X5MyNowyzdnaVHvkD3l/Bw8
t28C/3QXuTdDF9OQvaaVNE0raB19i6d2DNi1RNbAn3fnAi7ipLxevlF+ib+M
Kzgt3yTfKr/Kj+Aazslb5E3yUT6f/Itkl/yZ5/yWZ7BPZw/dT0u9/0HayR+g
WdpDZ+uX4b3zD6jGQr6KfrEvjyN6R/HAM8ZRwG35C72UXq9dx5yfwHGk5F/l
YzovzSvNj1BM/pnslSxtPuzeMvNa81JaTyed38OIPaecFz3rMdYnZzrr6DcU
dN7wG8xPkDbXYXnyPKzAd/lDvRt2nbXrOj1DVztbg1Vo1JnSafZfm/Af4g1n
9g==
          "]]},
        "0.046`"],
       Annotation[#, 0.046, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1FtM12Ucx/FH0KJCIci8KQVTBGykS1PXTZ5Ku2iKUZvKQcPYsGmbB9rM
bnSlXigKWpZopW6KAU1EzAuVtgqtzSOFh4l5Fp1nZznLXs+6ePN5Pp/v5/n9
n9//9+OfOX12/qwuIYQifxK6h1CaGsLmp0NYbV2bFsKo5BCOyo7hCH5HG0bL
f+oRws+o1O0jy8Br8n0YnRLCDbMprjEVkyP8LdzEGJ12/eP4AydxAmPl23TX
uGYiZsgK+SIU2ldM79AilFgX03u4i/H21stKrGvsnUYb+Dfl960n0B38+9a1
5mW0iZ8of2BdQH/gP7B+hH+xQ++crImmYbbzfIhW3SlIcJ+ldK/OHP0k/nH8
4jts1f9H93n6kPambWjRLbenu86L/CKzxfF7lc1FhjwTn8mWoF22EGdkf6ID
r3peKekhVJlX45T5Ikw0y0eNbD3OypahUFaELbKtqEWlvNVZEp17unOUm79H
Z9IkWTez07QDqUjnU+hF+zagQu8jNLvWrnifZp06z9ELsYd+uCp7Ic7s2YJD
fH9+vs/Kpdf4nNgza8ARfiBfYf4SzYsdeQsdjEzzQbQvvUKbzVY7xyf61XQh
HS4fFt81sxy9vc5XZbYK/fkb8h/NX0EWn40B6OR3m43wDHfSafaV4Gv5cNkw
rNG7Hj+T7jFb6ZqVGKLzmGcylH7sDMtlC+gK2qQ/1N7ttM2+9TpD+JexzjrZ
vhqajS/j+6yzNt6v+ef0KfMvaB+sxG3zyvjdmC+nT5qvoLl8Dr7xOb/qLJWV
OcMcZ5iHuXhCnqWzlC7DEnwav1dZBlbZ+737akA9ptr/Ny7p/UXfdY1J9G36
DgrQiTcwDmPwOsbiMi7iEi4gz3ubRbMxAKfwLHohHT3xDNrQFd2QgL7x94Du
p+ed4wCtcLb56GV9Jz6neM9odC8tsj3x94E/yDfae1XnGjqRbF6PZp1J8bcM
dXp1sp2yfP43fiN/ht9Ev+O/pdv5t+Jn8V/xHfw6ui0+O1rHj4v/W/x+VMu2
ykbGdxibnKVKdjj1/9/c/wD0Zcm+
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1FtMznEcx/FfZiUr5LApORSbUYYbpws2d04rljksTA55YpVic2gk5FSb
w4UL5oqKlIqNzWnDbMJcZGzIIQzjxqZNDpvXfy7evb/fz+/7e36/53n+TxkF
JYuK40IIqf50DgghhvMpIVxAXXIIr/XxXMSNsouo17/RJ/BGbpI1o0H/Vt+H
N/HqgSHcSwqhgHv6hbAWa9TroprPR6/LRfoYx/gPLkevy9dkefb/VS/nG/pi
9S3rcf1DKFEXy0q5hG/LK7hMv513Ygfizd53vx53+oWfSDf7mzu4w9pTfspZ
XCnfgyqUO3eEfCT266vxXFaBSbLJqJHV4oWsEjNls3BMdhwvZVXIkeXilOw0
3skOIV+21Ll1svrojvpe3kMPx3Ehr7a+nmPYgN7eV6e1M/a3q5PUNzFQ3Y9T
OJnfY5O9afxFlsqf7TmLx/ph0Wdo/QOvcvZKZKhTrd3i4ZzJaZwezeOTvtn+
w+5xBAex1Wsc4ib5Ad6i38/lXB09N/J9XKav4s28lxvle7hUv4srsTt6/uQT
fW/1/MSZJ5w5CkfVxznfPVcg28wQfS2+W6vhLNlhThzkftwXY2ULzefioP6I
2UQ+gAfqk87JNHOCP6LNXGt0hnsVuE+tbHb03ei79Yt5KefxMv6Gr5iPBZiD
eZiLL5iOGZiCaZiKLmRjAsZFzx7GoxPDkI6hSEMqnuE5EjDGufE8mtv5IY9z
v0L33MHbsQ3d+BH9FuWPcMfcdfN3+QYP5it8jTPMZUbv22fSgqvRrM9mSfSs
6Fv1Xfo2btZfilC32N/CrZzI53DZXI59d9EgfzXg//+Xf6NqnaU=
          "]]},
        RowBox[{"-", "0.046`"}]],
       Annotation[#, -0.046, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl09tPz3Ecx/FPNTd+9ftVliIuLMxdwwU3Pxs3cuGwCgllkVOnzSgbZWOG
Dd0wGeb0F5BfJ+XUwYVsxpY53DhdMMVVzIU8vnPx3PPzfr3fn8O+v5pX01Ta
mBZC+IbTiRDKMIKn6M0KoTsnhD7u4Swezw4hzhO8JTOEn5xQ/+Bs7jPXz718
n884J4fLOWbmT66z7cu0DrKGeAh1sgbUY8BsCx/GQTTjEKZFb9IbdO4whlCo
PuKco2jFPufGzR3gQXtanJ0b3Y+35t/hpfVfvSX2zlUv5UuyLzzGYUYISeti
vRV8RfaVP3C6Xol1Um8NX5dN8CfO0FtvvU5vA9+UjeM7xrzjNQJ+60/pf/bG
dtSqd6g7vf8e0r0tA1PyNA54b981s5v1NyEmm45+eYKzkYk4svBRftt8t7t7
kEKdO7p4q/2VKDAzarYAXer9+s3uzJfny2YiFX1TeV5Uyyui3zx6p3P2yu9w
J+7ilvsW8WXeaKYc7fadR8zes3wOpfIydJgrMn+BS9SrcUL/OKqdvdNbqngX
/8Ik5phv029zXisf40LZbJx0zkpnNFk3Rn9XWGv/DU7K9/Bu1GKV/Covl1dz
FbZjmbyDF8u3Rd8KlVEtv8jF8nr3vELK3W/kT7yjTv1C/cwbR6P3mpulNxl9
Z34ke4wF6oWoMf8cQ3oD8od4gDx1irtQZG4+KsyNYNj9pxL//0f/Aavvfvo=

          "]]},
        RowBox[{"-", "0.092`"}]],
       Annotation[#, -0.092, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl0klIVXEUx/EjZBn0fA4NJLQIikKjRTaS1MIomgyExHKTuqkW1stqUZto
gGhy1SK1ds1lrSJwFxRZvmcvskWBSLSohQ2QFjjU59Liy/ec3/kP917uwpZD
9W0FEfEUF9MRu9GH8bKIgVkRr9SXcBkNSJVETJi9MStWT5Nl9EdwGJniiHZ+
kYooNT+uT/MxLuGU9W9LI/Lm2aTmHFdyI+/BO2dnkFaf5DX2rcYJZ8yRzcaQ
Mz7bM6j+67716uWyGu6UfeFhLiiP2KyuMdvCN2Qj/Cm5x3mDGNP/Mh9N9rjv
PG6a30I7jmIMU8k6nuRR/mDvLs9Vh9/66fo/3MtTzpvB4zyJCXzUdzv7kTWP
8QA9eIgu+T2+j9u4izvolC/iq7zTPTtwxjnnvHMhb9UXecfT6rOybfrtWGDP
Jq5FGU6Zl3ODZ9/rWZrwNflO+KGez9/4O0bw07p5srnYaN8GHFQfwH7Uml/n
dfIWbsY+rJVf45XyVXjvuZ+4e0i+Ql+d/BOyvGzAuivowEx3FmGxdZXWFHqn
XPJvyKr0y5C3L6vvl7+2J8v9XMHP5M95iXVL0WptDi/NL6T//9v/ABPAbbY=

          "]]},
        RowBox[{"-", "0.138`"}]],
       Annotation[#, -0.138, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl0stLlFEYx/HHdWo6k+C2Ta2SEFLUcmUXFYzauG/VpsncBG7KVTvJEsEI
F13AP0Doskxs22RCM5MmMpqQ4bUyr/U5tPjyfc45z/N73/fMnLxx+3quIiLG
8aEKtREfkUdjJuJWZcQv/ns84hFO63nIp3iYR1CVjXjMeezjrpku7sSCvk/8
qjriNbrV5+Q14V1NxBR29Ddb7zorooTPWDFb4Dl8QYd1O7+R8xZX1fVYtb/C
W7zE37CMpzLr5B+qj9K5b9rEtr6sd16zt45NbGBcfxe60YlrnnGfB3FCzqD1
PfWo9y2oiziQddHZJQzZL2NL1jZeoNd5P790foef4xke6FvAmP0n+J5mvNtP
/avqnnRv8n/wBc9sx7S+99gzdzndF7dZT6vP85L5MlrV8+nu8RWtKYNbeEZm
mWvSnfIsL3IPH2BX1oDcszL6+I/1Dn6j13nGvdWiZKZeT9HzCuk/k75R9gRy
5q7ov8kNes4gr79Sz7H0O2LG2WT1///cP0e4bOg=
          "]]},
        RowBox[{"-", "0.184`"}]],
       Annotation[#, -0.184, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl0slO1EEQgPFCcUMcURKJ3LzpWQ8KJsyAo6IzMILLXblJwIsLIq6AywMY
EzUuEIh3Ly6AC3H3AdBojA9giKgxCgm/DocvX1d1VXX3f2bD4e62rrKIeIgt
ayPymYjNvINXronYuCqigjdxJf+vjPhYFVFeHXFV7pbce/EHtOsp6N3PRW7j
7OqIHEasH6Bbf17cw/fVDOEeRu3VmrEe38w9ruYETqJMLjCjpyndDY1o0ZPn
7+qr1Z0ST4t7eCcW6dnFi7mB/6nJ8hHuwJR55/Dc+gWe4bJZL/mdntP8OX0T
uXrvzMp1ylV5+1G+hFJ6Szobv80a5D88wBl1/XxD39e0h1fm9blDKd3R3hke
luvlt+re4DWuOPNm+t48qe4a96Z69OGg+hZ9h7iVD/C+9M1Rwl3cwbD8MvO+
OHtOvNT6mP6ifIO4kPbdc7l8TpzFL7V1XI9t2KtmO+9JPXr/2v+UfjvxbpTr
beYlvFW+glvT+6xXWNdgHW6Lx/SP4ykGvWmCJ+094QHxY+7n8zybfh++iAu4
jrP4KV/kQvru1o+40Xk/zG/iXGbhvzwPt6NeTg==
          "]]},
        RowBox[{"-", "0.22999999999999998`"}]],
       Annotation[#, -0.22999999999999998`, "Tooltip"]& ], {}, {}}}], 
   GraphicsComplexBox[CompressedData["
1:eJx0nXdcU9f//6N1r1q1aq1atda9Z109rrrFuusAN04cVFG0aETAXXEW66KK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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV03loz3Ecx/HPTOQem9vGaiOUP1DIHyjH0iYm5NaY26YdmdrKXZQjZw05
NqEImc2UY8TKhGyuRlEm1NyMJTz2x3PP7+v9+bw+3+/vt+8vPi0zNSMihJDs
z8D2IWzqHEJRVAj5XNozhASzyWjtug3WdA/hmPWibiG0lQvsK+NEe/K7hJAj
/44NIZdnyCnmWa6zcVgvWz8XOehtba09EfrNENAckZhuvkrnouue9q2R87DY
7JxZtNlELJIb3C+d9zp/qXP38TKOsZ6r86uHZ0ID5tvXyKnmUzFXnoddOt/N
57jO8tmy0V4/CelN90Uf3DWvwj2M6OocLuSDSEYKJuGK8457huv8yJ5qhA4h
ZLhHBM9xVl/Umr/ASwzX2+OzFekVo8bsMXrHhfCEK50V7fqzfZ34E2fihnl/
vROe/bJ95djo7FLzBOtlvEnOct3CvS95hjh7WzWdza35tj1DrA/FH7MK+0vw
3P7XZh+bPhNqUInlvpdmzumAH561P/dCjP218juU4D4q0NguhPHWp2IEVmEB
Euz/EeMe/ofv+J9z73ONXMtpnquCK+USrnfOOj4rb+cq/ZN8RS7kr/qL+aic
zdP0Z/IeOZlf6XfjtXIiX9MfxVvkwVyn/zc6hJVyGzlJ/4s8X37D1frneZx8
lS/oP+VU+S4/09/Pw+UiHqm/lQfK+XxHP4k7ytO4WD+T4+Q0rtIfxM3lkTxI
P54bevluuVz/VSfvi1zPB/Rbmn+QG+Vr+nf4oVzN8frlfEs+w2f08/i0vJm3
6R/hUnk3X9Cfx4fkFRyjP4V3ymP5iH4U58qxXKA/jDfIA7hY/2dHv0M5Um6p
/16eLb/g3fqneIx8iVfrP+BJ8k0+oL+Dh8oHudH6eu4n5/Bm/dHcTk7hhdaX
cA95Fm/T78tBHsL11rvzN7+Ptpyn/8wzPZfreLr1f/xW/soF+lHe/4++/1ie
0PR+eU9LsA6F2I6b+A/828UK
          "]]},
        "0.42600000000000005`"],
       Annotation[#, 0.42600000000000005`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV1H9IllcYxvFTLRk1s7L8sRS0pvsBMraxFtIqVluMGpUZWGMbCm3lmlZz
2srUoG3JtM1Aw0YG1TQoSHADZZNpm1JBDQpS3MBBhgUKM8rNRbrP88f1fs91
3/d1znmf99H0guKcomkhhBwfGxJDeDchhG1UtzCE7kUhvDonhCp+nvV8Opoc
QuvcEGrxYkRzv6i/bG4zfc0/Tg2hBi/oV8p+Zf1I7Qhu5b/ETpkXzVfwsdaV
9quiCjob5cx0qD9rZhPlmatQa1fL5A/ypfx/9i3DpqQQTlKTbKk9UsysoyK9
mTLFGION+jusW60T9LfYJ5emng7hI/Uye+yjhXql6v/Qv1RCx2Tf9ozq8KH5
cSox+xnFmW/Aeqqyd4b5Put+mhEXQprcE7gbs/Qm1acouOsbasv0vpH7NnrG
tDz6DTAdF9Nv1p84Y8zsZecX4k7aQT/xLfb5FZvxEr5ufgV9rj/onAT7L7Eu
pwO0n9p93z04Uy/fGU+aK4jOkn8TG+k47VH/Hh+Z78I26pf5S3aUOugm9UZ3
csdpnssceuAuz+Eiijc/wA9TG12jLpqIdZZ+Di2jXfQBLTH/YIEz/L7DOGnf
a3iTH8B8d+zCXr4NR+xTiReidw+vyjdjB9+IY/Lb8RS/F3Pl8/AYvw4H5RNx
H/8MdsqvwC/4l3BI/nF8CB/zs/i18n/z7/O38Yb8RVzD/4yt8rdwE38F++Tr
8TX+NGbLH8EX+APYI78W5/K5eEa+CFP5fLwqn4Uz+GzMkk/D8RTPFtvlB+eH
8Cc/gg3yMer3+Am+U74Hf+dvYJp8O17iz+N5+TI8xx/Gavkm/JGvw1b59/A7
vhDj5TfiUX41NsnHYQmfguXyS/EQ/zyekX84z98YP52Pkb/Lb+P/wDr5FlzF
/4DF8tfxHb4bG+Rr8RX+BE7oV2Em/ykell+JT/HrsUD/Q0zmt2K1fAZORf+r
cEQ/Ce/zs7FMvs+d+vkh3KI/iXf4MSyXj/P+j3r+KfhW9H55T9uokhqphrrp
f4j4yzM=
          "]]},
        "0.35500000000000004`"],
       Annotation[#, 0.35500000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV03loz2EAx/FnjlzZmGObYXOFWTkK+QPlWDlybIRCI+Scq1wrd9mcc8+K
kB3mmEZtZRplolAUQlFoaMoxamHy+v3x2fv3fp7n8zzP77vvr8fC1elZUSGE
Cf4Miw5hphzpFEJcYghHcWfnEHZIB34kwZgcksOSJ+2MH7Rur89X24WQg1cw
F3ON/+1mDkvjQ7goJeZ2m0t1TgkvliIpNL7DeD/j6VJo7IJsNrZJko1NkU32
aunMzZjtXlukBV9lTZaskAJ7rcRl1pTEhXCSL+cnMKZ7CO3tm2nuT5cQFkTu
iD3skSzx1s92zi1rqiL3NT/Y+E3MxRTn7bTmk7nP0sfnATEhfPS5VnKs2SNR
1jflTbAZ1ssg61KsH4g/+He5Z36M9Y3YkZ/DJH428j9w1wSJlzKeZv4adsdL
kXksxQR8LI8kxXe553sOwFS54rzLssRYkbsXyzjJlxOyJvKc8bd9qrFcXrjf
W50vUilPpUaWei7Bnm2l3vPsi10k1vqXvFbK5aFUS0NbZ5mfLsNlhcyXntbX
d3SGd6MWG+37EJ/yl5jprtVYw8uxzj5b8RLfhw/0C7GS5+M3/UV4hq/FDP1Z
eJhPxDf6nXEj74VV+iNxNx+E7/X/dvCO8FY8Tf8rn8ff4RP9qziW38Qy/Wc4
jd/H5/rHcBg/hyP092B/vgXv6qdhDM/A8/qrsCvPxAf6qdiEj8BU/ST81dWz
xQr9N7EhvOZ1eFy/ufFPvIFX6d/Fx/wJJulX4B1eiqX6G7CY78Ic/dN4g+dh
mf5cLODLMFZ/Kh7gY/C0fjSu54mYrT8Ut/N+eF7/Z3vvGI/izfU/8jn8Febp
F+Fofh2z9B/hZH4bj+vvxyH8FDaY34Z9+DrcpT8K2/BJuMD8YoznszFHvzf+
87sZjHXm4/A7b40b9J+70wv+HmeYb8QP/Btm60d7/794/ok4PvJ+eU/LZavk
yz65Lf8BMiLESw==
          "]]},
        "0.28400000000000003`"],
       Annotation[#, 0.28400000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV1HtszWccx/Fv1cyttVovekxpCUsaGWMmy7ZkRsosI50gcakYGbKtLiMZ
akvFJC4hcaqV1oJtlqJdWkKClISErBjJCBJsxJZUMovLmmWK1/nj3ff5fJ/n
8/wuPW3h3PLSL9IiosSPZE5En74R1fkRNUhiO6qQbZ6ZF/El5+dGLOME9zTr
gRGZEVPwc5+IRmzS2YhhZg3yAexH/UsR68yHmh93xjrX/Mms0mwvr+VvzCp4
j7ySV2MVutnfybXSkeHah+RC53T43I8/RJ1r1GKx/eXIMyuzfzZmocaZ9bzC
no5ExFNsl6twq5dn45uc5G3Ykergd7N051X7XMc78R1q8YKzsgsicpGDdPe/
1LnL8KK1Lnhdv1h3OD+WH6GTbhreMtts7Wm/iGbdNz1Xf+sDcFAu4fEoQiEa
Ur+n1DvlBP+Kx/qtnm0/t+NffCqf924uoAS/IM37qOJyne/5P2e0cBOuuI+b
evdwBJdwOnWOzjP32dPzPODBnI8s+6/Kd9GEVrSgPSPifeuTMQqLMAuF9j/I
dg3PeZefOLeVL8lXucz9tvBpuYnbnFPB++QNfFb/Bz4iV/N9/U94p7yYS/Wn
8lZ5At/Qz+EVchEf03+b18qv8W39/1+OWCh3lcfp/y3PlP/gi/oNPEY+yo36
v/Ek+Qxf1t/Gb8i7eLT+t/yq/BWf0h/HmXIp79b/jPvKZXxWv5jT5NFcrF/A
j17xbvmw/o3eEdflNk7qdzb/S26Xj+mf4vPyRS7QP8wn5Xqu11/Oe+VKXq9f
xwflLdyoP4N3yAs4S/8j3iS/x3X6GbxUTvBK/ZH8tTyEd+s/zIqYL4fcWf9P
ebp8jbfo/8jvys38uf45niif4KT+Rh4u13C79TU8SF7ClfrvcHf5A55jfR7n
ydN4vf5A7vD3MozbrOfyP3I3Xq5/2T1dkW/zx9af8B35fup/i36G7/897z/B
Y1PfL9/TJlSgGhtwAs8BFa7BsA==
          "]]},
        "0.21300000000000002`"],
       Annotation[#, 0.21300000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV03tIl1ccx/Ez3DK3vLSpaeXogrs4aI01pJzpKmQNlhm0QRtt5JarhubQ
lSPdDHK0QUZWdoNsK7CLoEFBhhYJi6mQY4oFBjlqoDDDBtKqub2ePz6+z+d7
vp/vc57H85u7oWxN6VMhhHX+ZCeEcG1WCAdSQjifFMLh9BDa8AimqqdQ4owQ
vsZtVEmzUkPYjntl0qxjXwxhBsbrq1dLtb5qf5HZz6q9gZ3RLPWzZteZfQa/
x0T1BIrTtxXLqYyS9X6FO82bYi/LjMt8knqtWiK286+qP21/Hp5IC6GJjkcy
v1zfwwxzcTI6Jz3jrFNxi9ojTJdrMyfGerNaifUXVGXGN5Rmfx8/HQ/zh6iR
ZuvP8dwlVOxZuXyc2Uvxc75XJs/6M+se6zlmd1o3eecT1GfGb1RmbtdMZ8RX
9NdF74T55uZRZmIIq/UtUSvEHCzCxVhgf8j+p9b/JHtH/l8qpQ7PyjLrNWrX
f5nyrPOp2n4NfRt9Nxq2d4e+s77knJUYa25xdH7qNqsAf6XgnPuxVP9P+FB/
B7bSgMxt2VG6SH3URRtlJr3zc95zHDMxjZL0D/J3qZW6qYMm4kNYZr+Q3qJN
9DHN0T/uPW/7n97Fx+Z2Yx8/iOudswO7+FYcMWcHnuZ343X5n/EifxDH5Dfg
Mb4Mi+TXYj3/Lg7JJ2MlPxfb5XNwJ78Ah+UfveC+8LH8Cvm/+I/4O3hD/hy+
w1/CFvnfcRX/C/ZHdwsX8ccxW34Xvsxvx2vyKzCeL8Im+S04k1+P1+WzMPDZ
mCWfgX/P9m3xgvzQ8yHc4kewQT5G/U9+gm+PfvvYy9/ADPkLeIVvxmb5CjzF
12Kd/FE8z+/BFvl1eIgvwST59/FHPh+Pyk/Dcj4dq+TfxBr+JWySfzDdHeP/
wxj5e/ghfxP3yJ/EXL4Nv5Tvwff4TmyQ/wEX8o04Yb8a5/NbsVb+bYzjV+In
0Z3GVP4DrJOfh0/8Jl7HEfspeJ+fihXy/c40wA/jGvuP8Q9+DKvkp7n/o75/
Oi6P7pd72ko76CDtpk76H3ENxrA=
          "]]},
        "0.14200000000000002`"],
       Annotation[#, 0.14200000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV03tozWEcx/Fni7kMYzOGszG38gcKKbeYEcplhEJzmbtdXCaWEUKhNiHU
yD0TMmShRlGWURRCkUuoKYSMGOZ1/nif9+/zPM/n+/t1fuek5yyfUhATQsjy
0adVCE3bh7AhJYT1KEZSuxAKOoWwJTmEJvbayGlpISRysbU8e/nIRStrjZ1Z
5nqSWUv5uBlHW4ewztlKuZv14EwMWjgfZ1Y8N6AZUuXmfEhnZYcQDvIqTtP7
Y70jXzInznWemfXcCBG9xjzf2kL7i7AAsdZ+Rdddj9PN4TLPtNfcHGfPy4nW
65z5gRjMsxb4O0Y6m4kMjMYo7NI95flLuZw7WWvp/q3NibhORYPZc835xxu5
u1mxCSFkc0b0O9RLxTBkYpC9OdbHuh6D83rd5Nu6BebWW/uDfOQhF1Xuf4ZP
OZvsnuXcjl8i2bx0e0VYizW4Ep1pr9D1apSYX4oac3byHVRH30VL3w/nO3uM
f+pVcQUem/tC9wMq8QC3sNAz/vWemncM4Qt35/ZIcP6J/BYVqEEV6twjw/5E
DMASzEJn57+0dY9UHf5tbg0/kJ9wtmet4ltyBdeaU8yn5e1crX+cK+V9/El/
Hh+UCzhLfyqXymP4uX4Sr5a78DX9wbxZ7s2v9X8lhbBYjpMz9T/KM+VXfF//
LI+Qr/I5/Yc8Qb7Nj/R3c3/5MA/U38o95bV8Uz+TW8hZfER/GXeQs7lavxc3
RHS5l36Ev8kJfFn/eWIIz+Ra3qMfa/29XCdf07/J9+T7HNG/zDfkci7XL+ST
8ibepl/GF+USPqc/gw/IizhBfzzvlIdzmX48r5BTuEi/H2+Qe/AR/a9t/P/k
fxyr/46ny0+5RP8ED5UvcK7+XR4nX+c9+ju4r7yf6+yv567yct6kP4SbymN5
tv0cTpan8Tb9dK73P+rDtfbb8me5CRfqP/JMj+XXPNn+b34jf+Ii/Xi//w/e
RwqPjP6+/E4rUIx92I7r+A+80r+2
          "]]},
        "0.07100000000000001`"],
       Annotation[#, 0.07100000000000001, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV03loz3Ecx/HP3DbmtjHGQvlj5haGMcwxDHONuTdyF0WhHJtQFBpFEeYo
ashcjdrKcisWikKoKZNrCbke++P5e/Z6fz6v7/fz+/2+34QFqyavjAgh9PHR
ODqERW1CWIyFyEUOGpnP5wWYg3mYiyjzbJ6NLMzCTESaT+cZmIJpmIqG5pM5
ExmYhIloYD6eJ2AsxiEd9c1H8xiMxCikoZ75cB6BoUjFMNQ1H8IpSMZgDEId
8xMxIZzEcRQiMt73QhRqW+8WF0ISEtED3bHc/FKrEP61D+EyFyPZfBD6YyAG
oJZ9WzkPW7AN+bhgf1/uh17og95YZv/t2BDu4KuzVOMbrjUN4V7rEO5jJ26h
HH8bh7CPl9t/lL+7bgkXoaJJCC9036MYD1GGHPf4XfObtw3hE3fi1oi2/4n8
BkW4jRJUu8dQ6+PRG4sxE/H2f2rpHn6DN/yz5uz8UH7C2c5cwmVyEVe6zno+
LW/ncv1jXCwXcJX+PD4kr+AM/UzeLafxc/3mvEbuwFf1B/BmOZFf6f9o4TmV
68qp+h/kLPklP9A/wynyFT6r/4jHyTf5sf4e7iUf5r76edxFXsul+qkcJWfw
Ef0lHCtnc7l+V/7bTpe76sfxFzmaL+o/bx7CM7mS9+pHmL+Tq+Wr+qV8V37A
cfoX+YZ8ik/pr+ZCeRPn6x/k8/IuPqs/gw/IuRytn8475SF8UD+SV8kxvE6/
J2+UO/MR/c/NvMvyH47Qf8tT5ae8S/84J8vneKn+HR4tX+e9+js4Sd7P1dY3
cIK8kjfpD+T68iiebX0+t5SncL5+R/7lPejGldZb8Ee5Hq/Wf+xMFfIrnmj9
J7+Wq3idfqTn/73/I4aH1TxfntMirEcBtuM6/gOp5q6k
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV1H1I1lcYxvETbFGBpbVsmdpeguiNNhnTttasVYOytjTTVqTVFkal6VjQ
gmIJPTLdarRBW0UWGK3hgoQKDSySWcO2aRZozGGRbr24MWmrpbDP74/L7++6
z33d5zzP7/g8v64ku3hICOE9f7oSQ3h9ZAhrnw2hkAoofWwIPyWFsCM+hE3W
s+NCqJ4Qwmz1LP66tRvURu10jYrV8/RNHR/CNJpCu+Tny7xFmfZYomeh539S
QliAW/kCmW/NXsLfNGe//T+RS5Pv5DtonuwafSf0ZemLWS+VXaf2nVqO2rv8
73r/oDvUSz1Upr5BX7p5GfQqfSq/QiaX5pu9TE++5wf687APP1Qrkls8LoRF
FO9coyifVtJnZhxVr6ZMvf30Ny0wL1ftoDkpzvYNpmJLtBd2jgrhR8zQm04r
zMqjHMql5TRbPplu6t2Ln9NV+Z9pSKp3gyXOuJWa9f9AR3yuc84Ub72D76Qa
tYtqCWqP+f+id2vmTPMGPQ/QE7oSvSNniqlXUKVzVVGTegWm6YlFngZ9J1/g
pmhPfKinHmupzexO+V6qi85MF2i97+SJswxz3j58AcdSnP52vptqqZnqqd8e
b1rPojTaQCspRX/fM/Zwf7rxkbnNeJVvx1U+az1e4Guxx5zteJzfg03y1VjH
78d78gX4Nb8Zl8pnY1V0P7FDPgHL+FQ8K5+Bu/hp2CX/75gQPuCf4ufK3+Xz
+V+xJbrbOIc/gyflf8HF/CVsld+LL/OH8BX53TiJ/wgb5efiCH4pHpYvwnH8
KmySn4yDybI4WT4J/+Lj8LR8x2j/q3wP7pMP6rf5fv6sfCNe4VswSf40nudr
sEa+FI/xO7Fc/gCe4ivxpHwefsW/j3Hyi7CCfwMPyA/HYj4Rt8m/hDv4F/Gw
/J8Jfnv4AQzyt3A5fx0r5Y/ia/z3uFH+Mr7NN+A++RjO4L/Efusf43P8Ftwp
PwuH8gtxtfVCHMPnYLn8RHzs7k/HHuuj8T7/NJbKtzpTG9+F71h/hL/x93Cb
/HD3v9f7SIx+46L75Z7W0vbot4z2UAP9D0F+0sI=
          "]]},
        RowBox[{"-", "0.07100000000000001`"}]],
       Annotation[#, -0.07100000000000001, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV1Htsj3cUx/GzzKKk7lq6YUHSNra1SrC5bxht06JkttYwc6nL3EKChGQS
XWKJChKCYImhSUkIlVpCogldVLRMQoIYq8R9azYWk3Sv549P38/nnPM5z/f3
/J5f+85dXrLsrYiY4k+/tIgXvSP6Y3nniHYZEYuwPf76bkQ9ladHjO8Qsfu9
iExz+Xw2/imXhYv4ZTI5rlvUPsLhHSMmyuyRyeVX6ve0cxVm4BV7G2ixbJG5
/eaGmCvgC6lY7YDaULX1Mk1mR7huxOE4yv6pZrLt2qhf2TNiG32m948zfIrL
7Zlp5rA9BXwxP5m+VjuiVqhWIfvAzj/oPpWojbP7m+RMPZyBptF06mB/x+Qe
dpTiU/OpOI4vw7LkXDiaRtJs95hFf+lP6uT+ajnJ58VcHGrnEBpGg/iUPhGD
cWDyvNXyZerl8/iX3SMWqi2g+XTKubP0sqnGvjPUZK7VXI5+OMs1vsJ1jj3p
2IPq1L/CvOQMVGGmjt74vNtwsdp+/NtcLVZTk/wtmYd0khroPM31nF57/m09
h2fYF7snz8T8df4eVdNFqqUW9xitX0h5NJ++pF7mnzn3Ld/bPXxl70Vs4K9j
qc9ai+f5amy2Zy3+zG/GOvkDeJLfjo/lZ+FufgkWyU/FH/nxeFO+M67ke2ON
/DDcyA/AO/Ivu0XM49/mx8o/4mfwt/Gy/FEcxZ/GKvmrWMBfwEb5rTiQ34uD
5b/H/vxqPCc/FtvxRbhPfiGm86VYJ5+Jb3rJYmbym8EXfCqekL/ZNeIG34yV
8q14n2/BGvlzWM9fxgz5E/gLfwgPya/An/gNuEl+Fx7nt2CV/Be4k/8WU+Xz
8Qd+JO6ST8Hv+DRcI5+L6/l+uE/+eZeIOfx/2Kr/O07jf8Mt8gfxE/4Ylutf
ws/5s1gpX4Ef8juwRX8dvs8vxQ3yH+M7/AQs05+NXfkS3CTfB//12/gAm/W7
4BO+Da6Qb0z+p/B3sFj/Fd7lH+Ma+RTv/0PfRxqOSd4v72k1raXttJnO0v9B
scC7
          "]]},
        RowBox[{"-", "0.14200000000000002`"}]],
       Annotation[#, -0.14200000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV03loz3Ecx/GPc2oxs40x5pz7mJscc4coVsbINSLnlLucGUWORlGWIxQ1
aitqxPCrUZQJZYVQU45RSzZX8vj+8dyz1/vzeX2++31+31/nvPzs9Q1CCJP9
Gdc8hCZtQ2iKxmiGODxsF8LK1iFMsH4qLYSMlBCmyiNahDDRLMWe1tiVGsJu
jDSfYt7GbEPLEPaZFWA/ppkXOSPTGZutpduzhTvyELMM3i534x7ojkrPX+t5
6zDOnio5i7M8J8d5mfYMRKHzj2O8+QLzM20801mzeDaykWf9nf57LJWXIMf+
RJ7LpdZLcBflyDW7x7c97zN/QQ3S7L9g1sVn6Yau+JvsIq01RAP0Swghyb6+
/E3+inrUYYjZTz5sPaR7hrP6RvcU/c/oiV6Imfcxv8/9uKfeUK6I7oqHYxjO
yIOjczFPL5P7R3fuzmL44z6O8apoL3+3v4yL8dSZVTofUIrHKMdSn/1X9D64
qxruxEmIt/+Z/BbFqEAZaj1jjPXpyMRyzEWa/TXupqqDDtc5t4Ify88413dU
xuVyMVc7Zytfkgs4pn+WS+VC/qS/kE/Jq3lG9B3zIXkiv9RP4A1ye76hP4x3
yr34tf6PpBCWyQ3lLP2Pco78ih/pX+bR8nW+ov+Ep8n3uVL/CA+QT/Mg/T3c
Rd7Id/SzuJk8g4v0V3CKnMsx/Qz+016XM/RT+ZsczyX6L1uF8EKu5qP6//i9
XMs39O/wQ/kRp+qX8C35Il/Uz+fz8g7eq3+Sr8kH+Yr+HD4h53G8/lQ+II/i
k/pxvFZO5k36/Xm73JmL9L8mhrBY/s3/rL/jbPk5H9Q/xyPkq7zS+gOeLN/k
o/r7uY98nGutb+N0eQ3v0B/OjeVJPN/6Ik6UZ/Ne/Q5c7zfXm6utt+TPciPO
16/0mZ7Kr3mm9Tp+I3/iTfpx3v8Pvo9kHhu9X97TYmxFIQpwE/8BQ5a5FQ==

          "]]},
        RowBox[{"-", "0.21300000000000002`"}]],
       Annotation[#, -0.21300000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV1Hloz3Ecx/EPQk22+f3G2OFc5j4SylWuWJSRM0eOyLnlyLXJmGNEjXJM
jpqiRlHUkDkKZeUY2coxas7QRsiReOyP157f5/vzeX32+35/363D/OxJWQ1C
CCP86JMYwpNWIcxpHcJsmSVPWoZQZVYpd5JCiG0TQpw0lxYSL+XmafZFXS+P
DyGCKzABu5t/SQ2hB2ab5Tlzi2yVsc1DSLVnjfkOvlMKZLx5R/NOsovvliGx
IWSa1/gcU7A42bUz3/G3UuUz9LS/l/SQPGf2xgx7munHtg0h36yf2TY86l4z
rVXrTcSX+CLO/Zgf5G2cfwiT8CGOxgostL5P9kuSbDTPkQ2ySXLld0IIK533
B//Z88bv7+zsiOvDzki2pwhTsBa7Wi/DfOu1ruvq97tOl+vm3fg1HG5/unM6
8ys8DbtIuhzlEYzKNL2ZMsYzW+ZZLa5f94y+2lOKJfLAOVX2vJbzUi5lMlfn
p2fU2Of/iO0wIjH2V/BqKZFbUip1zh9iPUN6ywKZKkn2f3T/Vb77avzu3FtY
zitwuu+gFMt4CdY4Zy2e5Pl4U/8YnueF+F5/Fh7kS3Cc/gTcxUdgpX4sZvNk
vKjfH3N5F3ym/y0awnzegA/Tf8en8Kd4V/8UDuYX8LT+PRzLb+B9/T3Yixdh
X/3N2IGvwqv6w7ApH4dH9BdiAp+ON/XT8HeKLqbpJ+JnHoPn9CsjITzmNbhX
/y++4nV4Uf8q3uF3MVH/HF7mxVisn4UneA7m6R/As7wAT+tPxv18Hsboj8Ht
fBAe0G+Cy3gUV+v3xPW8PR7R/9TC/wb+C/9af4kT+SMs0D+OA/kZXGT9No7i
l3Cv/jbsxvdhnfV1mMqXYo7+AGzER+IM67Mxnmdinn4K/vA30BVrrMfhB94Q
s/Tvu6cH/BmOt/4dn/P3uFq/iff/te8jikPr3y/vaYmslULJl0vyH4FVu4c=

          "]]},
        RowBox[{"-", "0.28400000000000003`"}]],
       Annotation[#, -0.28400000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV1H1MV2UYxvFHp27MFEHEBIKca4ZR842mFX+QCxEwHWEkylLEZTRs2bDN
Vi5qQ5QiN3Sx1M0SNnSTDTZo0003YNP5EkXiK5Y400GLGjRfZq3P+eP6fc/1
3Pd1n/Oc3zlndtn7hVvHhRAK/fw8I4S+xBBmY25yCL9gDvbh8sirdSeF0ENP
zAqhYloIk/E97NezX0/2lBCu6HsNG/nL1tPpqrUrOD7VMc6jAvWVtILyKY+u
6TtnfrK5KZREH5p/Xf/BqM/cG3oG6Ja1Ceb9hoP8Ehx7KoSl+IlM3ZMhfElf
UZHcHT1r8Yg5d6M90T1q4kusD6vn8uv5YVyL66Ia/al2w3UNUKZr2mX+YqzF
IWvDtMl5ymmzWaN8iRmlURYbcB8tnBnCAvrLvJKpIZQ5fs5xLe6mPZRNi2Ld
D7PyaQVl6CnAVEyjLrNK5R/pvZ8QwkN8QOXU7pqeUf/eedtk2ulHmX/1Zaj/
h738Z47TnScBl+iZgYdk3sQSyjG/wl7eUftW/99qHdhCF+X69dymVjpLJ+lt
mfvuywT7H8JUjKMY/b38ALVQF3XQiPkvq+fSC1RGa2iW/iHX2e//HMAxc7vw
LN+LxfbYgSf5Fhw0pwq/46vxtPwBbOXr8a78OtzHb8E8+ddxF5+Nl+Sn4FY+
CdvlF+PH/Fy8Jj86PYSNfOCz5H/ni/ireEa+CV/i27BZ/jwu50/hBfk9+Dz/
Dc6X/xSf5j/AE/JZOInPw0b5cpzOF+Np+Tn4MEUW58gn4h98DB6XvxTvHeYH
sU7+Mf7Kj2C7/Ans4c9govxx/IE/jIflK/EQvwN3yjfgMb4Gm+XfwL38BoyR
z8Ev+KXYID8RK/h43CafgR/xadgoPxznmeUf4GP1m7ia/wlr5A/ii/xR3Kze
jcv4TqyT/xzT+a9xRH07pvDv4g75TBzPv4pvqa/HWH4V7pRPxn+8F8/ioPpU
vMePw0r5C/Z0MfoGYb76WPTdib4VuE1+ouf/tv8jHl+Jni/PaQtVUT1VUyf9
D11hzec=
          "]]},
        RowBox[{"-", "0.35500000000000004`"}]],
       Annotation[#, -0.35500000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV03loz3Ecx/GPUPvDMMyxmesP5EjOzE0hymaHOXNtrnKWO1YoylxFkSOE
Gpkioygyx4YJRTnm2MzYMMeE8geP7x+vPb/Pz/vz+myf376/jnOXpS+tF0JI
86NPfAh1SSH0xd7yQg4nhjA+NoSXLUNIxY5tQljTNIQOuBZf2ZPQLoQyfGPP
a6zP32A5fysZesedM9TaBp281iGk8GGNnYmV9lSYTfScLmnRjL+XRGdV4kd7
qqXK8wnzKc6s4bPxDE+xvtnZ+5ydwTMlSybJ56hrPtlzDU7BVPxiPVv/bLSH
Z0ZrUsBzrH81X4CD3DXP2cm4A78nhPBDvspU+7/haczVS9K54W8okunu175V
CDMwG7tE97T+TiplmLVWTZytt0O2ywB7dmI/HG1ean6X9+d/WoRwW++W3JRC
f0tns5N+98XIpdjznWi/Xk/9AVjMS6zPdEY3Psi+ezwZ7+NgPIrDcYTM0JuH
i9x9Ph7U+2ZeiPnywBlP7amQc1IiV2Wme/7yGdX3eVRjEjaVGPsf8jLJlyIp
lNroszUfKz1ljmRKa/ur3fWpd7EM65xbhCX8IWa5dyFe5flY7pyVeJxvwuv6
h/Ac34VV+tNwH1+A4/Qn4DY+Ap/oN8IlvA1e0O+L63lnfK7/o7l3jf/DIfrv
MYM/w2L9k5jMz+Mp/fs4hl/DUv3t2IPvx176G7E9X45X9IdgQz4OD+hnYzOe
hdf1O+GftrrYST8eP/EYLNB/0iyEx7wc8/T/4mteixf0r+AtXozx+gV4mR/D
Y/qL8Qhfh7n6e/EM34qn9NNxD5+FMfqjcQsfiHv1G+AiHocr9Lvjat4OD+jX
xHmv+G/8a/4KU/kj3Kp/GPvz05hjfhNH8UuYp78Zu/LdWGu+ChP5Qlyn3w/r
8ZE42Xw6NuYpmKufgD99D7pguXksfuABF+uXutMD/hzHm9fhC16FK/QbeP8r
/D/iou9L9H55T/NlpeySTXJJ/gPhhM/f
          "]]},
        RowBox[{"-", "0.42600000000000005`"}]],
       Annotation[#, -0.42600000000000005`, "Tooltip"]& ], {}, {}}}]},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1.5, 
      FormBox[
       RowBox[{"-", "1.5`"}], TraditionalForm]}, {-1.25, 
      FormBox[
       RowBox[{"-", "1.25`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.75, 
      FormBox[
       RowBox[{"-", "0.75`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.25, 
      FormBox[
       RowBox[{"-", "0.25`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.25, 
      FormBox["0.25`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.75, 
      FormBox["0.75`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}, {1.25, 
      FormBox["1.25`", TraditionalForm]}, {1.5, 
      FormBox["1.5`", TraditionalForm]}}, {{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2, 
      FormBox["1.2`", TraditionalForm]}}, None, None},
  ImageMargins->0.,
  ImageSize->504,
  Method->{"TransparentPolygonMesh" -> True},
  PlotRange->{{-1.6536749816983884`, 
   1.6536749816983884`}, {-1.3536749816983884`, 1.3536749816983884`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 419},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[10]=",
 CellID->303345165]
}, Open  ]],

Cell[TextData[{
 "Arrows for velocity field are green, equipotential lines are red, and \
streamlines are blue.  Note orthogonality of equipotential lines and \
streamlines, streamlines everywhere parallel to the velocity field and \
outward flow associated with positive ",
 Cell[BoxData[
  StyleBox["\[Sigma]", "TR"]], "InlineFormula"],
 ".  The thickened streamline along negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis suggests a branch cut for the stream function."
}], "ExampleText",
 CellID->680409942],

Cell[TextData[{
 "Look at the properties of the stream function as the negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis is crossed in a counterclockwise direction."
}], "ExampleText",
 CellID->635547329],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ic\[Psi]BranchJump", "=", 
  RowBox[{
   RowBox[{"Limit", "[", 
    RowBox[{"ic\[Psi]", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"x", "<", "0"}], "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", 
      RowBox[{"-", "1"}]}]}], "]"}], "-", 
   RowBox[{"Limit", "[", 
    RowBox[{"ic\[Psi]", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"x", "<", "0"}], "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}]], "Input",
 CellLabel->"In[11]:=",
 CellID->81019929],

Cell[BoxData["1"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->279799763]
}, Open  ]],

Cell[TextData[{
 "There is a branch cut along the negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis for the stream function.  This abrupt change in value of the stream \
function as the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis is crossed causes the thickening of the streamline unless special \
care is taken with the options of ContourPlot."
}], "ExampleText",
 CellID->78970066]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[{
 StyleBox[ButtonBox["ICSourceConstant",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICSourceConstant"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICSourceLinear",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICSourceLinear"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICVortexPoint",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICVortexPoint"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICDoubletPoint",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletPoint"],
  FontFamily->"Verdana"]
}], "SeeAlso",
 CellID->745892494]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["2.1 Introduction to Influence Coefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"]], \
"Tutorials",
 CellID->178955562],

Cell[TextData[ButtonBox["2.2 Point Singularities",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/PointSingularities"]], "Tutorials",
 CellID->67455226]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["InfluenceCoefficients",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/InfluenceCoefficients"]], "MoreAbout",
 CellID->19239157]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"ICSourcePoint - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 0, 55.8624000}", "context" -> 
    "Aerodynamics`InfluenceCoefficients`", "keywords" -> {}, "index" -> True, 
    "label" -> "Aerodynamics Package Paclet Symbol", "language" -> "en", 
    "paclet" -> "Aerodynamics Package", "status" -> "None", "summary" -> 
    "ICSourcePoint[r_f, r_p] computes velocity influence coefficients at \
field points r_f, due to point sources at r_p.ICPhiSourcePoint[r_f, r_p] \
computes velocity-potential influence coefficients at field points r_f, due \
to point sources at r_p.ICPsiSourcePoint[r_f, r_p] computes stream-function \
influence coefficients at field points r_f, due to point sources at r_p.", 
    "synonyms" -> {}, "title" -> "ICSourcePoint", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/ICSourcePoint"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[9071, 275, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->407763394]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 192049, 3847}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[606, 21, 76, 1, 70, "SymbolColorBar"],
Cell[685, 24, 191, 5, 70, "LinkTrail"],
Cell[879, 31, 3566, 81, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4470, 116, 56, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4529, 119, 2079, 68, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6645, 192, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->158200725],
Cell[7228, 211, 364, 11, 70, "Notes",
 CellID->1067943069],
Cell[7595, 224, 247, 7, 70, "Notes",
 CellID->206410608],
Cell[7845, 233, 433, 10, 70, "Notes",
 CellID->30234373],
Cell[8281, 245, 676, 22, 70, "Notes",
 CellID->333352942],
Cell[8960, 269, 74, 1, 70, "Notes",
 CellID->529540534]
}, Closed]],
Cell[CellGroupData[{
Cell[9071, 275, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->407763394],
Cell[CellGroupData[{
Cell[9557, 292, 148, 5, 70, "ExampleSection",
 CellID->113239048],
Cell[9708, 299, 128, 6, 70, "ExampleText",
 CellID->513831186],
Cell[9839, 307, 100, 3, 70, "Input",
 CellID->267685788],
Cell[9942, 312, 321, 8, 70, "ExampleText",
 CellID->659456463],
Cell[10266, 322, 315, 12, 70, "Input",
 CellID->64953589],
Cell[10584, 336, 140, 4, 70, "ExampleText",
 CellID->326501090],
Cell[CellGroupData[{
Cell[10749, 344, 209, 7, 70, "Input",
 CellID->560964283],
Cell[10961, 353, 547, 19, 56, "Output",
 CellID->23414792]
}, Open  ]],
Cell[11523, 375, 91, 1, 70, "ExampleText",
 CellID->44302699],
Cell[CellGroupData[{
Cell[11639, 380, 168, 5, 70, "Input",
 CellID->575890672],
Cell[11810, 387, 311, 11, 58, "Output",
 CellID->106111450]
}, Open  ]],
Cell[12136, 401, 90, 1, 70, "ExampleText",
 CellID->472419597],
Cell[CellGroupData[{
Cell[12251, 406, 166, 5, 70, "Input",
 CellID->8664355],
Cell[12420, 413, 262, 9, 54, "Output",
 CellID->157697989]
}, Open  ]],
Cell[12697, 425, 315, 9, 70, "ExampleText",
 CellID->53987023],
Cell[CellGroupData[{
Cell[13037, 438, 195, 6, 70, "Input",
 CellID->30542685],
Cell[13235, 446, 213, 7, 54, "Output",
 CellID->392975652]
}, Open  ]],
Cell[13463, 456, 203, 4, 70, "ExampleText",
 CellID->9427774],
Cell[13669, 462, 408, 12, 70, "Input",
 CellID->258291294],
Cell[14080, 476, 967, 35, 70, "ExampleText",
 CellID->78226744],
Cell[CellGroupData[{
Cell[15072, 515, 298, 10, 70, "Input",
 CellID->13857323],
Cell[15373, 527, 209, 7, 36, "Output",
 CellID->394611595]
}, Open  ]],
Cell[CellGroupData[{
Cell[15619, 539, 321, 11, 70, "Input",
 CellID->692553449],
Cell[15943, 552, 209, 7, 36, "Output",
 CellID->433802873]
}, Open  ]],
Cell[16167, 562, 264, 5, 70, "ExampleText",
 CellID->78531558],
Cell[CellGroupData[{
Cell[16456, 571, 3716, 104, 70, "Input",
 CellID->1746043248],
Cell[20175, 677, 165465, 2968, 70, "Output",
 CellID->303345165]
}, Open  ]],
Cell[185655, 3648, 538, 12, 70, "ExampleText",
 CellID->680409942],
Cell[186196, 3662, 231, 6, 70, "ExampleText",
 CellID->635547329],
Cell[CellGroupData[{
Cell[186452, 3672, 662, 19, 70, "Input",
 CellID->81019929],
Cell[187117, 3693, 159, 5, 36, "Output",
 CellID->279799763]
}, Open  ]],
Cell[187291, 3701, 434, 11, 70, "ExampleText",
 CellID->78970066]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[187774, 3718, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[188194, 3732, 643, 21, 70, "SeeAlso",
 CellID->745892494]
}, Open  ]],
Cell[CellGroupData[{
Cell[188874, 3758, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[189296, 3772, 210, 5, 70, "Tutorials",
 CellID->178955562],
Cell[189509, 3779, 168, 3, 70, "Tutorials",
 CellID->67455226]
}, Open  ]],
Cell[CellGroupData[{
Cell[189714, 3787, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[190136, 3801, 166, 3, 70, "MoreAbout",
 CellID->19239157]
}, Open  ]],
Cell[190317, 3807, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

