(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    243231,       5136]
NotebookOptionsPosition[    235947,       4902]
NotebookOutlinePosition[    238080,       4955]
CellTagsIndexPosition[    237993,       4950]
WindowTitle->ICVortexConstant - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"2.1 Introduction to Influence Coefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"], "\
\<\"2.3 Isolated Panel\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IsolatedPanel"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"ICVortexPoint\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICVortexPoint"], "\<\"ICVortexLinear\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICVortexLinear"], "\<\"ICDoubletConstant\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICDoubletConstant"], \
"\<\"ICSourceConstant\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICSourceConstant"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"InfluenceCoefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/InfluenceCoefficients"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/ICVortexConstant\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/ICVortexConstant"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
ICVortexConstant.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/ICVortexConstant", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["ICVortexConstant", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICVortexConstant", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes velocity influence coefficients at field \
points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     ", due to connected vortex panels of constant strength whose endpoints \
are located by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICPhiVortexConstant", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes velocity-potential influence coefficients at \
field points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     " due to connected vortex panels of constant strength whose endpoints \
are located by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICPsiVortexConstant", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]",
     "computes stream-function influence coefficients at field points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     " due to connected vortex panels of constant strength whose endpoints \
are located by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, 
   "Rows" -> {None, None, {None}}, "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->481824839],

Cell[TextData[{
 "Input arguments are collections of vectors of the data type ",
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], "InlineFormula"],
 ".",
 "    ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation may be used for single vectors."
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 "The circulation density for a vortex panel of constant strength is written \
as ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["\[Gamma]", "TR"], 
    RowBox[{"(", 
     StyleBox["x", "TI"], 
     StyleBox[")", "TI"]}]}], 
   StyleBox["=", "TI"], 
   StyleBox["a", "TI"]}]], "InlineFormula"],
 ", where ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " is the strength parameter for the panel.  ",
 "For collections of panels, strength parameters are written in a list \
ordered according to the clockwise numbering convention for panels."
}], "Notes",
 CellID->25338845],

Cell[TextData[{
 "Output for velocity influence coefficients is a collection of vectors of \
the data type CartesianVectors.  Influence coefficients for velocity \
potential and stream function are lists.  For the special case of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation being used to specify a single field point, ",
 "output is simplified as illustrated in the examples",
 "."
}], "Notes",
 CellID->30234373],

Cell[TextData[{
 "The velocity, velocity potential and stream function for a panel are \
written as products of a geometric influence coefficient and the strength \
parameter.",
 "\t\t\t\t\t\t\t\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["V", "TI"], "=", 
   RowBox[{
    StyleBox["a", "TI"], " ", 
    StyleBox["ic", "TI"]}]}]], "InlineFormula"],
 "\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Phi]", "TR"], "=", 
   RowBox[{
    StyleBox["a", "TI"], " ", 
    StyleBox["ic\[Phi]", "TI"]}]}]], "InlineFormula"],
 "\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Psi]", "TR"], "=", 
   RowBox[{
    StyleBox["a", "TI"], " ", 
    StyleBox["ic\[Psi]", "TI"]}]}]], "InlineFormula"]
}], "Notes",
 CellID->495497047],

Cell["New in 1.0   |   Last modified in 1.2", "Notes",
 CellID->529540534]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->103271997],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->346306084],

Cell[TextData[{
 "Load ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->513831186],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->267685788],

Cell[TextData[{
 "This example demonstrates some properties of a single panel.  Algebraic \
equations for influence coefficients are simplest in a local coordinate \
system where the panel extends from the origin along the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis to ",
 Cell[BoxData[
  StyleBox["\[Delta]", "TR"]], "InlineFormula"],
 ", the length of the panel."
}], "ExampleText",
 CellID->659456463],

Cell[TextData[{
 "Specify an arbitrary field point and panel endpoints.  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation is used for a single vector."
}], "ExampleText",
 CellID->250733239],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"rF", ",", "rP"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"x", ",", "z"}], "}"}], ",", 
     RowBox[{"MakeCartesianVectors", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "\[Delta]"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}]}], 
  ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->64953589],

Cell["Compute the velocity influence coefficient.", "ExampleText",
 CellID->326501090],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"icu", ",", "icw"}], "}"}], "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"ICVortexConstant", "[", 
     RowBox[{"rF", ",", "rP"}], "]"}], ",", 
    RowBox[{"\[Delta]", ">", "0"}]}], "]"}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->560964283],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"-", 
      RowBox[{"ArcTan", "[", 
       RowBox[{"x", ",", "z"}], "]"}]}], "+", 
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}]}], 
    RowBox[{"2", " ", "\[Pi]"}]], ",", 
   FractionBox[
    RowBox[{"Log", "[", 
     FractionBox[
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"], "-", 
       RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
       SuperscriptBox["\[Delta]", "2"]}], 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"]}]], "]"}], 
    RowBox[{"4", " ", "\[Pi]"}]]}], "}"}]], "Output",
 ImageSize->{347, 46},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->201437575]
}, Open  ]],

Cell[TextData[{
 "Display the algebraic equations for components of the velocity influence \
coefficient, replacing ArcTan function by named angles (See ",
 ButtonBox["2.1 Introduction to InfluenceCoefficients",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"],
 " for sketch defining named angles).  Although the substitution of named \
angles is made to simplify the appearance of equations, note that the \
original equations are maintained for subsequent computations."
}], "ExampleText",
 CellID->120984796],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"icu", ",", "icw"}], "}"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
     "\[Theta]2"}]}], "}"}]}]], "Input",
 CellLabel->"In[4]:=",
 CellID->176773044],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"-", "\[Theta]1"}], "+", "\[Theta]2"}], 
    RowBox[{"2", " ", "\[Pi]"}]], ",", 
   FractionBox[
    RowBox[{"Log", "[", 
     FractionBox[
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"], "-", 
       RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
       SuperscriptBox["\[Delta]", "2"]}], 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"]}]], "]"}], 
    RowBox[{"4", " ", "\[Pi]"}]]}], "}"}]], "Output",
 ImageSize->{193, 46},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->21131738]
}, Open  ]],

Cell["\<\
Compute the velocity-potential influence coefficient and display in terms of \
named angles.\
\>", "ExampleText",
 CellID->44302699],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ic\[Phi]", "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"ICPhiVortexConstant", "[", 
      RowBox[{"rF", ",", "rP"}], "]"}], ",", 
     RowBox[{"\[Delta]", ">", "0"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"ic\[Phi]", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
     "\[Theta]2"}]}], "}"}]}]}], "Input",
 CellLabel->"In[5]:=",
 CellID->575890672],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "2"}], " ", "x", " ", "\[Theta]1"}], "+", 
   RowBox[{"2", " ", 
    RowBox[{"(", 
     RowBox[{"x", "-", "\[Delta]"}], ")"}], " ", "\[Theta]2"}], "+", 
   RowBox[{"z", " ", 
    RowBox[{"Log", "[", 
     FractionBox[
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"], "-", 
       RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
       SuperscriptBox["\[Delta]", "2"]}], 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"]}]], "]"}]}]}], 
  RowBox[{"4", " ", "\[Pi]"}]]], "Output",
 ImageSize->{261, 46},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->128867656]
}, Open  ]],

Cell["\<\
Compute and display the stream-function influence coefficient.\
\>", "ExampleText",
 CellID->472419597],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ic\[Psi]", "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"ICPsiVortexConstant", "[", 
      RowBox[{"rF", ",", "rP"}], "]"}], ",", 
     RowBox[{"\[Delta]", ">", "0"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"ic\[Psi]", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
     "\[Theta]2"}]}], "}"}]}]}], "Input",
 CellLabel->"In[6]:=",
 CellID->8664355],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "2"}], " ", "z", " ", "\[Theta]1"}], "+", 
   RowBox[{"2", " ", "z", " ", "\[Theta]2"}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", 
     RowBox[{
      SuperscriptBox["x", "2"], "+", 
      SuperscriptBox["z", "2"]}], "]"}]}], "-", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", 
     RowBox[{
      SuperscriptBox["x", "2"], "+", 
      SuperscriptBox["z", "2"], "-", 
      RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
      SuperscriptBox["\[Delta]", "2"]}], "]"}]}], "+", 
   RowBox[{"\[Delta]", " ", 
    RowBox[{"Log", "[", 
     RowBox[{
      SuperscriptBox["x", "2"], "+", 
      SuperscriptBox["z", "2"], "-", 
      RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
      SuperscriptBox["\[Delta]", "2"]}], "]"}]}]}], 
  RowBox[{"4", " ", "\[Pi]"}]]], "Output",
 ImageSize->{521, 37},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->514646365]
}, Open  ]],

Cell["\<\
Write the velocity components, velocity potential and stream function as the \
product of the strength parameter and the geometric influence coefficients.\
\>", "ExampleText",
 CellID->9427774],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"u", ",", "w", ",", "\[Phi]", ",", "\[Psi]"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", " ", "icu"}], ",", 
     RowBox[{"a", " ", "icw"}], ",", 
     RowBox[{"a", " ", "ic\[Phi]"}], ",", 
     RowBox[{"a", " ", "ic\[Psi]"}]}], "}"}]}], ";"}]], "Input",
 CellLabel->"In[7]:=",
 CellID->258291294],

Cell[TextData[{
 "Verify that the relations between velocity, velocity potential and stream \
function are satisfied.\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["u", "TI"], "=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Phi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["x", "TI"]}]], "=", 
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Psi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["z", "TI"]}]]}]}]], "InlineFormula"],
 "\t\t\t",
 Cell[BoxData[
  RowBox[{"w", "=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Phi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["z", "TI"]}]], "=", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{"\[PartialD]", 
       StyleBox["\[Psi]", "TR"]}], 
      RowBox[{"\[PartialD]", 
       StyleBox["x", "TI"]}]]}]}]}]], "InlineFormula"],
 "\t\t\t\t"
}], "ExampleText",
 CellID->78226744],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"u", "\[Equal]", 
     RowBox[{"D", "[", 
      RowBox[{"\[Phi]", ",", "x"}], "]"}]}], ",", 
    RowBox[{"u", "\[Equal]", 
     RowBox[{"D", "[", 
      RowBox[{"\[Psi]", ",", "z"}], "]"}]}]}], "}"}], "//", 
  "Simplify"}]], "Input",
 CellLabel->"In[8]:=",
 CellID->13857323],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->21822114]
}, Open  ]],

Cell[TextData[{
 "To complete the next evaluation, ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  StyleBox["z", "TI"]], "InlineFormula"],
 " need to be declared as real variables."
}], "ExampleText",
 CellID->421446271],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"w", "\[Equal]", 
      RowBox[{"D", "[", 
       RowBox[{"\[Phi]", ",", "z"}], "]"}]}], ",", 
     RowBox[{"w", "\[Equal]", 
      RowBox[{"-", 
       RowBox[{"D", "[", 
        RowBox[{"\[Psi]", ",", "x"}], "]"}]}]}]}], "}"}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"x", ",", "z"}], "}"}], "\[Element]", "Reals"}]}], 
  "]"}]], "Input",
 CellLabel->"In[9]:=",
 CellID->471042009],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->390033007]
}, Open  ]],

Cell[TextData[{
 "Flow visualization provides a plausibility check on the computed flow \
field.  Set ",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Delta]", "TR"], "=", "1"}]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  RowBox[{
   StyleBox["a", "TI"], " ", "=", " ", "1"}]], "InlineFormula"],
 "  to provide unit integrated panel strength, ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", 
     StyleBox["\[Delta]", "TR"]], 
    RowBox[{"\[Gamma]", 
     RowBox[{"(", 
      StyleBox["x", "TI"], ")"}], 
     RowBox[{"\[DifferentialD]", 
      StyleBox["x", "TI"]}]}]}], "=", "1"}]], "InlineFormula"],
 ".   ",
 "Circular regions near the edges of the panel have been excluded from the \
vector plot to avoid showing vectors of very large magnitude."
}], "ExampleText",
 CellID->78531558],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xMin", ",", "xMax", ",", "zMin", ",", "zMax"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "2", ",", 
     RowBox[{"-", "1.2"}], ",", "1.2"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"uPlot", ",", "wPlot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"u", ",", "w"}], "}"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", "\[Rule]", "1"}], ",", 
      RowBox[{"\[Delta]", "\[Rule]", "1"}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Phi]Plot", ",", "\[Psi]Plot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\[Phi]", ",", "\[Psi]"}], "}"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", "\[Rule]", "1"}], ",", 
      RowBox[{"\[Delta]", "\[Rule]", "1"}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gV", "=", 
   RowBox[{"VectorPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"uPlot", ",", "wPlot"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"RegionFunction", "\[Rule]", 
      RowBox[{"Function", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"x", ",", "z"}], "}"}], ",", 
        RowBox[{
         RowBox[{"0.005", "<", 
          RowBox[{
           RowBox[{"x", "^", "2"}], "+", 
           RowBox[{"z", "^", "2"}]}]}], " ", "&&", " ", 
         RowBox[{"0.005", "<", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"x", "-", "1"}], ")"}], "^", "2"}], "+", 
           RowBox[{"z", "^", "2"}]}]}]}]}], "]"}]}], ",", 
     RowBox[{"VectorColorFunction", "->", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"RGBColor", "[", 
         RowBox[{"0", ",", "1", ",", "0"}], "]"}], "&"}], ")"}]}], ",", 
     RowBox[{"VectorPoints", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"18", ",", "18"}], "}"}]}], ",", 
     RowBox[{"VectorScale", "\[Rule]", "Medium"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Phi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Phi]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "13"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Red"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Psi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "7"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Blue"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gPanel", "=", 
   RowBox[{"Graphics", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"GrayLevel", "[", "0", "]"}], ",", 
      RowBox[{"AbsoluteThickness", "[", "3", "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"Transpose", "[", 
        RowBox[{"Components", "[", 
         RowBox[{"rP", "/.", 
          RowBox[{"\[Delta]", "\[Rule]", "1.0"}]}], "]"}], "]"}], "]"}]}], 
     "}"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gV", ",", "g\[Phi]", ",", "g\[Psi]", ",", "gPanel", ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
   RowBox[{"FrameTicks", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.0"}], ",", "2.0", ",", "0.25"}], "]"}], "]"}], ",", 
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.2"}], ",", "1.2", ",", "0.2"}], "]"}], "]"}], ",", 
      "None", ",", "None"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", " ", "72"}]}], ",", 
   RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellLabel->"In[10]:=",
 CellID->1746043248],

Cell[BoxData[
 GraphicsBox[{{{}, 
    {Hue[0.67, 0.6, 0.6], 
     {Arrowheads[{{0.009116798619914601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9838697719654657, -1.2192661282485955`}, \
{-1.0161302280345343`, -1.1807338717514042`}}]}}, 
     {Arrowheads[{{0.009569687754266346, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9842819183200495, -1.0800036144948315`}, \
{-1.0157180816799505`, -1.037643444328698}}]}}, 
     {Arrowheads[{{0.01003260438058277, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9849907854779197, -0.9408700526751171}, \
{-1.0150092145220804`, -0.8944240649719418}}]}}, 
     {Arrowheads[{{0.010494535920320257`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9860676623265575, -0.8018182182871996}, \
{-1.0139323376734426`, -0.7511229581833889}}]}}, 
     {Arrowheads[{{0.010939521084517059`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9875817038932192, -0.6627686487870799}, \
{-1.012418296106781, -0.607819586507038}}]}}, 
     {Arrowheads[{{0.011346123674529864`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9895846990804944, -0.5236035400388053}, \
{-1.0104153009195056`, -0.46463175407884216`}}]}}, 
     {Arrowheads[{{0.011688143720156714`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9920886178369893, -0.38416858183854113`}, \
{-1.0079113821630108`, -0.3217137711026357}}]}}, 
     {Arrowheads[{{0.011937480299842701`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9950407430559669, -0.24428998699210255`}, \
{-1.004959256944033, -0.1792394247726037}}]}}, 
     {Arrowheads[{{0.012069481079506499`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9983087374695213, -0.10381021197286217`}, \
{-1.0016912625304788`, -0.03736625861537352}}]}}, 
     {Arrowheads[{{0.01206948107950649, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0016912625304786`, 
        0.03736625861537314}, {-0.9983087374695214, 0.10381021197286176`}}]}}, 
     {Arrowheads[{{0.011937480299842703`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.004959256944033, 
        0.17923942477260332`}, {-0.9950407430559668, 
        0.24428998699210216`}}]}}, 
     {Arrowheads[{{0.011688143720156714`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0079113821630106`, 
        0.3217137711026353}, {-0.9920886178369893, 0.3841685818385408}}]}}, 
     {Arrowheads[{{0.011346123674529873`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0104153009195056`, 
        0.4646317540788418}, {-0.9895846990804944, 0.5236035400388049}}]}}, 
     {Arrowheads[{{0.010939521084517033`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0124182961067807`, 
        0.6078195865070377}, {-0.9875817038932193, 0.6627686487870795}}]}}, 
     {Arrowheads[{{0.010494535920320257`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0139323376734426`, 
        0.7511229581833885}, {-0.9860676623265575, 0.8018182182871991}}]}}, 
     {Arrowheads[{{0.01003260438058275, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0150092145220801`, 
        0.8944240649719414}, {-0.9849907854779197, 0.9408700526751167}}]}}, 
     {Arrowheads[{{0.009569687754266346, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0157180816799505`, 
        1.0376434443286975`}, {-0.9842819183200495, 1.080003614494831}}]}}, 
     {Arrowheads[{{0.009116798619914589, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0161302280345343`, 
        1.180733871751404}, {-0.9838697719654658, 1.2192661282485953`}}]}}, 
     {Arrowheads[{{0.009776473983450114, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.804915554827524, -1.2194824863382105`}, \
{-0.8421432687018878, -1.1805175136617894`}}]}}, 
     {Arrowheads[{{0.010342744314054698`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8050900680510346, -1.0805623587736346`}, \
{-0.8419687554783771, -1.0370847000498948`}}]}}, 
     {Arrowheads[{{0.010937677956452644`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8055952571169903, -0.941877695175277}, \
{-0.8414635664124214, -0.8934164224717821}}]}}, 
     {Arrowheads[{{0.011550188090361797`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8065436107100611, -0.8033940158172144}, \
{-0.8405152128193506, -0.749547160653374}}]}}, 
     {Arrowheads[{{0.01216112908340845, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.808062428429387, -0.6650296413195764}, \
{-0.8389963951000247, -0.6055585939745416}}]}}, 
     {Arrowheads[{{0.01274059638075665, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8102746410205225, -0.5266345889484167}, \
{-0.8367841825088892, -0.46160070516923063`}}]}}, 
     {Arrowheads[{{0.013246569582145831`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8132589946485415, -0.3879760206041924}, \
{-0.8337998288808701, -0.3179063323369845}}]}}, 
     {Arrowheads[{{0.01362784987194159, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.816989692319574, -0.2487510543761716}, \
{-0.8300691312098377, -0.17477835738853467`}}]}}, 
     {Arrowheads[{{0.013834467068875205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8212795642619832, -0.10865132097332239`}, \
{-0.8257792592674285, -0.03252514961491328}}]}}, 
     {Arrowheads[{{0.013834467068875205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8257792592674285, 
        0.032525149614912895`}, {-0.8212795642619832, 0.108651320973322}}]}}, 
     {Arrowheads[{{0.01362784987194159, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8300691312098377, 
        0.17477835738853428`}, {-0.816989692319574, 0.2487510543761712}}]}}, 
     {Arrowheads[{{0.013246569582145831`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8337998288808702, 
        0.31790633233698407`}, {-0.8132589946485416, 
        0.38797602060419195`}}]}}, 
     {Arrowheads[{{0.012740596380756657`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8367841825088892, 
        0.46160070516923024`}, {-0.8102746410205225, 0.5266345889484164}}]}}, 
     {Arrowheads[{{0.01216112908340845, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8389963951000247, 
        0.6055585939745411}, {-0.808062428429387, 0.665029641319576}}]}}, 
     {Arrowheads[{{0.011550188090361797`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8405152128193506, 
        0.7495471606533737}, {-0.8065436107100611, 0.803394015817214}}]}}, 
     {Arrowheads[{{0.010937677956452644`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8414635664124214, 
        0.8934164224717815}, {-0.8055952571169903, 0.9418776951752764}}]}}, 
     {Arrowheads[{{0.010342744314054698`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8419687554783771, 
        1.0370847000498944`}, {-0.8050900680510346, 1.0805623587736342`}}]}}, 
     {Arrowheads[{{0.009776473983450114, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8421432687018878, 
        1.180517513661789}, {-0.804915554827524, 1.21948248633821}}]}}, 
     {Arrowheads[{{0.010477306125975762`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6255947566665606, -1.2193173857476642`}, \
{-0.6685228903922629, -1.180682614252336}}]}}, 
     {Arrowheads[{{0.011183628626149135`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6253888688839033, -1.0807438366226534`}, \
{-0.6687287781749199, -1.036903222200876}}]}}, 
     {Arrowheads[{{0.011949495453044802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6255130117197387, -0.9425558182023804}, \
{-0.6686046353390847, -0.8927382994446785}}]}}, 
     {Arrowheads[{{0.012768289697852271`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6261280893497236, -0.8047603565670728}, \
{-0.6679895577090996, -0.7481808199035157}}]}}, 
     {Arrowheads[{{0.013622335338254642`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6274465394599497, -0.6673093440013596}, \
{-0.6666711075988737, -0.6032788912927585}}]}}, 
     {Arrowheads[{{0.014475369768919472`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6297234816075017, -0.5300505149717056}, \
{-0.6643941654513217, -0.4581847791459418}}]}}, 
     {Arrowheads[{{0.015263437675282373`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.633205847216034, -0.3926627923226784}, \
{-0.6609117998427894, -0.3132195606184984}}]}}, 
     {Arrowheads[{{0.01589064678137358, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6380038552628454, -0.25461503207595015`}, \
{-0.656113791795978, -0.16891437968875614`}}]}}, 
     {Arrowheads[{{0.016244935134227048`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6438951300916238, -0.11524939248983948`}, \
{-0.6502225169671995, -0.025927078098396193`}}]}}, 
     {Arrowheads[{{0.016244935134227048`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6502225169671995, 
        0.025927078098395805`}, {-0.6438951300916238, 
        0.11524939248983909`}}]}}, 
     {Arrowheads[{{0.015890646781373582`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.656113791795978, 
        0.16891437968875572`}, {-0.6380038552628454, 
        0.25461503207594977`}}]}}, 
     {Arrowheads[{{0.015263437675282363`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6609117998427894, 
        0.31321956061849804`}, {-0.633205847216034, 0.392662792322678}}]}}, 
     {Arrowheads[{{0.014475369768919463`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6643941654513217, 
        0.4581847791459414}, {-0.6297234816075017, 0.5300505149717052}}]}}, 
     {Arrowheads[{{0.013622335338254642`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6666711075988737, 
        0.603278891292758}, {-0.6274465394599497, 0.6673093440013591}}]}}, 
     {Arrowheads[{{0.012768289697852282`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6679895577090997, 
        0.7481808199035153}, {-0.6261280893497236, 0.8047603565670723}}]}}, 
     {Arrowheads[{{0.011949495453044802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6686046353390847, 
        0.8927382994446781}, {-0.6255130117197387, 0.94255581820238}}]}}, 
     {Arrowheads[{{0.011183628626149135`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.66872877817492, 
        1.0369032222008756`}, {-0.6253888688839034, 1.080743836622653}}]}}, 
     {Arrowheads[{{0.010477306125975775`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6685228903922629, 
        1.1806826142523352`}, {-0.6255947566665605, 1.2193173857476636`}}]}}, 
     {Arrowheads[{{0.011199963345734237`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4459512131949135, -1.2185978856335782`}, \
{-0.49522525739332157`, -1.1814021143664215`}}]}}, 
     {Arrowheads[{{0.012072512361463297`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.44518705599077374`, -1.0803152974673846`}, \
{-0.4959894145974613, -1.0373317613561448`}}]}}, 
     {Arrowheads[{{0.013051586754250093`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4446816671464303, -0.9426034291576108}, \
{-0.49649480344180474`, -0.8926906884894481}}]}}, 
     {Arrowheads[{{0.014144384224217989`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.44463017617339656`, -0.8055550736990104}, \
{-0.4965462944148385, -0.7473861027715781}}]}}, 
     {Arrowheads[{{0.015348449856734201`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4453353714090012, -0.6692318970854733}, \
{-0.49584109917923386`, -0.6013563382086446}}]}}, 
     {Arrowheads[{{0.016637991562921856`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4472494312859861, -0.5335905261025837}, \
{-0.49392703930224896`, -0.45464476801506376`}}]}}, 
     {Arrowheads[{{0.017936440162617962`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.45098254528183845`, -0.39832226420526495`}, \
{-0.4901939253063966, -0.30756008873591184`}}]}}, 
     {Arrowheads[{{0.019074142779961948`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4571397340911158, -0.2625861436972978}, \
{-0.48403673649711926`, -0.1609432680674084}}]}}, 
     {Arrowheads[{{0.01977276963429679, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4657378637744819, -0.12486817945258294`}, \
{-0.47543860681375316`, -0.01630829113565275}}]}}, 
     {Arrowheads[{{0.01977276963429679, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.47543860681375316`, 
        0.016308291135652353`}, {-0.4657378637744819, 
        0.12486817945258255`}}]}}, 
     {Arrowheads[{{0.019074142779961948`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.48403673649711926`, 
        0.16094326806740802`}, {-0.4571397340911158, 
        0.26258614369729744`}}]}}, 
     {Arrowheads[{{0.01793644016261796, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4901939253063966, 
        0.30756008873591145`}, {-0.4509825452818385, 
        0.39832226420526456`}}]}}, 
     {Arrowheads[{{0.01663799156292185, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.49392703930224896`, 
        0.45464476801506337`}, {-0.4472494312859861, 0.5335905261025833}}]}}, 
     {Arrowheads[{{0.015348449856734198`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.49584109917923386`, 
        0.6013563382086442}, {-0.44533537140900126`, 0.6692318970854728}}]}}, 
     {Arrowheads[{{0.014144384224217989`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4965462944148385, 
        0.7473861027715777}, {-0.44463017617339656`, 0.8055550736990099}}]}}, 
     {Arrowheads[{{0.013051586754250093`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.49649480344180474`, 
        0.8926906884894478}, {-0.4446816671464303, 0.9426034291576104}}]}}, 
     {Arrowheads[{{0.012072512361463297`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4959894145974613, 
        1.0373317613561444`}, {-0.44518705599077374`, 
        1.0803152974673842`}}]}}, 
     {Arrowheads[{{0.011199963345734237`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.49522525739332157`, 
        1.1814021143664213`}, {-0.4459512131949135, 1.218597885633578}}]}}, 
     {Arrowheads[{{0.0119128240533511, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2661113500275943, -1.2171367569582179`}, \
{-0.3221239440900525, -1.1828632430417818`}}]}}, 
     {Arrowheads[{{0.012970645227741053`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.26460406742020326`, -1.0789957761908446`}, \
{-0.3236312266974435, -1.0386512826326846`}}]}}, 
     {Arrowheads[{{0.014198637077553295`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.26317639494206313`, -0.9416063510240228}, \
{-0.3250588991755836, -0.8936877666230362}}]}}, 
     {Arrowheads[{{0.01563161768674601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2620007739872351, -0.8051868693066164}, \
{-0.32623452013041165`, -0.7477543071639721}}]}}, 
     {Arrowheads[{{0.017308930966610733`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2613963465182258, -0.6700092231753749}, \
{-0.3268389475994209, -0.6005790121187431}}]}}, 
     {Arrowheads[{{0.019265812323821695`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.261959378899017, -0.5363710273499133}, \
{-0.32627591521862975`, -0.4518642667677341}}]}}, 
     {Arrowheads[{{0.021497164399658767`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.26481138932838394`, -0.40443459392157977`}, \
{-0.32342390478926286`, -0.3014477590195971}}]}}, 
     {Arrowheads[{{0.0238257974448833, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2719231465601457, -0.2735671544867973}, \
{-0.316312147557501, -0.14996225727790896`}}]}}, 
     {Arrowheads[{{0.025567923001979954`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2854165171276217, -0.1405173742034792}, \
{-0.30281877699002496`, -0.0006590963847564607}}]}}, 
     {Arrowheads[{{0.025567923001979975`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.3028187769900249, 
        0.0006590963847560444}, {-0.2854165171276218, 
        0.14051737420347887`}}]}}, 
     {Arrowheads[{{0.0238257974448833, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.316312147557501, 
        0.14996225727790854`}, {-0.27192314656014577`, 
        0.2735671544867969}}]}}, 
     {Arrowheads[{{0.02149716439965878, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.3234239047892628, 
        0.30144775901959664`}, {-0.26481138932838394`, 
        0.40443459392157943`}}]}}, 
     {Arrowheads[{{0.019265812323821702`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.32627591521862975`, 
        0.4518642667677337}, {-0.261959378899017, 0.5363710273499129}}]}}, 
     {Arrowheads[{{0.017308930966610733`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.3268389475994209, 
        0.6005790121187427}, {-0.2613963465182258, 0.6700092231753745}}]}}, 
     {Arrowheads[{{0.01563161768674601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.32623452013041165`, 
        0.7477543071639716}, {-0.2620007739872351, 0.805186869306616}}]}}, 
     {Arrowheads[{{0.014198637077553289`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.3250588991755836, 
        0.8936877666230357}, {-0.2631763949420632, 0.9416063510240223}}]}}, 
     {Arrowheads[{{0.012970645227741062`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.32363122669744354`, 
        1.0386512826326844`}, {-0.26460406742020326`, 
        1.0789957761908444`}}]}}, 
     {Arrowheads[{{0.01191282405335109, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.32212394409005246`, 
        1.1828632430417816`}, {-0.2661113500275943, 1.2171367569582177`}}]}}, 
     {Arrowheads[{{0.012572289032024968`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.08630566526264921, -1.214778192421174}, \
{-0.14898845238440928`, -1.1852218075788261`}}]}}, 
     {Arrowheads[{{0.013818604383719674`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.08392157866566363, -1.076518731992283}, \
{-0.15137253898139488`, -1.0411283268312463`}}]}}, 
     {Arrowheads[{{0.015308797801352584`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.08132264964345728, -0.9391129191740502}, \
{-0.15397146800360123`, -0.8961811984730089}}]}}, 
     {Arrowheads[{{0.017116898560857397`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.07856410315335052, -0.8028926802244971}, \
{-0.15673001449370794`, -0.7500484962460914}}]}}, 
     {Arrowheads[{{0.01935172228882581, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.07579740709511018, -0.668359363429699}, \
{-0.15949671055194828`, -0.602228871864419}}]}}, 
     {Arrowheads[{{0.022183664080880645`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.07339572528221622, -0.5363081116974457}, \
{-0.16189839236484227`, -0.4519271824202017}}]}}, 
     {Arrowheads[{{0.025900636613884823`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.07232003315022961, -0.4080894750326979}, \
{-0.1629740844968289, -0.29779287790847897`}}]}}, 
     {Arrowheads[{{0.030997853497116964`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.07552662898693757, -0.2860939099280415}, \
{-0.15976748866012092`, -0.13743550183666475`}}]}}, 
     {Arrowheads[{{0.03754378372508138, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.09470663716635123, -0.17148861752615438`}, \
{-0.14058748048070724`, 0.0303121469379187}}]}}, 
     {Arrowheads[{{0.0375437837250814, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.14058748048070718`, -0.030312146937919143`}, \
{-0.09470663716635132, 0.17148861752615405`}}]}}, 
     {Arrowheads[{{0.03099785349711698, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.15976748866012086`, 
        0.1374355018366643}, {-0.07552662898693761, 0.2860939099280412}}]}}, 
     {Arrowheads[{{0.02590063661388485, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.1629740844968289, 
        0.2977928779084785}, {-0.07232003315022958, 0.4080894750326976}}]}}, 
     {Arrowheads[{{0.022183664080880638`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.16189839236484227`, 
        0.4519271824202013}, {-0.0733957252822162, 0.5363081116974453}}]}}, 
     {Arrowheads[{{0.01935172228882581, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.1594967105519483, 
        0.6022288718644185}, {-0.07579740709511018, 0.6683593634296986}}]}}, 
     {Arrowheads[{{0.01711689856085742, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.156730014493708, 
        0.7500484962460909}, {-0.0785641031533505, 0.8028926802244968}}]}}, 
     {Arrowheads[{{0.01530879780135259, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.1539714680036012, 
        0.8961811984730084}, {-0.08132264964345728, 0.9391129191740497}}]}}, 
     {Arrowheads[{{0.013818604383719667`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.15137253898139486`, 
        1.041128326831246}, {-0.08392157866566363, 1.0765187319922827`}}]}}, 
     {Arrowheads[{{0.012572289032024979`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.1489884523844093, 
        1.1852218075788254`}, {-0.08630566526264918, 1.2147781924211734`}}]}}, 
     {Arrowheads[{{0.013127617905417648`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.09314116969478414, -1.211462549941464}, {
        0.024505889128745635`, -1.1885374500585357`}}]}}, 
     {Arrowheads[{{0.01454299001946831, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.09641191187742296, -1.0727412918205796`}, {
        0.02123514694610682, -1.04490576700295}}]}}, 
     {Arrowheads[{{0.016272107256669013`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.10025320680570056`, -0.9348203359521523}, {
        0.01739385201782921, -0.9004737816949067}}]}}, 
     {Arrowheads[{{0.018427008297575858`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.10479658034701134`, -0.7980532651231155}, {
        0.012850478476518434`, -0.754887911347473}}]}}, 
     {Arrowheads[{{0.021183581531526007`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.1102159532981092, -0.662999352729812}, {
        0.0074311055254205755`, -0.6075888825643061}}]}}, 
     {Arrowheads[{{0.024844937579490488`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.11677415102885273`, -0.530595779955316}, {
        0.0008729077946770386, -0.45763951416233134`}}]}}, 
     {Arrowheads[{{0.030017486788541176`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{
        0.1250082272900642, -0.4025814873780152}, {-0.0073611684665344235`, \
-0.30330086556316166`}}]}}, 
     {Arrowheads[{{0.03825714390929146, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{
        0.13669862743101754`, -0.28285178359554436`}, \
{-0.019051568607487773`, -0.1406776281691619}}]}}, 
     {Arrowheads[{{0.0557576247082935, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{
        0.164103698563034, -0.18253503508526903`}, {-0.04645663973950423, 
        0.04135856449703333}}]}}, 
     {Arrowheads[{{0.055757624708293575`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.04645663973950437, -0.04135856449703389}, {
        0.16410369856303414`, 0.18253503508526875`}}]}}, 
     {Arrowheads[{{0.038257143909291504`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.01905156860748783, 0.1406776281691614}, {
        0.1366986274310176, 0.2828517835955441}}]}}, 
     {Arrowheads[{{0.03001748678854118, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.0073611684665344235`, 0.3033008655631612}, {
        0.1250082272900642, 0.4025814873780148}}]}}, 
     {Arrowheads[{{0.0248449375794905, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.0008729077946770247, 0.45763951416233095`}, {
        0.11677415102885275`, 0.5305957799553157}}]}}, 
     {Arrowheads[{{0.021183581531526018`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.007431105525420562, 0.6075888825643055}, {
        0.11021595329810921`, 0.6629993527298116}}]}}, 
     {Arrowheads[{{0.018427008297575858`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.012850478476518434`, 0.7548879113474726}, {
        0.10479658034701134`, 0.798053265123115}}]}}, 
     {Arrowheads[{{0.016272107256669017`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.017393852017829198`, 0.9004737816949062}, {
        0.10025320680570057`, 0.9348203359521519}}]}}, 
     {Arrowheads[{{0.014542990019468334`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.021235146946106793`, 1.0449057670029493`}, {
        0.09641191187742298, 1.0727412918205792`}}]}}, 
     {Arrowheads[{{0.013127617905417648`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.024505889128745635`, 1.1885374500585355`}, {
        0.09314116969478414, 1.2114625499414637`}}]}}, 
     {Arrowheads[{{0.013529796440564318`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.27186580115168024`, -1.2072829857003338`}, {
        0.19872243414243776`, -1.1927170142996661`}}]}}, 
     {Arrowheads[{{0.015071701293711246`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.27586274803524263`, -1.0677518913188349`}, {
        0.1947254872588754, -1.0498951675046948`}}]}}, 
     {Arrowheads[{{0.016979080664775204`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2807448484326464, -0.9287888946288141}, {
        0.18984338686147162`, -0.9065052230182449}}]}}, 
     {Arrowheads[{{0.019387547870146127`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2868126126691828, -0.7906512430854068}, {
        0.18377562262493524`, -0.7622899333851818}}]}}, 
     {Arrowheads[{{0.022503892202001684`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.29451538076196626`, -0.6537267357753135}, {
        0.17607285453215174`, -0.6168614995188045}}]}}, 
     {Arrowheads[{{0.026656977789932933`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.3045725481181197, -0.518578521985385}, {
        0.16601568717599835`, -0.4696567721322625}}]}}, 
     {Arrowheads[{{0.032391686940352236`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.31826159255747455`, -0.38590357392354324`}, {
        0.15232664273664348`, -0.3199787790176336}}]}}, 
     {Arrowheads[{{0.04058780375872302, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.3380728313728183, -0.25592721847680316`}, {
        0.1325154039212997, -0.1676021932879031}}]}}, 
     {Arrowheads[{{0.052037501067732875`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.36784326800963585`, -0.12536552556101804`}, {
        0.10274496728448215`, -0.015810945027217624`}}]}}, 
     {Arrowheads[{{0.05203750106773292, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.10274496728448204`, 0.015810945027217208`}, {
        0.36784326800963596`, 0.1253655255610177}}]}}, 
     {Arrowheads[{{0.04058780375872306, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.13251540392129962`, 0.16760219328790268`}, {
        0.3380728313728184, 0.2559272184768028}}]}}, 
     {Arrowheads[{{0.03239168694035226, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.15232664273664343`, 0.3199787790176332}, {
        0.3182615925574746, 0.3859035739235429}}]}}, 
     {Arrowheads[{{0.026656977789932933`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.16601568717599832`, 0.4696567721322621}, {
        0.3045725481181197, 0.5185785219853846}}]}}, 
     {Arrowheads[{{0.022503892202001698`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.1760728545321517, 0.6168614995188041}, {
        0.2945153807619663, 0.653726735775313}}]}}, 
     {Arrowheads[{{0.019387547870146127`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.18377562262493524`, 0.7622899333851813}, {
        0.2868126126691828, 0.7906512430854062}}]}}, 
     {Arrowheads[{{0.016979080664775215`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.1898433868614716, 0.9065052230182443}, {
        0.2807448484326464, 0.9287888946288136}}]}}, 
     {Arrowheads[{{0.015071701293711253`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.19472548725887537`, 1.0498951675046941`}, {
        0.27586274803524263`, 1.0677518913188342`}}]}}, 
     {Arrowheads[{{0.013529796440564318`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.19872243414243776`, 1.1927170142996657`}, {
        0.27186580115168024`, 1.2072829857003333`}}]}}, 
     {Arrowheads[{{0.013740997549939769`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4495540150062986, -1.2024998828640752`}, {
        0.37397539675840763`, -1.197500117135925}}]}}, 
     {Arrowheads[{{0.015350155243130648`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4539594286356449, -1.0619029878735429`}, {
        0.3695699831290613, -1.0557440709499866`}}]}}, 
     {Arrowheads[{{0.01735114455407311, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4594302893993537, -0.9215107932804454}, {
        0.3640991223653526, -0.9137833243666136}}]}}, 
     {Arrowheads[{{0.019888137372837372`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.466355465429616, -0.7814147380883764}, {
        0.3571739463350903, -0.771526438382212}}]}}, 
     {Arrowheads[{{0.023170576259998205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4752991564963728, -0.6417440315971059}, {
        0.3482302552683335, -0.6288442036970121}}]}}, 
     {Arrowheads[{{0.027496482743281668`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.48706609021495806`, -0.5026544596648452}, {
        0.33646332154974823`, -0.4855808344528022}}]}}, 
     {Arrowheads[{{0.033253625406084175`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5027173474363977, -0.36423497110382186`}, {
        0.32081206432830855`, -0.341647381837355}}]}}, 
     {Arrowheads[{{0.04080882460299786, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5233093533213213, -0.22619529480784276`}, {
        0.30022005844338495`, -0.19733411695686354`}}]}}, 
     {Arrowheads[{{0.050108447190220146`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5488455857875305, -0.08737700370833855}, {
        0.27468382597717583`, -0.05379946687989713}}]}}, 
     {Arrowheads[{{0.05010844719022016, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2746838259771758, 0.05379946687989674}, {
        0.5488455857875305, 0.08737700370833816}}]}}, 
     {Arrowheads[{{0.040808824602997874`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.30022005844338484`, 0.19733411695686312`}, {
        0.5233093533213213, 0.22619529480784237`}}]}}, 
     {Arrowheads[{{0.03325362540608419, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.32081206432830844`, 0.3416473818373546}, {
        0.5027173474363977, 0.36423497110382147`}}]}}, 
     {Arrowheads[{{0.027496482743281696`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.33646332154974823`, 0.48558083445280176`}, {
        0.48706609021495817`, 0.5026544596648449}}]}}, 
     {Arrowheads[{{0.023170576259998216`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.3482302552683335, 0.6288442036970117}, {
        0.4752991564963729, 0.6417440315971055}}]}}, 
     {Arrowheads[{{0.019888137372837372`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.35717394633509025`, 0.7715264383822117}, {
        0.4663554654296159, 0.781414738088376}}]}}, 
     {Arrowheads[{{0.01735114455407311, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.3640991223653526, 0.9137833243666131}, {
        0.4594302893993537, 0.9215107932804449}}]}}, 
     {Arrowheads[{{0.015350155243130648`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.36956998312906136`, 1.0557440709499861`}, {
        0.453959428635645, 1.0619029878735424`}}]}}, 
     {Arrowheads[{{0.01374099754993978, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.37397539675840763`, 1.1975001171359243`}, {
        0.44955401500629866`, 1.2024998828640745`}}]}}, 
     {Arrowheads[{{0.013740997549939763`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6260246032415928, -1.1975001171359247`}, {
        0.5504459849937019, -1.2024998828640752`}}]}}, 
     {Arrowheads[{{0.015350155243130638`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6304300168709391, -1.0557440709499866`}, {
        0.5460405713643556, -1.0619029878735429`}}]}}, 
     {Arrowheads[{{0.01735114455407311, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6359008776346479, -0.9137833243666136}, {
        0.5405697106006467, -0.9215107932804454}}]}}, 
     {Arrowheads[{{0.019888137372837393`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6428260536649102, -0.771526438382212}, {
        0.5336445345703844, -0.7814147380883764}}]}}, 
     {Arrowheads[{{0.023170576259998184`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6517697447316669, -0.6288442036970121}, {
        0.5247008435036278, -0.6417440315971059}}]}}, 
     {Arrowheads[{{0.02749648274328167, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6635366784502522, -0.48558083445280215`}, {
        0.5129339097850424, -0.5026544596648453}}]}}, 
     {Arrowheads[{{0.033253625406084175`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6791879356716919, -0.34164738183735494`}, {
        0.4972826525636027, -0.3642349711038219}}]}}, 
     {Arrowheads[{{0.04080882460299786, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6997799415566155, -0.19733411695686343`}, {
        0.4766906466786791, -0.2261952948078428}}]}}, 
     {Arrowheads[{{0.05010844719022012, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7253161740228246, -0.053799466879897055`}, {
        0.45115441421247005`, -0.08737700370833862}}]}}, 
     {Arrowheads[{{0.05010844719022016, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.45115441421246993`, 0.08737700370833824}, {
        0.7253161740228247, 0.05379946687989666}}]}}, 
     {Arrowheads[{{0.04080882460299787, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.47669064667867905`, 0.22619529480784245`}, {
        0.6997799415566155, 0.19733411695686304`}}]}}, 
     {Arrowheads[{{0.03325362540608421, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.49728265256360266`, 0.3642349711038215}, {
        0.679187935671692, 0.3416473818373545}}]}}, 
     {Arrowheads[{{0.027496482743281696`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5129339097850424, 0.5026544596648449}, {
        0.6635366784502523, 0.4855808344528017}}]}}, 
     {Arrowheads[{{0.023170576259998205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5247008435036277, 0.6417440315971055}, {
        0.6517697447316669, 0.6288442036970117}}]}}, 
     {Arrowheads[{{0.019888137372837372`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5336445345703844, 0.781414738088376}, {0.6428260536649101,
         0.7715264383822117}}]}}, 
     {Arrowheads[{{0.01735114455407311, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5405697106006467, 0.9215107932804449}, {
        0.6359008776346479, 0.9137833243666131}}]}}, 
     {Arrowheads[{{0.015350155243130638`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5460405713643555, 1.0619029878735424`}, {
        0.630430016870939, 1.0557440709499861`}}]}}, 
     {Arrowheads[{{0.01374099754993978, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5504459849937018, 1.2024998828640747`}, {
        0.6260246032415928, 1.1975001171359245`}}]}}, 
     {Arrowheads[{{0.013529796440564323`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8012775658575627, -1.1927170142996661`}, {
        0.7281341988483202, -1.2072829857003338`}}]}}, 
     {Arrowheads[{{0.015071701293711265`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.805274512741125, -1.0498951675046946`}, {
        0.7241372519647578, -1.0677518913188349`}}]}}, 
     {Arrowheads[{{0.01697908066477522, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8101566131385288, -0.9065052230182448}, {
        0.719255151567354, -0.9287888946288141}}]}}, 
     {Arrowheads[{{0.01938754787014612, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8162243773750653, -0.7622899333851817}, {
        0.7131873873308178, -0.7906512430854067}}]}}, 
     {Arrowheads[{{0.022503892202001687`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8239271454678487, -0.6168614995188045}, {
        0.7054846192380342, -0.6537267357753135}}]}}, 
     {Arrowheads[{{0.026656977789932926`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8339843128240021, -0.4696567721322624}, {
        0.6954274518818808, -0.518578521985385}}]}}, 
     {Arrowheads[{{0.032391686940352236`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8476733572633569, -0.31997877901763355`}, {
        0.6817384074425259, -0.3859035739235433}}]}}, 
     {Arrowheads[{{0.04058780375872303, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8674845960787008, -0.16760219328790302`}, {
        0.6619271686271823, -0.2559272184768033}}]}}, 
     {Arrowheads[{{0.052037501067732896`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8972550327155183, -0.015810945027217485`}, {
        0.6321567319903646, -0.1253655255610182}}]}}, 
     {Arrowheads[{{0.05203750106773292, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6321567319903645, 0.12536552556101782`}, {
        0.8972550327155183, 0.015810945027217083`}}]}}, 
     {Arrowheads[{{0.04058780375872307, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6619271686271821, 0.2559272184768029}, {
        0.8674845960787009, 0.1676021932879026}}]}}, 
     {Arrowheads[{{0.03239168694035226, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6817384074425259, 0.3859035739235429}, {0.847673357263357,
         0.3199787790176331}}]}}, 
     {Arrowheads[{{0.026656977789932947`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6954274518818808, 0.5185785219853847}, {
        0.8339843128240022, 0.469656772132262}}]}}, 
     {Arrowheads[{{0.022503892202001708`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7054846192380342, 0.653726735775313}, {0.8239271454678488,
         0.6168614995188041}}]}}, 
     {Arrowheads[{{0.019387547870146123`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7131873873308177, 0.7906512430854064}, {
        0.8162243773750651, 0.7622899333851813}}]}}, 
     {Arrowheads[{{0.01697908066477522, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7192551515673541, 0.9287888946288138}, {
        0.8101566131385289, 0.9065052230182444}}]}}, 
     {Arrowheads[{{0.015071701293711265`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7241372519647579, 1.0677518913188344`}, {
        0.8052745127411252, 1.0498951675046941`}}]}}, 
     {Arrowheads[{{0.013529796440564323`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7281341988483202, 1.2072829857003333`}, {
        0.8012775658575627, 1.1927170142996657`}}]}}, 
     {Arrowheads[{{0.013127617905417648`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9754941108712548, -1.188537450058536}, {
        0.9068588303052163, -1.2114625499414642`}}]}}, 
     {Arrowheads[{{0.014542990019468313`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9787648530538937, -1.0449057670029498`}, {
        0.9035880881225776, -1.0727412918205796`}}]}}, 
     {Arrowheads[{{0.016272107256669006`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9826061479821713, -0.9004737816949067}, {
        0.8997467931943, -0.9348203359521523}}]}}, 
     {Arrowheads[{{0.018427008297575858`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.987149521523482, -0.754887911347473}, {
        0.8952034196529891, -0.7980532651231155}}]}}, 
     {Arrowheads[{{0.021183581531526007`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9925688944745799, -0.607588882564306}, {
        0.8897840467018913, -0.662999352729812}}]}}, 
     {Arrowheads[{{0.02484493757949049, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9991270922053235, -0.45763951416233134`}, {
        0.8832258489711478, -0.530595779955316}}]}}, 
     {Arrowheads[{{0.030017486788541155`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.007361168466535, -0.30330086556316166`}, {
        0.8749917727099364, -0.40258148737801525`}}]}}, 
     {Arrowheads[{{0.03825714390929145, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0190515686074881`, -0.14067762816916182`}, {
        0.863301372568983, -0.2828517835955444}}]}}, 
     {Arrowheads[{{0.05575762470829352, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0464566397395045`, 0.04135856449703358}, {
        0.8358963014369667, -0.18253503508526925`}}]}}, 
     {Arrowheads[{{0.055757624708293596`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8358963014369666, 0.18253503508526903`}, {
        1.0464566397395048`, -0.04135856449703412}}]}}, 
     {Arrowheads[{{0.038257143909291476`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.863301372568983, 0.28285178359554414`}, {
        1.0190515686074881`, 0.14067762816916135`}}]}}, 
     {Arrowheads[{{0.03001748678854118, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8749917727099363, 0.40258148737801486`}, {
        1.007361168466535, 0.3033008655631612}}]}}, 
     {Arrowheads[{{0.02484493757949048, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8832258489711478, 0.5305957799553157}, {
        0.9991270922053234, 0.45763951416233095`}}]}}, 
     {Arrowheads[{{0.021183581531526007`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8897840467018913, 0.6629993527298116}, {
        0.9925688944745799, 0.6075888825643055}}]}}, 
     {Arrowheads[{{0.01842700829757587, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8952034196529892, 0.798053265123115}, {0.9871495215234821,
         0.7548879113474725}}]}}, 
     {Arrowheads[{{0.016272107256669024`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8997467931943, 0.9348203359521519}, {0.9826061479821714, 
        0.9004737816949062}}]}}, 
     {Arrowheads[{{0.014542990019468313`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9035880881225775, 1.0727412918205792`}, {
        0.9787648530538936, 1.0449057670029493`}}]}}, 
     {Arrowheads[{{0.013127617905417648`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9068588303052164, 1.2114625499414635`}, {
        0.9754941108712549, 1.1885374500585353`}}]}}, 
     {Arrowheads[{{0.012572289032024979`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1489884523844096`, -1.185221807578826}, {
        1.0863056652626495`, -1.2147781924211738`}}]}}, 
     {Arrowheads[{{0.013818604383719657`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1513725389813954`, -1.0411283268312463`}, {
        1.0839215786656642`, -1.076518731992283}}]}}, 
     {Arrowheads[{{0.015308797801352579`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1539714680036015`, -0.8961811984730088}, {
        1.0813226496434576`, -0.93911291917405}}]}}, 
     {Arrowheads[{{0.0171168985608574, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1567300144937083`, -0.7500484962460914}, {
        1.0785641031533508`, -0.8028926802244971}}]}}, 
     {Arrowheads[{{0.019351722288825802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1594967105519487`, -0.602228871864419}, {
        1.0757974070951106`, -0.668359363429699}}]}}, 
     {Arrowheads[{{0.02218366408088065, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1618983923648427`, -0.4519271824202017}, {
        1.0733957252822166`, -0.5363081116974457}}]}}, 
     {Arrowheads[{{0.025900636613884812`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1629740844968293`, -0.29779287790847897`}, {
        1.07232003315023, -0.4080894750326979}}]}}, 
     {Arrowheads[{{0.030997853497116947`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1597674886601212`, -0.13743550183666475`}, {
        1.075526628986938, -0.28609390992804146`}}]}}, 
     {Arrowheads[{{0.03754378372508136, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1405874804807077`, 0.030312146937918644`}, {
        1.0947066371663519`, -0.17148861752615432`}}]}}, 
     {Arrowheads[{{0.03754378372508138, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0947066371663519`, 0.17148861752615402`}, {
        1.1405874804807075`, -0.030312146937919116`}}]}}, 
     {Arrowheads[{{0.03099785349711697, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.075526628986938, 0.2860939099280412}, {
        1.1597674886601212`, 0.1374355018366643}}]}}, 
     {Arrowheads[{{0.025900636613884833`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.07232003315023, 0.40808947503269755`}, {
        1.1629740844968293`, 0.29779287790847847`}}]}}, 
     {Arrowheads[{{0.022183664080880666`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0733957252822166`, 0.5363081116974454}, {
        1.1618983923648427`, 0.45192718242020125`}}]}}, 
     {Arrowheads[{{0.019351722288825802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0757974070951106`, 0.6683593634296986}, {
        1.1594967105519487`, 0.6022288718644185}}]}}, 
     {Arrowheads[{{0.01711689856085741, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.078564103153351, 0.8028926802244967}, {
        1.1567300144937085`, 0.7500484962460908}}]}}, 
     {Arrowheads[{{0.01530879780135259, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0813226496434578`, 0.9391129191740497}, {
        1.1539714680036017`, 0.8961811984730084}}]}}, 
     {Arrowheads[{{0.013818604383719657`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.083921578665664, 1.0765187319922827`}, {
        1.1513725389813951`, 1.041128326831246}}]}}, 
     {Arrowheads[{{0.012572289032024979`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0863056652626497`, 1.2147781924211736`}, {
        1.1489884523844098`, 1.1852218075788257`}}]}}, 
     {Arrowheads[{{0.011912824053351105`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.322123944090053, -1.182863243041782}, {
        1.2661113500275947`, -1.217136756958218}}]}}, 
     {Arrowheads[{{0.012970645227741053`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.323631226697444, -1.0386512826326848`}, {
        1.2646040674202037`, -1.0789957761908449`}}]}}, 
     {Arrowheads[{{0.014198637077553295`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.325058899175584, -0.8936877666230362}, {
        1.2631763949420636`, -0.9416063510240228}}]}}, 
     {Arrowheads[{{0.01563161768674601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.326234520130412, -0.7477543071639721}, {
        1.2620007739872354`, -0.8051868693066164}}]}}, 
     {Arrowheads[{{0.01730893096661073, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3268389475994213`, -0.6005790121187431}, {
        1.2613963465182263`, -0.6700092231753749}}]}}, 
     {Arrowheads[{{0.01926581232382168, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.32627591521863, -0.45186426676773417`}, {
        1.2619593788990173`, -0.5363710273499133}}]}}, 
     {Arrowheads[{{0.021497164399658753`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3234239047892633`, -0.3014477590195971}, {
        1.2648113893283845`, -0.40443459392157977`}}]}}, 
     {Arrowheads[{{0.02382579744488329, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3163121475575015`, -0.149962257277909}, {
        1.2719231465601462`, -0.27356715448679725`}}]}}, 
     {Arrowheads[{{0.025567923001979943`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3028187769900255`, -0.0006590963847565162}, {
        1.2854165171276222`, -0.14051737420347915`}}]}}, 
     {Arrowheads[{{0.025567923001979947`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2854165171276222`, 0.1405173742034788}, {
        1.3028187769900252`, 0.0006590963847561138}}]}}, 
     {Arrowheads[{{0.02382579744488329, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2719231465601462`, 0.2735671544867969}, {
        1.3163121475575013`, 0.14996225727790857`}}]}}, 
     {Arrowheads[{{0.02149716439965876, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2648113893283843`, 0.40443459392157943`}, {
        1.323423904789263, 0.3014477590195967}}]}}, 
     {Arrowheads[{{0.019265812323821702`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2619593788990175`, 0.5363710273499129}, {
        1.3262759152186303`, 0.4518642667677337}}]}}, 
     {Arrowheads[{{0.01730893096661073, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2613963465182263`, 0.6700092231753745}, {
        1.3268389475994213`, 0.6005790121187427}}]}}, 
     {Arrowheads[{{0.01563161768674601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2620007739872356`, 0.805186869306616}, {
        1.3262345201304122`, 0.7477543071639716}}]}}, 
     {Arrowheads[{{0.014198637077553295`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2631763949420636`, 0.9416063510240223}, {
        1.325058899175584, 0.8936877666230357}}]}}, 
     {Arrowheads[{{0.012970645227741075`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2646040674202037`, 1.0789957761908444`}, {
        1.323631226697444, 1.0386512826326841`}}]}}, 
     {Arrowheads[{{0.011912824053351105`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2661113500275947`, 1.2171367569582174`}, {
        1.322123944090053, 1.1828632430417814`}}]}}, 
     {Arrowheads[{{0.011199963345734237`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.495225257393322, -1.1814021143664217`}, {
        1.445951213194914, -1.2185978856335784`}}]}}, 
     {Arrowheads[{{0.012072512361463282`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4959894145974617`, -1.0373317613561448`}, {
        1.4451870559907742`, -1.0803152974673846`}}]}}, 
     {Arrowheads[{{0.013051586754250093`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.496494803441805, -0.8926906884894482}, {
        1.4446816671464306`, -0.9426034291576109}}]}}, 
     {Arrowheads[{{0.014144384224217975`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4965462944148389`, -0.7473861027715781}, {
        1.444630176173397, -0.8055550736990104}}]}}, 
     {Arrowheads[{{0.015348449856734189`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4958410991792344`, -0.6013563382086448}, {
        1.4453353714090018`, -0.6692318970854734}}]}}, 
     {Arrowheads[{{0.01663799156292183, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4939270393022495`, -0.45464476801506376`}, {
        1.4472494312859867`, -0.5335905261025836}}]}}, 
     {Arrowheads[{{0.017936440162617952`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.490193925306397, -0.3075600887359119}, {
        1.4509825452818388`, -0.39832226420526495`}}]}}, 
     {Arrowheads[{{0.019074142779961934`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4840367364971196`, -0.16094326806740844`}, {
        1.4571397340911163`, -0.2625861436972978}}]}}, 
     {Arrowheads[{{0.019772769634296784`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4754386068137537`, -0.01630829113565277}, {
        1.4657378637744825`, -0.12486817945258291`}}]}}, 
     {Arrowheads[{{0.019772769634296784`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4657378637744825`, 0.12486817945258252`}, {
        1.4754386068137537`, 0.01630829113565238}}]}}, 
     {Arrowheads[{{0.01907414277996195, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4571397340911163`, 0.2625861436972975}, {
        1.4840367364971196`, 0.16094326806740802`}}]}}, 
     {Arrowheads[{{0.017936440162617973`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.450982545281839, 0.3983222642052646}, {
        1.4901939253063972`, 0.30756008873591145`}}]}}, 
     {Arrowheads[{{0.01663799156292184, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4472494312859865`, 0.5335905261025833}, {
        1.4939270393022492`, 0.45464476801506337`}}]}}, 
     {Arrowheads[{{0.015348449856734189`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4453353714090016`, 0.6692318970854728}, {
        1.4958410991792341`, 0.6013563382086442}}]}}, 
     {Arrowheads[{{0.014144384224217975`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.444630176173397, 0.8055550736990099}, {
        1.4965462944148389`, 0.7473861027715777}}]}}, 
     {Arrowheads[{{0.013051586754250093`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4446816671464309`, 0.9426034291576103}, {
        1.4964948034418053`, 0.8926906884894477}}]}}, 
     {Arrowheads[{{0.012072512361463311`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4451870559907742`, 1.0803152974673842`}, {
        1.495989414597462, 1.0373317613561444`}}]}}, 
     {Arrowheads[{{0.011199963345734237`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.445951213194914, 1.2185978856335777`}, {1.495225257393322,
         1.181402114366421}}]}}, 
     {Arrowheads[{{0.010477306125975746`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6685228903922633`, -1.1806826142523357`}, {
        1.6255947566665612`, -1.219317385747664}}]}}, 
     {Arrowheads[{{0.01118362862614915, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6687287781749205`, -1.036903222200876}, {
        1.6253888688839038`, -1.0807438366226534`}}]}}, 
     {Arrowheads[{{0.011949495453044802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.668604635339085, -0.8927382994446785}, {
        1.6255130117197392`, -0.9425558182023804}}]}}, 
     {Arrowheads[{{0.012768289697852268`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6679895577091002`, -0.7481808199035157}, {
        1.626128089349724, -0.8047603565670727}}]}}, 
     {Arrowheads[{{0.013622335338254661`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6666711075988743`, -0.6032788912927585}, {
        1.62744653945995, -0.6673093440013596}}]}}, 
     {Arrowheads[{{0.014475369768919472`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6643941654513221`, -0.4581847791459418}, {
        1.6297234816075021`, -0.5300505149717056}}]}}, 
     {Arrowheads[{{0.015263437675282373`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6609117998427898`, -0.3132195606184984}, {
        1.6332058472160345`, -0.3926627923226784}}]}}, 
     {Arrowheads[{{0.015890646781373575`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6561137917959785`, -0.16891437968875617`}, {
        1.638003855262846, -0.25461503207595015`}}]}}, 
     {Arrowheads[{{0.016244935134227048`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6502225169672, -0.025927078098396207`}, {
        1.6438951300916242`, -0.11524939248983948`}}]}}, 
     {Arrowheads[{{0.016244935134227045`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6438951300916242`, 0.11524939248983909`}, {
        1.6502225169671998`, 0.02592707809839582}}]}}, 
     {Arrowheads[{{0.01589064678137358, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.638003855262846, 0.25461503207594977`}, {
        1.6561137917959785`, 0.16891437968875575`}}]}}, 
     {Arrowheads[{{0.015263437675282363`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6332058472160345`, 0.39266279232267803`}, {
        1.6609117998427898`, 0.3132195606184981}}]}}, 
     {Arrowheads[{{0.014475369768919463`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6297234816075021`, 0.5300505149717052}, {
        1.6643941654513221`, 0.4581847791459414}}]}}, 
     {Arrowheads[{{0.013622335338254642`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6274465394599502`, 0.6673093440013591}, {
        1.6666711075988743`, 0.603278891292758}}]}}, 
     {Arrowheads[{{0.012768289697852282`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.626128089349724, 0.8047603565670723}, {
        1.6679895577091002`, 0.7481808199035153}}]}}, 
     {Arrowheads[{{0.011949495453044802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6255130117197392`, 0.94255581820238}, {1.668604635339085, 
        0.8927382994446781}}]}}, 
     {Arrowheads[{{0.01118362862614915, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6253888688839038`, 1.080743836622653}, {
        1.6687287781749205`, 1.0369032222008756`}}]}}, 
     {Arrowheads[{{0.010477306125975746`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.625594756666561, 1.2193173857476638`}, {1.668522890392263,
         1.1806826142523354`}}]}}, 
     {Arrowheads[{{0.0097764739834501, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8421432687018882`, -1.1805175136617894`}, {
        1.8049155548275244`, -1.2194824863382105`}}]}}, 
     {Arrowheads[{{0.010342744314054698`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8419687554783775`, -1.0370847000498948`}, {
        1.805090068051035, -1.0805623587736346`}}]}}, 
     {Arrowheads[{{0.010937677956452644`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8414635664124217`, -0.893416422471782}, {
        1.8055952571169906`, -0.9418776951752769}}]}}, 
     {Arrowheads[{{0.011550188090361797`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.840515212819351, -0.7495471606533741}, {
        1.8065436107100614`, -0.8033940158172145}}]}}, 
     {Arrowheads[{{0.01216112908340843, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.838996395100025, -0.6055585939745416}, {
        1.8080624284293876`, -0.6650296413195764}}]}}, 
     {Arrowheads[{{0.012740596380756634`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8367841825088895`, -0.46160070516923063`}, {
        1.810274641020523, -0.5266345889484167}}]}}, 
     {Arrowheads[{{0.01324656958214584, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8337998288808706`, -0.3179063323369845}, {
        1.8132589946485418`, -0.3879760206041924}}]}}, 
     {Arrowheads[{{0.01362784987194159, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.830069131209838, -0.1747783573885347}, {
        1.8169896923195743`, -0.24875105437617162`}}]}}, 
     {Arrowheads[{{0.013834467068875205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.825779259267429, -0.03252514961491329}, {
        1.8212795642619837`, -0.10865132097332239`}}]}}, 
     {Arrowheads[{{0.013834467068875205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8212795642619837`, 0.108651320973322}, {1.825779259267429,
         0.0325251496149129}}]}}, 
     {Arrowheads[{{0.01362784987194159, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8169896923195745`, 0.2487510543761712}, {
        1.8300691312098383`, 0.17477835738853428`}}]}}, 
     {Arrowheads[{{0.01324656958214584, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.813258994648542, 0.387976020604192}, {1.8337998288808708`,
         0.3179063323369841}}]}}, 
     {Arrowheads[{{0.012740596380756657`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.810274641020523, 0.5266345889484164}, {
        1.8367841825088898`, 0.46160070516923024`}}]}}, 
     {Arrowheads[{{0.01216112908340845, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8080624284293876`, 0.665029641319576}, {
        1.8389963951000252`, 0.6055585939745411}}]}}, 
     {Arrowheads[{{0.011550188090361797`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8065436107100616`, 0.8033940158172139}, {
        1.8405152128193512`, 0.7495471606533736}}]}}, 
     {Arrowheads[{{0.010937677956452644`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8055952571169909`, 0.9418776951752765}, {
        1.841463566412422, 0.8934164224717817}}]}}, 
     {Arrowheads[{{0.010342744314054698`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8050900680510351`, 1.0805623587736342`}, {
        1.8419687554783777`, 1.0370847000498944`}}]}}, 
     {Arrowheads[{{0.0097764739834501, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8049155548275244`, 1.21948248633821}, {
        1.8421432687018882`, 1.180517513661789}}]}}, 
     {Arrowheads[{{0.009116798619914601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.016130228034535, -1.1807338717514044`}, {
        1.983869771965466, -1.2192661282485957`}}]}}, 
     {Arrowheads[{{0.009569687754266346, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.015718081679951, -1.037643444328698}, {
        1.98428191832005, -1.0800036144948315`}}]}}, 
     {Arrowheads[{{0.010032604380582736`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.0150092145220806`, -0.8944240649719418}, {
        1.9849907854779203`, -0.9408700526751171}}]}}, 
     {Arrowheads[{{0.010494535920320265`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.013932337673443, -0.7511229581833889}, {
        1.9860676623265578`, -0.8018182182871996}}]}}, 
     {Arrowheads[{{0.01093952108451705, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.012418296106781, -0.6078195865070382}, {
        1.9875817038932195`, -0.66276864878708}}]}}, 
     {Arrowheads[{{0.011346123674529826`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.010415300919506, -0.4646317540788422}, {
        1.989584699080495, -0.5236035400388053}}]}}, 
     {Arrowheads[{{0.011688143720156707`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.007911382163011, -0.3217137711026357}, {
        1.9920886178369897`, -0.38416858183854113`}}]}}, 
     {Arrowheads[{{0.011937480299842701`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.0049592569440335`, -0.1792394247726037}, {
        1.9950407430559673`, -0.24428998699210255`}}]}}, 
     {Arrowheads[{{0.012069481079506493`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.0016912625304792`, -0.03736625861537353}, {
        1.9983087374695216`, -0.10381021197286215`}}]}}, 
     {Arrowheads[{{0.012069481079506483`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9983087374695219`, 0.10381021197286175`}, {
        2.001691262530479, 0.03736625861537315}}]}}, 
     {Arrowheads[{{0.011937480299842701`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9950407430559673`, 0.24428998699210216`}, {
        2.0049592569440335`, 0.17923942477260332`}}]}}, 
     {Arrowheads[{{0.011688143720156707`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9920886178369899`, 0.38416858183854075`}, {
        2.007911382163011, 0.32171377110263527`}}]}}, 
     {Arrowheads[{{0.011346123674529857`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9895846990804948`, 0.5236035400388048}, {
        2.0104153009195063`, 0.46463175407884183`}}]}}, 
     {Arrowheads[{{0.010939521084517033`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9875817038932198`, 0.6627686487870794}, {
        2.012418296106781, 0.6078195865070376}}]}}, 
     {Arrowheads[{{0.01049453592032021, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.986067662326558, 0.8018182182871991}, {
        2.0139323376734426`, 0.7511229581833885}}]}}, 
     {Arrowheads[{{0.010032604380582736`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9849907854779203`, 0.9408700526751167}, {
        2.0150092145220806`, 0.8944240649719414}}]}}, 
     {Arrowheads[{{0.009569687754266346, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.98428191832005, 1.080003614494831}, {2.015718081679951, 
        1.0376434443286975`}}]}}, 
     {Arrowheads[{{0.009116798619914601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.983869771965466, 1.219266128248595}, {2.016130228034535, 
        1.1807338717514038`}}]}}}}, GraphicsComplexBox[CompressedData["
1:eJx0nXdAVMcX79fea9TYS2JL7LHXwdh772IXKxZErOiKIJZYYu8So7FHVFRU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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV0l1ojgEYh/F72LLNhtfma6tZStQsJ5QtB6ZMTibbcjJCKAr5iEIcjCQO
RsqmkGKKWdgyi83SSEn5PDFGUtQwyrTNmN8O/l3PdX+9e989+eu3r9yWFBE/
JTE5YpI0ZEf0TYrYNC3iNzby6/IvPaJWbUBthQxKHR9Wv6E/elxEr/0f8l2y
x0cMmXmXE/EHb5lpkr+ez9obY76Zp+H5kTvqaySyIi7wdPVW/QyMKRFJMuzu
dHfHmOl2dzTeM5OMz3gKbnSjTS3L3iV3xqptVUvFyzxbvUN/Gl7haeq79Gd5
3o3lPqNCKuU2/+RuCy7ii6UoM+I+dki7vNVvw6GpEQXmuvhj/spcp5uFaq89
d/mcYs+d2CI1I3+f73RTzsgTaZdSs02JiP3u1OOxjIjjuI3XYjc/iOv4Dqx2
ZxGW8uX4QH8DVvBVeIDPxCI+D1+6Pxnn8lTcY//1RP97/hEX6g/hDN6L2+3f
xZSR74WP9BtxYHrERdxsfyu+5/uwQP80fuVHcb39cnzB12Kr/lJ8yBdilf1k
vMYTmKdfgHd4Hlba75ngHeH92KDfjSf5cyyzfw738quYGPnd8TBvwWX2q3EL
r8EL+ntwNd+MJfbn4xJegsn6VbiCl2Gx/RxcwGfjKf1MnMOTcL79p97J8fwN
9pv/hbn8MxbyJhzFO/CI/Xrs827U4Wz7G7GL78Qe8yfwCz+E+TzXe9Ps95+D
z/k379qgPJIP8lJSzf8HayuOOw==
          "]]},
        "0.42600000000000005`"],
       Annotation[#, 0.42600000000000005`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV01toznEcx/HfsuUwGxvbMM1wYYXcUEhqwxyyTSNhK6dh5BhTtrgYyeFi
FqEcUkwhOWUWIbYlSzYiUXO6mJo5beqZLHk9F5/n/f/8vp/P9/8c/s/IVVsK
N8eEEDK8dMSHUDU0hOmDQogdEMKy1BCKqJi+mV1ICaETj8j8kMmmi85+4vP0
EH5hDq0x78JVWEKr6ZJcMr636wO1Uj/3+C13zHnE3ssyV6jbWbN9EbzGz8c6
Pg8H6pTqrqcNFPqHcEMmDk/Z80+mlr9NS1zf01uKaXrb5XdQGfWXvyuTiOf0
eg8O4SH/gNbKP9Bbh8P1dsmXUwWlyj+SeUzxOjnOZtIMqnfWQF+pI7rbWQLV
2fONz3Q9l+bRHN35mE95FEd96VNiCE34lGbLHPTecnFBtIcF0R3Yad9Te3tw
dPR+ej3Oe7teir+ojU55v5/T3J/eUK8hvl8slP+YHEK1z/kajyX4/Lifb8KP
5s3672iq+9RjLVXZd17/Oh2nJrpPufI39cr1a/CAfYdwE38CW/nduILfipX2
TMNcfi4+Ml+NC/nFWMGPwin8BHxpfwqO4/tgmf6rJM8U/wknm//FEfx33Kx/
F+P4J9hofhW7h/m9sVR/I77nd+FY86P4ld+PK/UL8QW/HOvMZ2EDPxmL9GPx
Mp+EGeZj8Q6fgYv02weGcIaP4BXzVjzCt2C+/mncyV/CJPN63MvX4mz9SlzP
V+FZ8zIs5ksxW38i5vDZGGtehAV8Pk7VH4aT+DFYbZ6AWXzAifrPPNuJ/FuM
yHdhOt+G4/mbGMM/xH36NdH/q+fxJI7RL8F3/DZslz+MX/g9mMmne25u+f6z
sIXv8Oz9oUb6QC+pj/x/bqKpxw==
          "]]},
        "0.35500000000000004`"],
       Annotation[#, 0.35500000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV0mtozmEYx/F7NtvYZo+zjWHSlkNSZBbltIkYbSiR0aamMBQ5xAsk8cIx
FEYxOaQ0NbNohJRkcwpzHGViiC0me8FnL77P9/nd13Xd//v/704vWlVQGhVC
GOwnISWEB71CqMUiFOJTYgjfE0I40zOEFt6vp7l7COfks4jIL/TV4znikkN4
yVP1vOcEuYET+QNPs17VN4TpnGStyFoxlqLN/o3c1fpyXoGViHKGn9yM1ViD
r9ZiUeUMV9GhRwglzhLFy/iatX4c3TuEGHRALDqiv/VO3BnxSPO8BC51pvH2
ful8A60lW0vnIdbWqU0xl4NT8un2d00KoZzj+Yb1TM+u1jfL/9mYgTzMxAk9
xc70Sn09l+JXF9+D81FkdgKPQbn36tnHOZGIkcjEQv1zeRGyMQOTUWbviLkU
PLb/cG7DIDSZabN3rP/z+Qc+4qhnvPd+X/Gs/fvYp5Xz9b/rFsI+3+ApH7R3
De+Q7/E79Vrz9cj2rFtciT32O2X+Eg7hHq4jV3+FuY3my3mn/XbxCvkwv5Y3
82J5FW+1zzjOlafxTfUiLpDn8SY5ncfKI/iR/XvwMDmO15p/0tUdkhs4S/0v
95e/8Urz1Rwj3+U76he5NTWEk1xifjm/kTfwUPUD/EXewUvM5/NDuZCr1HP4
tpzFC8xH83k5wmnqQ/mKnMZzzH+OhHBc/s0X1F/zXrmO88wf43XyOY6o3+Jt
ciVPNb+Vl8l7uEx9LS+US3ii+VE8SZ7I0eoLeJacx9nmU3i0nMH71BM5U/7n
ro8yf5+T5Bf8W38zp8ofebhcwUGu4e3my7nFHT/CGeaLuV5ezZ/17+ZGeQsP
kFPdm8u+fybXyU3u3h/cwVs8Qpz+/wyvocc=
          "]]},
        "0.28400000000000003`"],
       Annotation[#, 0.28400000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1HtojmEYx/Ebbf9svBg7YOIPK5YUYSlGaCW0zXHCbDFyLOSQQyM5lEPO
x0jLKRKFFEsjJYU5yzk225Dl8A5z/Lx/fPe9f/d1X9fzvM+e9+1cPC9/bpMQ
Qn9/6hNCqEwOIS4Swh2O57vcPC2ERNy3foB7eISHaGV/XZsQonq3W+ckhTBb
rksMocHeW2eqUIgpKMIP+9XcwvxpXILp+G1/txkjzWitVmMviWs5z943Tpa/
cgOiSHW+kX/hJ/7E5iDd/kb3EXEfh61n6J8v18stkZjic2G5syvwxV4Kulln
4m37EN4g2ToF2RhqzmVzjlgfxTFUy2Ps9zf7ifMD+J298fZqeSzX8Diu40Je
2TaEU81D2Jwawk6sQxkO4HcLtA7hkFlRPumeaniP/ImL1Zc6V4qpmI9ZOGJe
gbnFeOo6i3gOvjmfw7kocm8DuDfKzG2jLx0J6IEMTHB+FE9EFoZhUOzezI/o
S8Vd8zO5EZ3xXk+j2XHW42LPGFXY6xqvPeMPeIgm5jRwrvMvfZYtPtN93mZ2
Oa+Rb/BL9Vux54ks16rgc9gU+1/qP40duIFLGOL8GX1L9JfxWvPW8yx5Jz+T
l/FkeS6XmtOPh8g5fEW9iPPk0bxU7sR95e5caX4Sd5PjeYH+e628R/Ir7qP+
k9Pljzxb/0VuJl/na+onuaFdCAe5RP9Mfi4v5q7qW7lOXsOF+nP5jjyJL6gP
5qtyHy7Q35SPyxHuoN6Vz8sdOF9/bcsQ9stRPqH+jDfLt3m4/n28UD7GEfUK
XiWf46H6S3m6vIkPqC/gCXIJZ+vvyQPlbG6qXsAj5OGcpT+Ve8ldeIt6AmfI
f31/e+q/yYnyY446/5nT5CrOlM/wP9+Vcl6tv4y/yLu4i/5ifiLPi/0+OL+B
q+Xl3FFO896c9fwz+Lb83rv3HdfwApWId/4/NA6/WQ==
          "]]},
        "0.21300000000000002`"],
       Annotation[#, 0.21300000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV021olXUYx/FroaVbnU2nbjZLrR0wCcHIRi9SaUvIBdMJMtNsy5Zu5oaK
U5LaQmdq28HHSkWRg2goomAmxQgdhYSaFaH4/EJzbqJMnNESpc9efM/3/v2v
h/u+D+eM/qC+vC4jIqp93MuKWD0kopvLh0VMR/bwiAQWOp+cG3G0IGISN8tr
UOL6e2fFfOvpiB6zm/S/JQ/Ijrhkx0C+zEOc52KtuV59W11P1bdIrsN/zr5y
Vuos28ww13kYijJnXfY/0tNpVxfqUI8Zaj94hnJeZ896zHY93Ny7fMfcU3is
N9/eBl6OSrU2c+9zq5kUal2PNFfD3WYGIZEX8by5z8w0olA9iRdRr2+DuRfU
c9TmuW51/mHfc3ALvkQKvwyNKLCvWi1LbyYWJCKa5Br+W/0mOlDirFT9BNox
ALV6kpxWL1bfxluQ80zESs/YhAasRzOW6Z/muU71fad82L0zPO8e7/wkz1df
Zz6Fu/Jqnqv3Uz5gZ2u+/ViDNHbgoZmHgyN22dHD++28yV/Ld7hKfYW+RszD
YtRij30V9lbhgnst449xX/8ULkOld3qDX0Xa3lxzI5CJcUhilv5yno0ivI3J
fc9mf8JcHn63fyz3YhQ6zfTa3c/1TL6L6/jGPa75nrrwF8KeB1ym/4p3SXmn
P3mj3W28Sj7JV9RPmz+PIvc6zkfQYt9u8wexGSfxI4r1HzK33Hyam+37gmvl
LXxR/oTfkxdxoz2vc7E8hX9Sr+Rp8gxeIY/k1+SX+az9g/kluT8vMf/HIP8h
+SpPUP+XR8i3eaH5Y/yE/DO3q+/nB89G7ORq8zV8SW7gMeob+Ja8iueaL+Pf
5Dl8VP1NPiFP4Iq+3xnvkxNcoD6Gv5MLeLr5jpyI7XIPf6t+kVvlM1xqfhsv
lfdyQv04N8lHuMR8I38kt/AO9SU8S67miebH8yR5ImeoV/A7cikXmc/jV+RC
TqlnclJ+5H8y3vyvnCWf4x793ZwvX+ex8iF+7L/Sxp+bT/M9eSsXmq/i83Id
d+hfyzfklfycnO93c9j3n+Qzcqff3j9ox2WcRX/9/wNoAMoG
          "]]},
        "0.14200000000000002`"],
       Annotation[#, 0.14200000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV03tsTnccx/GfW6iootbyPOIyjZJFQla1iEtUinRJbcIqo1Fd3ebWuhWT
ou6phqm4ZSKPZrTLhrBITMREVoKVZZmwmT/IzEbtUqKRib3OH+++f5/z/X2/
53eentOveMnkxa1CCPP8ad0phCdpITThMbqnhDA1NYQpeKy2pHsIBdbfxEP4
gKvkJNcP9QyhUI7zUn0teIGY/FS9Az6znmFPTzOLeRbK9JeixPq8mR/xTjnV
/lr7F8pv8iqz4vrap4fQT+6Lf+zphiPWC+zro17GF8wp5Wa1HjganUnebe6n
iLlW59pK15bJA6P51plczuN5Aurcsx5F1rPwnXUDLiNXXyU2IEUewJ/Ysxbr
5Q2Y69oc1EXP2Nnv5nyN6lM4NTmETu51zFlTeJX6mR4hnMVXOI2TkaPzcz2y
zByB4Vhk/6E3QsiwTnCOe+znGqSYvcbvtA4rsB2bsFzPJPe+6gx5fMLs4N61
ztCO56hv01+NJnkjF0bPxF+YucMZarAZCRzESz0vuzmLGc1cb+ZvvFd+wkXq
5fZVoBilmI9a8wrMLcJt91rOC/Cv/bmcj5meaSS/jUT0v9YXRxIGIwPT7H+f
P0Q2JmJMdDbzk/Wl4Yb5g7gFffBIT4vZbaynchPuY5973PM7/YEf8RrPkG//
Xc9S7Zl+4F1mn+NKuYHvql/TfwvZ0fvHp1Bl3mH9X2I3GnAWOfYf17dSf4I3
mbeF58s1fEdezTPkhVxhzjucI+fyefWZPEmezOVybx4mv8WN5nflgXJbLtN/
s2sIneVfOUv9BcflP/lj/We4lXyJL6rX87OYb5ZL9M/jn+UVnKm+k3+XK7lQ
fz5/L0/nr9XH8rdyFhdE7xkflZM5pp7Jp+UYv6f/YZcQDsjNfEz9Du+Qr3Oe
/v28VP6ck9Uv8Dr5FI/TX8Gz5So+qF7G0+QSHqV/CI+WR3GIvkV+V87jbP1p
PFTuz9XqSZwh/+c7GaL/CneUf+Jm+//idPk+D5KP8yvfyjlerz/Bf8t7uL/+
Ir4lL+KH9m/lB/Ia7iWne29O+P0z+Lr8yDv8HBfxCxrR1v7/ActSzAE=
          "]]},
        "0.07100000000000001`"],
       Annotation[#, 0.07100000000000001, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1HtM1WUcx/Gn9A9bSNwRSE1OIl5REa1FLpqbw4K8DWcyWVrzkhOal3IN
m4KXKbNFy8tk7oyhYabOC5ZoylzUyijXvGN/4DRqMV3YdC6Xvc4fb97n83yf
7/f3e86ew5AFlTOXPxFCqPDnm7gQxqWHMB5jMBZ5OG19FI/GcIzECJyyPoxz
8TxyMBSt1rM5gsEYgudw0vpAHoRMPIssfG19AGcgFelIw1fWkzkFCUhCIk5Y
b08O4XxWCN/xt7iTGsJdvJYSQgmuqb3OLfYW8GV5Ik/AC7giT+Ij/Z2XL8n5
PA4bMkLYiPWoRQ2Om9OkthdL8AU+R2V8CGXPhNBhzyxOMu9p77PPvHh+X71l
gDPiSxzF4ZjN28vNyE8L4UVMxDL7G5wj4nOUizxjB9cj3uw1zr8WK7EZNVih
p9Szf/AOxXzI7MdJITR6h77e4R31Tfrr0COv53J7P+T9Zm71DvWoRRS78FDP
QzMazOjlZjNv8WfyX1yhvtq+aryFSixGo3ll5lbgimet4KX42/4pXIL5zvQS
j0fU3ER9meiHUYhgjv3TeS4KMBWTY+9mfpy+VPxsfi4/wCB063lg9pM+z+Ye
dGG7Z/zme/oDv+I/3EOJ/Z3OUudMF/hjs1t5ndzOneo/xu4GCjzrTOy+YIt5
e/QfwCdoj91hFNl/UN8q/VGuMW8DL5br+ar8Ac+T3+VqcyZxkTyFT6vP51J5
Bq+WB/IEeQR3mJ/Aw+Q+XKX/l8QQ+ss3OF/9PmfKf/IS/S0c5HPcpt7M9zJD
2M0L9S/ia/JKzlHfxr/L67hcfwn/JL/Jx9Rf4bNyPpfF7lmCeyzHyRnqOXxU
zuA39N9W3yn38j71q7xVPs/F+nfwe3ITx6mf4bXyEX5VfzW/LW/hXepVPEde
yIX68/hluZAfx+4eT5OLuUBO5bFyNtfp78cR+V+/kzz93/NT8kXutf8Op8ld
nCsf5Ed+K638kf4o35U/5Wz9FXxJXsa37d/IN+U1nCWnuTeHfP+R2P8rudsd
/gdtuI4O9LH/f6NyxCE=
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV03tsjXccx/Ffa1FZFaWlF/5oE2pjLl23iBJFWJeUTaViRCOduCatMJdI
mdW1GqJCJ5PlTLBulLXSRUSQLS5d1aVIS0lGZrZMtqxkIgR7nT/efZ/P73t5
nuf0ORklZYWlMSGElf580yeEmIQQRqWGUNM3hJyeIUScnZA7uofQnhzCLfRP
CuF6egjp3Can8j05jd8xM61fCNNRiDZzN/Rc4+P29NOz086+3CBfdd6qXuks
2VmrPUl8zVlv/lXuw4PtHW9fFk/gZnMtei5wnT099FTYkcBH5PPOm9R/5sNy
d+cXo/34Fxf1JnrGJu6U031ep+9zNPr8I+p9F3H8A291PtqOLZzLXZzH4g1M
xgeYhDFq+fw2d+UhfNMzDOWZ7n2ws9c9PAcHeQSPxGL1afJC/h7folRfkbMW
1yzkRPfxpvs9ZF8Cr1RvTPH/wRE04FjUnvkg1yLb7lF4D0v07/OsmT5HOM81
argaCXav9r2uxXJsQQWWmZni2k3uIZ+P2v2qdwj73UMX9zBPfbP5KjySv+DZ
etfwd3Zucw/V2IAI9uKZmWd27LOjk2vt/I13y39xsfoKfeWYi1IswH77iuwt
RptrLeNF0f+n/olcgDmeaTSPRMTeXuZSEYchyMQM/R/xTORgMsZG783+eHNJ
uGx/Fj/FADw089TuGJ+n86PoO489rnHX9/QHWvESj1Ggv8OzVHmmq7zD7pO8
Xj7HHeq/RN8N5LjWaa5HpX1fmz+MnTiHE8jTX2fuM/MRrrBvIy+Qq7ldXsWz
5MVcbs/7nCdP5FPqc3iK/DGvkPvzu/Jb3GJ/Tx4kx3KZ+SuJfjfyHc5W/49T
5T95oflGfp0Wwk98Vr2WH8tfcYn5+XxLXs4D1bfz7/J6nm2+gC/Jn/Bx9XF8
Rs7mouh71st7LMfLKeoDuUFO4anmH6h/KXfyIfV23iY3c775Gl4qH+B49dO8
Vq7n8ebL+VO5kveql/EMuYRzzQ/jMXIuv4q+e/yhnM85chIPlzO4ynwcZ8rP
/U6Gmb/A3eQb3Kn/b06W73GWXMcv/FZO8jrzEf5H3sUZ5ov5pryEH+jfxPfl
1ZwmJ3tvjvp/ZHKz/NA7/ARncRstiNX/P8Ue1dg=
          "]]},
        RowBox[{"-", "0.07100000000000001`"}]],
       Annotation[#, -0.07100000000000001, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV02tMVgUcx/E/vbBFphkUD1rJU8gqRstYLwSLRmu0sBthXiZr2qTLFgo5
qbCZXYFq0RRZrj1z1XClm7hWc4qsfLRSipqmRa4wkVpOF7ac62Lzc158+Z7f
+V/OOZzzpJcsq2vKiYjl/hy/PGIUrYURT2MlnsUz6MuLmJEfcXBaRDH/ou8Y
rnU8oD42MeJoco53ydc4f+vkiKUFEbO5kSudr8DP+pp5up7lXMQtvMk1jqiN
2LFTvtr5Xueu4h3JnPPD6sMcl0R0XxGxDuX29+jb697W83d6DiFl7mNzh/Qf
kIf4Nf2d6ECpuXb9u829ykN6vsEUc/PU52MBBp3bY/Yhx1NRiBRKzN9gfymu
R5sdd3OW+9S3ocyu/fbvdu5BtZ/4fq7DA6jHLD3Vdn3u+A4u1fNvym7+DJ/i
Q/ki3mlnP/Z4/hfNZXl6ch21Ss4mNWxwz2nOcJW+bu7CRP2t3sUqtOAVrEHz
pIg5rv2Fe6jhLZ733GURG937BXY/ov6y+U6ckJ/nhcm3wZvs7HB/XXgBGfTg
rJmzdmywY5x77TzGa+XfuUF9hb42PIwn0YiN9tXb25C8R9dq5sfwh/5qrsUi
zzSLb0LG3snmUpiQ/N+Qxlz99/I8lONOzE7uzf5cc3n4yv4SPoMrMWbmjN3h
uI5PYATrXOOI/9Ov+Bb/4TRq9Q97lk7PNMRv2L2dV8tZHlb/MvndoNy1+nkr
2u17x/wHeBNZfIIq/ZvNPWU+w2uS98yNchcfllfyAvlxbrPnFq6Sq3mH+iKe
I9/HK+RpfLN8HQ/aP4lnyDncZP7rKREXyz/yTPW/OCX/xo+a/4j/n+o75AH1
Xj4tv82LzS/l7+UWLlZ/nY/Lq3mh+VreL8/nbeq38S55Jtcn39mlEe/JuXKB
ejH3yQV8j/lR9fXyOL+vfpg75H1cY76bl8nvcq56P6+St/Lt5tt4idzOPepN
PFdezBXmy7hSruBzybfHd8k1XC7n8Y1yEXean8Bp+W+/kzLze/lC+QCP6z/J
+fIIl8ib+R+/le38nPkMn5Lf4iLzDXxQfoJH9b/ER+VWLpTzfTdbvI8075PH
fMN/YgA/YBA5+s8DM1bYlQ==
          "]]},
        RowBox[{"-", "0.14200000000000002`"}]],
       Annotation[#, -0.14200000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV03lsz3ccx/GPiKOu0tJaZWkFWZZUtL/IoqToQtHNLWg0UuKctUVXWcqM
baIyUXHF9SM0ZCFq2UJdjSiJKrJMHCVzH8GW1UI6C2KP3x/PPr+v9/H5fn+/
fn89Z5ZMKG4RQhjpz6+JISxKDiEnPoQSHsoL+dMP1PCF6wWYi/mYh2z1u91C
+IrLUIqeXUO4rfYH7nUIoWXHEKJJIezEQGcW2rutvkPejm34RD1fvVH9qDN2
eZaI2mS1TJ7Cm9XGcbo8msdiDCLmR/BIDEMuhsf21C97hhKO90zF3Jkb1C7g
rHv9i1Xu/wOy7AzAGbVmHHG/5+ae4bB+NQ7htPqo2OfmlZzhzA/Vj+ME3pk/
xjU4in76K8zleu4q1+19H3Vyrf3hapXm27r+Rq8NBzvfquWb+Zr3m6/obg4r
EcUWNHdCgu+vRwhNvM8z3+cN8lMu0C81V47p+BKzsdt5E51bgKvut5Dn4m/z
OZyHaZ5lAPdD1Lmd7CWjFT5GGiaZH82TEcEwDIo9m/Pj7CXgovP78Cv0wCM7
r5z9HuPxDHew0T1u+e4f4ze8wQvkmW/0Wdb4TJd5rbNreLlcx4365+1fQcS9
TnI1VsfeL/s/YR3qcASDzR+wt9h+lFc47zueLVfyNbmMp8rzuNw5/XmwnMPH
9afxZ/IYLpVTOFP+iBuc35F7y4GL7F/qEkI7+SZn6L/kZPkJz7H/C79L8V5x
rf4+fiFv5UL7s/i6vIh76f/ID+XlnG8/jy/IU/hn/Ww+JWfwRPtvO4ewV46T
k/R78WE5iT+3/0B/s9zEVfrXuEKu51z7m7hY3sNx+id5qVzNQ+yX8wx5NW/R
L+JJciFn2U/ngXIWv429ezxCzuWInMB95VReY78Vp8mv/TbS7Z/j1vLv3GT+
T06U73Af+QD/57dSw8vsR/kveT2n2i/gK/J8fmD+e74rL+HucqL35qD/RxrX
y4+8w/+gFjfQgGD+f0favhM=
          "]]},
        RowBox[{"-", "0.21300000000000002`"}]],
       Annotation[#, -0.21300000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1Htoj1Ecx/Gj0GguG9tcagsjTGy5bQwrJjOkn1yamMuGPzCE2hjxByLk
OmllNLkPJWmWLOVOZIxCuSZyiT8Imdfvj/fez+d8z/ec59lznl+3eSWRpc1C
CH39eR8bQss2IRxODKEKS5NCWIK3xlsYr+kQQk7nEM5yVccQDiFbnm9OEeZi
eLsQ5vFxcwo5S57FmTybq43f6BrCUT5gjwoMVpuh1mifPfJu7EK68anGG4z/
ww5j2zHBWD7uGfuD1e5jp/XOuZe78n453fUoc3KQjZEYgX7Gh/BQDEJPewzm
Mv3j1K7wSXucwmn0sdYalCFWfq4+yfU61PifvJHveZ63fBm9jddbp7XaEdet
+KpcqzbGXkUJIRRjrVoBRzATj9RLeAE+t3XfnIcC84Zyf1R6tjadQkhEc/RG
CqaYP4GnIgOjMQz77B2jLw63rZ/KP9AFb/T8sPY/TMYHvMBuezyLvnPcx298
RZ75jfEhbPGsd3mbtS9yuVzPjerX9T9Ehr1q+Qw2We+g/mPYjnpciL4L80/o
W66/ktdbbwMXyTu4QV7J0+WFXGqdgTxCzuFL6gU8Xp7IK+TOnC734lvWj+Ue
clNcCIv13+FW8lMeoP6dE+V3XKz/PP/t4gxwnXo1f5UruFD/fH4sL+Pu6lv5
tVzOM/Tn8U15Gp9Vz+bL8gCORM9qe9+WHCMnqHfnGjmB8/W/Ut8rf+Ej6g28
Wb7BudFvg5fIVRyjXstl8hkeqb+U58ibeJ/6Yp4iF3Km/jTOkjP5j/kRHivn
coYcx/3kZN6ivzmnyD+d2zT917iF/IC/mP+R4+UXnCqf4F/O+kVeo7+SP8k7
OVn/TH4oL+JX5m/kl/IqTpLjo9+e95ES/Y2IflvO8DfU4QluoQn/AeKSq1g=

          "]]},
        RowBox[{"-", "0.28400000000000003`"}]],
       Annotation[#, -0.28400000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV03loz3Ecx/G3oyjn2NxbzfXHsC3SyGwKhZTIkSMK8cfkiD+QZDRDbqPm
PkqOTa7lzpFjrqi5ytSGEoUoNDIevz9ee35en9f79fn+vr/fZ+kzF4yb3yAi
2vpzpm3E/c4RZ/FAu4j9lN8qYl77iHfNIxq3iCiT5XWMKMd8zJPvT444wVfq
nsRl/HG8Kq/V28cPtJ7qnGk0hXL0jpi5ZL/aTENnLzV32N5Fe6/t7eX38ned
uw+3+jxbKEt3vDOqzPylHbL+OiW4W6eUNlnf1tuM63SKKUNvqN59nToqkmXp
rcWdOiW0yvqmXiGu0VlN3fX66d3Q+UHnZY3sJctSqNb8dfu9rftQjfyRvVr8
lxJRTxXe7y0/RN4l8Q6USXl0yHPPy9OsB5vpY76n9duWEd9lf2kMfaBq2uZZ
rxK/CT2mOvpCI8y/aONdPfshbnBmBa7gb+AL+R39p5Sd+O6xjIqcV6p/lDYm
3pPOUa75Y3oL9ffgSuetwln8JqziF+NEfg4udU5fzOXz8YJ8Mo7kR+MivgNm
8T2w0vnNsCtfnxRRoP8Am/IvMVP+DVP49zhb/zT+6RRxDa/Ij+AXfhdO15+J
z/gFmC5fj7X8CpykPwLv8RPwlHwQXuYzcaz+79YRB/kmfLI8Hcv5ZBylXyPf
wX/GQ/IqXMvfxWH623EefwCbyC/hMr4MByfuOM7gi7BEXoDj+OmYo5+BA/gc
/G1+LA7nh2E23xp78alYrN8I0/if7mOG/i1szD/Bz+Y/YhJfjd34Y/jLna/A
5fp78BO/GVP1p+BTfi7WmC/EN/wSbMcnuTfH/R5pif/LxJ3uEPGVrtBzqqR6
+g8uf6cM
          "]]},
        RowBox[{"-", "0.35500000000000004`"}]],
       Annotation[#, -0.35500000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV0ltoznEcx/HfpqYWNraZw1Ybbqa2JdpqW26mGDcjxMWUibKxyaERM2WW
bC62JTuUYxrNqG1plhxaM7LIyAUycjM1kpDJ8HouPr1/79/30PP/P//00sp1
FVEhhG9yMSGE10khtCeGkDs3hDaeg+1Y5e7ftBBanFslKy6ERuxRf+7+jyx1
LkoOYbU0zA5hGW/S04sj6mftaJEGdzfdPXH3W07xgfkh1GOGvcexS31I7Zdk
OufZmS919mbxWj038JF6s51H+X07anA/PyBHnKtloZ2H8Jr+u/q/S5S7QR6N
idPt1N9td4908SgsUy+XnbJbdkm1jHlHPXoGnN/ZWxOZ5a2zQhj2GydneDbz
f3EYY/UVRN6D+mWz2c59ZjqlNvJOzVyRerkn3ZFnNduhv9IztWG1/TVYyhtw
hO/FDXw7VtmzBPP5crylvglX8TW4hydjFl+EQ/bHYjqfnOlZzT/GqfwVZqp/
xUT+EbeZv4m/54VwB/vVL+FnfgZLzG/FF7wC09RP4nt+GDeaX4kP+XrsUs/D
2zwTi81PxIdwnsfwBPU0vM4TsMj8qHoTH8cL6iN4gg9ioflGLOfnMEa9Dw/y
TiwwX4VbeC02q5fhWl6COeYzMJfn4IT+YlzBCzGbx+FinoJ15qMxlf/wDWWY
f4BT+FMc1z+G8fwNLuAd+NO30Bv5Hs234Sd+GlPMb8ZnfAeO6j+Gb/k+TOLx
vpur/o9UHOQf5oTwRfrlpQzJpPwHngCQpQ==
          "]]},
        RowBox[{"-", "0.42600000000000005`"}]],
       Annotation[#, -0.42600000000000005`, "Tooltip"]& ], {}, {}}}], 
   GraphicsComplexBox[CompressedData["
1:eJx0nXl8U8X398MiLiyCuLEoCorI4hdkEQE5RUWQVQQUEURFUEAB2VGEIgrI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    "], {{{}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], 
         LineBox[{3319, 4027, 3318, 4333, 4332, 4721, 4720, 3998, 3660, 5049, 
          3659, 4028, 3323, 4979, 3666, 3999, 3668, 5050, 3667, 4031, 4032, 
          3675, 5052, 3674, 3459, 3329, 4983, 3676, 3531, 4514, 3532, 4515, 
          4731, 4730, 4035, 3338, 4991, 3689, 4004, 3462, 3339, 4992, 3690, 
          3537, 4527, 3538, 4236, 3700, 4271, 4040, 4041, 3708, 3353}], 
         LineBox[{3425, 5024, 3813, 4021, 3495, 4379, 4378, 4270, 3636, 4217, 
          4218, 3823, 5068, 3822, 4065, 4066, 3834, 4862, 3833, 3501, 3435, 
          4699, 3645, 4461, 4462, 3840, 5071, 3839, 4069, 3443, 5038, 3845, 
          4024, 3504, 3444, 5039, 3846, 3649, 3853, 5073, 3852, 3506, 3448, 
          5043, 3854, 3654, 4869, 4870, 4868, 3855, 3507, 3454}], 
         LineBox[{3716, 5093, 3717, 4098, 4097, 4337, 4336, 4205, 4204, 4203, 
          3542, 3699, 5057, 3698, 4093, 4094, 3912, 4999, 3348, 4090, 3337, 
          4388, 4387, 4386, 3534, 3682, 5054, 3681, 4085, 3903, 5078, 3904, 
          4083, 4082, 3328, 4982, 3900, 3673, 3899, 4981, 3327, 4078, 3322, 
          4978, 3895, 3665, 4899, 4900, 4334, 4335, 3321, 4077, 4179}], 
         LineBox[{3805, 5066, 3806, 4139, 4138, 4844, 3804, 4295, 5092, 3633, 
          4683, 3634, 3943, 5025, 3426, 4142, 4143, 3949, 4951, 3948, 4952, 
          4146, 4857, 4858, 4690, 3640, 4689, 3641, 3956, 5034, 3436, 4151, 
          3441, 5036, 3961, 4154, 4153, 3962, 5083, 3963, 3841, 3969, 5100, 
          3968, 4158, 3970, 5085, 3971, 3847, 4963, 4964, 4960, 4962, 4961, 
          4162, 4163}]},
        "0.22999999999999998`"],
       Annotation[#, 0.22999999999999998`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl1WtM12UYxvFHgTSlUqjVi7L+VGqbLZaglJBnm6bYCbUFiOAB8YQlICqa
lWKHlZu9sEwFO61VptVsrtnB1NQNdctOaic1NMXUDmYJ1edZLy6+z3Xd9/38
zn8SpbPundkuhPCSP893DeFqejgthEcuCWEO5l8awguy7fw1WCWrptHyVfwO
eTeskc2lu+W1+KJsNV1L16WHsFhfAk9cFkI2ZVj3wRY6SW2pITTq3aWvAXfj
bfLj+nLwNJ2h22mO7B/9Z62T9NXyedYljn0HzuOT5b9bd8A6fpB1ufpgXMh3
lP9lnYZL+GHWlep34lI+Xd5qPVt2Ez/SugfehT0xdAnhCdf5JO1xvt/pb8Lv
MV/P03oS1in6HrNHX/4eeVa8bsqmzmprzB82d4R+pJ/oKL0sX2GuxfqMfU7i
WTyFa+XbzTfgBVk/WS7lRfLJZpPitfJDaCgNlKfJO8lH8qMoP9bic5VfKb+P
v58+td5p/62YoZbABGbg9diEezGb+lAW5VBf+oG+dZ0P2ne1PQ7ofZd/jwpl
a2QHZbfQcL2ZOALP0R9xXt8Ufa/qa1HbxL8fn53sNdmpeC/pT2riz+GbahX0
On+ez6U22sO34nq1SnojvjfxHaEke+7j2+Pbag/RW/G94Y9RM9XI3onPKb5n
WOneDDBb6DwD/y9u1LNZvRO1k12M7fE41qlN0zOdyqmCptIC+RQcaq9JOAQn
43z5RBzMl+IgLMN58hKcQEU0noqpVj7QO1SNGx3zccccwPenYn48FVEhzXLu
/e3XUc8GvspMrr48WiQ7JFuI/fgF8Zvk6/CCmbl4gK/BHPUqPMFXx29SfTZ+
E+8P9lafiT/HY8Z7qV6BX/NTMVO9WVZuvU5W5jx6yYqw1DlOoBLqqedL2qXn
C+yhZ3+8H+ZWyQri74lsNK6lB8yMoxvdlxtouGwsP4YK6Cqz3eTD5CvNb+ZT
KV3WGS/HDdgBUzAFk/EiXI9/26PN7Hlsjb8d2Cg/jb/xv+Cv+Jxshf0zrbNk
velWOsY3x+vCZ+hm6160j/ZShuwpWmr+Wcesx+Xxe5Dtju8Svy0+I+wer5O+
UtuEV2A9VanVx99nXIafy3aYnc5/bHYGvsJPw8V8qvo2fiL/ET8J1/FlOJ9P
Ud/KF/Nb4vuEjXxRfCfit6P+CT+W/4Afhw38o/IC6zH0WfzmnePO+JzivfO7
vUXPhzQqvjvxm5Utit+gc17Z9f//Q/8BJM/mMQ==
          "]]},
        "0.184`"],
       Annotation[#, 0.184, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl1Fdsj1EYx/FTRJB0EVvELCVG0Yo9YodqUDNKhFKjTbiw90jFjBER68Kq
Ulu4NCIuam+JESrmjV2xP29cfPs9z3N+zznvv2/7rz8hb0huTAihsx/ZcSEs
rBzCYizC5fgQ4mJDOJQYQiEO4oa6gGN5WZUQcs0ckFspvwrLsALLcUW/utwJ
+ZM4jrvqOWYump3N+epqyFfPVT+0Piv3iM/xWuesQbGzGuqtk8uXO6guQCO9
9Xqr9V5Y3zJzB7exx9w1mRT9LTJbZQrVm9Vt8EHmI1KRhvYR+j/5D3flbuiO
LuoKzivvjD7qvuiHnvpV9WP1B6szsEOvL2omhFALWe7s5v6x3J3r6l0w0wRN
kYSG5sfb324/w2wDmevRmeqJ+gM5HYPQ1N5zMy/QEclyHbgZZ8vukclyRgu5
ibxX/cZeW+R4viHqAv2hnCIzzcww6+HIxFu5T3ht/ZHTnZ3K08yO0is0O5rb
mZ1pdoz1LE5Tp2KG/WFmMjE0ekazwzlXfyR3Uo/g0RiFPP0S8y/x2V6B83L0
Dkfvwv5gZGC6+3/xb3yR+4o0vMM+M5Oiv1WU2h+IQZhq5ju/l9kvk23/mXtK
1MXqlzxFZjJaWveLniv6neFhdLb6Eb+P3hc/Nnubn6rv8DMeZzYLba3bIN3s
bhTbv66+yg/M3cd5dW3UQU1Us1eLz3EiV+R4rsAJfIzLchmO4Rguw0Vc6s4f
7vnC3/A1+j+1l+gdVEam+7b5jGkycepWvAkpcq3xynqX/AZn7eT1XEluHWeY
3Wi2iUw5vXrc3EwzJOOJer7cJZl5/Fe+Px6oB/A9+6fkzuA0akTvUj+RV8j0
ts4z14uXqxP0j8rdjN47F+EI4tXfZcrzArke1qX4hiT1Sv3GXF/2mvWB6F06
d6nPswT11PuxDwui76bo3cb//677BxM2rQk=
          "]]},
        "0.138`"],
       Annotation[#, 0.138, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl09tPz3Ecx/GPubBEySGHEMPoqJgNkU7kMJuaMcYyp0v+BIe7Elsox8xM
xrjAnZCcSVw5TOWQCBdOldPm9PjOxXPPz/v1eb8/n+/v+62xazeXbuoVQviB
+oQQJmLXwBC2xoVQMSiEpP4hXJK18EVuwGWMVDfyVVzBdVzDKPlNvoUbuIPb
GC1v4nu4i1Z1MyfzDvdsc1+b9SRZqrrCOo17DwhhcXwI6daZyMB2vVN4KiYj
G1mI0Vtp74DnP4SDeOOcTGdORlW0Rn9907jGzFJnHzYzQFbLJ80c4VP8U890
czMwEznqIA/289UF2C/Li86Ux2GBbCEWoUg+TDZEf4m6NHpX6qvqBm7kR+4f
w/fVTdzMd7nW7GJkqbOxyuxqtFk/w1O0oiWyM1boHW5/GF7LOnBe/o7fR+8B
b9GJ5/I1+udZF2MuMtz5Cu3oUXdjY/QbuBBdOGHmC2+Ifjvn4zPq5J94vTyP
5+Ajjss/8Dp5unNzrV9wp/f+ktOQI5uNWSjTn+I7LOfDejZ4zid6lliXINE5
Q7HM/jh9JTxEPRjz7RdjvP4UZ6UhFdeQrLfAXiHyUebcIq7Xm2Svn/lYrJTn
yuPlidH35KF8jvtxH47hWPTFWfwx9xe/Inue33xMXzd/VXdxD9fp3Sf/5r7v
OOOu06iRVUd/y3oeqI/qe2TmMR6iQ75XVh19A3PdmCCrRIXZKvke7Ea7bIus
3HkjrMuxU34h4f//9j/ClIx7
          "]]},
        "0.092`"],
       Annotation[#, 0.092, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl081TzXEUx/Fvm7JA3RgzVmWGOyOlhRYGS4yyiDAMK/0BNWPDzmMsQldG
DBbKjKdkFMpTbixQmelJ3QqJMeOywQxrXt+xeM/nnPM533N+93t/vyW19TV1
OSGEGTTOD6GlMIQLOI83+SGsmBfCdCKEd8hgCpMoUT+zIISUM50FIdThNE7h
Lpr4y/FH/BtZZ74iq/aNfkcq9qELQ3at5gX1v/Lb9rejQ/0OctQ75Ffte2nv
YT1H8MiZanqS34ij4jL947SPl6Vn5cdoA45jCx7zTtCtKOfvoJfoWrPXodOu
Lrse0O4Y4z7uYZHeHvqcvkCxuAiL9W9wdiMKPEsCVfYU66mhN+kt3MAyvaPO
jGEYQ/IRWooy7Ha20pwrzlXRpc7s9XybxbU0GWdgF39KPo1hMy7yt6m14q05
K1GOPeaN0zb1EX0T4klkkJS3m9UR7x3b9Uya95rXj1cYxAAO8PfjOq5hk97L
SJvxhN+Lp1goHzOjm6bjfdE+OogBVMb/wjM14JC4RO8o3Un7aWHcTxM0z29a
L35oz0G6Sq1HLS2vED+Ld2R+t1qv+JyZLchFHtaoFfEqoodP4lnsM+sX/Ykf
6m3x/UIrms2by88gxfusZ0Jcjy/x3ec3q5WJP9oxi/eYwQeUqs/Rm6RN+qrj
75DnxvuN30l8H/L/f3P/ANKcigI=
          "]]},
        "0.046`"],
       Annotation[#, 0.046, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl07lPlFEUhvGr1goyJhqdwVBrjFoJiX+A0WpMaHBJiCsuaCGNFoosamQZ
GAZwSVwAC43GCG4szTBgjRQuiSZamEGhNK7E343Fk+e+55x7v/t9MBW19emT
S0IIeRSXh3CqNISnGCoLoX9FCL+sf6MbC4kQ5sx0WX/lLP9BDt/kHp7nXl7E
X1xzxj1nLaiflt/gLd7jHUpQ6dx0SQjbeDdX8XX7JvQe8Bjf53Heot+HYTOj
zkyrjfGIvFW9BfWet8P8+ZUhXECdXpt6o3UTLmJe7araF/u3+wCveaM8y1f0
L6NdPmbuknUrNshJOcFrzI14xn4e5tU8o17HR+P3ifdST/Fj9ZfuNCE/53F+
wbPqu9w94exBc+vVB7icp/RfcYGnY1YvoAEp+S5P8STncQZJ+Q5Pct6+Quzz
Or6lvpZvc5nn7fTcpPURHMYh7HGfzzxq7hPXyAe5iDn0yzdxA9XyKp7xPZrj
+6HcejNnnN+FE3Kv3G2dxT57snIOH/V6uIhOfJAz3IG95jrNd8Rz5ON67dZt
2CSn5HNmzmLaesh9H3nPpfrLkPNuD+UB9WdmBuN35+/x/4T75AP8Az/jndSf
xL8jMqX/fwf/AMt3ffQ=
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0jlPlAEQh/Eh0QTQ3X13l+UUFU/4CBZqYRTiVVAoqIVivAoX5fAGSzk+
ALBepUejiaiolUcpKlE7vOLRmqiJlP6IxZNn5j+zs+8eTV3d7cWyiJjC31RE
bxLxGE9wKx0xwXPyPp7FB+zKRHzkPKpw296CbMRCrLNbzhs5nY/IIIUar9lh
dzsemC3T18iX8wq8VTfxa/NXKOYi2tydkg/oB3FUPWLnnPvncQEd8k5cku/h
3bgoH8QARuXHvO6HfBQvvPeY/Dl/lc/IVtt5ww36eqzS1/JKruOdZq34rW7j
WnmirkYBS/Ul9zah0u0K3PPsM/N3kdGncX3+O5Nf5Rxf45T8Cmf1JU74j5uX
1Z/wU/2ZO9xZr+7kL/pqXqRfjGn9S3zDIf2E57hrfkC/H43qLbwZ982q7DTI
8ryEW/Be3czDdp7aGeJnPMLfccR82Py05z2Ds9gnf2hnL5/S96MPQ/YO20/4
uN/xl7rV5zpp1oNerDF7J1/L9ciq6zjHk25udXMbCrJyWSUqsMGsIO9Wn0AR
JTsHZV246X3u6MfsPdLf0I8n///b/wCWqlt5
          "]]},
        RowBox[{"-", "0.046`"}]],
       Annotation[#, -0.046, "Tooltip"]& ], {}, {}}}], 
   {GrayLevel[0], AbsoluteThickness[3], LineBox[{{0, 0}, {1., 0}}]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.75, 
      FormBox[
       RowBox[{"-", "0.75`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.25, 
      FormBox[
       RowBox[{"-", "0.25`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.25, 
      FormBox["0.25`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.75, 
      FormBox["0.75`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}, {1.25, 
      FormBox["1.25`", TraditionalForm]}, {1.5, 
      FormBox["1.5`", TraditionalForm]}, {1.75, 
      FormBox["1.75`", TraditionalForm]}, {2., 
      FormBox["2.`", TraditionalForm]}}, {{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2, 
      FormBox["1.2`", TraditionalForm]}}, None, None},
  ImageMargins->0.,
  ImageSize->504,
  Method->{"TransparentPolygonMesh" -> True},
  PlotRange->{{-1.1536749816983884`, 
   2.1536749816983884`}, {-1.3536749816983884`, 1.3536749816983884`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 419},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[10]=",
 CellID->321588231]
}, Open  ]],

Cell[TextData[{
 "Arrows for velocity field are green, equipotential lines are red, \
streamlines are blue and panel is black.  Note orthogonality of equipotential \
lines and streamlines, streamlines parallel to velocity field and clockwise \
circulation associated with positive ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ".  Thickened equipotential line along negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis suggests a branch cut for velocity potential.  Also, note that the \
velocity field shows an abrupt change in tangent-component velocity as the \
panel is crossed."
}], "ExampleText",
 CellID->680409942],

Cell["\<\
Potential-flow singularities may have discontinuities in velocity, velocity \
potential and stream function across the panel.  Such discontinuities are \
called jump conditions.\
\>", "ExampleText",
 CellID->62898067],

Cell["\<\
Compute the properties of tangent-component velocity across the panel.\
\>", "ExampleText",
 CellID->673890120],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"icuMinus", "=", 
   RowBox[{"Limit", "[", 
    RowBox[{"icu", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", ">", "0"}], ",", 
        RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
       "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"icuPlus", "=", 
   RowBox[{"Limit", "[", 
    RowBox[{"icu", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", ">", "0"}], ",", 
        RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
       "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", 
      RowBox[{"-", "1"}]}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"icuJump", "=", 
   RowBox[{"icuPlus", "-", "icuMinus"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"uJump", "=", 
  RowBox[{"a", " ", "icuJump"}]}]}], "Input",
 CellLabel->"In[11]:=",
 CellID->98488054],

Cell[BoxData["a"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->304898429]
}, Open  ]],

Cell[TextData[{
 "There is a jump in the tangent-component velocity across the vortex panel \
equal to the value of the circulation density function, which is the constant \
",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 "."
}], "ExampleText",
 CellID->9527100],

Cell["\<\
Compute the properties of the normal-component velocity across the the panel.\
\>", "ExampleText",
 CellID->134318740],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"icwMinus", "=", 
   RowBox[{"Limit", "[", 
    RowBox[{"icw", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", ">", "0"}], ",", 
        RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
       "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"icwPlus", "=", 
   RowBox[{"Limit", "[", 
    RowBox[{"icw", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", ">", "0"}], ",", 
        RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
       "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", 
      RowBox[{"-", "1"}]}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"icwJump", "=", 
   RowBox[{"icwPlus", "-", "icwMinus"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"wJump", "=", 
  RowBox[{"a", " ", "icwJump"}]}]}], "Input",
 CellLabel->"In[12]:=",
 CellID->326115239],

Cell[BoxData["0"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->61288648]
}, Open  ]],

Cell["\<\
Normal-component velocity is continuous across the vortex panel.\
\>", "ExampleText",
 CellID->531709516],

Cell["\<\
Compute the properties of the velocity potential across the panel.\
\>", "ExampleText",
 CellID->211831706],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ic\[Phi]Jump", "=", 
   RowBox[{
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Phi]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
        "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", 
       RowBox[{"-", "1"}]}]}], "]"}], "-", 
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Phi]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
        "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[Phi]Jump", "=", 
  RowBox[{"a", " ", "ic\[Phi]Jump"}]}]}], "Input",
 CellLabel->"In[13]:=",
 CellID->189498322],

Cell[BoxData[
 RowBox[{"a", " ", 
  RowBox[{"(", 
   RowBox[{"x", "-", "\[Delta]"}], ")"}]}]], "Output",
 ImageSize->{55, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[13]=",
 CellID->258993274]
}, Open  ]],

Cell["\<\
There is a jump in velocity potential across the vortex panel consistent with \
the jump in tangent-component velocity.\
\>", "ExampleText",
 CellID->56731244],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uJump", "\[Equal]", 
  RowBox[{"D", "[", 
   RowBox[{"\[Phi]Jump", ",", "x"}], "]"}]}]], "Input",
 CellLabel->"In[14]:=",
 CellID->307784515],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[14]=",
 CellID->460749900]
}, Open  ]],

Cell["\<\
Compute the properties of the stream function across the panel.\
\>", "ExampleText",
 CellID->403496840],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ic\[Psi]Jump", "=", 
   RowBox[{
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Psi]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
        "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", 
       RowBox[{"-", "1"}]}]}], "]"}], "-", 
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Psi]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
        "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[Psi]Jump", "=", 
  RowBox[{"a", " ", "ic\[Psi]Jump"}]}]}], "Input",
 CellLabel->"In[15]:=",
 CellID->102020424],

Cell[BoxData["0"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[15]=",
 CellID->48664304]
}, Open  ]],

Cell["Stream function is continuous across the vortex panel.", "ExampleText",
 CellID->325279064],

Cell[TextData[{
 "Look at the properties of the velocity potential as the negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis is crossed in a counterclockwise direction."
}], "ExampleText",
 CellID->635547329],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ic\[Phi]BranchJump", "=", 
   RowBox[{
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Phi]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"x", "<", "0"}]}], "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", 
       RowBox[{"-", "1"}]}]}], "]"}], "-", 
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Phi]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"x", "<", "0"}]}], "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[Phi]BranchJump", "=", 
  RowBox[{"a", " ", "ic\[Phi]BranchJump"}]}]}], "Input",
 CellLabel->"In[16]:=",
 CellID->81019929],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "a"}], " ", "\[Delta]"}]], "Output",
 ImageSize->{29, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[16]=",
 CellID->44369461]
}, Open  ]],

Cell[TextData[{
 "There is a branch cut along the negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis for the velocity potential.  This abrupt change in value of the \
velocity potential as the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis is crossed causes the thickening of the equipotential line unless \
special care is taken with the options of ContourPlot."
}], "ExampleText",
 CellID->78970066]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[{
 StyleBox[ButtonBox["ICVortexPoint",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICVortexPoint"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICVortexLinear",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICVortexLinear"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICDoubletConstant",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletConstant"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICSourceConstant",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICSourceConstant"],
  FontFamily->"Verdana"],
 " "
}], "SeeAlso",
 CellID->745892494]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["2.1 Introduction to Influence Coefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"]], \
"Tutorials",
 CellID->152273840],

Cell[TextData[ButtonBox["2.3 Isolated Panel",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/IsolatedPanel"]], "Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["InfluenceCoefficients",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/InfluenceCoefficients"]], "MoreAbout",
 CellID->110102918]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"ICVortexConstant - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 1, 1.2444000}", "context" -> 
    "Aerodynamics`InfluenceCoefficients`", "keywords" -> {}, "index" -> True, 
    "label" -> "Aerodynamics Package Paclet Symbol", "language" -> "en", 
    "paclet" -> "Aerodynamics Package", "status" -> "None", "summary" -> 
    "ICVortexConstant[r_f, r_p] computes velocity influence coefficients at \
field points r_f, due to connected vortex panels of constant strength whose \
endpoints are located by r_p.ICPhiVortexConstant[r_f, r_p] computes \
velocity-potential influence coefficients at field points r_f due to \
connected vortex panels of constant strength whose endpoints are located by \
r_p.ICPsiVortexConstant[r_f, r_p] computes stream-function influence \
coefficients at field points r_f due to connected vortex panels of constant \
strength whose endpoints are located by r_p.", "synonyms" -> {}, "title" -> 
    "ICVortexConstant", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/ICVortexConstant"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[9688, 296, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->103271997]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 237850, 4943}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[609, 21, 76, 1, 70, "SymbolColorBar"],
Cell[688, 24, 191, 5, 70, "LinkTrail"],
Cell[882, 31, 3576, 82, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4483, 117, 59, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4545, 120, 2269, 71, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6851, 196, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->481824839],
Cell[7434, 215, 371, 12, 70, "Notes",
 CellID->1067943069],
Cell[7808, 229, 609, 19, 70, "Notes",
 CellID->25338845],
Cell[8420, 250, 433, 10, 70, "Notes",
 CellID->30234373],
Cell[8856, 262, 718, 26, 70, "Notes",
 CellID->495497047],
Cell[9577, 290, 74, 1, 70, "Notes",
 CellID->529540534]
}, Closed]],
Cell[CellGroupData[{
Cell[9688, 296, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->103271997],
Cell[CellGroupData[{
Cell[10174, 313, 148, 5, 70, "ExampleSection",
 CellID->346306084],
Cell[10325, 320, 128, 6, 70, "ExampleText",
 CellID->513831186],
Cell[10456, 328, 100, 3, 70, "Input",
 CellID->267685788],
Cell[10559, 333, 429, 11, 70, "ExampleText",
 CellID->659456463],
Cell[10991, 346, 205, 6, 70, "ExampleText",
 CellID->250733239],
Cell[11199, 354, 495, 18, 70, "Input",
 CellID->64953589],
Cell[11697, 374, 86, 1, 70, "ExampleText",
 CellID->326501090],
Cell[CellGroupData[{
Cell[11808, 379, 307, 10, 70, "Input",
 CellID->560964283],
Cell[12118, 391, 842, 28, 67, "Output",
 CellID->201437575]
}, Open  ]],
Cell[12975, 422, 570, 11, 70, "ExampleText",
 CellID->120984796],
Cell[CellGroupData[{
Cell[13570, 437, 436, 15, 70, "Input",
 CellID->176773044],
Cell[14009, 454, 694, 23, 67, "Output",
 CellID->21131738]
}, Open  ]],
Cell[14718, 480, 142, 4, 70, "ExampleText",
 CellID->44302699],
Cell[CellGroupData[{
Cell[14885, 488, 641, 20, 70, "Input",
 CellID->575890672],
Cell[15529, 510, 752, 24, 67, "Output",
 CellID->128867656]
}, Open  ]],
Cell[16296, 537, 113, 3, 70, "ExampleText",
 CellID->472419597],
Cell[CellGroupData[{
Cell[16434, 544, 639, 20, 70, "Input",
 CellID->8664355],
Cell[17076, 566, 982, 30, 58, "Output",
 CellID->514646365]
}, Open  ]],
Cell[18073, 599, 203, 4, 70, "ExampleText",
 CellID->9427774],
Cell[18279, 605, 380, 12, 70, "Input",
 CellID->258291294],
Cell[18662, 619, 947, 34, 70, "ExampleText",
 CellID->78226744],
Cell[CellGroupData[{
Cell[19634, 657, 339, 12, 70, "Input",
 CellID->13857323],
Cell[19976, 671, 208, 7, 36, "Output",
 CellID->21822114]
}, Open  ]],
Cell[20199, 681, 259, 9, 70, "ExampleText",
 CellID->421446271],
Cell[CellGroupData[{
Cell[20483, 694, 487, 17, 70, "Input",
 CellID->471042009],
Cell[20973, 713, 209, 7, 36, "Output",
 CellID->390033007]
}, Open  ]],
Cell[21197, 723, 823, 25, 70, "ExampleText",
 CellID->78531558],
Cell[CellGroupData[{
Cell[22045, 752, 4607, 132, 70, "Input",
 CellID->1746043248],
Cell[26655, 886, 197186, 3596, 70, "Output",
 CellID->321588231]
}, Open  ]],
Cell[223856, 4485, 661, 14, 70, "ExampleText",
 CellID->680409942],
Cell[224520, 4501, 227, 5, 70, "ExampleText",
 CellID->62898067],
Cell[224750, 4508, 121, 3, 70, "ExampleText",
 CellID->673890120],
Cell[CellGroupData[{
Cell[224896, 4515, 1134, 33, 70, "Input",
 CellID->98488054],
Cell[226033, 4550, 159, 5, 36, "Output",
 CellID->304898429]
}, Open  ]],
Cell[226207, 4558, 275, 8, 70, "ExampleText",
 CellID->9527100],
Cell[226485, 4568, 128, 3, 70, "ExampleText",
 CellID->134318740],
Cell[CellGroupData[{
Cell[226638, 4575, 1135, 33, 70, "Input",
 CellID->326115239],
Cell[227776, 4610, 158, 5, 36, "Output",
 CellID->61288648]
}, Open  ]],
Cell[227949, 4618, 115, 3, 70, "ExampleText",
 CellID->531709516],
Cell[228067, 4623, 117, 3, 70, "ExampleText",
 CellID->211831706],
Cell[CellGroupData[{
Cell[228209, 4630, 1019, 29, 70, "Input",
 CellID->189498322],
Cell[229231, 4661, 235, 8, 36, "Output",
 CellID->258993274]
}, Open  ]],
Cell[229481, 4672, 169, 4, 70, "ExampleText",
 CellID->56731244],
Cell[CellGroupData[{
Cell[229675, 4680, 165, 5, 70, "Input",
 CellID->307784515],
Cell[229843, 4687, 162, 5, 36, "Output",
 CellID->460749900]
}, Open  ]],
Cell[230020, 4695, 114, 3, 70, "ExampleText",
 CellID->403496840],
Cell[CellGroupData[{
Cell[230159, 4702, 1019, 29, 70, "Input",
 CellID->102020424],
Cell[231181, 4733, 158, 5, 36, "Output",
 CellID->48664304]
}, Open  ]],
Cell[231354, 4741, 97, 1, 70, "ExampleText",
 CellID->325279064],
Cell[231454, 4744, 234, 6, 70, "ExampleText",
 CellID->635547329],
Cell[CellGroupData[{
Cell[231713, 4754, 940, 27, 70, "Input",
 CellID->81019929],
Cell[232656, 4783, 205, 7, 36, "Output",
 CellID->44369461]
}, Open  ]],
Cell[232876, 4793, 448, 11, 70, "ExampleText",
 CellID->78970066]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[233373, 4810, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[233793, 4824, 655, 22, 70, "SeeAlso",
 CellID->745892494]
}, Open  ]],
Cell[CellGroupData[{
Cell[234485, 4851, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[234907, 4865, 210, 5, 70, "Tutorials",
 CellID->152273840],
Cell[235120, 4872, 159, 3, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[CellGroupData[{
Cell[235316, 4880, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[235738, 4894, 167, 3, 70, "MoreAbout",
 CellID->110102918]
}, Open  ]],
Cell[235920, 4900, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

