(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    287820,       6281]
NotebookOptionsPosition[    278244,       5967]
NotebookOutlinePosition[    280386,       6020]
CellTagsIndexPosition[    280299,       6015]
WindowTitle->ICVortexLinear - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"2.1 Introduction to Influence Coefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"], "\
\<\"2.3 Isolated Panel\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IsolatedPanel"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"ICVortexPoint\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICVortexPoint"], "\<\"ICVortexConstant\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICVortexConstant"], "\<\"ICDoubletLinear\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICDoubletLinear"], "\<\"ICSourceLinear\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/ref/ICSourceLinear"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"InfluenceCoefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/InfluenceCoefficients"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/ICVortexLinear\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/ICVortexLinear"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
ICVortexLinear.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/ICVortexLinear", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["ICVortexLinear", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICVortexLinear", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes velocity influence coefficients at field \
points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     ", due to connected vortex panels of linearly varying strength whose \
endpoints are located by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICPhiVortexLinear", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes velocity-potential influence coefficients at \
field points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     " due to connected vortex panels of linear varying strength whose \
endpoints are located by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICPsiVortexLinear", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]",
     "computes stream-function influence coefficients at field points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     " due to connected vortex panels of linear varying strength whose \
endpoints are located by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, 
   "Rows" -> {None, None, {None}}, "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->220476103],

Cell[TextData[{
 "Input arguments are collections of vectors of the data type ",
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], "InlineFormula"],
 ".  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation may be used for single vectors."
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 "The circulation density for a vortex panel of linearly varying strength is \
written as ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["\[Gamma]", "TR"], 
    RowBox[{"(", 
     StyleBox["x", "TI"], ")"}]}], "=", 
   RowBox[{
    StyleBox["a", "TI"], "+", 
    RowBox[{
     StyleBox["b", "TI"], " ", 
     StyleBox["x", "TI"]}]}]}]], "InlineFormula"],
 ", where ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 " is the distance from the beginning of the panel, constants ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 " are the strength parameters for the panel.  For collections of panels, the \
strength parameters are written in two lists, one for the constant term and \
one for the linear term in the equation for circulation density.  Within each \
list strength parameters are ordered according to the clockwise numbering \
convention for panels."
}], "Notes",
 CellID->45237691],

Cell[TextData[{
 "Results for velocity influence coefficients are two collections of vectors, \
one associated with the ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " parameters and the other, with the ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 " parameters.  Influence coefficients for velocity potential and stream \
function are two lists of scalar influence coefficients.  For the special \
case of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation being used to specify a single field point, output is simplified \
as illustrated in the examples."
}], "Notes",
 CellID->433990591],

Cell[TextData[{
 "The velocity, velocity potential and stream function for a panel are \
written as the sum of two terms, each term being the product of a geometric \
influence coefficient and the appropriate strength parameter.",
 "\t\t\t\t\t\t\t\t\t\t\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["V", "TI"], "=", 
   RowBox[{
    RowBox[{
     StyleBox["a", "TI"], " ", 
     StyleBox["ica", "TI"]}], "+", 
    RowBox[{
     StyleBox["b", "TI"], " ", 
     StyleBox["icb", "TI"]}]}]}]], "InlineFormula"],
 "\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Phi]", "TR"], "=", 
   RowBox[{
    RowBox[{
     StyleBox["a", "TI"], " ", 
     StyleBox["ic\[Phi]a", "TI"]}], "+", 
    RowBox[{
     StyleBox["b", "TI"], " ", 
     StyleBox["ic\[Phi]b", "TI"]}]}]}]], "InlineFormula"],
 "\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Psi]", "TR"], "=", 
   RowBox[{
    RowBox[{
     StyleBox["a", "TI"], " ", 
     StyleBox["ic\[Psi]a", "TI"]}], "+", " ", 
    RowBox[{
     StyleBox["b", "TI"], " ", 
     StyleBox["ic\[Psi]b", "TI"]}]}]}]], "InlineFormula"]
}], "Notes",
 CellID->352350115],

Cell["New in 1.0   |   Last modified in 1.2", "Notes",
 CellID->529540534]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->538786138],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->15938609],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->513831186],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->66474307],

Cell[TextData[{
 "This example demonstrates some properties of a single panel.  Algebraic \
equations for influence coefficients are simplest in a local coordinate \
system where the panel extends from the origin along the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis to ",
 Cell[BoxData[
  StyleBox["\[Delta]", "TR"]], "InlineFormula"],
 ", the length of the panel."
}], "ExampleText",
 CellID->659456463],

Cell[TextData[{
 "Specify an arbitrary field point and panel endpoints.  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation is used for a single vector."
}], "ExampleText",
 CellID->42111717],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"rF", ",", "rP"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"x", ",", "z"}], "}"}], ",", 
     RowBox[{"MakeCartesianVectors", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "\[Delta]"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}]}], 
  ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->64953589],

Cell[TextData[{
 "Compute the velocity influence coefficients.  There are separate influence \
coefficients for the constant and linear terms in the function describing \
circulation distribution.  Each influence coefficient is a single vector \
written in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation."
}], "ExampleText",
 CellID->326501090],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"icua", ",", "icwa"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"icub", ",", "icwb"}], "}"}]}], "}"}], "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"ICVortexLinear", "[", 
      RowBox[{"rF", ",", "rP"}], "]"}], ",", 
     RowBox[{"\[Delta]", ">", "0"}]}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->560964283],

Cell[TextData[{
 "Display the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-component of the velocity influence coefficient associated with the \
constant term."
}], "ExampleText",
 CellID->217164756],

Cell[CellGroupData[{

Cell[BoxData["icua"], "Input",
 CellLabel->"In[4]:=",
 CellID->14420391],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"ArcTan", "[", 
     RowBox[{"x", ",", "z"}], "]"}]}], "+", 
   RowBox[{"ArcTan", "[", 
    RowBox[{
     RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}]}], 
  RowBox[{"2", " ", "\[Pi]"}]]], "Output",
 ImageSize->{212, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->944684992]
}, Open  ]],

Cell[TextData[{
 "Display the algebraic equations for components of the velocity influence \
coefficient associated with strength parameter ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ", replacing ArcTan function by named angles (See ",
 ButtonBox["2.1 Introduction to InfluenceCoefficients",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"],
 " for sketch defining angles).  Although the substitution of named angles is \
made to simplify the appearance of equations, note that the original \
equations are maintained for subsequent computations."
}], "ExampleText",
 CellID->120984796],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"icua", ",", "icwa"}], "}"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
     "\[Theta]2"}]}], "}"}]}]], "Input",
 CellLabel->"In[5]:=",
 CellID->176773044],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"-", "\[Theta]1"}], "+", "\[Theta]2"}], 
    RowBox[{"2", " ", "\[Pi]"}]], ",", 
   FractionBox[
    RowBox[{"Log", "[", 
     FractionBox[
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"], "-", 
       RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
       SuperscriptBox["\[Delta]", "2"]}], 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"]}]], "]"}], 
    RowBox[{"4", " ", "\[Pi]"}]]}], "}"}]], "Output",
 ImageSize->{193, 46},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->1012007624]
}, Open  ]],

Cell[TextData[{
 "Display the algebraic equations for components of the velocity influence \
coefficient associated with strength parameter ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 "."
}], "ExampleText",
 CellID->118713566],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"icub", ",", "icwb"}], "}"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
     "\[Theta]2"}]}], "}"}]}]], "Input",
 CellLabel->"In[6]:=",
 CellID->25568170],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{
      RowBox[{"-", "2"}], " ", "x", " ", "\[Theta]1"}], "+", 
     RowBox[{"2", " ", "x", " ", "\[Theta]2"}], "+", 
     RowBox[{"z", " ", 
      RowBox[{"Log", "[", 
       FractionBox[
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"], "-", 
         RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
         SuperscriptBox["\[Delta]", "2"]}], 
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"]}]], "]"}]}]}], 
    RowBox[{"4", " ", "\[Pi]"}]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"2", " ", "\[Delta]"}], "+", 
     RowBox[{"2", " ", "z", " ", "\[Theta]1"}], "-", 
     RowBox[{"2", " ", "z", " ", "\[Theta]2"}], "+", 
     RowBox[{"x", " ", 
      RowBox[{"Log", "[", 
       FractionBox[
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"], "-", 
         RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
         SuperscriptBox["\[Delta]", "2"]}], 
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"]}]], "]"}]}]}], 
    RowBox[{"4", " ", "\[Pi]"}]]}], "}"}]], "Output",
 ImageSize->{496, 46},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->154777397]
}, Open  ]],

Cell["\<\
Compute the velocity-potential influence coefficient and display in terms of \
named angles.\
\>", "ExampleText",
 CellID->44302699],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Phi]a", ",", "ic\[Phi]b"}], "}"}], "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"ICPhiVortexLinear", "[", 
      RowBox[{"rF", ",", "rP"}], "]"}], ",", 
     RowBox[{"\[Delta]", ">", "0"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"ic\[Phi]a", ",", "ic\[Phi]b"}], "}"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
     "\[Theta]2"}]}], "}"}]}]}], "Input",
 CellLabel->"In[7]:=",
 CellID->575890672],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{
      RowBox[{"-", "2"}], " ", "x", " ", "\[Theta]1"}], "+", 
     RowBox[{"2", " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "\[Delta]"}], ")"}], " ", "\[Theta]2"}], "+", 
     RowBox[{"z", " ", 
      RowBox[{"Log", "[", 
       FractionBox[
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"], "-", 
         RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
         SuperscriptBox["\[Delta]", "2"]}], 
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"]}]], "]"}]}]}], 
    RowBox[{"4", " ", "\[Pi]"}]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"z", " ", "\[Delta]"}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox["x", "2"]}], "+", 
        SuperscriptBox["z", "2"]}], ")"}], " ", "\[Theta]1"}], "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox["x", "2"]}], "+", 
        SuperscriptBox["z", "2"], "+", 
        SuperscriptBox["\[Delta]", "2"]}], ")"}], " ", "\[Theta]2"}], "+", 
     RowBox[{"x", " ", "z", " ", 
      RowBox[{"Log", "[", 
       FractionBox[
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"], "-", 
         RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
         SuperscriptBox["\[Delta]", "2"]}], 
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"]}]], "]"}]}]}], 
    RowBox[{"4", " ", "\[Pi]"}]]}], "}"}]], "Output",
 ImageSize->{387, 94},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->251340842]
}, Open  ]],

Cell["Compute stream-function influence coefficients.", "ExampleText",
 CellID->22197235],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Psi]a", ",", "ic\[Psi]b"}], "}"}], "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"ICPsiVortexLinear", "[", 
      RowBox[{"rF", ",", "rP"}], "]"}], ",", 
     RowBox[{"\[Delta]", ">", "0"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"ic\[Psi]a", ",", "ic\[Psi]b"}], "}"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
     "\[Theta]2"}]}], "}"}]}]}], "Input",
 CellLabel->"In[8]:=",
 CellID->158994946],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{
      RowBox[{"-", "2"}], " ", "z", " ", "\[Theta]1"}], "+", 
     RowBox[{"2", " ", "z", " ", "\[Theta]2"}], "+", 
     RowBox[{"x", " ", 
      RowBox[{"Log", "[", 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"]}], "]"}]}], "-", 
     RowBox[{"x", " ", 
      RowBox[{"Log", "[", 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"], "-", 
        RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
        SuperscriptBox["\[Delta]", "2"]}], "]"}]}], "+", 
     RowBox[{"\[Delta]", " ", 
      RowBox[{"Log", "[", 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"], "-", 
        RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
        SuperscriptBox["\[Delta]", "2"]}], "]"}]}]}], 
    RowBox[{"4", " ", "\[Pi]"}]], ",", 
   RowBox[{
    FractionBox["1", 
     RowBox[{"8", " ", "\[Pi]"}]], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "2"}], " ", "x", " ", "\[Delta]"}], "+", 
      RowBox[{"4", " ", "x", " ", "z", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "\[Theta]1"}], "+", "\[Theta]2"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["x", "2"], "-", 
         SuperscriptBox["z", "2"]}], ")"}], " ", 
       RowBox[{"Log", "[", 
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"]}], "]"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          SuperscriptBox["x", "2"]}], "+", 
         SuperscriptBox["z", "2"], "+", 
         SuperscriptBox["\[Delta]", "2"]}], ")"}], " ", 
       RowBox[{"Log", "[", 
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"], "-", 
         RowBox[{"2", " ", "x", " ", "\[Delta]"}], "+", 
         SuperscriptBox["\[Delta]", "2"]}], "]"}]}]}], ")"}]}]}], 
  "}"}]], "Output",
 ImageSize->{584, 69},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->326456112]
}, Open  ]],

Cell["\<\
Write the velocity components, velocity potential and stream function in \
terms of geometric influence coefficients and strength parameters.\
\>", "ExampleText",
 CellID->9427774],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"u", ",", "w"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"a", " ", "icua"}], "+", 
      RowBox[{"b", " ", "icub"}]}], ",", 
     RowBox[{
      RowBox[{"a", " ", "icwa"}], "+", 
      RowBox[{"b", " ", "icwb"}]}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Phi]", ",", "\[Psi]"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"a", " ", "ic\[Phi]a"}], "+", 
      RowBox[{"b", " ", "ic\[Phi]b"}]}], ",", 
     RowBox[{
      RowBox[{"a", " ", "ic\[Psi]a"}], "+", 
      RowBox[{"b", " ", "ic\[Psi]b"}]}]}], "}"}]}], ";"}]}], "Input",
 CellLabel->"In[9]:=",
 CellID->258291294],

Cell[TextData[{
 "Verify that the relations between velocity, velocity potential and stream \
function are satisfied.\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["u", "TI"], "=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Phi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["x", "TI"]}]], "=", 
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Psi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["z", "TI"]}]]}]}]], "InlineFormula"],
 "\t\t\t",
 Cell[BoxData[
  RowBox[{"w", "=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Phi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["z", "TI"]}]], "=", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{"\[PartialD]", 
       StyleBox["\[Psi]", "TR"]}], 
      RowBox[{"\[PartialD]", 
       StyleBox["x", "TI"]}]]}]}]}]], "InlineFormula"],
 "\t\t\t\t"
}], "ExampleText",
 CellID->78226744],

Cell[TextData[{
 "The variables ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  StyleBox["z", "TI"]], "InlineFormula"],
 " need to be declared as real in the following tests."
}], "ExampleText",
 CellID->421446271],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "\[Equal]", 
      RowBox[{"D", "[", 
       RowBox[{"\[Phi]", ",", "x"}], "]"}]}], ",", 
     RowBox[{"u", "\[Equal]", 
      RowBox[{"D", "[", 
       RowBox[{"\[Psi]", ",", "z"}], "]"}]}]}], "}"}], ",", 
   RowBox[{"Assumptions", "\[Rule]", 
    RowBox[{
     RowBox[{"Element", "[", 
      RowBox[{"x", ",", "Reals"}], "]"}], " ", "&&", " ", 
     RowBox[{"Element", "[", 
      RowBox[{"z", ",", "Reals"}], "]"}]}]}]}], "]"}]], "Input",
 CellLabel->"In[10]:=",
 CellID->13857323],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[10]=",
 CellID->462937151]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"w", "\[Equal]", 
      RowBox[{"D", "[", 
       RowBox[{"\[Phi]", ",", "z"}], "]"}]}], ",", 
     RowBox[{"w", "\[Equal]", 
      RowBox[{"-", 
       RowBox[{"D", "[", 
        RowBox[{"\[Psi]", ",", "x"}], "]"}]}]}]}], "}"}], ",", 
   RowBox[{"Assumptions", "\[Rule]", 
    RowBox[{
     RowBox[{"Element", "[", 
      RowBox[{"x", ",", "Reals"}], "]"}], " ", "&&", " ", 
     RowBox[{"Element", "[", 
      RowBox[{"z", ",", "Reals"}], "]"}]}]}]}], "]"}]], "Input",
 CellLabel->"In[11]:=",
 CellID->499128776],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->25372728]
}, Open  ]],

Cell[TextData[{
 "Flow visualization provides a plausibility check on the computed flow \
field.  Since the contribution of the ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " parameter is illustrated in ICVortexConstant, set ",
 Cell[BoxData[
  RowBox[{
   StyleBox["a", "TI"], "=", "0"}]], "InlineFormula"],
 ".  Set ",
 Cell[BoxData[
  RowBox[{
   StyleBox["b", "TI"], "=", "2"}]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Delta]", "TR"], "=", "1"}]], "InlineFormula"],
 " to provide unit integrated panel strength,  ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", 
     StyleBox["\[Delta]", "TR"]], 
    RowBox[{"\[Gamma]", 
     RowBox[{"(", 
      StyleBox["x", "TI"], ")"}], 
     RowBox[{"\[DifferentialD]", 
      StyleBox["x", "TI"]}]}]}], "=", "1"}]], "InlineFormula"],
 ".  Circular regions near the edges of the panel are excluded from the \
vector plot to avoid showing vectors of very large magnitude."
}], "ExampleText",
 CellID->78531558],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xMin", ",", "xMax", ",", "zMin", ",", "zMax"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "2", ",", 
     RowBox[{"-", "1.2"}], ",", "1.2"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"uPlot", ",", "wPlot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"u", ",", "w"}], "}"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", "\[Rule]", "0"}], ",", 
      RowBox[{"b", "\[Rule]", "2"}], ",", 
      RowBox[{"\[Delta]", "\[Rule]", "1"}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Phi]Plot", ",", "\[Psi]Plot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\[Phi]", ",", "\[Psi]"}], "}"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", "\[Rule]", "0"}], ",", 
      RowBox[{"b", "\[Rule]", "2"}], ",", 
      RowBox[{"\[Delta]", "\[Rule]", "1"}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gV", "=", 
   RowBox[{"VectorPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"uPlot", ",", "wPlot"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"RegionFunction", "\[Rule]", 
      RowBox[{"Function", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"x", ",", "z"}], "}"}], ",", 
        RowBox[{
         RowBox[{"0.01", "<", 
          RowBox[{
           RowBox[{"x", "^", "2"}], "+", 
           RowBox[{"z", "^", "2"}]}]}], " ", "&&", " ", 
         RowBox[{"0.01", "<", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"x", "-", "1"}], ")"}], "^", "2"}], "+", 
           RowBox[{"z", "^", "2"}]}]}]}]}], "]"}]}], ",", 
     RowBox[{"VectorColorFunction", "->", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"RGBColor", "[", 
         RowBox[{"0", ",", "1", ",", "0"}], "]"}], "&"}], ")"}]}], ",", 
     RowBox[{"VectorPoints", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"20", ",", "20"}], "}"}]}], ",", 
     RowBox[{"VectorScale", "\[Rule]", "Medium"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Phi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Phi]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "13"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Red"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Psi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "7"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Blue"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gPanel", "=", 
   RowBox[{"Graphics", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"GrayLevel", "[", "0", "]"}], ",", 
      RowBox[{"AbsoluteThickness", "[", "3", "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"Transpose", "[", 
        RowBox[{"Components", "[", 
         RowBox[{"rP", "/.", 
          RowBox[{"\[Delta]", "\[Rule]", "1.0"}]}], "]"}], "]"}], "]"}]}], 
     "}"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gV", ",", "g\[Phi]", ",", "g\[Psi]", ",", "gPanel", ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
   RowBox[{"FrameTicks", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.0"}], ",", "2.0", ",", "0.25"}], "]"}], "]"}], ",", 
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.2"}], ",", "1.2", ",", "0.2"}], "]"}], "]"}], ",", 
      "None", ",", "None"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", " ", "72"}]}], ",", 
   RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellLabel->"In[12]:=",
 CellID->1746043248],

Cell[BoxData[
 GraphicsBox[{{{}, {
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.005832347130626452, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9904323069308713, -1.2129172174988423`}, \
{-1.0095676930691286`, -1.1870827825011576`}}]}}, 
      {Arrowheads[{{0.006052430457528339, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.990769191236421, -1.087578685265391}, \
{-1.0092308087635788`, -1.0597897357872406`}}]}}, 
      {Arrowheads[{{0.0062721513350817125`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9912410571630096, -0.9622719613144554}, \
{-1.0087589428369903`, -0.932464880790808}}]}}, 
      {Arrowheads[{{0.0064872077230530645`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9918678889851521, -0.8369757991079466}, \
{-1.008132111014848, -0.8051294640499482}}]}}, 
      {Arrowheads[{{0.006692164383243717, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.992666846924273, -0.7116608570818118}, \
{-1.007333153075727, -0.6778128271287149}}]}}, 
      {Arrowheads[{{0.006880477305597646, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.993648969273283, -0.586289377840612}, \
{-1.006351030726717, -0.5505527274225464}}]}}, 
      {Arrowheads[{{0.007044720770665285, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9948151332627119, -0.4608163105070833}, \
{-1.005184866737288, -0.4233942158087066}}]}}, 
      {Arrowheads[{{0.0071771014325310006`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9961519676428328, -0.3351925701304589}, \
{-1.0038480323571672`, -0.2963863772379626}}]}}, 
      {Arrowheads[{{0.007270292899646251, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.9976290190136603, -0.20937075214931736`}, \
{-1.0023709809863397`, -0.1695766162717357}}]}}, 
      {Arrowheads[{{0.0073185007110988425`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.999198834127021, -0.0833126801380812}, \
{-1.000801165872979, -0.043003109335603404`}}]}}, 
      {Arrowheads[{{0.0073185007110988425`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.000801165872979, 
         0.043003109335603015`}, {-0.9991988341270209, 
         0.08331268013808081}}]}}, 
      {Arrowheads[{{0.007270292899646251, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0023709809863397`, 
         0.1695766162717353}, {-0.9976290190136603, 0.20937075214931694`}}]}}, 
      {Arrowheads[{{0.0071771014325310006`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0038480323571672`, 
         0.2963863772379622}, {-0.9961519676428328, 0.3351925701304585}}]}}, 
      {Arrowheads[{{0.0070447207706652954`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0051848667372882`, 
         0.4233942158087062}, {-0.9948151332627119, 0.4608163105070829}}]}}, 
      {Arrowheads[{{0.006880477305597677, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0063510307267172`, 
         0.5505527274225459}, {-0.993648969273283, 0.5862893778406116}}]}}, 
      {Arrowheads[{{0.0066921643832437, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.007333153075727, 
         0.6778128271287145}, {-0.992666846924273, 0.7116608570818113}}]}}, 
      {Arrowheads[{{0.006487207723053036, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0081321110148478`, 
         0.8051294640499478}, {-0.9918678889851522, 0.8369757991079462}}]}}, 
      {Arrowheads[{{0.00627215133508175, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0087589428369905`, 
         0.9324648807908075}, {-0.9912410571630096, 0.9622719613144549}}]}}, 
      {Arrowheads[{{0.006052430457528372, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.009230808763579, 
         1.0597897357872401`}, {-0.9907691912364209, 1.0875786852653906`}}]}}, 
      {Arrowheads[{{0.005832347130626474, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0095676930691289`, 
         1.1870827825011572`}, {-0.9904323069308713, 1.2129172174988418`}}]}}, 
      {Arrowheads[{{0.006206248173771658, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.831249606784419, -1.2132190181102818`}, \
{-0.8529609195313705, -1.186780981889718}}]}}, 
      {Arrowheads[{{0.006473986083373183, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8315182216087881, -1.0880470547193088`}, \
{-0.8526923047070013, -1.059321366333323}}]}}, 
      {Arrowheads[{{0.006746016118932338, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8319435034888912, -0.9629386879881806}, \
{-0.8522670228268981, -0.9317981541170828}}]}}, 
      {Arrowheads[{{0.007017359038666378, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.832556944391264, -0.8378720548605944}, \
{-0.8516535819245253, -0.8042332082973007}}]}}, 
      {Arrowheads[{{0.007281202910676056, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8333891942556296, -0.7128130961420932}, \
{-0.8508213320601598, -0.6766605880684334}}]}}, 
      {Arrowheads[{{0.007528700129312999, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8344653478128451, -0.5877134290459461}, \
{-0.8497451785029443, -0.5491286762172121}}]}}, 
      {Arrowheads[{{0.007749041056843482, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8357980993157332, -0.46251004295979825`}, \
{-0.848412427000056, -0.4217004833559917}}]}}, 
      {Arrowheads[{{0.007930042898494031, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8373793951713624, -0.33712862033874996`}, \
{-0.8468311311444269, -0.2944503270296715}}]}}, 
      {Arrowheads[{{0.008059478677421766, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8391728081957808, -0.21149219882699083`}, \
{-0.8450377181200087, -0.1674551695940622}}]}}, 
      {Arrowheads[{{0.00812714091706064, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8411106392172238, -0.0855352172062821}, \
{-0.8430998870985655, -0.040780572267402504`}}]}}, 
      {Arrowheads[{{0.00812714091706064, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8430998870985655, 
         0.040780572267402115`}, {-0.8411106392172238, 
         0.08553521720628171}}]}}, 
      {Arrowheads[{{0.00805947867742177, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8450377181200086, 
         0.1674551695940618}, {-0.8391728081957807, 0.21149219882699044`}}]}}, 
      {Arrowheads[{{0.007930042898494031, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.846831131144427, 
         0.2944503270296711}, {-0.8373793951713625, 0.3371286203387496}}]}}, 
      {Arrowheads[{{0.0077490410568435, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8484124270000561, 
         0.42170048335599125`}, {-0.8357980993157333, 0.4625100429597979}}]}}, 
      {Arrowheads[{{0.007528700129312999, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8497451785029443, 
         0.5491286762172116}, {-0.8344653478128451, 0.5877134290459457}}]}}, 
      {Arrowheads[{{0.007281202910676083, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8508213320601598, 
         0.676660588068433}, {-0.8333891942556295, 0.7128130961420929}}]}}, 
      {Arrowheads[{{0.007017359038666378, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8516535819245253, 
         0.8042332082973002}, {-0.832556944391264, 0.8378720548605938}}]}}, 
      {Arrowheads[{{0.006746016118932338, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8522670228268981, 
         0.9317981541170822}, {-0.8319435034888912, 0.96293868798818}}]}}, 
      {Arrowheads[{{0.006473986083373216, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8526923047070013, 
         1.0593213663333223`}, {-0.8315182216087881, 1.0880470547193084`}}]}}, 
      {Arrowheads[{{0.006206248173771658, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8529609195313705, 
         1.1867809818897177`}, {-0.831249606784419, 1.2132190181102815`}}]}}, 
      {Arrowheads[{{0.006609637979218285, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6718694306609982, -1.2133998514845983`}, \
{-0.6965516219705805, -1.1866001485154014`}}]}}, 
      {Arrowheads[{{0.0069362387021059254`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6720232009263744, -1.088412895724931}, \
{-0.6963978517052043, -1.0589555253277008`}}]}}, 
      {Arrowheads[{{0.007274898430320748, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6723517910227894, -0.9635361085504346}, \
{-0.6960692616087892, -0.9312007335548286}}]}}, 
      {Arrowheads[{{0.0076204419236468425`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6729023984435495, -0.8387514350266985}, \
{-0.6955186541880293, -0.8033538281311965}}]}}, 
      {Arrowheads[{{0.007964925433469158, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.673726416879816, -0.714023838655797}, \
{-0.6946946357517626, -0.6754498455547296}}]}}, 
      {Arrowheads[{{0.00829689724012411, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6748738499788773, -0.5892954203611468}, \
{-0.6935472026527014, -0.5475466849020114}}]}}, 
      {Arrowheads[{{0.008600872015148477, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6763829634625054, -0.4644806958570009}, \
{-0.6920380891690735, -0.419729830458789}}]}}, 
      {Arrowheads[{{0.008857543275236931, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6782643643619118, -0.339466743970501}, \
{-0.6901566882696668, -0.2921122033979205}}]}}, 
      {Arrowheads[{{0.009045554088858487, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6804818292357129, -0.21412394786140607`}, \
{-0.6879392233958659, -0.16482342055964694`}}]}}, 
      {Arrowheads[{{0.009145498238515737, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6829385178808969, -0.08833191486270878}, \
{-0.6854825347506818, -0.03798387461097583}}]}}, 
      {Arrowheads[{{0.009145498238515733, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6854825347506819, 
         0.03798387461097545}, {-0.682938517880897, 0.08833191486270837}}]}}, 
      {Arrowheads[{{0.009045554088858487, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6879392233958659, 
         0.16482342055964658`}, {-0.6804818292357129, 0.2141239478614057}}]}}, 
      {Arrowheads[{{0.008857543275236931, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6901566882696669, 
         0.2921122033979201}, {-0.678264364361912, 0.3394667439705006}}]}}, 
      {Arrowheads[{{0.008600872015148496, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6920380891690735, 
         0.41972983045878853`}, {-0.6763829634625054, 0.4644806958570006}}]}}, 
      {Arrowheads[{{0.008296897240124104, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6935472026527014, 
         0.5475466849020111}, {-0.6748738499788774, 0.5892954203611465}}]}}, 
      {Arrowheads[{{0.007964925433469158, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6946946357517627, 
         0.6754498455547292}, {-0.6737264168798162, 0.7140238386557965}}]}}, 
      {Arrowheads[{{0.007620441923646855, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6955186541880293, 
         0.803353828131196}, {-0.6729023984435494, 0.838751435026698}}]}}, 
      {Arrowheads[{{0.007274898430320719, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6960692616087892, 
         0.9312007335548282}, {-0.6723517910227895, 0.9635361085504341}}]}}, 
      {Arrowheads[{{0.0069362387021059254`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6963978517052043, 
         1.0589555253277003`}, {-0.6720232009263744, 1.0884128957249306`}}]}}, 
      {Arrowheads[{{0.006609637979218298, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.6965516219705805, 
         1.1866001485154012`}, {-0.6718694306609981, 1.213399851484598}}]}}, 
      {Arrowheads[{{0.007039283303892644, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.512281988196229, -1.2133961497784338`}, \
{-0.5403495907511391, -1.1866038502215661`}}]}}, 
      {Arrowheads[{{0.00743757170467208, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5122573288411378, -1.088602777699429}, \
{-0.5403742501062304, -1.0587656433532027`}}]}}, 
      {Arrowheads[{{0.007860197031827642, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5124147708176979, -0.9639839842781797}, \
{-0.5402168081296702, -0.9307528578270836}}]}}, 
      {Arrowheads[{{0.008303002732570253, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5128204836986032, -0.8395339561212596}, \
{-0.5398110952487649, -0.8025713070366354}}]}}, 
      {Arrowheads[{{0.00875811577628384, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5135547649880046, -0.7152263902059124}, \
{-0.5390768139593636, -0.6742472940046143}}]}}, 
      {Arrowheads[{{0.009212223675285664, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5147082792064144, -0.5910024569537682}, \
{-0.5379232997409538, -0.54583964830939}}]}}, 
      {Arrowheads[{{0.009644502647382041, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5163702638839512, -0.46675602881590705`}, \
{-0.5362613150634169, -0.41745449749988284`}}]}}, 
      {Arrowheads[{{0.01002487650108977, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5186027567996456, -0.34232088266454797`}, \
{-0.5340288221477225, -0.28925806470387355`}}]}}, 
      {Arrowheads[{{0.010314658967867672`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5213967271335768, -0.21747335956739466`}, \
{-0.5312348518137915, -0.16147400885365834`}}]}}, 
      {Arrowheads[{{0.010473334637000196`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5246213120172166, -0.09197393643003729}, \
{-0.5280102669301515, -0.034341853043647314`}}]}}, 
      {Arrowheads[{{0.010473334637000191`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5280102669301515, 
         0.03434185304364694}, {-0.5246213120172167, 0.09197393643003689}}]}}, 
      {Arrowheads[{{0.010314658967867676`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5312348518137914, 
         0.16147400885365795`}, {-0.5213967271335767, 0.2174733595673943}}]}}, 
      {Arrowheads[{{0.010024876501089772`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5340288221477225, 
         0.2892580647038731}, {-0.5186027567996457, 0.3423208826645476}}]}}, 
      {Arrowheads[{{0.009644502647382041, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5362613150634169, 
         0.41745449749988245`}, {-0.5163702638839512, 
         0.46675602881590667`}}]}}, 
      {Arrowheads[{{0.009212223675285664, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5379232997409538, 
         0.5458396483093897}, {-0.5147082792064144, 0.5910024569537679}}]}}, 
      {Arrowheads[{{0.008758115776283857, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5390768139593636, 
         0.6742472940046138}, {-0.5135547649880046, 0.715226390205912}}]}}, 
      {Arrowheads[{{0.008303002732570253, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5398110952487649, 
         0.8025713070366349}, {-0.5128204836986032, 0.8395339561212591}}]}}, 
      {Arrowheads[{{0.00786019703182766, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5402168081296702, 
         0.9307528578270832}, {-0.5124147708176979, 0.9639839842781793}}]}}, 
      {Arrowheads[{{0.00743757170467208, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5403742501062303, 
         1.0587656433532022`}, {-0.5122573288411377, 1.0886027776994285`}}]}}, 
      {Arrowheads[{{0.007039283303892617, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5403495907511391, 
         1.1866038502215657`}, {-0.512281988196229, 1.2133961497784331`}}]}}, 
      {Arrowheads[{{0.007488184865313859, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3524956708276279, -1.2131272489954148`}, \
{-0.3843464344355297, -1.1868727510045853`}}]}}, 
      {Arrowheads[{{0.007971730703272825, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.35221166950053284`, -1.088516085672416}, \
{-0.38463043576262473`, -1.0588523353802157`}}]}}, 
      {Arrowheads[{{0.00849790733152536, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3520947626620233, -0.964161553454368}, \
{-0.3847473426011343, -0.9305752886508951}}]}}, 
      {Arrowheads[{{0.009065937037439643, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3522291092716275, -0.8400832362736612}, \
{-0.38461299599153015`, -0.8020220268842337}}]}}, 
      {Arrowheads[{{0.009671068739029808, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.35272844835686185`, -0.7162824639088241}, \
{-0.38411365690629584`, -0.6731912203017025}}]}}, 
      {Arrowheads[{{0.0103016695313731, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.35374041095070585`, -0.5927237999469818}, \
{-0.38310169431245183`, -0.5441183053161766}}]}}, 
      {Arrowheads[{{0.01093448574250738, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.35544332537967793`, -0.46930463563485086`}, \
{-0.3813987798834797, -0.414905890680939}}]}}, 
      {Arrowheads[{{0.011527423936015167`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.35802124410969893`, -0.3458101624593601}, \
{-0.37882086115345875`, -0.2857687849090613}}]}}, 
      {Arrowheads[{{0.012011406894346611`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.36158702870137627`, -0.22186555151362788`}, \
{-0.37525507656178136`, -0.15708181690742515`}}]}}, 
      {Arrowheads[{{0.012292846894785213`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3660217575965578, -0.0969534558201495}, \
{-0.37082034766659977`, -0.029362333653535105`}}]}}, 
      {Arrowheads[{{0.012292846894785213`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.37082034766659977`, 
         0.029362333653534724`}, {-0.3660217575965578, 
         0.09695345582014911}}]}}, 
      {Arrowheads[{{0.012011406894346611`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.37525507656178136`, 
         0.15708181690742476`}, {-0.36158702870137627`, 
         0.2218655515136275}}]}}, 
      {Arrowheads[{{0.011527423936015167`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3788208611534587, 
         0.28576878490906094`}, {-0.3580212441096989, 0.3458101624593597}}]}}, 
      {Arrowheads[{{0.01093448574250738, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3813987798834797, 
         0.41490589068093864`}, {-0.35544332537967793`, 
         0.4693046356348505}}]}}, 
      {Arrowheads[{{0.010301669531373096`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3831016943124518, 
         0.5441183053161761}, {-0.35374041095070585`, 0.5927237999469813}}]}}, 
      {Arrowheads[{{0.0096710687390298, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3841136569062958, 
         0.6731912203017021}, {-0.35272844835686185`, 0.7162824639088237}}]}}, 
      {Arrowheads[{{0.00906593703743965, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.38461299599153015`, 
         0.8020220268842333}, {-0.3522291092716274, 0.8400832362736608}}]}}, 
      {Arrowheads[{{0.00849790733152536, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.38474734260113436`, 
         0.9305752886508948}, {-0.35209476266202333`, 0.9641615534543677}}]}}, 
      {Arrowheads[{{0.007971730703272825, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.38463043576262473`, 
         1.058852335380215}, {-0.35221166950053284`, 1.0885160856724154`}}]}}, 
      {Arrowheads[{{0.007488184865313884, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.38434643443552974`, 
         1.1868727510045847`}, {-0.35249567082762795`, 
         1.2131272489954144`}}]}}, 
      {Arrowheads[{{0.007944382186897357, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.1925489072216697, -1.2124993960861528`}, \
{-0.2285037243572774, -1.1875006039138474`}}]}}, 
      {Arrowheads[{{0.008525747479929386, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.19191273072428586`, -1.0880261648573526`}, \
{-0.2291399008546613, -1.0593422561952792`}}]}}, 
      {Arrowheads[{{0.009175137603544032, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.1913921134461382, -0.9639019154439566}, \
{-0.22966051813280894`, -0.9308349266613065}}]}}, 
      {Arrowheads[{{0.009898828209760315, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.19107911829069277`, -0.8401873192892192}, \
{-0.22997351328825436`, -0.8019179438686758}}]}}, 
      {Arrowheads[{{0.010700806242011921`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.1911125505898818, -0.7169388368482482}, \
{-0.22994008098906532`, -0.6725348473622784}}]}}, 
      {Arrowheads[{{0.011579709836569835`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.19169781083505905`, -0.5941904678252355}, \
{-0.22935482074388808`, -0.5426516374379228}}]}}, 
      {Arrowheads[{{0.012522566715066396`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.19313149628899634`, -0.47191498978254365`}, \
{-0.22792113528995078`, -0.41229553653324624`}}]}}, 
      {Arrowheads[{{0.013490870058439075`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.1958231725319517, -0.34994145249351916`}, \
{-0.22522945904699543`, -0.28163749487490225`}}]}}, 
      {Arrowheads[{{0.014387468424857415`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.20026848155748425`, -0.22777759609271592`}, \
{-0.22078415002146287`, -0.15116977232833712`}}]}}, 
      {Arrowheads[{{0.014991625909092891`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.20674641547488898`, -0.1043034307360472}, \
{-0.21430621610405814`, -0.0220123587376374}}]}}, 
      {Arrowheads[{{0.014991625909092891`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.21430621610405812`, 
         0.02201235873763701}, {-0.206746415474889, 0.10430343073604681`}}]}}, 
      {Arrowheads[{{0.014387468424857412`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.22078415002146287`, 
         0.15116977232833673`}, {-0.20026848155748428`, 
         0.22777759609271553`}}]}}, 
      {Arrowheads[{{0.013490870058439075`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.22522945904699543`, 
         0.2816374948749019}, {-0.1958231725319517, 0.3499414524935188}}]}}, 
      {Arrowheads[{{0.012522566715066396`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.22792113528995078`, 
         0.41229553653324585`}, {-0.19313149628899634`, 
         0.47191498978254326`}}]}}, 
      {Arrowheads[{{0.011579709836569816`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.22935482074388805`, 
         0.5426516374379223}, {-0.19169781083505905`, 0.594190467825235}}]}}, 
      {Arrowheads[{{0.010700806242011921`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.22994008098906532`, 
         0.672534847362278}, {-0.1911125505898818, 0.7169388368482478}}]}}, 
      {Arrowheads[{{0.009898828209760322, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.22997351328825438`, 
         0.8019179438686753}, {-0.19107911829069274`, 0.8401873192892187}}]}}, 
      {Arrowheads[{{0.009175137603544029, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.2296605181328089, 
         0.9308349266613061}, {-0.1913921134461382, 0.9639019154439562}}]}}, 
      {Arrowheads[{{0.008525747479929392, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.2291399008546613, 
         1.0593422561952786`}, {-0.19191273072428583`, 1.088026164857352}}]}}, 
      {Arrowheads[{{0.007944382186897385, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.22850372435727742`, 
         1.1875006039138467`}, {-0.1925489072216697, 1.2124993960861523`}}]}}, 
      {Arrowheads[{{0.008390185008325167, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.03252235492067782, -1.2114173950800127`}, \
{-0.0727408029740588, -1.1885826049199875`}}]}}, 
      {Arrowheads[{{0.00907824855594103, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.0314436659006615, -1.0869925593744247`}, \
{-0.07381949199407511, -1.060375861678207}}]}}, 
      {Arrowheads[{{0.009866709049115414, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.03038032088342185, -0.9630013605336675}, \
{-0.07488283701131476, -0.9317354815715959}}]}}, 
      {Arrowheads[{{0.010773356935809516`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.029409710135948522`, -0.8395567477281658}, \
{-0.07585344775878809, -0.8025485154297292}}]}}, 
      {Arrowheads[{{0.0118184217824174, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.028664762797164473`, -0.7167957436846352}, \
{-0.07659839509757214, -0.6726779405258916}}]}}, 
      {Arrowheads[{{0.013024095866276243`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.028368820056074075`, -0.5948756159190252}, \
{-0.07689433783866254, -0.5419664893441332}}]}}, 
      {Arrowheads[{{0.01441325109110494, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.028893716205075005`, -0.4739575369972765}, \
{-0.07636944168966162, -0.41025298931851334`}}]}}, 
      {Arrowheads[{{0.016005795324081196`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.03086588163790118, -0.35415999569561263`}, \
{-0.07439727625683543, -0.27741895167280883`}}]}}, 
      {Arrowheads[{{0.01780230422200126, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.03540483648148225, -0.2354154994299008}, \
{-0.06985832141325436, -0.14353186899115222`}}]}}}, 
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.01780230422200127, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.06985832141325435, 
         0.1435318689911518}, {-0.035404836481482266`, 
         0.23541549942990045`}}]}}, 
      {Arrowheads[{{0.016005795324081203`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.07439727625683543, 
         0.27741895167280844`}, {-0.030865881637901188`, 
         0.3541599956956123}}]}}, 
      {Arrowheads[{{0.014413251091104943`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.0763694416896616, 
         0.41025298931851295`}, {-0.028893716205075012`, 
         0.47395753699727616`}}]}}, 
      {Arrowheads[{{0.013024095866276243`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.07689433783866254, 
         0.5419664893441327}, {-0.02836882005607408, 0.5948756159190247}}]}}, 
      {Arrowheads[{{0.0118184217824174, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.07659839509757214, 
         0.672677940525891}, {-0.028664762797164466`, 0.7167957436846346}}]}}, 
      {Arrowheads[{{0.010773356935809504`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.07585344775878809, 
         0.8025485154297287}, {-0.02940971013594852, 0.8395567477281652}}]}}, 
      {Arrowheads[{{0.009866709049115427, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.07488283701131476, 
         0.9317354815715954}, {-0.03038032088342185, 0.963001360533667}}]}}, 
      {Arrowheads[{{0.009078248555941032, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.07381949199407511, 
         1.0603758616782064`}, {-0.03144366590066149, 1.086992559374424}}]}}, 
      {Arrowheads[{{0.008390185008325167, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.0727408029740588, 
         1.1885826049199868`}, {-0.032522354920677815`, 
         1.211417395080012}}]}}, 
      {Arrowheads[{{0.008802376264060227, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.12745376167389916`, -1.2098054316679074`}, {
         0.08307255411557472, -1.1901945683320925`}}]}}, 
      {Arrowheads[{{0.009599122356071516, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.12903852597502588`, -1.0852890353097564`}, {
         0.081487789814448, -1.0620793857428752`}}]}}, 
      {Arrowheads[{{0.01053363507836227, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.13075861597191865`, -0.9612550870527122}, {
         0.07976769981755524, -0.9334817550525512}}]}}, 
      {Arrowheads[{{0.011639209071548822`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.13258344020879786`, -0.8378654771813479}, {
         0.07794287558067604, -0.8042397859765471}}]}}, 
      {Arrowheads[{{0.012959134364095145`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.13443756349159217`, -0.7153416401984702}, {
         0.07608875229788173, -0.6741320440120564}}]}}, 
      {Arrowheads[{{0.014551046397145804`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.13616394647845953`, -0.5939849406425087}, {
         0.07436236931101435, -0.5428571646206496}}]}}, 
      {Arrowheads[{{0.01649597266341882, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.13746159456394969`, -0.47420391514991156`}, {
         0.0730647212255242, -0.41000661116587833`}}]}}, 
      {Arrowheads[{{0.01892245515485331, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.13778141578464437`, -0.3565627728439466}, {
         0.07274490000482953, -0.27501617452447485`}}]}}, 
      {Arrowheads[{{0.02208847287830911, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.1361542361954097, -0.24193266057534565`}, {
         0.0743720795940642, -0.13701470784570738`}}]}}, 
      {Arrowheads[{{0.026742429303635833`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.13105234287738188`, -0.13220436676589237`}, {
         0.07947397291209203, 0.005888577292207778}}]}}, 
      {Arrowheads[{{0.026742429303635865`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.07947397291209204, -0.0058885772922082635`}, {
         0.13105234287738185`, 0.1322043667658921}}]}}, 
      {Arrowheads[{{0.02208847287830912, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.07437207959406422, 0.13701470784570693`}, {
         0.13615423619540967`, 0.24193266057534532`}}]}}, 
      {Arrowheads[{{0.018922455154853315`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.07274490000482953, 0.27501617452447447`}, {
         0.13778141578464437`, 0.3565627728439463}}]}}, 
      {Arrowheads[{{0.016495972663418828`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.07306472122552418, 0.41000661116587794`}, {
         0.1374615945639497, 0.47420391514991117`}}]}}, 
      {Arrowheads[{{0.014551046397145809`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.07436236931101435, 0.5428571646206491}, {
         0.13616394647845956`, 0.5939849406425083}}]}}, 
      {Arrowheads[{{0.012959134364095145`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.07608875229788173, 0.674132044012056}, {
         0.13443756349159217`, 0.7153416401984698}}]}}, 
      {Arrowheads[{{0.011639209071548827`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.07794287558067603, 0.8042397859765467}, {
         0.13258344020879786`, 0.8378654771813474}}]}}, 
      {Arrowheads[{{0.010533635078362276`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.07976769981755523, 0.9334817550525506}, {
         0.13075861597191868`, 0.9612550870527117}}]}}, 
      {Arrowheads[{{0.00959912235607154, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.08148778981444799, 1.0620793857428747`}, {
         0.1290385259750259, 1.0852890353097562`}}]}}, 
      {Arrowheads[{{0.008802376264060234, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.08307255411557471, 1.190194568332092}, {
         0.12745376167389919`, 1.209805431667907}}]}}, 
      {Arrowheads[{{0.009153816401990485, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.2872040287088841, -1.207634723830129}, {
         0.23911176076480029`, -1.192365276169871}}]}}, 
      {Arrowheads[{{0.010051378731496715`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.28930191006083184`, -1.0828460164477178`}, {
         0.23701387941285254`, -1.0645224046049138`}}]}}, 
      {Arrowheads[{{0.011125171169617538`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.2917198688458918, -0.9585216305239369}, {
         0.2345959206277926, -0.9362152115813263}}]}}, 
      {Arrowheads[{{0.012426287855545084`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.29450510224977744`, -0.8348477624923029}, {
         0.23181068722390696`, -0.8072575006655921}}]}}, 
      {Arrowheads[{{0.014024958414283722`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.29769390450679284`, -0.7120987931328636}, {
         0.2286218849668916, -0.677374891077663}}]}}, 
      {Arrowheads[{{0.016019498626009036`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.30128792648968344`, -0.5906803616473048}, {
         0.225027862984001, -0.5461617436158537}}]}}, 
      {Arrowheads[{{0.018551505501626712`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3052108476554026, -0.47118882224218833`}, {
         0.22110494181828183`, -0.4130217040736015}}]}}, 
      {Arrowheads[{{0.021835276505795517`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.30924758091091165`, -0.35448654843728833`}, {
         0.21706820856277276`, -0.2770923989311332}}]}}, 
      {Arrowheads[{{0.02621958712304588, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3130172584198053, -0.24178234365585877`}, {
         0.21329853105387905`, -0.13716502476519427`}}]}}, 
      {Arrowheads[{{0.03228642573359437, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3161666454087352, -0.1346315126780997}, {
         0.2101491440649492, 0.008315723204415107}}]}}, 
      {Arrowheads[{{0.0322864257335944, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.2101491440649492, -0.008315723204415565}, {
         0.3161666454087352, 0.1346315126780994}}]}}, 
      {Arrowheads[{{0.026219587123045906`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.21329853105387903`, 0.13716502476519382`}, {
         0.3130172584198054, 0.24178234365585843`}}]}}, 
      {Arrowheads[{{0.021835276505795535`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.21706820856277276`, 0.2770923989311327}, {
         0.30924758091091165`, 0.354486548437288}}]}}, 
      {Arrowheads[{{0.018551505501626726`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.22110494181828183`, 0.4130217040736011}, {
         0.30521084765540263`, 0.471188822242188}}]}}, 
      {Arrowheads[{{0.016019498626009054`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.22502786298400096`, 0.5461617436158531}, {
         0.30128792648968344`, 0.5906803616473043}}]}}, 
      {Arrowheads[{{0.014024958414283724`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.22862188496689156`, 0.6773748910776626}, {
         0.29769390450679284`, 0.7120987931328632}}]}}, 
      {Arrowheads[{{0.01242628785554509, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.23181068722390696`, 0.8072575006655917}, {
         0.29450510224977744`, 0.8348477624923025}}]}}, 
      {Arrowheads[{{0.011125171169617537`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.23459592062779258`, 0.9362152115813259}, {
         0.2917198688458918, 0.9585216305239365}}]}}, 
      {Arrowheads[{{0.010051378731496715`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.23701387941285254`, 1.0645224046049133`}, {
         0.28930191006083184`, 1.0828460164477174`}}]}}, 
      {Arrowheads[{{0.00915381640199051, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.23911176076480026`, 1.1923652761698704`}, {
         0.2872040287088842, 1.2076347238301286`}}]}}, 
      {Arrowheads[{{0.009416526954396768, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.44652925240499935`, -1.2049504032081135`}, {
         0.39557601075289556`, -1.1950495967918866`}}]}}, 
      {Arrowheads[{{0.010395370936828748`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4490656506754714, -1.0796964650583667`}, {
         0.39303961248242353`, -1.0676719559942647`}}]}}, 
      {Arrowheads[{{0.01158464230469338, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.452104365597503, -0.9548003183743136}, {
         0.39000089756039197`, -0.9399365237309496}}]}}, 
      {Arrowheads[{{0.013053197241524914`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.45578589092087074`, -0.8304276379803862}, {
         0.3863193722370242, -0.8116776251775089}}]}}, 
      {Arrowheads[{{0.014899165088660946`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.46029257528654643`, -0.7068395568716778}, {
         0.3818126878713485, -0.6826341273388489}}]}}, 
      {Arrowheads[{{0.017263677362387837`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4658511849946596, -0.5844531177545778}, {
         0.3762540781632353, -0.5523889875085806}}]}}, 
      {Arrowheads[{{0.0203492706226711, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.47271602258707407`, -0.4639326807572885}, {
         0.36938924057082084`, -0.42027784555850145`}}]}}, 
      {Arrowheads[{{0.024439701273845628`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.481104746983458, -0.3463011260177866}, {
         0.3610005161744369, -0.2852778213506349}}]}}, 
      {Arrowheads[{{0.029907240058843307`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.49105652280275847`, -0.23299201845115025`}, {
         0.35104874035513645`, -0.14595534996990278`}}]}}, 
      {Arrowheads[{{0.03717191494905455, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5022439145054034, -0.12564044300799102`}, {
         0.3398613486524915, -0.0006753464656935951}}]}}, 
      {Arrowheads[{{0.03717191494905457, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3398613486524915, 0.0006753464656931302}, {
         0.5022439145054034, 0.1256404430079907}}]}}, 
      {Arrowheads[{{0.029907240058843317`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.35104874035513645`, 0.14595534996990234`}, {
         0.49105652280275847`, 0.23299201845114992`}}]}}, 
      {Arrowheads[{{0.024439701273845635`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3610005161744369, 0.28527782135063445`}, {
         0.481104746983458, 0.3463011260177863}}]}}, 
      {Arrowheads[{{0.020349270622671105`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.36938924057082084`, 0.420277845558501}, {
         0.47271602258707407`, 0.4639326807572881}}]}}, 
      {Arrowheads[{{0.01726367736238784, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3762540781632353, 0.5523889875085801}, {
         0.4658511849946596, 0.5844531177545774}}]}}, 
      {Arrowheads[{{0.014899165088660954`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3818126878713485, 0.6826341273388484}, {
         0.4602925752865465, 0.7068395568716773}}]}}, 
      {Arrowheads[{{0.013053197241524931`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3863193722370242, 0.8116776251775083}, {
         0.4557858909208708, 0.8304276379803858}}]}}, 
      {Arrowheads[{{0.011584642304693391`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3900008975603919, 0.9399365237309493}, {
         0.452104365597503, 0.9548003183743132}}]}}, 
      {Arrowheads[{{0.010395370936828748`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.39303961248242353`, 1.0676719559942645`}, {
         0.4490656506754714, 1.0796964650583665`}}]}}, 
      {Arrowheads[{{0.009416526954396777, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.39557601075289556`, 1.195049596791886}, {
         0.4465292524049994, 1.2049504032081129`}}]}}, 
      {Arrowheads[{{0.009565930323572473, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6052447107511353, -1.2018861887148717`}, {
         0.5526500260909701, -1.198113811285128}}]}}, 
      {Arrowheads[{{0.010594915105533382`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6080557906511651, -1.0760064732828343`}, {
         0.5498389461909402, -1.0713619477697975`}}]}}, 
      {Arrowheads[{{0.01185805798421102, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6114986760938137, -0.9502915065508679}, {
         0.5463960607482916, -0.9444453355543954}}]}}, 
      {Arrowheads[{{0.013438837820147872`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6157932094177815, -0.8248315402376089}, {
         0.5421015274243238, -0.817273722920286}}]}}, 
      {Arrowheads[{{0.015460867364983227`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.621259567954375, -0.6997833610749346}, {
         0.5366351688877303, -0.6896903231355921}}]}}, 
      {Arrowheads[{{0.01811021140952181, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6283665721444255, -0.5754317790707026}, {
         0.5295281646976799, -0.5614103261924557}}]}}, 
      {Arrowheads[{{0.021667162918267806`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6377855985938415, -0.45231464983977876`}, {
         0.5201091382482638, -0.4318958764760111}}]}}, 
      {Arrowheads[{{0.02653810830123729, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.650390603069207, -0.3314632273070564}, {
         0.5075041337728983, -0.300115720061365}}]}}, 
      {Arrowheads[{{0.033238663421553655`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6669983433854657, -0.21476007296597924`}, {
         0.49089639345663955`, -0.1641872954550738}}]}}, 
      {Arrowheads[{{0.04221283494424637, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6874941855893525, -0.10503246397081062`}, {
         0.4704005512527529, -0.021283325502873977`}}]}}, 
      {Arrowheads[{{0.0422128349442464, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4704005512527528, 0.021283325502873512`}, {
         0.6874941855893525, 0.10503246397081031`}}]}}, 
      {Arrowheads[{{0.03323866342155369, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.4908963934566395, 0.16418729545507338`}, {
         0.6669983433854658, 0.2147600729659789}}]}}, 
      {Arrowheads[{{0.02653810830123729, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5075041337728983, 0.3001157200613646}, {
         0.650390603069207, 0.331463227307056}}]}}, 
      {Arrowheads[{{0.021667162918267827`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5201091382482638, 0.4318958764760107}, {
         0.6377855985938417, 0.45231464983977837`}}]}}, 
      {Arrowheads[{{0.01811021140952183, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5295281646976798, 0.5614103261924552}, {
         0.6283665721444255, 0.5754317790707022}}]}}, 
      {Arrowheads[{{0.015460867364983227`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5366351688877303, 0.6896903231355916}, {
         0.621259567954375, 0.6997833610749341}}]}}, 
      {Arrowheads[{{0.01343883782014787, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5421015274243238, 0.8172737229202857}, {
         0.6157932094177815, 0.8248315402376084}}]}}, 
      {Arrowheads[{{0.01185805798421102, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5463960607482916, 0.9444453355543949}, {
         0.6114986760938137, 0.9502915065508675}}]}}, 
      {Arrowheads[{{0.010594915105533386`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5498389461909402, 1.0713619477697969`}, {
         0.6080557906511651, 1.0760064732828338`}}]}}, 
      {Arrowheads[{{0.00956593032357249, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.55265002609097, 1.1981138112851277`}, {
         0.6052447107511353, 1.2018861887148713`}}]}}, 
      {Arrowheads[{{0.009585574334429407, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7632269222444318, -1.198655697922137}, {
         0.7104572882818841, -1.201344302077863}}]}}, 
      {Arrowheads[{{0.01062455381844845, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7660801909954578, -1.0720702426303474`}, {
         0.7076040195308583, -1.0752981784222844`}}]}}, 
      {Arrowheads[{{0.011904938068836662`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7695947884337615, -0.9454047293198438}, {
         0.7040894220925544, -0.9493321127854195}}]}}, 
      {Arrowheads[{{0.013517179629487482`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7740183612638332, -0.8186311198527056}, {
         0.6996658492624827, -0.8234741433051893}}]}}, 
      {Arrowheads[{{0.015600358484109221`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7797323936060458, -0.6917160132037068}, {
         0.6939518169202701, -0.6977576710068197}}]}}, 
      {Arrowheads[{{0.018376988806976354`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7873497330708109, -0.5646357037304981}, {
         0.6863344774555049, -0.5722064015326603}}]}}, 
      {Arrowheads[{{0.02221776087653521, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7978993769065688, -0.43744418145137987`}, {
         0.6757848336197473, -0.44676634486440997`}}]}}, 
      {Arrowheads[{{0.027756405929004024`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8131621812939129, -0.3105440107234221}, {
         0.660522029232403, -0.3210349366449993}}]}}, 
      {Arrowheads[{{0.036022721449077796`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8360526135956293, -0.18567719090610127`}, {
         0.6376315969306866, -0.19327017751495176`}}]}}, 
      {Arrowheads[{{0.04813251423664526, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8693939139986353, -0.06849529123717053}, {
         0.6042902965276806, -0.05782049823651408}}]}}, 
      {Arrowheads[{{0.048132514236645316`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6042902965276805, 0.05782049823651363}, {
         0.8693939139986354, 0.0684952912371702}}]}}, 
      {Arrowheads[{{0.03602272144907784, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6376315969306865, 0.19327017751495137`}, {
         0.8360526135956294, 0.1856771909061009}}]}}, 
      {Arrowheads[{{0.027756405929004045`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6605220292324029, 0.32103493664499894`}, {
         0.8131621812939129, 0.3105440107234217}}]}}, 
      {Arrowheads[{{0.02221776087653523, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6757848336197472, 0.44676634486440964`}, {
         0.7978993769065688, 0.4374441814513795}}]}}, 
      {Arrowheads[{{0.018376988806976375`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6863344774555049, 0.5722064015326598}, {
         0.787349733070811, 0.5646357037304975}}]}}, 
      {Arrowheads[{{0.015600358484109221`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6939518169202701, 0.6977576710068194}, {
         0.7797323936060458, 0.6917160132037065}}]}}, 
      {Arrowheads[{{0.013517179629487482`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.6996658492624828, 0.8234741433051889}, {
         0.7740183612638333, 0.8186311198527052}}]}}, 
      {Arrowheads[{{0.011904938068836662`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7040894220925544, 0.9493321127854191}, {
         0.7695947884337615, 0.9454047293198433}}]}}, 
      {Arrowheads[{{0.01062455381844845, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7076040195308582, 1.0752981784222837`}, {
         0.7660801909954577, 1.0720702426303468`}}]}}, 
      {Arrowheads[{{0.009585574334429407, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7104572882818841, 1.2013443020778625`}, {
         0.7632269222444318, 1.1986556979221366`}}]}}, 
      {Arrowheads[{{0.009471236451657348, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9204526771210444, -1.1955155810444777`}, {
         0.8690210070894823, -1.2044844189555222`}}]}}, 
      {Arrowheads[{{0.010476590251087013`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9230969369970288, -1.0682566300361593`}, {
         0.8663767472134977, -1.0791117910164723`}}]}}, 
      {Arrowheads[{{0.011710497567857265`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9263127655047004, -0.9406844792464225}, {
         0.8631609187058262, -0.9540523628588408}}]}}, 
      {Arrowheads[{{0.013258147025265083`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9302993704674841, -0.8126526731675918}, {
         0.8591743137430424, -0.8294525899903031}}]}}, 
      {Arrowheads[{{0.015252649417393726`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.935360551838766, -0.6839236496166874}, {
         0.8541131323717606, -0.7055500345938392}}]}}, 
      {Arrowheads[{{0.017915152185791726`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9419893645294491, -0.5540952775800169}, {
         0.8474843196810774, -0.5827468276831415}}]}}, 
      {Arrowheads[{{0.02164560872629149, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9510646165041325, -0.42245178722385457`}, {
         0.838409067706394, -0.4617587390919353}}]}}, 
      {Arrowheads[{{0.027261725681116195`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9644055723507341, -0.28765061890436594`}, {
         0.8250681118597923, -0.3439283284640556}}]}}, 
      {Arrowheads[{{0.036760391471791165`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9868193540143226, -0.14721507521609084`}, {
         0.8026543301962039, -0.2317322932049622}}]}}, 
      {Arrowheads[{{0.05575762470829354, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.035439905843183, -0.001362685741622495}, {
         0.7540337783673438, -0.12495310373206212`}}]}}, 
      {Arrowheads[{{0.055757624708293575`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7540337783673435, 0.12495310373206173`}, {
         1.035439905843183, 0.0013626857416220925`}}]}}, 
      {Arrowheads[{{0.036760391471791165`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8026543301962039, 0.23173229320496186`}, {
         0.9868193540143226, 0.1472150752160904}}]}}, 
      {Arrowheads[{{0.02726172568111622, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8250681118597923, 0.3439283284640552}, {
         0.9644055723507342, 0.2876506189043655}}]}}, 
      {Arrowheads[{{0.02164560872629149, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.838409067706394, 0.46175873909193493`}, {
         0.9510646165041325, 0.4224517872238542}}]}}, 
      {Arrowheads[{{0.017915152185791747`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8474843196810774, 0.582746827683141}, {
         0.9419893645294493, 0.5540952775800164}}]}}, 
      {Arrowheads[{{0.015252649417393726`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8541131323717606, 0.7055500345938388}, {
         0.935360551838766, 0.683923649616687}}]}}, 
      {Arrowheads[{{0.013258147025265083`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8591743137430424, 0.8294525899903027}, {
         0.9302993704674841, 0.8126526731675914}}]}}, 
      {Arrowheads[{{0.011710497567857268`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8631609187058261, 0.9540523628588403}, {
         0.9263127655047003, 0.940684479246422}}]}}, 
      {Arrowheads[{{0.010476590251087013`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8663767472134977, 1.0791117910164718`}, {
         0.9230969369970288, 1.068256630036159}}]}}, 
      {Arrowheads[{{0.009471236451657348, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8690210070894822, 1.2044844189555217`}, {
         0.9204526771210443, 1.1955155810444773`}}]}}, 
      {Arrowheads[{{0.00923284850957926, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0770114267049187`, -1.1927084333128477`}, {
         1.0282517311898185`, -1.2072915666871522`}}]}}, 
      {Arrowheads[{{0.010165145654038659`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0792404725454374`, -1.0649157380738263`}, {
         1.0260226853492997`, -1.0824526829788053`}}]}}, 
      {Arrowheads[{{0.011294651277663588`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0818590479101802`, -0.936653992186904}, {
         1.023404109984557, -0.9580828499183591}}]}}, 
      {Arrowheads[{{0.012688663947837972`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0849589539706883`, -0.8077128534171584}, {
         1.020304203924049, -0.8343924097407366}}]}}, 
      {Arrowheads[{{0.014448986064514146`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0886518091485597`, -0.6777535805630239}, {
         1.0166113487461774`, -0.7117201036475027}}]}}, 
      {Arrowheads[{{0.01673850409958773, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0930613267556397`, -0.5462010857677145}, {
         1.0122018311390977`, -0.5906410194954439}}]}}, 
      {Arrowheads[{{0.019839198345411414`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0982799133104846`, -0.4120044906628739}, {
         1.0069832445842526`, -0.4722060356529159}}]}}, 
      {Arrowheads[{{0.024299476814383568`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1041673430332757`, -0.2730183773164044}, {
         1.0010958148614615`, -0.3585605700520171}}]}}, 
      {Arrowheads[{{0.0313959980966581, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.109204036340228, -0.12399680312140632`}, {
         0.9960591215545092, -0.2549505652996467}}]}}}, 
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.03139599809665812, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.9960591215545092, 0.25495056529964644`}, {
         1.109204036340228, 0.12399680312140585`}}]}}, 
      {Arrowheads[{{0.02429947681438358, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0010958148614615`, 0.35856057005201675`}, {
         1.1041673430332757`, 0.27301837731640394`}}]}}, 
      {Arrowheads[{{0.01983919834541143, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0069832445842526`, 0.47220603565291563`}, {
         1.0982799133104846`, 0.4120044906628735}}]}}, 
      {Arrowheads[{{0.01673850409958773, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0122018311390975`, 0.5906410194954435}, {
         1.0930613267556395`, 0.5462010857677141}}]}}, 
      {Arrowheads[{{0.014448986064514183`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0166113487461774`, 0.7117201036475023}, {
         1.08865180914856, 0.6777535805630235}}]}}, 
      {Arrowheads[{{0.012688663947837972`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0203042039240489`, 0.8343924097407361}, {
         1.084958953970688, 0.8077128534171579}}]}}, 
      {Arrowheads[{{0.011294651277663631`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.023404109984557, 0.9580828499183588}, {
         1.0818590479101804`, 0.9366539921869036}}]}}, 
      {Arrowheads[{{0.010165145654038659`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0260226853492997`, 1.0824526829788048`}, {
         1.0792404725454374`, 1.064915738073826}}]}}, 
      {Arrowheads[{{0.00923284850957926, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0282517311898185`, 1.2072915666871518`}, {
         1.0770114267049187`, 1.1927084333128473`}}]}}, 
      {Arrowheads[{{0.008892896153838184, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.233081076372159, -1.1904073154111428`}, {
         1.1879715552067887`, -1.2095926845888574`}}]}}, 
      {Arrowheads[{{0.009724057314885109, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.234781640983646, -1.0622843140104894`}, {
         1.186270990595302, -1.085084107042142}}]}}, 
      {Arrowheads[{{0.010709964817197208`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2366602635329003`, -0.9336451129511557}, {
         1.1843923680460477`, -0.9610917291541076}}]}}, 
      {Arrowheads[{{0.011893633454494644`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2386976420039502`, -0.8042917521526894}, {
         1.1823549895749978`, -0.8378135110052054}}]}}, 
      {Arrowheads[{{0.01333354622363207, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2408190559981518`, -0.6739322034826936}, {
         1.1802335755807962`, -0.7155414807278332}}]}}, 
      {Arrowheads[{{0.015109155934006888`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2428214908665853`, -0.5421315498382004}, {
         1.1782311407123627`, -0.5947105554249579}}]}}, 
      {Arrowheads[{{0.017323942402916324`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2441975765847655`, -0.40825238362871713`}, {
         1.1768550549941823`, -0.4759581426870727}}]}}, 
      {Arrowheads[{{0.020084658062272935`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2436919678776388`, -0.271468968614242}, {
         1.1773606637013088`, -0.36010997875417944`}}]}}, 
      {Arrowheads[{{0.02334428484481492, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2382651018445074`, -0.1314205595494136}, {
         1.1827875297344403`, -0.24752680887163944`}}]}}, 
      {Arrowheads[{{0.026191028232887024`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.222491697657883, 0.008029264854853974}, {
         1.1985609339210648`, -0.13434505432853858`}}]}}, 
      {Arrowheads[{{0.026191028232887027`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1985609339210648`, 0.13434505432853822`}, {
         1.2224916976578828`, -0.008029264854854376}}]}}, 
      {Arrowheads[{{0.02334428484481493, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1827875297344403`, 0.24752680887163908`}, {
         1.2382651018445074`, 0.13142055954941317`}}]}}, 
      {Arrowheads[{{0.020084658062272952`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.177360663701309, 0.3601099787541791}, {1.243691967877639,
          0.2714689686142416}}]}}, 
      {Arrowheads[{{0.01732394240291636, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1768550549941823`, 0.47595814268707237`}, {
         1.2441975765847657`, 0.40825238362871674`}}]}}, 
      {Arrowheads[{{0.0151091559340069, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1782311407123625`, 0.5947105554249574}, {
         1.242821490866585, 0.5421315498381999}}]}}, 
      {Arrowheads[{{0.01333354622363207, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.180233575580796, 0.7155414807278327}, {
         1.2408190559981516`, 0.6739322034826931}}]}}, 
      {Arrowheads[{{0.011893633454494644`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1823549895749976`, 0.8378135110052051}, {
         1.23869764200395, 0.8042917521526891}}]}}, 
      {Arrowheads[{{0.010709964817197219`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1843923680460475`, 0.9610917291541071}, {1.2366602635329,
          0.9336451129511552}}]}}, 
      {Arrowheads[{{0.009724057314885143, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1862709905953017`, 1.0850841070421418`}, {
         1.234781640983646, 1.0622843140104892`}}]}}, 
      {Arrowheads[{{0.008892896153838184, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.1879715552067889`, 1.209592684588857}, {
         1.2330810763721591`, 1.1904073154111423`}}]}}, 
      {Arrowheads[{{0.008481405753401998, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.388876689934114, -1.188686486362938}, {
         1.3479654153290446`, -1.2113135136370616`}}]}}, 
      {Arrowheads[{{0.009198216701380624, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3900376206964475`, -1.0604396181301967`}, {
         1.3468044845667109`, -1.0869288029224349`}}]}}, 
      {Arrowheads[{{0.010025926888739125`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3911971155898544`, -0.9317217767923475}, {
         1.3456449896733038`, -0.9630150653129158}}]}}, 
      {Arrowheads[{{0.010985466362864344`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3922675860096068`, -0.8023962964656529}, {
         1.3445745192535514`, -0.8397089666922419}}]}}, 
      {Arrowheads[{{0.012099058541159505`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3930846287398957`, -0.6722936196739108}, {
         1.3437574765232627`, -0.7171800645366159}}]}}, 
      {Arrowheads[{{0.013384820889978475`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3933470651464237`, -0.5412257655667402}, {
         1.3434950401167345`, -0.5956163396964181}}]}}, 
      {Arrowheads[{{0.014841348159138826`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.392519780631493, -0.40905320652189614`}, {
         1.3443223246316653`, -0.4751573197938938}}]}}, 
      {Arrowheads[{{0.01640847670888386, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3897201708865403`, -0.27589535423375927`}, {
         1.347121934376618, -0.3556835931346622}}]}}, 
      {Arrowheads[{{0.017886158214873018`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.383785805196471, -0.142632804252653}, {
         1.3530563000666873`, -0.23631456416840002`}}]}}, 
      {Arrowheads[{{0.018854946538373912`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3741570338377262`, -0.011508839692170682`}, {
         1.3626850714254322`, -0.11480694978151393`}}]}}, 
      {Arrowheads[{{0.018854946538373912`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3626850714254322`, 0.11480694978151354`}, {
         1.374157033837726, 0.01150883969217028}}]}}, 
      {Arrowheads[{{0.017886158214873018`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3530563000666873`, 0.23631456416839963`}, {
         1.383785805196471, 0.14263280425265262`}}]}}, 
      {Arrowheads[{{0.01640847670888386, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.347121934376618, 0.3556835931346618}, {
         1.3897201708865403`, 0.2758953542337589}}]}}, 
      {Arrowheads[{{0.014841348159138838`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3443223246316653`, 0.4751573197938934}, {
         1.392519780631493, 0.4090532065218957}}]}}, 
      {Arrowheads[{{0.013384820889978475`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3434950401167347`, 0.5956163396964177}, {
         1.393347065146424, 0.5412257655667397}}]}}, 
      {Arrowheads[{{0.012099058541159521`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3437574765232627`, 0.7171800645366154}, {
         1.3930846287398957`, 0.6722936196739102}}]}}, 
      {Arrowheads[{{0.010985466362864354`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3445745192535516`, 0.8397089666922416}, {
         1.392267586009607, 0.8023962964656525}}]}}, 
      {Arrowheads[{{0.010025926888739125`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.345644989673304, 0.9630150653129154}, {
         1.3911971155898546`, 0.931721776792347}}]}}, 
      {Arrowheads[{{0.009198216701380624, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3468044845667109`, 1.0869288029224344`}, {
         1.3900376206964475`, 1.0604396181301963`}}]}}, 
      {Arrowheads[{{0.008481405753401998, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.3479654153290443`, 1.2113135136370614`}, {
         1.3888766899341138`, 1.1886864863629378`}}]}}, 
      {Arrowheads[{{0.008029516420398373, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5445970124388382`, -1.1875280591337964`}, {
         1.508034566508531, -1.2124719408662035`}}]}}, 
      {Arrowheads[{{0.008632388500394736, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5452824723332077`, -1.0593203175553567`}, {
         1.5073491066141613`, -1.088048103497275}}]}}, 
      {Arrowheads[{{0.009308361368073401, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5458489909159314`, -0.9307361200867315}, {
         1.5067825880314376`, -0.9640007220185317}}]}}, 
      {Arrowheads[{{0.01006332524814314, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5461903464254663`, -0.8017064281087493}, {
         1.5064412325219028`, -0.8403988350491458}}]}}, 
      {Arrowheads[{{0.010898370452419875`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.546141180936287, -0.6721715701520691}, {
         1.506490398011082, -0.7173021140584576}}]}}, 
      {Arrowheads[{{0.011803683976666346`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.545454051805939, -0.5421135290766046}, {
         1.50717752714143, -0.5947285761865536}}]}}, 
      {Arrowheads[{{0.012747611594505373`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5437858742302462`, -0.41162256576020845`}, {
         1.508845704717123, -0.4725879605555815}}]}}, 
      {Arrowheads[{{0.013660250109201997`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5407342641377548`, -0.28101043363324496`}, {
         1.5118973148096142`, -0.3505685137351765}}]}}, 
      {Arrowheads[{{0.014418887286838048`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5360051323996444`, -0.15093273814441943`}, {
         1.5166264465477248`, -0.22801463027663366`}}]}}, 
      {Arrowheads[{{0.014861811736278874`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5297576674847648`, -0.0223417478064045}, {
         1.522873911462604, -0.10397404166728011`}}]}}, 
      {Arrowheads[{{0.014861811736278869`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5228739114626042`, 0.1039740416672797}, {
         1.529757667484765, 0.02234174780640412}}]}}, 
      {Arrowheads[{{0.014418887286838048`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5166264465477246`, 0.22801463027663324`}, {
         1.5360051323996442`, 0.15093273814441902`}}]}}, 
      {Arrowheads[{{0.013660250109201997`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5118973148096142`, 0.3505685137351761}, {
         1.5407342641377548`, 0.28101043363324457`}}]}}, 
      {Arrowheads[{{0.012747611594505373`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5088457047171229`, 0.47258796055558105`}, {
         1.543785874230246, 0.411622565760208}}]}}, 
      {Arrowheads[{{0.011803683976666346`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.50717752714143, 0.5947285761865533}, {1.545454051805939, 
         0.5421135290766043}}]}}, 
      {Arrowheads[{{0.010898370452419875`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.506490398011082, 0.7173021140584571}, {1.546141180936287,
          0.6721715701520686}}]}}, 
      {Arrowheads[{{0.01006332524814314, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5064412325219028`, 0.8403988350491454}, {
         1.5461903464254663`, 0.8017064281087488}}]}}, 
      {Arrowheads[{{0.009308361368073413, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5067825880314376`, 0.9640007220185314}, {
         1.5458489909159314`, 0.930736120086731}}]}}, 
      {Arrowheads[{{0.008632388500394736, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5073491066141613`, 1.0880481034972744`}, {
         1.5452824723332077`, 1.059320317555356}}]}}, 
      {Arrowheads[{{0.008029516420398373, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.508034566508531, 1.212471940866203}, {
         1.5445970124388382`, 1.187528059133796}}]}}, 
      {Arrowheads[{{0.0075642286772638605`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.7003908763717748`, -1.1868533772880578`}, {
         1.6680301762598049`, -1.2131466227119418`}}]}}, 
      {Arrowheads[{{0.008062872078903572, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.700700305422889, -1.0587873299968147`}, {
         1.6677207472086903`, -1.0885810910558167`}}]}}, 
      {Arrowheads[{{0.008605961716456799, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.7008332692130659`, -0.930448598159291}, {
         1.6675877834185138`, -0.9642882439459722}}]}}, 
      {Arrowheads[{{0.009191389106786005, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.7006956272512557`, -0.8018176911758822}, {
         1.667725425380324, -0.8402875719820128}}]}}, 
      {Arrowheads[{{0.009811438055205082, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.7001601607908394`, -0.6728998424340704}, {
         1.66826089184074, -0.7165738417764563}}]}}, 
      {Arrowheads[{{0.010449121743499969`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6990662459329837`, -0.5437493094533038}, {
         1.669354806698596, -0.5930927958098546}}]}}, 
      {Arrowheads[{{0.011073551308869053`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6972353239492042`, -0.41450398388801557`}, {
         1.6711857286823755`, -0.46970654242777427`}}]}}, 
      {Arrowheads[{{0.01163606361045972, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6945181741890525`, -0.2854206431577522}, {
         1.6739028784425274`, -0.3461583042106693}}]}}, 
      {Arrowheads[{{0.01207166955237563, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6908846588981998`, -0.15687893949091172`}, {
         1.6775363937333796`, -0.2220684289301413}}]}}, 
      {Arrowheads[{{0.012312273747450541`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.686528253997264, -0.029302973382940543`}, {
         1.6818927986343157`, -0.09701281609074405}}]}}, 
      {Arrowheads[{{0.012312273747450541`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6818927986343157`, 0.09701281609074366}, {
         1.686528253997264, 0.029302973382940155`}}]}}, 
      {Arrowheads[{{0.012071669552375623`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6775363937333798`, 0.22206842893014092`}, {
         1.6908846588981998`, 0.15687893949091133`}}]}}, 
      {Arrowheads[{{0.01163606361045973, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6739028784425272`, 0.3461583042106689}, {
         1.6945181741890523`, 0.28542064315775173`}}]}}, 
      {Arrowheads[{{0.011073551308869053`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6711857286823755`, 0.4697065424277739}, {
         1.6972353239492042`, 0.4145039838880152}}]}}, 
      {Arrowheads[{{0.010449121743499969`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.669354806698596, 0.5930927958098542}, {
         1.6990662459329837`, 0.5437493094533034}}]}}, 
      {Arrowheads[{{0.009811438055205082, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6682608918407402`, 0.7165738417764559}, {
         1.7001601607908396`, 0.67289984243407}}]}}, 
      {Arrowheads[{{0.009191389106786005, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.667725425380324, 0.8402875719820123}, {
         1.7006956272512557`, 0.8018176911758818}}]}}, 
      {Arrowheads[{{0.008605961716456799, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6675877834185138`, 0.9642882439459718}, {
         1.7008332692130659`, 0.9304485981592906}}]}}, 
      {Arrowheads[{{0.0080628720789036, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6677207472086906`, 1.0885810910558165`}, {
         1.7007003054228893`, 1.0587873299968142`}}]}}, 
      {Arrowheads[{{0.0075642286772638285`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.6680301762598049`, 1.2131466227119416`}, {
         1.7003908763717746`, 1.1868533772880576`}}]}}, 
      {Arrowheads[{{0.007105765729122507, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8563490870514354`, -1.1865589674362338`}, {
         1.827861439264355, -1.2134410325637661`}}]}}, 
      {Arrowheads[{{0.007514390773930752, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.856384775972655, -1.058683394951912}, {
         1.8278257503431352`, -1.0886850261007193`}}]}}, 
      {Arrowheads[{{0.007947644178409857, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.856229053933876, -0.9306252301642465}, {
         1.8279814723819143`, -0.9641116119410168}}]}}, 
      {Arrowheads[{{0.008400220273752666, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.855810474493328, -0.8023929177965561}, {
         1.8284000518224626`, -0.8397123453613389}}]}}, 
      {Arrowheads[{{0.008862390520394009, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.855043765115739, -0.674019274709913}, {
         1.8291667612000513`, -0.7154544095006136}}]}}, 
      {Arrowheads[{{0.00931837251275419, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8538367663093358`, -0.5455744394626324}, {
         1.8303737600064542`, -0.5912676658005258}}]}}, 
      {Arrowheads[{{0.009745004988461734, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.852107316296782, -0.4171786638291032}, {
         1.8321032100190084`, -0.46703186248668677`}}]}}, 
      {Arrowheads[{{0.010111826059493317`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8498130090182303`, -0.2890070681230849}, {
         1.8343975172975602`, -0.3425718792453365}}]}}, 
      {Arrowheads[{{0.010384135251940032`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8469897043499912`, -0.16127359250823387`}, {
         1.837220821965799, -0.21767377591281917`}}]}}, 
      {Arrowheads[{{0.010530054264729322`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.84378085260945, -0.03418415958265317}, {
         1.8404296737063404`, -0.09213162989103144}}]}}, 
      {Arrowheads[{{0.010530054264729333`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8404296737063404`, 0.09213162989103107}, {
         1.84378085260945, 0.03418415958265275}}]}}, 
      {Arrowheads[{{0.010384135251940032`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.837220821965799, 0.21767377591281878`}, {
         1.8469897043499912`, 0.16127359250823348`}}]}}, 
      {Arrowheads[{{0.010111826059493317`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8343975172975602`, 0.34257187924533616`}, {
         1.8498130090182303`, 0.2890070681230846}}]}}, 
      {Arrowheads[{{0.009745004988461734, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8321032100190082`, 0.4670318624866863}, {
         1.8521073162967818`, 0.41717866382910274`}}]}}, 
      {Arrowheads[{{0.00931837251275419, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8303737600064545`, 0.5912676658005255}, {
         1.853836766309336, 0.5455744394626321}}]}}, 
      {Arrowheads[{{0.008862390520394009, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8291667612000513`, 0.7154544095006132}, {
         1.855043765115739, 0.6740192747099126}}]}}, 
      {Arrowheads[{{0.008400220273752666, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8284000518224623`, 0.8397123453613384}, {
         1.8558104744933277`, 0.8023929177965556}}]}}, 
      {Arrowheads[{{0.007947644178409857, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8279814723819143`, 0.9641116119410164}, {
         1.856229053933876, 0.930625230164246}}]}}, 
      {Arrowheads[{{0.007514390773930752, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8278257503431352`, 1.0886850261007188`}, {
         1.856384775972655, 1.0586833949519117`}}]}}, 
      {Arrowheads[{{0.007105765729122507, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.8278614392643548`, 1.2134410325637657`}, {
         1.8563490870514352`, 1.1865589674362333`}}]}}, 
      {Arrowheads[{{0.006667360476695942, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.012513619240506, -1.186543046349873}, {
         1.9874863807594947`, -1.213456953650127}}]}}, 
      {Arrowheads[{{0.0070010734352654105`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.0123631768363386`, -1.0588693179660567`}, {
         1.9876368231636625`, -1.088499103086575}}]}}, 
      {Arrowheads[{{0.007346589267291133, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.012030599288161, -0.9310818938544173}, {
         1.98796940071184, -0.9636549482508461}}]}}, 
      {Arrowheads[{{0.007697986317420561, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.0114663564971433`, -0.8032013765432072}, {
         1.9885336435028576`, -0.8389038866146877}}]}}, 
      {Arrowheads[{{0.00804635537870171, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.010617786332993, -0.6752670875966237}, {
         1.989382213667008, -0.714206596613903}}]}}, 
      {Arrowheads[{{0.008379259286396018, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.0094361580941884`, -0.5473425101179439}, {
         1.9905638419058127`, -0.5894995951452144}}]}}, 
      {Arrowheads[{{0.008680656128542423, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.0078883988865894`, -0.41951815397587583`}, {
         1.9921116011134115`, -0.46469237233991406`}}]}}, 
      {Arrowheads[{{0.008931755667815892, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.005972746368271, -0.29190800170220904`}, {
         1.9940272536317298`, -0.3396709456662124}}]}}, 
      {Arrowheads[{{0.009113226996403868, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.003734167999657, -0.16463562173976592`}, {
         1.996265832000344, -0.2143117466812871}}]}}, 
      {Arrowheads[{{0.009208710274008982, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{2.00127155893297, -0.0378094110550595}, {
         1.9987284410670307`, -0.0885063784186251}}]}}, 
      {Arrowheads[{{0.009208710274008982, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9987284410670307`, 0.08850637841862471}, {
         2.00127155893297, 0.03780941105505911}}]}}, 
      {Arrowheads[{{0.009113226996403889, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9962658320003437`, 0.21431174668128675`}, {
         2.0037341679996574`, 0.16463562173976554`}}]}}, 
      {Arrowheads[{{0.00893175566781592, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9940272536317296`, 0.33967094566621203`}, {
         2.0059727463682715`, 0.29190800170220865`}}]}}, 
      {Arrowheads[{{0.008680656128542423, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9921116011134115`, 0.4646923723399137}, {
         2.0078883988865894`, 0.41951815397587544`}}]}}, 
      {Arrowheads[{{0.008379259286395985, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9905638419058127`, 0.589499595145214}, {
         2.009436158094188, 0.5473425101179434}}]}}, 
      {Arrowheads[{{0.00804635537870171, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.989382213667008, 0.7142065966139025}, {2.010617786332993,
          0.6752670875966232}}]}}, 
      {Arrowheads[{{0.007697986317420543, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9885336435028576`, 0.8389038866146873}, {
         2.0114663564971433`, 0.8032013765432069}}]}}, 
      {Arrowheads[{{0.0073465892672910866`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.98796940071184, 0.9636549482508455}, {
         2.0120305992881606`, 0.9310818938544168}}]}}, 
      {Arrowheads[{{0.007001073435265359, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9876368231636625`, 1.0884991030865745`}, {
         2.012363176836338, 1.0588693179660562`}}]}}, 
      {Arrowheads[{{0.0066673604766959715`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.9874863807594947`, 1.2134569536501265`}, {
         2.012513619240506, 1.1865430463498723`}}]}}}}}, 
   GraphicsComplexBox[CompressedData["
1:eJx0nXdcVMf39zd2jd3YY42JsZfYjQ4ae28xltgVS8QuVnRFwB5LYm/Ebmyo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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV01toznEcx/HvHIdtNjbMbDOEkJXRJkKMjCQTCY3mlOZ0gWYoa41QQuJC
LHEhlrgQkkMSchYXI8eLOSRyyrl4PRefvXv/v7/P9/88//2fgsqV5SuSIiLH
n+SUiMFdIn5mRszuHvEL5+AsLHQ9xXwT/8Mr5K/U8lTXi8zT8V7XiPtyV6o6
WpwVkZcb8c/ZYmeSeDJvgcN5iWTq1dvT2rWFzrXBzTzL9ZHm2fjcvhfyTFbZ
m+xMvj1tcZQz7bA9b49j+GjJ1dtmT4pra+xNxe08z/Vx5n3ws3019n3C9djR
mZ72pOH4xHfCDjwDa+xYrL9EFsgiWShlzv3J9jxkiD1DpZtOEWZjYWrEG91J
zr7FyViT5j7mJTwX8yVPeklvGZF45tjovmd9zrpuEbtkrRyXg7LFjubOETd6
RDThNfe5gOf5TRzq+5zEE/wwNtqzEvfwDbhOfy8e4lvxnP4M3MIrsZ/+RKzh
I7BBvy3O55m4TH8wLucFif36HzpFTOe/sbv+S5zAH+Ju/QYcxBtxnv41HM7P
YYN+Pebz3ZiqX42deBXW6xfjz5yIUizXr8DW5tMS+/Vz8b35AAz9dHzOW2J1
4v3MiLjKn+J4/e/4gL/Dev3TeIZfwa/mR/EY349V+ktwJ1+NJeY78ACvxWr9
KVjHZ+Nr8zG4lhdhhf6/9Ii5PIUPNO+LS3l2Yr9+s/lU/gUfmzfhWH4Lp+nv
w/78COaZX8Rh/BRW6G/EHL4Nb5uvwjS+AEv1C/Gb928kZpjPxCTzssR+/Sx8
Y94LL5kn45PEb96zLNa/jpf5I2xl/hHv8FdYqj/Ae/vJ/6MYf/CLfhO3ZJ+c
kiPSJP8B8dWU1w==
          "]]},
        "0.42600000000000005`"],
       Annotation[#, 0.42600000000000005`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1F9MlnUYxvE7NZpT5K+mpKkH6BB1LREbZqgguFBnTA9avpAelAOmKSE6
1BpjjA5SmcuDAub0wBVreuDQtaxps/IPrvIAWzOdUqtWlii6cmqf5+Di+16/
677u53n38LzTN2yu3vRERJT6czc3YuuEiAKqfDqigpbRsPNCZ7PoZZ9vT46o
wpqMiHXyFKWnR4wYGzHbzJO4Ky/inpk1dJ9282nO58jX8rX8f7gOayhFz8mu
TYq4TufsfNP+b3EjPjTbamacHUXm5tOo8RGjp0SMxBK+1sxd9/Y6NujU6dZT
A03QW2QmDzvsGaOzlF9C9ebv6TXgNr0t5rcmHd9pqvlyM/m4Ry9br8VchbOd
OORed+EdfEd3hvuZSZ3ybMyh3ORM/yU7S5O9dryI43ERNprtGBfxEVY568Ju
6qG3aLG95eZWyMpwJTY5f5u20e98lZmbPt+iM3SF+ukr1z09MeIi9dENGqD9
rpflvn/2vdPwe9/1n5yIy/wDLLXvjL39VCE/gXNduxdP2Nlqxz5qok+om9rt
HNT9xo4BPGvn5/gZfw7n2XkUP+UPYa89m3A/34Lb9T/Ag3wHntRfg+38epyh
X4k7+BLs0U/DWj4H6/XnYAM/Ldmv/2d2RDX/L07Sv4bL+B+wU78HC/lerNE/
iy/wJ7FHvw2f5TtxrH4zZvF12KZfjPef8VzwFf0UjpKvTvbrT8Y/5AX4WJ6B
V/kR2Kx/Kcsz4n/Ccvkwfsf/hm36x7GPP41D8iP4Mf8h1um/gXv5Rlwgfx+7
+HexWX8FtvKv4i/yUmzin8eU/qPMiNf4MfwseT5u5Ccm+/UH5av423hFPoBL
+PO4Wv8AzuQP4xT5KSzij2FKfyfm8e/hBflmTOc3YJn+XLzjvVqImfK1GPLl
yX79XPxVPh2/kD+FPya/HZ5Fsf7X+CV/GUfK/8KL/HUs0y/wv/u351GcvOP8
Ke/peTpAx+gwDdD/CXKyzg==
          "]]},
        "0.35500000000000004`"],
       Annotation[#, 0.35500000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV03tszXcYx/HHhMVc2rUdrVYQl7XN/IHUMhORbi6jmE0WpLOysZjUwqqt
S0LTyCwRRPhDaIw/hGaZP4SylSDFDNlcdljHzC1sxqZx2xLl9fvjfd7n8zy/
5/n+zvmd03vm5+/NaxMR73hp7Rixr1vEfjSgNC3iOy7JijjAH8mNXMaT1JZ2
j2jbKeKQ2idq05Nrkj5eVD/Os9U/TvpI6xzRQX1g14hBaLWjzI6ZmIEhatdy
Is669hzO4Becx031X7kZF1Fu7288y46rPF/+gxdwudpX9uU66y+1KrUvuAIL
0Uu9hZeqV3E1ctxbf/XR7mEMsl6JGKqeyW/yWLVxyLe3AK/iuHMKeSpavN+i
/zXG2zufS7hPj4h+6IsJdlW7dhF22zlBf7nratABH8gZ6nl4nNQ4cNp9PciO
eIo7SPNdtMfmLhGDeRiyUYjeaPZZUs66gmFowTL7bvNYZ1zz/i4O4wJO4Ujy
HO0+iT24ihTWOSPdZ7ucF9GOf7b7n0zPSP6fh9t32N5TGKm/lwe453rea2eN
HWtQgZ3YjBV2Xjd7zI4UN9n5Pe+Xf+BBdn7L38hbud6ecl4nL+Yq8+t5i/wl
N5h/n1fIZdzP/Ciult/gOvPteLqcwZ+Zf43nyj2T/ebvZPhNy0842/zv/LZ8
htear+NCuZ4/NN/Er8sNXGe+lnvIa7mj+UpOl+dwrfkifpQbUczvmi/ltvoT
k/3mc/lP/Xxu1e/Cl+Q2XGn+9MuekdzMb+k/4J/kW1xrfjfvkQ/xff3tvEPe
yHPMz+LV8gIeor+KN8nLuNL8OK6Rp/AN/eFcIQ/kUvNP0yOmyS/JBfp9+VO5
W7Lf/HX98fK/fEE/xSPkEzzR/AbuL2/jPP1GHizv4lLzSzhHXsk/6s/jTvIM
LjY/gFv8b4Zymv5kfiaPTvabz+Sbci8+oN+eL8r/eRZF5o/yQfksv6D/N5+U
r3Cx+Xy/3XueRxE/lBv9j05gA3ZhG1J4DoVcwSg=
          "]]},
        "0.28400000000000003`"],
       Annotation[#, 0.28400000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV029MlWUch/Ebh8s0gUMGKpi1cuakEoeac64R5mZLVNA1PbmUmUELCgk1
XzDHtNpaOpcvWqLTqVPW5IUD1iRFw5YBM908OKcyQVeNlHJAZPPf53lxefG9
f+f7u885j+f5ko+LKpJCCMX+WTwuhJbMEFanhrCSVyGOR2NC2DIxhELztWan
nJVwKxc5+4nfl8/wWbShe0IIlc+EsAGrvKZHXslpY0MY/VQI7V7TEfWwHunO
U513+fsyPkApyhBzfo0r3FEeOZrbNZStyzflKrNe/pRvRa9xfps3y39xP+5E
eB+DPIQBDONfDDqfwbk4mBHCYRzCHDnHZ38Z0/EqXsHeFHvd8cQk93Afljkv
ir4nLMUSFCPDnks+w2J3LMdClCCOQ/bU8HaUYRM+QY/v43/fXZJuAafxlxiF
FT5TzPvKit57dBY9I+50x8D4EO6jDyn2jMQed8zkecjENDyHK+5I2N2NebiL
Gnv+4EXu6PH3bZxGFzpwxh2tdrejETeQwC53pPoOrnkmyfyb3f1Ph3BRvsfz
7TttbwcWmDdxjvddz012brVjB6pwFHuwzc5e3Z/tSHCbnSf4B/kXzrXzGH8v
7+d6ez7iXfJnvFH/G94nf87N+kW8TX6PX9R/kzfJr3GdfjKvlmNcpj+dP5Sf
jfbr96V7tvIwZ+pf5wL5Au/Ur+Npcj2/q9/Gs+VmrtOv5Wx5J4/Wr+ZUuZRr
9fN4KCuEfF6iH+cR5oXRfv2J/Kf5VH5gPpavyoGr9TtjnpF8hd8wH+Dz8u9c
q3+cG+VW/sf8MB+Rv+VS/XX8tVzJs8y/4u/kGq7Wf4u3yu/wTfP5XCXP4Lj+
/TS/c/lJ+SXzF3i9nBHt1+81f1v+m7vME/y6fI4L9XfzFPkAZ5m38Ey5geP6
W3i8/AX/al7OY+Q1nK+fw3f95uZyinkxP5QXRvv10/mWPJl/NB/Jl+X/PIs8
/bN8Ur7ISdFvnNvlbs7Xn+r/7h3PI48H5Ra/o3PYjQYcQAKPAWmgzMY=
          "]]},
        "0.21300000000000002`"],
       Annotation[#, 0.21300000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1H1sjWccxvG70WRjbbVoSztbilQzf0y1bBGRYhJv3eYl2GFbWhHKstNY
vZSUNDVLFsTmD6FZ1mWbNULEW8RLvBQ1zEgcIqwZ61Rqm1mHEjM+zx9Xv891
/+7rup/nnPM0rzw+5eOkEEKVP0kpIVRmhjAhO4TxNI7e6RXCTGspqTaZr8gJ
4V1rc7qH8EufEFroOk21lmZPsj2n5JqpjMpphlnHyyFMx090VdFs1/9am4UZ
ci/JrdL9Pl+h+ye5BXgef9ffSovlethXZ1+Zfe/xMeppbbW1cmtxmdv2Vrhu
w/nYS3+2Pbd0tVGcKiOZdbqHOFbrWU5LXd+VW4LZcnlyj+yt0VuNy+mheSc9
oJX25Tu7GRvMvqFhVMcPxf/teUo5ugp0Fdo7hAZTMRVRKU2idpnt+H2a+84K
4Tif1DeEpuhe3NsXtM1sv3vZbd9uLNbb27zVc/TFneZ5svn0D5+LM+3NxEvO
nxR9VzSWyihG38rU4Oro84qeneJ0Q/djZwbZ0dgdP6MXaJqz0z1fDj2I1qJn
xXPO6OgdwhNqp1Q9ybTFGYU4nLKogF6lq85I6G6h4XSPavS04Thn3HD9Bx2l
y3SWjjnjiO4ztId+pQRtcEaaZ77us+iCP+v+q2cIF/lHOELfUb1naYz5Xhzk
vhtxb/T707GOFtEPtIXqdN6UPakjgU06D+B+vhkH69yO2/ivsVHPQtzAL8PF
8l/iV/ynuE9+MtbxH2B/+bdwCf8G1st3wdl8Os6Xfw0rou836pdv7+G95B9i
VvQe4mj+Aq6Xr8cCvhFnyTfhUH4f1svXYi6/HrvKV2EaPw9r5Yvwfm4IJfi2
fAyTzEujfvk+eNs8H/8zT8Fr/LMMXfLn8Bh/FUeZd+B5/hbWyu/CPfwR/Nv8
O9zKb8J58nNwLV+Jxeaf42a+JuqXH4+r+On4m/kIXMS/jjH5J+l+8/yL/EDz
fjiXz4z65W+aT+Tv4mXzBI7kT2Op/EYcwDdgjvlBLOR3YEy+GrP5Nfij+UfY
jf8QS+QH4T3v3JuYaj4Fn/Jjo375DGzlX8FD5sl4he/0XRTJn8DD/EUM5nfw
DN+CJfL5frt/+j6Kov+h/EHv0WnaSDuogRL0HKg92GQ=
          "]]},
        "0.14200000000000002`"],
       Annotation[#, 0.14200000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV03tM1XUYx/En2bpMEExQIGyiI6d/pMPUGWtkudUSS9Hm0HBOG6ZtOlIU
FyMirVUyzJkpNd10xVzOJTqnNXXWMnCWW8fGVOZtXUAtJLsYXez1++PN+3zO
8/0838PvQOGiFeXL74iIaj9KhkYMyYiYMSyiDNOxODtiTU7EauSmR7yRH7HM
e/Pla3kR19GNq+hBrfcLnOvW7cFPqMmMeEmnGjn2P+fMKq9vFUSs5JflUTpN
dq+V++xZrbdWr5dvyL3I1R3p3Abnap2rN2/gV7DAjkr0O9eY7OZX+S++yWP0
mvXmJh2UOzt4eMRsfgazkCXP5DkY6/xG59frrsNkrzt4IrfzJD7PaTpdvMvO
dz2/LXgPD3uv0Gffia32nZK38/toGxRRlRuxFJVYjIXodOdb5k3Ix2ZsxG7n
D9q/z537eILnMMy+K55fAe81H+HOItyQ83mus9n8rZ3TPYNyTMNCVGCnTh2/
hiWowXJctPuWO29jKgbZ8TruxGx3Z/p98vBb8h7/yyfd0efz96Mb6fakYZs7
xvMU5GA07kenO1J2d2EKelFnzw/8hDsuen0VR3EGHTjmjiN2t6MNF5BCszsy
/M7nPIsB/LXd14dEnJb/5BL7jtrbgcfM9yffrc/dyvvtrLdjA6rxEbah0c5L
ul/YkeLjdh7ig/KXPM7Oj3m3vJ1b7VnGzfIaXqX/Dn8gr+MD+jO5Ua7kkfqP
c408iVv0B/B8OZOX6I/hF5LvN9mv331vxNPy75yjf56nyt9wk34Lj5ZbeZ7+
cX5IPsAt+g2cLzfx3forOUOu4gb9Yr55X0Qpz9Cv4DAvS/br5/KP5kX8t/lA
Piv/N9gu/ZN8TO7kR837+JT8PTfof8Jt8hH+xXwXfyhv4Sr9Rfy2vIInmL/J
W+W6ZL/+k1wvP8uXzUu4Wn6QK/T7s/zNy3fJD5gX8vNydrJf/5L5U/LPfMY8
xY/IJ7hMfxOPkndwnvlhHi/v4Qr9Wh4qr+evzF/ke+QFXKo/lnv9z03mdPNZ
/I88Ldmvn8VX5OH8qXkafyf/4bso1v+cP5NP823ne7hd7uJSucjf7jXfRzH/
Kh/2f3QCm7AHO5DC/6nL2x4=
          "]]},
        "0.07100000000000001`"],
       Annotation[#, 0.07100000000000001, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV02tsjncYx/F/a8X0hJ6pmKWTeYGUkBKpU0I2NoeI1EIEK7NkCGIvGmmc
EofGIbJRQuLUFRGqq1MQWxjCJB7SmFqrDnGc2ZxSh/K5X3z7fX7/676u6356
30+XqbPHzooLIXzrT2NSCBnZIWSiPdKRhpvO23I7JCMVKWhwnshJaI02+Bj1
zltyK7RAAj7C387jOB7NWZbye77h/C2/QxPe4DXqnFdmhrAbFfgFGZ1CyEIm
rqt/7bpRKEA/dHCei454mBzCnXT34vNdbsRtJMi3+C/9R1NDqO4QwjF+4PpD
dhxGNWrwK/o4/zkjhB3mb+ALerfwJlSlhFDse3yHiZiKyag1e4V6GXKwHmtQ
6foauw7YeYB7mZ0Z3Vuue+Z96p3tzMO/cg6Pd20aXzbzy5wQxmAoJqMI2/SU
8GJMx3z8gHqzX9nZjEFINmMZEjDW7hTfJxvPojN+y+fteOr+m3APiebEY6Md
PbkA6eiKTqi1I2Z2HQrwBCXm3OFhdtT7/AAncAXncNKO42afRVX0XiCG1XYk
Rc/F/yKOL5r9KC2ES/JL7m/eCXPPYbD6Qe7mviv4oJkLzViFOdiFjVhkZoPe
382I8SkzD3ONfJp7mLmHK+UtXGHOTF4tL+B5+tfyZnkJV+sfxYvkidxF/xCe
L/fh8ugd52/kFJ6u/3OeET3faL7+e+1D+Ep+zun6r/Mg+U8u01/OXeUKnqD/
FPeWq7lcfynnyGXcSv9cTpKLuVR/Pv/fMYRCHqm/iN/LI6L5+rP4rpzHr9Xb
8DX5XTuz9J/nk3ItD1R/yhfk21yqfz9Xycf5H/XtvFP+iYv1T+GV8izupb6c
N8gl0Xz9w3mhPI5vqvfnOXJ3LtLf1NY7L7eUP1P/hKfJadF8/Q3qX8iP+Yp6
jAfIZ3iE/nX8qbyVs9WPcE95Lxfp/5Ez5KX8h/r33FqexIX6u/ETv7m+nKg+
mt/IQ6P5+lO5Uc7lo+rxfFV+4Vnk6/+Nj8mXuNn19/msXMeFcp5396Hnlc//
yUf8js5gHfZiK2L4AGjH0J8=
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV03tsjXccx/FfEyYuxVq007pUWbuEpmTSCHXLCHVncxgRpObSVoPxRyON
jYRqaiIunRCERohQIlgQW1zDJDusY41bbGFYZ8PEPV7PH5++z+f5/j6f73n6
PCdjZun4+QkhhIX+vG4RwpftQpiUEkKMJtInbUOYHF2jLJ+7dAghG7e3CaG4
fQglVERdXZvozLzoc2IIc3Gl/JpWIWxzdjP/RP8EZz6nTOdb6OqC4/nOmMFn
4CqZj1PlMQsb5MY48wg36unoTJXODriBf+j6KPMK1wbZme76QEzDOeYjzYqp
3rwCv3B9PpZQnu86hE+0eyh+RgXUih+ORVQX7eJ/w18pX+d17IfXMEfHNvsW
81/Truie5WZH++kYX2TXRWe38N9TbcsQCt3bbJpCM2ga1bmXVeaVlErraA3t
dv6wngP6D2BPO9vqu5MeQnvcZ94x+r/SP3xqdJ/OJuEvOod/FMJYGkzTKEbb
ZcrwG5pFi6iYbup+budbGhA9Jx0rqBGNszvRvabQk+gavsYLdjz2/V/QPWqm
J4E22ZGDeZRM3Sid6uyI666nPGqgMj1/4BA7bvr8F52gK3SeTtpxXPc5qqUb
FKcqO5q75+v+FwEv6X6YHMJl/n/so++E3vPRe2F+ELN97xo8qHOpjtVUSrto
Ey3TeUv2Jx1xPKXzCB7mT2MPnXtwN78Fa/TMwSp+MS6U/w4389/iIfnRuIyf
gp3lB+Ei/lOslg84mU/EWfJZ+FX0fKN++XtJ3mX+KSbL/44D+J+xUr4au/E1
OEn+FPbiD2G1fDmm8pX4gfwCbM4XYrl8Lv6X5v3GEfIxfMcXRP3y7fBPPhNf
mjfFa/ybD3XJX8CTfB32N3+MF/m7WC6/H2v54/i3+Q7cya/HQvnpWMGXYE/z
lbiRL4v65YfiUn4C3jbvg6V8d4zJv2jtnecb813NO+FMPinql79lPox/hFfM
49iXP4MF8msxg9+KKeZHMYffizH5JdiGX45nzediE34q5stnY4PfXG9sZj4G
X/GDo375lniHT8Nj5gl4lX/mWeTK/4g/8JfxrfP38Rxfj/l8pnf3geeVi//y
R/2OztBa2ktbKU7vAXu10h8=
          "]]},
        RowBox[{"-", "0.07100000000000001`"}]],
       Annotation[#, -0.07100000000000001, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1H1o1WUYxvFbsnRrVpu6+VbWNIV0bWKI2VwtCjPJbOkMRUI3Myg2nVYM
GeuFxMSSMncUB4VNc45NxVyhsqL5QiV4DCklTcrMl8zUFDON9fn9ce37u+77
ue7n+T3nnN0zp7qsqltE1PszLzfiBVo4IKKob0QNjsK5apWU2Stik1qhWkef
iPuxmc9Qr9DvgSNpTV7Eamq8PWK7dV2DIj7D2dbMoQK5DXLdrZ3Fd2VFrONH
qK+WeaRfRApLsZ96Hs20rhrnUxXdZ+0C3GjudL0rZjTyw9Q/VbsX1/KX1afp
X8Qh9lviXG/Tu+anrLvubA04xZpnaLDcs/pTaRpNVjsjW+b5Q88r6QN6S75n
8u7Ug5aaUYJHsMna9ckdWNeanD25B76dWtXeV/vNzDbPh3GQc7Wo7ZbtpIc8
p3EcHsBX8TV6VH60881wL61mNvObqIWq1U8kd2/mgzKrkju+LWKbdW2ybVik
30fvuHfujy36d1qfT3/yeTjV2mw84FxP9I94mkppFpXTRzK1+DpVUg29REfN
vmLPG1RCtyZ3RDfRFHtnOWMuXUxq+C/us8d573KVTlKGOUEN9ijAMZRDQ2kg
HbJH2uwjNIbOUa05v+Jj9jjq+RTtpIO0l3bZY4fZe2gz/URpWm6PTO/8o7vo
6h3xrdlncD9/Gceat9PcvfSwdVtwuHM34RYzF5vxDlXRJ9RA9WYek/3KjDR2
mLkdt/Ff40gzN+IGfi02mTMPl/OLcIH8e7iGfwO3yj+F9fwMHJz8NrCGH40p
+a6ciOf4LL5SfhjOTT7fZL78Sf1J/CXMkT+MJfx3uEw+hUP5Jpwu34Gj+K2Y
kq/DPH4Z3iw/HzP5CqyTL8QLAyOK8Un5cvyPn5jMl++LJ/h8/Ee/J/7AX882
S34f7uIP4Xj98/gN/wvWybfiZn4H/qH/Ma7jV2KF/PO4lH8Zi/SX4Cq+Npkv
/zgu5svwZ/2xWMWPwHL5q3f4zvPd+SH6d+FsPjuZL39MfwJ/Fg/qp3Ec34kT
5Vfg3Xwj5uq3YwHfjOXyr2Bv/k3crf8i3sLPxGL54XjOb+4BzNCfjNf40mS+
fC88zg/Az/UDv+f/9lkUyn+JX/D78Yb1v+Oe5P8QFvP5vrunfV6F+Bff7nfU
SSuomRopTf8DWEvgEA==
          "]]},
        RowBox[{"-", "0.14200000000000002`"}]],
       Annotation[#, -0.14200000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1GlsTmkYh/HHOqitqKVaUh1LqBKCiKXWtgQTJp0JEaQmJMQWPjTS2BJL
ldobUWppRD8IMrGkpCbWWBIvGkaGEcQuZmwxY0n5nQ9Xr/O/n3Pfz1l63pTc
eRPm1gghTPInrlEIpa1C2IWd2NckhJ9bh1BPvTwxhMoWIexRG6u2m8dxhVo2
l8gjOQuZaOf8qS1DmIa0hBD2y7XMmSxXNwyhq+NN9tiIDOcPxmf1Luob1Iqw
xcwB6u/V18mFWIvN6qX2/ZrkWjndOevU0riQu/M29Y68Su7APyIVTVxHcnR9
SEQS2qKu+k09NZNDqOIbGKY2HBkYiiEYgSz309h52fw0un+1URgZrSETfd3H
6jYhrMEqrEUBJrqezp5BF/yqP4mvub8sa+MwBJORg9LGIeTxUkzHAszCXbM/
6P2CQWhgxgrUxE/mx3lGCXgT1fgTX7THa/f5EY+jd2rOt+g52SPNcR/EIxWJ
qLJHzOw76INXyDPnIQ+3x13HT3ES13EBp+xRYeZ5HMJfiEXvxB71Pafb3ld1
8xAum/2cr8rvuZ95J829gMHOO8ydXHcZHzZzsRlrMAf7omvGEjPv6f3DjBhX
mnmUf5fPcDczD/B+eQeXmTODC+WFPF//et4uL+Mj+sfwEnkit9OfwQvkXlys
v7qZdybHydP1d+Tf5NbRfP2PrY+W33K8/j95kHyFC/QXc6pcxr/or+Se8hEu
1p/PLeUCrq1/HteXczlffzr/29Y3waP05/BXOTuar78FP5JT+D/rP/At+XO8
Wfov8im5igdaf82X5Aecr/8gH5Ir+KX13bxX3sy5+qfwank297C+krfKedF8
/SN4sTye/7bej+fIXTlH/8emfk/kWnIH68k8TW4azdd/z3qm/IKvW49xf/ks
Z+sv4vZyCSdYP8Zpcjnn6F/EzeTlfM76TK4jT+IB+jvxK99mb65nfSz/H33X
0Xz9Dfm+3IaPW//mHdyQ33G6/tN8Qr7KX5z/hM/Ld6LfKTnF/+4z7yud/5GP
+Y7OogjlKEEM3wEeSsV5
          "]]},
        RowBox[{"-", "0.21300000000000002`"}]],
       Annotation[#, -0.21300000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1FdsTnEcxvG/EcQsLVpUtOECqZ2iVmIE0cRICRIR80KkFS5EqvaOStBq
NAiN6IVRjZKSEqsxLl4iRmxildRM7BGfc/H0e57/7/8853/6nvdNmZ0zKbtO
CGGkP6ntQpiYGMIkGk8HWoQwAasSQsjEffw43IuV1krtb9AshKK2IeyiQio2
G2lPXesF/E7aQSOsDae/TUPoaja3dQjlOg7o+GNtLd/G9UB7MiidBlB/irfe
B/tSL9ruHr1xj3x33MJ3w81YYK3I/tqo033X0Gpab5ZiT431FXweLacka4n0
wnqyc1XLT7d3a5sQ8mkbHaHDVEZPzOsnh/AUH1OOtRq5Z64XeIZhSc7r+rju
cnpjlm39NRbbO9Ys3noFNaYE+mi9Dv7AaueodZ6v0Zmoob5/WNDcs7ruR3GU
Skl0S29M/z3qR+9oqZ5nONxzPHD9ik7TDbpMZ9yjUuclOkr3KRb9H92jkfPd
6eBzig/hqu43eJ3/gun6Tuu9TEPsO4ZdnLsEj+lcpmMjLYzenejMlKfzoew5
HTGs0nkCy/nz2E3nITzI78YSPfNwC78Yc+S3YhG/Esvkx2EePxWT5YfiIr43
Fsr/bRXCFL4xP1u+M87h20b98i/Mx/KfME7+Lg7mr+Em+UJM5UtwsnwV9uTL
sFA+F1vzm7CefDY24mdhrnwafmjvPcYx8ln4mx8d9cvH43O+E34zb4C3+Z8t
dclX4xn+Fg4yr8Ur/FPMlT+MR/lKfGu+D/fz23GW/AzcwC/AHubrcCe/NOqX
H4HL+An4yDwdF/JdMUv+a5zfAL4un2LeAWfyLaJ++Yfmo/gavGEewwH8BRwt
n48d+WJMMK/A7nwpZskvwZb8KrxoPh/r89MwQ74Lvou+89jQPBO/88Oifvkm
+JhPxJPm/3wGN/nPmCZ/Fk/x1/GX/S/xEn8PM/hO3t3XPq80fM9X+B5doHwq
pWKK0X82O7/y
          "]]},
        RowBox[{"-", "0.28400000000000003`"}]],
       Annotation[#, -0.28400000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV019MlnUYxvGblJhNQRH/JLAXCqcZVLq5xJSt5tTswBW1PGhOWbQhQ1cH
zTGLRlNkssmUXOicMpOaITmxgLBB6ioPXprTmmhrTjNr6aRVTpy1Ps/Bxff5
/u7nup/3H8VVm17amBERFf7snB7RKmWTIgbzIopnRQxh07SIbfKl6yiMGMB9
MyL2yqqZEc9Lps7j8oVZSq8Xi7A9J6LH9UeuM8zfsKdajjs75CycbeUFrgtl
qV17dJbgB9jlvoPOxyZGzHHvevd+6uyAs7vOGvlh/l9BRCc2e03bpVX3STtG
3fOI3l6zXJ19+J5Og+xyfU9vd3Km0yBFOim5oZfS22E2Sa8Fs7HR3neSz4OP
502YibOlRGp8fhtk2PkinsZyPGNfvn0LktdvR6dnfSyfSMXD3rdcNa8xz9U5
6rxbPpM6Z9fNLrlnhdkIrsS/ks/R/Fn7RjzzF+lLni2npd8ze72XU9IlFyUt
zdkRWfoXvPf7UyO+tftXPMv/xIX29dl9WpYkrwVLPKcDj9q52Y5tUisHZbds
sfOS7ld2pHHAzuN4jA/iY3YexkP8Q+yw53Vs5m/iRv0duIe/i936q3ALfxUL
9JfiJv4Utunfz414hU/g6/UfxSo+Pdmvf9V8Jb+NOfo/4DP8O2zSb8Ni3oEv
6w/gE7wb2/TrMY834QP6dZjF12G9fineyvf94gr9SrzHlyf79XPxCk/hP+aZ
eJ7fnWKX/hns5+dwsfkf+A3/Gev1j2AX78XfzPfjAd6K6/Rfw628BsvM38dd
/O1kv/5zuJmvxsvmC7GWz8VK/b8nR7zIM3iReT6u5dnJ/uT3Zr6M38Bh8zQ+
zYdwuX4LFvJ2nGreg/N4J1bqv4WTeQN+bV6N4/gaLNcvwd/9b8zHB81fwDu8
Itmv/xD+xGfgCfN/fQff81Es1T+Jn/OzOOb+a3iK/4jlPOW3e933VYo3eY//
myFpkU5pl7T8DxaOuqw=
          "]]},
        RowBox[{"-", "0.35500000000000004`"}]],
       Annotation[#, -0.35500000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV01toznEcx/Hf5pAzs83s0Bhha8QFGnEhjbgQc06McSE5XmitoeghbTlu
shUzs2FjNiMJOSRRRhIXSKI5hLhwmIV4PRef5/1/P9/f5/vfnuf5ZxRsyFsf
E0JI97JsQAiRxBBSUkJoSgghGZsxFQu8v1Ia+Tkp7xtCPR41+9krhOG9Q0hw
PXZgCONkT1IIibzBmSpsd2a7/gn+Ny2EWlzOh+kdd10j++yswArnvzifYdbP
daZ9WRKxM45XOlOJX53ZZsd+3mHnAczng/X2RvdJxM4SPOj8W+fTzXq47ilJ
dm43L7a3Fy+TcpnN58g757fYF7wXI/+SQ+iEsTJUjvu8aqRESuWN8w043946
e+fhSVyAbe67GE/zhXgKF+FZPK9Tj83YgC141v3XuvciXo334+3BiE6h7JQ1
ckwOSnGfEJ45c83n0IpX3O88NvHrmGlPLdbww1jtb12Ju/lGXKdfgof4VmzU
n4HFfAGm6k/C9Xw0lun/7u9/5N34cv0huIInRvfrvzafzr9gH/0nOJHfxV36
ZTiYV+Nc/Ss4ijdimX4RxvNdGKO/FrvyfCzSz8bPqSHk4DT9POzgudH9+nH4
iqfjd/PO+Ji3x9mlfxsv80c4wfwj3uEvsUi/Hs/wS/je/AhW8X2Yr78EI3w1
jjTfgQf45uh+/SlYyGfhc/OxuIaPwDz9b/381njgg8xTcCnvHd2v/8x8Kn+H
D8xbcTy/gbn6pZjGK7C/eQtm8TrM09+Effk2vGm+CmP5QszRH4of/I7HYBfz
mfiDT47u1++OL/gAvGD+x3fwMPrcYbb+VbzI7+Ev59/gLf4Uc3i6567N95WN
n3iL5+eGlEqdVEir/Acqq6OI
          "]]},
        RowBox[{"-", "0.42600000000000005`"}]],
       Annotation[#, -0.42600000000000005`, "Tooltip"]& ], {}, {}}}], 
   GraphicsComplexBox[CompressedData["
1:eJx0nXdck9cX/+OoW+vee9XxddWtrQerto4OtVZt3XvXrXXHVXfrqnvgnq3g
xHmjIqKiMgJEiJCEEEIIWQQIyPjlxzP43U9eP/+h7557b+5znzvOOffc+zSb
unDkjJIKheJMdYXi//4V/jlY16J/GUxgK7AZ2AisA9YAhwOrgBUKkBPkB9YA
64CNwGZgK7AD2AXsBs4C9gDnAucBFwAr/KA+7GjRP6f8Png2AxuBdcAa4HBg
FbBCAXKC/MAaYB2wEdgMbAV2ALuA3cBZwB7gXOA84AJghR/UhxUW/bPL74Nn
M7ARWAesAQ4HVgErFCAnyA+sAdYBG4HNwFZgB7AL2A2cBewBzgXOAy4AVvhB
fVhY0T+r/D54NgMbgXXAGuBwYBWwQgFygvzAGmAdsBHYDGwFdgC7gN3AWcAe
4FzgPOACYIUf1EecP1JhvkqV3wfPRmAdsAY4HFgFLM1XspwgP7AGWAdsBDYD
W4EdwC5gN3AWsAc4FzgPuABYmq/k+rCZRf9M8vvg2QxsBNYBa4DDgVXACgXI
CfIDa4B1wEZgM7AV2AHsAnYDZwF7gHOB84ALgBV+UB9R30mS3wfPZmAjsA5Y
AxwOrAKW9CtZTpAfWAOsAzYCm4GtwA5gF7AbOAvYA5wLnAdcACzpV3J95HaV
3gfPZmAjMP7VAIcDq4Clf7KcID+wBlgHbAQ2A1uBHcAuYDdwFrAHOBc4D7gA
WOEH9WHHiuavD/L74NkMbATWAWuAw4FVwAoFyAnyA2uAdcBGYDOwFdgB7AJ2
A2cBe4BzgfOAC4AVflAf1q1ovETJ74NnM7ARWAesAQ4HVgErFCAnyA+sAdYB
G4HNwFZgB7AL2A2cBewBzgXOAy4AVvhBfdiJovfzSn4fPJuBjcA6YA1wOLAK
WKEAOUF+YA2wDtgIbAa2AjuAXcBu4CxgD3AucB5wAbDCD+rDThfxffl98GwG
NgLrgDXA4cAqYIUC5AT5gTXAOmAjsBnYCuwAdgG7gbOAPcC5wHnABcAKP6iP
+PfII+l98GwGNgLrgDXA4cAqYOmfLCfID6wB1gEbgc3AVmAHsAvYDZwF7AHO
Bc4DLgCW1nO5PuyA8H7EdFZgM7ARWAesAQ4HVgErFCAnyA+sAdYBG4HNwFZg
B7AL2A2cBewBzgXOAy4AVvhBfdihIn4lvw+ezcBGYB2wBjgcWAWsUICcID+w
BlgHbAQ2A1uBHcAuYDdwFrAHOBc4D7gAWOEH9WFdhPVdfh88m4GNwDpgDXA4
sApYoQA5QX5gDbAO2AhsBrYCO4BdwG7gLGAPcC5wHnABsMIP6sMOF72fD/L7
4NkMbATWAWuAw4FVwAoFyAnyA2uAdcBGYDOwFdgB7AJ2A2cBe4BzgfOAC4AV
flAfll/k702U3wfPZmAjsA5YAxwOrAJWKEBOkB9YA6wDNgKbga3ADmAXsBs4
C9gDnAucB1wArPCD+rAvi8ZLkvw+eDYDG4F1wBrgcGAVsEIBcoL8wBpgHbAR
2AxsBXYAu4DdwFnAHuBc4DzgAmCFH9SHzRD8i/L74NkMbATWAWuAw4FVwAoF
yAnyA2uAdcBGYDOwFdgB7AJ2A2cBe4BzgfOAC4AVflAfdqRo/kqV3wfPZmAj
sA5YAxwOrAL2at68nCA/sAZYB2wENgNbgR3ALmA3cBawBzgXOA+4AFjhB/Vh
b4T9Kfl98GwGNgLrgDXA4cAqYIUC5AT5gTXAOmAjsBnYCuwAdgG7gbOAPcC5
wHnABcAKP6gPKxD2b+X3wbMZ2AisA9YAhwOrgL015OUE+YE1wDpgI7AZ2Ars
AHYBu4GzgD3AucB5wAXACj+ojxTvIb8Pns3ARmAdsAY4HFgFLMeXyO8D8gNr
gHXARmAzsBXYAewCdgNnAXuAc4HzgAuA5fgS+X0ULe9dM+T3wbMZ2AisA9YA
hwOrgOX4K/l9QH5gDbAO2AhsBrYCO4BdwG7gLGAPcC5wHnABsBx/JbKTLhS9
nytMYBvtEViW82wT979dTHqfPBcSz9L+sMSh8HuhUL4a5GqQa0GuBbkB5AaQ
m0BuArnFpz14tkB6h0964a/kH3QC26hzUfs/F/9/xv+vfeXxwrPcvjLz79MG
8YayXG5vvj5qYC2wAdgEbPVpH3xe9A/i82L/5J5P6aBAYQIRy7eL8Ulif1IW
EseKNEz//8tPkF9+n/B++d9TpEN5Np/f2yiwXD7f/naf5+V/3w7vT5aLHA7t
HQa/Hw71D+Pr42X+/WigvBgoTwPlxUB5MVCeDspLgPJ0UF4ClJcA5RmhvCQo
zwjlJUF5SVCeGcpLgfLMUF4KlJfi0//59GmQPg3S24EdPv2R6x/e/s+XZ6Me
wvwh9z9+f8hOvYvkt+T+yudP9xmfvDwS2iPSR87nj4P0cZA+DtLrIb0e0ush
fTKkT4b0yZA+FdKnQvpUn/fRQWgvafzx71Npoy+K5OdEeQD0HxVwMPx+APQP
FXAw1A/et9IOv++E9cPJz69KJzxPBqR38exNn1ukLzM5PV+eC+QB2H99yufT
Gxiu/zyrgbXABmATsAXYhunF8SXtmxqAA4jnUOAMYIv4PDa5fJ4DkIlni+jv
S5N+DzgAWEs8G4BNwBZpHpPaFzgUmSA9sBbYBuxEudi+Rrk+PGuB1cDS+zDK
9eP5E7BNjG/Sy+XzHIBMkB7YAGwCtgBnAtvY26L6aeX68BwAHEo8q4G1wE7g
fGAbm1XU32Lk/s6zFjgUOACZID2wFtgAbAK2AGcAZwPnY/msRNHfcLn+PAcQ
z2pgLbAB2ARsAc7H/Ixfj0OBA2C9DgVWA2uBDcAm4EzgHMzPwN4CzoD1KxM4
B7gQ7SvGr2ehwJmw3uUAF6L9xvj1JBQ4ANYbLbAJ2AKcCZwDnA9cCGxgeUXr
23u5fjwHEM8mYAuwDdgJnAGcCZwNnAOcD2yg6cJ4kuvHswXYBpwBnA2cj/nZ
a2F+kt4XcACwmni2ATuBM4AzgbOB87F8cX9BL9ePZwPxbAK2ANuAM4E/AecD
S/qGUc7P8ydgi+j/TZHbj2cLsA3YCZwBnAP8CcuX9BW5vXg2AVuAs4FzgDPF
/mST0/PslPQzaD8HtJ/EhcAG8K+owb+iBTYAm4AtwDZgJ3A+cDqvv3vtP16f
Twf7zs7Px0qIj/Pq73z+YF7ufV7eXlGDPaIFudbHHsX68vaTneYU8Q45PV//
KHolyGXm08eDPJ6Xe+0zd1F/vCEzv56A/aSy0W9F7X1Rrh9vbzl4+0Xp8LGH
OPZzUUhRf3ok96ein5P9NwY8jwr2SajP+VJergM2ApuBrcBOH+btmXyQFwAX
Qnoj2E8mVijst4isgvQBxMtdIM/0YeH3JXtJ8n9JcjPYV5J9UXxekZcHgDwb
ys/2sZ+E91F8/g7tNV6uI7TXeLkRy/dh6bmEvzkgzwG5ZH8Wn0/j5WqUQ37J
/i4+j8bLrcAO4Awf5u2xTyD/hOfXwN6T/L/F5714uRbPe8HvBcD5p0yQZ4L8
k4+9JvzVyb/Hs3Reptie5OU6H3uSlxtBbsbf92Hensv0sff49PnwezY476MF
DgAOJ7RH8fwPL3cAZ/gwnz8b5Nkgz8fzQWB/6nzsWTxPg/YsnqdBexbPz6A9
i+dpeLkZ2ObDvH2YAZztY0/yv2eA8yihwAE+50/Q3sXzJ2jv4nkUXm4FdgLb
IL/Txx7l02dC+hyQ54A838d+5f3bocABPudB0N7G8yFof+N5EV5u9LHH+fQW
kFt8zo/w8gxgJ6TPAM4EzgEuhPIKQW4Q/0r6XiiwHvSxNGArsDT/S/kzgXOA
C4ENcH4hFDgA9gtDwf5Woz3uY+/z+TOBc4ALgQ0Qzx8KHOATz8//vsanvrz9
rQO50ed5+PRmkNvg953AmcA5PvH66O/g5QbG2++hwAFg32uhvlqQG32eh2cr
sMPHP4Lx8ugfwXh59I/w8nxgG0N/BcZnw/P6MJ8+ANsL5KHQPqHgj9D5tB/P
ZmCrj3+G/71s4HxgG8SDa4EDwB8SDvXV+jCf3uHjz8F4b16e5ePP4dPnA8v7
DXL9eQ6A+gSA/0QHciOwGdjq49/B+Gb09/ByC9TPAvHGapCrwb8TCvUPBX+M
G+QZEM/7CdgC8bWh+Hvg3wmH39P6MJ/e6uOPwnhaXu4CzvXxR/H5LYz3BwVA
fQNAroP6GYHNwB4ffxUf72mC8uX9Rpgv5fqB/ykL5JkQz2iA8mR/AvQHh8yi
/S63Jy93+sT/8XLZ3pbXRz69AePt4Pc0wDpgI7AZ2ArsAC7w8Z/x9YF4IaUd
/EdqWO/TfPb/pxWVHyKw0gns4PUhJcZ/FYI/Jxj0p1D4/RiQ633i17C+vL/P
RmWK/kO0S/0cJDTIOVHuAv+WdJ5Var948W+xvxD9d1x5fnYyF/XP03L9+fpG
8+m9cr599cBWn/afI7S33H4YX4H+TL6+EB+hkvSjp9ifxPKc8L7y+XgtpaS/
FsebCfNB8f1s4ngRORP8XVk+/ibeH2X28QcJ9Sm+Lwn9QbxcHkciZ/jc58PL
3T7+GuF5i++zQX+E8FcnPx/PeZDeBfZ+FrDVx17m7d8M4Gyf+yrQPsX7EXi5
x8feRvuXt59cwAU++7O8fZUA41cH8wPGe4VB/w2DeJkY4CRInwRyNervPvo8
r++GAgf4nH9FfR7P4/L6pgrZR79F/RjPE6L+iPodrz9pfc7LoX6H5/d4fUjj
o1/h+S7Ur/C8GOpHeN4I9SHUV/A8DeoreH4H9Qs874H6BZ4v4ddzlY++wK/v
MT7xZPx+D7DKwe+vFK8v8nrH998Y2J+Jhvi2JBhPBhhPen69864nGP/N/14q
lGeB8ixQHqzv3vUU11eMb+X5I6x/Wli/E6B+OuAE/nm87cvvh4XDflwE6Dfh
8L4iIH8kpI8EeQDEhygUPPtDe/pDfYKAVcDP4PefQXsFQPyIQsGzPzyfP7Rf
EMhVwM/geZ9B/4uD8lygbznheZA18HsaKC+W36/0/h7fnlron3B+wM8J9Y+F
+ifA+hAH4+MD/L6eSgntJI8nbn3zjrf5RfPLXbm/8/05Ep4vEn4vAuYH1I/D
oLwkYCleXZpPksX9LUnfCiKeU0X7KF3qn8DBYn6L/Dw8R4r2TYpcP56lciT9
KQ44iHj2F+P9dPLv8Rwm6jfxUvuzd0VytVxfnoOI5wTgGHovTPBMak+ew+B9
xwAn8azM5+0fr34urGcR4vvMAM7m2ZteWJ/EuH5lJrDkjzBAeVJcd5pob4v3
AilzgLPF9S5NLl/QV+zy+iXq63L/4zmfP/+hAvtJEQLj5SP0Zz0v9+bn5zep
HST9zx84ifH2RDDYFymiXLIn/InnICbYG5J9ECau95K+H0c8p4njIVl6/8DB
xLO/OD4McnkCJ4hcfM+z8DcBOAhYij/RyOUJ9kNE8XrE2X96nr32KT8/unzi
/fn4jQKYD9PpldAgotwB7Ab28OzNL+irYpy0UoqnF+1oZRbP3vRCfxXvvVLm
AUv3SSfJ5QnvV9Qflbk8e59PGC8WuX48S+fN0+XyxflRfl6hHMlufMb3x2K5
vP5w/bF4vDAoX+RMvn8qpfOk4rzpHa98f02F+TeN769KDy/3jn9e7uLnZ+/8
wfVfZS7IpfvBxPlZ+Qn6dzrMz3a+vyszgfP49EoXP58rs2B+T4f52g79P52f
r5UZvFyZDeyA9E4YLw6Yz92Q3gOcD+MvFfK7gOF+PCXcj+cdf3x6Ha9PK/H+
tkhYb/TAMTAek4D1jBuf3vWT5yB+/HrnH369SgC5XbS3YiU5cBBwMPEcBvOB
P+PGv/f3ufXQ+/u8PJ1x84NX3+DZn18/vc/Hy9MYv57GAEcyfn4JhvU3DOaf
MMatv9725eYf7/Pw8gR+fvLqZ9z8JLdfmtw+vDyFcet5sf4nPz8vD+Lnu+L1
VX4+nmX9Upwf8bwo+P8UYH96+zMffwj+Qz8pfk5azxPB3oTxoHhNkwV9WyzP
wLMKz4fB/r2fnvyLxIFyfXn9JRHqG0E5RelXMYl7Cyy3L2//PYP28Qd9/xnU
D+wnFfrnC/j1xPs+eHsoBvxtScCxfPt4+x+/3tuBMb5Tfr/4/sT/bwJ/oQXY
BPXRgv9NC+UZfMrn2QIM5/mUEJ+pks73F/tb+PYL4fuXCu/PhOf36re8/qoF
/gj250ef8718f7P7nNfgxoesn0r1TwA2QHo9yJOATZA+GeQpwBZIn+pznpxn
B5zfsjN+fU8GeysVWAv9Jw7W2wTgZLDP3Lw/2rueo5z3V7jh/bhhv8QN9nsc
rLcmsMeTYTylALvhfKnbpzx+fCTD+p0CbIH0qSBPA3bAeRI749fXON7eK+5P
cn14TvWxD/n52Qb2IuyXKSD+3s/F5/cyP16iwJ+jBo6C+VVLw4rkB+Txif5Z
tE/59x/Pz//e/snH18fz64Gfg49398P7Xu0M7UO8L54fDynAZuA0YCvoq24Y
L3EwHuJ84vl4fdQA/i075Af92SvH8cfbo254P24ffxw/HybA+NDBfjPe12pn
vD6ZAP1VB5wEbAROATYDp/FcvF7jeiSvI9zze/sH9md+vomG9TsF2tsM78sJ
crRHMB5VA+0fC+c1jKAPJIE8BuoXDeMV/LE6Bz9evNxAGJ9i+ZE8K7D/6WC9
MQKbgW2wfqWD3AqM+paL3x/1th/ak/x6FeYTL8uPPyu0fxq8rzSwD9Pg+WN9
7pvmx0savM80n/uqebmL38/0Pg/v38H4SSOsJ2ZgK3AGn99rr/Ny8Cd53ze/
n6oHe1MHbIH0aSC3gj8K43W0cP+CDljetxf/qiGeQQ3+ehvIbSA3wXkZM3Aq
+N8lu9Uss1CeFN9gArkJ5FJ95fsEIP7BCXInyC1wP4AVOB3892qQa4DjIH02
xEdk8azMBH9TDsRHeHhWZoP/qBDSF0D6fEivhfPyOmA949fbAJCrgIMhfSjE
W6iAg2A9LwR5AcxH+ag/w/l6HbCe8fONFvYzdcB6xunHfui/B1bZUV+C/Uod
sJ7x9lsarCdaON+uA9YzXh9wgb4ArJLOj7+Uy4f4c2A94/1fASBXAQdD+lCI
F1EBB4F/rRDkBTA/50N6A8SHGCCexARyE8gtILeAPAfiVTw8y/tLsXL9+fQF
kD4f0qshHlgN8S1OkDtBboH4aytwOuP9f2qQa4DjIL0B4l0MED9jArkJ45nx
PDrIs+H5snj2zn98fWwQb2uD+BkTxMuYgVPBn5gB+mnG/299lMcHz3r+fi6v
vi3qWyInAacAp8F9Xk7wzwCrIN5EEQnp1cDRkD4a7Ds1nHeG+wyL/UeC3A/i
P/3sVKnoP/yl+RP8CYlg7yb6xMfw8agffcrj4yUSwV5IBP+ipAdJ+1WRMB8D
e+W9BSa5Pf9fVkn3SQTL8ys3//q5+PhRP7zfSu4f8u8Lf4vvC+Q5A+JPM3j9
Sd7/kzgD9oNTRf+/pE/pYf8sGViKO5XP9wKnwX5xGsQ/REL8gw3iUW2gj0n2
h1SeHfbrHFBeDuhnuTwrc33284R0Uvp0cbzK+9PABijfAPGuetj/SwI2QX4T
5E+G9CnAFshvgfypkD6NZ3m/UZLbwf9oANYBJ4A/JgH8laGQXgUcBPmDIL8a
9FvZjpH7D58/BtgJ+Z18fqUD9ludkP8TxBfn8azM89mv5etvgPwGOI9cvI4K
f5OATZDfBPmTIX0KsAXyWyB/KqRPA3aC/u8C/d/Fp/cyZw949Rteng/l6xm/
Px0MHAT70fmwn61nvH9ADfp6OHAYpA8DfT4A0isUPPuDP8LfZ/+bT58N7AH/
ngf20z0YL8dgfQb7Ihw4DNKH8faHEvz33vrx7AH/vgf8jx5Yn/MhfwHkL4D8
BXg/J+PXZzXYN+HAYZA+DOwf2H/wPh/PHtBfPOBP9YD/MB/yF0D+Ap/zL3Ae
hUE8Pdhf4cBhkD6Mt8+8/ZNPr1Dw7A/6g7+Pv5dPnw3sgf0VD/gvPRg/wXj/
UzBwEPjD8jGeAuyZZOBUYCfYRy6wj1w+8VNcfmU2yPN94jHAHgTWASegvxz2
g0IhvQo4CPIHQf5IiGd0QDzHJ7A/83j2rkd8/Fge5E/HeBFgPdhPycCpEF+Z
CfElOWC/5fLs1bcEfTNJZj4+xAbnIazAaRBfkgbxI2r4fTWct4iE+JMY4HSI
N3FCeU6+PKUD4uucGF8H8SwGOM9hAHtUD/EtScAmyG+C/MmQPgXi+VwQr1J8
zopvP4fMoj0htx+fPgbYBvltkD8d0tuB033iK/nzI+ng70qH9TMd9gOs/H6A
Nz/HKhstKeKLTHo+fn2LAntUDRwF61s6rBfpsL+D94vB/Wx+eD4DWGUHe9MO
+z/psB6ngz0P5zuVcP+bn41URe/vuvz7WD9+vxnvMwO5t38J9om0nyTdHyXZ
qzHA6bw96yd9/wPvX5bszhSIx0+B+7x0YL8mARuB03zuuxLqnyLXl2Pv/MWn
V8P9UpIebZRZnE9EdoC/3w7+fikeuvg+Jl4uzcsS68H/rwd7SAMcB+njoDwt
xKPi9+kTYD8+iGclfo9dC/cpJcB5hAS8fwnOawbD/kOwz33BfHo9pNdDeoWC
3y/wh/0Bf8L7knh5OOw3RII8EvKrQa6D/HqQ6yG/AeTpYJ+kQ3ob2BvwvW8l
ft87DO5D8of92dewv6qB+dfiYx/x87HHJ56XL196LskfB99XVuL3k8Pgvh9/
2L+1gD4L3w9WwveClT7fA1bw+wv+oM8GAIfC/m8Y6M/hwGGgj6eDPB3Ks4Ec
95/xe7tOfr9X6YDzNXA/jhLvB86B/B7Ij/fj2OF+mgTgIGA96PdJoP8bgVOA
zcBpoP/j91gdsH9hB/3XBgz6t9IJ+nQ2v7+izIL0WT76Ob8fkwT6tRE4BdgM
nIYM549doH+7eH1bid9/hPNSSvy+oxX0Yzsw6JfF/maBi+OX5f4s6gdyfxfk
xfqgqA+IDPFCXv2Hi1dT2nj/uwru21A5QP/D87fRvNzPBfpYNOhXatAfY0Df
i4Z4WyvoR6GgP4b4nBfD+yp4eSjsd4TAfkeIz/dB+Ptr8fsfsD9RrJ+Lvwfn
eWT/tPydCDhf4wT9yeXDnD6kdMF5MTh/452POH1LmQPl4fkcn/tORZb0lzTQ
f6zAaaAvfeLzKz9Bejy/g/eDOnl9yNs+vP8Yz/vg9xksoN+kgX5jBU4DfQfi
sbzty+sDGSDH8z14/2Q+rOf5oA/geR28jzEc1o9IWE/VwDpIr/eJh4L1GNab
YDhfo8LzNnD/RAzM9xofhv1w8IdoCdcXfj5Nh/MzVuB0/n4JZTRvnykTYP5J
gPOkkTA/wvl8lZ4ii8q7Lb8P4fekeKww2C908faZfJ682D7j5F4Wxq+UPxnO
pxrAXjMCJ8N5Ven+Zfk+aHH/Uj7PCumz4Xmk84LF5wn59NL8Iv2+h2dlNn/+
0Nu/+PShYJ8Gw/6q1A/M8vsS0hffl8zvh6bB/Cr518TyvfoXX74a4uPUcL90
JOzPhoI9Gg4cCed5w2H/MQzmZzXUXw37uTHwvE5I7+TTe/VTPr0N7PN0qF8o
zLcq4CDY/8yH58mD9Uia74u/L8TPxxrgONjfDID5W6Hg2R/SO6C+dlhf1DCf
hwNL85603ybtx0nrURbsd+ZA/Tyw35kN65MN4vVsYM+Hgj8gWG4n6fn4+tph
vcoEuRv2HzMgvRrWwxjYL3TycqUD5Dkg94A81Cdembe3s0AfzAR9Lgv8gZk+
37Pj7c0PEC+eD/Zpvs/5M369ssP+RibI3bB/kgHptXCfUwKcdw2F+OFgyK+G
3wsHDvOx13h5DuxveMB+yobzr2rYX9EAx8H+SgCUr1Dw7A/p1WA/RsLvO2D9
t8N67+Tzy/fJxcv14e1Bf8gfCuWrgIMI4+/48vRQngXkaSD/BPbsJ5Dnw35I
HtiXn/j9HXk9kjiG8fJQXr8pXn+k9gZ9KBTiAYPh98KhfmGwv5QB7ZfBl+cd
D3x5OthvSQB71wrxhWmw/5ID+0U5sD8kxYtK9215eJbvP5H2TUxwn1cKnG+W
7luWyjMCJ8P551D4fen+V7vcvsJ64YD2dcjty+8nuXh7vVg/lNub/70M/veU
GfB7hZC+ENIXQnot1E8L+09xUJ8UXj9TOkGfiYH4rwSMX4H1IgXWhzTgFIi3
TODtFT8X77/wlo/+PX6+jYH7P7KA4Xvbfk6f739y7JXjeSS8r0b4K/ljtfx+
lBLuA1Pafc7v8ucHYb9JafeJF+XiTRUhEE8fDBwC/ppgOH+YCfEpNkgP3xfy
g/01JXwfVeWE9Rq/32MEe0jaFyr+PqnYX2UW+6vIoWCPhPL2i/cfn1+6X0fi
QshfCPlVYL8EwX6aDewJK7C0XybZJ+EgDwN5KMQrBsvtJPzNAP3XDedb3KC/
S9+P1cnPj/oyr49Ggj77CfTNPNA3pf0uSX/VQPo41EdBPw4HDuPPtyjTYD/W
AvsreJ5NBf66YNAHQ/E+TzhvEQ4cBvE7NpDbQB+0gz74CfS3PNAv80Dfc0O8
vhvj9SH+3gqcBvEUTlh/nRAPkQ3rt4dn73zKr+c5/H3cSil98X3d/PqSDeul
G9ZvN9wnIvtHcH4QWZ4f5PfL2wchuL+O8eNgL+gh/QeY7+JgfoyD+PYMmD8k
/UiaP9zgH3ET+l+Ev5L97uHZ275CexbfN8zND8oMuZ2Ev2qwP2LgfI8G+mMc
2ifQn6TzLVJ/CQd5GPQ3NfQ3NfS3UNC3Qn3uZ+XlQaC/yf4tmfF+Vl7uD/qP
7G8TWfa3ye2F96Py9qsB5iO4L9X7/2F/GOJPFQpML/yV76vj41O9/+D+F4j3
VCh4DsD9Xj4e1PuP32/Jhv4YzK8fXnuF1/f8wT8ixTtJ8czZvFzun9L6kAzx
oMkwHo28fqB0+Zz/5TkM+m8kcKrP+V64j4rhfVEQfwj2XSRvn8n7vVL/TIb+
qoX4ngQ4v6IB/SYG/MkOkNtBLtlx0vyQBPOFGeQpILdAPE4a6PM5sB/jgfcZ
AP6tIIb6Pu8fSgMuBP9SAdgLWvgeVwJDewHuA4Lxl8DwfmFuPDV1wH0ekp9P
Go8JMN60ML4SID45Ae8jgu8ZJYA+EQDyIJAXwnn3Auj/RvDXJIH/xAzyFJBb
QZ4G8kKIbyiA8eMCfcUJ40UN94/HQPyAEfwRSeBfMIM8BeQOWG/sEM8J3/vy
2m+8vpgI630i+B8jffaL+O8xREC8ewTEG8L9E157iY8fzILzD5lgD/syv35k
QXx6Jug7yDaI90mH+dYO8ZDAXntUI9RflMfB/nu8z33c/Pcs4D6spnB/h7c+
/PlFiNf02pM43oS/0nyaD/NlAc/e/svPn2Fwn2Yh2IfJYO+ZgCOleVj8qwZO
IoxHxPuIefsoFDgJ2ACsgv3iYNhfDva5/wPvE+f1Q3/YPw7zub+DT49sh/PO
dj6eSJntEx+E8UN43zBvH0jnGzGeuvh7OLy+9AH8H3E8+8F9+gpgFXx/pdj/
xiTm718ttlskFucvkXN4fUq+D7X4vJ34HPJ6yZ/3yoB4xXx4n5HwPiJ94gF4
uZ7wvgG4TxfOE0TCeYJI0P9jCO+r5O3DYNCP/MHfKu33SffnRoN/Kxme1wj+
qiSY3z/CfPYR9PmPEB/gD+fHpHPTxd9T5O27JNhPD4X91zDYf5XGrTT+Y2B/
MxP0Len+XI0s59djN6zHObCf4vHZT+H3a/xBHgr7H8E++xv8fkAerMda2C/W
AMeAPZEN8mywb7PAn1UI+7GF0F4F0F4W2O+ygD6ShvGNoG9YgdPgeU1w/sYE
9nIKjI9M3v8v+wek/QDJXpLkWrB3tWC/Sv3GJT8ffP8HOB36mxXaJx3Gfz6c
l5fOq0rrrcKP928Ugj1pgPgmKZ7mkcz8+mQAOX7/NRjie4PB3xcG+nc++KMK
eC7WB0R2+MQP82wA/UkP/uww0Ide+8Qj8voNxOvJ8XXS+0oDe1uaR4vj53iG
+Dlvet6+wni2fGxfiAdLh3gsDejbMbCf5w/2UA7oYzm8f0zpgf4i9S+zzFz7
eOcbfv7IAX94DrSHB9rPAucnrMDpcN7CAv5pK3A62LtqiI9TQ3vGwPgKhXjr
UNgPDuP9Dd71h29/A/jPkvj1VRkF+kuED/P2TrTP9/v4/aoouO8jA/RxYBXG
y4L+7gf7Q0rcLzJA/9EBJ4A+rwZ/bDhwGPQ3Nbx/DXAc9IdPoC/lwvmAHNCv
MmC8uHhWOn32w/n5ygPzVTbvz/SD+6f84L5qr33F26taWM+0/Hru1a/4+diI
+yugHwUx9GeCfQH+knS5naXf5+e/OBjfNqhfOqy/mfz+tawvSesh9H8V7j9K
+25yPDOsj3F8fb3vF79nwa03KgfcB5YA+zcWiP9Kg/awgD8tDeaXKJ/7a/n4
qUheX/aD7zH6of1sgP1Uvc944vXdSJ/xwz9PDDyPHvwb0nmtpzLz9moGv18g
n78oHh98fIMH/Ps66N8J0F+CwL8exPdn73zG+8czeP+0Xwb0nwSov87n+wuc
PuOdP3h95RP493Ll/lX/p3/LDk99L78fq8r/V88K/H6mkV3pkdTlrfd9XTf3
qfSNJxnsExdNWdjp7abZTlbq6gXnnf5miL/1akQD1k0I/qX4+5kTDgzbEnXQ
xu7lqE+xzvj9TBUbcLvHJ9tiK4ue8OTs5VKJcjzqn8vf/i/wYZrP9zQXfFhd
t9Q8qZ+Hs5YHAk++uGhmi/aeajQ3PEq2l9r9GT9i7bk3MB9o6NfPa65udb04
/jcqO3tV3z1S+d7xd6xCxpnJxfctxTwNf+ppaGL7Rg0f/XZguuzfPvLr+cj3
P2lk7r5nRHN3l5uyvfGPKXzsu3nF3+e0s/gSk03SPriOlUgxWW8oT8F8YaQr
45pl2zMl/cNM+y5PbjLJrmcNWe7j0l9Idq2O5UXat1w0FH/PM0MzRHN+Zxjo
axpKNzX6buKtRPn5HjZ6Xja7d6L8fp41UB4bOT+RtbjSrGPJEtmQ38EGWN1x
nc9FsXYjWqXvmK+D+UTFOp1pPqfeoFjWZfKrD+3aBcvyD/vb/TZ/PYN4USNF
vm/TtdfNWPn5dnjumLVvYlnSrAtTU7KLvyf6Z88m4TfWFn9PNOXCr+s/vC/+
nqhyaZ1hhzfmy0zdu1+a1l4P/n0NHRl9IadrfoT8/NHZvdcuSI2Q+9OKf+aW
ONU5gq243u7f9h+Lvy96uG1EvV3H9OD/D6eD1VosqjwvhPUMaVTt3vZYn/t0
x8zsoet09yV7VaP2Pne14u8dDOvcdcCaoOL7c4912ZepYNJ7lvwBVqITl1IL
koJYneo/7elmK/7+aH2jK+FQuUyZj4j1k+aTBXeqff5wW/H9u8Hi70v8k/j7
Ekvtxds3GrIYtxb0+uohSe/Lsuzt21oqic1UpdWX1W90LI6HnTNho9+RKU9E
LiDzT5Nf9vniGa0U21Mq/39i/5H8DV3F/iJxuNhf+P1mI7XWTT5f/3Lx/b13
45bd3BD+ilLE/iKl3yb2F4nNYn+ReKPYX/B7qKWr2n6JsEWLrKIB/ZcP+3xc
FKWK40nSt1uK40NipzieJfvfTxzPvD8gnMZvzc1vmRNHTcXxy8tdVHWA5sai
lx9IGs+SXJpvJH3lhDjf4PdTx/T5Mvi3KnrqJs4/vFxHmSsWhHcYIt8/Sc1P
Xf28TReJ86jrwgWdGrbRkzRf8fk1rMOuwNMhi5JIml/xe6uXhnWbsO4v73sS
51te7qaNgz/e/+1lcbxoyYh7XbYUmihEnO+l9fmgON/z/g4XHZv5alGDzVJ5
udRsn67yyi9TSCGuP7y9oqOK5TsPaXs3Ve4/E7pXsHfuZpH779qRlSrRFxZ6
IK5H+L3W7vc7FZbvmE43xfWPjwdQUX+He1PwCweupwTrqfz8kQVlTvy7ofj7
7E0Pbjg7f76Drt5Xvm72hc7ne+x3Kj55++8x2W9L9NOKhlOk+x294635FsPV
H6q56G7G+DX/eN6Tf8qVpWOOOZk7bcTLqu3N7PmjZaf97Fam/m7LqxUvE2lg
0mFDj4BUltJ9ZYUZw5Po27599tVqbWHLWyy+cPWbFIquTbWqdbGwoZnf1ZzX
+R3tnP0ktNFzE2sefygp6rWTpp2p8nLVQxOLO7v4XU6GlQ3e5F48JSuJjd9Y
ZtCA7hrmnPn+x42Dk1ibvTNvh5e4T1OfZKb3NCWxO3XPdzlaZz+b90Nsn9dL
dYxNywjWnc2kErm64IKDOjZw1uf/u9PnHSWX7/DQ8mMcu/j3bL+9Y3Rs0Ya4
rj+wKHZ/qr7PtdLPWd2nGesGzlazqtUjRg29wWh63tntYwZEsYJOO+s6b2XT
gpmJdQ61VrP5XyTMa/xVEmk3nl46r1IYm1DqyvE+hjRa0iapcoLnFfNsUFVo
E+Od32ceajbzwmv29mHFvHkslq74XarkN/sBo4KD/3meJFHs+VULQ3s/ZAcr
VJr1V9MUetjcFHmuE2ND78zd1c9op8AXfQdO6DaTjR44fYYjLY2sPzZ41L3e
fbpivzC7tN67Hk3fPfenqFA6tzFo4f1tOhYybYmywm+RVOtl1eNt2RPq/leP
keUyIqnfk0Gbdu6zU8f86B4rN0VS0s4zbVtcy6Dyo+cFnWsRSdv8K5o3xWdT
rdljby18Ekk74uaN3HYon+pvZ2UD9RE0wb7C/HtBDNX6df/Pri6J9HPfZ/P/
umWnBvtL/RewNZFSmjQPWW/LpBoz/WvkeftB32N3yt0/omH1/7ZMbZhloF+r
B+58cPwT7RneoFrwfgOFTJ3w/FBGGp0wr3t/7e9UmuT/R9L+P83MdDOydssp
VirbZfsV3aFsKm/+cPlYSytlTK32qtaWTBp+yrzsUW87xY8I/vObGCcNrnKy
V/c/nFSzx8a2W2w6tq/R0eZvolzUvMPHrw7dzKfbj9flD8x0Ufam+Sc7PvX2
5w1LW64YcY5JXFVgqv1xXP8mlUPowFeVp5aMucFqiqwUmCa3v1H3305qGms5
9emP2ZdpbK1ta5YPt1HC1d8mrzr9L1u/5/zbg7HF+4Mb47K/7tEK7zfV0cF7
oa8aXfuXNohyyf753dPg+fSlNjp8e/e55ee965S/Lik9Ar8HGUnX6/13ccSu
RbI980pgisxZea7VaRucB7bTF98dH/5+4Hna9PuwTudUz+mX54YpNUsfYQ2U
tfO2JjynE1+eu/lh4TG2QZRXF+RUX5SvEORk2qXZMine9zzx1sFpq9emqyhh
UfY/cZWL7zut8t+y5/dc8XRowKYlVea42OwmYYPK1tKS/nRm5eNzXax2nWMX
zozTU7sJzx63uuxiAzaUWJnx2kA/HFUcOnjPxexDMlV/1U2mMmt71st95mLn
awdWn6sz0bKFugXtVC5WMdma8XFkKk05X7bXpWsuNqRWiQ7/bLbQ8OiZf5Q7
7mIl6vT77nhEOili6t6N/dnFCq4PLhng94JMzSYdbVbDyZ6Pm/QstlUGlUxK
LlEjzcHuR1X41OqRiR3Q9u2Ulmdj80ev/f7CUwvbce3QOcsxM+tnDPp0dVMU
mdxDCj6+TWE51uptmtltdODx5fs/MDPbMDdlQGq4g7qsvDD/8u0UFlnYYvup
GdFs29H+pT6MS2b99E+/ap4ZTK3LXFTtn53M9NcTL5R8nktvRrxtO7VjMuvz
TZvLQ+y51Pbk0qWf3zSytYPuHLzwPp2Vmhr9y2CVnmVfTltxtOY5Us299LOn
n4EN3XjjwSajnl68uTj38A09Kzd8zuJ3bQwUVanZ034P9GxUQnrlpquSqcWP
8e4NnxnY2ROJloAtJjLXNj1UlTIwxfUW7/+xplK3AQGqZle8+fdE/Le+koX6
xMbtOnJOz6zdC8/H1cukmi1m/J79TM9+e/XX9rUPbWz8cqPfS89HZl7s0c4u
/5H9+TQkoOmZj2zJlCBX85aBLODbXZ37lP3Ihh55oW5W6TZ7UnNJ9P78eBZe
62aT/fOiaLcttt6uAR/ZyuFjeve3O8g8/c6aqSU/ssbLJ3xboaSTDg6YWHNU
lJa9UU9NimB5tOubrxbPL4xnZ7tm9qN9IbS7cYSpXk4Me9XxlGnFfT3tbtDj
Wf/YGDY7siB98yoDxW2eXjuqTSwbP5uFNKrt7T+/zmpzUhvLlrfa4XfaYaJf
zlbpNiMllp06f0/RfEYqhbePK+g0MZb9/WxGP//xFrp5cmH+0dwY9uzmZltN
i4uiy9Gnb5rGsL5fJ626ODybWmSfa7Jmtrd8RcOV+7vnk+rYrwHqarFsWpMd
RwpbhLCLrV/9PfNJOAvNP7D6cetkavS/pQ+DB79lZ7rNclOlVDoVnxRXmBHG
2lffPCfuFz1bPantmms/v2D9bUkNy9V5zXqOvRvQOzGYWRqPmlDV/JrplTvM
rkQVWxQfdXON4Tw1UZe78sP8F2zTngkTx6XF09FGPU/0GfWcZX4e1SR4nZZ2
1QsqnJ/3nIX2Prl6wi43zX5a7dGVRiFs+/IEW9VHbjoxuu2k+cqn7GhIv8HN
j3qobzvPrHh/FftmT+eTzQP1bPeO/Y+r7fqXjTb82F2b/4a1mxv4MKTTLRZx
tNGEB/luyu9ZYd3zqCss+ed/qzUocNP0/GYTG8y5wk5HjLnR4qGHBqcd79uz
XwCroA8/OvSZh1alrO2oXXCO3R7X507yXT3bO3XEFYPzAo0Vy28qlE9Seb2E
8ihc/L084feooljefKE8kn6vp/B7dCyj5LgNf+jZporndhze+JSk9osX2o+k
9u0itC/NEduzgdCepBTb87DQniS11yGhveiV2J4ThPYkqf36CO1HHeYoR1ZX
6Jn77qzfy74No1PDw3++X+YlG1y58oCDvd7Tva/LV5pc8jbVuz2149IZ7+i1
2D8aCv2DQj+bmDX/sYlqbeu9NebMWzor9peTQn+hA3Mvdtvz0kEfV/xteXDo
Hb3k+z+N4/s7HRf7d5TQv+l/Lbbk6Dql04Be/pmz6kXTc7F/Rwr9m2YdXpvR
59d8qncn1q0dHE2jxfE+ShjvlCqO9y3CeKfZ4vi+I4xvWimO/xvC+Kf34njf
JYx3WiGO91RhvNPCiYv3GK9l0eQppdtfZvEUJo73HcJ4p8uHev3wMjmdName
u+j0Fj19v+NDzwHP49jwk44bKfX1NESc/4KF+Y+k+a65MN9R7jVhfushzG9U
M3FY3y1V0ym+7rmppeJ0BPMt6fj5mH4X5/+dwvxPP4nzv0WY/6mPOmPn8ZZq
+iGh9NRxzhRS8vM/XR7ZbcKjTanMWmVCnfx3FtoZevvq+60eWh734OrNhRb6
o+aSZ1sPZVGrbXfDzz5Opx8T9jzuaNCzoFan6676zUXNZg3b9+2jaMrZsnPt
Lu+6CuspwfpJsF4SrI8E6yE1GX992cORNlIta3bqx0EuGheQF9u+azr1L9m4
683Vt8n5Jua7fhsdVCmsRZdGA27Rb6L8O0HOwkV9g9dv7NRK0DfYuuqnLh4d
7KKHq5IHjvDcoYMXZxyYNt5F7UYN9kyeF0intr5+/OOB4u8J7zW1n2Z+oZXj
X3aJzO/v2Cg36VGj4KMBNHjIV0vbBBZ/n0x9d2LElot2MuqvPdJf3Ivp2bPR
NGnhxHjKbDKwRtU2SqYS+a3AFHohv09Frz7XbN2aX9se+1f0Z9ro1aqCX4b+
dI3WTThe0vq5XY6v0f01YOzOd972a/lyzpaml+jDf9mDByywQzydnZStF+ia
VTtMDf692H9sOxe1rmLXn7j3kCpntnXOqumiUezWzre5T2jH2ZnU8PskuE/B
wH5ZOeFy3NDi7zV+GXZiQZXekly219g2/8/7/e9ZvHx/w+CwCtNm9S/+/mNi
5dcXZs4tpORY9b20Zk6mm1vfVS7ACPctmFjQ6yUlYjPl80OS/cmOpCrL/7tG
Je9fRabW928Xdgb86wG0eVfjwW2+kcpT+DUT7E/27YMuzQsCC2H/y8y6CfYv
67fu91bUpvh+v3PVJz0/NvuizLXeO1NqLXkE/t8A5l7evkqTvHQmjafWwnhi
WS+6qy50t8D9CBb2e+hDVj8rlf0+ZOGJH1Qv5PNM7RfM7nBpdhikD2V77wdr
kv+T9lm0NC3HnVV5YCrbwY9npm619NWS6Tn051dBl+/5m1nLamUWdG0fIT+n
8FfNrp/uNX7Vdyb5fSYFOGMLDMlswIfTNasvk+JgzGz+iBOB+UsdMpfvM7ZP
TpsMutR2z95xVZJZmXvpa2ofs8J9DhrWXvCfMGPvXTWndsL7CGX/CfOb2T9N
M/spxK8E0Mjzpuj2HgNbmbyzdIfRbjjPlUmlUnbPNWbqWUzHmILv58r3MErt
w378Z8KszFeSv1f2F7Efe1/u363iPtjvDmA3O9pfH5/+UbaP3IL/SG7v1n6f
G9de1bEa4vytFeZv2T8q+rtY7xA2Kft+JMQfhJLh9ZVR0edj2e/iejNFWG9E
eTat7NGhoIH/B7Zvedy4t5/ywB+dT5U7/W/kwjkf2MYBf3a9HmSH/VUdc39e
5M9j+ZXHb+gcnyjLp54YP7JZvgrSq2ig4O+Tnzfh98w6R6OjWYedC7ZUTNPJ
58+y/31cdWebOPBna2n86F9+PfilmrUT1/OBwnrOqs+bHbriQyj4o0PZzWrG
RoPbhbFjupWV+nX41+e+w61TIvYEGV6zroVL2xZ2i5bv/7lcfssXT/7nez/h
7MUjv63y+LXsT0+YdKJr82VvmaSf6AX9hPl32tyo4dE3sj97aVNDCfvItxDv
GUoVv+03eI7rrlz+lYIVnmN7JLbQb9+3DR616DHr+0fyzpDE4vsGVxxrNuvh
teL7BttUX1jhbbqO+i6brL3/5KbcX0d17tose+Z5mbd1LGLa+euLpDkhxfcP
nhPrK/mr/71Zq3fHYW/oYoVfIi/tVRHUh74S6yOlXynWR2LpffD+7lB2XXgf
dGFj80d9Wv4L8T7y+5D9cwsH/j7615kh5O/0TB4y4q1sf5fo8+7Rnn1q6nju
p16N6kj7+U46t/jHEyUCw6itbtvyiXn5cvpKA5ZHPn35kVSdz455kyPNM/L+
hLxeLsxNz07+TU03a6bt//QJ7wvMoB43Lgw33JL81x5aL+xnUN2RS8aubIj3
BTrY18J+Dm0Qx48UX3JatOd2CfYcfSH2fz7+xEjVvzo77MS14vsA68+LV1UL
jqXrQ2tpJ8/E+yWyaa0wnmm/OJ75+JV8qiWMZ0rYUJDUep4N/OdaNlyYv6iH
OJ9I8WWPt53pXO7CR6rXebe1k7M43q6EPkn7zVYdvVxrMnQt7YTzCFn0VNjv
oroPqp1qt674vkCXaM8zwZ6naf1KVAhR6uH+AiN9+1/jOvPdevn5184J8UtL
11NAxdyBr7tb5PTSfM370zOppDBfU4eKlaPHGz5B+Z/oxNpepe6tN1KE6K/4
U/BX0NBZ5pKfj4xkqc/aligzKoW+4v0XNN9/aPtzozLk/ep8cb3l/ekWtlxY
b6mj4XDftREh7OyBEUtuZlnk8dR02eYyM6ypcn88LOyH0kJx/ZPiE+OUi2P9
r+bI3NZo31jO4QH/fQ612qMeMuyPVPqt5d39Y/yvsR7mX17O7Vh8/8Lvwn4u
bbS0c09u7IH4lmwaEGP542KDdHoo+oMOCv4g6vnlUuPa7sls7eqDf71Z46Db
t9sO6bH+Lj3fPGZtwCqH/L5/FfafKUBbMiSwaZZ8XnRdkKl1nsEN50kzqcmF
S3N2JdhR/xPlsv5H4L+Sx3vyukG9+gc7aeGZqZuTpmbI8Tagb4npC6luRPim
58ecBPqivJ/x79GIH/ceKP7+60XaduIjK/7e607F+6pz70tspRu7Zh4ybyi+
v5G2hPb/x2uPgP5JUaJ+/pmhSD9nE43vGgQ9dNLioX02fmkOlu2NWoK9wb4T
9XvevpDrj+sHXdX9VuV+cjQl51+ZN2SbFL9joGpLmyqG/XOXhrZKOjv6Dxd9
trzsy5+rHaEdazq0rLvSRd8E/jBh/5TjdCb4mXtGiHR/hORvjSf2baOpDT+b
wyS+MaiIaWJMzyHV+9tp0Yjp2ruV/anGrj2qKYPsdKrOkl29V5ymp9fXGd50
jabXbfafYRcC6LJYv0ShftRRXK9GCOuV3J5fC/uxFFZhy2njoBfy878UWbJ3
unac1P8/k5Mq//WyXJt9z+TxEzSnV+etVSV7Qe4f7OuVZ26WWyx/r57d2Te7
w+pShRTtHLQ84pDUnvL+EYP+xKT+y+v7mdRU6L8sUOzvfLxaNvUXxpO8njcX
xjuD8cek8czr3/J4ZhfLNtpXqq5Ljq940npT+MqreP+jvP/HYH4S5fJ8x6T2
DhzQqfbITpK+Kc/nIufRojFlJu5rq2df/PmiXTND8X2PbyLDd5ZZnwX6ppUO
1Ts9ZXzZGHZZXJ94/c9G84X1VWR5PWXT3+Z9d3q/A+5flNdz+fkmhMbb9z1/
zwLF9VlKf3Z47sk07/rL53dRm7+W6I+YVWxrTOvA+/H4veMC2lmu5bKS9IzV
CQjcPSKv+P7G5sfeL3j04hVNfZ5sOLDwEXX+Xj+ctkfI+kp61qHzQ/voQX8x
0CxBP5T3Ew8L8RDURdQvJf3AKeq3/P64rN9SSuNOmhNefZpf31Xs+LtGJ13f
xdKD7weELM9VyfK0AZurBq16AfqArP9L+hW7JdgbFHrydPyRIA3sj8v2BtUU
7RV+vQygEYJ9RC0TA9eXPSrbX/T0q02Bh7vI9paYXs2uCfYegb1VXD/B/pP0
PSm+iDqK9qO03rUR7VM8j75PsE/l9h4nxBvRl6I9ycf3a2mqYL/K7fmNEA9F
YA+TZG/z+9UBLEOwt+nqhLFU6LXX+fPuJqYS/AciezUuIT6LQkR7Xkr/TPQf
8Ouh7D+Q1h8pPozAX0Hgj5DbU/SP0F3j3Z/uv/1/4kkvjGn731MHTTVFV+1U
7hyZvutf6Yo1RI6XW1i74uBbFfVwvjuBSlZcNWzEIyn+PYFKC0yfYrqtux3t
oImzqzYukaYkT2GpN7ZIB52dPvNYm0N76cykUzUfTCu+L+e0yNJ5zZq99ly8
83SJPL/vbdV9WrltO2ng2u/abvDaQ9u375pTpflK+v6Lp1+/S70m29PCvyOP
wg/XqDd58CK6+nVg5+eeS2x8i+3n3yY9o5Yr83YtnLeWuXsK5fP7gcX5H4n5
Dwr5ab34+32E36fp4u93Fn6fpPJXrSgqnwbS4gan5rnk+kvtLfnv7kVeerX2
kp6WPDzW5r85d1itGt/+r1IJ+XvTzO9Yny2Kd6Fw/0USHdqRsP/LRfdR32Ld
OmhmPrAks8bNBny9qZyDgf7F/jtc4umbTU9oak2/aY/vO9gIUf/rKeh/rAOv
f7Ihon5rEfRb1rnFjcBDK6PYz2ObWrNOmFj7B6MGTU2xsoGnmk8NrGFkT0T9
v4Gg/7NLZ+flNn6aQB8rt59eqb6eHRm5pPqxM09Zu+qnPeNYNKsi2lvPBXuL
lZsfcNlwMoHUj1o/aDAsmpUU7beugv3Gmp890HbadgOV73OhcYUf37Abov15
VbA/WRtxPl4tzMeszOd+LSI+5VPM6cJ7nh+eU+zskuerezLoh3XtMybUC6fj
T74tnRT6ifxnt8l56dST5D+6KPiPKJr3V9EA+6o+Jydl0v6UrxaWnmUn8BcS
6AtUeF4YT5uF8cS+Ty2xydQshC7Gvz/xdY8A0lcZ9o9xzkdqXPbPgWv3+NPq
6U3jG5fWU7OAzk3UdW+RuvzRWW+844eu/fN3jF7JAsY7no+P8up7O6p3/uL6
Xp/+vqTC1uDow0nswNzZDewLneybz/479dy2h5pFnfyip7+dXaoz+Vu//k9Y
1vHyZZv6W1lV3dpPh/LfsjkhyW/ufZnKGq+z7t107QOpjqn3H7GZ2dMHL6Oq
BsbRraF3R/nXSGXzJ7SNOJSSxloEPqicONLEGt67uXHC1xq6619hhvldIts5
37Cw2lI7q1azp6lBkIb5tUlqtf6Snc3afTCrpDuGdVyzYvGJ0omsVd6+L9aF
a9gme4nRZ8MS2ZeVjzRb/3cMazEkrlntiY+Yuv+LMqcSNezOhX7O3HgNla9V
gnpujmRzNsYvbh4dTl9M7xPwz8BQ9uzesHW1pugo/1blhbapoazZtbt3m33n
otDzVwc0qniXqb5bM+6HCS5aeOj5pGorA9ndpjlxS7z6baUnvY94ah9h0YYH
xyt69duRZyMr15x+nP3d7frWM2Fmykyt7/zq3hMae3XoXxMGpNLkMb8s1g5/
QpF1V61/Xa2Aap+/1OTndQ+o7dScnFEVrfT9gi33tY9f0fn5fjGZ37lplaX7
uFYz31BhyHe95g310MSQLrP1y99Q3d53HLGtPDR50rWObyu/p5ENqvTYvcxK
36p/WmG9Fk1Pt/2Yk17GRSW+/zQ85MYH2rZk2MIucVbaH5t18f/a/exHw+2k
HnnUObDUw8gxOppYpkv7ix3c1KmCudrXa5NowpOjXdZXcNHLaeN/6rzSRMcN
U/+a/GUu7ZsbkaS4Z6LR214uGHPCTFcjI8dcr22h7aNvf5jwZyp50s82Nuem
0t4/2884HuihYc+aTixbx2t3Bk4+1GeSm0yqWwkdv3dQ9Q3zbqoWOOjzsOaZ
M51O+mvNm1rrrzhoXHjWqHcJTpQzkDOIR2EQf8Lg95hbjGcZIcSzMGn8HRDG
H/tMjH8pK8S/sJ/F57ssPB/bwT8feybG0xwX4mnYLvF5RwnPy6T2CxXaj0H7
Mam9OwvtzcaI8Tq7hHgddkycT84J8wnbwb8/NlKMB2ooxAMxKR6othAPxKT3
20V4v0zqD6WF/sBGi/1loNBf2NdivFInIV6JtRH7409Cf2RS/FJFIX6JSfPf
SGH+Y1I8U20hnonZnzq+H33QSdMr9br/lzmEDRhfasTGe056Yhwe0q7UC1bG
qb1Zrq2DxvrRk+iLD5nU35VCf2dSf58i9HdWR+zvi4X+zi6L8Vi9hHgsVpaf
n9kecfy5hPHHxonjb6ww/lhf1aAfdrR3UUbJgBUfQx6yy476Xyf1c8B5Fq99
V3X9mInTbrF+uTOO9erpkuXR4vgdKIxf5h4aU6XDLB01Tq9wJvFUACv5eNqq
r9dKdvqRR7PF+aalMN+Qip9vaK4Y3/ZRiG+j1uL6VEVYn+iyGD+3WIifo8di
/FxtIX6OzojrYUthPaQKYjzdVCGejm6L8185Yf6jKuL6GCOsj7RLnG8/F+Zb
kubb6cJ8S134+ZZ2ivNtW2G+pbbifBslzLdEYvyfUYj/oxgxXnCuEC9I9fj5
nv7j13dSi/GI3wjxiNRU1A++F/QDmiTGJ7qE+ESS4hOnCPGJJK0vzYX1hSR9
Y6Sgb9BgPh6Sqojr12xh/aL64vr1VFi/6KW4ft0W1i8KF+MvBwnxl5Qhxmeq
hPhMktbHTGF9pEGiPlVH0KfoN3E9bSqspwT6FMH6i/GgpBH3P/nvk9pps7D/
ySLCEqp1Pyedb5b8E3oquWnVkPflbrEMsb/WF/qrkE6ZT6YNF/rUapXBziw6
FnJCUUj1fxv8WZ1jTrZt0pd96p/xvqcTj584PZJ+rKMTQnwdq1Hu8tUHA3Ty
+flP3wXV/LgtlGa1/pDQUxVEqWPm1NSOMNAexb6Am3vuUeJvilZz3AZqkXj8
/oKf7tMXV+aGzvDaA/z5fD19Pmf+PLX+Niu/8r+wVl595vXpmP8u1b0lP08p
4Xloq1g/f6F+9Mf7WrXrGCKoVN89tm3pY5jE7XoVMfmtOdG7hvq53H79ROb3
v2Oo2tCH9ok26fdkfZndXbZVM/JdDN163GzYhM/uUb/45FGRqg/0ts/ehFoz
VHTr4JkOymV2UsdHTHy77CkZG0y88iTMLsfrgX8I21+y7+hInfUXHp15K/tz
1rvanV/9i+SfMNGDwTVL/Xf/ibxf9VmlB9G9fn8u2599q7Z49HCONA8ZqG1m
22EfA1xy+T/+d/Jr7XP5PDiVHj607a1zLioZk9Tr6Ub4Xp2ygEpt7P/3qFYu
VlIZs6jjwQIqoaj+RauqLpYq2nfS+xsu6qf3Bf0U+6Ncv4uFefW/6bGXRQ0v
YWr9KF623/akXbJemqilb04fy1YkHBTTf6QHNVc7Fv1wglUT+5t0vi1z8pNu
WWYbxEvaacyGkYEd7c+wPdjZo/6Ty/VPpOr/Xbce3i2ND7l9WKdFjlc/btVR
9ckHGt0/7mJt66wu1apDEj3KOl7w+IH8PQap/Vi7sXVbXjAZ6eqOmGWXn7mY
s23fXyKap1Av+8aViU9c8vsT25d9V6bBws+emKnhidj9s2+62JQZU1suvZlG
S2Ir7pq31cUi1j6r2rytnW2YN7XuNGcc29dvcWVNfTtrON28pvXEeJbs13Tp
Zb2Rfu1cmGns8JaVYb/QYu88ZX/ftGRe6bdyfygt9AdG/kMrtaMEmhv2cOTo
EiGsyqs/FrRZaqSs3r1Gn1j9iO3NXL9k2ahkmq/t3WJd0mP2+PTrptmxbso5
Ua1txWMPWfqmisamRjfF7grpqukWJPefIKH/0d9i/tlCftovxlffF+KrKSxp
UunzcSbSBc7YZKrKaPqq3l+d/dNNgxeWbxC/NwT7OxnF5xsjPB/1q/Lm/EpK
oXYvd3yTfPit3F83COOBSovP7xSen34T48OXC/HhtErH5p05Y6HkWWFjI069
oYMmZ/LiPTZ2rU1A24BGiQTvm+B9ErwvWivGp0jn88uXjzzZZbRLHi+Hlx2s
sWmQi8aqP5uScv0uZSzSfVt5losS64+7O6judboXEfC/vlOjqFObKYO3PvGX
/FfUIvBN5ePXLtKxRV3MVc9FyfEvtwz/0aqCSJifomhd3vFzTe8fpYjCf2jG
JC3Fzzad2qE+yKTx8lgYL5TZ6GT+g00O+vGHXY39qt6isI3aR59fdNCnDn5D
cmaJ8TcqB2ke7Pvj+dNz8ni5JYxPeieOT8mfsit98KeVXzjI/O2SvWH7H9KK
hnM39u3hgvOPLioTe3T58dv36IjY3peE9mbtvvly3qPRdjgPaqQ2wnkgVi05
6R8nGeXv5bwW4/VqC/F67GuxP/xP6A9spfh+TcL7lf3D94TzRGya2N+GCf2N
+Ynz9XthvmYLb5c1r42KB39tHN1/fmbe+icquX5pwnkpZhHXswPCesbixfWs
gbCeMWn+1wjzv5jf4dVLis5TsSRxPZDOf2aI8xf/+/L8JY8/nTD+5PFpEcYn
zRHrz89/cfRQqD+VqirEqx4X4lXpa3E+mCXMB/J+1FjhfBt1GaOdZL6SIN9n
sahe78jfJrvh/Kmbftq9vElm9ZekFeerucJ8RcfE+aq1MF8RxKMTzL8E8egE
8y1BPDrB/EoQj04wn1LfwRX0GS0LaGrUmR+yNrtwPSPz7P8pmnvteWl8DV//
9OTaBS5q2bf15CvsEr3m5y/x/ZipknBejlV7vPDIihQnnfm5oHu7Q8/Y4raj
mn5exSW/34s723WYtumDvP7/sVlRfaEqlvTuwzklZzygFZ/vdZxkSdQr/MtX
F7tL5/FiKDd82/1x6bfo/eOeMwZdiJHvn12sqeQ3v0w0PTq0OswYc4Na336g
bhRd/H3bD4GHn+2IcVDIqqeJ78ouIVUJ9adv3juo0dKwct12H6JyTQd+CDwZ
ST+GTd/R4N4+Wjak+c8N8iLp5qBGbYJnHSXzpofhPd16OM8v71fK40Hcr5Tb
Q9yvZFe3T7xQ2NVKVU+6qnRd45LH48LXC6pd2yCxvH/JLjUalrAgXrp/xcW6
pDWMvdjNSV/pt3bYb3nNBtz86WW/UU64j8RrT87Ifhjy9CUbIPo3Jfush9h/
pfuvHH3O1HCssFDZb38f+na/dL46jX4ev+OM9q+H7BfxvI9LOO8jymNpR/Sc
tGlB98gl5v9MyE9S/l+E/CTVt5dQXxoo1pcfL05SCfWlCuL8xt/PaKZEIb6H
Akrcb/pdfga9ruBf+qBNiu9w03hhP4p2L5xlftjFLedPCRj3OshPRyww6hdN
QbQ8nv84+X54QWQ0LU87N6JUTBo1STy9qVcLab/GSv8I+2cE/mJaF3Xr4KDX
evm+dYiXZdJ6dfLfiuGPbNJ+po1Oz/tsatefXCxy9+RVft+myvF+EJ8r61sR
kTVbZpaR2ESWHn8+NPezyOVvPtXb7UqVvkfnpDVrvmp2dmcKC1m9rt7vCdJ9
KnomxS83E+KX2fYtxkePesXJ8iFiPPMIIZ6ZZcdsmLk8Kp6dHNC1QkQ1HftH
d+9tpQ6Z8n7jP41/TYr9M5NsndtEslgdW/Sw69qpS7Ll/cXK06bF9D6aTS/K
99NvbRTNgvuur9F1YL4sl+K7mwjx3eyqdVnzq7PzaVTmN7833aRmaT3Xbxkb
o5Pvh5Hi2XOFeHb2qo+jTT0vB1YNG7LWu/71PHMpyagKkdNL8e4/CvHuTOH4
pO/U5K68H/lIjH9vJMS/sw77rItn3s6X5VviyhtNT/LJoq34w6RJ79jF9pqO
u+frmbT+SPH924X4fjZswCrLqb/1zG3JfhQ1+xH7+WjjYMsdvXwfgnTe4LJw
3oDpCrUD5j/Ws/NLZy5bf2Y163VzzZ0j9+yyfj/g+v6NIZF2KrusuaXeX0q5
PGn/RDofsUg4H0G228E7Fp/Ws8b37y0+1+keSfWV1jvpfMcy4XwHjXr5MKvx
AD3raxxr6PT1S9rYcvPMdwec1H3z6qw2ySFkFdtf2h/99spvnV2BOlbum+Bb
dxtGUC+xvSW5dN7knHDehE5/FnG7bfMXLPb4z4OvVo6kXLvQ/lL6ZgsK1dWP
3acXm2dG//NdBHUU21+Sb6iePqrlmXyaM7H8412dpHgwA9XtlT+/SZ40Pk20
95g1KipL2k+10Mk1G/y2tIslqT9K+6/SeZlmwnkZWjzrzfevSmTT5AbLvzYc
+EBS/5TSS+dpmHCeRtY/f51mr3/whPRdCifVCrDXfaXRkjTepP3ZNfx5KVre
8Krt9aJ09qz29rFnVQbaJY4/Kf2ggI6L6mz9wOrrBh34qnvx93XC1lZe7r6t
l583JufFmp/Ky/ef0JMa6WeGXtSTND6l8tLF81LVhfNSJM0/Hbu8GNOkUirB
/EOlri7bcfODmR1r1u7MOL0Uf5RBmVeuHjqX58D5jpaxv1uvKtCxKfWilncO
K7aHJftW0qenCPo0k/Tpqh2L9Gm2bmcbY1JEtJz+vRifUqJtUXwK+7XchxoT
vet37+k5Soq9wdbXtQU+G6GG/cNoKlA+Prj1tPR9exu9FOLj0f6mXcZzTerO
c8n5J4jxMuuFeBn2uRgv06duUbwMm5Y7aUeoMoHajumaPXWKdJ9hIg2edf6b
JqbL7Om/c65VTfko2zc/1DPdvjn3I93L1tT4Zbs/6/e+ifHDhgQq1avE97/N
ku5bTKS+Qn5alN7WGtAvksbHl+h0Z9Rm1n1q5e336kXK90EtFOUNBDl9Kcp7
C3Jqk5vkPPfESVMufU/1goPpM5d/pTEVXLT14ulGHb5R4flTBu+LNc5t+6RT
+XD67Mto18LjTja53cA7Vd5F0v12LP5VjLReye+ftdOf/Va9zuVdt9e9tHdz
sjMNu1+/q3TRraS9o190dbKp4nlZo3Belkn96bjQn1itzqvNJ/+no6qqKU2H
PU5l7Tcby4/ZracBrw3BTbNSmaNH6K2kl0ba/aJ31qbeFnawrrZG9e7J9Gjk
3R3OvhZW/u6VxY4a0r0nZvZcX/bSxYkW9jowp4b+h1T2aMrhyrVC01ip5D/H
NT9gYo+6tCtTbmwa6/yXrbT/qhQ24LvXEe6/Iyh5iOGP0pNTGIwHZtzae6Ky
tpVuTOn149FtqcxSZ9ya92vSqVmNtZ0+dEplK7OqZ41tbyf/si37NrlqZn+U
Wrf9U3KuHM/z7cvfx73dn0sx/QI9/91LZivF8f5CGO+sp3j+uJFw/ph9K473
hsJ4l9f7N8J4Z8eqx3wReiyJ0h6PqnHAopfX/1hh/LO7NYb4N1ltpPYr+ii/
/6Rnq2ul1ly2O4U+a564vXyWlN5Cj4T5gZW+s3vwzPFm6pEdU2Zrkp79V3p0
76EpNrjP0M4edJp8dXsDLXvxw9YJ+jkfZfnt/9W690fDBKZZ/kuf+Q3jWac+
3RT6Kglsd8e9M+tTPNv2Q9yIsDeBcvpZT16csFrvMoNrz8hOeXGyviLOn2yM
eH67hnB+mw0Y37TDhefR9Pqnep/fOJQg6zN1hPmVPaoUZD+6xEF7Mr//8s++
CSy6uv7Nmat5crzT/gqtLqpW5VHO1Ucjp69OYPYrr3f0WJdHnf/cPnLNhAQW
5DLfCs7JI9Pe2nt2/vtBbu86wnrCjiRUnt/45yRa1n34nyXXxMrt/bewvrB5
PSdMHZxspG9dU076RceyQQvKWmpUSyFPI7f9ujFWbu8TwvrDUj7b/X/KuvJ4
LrPvXw1F+6Ii0UqNmQqtkk4N0aaUpNVQKmmxFtFCzGhPkUrpW6OoJCFZp8Nk
35Iwwtiyr5/PJ2RLPy/33qfX8/n9eV/Pc5/nnnvPPed9zrnn3LFn/64DW+ly
F59b/2KZnvebPz+JOPxzUsNssPpWEcRUpM73ulmIa/ffKbc8LYJN/05ZfLHt
X7Si+smS6Cd8SPPps0k+Perw9TEyfVtE9C2q0Hz7FSTfHpWovk0m+hZZPr0C
yadHpm+PEn2LDB+sJvgAF88XThG6pnP459xOxVE4Kg39Gqdcz0pIxY1Xh9ds
WZiBYc7Ppd2i3qLL0Vv5coMecfa6uZbGFn/7IFg/8snJ5lepuEA9KzDhlA/8
prWnZafsWxw3BOIsNH1hcHhX9FOveDTM9zrrGdTG9V97ydNd6kon19YtcB7y
wroTAh9PEZxLTca5RjbF7050Qlbz8dPVs5NRe+6NsXMedsLo/bN/tzV/i80U
H80m+AjzEor+UN6byeGzItPS+MPfMvBZn0XG9qXhaLtg/sgDJzNRy29EWJm/
D86bY9loYt2PM80CK5Ms7uP+NzfOdH9r486bxDRWWssldHJtpwMJNtLhnSAY
v+azY3wEOpiHRPwvqRN0LW0MI3d6YNYFpwqY0ce9Xyo7VHqvTB+I2kJR5B3V
ry8XjT41rd/+M7NbtglfYNj2MVJSSn1wPyHIdujDM5ivsQGC+vu3+JWXvtAM
4PBkle0AnoR8Sh/Td4yeUYQeYPQMI/RACaXfl9AP+yh9rD+jj7UZPTWHB+gB
Rm8VoRcyKX3sfTb+kviB8UMeHb+AjB+KKf3S7QP0QwalfzyhHxi+rib4GpZS
fmT4lvFfAOE/YPx5m/AnnKT8yN5XpPy3kPAfDKX81xs2wH9gQPl1FeFXYPzI
+jN+ZG0dym9d+wb4DdZQ/gwg/AlzKH/+Q/gTmP0SROwXYPU5vEh9DviD2h+1
xP6AY3z5A/VUvtgR+QJMvjD86kjlSxiRL7COypeNRL7Ad1o/xILUDwFmryUQ
ew2YPbaF2GMQRfUFP5+gFSVUB/QFpFF9wZ4zffGR6Atg+uIC0Rdwk+oL9r4N
1RflRF8Ak/eXiLwHJu/Z+zFUnpcSeQ7eVP6LiPwHAZX/akT+Qw+1Z32JPQuR
VH+qEP0Jknz9CAm0Xktv10C9FmD2bjOxd0FM/wPT//lE/4MMxSvsvONbik8G
EXwCCRSfqBN8AukUz7wjeIazN9yJfQ+r+PgFyiheCSd4BRopXplO8ApnnzgR
fwCwejZmpJ4NOLQHG+jYCOGy0STXh/K1nL3xkfgbYDLFa+MIXgMxvAb1tJ6O
DqmnAyMOzYgJ9a0EqSb9nl0TGzh7pZH4K+ALxXdXCb4DT4rv3hJ8B/b8ejzg
9KEzojSxBpKLvG7ILm0AMfwJYviTmy/qbwExfAti+BZCE+d92+5eAJd2rch0
mPKj/j31z4D5k3UhliatsPuEgm6iEnveDE/qtm1YZxbBtQNJG+VX55nuXdEE
V2+GdmtPjsJ3IYkKZd5N4NqnsKO78Q2W0fzcLJKfiwelaycbmX2EV55Hs9Yk
PuTw0RwS38D7NL7B7IsIGt/g11v8CBEnGxRVo+/igeTrZ4alNIPsnoztPsmh
ILHm+7ASuRYu/peWN2Lcon7+4Mcrm2F1Q4qyj2I4aFD/A/OPsno/L98N1PsB
HeqPSLcb8EeAOf3fNPI/HEL/x+LbKfR//Hh1M+iQ/+GR3YYjThsLwDDpq+cg
r0j4yVe/YA4IxOIjAqi74miLHuw+IwEIZ0/AhP3hAJNjp/XuE3D+3s/BmotU
ftSTovOTA7JO3VeDfIUolr+OYuuPlpvqg2Vkm/n3zfevb/Rz25oiCxFOWC4c
/ffsPij23mWb7ibC9iMr/LNO1mJinM/5x50sn6AW58/yVFF/wOWhwokxdatz
IupxFN0fw8n+4PzDpnfeNqmqN+Bx9eiGOaWNYvXuc+CJrqLvpL4a/MNK0iO1
KIfGw2sR+Pn+qPpmykqRQbdYfeduOBSsb7e2vhrVzOzTbhUUYLlyqlu+EVc/
Hj/+aXFMuaUKJwRpTT6xrQVLLpiXeWqzfIMWXPN5WGXesgo0X93p26nE7IVy
XDfxdW67fjl0tVgZa+6uQMPMWfIP7SrE8mkr4FFvnHqKWwW6mlxc5F9cBHdX
9TYI7ctxvE6RjVpoIfPDcvGk3AfXA1rXluOa7+rhu96XcPd7KJ+q0JDb2Mu9
X3RRYYNDZCkXH7s9VKg11S1GLF4WhW2GaS/KPItQOS/DMG6kiMunPXfb9Kee
yjK8W1azeMHmAuyIz3aOn/gOlsk5ZZ9TyUeJ/Wov1R2SxPIlEuFC8jltl8f5
WEf9yfHEn8zR66By+aufXQGut3rd/ndXMOxyO2GxanA6Pvv5zxTbn8O4eF3X
rx+nTvkrFwK2VHywSsvApi6fV9PcP4jlt+ZC+Yuvo4ZdT+fkw+IREWOsZmbj
4Kn+U8qWf+S+9zL9gEHI2wqQGueRcc8ynRvPZdGeacLdaXgsOzc8wqAZOrul
PgVZpGCbEirHKzaJ/a8ZrHIqLO6PTuXsYz/Nu0pxmUmo+VSvpTW2mYsXsHgX
P97QBptIvAtLmvuqHuzq5J5fV1SbYWnnycUf+uj5Fyty/oX+rxDqSf1FfE7j
QSw+VPPGdrnyhD6uP96+U/7SjZ3zuBNn3Kn1aIcSOz9xJ86d/o/Jkyran7VX
0fXZTtYHHtH1YfGCX247zZ66PRtWxViervdIhBXr3D5L9u9Hfv36TPjwanp6
9ukEOE7nt53ML3yl88vPR2mGY2R+OX3jS+YXtOj8su8nd+17H9sr4MbzH51P
9r2ldeO8ftkkAJnaXGml7hxIa9c+HOgtEKufLQDPIao/6W3Igbg/Lz+wm9QJ
Iue3ZoV6OTCbxpdZPmox3U+s/RfdT/x81SjsIPsJGH85En6HI7TeUQmpdwQ+
fHsaWL0jQ1LvCJj93EHsZ/Ck9Y5CSb0jYPuV/X88lU+fiHxi84ebiHwCSyqf
+PdPCKBQkFEktbQMVuhaWfuYCLj7cVh9p5ukvhOEWcyWNjPogMXn7Q1+yqb+
XZcOuKPjclAnrgTmUPnD8nOG8+tbAfPX1BN/DbD6VjWkvhWcov4ZCeKfgZG0
vpUmqW8FTP6z+pHdwc13JvuJ4Nq60wlKqdUQ6a+nYC0rErvfWwQlTT7eWY0/
7huXfuS0PGBNNRxxnZAgFc/uE6uG+VQ/8Pt3w36iH7h4l9fG8M3jrtZDN7+e
GJzk+7/o+/3/f79qwxQLIYjVLwOx83EgFu8EMX3L7Q+qb0Es3gnLlUckTbFp
gcYJl07O9EuEfQdnHSnzE7tPIl4AOeR8BhZMaIke3W/vjM3TH7lLcx24Uv8x
P38zH3qJ/xj30Xp6h0g9PTxbaXNw/4h8Dj/Jlew/4/chn8NPshTv3SJ4DxIp
3nMmeA/6Ngd82V2fCOMHTUv49JL5j5OhUO9uq8XREE6fLDzz18Ke7Pt49ULD
CPc/Ezm810v7DyH9gfXPI/2B9Z9P+sNF2p9/35IQAkxf/5ryKhGyNF1Glbb/
qF8uVm8HxfAuPntRezVRg+UxifAUxeMpBI9z8exb6wu1Dyo1ILMnrhB7Ahl/
xKi2zy88X4OMn68Tfub8y5RfkfErgbfVOHWjQP56VhfML6/5fUMI+1435OLL
3LH9+OfhbZ9pou9dHB4i/NwFOloyp4zM6tDccZ/FrfuNEJm8afq49f12tsmW
OWv77Qw+PmmEiacPRXyzrsAHQ67kmTT2QJrfqZjbjhU480bN6keyvRz+SBGN
aW+4IIQzf2U+i9tczNGnP05RUT7lE47dOfKp/Q0hd9+zaU3QeF0JIT8f00UI
5bmPrvxsWYz2NN48g8SbufHcc2271xxQgElU/jP9zuLd2STejSy+zZ5bK4VV
P+39BhoXiy7lKrzDDu178sVSfZz+7KR4w5/gDc5eUid4AvrkCZ5g+iaY4glJ
gifAKLtOY25oPBpO27teO7gALmzoWd1t9hbZfcu/Uf9tGvHf0u8Xgk1GUOPO
sDI4R/HfHYL/YCXFd0xenxiqon3Tpgn3LItd/HQOJ49w0aeJO3MVauD4FpPY
Z1pNyOTZQdXxn6odSmHh/cCPl9UDMI7GYxj/D6fnNyzI+Q10puc3NHQHzm/Q
9ciFNwpG1yYdugav0wuDytzSwSMq5/iSnUJ81xRgYKwkhCHGW0edihRipszp
N6GHq3CL1J3KCV8FaB+37tDKnmr8+dvyJrXeVnSOnDo1LtIb0yJd9PsaWnDS
1H1p/xvTAbNMfxpz8FwLhl3ROlgn+xUMZHqlA9ua0Lj4ksZb6XfokCT/BLMb
Ud4r59bdxyUwN6nM8x/jerRUsK+eadQA26r2uEf8V49Vt3zzz43tBMlZiUZ3
9Rtx8OtNs8pe5WCWTHQg6tZhTJT0UsnPH3H9q5zK7F01uCHgcYivYxoYO67R
lpGvwZC4e4PPFBSAoes19cEOdRjl6+rlOlcAV79abH2/qg7t+fOPytaD/QbL
hUJv4EZrucmfMdrEcXSpXg+0f1OerBrwGdU3fhAFNhfjXOvAuYbCMvSdpqt4
RjoArx8+Vi8aVIZyizvKNh9rAZnEG3J2xuU4c7hW0ta16SDq0a8qDSnEUKpv
lxB9i/56M/8JGfQVrGXzhmbe+4RbKb8ZEX5DqejXm0T933uaELNeoZ/OZ9pe
N1Q3f4E/Cq2P3FPNQ/Wu3mjNxwUAuRYPlkz6gEsoPppI8BHG8vEPLqD4Tofg
OzQ+sKNbXaodJp11eB2zYhfXXuI80AZPgwaXX+MK4Xe18d/+ffI3sP02n+w3
GM4fHzyn43Mn4wM3am/4EnsDRNTeWELsDZhG50dI5gfE5AE85s8PLKTzr0zm
Hx7T+b9G5h8M0iVXJyz6ALNMejIy8otBnr8ekMqXZ1AXphRZ+PQ8tE+vf/V5
bwWw9e8h6w+6fHsPDlD5+obIV/Cj8jOZyE+I4fMLiNmfwPh1HeFXUE+WqNP1
j4c9hRkSSi+qYC2ff+HljolmR19+gUkJsYP11leBHH+/ANsvhmS/gALVH/OI
/oBw/v6DPmq/pxH7HRzpfv6F7GdIbbeylf81Hh74JgTuTxWIywcQkw8wPMZ0
HSR9gWGp4csz/QQw71mEteqH72DskfhlZ6AA3GJDj81IKYSNgWMabq4UwcFt
3veOJAj+332o94k8QuUc2WOhni3wJNvafo3Vc/ivo7TuqHQrfDC+YlMY5A9O
vbEe1+a9486LONI289+c+tl3pf+JVpC6YjnG0PMOeGxfvMFk1o/7WoZDxLAe
rMScMOfD5jVCDPE/c2Ve0zPY9Nvn36qVBeh9ObF3wy9vsH2UU2aMZTP+PuNx
6NfZDXi2duq1XyPrceVVK6GsYzrabp70TUK2AdX464fmmX6jUwLaQMOp95XN
40pUCSf81UH4C7dS/lQi/Im/T0o7jr29kOHktk/t+idUX/Lvk380iwGON8uo
5XzEWscnOzuCUzHW1F0maXY6nrexmzm3OZ07L8ieR5DnoNJwMTfhn1YselZi
cFiiAEzp99PJ94GNbxkZH+yj9LkQ+mDt4bUCxa+dUDg6MNz8cj0w+StB5C94
TFCVHNPWBvlRcvlVTa3i8wlRoX7O9yEFhq73eyh56DVsbzF5rxVYAQ+GL1+6
+kgELHou1IuqKIKePIcRpho7uO8VkO8h+/8n8n8Mpvw/kfA/qlJ5t5LIO1hA
52slmS/Qo+tjR9YH2Hq2kfWEQP56g5h+6+fnLVrH/3Dj/Heq70Mt4iyNfvgf
aZudB+2MJu+L30d+U14rcuXJvyE4aiRMdagCfYnGrlqvOFiWEX++1b0KFkSl
bFsS/OO84AMf1wP6y9j55FrQ3KPtAb8gqgyHOSO/58KQUNuC9Cd3QfJlW83Z
C22cP+LCfdkyF2EbN17NeSra11LbOP5QovvpFdlPKLY/UWz/oiRfv8ILvv6E
3Xz9Daep/k8h+h/eebZNfybxGa6anfZKco7G/Bz3HXvb/oNt3no+z8+w++oq
Qe2ox8ixMlFYPFd+i/4BIazKDX4Y25LKzZ8XmT9ca2/beGLzj/uX/g8fFOW9

    "], {{{}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], 
         LineBox[{3160, 3522, 3811, 3810, 4032, 3986, 3987, 4263, 4262, 3346, 
          4031, 3507, 4455, 4457, 4456, 4133, 4132, 3337, 4252, 3145, 3257, 
          3777, 3499, 4647, 3144, 3808, 4447, 4243, 4244, 4241, 3329, 4242, 
          3485, 4713, 3486, 4127, 4126, 4236, 4235, 3130, 3251, 3479, 4442, 
          3480, 4443, 3328, 3472, 4634, 3128, 3770, 3769, 3471, 4633, 3127, 
          3802, 3464, 4710, 3465, 4123, 4122, 4229, 4228, 4435}], 
         LineBox[{3245, 4700, 3629, 3793, 3794, 4121, 4556, 3630, 3967, 4212, 
          4072, 3637, 4071, 3636, 4560, 3293, 3953, 3952}], 
         LineBox[{3246, 3292, 4202, 4201, 3432, 3625, 4731, 3624, 3922, 3737, 
          4743, 3738, 3618, 3919, 3240, 4695, 3732, 3617, 3731, 4694, 3239, 
          3917, 3234, 3288, 4199, 4198, 3429, 3605, 4727, 3604, 4383, 3425, 
          4380, 4382, 4381, 4543, 3910, 4624, 4625, 4623, 3715, 3907, 3906, 
          3216, 4189, 4188, 4187, 3417, 4530, 4531, 4529, 3583, 4008, 3409, 
          4361, 4362, 4062, 4061, 4518, 3897, 3898, 3574, 4724, 3573}], 
         LineBox[{3477, 4758, 3478, 3470, 3663, 3766, 3930, 3642, 3926, 3964, 
          3963, 3469, 4441, 4096, 4097, 3845, 3929, 3662}]},
        "0.162`"],
       Annotation[#, 0.162, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl1VlQllUcx/FjWtIoslhaipa5lS0KJC6IM6ItFhihBOikhSVkBeVMpC1T
F0EIlqIpKrY4lQnaVGYXrWqlNZlZFzZTVm5pXWRhTUkZ0OdMF1+/z/93fuc8
z/vIC0PKqgoru4UQNvvnyeQQ0lCUEsItaEgMYRn+7G0tKYS/uI7PoB6nzUu5
nRv4XzyFv83L+B9+mjuwIu4zL489buQudKLEvYrRIR/k/idc/4S8PiH8zGfp
dEe++YLUELL0epgnxPvr16MOXeabdZp0CvgcnUJ+zDyTE8w9MUkvTTZLtsHn
K+JB5hz5udan8BBzsfxF64nxGV0/Livll2R/eK45rtv4d5xCkt58vgPNnqen
3npO4Lm6l9mf5+wUvVSUyU7r3s7t3E92P79pz3Zsw4X2vsU7ucr+XTwUV8qu
wmiUOXMPf4JxGI/dsnIMc+Zw3Oce3e2v4h58hazRvQ7pHsYPOIojWCU/yb/i
F7ThN6yWt9n/sHMnOuOk59hr3mt+BF06QecmLsBXshokyAp0i2Ul6HT9gP0d
nGqthFP4PPTF22h37hp7b9Tr43ofvjY3IU+WZJ5n3wDd23gg78FunLG2UW+W
3gDX+5GGxeaB3IkF9lzES2SD+Rv9TSgyX2wutz6ch+Fb+Zc8Eg9ZH8Fd2Cqf
E99lfAfuW+GzZXA6PsNB61swW+dynW7eecAbsvHWq9xjHFfyF7wf+5Cpe0Lv
OJbgXutbOEP+Ix/DYtwjb+V0+VE+ggdxt7yFx8gP8yFUY6F8tOxWz/O6Z8hy
r7tkmTwWV+M1+UzdVi7V26A/Cs0oMbfE75j1oea1qJU1cYFsdeyZ18TvE1ai
xtzIM6wv5/XmFdwPDXjCXM/TrdfFM81LOQXXyda53wxZjTk59pFvXiufFn+G
+QDmefdzMdvnOd/nqJfl6gX9QlkvWSJ64xVkx98D8f1az9Gr1c80P8qfY7qz
rsc0ezu5WnYtXxMzTJKf4kXyqZyLKciSH+NK+WTOiV2MkR/gCnk2T8QEjJR/
yvPlGZweu/gQu1Aq34GP9Ko998d8PP4cYJG5RacVm9HX2naM8nny7bk0no8R
eA7P4gb5O9iqd6f9r/ICPps3xd8t9k61vtP5O/Cy7BJZrux78wvmheaN/IH5
eR5snmz9u/j/a64wN/P75nXxu2nOtn7QvMpcbn6G3zW/h0bXK+N71qnn/tbH
ut4Wv8OerTb5/79N/wEFCeNQ
          "]]},
        "0.108`"],
       Annotation[#, 0.108, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl1Hloz3Ecx/GvazIzc4U/ZJfZjI1sKZojhkyusdUcseEfQyJ/UIpEckyT
Y+4zhlxhK9dcueU+cswfNrnL/IMpj2/+eO75fb0/r8/n+2m//RZXOHfcnAZB
EKT5sS4mCGIxuVUQnIkKgilcIq9HHD7KZ80/cWTLIGiOvGh7WgfBQPMoeShv
1t2CTWjUIgjKeJh5kl653BgJZp+dU2n+haPtLXDWV8/f8Q2H9SabLbVvEsfo
nDQrlk9xK6Q4pxsuOGcc2oV34jSdInsu6lxCupxvXqUbL6dyd1wxm4ZO9s3k
TL1i+xLkeKxxj0d6j/EQ6fY+4Vf8mjP4FWfyG67mT9yXP3I//szv+Q8P5t/8
F/XY4/w67/sZYu1AmPmBuyxHhNwU43UnIA+rzAe5Z71ePiqtR9hfwU34ifUS
dJA7osie6WhhbaF9UfwL23XGym0838dV3WuowhVcxm/z3XpdPCchEdO9s6N5
B9ySb2KG8xM4Hjfw3J5dyHV+e72/qMcBs3TrPdEdaeiB/ebdOBVdkYJk7DMf
6XPYGX6+zlrnjBx5RPjZeC4zW8tF7hSrX8hxvEN/uM42HqOzUqc1VmC0vNU8
2/qGsCcvM28a/q1hmFxqPsD6ah4sLzEf7+xoZ+dySy7HIWTpReos0mnGiznT
LANLwvv73eRgBLLtrcV8vRr+adaff3AW1/E7/sB9+D3Xogal3vWCX+IZell/
zk/5DqfwLU7m23yX53nPPc6QeyPR81WeGH4XcM1dr5v11CuQa+X53rMAR/Xa
WTvCbbki/N7ojdI7jzPybPksR+IEkuUca+dwTJ4lH+cIHESiPMRaBaZ6xzRs
dH5Da/FcGP5OPXfRy9Z56z7V2GvW2WyA2WlM0lsb8/9/1j+JfqIq
          "]]},
        "0.054`"],
       Annotation[#, 0.054, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl009Mz3Ecx/FPxYb+/1kZLr+rySGTbCYZbWzkXzaEdLEVxcGw1bI5SMmB
sHGQf/Nv/dsw+mf+X6qfyUX+bOagzUlTyZ/N4zeH556f9+v9Z99aRSpqNlXH
hRC+4VFyCPvTQ6hBNT6nhLAlKYT2tBA60IZsM72ozgihh/v1NyNX3aeegyN2
j+IwXqqf6Jfhuf0XWIx81OlH9Wt5iAdl83mAoxhCs94t39Hg/m2+ibs4qb7D
79CORnUHt6ELTepObrE/4c4kxlHo/k/+yuf1RvkCF/M6+XoM+tZ65KeGcIKX
u1Xr1qSZOi5UN8Ry/VO8Sh0nL9XfZj9R/UaexCvM1Oslex/jFI7nInmLmTXq
GeoddsvtdvuWHsyUZXOZfI+817sPs+Q5vFNeIe/3TpStdeece5Peu/QSeLWs
WTaMe+buIw1b9bfbfeA9zVyxudNmJrw7YxmmIwElZsewzMxxM1EMmPsuy3Aj
E1kYk733M31AK1bqT8mLuMXuW9kX70u8lEf1CvgTCvQPuruEP6oXYUQ/j8/K
hu2MeF/khfxaL5fP6L2SNfnORkS842WVviWOq3iebC7K3c+T7ZVF7Pa58Tj2
+8MhuzlmfvFvTOEv/mC22U4zOdzFpe5kmd3IC9wrd69bL1O22/tH7O8N47Hf
tfy6nVS+wRvsXOMU9RVO5qtcIm/lJPXl2N8PV9nfh0o8VD9z+ykOpP//P/0H
eYaESA==
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl00tIlFEYxvGjZW0aLcFKuli7IEhCyUAod2JWkK0zUCd0kZbghS5WI9Su
dboxqlVlQkF235VmECJSmy5OF8pJR7tBF6h+hxYP//d53vec853vm1nf0FbX
mhNCeESHCkMYS4TwMJ+nEfWRZSGM4lFchTeXhrAah3HfkhBu4xr+Fq7FO1gv
v4cl/F1ch3+tT9rzDzbhyoIQnsgPqDc7d781xbIklvPN8kn9oG5R59pnAS2M
55groVb5LB7ELG6gjZThP9GN2KcZ2iSfdPY29bR6O2aww35D2OmswejxGj6g
+3SaT3qGM/jNs/Wa/4rv7VVrjy/qd+qUvIKfUFerC2jAunxM0Iz15/V24EW8
RBdojn9qjxRVmp/nd9P3+I6sW0G/+Sq9IfN7+V18rvpXfFYcpDr5Tj5H/ROv
4hXaI6/lL6t/YBG/nLa6W5bvdk6vvbuwIt4fU3xnfIf6i9SLKY/aZaVm+uw1
rXcOM9iPaWw3c9jMMNWbq8RjsoH4LSntjun4DtRT+Jbe0Gd5i/l5bDR/Xb8r
3gEb+Ky8WX8OX5l/TS+oRv8lTsl79I/Tc+vOyvLccUz/MT2TbYm/I3Mf+RP6
J6knfjc6RSPycXNl5prMfYhrE///E/8A8t95GQ==
          "]]},
        RowBox[{"-", "0.054`"}]],
       Annotation[#, -0.054, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0rtL1WEYB/CnJSLKNKrBqYaCLoM1JURaYJeli3+AQUOBl3NSk0AhldJq
sqDLUBrmbdUUKnDRTMyh0oMedUgIB62h7N7m50fDh+/7Pu/7POc9R3ddTJem
1kXEINObI27kRUzJm3IlJ6JiU8T63IhHWyIOqD+U++VbOckEd91tY9zd8oTa
Eb0pfWmuMOt8hixFFHOU4xxj+9aIbVygR+91PY18Nu+SeSvysvwim9Wb6HVv
h/t/1f8xyD37Idkp++VzBvikr1pPDT/t53xmif55eUJ+V/tBJRvdqXX/m/U1
WWjWV+sqZpmh2G9QJ7PJLBaYp4Ba8zbouSoP2R+kyZxcc59Zn9TbKU/JLpmn
3iNP23fLPnpp1rNqxhNvbOcxT+lI/ka0uNNKijT17g/Ls87OcYZSzlNtdr2z
F7ykwf6VXNRT5q0dPqedP2rd8pfskr/lkjtZ6zmGyPBRLWNuoXWV91dSwWH7
fXr2spP3fOAd00yxrPe23jvJd+AWreQ7G2fMeZs3vUl+S/Z46wOZkWPyNTXs
tr8vRxlJ6vomcv7/H68BeFZyIg==
          "]]},
        RowBox[{"-", "0.108`"}]],
       Annotation[#, -0.108, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0jlPVXEQxuGxAgKyXD4BjVa2JigEZMelMRBJlLBaKUvY5AJXWTskNBhJ
ANHaFmP0A7igpUbUCgIUbKGl4jmh+OWd5Z2Zf05OSWff/d5LEfEKywURQzjA
Yiria17EofgKrmLrckRWYUQ2evRyaB+9WRRRhhsY4LlL7+E2MvI7dIdvkH/J
3kG1crUh+t7eEfVnaBI343p+RIW8Et/M9aOBvxH1mDA3h267ZmkNfzWmxR1q
R/wz4kPaLp8STyY+s2O0mtahFv3yUvoPf9Fgz3dzw8hzPxfbPCfqx0irr3nj
G6xgFfNuLOAlRvVPk+9IP/K3i9tQLs7or9Pf6Farpy10Fnv8j+k+zbg5gXf6
xWZ+0Q96ae+Ypp/wGS/kVXrPeSrprSRWm9KbxAZG5TN03L4xrPK+5Uslb5Wn
sevmQ55HaMU1/Df3gHahA53JPnNn9I/eD7pmx0/6JPkW4qf2fMGm/DXWUxf/
1DkRL1VM
          "]]},
        RowBox[{"-", "0.162`"}]],
       Annotation[#, -0.162, "Tooltip"]& ], {}, {}}}], 
   {GrayLevel[0], AbsoluteThickness[3], LineBox[{{0, 0}, {1., 0}}]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.75, 
      FormBox[
       RowBox[{"-", "0.75`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.25, 
      FormBox[
       RowBox[{"-", "0.25`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.25, 
      FormBox["0.25`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.75, 
      FormBox["0.75`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}, {1.25, 
      FormBox["1.25`", TraditionalForm]}, {1.5, 
      FormBox["1.5`", TraditionalForm]}, {1.75, 
      FormBox["1.75`", TraditionalForm]}, {2., 
      FormBox["2.`", TraditionalForm]}}, {{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2, 
      FormBox["1.2`", TraditionalForm]}}, None, None},
  ImageMargins->0.,
  ImageSize->504,
  Method->{"TransparentPolygonMesh" -> True},
  PlotRange->{{-1.1536749816983884`, 
   2.1536749816983884`}, {-1.3536749816983884`, 1.3536749816983884`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 419},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->251107722]
}, Open  ]],

Cell[TextData[{
 "Arrows for velocity field are green, equipotential lines are red, \
streamlines are blue and panel is black.  Note orthogonality of equipotential \
lines and streamlines, streamlines parallel to velocity field and clockwise \
circulation associated with positive ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 ".  Thickened equipotential line along negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis is indicative of a branch cut for velocity potential.  Also, note \
that the velocity field shows an abrupt change in tangent-component velocity \
as the panel is crossed."
}], "ExampleText",
 CellID->680409942],

Cell["\<\
Potential-flow singularities may have discontinuities in velocity, velocity \
potential and stream function across the panel.  Such discontinuities are \
called jump conditions.\
\>", "ExampleText",
 CellID->62898067],

Cell["\<\
Compute the properties of tangent-component velocity across panel.\
\>", "ExampleText",
 CellID->673890120],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icuaMinus", ",", "icubMinus"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{"icua", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}], ",", 
     RowBox[{"Limit", "[", 
      RowBox[{"icub", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icuaPlus", ",", "icubPlus"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{"icua", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", 
        RowBox[{"-", "1"}]}]}], "]"}], ",", 
     RowBox[{"Limit", "[", 
      RowBox[{"icub", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", 
        RowBox[{"-", "1"}]}]}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icuaJump", ",", "icubJump"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"icuaPlus", "-", "icuaMinus"}], ",", 
     RowBox[{"icubPlus", "-", "icubMinus"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"uJump", "=", 
  RowBox[{
   RowBox[{"a", " ", "icuaJump"}], "+", 
   RowBox[{"b", " ", "icubJump"}]}]}]}], "Input",
 CellLabel->"In[13]:=",
 CellID->98488054],

Cell[BoxData[
 RowBox[{"a", "+", 
  RowBox[{"b", " ", "x"}]}]], "Output",
 ImageSize->{41, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[13]=",
 CellID->576135760]
}, Open  ]],

Cell["\<\
At a given location on the panel, there is a jump in the tangent-component \
velocity across the panel equal to the value of the circulation density at \
that point.\
\>", "ExampleText",
 CellID->9527100],

Cell["\<\
Compute the properties of the normal-component velocity across the panel.\
\>", "ExampleText",
 CellID->134318740],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icwaMinus", ",", "icwbMinus"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{"icwa", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}], ",", 
     RowBox[{"Limit", "[", 
      RowBox[{"icwb", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icwaPlus", ",", "icwbPlus"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{"icwa", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", 
        RowBox[{"-", "1"}]}]}], "]"}], ",", 
     RowBox[{"Limit", "[", 
      RowBox[{"icwb", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", 
        RowBox[{"-", "1"}]}]}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icwaJump", ",", "icwbJump"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"icwaPlus", "-", "icwaMinus"}], ",", 
     RowBox[{"icwbPlus", "-", "icwbMinus"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"wJump", "=", 
  RowBox[{
   RowBox[{"a", " ", "icwaJump"}], "+", 
   RowBox[{"b", " ", "icwbJump"}]}]}]}], "Input",
 CellLabel->"In[14]:=",
 CellID->326115239],

Cell[BoxData["0"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[14]=",
 CellID->32578111]
}, Open  ]],

Cell["Normal-component velocity is continuous across the panel.", \
"ExampleText",
 CellID->531709516],

Cell["\<\
Compute the properties of the velocity potential across panel.\
\>", "ExampleText",
 CellID->211831706],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Phi]aJump", ",", "ic\[Phi]bJump"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Phi]a", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", 
         RowBox[{"-", "1"}]}]}], "]"}], "-", 
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Phi]a", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], ",", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Phi]b", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", 
         RowBox[{"-", "1"}]}]}], "]"}], "-", 
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Phi]b", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[Phi]Jump", "=", 
  RowBox[{
   RowBox[{"a", " ", "ic\[Phi]aJump"}], "+", 
   RowBox[{"b", " ", "ic\[Phi]bJump"}]}]}]}], "Input",
 CellLabel->"In[15]:=",
 CellID->189498322],

Cell[BoxData[
 RowBox[{
  RowBox[{"a", " ", 
   RowBox[{"(", 
    RowBox[{"x", "-", "\[Delta]"}], ")"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", "b", " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["x", "2"], "-", 
     SuperscriptBox["\[Delta]", "2"]}], ")"}]}]}]], "Output",
 ImageSize->{146, 32},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[15]=",
 CellID->468990507]
}, Open  ]],

Cell["\<\
There is a jump in velocity potential across the panel consistent with the \
jump in tangent-component velocity.\
\>", "ExampleText",
 CellID->56731244],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uJump", "==", 
  RowBox[{"D", "[", 
   RowBox[{"\[Phi]Jump", ",", "x"}], "]"}]}]], "Input",
 CellLabel->"In[16]:=",
 CellID->114820653],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[16]=",
 CellID->317836186]
}, Open  ]],

Cell["\<\
Compute the properties of the stream function across the panel.\
\>", "ExampleText",
 CellID->403496840],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Psi]aJump", ",", "ic\[Psi]bJump"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Psi]a", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", 
         RowBox[{"-", "1"}]}]}], "]"}], "-", 
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Psi]a", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], ",", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Psi]b", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", 
         RowBox[{"-", "1"}]}]}], "]"}], "-", 
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Psi]b", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
          "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[Psi]Jump", "=", 
  RowBox[{
   RowBox[{"a", " ", "ic\[Psi]aJump"}], "+", 
   RowBox[{"b", " ", "ic\[Psi]bJump"}]}]}]}], "Input",
 CellLabel->"In[17]:=",
 CellID->102020424],

Cell[BoxData["0"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[17]=",
 CellID->111628474]
}, Open  ]],

Cell["Stream function is continuous across the panel.", "ExampleText",
 CellID->325279064],

Cell[TextData[{
 "Look at the properties of the velocity potential as the negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis is crossed in a counterclockwise direction."
}], "ExampleText",
 CellID->635547329],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Phi]aBranchJump", ",", "ic\[Phi]bBranchJump"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Phi]a", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"x", "<", "0"}]}], "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", 
         RowBox[{"-", "1"}]}]}], "]"}], "-", 
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Phi]a", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"x", "<", "0"}]}], "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], ",", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Phi]b", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"x", "<", "0"}]}], "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", 
         RowBox[{"-", "1"}]}]}], "]"}], "-", 
      RowBox[{"Limit", "[", 
       RowBox[{"ic\[Phi]b", ",", 
        RowBox[{"z", "\[Rule]", "0"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\[Delta]", ">", "0"}], ",", 
           RowBox[{"x", "<", "0"}]}], "}"}]}], ",", 
        RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[Phi]BranchJump", "=", 
  RowBox[{
   RowBox[{"a", " ", "ic\[Phi]aBranchJump"}], "+", 
   RowBox[{"b", " ", "ic\[Phi]bBranchJump"}]}]}]}], "Input",
 CellLabel->"In[18]:=",
 CellID->81019929],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "a"}], " ", "\[Delta]"}], "-", 
  FractionBox[
   RowBox[{"b", " ", 
    SuperscriptBox["\[Delta]", "2"]}], "2"]}]], "Output",
 ImageSize->{70, 35},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[18]=",
 CellID->67847108]
}, Open  ]],

Cell[TextData[{
 "There is a branch cut along the negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis for the velocity potential.  This abrupt change in value of the \
velocity potential as the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis is crossed causes the thickening of the equipotential line unless \
special care is taken with the options of ContourPlot."
}], "ExampleText",
 CellID->78970066]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Scope",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->1293636265],

Cell[TextData[{
 "The general application for these influence coefficients is the \
construction of the panel method ",
 Cell[BoxData[
  ButtonBox["PMVortexLinearNeumann",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/PMVortexLinearNeumann"]], 
  "InlineFormula"],
 ".  As part of the tangent-flow boundary conditions, velocity influence \
coefficients are computed at each mid-panel collocation point due to the \
vortex panels representing the discretized airfoil.  The dot product of these \
influence coefficients with panel unit normals gives the normal-component \
influence coefficients.  This example uses the NACA-4412 airfoil discretized \
to six panels."
}], "ExampleText",
 CellID->246788676],

Cell["\<\
Compute collections of vectors locating panel endpoints, collocation points \
and outward unit normals to panels.  Each of these collections of vectors is \
of the data type CartesianVectors.  The final result for this example uses \
only the components of velocity influence coefficients normal to vortex \
panels.  Because there is no jump condition for normal-component velocity, no \
numerical problems occur by placing collocation points on the panels.\
\>", "ExampleText",
 CellID->67964247],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"NACA4DigitAirfoil", "[", 
     RowBox[{"4412", ",", 
      RowBox[{"NDiscretizeUnitSegment", "[", "3", "]"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rC", "=", 
   RowBox[{"PanelPoints", "[", "rP", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"uvN", "=", 
   RowBox[{"PanelNormals", "[", "rP", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Head", "/@", 
  RowBox[{"{", 
   RowBox[{"rP", ",", "rC", ",", "uvN"}], "}"}]}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->27822125],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "CartesianVectors", ",", "CartesianVectors", ",", "CartesianVectors"}], 
  "}"}]], "Output",
 ImageSize->{376, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->186695972]
}, Open  ]],

Cell["\<\
These are all two dimensional vectors.  The seven vectors locating panel \
endpoints describe six connected panels specifying the discretized airfoil.  \
Components of vectors are arranged in simple lists.\
\>", "ExampleText",
 CellID->363116584],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Dimensions", "/@", 
  RowBox[{"{", 
   RowBox[{"rP", ",", "rC", ",", "uvN"}], "}"}]}]], "Input",
 CellLabel->"In[2]:=",
 CellID->131208994],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", "7"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "6"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "6"}], "}"}]}], "}"}]], "Output",
 ImageSize->{157, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->297842021]
}, Open  ]],

Cell["\<\
Compute the velocity influence coefficients associated with the constant and \
linear terms in the equation for circulation density.  Note that each of the \
two collections of vectors are of the data type CartesianVectors.\
\>", "ExampleText",
 CellID->128983040],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icA", ",", "icB"}], "}"}], "=", 
   RowBox[{"ICVortexLinear", "[", 
    RowBox[{"rC", ",", "rP"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Head", "/@", 
  RowBox[{"{", 
   RowBox[{"icA", ",", "icB"}], "}"}]}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->689942839],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"CartesianVectors", ",", "CartesianVectors"}], "}"}]], "Output",
 ImageSize->{253, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->673368701]
}, Open  ]],

Cell["\<\
These are collections of two-dimensional vectors with components arranged in \
matrix form with six rows and six columns.  \
\>", "ExampleText",
 CellID->70673352],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Dimensions", "/@", 
  RowBox[{"{", 
   RowBox[{"icA", ",", "icB"}], "}"}]}]], "Input",
 CellLabel->"In[4]:=",
 CellID->523446322],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", "6", ",", "6"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "6", ",", "6"}], "}"}]}], "}"}]], "Output",
 ImageSize->{143, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->368646767]
}, Open  ]],

Cell["\<\
Take the dot product of each of these collections of vectors with the \
collection of unit normal vectors to compute the collection of \
normal-component velocity influence coefficients.\
\>", "ExampleText",
 CellID->242002433],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icAN", ",", "icBN"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"icA", ".", "uvN"}], ",", 
     RowBox[{"icB", ".", "uvN"}]}], "}"}]}], ";"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->664517122],

Cell["Note that the results are lists.", "ExampleText",
 CellID->741805088],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Head", "/@", 
  RowBox[{"{", 
   RowBox[{"icAN", ",", "icBN"}], "}"}]}]], "Input",
 CellLabel->"In[6]:=",
 CellID->237716289],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"List", ",", "List"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->171159487]
}, Open  ]],

Cell["\<\
These lists can be represented by matrices with six rows and six columns.\
\>", "ExampleText",
 CellID->25104228],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Dimensions", "/@", 
  RowBox[{"{", 
   RowBox[{"icAN", ",", "icBN"}], "}"}]}]], "Input",
 CellLabel->"In[7]:=",
 CellID->157798943],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"6", ",", "6"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"6", ",", "6"}], "}"}]}], "}"}]], "Output",
 ImageSize->{107, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->101135789]
}, Open  ]],

Cell["\<\
Here is the matrix of normal-component velocity influence coefficients for \
the contribution of the constant term in the circulation-density function.  \
Chop is used to replace numbers close to zero with zero.\
\>", "ExampleText",
 CellID->257626546],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Chop", "[", "icAN", "]"}], "//", "MatrixForm"}]], "Input",
 CellLabel->"In[8]:=",
 CellID->101065266],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0.1728362320479963`", "0.08223851071460127`", 
      "0.0831124055323044`", "0.16166546715853025`", 
      RowBox[{"-", "0.08595926215825454`"}]},
     {
      RowBox[{"-", "0.17687662730519638`"}], "0", "0.17700828509573557`", 
      "0.16124476201769958`", 
      RowBox[{"-", "0.028998794181023954`"}], 
      RowBox[{"-", "0.16683661840070965`"}]},
     {
      RowBox[{"-", "0.08195494538979214`"}], 
      RowBox[{"-", "0.1727259884575746`"}], "0", "0.1129483912826577`", 
      RowBox[{"-", "0.15217312012490883`"}], 
      RowBox[{"-", "0.08123871452342933`"}]},
     {"0.07594635175874308`", "0.1441834065134932`", 
      RowBox[{"-", "0.12577678973495487`"}], "0", "0.17286217949671967`", 
      "0.0785190739201845`"},
     {"0.16202922599240596`", "0.03281156740374286`", 
      RowBox[{"-", "0.15023249970581934`"}], 
      RowBox[{"-", "0.1771172467271518`"}], "0", "0.17481751573082782`"},
     {"0.09046787840725361`", 
      RowBox[{"-", "0.15637110443330904`"}], 
      RowBox[{"-", "0.07834038906500142`"}], 
      RowBox[{"-", "0.08244754387774987`"}], 
      RowBox[{"-", "0.17491563509809227`"}], "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 ImageSize->{483, 87},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]//MatrixForm=",
 CellID->233772780]
}, Open  ]],

Cell[TextData[{
 "More details for formulating the model are given in the ",
 ButtonBox["implementation",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/Implementation#336729780"],
 " section of the chapter on panel methods."
}], "ExampleText",
 CellID->198004268]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[{
 StyleBox[ButtonBox["ICVortexPoint",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICVortexPoint"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICVortexConstant",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICVortexConstant"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICDoubletLinear",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletLinear"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICSourceLinear",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICSourceLinear"],
  FontFamily->"Verdana"],
 " "
}], "SeeAlso",
 CellID->745892494]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["2.1 Introduction to Influence Coefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"]], \
"Tutorials",
 CellID->178928221],

Cell[TextData[ButtonBox["2.3 Isolated Panel",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/IsolatedPanel"]], "Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["InfluenceCoefficients",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/InfluenceCoefficients"]], "MoreAbout",
 CellID->594394]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"ICVortexLinear - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 1, 7.3752000}", "context" -> 
    "Aerodynamics`InfluenceCoefficients`", "keywords" -> {}, "index" -> True, 
    "label" -> "Aerodynamics Package Paclet Symbol", "language" -> "en", 
    "paclet" -> "Aerodynamics Package", "status" -> "None", "summary" -> 
    "ICVortexLinear[r_f, r_p] computes velocity influence coefficients at \
field points r_f, due to connected vortex panels of linearly varying strength \
whose endpoints are located by r_p.ICPhiVortexLinear[r_f, r_p] computes \
velocity-potential influence coefficients at field points r_f due to \
connected vortex panels of linear varying strength whose endpoints are \
located by r_p.ICPsiVortexLinear[r_f, r_p] computes stream-function influence \
coefficients at field points r_f due to connected vortex panels of linear \
varying strength whose endpoints are located by r_p.", "synonyms" -> {}, 
    "title" -> "ICVortexLinear", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/ICVortexLinear"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[10647, 322, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->538786138]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 280155, 6008}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[607, 21, 76, 1, 70, "SymbolColorBar"],
Cell[686, 24, 191, 5, 70, "LinkTrail"],
Cell[880, 31, 3562, 81, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4467, 116, 57, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4527, 119, 2283, 71, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6847, 195, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->220476103],
Cell[7430, 214, 364, 11, 70, "Notes",
 CellID->1067943069],
Cell[7797, 227, 1006, 29, 70, "Notes",
 CellID->45237691],
Cell[8806, 258, 633, 16, 70, "Notes",
 CellID->433990591],
Cell[9442, 276, 1091, 38, 70, "Notes",
 CellID->352350115],
Cell[10536, 316, 74, 1, 70, "Notes",
 CellID->529540534]
}, Closed]],
Cell[CellGroupData[{
Cell[10647, 322, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->538786138],
Cell[CellGroupData[{
Cell[11133, 339, 147, 5, 70, "ExampleSection",
 CellID->15938609],
Cell[11283, 346, 132, 6, 70, "ExampleText",
 CellID->513831186],
Cell[11418, 354, 99, 3, 70, "Input",
 CellID->66474307],
Cell[11520, 359, 429, 11, 70, "ExampleText",
 CellID->659456463],
Cell[11952, 372, 204, 6, 70, "ExampleText",
 CellID->42111717],
Cell[12159, 380, 495, 18, 70, "Input",
 CellID->64953589],
Cell[12657, 400, 361, 9, 70, "ExampleText",
 CellID->326501090],
Cell[13021, 411, 444, 15, 70, "Input",
 CellID->560964283],
Cell[13468, 428, 217, 7, 70, "ExampleText",
 CellID->217164756],
Cell[CellGroupData[{
Cell[13710, 439, 72, 2, 70, "Input",
 CellID->14420391],
Cell[13785, 443, 401, 14, 54, "Output",
 CellID->944684992]
}, Open  ]],
Cell[14201, 460, 662, 14, 70, "ExampleText",
 CellID->120984796],
Cell[CellGroupData[{
Cell[14888, 478, 438, 15, 70, "Input",
 CellID->176773044],
Cell[15329, 495, 696, 23, 67, "Output",
 CellID->1012007624]
}, Open  ]],
Cell[16040, 521, 243, 7, 70, "ExampleText",
 CellID->118713566],
Cell[CellGroupData[{
Cell[16308, 532, 437, 15, 70, "Input",
 CellID->25568170],
Cell[16748, 549, 1363, 41, 67, "Output",
 CellID->154777397]
}, Open  ]],
Cell[18126, 593, 142, 4, 70, "ExampleText",
 CellID->44302699],
Cell[CellGroupData[{
Cell[18293, 601, 753, 24, 70, "Input",
 CellID->575890672],
Cell[19049, 627, 1742, 54, 115, "Output",
 CellID->251340842]
}, Open  ]],
Cell[20806, 684, 89, 1, 70, "ExampleText",
 CellID->22197235],
Cell[CellGroupData[{
Cell[20920, 689, 753, 24, 70, "Input",
 CellID->158994946],
Cell[21676, 715, 2158, 66, 90, "Output",
 CellID->326456112]
}, Open  ]],
Cell[23849, 784, 190, 4, 70, "ExampleText",
 CellID->9427774],
Cell[24042, 790, 750, 26, 70, "Input",
 CellID->258291294],
Cell[24795, 818, 947, 34, 70, "ExampleText",
 CellID->78226744],
Cell[25745, 854, 253, 9, 70, "ExampleText",
 CellID->421446271],
Cell[CellGroupData[{
Cell[26023, 867, 584, 18, 70, "Input",
 CellID->13857323],
Cell[26610, 887, 210, 7, 36, "Output",
 CellID->462937151]
}, Open  ]],
Cell[CellGroupData[{
Cell[26857, 899, 609, 19, 70, "Input",
 CellID->499128776],
Cell[27469, 920, 209, 7, 36, "Output",
 CellID->25372728]
}, Open  ]],
Cell[27693, 930, 1033, 31, 70, "ExampleText",
 CellID->78531558],
Cell[CellGroupData[{
Cell[28751, 965, 4691, 134, 70, "Input",
 CellID->1746043248],
Cell[33445, 1101, 217847, 3973, 70, "Output",
 CellID->251107722]
}, Open  ]],
Cell[251307, 5077, 669, 14, 70, "ExampleText",
 CellID->680409942],
Cell[251979, 5093, 227, 5, 70, "ExampleText",
 CellID->62898067],
Cell[252209, 5100, 117, 3, 70, "ExampleText",
 CellID->673890120],
Cell[CellGroupData[{
Cell[252351, 5107, 2392, 71, 70, "Input",
 CellID->98488054],
Cell[254746, 5180, 204, 7, 36, "Output",
 CellID->576135760]
}, Open  ]],
Cell[254965, 5190, 214, 5, 70, "ExampleText",
 CellID->9527100],
Cell[255182, 5197, 124, 3, 70, "ExampleText",
 CellID->134318740],
Cell[CellGroupData[{
Cell[255331, 5204, 2393, 71, 70, "Input",
 CellID->326115239],
Cell[257727, 5277, 158, 5, 36, "Output",
 CellID->32578111]
}, Open  ]],
Cell[257900, 5285, 102, 2, 70, "ExampleText",
 CellID->531709516],
Cell[258005, 5289, 113, 3, 70, "ExampleText",
 CellID->211831706],
Cell[CellGroupData[{
Cell[258143, 5296, 2082, 57, 70, "Input",
 CellID->189498322],
Cell[260228, 5355, 426, 15, 53, "Output",
 CellID->468990507]
}, Open  ]],
Cell[260669, 5373, 162, 4, 70, "ExampleText",
 CellID->56731244],
Cell[CellGroupData[{
Cell[260856, 5381, 159, 5, 70, "Input",
 CellID->114820653],
Cell[261018, 5388, 162, 5, 36, "Output",
 CellID->317836186]
}, Open  ]],
Cell[261195, 5396, 114, 3, 70, "ExampleText",
 CellID->403496840],
Cell[CellGroupData[{
Cell[261334, 5403, 2082, 57, 70, "Input",
 CellID->102020424],
Cell[263419, 5462, 159, 5, 36, "Output",
 CellID->111628474]
}, Open  ]],
Cell[263593, 5470, 90, 1, 70, "ExampleText",
 CellID->325279064],
Cell[263686, 5473, 234, 6, 70, "ExampleText",
 CellID->635547329],
Cell[CellGroupData[{
Cell[263945, 5483, 1911, 53, 70, "Input",
 CellID->81019929],
Cell[265859, 5538, 307, 11, 56, "Output",
 CellID->67847108]
}, Open  ]],
Cell[266181, 5552, 448, 11, 70, "ExampleText",
 CellID->78970066]
}, Open  ]],
Cell[CellGroupData[{
Cell[266666, 5568, 221, 7, 70, "ExampleSection",
 CellID->1293636265],
Cell[266890, 5577, 720, 15, 70, "ExampleText",
 CellID->246788676],
Cell[267613, 5594, 507, 8, 70, "ExampleText",
 CellID->67964247],
Cell[CellGroupData[{
Cell[268145, 5606, 626, 18, 70, "Input",
 CellID->27822125],
Cell[268774, 5626, 265, 9, 36, "Output",
 CellID->186695972]
}, Open  ]],
Cell[269054, 5638, 256, 5, 70, "ExampleText",
 CellID->363116584],
Cell[CellGroupData[{
Cell[269335, 5647, 163, 5, 70, "Input",
 CellID->131208994],
Cell[269501, 5654, 361, 13, 36, "Output",
 CellID->297842021]
}, Open  ]],
Cell[269877, 5670, 274, 5, 70, "ExampleText",
 CellID->128983040],
Cell[CellGroupData[{
Cell[270176, 5679, 339, 11, 70, "Input",
 CellID->689942839],
Cell[270518, 5692, 234, 7, 36, "Output",
 CellID->673368701]
}, Open  ]],
Cell[270767, 5702, 173, 4, 70, "ExampleText",
 CellID->70673352],
Cell[CellGroupData[{
Cell[270965, 5710, 153, 5, 70, "Input",
 CellID->523446322],
Cell[271121, 5717, 322, 11, 36, "Output",
 CellID->368646767]
}, Open  ]],
Cell[271458, 5731, 237, 5, 70, "ExampleText",
 CellID->242002433],
Cell[271698, 5738, 274, 10, 70, "Input",
 CellID->664517122],
Cell[271975, 5750, 75, 1, 70, "ExampleText",
 CellID->741805088],
Cell[CellGroupData[{
Cell[272075, 5755, 149, 5, 70, "Input",
 CellID->237716289],
Cell[272227, 5762, 209, 7, 36, "Output",
 CellID->171159487]
}, Open  ]],
Cell[272451, 5772, 123, 3, 70, "ExampleText",
 CellID->25104228],
Cell[CellGroupData[{
Cell[272599, 5779, 155, 5, 70, "Input",
 CellID->157798943],
Cell[272757, 5786, 302, 11, 36, "Output",
 CellID->101135789]
}, Open  ]],
Cell[273074, 5800, 262, 5, 70, "ExampleText",
 CellID->257626546],
Cell[CellGroupData[{
Cell[273361, 5809, 136, 4, 70, "Input",
 CellID->101065266],
Cell[273500, 5815, 1831, 44, 122, "Output",
 CellID->233772780]
}, Open  ]],
Cell[275346, 5862, 282, 7, 70, "ExampleText",
 CellID->198004268]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[275677, 5875, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[276097, 5889, 651, 22, 70, "SeeAlso",
 CellID->745892494]
}, Open  ]],
Cell[CellGroupData[{
Cell[276785, 5916, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[277207, 5930, 210, 5, 70, "Tutorials",
 CellID->178928221],
Cell[277420, 5937, 159, 3, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[CellGroupData[{
Cell[277616, 5945, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[278038, 5959, 164, 3, 70, "MoreAbout",
 CellID->594394]
}, Open  ]],
Cell[278217, 5965, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

