(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    195405,       3981]
NotebookOptionsPosition[    189863,       3802]
NotebookOutlinePosition[    191783,       3852]
CellTagsIndexPosition[    191696,       3847]
WindowTitle->ICVortexPoint - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"2.1 Introduction to Influence Coefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"], "\
\<\"2.2 Point Singularities\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/PointSingularities"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"ICVortexConstant\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICVortexConstant"], "\<\"ICVortexLinear\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICVortexLinear"], "\<\"ICSourcePoint\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICSourcePoint"], "\<\"ICDoubletPoint\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/ref/ICDoubletPoint"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"InfluenceCoefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/InfluenceCoefficients"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/ICVortexPoint\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/ICVortexPoint"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
ICVortexPoint.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/ICVortexPoint", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["ICVortexPoint", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICVortexPoint", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes velocity influence coefficients at field \
points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     ", due to vortices at ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICPhiVortexPoint", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes velocity-potential influence coefficients at \
field points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     ", due to vortices at ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ICPsiVortexPoint", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]",
     "computes stream-function influence coefficients at field points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     ", due to vortices at ",
     Cell[BoxData[
      RowBox[{
       SubscriptBox[
        StyleBox["r", "TI"], 
        StyleBox["p", "TI"]], "."}]], "InlineFormula"]
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, 
   "Rows" -> {None, None, {None}}, "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->60470433],

Cell[TextData[{
 "Input arguments are collections of vectors of the data type ",
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], "InlineFormula"],
 ".  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation may be used for single vectors."
}], "Notes",
 CellID->1067943069],

Cell["\<\
The strength (circulation) of a vortex point is denoted by \[CapitalGamma].   \
For a collection of vortices, strength parameters are represented as a list. \
\>", "Notes",
 CellID->358739961],

Cell[TextData[{
 "Output for velocity influence coefficients is a collection of vectors of \
the data type CartesianVectors.  Influence coefficients for velocity \
potential and stream function are lists.  For the special case of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation being used to specify a single field point, ",
 "output is simplified as illustrated in the examples",
 "."
}], "Notes",
 CellID->30234373],

Cell[TextData[{
 "The velocity, velocity potential and stream function for a point vortex are \
written as products of a geometric influence coefficient and the strength \
parameter.\t\t\t\t\t\t\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["V", "TI"], "=", 
   RowBox[{"\[CapitalGamma]", " ", 
    StyleBox["ic", "TI"]}]}]], "InlineFormula"],
 "\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Phi]", "TR"], "=", 
   RowBox[{"\[CapitalGamma]", " ", 
    StyleBox["ic\[Phi]", "TI"]}]}]], "InlineFormula"],
 "\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Psi]", "TR"], "=", 
   RowBox[{"\[CapitalGamma]", " ", 
    StyleBox["ic\[Psi]", "TI"]}]}]], "InlineFormula"]
}], "Notes",
 CellID->777516543],

Cell["New in 1.0   |   Last modified in 1.2", "Notes",
 CellID->529540534]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->196781413],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->219571741],

Cell[TextData[{
 "Load ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->513831186],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->267685788],

Cell[TextData[{
 "This example demonstrates some properties of a single point vortex located \
at the origin.  Specify an arbitrary field point and the location of the \
singularity.  Single vectors may be entered in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation."
}], "ExampleText",
 CellID->659456463],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"rF", ",", "rP"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"x", ",", "z"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->64953589],

Cell["\<\
Compute the algebraic expressions for components of the velocity influence \
coefficient.\
\>", "ExampleText",
 CellID->326501090],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"icu", ",", "icw"}], "}"}], "=", 
  RowBox[{"ICVortexPoint", "[", 
   RowBox[{"rF", ",", "rP"}], "]"}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->560964283],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["z", 
    RowBox[{"2", " ", "\[Pi]", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"]}], ")"}]}]], ",", 
   RowBox[{"-", 
    FractionBox["x", 
     RowBox[{"2", " ", "\[Pi]", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"]}], ")"}]}]]}]}], "}"}]], "Output",
 ImageSize->{192, 35},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->112915787]
}, Open  ]],

Cell["Compute the velocity-potential influence coefficient.", "ExampleText",
 CellID->44302699],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ic\[Phi]", "=", 
  RowBox[{"ICPhiVortexPoint", "[", 
   RowBox[{"rF", ",", "rP"}], "]"}]}]], "Input",
 CellLabel->"In[4]:=",
 CellID->575890672],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"ArcTan", "[", 
    RowBox[{"x", ",", "z"}], "]"}], 
   RowBox[{"2", " ", "\[Pi]"}]]}]], "Output",
 ImageSize->{98, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->191967185]
}, Open  ]],

Cell[TextData[{
 "Replace the ArcTan function with a named angle (",
 "See ",
 ButtonBox["2.1 Introduction to InfluenceCoefficients",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"],
 " for sketch defining angles",
 ")."
}], "ExampleText",
 CellID->53987023],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ic\[Phi]", "/.", 
  RowBox[{
   RowBox[{"ArcTan", "[", 
    RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}]}]], "Input",
 CellLabel->"In[5]:=",
 CellID->30542685],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox["\[Theta]1", 
   RowBox[{"2", " ", "\[Pi]"}]]}]], "Output",
 ImageSize->{34, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->41679754]
}, Open  ]],

Cell["Compute stream-function influence coefficient.", "ExampleText",
 CellID->472419597],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ic\[Psi]", "=", 
  RowBox[{"Flatten", "[", 
   RowBox[{"ICPsiVortexPoint", "[", 
    RowBox[{"rF", ",", "rP"}], "]"}], "]"}]}]], "Input",
 CellLabel->"In[6]:=",
 CellID->8664355],

Cell[BoxData[
 FractionBox[
  RowBox[{"Log", "[", 
   RowBox[{
    SuperscriptBox["x", "2"], "+", 
    SuperscriptBox["z", "2"]}], "]"}], 
  RowBox[{"4", " ", "\[Pi]"}]]], "Output",
 ImageSize->{83, 37},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->229221347]
}, Open  ]],

Cell["\<\
Write the velocity components, velocity potential and stream function as the \
product of the strength parameter and the geometric influence coefficients.\
\>", "ExampleText",
 CellID->9427774],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"u", ",", "w", ",", "\[Phi]", ",", "\[Psi]"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalGamma]", " ", "icu"}], ",", 
     RowBox[{"\[CapitalGamma]", " ", "icw"}], " ", ",", 
     RowBox[{"\[CapitalGamma]", " ", "ic\[Phi]"}], ",", 
     RowBox[{"\[CapitalGamma]", " ", "ic\[Psi]"}]}], "}"}]}], ";"}]], "Input",
 CellLabel->"In[7]:=",
 CellID->258291294],

Cell[TextData[{
 "Verify that the relations between velocity, velocity potential and stream \
function are satisfied.\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["u", "TI"], "=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Phi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["x", "TI"]}]], "=", 
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Psi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["z", "TI"]}]]}]}]], "InlineFormula"],
 "\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["w", "TI"], "=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Phi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["z", "TI"]}]], "=", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{"\[PartialD]", 
       StyleBox["\[Psi]", "TR"]}], 
      RowBox[{"\[PartialD]", 
       StyleBox["x", "TI"]}]]}]}]}]], "InlineFormula"],
 "\t\t\t\t\t"
}], "ExampleText",
 CellID->78226744],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"u", "\[Equal]", 
    RowBox[{"D", "[", 
     RowBox[{"\[Phi]", ",", "x"}], "]"}]}], ",", 
   RowBox[{"u", "\[Equal]", 
    RowBox[{"D", "[", 
     RowBox[{"\[Psi]", ",", "z"}], "]"}]}]}], "}"}]], "Input",
 CellLabel->"In[8]:=",
 CellID->13857323],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->372418502]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"w", "\[Equal]", 
    RowBox[{"D", "[", 
     RowBox[{"\[Phi]", ",", "z"}], "]"}]}], ",", 
   RowBox[{"w", "\[Equal]", 
    RowBox[{"-", 
     RowBox[{"D", "[", 
      RowBox[{"\[Psi]", ",", "x"}], "]"}]}]}]}], "}"}]], "Input",
 CellLabel->"In[9]:=",
 CellID->692553449],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->12270635]
}, Open  ]],

Cell["\<\
Flow visualization is shown below for a point vortex of unit strength located \
at the origin.  A circular region near the origin has been excluded from the \
vector plot to avoid showing vectors of large magnitude.\
\>", "ExampleText",
 CellID->78531558],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xMin", ",", "xMax", ",", "zMin", ",", "zMax"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "1.5"}], ",", "1.5", ",", 
     RowBox[{"-", "1.2"}], ",", "1.2"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"uPlot", ",", "wPlot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"u", ",", "w"}], "}"}], "/.", 
    RowBox[{"\[CapitalGamma]", "\[Rule]", "1"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Phi]Plot", ",", "\[Psi]Plot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\[Phi]", ",", "\[Psi]"}], "}"}], "/.", 
    RowBox[{"\[CapitalGamma]", "\[Rule]", "1"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gV", "=", 
   RowBox[{"VectorPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"uPlot", ",", "wPlot"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"RegionFunction", "\[Rule]", 
      RowBox[{"Function", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"x", ",", "z"}], "}"}], ",", 
        RowBox[{"0.025", "<", 
         RowBox[{
          RowBox[{"x", "^", "2"}], "+", 
          RowBox[{"z", "^", "2"}]}]}]}], "]"}]}], ",", 
     RowBox[{"VectorColorFunction", "->", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"RGBColor", "[", 
         RowBox[{"0", ",", "1", ",", "0"}], "]"}], "&"}], ")"}]}], ",", 
     RowBox[{"VectorPoints", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"14", ",", "14"}], "}"}]}], ",", 
     RowBox[{"VectorScale", "\[Rule]", "Medium"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Phi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Phi]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "13"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Red"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Psi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "7"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Blue"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gV", ",", "g\[Phi]", ",", "g\[Psi]", ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
   RowBox[{"FrameTicks", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.5"}], ",", "1.5", ",", "0.25"}], "]"}], "]"}], ",", 
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.2"}], ",", "1.2", ",", "0.2"}], "]"}], "]"}], ",", 
      "None", ",", "None"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", " ", "72"}]}], ",", 
   RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellLabel->"In[10]:=",
 CellID->1746043248],

Cell[BoxData[
 GraphicsBox[{{{}, {
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.004289048054484118, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.4926153846153847`, -1.209230769230769}, \
{-1.5073846153846153`, -1.1907692307692306`}}]}}, 
      {Arrowheads[{{0.004548525024194409, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.4929725693880864`, -1.0257660469703969`}, \
{-1.5070274306119138`, -1.0050031837988338`}}]}}, 
      {Arrowheads[{{0.004804932811407674, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.493583771055968, -0.8423540885848438}, \
{-1.5064162289440317`, -0.8191843729536177}}]}}, 
      {Arrowheads[{{0.005044529625747916, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.494499500898383, -0.6589228619254571}, \
{-1.5055004991016172`, -0.6333848303822349}}]}}, 
      {Arrowheads[{{0.0052497705738811535`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.4957448648648648`, -0.47536765072765064`}, \
{-1.504255135135135, -0.4477092723492723}}]}}, 
      {Arrowheads[{{0.005401385452458507, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.4972973220416572`, -0.29156258253076733`}, \
{-1.502702677958343, -0.2622835713153863}}]}}, 
      {Arrowheads[{{0.005482290651234551, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.4990719170007074`, -0.10738904104619727`}, \
{-1.5009280829992926`, -0.07722634356918719}}]}}, 
      {Arrowheads[{{0.005482290651234551, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.5009280829992926`, 
         0.07722634356918733}, {-1.4990719170007074`, 
         0.10738904104619741`}}]}}, 
      {Arrowheads[{{0.005401385452458507, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.5027026779583428`, 
         0.26228357131538643`}, {-1.497297322041657, 
         0.29156258253076744`}}]}}, 
      {Arrowheads[{{0.0052497705738811535`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.5042551351351352`, 
         0.4477092723492724}, {-1.495744864864865, 0.4753676507276507}}]}}, 
      {Arrowheads[{{0.005044529625747916, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.505500499101617, 
         0.6333848303822351}, {-1.4944995008983828`, 0.6589228619254573}}]}}, 
      {Arrowheads[{{0.004804932811407674, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.506416228944032, 
         0.8191843729536177}, {-1.4935837710559683`, 0.8423540885848438}}]}}, 
      {Arrowheads[{{0.004548525024194409, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.5070274306119136`, 
         1.0050031837988338`}, {-1.4929725693880862`, 
         1.0257660469703969`}}]}}, 
      {Arrowheads[{{0.004289048054484118, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.5073846153846153`, 
         1.1907692307692308`}, {-1.4926153846153847`, 
         1.2092307692307693`}}]}}, 
      {Arrowheads[{{0.0047169012218768284`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2602993675899943`, -1.209446674812358}, \
{-1.2781621708715443`, -1.1905533251876417`}}]}}, 
      {Arrowheads[{{0.005068874973481217, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2605034965034965`, -1.0262937062937063`}, \
{-1.2779580419580419`, -1.0044755244755246`}}]}}, 
      {Arrowheads[{{0.005431303348233201, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2610326669396328`, -0.8432941092695778}, \
{-1.2774288715219053`, -0.8182443522688836}}]}}, 
      {Arrowheads[{{0.0057848314074472156`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2619973746390127`, -0.6603622998162246}, \
{-1.2764641638225254`, -0.6319453924914675}}]}}, 
      {Arrowheads[{{0.006100506649715706, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2634847838293093`, -0.47733992139247605`}, \
{-1.274976754632229, -0.44573700168444685`}}]}}, 
      {Arrowheads[{{0.0063421290257965135`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.26550467267034, -0.2940010194917104}, \
{-1.2729568657911983`, -0.25984513435444323`}}]}}, 
      {Arrowheads[{{0.006474227895712772, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2679364581488881`, -0.1101044696835554}, \
{-1.27052508031265, -0.07451091493182906}}]}}, 
      {Arrowheads[{{0.006474227895712772, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2705250803126502`, 
         0.0745109149318292}, {-1.2679364581488883`, 
         0.11010446968355554`}}]}}, 
      {Arrowheads[{{0.006342129025796524, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2729568657911983`, 
         0.25984513435444334`}, {-1.26550467267034, 0.2940010194917106}}]}}, 
      {Arrowheads[{{0.006100506649715706, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.274976754632229, 
         0.44573700168444697`}, {-1.2634847838293093`, 
         0.47733992139247616`}}]}}, 
      {Arrowheads[{{0.0057848314074472156`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2764641638225256`, 
         0.6319453924914676}, {-1.261997374639013, 0.6603622998162247}}]}}, 
      {Arrowheads[{{0.005431303348233201, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2774288715219055`, 
         0.8182443522688837}, {-1.261032666939633, 0.8432941092695779}}]}}, 
      {Arrowheads[{{0.005068874973481217, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.2779580419580419`, 
         1.0044755244755244`}, {-1.2605034965034965`, 1.026293706293706}}]}}, 
      {Arrowheads[{{0.0047169012218768284`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.278162170871544, 
         1.190553325187642}, {-1.260299367589994, 1.2094466748123582`}}]}}, 
      {Arrowheads[{{0.0051917232721398155`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.027641491940075, -1.20936350179742}, \
{-1.0492815849830015`, -1.1906364982025797`}}]}}, 
      {Arrowheads[{{0.00567275165995242, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0275309653739342`, -1.0265636105878468`}, \
{-1.0493921115491425`, -1.0042056201813838`}}]}}, 
      {Arrowheads[{{0.006195291634254822, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0277948717948717`, -0.844102564102564}, \
{-1.049128205128205, -0.8174358974358974}}]}}, 
      {Arrowheads[{{0.006736282984742626, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0286530656954294`, -0.6619174630993783}, \
{-1.0482700112276473`, -0.6303902292083139}}]}}, 
      {Arrowheads[{{0.007250038641562057, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0303460745440125`, -0.4797982553528944}, \
{-1.046577002379064, -0.44327866772402846`}}]}}, 
      {Arrowheads[{{0.007665957668748521, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0330175550590488`, -0.2973380146824129}, \
{-1.043905521864028, -0.25650813916374077`}}]}}, 
      {Arrowheads[{{0.007902685346588603, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0365330720235177`, -0.1140029397354237}, \
{-1.040390004899559, -0.07061244487996075}}]}}, 
      {Arrowheads[{{0.007902685346588603, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.040390004899559, 
         0.07061244487996089}, {-1.0365330720235177`, 
         0.11400293973542384`}}]}}, 
      {Arrowheads[{{0.007665957668748521, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0439055218640279`, 
         0.2565081391637409}, {-1.0330175550590486`, 0.297338014682413}}]}}, 
      {Arrowheads[{{0.007250038641562065, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0465770023790641`, 
         0.4432786677240286}, {-1.0303460745440127`, 
         0.47979825535289455`}}]}}, 
      {Arrowheads[{{0.006736282984742626, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0482700112276473`, 
         0.630390229208314}, {-1.0286530656954294`, 0.6619174630993784}}]}}, 
      {Arrowheads[{{0.006195291634254822, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.049128205128205, 
         0.8174358974358975}, {-1.0277948717948717`, 0.8441025641025641}}]}}, 
      {Arrowheads[{{0.00567275165995242, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0493921115491425`, 
         1.0042056201813838`}, {-1.0275309653739342`, 
         1.0265636105878468`}}]}}, 
      {Arrowheads[{{0.0051917232721398155`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-1.0492815849830017`, 
         1.1906364982025799`}, {-1.0276414919400751`, 
         1.2093635017974202`}}]}}, 
      {Arrowheads[{{0.005695805863805758, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7946691465826103, -1.20876558920845}, \
{-0.8207154688020045, -1.19123441079155}}]}}, 
      {Arrowheads[{{0.006350149426125195, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7939953763414692, -1.0262799016864186`}, \
{-0.8213892390431459, -1.004489329082812}}]}}, 
      {Arrowheads[{{0.007110657655641629, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7936407408537514, -0.8444304763067159}, \
{-0.8217438745308636, -0.8171079852317455}}]}}, 
      {Arrowheads[{{0.007965374958327644, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7939780219780218, -0.6632967032967032}, \
{-0.8214065934065932, -0.629010989010989}}]}}, 
      {Arrowheads[{{0.008856650564581726, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7955815384615382, -0.48273230769230757`}, \
{-0.8198030769230766, -0.44034461538461533`}}]}}, 
      {Arrowheads[{{0.009649270221388078, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.7990670337697363, -0.30208012586390953`}, \
{-0.8163175816148788, -0.2517660279822442}}]}}, 
      {Arrowheads[{{0.010134686102653235`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8045206719147088, -0.1200595053616809}, \
{-0.8108639434699062, -0.06455587925370355}}]}}, 
      {Arrowheads[{{0.010134686102653235`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8108639434699062, 
         0.0645558792537037}, {-0.8045206719147088, 0.12005950536168104`}}]}}, 
      {Arrowheads[{{0.009649270221388078, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8163175816148787, 
         0.25176602798224423`}, {-0.7990670337697362, 0.3020801258639096}}]}}, 
      {Arrowheads[{{0.008856650564581726, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8198030769230767, 
         0.44034461538461545`}, {-0.7955815384615383, 0.4827323076923077}}]}}, 
      {Arrowheads[{{0.007965374958327644, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8214065934065932, 
         0.6290109890109891}, {-0.7939780219780218, 0.6632967032967033}}]}}, 
      {Arrowheads[{{0.0071106576556416, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8217438745308636, 
         0.8171079852317458}, {-0.7936407408537514, 0.8444304763067159}}]}}, 
      {Arrowheads[{{0.006350149426125195, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8213892390431458, 
         1.004489329082812}, {-0.7939953763414691, 1.0262799016864186`}}]}}, 
      {Arrowheads[{{0.005695805863805758, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.8207154688020046, 
         1.19123441079155}, {-0.7946691465826105, 1.20876558920845}}]}}, 
      {Arrowheads[{{0.006187843129446404, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5615526954271319, -1.2073896064884349`}, \
{-0.5922934584190214, -1.192610393511565}}]}}, 
      {Arrowheads[{{0.0070549087935796295`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5600171807887542, -1.0249902381882077`}, \
{-0.5938289730573992, -1.005778992581023}}]}}, 
      {Arrowheads[{{0.008145776882472604, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5584826812957993, -0.8435750610659511}, \
{-0.5953634725503542, -0.8179634004725103}}]}}, 
      {Arrowheads[{{0.009511317872054843, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5573687830976686, -0.663613037069389}, \
{-0.5964773707484847, -0.6286946552383031}}]}}, 
      {Arrowheads[{{0.011151524941658707`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5577230769230767, -0.4855384615384614}, \
{-0.5961230769230766, -0.4375384615384615}}]}}, 
      {Arrowheads[{{0.012874578596638543`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5615680704211261, -0.3089126738021405}, \
{-0.5922780834250272, -0.24493348004401316`}}]}}, 
      {Arrowheads[{{0.01410156059771489, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.570782671306852, -0.13068522740909624`}, \
{-0.5830634825393013, -0.05393015720628822}}]}}, 
      {Arrowheads[{{0.01410156059771489, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5830634825393013, 
         0.05393015720628836}, {-0.570782671306852, 0.13068522740909638`}}]}}, 
      {Arrowheads[{{0.012874578596638554`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5922780834250272, 
         0.24493348004401327`}, {-0.561568070421126, 0.3089126738021406}}]}}, 
      {Arrowheads[{{0.011151524941658707`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5961230769230766, 
         0.4375384615384616}, {-0.5577230769230767, 0.48553846153846153`}}]}}, 
      {Arrowheads[{{0.009511317872054843, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5964773707484847, 
         0.6286946552383033}, {-0.5573687830976686, 0.6636130370693891}}]}}, 
      {Arrowheads[{{0.008145776882472604, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.595363472550354, 
         0.8179634004725104}, {-0.5584826812957991, 0.8435750610659513}}]}}, 
      {Arrowheads[{{0.0070549087935796295`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5938289730573991, 
         1.005778992581023}, {-0.5600171807887541, 1.0249902381882077`}}]}}, 
      {Arrowheads[{{0.006187843129446404, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.5922934584190214, 
         1.192610393511565}, {-0.5615526954271319, 1.2073896064884349`}}]}}, 
      {Arrowheads[{{0.0065968484472382904`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.32868440265777216`, -1.2050392625469444`}, \
{-0.3636232896499196, -1.1949607374530558`}}]}}, 
      {Arrowheads[{{0.007680134209140217, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3261186772505604, -1.0222147866016444`}, \
{-0.36618901505713136`, -1.0085544441675862`}}]}}, 
      {Arrowheads[{{0.00915443585106048, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3228639053254435, -0.840473372781065}, \
{-0.36944378698224817`, -0.8210650887573965}}]}}, 
      {Arrowheads[{{0.011239593633760892`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3188476023480976, -0.6607821910497826}, \
{-0.37346008995959407`, -0.6315255012579095}}]}}, 
      {Arrowheads[{{0.014280919927654357`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3146658461538459, -0.4851544615384614}, \
{-0.37764184615384583`, -0.4379224615384615}}]}}, 
      {Arrowheads[{{0.018585874902764508`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3141538461538459, -0.31692307692307675`}, \
{-0.37815384615384584`, -0.23692307692307688`}}]}}, 
      {Arrowheads[{{0.022997873006245586`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.329821895946377, -0.1535525055857005}, \
{-0.3624857963613147, -0.031062879029683972`}}]}}, 
      {Arrowheads[{{0.02299787300624558, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3624857963613148, 
         0.03106287902968413}, {-0.329821895946377, 0.1535525055857006}}]}}, 
      {Arrowheads[{{0.018585874902764504`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.37815384615384584`, 
         0.23692307692307701`}, {-0.3141538461538459, 
         0.31692307692307686`}}]}}, 
      {Arrowheads[{{0.014280919927654357`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.37764184615384583`, 
         0.4379224615384616}, {-0.3146658461538459, 0.48515446153846153`}}]}}, 
      {Arrowheads[{{0.011239593633760902`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3734600899595941, 
         0.6315255012579096}, {-0.3188476023480976, 0.6607821910497828}}]}}, 
      {Arrowheads[{{0.00915443585106048, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3694437869822482, 
         0.8210650887573965}, {-0.32286390532544357`, 0.840473372781065}}]}}, 
      {Arrowheads[{{0.007680134209140217, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.36618901505713136`, 
         1.0085544441675862`}, {-0.3261186772505604, 1.0222147866016444`}}]}}, 
      {Arrowheads[{{0.0065968484472382904`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.3636232896499196, 
         1.1949607374530555`}, {-0.32868440265777216`, 
         1.2050392625469442`}}]}}, 
      {Arrowheads[{{0.00683430607684869, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.09663489021055868, -1.201802858189813}, \
{-0.1341343405586715, -1.1981971418101869`}}]}}, 
      {Arrowheads[{{0.008062271070084417, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.0933060840230649, -1.0178935394029733`}, \
{-0.13746314674616528`, -1.0128756913662573`}}]}}, 
      {Arrowheads[{{0.009823014473758204, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.08856856693646978, -0.8344936819425842}, \
{-0.1422006638327604, -0.8270447795958772}}]}}, 
      {Arrowheads[{{0.012552260161081736`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.0813277550632307, -0.6522354283540933}, \
{-0.14944147570599947`, -0.640072263953599}}]}}, 
      {Arrowheads[{{0.017318159203737356`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.06907873303167397, -0.4731149321266967}, \
{-0.1616904977375562, -0.4499619909502262}}]}}, 
      {Arrowheads[{{0.027463307553181465`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.04551479289940803, -0.3060355029585797}, \
{-0.18525443786982215`, -0.24781065088757395`}}]}}}, 
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.02746330755318146, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.18525443786982213`, 
         0.2478106508875741}, {-0.045514792899408046`, 
         0.3060355029585798}}]}}, 
      {Arrowheads[{{0.01731815920373735, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.16169049773755617`, 
         0.4499619909502263}, {-0.06907873303167399, 0.4731149321266968}}]}}, 
      {Arrowheads[{{0.012552260161081734`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.14944147570599947`, 
         0.640072263953599}, {-0.08132775506323071, 0.6522354283540933}}]}}, 
      {Arrowheads[{{0.009823014473758204, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.1422006638327604, 
         0.8270447795958773}, {-0.08856856693646978, 0.8344936819425843}}]}}, 
      {Arrowheads[{{0.008062271070084417, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.13746314674616528`, 
         1.0128756913662573`}, {-0.0933060840230649, 1.0178935394029733`}}]}}, 
      {Arrowheads[{{0.00683430607684869, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{-0.1341343405586715, 
         1.1981971418101869`}, {-0.09663489021055868, 1.201802858189813}}]}}, 
      {Arrowheads[{{0.00683430607684869, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.1341343405586721, -1.1981971418101869`}, {
         0.0966348902105593, -1.201802858189813}}]}}, 
      {Arrowheads[{{0.008062271070084417, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.1374631467461659, -1.0128756913662573`}, {
         0.0933060840230655, -1.0178935394029733`}}]}}, 
      {Arrowheads[{{0.009823014473758197, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.14220066383276098`, -0.8270447795958772}, {
         0.0885685669364704, -0.8344936819425842}}]}}, 
      {Arrowheads[{{0.012552260161081737`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.14944147570600008`, -0.6400722639535988}, {
         0.08132775506323132, -0.6522354283540933}}]}}, 
      {Arrowheads[{{0.017318159203737353`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.16169049773755678`, -0.44996199095022615`}, {
         0.06907873303167461, -0.47311493212669675`}}]}}, 
      {Arrowheads[{{0.027463307553181448`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.18525443786982265`, -0.24781065088757384`}, {
         0.04551479289940874, -0.30603550295857984`}}]}}}, 
     {Hue[0.67, 0.6, 0.6], 
      {Arrowheads[{{0.02746330755318143, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.04551479289940877, 0.3060355029585799}, {
         0.18525443786982262`, 0.24781065088757398`}}]}}, 
      {Arrowheads[{{0.01731815920373735, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.06907873303167462, 0.47311493212669686`}, {
         0.16169049773755678`, 0.44996199095022626`}}]}}, 
      {Arrowheads[{{0.012552260161081729`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.08132775506323134, 0.6522354283540934}, {
         0.14944147570600005`, 0.640072263953599}}]}}, 
      {Arrowheads[{{0.009823014473758204, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.08856856693647039, 0.8344936819425843}, {
         0.142200663832761, 0.8270447795958773}}]}}, 
      {Arrowheads[{{0.008062271070084417, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.0933060840230655, 1.0178935394029733`}, {
         0.1374631467461659, 1.0128756913662573`}}]}}, 
      {Arrowheads[{{0.00683430607684869, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.0966348902105593, 1.201802858189813}, {
         0.1341343405586721, 1.1981971418101869`}}]}}, 
      {Arrowheads[{{0.0065968484472382904`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3636232896499202, -1.1949607374530555`}, {
         0.32868440265777277`, -1.2050392625469442`}}]}}, 
      {Arrowheads[{{0.007680134209140217, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.366189015057132, -1.0085544441675862`}, {
         0.326118677250561, -1.0222147866016444`}}]}}, 
      {Arrowheads[{{0.009154435851060487, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3694437869822488, -0.8210650887573964}, {
         0.3228639053254441, -0.840473372781065}}]}}, 
      {Arrowheads[{{0.011239593633760892`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3734600899595947, -0.6315255012579095}, {
         0.3188476023480982, -0.6607821910497826}}]}}, 
      {Arrowheads[{{0.014280919927654349`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3776418461538464, -0.4379224615384615}, {
         0.3146658461538465, -0.4851544615384614}}]}}, 
      {Arrowheads[{{0.01858587490276449, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3781538461538464, -0.2369230769230769}, {
         0.31415384615384656`, -0.31692307692307675`}}]}}, 
      {Arrowheads[{{0.022997873006245544`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3624857963613153, -0.03106287902968407}, {
         0.3298218959463777, -0.1535525055857004}}]}}, 
      {Arrowheads[{{0.022997873006245544`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3298218959463776, 0.15355250558570052`}, {
         0.3624857963613153, 0.031062879029684222`}}]}}, 
      {Arrowheads[{{0.01858587490276449, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.31415384615384656`, 0.31692307692307686`}, {
         0.3781538461538464, 0.23692307692307701`}}]}}, 
      {Arrowheads[{{0.014280919927654344`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3146658461538466, 0.48515446153846153`}, {
         0.3776418461538464, 0.4379224615384616}}]}}, 
      {Arrowheads[{{0.011239593633760883`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3188476023480983, 0.6607821910497828}, {
         0.3734600899595947, 0.6315255012579096}}]}}, 
      {Arrowheads[{{0.00915443585106048, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.3228639053254442, 0.8404733727810652}, {
         0.36944378698224883`, 0.8210650887573966}}]}}, 
      {Arrowheads[{{0.007680134209140217, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.326118677250561, 1.0222147866016444`}, {
         0.366189015057132, 1.0085544441675862`}}]}}, 
      {Arrowheads[{{0.0065968484472382904`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.32868440265777277`, 1.2050392625469444`}, {
         0.3636232896499202, 1.1949607374530558`}}]}}, 
      {Arrowheads[{{0.006187843129446422, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.592293458419022, -1.192610393511565}, {
         0.5615526954271326, -1.207389606488435}}]}}, 
      {Arrowheads[{{0.007054908793579613, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5938289730573998, -1.005778992581023}, {
         0.5600171807887548, -1.0249902381882077`}}]}}, 
      {Arrowheads[{{0.008145776882472588, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5953634725503547, -0.8179634004725103}, {
         0.5584826812957999, -0.8435750610659511}}]}}, 
      {Arrowheads[{{0.00951131787205483, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5964773707484854, -0.6286946552383031}, {
         0.5573687830976694, -0.663613037069389}}]}}, 
      {Arrowheads[{{0.011151524941658695`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5961230769230773, -0.4375384615384615}, {
         0.5577230769230774, -0.4855384615384614}}]}}, 
      {Arrowheads[{{0.012874578596638543`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5922780834250279, -0.24493348004401316`}, {
         0.5615680704211268, -0.3089126738021405}}]}}, 
      {Arrowheads[{{0.014101560597714878`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.583063482539302, -0.05393015720628826}, {
         0.5707826713068527, -0.13068522740909622`}}]}}, 
      {Arrowheads[{{0.014101560597714873`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5707826713068527, 0.13068522740909633`}, {
         0.583063482539302, 0.053930157206288395`}}]}}, 
      {Arrowheads[{{0.012874578596638543`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5615680704211268, 0.3089126738021406}, {
         0.5922780834250279, 0.24493348004401327`}}]}}, 
      {Arrowheads[{{0.011151524941658695`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5577230769230773, 0.48553846153846153`}, {
         0.5961230769230772, 0.4375384615384616}}]}}, 
      {Arrowheads[{{0.009511317872054843, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5573687830976692, 0.6636130370693891}, {
         0.5964773707484854, 0.6286946552383033}}]}}, 
      {Arrowheads[{{0.008145776882472588, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5584826812957999, 0.8435750610659513}, {
         0.5953634725503547, 0.8179634004725104}}]}}, 
      {Arrowheads[{{0.007054908793579613, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5600171807887548, 1.0249902381882077`}, {
         0.5938289730573998, 1.005778992581023}}]}}, 
      {Arrowheads[{{0.006187843129446422, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.5615526954271326, 1.2073896064884349`}, {
         0.592293458419022, 1.1926103935115648`}}]}}, 
      {Arrowheads[{{0.005695805863805758, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8207154688020053, -1.19123441079155}, {
         0.7946691465826111, -1.20876558920845}}]}}, 
      {Arrowheads[{{0.006350149426125195, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8213892390431464, -1.004489329082812}, {
         0.7939953763414698, -1.0262799016864186`}}]}}, 
      {Arrowheads[{{0.0071106576556416, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8217438745308643, -0.8171079852317455}, {
         0.7936407408537521, -0.8444304763067157}}]}}, 
      {Arrowheads[{{0.007965374958327644, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8214065934065938, -0.629010989010989}, {
         0.7939780219780225, -0.6632967032967032}}]}}, 
      {Arrowheads[{{0.008856650564581726, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8198030769230773, -0.44034461538461533`}, {
         0.7955815384615389, -0.48273230769230757`}}]}}, 
      {Arrowheads[{{0.009649270221388069, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8163175816148793, -0.2517660279822442}, {
         0.799067033769737, -0.30208012586390953`}}]}}, 
      {Arrowheads[{{0.010134686102653225`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8108639434699069, -0.06455587925370357}, {
         0.8045206719147094, -0.12005950536168086`}}]}}, 
      {Arrowheads[{{0.010134686102653232`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.8045206719147094, 0.12005950536168103`}, {
         0.8108639434699069, 0.06455587925370371}}]}}, 
      {Arrowheads[{{0.009649270221388069, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.799067033769737, 0.3020801258639096}, {
         0.8163175816148793, 0.25176602798224423`}}]}}, 
      {Arrowheads[{{0.008856650564581726, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.795581538461539, 0.4827323076923077}, {
         0.8198030769230774, 0.44034461538461545`}}]}}, 
      {Arrowheads[{{0.007965374958327644, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7939780219780225, 0.6632967032967033}, {
         0.8214065934065938, 0.6290109890109891}}]}}, 
      {Arrowheads[{{0.007110657655641614, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.793640740853752, 0.844430476306716}, {0.8217438745308643,
          0.8171079852317458}}]}}, 
      {Arrowheads[{{0.006350149426125195, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.7939953763414699, 1.0262799016864186`}, {
         0.8213892390431465, 1.004489329082812}}]}}, 
      {Arrowheads[{{0.005695805863805758, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{0.794669146582611, 1.20876558920845}, {0.8207154688020052, 
         1.19123441079155}}]}}, 
      {Arrowheads[{{0.005191723272139785, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0492815849830022`, -1.1906364982025799`}, {
         1.0276414919400758`, -1.2093635017974202`}}]}}, 
      {Arrowheads[{{0.00567275165995242, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0493921115491431`, -1.0042056201813838`}, {
         1.0275309653739348`, -1.0265636105878468`}}]}}, 
      {Arrowheads[{{0.006195291634254822, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0491282051282056`, -0.8174358974358974}, {
         1.0277948717948724`, -0.844102564102564}}]}}, 
      {Arrowheads[{{0.006736282984742626, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.048270011227648, -0.6303902292083139}, {
         1.02865306569543, -0.6619174630993783}}]}}, 
      {Arrowheads[{{0.007250038641562057, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0465770023790648`, -0.4432786677240285}, {
         1.0303460745440134`, -0.47979825535289444`}}]}}, 
      {Arrowheads[{{0.007665957668748521, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0439055218640285`, -0.25650813916374077`}, {
         1.0330175550590492`, -0.2973380146824129}}]}}, 
      {Arrowheads[{{0.007902685346588598, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0403900048995596`, -0.07061244487996077}, {
         1.0365330720235184`, -0.11400293973542369`}}]}}, 
      {Arrowheads[{{0.007902685346588598, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0365330720235184`, 0.11400293973542383`}, {
         1.0403900048995596`, 0.0706124448799609}}]}}, 
      {Arrowheads[{{0.007665957668748521, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0330175550590495`, 0.297338014682413}, {
         1.0439055218640287`, 0.2565081391637409}}]}}, 
      {Arrowheads[{{0.007250038641562057, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0303460745440132`, 0.4797982553528945}, {
         1.0465770023790646`, 0.4432786677240286}}]}}, 
      {Arrowheads[{{0.006736282984742626, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.02865306569543, 0.6619174630993784}, {1.048270011227648, 
         0.630390229208314}}]}}, 
      {Arrowheads[{{0.006195291634254822, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0277948717948724`, 0.8441025641025641}, {
         1.0491282051282056`, 0.8174358974358975}}]}}, 
      {Arrowheads[{{0.00567275165995242, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0275309653739348`, 1.0265636105878468`}, {
         1.0493921115491431`, 1.0042056201813838`}}]}}, 
      {Arrowheads[{{0.005191723272139785, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.0276414919400758`, 1.20936350179742}, {
         1.0492815849830022`, 1.1906364982025797`}}]}}, 
      {Arrowheads[{{0.0047169012218768284`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2781621708715447`, -1.190553325187642}, {
         1.2602993675899947`, -1.2094466748123582`}}]}}, 
      {Arrowheads[{{0.005068874973481217, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2779580419580425`, -1.0044755244755244`}, {
         1.2605034965034971`, -1.026293706293706}}]}}, 
      {Arrowheads[{{0.005431303348233201, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2774288715219062`, -0.8182443522688836}, {
         1.2610326669396337`, -0.8432941092695778}}]}}, 
      {Arrowheads[{{0.0057848314074472156`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2764641638225263`, -0.6319453924914675}, {
         1.2619973746390136`, -0.6603622998162246}}]}}, 
      {Arrowheads[{{0.006100506649715696, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2749767546322297`, -0.4457370016844469}, {
         1.26348478382931, -0.47733992139247605`}}]}}, 
      {Arrowheads[{{0.0063421290257965135`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.272956865791199, -0.25984513435444323`}, {
         1.2655046726703407`, -0.2940010194917104}}]}}, 
      {Arrowheads[{{0.006474227895712772, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2705250803126509`, -0.07451091493182907}, {
         1.267936458148889, -0.11010446968355542`}}]}}, 
      {Arrowheads[{{0.006474227895712767, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2679364581488888`, 0.11010446968355553`}, {
         1.2705250803126507`, 0.07451091493182921}}]}}, 
      {Arrowheads[{{0.0063421290257965135`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2655046726703407`, 0.29400101949171054`}, {
         1.272956865791199, 0.25984513435444334`}}]}}, 
      {Arrowheads[{{0.006100506649715696, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.26348478382931, 0.4773399213924761}, {
         1.2749767546322297`, 0.44573700168444697`}}]}}, 
      {Arrowheads[{{0.0057848314074472156`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2619973746390134`, 0.6603622998162247}, {
         1.276464163822526, 0.6319453924914676}}]}}, 
      {Arrowheads[{{0.005431303348233201, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2610326669396335`, 0.8432941092695779}, {
         1.277428871521906, 0.8182443522688837}}]}}, 
      {Arrowheads[{{0.005068874973481217, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.2605034965034971`, 1.0262937062937063`}, {
         1.2779580419580425`, 1.0044755244755246`}}]}}, 
      {Arrowheads[{{0.0047169012218768284`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.260299367589995, 1.209446674812358}, {1.278162170871545, 
         1.1905533251876417`}}]}}, 
      {Arrowheads[{{0.004289048054484118, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.507384615384616, -1.1907692307692308`}, {
         1.4926153846153853`, -1.2092307692307693`}}]}}, 
      {Arrowheads[{{0.004548525024194409, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5070274306119142`, -1.0050031837988338`}, {
         1.4929725693880869`, -1.0257660469703969`}}]}}, 
      {Arrowheads[{{0.004804932811407674, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5064162289440326`, -0.8191843729536176}, {
         1.493583771055969, -0.8423540885848437}}]}}, 
      {Arrowheads[{{0.005044529625747916, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5055004991016177`, -0.633384830382235}, {
         1.4944995008983835`, -0.6589228619254572}}]}}, 
      {Arrowheads[{{0.0052497705738811535`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5042551351351359`, -0.44770927234927227`}, {
         1.4957448648648657`, -0.4753676507276506}}]}}, 
      {Arrowheads[{{0.005401385452458507, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5027026779583434`, -0.2622835713153863}, {
         1.4972973220416577`, -0.29156258253076733`}}]}}, 
      {Arrowheads[{{0.005482290651234545, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.5009280829992933`, -0.0772263435691872}, {
         1.499071917000708, -0.10738904104619726`}}]}}, 
      {Arrowheads[{{0.005482290651234545, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.499071917000708, 0.1073890410461974}, {
         1.5009280829992933`, 0.07722634356918734}}]}}, 
      {Arrowheads[{{0.005401385452458507, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4972973220416579`, 0.29156258253076744`}, {
         1.5027026779583437`, 0.26228357131538643`}}]}}, 
      {Arrowheads[{{0.0052497705738811535`, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4957448648648655`, 0.47536765072765075`}, {
         1.5042551351351356`, 0.44770927234927244`}}]}}, 
      {Arrowheads[{{0.005044529625747898, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4944995008983837`, 0.6589228619254572}, {
         1.5055004991016179`, 0.6333848303822351}}]}}, 
      {Arrowheads[{{0.004804932811407674, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4935837710559687`, 0.842354088584844}, {
         1.5064162289440324`, 0.8191843729536178}}]}}, 
      {Arrowheads[{{0.004548525024194409, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.492972569388087, 1.0257660469703969`}, {
         1.5070274306119145`, 1.0050031837988338`}}]}}, 
      {Arrowheads[{{0.004289048054484118, 1.}}], 
       {RGBColor[0, 1, 0], 
        ArrowBox[{{1.4926153846153853`, 1.209230769230769}, {
         1.507384615384616, 1.1907692307692306`}}]}}}}}, 
   GraphicsComplexBox[CompressedData["
1:eJx0nXdcU9f//6N1r1q1auuoWuves64eR91714FbnFiLiqKiEUHc4izWRRUV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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV021IV2cch/G7NejFVtlysywtoYwaBStrqUwbZIuxbGq6nqjQrcbaZqPF
HOQooUE+NGhjghPcyFiGQcG/QcEEbdOVYQOF2XRLm4bJ7MGoDRmrz3nx9TrX
/bu/t+cczj+luDT/w0khhDx/lj8fwoOkEFbgS/K71M8J4fWpIfS9EMJGTJkd
widxIczHMvzDnsTkEPrxhj1/4mR+Awf5gBTofeecV6wd1KmaFUIuz5rmTByy
56bZm67zJS+a8WGZ46whHLHnttxyfcJ8szNH+S48zXOtVzj7K2cX8E1SJIXy
d9Q1f8v1KG7GjThmvUS/OdrDN0Vrcoa/bf2u+R7M8KxVzk7HaryfGMK43JUt
9t/DJvxML0mn1T20yTbPNy8hhO1Ygoui57T+lwxJlrWE6c7Wq5ZKWWVPDaZh
jvlV88t8Jf83PoSf9X6SS3LevaSaNfrfscilw3V7tF9vqf4q7OC/WN/hjCU8
w74rPB07MRMbMBvXyHa9d3CvZ/8IT+r9Z96KMbnujEF77shF6ZEOec9zPuVd
TJeH3tVinCvx9vfxEYlJl7TKhPPXmRdIunwgu2SB/Q8966BvcQQfO7cLe3gf
FnvuVuzgMRxzziE8w2uwU/97vMjrcFx/N37L92Oh/hb8kr+BA/qz8FO+EFv0
s/FzvhyH9f+fGcL7/Bm+PvoG+E4+hN36ZzGH/4jn9H/DfH4Fe/W/xtX8BGbq
H8UX+UFs11+PM3ghNuqXYjIvxk79Zfg0z8Rl+in4z1zvFi/oDzznt8jHsFZ/
ivVRPsFb9NvxV96NKfoX8BJvxmb9MmziR7BSvwF/4MfxnP4OrOd7MV4/D7/g
a7FBPw4P8CQs138ZK/gSbNR/NCOEd/lkPkX/Nt/G+/G4/il8lZ/HffrXMJe3
Ya3+MUzj3+CE+WFcxD/GI/prcCrfgCXmezCRb8VK/VQMfAWOmc/GB34Xz2KZ
fq97us6Hscj8Md7i41iuH+f7v+P9J+Fr0fflO43JIamTGmmTJ+DQ0nY=
          "]]},
        "0.42600000000000005`"],
       Annotation[#, 0.42600000000000005`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1G9MlWUch/Fba74IEgtNETRY2QLxBZmFvtBcERq4JaMZIDZRa5FTJ220
qUM22qTAzRZs1KIt/LfZFvPUpptsWOKmDnHQBFEnlQ4UdJJkoZP8PC++Xud6
7t/3d55zzoNpZVsLtkwKIRT4p3tGCD3PhZCGK5JD+B3fxh7MjdzZqdkhdEh8
Ugjl00KIw0/woplGM8ufDqHPXA428V7X0+WSa304ea7XmCH5zlfJSsmTd6Tf
3Dn7k+1NkdlSYf9l899Fc/ZeMXNV/nDtSfsG8E+ejWNzQliMu3TqZoVQL3ul
UO+GmSLcb89g9JlkSA7wYteHna/ga/kwFmFJdCZ3nF1xX1dlkXvaY/+rWIu3
XBuWDd5no2yy6x4vtqM06uLX2CCvzAwhS+7aVzw1hDKv53tdi1/Il7JcFib4
PuzKk5WSaSYf5+Lz8ptdpfoPzP47PYRx/E82Ssw9zXPe4n2P6sSkS+eRuUzn
E3iBV3ud7n2mY7aZGdisswZLJNf+cp9lm7P95h86a8dY9FvqDZi5LcelR07L
xzqTfPapMub7eTn6LSXRfD8flJh0SruM25/jvECyZbN8IC+YH3OfA37PQZyw
txN7eD+u9xnb8TSP4Yg9Vfgjr8Oz+gfxOG/CUf1N+D3fjoX67+NXPA+v6c/E
z/iL2Ka/FD/nWXhd/1GiZ50/xXP17/J1/C/s1v8J3+InsFX/Iq7mZ7BXvwFf
5z/gEv09mMF3YId+Lk7jhdiivwXn8PV4Vn8BPsGX4AL9VLyf4rvFY/rXnvWM
8hFs1J/i+k0+ztv0O7CLd2Oq/jH8lR/BI/qVeJjXYK1+M/7C92Grfil+y8sx
Uf9d3MvfxGb9BPyUp+BO/dewmqdji/4/z4TwEZ/Mp+gP8WJ+GffpH8I3+M+4
Vf88ruInsVG/Hhfyb3Dc+W58iVdgjf4yjOf5WOb8Q0ziRVirPw//j/4GccT5
LPybx2Glfq976uPX8T3nE3iDj+JO/QTP/23ff0r0/2L0fHlOY1IlTVInJ+Ux
VAfQ5Q==
          "]]},
        "0.35500000000000004`"],
       Annotation[#, 0.35500000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1HtszWccx/EviT+QnnLq2hZBKVvdJiMS2+LWGJJ1btsEQRCX0biE0LFO
hWJLSoZaWLIuSEhISkIwJUFYbFklbYNok6EklbRiSxr31/nj0/fv/X2ez9Nz
np7TvosKp69uExHj/RjePaKmW8T8HhHzZK7UdI2oM6uVG5kRiZ4R6ZImnaWT
/GmeY1+G5287RSRxFXbBD82f94rIw0KzYmf+INtkclpEL3vWm+/gO6VUppn3
M+8vu/huGZuIKDB/6HXMwoosz858whulzmsYYv9QyZNiZw7Dz+3pqJ/oHVFi
NtJsOx72Xgus1et9iQ34IN37MT/Aezr/IGbiPzgJq7HM+l7ZJ5my2bxINskW
+U5edolY47xX+M6ex37/QGcnPZc7I8ueQ5iNzTjY+h9YYr3Zc0tqv+dcqTL/
gF/GcfbnOmcgv8BzcJDkymGexAyZrTdH8t3ZcndVaPa7O3ppTxVWpu7LOQ32
PJPzckeuyzKd8NrT5IW7ysVMSdp/lzdKpdyWKml1/sTUHcpoWSnzpZ/9L7z/
Bn/7Rnzj3Nt4h9/FBf4GVXidV2KTc7biSb4Hb+kfxfO8HFv0F+OvfA3O0P8K
9/IpWK/fDTfy/nhJ/xPczofjQ/3XGREreHuer9/M5/F/sVr/FE7gF/G0fg0W
8JtYq/8zjuK/4Rj9nTiYb8Zr+vmYzmdghf4qzOYL8JZ+HrblYzBPvw/+n+1u
8Zx+fTLiPm/C/frtzJ/yVn5J/xr+zauxj/45vMpP4An9DXicl2Cp/hE8y8vw
tP5c/IUvx6T+F/gTH49H9BO4jmdhkf7HWMwHYYX+f50jlvI2vJ3+E/4Nv4dl
+sfwM34GV+v/hdP4Fdyv/yN+xA9hq/XvcQBfiyX6n2JHPhUXWl+CPfjXWKqf
g299B0Zgk/Xu+Jx3wA36tV5TXer/As60/gYf8RYs0k/4/D9z/1mp73Pq8+Vz
WilbpVz2yBV5D9COvOU=
          "]]},
        "0.28400000000000003`"],
       Annotation[#, 0.28400000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1Hloz3Ecx/HPNGeGOTY2JgzTJpZhys3kKMfUEDJXjsxVUxTKsEI5a8SK
haLIj6LQ5qYQCo0iR6jNFTIy8fj+8dyz1/vzeX2+v9/n9/uty7wV+cvjQgh5
/gxPCKFhhxAaIR5N0Bi3U0JYlBTCSOtlqSF0bxfCWDm3RQijzNrZk4QN7UPY
iEHmY8yTzVa2CqHEbDO2YJz5QWf0dUaxtTR71nBnzjHrzmvldO6JHnjg+cs8
rwjD7amWh/EwzylwXl97srHb+XswwnymeXmyZzprMk9BPuZZf6X/GnPlQhTY
n8jTOGb9DKpQiRlmV/iy59VwLT4i1f4Ks67eSzq6ob6ti7TWAHHo3TKENvZl
8Wf5E+rwEzlmv3iH9ZDmGc7Kiu4pes3IQC9cM880v8q9OUOvP9+M7ooHYgDK
5X7RudFr1svmPih1Z7cQ532U8XKzCv5tfyXH8NSZL3VqcQGPcAOLdP651+bu
6lv0WXAHJNpfLb9DDHdRiTp3Psr6ZAzAUsxGF/u/uZuXnXS43rl3+ZFczXN8
RpV8Q45xjXPW80l5G9/RP8oX5DL+oj+fy+WVnK9fwLvkcfwi+k7yGrkrX9If
zCVyH36j/6dNCEvkJnKe/md5lvyaH+qf4pHyRT6t/5gnybf5if5e7i8f5lz9
rZwhr+Xr+nncQs7nI/rLOFWew3f0MzlOzuXM6HfAPzq6Wz6v/6J1CM/lGt6n
H2/+Qa6TL+lf5/vyQ07TP89X5BN8Qr+Yj8ubuFT/EJ+Td/Lp6PfBB+TFnKg/
kXfII/iQfgKvllN4nX4Ob5R78hH974khLJSDHK//Xp4uP+Od+sd4qHyWi/Tv
8QS5ivfpb+dseT/XWd/A6fIq3qQ/hJvJ47nQ+gJOlqdxqX43/hv9H+Ea60n8
VW7KxfpPvKan8huear2e38pfeJ1+gu9/rftP4dHR98v3NIb1KMM2VOE/J824
9w==
          "]]},
        "0.21300000000000002`"],
       Annotation[#, 0.21300000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1HtsT3cYx/FH/DGspXOpFhWX0Eayn5pVE5fFpcm0orR2i1IZhhrKQoKY
hMQvQYJM0skIlm0uySSdrF3WpJaQjWFRQ5CQuNSSiqFEI8jsdf749H0+z/N8
nvM9pye/wfNrK1d0ioiZ/gzpE/EoL2IoLsmK6JobUYPd8M9+EWdoSXZESWbE
nv4Rw82V8gX4WC4fa/gVMinXT9TexrHdI96X+UZmJL9KP8fOLzAX/7L3PC2V
nW5uv7kic2X8NCpXO6A2Rm29zEWz41y34FicYH+FmQK7NurvzInYRZP1njnD
JKy1Z46ZQ/aU8eX8DJqrdlhtmlpa9q6drXSHKtWm2P1pcqa+zkCz6APKtL97
cg87ZuMD8xk4ha/CquRc+B6Np3nuUU3t+lN7uL9aKnleHIlj7CyiYnqH7zIw
YjQWJu9brVTmjPwovqN3xGK1RfQZ/ezc+XoF1GjfL3TR3GtzKf1wlr/5tOuU
PdnYl06pf4KFyVkobeZ3Cs9ch8vVvsXn5pqxnq7I35S5T43UkuxJziLzn/f/
pvfQjsMwh7LMX+VbqZ7OUjN1eKeT9WdQEdXQHBpkvt25b/q/teJLe89iC38V
qz1rM57i67HNng14lN+Kp+W/w0a+Dh/Kz8d9fC1WyH+IO/mpeEO+N67hB2OT
/DjczKfwtvyLXr57/g2+RP5fvoq/hRfkf8RJ/K94TP4SlvN/4GX5r/Bd/gAW
y2/BfH4tnpQvwUy+Ag/Kf479+Go8LT8Cgy/GEfJ5+HSAd4sN8jd6Rlzn23C3
fGf1f/gOvkn+JJ7nL2CefAP+xh/BI/Kr8Qd+E6bl9+Jxfgcek5+Ne/jFmCU/
HbfzE3GvfAau4nNxnfxo3MgPx4PyT96KWMi/xs7y9/Bj/hrukP8eJ/A/4TL5
c1jGn8Dd8tuwkP8aO/S/xKH8StwkPx678qU4T38BZvMfYVp+CL5KfnuwTb8P
PuK74Gr5y850hb+Nlfov8Q7/ENfJZ/j+73v/ucnvQfJ9+U7raQPV0VY6Qf8D
Sfu/9g==
          "]]},
        "0.14200000000000002`"],
       Annotation[#, 0.14200000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1G1olmUYxvFTP1ik9cymOW1aRgZLSpjhRActp5W1xXzZpoZaE01SttWE
BIsUGi5rhsoKJrSphUrCChQUZjDRyXrZwK2lW5Si5gZO1FjhfOt3f/j7f47r
Oo/rvrbnnpPLKhaWD4uIZf7587GI2Y9EvJ0R8RZWImdsRPuEiE1pEevsL3w4
ovHxiFzrBfJv9rpxBl3oRLn1UnPPjo+Yiix8rD9XJx95nlFo5mWf/50YMY8r
5ZU6B51dKPc6Z5fnb9HL1u+Rz2GO7gpzB8wVmNtq/z3dMmvfWVtkrUjuM9uP
y7iCv/G+9TXmcpw3EzOwTb9Epxhznb3AzBKfB8yX8jWusrZW7/VxEa8hzb1S
WIKlqHXGHuuNyDP7D25invOKre12zkR3q+dJ/EvyLO5JRfzEM83moMRZpViE
YixGrn4mes1+wdvxq34Hhk3y3XCFO1bitPlWNPi5jrpTmv1zcg++tdZibbS1
W/JQ8t06c5rz7vp8B7fRlnxH7rTVeg0+c6/PcdJ6DWebqU4y7vud1PF6a438
n5lmbkKXs//Q78cRdOAEVvud3HWXh9z3Oj/N45Ay3y1fRBPa0IxBz3jJ/ht4
AWvxJp4wf32MZ3h/LvKQc9u4Q+7m5X7WZj4hN3GfczbxAbmGW/X38hG5jgeS
9553y+VcpL+Yt8uvcK9+Om+Qn+Rj+rN4i/wcn9e/lR7xjjxCzte/Ki+T/+L2
5P3kPPkoH9I/w4XyKe7U38HT5a95hv4n/Iz8Abfo5/MouYgb9N/l8fJybtXP
4vuZupyln8k35RQf1u99NOKs3Mc79YdbvywPysf0W/hnuZ0z9Q/zj/J+3q9f
xd/Im7lav55/kGv5kP5S/kpewyn9At4mv8j1+iO5Us7gjfrZ/JE8hRv0b4yO
WCXf4+H6l7hE/p1r9fdxrvw9r0v+dni+fJx36n/K0+QvedD+h/yUXMGb9Wfz
g/KrvMJ+GY+Vi7lafzLf9u4/z332x/A1+QGu0u90py75PC+wP8QX5AHeqD/S
+9/v+8jgOcn75T1tSv7vRB1qcBz/A9Vw1Fw=
          "]]},
        "0.07100000000000001`"],
       Annotation[#, 0.07100000000000001, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV03loz3Ecx/HPHMNmcxvmDLU/mGMIc86xOYcZxjBz5S6KQrkmR1PUKAqZ
q6ghc7UpCzVnIRSFUFNojiXEeOyP5+/Z6/35vL7fz+/3+3475a6asjIihNDH
R0xsCItah7AY87EQC9DQfB7nYg5yMBfR5tk8G1mYhZmIMp/OMzAV05CJBuZT
OAPpmIxJqG8+gSdiLMZjHOqZp/EYjEIqRiPSfASPxDCkYDjqmg/hoUjGYAxC
HfMTcSGcRCGOI6q974Vo1LbePT6ERHRDT/TAcvNLLUL41y6Ey1yMZPNB6I+B
GIBa9m3lbdiC7cjDBfv7cj/0Rh8kYZn95a1CuINvzlKF77jWOIR7LUO4j90o
x01Ux4RQwMvsP8o/XLeEi/CkUQgvdT+gGA9RhgXu8afmN28TQiV35paItf+p
/BZFKEcJqtxjmPUJSMJizER7+yubu4ff4C3/qjk7P5SfcrYzl3CZXMQVrrOe
T8s7+Lb+MS6WC/iTfg4fkldwun4G75FH8wv9prxG7sBX9QfwZrkbv9b/2cxz
KteVU/Q/ylnyK36gf4aHylf4rP4jHi/f4sf6e7m3fJj76m/jrvJavqGfwtFy
Oh/RX8Kt5Gy+rZ/A1W11OUE/nr/KsXxR/0XTEJ7LFbxPP8L8vVwlX9W/wXfl
Bxyvf5Gvy6f4lP5qPi5v4jz9g3xezuez+jP4gLyQY/XH8S55CB/Uj+JVchyv
0+/FG+UufET/SxPvsvyXI/Tfcab8jPP1CzlZPsdL9e9wmlzK+/R3cqK8n6us
b+BO8krepD+Q68mpPNv6PG4uT+U8/Y7823vQnSusN+PPciSv1n/sTE/k1zzJ
+i9+I3/idfpRnv8P/o84Hl7zfHlOi7AeBdiBUvwHqf6upA==
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV03loz3Ecx/HPZGP4scOG2W+OKX84Qv5AOea+QqMQwhw5tmmOOeYoK0fO
GuUOZbLa1MRqFDkaRSEUZUJNIdScmePx++P5e35fn8/n9f5++/2+v255q3IL
40II03z0bRtCyw4hbOkYwmaUIDU9hMLOIZSmhdDCXrKclRVCCpdYy7dXgJVo
ay3emRWup5q1nM+acTophE3OXpazrQdn4tDG+QSzWvM/JCIqt+ITOkWdQjjO
qzlLr8l6Blebk+A638zf3ByZevG8yNoS+0uxGM2s/Yqtu56gm8fHPFOZuXnO
Vskp1r858x1xWGgt8FeMdHYUcjAGo3FAt9zz7+fz3NlaxP2TzMl0HcU/sxeY
85e3cQ+zmrULYR7nxL5DvSiGYhQG25tvfbzrcajSy5bv6Baa+9taEwqQj5W4
5v4VXO5smnue53SuR5p53extwHoUoyY2094a12uxz/z9uGfObq7DTfyJ+H54
hbOn+JteLVfisbkvdN/hEh7gBhbFntHv1DIjhE/cndMQcf6J/BqVqEMtGt1j
uP3JGIClmI2o85/au0dUh3+aW8cP5Cc8x7PW8g25khvM2cjl8g6+rX+aL8ll
/EF/Ph+V83mKfi7vlcfwc/1kXi1ncY3+IN4m9+J6/R+p3im5uZyj/16eJb/k
+/oXeJh8hSv0H/Ik+RY/0j/A/eUTPFB/O/eQ1/F1/RxuJU/hk/rLuIM8h2/r
9+Q/mbrcUz+Dv8gRrtZ/nhLCM7mBD+oH62/lRrlG/zrfk+9zhn41X5PP8Tn9
Ij4rb+VS/SN8Ud7DFfoz+bC8mCP6E3mXPJSP6CdyoZzOxfr9uETO5pP6n5P9
p+QmDvpveIb8lPfon+EhchUv17/L4+SrfFB/J/eRD3Gj/U3cVS7grfqDOUEe
y3PtL+BUeTqX6nfhX/5HvbnBfgp/lOO5SP+RZ3os1/NU+z/5lfyBi/UTvf/v
/B7pPCL2fnlPK7ERZdiBq/gPDwS+FA==
          "]]},
        RowBox[{"-", "0.07100000000000001`"}]],
       Annotation[#, -0.07100000000000001, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV03tI1mcYxvEnqqnDPM3Xw7KajcVygy0WlDUPf3Sghls12MLBiixqa3kY
uSywLUEFBxlW2JEUHKyygcIElVkktJHClBYUVFhgsEYLZEznau3z++N6v8/1
3Pd1/57nfX9v7raKTeUzQgilPpYlhXB1bgjHYyF0p4RwMjuELjyFGfZjlJwZ
wle4j6ppbkYINXhEJss6bn4ImThHX7O9DOsr6kvNftHeEhyIZtm/aHaD2Rew
EZPtJ1GCvkqsogpK1/sl1pn3glqeGf18iv1D9pKxj19sf5b6QmzPCqGNzkUy
v0rf1Dxz8b/onDTbWeNxt71pzJbrMmem9ef2dlrvov1mHKAs9RY+FU/yJ6iV
cvSv9NwVVOZZBXyC2YW4gx+WKbLebj1k/YrZA9Zt7txOI2aMUoW5gy87I76u
vyG6ExabW0SvJYewQd8Kex/gStyI+bhG/Y76Vut/0t2Rf0bl9JNn5Zn1BvXp
76ci62KqVT9IX0ffG91XG6NvrHudsxrjzC2Lzk/XzVobkZ7O8X3gZ/rP4l/6
e7GTRmVuyz6kbhqmy7TN3abdOc49H2MuplOi/hv8GHXSNeqlCc8oVH+PltAO
2kw5+h+7522/6RhOmnsNh/kbWOqcvXiZ78Rxc2rwO74eB+XPYTffgo/kP8UT
/G4skd+I3/Kr8JZ8Clbx87BHfhke5PPwrvzfL/mO+Jl8sfzv/Mf8HRyS/x4L
+B/xvPyvuJ6/iiPyh/Ft/jS+I38IX+X34oB8MSbwJXhGfidm8KU4KL8In+bI
4iL5bHzCJ2KX/K20EG7y49gs/xwf8BPYE/0v8Rd+CLPlu7Cf78AO+Ups52ux
Tr4Vf+Cb8Lz8R3iML8NE+XXYyL+LrfLxuIePYbX8W3iAX4hn5P9M9f7y/+Jz
9fv4If8bNsm3YT5/CXep/4xr+D5slm/AN/mjOKG+HxfwX2Ct/HKcza/GT9S3
YBq/Cevk5+NU9D/BcfVU/IOfhZXyI+40yt/F99Un8R7/CKvl473/D/0eMSyK
3i/vaSfVUAvVUx/9D2v3x3Y=
          "]]},
        RowBox[{"-", "0.14200000000000002`"}]],
       Annotation[#, -0.14200000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV03tojWEAx/HHhtHcnTFjmPIHci/k+oe7ppBLlMklhGzZcp1ym3KtMRs2
lxDWMXc1YkUhxISiXEZNIdRiE7J8zh/f8z2/53l/z/M+73lP2oLMaSsbhBDG
+yhICiG5YwhFHUI4iAIU4gAixlu0DyGHO7QLIZtTuJmxRAxsEcJ0XEgO4Tx2
6+xCP2Nl8jlEUdoqhDzjvY3ftEaePc8Y22LsNG/lTcY28gl5PediA5q6Ps5e
8Whu76tymnX++Z7K6SixRzGyXJ+J9sbmuT4Dc3HQmqW82jX/UkKoR6F8AFUt
nY3fcQHvx6FYB++NxVuvyPcSPoKjKEYja0U6h9AOSYh3/6usm40Ec40xQL+X
bn+ulX8iTrcBhhrbY64+NYTLuoOdq4v5rrgij+cJ6IY0lMV+p9gz5RSuRK3+
I2eL8i/UYYn82LN5gnF4gL/NQ8jnpTol/MMa5RzFU/fxSu8jLsXWQwXme4a/
3Wdj5/nKXbktEl3/TK5CFHdRjhp7jDA/Cf2wCLPQ0fVfI/Zwziqus+5dfiQ/
49nut5wr5ChXW2cNn5K38R39o3xJzufP+nO5SF7G6fpTeKc8ml/qt+QsuRNf
0x/EG+Ue/Ea/tm0IC+U4eZT+J3mm/Jof6p/h4fJVPqv/hCfKt7lSfw/3lQ/z
AP1N3E3O5lv6o7iJnM7F+os5SZ7Nd/S7899OutxdP5m/y4l8Uf9lmxBeyNW8
V7+eP8g1fE3/Ft+XH3Ky/kW+IZ/kk/qZfFzO5c36hVwm7+Cz+jN4v7yAE/Un
8HZ5GBfqJ/AKOcI5+n14nZzGxfrfWvt/yX+43vx7niY/5x36x3iIfI6XmL/H
Y+XrvFc/j3vJ+7jG/FruLC/nXP3B3FAew3PMZ3BreSpv1k/lX/4vPbnafCv+
Isdzpn6lMz2V3/Bk83X8Vv7MOfoJ3v+Pfo8Ij4y9X97TKNYgH9twHf8BSFvB
1Q==
          "]]},
        RowBox[{"-", "0.21300000000000002`"}]],
       Annotation[#, -0.21300000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV02tojmEcx/Frzq3Y2NjmuBU1M6cXKKcXziKn0BRiQk5TFGPKOZscZkIm
Z7Z5HLaiRraiWE3ZQlEINTGh5pjTyud58d33/l3/63fd9/M899Ky187KiQkh
TPZnaIcQ5qCocwhJ3UI4zDu6hLAdCXJRijUcxCEUIt76Afv2ur4WH0I+X+UC
LrD+r4cZR5JDuIQys11mme5TJpeiBBetb7eebn0WLlq7gE3WcpFqbRpyndXO
PTdxnufajLbyGntysArFzlrNK+wpSwrhmLxSPspxPUPo6NxFZn+7hrA4+oyc
5oxUJNuf5T7V9lRFn9d8sPXbXMAZ7rfDnkazD+jjul9cCO9dv0O+PXsQY39L
uQW34m8YZF+G/QP5q/wFNeZj7G/mRPks95LPRH8Dz5qCZJTLE8wruCdfjs45
wilch4fI8FlqfM5+nImr7ncFy6yVePZSjMdxFGK5zgn+6pxKjqDe8z3TeYsK
PEA1FvlefvkeWzv3I/fiToi1/5H8ChHcQyWa2ocw0nwyBmIJ5qKr/R8T3cO7
8Yp/OPceP5AfcZZnreRqOcINztnA5+WdfFf/JFfIhdyoP5+Pyit4iv50LpDH
8FP9DrxW7sY39IfwFjmdX+h/TwghW46RR+u/l+fIz7lWv4RHyNe5VP8hT5Lv
cJ3+Ph4gH+fB+ls5TV7HVfqjua08hYv1l3KinMV39Xvzn+663Fs/iT/LsVyu
/7RTCE/kBt6v38xv5Ca+oV/FNXItJ+mX8y35HJ/Tz+HTch5v0z/CV+R8LtWf
zUXyYo7Vn8i75eF8RL8Nr5ITeL1+f86VU7lY/1PHEBbKv7nZ/DXPlB9zvv4p
HiZf5mXm93mcfJP36+/iDPkQN5lv5B7ySs7TH8ot5bE8z3wBx8szeJt+d/7p
/6YvN5jH8Qe5Befo1/lM9fILnmr+g1/Kjbxev433/63fI4FHRd8v72kEG1CI
nbiJ/7lCwuA=
          "]]},
        RowBox[{"-", "0.28400000000000003`"}]],
       Annotation[#, -0.28400000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV03to1mUYxvHbdOL+mHNT3zzM0oaHQE1Eo4MTsjAPkGfUNMqROoXNFLfZ
1G1YNKHVnKi4yIGhk6k4UZiLGVoqTEgxwTMqTMiwYEieM/Xz++N6v8/13Pd1
P7/3dxiUv2JmUaeImOln2ssRC1IRH1Nt74hf+0eM7R5RyWdZZ9P3fSMO9oj4
DpsS6jtmf7S+WfQt//+AiGrcr14hW2X9n72NOJ//Bn+ReUN/OZ9hXWFeJZXT
riSn52f7Q/XMoHn6yu212BvCr+dL+CfmlmJ9n4gdVC9bYkaOnqlUpJYmswK7
Yp16gfVB65T6HHNm0/N+EUvtl5qxhnqrldh/SI9oNW2Wnewe1eJ9/Q9otd5i
ytS/DbdSpdmD9V+yvkydMyMGynXBL3CE2jP7zylc6wR7b6nVyG1K7jGNS54B
DsLX6KR1oTPu6m1z/nJcRgXUyu8x5wQ24G+Yp388fal+0zkp83Ot19FaKqMW
/3clpqktckY3ffnJWfIf4HaqoSX2f8C7+puxkc7KXJS9RU10mo7Sp67xoWvo
4t7cwVcwi9L1n+OvU2NyrdRMHRkR76pPopGUT3Oor/47vZzh+V7He+aewNP8
OZzrGpvxKN+I7eYU40/8Bjwu/yM28TV4W34BbuULcIr8R7iRfw8vyGdgEd8P
D8uPwbX8ULwq/29P94kPPk/+T342fwXb5HfjO/whbJD/HT/kj+GZ5NvAEfx2
HCVfjgP5ldgqn4dd+SlYJ/859uTn4nH5XHycI4u58in8h0/HA/IXsiPO8+1Y
Lf8Ub/IdeFi+FU/xbZiSP4At/E7cKV+I9XwZVshvwX18FTYk3zfW8p9huvxE
/Jp/G7fIp+FyPhtXyQ/HUv5VrJP/OyviE/4RPlW/gdP5P7BKfge+ye/Fxeon
8X3+CFbLf4Wv85uwQ70Ec/hlWCY/Fl/iJ+A89YWYyU/DCvn+eN83MQzb1bvj
X3wnLJQ/4z+d5a/iVPV7eI2/javk07z/tzyPbByXvF/e00YqphraQEfoBW86
yCo=
          "]]},
        RowBox[{"-", "0.35500000000000004`"}]],
       Annotation[#, -0.35500000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1Hloz3Ecx/HPGM01tplrbCxnyB8o1x8oZ9toNOU+5irnDkwoRxSFGhFW
mNUoZNrGViZHE5Nr5Bg1hVAjZI4/eOyP557f1/vzeX2+399+363X4rXpayJC
CCl+DIwOYVd8CIUdQtjCZQkh9Dabjtau22Bj1xBOWS/sEkJbeat95dzHni2d
QsiRf/cIIZdnyanmWa6zUaCXrZ+LHCRZy7MnQr8ZAiLRHBnmq3Uuu06wb6O8
CcvMLprFmU1Fptzofkv5kPNXOPcwr+SO1nN1fnXzTGjEAvv+cLr5DMyT5+Og
zg/zua6zfLZsROtPxtKm+6In7prfQw1GdnYOH+PjSEEq0lDhvNOeoYof2/ME
oX0Ia9wjguc6qy9emdfhNUbo5ftshXpnUGv2FEmJITzjamfFuf5qXyx/4bW4
bj5Ar8izX7HvKnY6u8y8t/Vy3iVnuW7p3qWeIdHeVk1nc2u+bc9Q68Pw1azU
/mLct/+p2VtcwB1UYr7fy0/P2NxZH7kHd0CU/Q/kOhTjBkrR0C6EUdYnYTAW
YSa62P+xo3v4Duv4u3Nv8B35AWd4rlKulIu53jk5fErezlX6x/mCvJ/f68/m
Q/JynqKfynvksVyr35ZXy125RH8ob5b78gv9b3EhLJT/8Rj9dzxDfs7V+md4
pHyJi/Tv8UT5Gtfo7+VB8hEeor+Vk+R1XKE/hlvIU/io/hKOlTO4Sj+Zf3XX
5WT9eP4sR/F5/drYEB7J9bxP/y+/kRu4RL+Cb8nVHK9/nsvlk3xSfxUXyHm8
TT+fz8m7uUg/nQ/KCzhKfwLvlEdwvn4kr5RjeL3+QN4gJ/JR/U8x3m+5kf9a
f83T5Ie8W/8ED5fPcqb1mzxeLuN9+ju4v3yAG6zncoK8gvP0h3GEPI5nWZ/D
0XIab9Pvxj/8PfTjeuvt+EPT/xdepV/jM92XX/BU69/5pfye1+tHev/f+j5i
eHTT++U9LUYO9mM7yvAfSfrCiw==
          "]]},
        RowBox[{"-", "0.42600000000000005`"}]],
       Annotation[#, -0.42600000000000005`, "Tooltip"]& ], {}, {}}}], 
   GraphicsComplexBox[CompressedData["
1:eJx0nXtAlEXU/8nM7K6ZmWY3M83MrMzKbqfSSkuttHuWpl3N7G6ZlZiZmVaW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    "], {{{}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV0j1I1HEcx/HfDUJQllq0JJmUOUVi6eADZQ0K1eIaBjq4pZk62NADlA5q
YU9ehZlnnVnnIg4aWSGC4GC2qIMFBl3aUugc+voPb97/z/fh97v/3eU3NNc2
xUIIH1C5N4SKfSGUI54TwjP04wWe406mnB3CbY7zxz0hlJmd4hX9hP1yDGFZ
nlQv1Z/gJXlQ/YbdHrsd3MtH1PNQbO4hP0JfdKfZx1zIT7Hl+QkXmTse9fBN
bdTZJ9SSvIHpLJ8LV51/wMy6mf183V2Nas3cgmvYpVdgt4vvmunke5yndhh9
zhtzVgrvcdl+vb0e9dfyMBKoVf+HtP2/fN7MRa7iC3yO/3AJV/EpPsunOc0F
XMpHuYSP8XfO4SLO4pOczcv8yz0rfMvdN5HvORa9n8/VLrehFQfVN/mK+jzG
vdsXtWn7dfKCPG5v3cwG0tit/zt6X14zl+Km6PvET6zhkt0ZvNJL2h/iEc7g
eWfG+Uf0/+F3cj+n5Go7c3LC7DAWnRXT+8qBH+CtuTPmPqNXXpXv87bczW/k
Cs+fMOCMl1iw/19u4EE5aWYEh+RcVOrNurfGb1qNHSqLbP0=
          "]], LineBox[CompressedData["
1:eJwV0V9ojXEcx/HvsiaLzcbtDsMVzjC7IY1cuMDNNn92WNG2s50Ltras/Jc/
N1jYysSFHReuTUrZSY4oduOGlKIIUYy2C5Qar+fi3ef7+fw+v+/zPD217b3N
PSURUUBrRcTmyohJ2jg3IjU/oga3qiLymJF18C3VETuwCzvxQz+D3eYMWjE4
L6JN9tO8l05jSPaOH6al6LKrGzlssfuXzjY6ptNuvqnTQe/y2+V/zKOyHJ8z
Z2k3uvCXv+MdP9n1GR9Rrnsf48m7uj8lW8D30X5MyNowyzdnaVHvkD3l/Bw8
t28C/3QXuTdDF9OQvaaVNE0raB19i6d2DNi1RNbAn3fnAi7ipLxevlF+ib+M
Kzgt3yTfKr/Kj+Aazslb5E3yUT6f/Itkl/yZ5/yWZ7BPZw/dT0u9/0HayR+g
WdpDZ+uX4b3zD6jGQr6KfrEvjyN6R/HAM8ZRwG35C72UXq9dx5yfwHGk5F/l
YzovzSvNj1BM/pnslSxtPuzeMvNa81JaTyed38OIPaecFz3rMdYnZzrr6DcU
dN7wG8xPkDbXYXnyPKzAd/lDvRt2nbXrOj1DVztbg1Vo1JnSafZfm/Af4g1n
9g==
          "]]},
        "0.046`"],
       Annotation[#, 0.046, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl1FtM12Ucx/FH0KJCIci8KQVTBGykS1PXTZ5Ku2iKUZvKQcPYsGmbB9rM
bnSlXigKWpZopW6KAU1EzAuVtgqtzSOFh4l5Fp1nZznLXs+6ePN5Pp/v5/n9
n9//9+OfOX12/qwuIYQifxK6h1CaGsLmp0NYbV2bFsKo5BCOyo7hCH5HG0bL
f+oRws+o1O0jy8Br8n0YnRLCDbMprjEVkyP8LdzEGJ12/eP4AydxAmPl23TX
uGYiZsgK+SIU2ldM79AilFgX03u4i/H21stKrGvsnUYb+Dfl960n0B38+9a1
5mW0iZ8of2BdQH/gP7B+hH+xQ++crImmYbbzfIhW3SlIcJ+ldK/OHP0k/nH8
4jts1f9H93n6kPambWjRLbenu86L/CKzxfF7lc1FhjwTn8mWoF22EGdkf6ID
r3peKekhVJlX45T5Ikw0y0eNbD3OypahUFaELbKtqEWlvNVZEp17unOUm79H
Z9IkWTez07QDqUjnU+hF+zagQu8jNLvWrnifZp06z9ELsYd+uCp7Ic7s2YJD
fH9+vs/Kpdf4nNgza8ARfiBfYf4SzYsdeQsdjEzzQbQvvUKbzVY7xyf61XQh
HS4fFt81sxy9vc5XZbYK/fkb8h/NX0EWn40B6OR3m43wDHfSafaV4Gv5cNkw
rNG7Hj+T7jFb6ZqVGKLzmGcylH7sDMtlC+gK2qQ/1N7ttM2+9TpD+JexzjrZ
vhqajS/j+6yzNt6v+ef0KfMvaB+sxG3zyvjdmC+nT5qvoLl8Dr7xOb/qLJWV
OcMcZ5iHuXhCnqWzlC7DEnwav1dZBlbZ+737akA9ptr/Ny7p/UXfdY1J9G36
DgrQiTcwDmPwOsbiMi7iEi4gz3ubRbMxAKfwLHohHT3xDNrQFd2QgL7x94Du
p+ed4wCtcLb56GV9Jz6neM9odC8tsj3x94E/yDfae1XnGjqRbF6PZp1J8bcM
dXp1sp2yfP43fiN/ht9Ev+O/pdv5t+Jn8V/xHfw6ui0+O1rHj4v/W/x+VMu2
ykbGdxibnKVKdjj1/9/c/wD0Zcm+
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl1FtMznEcx/FfZiUr5LApORSbUYYbpws2d04rljksTA55YpVic2gk5FSb
w4UL5oqKlIqNzWnDbMJcZGzIIQzjxqZNDpvXfy7evb/fz+/7e36/53n+TxkF
JYuK40IIqf50DgghhvMpIVxAXXIIr/XxXMSNsouo17/RJ/BGbpI1o0H/Vt+H
N/HqgSHcSwqhgHv6hbAWa9TroprPR6/LRfoYx/gPLkevy9dkefb/VS/nG/pi
9S3rcf1DKFEXy0q5hG/LK7hMv513Ygfizd53vx53+oWfSDf7mzu4w9pTfspZ
XCnfgyqUO3eEfCT266vxXFaBSbLJqJHV4oWsEjNls3BMdhwvZVXIkeXilOw0
3skOIV+21Ll1svrojvpe3kMPx3Ehr7a+nmPYgN7eV6e1M/a3q5PUNzFQ3Y9T
OJnfY5O9afxFlsqf7TmLx/ph0Wdo/QOvcvZKZKhTrd3i4ZzJaZwezeOTvtn+
w+5xBAex1Wsc4ib5Ad6i38/lXB09N/J9XKav4s28lxvle7hUv4srsTt6/uQT
fW/1/MSZJ5w5CkfVxznfPVcg28wQfS2+W6vhLNlhThzkftwXY2ULzefioP6I
2UQ+gAfqk87JNHOCP6LNXGt0hnsVuE+tbHb03ei79Yt5KefxMv6Gr5iPBZiD
eZiLL5iOGZiCaZiKLmRjAsZFzx7GoxPDkI6hSEMqnuE5EjDGufE8mtv5IY9z
v0L33MHbsQ3d+BH9FuWPcMfcdfN3+QYP5it8jTPMZUbv22fSgqvRrM9mSfSs
6Fv1Xfo2btZfilC32N/CrZzI53DZXI59d9EgfzXg//+Xf6NqnaU=
          "]]},
        RowBox[{"-", "0.046`"}]],
       Annotation[#, -0.046, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl09tPz3Ecx/FPNTd+9ftVliIuLMxdwwU3Pxs3cuGwCgllkVOnzSgbZWOG
Dd0wGeb0F5BfJ+XUwYVsxpY53DhdMMVVzIU8vnPx3PPzfr3fn8O+v5pX01Ta
mBZC+IbTiRDKMIKn6M0KoTsnhD7u4Swezw4hzhO8JTOEn5xQ/+Bs7jPXz718
n884J4fLOWbmT66z7cu0DrKGeAh1sgbUY8BsCx/GQTTjEKZFb9IbdO4whlCo
PuKco2jFPufGzR3gQXtanJ0b3Y+35t/hpfVfvSX2zlUv5UuyLzzGYUYISeti
vRV8RfaVP3C6Xol1Um8NX5dN8CfO0FtvvU5vA9+UjeM7xrzjNQJ+60/pf/bG
dtSqd6g7vf8e0r0tA1PyNA54b981s5v1NyEmm45+eYKzkYk4svBRftt8t7t7
kEKdO7p4q/2VKDAzarYAXer9+s3uzJfny2YiFX1TeV5Uyyui3zx6p3P2yu9w
J+7ilvsW8WXeaKYc7fadR8zes3wOpfIydJgrMn+BS9SrcUL/OKqdvdNbqngX
/8Ik5phv029zXisf40LZbJx0zkpnNFk3Rn9XWGv/DU7K9/Bu1GKV/Covl1dz
FbZjmbyDF8u3Rd8KlVEtv8jF8nr3vELK3W/kT7yjTv1C/cwbR6P3mpulNxl9
Z34ke4wF6oWoMf8cQ3oD8od4gDx1irtQZG4+KsyNYNj9pxL//0f/Aavvfvo=

          "]]},
        RowBox[{"-", "0.092`"}]],
       Annotation[#, -0.092, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0klIVXEUx/EjZBn0fA4NJLQIikKjRTaS1MIomgyExHKTuqkW1stqUZto
gGhy1SK1ds1lrSJwFxRZvmcvskWBSLSohQ2QFjjU59Liy/ec3/kP917uwpZD
9W0FEfEUF9MRu9GH8bKIgVkRr9SXcBkNSJVETJi9MStWT5Nl9EdwGJniiHZ+
kYooNT+uT/MxLuGU9W9LI/Lm2aTmHFdyI+/BO2dnkFaf5DX2rcYJZ8yRzcaQ
Mz7bM6j+67716uWyGu6UfeFhLiiP2KyuMdvCN2Qj/Cm5x3mDGNP/Mh9N9rjv
PG6a30I7jmIMU8k6nuRR/mDvLs9Vh9/66fo/3MtTzpvB4zyJCXzUdzv7kTWP
8QA9eIgu+T2+j9u4izvolC/iq7zTPTtwxjnnvHMhb9UXecfT6rOybfrtWGDP
Jq5FGU6Zl3ODZ9/rWZrwNflO+KGez9/4O0bw07p5srnYaN8GHFQfwH7Uml/n
dfIWbsY+rJVf45XyVXjvuZ+4e0i+Ql+d/BOyvGzAuivowEx3FmGxdZXWFHqn
XPJvyKr0y5C3L6vvl7+2J8v9XMHP5M95iXVL0WptDi/NL6T//9v/ABPAbbY=

          "]]},
        RowBox[{"-", "0.138`"}]],
       Annotation[#, -0.138, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0stLlFEYx/HHdWo6k+C2Ta2SEFLUcmUXFYzauG/VpsncBG7KVTvJEsEI
F13AP0Doskxs22RCM5MmMpqQ4bUyr/U5tPjyfc45z/N73/fMnLxx+3quIiLG
8aEKtREfkUdjJuJWZcQv/ns84hFO63nIp3iYR1CVjXjMeezjrpku7sSCvk/8
qjriNbrV5+Q14V1NxBR29Ddb7zorooTPWDFb4Dl8QYd1O7+R8xZX1fVYtb/C
W7zE37CMpzLr5B+qj9K5b9rEtr6sd16zt45NbGBcfxe60YlrnnGfB3FCzqD1
PfWo9y2oiziQddHZJQzZL2NL1jZeoNd5P790foef4xke6FvAmP0n+J5mvNtP
/avqnnRv8n/wBc9sx7S+99gzdzndF7dZT6vP85L5MlrV8+nu8RWtKYNbeEZm
mWvSnfIsL3IPH2BX1oDcszL6+I/1Dn6j13nGvdWiZKZeT9HzCuk/k75R9gRy
5q7ov8kNes4gr79Sz7H0O2LG2WT1///cP0e4bOg=
          "]]},
        RowBox[{"-", "0.184`"}]],
       Annotation[#, -0.184, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0slO1EEQgPFCcUMcURKJ3LzpWQ8KJsyAo6IzMILLXblJwIsLIq6AywMY
EzUuEIh3Ly6AC3H3AdBojA9giKgxCgm/DocvX1d1VXX3f2bD4e62rrKIeIgt
ayPymYjNvINXronYuCqigjdxJf+vjPhYFVFeHXFV7pbce/EHtOsp6N3PRW7j
7OqIHEasH6Bbf17cw/fVDOEeRu3VmrEe38w9ruYETqJMLjCjpyndDY1o0ZPn
7+qr1Z0ST4t7eCcW6dnFi7mB/6nJ8hHuwJR55/Dc+gWe4bJZL/mdntP8OX0T
uXrvzMp1ylV5+1G+hFJ6Szobv80a5D88wBl1/XxD39e0h1fm9blDKd3R3hke
luvlt+re4DWuOPNm+t48qe4a96Z69OGg+hZ9h7iVD/C+9M1Rwl3cwbD8MvO+
OHtOvNT6mP6ifIO4kPbdc7l8TpzFL7V1XI9t2KtmO+9JPXr/2v+UfjvxbpTr
beYlvFW+glvT+6xXWNdgHW6Lx/SP4ykGvWmCJ+094QHxY+7n8zybfh++iAu4
jrP4KV/kQvru1o+40Xk/zG/iXGbhvzwPt6NeTg==
          "]]},
        RowBox[{"-", "0.22999999999999998`"}]],
       Annotation[#, -0.22999999999999998`, "Tooltip"]& ], {}, {}}}]},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1.5, 
      FormBox[
       RowBox[{"-", "1.5`"}], TraditionalForm]}, {-1.25, 
      FormBox[
       RowBox[{"-", "1.25`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.75, 
      FormBox[
       RowBox[{"-", "0.75`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.25, 
      FormBox[
       RowBox[{"-", "0.25`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.25, 
      FormBox["0.25`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.75, 
      FormBox["0.75`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}, {1.25, 
      FormBox["1.25`", TraditionalForm]}, {1.5, 
      FormBox["1.5`", TraditionalForm]}}, {{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2, 
      FormBox["1.2`", TraditionalForm]}}, None, None},
  ImageMargins->0.,
  ImageSize->504,
  Method->{"TransparentPolygonMesh" -> True},
  PlotRange->{{-1.6536749816983884`, 
   1.6536749816983884`}, {-1.3536749816983884`, 1.3536749816983884`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 419},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[10]=",
 CellID->670804276]
}, Open  ]],

Cell[TextData[{
 "Arrows for velocity field are green, equipotential lines are red, and \
streamlines are blue.  Note orthogonality of equipotential lines and \
streamlines, streamlines everywhere parallel to the velocity field and \
clockwise circulation associated with positive ",
 Cell[BoxData[
  StyleBox["\[CapitalGamma]", "TR"]], "InlineFormula"],
 ".  The thickened equipotential line along negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis suggests branch cut for velocity potential."
}], "ExampleText",
 CellID->680409942],

Cell[TextData[{
 "Look at the properties of the velocity potential as the negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis in crossed in a counterclockwise direction."
}], "ExampleText",
 CellID->635547329],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ic\[Phi]BranchJump", "=", 
  RowBox[{
   RowBox[{"Limit", "[", 
    RowBox[{"ic\[Phi]", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"x", "<", "0"}], "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", 
      RowBox[{"-", "1"}]}]}], "]"}], "-", 
   RowBox[{"Limit", "[", 
    RowBox[{"ic\[Phi]", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"x", "<", "0"}], "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}]], "Input",
 CellLabel->"In[11]:=",
 CellID->81019929],

Cell[BoxData[
 RowBox[{"-", "1"}]], "Output",
 ImageSize->{19, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->633735606]
}, Open  ]],

Cell[TextData[{
 "There is a branch cut along the negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis for the velocity potential.  This abrupt change in value of the \
velocity potential as the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis is crossed causes the thickening of the equipotential line unless \
special care is taken with the options of ContourPlot."
}], "ExampleText",
 CellID->78970066]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[{
 StyleBox[ButtonBox["ICVortexConstant",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICVortexConstant"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICVortexLinear",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICVortexLinear"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICSourcePoint",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICSourcePoint"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["ICDoubletPoint",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletPoint"],
  FontFamily->"Verdana"]
}], "SeeAlso",
 CellID->745892494]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["2.1 Introduction to Influence Coefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"]], \
"Tutorials",
 CellID->31486283],

Cell[TextData[ButtonBox["2.2 Point Singularities",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/PointSingularities"]], "Tutorials",
 CellID->98288976]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["InfluenceCoefficients",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/InfluenceCoefficients"]], "MoreAbout",
 CellID->170071063]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"ICVortexPoint - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 1, 12.0084000}", "context" -> 
    "Aerodynamics`InfluenceCoefficients`", "keywords" -> {}, "index" -> True, 
    "label" -> "Aerodynamics Package Paclet Symbol", "language" -> "en", 
    "paclet" -> "Aerodynamics Package", "status" -> "None", "summary" -> 
    "ICVortexPoint[r_f, r_p] computes velocity influence coefficients at \
field points r_f, due to vortices at r_p.ICPhiVortexPoint[r_f, r_p] computes \
velocity-potential influence coefficients at field points r_f, due to \
vortices at r_p.ICPsiVortexPoint[r_f, r_p] computes stream-function influence \
coefficients at field points r_f, due to vortices at r_p.", "synonyms" -> {}, 
    "title" -> "ICVortexPoint", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/ICVortexPoint"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[9046, 272, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->196781413]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 191553, 3840}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[606, 21, 76, 1, 70, "SymbolColorBar"],
Cell[685, 24, 191, 5, 70, "LinkTrail"],
Cell[879, 31, 3566, 81, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4470, 116, 56, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4529, 119, 2079, 68, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6645, 192, 579, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->60470433],
Cell[7227, 211, 364, 11, 70, "Notes",
 CellID->1067943069],
Cell[7594, 224, 202, 4, 70, "Notes",
 CellID->358739961],
Cell[7799, 230, 433, 10, 70, "Notes",
 CellID->30234373],
Cell[8235, 242, 697, 22, 70, "Notes",
 CellID->777516543],
Cell[8935, 266, 74, 1, 70, "Notes",
 CellID->529540534]
}, Closed]],
Cell[CellGroupData[{
Cell[9046, 272, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->196781413],
Cell[CellGroupData[{
Cell[9532, 289, 148, 5, 70, "ExampleSection",
 CellID->219571741],
Cell[9683, 296, 128, 6, 70, "ExampleText",
 CellID->513831186],
Cell[9814, 304, 100, 3, 70, "Input",
 CellID->267685788],
Cell[9917, 309, 321, 8, 70, "ExampleText",
 CellID->659456463],
Cell[10241, 319, 315, 12, 70, "Input",
 CellID->64953589],
Cell[10559, 333, 140, 4, 70, "ExampleText",
 CellID->326501090],
Cell[CellGroupData[{
Cell[10724, 341, 209, 7, 70, "Input",
 CellID->560964283],
Cell[10936, 350, 573, 20, 56, "Output",
 CellID->112915787]
}, Open  ]],
Cell[11524, 373, 95, 1, 70, "ExampleText",
 CellID->44302699],
Cell[CellGroupData[{
Cell[11644, 378, 168, 5, 70, "Input",
 CellID->575890672],
Cell[11815, 385, 283, 10, 54, "Output",
 CellID->191967185]
}, Open  ]],
Cell[12113, 398, 317, 10, 70, "ExampleText",
 CellID->53987023],
Cell[CellGroupData[{
Cell[12455, 412, 195, 6, 70, "Input",
 CellID->30542685],
Cell[12653, 420, 231, 8, 54, "Output",
 CellID->41679754]
}, Open  ]],
Cell[12899, 431, 89, 1, 70, "ExampleText",
 CellID->472419597],
Cell[CellGroupData[{
Cell[13013, 436, 202, 6, 70, "Input",
 CellID->8664355],
Cell[13218, 444, 311, 11, 58, "Output",
 CellID->229221347]
}, Open  ]],
Cell[13544, 458, 203, 4, 70, "ExampleText",
 CellID->9427774],
Cell[13750, 464, 441, 12, 70, "Input",
 CellID->258291294],
Cell[14194, 478, 967, 35, 70, "ExampleText",
 CellID->78226744],
Cell[CellGroupData[{
Cell[15186, 517, 298, 10, 70, "Input",
 CellID->13857323],
Cell[15487, 529, 209, 7, 36, "Output",
 CellID->372418502]
}, Open  ]],
Cell[CellGroupData[{
Cell[15733, 541, 321, 11, 70, "Input",
 CellID->692553449],
Cell[16057, 554, 208, 7, 36, "Output",
 CellID->12270635]
}, Open  ]],
Cell[16280, 564, 265, 5, 70, "ExampleText",
 CellID->78531558],
Cell[CellGroupData[{
Cell[16570, 573, 3737, 106, 70, "Input",
 CellID->1746043248],
Cell[20310, 681, 164794, 2956, 70, "Output",
 CellID->670804276]
}, Open  ]],
Cell[185119, 3640, 559, 12, 70, "ExampleText",
 CellID->680409942],
Cell[185681, 3654, 234, 6, 70, "ExampleText",
 CellID->635547329],
Cell[CellGroupData[{
Cell[185940, 3664, 662, 19, 70, "Input",
 CellID->81019929],
Cell[186605, 3685, 176, 6, 36, "Output",
 CellID->633735606]
}, Open  ]],
Cell[186796, 3694, 448, 11, 70, "ExampleText",
 CellID->78970066]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[187293, 3711, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[187713, 3725, 643, 21, 70, "SeeAlso",
 CellID->745892494]
}, Open  ]],
Cell[CellGroupData[{
Cell[188393, 3751, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[188815, 3765, 209, 5, 70, "Tutorials",
 CellID->31486283],
Cell[189027, 3772, 168, 3, 70, "Tutorials",
 CellID->98288976]
}, Open  ]],
Cell[CellGroupData[{
Cell[189232, 3780, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[189654, 3794, 167, 3, 70, "MoreAbout",
 CellID->170071063]
}, Open  ]],
Cell[189836, 3800, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

