(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     42979,       1262]
NotebookOptionsPosition[     37460,       1073]
NotebookOutlinePosition[     39088,       1119]
CellTagsIndexPosition[     39002,       1114]
WindowTitle->MultiplyByList - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"A1  Data Type CartesianVectors\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/CartesianVectors"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"MakeCartesianVectors\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/MakeCartesianVectors"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"CartesianVectors\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/CartesianVectors"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/MultiplyByList\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/MultiplyByList"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
MultiplyByList.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/MultiplyByList", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["MultiplyByList", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"MultiplyByList", "[", 
       RowBox[{
        StyleBox["a", "TI"], ",", " ", 
        StyleBox["u", "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]multiplies a collection of vectors ",
     Cell[BoxData[
      StyleBox["u", "TI"]], "InlineFormula"],
     " by the list ",
     Cell[BoxData[
      StyleBox["a", "TI"]], "InlineFormula"],
     "."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->130288823],

Cell[TextData[{
 "If the length of the list is equal to the dimensionality of the vector \
space, each component is multiplied by an element of ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ".  This is used for scaling vectors."
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 "If the dimensions of ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " are the same as the dimensions of components of ",
 Cell[BoxData[
  StyleBox["u", "TI"]], "InlineFormula"],
 ", each vector in a collection is multiplied by a corresponding element of ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 "."
}], "Notes",
 CellID->262460257],

Cell["\<\
This function is defined because I was unable to overload Times to give the \
desired results.\
\>", "Notes",
 CellID->154613871],

Cell["New in 1.0", "Notes",
 CellID->529540534]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->475611956],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->507038559],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->368492904],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->305512418],

Cell["Specify a collection of five two-dimensional vectors.", "ExampleText",
 CellID->243599311],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uS2D", "=", 
  RowBox[{"MakeCartesianVectors", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Array", "[", 
      RowBox[{"ux", ",", "5"}], "]"}], ",", 
     RowBox[{"Array", "[", 
      RowBox[{"uy", ",", "5"}], "]"}]}], "}"}], "]"}]}]], "Input",
 CellLabel->"In[2]:=",
 CellID->101936636],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}], ",", 
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "1", "]"}], ",", 
     RowBox[{"uy", "[", "2", "]"}], ",", 
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}], ",", 
     RowBox[{"uy", "[", "5", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[1], 
     $CellContext`ux[2], 
     $CellContext`ux[3], 
     $CellContext`ux[4], 
     $CellContext`ux[5]}, {
     $CellContext`uy[1], 
     $CellContext`uy[2], 
     $CellContext`uy[3], 
     $CellContext`uy[4], 
     $CellContext`uy[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{479, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->29093578]
}, Open  ]],

Cell["Specify a collection of five three-dimensional vectors.", "ExampleText",
 CellID->250652535],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uS3D", "=", 
  RowBox[{"MakeCartesianVectors", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Array", "[", 
      RowBox[{"ux", ",", "5"}], "]"}], ",", 
     RowBox[{"Array", "[", 
      RowBox[{"uy", ",", "5"}], "]"}], ",", 
     RowBox[{"Array", "[", 
      RowBox[{"uz", ",", "5"}], "]"}]}], "}"}], "]"}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->622734421],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}], ",", 
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "1", "]"}], ",", 
     RowBox[{"uy", "[", "2", "]"}], ",", 
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}], ",", 
     RowBox[{"uy", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uz", "[", "1", "]"}], ",", 
     RowBox[{"uz", "[", "2", "]"}], ",", 
     RowBox[{"uz", "[", "3", "]"}], ",", 
     RowBox[{"uz", "[", "4", "]"}], ",", 
     RowBox[{"uz", "[", "5", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[1], 
     $CellContext`ux[2], 
     $CellContext`ux[3], 
     $CellContext`ux[4], 
     $CellContext`ux[5]}, {
     $CellContext`uy[1], 
     $CellContext`uy[2], 
     $CellContext`uy[3], 
     $CellContext`uy[4], 
     $CellContext`uy[5]}, {
     $CellContext`uz[1], 
     $CellContext`uz[2], 
     $CellContext`uz[3], 
     $CellContext`uz[4], 
     $CellContext`uz[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{487, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->187496915]
}, Open  ]],

Cell["Scale each component by a different constant.", "ExampleText",
 CellID->150923736],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MultiplyByList", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"sx", ",", "sy"}], "}"}], ",", "uS2D"}], "]"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->789413974],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"sx", " ", 
      RowBox[{"ux", "[", "1", "]"}]}], ",", 
     RowBox[{"sx", " ", 
      RowBox[{"ux", "[", "2", "]"}]}], ",", 
     RowBox[{"sx", " ", 
      RowBox[{"ux", "[", "3", "]"}]}], ",", 
     RowBox[{"sx", " ", 
      RowBox[{"ux", "[", "4", "]"}]}], ",", 
     RowBox[{"sx", " ", 
      RowBox[{"ux", "[", "5", "]"}]}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"sy", " ", 
      RowBox[{"uy", "[", "1", "]"}]}], ",", 
     RowBox[{"sy", " ", 
      RowBox[{"uy", "[", "2", "]"}]}], ",", 
     RowBox[{"sy", " ", 
      RowBox[{"uy", "[", "3", "]"}]}], ",", 
     RowBox[{"sy", " ", 
      RowBox[{"uy", "[", "4", "]"}]}], ",", 
     RowBox[{"sy", " ", 
      RowBox[{"uy", "[", "5", "]"}]}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{$CellContext`sx \
$CellContext`ux[1], $CellContext`sx $CellContext`ux[
      2], $CellContext`sx $CellContext`ux[3], $CellContext`sx $CellContext`ux[
      4], $CellContext`sx $CellContext`ux[
      5]}, {$CellContext`sy $CellContext`uy[
      1], $CellContext`sy $CellContext`uy[2], $CellContext`sy $CellContext`uy[
      3], $CellContext`sy $CellContext`uy[4], $CellContext`sy $CellContext`uy[
      5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{330, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->34368681]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MultiplyByList", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"sx", ",", "sy", ",", "sz"}], "}"}], ",", "uS3D"}], 
  "]"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->731879500],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"sx", " ", 
      RowBox[{"ux", "[", "1", "]"}]}], ",", 
     RowBox[{"sx", " ", 
      RowBox[{"ux", "[", "2", "]"}]}], ",", 
     RowBox[{"sx", " ", 
      RowBox[{"ux", "[", "3", "]"}]}], ",", 
     RowBox[{"sx", " ", 
      RowBox[{"ux", "[", "4", "]"}]}], ",", 
     RowBox[{"sx", " ", 
      RowBox[{"ux", "[", "5", "]"}]}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"sy", " ", 
      RowBox[{"uy", "[", "1", "]"}]}], ",", 
     RowBox[{"sy", " ", 
      RowBox[{"uy", "[", "2", "]"}]}], ",", 
     RowBox[{"sy", " ", 
      RowBox[{"uy", "[", "3", "]"}]}], ",", 
     RowBox[{"sy", " ", 
      RowBox[{"uy", "[", "4", "]"}]}], ",", 
     RowBox[{"sy", " ", 
      RowBox[{"uy", "[", "5", "]"}]}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"sz", " ", 
      RowBox[{"uz", "[", "1", "]"}]}], ",", 
     RowBox[{"sz", " ", 
      RowBox[{"uz", "[", "2", "]"}]}], ",", 
     RowBox[{"sz", " ", 
      RowBox[{"uz", "[", "3", "]"}]}], ",", 
     RowBox[{"sz", " ", 
      RowBox[{"uz", "[", "4", "]"}]}], ",", 
     RowBox[{"sz", " ", 
      RowBox[{"uz", "[", "5", "]"}]}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{$CellContext`sx \
$CellContext`ux[1], $CellContext`sx $CellContext`ux[
      2], $CellContext`sx $CellContext`ux[3], $CellContext`sx $CellContext`ux[
      4], $CellContext`sx $CellContext`ux[
      5]}, {$CellContext`sy $CellContext`uy[
      1], $CellContext`sy $CellContext`uy[2], $CellContext`sy $CellContext`uy[
      3], $CellContext`sy $CellContext`uy[4], $CellContext`sy $CellContext`uy[
      5]}, {$CellContext`sz $CellContext`uz[
      1], $CellContext`sz $CellContext`uz[2], $CellContext`sz $CellContext`uz[
      3], $CellContext`sz $CellContext`uz[4], $CellContext`sz $CellContext`uz[
      5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{338, 50},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->156415198]
}, Open  ]],

Cell["Multiply each vector by an element of the list.", "ExampleText",
 CellID->127216119],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MultiplyByList", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a1", ",", "a2", ",", "a3", ",", "a4", ",", "a5"}], "}"}], ",", 
   "uS2D"}], "]"}]], "Input",
 CellLabel->"In[6]:=",
 CellID->49749319],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a1", " ", 
      RowBox[{"ux", "[", "1", "]"}]}], ",", 
     RowBox[{"a2", " ", 
      RowBox[{"ux", "[", "2", "]"}]}], ",", 
     RowBox[{"a3", " ", 
      RowBox[{"ux", "[", "3", "]"}]}], ",", 
     RowBox[{"a4", " ", 
      RowBox[{"ux", "[", "4", "]"}]}], ",", 
     RowBox[{"a5", " ", 
      RowBox[{"ux", "[", "5", "]"}]}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a1", " ", 
      RowBox[{"uy", "[", "1", "]"}]}], ",", 
     RowBox[{"a2", " ", 
      RowBox[{"uy", "[", "2", "]"}]}], ",", 
     RowBox[{"a3", " ", 
      RowBox[{"uy", "[", "3", "]"}]}], ",", 
     RowBox[{"a4", " ", 
      RowBox[{"uy", "[", "4", "]"}]}], ",", 
     RowBox[{"a5", " ", 
      RowBox[{"uy", "[", "5", "]"}]}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{$CellContext`a1 \
$CellContext`ux[1], $CellContext`a2 $CellContext`ux[
      2], $CellContext`a3 $CellContext`ux[3], $CellContext`a4 $CellContext`ux[
      4], $CellContext`a5 $CellContext`ux[
      5]}, {$CellContext`a1 $CellContext`uy[
      1], $CellContext`a2 $CellContext`uy[2], $CellContext`a3 $CellContext`uy[
      3], $CellContext`a4 $CellContext`uy[4], $CellContext`a5 $CellContext`uy[
      5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{330, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->16269057]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MultiplyByList", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a1", ",", "a2", ",", "a3", ",", "a4", ",", "a5"}], "}"}], ",", 
   "uS3D"}], "]"}]], "Input",
 CellLabel->"In[7]:=",
 CellID->67325663],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a1", " ", 
      RowBox[{"ux", "[", "1", "]"}]}], ",", 
     RowBox[{"a2", " ", 
      RowBox[{"ux", "[", "2", "]"}]}], ",", 
     RowBox[{"a3", " ", 
      RowBox[{"ux", "[", "3", "]"}]}], ",", 
     RowBox[{"a4", " ", 
      RowBox[{"ux", "[", "4", "]"}]}], ",", 
     RowBox[{"a5", " ", 
      RowBox[{"ux", "[", "5", "]"}]}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a1", " ", 
      RowBox[{"uy", "[", "1", "]"}]}], ",", 
     RowBox[{"a2", " ", 
      RowBox[{"uy", "[", "2", "]"}]}], ",", 
     RowBox[{"a3", " ", 
      RowBox[{"uy", "[", "3", "]"}]}], ",", 
     RowBox[{"a4", " ", 
      RowBox[{"uy", "[", "4", "]"}]}], ",", 
     RowBox[{"a5", " ", 
      RowBox[{"uy", "[", "5", "]"}]}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a1", " ", 
      RowBox[{"uz", "[", "1", "]"}]}], ",", 
     RowBox[{"a2", " ", 
      RowBox[{"uz", "[", "2", "]"}]}], ",", 
     RowBox[{"a3", " ", 
      RowBox[{"uz", "[", "3", "]"}]}], ",", 
     RowBox[{"a4", " ", 
      RowBox[{"uz", "[", "4", "]"}]}], ",", 
     RowBox[{"a5", " ", 
      RowBox[{"uz", "[", "5", "]"}]}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{$CellContext`a1 \
$CellContext`ux[1], $CellContext`a2 $CellContext`ux[
      2], $CellContext`a3 $CellContext`ux[3], $CellContext`a4 $CellContext`ux[
      4], $CellContext`a5 $CellContext`ux[
      5]}, {$CellContext`a1 $CellContext`uy[
      1], $CellContext`a2 $CellContext`uy[2], $CellContext`a3 $CellContext`uy[
      3], $CellContext`a4 $CellContext`uy[4], $CellContext`a5 $CellContext`uy[
      5]}, {$CellContext`a1 $CellContext`uz[
      1], $CellContext`a2 $CellContext`uz[2], $CellContext`a3 $CellContext`uz[
      3], $CellContext`a4 $CellContext`uz[4], $CellContext`a5 $CellContext`uz[
      5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{338, 50},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->19603643]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Scope",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->1293636265],

Cell["Components of the collection of vectors can be matrices.", "ExampleText",
 CellID->201092444],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"uM3D", "=", 
    RowBox[{"MakeCartesianVectors", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Array", "[", 
        RowBox[{"vx", ",", 
         RowBox[{"{", 
          RowBox[{"4", ",", "2"}], "}"}]}], "]"}], ",", 
       RowBox[{"Array", "[", 
        RowBox[{"vy", ",", 
         RowBox[{"{", 
          RowBox[{"4", ",", "2"}], "}"}]}], "]"}], ",", 
       RowBox[{"Array", "[", 
        RowBox[{"vz", ",", 
         RowBox[{"{", 
          RowBox[{"4", ",", "2"}], "}"}]}], "]"}]}], "}"}], "]"}]}], ")"}], "//",
   "MatrixForm"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->418402993],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnb2S4jgUhdnaJ9nHGB5jn6BrAqqzJSTrkBAywk6mqsPOmrBDQh6AoEPC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    "], {{0, 57}, {420, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{420, 57},
  PlotRange->{{0, 420}, {0, 57}}]], "Output",
 ImageSize->{424, 57},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->164101680]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Dimensions", "[", "uM3D", "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->478310181],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", "4", ",", "2"}], "}"}]], "Output",
 ImageSize->{61, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->918624515]
}, Open  ]],

Cell["Scale each component by a different factor.", "ExampleText",
 CellID->235433920],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MultiplyByList", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"sx", ",", "sy", ",", "sz"}], "}"}], ",", "uM3D"}], "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->228073273],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnLF2ozgYhX12u32LfYz1Y+wTzJnCZ7pNmW7KlHGXcsopp3PKlCnzAC6m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    "], {{0, 57}, {522, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{522, 57},
  PlotRange->{{0, 522}, {0, 57}}]], "Output",
 ImageSize->{526, 57},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->153223563]
}, Open  ]],

Cell["Multiply each vector by a different constant.", "ExampleText",
 CellID->7567766],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"MultiplyByList", "[", 
    RowBox[{
     RowBox[{"Array", "[", 
      RowBox[{"a", ",", 
       RowBox[{"{", 
        RowBox[{"4", ",", "2"}], "}"}]}], "]"}], ",", "uM3D"}], "]"}], ")"}], 
  "//", "MatrixForm"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->39046019],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnb1y6zgShT21T7Kv4cfYJ3BN4HI2SrZK2Q0V2plDhw6d2aFChXoABa6N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    "], {{0, 117}, {481, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{481, 117},
  PlotRange->{{0, 481}, {0, 117}}]], "Output",
 ImageSize->{485, 117},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->72794514]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Possible Issues",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->1305812373],

Cell["\<\
If the number of vectors is equal to the dimensionality of the space, then \
multiplication of components by elements of the list takes precedence.\
\>", "ExampleText",
 CellID->85804112],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MultiplyByList", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c"}], "}"}], ",", 
   RowBox[{"MakeCartesianVectors", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"x1", ",", "x2", ",", "x3"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"y1", ",", "y2", ",", "y3"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"z1", ",", "z2", ",", "z3"}], "}"}]}], "}"}], "]"}]}], 
  "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->247157972],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", " ", "x1"}], ",", 
     RowBox[{"a", " ", "x2"}], ",", 
     RowBox[{"a", " ", "x3"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"b", " ", "y1"}], ",", 
     RowBox[{"b", " ", "y2"}], ",", 
     RowBox[{"b", " ", "y3"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"c", " ", "z1"}], ",", 
     RowBox[{"c", " ", "z2"}], ",", 
     RowBox[{"c", " ", "z3"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{$CellContext`a \
$CellContext`x1, $CellContext`a $CellContext`x2, $CellContext`a \
$CellContext`x3}, {$CellContext`b $CellContext`y1, $CellContext`b \
$CellContext`y2, $CellContext`b $CellContext`y3}, {$CellContext`c \
$CellContext`z1, $CellContext`c $CellContext`z2, $CellContext`c \
$CellContext`z3}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{346, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->607562134]
}, Open  ]],

Cell["\<\
Since CartesianVectors was defined primarily to simplify the manipulation of \
large collections of vectors, this ambiguity should not cause problems for \
typical usage.\
\>", "ExampleText",
 CellID->484040969]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[StyleBox[ButtonBox["MakeCartesianVectors",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/ref/MakeCartesianVectors"],
 FontFamily->"Verdana"]], "SeeAlso",
 CellID->929782353]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["A1  Data Type CartesianVectors",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/CartesianVectors"]], "Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["CartesianVectors",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/CartesianVectors"]], "MoreAbout",
 CellID->202359178]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"MultiplyByList - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 1, 30.9936000}", "context" -> 
    "Aerodynamics`CartesianVectors`", "keywords" -> {}, "index" -> True, 
    "label" -> "Aerodynamics Package Paclet Symbol", "language" -> "en", 
    "paclet" -> "Aerodynamics Package", "status" -> "None", "summary" -> 
    "MultiplyByList[a, u] multiplies a collection of vectors u by the list \
a.", "synonyms" -> {}, "title" -> "MultiplyByList", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/MultiplyByList"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[6153, 188, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->475611956]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 38859, 1107}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[607, 21, 76, 1, 70, "SymbolColorBar"],
Cell[686, 24, 191, 5, 70, "LinkTrail"],
Cell[880, 31, 3077, 72, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3982, 107, 57, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4042, 110, 599, 19, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4678, 134, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->130288823],
Cell[5261, 153, 277, 7, 70, "Notes",
 CellID->1067943069],
Cell[5541, 162, 383, 12, 70, "Notes",
 CellID->262460257],
Cell[5927, 176, 139, 4, 70, "Notes",
 CellID->154613871],
Cell[6069, 182, 47, 1, 70, "Notes",
 CellID->529540534]
}, Closed]],
Cell[CellGroupData[{
Cell[6153, 188, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->475611956],
Cell[CellGroupData[{
Cell[6639, 205, 148, 5, 70, "ExampleSection",
 CellID->507038559],
Cell[6790, 212, 132, 6, 70, "ExampleText",
 CellID->368492904],
Cell[6925, 220, 100, 3, 70, "Input",
 CellID->305512418],
Cell[7028, 225, 96, 1, 70, "ExampleText",
 CellID->243599311],
Cell[CellGroupData[{
Cell[7149, 230, 317, 10, 70, "Input",
 CellID->101936636],
Cell[7469, 242, 1047, 34, 36, "Output",
 CellID->29093578]
}, Open  ]],
Cell[8531, 279, 98, 1, 70, "ExampleText",
 CellID->250652535],
Cell[CellGroupData[{
Cell[8654, 284, 390, 12, 70, "Input",
 CellID->622734421],
Cell[9047, 298, 1438, 46, 54, "Output",
 CellID->187496915]
}, Open  ]],
Cell[10500, 347, 88, 1, 70, "ExampleText",
 CellID->150923736],
Cell[CellGroupData[{
Cell[10613, 352, 187, 6, 70, "Input",
 CellID->789413974],
Cell[10803, 360, 1471, 41, 54, "Output",
 CellID->34368681]
}, Open  ]],
Cell[CellGroupData[{
Cell[12311, 406, 201, 7, 70, "Input",
 CellID->731879500],
Cell[12515, 415, 2073, 56, 71, "Output",
 CellID->156415198]
}, Open  ]],
Cell[14603, 474, 90, 1, 70, "ExampleText",
 CellID->127216119],
Cell[CellGroupData[{
Cell[14718, 479, 223, 7, 70, "Input",
 CellID->49749319],
Cell[14944, 488, 1471, 41, 54, "Output",
 CellID->16269057]
}, Open  ]],
Cell[CellGroupData[{
Cell[16452, 534, 223, 7, 70, "Input",
 CellID->67325663],
Cell[16678, 543, 2072, 56, 71, "Output",
 CellID->19603643]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[18799, 605, 221, 7, 70, "ExampleSection",
 CellID->1293636265],
Cell[19023, 614, 99, 1, 70, "ExampleText",
 CellID->201092444],
Cell[CellGroupData[{
Cell[19147, 619, 651, 21, 70, "Input",
 CellID->418402993],
Cell[19801, 642, 3245, 61, 70, "Output",
 CellID->164101680]
}, Open  ]],
Cell[CellGroupData[{
Cell[23083, 708, 109, 3, 70, "Input",
 CellID->478310181],
Cell[23195, 713, 213, 7, 36, "Output",
 CellID->918624515]
}, Open  ]],
Cell[23423, 723, 86, 1, 70, "ExampleText",
 CellID->235433920],
Cell[CellGroupData[{
Cell[23534, 728, 237, 8, 70, "Input",
 CellID->228073273],
Cell[23774, 738, 3692, 68, 70, "Output",
 CellID->153223563]
}, Open  ]],
Cell[27481, 809, 86, 1, 70, "ExampleText",
 CellID->7567766],
Cell[CellGroupData[{
Cell[27592, 814, 316, 11, 70, "Input",
 CellID->39046019],
Cell[27911, 827, 5293, 94, 70, "Output",
 CellID->72794514]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[33253, 927, 231, 7, 70, "ExampleSection",
 CellID->1305812373],
Cell[33487, 936, 197, 4, 70, "ExampleText",
 CellID->85804112],
Cell[CellGroupData[{
Cell[33709, 944, 508, 16, 70, "Input",
 CellID->247157972],
Cell[34220, 962, 1049, 30, 36, "Output",
 CellID->607562134]
}, Open  ]],
Cell[35284, 995, 221, 5, 70, "ExampleText",
 CellID->484040969]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[35554, 1006, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[35974, 1020, 195, 4, 70, "SeeAlso",
 CellID->929782353]
}, Open  ]],
Cell[CellGroupData[{
Cell[36206, 1029, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[36628, 1043, 174, 3, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[CellGroupData[{
Cell[36839, 1051, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[37261, 1065, 157, 3, 70, "MoreAbout",
 CellID->202359178]
}, Open  ]],
Cell[37433, 1071, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

