(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     23886,        710]
NotebookOptionsPosition[     20208,        593]
NotebookOutlinePosition[     21958,        641]
CellTagsIndexPosition[     21873,        636]
WindowTitle->NACA4DigitAirfoil - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"1.1 Introduction To Airfoil Geometry\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToAirfoilGeometry"], \
"\<\"1.3 Airfoils From Thickness and Camber Properties\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/\
AirfoilsFromThicknessAndCamberProperties"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"AirfoilSurfacePoints\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/AirfoilSurfacePoints"], "\<\"PlotAirfoil\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/ref/PlotAirfoil"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"AirfoilGeometry\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/AirfoilGeometry"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/NACA4DigitAirfoil\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/NACA4DigitAirfoil"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
NACA4DigitAirfoil.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/NACA4DigitAirfoil", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["NACA4DigitAirfoil", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"NACA4DigitAirfoil", "[", 
       RowBox[{
        StyleBox["id", "TI"], ",", " ", 
        StyleBox["x", "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes a nested list of thickness and camber \
properties at a list of ",
     Cell[BoxData[
      StyleBox["x", "TI"]], "InlineFormula"],
     "-values for the NACA airfoil identified by the four-digit \
identification number ",
     Cell[BoxData[
      StyleBox["id", "TI"]], "InlineFormula"],
     "."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->109997341],

Cell[TextData[{
 "If the four-digit identification is written as ",
 Cell[BoxData[
  RowBox[{
   StyleBox["id", "TI"], "=", 
   StyleBox["ijkl", "TI"]}]], "InlineFormula"],
 ", then ",
 Cell[BoxData[
  StyleBox["i", "TI"]], "InlineFormula"],
 " specifies maximum camber in percent chord, ",
 Cell[BoxData[
  StyleBox["j", "TI"]], "InlineFormula"],
 " specifies location of maximum camber in tenths chord and ",
 Cell[BoxData[
  StyleBox["kl", "TI"]], "InlineFormula"],
 " specifies maximum thickness in percent chord. "
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 "Output is the list ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{
    StyleBox["x", "TI"], ",", 
    SubscriptBox[
     StyleBox["z", "TI"], 
     StyleBox["t", "TI"]], ",", 
    SubscriptBox[
     StyleBox["\[Theta]", "TR"], 
     StyleBox["t", "TI"]], ",", 
    SubscriptBox[
     StyleBox["z", "TI"], 
     StyleBox["c", "TI"]], ",", 
    SubscriptBox[
     StyleBox["\[Theta]", "TR"], 
     StyleBox["c", "TI"]]}], "}"}]], "InlineFormula"],
 "."
}], "Notes",
 CellID->923907834],

Cell[BoxData[GridBox[{
   {Cell["      ", "TableRowIcon"], 
    StyleBox["x", "TI"], Cell[TextData[{
     "input argument, list of ",
     Cell[BoxData[
      StyleBox["x", "TI"]], "InlineFormula"],
     "-values"
    }], "TableText"]},
   {Cell["      ", "TableRowIcon"], 
    SubscriptBox[
     StyleBox["z", "TI"], 
     StyleBox["t", "TI"]], Cell["list of thickness values", "TableText"]},
   {Cell["      ", "TableRowIcon"], 
    SubscriptBox[
     StyleBox["\[Theta]", "TR"], 
     StyleBox["t", "TI"]], Cell[TextData[{
     "list of angles that thickness line makes with ",
     Cell[BoxData[
      StyleBox["x", "TI"]], "InlineFormula"],
     "-axis"
    }], "TableText"]},
   {Cell["      ", "TableRowIcon"], 
    SubscriptBox[
     StyleBox["z", "TI"], 
     StyleBox["c", "TI"]], Cell["list of camber values", "TableText"]},
   {Cell["      ", "TableRowIcon"], 
    SubscriptBox[
     StyleBox["\[Theta]", "TR"], 
     StyleBox["c", "TI"]], Cell[TextData[{
     "list of angles that camber line makes with ",
     Cell[BoxData[
      StyleBox["x", "TI"]], "InlineFormula"],
     "-axis"
    }], "TableText"]}
  }]], "2ColumnTableMod",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, 
   "Rows" -> {None, None, None, None, {None}}, "RowsIndexed" -> {}},
 GridBoxDividers->{"Rows" -> {{True, True, True, True, True, True}}}},
 CellID->401597287],

Cell["New in 1.0", "Notes",
 CellID->529540534]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->470438946],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->56885981],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->467046585],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->145894596],

Cell[TextData[{
 "Compute nested lists of thickness and camber properties for a NACA-4412 \
airfoil at the list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values."
}], "ExampleText",
 CellID->63338921],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"x", ",", "zT", ",", "\[Theta]T", ",", "zC", ",", "\[Theta]C"}], 
   "}"}], "=", 
  RowBox[{"NACA4DigitAirfoil", "[", 
   RowBox[{"4412", ",", 
    RowBox[{"Range", "[", 
     RowBox[{"0", ",", "1", ",", "0.1"}], "]"}]}], "]"}]}]], "Input",
 CellLabel->"In[2]:=",
 CellID->684504875],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "0.`", ",", "0.1`", ",", "0.2`", ",", "0.30000000000000004`", ",", "0.4`",
      ",", "0.5`", ",", "0.6000000000000001`", ",", "0.7000000000000001`", 
     ",", "0.8`", ",", "0.9`", ",", "1.`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "0.`", ",", "0.04682770423823951`", ",", "0.05737542990236251`", ",", 
     "0.060017266393970294`", ",", "0.05803010847647902`", ",", 
     "0.05294025200057157`", ",", "0.045633690658677804`", ",", 
     "0.036639067126780174`", ",", "0.026231179804725006`", ",", 
     "0.014477172714718477`", ",", "0.001259999999999961`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "1.5707963267948966`", ",", "0.16717100260042894`", ",", 
     "0.057639457820609204`", ",", 
     RowBox[{"-", "0.00007777267655939293`"}], ",", 
     RowBox[{"-", "0.037230752653313264`"}], ",", 
     RowBox[{"-", "0.06302740734581672`"}], ",", 
     RowBox[{"-", "0.08197024037979789`"}], ",", 
     RowBox[{"-", "0.09698198462109772`"}], ",", 
     RowBox[{"-", "0.11031266314794418`"}], ",", 
     RowBox[{"-", "0.12387740424635431`"}], ",", 
     RowBox[{"-", "0.13939996933185547`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "0.`", ",", "0.017499999999999998`", ",", "0.030000000000000002`", ",", 
     "0.0375`", ",", "0.04`", ",", "0.03888888888888888`", ",", 
     "0.03555555555555555`", ",", "0.02999999999999999`", ",", 
     "0.022222222222222216`", ",", "0.01222222222222222`", ",", "0.`"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "0.19739555984988075`", ",", "0.14888994760949725`", ",", 
     "0.09966865249116202`", ",", "0.049958395721942744`", ",", "0.`", ",", 
     RowBox[{"-", "0.022218565326719054`"}], ",", 
     RowBox[{"-", "0.044415215246910854`"}], ",", 
     RowBox[{"-", "0.06656816377582382`"}], ",", 
     RowBox[{"-", "0.08865588186743747`"}], ",", 
     RowBox[{"-", "0.11065722117389563`"}], ",", 
     RowBox[{"-", "0.13255153229667402`"}]}], "}"}]}], "}"}]], "Output",
 ImageSize->{500, 135},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->628138808]
}, Open  ]],

Cell["\<\
Thickness and camber properties are used as input to the function \
AirfoilSurfacePoints used to compute endpoints for panels specifying the \
discretized airfoil.\
\>", "ExampleText",
 CellID->15047424],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"{", 
     RowBox[{"x", ",", "zT", ",", "\[Theta]T", ",", "zC", ",", "\[Theta]C"}], 
     "}"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"PlotAirfoil", "[", "rP", "]"}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->171426102],

Cell[BoxData[
 GraphicsBox[
  {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxTTMoPSmViYGAQBWIQ3RPx8Miif+/tTY+eP11WFrI/TPEM45b9b+w5FLbk
ue9P3L+cxezrpLaX9is8HVZF/M/fX881yTo/6Jn9lnNzuirvVO1/fuAg2yyl
x/Y2v7dLO/S07Bfs0r6y6819e4n0ImuZAz37Z80EgZv2Suf+S9SUTdr/kr2B
3S/lsn1VzeKfSr+n7X/wTW/vmpBT9o7sGsYr987eb/CgdtL1hN32a2NTzrk0
zN3PgAZYgrcuiL603T7R4euhy6Eb7ENsWMra7p2w153Y4zTZe5t9fdyzOAmm
y/ZBQnM+/v64wx5mPwvHO98GiZ32P6xeTV3E+cC+rDtc2qlxu33j0iXxZ5wf
27vtEay5d3SLvf8hq4mxVc/s3WJy+j5wbrTv1ftru2DNS/sNBakLEw+usHep
ODv96c039pvcV1++bz/bfldw/9x1DB9g4WcPALCSpoI=
    "]]},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2000000000000004`, 
      FormBox["1.2000000000000004`", TraditionalForm]}, {1.4000000000000001`, 
      FormBox["1.4000000000000001`", TraditionalForm]}, {1.6, 
      FormBox["1.6`", TraditionalForm]}}, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}, None, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}},
  ImageSize->{504., 180.},
  PlotRange->{{-0.05, 1.05}, {-0.1, 0.201}},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->337545223]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[{
 StyleBox[ButtonBox["AirfoilSurfacePoints",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/AirfoilSurfacePoints"],
  FontFamily->"Verdana"],
 " . ",
 StyleBox[ButtonBox["PlotAirfoil",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PlotAirfoil"],
  FontFamily->"Verdana"]
}], "SeeAlso",
 CellID->226809566]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->19724],

Cell[TextData[ButtonBox["1.1 Introduction To Airfoil Geometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToAirfoilGeometry"]], "Tutorials",
 CellID->775622],

Cell[TextData[ButtonBox["1.3 Airfoils From Thickness and Camber Properties",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromThicknessAndCamberProperties"]], \
"Tutorials",
 CellID->322803794]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["AirfoilGeometry",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/AirfoilGeometry"]], "MoreAbout",
 CellID->537983610]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"NACA4DigitAirfoil - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 1, 34.8936000}", "context" -> 
    "Aerodynamics`AirfoilGeometry`", "keywords" -> {}, "index" -> True, 
    "label" -> "Aerodynamics Package Paclet Symbol", "language" -> "en", 
    "paclet" -> "Aerodynamics Package", "status" -> "None", "summary" -> 
    "NACA4DigitAirfoil[id, x] computes a nested list of thickness and camber \
properties at a list of x-values for the NACA airfoil identified by the \
four-digit identification number id.", "synonyms" -> {}, "title" -> 
    "NACA4DigitAirfoil", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/NACA4DigitAirfoil"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[8225, 249, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->470438946]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 21730, 629}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[610, 21, 76, 1, 70, "SymbolColorBar"],
Cell[689, 24, 191, 5, 70, "LinkTrail"],
Cell[883, 31, 3382, 77, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4290, 112, 60, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4353, 115, 711, 21, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5101, 141, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->109997341],
Cell[5684, 160, 553, 17, 70, "Notes",
 CellID->1067943069],
Cell[6240, 179, 499, 20, 70, "Notes",
 CellID->923907834],
Cell[6742, 201, 1396, 40, 70, "2ColumnTableMod",
 CellID->401597287],
Cell[8141, 243, 47, 1, 70, "Notes",
 CellID->529540534]
}, Closed]],
Cell[CellGroupData[{
Cell[8225, 249, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->470438946],
Cell[CellGroupData[{
Cell[8711, 266, 147, 5, 70, "ExampleSection",
 CellID->56885981],
Cell[8861, 273, 132, 6, 70, "ExampleText",
 CellID->467046585],
Cell[8996, 281, 100, 3, 70, "Input",
 CellID->145894596],
Cell[9099, 286, 224, 7, 70, "ExampleText",
 CellID->63338921],
Cell[CellGroupData[{
Cell[9348, 297, 334, 10, 70, "Input",
 CellID->684504875],
Cell[9685, 309, 2153, 48, 156, "Output",
 CellID->628138808]
}, Open  ]],
Cell[11853, 360, 213, 5, 70, "ExampleText",
 CellID->15047424],
Cell[CellGroupData[{
Cell[12091, 369, 332, 9, 70, "Input",
 CellID->171426102],
Cell[12426, 380, 5425, 124, 201, "Output",
 CellID->337545223]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[17912, 511, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[18332, 525, 347, 11, 70, "SeeAlso",
 CellID->226809566]
}, Open  ]],
Cell[CellGroupData[{
Cell[18716, 541, 415, 12, 70, "TutorialsSection",
 CellID->19724],
Cell[19134, 555, 193, 4, 70, "Tutorials",
 CellID->775622],
Cell[19330, 561, 222, 5, 70, "Tutorials",
 CellID->322803794]
}, Open  ]],
Cell[CellGroupData[{
Cell[19589, 571, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[20011, 585, 155, 3, 70, "MoreAbout",
 CellID->537983610]
}, Open  ]],
Cell[20181, 591, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

