(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    372788,       7488]
NotebookOptionsPosition[    360693,       7087]
NotebookOutlinePosition[    362778,       7139]
CellTagsIndexPosition[    362691,       7134]
WindowTitle->PMDoubletConstantDirichlet - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"3.1 Introduction To Panel Methods\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods"], "\<\"3.2 \
Validation Example\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/ValidationExample"], "\<\"3.3 \
Implementation\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/Implementation"], "\<\"3.4 Flow \
Visualization\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/FlowVisualization"], "\<\"3.5 \
Convergence and Timing\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/ConvergenceAndTiming"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PMDoubletQuadraticDirichlet\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PMDoubletQuadraticDirichlet"], \
"\<\"PMVortexLinearNeumann\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PMVortexLinearNeumann"], \
"\<\"ForceAndMoment\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ForceAndMoment"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PanelMethods\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/PanelMethods"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/ref/PMDoubletConstantDirichlet\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/PMDoubletConstantDirichlet"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
PMDoubletConstantDirichlet.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/PMDoubletConstantDirichlet", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["PMDoubletConstantDirichlet", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"PMDoubletConstantDirichlet", "[", 
       RowBox[{
        SubscriptBox[
         StyleBox["r", "TI"], 
         StyleBox["p", "TI"]], ",", 
        StyleBox["\[Alpha]", "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]implements a panel method using doublet panels of \
constant strength and internal Dirichlet boundary conditions. Panel endpoints \
describing the discretized airfoil are located by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     ", and the angle of attack of the airfoil is ",
     Cell[BoxData[
      StyleBox["\[Alpha]", "TR"]], "InlineFormula"],
     ".  The function returns a list of strength parameters for the doublet \
panels, the strength parameter for the wake panel, locations of collocation \
points and a list of surface speeds at collocation points."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->116858593],

Cell[TextData[{
 "Collections of vectors are of the data type ",
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], "InlineFormula"],
 "."
}], "Notes",
 CellID->392463071],

Cell[TextData[{
 "The discretized airfoil is specified by",
 " ",
 Cell[BoxData[
  StyleBox["N",
   FontSlant->"Italic"]], "InlineFormula"],
 " ",
 "straight-line panels, where the doublet density for each panel is constant.\
\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["\[Mu]", "TR"], 
    StyleBox["i", "TI"]], "=", 
   SubscriptBox[
    StyleBox["a", "TI"], 
    StyleBox["i", "TI"]]}]], "InlineFormula"],
 "   for  ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["i", "TI"], "=", "1"}], ",", "2", ",", "3", ",", "...", ",", " ", 
   StyleBox["N",
    FontSlant->"Italic"]}]], "InlineFormula"]
}], "Notes",
 CellID->46829889],

Cell[TextData[{
 "There is one unknown strength parameter for each airfoil panel for a total \
of",
 " ",
 Cell[BoxData[
  StyleBox["N",
   FontSlant->"Italic"]], "InlineFormula"],
 " ",
 "unknowns.  Strength parameters for the discretized airfoil are grouped as a \
list.\t\t\t\t\t\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["a", "TI"], "=", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["1", "TR"]], ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["2", "TR"]], ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["3", "TR"]], ",", "...", ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["N", "TI"]]}], ")"}]}]], "InlineFormula"]
}], "Notes",
 CellID->156357560],

Cell[TextData[{
 "Additionally, a doublet panel of unknown strength ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["w", "TI"]]], "InlineFormula"],
 " extends from the trailing edge of the airfoil to a point far downstream on \
the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis.  This panel is used to satisfy the Kutta condition."
}], "Notes",
 CellID->61542297],

Cell[TextData[{
 "Boundary conditions at collocation points provide ",
 Cell[BoxData[
  StyleBox["N",
   FontSlant->"Italic"]], "InlineFormula"],
 " equations, and the Kutta condition (zero circulation at trailing edge) \
provides an additional equation for a system of ",
 Cell[BoxData[
  RowBox[{" ", 
   RowBox[{
    StyleBox["N",
     FontSlant->"Italic"], "+", "1"}]}]], "InlineFormula"],
 " linear algebraic equations.  The system of equations is solved using the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function LinearSolve."
}], "Notes",
 CellID->109779479],

Cell[TextData[{
 "Output of PMDoubletConstantDirichlet is a list of strength parameters, ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ", strength of the wake panel, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["w", "TI"]]], "InlineFormula"],
 ", a collection of vectors locating collocation points, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["c", "TI"]]], "InlineFormula"],
 ", and a list of surface speeds just outside the airfoil near collocation \
points, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[LeftBracketingBar]", 
    SubscriptBox["V", "c"], "\[RightBracketingBar]"}], TraditionalForm]], 
  "InlineMath"],
 "."
}], "Notes",
 CellID->73919694],

Cell[TextData[{
 "If a list of angles of attack is input instead of a single angle of attack, \
lists of ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["w", "TI"]]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  FormBox[
   TemplateBox[{SubscriptBox["V", "c"]},
    "Abs"], TraditionalForm]], "InlineMath"],
 " are returned."
}], "Notes",
 CellID->19186463],

Cell[TextData[{
 "The same example airfoils and computations are used in the documentation \
for ",
 ButtonBox["PMDoubletQuadraticDirichlet",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PMDoubletQuadraticDirichlet"],
 " and ",
 ButtonBox["PMVortexLinearNeumann",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PMVortexLinearNeumann"],
 ", providing a convenient way to compare results among the three \
panel-method functions in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 "."
}], "Notes",
 CellID->181230849],

Cell["The following options can be given: ", "Notes"],

Cell[BoxData[GridBox[{
   {Cell["      ", "TableRowIcon"], 
    ButtonBox["Epsilon",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/Epsilon"], 
    RowBox[{"10.", "^", 
     RowBox[{"-", "10"}]}], Cell["\<\
offset collocation points normal to panels to insure that they are inside \
discretized airfoil\
\>", "TableText"]},
   {Cell["      ", "TableRowIcon"], 
    ButtonBox["InterpolationOrder",
     BaseStyle->"Link",
     ButtonData->"paclet:ref/InterpolationOrder"], "1", Cell["\<\
specifies order of interpolation fitting velocity potential for computing \
surface speeds\
\>", "TableText"]},
   {Cell["      ", "TableRowIcon"], 
    ButtonBox["XWake",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/XWake"], 
    RowBox[{"     ", 
     RowBox[{"10.", "^", "10"}]}], Cell[
    "locates the downstream endpoint of the wake panel.", "TableText"]}
  }]], "3ColumnTableMod",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, 
   "Rows" -> {None, None, {None}}, "RowsIndexed" -> {}},
 GridBoxDividers->{"Rows" -> {{True, True, True, True}}}}],

Cell["New in 1.1   |   Last modified in 1.2", "Notes",
 CellID->918694056]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->686933026],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->107487309],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->77267041],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->326492779],

Cell["\<\
This example uses the Eppler-205 airfoil specified by a data file located in \
the folder Airfoil Geometry.\
\>", "ExampleText",
 CellID->48545141],

Cell[BoxData[
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<Aerodynamics\\\\Airfoil Data\\\\e205.dat\>\"", "]"}]}], 
  ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->317289154],

Cell["\<\
Here is the number of points in the data file specifying the shape of the \
airfoil.\
\>", "ExampleText",
 CellID->136872357],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NumberOfVectors", "[", "rD", "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->59487574],

Cell[BoxData["61"], "Output",
 ImageSize->{18, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->135961514]
}, Open  ]],

Cell["Specify an angle of attack for the airfoil.", "ExampleText",
 CellID->20057380],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Alpha]", "=", 
   RowBox[{"6.", " ", "Degree"}]}], ";"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->163234459],

Cell[TextData[{
 "Solve the model.  Output consists of the list of strength parameters ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ", the strength of the wake panel, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["w", "TI"]]], "InlineFormula"],
 ", locations of collocation points, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["c", "TI"]]], "InlineFormula"],
 " and the fluid speed associated with collocation points, but just outside \
the airfoil ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[LeftBracketingBar]", 
    SubscriptBox["V", "c"], "\[RightBracketingBar]"}], TraditionalForm]], 
  "InlineMath"],
 "."
}], "ExampleText",
 CellID->442126641],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "aW", ",", "rC", ",", "vMagC"}], "}"}], "=", 
   RowBox[{"PMDoubletConstantDirichlet", "[", 
    RowBox[{"rD", ",", "\[Alpha]"}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->53812608],

Cell[TextData[{
 "Flow visualization provides a plausibility check for model results.  The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function ContourPlot is used to plot streamlines for the flow field.  The \
steam function for the flow is obtained by the superposition of the stream \
function for uniform flow and the stream function due to the collection of \
doublet panels describing the discretized airfoil and wake.  Although the \
strength of the wake panel is included in the output of \
PMDoubletConstantDirichlet, the location of the endpoint of the wake panel is \
the value of the option XWake for the function."
}], "ExampleText",
 CellID->205450438],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"xW", "=", 
  RowBox[{"XWake", "/.", 
   RowBox[{"Options", "[", 
    RowBox[{"PMDoubletConstantDirichlet", ",", "XWake"}], "]"}]}]}]], "Input",
 CellLabel->"In[6]:=",
 CellID->544103304],

Cell[BoxData["1.`*^10"], "Output",
 ImageSize->{54, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->35643753]
}, Open  ]],

Cell["\<\
Append the wake panel to the airfoil panels and the strength of the wake \
panel to the list of strength parameters for the airfoil panels.\
\>", "ExampleText",
 CellID->33026124],

Cell[BoxData[{
 RowBox[{
  RowBox[{"rAll", "=", 
   RowBox[{"Append", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"{", 
      RowBox[{"xW", ",", "0."}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"aAll", "=", 
   RowBox[{"Append", "[", 
    RowBox[{"a", ",", "aW"}], "]"}]}], ";"}]}], "Input",
 CellLabel->"In[7]:=",
 CellID->81885161],

Cell[TextData[{
 "Specify an arbitrary field point using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation for a single vector.  The stream function for the doublet panels \
can be written as dot product of geometric stream-function influence \
coefficients and the list of strength parameters."
}], "ExampleText",
 CellID->19233943],

Cell[BoxData[{
 RowBox[{
  RowBox[{"rF", "=", 
   RowBox[{"{", 
    RowBox[{"x", ",", "z"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ic\[Psi]A", "=", 
   RowBox[{"ICPsiDoubletConstant", "[", 
    RowBox[{"rF", ",", "rAll"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Psi]", "=", 
   RowBox[{
    RowBox[{"UniformFlowPsi", "[", 
     RowBox[{"rF", ",", "\[Alpha]"}], "]"}], "+", 
    RowBox[{"ic\[Psi]A", ".", "aAll"}]}]}], ";"}]}], "Input",
 CellLabel->"In[8]:=",
 CellID->133172956],

Cell["\<\
Generate the graphics for a streamline plot for flow over the airfoil.  Some \
experimentation with option values were made to obtain the plot below.  The \
number of contours plotted was used to avoid a streamline very near the \
airfoil, PerformanceGoal->\"Speed\" was used to obtain a contour plot in \
reasonable time, and MaxRecursion was set at a minimum value to avoid \
noticeable numerical artifacts in computed streamlines.\
\>", "ExampleText",
 CellID->76299467],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"gStreamlines", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"-", ".5"}], ",", "1.5"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", 
       RowBox[{"-", "0.4"}], ",", "0.4"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"Contours", "\[Rule]", "18"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "False"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "3"}], ",", 
     RowBox[{"PerformanceGoal", "\[Rule]", "\"\<Speed\>\""}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"grD", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gStreamlines", ",", "grD", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", "\[Times]", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[9]:=",
 CellID->808226483],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJx0vQmcj+X3/6+04hNpL9GiSMQnKUpdfYq0F0VKlrSXJaWFFipLRVSyJEtI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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{5005, 5748, 5731, 5746, 5747, 5091, 6069, 6435, 6434, 6768, 
          5568, 5570, 7523, 5569, 4851, 6245, 5688, 4921, 6066, 4922, 7107, 
          5040, 7106, 5039, 6942, 6941, 6940, 7086, 7085, 7349, 5988, 6994, 
          6993, 6992, 5531, 6249, 4857, 5834, 7125, 5095, 6436, 5096, 5754, 
          5753, 6095, 6094}]},
        RowBox[{"-", "1.4500000000000002`"}]],
       Annotation[#, -1.4500000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{5008, 5749, 5732, 6391, 6392, 6390, 6389, 6091, 4939, 6395, 
          6396, 6393, 5006, 6394, 4776, 5623, 5300, 6229, 6230, 6231, 5792, 
          5943, 5944, 5378, 6058, 6059, 5649, 4856, 5969, 5833, 5744, 5745, 
          5572, 5640, 4835, 6412, 5019, 6214, 6213, 6212, 5000, 5717, 5716, 
          6084, 6083}]},
        RowBox[{"-", "1.5`"}]],
       Annotation[#, -1.5, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{5799, 5693, 4844, 6335, 6336, 6334, 6400, 6399, 6799, 7487, 
          5007, 7539, 6398, 7538, 6397, 6092, 4940, 6403, 6404, 6401, 5009, 
          6402, 4777, 5624, 5301, 6232, 6233, 6234, 5794, 6678, 6679, 6330, 
          4976, 6166, 6167, 6606, 6607, 6221, 6910, 6911, 5791, 6220, 6219, 
          6218, 5292, 5719, 5718, 6370, 4999, 6369, 6372, 6371, 4934, 6283, 
          4933, 6284, 6374, 6373, 4770, 7263, 5289, 6603, 5290, 6604, 5277, 
          5938, 6215}]},
        RowBox[{"-", "1.55`"}]],
       Annotation[#, -1.55, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0btLFmAUx/HjINnktXxfL5XU5tRf4KK4pIjgEF4SQbGliGqyocFJN0Er
qCHCC4WO5aUEQY2yMnPV0fS9WgotCvlx+PI7l985z8Pz1PXdb79XEBFjOK6I
eFQT8Z0+pD/oY5orj8hjoCxi6HLEE9y4FHEdA5UR/9UHxac0zddcG5GiG3wn
avXyb3b9SzoDy+K/9BM9op/53vHVJSKu4Q56kFM71P+DHD7y5+k6/1X9KxiR
N6m3OqNbfrE6Iqu2pJamizRD18zU6tdgWN6o3mKmS15kJqW2oLZP5+kBXTVT
rV+Fp/IG9VtmOuUXzGTc77n4BcbxDBMo5H2r1+ft3pv5Lf9Ap+ybRKIqIokl
3gfeKUGT+MW3jTZx2P+ad1N/S23O/E86S3vtva2X4kvjAPv44i+az/9KfIjM
+Rsiizfu02Nu2vxXe2boKzteosxdiv1fKS2h5fSuPSPuMIo587PYteOmM3bo
Kc8JOvj37O3nb7NrRf8MKsZj8Q==
          "]]},
        RowBox[{"-", "1.6`"}]],
       Annotation[#, -1.6, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0LtPVGEQhvFZ7KBilwUBIbHijwCUSwMdLCFaEZdLYigA6dUocl3AUpCE
BjsIS2ejiSTcxNgABVSAMdTgpUERfxRPnm/O986cOedu92BmIBERb/AqFXF4
O2KUj7iHE7yHmqqI+WTEvvNbbr8TkUEbFkoiOniO18vd8yee50xpxGNzNtSL
6vv6G9CIBXfHZl1VRDSpt2Q2UeRciH59eU67L8Gi/Hf5Vs9asGzeV/lf6Yjf
mJL/4XmOf95keLcs4ty51/5ZvRfOXeZk7ZvXf6B/jd95toQzmQ8yHzFbGTGD
YXP69U/YYRLP9NTqecp1/JyPZf7ov6fOemc9X6qn5F+6b1C/4EYe4RP5K/fN
6m75Jv6rzslPY0xmHKOYxARO9YRsAlW+91r+Hx7Yedr9DHJ4jVl8ky+QvYVq
+TLfsMJpXuZSfqg3718Uq1c5yWu8bZ8h/swDvMODfCk/zLvqJ/yF+/Q8MquX
U/ZJ4r13ptSdvMP/Af54Xpw=
          "]]},
        RowBox[{"-", "1.6500000000000001`"}]],
       Annotation[#, -1.6500000000000001`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwN0D1Pk2EUh/FTRKZitKWKtonxY+jKBAONWBMYHCTO8gF0YJCBxDcEJxlo
i5q4KPIFShugRnRDa1+UFxtfZsNAEP0NV67n/M+5z33nuTQ5NXY7ERGz+J2K
mB+MWMBjPMEcXmcjXurVLkRU0ac+iV+y7+cjutjFPvZQOxvxQm/NbAW9Zk/g
p+yb/g7a+IoOquaf62XcNYACruGHrKnfQgOrAxFfeM18Wj+Fu+or8n/2H2NM
1sOf5W/l27zCn7ji3Bn907ijviz/a/YIV2UJfsQj6YgHPMwP+T7/4WXvOcj4
V+pRdR5d2Q17L9r3Rr2CJPpxCiV31PXK/I63nN3kono8FzGBhH8UmPbddqaD
Fpqon4sYcqbsbctYQglFPHN3wZnrqNq5aGeN6+b77Ut6a8X5Gd95b7wlz+lv
cJbX+b09T/Uadh2ae+XOSfk9+U3+wB+xhU31f2RVXYg=
          "]]},
        RowBox[{"-", "1.7000000000000002`"}]],
       Annotation[#, -1.7000000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ctLlWEUxeGdOEgkA2+pqeSscSpWXgiicSgWYYp6Ch1oOu0iUZBHAydd
MfHSLA1LLE2bFAQNKkotDXRwKv+FBl6a9Dj4sd717r3Xfj++skRvQ8++iBjH
+eyI9vyIBL7kRKwejvhMR/hNtUH+LvoxgCRaiiPa1H7mRbxFxaGIcqTM7ar/
wxZ2sI1m/a361/RmYZo/QF/QJ/b8UVsskIV5LOANzrlPFtmL1/yQrF475pwz
ndvM9vCzdl+hN0vscfeRH5U9hkv6LqNC/SlfSaf455jEIz6jMGI/Dsrt5l/R
r/SIu2+0jKZ7Q6f8Y/yEnHKaMN+O93i3t8dcs/1dai+9YQYX+SY0+pYLtIF+
cF8qswSf9J6R22f2lIwbdDM34o4daXae5pN6ftNdetRMyvkXalGHlMzb+pfN
r2AJP/AdHWbu2ddJ6+2v0b+h/7HzOq3mT+I6riFHX425q8713nXfW87K3vaW
HTw0d0LtOKrwQP2vfzksa8zcLfOjdALje/+If6ZvCn3yJukq/gNjQWOb
          "]]},
        RowBox[{"-", "1.75`"}]],
       Annotation[#, -1.75, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ttPz3Ecx/F3QsjUKsovLoxqzOkmJtSFs2LjkrvYXNgc5lAySrUVV06L
zZlVjAubCy30Y/wBzIUJFTPnzWEzYQ6P38Vzr8/7/Xp/Xp/P5/v7TareunZL
WkT0Yn9OxOtxEfX0SiLiMiIv4lVhRNnYiCnq7RP5uRGn9NLVLRMiWlFfELFI
r4EuNvPATGJ8xO38iDtYLXeN3GZ7kuoac7uRY66P14/nzrrI++isT/hn/1/U
2VfEL0aTfpWMVViBSqxEnpxGXpqz0zEEDeplvOVYggPqpXSTnA7rXDOdNI8O
Ovukd/yibfQnPUETcgesm6xf0gr3K0eFt5ZjIfr1G/mlspK0jr8X7epq523E
BhSmspz9wp4+TE29T28abafNvE5zHZhr731Zf/Svy8rnFaDMeh4eWj9CLY6b
6dbLtL4ptwtFKEYG7y5t9a0PoiX1++gdoiX68+177P5dNGF/IbY5e447lGKz
uUH3+4HRvG57bmE6ZiBT1r1UDm8BLqlnqmdhNrJ92yxUye/lP8MN+ZPpKLP7
Ut9c9m/0mn+K7/ol6jZ6mP/F/Sr1q1Ap57P6mP43eoR+pUfpgP/JW/oOb/AB
79HBX0fb6Xp62twT7zpD0+kQXMNVjPHmHc7eiaHqHvVZcz3ue44O1xuW+u/y
dyGLX0NrkaGfVJ83lzR/gY7UG4E6/h5k8/8DDZyANg==
          "]]},
        RowBox[{"-", "1.8`"}]],
       Annotation[#, -1.8, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV00lsTlEUwPGLEl0Qc1HVlqDVyayhkhojLNCaac3sEBoENduK0lJCKFZs
0BISNS1NRT+Cjxi7NESiJNrwe4t//++ce8659773NX3lhqL1bUIISf7M6BbC
zF4hTO8Xwv4eIdzoE8IB/iz3CSN6hzAcm/qGcFA+IymEqZiCJXIVPUNYzEd4
KY/nAgxKCaHc7INm7OQPySF8xHv86x5Cgv3ao1Btolkd8Vxtjtwq+2Tzas7l
FvVdra8RX+Dven5grfgoj7ZXippp1vrzGHGTtUzzqqzni9Pkq6yn8lhxpfxP
3MJ0cYL6OuvtuUlfm8jO2i56F9yWv3AJ17jnWZzBH/Pi8r95gLMORCrSkRbd
z6wMzsQQLNM/lP+q72K/5eLz/NU5vmEM7lpfIZ/FK/kBP8Qi51yICWoG27vc
8yCO6c/23bKQLv5oz0nONBHlvt0h9RPVFOK4tWqcNXe+mWd4AV/2jRrkK8w4
gsOYGb1fLjGzk706Y56Z85Hp/NXmZfBccTHKrNeIb3rOwDnPuXKJ3GpWR84R
x6LviO2ed2AbZok7OMdsjjlHI07Ye464mov4JJc6y2n3Ga8ny7wTzjCUx4lP
ye/Vtye6o7rjuG1mjnw9Z3MDP8UjPMFj7HH3Z7yXL+m5iA7Rb0/9AHM3y+8z
s4wHivPky+y7GSPcczg2eM9b5SfrjevdZd5utDpzC17L1ap/wwX8iv9Y36Ln
F/9GM4rtUWyvl+btc68XvJ+v2eM6rqIOtYirvWfOWn33eR3H1dbrTzb3irgf
X+VK68XOfjt6lzxS/MuMUbxR7q2+POeqEpdE/4v6GnFMfMf6Vj1jPTfryedN
cu/0DIt+U+Jl0W8ZMT2lzlXq+T9OqaA7
          "]]},
        RowBox[{"-", "1.85`"}]],
       Annotation[#, -1.85, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV00dsj3Ecx/EfqkRLjdasWm2NQ43WKAkxDirU1hqxDqXrX+OAoCctYkZt
iZW2TiQkdh3sPSoRnDki1sVoeT2Hdz+/7+c7ft/n6f/pt7JiTqxFCCHLn4SU
EH51DWFUcgilPUP4102iewgNvFviBjTzRsqv7RFCO/UVdIT4D/8vmnBD/U21
N/BbPFw+pu41Hc0bgzJz03uFENclhLfq3zl/d85Q94Nm0iG8wXijb1NqCNdw
FeVmLuQXogAn7dGpN9/MGN6Z9x4D+f34FZ1DqBTH6LfoHnzFT/xAlvnl7htG
y+hQ2tk9XfDYrk9w3Nx8exWpX4WO6pLQ4Hze3BZqp+oLNI9OcPcid4+nF+Q/
8+aJv9Bp9s9Dunm5vE+8Guc6s86hHvHiXWpa0/52f+280ayh6tvzdos70GHi
Tfx0NYni5XY9LV5Gz9AV9BTNkc9GEfby9uChGRfkSpx70kSzjtEEepR+t/dY
uXXOuXQ9HUfXRu/VnL4oUdNozhZetr1zMBJ7eWPoaIxCs/fShCn613iXtXZt
7/2liXtjg/ol6hZjEc6al6HusvmJcoOcByIT072v22bdQa3eOhwyK59/kB7B
YcwQF+stwQPzVtNVKDZzpp5ZyMcczMZ2NfGevQ1aY4e4uz3nyc3HXBRgAXbK
DYp+Y9hsXjkvFv2ucTP6LqLfP23jvni0smtLnLBXob1SnAtoMq1GFT5E35W+
j3SSnkrPmur+tna5L/8A9/AID1EvN0B9rnv6R98Vfczfoncr9osL+fvoQjqZ
1w4JuM/7i0vRtyG+SP+Ib6FMXIorZqUjAy/0v8QddUlyd6P/u/gVlto3U803
vdUYbt8Uux1QV4MqXhxvG+3g+Z95rud4ipd4gUnqq+hEWk0b8Qo17smL3pE7
ruvdTvuYmYYc+Wz8BzIUo+g=
          "]]},
        RowBox[{"-", "1.9000000000000001`"}]],
       Annotation[#, -1.9000000000000001`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV01tM1nUYwPGf3XiVicLL++Kx5QrcNIVwM9OltVnKKedh5qEFoiQbeADU
Gpp5hxFYtBagbnksSy90U8BDbSVONy+yRFNx09QrTVti2PHzu/jy/T/P73me
3/N/ed+nS6vnVA0IIczx59jQEIamQujgdF7AvcNDuI7bWSHcwdkhIYzIDOEc
70mGsBdbhoXwAd5Sn6nvFGfwSU7ww/QQWszo4095o3yr+npeNSKE7ebuwDXn
ueKE/FR3ZHKeuFd+YCKEAr0Nnu+rfYDf8Yn8WbO+VZt013ec4k40qu2x5798
kTc7W4t39OzS+wX6xVf11/Az3mUMjqhN03+Yc9XkYSKqM0J4gau4wX5PxLvM
aueX43vwYDQ6e1fN3/Z9j/v5LzxGufoU15rdatbn6NNzTf4quuTPyVWYVxJ3
tdvH8T2xTnzdvF7sdMeA+PnqybFzY9xNT0quxVmD+EOUym3lenUrURnrxR12
yOZOrlJ/zL2XPRfLXeISfl9+EzqdFYln4Ednr/AFns49/Cr/xBfxMzaor5d7
290b415Y4DmP15uXZsfbGCVXprbCTq85Wyx+nWdiNpaIZ/FN3MB8/8dsuQ49
g8yZJy6UXypXwJfj7vgFbzorkV/EC5FpryQSWCxeZ8YW99bxGHW5eurMrMVT
divEaPllzms9T/Ncx8/hMzXP8lj9X8Z30XsLbyBH3Yn4XVNzPP5W3DcSWRiO
YTiv75CzH/xPv8cs+/waPxu5r8ybYcYBHh/3MGcc/6HmipoVapp8Jyp4m97m
+F0RF6sp50l8EO3ybaiUW4l/1H9t1jfIcH86jso9KT6jrhunMdc9aXac4K6J
2GfWdPvsj78PtKlvR7ezIv1LkB9/T+Ih6nZzs/MmTHHvi5iM4Ow/c9f4rq7G
82Zt5tHuHIUN8bMX12AtNpmzRm41qrEqnsu9pLaGp/L9eLf8Ay7TU4pydyzD
Q7/ffPFdd9/Db1gu16f+ETrsfFz8kfft4iY+wY/MnaRvtn3HyVXK/am+H116
xnOz/D7exkXxf6onZacsTvL/PFbDdA==
          "]]},
        RowBox[{"-", "1.9500000000000002`"}]],
       Annotation[#, -1.9500000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0VtMz2Ecx/FvZBRhTh1l3CUUF7okGzZmGeaC6sJhHXTY3MZNNkS4cbxg
XDC2NmaUzZxuHLbaHC5M2Eoh1YQibA6v/8V779/n+3yf5/k+///cbfUb6pIi
4iUapkecyI44hZPYLH9Ij3iYEfEIT/AYzdMiVuRE7JkRUcx7OUdfNt5nRfSi
Bw3qy61vcc5p551Bv/oA2p0xyB28Vs9BvWu4iXu4F114h24803eEj6IZA/qO
cR8f5n5u4k98iEe5f6aMC+YqNMPSWRHlZihDi3yH09SuWC+Si3xXeF8l9qsd
QLn9BeYs40JuMUey+jisdkfJ7IhVfFy9zf40d0/GJEzFFLRa+2P/X1xFF2rU
kvTXcrd8DSH/42X8i3/jnHvO4o3vt4kec6ZY/2zfZfUhTpXnmXkM35XPq3/U
24cH8kX5EpKtj8V9tUb5Hnck/gu8Nmcnnqq36xlwz23fg9whv+J05+V6b6l9
xWor5a2J9/NGv0Gq/RMxHimYgF166/TWY5O+YXeN4Ll6nvNfcK61ObhufTbn
YJ16Pi/ADfVFvDCRzbGYl+Bm4j/hAuzWP+y+GrVvXMsjXMVfE3PwEFfzFy7R
n2n2NjmDb3EWt/IP531HnjftyIzYie3YZ+5GrLe3Uq5CBebrq+ZOs/1MzCiX
els+jyb+E9//AXT7hQc=
          "]]},
        RowBox[{"-", "2"}]],
       Annotation[#, -2, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0DtIlWEYwPFH3BIyvOs5Qg45nKNrJIRWVg7RkhFKg3kcM8tF8xxRoSXI
oIsauWSuTlHYRSlSjKYyTB11KKghMkijzd8Z/vzf5/q+31eXuX6hryAi5rBe
EpGsjXhbHpHgd1zLz7ixMqIBryoiHul7XeOMPeziq1xKPY3xsojzyYg7PCU/
b2aSX+Z71TPVEd34jxa5E2jGKZxE2n0L5hcxKZ5Cv/lRe27whHjNnpS+pPPf
0ohhvZvuy/IG57hJ/yCvigf4C9/k3/o31HbyTkT84U2e4Xo7L/nmI7yup1Cu
QfzZ7HHnD5wS//OOj+rf5M6Kv/NjHPBN01zECTv39B311l2uEVfjmjf0Yg7t
ahex7L55LrbzEHrEVXorUYEV917Vf1DtF2fUy+TLcdtcl7gbXd5yhc+5b8k7
WsXLvIBb+t7wfTvu4ZOep1URs8jZk8Vl9SEeMjfizlZ3/fCPf+Kw+TqMyZ+W
H+UzfNeeB/n5/G6ewEN0ip/Y3cEzPGZHm942dw8755DFC/UV/c95XG1bbgt9
9h+zP202hSW8R4l/uw/8JWeE
          "]]},
        RowBox[{"-", "2.0500000000000003`"}]],
       Annotation[#, -2.0500000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0L1L1mEUxvEjkZBtz2OWWailvRCCW6BWCmHPoKUOLmVETpov2Oy7bYmK
g4aDUTpFVBZuig5i5KLi4lKrpVRY/gF+fsOX6z7Xuc79Vvy0u6krKyJGUZOK
OHEm4s2FiLdow1puRDtdoa20Wf+Alp2PeHIuYpV/lB9xXX0NqbMRaUzLnboY
MUVzaIb3Mi/iqzN+yH1Hp9mb9jrJz0aPulL9nFbRxXTEeoEZ9Mn34675jGyG
1jv3PupQaOYB3THzQf+n8/ZQxG/gnzabg0brJvzSW1CP2eej/Dgdkq117jC9
rH8Jm/zfcn/wGV8wwXvvTq+cNeIPbtFhepu+oH9lyhPfux/x5qz37XWAWfUn
s6/pAr2TvJG/IfNMvhMd6OV30W60oUXdK/+YzsvesK5w72r1THJP/7yFUusS
HOr9wzveEf2PXdkreleRx9/GQ2+558399htI/hlDGMSW3qS5JXPf1Mu0Xl2X
oHcM8o5TXw==
          "]]},
        RowBox[{"-", "2.1`"}]],
       Annotation[#, -2.1, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0L1L1WEYxvH7DEGBZKSmZg7nmNafkGGKkNoJUqnMlzLFocCh4xtWY+Cg
gWaBQg5NpTa0hJ7+gsBBN21vSexFrdwK6/MbvlzPfV/Xc9/P75ceyF1/kIqI
OVQXRcyXRrRVRLTiGhZKIgbLIg5PRkzxqvQySOM+7x7vgDfBO6FXiOPo5G3o
DfDGT0U8xJfiiMu8JjSjhZ+2M2RfnYlI0fZkPy7o11VG/HPnDzqc/9KXctu0
xf0ryOKp2Rn5q84fveex86zzI7oov4TXOGL+GzotP4Mhc2rVOXqRTnrrO/2l
8ohlLGKU95Z+Ph1xSaYeX83eQSnvGy2jjTKb9pV7+5i6mxbbt44ecxvcG5Id
Rg6jGMEa75mdeXfX1B+S71BnE3g3ze3ADXTiFrZk9pNvtqcn+Yd0T/3EG7v5
Veouejbx6Sf59/wfst9RrX9b/6i33aE16l39FfdXsSL7U72PPZzj98odk79L
z6t/6edln3vjqvwLeqD3G/0yfSjg/wdWVVYH
          "]]},
        RowBox[{"-", "2.15`"}]],
       Annotation[#, -2.15, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0D1Pk2EUxvHDgpDAom21pQgyMKD4yupLVBxgsiYsbiaVmkgZsK06YkhQ
P4JfQFuVvqh8ALowaIyGdMEgIFvDZ/DX4Z/rPtc557qf5z73aDlX7IuINeRP
RnSSERPIn44YH4kYw2P1cTZi2HkIW4mIgvpZJmIFjVMRbf59umNv60zE5GjE
qry28yt61fw1XEHB/gxdpAnzv2RcUl/GkpynyMkq0gf0nYzfZtpy1lMRr9Hk
l3hlVHCPN6t/Ix1xE9eRkH+L/jTb1U+pF9yXpB/cVcV79Pu/2+bS/D39DXl1
1Hp3ydzn/cVdM3cwqLdr7w+mnC/gPE7IKdAnGOjVeGO/Lqdm7yOq+IxP2Nab
9m8XcSD/EG95DfN1/ZLv2aBl2qAHvXcyO+dts7w53j87R2jZaakr/CZ9Tr/R
TXzBC/XXXpbdUeeutz1L53kd75OR8VL9kH73Hz+QxH8blk11
          "]]},
        RowBox[{"-", "2.2`"}]],
       Annotation[#, -2.2, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0LkuhHEYxeFXwSBRWMYyQ6HSWAsi0U1s0ygsQWWrLKGxdRrhHpgZSiQu
wAUoUBHb0EtIuAbPFL+cnPNu/+9rX96a3CyLiBUM1UectEacYiEdsYg+2URT
REcqoqIlIoGl5ohqtQKdUc/RaZqnB/p+GyNSDRFz5tL0j39SHzebxQjGMFqi
LmLHrl1sYx97uJU3m/+07wNFTCXN2jWMJrV5PQ+yazTySTy686Ne5+4NX0u/
+Q16r+8Os/J1/kp+iSL/jkHvqZFn2yI6fcuzmy84k58jhwLyuDB36P6Af7Vq
pp+u0WP5EcpL/wt9eqvkr/a/4UstwffKK2kX7UE3MvZlSm/FP5/INao=
          "]]},
        RowBox[{"-", "2.25`"}]],
       Annotation[#, -2.25, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwNzksug2EUh/EjMUJc+rkEFboBA5dKEZPu4ItBW0akjDCSsIAyNzGukmi0
JqzBEhDKUJuwCL/Bkyfn/57Lm9s7SY/7ImIbW5MRBzMR9xMRVW7yIVe4NhWx
w7toJxHJbEQGY7hWP/AVP2cjcnqeeIH3zc3zoj1l/pOXuKN+ybg3F3E77R7f
cR0N3KDlP5d6vu1IzbyaecM7UlnDrq67PbRlj7jQ/2uuxi31l75kPGLTvn47
NnhU/SH/RAdd/T2MyAveh3mdz82f4dTfBmU/7gzxmrcBXuU8lrCCZRTdLdp1
hH8iiDEQ
          "]]},
        RowBox[{"-", "2.3000000000000003`"}]],
       Annotation[#, -2.3000000000000003`, "Tooltip"]& ], {}, {}}}], 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdk30wG3YYx5lT61Adxm3OMK509dbSpdrVd5ptZah5K/Uy0izGKMtWi3dV
VFALahKltqxa6qWaeK3L/NRwipY2Y7KezVuuWpWE9Mpym0629Z89d889z5/P
53Pfx+pkUgDjJQ0NjcNbrZ7/lgIvNhUtTxG7IEecebDvzuTnvUwz/VmHUjkM
pp67jXEpxEYmeMrXlSPd+PrRubQgsr3Igi4Lk8H3ARHer/uUNJ7197p0ZgWD
nL0sB6sk0npkxK8x8wkiSTQlIfw0KTCR3F0PXMZ0p12bx65Ucqln7UStzmM0
t0xHfe2XRSoklhfO1yxhqOgNn58UOcSe8fp8qOlDmDjnhLOUeeTOIyFvOkOK
Ff1e56q32WQgI1Xz2b0FVAr1mM3FxWT5+LiX3GweEc0Nt1z1OGShP6fUPnoW
y8V5nTXeZWSHUqo9fXQGtASrYY/tF4iiaDZ7j0SCxLj9W2gVxIJ0PRrPmoKp
aGmPp0slKZQPNJtSxVg9I9wdksIlB60/Ss91moBdd6h/bhyP3AspjbG/OAIp
jT/lKucRup3tr8GegwjIClgRr/IIoySN/SGrD1X6+24Pn+CR1d0aQy6x3Vg0
2FabbsAlV2kbbJ6hAIbroaoJ7W9JtiikxWfjCjT3dyS0OJUTo2pOPHWQh3Nh
YtGCcQlZzK290ziej/c+N7JdP5RHclWVbpEmdHT2esaO3jxNVmMX2B0bT92d
Ei9rirSPoPXlza5y83hoHBzjmkZnw/47aeoXrHPwYQZLWB+UvuCC6v1am0iX
KpgUuJCLWnX4R58FH94WPd/3Mq7jK+8mKybnCn4xXPMYj+mA5f2kKJ2717C4
/s5fEo4ITEsbXV5fC7Sox7vSjvWDflLJPNVwA6ydhwKXREPo2xX/M9VViDUd
rfEfC0aRXcfWa0ppA4X5JaH6T8CTZJof+KwdiT5+ofOaYoS4jypnZe1w+SOm
2kw8Ca98Qb6lTgeSFPm3g25Nw7zCWfystR2r9LFI7+EHKI/YpKpW2sDOV4v6
DZEf/76vflSInr2HaYU6cxAMaI1cjRCAEnReSeHPQ2/4E/lmSitozAPKJrdF
DAgct4w1o61DLVIKo+5kil9yA9THO/o/xLvrbzq6yerA72Uc4w4uoSGgTScs
hw/2q/2XS6wfQ53OoIIa5M7cOOUfswxfh/CtyFVCUT/kTSl8gg21JuMy1Pxg
ffPPghWUVc9ty8hiYzh9ciQvSobkbzQlyzPZKHuFruuxQw7thmuM+Ewm5riq
1MAyOerfem3I5GzEf3xySG0n3c05bvj///4NQNjiEA==
     "]]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  ImageSize->504,
  PlotRange->{{-0.5, 1.5}, {-0.4, 0.4}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 224},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->2581982]
}, Open  ]],

Cell["\<\
Note that the flow appears to leave the trailing edge smoothly (Kutta \
condition).  There are no streamlines inside the airfoil indicating that the \
computed velocity there is zero, as it should be.  The streamline nearest the \
airfoil shows some waviness, and increasing the value of MaxRecursion in \
ContourPlot does not improve smoothness of this streamline.  \
Constant-strength doublet panels are equivalent to contra-rotating point \
vortices at panel edges, and it is the effect of these point singularities \
causing the waviness of the near-airfoil streamline.\
\>", "ExampleText",
 CellID->14395635],

Cell["\<\
The function ForceAndMoment computes airfoil lift and pitching-moment \
coefficients.\
\>", "ExampleText",
 CellID->80454523],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl", ",", "cm"}], "}"}], "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Doublet\>\"", ",", "a", ",", "rD", ",", "\[Alpha]", ",", "rC"}], 
   "]"}]}]], "Input",
 CellLabel->"In[10]:=",
 CellID->499718542],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.9980191091077664`", ",", 
   RowBox[{"-", "0.3066586747308935`"}]}], "}"}]], "Output",
 ImageSize->{149, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[10]=",
 CellID->227599051]
}, Open  ]],

Cell["Locate the center of pressure.", "ExampleText",
 CellID->8410041],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"xCenP", "=", 
  RowBox[{
   RowBox[{"-", "cm"}], "/", "cl"}]}]], "Input",
 CellLabel->"In[11]:=",
 CellID->342608962],

Cell[BoxData["0.30726733780182597`"], "Output",
 ImageSize->{60, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->203117986]
}, Open  ]],

Cell["\<\
Note that the wake panel represents the starting vortex which should be equal \
and opposite to the circulation around the airfoil.  For the dimensionless \
formulation of the problem, the lift coefficient is just twice the \
circulation.\
\>", "ExampleText",
 CellID->119738758],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"2", "aW"}]], "Input",
 CellLabel->"In[12]:=",
 CellID->288057467],

Cell[BoxData[
 RowBox[{"-", "0.9980191091077661`"}]], "Output",
 ImageSize->{68, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->643710237]
}, Open  ]],

Cell["\<\
Surface pressure distribution is computed using Bernouilli's equation.\
\>", "ExampleText",
 CellID->753558053],

Cell[BoxData[
 RowBox[{
  RowBox[{"cp", "=", 
   RowBox[{"1", "-", 
    RowBox[{"vMagC", "^", "2"}]}]}], ";"}]], "Input",
 CellLabel->"In[13]:=",
 CellID->435840926],

Cell["\<\
The function PlotSurfacePressureCoefficient provides the conventional plot of \
surface pressure coefficient versus the chordwise locations of collocation \
points on the airfoil.  An inset showing the shape of the airfoil and the \
location of the center of pressure is included.\
\>", "ExampleText",
 CellID->139054009],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"xC", "=", 
   RowBox[{
    RowBox[{"Components", "[", "rC", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"PlotSurfacePressureCoefficient", "[", 
  RowBox[{"xC", ",", "cp", ",", "\[IndentingNewLine]", 
   RowBox[{"Epilog", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Inset", "[", 
      RowBox[{
       RowBox[{"PlotAirfoil", "[", 
        RowBox[{"rD", ",", 
         RowBox[{"Epilog", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"EdgeForm", "[", "Black", "]"}], ",", "Yellow", ",", 
            RowBox[{"Disk", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"xCenP", ",", "0"}], "}"}], ",", ".012"}], "]"}]}], 
           "}"}]}], ",", 
         RowBox[{"Frame", "\[Rule]", "False"}], ",", 
         RowBox[{"PlotRange", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"0", ",", "1"}], "}"}], ",", "Automatic"}], "}"}]}]}], 
        "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", "1"}], "]"}], "}"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "1.1"}], ",", "2.2"}], "}"}]}]}], "]"}]}], "Input",
 CellLabel->"In[14]:=",
 CellID->671472236],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3V+IHdd9B/BtWtNAU1tvdfIkPzTVUyvahyoLTQT9g4UfHBVCDEKt6sJF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    "], {{0, 324}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 324},
  PlotRange->{{0, 504}, {0, 324}}]], "Output",
 ImageSize->{508, 324},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[14]=",
 CellID->155318168]
}, Open  ]],

Cell[TextData[{
 "If a list of angles of attack is specified, lists of ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["w", "TI"]]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["v", "TI"], 
   StyleBox["c", "TI"]]], "InlineFormula"],
 " are returned."
}], "ExampleText",
 CellID->571494656],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Alpha]List", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "5."}], ",", "0.", ",", "5."}], "}"}], "Degree"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"aList", ",", "aWList", ",", "rC", ",", "vMagCList"}], "}"}], "=", 
   RowBox[{"PMDoubletConstantDirichlet", "[", 
    RowBox[{"rD", ",", "\[Alpha]List"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Dimensions", "/@", 
  RowBox[{"{", 
   RowBox[{"aList", ",", "aWList", ",", "vMagCList"}], "}"}]}]}], "Input",
 CellLabel->"In[15]:=",
 CellID->42630738],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"3", ",", "60"}], "}"}], ",", 
   RowBox[{"{", "3", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "60"}], "}"}]}], "}"}]], "Output",
 ImageSize->{153, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[15]=",
 CellID->57343977]
}, Open  ]],

Cell["\<\
These results can be input to the function ForceAndMoment to obtain \
aerodynamic properties of the airfoil assuming the angles of attack are in \
the linear range of the lift curve.  Listed below are values for lift-curve \
slope, angle of attack at zero lift, location of aerodynamic center and \
pitching-moment coefficient at zero lift (couple).\
\>", "ExampleText",
 CellID->24035690],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl\[Alpha]", ",", "\[Alpha]0", ",", "xac", ",", "cm0"}], "}"}], 
  "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Doublet\>\"", ",", "aList", ",", "rD", ",", "\[Alpha]List", ",", 
    "rC"}], "]"}]}]], "Input",
 CellLabel->"In[16]:=",
 CellID->34907031],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"6.7180835115881115`", ",", 
   RowBox[{"-", "0.04402568206056107`"}], ",", "0.25662302616339816`", ",", 
   RowBox[{"-", "0.050645916659026485`"}]}], "}"}]], "Output",
 ImageSize->{298, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[16]=",
 CellID->617452414]
}, Open  ]],

Cell["\<\
Note that the location of the aerodynamic center displayed above is close to \
the thin-airfoil theory result of the quarter-chord point.  Display the ratio \
of the lift curve slope to the thin airfoil result of 2\[Pi], and the angle \
of attack at zero lift expressed in degrees.\
\>", "ExampleText",
 CellID->47127959],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"cl\[Alpha]", "/", 
    RowBox[{"(", 
     RowBox[{"2", "\[Pi]"}], ")"}]}], ",", 
   RowBox[{"\[Alpha]0", "/", "Degree"}]}], "}"}]], "Input",
 CellLabel->"In[17]:=",
 CellID->274981366],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1.0692161989734064`", ",", 
   RowBox[{"-", "2.5224857722549707`"}]}], "}"}]], "Output",
 ImageSize->{135, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[17]=",
 CellID->24842389]
}, Open  ]],

Cell["\<\
Consistency between these two approaches of computing aerodynamics properties \
can be checked by computing the lift and moment coefficients at 6\[Degree] \
using the linear properties. \
\>", "ExampleText",
 CellID->328772922],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"cl\[Alpha]", 
    RowBox[{"(", 
     RowBox[{"\[Alpha]", "-", "\[Alpha]0"}], ")"}]}], ",", 
   RowBox[{"cm0", "-", 
    RowBox[{"xac", " ", "cl\[Alpha]", 
     RowBox[{"(", 
      RowBox[{"\[Alpha]", "-", "\[Alpha]0"}], ")"}]}]}]}], " ", 
  "}"}]], "Input",
 CellLabel->"In[18]:=",
 CellID->762946865],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.9992842689444069`", ",", 
   RowBox[{"-", "0.3070852697530192`"}]}], "}"}]], "Output",
 ImageSize->{149, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[18]=",
 CellID->154679881]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Scope",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->1293636265],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Alternitives for discretization",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection",
 CellID->421192202],

Cell[TextData[{
 "Fitting a B-spline through the points in a data file provides a means for \
alternate discretizations.  In the current version of ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ", this is accomplished by the function FitAirfoil."
}], "ExampleText",
 CellID->117609193],

Cell[BoxData[
 RowBox[{
  RowBox[{"spline", "=", 
   RowBox[{"FitAirfoil", "[", "rD", "]"}]}], ";"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->559405563],

Cell["\<\
Graph the data and spline fit to check that the spline provides an adequate \
representation of the airfoil.\
\>", "ExampleText",
 CellID->299806311],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"grD", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gSpline", "=", 
   RowBox[{"ParametricPlot", "[", 
    RowBox[{
     RowBox[{"spline", "[", "t", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "0", ",", "1"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"grD", ",", "gSpline"}], "]"}]}], "Input",
 CellLabel->"In[2]:=",
 CellID->249332617],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3f+PFPUdx/Fr+pf037hEueMOFAk9D4vYWmrhZAtt1Wo0MagRalsIQrrx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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->906358222]
}, Open  ]],

Cell["Rediscretize the airfoil using the spline.", "ExampleText",
 CellID->27068391],

Cell[BoxData[{
 RowBox[{
  RowBox[{"n", "=", "60"}], ";", 
  RowBox[{"layout", "=", "ConstantSpacing"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"spline", ",", 
     RowBox[{"NDiscretizeUnitSegment", "[", 
      RowBox[{"n", ",", 
       RowBox[{"Layout", "\[Rule]", "layout"}]}], "]"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->343427282],

Cell["\<\
Graph the endpoints of panels for this discretization and the spline.\
\>", "ExampleText",
 CellID->126580234],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"grP", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rP", ",", 
     RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"grP", ",", "gSpline"}], "]"}]}], "Input",
 CellLabel->"In[4]:=",
 CellID->274596420],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3f9vFHUex/Fe7i+5f4NEaSmoXA9r9biaYEWo7omgnp7kLkUjmlOMStzA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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->24518823]
}, Open  ]],

Cell["\<\
Solve the model using this discretization, and compute the airfoil lift and \
pitching-moment coefficients.\
\>", "ExampleText",
 CellID->33075010],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"aP", ",", "aWP", ",", "rCP", ",", "vMagCP"}], "}"}], "=", 
   RowBox[{"PMDoubletConstantDirichlet", "[", 
    RowBox[{"rP", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"clP", ",", "cmP"}], "}"}], "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Doublet\>\"", ",", "aP", ",", "rP", ",", "\[Alpha]", ",", "rCP"}], 
   "]"}]}]}], "Input",
 CellLabel->"In[5]:=",
 CellID->252090154],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.9324043841538483`", ",", 
   RowBox[{"-", "0.2756074283579629`"}]}], "}"}]], "Output",
 ImageSize->{149, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->189895069]
}, Open  ]],

Cell["\<\
Compute the surface pressure coefficients for the user specified \
discretization and plot in red.  For comparison, also plot in larger blue \
symbols, the results previously computed using data from the file for \
discretization.\
\>", "ExampleText",
 CellID->511124957],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"xCP", "=", 
   RowBox[{
    RowBox[{"Components", "[", "rCP", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cpP", "=", 
   RowBox[{"1", "-", 
    RowBox[{"vMagCP", "^", "2"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gCpP", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xCP", ",", "cpP", ",", 
     RowBox[{"PlotStyle", "\[Rule]", "Red"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gCpD", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xC", ",", "cp", ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Blue", ",", 
        RowBox[{"PointSize", "[", "Medium", "]"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gCpP", ",", "gCpD", ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "1.1"}], ",", "2.2"}], "}"}]}]}], "]"}]}], "Input",
 CellLabel->"In[6]:=",
 CellID->59705953],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], PointBox[CompressedData["
1:eJxFlWdUU9kahiNYUBxGRimGYig6FEGUjugHTtRIE1i0iyJFKWoUREpEVMrI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     "]]}, 
   {RGBColor[0, 0, 1], PointSize[Medium], PointBox[CompressedData["
1:eJwB0QMu/CFib1JlAgAAADwAAAACAAAAvxJCeLTx7z+eOSzV8rHUv35MSGjL
ue8/njks1fKx1L+Mw17q503vP87CmgGbYtK/Se3nK0iz7j+Ui2Ca1KPMv47d
N9bF7e0/7LNYrmHuxb+IYI3MI//sP7hH2mcCTMO/YztRD9Ho6z+Q82djlFnD
v1l7ZRTLreo/hDhek44IxL8uXol2FVLpP7jC33Ves8S/kLKcuvLZ5z+EPv/s
LHnFv025Zb0YSuY/hLg1KTVGxr/uwxVNZ6fkP4yVW01RJse/YfmjpfL24j+g
l9QAPiHIv5gcKO0NPuE/rIRiaj4xyb/u33OTGATfP/TBjUbBYsq/iGriwoGQ
2z94tdRSd7/Lv6pfy7T9K9g/yJ5UGF5Czb+pqj3L8+DUP2wK9z6xAM+/uwIH
lE250T/IK86hMIrQv+Ovka6ZfM0/VBb+QJbT0b+3vFsgQfHHP1iPQNg1d9O/
ybNRMc7fwj+EtD728JvVv6XKfmtgq7w/pnLfV4h72L8y5IMnZr20Pyi0VSNp
dty/WbG9374OrD93/iHjF//gvzPauDF3LaE/atXtm8Dk5L82HbVpjNaRP8G8
f46RMOq/9x92H0KVej/7JSQttZDvv0+ZE75OQFM/6l1Jx61E5L+iHAI8hEdb
P8THp5Iv4v4/TuHsLNU+fT876NETjUMAQD7nVbQHWpE/2PikOqp2/z+bkN9U
xr+fP7JOx1d2qf0/qERNsKw0qT+IMNbIfsT7PwQpS0nXTLI/0kQ4NAEI+j9R
vbGfVPu4P1hnglYwhPg/ecAE2zNLwD94SX2vBi73P5YcUoWxhcQ/dHYwIkYH
9j/dQCYRUyLJP+6iD3dcBvU/b/T4piIVzj/MotGf4Cf0P/g44guTqdE/Lh0B
f1tK8z/1Xdv5fmrUP8Izg0sNJPI/g6h1MjhK1z/o8uDq/azwP69ypxPQRNo/
0NN62TkC7j9O9ZAYBFbdPx6WVQ3eXOo/4q5z5bM84D/m0r8Mc4bmP8YHY6pg
1OE/uLoV7uXH4j9oiWDzcW3jP3hjGCe1Cd8/O6f5HYoC5T8MntN/z3HZP8XW
u4dLjuY/UEcbTb6h1D82U48KYwvoPwQDEI0Gb9A/XuQReZJ06T/obLaNwErJ
Pw1xt4jvxOo/MCtgJHmPwj9iurlcxPfrPyDJwebfsbg/VRVzApoI7T8gBxPz
fuSqPyFn+sHA8+0/gG7tkKaQhT/fBxrr4rbuP4BZWqgMA6e/s6usQspP7z+o
QvBS47e+v766qCaIuu8/qH/d6Aphyb+Mt6tp3vHvP3ovI0n/XNC/tKL+Mw==

     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox[
    "\"\\!\\(\\*SubscriptBox[\\(C\\), \\(p\\)]\\)\"", TraditionalForm]},
  FrameTicks->{Automatic, {{-1, 
      FormBox["\"1\"", TraditionalForm]}, {0, 
      FormBox["\"0\"", TraditionalForm]}, {1, 
      FormBox["\"-1\"", TraditionalForm]}, {2, 
      FormBox["\"-2\"", TraditionalForm]}}, None, None},
  ImageSize->504,
  PlotRange->{-1.1, 2.2},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 324},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->118437511]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Options",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(3)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->2061341341],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Epsilon",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection",
 CellID->30521753],

Cell["\<\
Collocation points are displaced a distance Epsilon from panels.  When \
Epsilon is positive, the displacement is opposite the direction of outward \
unit normal vectors to panels.  In the following example, the model is mapped \
to three values for the option Epsilon.  The three values place the \
collocation points inside the discretized airfoil by the default magnitude \
for Epsilon, on the panels, and outside the discretized airfoil by the \
default magnitude of Epsilon.\
\>", "ExampleText",
 CellID->120830056],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"aMinus", ",", "a0", ",", "aPlus"}], "}"}], ",", "aW", ",", 
     "rC", ",", "vMagC"}], "}"}], "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"PMDoubletConstantDirichlet", "[", 
        RowBox[{"rD", ",", "\[Alpha]", ",", 
         RowBox[{"Epsilon", "\[Rule]", "#"}]}], "]"}], "&"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"10.", "^", 
          RowBox[{"-", "10"}]}]}], ",", "0.", ",", 
        RowBox[{"10.", "^", 
         RowBox[{"-", "10"}]}]}], "}"}]}], "]"}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->210128799],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"LinearSolve", "::", "luc"}], "MessageName"], ":", 
  " ", "\<\"Result for \[NoBreak]\\!\\(LinearSolve\\)\[NoBreak] of badly \
conditioned matrix \[NoBreak]\\!\\({\\(\[LeftSkeleton] 1 \[RightSkeleton]\\)}\
\\)\[NoBreak] may contain significant numerical errors. \\!\\(\\*ButtonBox[\\\
\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/luc\\\", ButtonNote -> \
\\\"LinearSolve::luc\\\"]\\)\"\>"}]], "Message", "MSG",
 CellLabel->"During evaluation of In[1]:=",
 CellID->285829413]
}, Open  ]],

Cell[TextData[{
 "There is a discontinuity in velocity potential across a doublet panel (see ",
 ButtonBox["ICDoubletConstant",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletConstant#62898067"],
 ") so large errors may occur."
}], "ExampleText",
 CellID->937232338],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Max", "[", 
    RowBox[{"Abs", "[", 
     RowBox[{"a0", "-", "aMinus"}], "]"}], "]"}], ",", 
   RowBox[{"Max", "[", 
    RowBox[{"Abs", "[", 
     RowBox[{"aPlus", "-", "aMinus"}], "]"}], "]"}]}], "}"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->98885038],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3.663079372544422`*^16", ",", "3.663079372544422`*^16"}], 
  "}"}]], "Output",
 ImageSize->{199, 18},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->24909326]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "InterpolationOrder",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection",
 CellID->233233938],

Cell["\<\
The list of external surface speeds associated with collocation points is \
computed by using the jump conditions for the velocity potential to determine \
the velocity potential just outside the airfoil.  An interpolation function \
is fit to these values of velocity potential, and the velocity is computed \
from the gradient of the velocity potential.  The default value for \
InterpolationOrder is 1 giving linear interpolation.  Higher-order \
interpolation may give better results, but risks excursions of the \
interpolation function fitting the data. \
\>", "ExampleText",
 CellID->381223408],

Cell["\<\
To prepare the graphic below, the model is mapped to values of the option \
InterpolationOrder of 1, 2 and 3.  Flow speed just outside the airfoil at \
locations associated with collocation points is plotted versus distance along \
the discretized airfoil.\
\>", "ExampleText",
 CellID->507899588],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "aW", ",", "rC", ",", "vMagC"}], "}"}], "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"PMDoubletConstantDirichlet", "[", 
        RowBox[{"rP", ",", "\[Alpha]", ",", 
         RowBox[{"InterpolationOrder", "\[Rule]", "#"}]}], "]"}], "&"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"s", "=", 
   RowBox[{
    RowBox[{"Accumulate", "[", 
     RowBox[{"PanelLengths", "[", "rP", "]"}], "]"}], "-", 
    RowBox[{"0.5", 
     RowBox[{"PanelLengths", "[", "rP", "]"}]}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"data", "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Transpose", "[", 
       RowBox[{"{", 
        RowBox[{"s", ",", "#"}], "}"}], "]"}], "&"}], ",", "vMagC"}], "]"}]}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"data", ",", 
   RowBox[{"AspectRatio", "\[Rule]", ".5"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<s\>\"", ",", "\"\<\!\(\*SubscriptBox[\(v\), \(c\)]\)\>\""}],
      "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7.3", " ", "72"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"Green", ",", 
        RowBox[{"AbsolutePointSize", "[", "6", "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"Red", ",", 
        RowBox[{"AbsolutePointSize", "[", "4", "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"Blue", ",", 
        RowBox[{"AbsolutePointSize", "[", "2", "]"}]}], "}"}]}], "}"}]}], ",", 
   RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->49740593],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3T+IK/lhB3AlwWGbXLZ0DIlJCrNNSHHNsnB5uSLPDz9yBGMwr7kYQ5Ti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    "], {{0, 281}, {526, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{526, 281},
  PlotRange->{{0, 526}, {0, 281}}]], "Output",
 ImageSize->{530, 281},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->9889717]
}, Open  ]],

Cell["\<\
Results for Interpolation orders of 1, 2 and 3 are displayed in large green, \
medium red and small blue symbols.\
\>", "ExampleText",
 CellID->520516665]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "XWake",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection",
 CellID->104403840],

Cell[TextData[{
 "Results should be insensitive to values of XWake if they are sufficiently \
large.  The model is mapped to three values for the option XWake, ",
 Cell[BoxData[
  SuperscriptBox["10", "10"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  SuperscriptBox["10", "5"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  SuperscriptBox["10", "3"]], "InlineFormula"],
 ", the first value being the default.  Strength parameters for the latter \
two values of XWake are compared to results for the default value and the \
maximums of absolute values of differences are displayed."
}], "ExampleText",
 CellID->391229248],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"aD", ",", "a5", ",", "a3"}], "}"}], ",", "aW", ",", "rC", ",", 
     "vMagC"}], "}"}], "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"PMDoubletConstantDirichlet", "[", 
        RowBox[{"rD", ",", "\[Alpha]", ",", 
         RowBox[{"XWake", "\[Rule]", "#"}]}], "]"}], "&"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"10.", "^", "10"}], ",", 
        RowBox[{"10", "^", "5."}], ",", 
        RowBox[{"10.", "^", "3"}]}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Max", "[", 
    RowBox[{"Abs", "[", 
     RowBox[{"a5", "-", "aD"}], "]"}], "]"}], ",", 
   RowBox[{"Max", "[", 
    RowBox[{"Abs", "[", 
     RowBox[{"a3", "-", "aD"}], "]"}], "]"}]}], "}"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->451142620],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1.3479012308525995`*^-6", ",", "0.0001348201233417612`"}], 
  "}"}]], "Output",
 ImageSize->{177, 18},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->632083273]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Possible Issues",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->1305812373],

Cell[TextData[{
 "In the specification of some airfoils, the trailing-edge points for the \
lower and upper surfaces are not coincident.  This causes a problem for the \
doublet models with internal Dirichlet boundary conditions implemented in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ", but not for the vortex model with external Neumann boundary conditions."
}], "ExampleText",
 CellID->416625656],

Cell["\<\
This example uses the Clark-Y airfoil specified by a data file located in the \
folder Airfoil Geometry.  Import the file and plot the discretized airfoil.\
\>", "ExampleText",
 CellID->148664056],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<Aerodynamics\\\\Airfoil Data\\\\clarky.dat\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"gD", "=", 
  RowBox[{"PlotAirfoil", "[", "rD", "]"}]}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->112529997],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3c1rFOkWx/Fc7l9y/43AMAtxIS7ECyGLkI0XGt3MbHSjm2SjmwQa3RiQ
60INJLmLyFxKhAzIBMTAMIFgQOyBIcIQFwZBEW5gIPekT+dJpV46/VIvz3nq
+6HmhY52n0p3/erUU091/ePaj/+89veJiYn/yj//+9vExPH/HwEAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAA4I2oNSEm252sh9OPAwDGUnXqyhO3ou5/zzx3
p92Oej8m6QGgOJWnbqc92X3B1EueVkTOA0Bhqk/dkx1L3lNHrd6PeyYAACMZ
LXXH13/PcnIccSbniy0AAJrAheewqVuAnJEiLab7UNSO/YCcB4ARnIbnkKlb
iLNnfnsV9M77dsWPIch5ABhBPDyHSt3qkfMAMAJD4WmoVADwh6HwNFQqAPjD
UHgaKhUA/GEoPA2VCgD+MBSehkoFAH8YCk9DpQKAPwyFp6FSAcAfhsLTUKkA
4A9D4WmoVADwh6HwNFQqAPjDUHgaKhUA/GEoPA2VCgD+MBSehkoFAH8YCk9D
pQKAPwyFp6FSAcAfhsLTUKkA4A9D4WmoVADwh6HwNFQqAPjDUHgaKhUA/GEo
PA2VCgD+MBSehkoFAH8YCk9DpQKAPwyFp6FSAcAfhsLTUKkA4A9D4WmoVADw
h6HwNFQqAPjDUHgaKhUA/GEoPA2VCgD+MBSehkoFAH8YCk9DpQKAPwyFp6FS
AcAfhsLTUKkA4I96wzNqyetPTLY7Zx/utCfl4VZ09lFyHgBGEA/PoVK3APJ6
3WeNWqnXPH5Rch4ACnAankOm7vhOnzTjJcl5ACiGC89hU3d8JzsWch4ASuTC
c9jUHd8I/bxTdC0AEJp0ZlbfzzM+DwAVqHF8/ih55jd+SDGRPiVMzgPACPLn
25yTutUj531zcPBt+7c/08vmL388/vev5y4//vBTIcviwmafV3kRvUtXKJXX
/csDqmMoPA2VapHLQAlGF5L37r6KJ+rlS4+//27JLVevPMkM3ju3Xw6S85n7
iBGW5+u7fV4lsQq6SOXxFZEl/tP5uY3M3cTh4V91v0vAKAyFp6FSPbS//0WS
6ueN3xO9dDroJBjzmuGvXw/rXo+yxFfT/ZYSu4mLFx7p70r2d+mjibXVHf3r
nfef6l4b4AxD4Wmo1BrpWIq2uLduRq53nZ5ajneqLtPqrtck2d+ljyYe3H+t
yf+va/9J7A7cr33zlz/4taN6hsLTUKmVcTnj4kXHUrTJ3HrzgbHoGrndgTtA
uHP7Zfwwyg18LT3ckp/KW8kAEcpgKDwNlVqSzvtPeopTGnXpz3WwRSOd4QKL
3InsZ0+35U2UtzI+QDQ7sxI/2cEuACMzFJ6GSi3K3t5naQWl2dMOUDp23eql
Ud/f/1J3dSiXvPvxyUvxXcCN6+vy/w/uv3ZjQeQ/+jAUnoZKHZkc6et2rZNb
pKObn9uQZo8RXcTtvv0oH4m11R03FhTPf3lQPzPyx+quFF4wFJ6GSh2KHLxL
0y7H7NKuS7Zrxx725BaURPNfPj96DCix704H67lgPQXAKZumMRSehko9l2xo
L6J39+6+mp5avnrliWyDsgEywI4y6OlgPRespwB0CpbsBW7djNzsWfqKgBkK
T0Ol5pF2Sxot6dtlQ5OQl+2LYXbURT6NW28+uGsEpO2/eOGRu4BCfkTjEQxD
4Wmo1DhJcslz6dhlO5IO6tnTbTYf+Onw8C93QbS0+jpTV6f9uDEf2n6LDIWn
oVKPuvG+trojW8r01LI0SHLUzAYCi3Tajxvzoe23yFB4mig1Hu8P7r9mE0B4
Mtt+N+lXWhrm+fjGRHgqn0s9OPhGvKPJ3EV883MbOs9ndmZFz/PK42wR9Soq
PPVm4aV+fbGfOS/di3ywr155QrwDcXt7n/U8r/T5rufX6Z1c6FexwnK+c5zu
nXarvC+p9yrn5TO89HBLp0RK1NddDmCANEI6vVO/uEPH+RcXNvWrPrmktzxF
hmfWnagK5EPOy0fxRfRODkvlmPTZ022uNwFGpuP8z9d39as+JfYl/PlmjzIU
F55Rq3v7qU4UBTluI5EuHz+dPMNpJqAMku3xb+rThl/2ArIvYKMbR1Hh6Ybn
J0q5keyxunJePnt64ap8/Gjggcpow7+2urO4sKnndt0IPzP5h+LDYMiAqi/V
JfyL6B2Dh0Dt3Ai/zuS/euWJm9LDOE8f5Hwm6dulhdCEr+xFAQxFtlM3pUe2
Vv3GNmI/jZxPe76+OzuzIv+u5uUAFEK/sY3YTyPn4+So8Mb1denkGfoDrMuL
fTkEaNqJNnLe0QtaOa0PBMnF/q2b0dUrT2Rx93oI/uwbOX/U/QDIO/7g/uuS
nh+Ab6Sld/duu3jhkd67Ta/Yqru04pHzOlYj73gZTw7ABL0Xs16xpRM4Fxc2
X0Tvwvgyk4bn/NabDxLy8hYX/swA7JJ4f76+e+/uKwl8vVzL9MB+k3Nedta3
bkbBD80BGIderuUG9qenlufnNvRO63WXNqjG5vza6o7srAt8QgBNsL//5eeN
3/VO63qb3Qf3X0vT6POwQDNzXnbNnHUFML7dtx+1aZydWbl86bFen+vbHJ4G
5ry8C7IU8lQA4Hz9eqjX5+ocHj2ZK81/7VdpNS3nZc9LJw+gAnoyd35uY3pq
We9VIflTyxU6jcp5OZjixCuA6h0cfNN5mzeur7sJPJWdyfUz56NWxh0IxyxV
fs+zMyt8oQGAerkJPHom103aLC+d4uGZma41kDq6X2KfuD/VmDnPDf4AeMhN
2rx86bG7PqvYIf3T8MxJ1+p12pO9e5WcLWWcnO+8/yS/wLFLA4ASueuzdEj/
zu2Xa6s741+T68IzL12rd7LDych5Z9jnfPZ0e+nhVnE1AkC5Dg6+SWpJg/r9
d0vS6o/wTenpzMxL1+qV0c8zlxJA2fb2Pm//9md80WmWiWV+buPHH35KLNK9
S54nlhvX190fGGfY2cN+vozxedkVLi5sjl8agDDo1xQnlrXVnUQm6zebJZbZ
mZV0Jssijyf+pF42lVgksdMvXeo353g4Pn9Uwnyb/f0v8hYwoxIwavftx0Qw
6o1iE4vkajqW9WayiUzW244kFkn1xBPqNxUnFp+/4iCTj/Ntcow534ahG6Aa
0lals3HwZM7sluMjGLrMz22kn23rzYfMl274hGo/589nGr/UO7dfcl9vIC3d
Letkv8Ry7+6rdA+czuTpqeXMpnrAZDbXLfuvUTkv5IO69HCLARzYpVfZJJbn
67uJ/NQvVBzkZF9mt6wX7yQWaZPSL1337wPna1rOH3XnWMqnmmumUI3O+0+J
YNTb1Q3YLaeTWa+aTyyLC5uJZ9MvSK/yZB+81cCcP+puenIIKVuQ9Dzc+BtO
5sBy5tS4zIFlneqcWOTBxB/T208P2C2TzBhfM3NeyRakvb00SLLNav/DkI4h
6enKZQwsZ06NyxxYDuNeoghPk3PekWyXbVbHMyXzZWPXTVunuZb0og1U+MBy
eroyA8tAGjmfJgfveqiul63FR0QlghK9XMAtXHpgOW9q3IDJzMAyUAtyfhCu
EZUISozNxodk01depC+7cIv0sZljDiMveWf33CKJmtn95nXL6YHlvKlxJDPg
M3K+QOkrqdOXUQ+SuqMteWf3BtmzkMlAwPwPT8dQqQDgD0PhaahUAPCHofA0
VCoA+MNQeBoqFQD8YSg8DZUKAP4wFJ6GSgUAfxgKT0OlAoA/DIWnoVIBwB+G
wtNQqQDgD0PhaahUAPCHofA0VCoA+MNQeBoqFQD8YSg8DZUKAP4wFJ6GSgUA
fxgKT0OlAoA/DIWnoVIBwB+GwtNQqQDgD0PhaahUAPCHofA0VCoA+MNQeBoq
FQD8YSg8DZUKAP4wFJ6GSgUAfxgKT0OlAoA/6g3PqCWvPzHZ7px9uNOelIdb
0dlHyXkAGMG54TlUFA9Hnrr7BFEr9fTHz19kzjdzH9HAtWaVm6CBq3xUagAO
GcVDOf37Gc9OzheggWvNKjdBA1f5qMwAHDaKh3KyDyHny9LAtWaVm6CBq3xU
ZgAOG8X9niiu+1wj9PMAgBEUGMXDKXNQCAAwkJKj+OxJ3vjRw0T8BwCA8hDF
AAAAAOqSM/8/QDlrGvKBV783N9AzPH1WWa9pCe+N7nsJT4grfKqIy5SaoN/5
hbDkrGmn3Y56Pw7uN9Dvze1mQ3jbR/4qSyIE9wZ35a3yyX78dCg5TIH2K4Xq
O18oKOeuaXi/gD6rHLVaUYjbR+4qn/S24e3b8t/lXsAfv9f1lFaNED/HRes7
/z8o561pgFtD7irruoa4feStsmvmwzts6/fBDvSg7awQP8dFo58/+Wk7vI9K
ziqftrbhjd3mvcsBf8779fPazof2JieQ8wNo/Pj80WmzF7UD+xX0f3OD3D7y
Vvkk7QJc6T6r3H082BMTPQG+pWVo6nwbN3p52tyG93HJu7jjWKDbxznXszRo
ld2BW4jrfOLsbLnQzzkDAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAABCI
3rcmttrH/+XLBAEgRMdRLzkf4K1cAAA9wd/VCACaTO/+wD0gACBInd64vI7T
09QDAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAABDzf3mA5o8=
    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->342316319]
}, Open  ]],

Cell["\<\
Note that the trailing-edge points for the lower and upper surfaces are \
close, but not coincident\
\>", "ExampleText",
 CellID->390242666],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ptsTE", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"First", "[", "rD", "]"}], ",", 
    RowBox[{"Last", "[", "rD", "]"}]}], "}"}]}]], "Input",
 CellLabel->"In[2]:=",
 CellID->238007975],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1.`", ",", 
     RowBox[{"-", "0.0005993`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.`", ",", "0.0005993`"}], "}"}]}], "}"}]], "Output",
 ImageSize->{241, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->261210594]
}, Open  ]],

Cell["\<\
Specify an angle of attack, solve the model, and compute the lift and \
pitching-moment coefficients.\
\>", "ExampleText",
 CellID->25666953],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Alpha]", "=", 
   RowBox[{"5.", " ", "Degree"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "aW", ",", "rC", ",", "vMagC"}], "}"}], "=", 
   RowBox[{"PMDoubletConstantDirichlet", "[", 
    RowBox[{"rD", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl", ",", "cm"}], "}"}], "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Doublet\>\"", ",", "a", ",", "rD", ",", "\[Alpha]", ",", "rC"}], 
   "]"}]}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->592619],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "0.1896980240555135`"}], ",", "0.23030473695908804`"}], 
  "}"}]], "Output",
 ImageSize->{149, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->573096007]
}, Open  ]],

Cell["\<\
These are not plausible values for the lift and moment coefficients.\
\>", "ExampleText",
 CellID->722709344],

Cell["\<\
Plot streamlines near the trailing edge, using a scale that makes the gap at \
the trailing edge visible.\
\>", "ExampleText",
 CellID->492689119],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"xW", "=", 
   RowBox[{"XWake", "/.", 
    RowBox[{"Options", "[", 
     RowBox[{"PMDoubletConstantDirichlet", ",", "XWake"}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rAll", "=", 
   RowBox[{"Append", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"{", 
      RowBox[{"xW", ",", "0."}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"aAll", "=", 
   RowBox[{"Append", "[", 
    RowBox[{"a", ",", "aW"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ic\[Psi]A", "=", 
   RowBox[{"ICPsiDoubletConstant", "[", 
    RowBox[{"rF", ",", "rAll"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Psi]", "=", 
   RowBox[{
    RowBox[{"UniformFlowPsi", "[", 
     RowBox[{"rF", ",", "\[Alpha]"}], "]"}], "+", 
    RowBox[{"ic\[Psi]A", ".", "aAll"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xMin", ",", "xMax", ",", "zMin", ",", "zMax", ",", "contours"}], 
    "}"}], "=", 
   RowBox[{"{", 
    RowBox[{"0.95", ",", "1.05", ",", 
     RowBox[{"-", "0.03"}], ",", "0.03", ",", "13"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gStreamlines", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"Contours", "\[Rule]", "contours"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "False"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "3"}], ",", 
     RowBox[{"PerformanceGoal", "\[Rule]", "\"\<Speed\>\""}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gStreamlines", ",", "gD", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", "\[Times]", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[4]:=",
 CellID->266881112],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJxs3QW4FdX6P3BAUCzs9mdiB3br2IXdiEqIYAvYAQqimIiICIoYYPe1MLfd
XuvaehXk2p697db/nJn9WctZ538en8fny5q9ZsXb77vWLN33mD0O6dCuXbuv
l23XrvX//Yu/lmzqEmd9tfvUSbUFj3w8/y/iZwYt/uygxesBn3Bi61/Eyw95
fYUhr0d8ROvPF2wEvNXWrX8RL/Zsa4cRf7/DVT/scFXEL7/U+hex8c1YqPvQ
u2ZcFsYHGx9sfLDxwcYHGx9sfLDxwcYHG9/c87T+XRzGBxsfbHyw8cHGBxsf
bHyw8cHGBxvfyfPfvt20U0aE8cHGBxsfbHyw8cHGBxsfbHyw8cHGt/1in98z
bJcjwvhg44ONDzY+2Phg44ONDzY+2Phg49urxJnxwcYHGx9sfLDxwcYHGx9s
fLDxwcZ3WrmeYXyw8cHGBxsfbHyw8cHGBxsfbHyw8c1b0mMYH2x8sPHBxgcb
H2x8sPHBxgcbH2x8n5b8HMYHGx9sfLDxwcYHGx9sfLDxwcYHJ/I5S+Rzlsjn
LJHPWSKfs0Q+Z4l8zhL5nCXyOUvkc7bLb1N2/W1KPdupoMfza28PX+2d4as1
At7si0vy/1qy7gU+NbTD2tsVf9tv0lL+BTzph9YB1AP2Plh/sP52L/vP9Afr
D9Yf/NffrX8R62+3cj7hedj70/aS/seGdlh/uxTPXxHkC9x79vvy/xoBW29Y
/6eU/QVM3hu/du+Htdsf6wGjHxi9pPtLfpsPeWm97S+s3fu0Gz+s3fPaYeP3
vPHB2tHDHZ163tmpZ6MN/aX6JqWfVL5qtz4pPXre+2HvT+k1pS/y0nqhJ+/z
PH5O6TeV39q9P6XvlF5T+ard8+hXfyn9p/I6pf9dS/oN84WtL4z+U0ze+D37
MGkP9JDSe2p/pfSS2gva4XT/7K/9SvVlul/peqb6K11P8j+Zb5C/1sf+4Uf9
p/ae+dAX5lPd3xGJvBqR3VsscCObXoxnZO3m4q8lO6loP66mHda+f/G+ntmD
D7X+me+w0B+s/cFifqNCOzx9g1GfbDCqHt6/9DKtf402eK6yv1ox/Hdasi3L
99f2v7OVYOsBr7V261/E+k+x8V/4Ses/NNq0Dy3nm40pNihi40/b0/mW+zsy
9A+bj/mjZ/NFb+l40v2wHrD997z9h1P5ZfzGY/zmg771D2s3H+2wdvPDH7D3
w62tKwxpyWa6cr1XD5vpyiY9NwLW/33l84H+rA/6h2drFRe9436TB7D9Mf+q
fTcyjA//GV/HcjyZ8cF+b/31b7/sp/Wn/1L/wX75PfpEH9qth3bztz/Wx37q
n37Xv/UkD8hP9KAdv3kf/kr53/r6fYo9b3zWX//GS36Zj/mRn35vv+gX9IL+
jB/GX/bT+Mhf2HjsH0x/mC/+I589D+vP+z0Pe976pBg9eV5/9j/tP5Vn+rO/
KU75J90f9Gb+MP7yPP6Ctaf2k3Y4lZ/4wfraf/ILfVTtmWFBfumv6t+ODfLp
2TG9Vv316TGBf9kT2p8u24N+TekntU+tp/U2P+ub7ofx0VfGTz7A1juVp8bv
/TB6Tu0H8ilpD/4GfrFeqXxN4zPo0fjNH/a89Uz5xf5qTzF7yfxS/ziVv+br
/dabfCUv4DReYT7Wx3zgNB6lP/Rof9Bjqt9T+5C9aj9TeoPRD3qx/t5vvt6H
flJ6sZ74UX8weoP9nv1PX7BfYfRMnvq9/dSOnqxXau+Zj/dZH5g++rV1eXeN
/aEX7TB7GL2l9Eve2Q8Y/af2ZBpPs376T+3nND5nPT1vvR4rHqhnZxTP98sK
8ryqJVuveH/32vgJrX/1rH+Jg75N5V1qf9kf7dYDPXk/fkjnn+of62d/U3mc
zt9++j2MHlP7Ar2Qd/rzfPr71P8xH5h+tf6pfKKPrCesf++v+m/DEnk0sjmP
RnZZsU/nNvtvZKuP+6Pv893Ormkvx3diaO9S/PuggDsV8uzCJn/H359WCKCI
C3H9UqSPsv96dlxJPwFrL9jlt5bQXvZXb0Nf6A+evxxftn0rOf4QsfHCA1uH
+2w9W7McX9axlV3vjNjv4Q8+bP2rZ1eU6xX60269Cum+3uimvx/naz9h/APj
r9Rego3P+uN/+kK79cAP2o1XezW+Hf0l/Ge/8K/9tB8F+3zS0tTfw5P4+LCw
fun+G792WP9pu/cZP6zdfOj7dH2NH31YD+9DD+jNfMzP/uK/dDzkC0weeL/+
4c9LfsoK8hvUkv23oKsra4OLf2gEjJ6sB+z96Au2vjD6tj5+Dyfvz5L3h+dT
/ZPa1+QN/rV+1XzIsMBv2qv5kWHhfdrth/3ca+/Wv7b8Qv6hf+3ks3b7jR7I
f/oAPbUv5VnYT5g9QD6fU/zF9ab/Uv6A2Tcw+U2+w9bbfFJ6v6ewH4aG8cH4
N/WvYc9bH/1ZD/QEkw8w+tIf+xI9wNpXbVWHw2P/sPebXyqv0/gOeoU97/fo
BX/D+qOfrE/Jz+Oyqr4cG55P/SP8B6f76X3Wd4NS/of+YPSW+tsw/x09wCm/
WR/8VNqjQwP/wPgjtWfId/qa/sePxqMdfX3eOpzN4vzNx3rZX/IN/7M36U/t
3qfd+O0v7Hm/t96w+VoP+hn/oTf7BWtnP5I3sPc/Vb4vYOuBv1J+ZC9V49En
BnvF+61fKo+0W0/0D1sfz6f8Y72MJ8VpvLSaf4/0oz8Yv6T7l8Yb8Rt5Yj3J
jzSeab7oBf3bT+tHPho/+x8mD6yn38PmD9tPmP2uP/xGP2jHf+zR1L9J46P4
13jM13rD+vc+62X92R9p/MXz3g9bf/aLeAH7wnrzP+gz4yF/0vHhf78nL8gb
/o39sH/kXWq/6w9GH34P43f8COM/mLyGvd96w+wB2PNV/yTuj/ZUX+JP+5X6
w55HTym9pfaNdlh//FfPw563XujNeuJX7ak81T+c+kv6Iw9SfyOVv/pHDyn2
vPHB5CX9zB4V7zO+1B5K/VPyQ3uKU/unGi9ptNEHVf0ax0t/wOzpNH7Jnrb/
xm/90nix/bCe9oP+sv7GU5VvsT/04/fJ+GrJ+IJ/anz0TepPpPGj1J/2fvyT
1iek/ka1Pij6O/QLXMij+RrZv4rnjw7xAutFHsEvFvOrZ18W9DEg8Cv7wfqg
T/IJRp/kqfhTaq+l+0ke+b3x6z/FKT2zn8mPVP+n+QjrD5sffcz+1F9qT6b0
lsZzyFv2hvGyB7wff6IP80/zt6k9b729z/ir+ddo37LvyD/8DbNH+dvGa79T
f9l6sG9Tecg+xn/eh//tJ3kOW890/cRzrE+qH+Fq/UDUV+hLO8w/1X9pD8Tf
V+OnJwZ+N1/zIU/gNH+m3frgb/Sf1vuk8p2/5fepv2V81tvvxR+sP/633uR1
NT4Z23sV/F/PHi3lR7ZAMf+I/d78YOvj+eeL/RwcxlP1X+qJ/1JvYy/qj73l
fbD1Ji/5b/xr9jb5A9tP40Hf+Et/9kd/4i/6g+2/8dr/NN6a2t/wTYV8qWdr
F/r8hMDf+I98h8n3/5byOthf9tvv4eL9izSyJ8v9CPEp46vWq41N5ndxmL/x
wvbLfqT9pfWi7F3ylH4o61Hj/M3PeD2f1o+k2P6bP34xf+tlPOZj/Kn9YD2q
9ddjgz6Qj/W8+cLmix7Mt5r/vTjQi/7k3+TH7Jf+jUd//MOUXsxXfi2td/e+
ar15rO9jbxivelXrZ3zq9YyPPCdPtONX8hf/aP//52/Hhv01X/KaPCfvU3+J
/iP/2cvknf68D79MLuRZPVui6G/jkI+s2pMx32v9/d7+pfUA6fzT/Bp6KXCX
RrZayQ/B3ibf1y/kTZSvMP5h/6T5J/qO/eT92tP6L/LQfPnH5Lv30U/6h1P7
LI23pfSCfvFval/IjxT9LRnXx/6n+R/0hR/SeKb1gb2/tG9ivMN6aIeND/+g
J/RnvWD6Bz3J/xtvWj+Rxg/Qt/VK/f00v5v659r9nr3m92m+FX9V6/eGlvTV
IfoXMHmNf9gL3xTiI2LPpxg9W4+0vo69g36105/GYz4pv8KFuO/cCP6O8fo9
PFvxuz414ydP/R42fs/TXzB+9jzs/caHXs3n6GI/9gr2q/1kn+kP/aJX2PPo
NcUVeyqfj/eZ79klDvs3tJSHYXxf796qIYYF+k7t9zQfV9G3+fvGFb87Obxv
YolDO+z3+Af9krf2G+bPmk81vjUyYPxqPY2HPEUPhX3eNY53WKGvo37wvPXR
n/F7fsGSHoK8Rv/o2Xr7/cyFfzkkzH+fwi+7NPze/lu/dP+MT//ayXuYPLc/
MHuI/WQ9YPvLXiAvvM96wOl66T+sX/P9+Kfijy7QyFYq5Gn/YH9U5O0i0d4r
hrNwI7ukWP/Dgr2V1ouwZ+j/FKfncbSThyl2fnRkQcDxfI/zVrDx6J/+gLWn
5xu0k4f0B5zWR3Ur9V0Yj9/D9K/xweJdqX62vn5P/sL0PX1l//SvXf8w+pZf
YR/B+mefwKm9yJ5m77Ifvd/z3l/1F6K9qf8Uqw/XH+x8nfU9pNjPqN/p44p/
NF+U/9rFc7Rf+8bMGx119ZkhfpOeH4Ctn/hV1d+9LPgr9os9hl+L5pXz/Sjl
TbBH7A95j3/Z3/QJfUleeZ/19T72uHb1qCWO9an8AZg+sZ74EUbv1h894Af5
HvWE1jOtd7c+5mM96HeY/Kzk7/N264P+2FPsPfIHvXgfetHu/al9wn/zPu3k
T4rpF/uVxmur533rgd6MD78ZX5p/Fv803/S8Gf5gv6jntd+w/YbRr/0vxNtD
Mf5uPeg/803tUe3WH78Yz+XFOkwI9ib7Bb2m+ek0Plby2xlJPUE872R9gj5s
rh/9zN71fCVfnduH/Cv2kOfFK6xHuT4twd6E6fdk/YJ+5z/xt1J7kD1iPLB2
9gx5z5+A8VfI5zbr/WD1QujPePE3TP6k9jh/g/2QtsPa9Wc+sPl4nv1MXvt9
ag+bP/uHPPQ8foTxL/rBX9rpV/EO+2t86Bv2PIzf1cvbb/YD+hQfQJ/8Z/LB
fGDt6Be/k7/oB/3irzQ/mvpXFXru3PZ589E//836W1/8ntYXpv6Z9dfu9+gB
PcHGSx/b7yK881Quv/NfL9R9Uvi993uef8Q+S/039AfTH+JHlfq5zlG/mL/5
kC/peqBf653mk6y//swXNh/9a4etr/5T/5+8hCv50g6NNvlA863aN0ODPcS+
gMlfz6Mf7egHNl/7pR09aC/I+84oT2H7JT+qHU7lCf/H+NJ6mkq+r0t8Hr1X
4mdLxv0vujuyBf0F+amd/IS1p/Rn/NYfJr/pQ/7rlyU9h3b7qx19wOihWj9z
Yminf+yP8VoP2Hqm9dmpPE7rL/Cr+aIn70/lvfHpz/hg/r/xmJ928QdYvM3z
lfO9XaI8Ik/E29ln1fts4vpuWthHbfk1rQdkf5vvw/t9d/5ytXNCfDPNh6b1
Qfo33vS8lHgq+q/W98Xzaux9z1froceG35f25yUhPs9+kJ/Tf7WedEzw31L/
nT+OH9L7rTzPv0vvy0r9Zf6E/tn3/ItqPWjEfp/eb2S/xKvIIxh94A+46p+O
SPzdESG+LF5Wva9mZHLfT/Rv+Yv4QfyAvoBD/X4ZjwrxH+sJW3/2gN+jN5g/
Jj7HXknPm4lHyLeiB+vPn1KfkNYLoGfyznqxL/k/noetZ1ofzL/hz6CfUl9E
f4V/k/ovaT6fPi3pq9Emf2V/jI985j+k9svHxfsuDPae/Il2+o48SM/nVO4X
6BLjn94HkxepPSv+iZ5g/knlPPZ8Uf+J56XxZs/rn3znT3g//8b72F+weK75
W1/ypBqPGRPWo7TnLg6YvFKvUK0/ifkT8j+9Tww/2h/7DafnT7WzDyr5987R
Py2aZ2lk6xT20cCANypxWC/6krxJ/TkYfVbrBxpt6pth+8l/RT/Wx/55nn2V
1pOn9MAeCOd5m/YP/e739N2kkn+Dv2k/zA/2vPgOXK7fGQGn8f/Uv0vzldoL
+fVxPVu3jF8H+kzrza2v8dIHWxb2QT3sXyEfnq9nsxbjPSrsX1p/4/36g/3+
qjI+HvjDelhP84etD36snmeL8go9wOjhsdL+CPFM+hEmb9SfoRf1aWl9V+X+
ha4xHwx73vrgd+sBW9+Wgr8HBTyloJ9jg3zAX+xx+wWn9Tfa6Vf68LXCfmqr
D9kTnoc9n9oT+F87/5x+Jz/E7/j/MPlGP6X1HMbLvvB78qay3lNifIH9Kr5A
v9BXcJovl/9k7/Kf4TQ+zv7HX+gZ/Vsf+6ndfNgrsPlaH/tD/7NHYPtDX5uf
+EsZXpwQ6MF64A/YfFY+4KInflzxgpD/sJ7sN9j7/N5+pOe5q/cxRv1lvdGH
9Ur1Kez3lXrVRRpt6rX4t9abfJJfSOs1Un7FL8YPs/fZWzB7SHzb+lu/6n60
JPZaS4j/ygehf/tn/8lH9Ecesg/ZkzB68j77Bxuf94t3oz/+FfqB0Sv9RX+Q
b+wb+wcbH3rBr3D1/tCxwd4gH6r3dw4L9on1qD4/POOfpvUBMP1Jv8H49fbW
9Xs/8iv6Y5/C7Cf0SL/wH/mbnkfvnsdv9tP4YPOxn+gN/9s/9GS/7Cdsf/AL
nJ73sN7sB+tjPfBTtZ7p7DCfOd6ePHDT2UaGfCh644/C1sf4q/XwYwJO76Mz
f/1X+S/2Zz7WH73qT7vnzQe/GS//nz5L80/pfa/eZ335e/jbfuDvarygbbyE
PYKf+J/kpfhRNX89Nvg73g/z7/gH+kdvcMFvP9eD/Q6/WOxP3yBvK/fPbdwI
9lq4v65pP/PX2M/hPrtmeyGfOjayjkV/u0d/LKnHSOtZ6JtCPM3byMYUeOvQ
XrHvPo72q/l4X8Ffi+X+Q2l/1din7GXvP+G9zW+Y87jzgj1Gf5Dv7Ff7T3+g
T/RNPpEX5EPJP5G+2Y/G27ecX8DmK56dxqfLep2zAsafxpfW77Av+RPi49ph
+j7Fab4qrQe0/94/uFzPeL9q076HzadY3/aN7M+Jhfypvdq64X9GeaQd9n7+
pefT39dKeq4t2spe39ZDvU5h739dD/U++vd8wT831LN3ivXbN9A7e9D72SPW
L62PS+ulKueNu8R8nvVN6+fSfKj++dvGh79g81u8pP/M/Btle/CX8V+/VvH2
aVw//ARrR48XtE5/ar3JL6cE+9j+oF/j9ft0/Y3X+NLx41/9aYetF2y9rQfM
XhQPxa/kNf8wtcfS+/88X/J3rB9R35HW25b5jIuDf6Xd7+kf7eSddvylnTzW
zj7RTj9pp//oN+8nL9J6oYr/c0nbeDr5R36l9S3p+XvyIJx3T/QBecf+qZw3
+DnaZ/iZvMavsP48bz0q9yXPF+OV+sP/3oc+9Qd7n+fFJ+Sr8FN6Xw7/H33z
/z2PHvGz9kK8TG0J9M7+Ig88j7+L/Zg1Yu34nX1hv5ZqVW/XRv56q3UDb6tn
txTtBwV5hT6Mn30AP1k+H/iffe395Aec+mvsVfEN9qjn2Tdwul6eL9TNHI1m
PO7YGkzfs3/IC+uF/sgb9pD5pfpEf+gpzR9X5MvdLWE9YftJX9pP9h1+YS/A
9st4y/7ifrGH+KP0b5o/TTH7w3rpH7+m9GS81of/jl7053nrhZ/S8zP2Vz2U
+mPyV76DPIPZb+hNPoj8Fs9TL0TeV+J7uX5fp/j3M0P8GX9bD/YdXOY3Twr6
8IkSl/GnRyOu3p80pjm+GN8t42v1YG9aL/vzS6s79Epbe5w/Z//IW/yXxl9h
9O95/aXnd61P4W4v28g6F/bkiUF/kmeFGP2/yG+l/x7Hp54a/Thfflwx38vD
+8UHjD+tp+B/qoelb/nH9hu9y3dZn8p5vPnS+1WivS5eoj5Q/Ee8hD6UnxUv
Sc/3Vs4HrRHrOcWj1FvQp/xD9L9iES+YGOrpYPHq6n1Y0Z9zPtb+hvrSpH4U
ps9aFm39/SpBvt1e9LdK0K+p/oXxC3lufVN9m95Pb331R7/qj7xl//K/4LSe
y/rC1oM84l/Axkfes1/pA/qyoMelGtkexbofH+w//jj+T/0D8lC7/ryPvmQ/
qC/yPPp1P4T9RO/ihyU9NLKry/xgiBegd/TP/oXZk+ghtVfwH/nOn2Z/mB+c
1stoJ+/JL/KNveZ91pP9lfpr5LHfw97n955P7UH0BFe+19Ax2iPsdesFV+rR
7476VzxZu3hBev4ENn72Fqw/40/9Nf4WjP+MTzyR/bZ/q/rZb3h4vvK9kvZR
H4f7HhJs/ek78h+mH/A3/4N9g77U26T+qfmYP8we5i+yd9gL1l9/cHo/EXtI
PDPNT+M/GP1aL1h8FD9pt39wun/eDxuf92lH7+QtTB7j97QeNr3fPo3f8+/k
1+gr9SPsf/HJ9L5+/iV9xh8s+lsvylv3KZBPxo/e6Z90fvw/80vv5ySP5ANg
7fxj8VT+ifGiH/q8eh9YI7m/K86HPKIv6JO0Pp59w39hz8H0D32vHX2S9+g9
Pb9Zuc+vS5Sn9B+s3fzFz6v2Uj3EB2HxQfrC+2Dyj7+Ynm9M69vTeopUHpOv
6MF6wOl5ce36T8/XpvoFP1ufiv07a9Q36A//eb5SDzVr9Pdh+sb7/J489Dyc
+g/kNWw8/Bvvpy/S+J/1Nh7yDyaP2DfsO/YG/rMfMPsDv+tvYGkPhfHoz+/T
fLj5s3/wv/eTZyH+0cT2O5Wn6fnRVF9qD99vS/g31ZeV87I/1bMVi/62CPYH
+a39l4LQtgjjMR/5B/rR+ZiKP3VJ2/rRtP4nvW9Wvr16H/vFIT+Q1lOw/0p/
Ldp/pXyMGL/gB+NHj/w99jn/gj9G3sLo3/7hp5S/03i997P3YfKIP0bfz1U4
iCMy+v3SQp2dnvGH1C+X/nY8H4E+UnuTvEDf2skz8go9p/o7tUc9j761+31K
r9rxs98bj3bPp/6F8XofTN9UznN0bFS/l/Jx8v3Tn6M94vnUPsHP9B1MH6fn
8dAb/re/xoveYPKP/5Ha56l9n8rf9LwNetOe2v/GY/3Fm8hj74fZi+wj9FWw
7xONbHjhsE8K8jw9H0Ve2l/8EuK7iTw0PuOt3C/Xoa3+gvEbrL80fmm+fm89
tVtP7Skmr7zP72H0a/wpTs/jaIf1Z3/SeC7/kzyhv/QPW2/YesP6S/0N+1XY
o3PF/dE/+1H/7A04ve8j9fdTnJ4v004+er/nySMYv3q/du9nH5gv/kC/5Kf9
F6/2vHgHnH7vgr8hfgaLZ/E/5Mur5/eiv0CfyS/JT8Ha1TuobxLvY1+X9TUT
Az+k9ph8ini7+jT+6+mt7D004tJ/HtPM/8d8NXoiz+DK/cKzxnb7AXs/f0J8
dL9yvQPuUvrbGXpEr/Q5TD/gf/ZaKv/49+H+6SZm36f1PuoVjGfPgmAiXv3X
1h09t9yf5WM8XLxMPBhGP/I72tlH4vn82er3+iKmf8RL2S9p/la8VjxD//ST
/tQLpd+r0j95pH84/X57el+AdvVq6NX4YfFs9pz8RpofTr/Xzl83/jS+TR/Z
L/xgfjD/OL3v1f0Onq/c95D3f11OnW9PHh3qeavnq2K9VCGvVm1kyxf8eVqI
j1e+D7FGtF/EJ6ryJO6H8y7i4dbX+sknsUfE46wn7Hnyw/ki6+F52PPG7/lq
PmlkwOq3nA8SryffrQdsPdAreeF94tXWS30U+ch/gz3PP6jEK+L5oSw5PxT2
L70fGf/ITzhfxN+BzSf9/rL1sT/OB8GV++hGxXb6kbzEvzD5R36S9+rp8avn
tXtefal4Jkxeyc/J/1W/Lzs0yPc0XmT9xe/RK0zeFOLgxJasYKeBVwZ/0vOw
5wvxODw+L59nfuIJsHyk+ZIX7Bn+KP84nE9p0qP1Yp/gF/Ypfqrcr75ko839
U9bP/sHsGfqA/mTf2F/2FnnLfijrO+qhvtb80/sz2L8w+lS/RB4aP3scrt4X
G7H4P3vVesD0L3uc/oWL6czcyK4pNzToY/Ew8TjxMPNL44npeWjxGTjNz6T2
ufk4742+xAv4m96Pfqxneh8Z+uG/84ftZxr/Ie/4A+Sb8cLsLfl1/bGXYPLO
fMgn8S7xNfn/Sj5nSsTiA+hVPoH9hL7T84mw59lHnpfP9DzsefkW+la8CzZ+
+2O81hO/8t/wj3aYfLB+4gWpf2d8fg/zH4wXNl/7ld4HST6pJyCfYPkN/Ie+
2Sewel78RL7hX/UT7GGYvILRn/oB+J5Wef9iPXuqmM+hYf34kzB6Rj/ycehL
Pg6WTyn1dVxP/Eo+oQfxCfQCy/ek32tlj9Ef6mnY1/yr6vcRx4b7pZ03dR4e
5n+xj+Rf2EMw+wuu5KsXiPEb+Wv+J3vf+9g/7BHylf1CHmsXD07tGfIUtr7s
oer3iiP2fvaV59P7oNP7gdP8F/oo9M0v9WzHYj3WCfjmEof8Gnmqfrxcp7OC
P5vSi3oI9AHjZ/YQ+Yj/6E/yjbxlz9DH5AN5kd7Ha7z4Nf1+AXsOf8LhPE5z
fnApT0aG+dJX9An61z99g3+tv/el9ejkJ/2anudlj9JH5D1cuf+4Q8zHik9Z
b/oV9r70+zb0tfUmv+Tzy/hEPJ/ovDH7uVovXK+ezx8Vsfb0/Lb6F+PzPJx+
/0k7+0m+qNjHNyO9ozfyEj3CnqePvA/9sTdg71u19J8CLut5h4Tf0yf0A3qB
K/bhT5H/4DRfhF9h9GQ+aXyJfnI/qPfD6X3x2gt9vVCu74v1PDzEQ9L7P/nH
5CV/0/PiA+IrMP+GvA752vI8SpCP6J885L/B1fvhozxkj6Tnacr8zUXJfUqj
Q76aPck+s/7h+wVNrJ6Ef1T+PsYDjB92/on/LV9h/fnL6Fv8Edaenpeq1itE
/1o8mT9ifWHrM75QvxcEfUr/qoegz40X5n+n9Y/iLfSp/Ix2mP+anoeD2Vdl
/G98/D5EWa+ZFeGQUdE/KJqfjfdToUe4rF+P+RPyl/4iT2D2X1pvKh4d7kNq
yotif3P5tUlBT72DvkGv3uc+IvoA1p7eT45ePA97vnzuwuA/8G/QM6yd/c7f
pU/ID/pNvEM8v6D/Rj1bo5xf4Ff9iQ97Hyz+b730Z730Rz5Xvmf2caw/Y0+R
d55nH/Nn4CJ/Oz3ay+yjCr3M0qjGh7pG+4+9RL/C9Kv4En2KH2H0Kv7AniGP
2A/WD6avxCf47/ZLfNz+pOeb0vvujId/wD+B+VfBH2vqU/2hDzi93047/WG8
+BlmbxsvnNZzshf9HvY8/cYeS+/Tt3+hPqD5vPWHtet/bOt6H9aSFcs3/6Tg
74gHwfjfePi7Ib+T1CPYf/ya3jcHsz/wB5zm+9P6X/IKfWiH0/vZtBfh4Ny+
u7Dglx4B/1To/wPDfOiX6vmMkcEfcT6Kv+A8LswfsX7sEfFH/qB8Hv0N0yfq
++TL1JdX8meXtLTRT/w3OP2ejXb1LewZ65vel8bfJP/QH32a1p/D4vfqW9wH
4rwD/QtrT+8vc57ceTL60vhS/cm+0l/VP477qd6G/pG/gslf/YfvvTf1MZze
f6y90IfbxfPM1fsMLgvxav4I/0P8hr0Aaxffog/wm3jmhcV2HRP8KfIRf4kn
wn6PXvkThX6YUc/2L9bpkKBf5i7sgwGBv8kf513S+6fS+4zQT7F/z+X+aGGf
HRnwjEK/x/uO6A/xoFNK+zzYq+I7+ifvvB/u0cpfT9ezYYU/F/EBxe8HB/+Z
vHe+hryH6QvrYTz44ZHCb3JffbS3YfxeTG+ntvYqf1R/7AGYvY7ftMtPspfl
L8nDqr06LPAfjF8q9cb3xXyK/mH9p993Uk8mXoOftVfvq297vwZ/Cb/oj750
/4r4Gv6BK/fZjYrnLWH5XP6788HmL37mef4W/418IE/oA/nG9PvgxmP90/1I
v2+jXhhO73sUL4Er9U47tK2Xof/IE5h8q5wfWDue9yK/YfameACMHsgj8Qz+
CXuPfwKX8ax4PoG8Mv5w/i+pL4bFY9gT7H+Y/c++MD6YvkvriWHxDPMTfzM/
/YsPWS/2A3oif9VLip8W5mot1rOp96QP1SvA4oX0if0if+1nag+SZzB5zD6E
+fvoE/2Rd7D++DvsNfyE/u0H7PfsJb8nv2D7j//RO/5HP7B4IvmtHdZuv40n
1Fc364vC98yaWLvxaofV3/CXYfLIfNt8b6OJ0a/1hdPzwWm8F+YvirdZD/TP
n6jY2wvE/RKvoo9g9qd8BHuTvCEf0bd8PHvIfvEvK/f9d437Ub3/bGiIv5Mv
5if+LR4B40/Y/NN6U/qGfnX/C3lOvpH3sHit8xr8CfEm/oZ8KCxfSl6Il7IP
2ePomz9gv2DzUb8pfsl/xV/0G/ry/QX3IbF36d/q9yFj/ST5aP3Yf+w9+5vu
N/moHtr8zY99YD7yc9V6wHh/PnoRX6Yf5Ldh/Xu//tGP/mHt6X3yMPnDf8G/
fm99YfEL8W/6Lb0v237C1f2P9VP8P/4WnNor6fdU3X9BPrDn8Q97nzyDrR9/
0HrA+IN9oB5HfSL7WryavS4/wt+Hjafij+wd5TGc2q/iT+xF2PpU76uO9X/s
ycr6LdJoc59p5T7LRWK8HP9Uvw+b3icQ9RX6Fy+H2Wf2D3/iX/ae8xdf5L9e
7PMRIf/kPAZ/GD+Jf+NH8pQ/yj5Qn2u8sPGwZ6vf307v5z4x1PtU7vO/JOYD
xKfL7ZgY+JW8xM/kNXqk70L9SNPfQt/403rB7K+yHvKM4M+gL/aU+EO5frFe
yf7wR42XPoaNj74Rz4HFU8n79P6ayvcFO0b/nD6ij2H5ef4eeQ/TB+RJ9fu7
LWE87t9Oz2uQl8bHnhUPdX8Jewm2X+QNnNpT+Be/iydYb/RUmA8rxfgGfqx8
v2zJGO+gP2H7I55Jf1TuO380xoPpT+utng0/i8+Rf+QHfoDJO+ttfdhPnoc9
r54j/d4GeSb+YryBP5r8Ir9Fv8L4i//ieetJn9Nn6AFGL+SNfCh6we8w/0C+
XbzYfqBP/I5+Ye+nT2H6FP/oDxZ/N3/0CosXo2f7rX/7wZ8X76NvyTuY/mPv
sifFc6w/fYF/YfFX+229xXdg+sN86CeYvDd/8o//pn+YfWC/Kv7WrvF8B5ze
BytfSd+Tl9YDlm/hj8gP8UfYw/xLOL3fgrw33zS/wl61n+wz7fxd7en9Xp53
/zx6gCvfJ1ojrhf9CYsvkQ/0H0w+my9svvwp+4Xf2WPkASy+KP7qPIf1IR8r
9zttHev12QPi+zD7yP5rh7UbP/4Tb4Q9T59U45tDgz6unJfaoVG9T2LtGP+H
2ZviR35fdPdUjBeRj+xL60G/GF/1ey6NEO+D8Tv7ir2m/oB/pB4Apj/IE/lW
8h8m//nLqf8LWy/6wPPku+dh8qrC78OjvmbPp/eFkOfkFUz+21/ywO/5w56H
6QP2Mv3NXq5+jzfWt1hf+wvTR/wZ9jLM38Uv6gGS7/uEfJH7jmD2MflvPPwP
9XHV7xOMDPKBfc+ehtmr5DV7nn1qP8mrkJ9qyjf0n96nod37yRP+Kv1qP9P8
tvmJX+AX9hf7jDxlX7EfYOPF7+4j5g9rR7/oBz3SF5X7fNvHeDT9b/xw+j11
559g+lj/3he+n9q0V9lf6It80T9cuf9jVMzH4i/vg/mP/GH0zh/2ftj42Ksw
fyL9ngks3sI+qH7PZnjIX8mPVO/nPjv4N/xN9heMHuy3eibykjyFzZ/9D5s/
eaM/mDxhrxi//YPZO+JD5AP9UL2vfWSov6JPxB/ID/Li4tI+C/EF8oH8gPmP
5AF5AeNH8yMPYOfv+DchP9ykL/uF/vAbzB+v2JujIg7ff2nSF4yeqvbdiOQ8
74hwPg19WL9q/LORfJ8uymv8rj/Pw9X7yy8O+UP7Bcv/8afQF33ivkL+v3b6
T/yi1J/x/MdnrcP5q56NKupTFgu4y+IFDvOt3N/YO2Lyu5LPzP3Xhwu+it97
YL+Th+LF/Gf+SPH72XL/rKCvA0P8wnoW/sCX9VCPwl+qyNuHYn/0qfyR9WCf
i9eIf6gPuKEsCAvxJfVR6mudhxKvuLyQJweHeID6Yf5Q5fxF6/2/xfrtEvwd
8kD7K2V76W/n+uqPgh62C/n79L549Uby7zD7K61feq+MxwY8qry/JMRn2E/m
a7zqAayPegPxb/pI/wW/Tozrd3jrdoyLWPyDPpR/wf/qg+VXzJ89L59kvuxV
9qf4/NWFuDqhlJ+1iNl/8h3sA/4ff6xyv9fX0X+XP4bFF9J29gp7sPK9jgkx
f0LfiC+E+1+b66UeS/1iQc/317OpJQ71k8sW498n1GduXOIgb8r46diAn/qx
NUJ5aYiHs7/4D/xVz4sPsZ/IH/ED/gR9J35vftrD94CTfIz9w7/8DfY9/Uce
VeNRsZ5Kf+wP7ezb6v3IsT68ffG+0SH+kManq/mHyN9w5b6PHaK8LOb7e+SX
Qt/l9hfsvI39VP/iPvAQv2rSI1yxlx9taVNPa//ZV+X57gsCPaNP+lm8iDxP
78uVz4fVF5H35qv/in1+Z8wP+738HyxfBPMPYPctweYPiw8W85mzEeqlxAfS
+6n54+w/78dv9A99hx/Vh+1a3j9R2utbxHgIeoXFm8t8xdgQb4HVM+IH9Yyw
+Ih4LHss/f5I+n1U/hb/Tr63er481vvx18vxR3uBfWH+VXsj5tsq9vvG8fsf
8hfp9y/FK/Ab+iW/G0X/fUK82u/xr/WDy/uyY35OPQj7UH5O/I49TP9PKPMP
wT4WbxI/Qz/2f44y/xTwmuV93UG/uk8Bvrak5xCfEb9Cj54v7dO4/qU9X6/6
m0s0sjWK9x0bv0fZ1I/sT/QnfoOf2Qfyr8YnPsF+qZw/zu2TB4r12CPMp/L9
wbz9tbI92A+ed96l8v3Qi+vh/hb+9cmFnRa/P1PJZ+T9vVTaQwGzj/jL5In8
Dew8rfgNfUdfi7fCzgOkmLxEz+n3WNN6KvqMfb1MWc8Z7MX0/qX0/nr2s3b5
f+3iY+IX6f1u6fdbtMMV+2nWtt9v8H7Pq0cq6Pz7enZdsZ49AqZv6Cf2XSEf
PorzL/zXB+vZjNJACPRivOxN8p2/6TwY/9/5+VI/x+8Li2fwh8iXIeV9SuF5
8hW23zB/z/3x7EXytvJ9kksiP6Hf6n3ocb2tb3o/Xvo9EvcpaRdP83sY/+EP
2H2R4gGV75v+FOtz2Zunlfo93P+gHpj8q9y3MUv0VzyP/wt+mBz9B/RqvdAL
f49+lo/QX/V7FZE+CnVzT0s4r0D+WV/vtx6F/Lsr+o/kF3sKrnwvu2OjzX3X
2o3v1HJ9wvPp9yPoy9KNbwn2N3tL/X1B3zm/lPbESaF/4yv8xNfq2fWlvxnk
dcUfnx7rk/hnMP0kfsi/kt9Av+zbcB9Ssx5Pflc9Q2GeTIn1DOpLKvGuDtE+
htnH7GHxhmL/20X/hryExffEn/zeeVTyRn4bf9Cn9o8+YN+RX+Rren+meiHx
gEq8+tG4nvYj/V6UeDR/XX0ufta/80Hezx+YUdo/AYs3+L3nC/NmpkaQr7Dz
A/iHfPH9DPwi/gbTXzB+r+jzWWI8A72m7eQPjB/xv/21vuKZxfrfWs+WLcY1
PMg39+OKb5OP9Ln4oP6Nhz9svDB5oP5P/sR+OF8NO4+tXkY9jHwD/Q/bb/TJ
noTT+19h9jT9w/8o/fWzQr1n+r0x+Qr9p/dNWx/tMH8cPXs+/T4c/oXxt/W2
PvaT/8E+8nvPp+fXK+dV//E9RvRjPJ63/zB6db5APFh8V70C/V35Pnnv5Puw
+fvT+zDwk/t+nR8uxNMjMd6IHvEn/sC/MP5mb6Mf6+/39qtSzz8j6oOwPk15
SH5V6lVb17OMNwT5sEvpfwTMH/G8+IT3q9cq2PKJRlaK40nBPkQPsP0233Q9
vQ89qP/SbnyBPprj1y4eZfzW0/hTjD6NByZPjJ9+cH4bZq+IR7N3C3r/JtrD
9JP4uuedb9YfLF6BvqyvfK16GfaselpYvSP7UzyGfSGec1mpr4O/zB9DP+xB
8RT1K+wL582cR0vP7/Kvi3TVko1s9jJAGOxx9oPxp98PerQ83x/y2drZw9pL
fz7m2wr2WbAR4mfsF+tHvqfxwPS+RvJBvLbsJ97XmN7HmN63mN6nyL6B0UPI
xzb1H/nEP2cvOp/OPoPx/zOFPoj3BdJH7B/2Dpze/5fe7weLl+uffYOf5RvS
+7+9vwhfnFIP5/3YH/Jl7A/1QPiRvcy+kh+kT8gf4+GP+D15gN/T+3mMHxbf
tT7iS9b31PIixDD/9L619D61Sv3vfI029/eTF+LnMPlnPp4nf+yH8bAvK/WH
T9Szrwr/4rhgb1qvgr7/HeUt/8D9CMZDHsLkdRGvuKmejX6ilSCGZ0+3LtgH
uf1RxldDPU96n5f4fPHYhy0hPuf872elPxTi3XOW8byQP6Zfya9BJX0GfhZ/
oW/Z//aP/LJ+sHy3fDX/kP0sXmT/+C/iy+JX4m3sMfWe8pPqP4r8V76f8y/c
OqP1A76n2O/1S3mycGK/fxLjv75PJ5/Lflmx1sogFwV7C33TB/xVz8t/yu+S
/9af/cOfln82P/RGf7FvrdfK5f3LoZ6DPCzk2/Vt7fHUXmbf6T+NB4sf0Efo
Vz2E/aGP6VP2An+dvkrvo6jQTy4/nH/mP7O/9P+/kt9K+3v9Rsin8oetL39Z
vLqQP/NFfS9/QH+W8jzedyP+63vVzotNL+Jnl4f8+UrF+p9c6q83Yvy82L98
/b4pxtsv6G/xbvE29aEFP+fyfZ9y/jXjFe+hf8UnxPvV12iXr3Mfkng77D4A
/s5+5f3m4TyF+3DY8+St+g3xBvJE/NJ4+E+ntk7os9xeyUf3xsynZOIt5Cv5
xh9EbzD7rXLf/RzRfivG+0js33lW/kAhf06vZ58U+3Ve0Ff0X/X858Cg7+QL
xMfJI/UxMP0jXlP9vtgZQf+l5yu9Dz/RH0Xz/NFfKZb/i5Zw/zz6837yVX09
/TepEGDjMvn/Mr4/Mvk+/KUh/za5rF8L8q4S/1413q+KX8ULYPIWVg9e6LN7
6tnOhT4ZGupp8CN+JW/52/uW+c1AD+n3Tum3Qp+eXw/3z5MX6tPYt5Xvcd0c
7Xfvky+j75xHon/Yc8ZjvQt//Paoj8XX+Uvmzz+U/6Jv6Qf1+fwR9n3x/pw+
nivswUMC/4q3O69H3lXPy50Z9sN5LfFm/lb4HmnTHivCC6tHfww9sNdh9dzy
ZfLb8kH6E19kf3u/8chnwu4ft/7Wy/2P8u3sD/47+4085g/Sh/Kr6tHU24m/
0C9+T/6QX/pDf84nsL/Vj8gvwfKB8lHkI3mJ3ui/cF6vcp4w2vv0o/gDeZb6
8/x37TD7Mv3epv6L/T+6JbuhINArQ/5cfgY9iS+JT6TxBvFG/af5K/lD9jb9
pn/zN1/P6x+WbxD/57/Kj3uf9YfFI2HjLfZnsZhfIL/ZF/IlztfzT2H+Ff8J
vbGP+AvrlfnjcF8IenJeUb4iyMtmvRp/Hi7kx+stbeodYPXDlfqPvRvJeddY
v8D+VV8k3saehK238/H0Af5lzxb6bJ1GqE9jP3i/+kD+nfguexeW7/Y+8XSY
vMEv5DV6qHy/eIvIf+jXeNTPlfrz7JBv4M/C+IG8pY/Z05X7g9vH+3nUH7Hv
2OPqZcTT0Af/inwzH9j7Upx+fwi/s+/Fa7SLP2unv8TntMtP4GfyL/3+jPix
dusDox/r73lY/+Kx+ofZJ8bLv9F/5b7QRvSv+VPoifyRPyUPUoxe1P8dW/jD
8fvR8gH8k7QeENaejhc9a+f/F+yyaCPYF8aTfq8Q1m7+sHxQwe8/Rv8X/dpf
8Tz7L1+Z1gvyh9lble95DY/nrdRfq//jn/A/yPsyPhHtFfaL+Ct5Jz5Hnqlv
w9/8Ue8X/1Nfxn5yX2ppjzRCPpW9x7/Bv+QX/qzkp06M84XF+9S3qidk7w0u
6OfCIH/lS2D16eSp/Kv5Gy9sPuYrv0Te6p//MkvhL10e6hfJN/Fo9I+/YPKC
vaCdv8uf4Z/hD/2TJ/i9aO4S+Vf/npe/Ig/IQ/yof+PBb8aDn2DPV+6LXSHa
X9rFN2DyU3/yM/grxfI/4h/4A/3w7+DUvkJP/EH6gf3Jf9UunlDer9uSxF/b
4jR/TT6Sr+SH/dUe6ov+ed5szpgfZt+TZ/xh9S1+r93v8ZP30V/okf1t/eWD
yG/7C8vnoofHynh6oG/1K+gN5h/yT9lb5A37iDwK8dVmO3+9el43+p/Wh3wT
n3uncMDPDPUX5K96e/Eh+68etbB3zsr9o4KfzwvxJfE18QL2KPtznlIAh/Nh
4tXq/fkv6sH5f/SV5+l7+oP/k9oL5JF2+lx8B/3A6f3r+B89WX/2OHuxer5g
ZNAn7DnyEb/JF+JH7+cvkf+e5896nj0HkxepPZTm89P7askz9Oj37Fn2Bf+H
PRHu72jSL6zdfrGXrL/1NF/y3fvYk/qTL+EPi5ebn/61p/6ddvxrvOwX8wv5
6GY7nNpT2s2HPjEf/p33ex7WX6g/bNoj6sPxr/gxLL8CWy/6wnrJL6r/Js/p
/zIeNiqst/pYvxcvgtnr4h32T7wirQckL2H7Z/3ED2DylPxnn5PH/GP+Rerv
pPJe/I48Ee9E7/oXz9W/8ZqP9UnnZ/7kOf4xH/FlmD5AL+KZ6HXvsv4v4G5l
PD/kK9mD/PvK9+Fy///4Mj9Y0vNxMZ95dTH/RjatzBeEeB//WPyt8n3w1aO9
Sb6In9nvtP7BfGH2jf0QP5d/g8mjyn2LjXjezHrTr+gxtafIe78nP7XbL+3k
DWx/5VfTdvqB/OU/2/+K/TFLjA+hn8r34mZutKEP/AKzP4wnbedPkV/Wk7xi
/3pePMB4PG884o2w36svE98gv9Q3sC9h7fwh+c7Ni/VZM7SjJ79Psd8X5zH2
aFS/331kjM+opyXf0LP4v3pJ9Gw+/DH5BfVzsPMQ4qHa+Q9pvIP9z9+CteN/
9EvewWm9kXiq9bI/cFpPhN71V8lndWxbb+R58px/QR7yJ+hv+0+/2e9gDzcx
eq18T7UR22Hykjwkz/WPvtEz+vB7OHyvL4mn0see598ZH4ze03pWv9c/ftY/
+0h/7EWYPBDv+Kzof7MQL0nrYdPv5aXxkpHl7wO+rRj/lmF+/Bfvh9G//UAP
1o//BYtnb1vUkY0I8iqN96f+t3bj5++ZH/sUVp9n/NpDPDJp5y96n+fRn/eb
H3mJXyr3r3aM8VG/V/8G4z/2iPWj/6yX8cDqiTxPH6Trjb9g/Ge+/C/+AX6F
tdvf1B/wPH2F/7TDQR41sfHA5EOaX9FufrD6trT/yn2Qs7X1r8lT65/aF9r1
X8nnzRL9k2L95o76lzyGjUd+hT6Efe+KvQuzj/XnvhX21Aet5PPe6HA/DXu0
yB+sGfOZ4mniZUW48rYY/yvtvYjps+r9qrEeSr1i+v01mLy3nqGeLbHP7Af7
TLv1wh/iBeST9ajUSz0Rz/eH8z5NeoDRE/60Pr4nXMk/rRf1ufWF2Qt+X4w/
xx8XA1szYPaH9XL/j3oT8Xj1i+q51G/A+EH+XT0k/tVO/tGn5Jn8DfvL+Tj7
Q56QDzD7B7267yLUkzfP08HqUfjHFfk/I7nv48e4PrD1sz/i9ej96KJgdnSI
j6NX9hr+kE9E72X9XEtWDG+jK2tF+f35EYsHk/fsA/V27AP8CrPH2W/ogX1Y
+b7vPW3zIeKb/AVYu3oI60HfwPQF+zK139J6dv6K+Cl/B73Q30W4r3MjO7TY
7/1DvF18EJa/DfmKZrv8v/sY5B/VX6oHgAt7/a3oz4l/o0f1INcW29k/1A+q
H4Er3yN9JJ7Xc7+/+uxCPz0WzxMU469F7D4c9SbOZ4v/qT8UL4TVGztPz1+G
2fdF9z+0VO+jXyPmT/jD+A09ksfsY/le2O/JT/fH4F+4EBc5v75T6I+BIX7I
X3D+kjyG03qClN7kk+XvySP3y6DfcP9OIp/5w+LB7Av6FxbfVN8Fu++kkEf9
Y32572up74Dtr/i0+Ax9UznfOiVi+Rjxner3CAbW1Ctbb8+TJ56HPU++BHun
6a+Fepfm+gd+b+LgHzefZw/D9C3+936YPiAv7Ad/Dma/Gg/+grWH9zf3F67E
c26O8UT5hkp98Evx/olQ79Gs53bexPkJ9Zjqy+CK/Hk8vk++V3zeeQj1IO57
cR7YeWbYeY7qfcTxe6DsLfUj2u0XTD96Hv05L+F+C1j8vwy7xfmwR9hHMHuQ
fMTf5LP3k8fyW86bwM57sF/Up/lesfMsMHtEPJA/I/4IV+5/ui+e3xN/dP5V
vhuWv6Gv1Tewb2D0KH8Esyc8z56E5T/EB2H5Jf4Mfa1dvgqWX/W8+hb5eu+z
X+Il5Dn7n30A89f4C/L3+uNP6U88qHKfaJeI2ev6g/l7fi/e7/ew58X/xZPJ
c/ZSivlH7B3xDPVI7jdhT7KH2Aew/Lv9IA/Fe9ATfYneYPUtEwp78rRg36fx
kvQ8aOV7Kh3jeQLysPp9peifqF9W/6xeDmYPV+q9Foz+NXvA/cDkq/Nq9CWs
fsV5Y/Ws5BNMftGnlfssr4ryBv2KZ9L39CnM/pZfYq+rp9Pu+7janbdJ61PS
ehTyTr6p8n3iE2O+wn00sPGjX/vN34TpA/lTWDzEeQr9wdaffyYewp6F+Vfs
He1wNf7cSO4nifWiBT0cFs9P0mfqaegz+pU+g8kr8gCmH8kv8gA/8YfxZ1q/
Agf+bPqj+D2tl4ELe7pez64s5H3vkD8TbyF/YPU0/Cv+q/d5v/yK9/E3+Kfk
l/HD/I99Cwfi0CBv0vM+7CX6jX8gv125b2+WWN8Gyy/D/An9weYnX6p/2PPk
o3ZY/6m/ab7iifY/xKeb+8+fgMk/z6Mf/IJ+rBd+hr0PvXsffvA+2Ps8T/+Y
P/6ib8nHyvcMfor6E67cT/B7tI/xN/lrPtrNR7v7YtAf+uRf2480X01fmR8c
4sNN+5G/nOZXyC/Y+vMvnHeH+bPkNfke7iNt1ieG80JN7PzSEYUAPzPIM/SA
vuwvejQe9Gu81gNmv7G3nBdT78X+g42XvnKfh/Ojxg/Lz5u/96X1zuxL+lk7
/W1/0Y/8BPuG/cwelr8Rb2HvwPI3sN87j8We5q/D9DX7mjxmv5JnMPll/WDx
IPJMfJW/yR6C6V/jc56o+v20s8L4fE8Jdr8IrH6J/CDv0Bf5iP9g7eQZfZ3m
dyr1JTOiPcq+5X/hH/k5/AsH+7eJya/UH67cN/hlrAcI9QTN/IL1J5/Zd+QV
e9r4YfJLvntY8fwOof7O97hg9qLzjexD9hP/CLb+6pvT71eiB/ZYen8f+4o9
Qz7aH/JCeyU+kbePK+N9QZ94njxmD6FH/hd5D2vnj7Hn3TemfpT/BFuPwpy8
P8YTi3jegxGTP5V6+Nui/+m8HPuAfILJy9Q+QA/iZdYbf4s3oRcYvZH35q9/
6wNbD/wEs0/oB+tDvxk/TB/xt+kr/I3eyQdYe6Weae5o34tXyS+JZ6X3X6bf
c9JO3qPfYv45feIX+gS9qielj2D6S3/a6cO/Wrntj+Hh+2rksfN2sPO76q3w
P31Xns9shPil83z2S36PP1S4G3/HeDL+EH+Dxd/IT/YIe5g/jH7kd8Qv0++9
wORppT5zRqw/LLb7yGhf2C/8QT7TR/RJWr8kfoU+6We/Jz88n7an9er8M/UL
2ot4/3cx38C+I4/JO/KYPIT1T97r76dCHu0e/A/7aXz4DyZ/jU87e8r5O/YF
zH5QnwdfXt5HHs6P2D/3Udgv2H6yF8kX62t/jC+NF5MX/E/+qvs7xCNg93U7
H+i8svMr4hXuL+bPw/z/Ur7E8+fl+kQsPyu/CKt3oU9h8VH1BPLBle+RrhHr
wdiz+N16lvnLM0K+m72BH9T/kHewdvEk8Uv0jb+tB3mv/h1Wjw9Xvm/wS71N
/FA7eyiNF5KX6Es+yPjFr8gP9gFs/uSJdvSMPum/ynmF2aK/j/7IO+tlvMYD
y3fgV5g9RZ/K35KH5CVsfsG+bO5PqI/+5/1/a8T4lPye+yLli2HtlfxgI77P
/Rj8D/xC/8HyBeobyF/5cdh5evlT9yc7PwWLJ7EntbPPyE/2XeV7xbPE+ln2
Mly4V7k8OajIp/cN9i9/y3qn5xXM3/5Zb/WT9gOu1Hev1Kiez1y1rf3KftE/
eUze4w/+LPoiH+kj+ja9b5Y+pl+sF/5Mz1Npx+/pff5pPZd4hv7Qd8Efv+fy
obxfIdjj5In4h/X1Pli7/bH/fh/yI83nzQ/W7veFfsyx8dBv/FXyoPL9wjna
1i+wH8gH/jV9q97F+Cv3TX1Rb+OvkTfyMTB/0fey+IMw+ej8Fez+Hvd/iH/D
7pdQL+A8Ov0G40f+G/4lL6wf/yd8H6aJrRf/B3b/EHom39nn8nXmr54KFq/g
L4tnsOdTnN43rd4Jdn6cPlJvgz9TLJ/uPmz+r/tQ3Oft/k1YfpW9oV3+WP23
+/ec36/cp3xkfB973fkp9SHkgXoSWDt+5R/hV/pefRP7n71erM/BMf6PX9mX
6J39CJNn+F+7+3N8b0C+1/1l7Bv2HHrz/RbYeOSrm/erhfy1fNGOxfmRWJ9R
1sNFTJ/Tx+H+oWb+mj2lnT2Anqrf2zs25B/cF0lepPe7sW/ZJ5X6xBlt8weV
eo3+Ub86j0ieuz+jch/sUpHfYPJcvc7hhf04MMhTz+MP/ijMX8VPlfzW1Ghf
yodW6s/vifXqcHq+TL4gfK+6icP3nJrjo//5u/xB+6vehv7lb+Bv/Vl/+lg8
iHzm/9hvONR7/9P/yTH/SH0T/4b/CbuPQDwFVt/D/xd/DvePNOtVfJ9HfUrx
vv1iPQF/wnl39qJ6W/YhrL3JP+F8f3o/iXoK/hV6dP7bepMX9CH5gJ/cN0fe
u4/w0zKeG/jHfZGw71+pV1Uf5jw++iYv1Y+Qj7D7FrwPfryMHwd5674kv4c9
X+53xFeU52Ojf9y0L8X7+UOeV/+C/9An+0e+SX7U94v427B44PbFAbmI7xtW
WADhe130H3lMv6knho1XfEs+Q7v4Vnp/pP7Fkyv3438c9x8mb/Gf58WvYPUU
jxZxr1NDvQP9gJ9Tf5t/U8Q3c/x6ob82Le2jHyPGr/I37rMUr8S/sHb60P0e
7C/n2533htljzr9qJ+8mlH+hnls9q/Pj4hWw+kTY98PUQzgfqR4DZl/hF/VK
sPpK/gB7gX/mfiT1FOoj0AusHl19ACx+I76gviCNN9Av5X1aUf/7vXo+43ff
JHsHtn/l93oiXq3QHxOCvFB/x37yfSz8D2t3vyN9Q5+p74S104fom3xQn3B2
MZ1eIT9U8RdWjecx1A+sVYz/sHD/BvnvvgP+DSy/wX7Qbj3IU/fvyj+6TxXW
Ln7K/6zch3ZPjF+QX+Ir+NN9nuhfPod9URlvx+hfiIfA2vkj4fsdTcx+4Y/A
8jfep73Y7gEtWbG8v14Z9rviX74esftK7SfMP2VPVL7H/U28T4N9AWvn/7F3
xAfYy7D6+sULeXJAwK8W8urA4O/hX1j8RXyz1CcXZ+KbMPrw/TX2OH6ln33/
hX8Iaxc/ZN+iH/Q8pJAbA0J+Hn3C8rH0D1ysT453L/yBowM+obBvjgr3+7KP
3SdNvuF/WHvlPOmt9fC9QPzLHiZ/xOfJz/R7qOK95Kl25yfUE6lPEi9iT/AH
wvmdJtbOnmV/8wfpT/YtzD8M9nuznT9BH5GnsPmKF3hePpi84A+L71W/zxO/
pyn+4H4s9cyehz0f7NPm8+r7jI/+E99g76bfB1LvrX6ZflQPDqv/dv80+ncf
of2AxW+q+YJGm3iK+Cx7mj0gvkq/qh8RT7Jf7En5wMp9012jvnXfAnkivoN/
y/1Lv9fXSOId8XsBsHiG91XuP1w2xifZk+ZvvLD5WA/Pi2+bv/Pr8kHpeV0Y
/7AvA/807Un8zb8M329vYvEA8Qz+Y/r9O/pD/py+4d/D2tGH9sp9lh3i98Vh
86efrI/1tn6w59nj4qfiMeQpzF4gL+gn+wNb/9Qf9n7rD7PfjJf+Il/pL9j6
8Df5D+wn/FO5D3GOmA+x35X7ludoe/+J9ZUvQE/GA6uHk0+A03wxe2yTVnL4
Pvof7CPxe/oT5j9U6gt714M9C1f1c6P6PbQFYrxGfUQlf3Zk1PewfAz5lPoD
4wt2vyDoM1g+ID3PLZ4I0yfohbzBz3A4r9i0j8J5xSYWL1bvR7+IF6N3/gD7
F0bf+A9Ov6+G/sw3/R6W+J71grXzf7XD+uNPep6/6HlYu/iQdhg/6S/cx9SM
H4m3kPfma7yw+cH693vYfotHhe/BNP07++18mXb2eyEv/4j5IPV06X1gcGEf
tY/5Gutnv9hL6f1tsPoq8k47+Y9fxB/g6vcq6+H8qXyJ+9Fh/Ol8En+fvweL
z4hHk6fsE/4DLJ/MXjE++pR9WKl/3SF+H4C9ArNX1Q+F7z83623D95CbmH+q
/ha90X/ogbwyXvJG/Ia/Lr4Dk1+V73fk8/N9HfsFs9fRo3gP/4q8l/8xHudN
xO99b8J6w+iPflIvWMn/XlVv8/3sSn3SlIjNN/2eH33EX6CP+Pf0GXmoXT0l
+9l5E/NTT6neRfwkxexl/Fe5X+rO+H0T9AmT1+W6XhjsVftBX6aYfUHfy5/K
bxkffeR9sPnxD9znVdrv0b6QfyOP4Gq8qa39Ql6YXyVe8EQ9+P/sO/5UuC/o
n/7W1xGH+3qa+lF9Fn823C/SfJ6/zR7BX+gZf4t/wOSB85fsAfrT+RlY/ZD8
lfNV5k/+wewv9mN6/7R4vvqKMwv53yfE4/gHMP+goKMDY76CvrFe6gfU55I/
6Il9CVtv+Xr2lfyheK74MPkDe5494Xn3YeAH8VrxBv4Zf41/BJuv/C//0vfs
0b/7m/mv7Bf8DuufPPA8rD/naclr+VX2AoxfyEP8IR/Bvic/4aB/m/oTRj/q
V8UH+VMwf6vin70Uv7cF07/yXe5/YB8ZD/sIVg8hH+f96gdg9gD9Yj72N51P
+n0a8R36Bqb/xG+sr/dbX/ku2Hjks8hT8RPY9xTYY/iDvBBPdT8X7H4tWP2z
fMgSRf+bB/sTf1lf2PqS5/zXSjz/42hvVuurjw3t7H/tsPiL/IX7Pcr7Ks4L
9j35r96K/ob5v/Ih7Cnnz/Eje0/8Q70zeUzeVe7X+zrel6EeLo3PsBfEP/CX
+LT9LPa/U8y/kz/4BeZvhvOwTf+KvUufw+pD7I/6PevvPlHrD4f6hKZ/gJ7l
I/nPvn/FH4fRi/1SbyDey3+r3J+8TBIfvjnWGzrf6/sjsPpD8Tjft4Ttr3oJ
/akP0J94Cn/ffsHsU/E09ilsPeTDxWvoX5j+dX6XvcEeY2/gP/KbfcjeJn/Q
Q6G/3o31IuQ1LJ7CXhBv5z/Qz+qTnMfGP+7rxW8we4z+dj8MeVQ5//5z5Bf2
jfgazH+wPuIj1kd8QbxR/ML9TO7rcT8/bP74wfz5D/i3eX9PiLeqJ4L5b8ZT
+d5d+4jtH33LfqBPxDfT78njd/40+e58EH+EfeF7hrB8DPlAnsH2U77WftK3
noc9T/9W4tU71MN6sc/5uzB7ib8C81c8zx6CyXvn+7SLf/k9/xtmL3ifdvct
aff9jNI/Hhew9SO/zJ9/SR6QdzD+d95dO6w/+TPY9+fYI/QNewFmH3gf/0S+
kz1bsS8WiPWPMP/T+9J4D/rDj+Kd4qfu04D1Tz85j8CeLtd7RGh3PpN/XTm/
f23MD9Ov/F/6Sf6APqrIr48jRi/0r/OF+N/9BuQdfYK+YfKtvM+wJXyvqrzP
KWLyiv8Ci2d91zrev2P8izzkz/G3fO8NPYkHyqfLN4hXwp5P67/T8+8w/5N8
IR/JK/F28UTxZ/45+eV8sO/zqb+Tv7Oe6N33QKv1GecEfzT9njt5D9MH3s8e
Nz/+tO9L84fId/6S+ibyRT0U7H4Y9ErfpvY//xi/Gx/+oK/JD9jz6fkW8THP
w+IN6ffX6Q/7yR4o6GuW6I+rh3d+Ek6/V2L96aNK/KI1H9X0x71f/Qacfm+A
f4D/xePFZ6r5w1jfrL4QfcnPwuqT3V+Hfvgf6i/V39K/6Bv94C+YvUXesk/F
o8gDmH4o42tRXvOP1H/yL9PvdbMXnd9Sr6cej78Es8fFY8hvmD/P/vM+8hO9
kJ/oib0ifqR+vnKfTO9YzwVX4muLxXofWP0QfcLfJ/9g/uh8xX71C/KJ/C7o
5x/1LOqX+IvyP7B4l/1BH/xD8sL+wpV892KN6n2Li0b+rfqLIwNWvyk/wN8Q
72SPoJ/UXlDfS176ng15Cdsv37uC5e+sN3+wkG+/xnia+Ab6o3/4K/gFFo9W
XwTzf4rxtsR6NftR+f581xhfs//om/0Bk0/p92HgYj1ye3taWS8W5Ht5v8Nl
Ib9hvdZv3v8tn6H+iHwVz4bZR86Lw+pv8LN4MHtN/lw7fez3cKg3bMYDxHdg
8R/07PmU36wXf7Ry3mXr+Lx4Cvsexh/sZ/QMi7+zp6vx+hivZN+gL/a0fBbM
/vd9GPFT8po/CJNHhZ//bYyfk8/kH3lXqa9/N9ov/EH2lni5/XJ/C//c/Qql
/3h6qK+G2XfkhXwneYKe0/OxlfqKn6J+FJ+g/8SL6UfxbvGOan423pdW+b7e
4/F797DvBaqH8j1g9iP9TB6Kj8vnVOqr74v60/rjd/oGZp9Xvnf9Zoz3wehf
vJX+KL5H/WLUF/xt44HF+8kL3+Nir+Fn9Tvis95f8ZferLe5r5L8xl+wfEwq
z/mzMPonz8Uj7Y96Ofujng7GT8Yjn8c+4s/Alfq2u1uq36O8L2L7L95qv8Xv
xHvEy8TnUsw/13/l/rfHI3Y+g38gHgiLv9HX7DP3cxof/WT8sPU0f9h6qX8n
v8SPfE8SFl9Sj0Df8C/wF/2jvtx8ydtQH9+UF+Sr/YTRB3piD8D8afFN/gp5
5P7pUB/VPO8jPgCzP9Wvkm+w8yLus6JvKvy6QKOav1ggxh/YF+LT7FP0BtPn
xqtegf3jvib5JflH/AOzZ9X/wvSt+fm+Kky/kWf8b/6z/mD9hftl/3kedNZo
H6tXqtzP3DvWt7L/0B9c0a9zRP8X5l+yF/Ab+8D5TfNznlJ9Aqyewfkn/FWp
95ka/SXym7+lflb8D/+SdzB6xh/kHaxdfLXkkytCvJ59IF4vvwJ7H3lDHttP
+sB86Av+AXuW/ao+Hj3iJ/Yw/S2/Ix/Gfobxz7LFOkR73vkc8QfxLvEHWPwh
/f6H85LimeRH8r294O+wp9hP7pdkr4gnwuQT+cweQG8weSA/jv7UF1pf9gn7
pfL99Fni72H942/j4d+wp2DylTynL8l7mL1LP9Kfab6DfqLPUn3FXpbPSvWT
8ZOvvt8e7rtp0gN6Io/Qq3ok8RDygfyAxdf5m+Jt/E2YvWi/0L/z0fiJ/kSf
9Cv5ix9SeYw+w/czmvTrfAN65R+xr9mL8kGw/BH7t/K9mDvbrm9qj8L2S70g
TB7wrzzPXq3kczvH+CRsfdCn+ZuveFj1+22jQr4af5CPMHmHn80fP8Pkpfnj
HzjlV/M3X/Xl5sv/t/7sJ9h+yS9V74eI8QbYfOWb9ed9sP7Yu+aHX2HrQV9o
9/v0PupDZ9x74p0nHpzdNf/981zQel/RU/2vO3/gJrU7Dtlpw/d+rWef3TFj
5aWGb5qdfdw5v6+dt3+x8oSnH3x+o6zzbwvf8UWHnF43vneluf7onT14yVdP
zvdHPTvqsMV2frF979r469b54br89//39omXb35L7+z4bddb9OOZGtmHP35x
0U4/7FM75/bzFv3v37G+fvj4bt/N3y7GVw9s3/3vCT//43utd+93S/9fcvk7
pcPA65/ep/ZBxwdfPumXGE++qO9Z/R/JxzOwz4iJF++6Se2G62a/ZdF/3Bfz
0wXzH3rbr7G+gP/35I8df5zptwNqK1635U/bzpTkP/L+/n3qlCW+GHZgbcR/
Dpx/8db7UH5/asSTm6+T3f58t58/yvF2k9894tGZNs5eW3X8Esv94/tW822x
xDqvzRTPf087/d0t/pfjW8ct9t8Ff9y9tsrcw7v9nOOfJ682eN79dqvNfuHA
nzvl893nqdE7bT1h92zB8Tus0Dfv79Epm494cb/dsznvGdnuwHy+t6762L7D
u+5RO/GUPXZdK8cbnjDn2++tsVvtrNk7r9qt9XsDG/z3zafX3jY7seXGJTvl
6/nZLH90WX+hPrV2q08/s0/+vu+++nTi/zruX1t7yRfH79/6fdjz91txh0X3
y8b+8vveB+R4t4fb9frm5n1DvGm/P/Y56pends5mn/zy4ufk61dfY6fT5+m1
f23j/ddYfsdfov7tdvoDj9RnivJstQ7bbdwaH1/y7t1nuX7GdrUHrjv637vm
/sBC83X7cuRRq9VOWHzyuqfk9PLH8lt3Hf/5GtnoK7tduUquT3vsvGT/zwev
km2ya7ta6/qesO/MXZZftE+w19/d+N/jhizcp3bpPM+c9Eve/r8pxx81csQm
2eAbL1j+v/n7xq758lmzHrJtuG9j+VuPGbZ8/42ye598YN2Ff4v+6SZLL/pg
6/O73vjC11c9dWA4v3joqJFjj9+sR3bHD1seflg+3tWu/eDPt4etEt7/n097
vXltl67Zev96/oYD2kd7/uHNtt17oz9j/Fh/h6/Zrd+ZL+yaDZ3a5cO+efse
xzw0euezVgv8t8x/Pjzj4DuXrw2a+tYR5/zj+yFHPP3+0q30MP78bzr976j9
sn+3W6d75xx/u8s+q+3Qa99s5ueyl2b6OX4PbatNl7xm7tbvZY7qt/fZ7Q7K
Hjhgg4u/+TXWO3z2yXcvX5i/b8wmN7y60cwr1JYe1vPT7jke/lbt3PMv7Jr9
eeZRP/TIxzvo662nzfThbtm1U6atc0yOT+i2Wvs35tsre+Dc1W/6rEP0Vy4/
9ZPsoI7xvB5+Of/ghe684oWNsmknTLx7eoeYX1nmmqee/vinejZxmztG3Ndt
7+y0a+7puWFOn9O+WPOijdqvUFvuxwFDlsvnc+dpW7x41+27ZQec1vWwRuv9
vaN2XujVd/bMXplp2F/z/xzrxUZsdcpKx+XvX7v7rd3+881e2VPrLXldj7y/
DUa9NHqPS7sG+l33wQ/6ffbp/oEeflz1lb3abbB5Lbv13Ld6/xrzUx+8/X9d
J+f4qCtHrffRB1vW/jNzo6XX7zFe+/wNJ79y7D++5/DUJ5d3+d8/vmew5xWj
5vmplR8ev/J/S86zfe2moZMuap/3t/PMs3W9feJ2QZ4M/WC9pSf8J6vNvUb9
hh7/uJ/19i7L9103x1/OseUGQx/astZlyJ2HvpX//uTL79r8z4961v7d/8Jn
B//j/um+p+y/aSt//b7dHl8/k/PXwZsd83nPf9xfm01fdpPW9vQ+2Dvevv9f
5+Z4t9m+f+jWDrH+UDwo+/jMjsdO7lZ7fo9x3y6Ytz9+8BZf7/TpOrUD5h+2
/sr576+54KQh/ec7INi/+/xwwAn7nNM7/P6NSxuf3Dm5d+3RvbPed/0W6zc8
v/7vPabu/U632oa/3njVW/n6DtzyzaV++H2d2vzTTx32+58x3yLfPWLfHZcd
2bJi7buJZ93TqX2jev9Bvt/33Xjtdfu/v2LtyX5///7tTzE+ddu2r3z/Y07f
J2xxRp+FF4rnt+49bsQZ745fJbtp28c37fVLjF+tcs1zA/4vX+/RP0yZ6fMr
tsgefvTxZ9b5JdbvLrb2rXs+9kv0L45/+bRD+uXjeXvPEd3XeaVv1v+Hi/oc
03rfRG3ieV8/3zfww/xjTz52geX7ZD3W+6Tn4PaxXmW+dV5a+aeZoj/eZeWn
7jo5H9/SX/V+ZOGf+2a3LXfKcX/n8nGHLmPHrvnfvoF+d9nxjk+/OKpPNtf4
Yae9/UesP9hn7HO9uuTtp2W7XLrRBn2yRT6/anT7/P3zPvtIu3Pm6l3rv177
c2/I8fMnXP7ZJVsdVJvntgeOv7w1n9Czw8gh13bPdjj32F+65vSz6BMX9jhk
pp5Z1uuridt3ifE18ZDhnVs+W6TdZeH8406fXv/a/FddFPyrxbbb5Zxvp0e8
2+vT5vrsyoNrGx969wejFojxGO3H3Tb2/VO6jsz0N+2l3XrvPvzibMzLk56/
NX9+8J7dH/lkhRHZSo++c3nvVXP/4f8OGnb9H6fWHjvk27vvub6evbLFf+b+
adxStV2eb6x8wIO5PbnWElM2/OGK8H2NVV8b89qof19eG735iIVfX6uRHbTs
jJFPbXtp7be/b5n5kZy+1tmh3RmLbLhwbf/r111ywdbvWb1/1iXHnzdP7aGd
ausfk+Pv3l9wl8n3d6ot9v32uz2X497tt/7yvnsaj54161Vjrs7pb8nXj9ri
rwUWrtX7f7ntca3fj3h74EWzDpqn1uOpdX/fLsf7P/TZgyuN71R7YulhdyyW
48cufP+2MyY2Hj33nQufbbV/Fj/45Bk3t3yy2UfvzLzG2Tme0vmWMeO36ZCt
++7p41fL8eCp6x959kVzZee/+0u7N/PxrrjO9GUePHLB7Ma1Vrpt8L6NbOTX
E64689ghtX7vdDjszwMa2UktL+30yGmn1fo9vuOBu62er0fvi95a841Tsn0u
Omy7MyfXw/qLb/z8c8ffhi0yIttz9dt2G7tg/N4Z/XTgla9e/Um/I2s/LtBv
hZe2i/HOjS55/6IRuX/zr6cuuuygQ47L7l/k2w9OyvGGd9zXsde5x2Xdfpq5
462do/93xN8vDnhqeEu2eN+l1lvxmCtri86YebYvZ2tkowcve/Nda+6fLXVK
x3eXa60PGj5Pr2ff75mdvtV/HhnwWD2bb4f+10494qLaU7s+PvfU3H98s/uq
ny+x1/G10698fpNe+fNLHXbukrPv0bO21Cq7bfbHZ/Xso6dfHXfPiz2y85/d
7NYLH4j+0RuXvbj2Ql/EfHW4X65rjwfGNI6s9X5vpXevW77epv51q32nbv78
vyZl/z29W6fZ8/VeaN0LB8w//ZJaz++eWeeTpZPvKeT+Sd+T59pxzVeOyL7f
qfcaU3P/8P3F7n7vwDkG1e799c5Ffu7UGl/vfvOkTwdmJy/8xvUfTY3v498u
ctVFa/z1ysm17XtevvY2Mzeyry+b9szQ1Q7Jtlrg4+lH/ZD7AwutM/Ks8f1q
J//a6dW18/ZpR1w17eqFD6kNvGLlvU4d2JL9vcZe1/3+1JW1887sNdMW97Vk
T4/9e9q/9u9Xm3fsoFfG5v7WMifffPXzHxycrfPavrcfvmCsvwzfV9zvk46b
zrNQ9kGXn47cN++/07b1vy9edfva4WseeMEs+fgntftk7udO2qU2yzWv9ZyS
48e7jD91oXO61yZs13+JmfLxrffdUlN2e3qnbOeOx37Uu/U8cY8bdrx36o7Z
Jy8dd8XkH/P5rlTfZZvaztmYY/db7Zebor+96N7nrvnsAtHf3H6/u3b+fPYY
r/h94D6/fX9PvP9b/OqBr2baf+zWPWrdO3y9yu2t4zt60Mwzb90922bf5Sf0
bv0+2+fP3HVeu+2zX5b67Kg78vaVGmtO+36P7tno9U/bb4Ecd1uo1/QL19sl
G7vUmx9dk6/P42++OsvRl22fLf9Yu/rlz9Wz5SbONsepM3apPXHaO5OXX6yR
7XP1F9fMvsg2tT5/PrPa9bl/+MzfOwxf/extQvxtg4XPHvX0d1fWxt27/8Fb
vhHz3Wv//dV29SnRX/mrfszLz92Z3Lec48b02ther/SsDbr95Wfu/aCeHft2
hx6bTh1Ym9Rhh02O/iHahz2fmHn0z4/UQ374gONvX2HCd7m9t823qy5zxkm1
WWZ7/utvW78vfdcuO705/0611adt+8rE7+vZGTd/Pd+K07ev7bbiyY9993Q9
O3GWU7d94v6B2bhB03+aK/d/b/vtw7Uf7z6oNudPH97U96Vc/w7+s6XDdQOy
81facK1LX6xnh2/28Vs3XHVodsIa26y83125vH7soO5bdTy9NqXXtg99NqGe
nb/5MaOv/nVg7a1h7bKDcvzLPo07tvx7YIhn7Xfcm492eKdTdsHDv828y3y5
/J99519P2+a84O92OO2ISVfef2Rtqyt/vyu7uCWr99555r9vG5vtt8LtR7a7
vSU79MBHn2x38Rnh+3Hj3n98i+XOPqPWp3HFtNP/8f2481/ruGjXfSN/Oo8/
6xr3djzgm6Ozw94dvM4mX7dkyw3tP/XuRa+o7dT91ZevfaCRvXXq6pP37ntB
7bw7rlpxfL7/C7301hK/Dx9U+8/J89RvmtrIxqzw7dX7LLdhrf+B87y8xb2N
bJtP733pt/POz/r9MO6xhdZpZL/133qHNboNrfV46+HJ9dyeOPGuscfvtdim
tfvGNAZskOvnrd7/aY+f/rtR7dkVev5+b96+5O89l/zmyA1q7zz89Ki1c/zr
oo9/9clv64b6h7memvXsBXY4rjZj8jbvLX1CrNd5/uY+556wXfRvxeeuWPvO
90/qt2G249N73fLZtHp2TvdlF1t3u8HZlH/POfcq03N7bPDpR561xrFZ1+HX
3HpV6/3gB/Ya/vqwE0I+aMkhRy7y7NTTQnzr2mXaLb//0aeG/fnj6XfmemHO
ziF+8fFM7zde/fHkQL9D/tr9/5589oyQP5nvvfYXDLt5XPbgLI9e9cACsX50
1kcvuXmLL2P9zUv/nrzQoV/E/Ea3gd+2f2WzRvbcpm8dv8rhO2W/H/TS+GPm
iPUxnYcfcG//K3L7ZfMjv//+uLOzlyZtPeCCsfUQbzKef42Zs2XV6UfW3hl6
ec/7H479H3xo7da7547+vvjU6JNuemvXCQcE/+zKTnvN2/lfQ7LBLTeefWRu
P621zxsf9DlrcK3X6OF/nprbXz23O2jtMw8/Ktthvp3fXqie43vePP2Zq47O
Llx8tYM65fbdnv337rXgtMHZl98smn33aDwv8cNKm7zyyK9R/o87tj64nsvD
U0fPcdH1Kx5T2/OkcS8/3hpPu279Jw5f6dDsme0/OfaVHJ/bv988l100ILv4
uz2njMt///JV2W1zX9u71vn62w/5PG9/dMeNP3pw30Nrq9eXnvR3jofstt4z
L+w9oHbdFnsedf2cjez+1d+7ePDYQ2uX7HDAkTd/FeXRJ0vP+/2E2Rtt6vMe
mP2HH186bafssolHPrzm7DG+0ZhzhXvny/uf9NlRf215yd61Jy47auPV8vmP
OmWbkyc/vVe26iL7fHZVS+5v33n6bCu/sWH2x4UP7/xM3r7Y0Q9c/+DW64fz
v09svOAjZ901qDbrSzf3fW/WRvbNGSvevmu/U2pnf3b/zr2eivQ+fuVdzrzp
h+hvLfPey0PfvS9f77dXmJJtMSSbdsgp39yQy5MbX3nopM++HZtt0PnCead/
k/PvxhM2yDbYs/bSwjstN/Qf34vb8OFL7j7qx4gf/3ax9Y/tFOMXly9xyTwf
LtLIep/5vwt6Dji6dsXa95xy5vcxfkCe7dtlyEf7DN2k1vuw4554/4fo77z+
3uO/XDAt1quK731w/22d7trz+HBe95ABr5x6zv8dXHvpost3HJrzx5xXfzHX
5L8OqZ18yiFrt35fbrMff5xrltkGhPzpe//639S/5j2jttOcp3y39cKx3mqb
xVc57c6Fcnpe4NSJ13Q7PNyntsvlq8w++uOhtaeWf36POT+P97lrf/DMV3vu
2zK6ttbub+y//r0x38YemTjxwOnn7zY62NMf3Pjofw967NzamEEr7Xf/65Gf
tC/+zp0TB7+0Z5BHL4wZ8nS3sWdm6t/nuL3Ln2stNDob886FH3a7OtZXvH/G
i99t0ze3V9t3+HJgv0tqPcZsMf28QxvZxJWv/36zhS+pXb/orG/Nnq/Xp5d/
0LVvy6FZlwvmGTTXgjHfO3Cpk9e+plc9W3lan2VeHDUq+3TN+jr/65frp8Z+
O1y30KhstS/fOm+PnL8/mn2f2gurjs76Hv/metdMaMnWv+7Nq09a94wgv457
p999KzywTPbwgz/cN1fu7211xw4brHnnkOy8cz5a5Y+c36du3+fUp18dkv04
vfMV69zcyG7uUtv6nm8n1trPtt7dh0yO9/Mc8dGAGZ+ukcu3HyZ9Om/vUbVL
H54xT0suP87ZssegC+YeUjt6yQGHPNV6v1aXbLvt5xoS5O09y9942+Q9Fwj4
jOf+77kjj5o32/HrOffumsuTVyfN9fW4lTarnT+hZZEv899Pf+iGDwdeu1Ht
rq1m/WCVvL3rrtueuNIquT75ZtlJR+X4kOkz5p5+97q1RZ/+ftBlOzayCa9c
MWKmRU+v3bJHbcNjcv/723kX7LzcBgOyxjxdj+vaep5i4pPZzHMOyDbscWOv
A46L/sD1/+q14n35fmx47Y1b1T88Pntsvu6X9pw3n/+/111uy6svyL46+cdH
lshxfetzp+2214VZj7WWGPzGvjHfSv++9Ow5tw647eLsgB4v7DR2fMxHjpp+
630n79rIrp69x8Sdnh6VLfHC8Nsf2CXXr1vNvfpDg0Zljf6TPn5ryVy/frzi
Rnd+OyE7Y/TSX+1/dy4PDtx+/zsnnZ7tcf6JS46ZI9Yrvd3p5sf/erglW6Jd
93s337V3baazR5/2So6Xeqvx5Ij7+tROfC9rjM39xVVuzP5cYLZJ2erdjlj2
lsfr2ay7f3np3VueUHt/98lzzvVEbo9v8drU//11fO355c9/Y95c3t1wy/jj
el+6dzbg5J8P6ZTLqwcXX2vcZ9f3yWbe/r9dDnw15qfOOWPvm+q5/fDh33Pe
vtcjF9RaJq/13KfP1rMv6udd8uTSF9bOO2rvHh9eF+XPi2s/PGPNXD8v9NXH
Peee3qf22rn3XPdJjvvMcuPjU2/uXbtz5oM+7Z7Lz1v3WPLBd7bvUzvm6I5b
7pLjcSsdtP5Hb/eudeg+Zlqrvz5Pn12nnL7XXrU3tvvP6Uvl49tkg9Gdjno2
nn/p9sKGk78+98raTkss0KFf5yg/5Gseue2DA/915eG1rh+dc0u/lRvZy5/e
9OFzPU7Isl4brvDCKrEekf5f6tDbdrxhzLHZdocv+sSSf+b2/oHrfNF+0rHZ
HZd+ceLnOf8sdsBlMx5YM97Ps02/V7Zb9LD8fS+MeHnLTtFeQ++b9Jz7lHM3
HJB17vb3O4M7xfra5c9aenwt11/9Xl1zlse6HFub/e8l/pyW41VumXW3Ny8e
XGuZcvzmc+XzP/XXDdv3PPvoWufFx+45e2v9/L5b3PVXy9G1LRYfc9AXc8b5
vj33/Vstfk9uf397xdk/dVqrpj7p28tfGX7toNNrl66y47onfhPPf/XtPq3j
LD/E+rraDqfv9/1M0R5hL5y12gdrDL8qft+u84Bxyz4+5Jjadn98vs+v38f6
gMmbrrXpWrk86fxrv8/XXHdw7Yz6Ssve93O0h8R7tnj/nQ2Oe3NQbaH56ovu
82uUt9rPe6HH6GM3OaR28OTHt5mwYDwvd9rrPx268fyNrON/Zr5x6t5HZ11f
+Xf9iBxv0W7C+OX3PiY7c/alfl4l1zddDrtrs307Ds5WePeAT1+9O97/8czH
l70xU75+fQ+Y+Ohn3TfNTvvpr/37XRf12xXtvnj39nfq2f8j68rjqfq6d4YM
IWOGiExlSMbMbHNmiczzPCUyz0OZi8wSMpWSIVGJtDWQIaWkkqJISriXEEp+
+77f373n+3nfP5/POffcc/Y5e+1nrb3Ws168HwzjfZcEyXd/aK9H178j+EBu
+agmFFb+atmBfh9L+UFuPEQemttVW1fTYHyV+L3dv7JvR9pBO8AnIcZ5dicW
3xxRsUkp/ld/vZho/m8aq1g8u/TK08GGNmy/LTEyKXQJ8Y1nZ6juUkuGw2vu
6Vwr8xh/+GPwu7mJBuN7xP9Pd72xj7bDFu6uNs14CJB9ed6dbXQygbTfbcor
/Nf5ZzycvvOHw0ABT8oXlP6cxmbxEXuffpxexwIZsPzRNwtP1M+cXgRfVNeT
gt0LwfOeIqdL24tgxjV0l6x5Hmm/PMUut+OKSgFIuLz0KbBjEeThWoJvruSD
/If5Z3mYED9jUGjUFboAuS+Sq7kvYvkaxHiZB/Btb2q6SKovy5E4dNT4bzH8
MRZHNVqP1Zu198pYCM5g9WWWtFQrGeewfBRi/c/TNqnZ9/a5JLxuOMNdSh4M
e3koy8tZMX+KyF/y3VOPFXVj9UP9kif3eGglA47czNTvGtj+LzEf9mJH8hRl
UCEIO/iQs9Ifqy87e0RovfQMVk9GvJ5iwndr9owSwJw1njfBjwfHFjw564SL
oODjYzGRsZh/rSiYGaJsh+UnEff/e+cEU8QfZsIWN4dz1z4tgqX8yOhXRQkw
XDmWqXgGjbfH8Yt5lgmk/ADmX9eP5utFk/J/QCrHo+diQTAbPU3LVSw/ekV4
7+JLIYy/EOM/vzMGyW4VlZLitfOUmw89oB/Jn6jo2bkji8ubdH2NTyr6rjTJ
0IXDXbKvEstHOGXbeCdWFflzdP4KyjkpIPvF+6qKXVh+Rxidh9L4NyxflJgv
p0g5/jGe1h4S83Mj2ja4lHQcIFt97fw75O/d+m35I/atK9AfJD/6phP7vax7
AO+xA3jgTZuWNL0eDPSG7z2WQnyp9M3zH4+/5oLYvLM2RxCOTX+nuobPBWrS
YXeOo/X+PMPpUw3D+SA4t0R5Cx0/1PO1ZEdcPtixdO+aAg32vRD1FXZr9ku8
f51J4v9G+QrLLswZoD46P389E0eqh76c3/FnlAoP3pu39FnRB0FFA+X8AUJ/
jqO8U3E7g0j543ytzUPyG66gN0R8V+kIZh/49+XnmrNj65Wnq+i8NeLL82Wm
IqbN2rDkSlxr7Vsc6PN93rHcdgb65z5ReIKw5fCeX1UXzsB3d7IPz4yh8XkR
wnUmIRZ2pe24aIKwnm1ExohrIqRuOWJk8B4HbK69/5auEk2qF+cLZLVLbPcg
5fMdtq7ctLQ5BYouGR6NeIrlQwX2u3N9QfygpZVH76tzAEgQHb+RNYADuftT
u5RXfEEqxcDAE8THXbXk3qguxsKvPffN59F86z7H7qX0IB20Htvb/wPhjEKd
tm8wHbi52MX8sMCBnxubzcA8Eu4m13542AoHdt4ZjPwhHEmqL1DvmR3btoyE
lT851t4g/+pW8vB6MgwGfW7yMxcQTvh0UC6Z/DTwLn5u1MSH2TNiPoiD8cge
oUOhsDe87mfzPkzvi3950HQF+TtU4/ixbBs/sOHvV9yIvg8K1zNl2gNZoLiC
13cc8c8qxXJtiQfnSPbjzJdSFUdwDKidGOpYZsPyGYWNYt658GP6HaPUNNIs
4li+HzHe9tpyWfd6VyGQ3/Gz/BIav2sCrcNnKIOgqJyo0+dBHDjQ5ZDBKB5I
iueeDtY3kDmL5Yt+WR/J+2Z1DtBYf3jL+R7zx+k/JKvGf8fs+4dCjkSG71h+
JjFfqPGPctlv2RhSPtKlTpVCk6FY+NOQ4Xi2Gh7kxhuq6p1IgIzeSU9tlfCg
R4maQ3MbrSfmV/oyKREfcRrsNe0whx3SdfuK0Pd6oMXpXYpcDOj8zHSJagyL
jw7mODWGofHV3DimAjZPgWzPz7xJCM9HVGhnhAQBuQiOEwcQf1DN1zl78rQ5
XLEQ9WNC4yvytlJi7+1McKr3lNt7fqxe4lf/p4mnuTgguptN2vpkOsk/Y6No
NuGYiYRZa37ZBoi/tbNOOnCG6EGLuquKRej6biv7528+0YH9v+m3+9fQ93X9
B9v591rQwtTKaA7hYTKd4pYhXcDhn1blgfDt4sfKerOaQIxxu18X/X5k7att
HYUOqHF3Ke9AfPBVXNdiw1Is4BijDCxAfKrnA7lZeF4s6KtVtRJC/CTS0y7w
Q2sK4I7No89G30eRsXxQk0oaSLZeNmlD+LnI95PHh1IBc0DPmsQLHMgPi2CV
Pu4Lbz77nvPhE8aX+I/aSvnjMPzoEzfP2Bcs/nrmWclHgr6KhO/7nuv1mZB+
Q8veAV1vv0KHfjtzJsl+84rlVvs2xcLjg2k2CeRY/vUIZ6baKXKs/9hEc/rA
xx14gP+ZtlxnEwBj9TtOivzFAVu/uGyX8pPg2leeFjYyNB/51SQNA04C5/WD
HGfRccnPfMMhMgHwSYK67hri0wNCrpVvNd3hlvRM3BTii/cibC3zcG7wRv5B
bbIlxL+0Ug6ZMfnBvp15cZ/R/4ngQ0oMA9yh9UMhXC26fibrBq3ULndIEXSB
wbMOB06FrchS+thA3HzZQg9az8RH/Wr9q61I+yGWg7b9wsNW4NhvqVXxOSze
QrRv7V/0OZsEzUj5irTvF7TDx/XBh/F9JvdnMX/mCe17T0k8Fl8inr8q/vl8
WP0xuNek39+TBos3EflY8em7zjuyjOH0N4V2Vxos3kTSl8wrDRuGxmDLvn28
F60HVgW3qrhu2cAgsXiWFsJ+SVMA+Y5pK7g7kbkhnZDP8Vfj5cd0G6jQzbl0
HuGG55aJk7LW8IXn9xk6xA9zcafqaCnNQTHvh/XpHzjgHPa6i8zUHFgIWDW9
ReM74ZDyOH/dGlCnvWobR7j+8GDN5qA1iGld7LmBnjcufii7N+0o+HVDakam
A/n/Lu4P6t5FA+Huq47VaH6mQvexirAAsGwuZGaK/s+IPy/b/qANfH2FbYsC
+QOrj+9rLrvYwotCn+al0fz4vV+/VEjRBlqaRNQfQnjPfaOWxUxr+PibaPXZ
NWw8SfqgTw+5U/w0hbJhWep5PVi86evO+v7BQzhwc8eZqVreizDT4ux3am+0
PsiEnd5/vQiaT38Tr0f+rkThxRGjsHJI9atwOKAJB2IUra1Ce3NhZen5l2FR
eMDkuauaRTEPskoP2LmkIP/L7eV+o8xc2E/LGs4ehwe+jGVV4UGhUNxl9bU9
wncnhC822IdCGbUTPKLReBBfcIhnsLqIxP9mcDqusCoPGnh985wPxOoNiHr+
/n6fkimpEkn+oOK6uMXXPG3IvZvBd3wnlq+QftxJ9ynlf+sP4oEeT5n+2l5t
0D2rJfaJEstXKZvv+La+E8NbC6bhLej4Ownjz12BWlDkhvjqV0I+xqCtNbJY
8OfmwwZNQv6YWyjf8TQdeIsVmXM0/lFa1QbLJ/zA3h9TtzLQ8Q9yfUI/5n2B
1MW0e5YI7zuzfca20Rc6cp8qKET40KctGSpFXxh1zeL3KzT/ND0mKJ3ik0j7
IZmDDpe6RE8DxT8gipAfQ8w/ZHj68NzWMg4o3Oc96Z4dDDa57foety8C4SJl
wVe7bGDI+b49AvcWQekv9twOSRt4vrEIZ0mNB66lSe2xGceA+MVM8BLNh6sH
UxnFLtqASTt7S4jw/flPTQo7rQGDvGPe6CNsPSXyFbLS39RJ7uehgPVF2i70
vb8+SBtD+cwN3PGcY/RCmDxEO/aDthOwCmIMeyKK1tMBeqXrV8KAZoVGuwc1
pt9BzE9+5+Q6/iP0FJQ0i6LTo8bqx4jHX6ZGG7+UPgVGd+oP5yL/6ejY6plq
9lAo+vWCNhnyH9u+hjEGywTDxlsR7I63se+/87u9gBDiw3WVepR4J13oUM0/
g6fFA4b9Aao953Rg1tTKNguyX/ys+QzWL4MgzQ+7WS80nlfrOuozagJgALVS
1wlkzz2GPgvO5J2Gy+sBrK5ti8DFX/oqs6kjDNDLtVxD/vDLIvK1qGpnUv5D
72x24xytFQg6WRbHwo75q8T9gxaDB7lmkSowuaJ9dpsJ+z43t5nSVd79bz9c
x+imqnsTyZBDLaHt2ntsfyDBcG8+YX+XmO+0UvNE2Wgnhv8IDX0/vYTx3SDP
8vyMJez3ERMn+t9SYvaVmI/mW+WTGP/QmBTf1RiXatEZKCfVG1YxUxpP61fA
Xr6PDSX9/9tfF9d0W6dx0I9UzxAeKrrHP7kQnonffOYcgANeF3LSs2sz4EsN
XZ+HfjgwOR3gPPQsA0aB88t3ruFAesgf90WmM8BH8JjBawaMPxZZOh3hRPY0
fVpsfCgjEqym423cunDAO/yEZrR7JOgpNzwTLoX4cyM/zjg2GlbcvM9nIoL8
u490J+6KxkCr3J+Pm4eR/fRYeEjn5Q6W9qQ3EOoZif56YDO+MpsV01si8s8r
fateoDedpBfsklDxSUcvAv5WGLrEYouNpyd/xHNdQTwQuh8ff/NIBEzBbRnB
x9h8JdbnPKU4JLtmdBrU0WUwxnzG+D6xnjXsslH2seEAUjw/dHNfRlJYHsy5
FRYtLY0HFVYVeuxn42ArT1jwLik8oJPL6X4kGwf9hSS6d0E8SNx9wOjb9Uuk
+mkp7h4y1+dxMN3H7kbb70WACzH0kTfB9IX8f+6KmWL+qU7Erk8v8vJYsJL8
0zW2l9TlNpyk85NKB1kOF1CQ8tXKtKJ9uxLESLir8a1PO6cgCTdI+w17uXOQ
cAejYcU9MTIS7k0JfkA+va5OxAFq0iwbVHtI8cAGeveOkjl+Er4e3fijzVwU
SK/uFTpJ0AcSHFC87SYO13rD00oI/cY/3pzV2DgIs2rCBHoQrjeCUq8rheD+
xNCuJYSPdQxFeevzw9sOITa86Hqr5u0pq5PckPF98G9/dFyQnpE1+5g4rDlk
fpCKkF/1t3n3zeD9gNak2c0ZYTW1ve7HBgTBqZMMFe0IS8WevYsXOEiK7ymO
NQdRd54Eyye8Rt6xYPkwRxksRegPYfViczn8xQL0WD65a3mfqxCyd49Srq+1
S8SS+A5rlkghl38MXGgMm85GfKnne1FoZWosiV/dXBa61sIWC6KmghVFET95
g7tjc6wzGtxafPXSDdk/8jPnjih+sALbLMLsffvxgD75Z7GISxhMZeQDB+aw
/GrDd54Ox77iQKG4MgcYsIa/thQ5hRB+KleYpCVnA08B/PuviJ8ULt0W3HXL
GVh1309VWkD+sbyFkKe9I9g+UMl5/NQikFsh06EKSyDVB18qqniYJVkA1x4L
jpgex4ON14z2pWwFUE3hkMgosmc4B4uo0HprGHR9Y2DgHca3ifZv8H5P01KB
A0z+HKM3gp6fMf73xccJQeBEyeGU63vxYEFE2Cq10JlULzOSIq6UNe4AHPy6
nXh3Yvmly3xazO+RfdvdLeQmz+IGVHpfxpSsYc+PP/PrmuK/+lOPCfS+KSPU
F3et959rdoMD8kN/Nn4R+OnlQwZs7jBM0GvtDyWWj16YqUy9RI7Fj42reIsO
oPVogOlWVlsJgDLvJ3ZrI7zaSGfYddIADgTFWxcge6FxLjA/yVEH3jPd4t6a
wYGahvGWw/HacHmRO7uQoEcTrRbeKXscun+8d8SMgGXDBpSLj8GqBgclKYQN
XV1Y8dw68A+V/Nn3aHxc1dOdTefMwKrpj3wOZO/ZOs5XXBzVAkIURXdqlv63
P3W2c1bz2xx/kv9CZrMUd7zmNLR0itopS9An4v06muQVAfU2f0wlINx9z+YM
x4dwmFGuA++j5/cUuTSZp6gFppMZ+0wQlmTjjjxiogNqh9kMFND9zd39upNu
Uxv8XDHRfnsLBxaL3HIpLWzgg9lWG4FWHHhp28UZOWwNpY8pzf5E69k7czKP
ndZOwGGx324RYS1eu3jcAWcQce0ZV9ISFu8NpCsS8ZzE/G2LsXsrn9Bx8Yzu
UjYpcxhpWCP9A63nO9iS2F/ZOsIjIn5izOj9MgcUWUsLuEBwVv7Jd3R9NgYj
xxZ1J3hE79voFML00/tfFGg5Q85jg20EPt3tUnWVTE4AjNwe+fOb0N9q8Ne7
WZwkuBR3TeQXQd9R38qmO/4AzPXOK3RC/nDR51bOpzUCsEzcVloobhGkj0/G
tiL+ekFlD7kEwnjD3NJChMvFF8bnruNBmhwL19P4bFiZ8Dm5jB756wZXtQ+6
xUGm9dxUtXUsnk7k829Kkp+WZiqS8Cpl3ekGJw2SfeaNUw308JMi2e/7H1V7
nnXKgbZERpnODSz/PI5rkWFpE8Nn/EydF35jeJfUkwjCfgORDzPy/ZZ6sI5h
Uv50Uomnm4MaCUdVJ29VFxwh6cWJH3E3ghUeJH2OEOm0Ut1vYXDr6Ds6CRk0
/u9udJadSADxeymjDCjw/9PvYOisFsMdizjAY1V4kvcnVk+0Nqj1slkMyx8k
9gv+Wz0hGWwcAxsazArlD2LxUHWLwgH2W1i9zLLbbbuvdJh9iEoey4+owgFl
TdmrN8lSQFy9hd6by2i85QeKXP1SSPE6y3lWEwr945BsVgnP3ov5s8T8l85L
W9zDlBbwSZq95LMP2HHi+/CTfUE5eSaG9HzXnDl+RcRGwQ/7TYN/rGD1gkS+
2MSvRf7XxQ2wKZeyX2bC7tfn8FDdA3ps/0ucfmu3GPoem0/4npYTcoGJA4ba
Suh91bF0djx+4Qip6BmEKdH8/CT01+skrxPMrCv9rYH4FeWtnJ1OtZ7QTxMI
j6LrvcoUtiqv8QXMZ9TIixCeNHt7hnbRE0y0KUVdRnjWmKtRwssL0PSoMtBP
/W8/8C6+5MGdFvpwiy0K8uzC8iXarw01pSDsFyqndS7diuTvP3OlP7PMxwLY
jg7b+qD5ulu4TkmuUQo01Qdb8SK+XuBX+ejyChY/3boSkpvvy07KR6T6K9Hb
JaQGVZzvDo9/x+aLaF9efhq63v70iIk37EIwz/HOX0NkHyTIf37aunoA1qbZ
XOBbwfK3Rx+mPhpdxfCY5JX7a//qDy4pW7GptBPLx5bZVeQqTIVhIt+191MT
S/WOhYdkrG1afmL542xvXWI6VjHseULByBvh3rc/HqbtBqT9wIKP5/aLfNWB
yQIDA3dWsPoD8Pn65Bg6H7d2o1uzVAPSnWFmCl3Cnpdoz6MEOkGepjKsn/Yd
uJyyCCqffj/4oDgB7mB6riZ8bhF8W3VJ7BxKgLf5B1ZPLGN6bOI6Qvf8xf+3
X/jHyM1X+15fAB3k190CHbH+FoIJIlWTzniwU7qkRTsxHzK9fM+UjPiyRry0
WssrP3DznH/I5Ekc2Ezs/D539DzQM1WLuYi+vwyNbGW3OSEYbCxteAvNdw+z
dv1u+/1w9M0JC+HfWH05sV7Aju3V0HIh1p9c+E/LndxH3kB6wDWmAK0fe1m3
7xaFJQD/d4ERtA/wYM9LDbEPzhXAU9JmoAS9H67qPUar7u5AV843fgWtFxds
wkyfixyDMkOeU2fR8UcTSyffxbhDFuFA7rsId92Tt3cydoOU8rr8TvTY+v6c
To3BFfGfhufCTOeX3UDH5wtrWcjeNHAFN2iFqcJ4s8sRDxDeuG7QVEyvSrLP
/T+PPsuitINDMapJxymx/UWivzVcrXtwbdweCpaMXL9Lg9Wj2y8LONoG4Uj1
gMT8CL67odZn7+ZD30NfynY5YnqsTG8UivkRH+sqAtGtSn7gZU+cxgl0v1zL
G0sGu7yA2cJV0+mdmD69Aw5uEfa76XFjLJKN/rA6sfEw/W7C9UcF1dcwvXnr
COOEzFlBEvYObJDMOiYE1BM3+hnQfG6bohEy9ReFXeFneN4Q8rOKP768RSkK
Py/AvAw0njuuPktREJWFdQkVok2ID+hc+vjzrst+0vMXBf4sLn3tTuL3rH42
6RFmXqAnpm+NkA9D1L/ymb/JnYPsGYv/nZAd6Z6k9c1I106mssUL7Blx2UjY
wPqJEce/q9Z00vO6D1wvjTlbQIj33BPb1+DpBu8e8bx5ghwbDwZpuqxW9L5H
GQsinugGATHu0xPtCKfRPLtYJxYEDn3cKzC4Aw/GQ11eH4vkBSXeRvqE+KxI
p5Pm5XYhsFX/hmkT4Ser/d/PXdsPJPV2tg0h/NrD9qXZH0Hg+lnWugrh6ZFv
7SzHDoK6ovey19D9FF7aP3r/hSDIC/nzfgLhZVpqUdMQWeAWv7F1Ac0P4ZjF
kol2bhBrolFPgca7O6/nsOWpQ4A2JnLpFhrPTxwUZIuU+0FW+jG7QjTeD2Iv
cgm0y4Ah0Y88s/+qN2ykTNxjRYPV/9SR33xotojVh2XTV9LvQvOLaX3/RYuA
Q7D6y++7YgjPbEoyHvMTh8b6L4pGkL8j3T3vNiEsAvnn9D4T6vMS2j0mtV4J
wbWsB+ICCA/d/GBfF88PByTkw/vRfFY7fv0mhxUPtHWMrgz5gfF7g0gdb3sa
rN6X+g2V7Ec03i1W6/hOXABab52Efixh8UqifZOUiWnP3xEMwNzIUQ70/TJ6
D53bmRwGr5OZ8qogfLn8SKAQaxg0rFhbk2fAAdUrE/GcYqUwu+85boAX6wdi
C7IkkmoWQfI9P6lw1Tz4boLzIUMEln9K7O8QSvfYl7I+Hxw8xolTEcUD5xcq
GhLOZUDd8snk9XVMT1fEyCGIkH/qaaUwntgaDCh532UYIPytv+OLnvJpEDvD
SC+E1vPhjri576y5QPP5g6UohH0HeSbV7uQBZbnP0Q/Q/RW+Tafl3Y6BwR17
lDNpMD3DV6E8EezsWD1/x5up+Ik96P6MqmRpKcJAUNxXeSp0/MI2lV44DANS
+4p8jRCOeMgtPV0eAd5a4fbzE36Pq45eywkHH3s5nk0yIvsxFLyQ5aJNik+d
7/yrB77qgTdzh/LJprH5NBR8xFgLfU+BnhTZrfPBcNBoouo//SN34Y92/QiG
j0KVDOL7MPtdyPBlWY0ND5akM/r3/k0DoowFT2kP4IHEzQO72tciSfX+lctv
ImuPh8GFZBXBBl5MfytKNfiFlDGOVF9E3M+VEVNSWg4pgxdYjujp92P6uHQm
ORoi6PuvH+vP539ZQspPdaNMat2WLCHtB94KKZUKYs+Hzk4NYUyIjxPjy8R8
asohG8uk/kKY8ijGkTMI0098VHvU2eo8pndDjIeJaJf98Ky/AH208auy1zA9
r/ORQb4Sw5h/kvg4KdGNA7PvBSJ2BZZymB4GUV9ox9I+vrqdjx9MCe0pNryF
B89qGnON0PMS98MFA68HsboWwFkjE3Hxi2h9eWayIlSSD3fX8Q7cF8CDvNO3
i4o4iyAu4NIEC1qPjrVO2g7cKCTlq7gp3R67TVsIDTayF1+h/1t+KVblrFYI
29hqT+xRwgH3+CI7rzfnoRLVamGRBg58X900Lz92HlLWJKYT8rfUL+6zPy0e
C9/sOMtjiPCtegpxIBtLqp9zJBeZ/qo6q07ErcVSwsMnqEjrt3PxoL5sICsJ
jxpQOUtM7SH9vox/YWtUjY6EeZwvxVQHrj4g9h/c1x91tKm4HEjPzM29bsf6
aRPPP/d35J7rjD8cPfjaybgI03Mg6oFUXX5eWbN5BvaHTR+zysT0W6hd5PbS
oPnK9Es80kcM6zfopvLzd2NnKNi6+JWjfBc2/4j5Cj/3T51N6D0NM+9YvtlK
Rf63hPKZC2anYZAQ42v5LBwYfCZ1d896MDwTEfK9NmoR9Nbi+0+XlpPq7+WS
6YuMLcqh/oAT80byIvCIeWB7KLAcLn3ZmGh+j/UvIOkJeN0hg392QZkH11O8
fmD6leLTW+Q8aL03TjTIHqsIgQ5HV/fwI3zhiFtRak0IfKexN/HwVxz47Tyj
5/XbCUQumrabIHyS4tGRtlInkl7CXLZ+9sOeBOiz10KBNoawPxXSnPs2CyzP
mUqktWLxA+J+douc/Mfjr33hzv6TCg18eDBSduve00Bv8E00Yk5KCA9uX5xJ
/3v9FKBN7uZ6IIjF9xNX1y2M0HwVMcnvK8kPgixLZEfY7v/L/6FT8777DvP3
ic/ftW7WmMjiADRXNRlqKbD8ka/t0dRRq5jeuWm4OF/MDKEemO/Fq2MR4H76
d3H5r1g9DFGft5ujiCeMNpLU3970pQGeZawcHg74VbhVi/HveLWE0vlxbLzt
rnZx8CH/7gBfMg4PTkO5x283QtH7qpnmS8yZi4Rat2OjzyFc589Q1FcaCf2L
RpvfITzKWKbf9SccNlygCH6OsPSSXvDw3gjY4GRYEYb4s6L4bPjZWgBvx5C5
cCB7HGxqysecpQLrnWNGfFew9Vqtxmvf7RVML7Hqw8bfmQdofl5TyvkoEAEy
c5K9ZhHfvTel963nhhdk3ONhXo/8ma8attmNzhHwhv/88v29ePB1KOTg+q4g
+LzofN0KwmTWDfGjGkHwNi9b+KnnOLBA1XCFNdwTaHNYeEW8wPTjiHqMCW1m
EgUnPIB7577o04M4YG//yP15vw+o/rJWRL+CxROI/g2u1mS+1loaJlr29oR4
48Cawi+Zi4nh8PfBy+7UHjjAemf15fRMOPTZoBLr/Yjp4RHzU8urZ0IFeQrB
6EzKh7wzi+BHGq6Ke60A/KDjo/R9tgjmV6VdlkrySfpeXiJO/QHeBaR6oEuv
mTVXXuWDwrsC2r8HsP5jnHFfNt2k8aDVteF7gmAF2Dw7NMUkhdmXV4eyr3y7
jgcnDRbejMfaQsq8974Fclh9HNF+UzMYsXjoYXocBp5qzNsuCSR9j+QQVTU/
dU1SPMDkaOL9lKvS8EW8v6Pfb6y+mX9k3d9gE8MbbNF9lYhvnN/vY+KsLAv5
BRNiCPXJTVEszxZ2yIBgco0D/ArIv67ZJfB1WgpkzUtF6+Yhe6ocq+fEngIv
Gcvce06P5XcQ9ZDXB9WuHpYLh/5SzGtvkD+f3uU0+lU7FAhycvoNx+NA1sT7
97a6IbCQz56PHtm/Jb9MNmvOCOBYEvnoIMI5Hrs7s3+EA5WiR5ejRjF7ylPb
sV70Bq1XqSYKdpwpMDXCNZ4cYa3Arh1F11Kg8dKSVRf6f8ENr4up79yg/fVz
jG0I+9c+1zW45gbp973IHG1B3zffmkuFdADMqjn0fmAIB+Y1dc5v0miQ+MDH
vWYzTPskYXIVsKF+gfnDOhFSmbAfi08Q91ekFkp5GzK0YNq0+B/2t9j6ICZ+
/qkw4udbunptauEhJP2pO/bCA5sB4fBjVqJpxQSW79BIKXoiuwkHxv2eZIlb
JQHl2YCKyUbkH//0qjpyLgl4/XA6sq8Ey9cl6lONawk7UJsHw8zU9v4aZP+m
/w7Fnr4fDWqMkkXpEZ+vXxdt0feLJsWbyl5MBLIWnoZcVnJxr3sx+2Nf9Vpz
7R4O0Mu43A/vjYYuOqlN2a9x4OjTG5kh44Iw9TfvjSU03tqX4LTETQ4odJpG
I6sdB1QElWcjdscCBu3MR+vdmL4PM1/cr1FuTG+NqG+0kf9sUup4EMyEfkzP
kH2tahyP2DCUBa5/fVoE17B4IFF/M6eh9/3BSRVA7378GkCYUyC17FXYEYge
3YaQ72DAeFCrqjYYOKtsR/VyIv/G5dzr90YBpP4sYblLfGeE/SFr749pYS5M
r524//9DeGZtofwS3Dd2pQNuY3yeuU9vRj0JfR/zlftq5AshhUoWlwSyp9P8
YnfU7YtJ+V1+YWFl9eZZkD6ALkn6MR4kdPJWG0llgyJTWxc5FjzQM3WqCHnn
CTgfqlrbIGwxHjG9Zu8FElsDT2e1LwIn8Rfsv2zKYKTAqyWKu5jeavu4aWik
G/6/9Kvx4EcgkL4dFk2KB7gprRfY6vkDtTtz/YkBeGDg9WOQ5UoEMJ39Eu6O
/m+gyux2dPY5QNTPutYV0bvrcxlU/mHNfB+9jz9r9Z+na0pA3imrKyI86Pp6
9MP7BkoQvyfnHctaBE3N8PJu7XL4kTdnMPwxDqQd2+Jt4QiDh5653ntPg/8f
fd0jgRYLqu7pcEVRYmzmJVFfNR0uaU6uq1zB4mFxDpkS23M4cHZbtdkDrXdP
09/aRqH32VUeoSGi4g6nh/7KnMZhehgi/vrcxci/yGGgXfu90waOORuvzNNg
+hjEeFlzsMj187Zy8FMA5z5dZH8oZPhSBV77AI/1c22hiA/9I/dXDju0NZ8F
Cv9Xv3rEL4RqKus3LweBWy8ueUoQ9GI/KHM8rAgh6TW9E0vVeOjgBmZaK8vy
uLH4acxKlwcZFRZf+NMh4W7WhsX7klg/sT5DfGduvuXy5zxh5F942skvYv4N
F/VkSsgvbD0kvt+73pTUOgkeIFJRtenyKqb/p+htctB59b/6haPzjcNb3cX/
BsGfP2WaniJ7T9Q/nJVhKOhE/tMjHaO+j3lBQHBMO9FlD7ZfmCau682LcEHQ
q885A8EgDyf49AU9xq9I/XqH0/hw84HA5Ob1qhRCf6QsZ5EultPg+D1j0WrE
Z/LMrcycXiD7bvp0pRXh9hV8gnBACFSuN9mUR/jXE/xyll4YZBuhPCKHcMn5
hkUD9zA4s929PIn+//Xjz1ppnzWhe9UOL2/0f6lrR81uh0rDKZ0fexKQPZqa
Tb8XoOsG5VrNJQMWkD8uE3JhdjEUXkjsiifU3xH1L4n69FrSeDeBc/Fwuraf
/FDQIlBsFQoTCy0ENlzlGcss6P4SKD/7PM0FfA9GyxSZkD37OiFrMJ4LIhIY
5VRxi2CPQGeRRHcekDjRWCVRhvU7I86nvDqBa7QnC0jx4z0nb8Nmmkuw/2/7
/XVWTI+SWO9Rnbr98OjlDBBRdD9cSQfLDyDq37VyVPJ/a0sF31QEEk1yMX3g
bYn9vB9U8GDqXd7HtfkC0vpvcGZS8+SuMqBAJ5RTyU/SFwRVzy9c8dZD9qjk
wZxhfSZwXFY8dY0Ly3cn+iMef9p/aTZHwlHLnXcs6TD9PqJe5tFEW2UHo2jo
IbUhojGD5X8Q9Xf9lv312F44A5GTSw6iiH9Wk59oPHAnGtZ+dvtdh+bfngNx
xefsvMHTtzK3Enox/cUpoyYmR7SeegjvnQy67g+oN2uneJG/f1nsxInoipPA
cXH71t0hLF6wQ9yGTO4Gpn/KYeBJo9mDAy+OOSfNOAQD8gYzQcl9eHDwFG2o
veVpkj43dV3fo29Lp6HHkzZKQn2ysVHD7ZRiP7BfKzf7LrKPUrWJyfL1WcA5
dK03hAWLbxH18g4+YDj/6pQRsDe4+MaFDcuHydsV4Hy+Do3P8ab6zyPppP0p
+9mCeYeL8aR4QUh6mQhfmyN43hmrkUyH2cvTD8ZaeND3bfh0JCFW0gXW+F+N
74F48GvVOjkmswLwdvmGtjTgAFlhQ1Tg2URSPrTESp5XUZUuSR808VAhnUtN
EJzk7c64h/xTtVjdu/rhaWCryXsXYMOT+vWtXHsirMWOJ+kd2kUmhpSzY/mj
uy4e3pXO/t96vTgQw1i1J9nnAtRZaBl1/oLpv7AwUIhtVOHA6+Va9rXPZ0H9
w6rIklBkr5dW/ogWngPHWlLYMhCfmUk4rviw8iSkuSn3nr4DvT9WHQmxKS/I
NjY/9LQdi2cQ9Rm/4NRkJ194w5+O3OrOd7B42FB2tlg0M9Yv6lV4ZuJuUYI/
c5z164Vo2NyRMlUmht53+9JLVb0YKOvCGWqUj+2fE+MrEraZd1O++8HmIpCU
cg2Ln4vmXnMtUMb0Yqw5gVd3HoaJv5czmr6/v+ssALXHHQSZsXrDns3McxQ1
WP0JUW/mplfu3odHLoGyZ8YqprcXgZLy8L22jVx4wIzhU8qdRWBbysRyeyWX
VJ8hddFVRbI2D3qeUGX8gN4/Uf+SGE+6Jff07bXYdPiUovZ6TwBW/zebH1nE
rY3sGYfR9RKNKBjilXdsTBf5PzY53/bsj4Jqbm09t9B6llG+j7aI8wD8eoOb
W/cvDjBwrOT0XdoP/aXjTPQR/6H3ypGl8BeCNW49amQIB331vv2qmx+O5zMc
7kDn1w4lBupI8EJGoffU/gib3jt7qFSNE2ps+QSzIEzv6sPeXsIMg9/8en8P
/V8qc8mOhtmdsKo5VccV4e8PH1x7obL0wPGOVNhJdP4R5vmsF1K8QGxWLO4D
Ou5mtyjFnS4EYBSF8FGEbRQdPPYNTql7i19Vr0Q4IFHQTomJHDB91LfZIMSD
liXtb5xkBO3ZP4KPE/SbZHibsj3ZwYnjI++uIrzXQ0qbtYMHKGwcUl1EWLdB
46/DLn6QLPZLRMgSD5oqK4wVyOJJ/Y18AgoCP75F73ePRTafCB40ZEW6ee0r
I9nvxptJxn0nrEA0d9b0LzU8UL5bOVPslEqy/zuoVnBjw1FAWmMkOG8O0+O9
GHChvuw6Fq/+pz8RDuw8b9Cqt2IDC0Y316l/oPlyrNzdNcQRDuQq8ukgPKHF
b6tj5ECaH29nnFqfDseQ9EkndHN6rjdHAd/ahxMtHdj/Eb8XnHF4Mov2OZC2
uEhvEYkj9XOu9S2S00jAkfKJ/tG3xgEuI8pkpsIsIHi3J90Y8ZnZJmWm4wVB
wG+vg+n3esT/nX+fmP0YCCkfSOWfQHx8lU7lVsH5IGj2grHRF/GBcuURi4/L
biT7dyZstHAvmyScXalsy9qF6Wl5XNeijUDnz9jz6isWH4PVf5ltWXbiwb44
Wcdb4makeiKeiPpfEJcIXyfjy8TQcYdANivxSjPwxbXcdGEvHjgZdld/x+lA
SvrfC7y8eGBTkVV5uFUXJmt2xb8g1DMoWP1+nhgHf6ulNVcjHJvvtc9JPx4+
a3kS4IDwk1wcnwRtIpyhqg7ORfhGRlQ2l2ACqT8k9SQDN4tvBIhYMMpTRfzK
LNI/3GPgGJz7VcRB8FfWAqyoVZdMIZf6jmZJhPMOvbss6WoKheWVFmTQehfd
+lviRmwQqBWdqBxBfPDNYr4Y3iUO8pQX/S1EfJCvNk4/6XccjJbaKmCuwIFg
pTe9dq1BMFezOwE3hekjMysqdLWyYf01iHreZA8s3xi2e5Lq62/nxYwZP8oD
++7a1+0+gAcrVn8bb41qwWO0OM55hK9EUP3g39Ak1aOUqgT8mjjrDFpMc6d7
kX1f4JyrDmoIB1KRzYkqEAc+2zn7rL0NB+UiabgahLlkRSc9ksJBb7NhQiYH
ntTPo94oN1eKFru/Pm9OI1YGDBP5o6/yvXudh92hW/9aOD0tVs9K9GcV2N9K
h1Olk/o9zcmccFsbTwWmFi/D2pD/S9S3FUxkLaO4icVz/I/yxc1fRb93byVT
nj4FKXYpJBH2O3ove/xWKfYF7Pd947Lv4sBQCvfc6jMfSB3Zf9sUzSeaS1kH
d6j7QD2VlklDwvms65O3j1nDc28v+WYg/ubFkODQOVYApH0KThH4snrp6O03
m4VAmlyai8Cnd1UM/mKrKQQLuMmYnGnM/2VwePCiC/2e2D9tu2NgjKCHo5DO
U3vDpZTkf5YP4rXN3XOhedTb9eZzOFI/YmI8uKQy+WPnvkKSXkDSTZ+xqYJc
uFkqppl5H9Mbnnnr4Hi2C0+K7xL90y1aRrL+GDnAKeIwckEB0y948pUhr9Qd
2be7p6lspfOBud/+TpY2LL/wn3pRPDhsPSPgFp4BiHoLc3lelTUrOeCN304v
7l5MT5Z4Po00RxrtqTJgwHKgsJEZO+7L7XcpDOEFVoNo3M0ckDb4gkzlGhaP
Js53vS/T/VVfD5DeP/Upvkd9OfmkfHu3jA+30g2VSVhziU25+e1hSPnu65jb
Tsy+EI+HltFcyNSSIOnN3Myx39xOVALTnJ9G0nZiepr0zOq/DqH5cCL8vfA+
qQSY0SV59dYf5D+mqd9xr/AGnM/atzjI8eCUvrqAL7s30A+pp+dYxwHZ20ea
xpR84a3zW0eETfAgQObtN+vH54H1qP0b6t1Y/GOywJ1ponMRWDXOc7RF2MNr
v6Z266LvTVr2fle3ljXQ74saVh7EgUz9b8nX57Mgw5dyYDCP+YNE/9YrQ9Ck
4VsUeHygqFtLAuvfQ8xXEgtTbUxvDAc6ruNW7ojPfeD5XG0okQRYKL+oGiPc
c8zYwbEvCdA6SLzgRRhvv5IgVZgMbtN1/DqC8EpgurL9HqzfgIu+g1pgVAJ8
v9Tpz9iE6YPPN5bP5xH6GcrZDXj7xYBd+gsuFQgbF3Xsc1+MBYfqL1NdQtjB
uV5Nvyoe/LrfLlSG8K+YgNGnXHHgj8p6dxDCDao8nR/XE4H8L++1MwhrsGrw
MhgkglH5FaZihNci6YY6X8YDI+0dblmE3x+K6Pc6kAgc5mx9Z9kw/30mR+dL
NZqfTE/WJNeeh4BIU4l95xE2XNp1fDU+BBzI1I0i7AdfPtW66c9xEiQP339M
8G/7mGalH7cFArEHSa7f9iP/WJ+mNT5AFx5OeLRxFuHjl77R+XrpkvYntnfQ
mB0bjgNdSzNLq7exfsNqaqJ7riL+YAAftMs4BUGXszu7DyJ+rIj3PN2RFAT5
e44b7h9B6+X0iQnRrTRY207lyaiN5iMHfX1tcAIkpxKp+Nq1CMz0OyNpv+QD
SjGr2IWmRWCRRpnNLFoAVPuM7NRbFoHsu9vbrbwF4IhZrBEDKw54nRenSWC6
AOkK6StjlxbB1+dvD+/6dAHG0nPgyLYXwbPeJ+t9Fy78o8c5vQhotCIVAhjy
wZ/sbm6rM1i8mdjf4oqqh9cyfTFpP6puUi8pQDSNpHcemPxhied+FlRULn7d
oIADmzJchi/XomBblXt4lRJa/+iUQvJeRcGxTkGDk7VYvKLS1uJP1hUcqR8J
0X+pWXtdI0J3Bs6nff61hPxbNb1BdibdFKCoE/3pHlqPTHpsFA+zuAEF+wrF
duSPjT5sFvZRKQBCakce5CN7QtW/LZQ/lgfEcx7bB6bgSP1QblVwPNnIwZH0
wYn91LO6HxWGbGeA9O2xJc+XmD/zVMupfxutZysF0X+9LmtDRksLK6W3yP8J
P79D0+os/HbgSyod8m+XLLiOf/8YA5U94v/SI74wu/u+eRbiJ1SWB73NkP23
+6efJKBQvf7srjEOpPBEt71rj4QHOt9QnED8xL+Omw5vE0LqD8ErKUDz6low
dHL0ToA8WL8KYjzg5OkTezw8E0j9nZQ1bp9z54tG6wxPKN1JrN6YyLdOLJik
HYozhLtv2hgGIPvWm/tpd+pbC+irFvjSEOF3KQ+aH1Ieh47keyEhf+Xlm5mo
jnsW8ODPg14B45h+gb23sGECOp/G0FOCb6clKBfTULdFuPYN7w+BKXMAnzoJ
n13FgeT5aVcNHwv427RvgoHQT3gvB6PYHwsoHifbV4+Ob1lwr+0+fhzyVlVH
d+xH4+XH34nWOzCUN9v9hhP5V83ieS0FxeAwo2U7OcJubdV6p5SLAefz9VhX
9PvlgxP1J2MNYNgq64c+hI8lsPfS6x6Fwk2GjE0rOLD/fEWsZYUOfF3HoWWG
jtsMrj6X49cD0W+CNpwRH6Ow2a6916wPtmoPjBP0aqMHJTn2KcSi+/A/mvgc
B86oZQgoGviR+mfs67D8xPLeA7oXRN9zXcHqNwU92yUI+oRE3D9R4ulPhfEX
Yj7hppTsVW1dZzgt1xJ7e/pf8cuFDG+bWWz/kbhf2HDV5OmCrRO4nnBAtgn5
J9rDPG2digEwZf8Q/a1v2P7cqHqOCCEfmRifSNpI41NcwTAxn+XO5PDb8Fkj
cHiFjN52AYs/3uuaury5jO1fEM83sUzeZ66tB58FeRy6u4jFZxfG2JufUGH1
PMT6KbD7k/bDCDM4Ery72oEWy1f0rApWOkmNYWJ+V/JzEZ9BNQBr59780KXG
9IfbcqHwAi2G4zMcI5ZosfomYvxpslErSv6CJeA0T595QYXFZ4j349VhLKPE
Ygb2Fo21h1Jj8RmNHS37jtJi+H1s5JVTaLyYBx7Mn9hjD2hOJv/2RDhG2pI6
XNQG4Fze2RGOe0fJih66YQ3Omkgq6RPyRwuSc4f3nABHV+5djaPD9CLjzwxS
yU/gwOVYIybvM/4w2nDkOw3yv2heu40krEWDkoxKT4ZVbLxtCctpDw4UuT4m
K10NAvelP9/fuIhH90WWYzBfDttuJqV0lBD6o9exWW6Xw6kKr265/kVQxHpN
6uzVXLiyyK6Z8WoRnBNxeOYWlwuHVRhNd5XiAQMc/3z26AWYKTMRmYp+zz95
fP6z8wXY5PD3NP8lxHf6pO24zL1geO2IRkUBHliovX5uFOYHl1s1upjR75+z
jd98QJcKrlq+Zf5UhN7vXm7GNKcUwHuEyW2IGfu+Sf1aFa+5dVcWkOq3q1QO
GjtWn4PivlTglg7Wf/aHXdjbyZ3Y+/V8U665819YrIJbiYoGq1cjvs9TFCNP
1C/KkzB3oZOp4UttKO0rwvnnX/rR1UE3PyZQYJglUrGG4l/1mcR8QPZk/iGR
aUDCCeNZRQevK5BwtznFG/ckKXjnyXEGjp1Yf77Jpb7vhPpKIhao3ihb/dfx
NZ6UAtZ/1Xsq6Eh9yKXAMPH6qZ29J7zsJUmYriCZcWlUnoSNWDU/OQ2rk/Cr
vZ8EzNl0wOmMo1y0NNj1iONB/nB6+tF5bRL+OuawcplPHviGOxnlo/vDSe10
pnhsCGua+4/JIex660up7nM9SK7BJNCMrs9yV1L/nIkB7M85pyCBjl9YvT5/
l0cDEvNBh3Z4n1QZj4VcTSw0b+pwwKlbe/fjuCR4OiAy9RGhPnY+CfeLOxio
haivzaD1Mu1+zQnfCzYgwzrdPBDd702OymSrZ0bAOn77vRuarwn7DuCjnx8H
vTTvvxP0UeWOzmg+bz8OHpZv6eARPxigyte75GUEFbhqhZWWMb5LsmdfGe+I
49zADbtntwh6pCNq1LeG0+3Bs0U/6U1+PDjAfc7xVLw/uHOWgXxoJ8YHla85
hBjdXgQMHbtuSvvYkdbbD0utIcyI3x04aDKwiP7//WKq/riMEYz0fSJahPju
l9Mbz6IK7WHcUFZX5gdsv5V4PxVvDzVO74qEn19shrag/1Mr7FLrQMcp+eQ1
Kndi9vI/8TxCfwPJV9MK0ISUL9SU7/Vhz1YQPEnFXhpfuAh2RdYI6V8sALFO
5nvTpfGkfoLBLg78PQS94fscU0v7I2F/nrTCrZ2YP9FIrzIdP4XpQ4mCdRqJ
ZximXCUPuvIKB75trxbT53uQ4gMxMj+38DSXSP3nqJtGtBfI4kn94b5co6CU
vFIGl2Wvqr+8uggOmx29k3SvDM6KfqOBVYtgu3w5TuV1GaneLm57idqOX5mE
maNe1kYMHyHh6uWbmoUJ0vBEFExdJsf8s6iNb+RfKDD8i4qn4/u/9Naf3KAY
9f7X+cTrGY0/pa5+IUXK7y0raWtu3X8EeHgzWBH6HxP9uebcUwfJ/oU9x95+
IKPEcGeIpCQHBYbJTn8PT9jEgbLPFWyiLfPqC7Er0cy/Ed+PkF4wYmUAY2nb
8TUId18qHzgQxwW4Au1/ryD/8PlNZVW2pEV1SgkDBw+EvdQVtzvMGQDuh3zX
CDpf89vNiJIkLvDikHhMF7p+gy4FK6vUQbA0aWW0dweyv8oPuAClD3jQsCjV
u40DzbXsOBdqHzBff2djBJ1fijOs5972g11kVy/9RvPnRN/OoOuqsbAWepwp
J+TfvnOvPchjDz6a3qgrnsf2Z9peDHV3NKPvO3eqk2ojETDetv5oex8HEo5+
VH805gGtWm6W7kF8tiVpOm6Q0hYKMLt9FcVh9aOOmvfU/mxg9WvE/cXFuoJ2
qmg3uPbq5Ag7uj+HSXtPEVlXeNHB6ngB8oelZR5k/3rhDrUW3hyTocDy3RXC
vVu/oPfhYg+epjv6w9IUHLcmOn5Xsb084Jk/yL0wtuNdG5Zv8TAybSxDEA+2
QbTdF08dUr+yIMPvfX/JdWGQ/J9oW0FsPenZt/8oOeJL9JurGc4VGjBuQeXu
HWQfxZ7w/qmksYCH+NTUg9F47QuKaTd+YAQ5Mgt22KDxdP1OK5T27TgcNgvq
MEOY+0lOfU7OcRjUzsVoTtAb9TQLZ3RXgwf3ptF/QtdLnz3F1iesD6+d3XJ8
TeA70edjG8ePwkvba1tSBPt0K4J98oAPwHlMNVc0YP2liP1KfZJdH1peTwLq
T9Kf/76BA30PR405eJMBl/ljuUcIdxXQjw4fTgYay7rXCPUh3ZqqCeSPAKk/
mQMFQ19srDooNWy43/wTB8qZpvB+9hZwlE+y8SfCzUPhHjsmzWEzIy1fH3qf
03zlPx9am8A+nh/RzQhvdrazXzjiANPfpDLGISwofcrnCYMTJOZD3jILi6md
OEd639Ls3W/8rKShH6jZjEV82vA/4y0Hpdyy6o3WMUw8n9uZ1kXEUAK0Hbyc
egvh+n+OA66DtQE0EThA6xyrkmsbCoUqbDbfMyH+8uP3qPmwPVyNSd/9hhEP
9ogk9adpOQF1k/03ZNHxHc0ww53aBdRPZXrqIn/dWYBlT5iPOzic9kyHAeGO
qis9Ap9cQf2uq77ktNh+/NNOlpXkMhxJ34fobxZnKmvQZacA3LUCBa8nOKC7
9xfzKQYbeEZl39tuhAOlcFFyZtbQovN9gBK6f9gyeao+NRrGme2s+Yj4XM0H
ufa9G1GkeCZe05o7g94XiDBy/V6lwfQFKa2mokoYMEyM12/p3lOM9fWEt7f4
RsY6sP17Ur9C7snaO7MWpHomfHT2xw5DJVJ97jcRsp/LZyWhMV/81Tkcls+1
sLtRdi/6/qZ8vBxvvlAEs2kXJ7ybsf5RHLybc6bo+xj9E0O+sC0HM4UTfgej
9XZ1h9Vui72KsC8nbEHqF8b/ifk3+xMW6bNpjsG/czGWSZQYvxe2dX69QI3V
YxH7gX+u7pn88MoCbuT/bRP7hdUnEa93DXdAQC5IG0yPvN10XMPyIYnHXbic
4igU9CB+nfbrigbil1v1eQYeF8ASRwQzVMSBHLPekvyrF0j59Cxxs1qisvlw
IfXvgT4WrH6NuH+72WpyWnHWi1Rv792sKa4emQ02zooYE/jRB6PPZefq5CEX
o8RMIUE/f/3Z+SxDCbjaD/8koOftpDbSk2JWglrRXtKr/+lf1HSZokEOLuDS
KebQ968s0NmzRziA1K/3YG0dxcsDFeDW71d/ZbrQeoXuhsOoArj3xlaWTWH2
tLtYYjEW2RtVjbCc8E53KNOU4PYU4XM5fV6CEa6k/ZNfR3NZ+R3tYZ3lz5J5
OizfxcfYopebFrOvLByZ5jcI8efcVfYAXw/gqgJ55b7gQI9uAfsZUXcgkcv7
KAB9n0768IiEhD+85tvNd34e4wsjdlbSfuj5nbRbIhMueEOTIYGPlsg+S4+e
fJ3I5AlFe1+dTUdYnElGM87UE3A/dr9qhnAos0G6f6UnuDK08zkhnipJp2JE
0+YNGobTaXfvxvQ5Dja/bkx+iAfFWoFhi20VwAqX4PiBFlt/+14aiBD4Jatk
3c9LNcpQ99TCKxZC/e4V3wfnbKThhlF5qzgh39Nkjq9hKxDu16vz6EDrk6a7
iV7gaw94f7z81g70/eAVz89cYfKGAlYBJj/RfF2NL48Y/O0FBdtX3Qn2iBiP
9/10zpUN3f8H26QzZraukMzg+seXhP5Jj9it/3S4wrYjP+PPIUw1FSLMfVER
TK7UUFSj+/O0zVX94HoIyL/HnSAn8LHeGJZ3EnLA+dcr3RcIU+DfXmoVVgaM
rjETcwj/YXI48atFkmS/93tqX7+aewR+lKFpXSbUZ9XrTNTnuoDJ0BrzrwhX
nrjf433RGcyoJthdQ/il6zblp1J7wDi75k7Q45a5SmOgvdsJvDZorJgi6CcV
d253gBOgd7D50Rjy94f4lo58JbMDDLJqT1mQvXB94Lz/87wNKKL8mqiJ+Mgh
5bzGFkEWyEKP+3ITrd/JfR3+/Ye2HgzES0dZIpz4PoP2wH5xyF/EVJSFzpeU
X8hcnOOFAq3qQgqI36Qm/YxhZmCBrfR8wv0IuyngXh/x/vOA2exSjf8cDij0
VNIHOrjAb3zZUckIyzlMZyXwusAbzNFs1Oh9bxor4Y5EuUAf3fGuP+h5lF/d
NFJ84Qyfmcc/nET2NDTJEq/h6Q+YfGncrz7E6hn8V9Lu7Lq4CMgzbuSHu+eR
8m9zNJqlSvtyIbjLKdbXvkjaL391Kbq4NBD5Wxt2VI9WCkj7t2HdrMod5QVw
YrhQ3F4MD3x3z53mqS4DOx2rS2SYsf7QLNbhTwzQ/YVlR+2UovD/J37YhwPh
26rnvX4GAND2hvI/+nj/H19U7unlT+bDA7t9HkWH0PpWXSPXTzOwCBQblqfH
f16CbmNGaxzIf/80/PrK8ceXSPc/90xqrvhAGRqvhZgdD7F+qk37TI4MGeIA
30bHEm6lGEoUF4msIr72blzekXvpEszntvrchPhjXfgeWaeEi/DbR/W78xl4
IBGlEBWvlwvvUKR2lbDjQbl133ryHh8S308T7nirk1ABiOu9WeYv/KNb2H4P
Wbh6dMpcGbShHRC/W4Ij1QMS+VeUaMsfA/4ikn5ltfCP4jmVbFJ9ReU1w5x3
Ldyk+on8EgtJy7/8JPzmAxnHfcEDYDLL9bIawrt5CPMfO7+beqSyzekg6XqW
x3OY7ZQESXhYqZhy+yAvbG1tV36McPZ/1j9uyHfjsizZHwwT9Qe23/Ksr9H4
kNYLZVGm/tmnoeCDyhnHqYJFtF6uLXKyl5P0Ub7MijTSc5fDhmSpgv3IfhaN
Gct8pggH4QmWG4T9Penb5R/HT4SBFJWqhQ/JWP4sXfFlrn6CP91+JcLOMQZw
tdqXEvSj/rtfJv/bzhcMsBz2ZZh+cOrGgZszP65SToaBvVzMh+8jnFn5WpA7
MwxojCct6nzEge+ni2Xb8FRg7jXFbfV9eOBxZ/jv2ln3f+rfkf0efrZD9UZJ
CKnfJM+NSJ6URH9YddbYuZ7gb/9//J3YH1lsl9W3yuggGL75CT/fhQNhd+pV
d5+PhMyfMhvDkX/glzCuedY1ClYoUjXZ3MD0CIn8dafIh/h06kA4uk9n3Wgc
B9Yn5fqplWxAmdo8jpB/TLSvS7jduoT9fSKurhwIj2rBgeVSxZ6mmD3gea10
p+otbP+PeP+7iinD6rI4oZuoz9QiNba/RYx3WCa8z6d0jAYZ11xNub8if/z+
DQOx5xGARSTSyuE9Vu9x1eMdOcU41q+e2F9eL5AjgVw4HBL1eHYXuS5oR2nA
lycVDhP0/ilSwV+YakmqL5D9uLD0948l6GOMs0lCWOWf/qFATbR/7d0SFp/t
ebmbjAetN+4bws9Ve12AfRzucSc6P97HYSbuvRPgGjvfsLyI3Q/bI9OgZw/R
fG5kW+4rDgUFVLxidHux9dKrIz/q8AtkPy9TH//90ANYUA7LDA9g9ShthAk8
iAOPeJv3pn32Id3v3TtlLBOuCuBD2blSQj04kX8LDIptxfzE+Hrqp3khxnrk
n4S08VxpSgYZIJbCG31vBRa2HOTBgbDOya3lIcILuGQTc7aTsMLDXexK6CIp
n4y4fy224q310jIf0C6cpnn+cRFEWxVIU8jkAx/RPQNbCLsI1Ok/l8oH+Ts5
ybaQ/X2vpy5XXFQAy88aCrGZ4oFUpZnEQ/UCaKf226vKEA/s15tT11wKIL01
VXbtITx4fDju/kOhcpK92k8tMHHHVBtaHXKlp5nC9JBUfkh2G3/B+qs+MH8y
SNaN4f/YA4gD5/+qHPeUDgdMj795HCfUA7iJx2wFBpDiC4c4Z3vzVk8DrVe1
f6ZycOBC9BWPmQvBUGUgeuRVHg6kO5WWFHsEw6SDO1Lke3Ag1ksa7P0eArn7
L+ld6kX+WN768Xvzp+EHvmdUYncxfYkU+zubLxE/sfHomxKTUwTVTNZ3RBAf
cRUqMI59rQR3P8nh9+VEfPFx2S/6In+YXhqamrSwCE5ncKhcX70AxSe42TX7
Ed8+W2ksrZ4Je1geuV5F61V64CuLfTPZ4IZ2r00iwjcG5K+7B+eAoh1MQxlo
vtIZF8vfoxMHYn/Gg2MR5poMHWNmFgetFs2TsSfxIPL6qZALnZHQ+c4mxTMf
PGA6/sPeaiIKGtQxZKcj/kzMFyLy6U8y2jmP9M+BOaHrJ+Vt8eDSYVvdbKtc
EKx6+0OEEdZv9j/+kTEeyFc07xooOw/O/dUwv8GL5bMS43EVo2q3DT6XALns
4d29D7H5kdbd0Px2GAcsuDMlXO0yIP28/faNl2h+fbdUSJlIhy+yfugr9GHx
5j35ZrO3r6DxF3b8/OSiBfRk1TI3Q1i+t28Ha4Q57Bd+rGO7G/NXLN3xibXT
WLxR9kssL9cUNr/+kw+GcBJrb595iC+pPvx2957zZl4hYCx9kL4b8a1jB5eL
VCkDgPKNSNZVhHs9Qu8nnAwAXGZ36FaRf8D/uv4dXVQwtLm4kWTOhuVTE/N3
ym4t20pdPg3qJlTk00cx/4CYX5uyFRtz2cwV6jjKpRH2c06eWpO/Mx4Ka8Kk
fPMV0fMwPxX0+RZP0ucyLYxM7hbOA3kUBw4uMeBIfGZ7ZMN9vW4R1D712+Q+
WgAiNlmPKFQuAsbgDjp23wLQrHGV7wayh//H1HXHU/n9cSMRISNlVMoICZGi
5JhZ2XvvnT2yN5WQlURGkqwISQqHZBTRVFKIhOjeq8ySfufqe+/z+/P9Os9z
n+c+55zPOJ/P5/2pOBwtw3jNE7S3ri7yKBFAhv9Pdo7kC8B3hMNbGO3fLQd/
XUrJuAgq+ZTuyCQRAJfsh96QU7nAh9brp60NHozrLc4+W0yGtxPYw4b4CGDJ
0HwPE2M+ONYqfekLPQHs4XoZzakbCu8yOP5VR/5MkdAPgu3zEFhBW5dBjI/X
iTo57gmwB4S/XqPFaH62fbvkypLkBByzPVdte/CgiuP59Ntxb/DI5E1iOfLX
cf/ipcCf7XSbNtKPlolnXkb7+ENaT8UabfR/3jezBh6T1QGAys7kKlq/v62f
Lx3u1wK2Bt78p1IIQPWBgsNQuw/8lnxs4acsAZSHXPktLJwAkuN0WV6lEOVV
zq3CI/FgjZsrqKKIAEzy1H3EFPJBTHf0U6FSPNh997eu0v1MKAi87BtCkf6+
9Xg97MSF/+K3aD2yXJceSwiFAQ3hk0qnMP0Uxzj+yk0H43dXSropfkmVAJL0
Jt2/njID6qyUbqWnMH5ivorUxd8zeDB4gk06504o2GOS/rCb6B+u6cnfyrED
zV720pcRNls+9X32oC3wPu+zWwTJ9y3rVb3PZizhi4clOVFIPg6e/Sb1w9sK
vtGcNH6Nvi+3g5yBg7MN5MuAz3q+YfHKzXwchL8J7ITezrYwfcG0iMgv9vjp
UMIHFTu4SJ1fPfgIydfRXuYrRaH/8tfbkL9sX0gn7lUIToc3UHtAAojbJIAv
BPFHtbTEYvHAQki4u4jpEhi0v9NdiOwFUj44yT55sPdXT8s5X0DKB/K7UcXn
vegNSXxcnF1JZaFeKjAh6A51Kyd2/qe08Cx8eBhP5vt5Qj07YTSM5atIWKT+
zmnExisyrcbo0P+vN+u+OFESCPafNL1ji/an6PH6zsfzjnAw//jPZzgc2J58
Vut0Zya5PmTrIXrL6hRlqKeWEEQJsf7etan7FVbRfnb+YW2/rJ0Djnx9G3wh
B8kT89zbVDHnMf7pIvuFfu4rIHb5LvU40g+SbMyf1rvTIW8N4YxPDwG8kM/i
7W/XADki9I/ouwmgp3dVuEb0NMjYV2pljfQhbZChR292CLC/p9eyhjC1YVON
sVooaFWLitiL/t/Uiau3FfyiAIvC29NTCCsG3aM9MxsJSP3r+vCWlw9tywai
14va6BvR/t8sGMgG6XtaOTQZsXyyVO6e5JYSJL+Eo/xmfOIg2+3lHfvQ/iHV
N+KOnehOSUHyb0aSwsH8IkjXP3jh/jv8f/X28eR8kb/qzXvkZELI8QUzWfu/
xQcyocLQ7/KDxfj/7IlEkCZjKqcvRNI318nn+/HEnMbpCKDcEdN8eAr73pvz
gdazxafxEZORYHC4rUHrzT203p0MY0ayPOB7Q8/H8ano97WVc7W8siFlf1kw
Mf+HVI9hzrV1Fw+afz6Vb5U6c84AgIvRAOGKeH2zjARnYHux7sbrnQQy//6G
yUdXYj+x0dSfdTfw8WBfyO/DzUi/HO8XiDB5ePxffp4EAbiJpdZ9nBeA3/L8
7bnFCUB5I09s6qYY/JwW5ZUnRgBgP9No7CsJyB4v7+b+mAD6Dqrujw3Mh49c
hdT7nhDI/RL8Nx0GNL9tW6xShfOhLkuDdI8uHvx2q1bZLZaC7JUC7uMIl6vd
6PSVTSH3w8Xd+XCDSvwgGTPQqkhdZeAjY/12i4umL/aQ8RX/w9UxF3eScbEG
RaFhIj0Zi1pp9Wjhf7aRcGR6y2U9s2/yJBz1KvPpOB8tmV/HRSG/aeg9Gxnn
7PKib93HQ8aO54pq9KUOkPFvB7r3T0wEgYYhbowP4Yfxt6o3JgWgyi4npxHk
3+rkXNIxCuSFXK6zJUS+FK82AVc7lt3wkTaP2CqRf9rZImPKgQVaHdVt5kf3
N+7ae+6tGw38wxWnoo+w9DOmq3AXoe13bYKKFbr+T59Ox4oDPzyKG12KRvhO
3amcysb90EtU9vZNhB+ImQ893LEXektPa/Oj+f23PxJAyEN1o8VqAlDym7ZJ
r7oOuBsNDpsm4YEL7xPLbfeT4KrR0VfDj7B+kdOUU3W59Bg/ikwaTrQXYXEW
FuseA3NQukDY39CP9Y8g1ZNamfqLlitfgjyPqpauIP/sXfmxUQa2dOhLc89E
Fcn/cgXhth5mc+D4guslLRvWz2I3Fet44Cyy3weYX7de9Yaf9zjbKsxi+bGk
fKI9PCd86R9pgbCGE6eSEP7dGd+zP10LGD5MjXFE+688q+XUGF8gyFn8rNwk
hOU7HOtI/HhrC9ZPODQx/wPxfOxVcdhvtj2OwO30UsABJL+19k1UjJY5A5XS
wNk15P9mTL77lPDOGZ738EzwOIsDFOL6pVfeFcA5K1XFL804cj4NqZ/E9W/F
aQ82HOCldxn2xN8blfCYkSo8DXPbFx7yLWH9uN3PtXyV+vl//bl7pcdnf+AB
29iuvU1H1QCrmJkKM8KHriQLGN/QBKkcDmXE/CCqS9yL3qf0gK+EUR2xn3jr
Q+7s5jUVcM/zd5ACet/FrJWQtazT4KKGzkIrGu+/kq8fvFUDqHHnf9FG71PY
uFOd2UcNdLsoPqt8j+WzcWqox20gf1CANehjzoAerKD0CJNH/pjIA96XX67r
wUVexUc1RP4wQUEF5zBt2NvZorsd4VHust9LvKrQI5hHem0/AVC+zLlmqh5M
Pm/55TRnvewaDJsmy6gZkD1F90/+QcGi5svFSB8NP41luMQVAsv3/BB68xgP
Sv/xnQCPLwfojnTigXs1tcSIdgCY7Yx5G4vwSTXZPG2KAEDii3gdlXiH90M2
SL1ZN+aTggM1scsFb7qyyfoi8FOrytq1mH/xjn4cUDH94amgFwNbiodCVV7h
wLF2ugQesRi47GX3YQBh3rrEN5YiMdCH+sUN2jbkL9ooGvanxcDDyWcZpdF8
332vebngagxU7V81DOvCgR/bhGuNgmIg043cFy69ONA07St3yC0GlmeoMo0Y
EAD+c64FdaDrf3FxAgg7XLln5p478Fc5N2DWQQDVf39e1TS6BAfunv9Bewzp
x7qtWSnzsUCjHidCifDpjfj1wJlY0Pe3lKJEmgBe2tSp7Q6NgtbZq68idbH8
lihwxMjcB8nz6ddxCkFB/+Xb4cGilvcRbp4gKHHUYN8xIp/KaZzPMmNLm9bP
V3VyUli+Nane8KqQ++ElOiEgdW9rArF/uuimPosAYm+Oj7Ihe3qgjKtLesYA
zHvb7BJsw4Mfy+XRn2gd4QHWJe1QhJV1q1yo7juQ+Z+N5P0tjaDSP/5TZoQ7
Gyd7pM3I8vnMouE2+gp/eHhfi0Qlwkfn2h3uF/uT+QPrEqZzPrsbg5L55eLM
Eey8YqREtu8D8Xw+6/4Z/FNzsL4xtSDFjPVLIj1vUN+DaTzcGlD1r/Uv0mH1
rQ8EVh5BhB91VXy2NHWH3RNJiXu2YfxXtVQul5kR9s9cKnr50gVCXxVDa0Zs
vKld7N72OaTPro5xWa4rgltMD+vtcHiwLHvUnX9NA2gNOM9qInlnl2KxI9zE
ELL0r+3bjfwfKnoxqowvpyHp/GtvjiTllJomPH6v3mIO6Xeniqm4lh49+Jqe
JSYb4VnJiGLxXH3gGtFEPbqNWC9pXOw8rULOLxOrkpjMi1AF2cqqZ1OJ/IAi
3xvb6Azg/ZKtwlQ4rB6d9Dy7G346xwsUgNgTHEMsAQ9yqwLKnI9Ik/mYj/+R
8mRPC4TGenSllWj/OxUOryt6GQGWkYidLEj+3JOG04XX9AF+5vJMFtr/eyfm
Lu5LNASB032sdj8wPoCXvXeELOax/u+uZqp5RwgY3tFsrZiExo8mPHr8xcYU
NorK6DCi7/PHq9entcQYju4Jr7uMrpd0/4r/420C7zw4JFiDcGLujwG6fmPY
NPf4Vgr6f9lz7KFU5/XJ37NZK6XbqU0amrOsukij3+NZEFQTopWCgzJiifzo
eg53FpYh/yNQueMSOzv6f2tvJaN7RJWBI1goTUHvzxouHF1JhfEPjo4+7bPr
VASzOlk7XyMstCmvFcER4nkfkt+5EoYBNm1ywENf+rcxwmV/YW2ykyIst/db
IfKv+LMZel7IloFrdIHz+Tux9VNdI2tC7Ff54/MYjSa9KxRp3YvriSYAJmWn
wz5qMeR6vRmBLSy+366Ba/ztFS2ZBKBYnz5f230NfD/QOrmOfu/h3Mg4jiIe
nDYfsZlFmEOcMui6bTy5vrd7y4dEV+MYOBwU7J92DQfcAvjbCEdjYNUJ7UKL
P3jA6M7QXDwQCa7cXyndUUEg80f9q+cjgKr+g/JTSgVw4ZRZUBsxP6z1LteL
KkkYSle7fy+SD2fkr55pEBGDnDQRiYahyP5tfTpTmxf5X/0WAaSvqFgfeRIB
ZMPHCudZCP/lm6eCgXaWpzTIHg9pUzoXkJ0KXp85g1MyQvdHZrB026aT5aXy
XEcexdV0MOxNq3nmLsYXvZnuhfBwEVUubksBsLttvmW8AAfOez67qjJzHf47
psGBkyGEJaWN69C/u1Y+2pBUH5oO+Jele3cifbWPiXalXysPOOroBRXQEoBc
8oGDDacNQNy5y1rNCE/rF0v5UhmCK7INeyPQ/tITpKlRlTQAQ4INO3AIX6BI
reT3NAKNbFQcLNsw+8JHaa1xK8Klk1/9Los5gKlU/mlVPBa/I8XTHO9zuN2U
MIKZr3eYMDJi8Wm897118W3Y+fOA7IDixGfMXiDl+77xdivjcDEALx6L9D5a
wPLBFeo0s2zmMVwbvOZBHB9OdNi11OYADkyquwQhfLLMZXmXkj0oZv30zonY
/w1cmdl71QHIg30eVxAuFl6eYLa2B/1nUxktFjB+nZmbsq57DmL19+bxSrFz
89j50x3T+xxm81j98819raeIfJF39d3swqxcwLntj0XiEF4ITtgFSl1B3BnL
znK0f3RvnDnQo+cKzgocrLdH2C2GtcoU5wI26wdX8eDp3+7YRU4n+FiyV2Ye
rcfy0L9TsixO0MY/bU8NNZbfvFkvQMzPkr3osP+eE7y5Rdn9D7I3CROtdAXc
XkByp+zNYnbMf9/9461nJMLbj67v8XjkA4Zu5msJIXtOi3/rgMcBJxjr9VVO
6R4O2V1D7eZ8DrDaMylaGK1/uhFzs2C9w0CjU2HqGXre/dt3r3gclSTLZ0Wm
bwn8G4eAjLptInG9NB4cii3rPATO7avdyGHE8iNJ+Rgvch3ECgRlgLjUeTkK
5P+LzunPVUvEkvM1pG597qu5JAlPLfByBaL7dzYVull8l4KL5wY504j1av0a
+S+ZjsLbEt/cbZB+6/i7p6dHwwzYUxf8METyMOTTiwN8fVaAc1fU2fNIPnV0
CPwxu47m96eoM5GPNUI5i56HxwGYHRxVckT3z7TkRQrIm8H0487ziQinbs6v
OZxWY5mbeoEjn0+Q6jN0Xk2VK/hEw/MVroK4EWQ/8a6sx72Phg5vdovpIF9Y
c8K869JMFLl+NK856YyDQTzgXPn8irMM+dsLtvdT++LBXuBlbo78F7oLx546
/40HwTuV/es3kDyNbTkxt+ICcc+5qFgFCaDTdzl54oEf2BI+65LAQiD3+7uW
jR/Ych4PPlfEUU/JJgMuWb0PNQIYP4B4/tkKQWRvDqU0h80oBYLcy3vKoxFO
Zq96u/tnALIj3cVYkf9hfLnsCNMzV3K/n+0NEjXb+pzhsZCGGPFZLP+FNJ70
5OBq5bIKqPzY5Rc++3/5I2riAiNofdkVO9tYhiSBlZM/lVzRuHVCtbZmlhPZ
Hnok+uBT1TFH+Ezq6noB8ucKmwtq0uId4R+FtssmBOz3SPn3DPNZ3smFpuR8
m8IbQxv7VpXgc7FxRmEclp/yvVn6cDAjlp9OWk/SlLLqGUEG5H5BEebH7yRG
AfDk9sW4rm1YvnDKxfA1A1oMPzZj3JexBeOrvHxDjf8SDYb3d9UzXEP+Ru2b
+iq5+8aw8uY+lXg0/vvDOtScsIABhPapRGK/3sAvLVc1zaBJvrkmMV95+fvk
ve1ClvDyaTXzIDRu8btqRzSdMbwQO+FTiMZ9km4vaxgaw1m2vOCLaPywT/FO
i/OmMOIEnerCd6yeYL0/iAr/DQ96rQ7wru8yBIzzGYzE/i4k+3FUv6Q/jwbD
QcM+KcT3uVYcNVHw2Qjk3FuXyUdY/fYjt7U6U8AyTB+7G82XzeDCa3NnT7iP
k3UpaAp7Hmk+KIP2p/q81wADBj9y05H915EQ2efNcxo0cdNbry9h15PyhYaU
JrWC/2gC2UwHWZqfmH1kmmqaI7yCYY1VtxnVJQy3T8gKriF7RRjX/URpzBxy
yHRYaKL7j80KG1Cwm0AeOZh1GmFqTVoGejUTWNL6RW0e6Z+MkRaLyL0WMNoo
8cEOtJ4ofz2R9FUwJ6/fv3fb6+536cOFPuf2jFksn8r9/YGom9+I/G6CgOu2
Dgwb4eeMQXinVfBa+F5dyO3itU7kc+FIWONTdbWAe7O98q2WiPFhX+bxHjMY
0LbnhQvx/6sXs+p6GMNd1nrM2gjP7zu64xC7KTRRn7hGif4vM512tj6a79fL
8vceXEfrdYhJYW4uDwj/2agVvEYAP5vktTQT84CQfC79h3oc6A93Nj+XEwMn
CEy7y8pwIPIa93hPcAwMFGwUvnITB0qfrceIn42B76zf2EsT/WOumnmlEDdQ
eeqRzw6EpeyCA+qH3cj1Y6Jwg0ndPho8kTJaaA3H+sNWX/S1bEHyZRg/PoBL
SANPGf6AMYR/1Ci1PeBOA1OURxyJ/jiJ74Gd3aJ+hshnHKriTFXpBh0LH1e5
b8fqOT7+LNhymoEAPommCJUMaIGx3th7RP79EZP5wEdvLMDxnxf7ifHov57n
PnhdNwaZIdw+oUj/3r0UyvOgypKsX9q/vNshVKxC9q8auT8wsV6WgUmX6ntv
If+eRZbt2uU/sVBF3f2wHMJRS5R/ujrj4C7K6/7yz/Dgo8WJwcdnfMn6xenh
5wGNZD0YXNLEH4ber/8La5j3G02YaH/9I7EfQO6jQZ1phzNgp+RSPRvCpQ+G
PUsP6JHrU8+9yO5KtNMCjYtVUsTzk8HN76cDAmYpju79P3xri/4zYnycJI9I
+XdlT0Zm/zw2BJxxASet0P/Nbi7lyZKxBYXL+uPE/j3ta1lXk0btgNPZhK/e
03gkhzblOTiRmE7ji/abv+SV3SWWoUAVBM5TIjx6wP8K/95Q0DinkPKrDutv
Q4rXy+JMH02KRMEPzUf/6PITyP1rNvUZH/K3T5ThM9zOwQxv45Ypf+TPe+mJ
XigJA0yNQpP+zwhA3qxOSWYxDxpR0GU97kX2m6HonDhzPvz5UC1ThhWLJ5LO
Vz/yH1jwOBMJHy9XTTtREYCkIPXbUE0WGPr13NgdhHl4aTK2Oqy10T/xWRxD
+kDB/0iqgj0LTI3PLLmK8Pv8O7fv8v5qW3eeSO6hIq7nNlYeZhE4JXeeXRxh
07m5kEqwBzb/tK6sQfbV9p2CDXONdsCzdTvTlu+Y/Vajw62chtaD7xxzUf5j
S7g060rB9g6Ln2TfVNZ2IGB8N/R/v0zNo/X8mKKt8OyIPUho79R/hMf0H9tk
0XA0wrKrZVFvXjrBAOEYs2KEl1XL3pt0OUPx8o1TugTMfg1YNG0NJPbjLvuw
rvvBELBQ6+1fJd5fcjlRW9cA2IYUHWtD8u3lpj43AJ9dHI4eQ/rQyO+RacAt
f9jdekH8FPoeveIi795U+EPN0qFiXaS/2Is+ZfX5ngKWTw5fnUfrqb/S6+hv
e3UQ+9Z4YgjhZ/0+N3IH1UBr8dtGTxqMz0YtnOXvFLHfxVhcwcNBOxBcVv6U
2B+qhG9ReG6LJbmeQ6poY6Ms0wxkvx7sKEP41j97CNwrmDrPiP6fxZ46aw5T
BXj7Nl/oyBL2PV8NbmdjQ/KY5ub7mEkNO/hCe6XvCR7j0zjwZriJuP9J1+t/
/VBoQIPhq7H0l8fR+P7FtNCTo5bQyffQ9RGE2Y5+xCt32pHzl1+upGZLl2/I
k/Bvi8TmtgheMh/XL4G2v12hAuTrHfmL/D+WssA4FiMuDvS+Wo8uNAW+ogTu
p28nryH91M2zVJw4SQUsSuhFPiP/14z+SDjXkBDoq95lQ8yvp0pjdKr8vSrf
f4WTY4OYT1p/8fYefU5gbGLBHYjGE/rSuH+w8gHRQyejd/5C+tiXcFGTVxiY
rwi87ELP1wsQ+akduyI/TSlxuRfdH4AXe3dYnZOsH+3xo5WX70iSsWMR52fK
XH4wN96uV4Pw9jqmcotGCdCXjavtQ+8v9tb9x63zHDDM/fDqGTSuojScKuDI
AS1jt1fdQO9z72lyyN3d9HDHWccHQuh5hH27+i7m0MPDozdEtNH7ld0N+NN0
7BBkP8+cdAddHyz4tEHghAD8LR75mRHhGJlTbwTD98LrvBme/SsYnyfJPxmK
bTRKSBck4/3vF865+0uA8qshbbv1CMAsZkt1P40zEL5c/G7KHcsXaIuf45RH
7+vXzu46Eh0JpPWvN1eLIP/6t16yqnb4v/MNUQIQ3BZllvc5nOw/Mv5pPLDX
3xS4nVaTs2TC7JkLTjnZ7TsI4KgOm1+Znz0IZzg0KcOC+aeUwy+DU9H6yubj
3ZAvsQSZ8SsrRHvLEDSsS4jagNTDQbtPIdzLHSjRkWQHftAx0ZogrPuFw487
wxasKJhGbEH6p75/wb+I2RmcKV01XkX7z7tSsaKowhm4mOHj8tF83HlvGy4a
aQW7B63w1QjvNqq70RZoDeV5DdI00P/tP6CcPWeJ1m/TYI0g+r39/WdelMs4
Q+3skRv7EQ5QeuTMEuIMaU6/bAhEzz8ey3Wt+pMlPCi/rV2PmM8tX5Z67qwl
pPtuvGRH7G9kcVf1yZo1PDT/cYSPiBVgaIibHVRuvjIihXDK9xKbPZO2MMa+
+9sBZA8/NjgcIDPhBAubu0MLtmPngwFjhqeJfJy3Q+15d7xyJZ+f7bh9q8+Z
ZSc4PspaZYfsn6WFrgctHTtB+LObbXMIH0qek6Hy4gBvU/NOTSF5q+AR/vDM
dzYw3zRQyoFw2nmOfsYjbGR5En3qRunGDkYyNunjTrjzSgBya60pdxLPC+KK
LdKGmUF029rQDJIvI0Hvpa5X7QAj/s3cinjsvK3PHS8ejOw1wmH9Im9zOdjE
/RYfieSNGPPavoeBvOT+KklGbxblAnXg7gsv2rMQLvqnj+EXntSNmQXM3rtF
U31MCOEW3rWfUi2G5PqxiWSOc9WZ+lBW9TutK8LN/+QzTPqiyUCUR190Okq8
LpyBs6vLcnQI/2wWpxSU1yM//6LGKcfECg2goaL4FI/w+uZ6VSOfp/mp855h
tTpN/h4zbsvBIT6SIGErlcYrApYPRhp/zLqX6qOHIqj+ZkEgvj/pfC6nIZ+2
HK23/n+/D8+z5e14+xPDUm67bir+wPwj0vN3RTo85RxThA6dV2pr0P2rmwTn
ijDg+cfawJ8YDrfGh7Wg97epYKUq/nASXhf+JPkd4R93f0dPQWUoyrm7QAHd
/+XI4F/5bCW4Mt02JI3m63UFA7fUymFIzXQ0sgS9r9JYScqDwhPkek6Z1I2a
Cw18ZMzDatGWI7OdjE0lwtqdrbeS6y29qGSY1uUOgDM+C2PE/fqlK22IWZQF
6nT3uDxA45dYZpqzPFlgD1X+/H5qAvjmIBbwcN9WmMj6ZG0c2QuGAfs9cm6J
wIuM9/KPEM9TNrSP+13jhyl0pafiiOcnbEt0uVx74Mw1N6l1hFMm750pL6SB
LJ03zJlL8UAgV7KnxDQBBLoJbFxC8y2xe+jBxeeOYLee/OP6rZi8GUg8c3Eb
2s+0FnWX2cYdgRoVFzMTwomsZopHXRwBIVNhf8VP7HzMEpbe1kPfc6dhx64G
LSfIG8TB/Zq4nlmq68IVHGGyWV4M8XyoRJeOvXXQHl6QvHcvAmHxJP2iol0O
8GZrDocD+t7HAnMjnko7QeeXh5d0EVYOnN4rQeEIq+4+UFJB+MH3hK/dmQ6w
er5iXAOP5S/l//ocegi9362nbUrmTcgfn39LeRjhNXX5i6aXHGFGshKNJpIH
Y47Sv/ZaOQGNjq9viPUXu80qPYaZLMH9B09Vr9EQ+0e+uL2LxwxMODsEBS9h
+n6ied9gJA3G37Z2cmbqBJIPgZmFBTJFHmBt9UGRNvr9p9TuzM/zncj2/uwW
jyn508LwYROd9ASSt6WCGV+eZPOT642Hdxw3/fVrDxk/97/ezyjGA1RW8cPs
6PrrfyeC3W8cgG4ECbYpNP+3KzldlYELKI+vs/+D/r8Fk8DNs0qu8NaxB1GT
CP/wWGRKeuwCG7tjLHb/xoM3/I0hVk3OkO/x67zGdXS901Zx/UAX8vqMGFya
5tF0hEdfbljuQThvM57nCLl7pO7tQPabRZORsR/DF/lfo7PgFML86VFWa7ZU
YHi5qM8dYacftx05WpnBAyYT0xyEzSvdGjlTOcAE+BRsgey9xynvKVUFp+S3
fi4TEUN43HVv+o0sKiAS5ztKiXBsdfdY3SdmoMN3IvMtuj83pGnX5ywOYK3y
ob+KGB9vsayeeMcDxgTVVS4grBf3LEqcYz8ICVrsbUbfw1jhrLex3A/5nfav
g/wQjjF/ePPeCyZQd6b+gDDCNhr71IsUuUBgcV5wDcIeZx7ezTksBGSvGu+7
iJ6vk/6ZM8VzQf6SuOfuVwib3ZQz4sxjAp96Yli40fWy4In2McAFTN+lf9BF
808ZxaVo8nM7UKFjLioh1ttuyaP11dkOVuOVzzGh/VZJ2+piqEEBjB9Vb91O
3M+ZYcxqYxSA+qmzSSo1kU9jcjXMlxuspViHnSLu726z6eRGPoDXMy78jp7X
9h0YvK8XJvNr6NeycbvckwRp8pHxamh/ZkUZGYdUc4A59mURYv+faJvzB8++
4wC9nBDHitbLcozLtcyIA+R+josTNekJthPyJHzw/lPP+ncU5PG4BteG+F2M
ZBz8Pi6ysoqdjA+909+f1ccF3Lg5J4XQ9yflz7f3Rbwq/4vl0/MFDO2tpcH4
Ekj1yOrbXdrs686A9LyDA2wMWLyP8vHzmWAcHlxxVOerNDIlP8955Fj2DSkO
cj79Txe1c61sO8h46UvtvfppajLeQVCeqyjCtdHfbbJ8guwnnStn0tjnVAHH
Rb2fLsh+Iumrw10vRoh8AKUvnvTJJPqR+ZpWKBx5LrL6QR2/8D5mhMOJ5VNR
fpDl8YftArsIoOVl7meGA+7/7G+EO8cFaUSfuEPOtPdfiPUbtzYJftOBVV9P
BpGfUWBe8bImUwb41R1UShw/9tDMfLw8HeTLfW4SlUL+/4fiohaewbaILNYL
xH6oXNdLRLI5eCEpPiJzvUG5LOM6zLqkrve4CvlHYTibc8PXYeUhb0mFGgLI
iKudz6m/Dpvvn061aUT6pDzo9ITSdbiwZ2wbLfJX9Q6pztO8jAAnKwUa5BEW
i/Dic2OPBHk9NnkPGwhAO1vb2siqAAxWxvKGoeep/HaTY6crQN9pcKAxH8un
UhrfRtmO7C3H9cY5igJjMJb9/H4g8j/rLz1T/HnTEpDy26/KbCXcKTYD7w54
KTIOYf7qlGvSdYNbyL8wEz5gviwHhbLvnNlA+ufsyZv9On4qcKaW88q3Doxv
zmNpxmQB6YPCWjXB8l30oObnwHAA0ierCapaMXeEgdwFUc6zSP8fFexyj4N0
ZPujMCPy/uSoGCj9ncexYxmrTyT5H7lthz/9cjsELRV7O87F4kDtFu2nF6mu
AK/IjUXOo8g//Xvn+1BfDJj68uwJpxTGR6qU8p3z9ir2eyT/wPfDkNgjD0ky
VormothzSoSM+1+kRPpq8sIcBWbmGeRvpOkPSzqdF4eK399I5RDPy+LnqTKD
RMn8c89ZDei7hhMgiQ/x5M1Zib2RATD2bdZqXQge2LF0mZ3nCYT911WmJ8Pw
4KZtQvx0fwDMibQSf0vs3/Hjzpaxbn+4TTomlJ2eQM5XK39mSx+2lwCKS9h6
7AKdgWEy28bD7Vh8Kuk440E/hBuuiogwRNtD0ek9LdHteGA9Z7FTMd4Ops+1
j8qPInvX4FKbOt4IGPT7wW9IvnzAaX8uveUCqJNmrHrpsPpHzvDnXhtIHpH4
IG+xzyj5EPGJu725X8PI561pmsY/TO/sAlRnYzxav+DBJKWciEErB2ilIGge
msKD+OdFaswlu8Crm78ZOZH+bWQOOxQ0Jg47A2cUaIj9AQ92FJy5iM3vVOLH
+nGbY3DunUL9LWK8e/N9jsPByvgwHSJf6jjFydcgHfwseRUthfD7jrzX1YXp
YLOdTBoOcISkn3A6XoD0b500yMQBqTWaGmu+AthnsUPfNh0HRhe7ttaLFsAn
JxzjriTjwMHqbdoqGgWwWe9A6EoLjmx/eNZcfE3MZyXxKX2UPxYfjvDDdpFy
AV8/aHIyu+w8sV/tl1f6/Rf8YL2nrNOxOeSfl2dSV8b5w5Vqyy2ryF9M2sfh
28l5DOp/TRhjuEfM59ZXnowyhoT1bmVrYnxgoCf2hYUXuT5XoTAHH2bmDU60
OPuUI/xlM17rBWau+/lEo+9johPx9JmFI/BuKlW3XMX4oqSbfFqI+U1Nb85H
6487w+ADWmefsRPAjh9mfg9afUHO6Nh8KZF/5ksmDc8xCRjhVv++DM33TRk3
vQ4aS3h718r+hkIc2N24IzzDJxuQ4re8DOaPPJOyyfzSkWEsfvqvs2FyevsY
kzIBPNu2bUmjIhvSJLxi81QggJe7WVnSOrLhUF6cW8JJND83Tz9zJWTDwJf7
8x+9xYNyVi727VK2YGoyQLNlAsn7zfXtCj4/Z4meRLjYcWhcJscFiAdnbR9H
uKKwN6DlhgsouT2uaqWL8QEWtHsEvXyEA+sMtbU+Ry3hpvqASL5Zp71lTy8E
KqkdO2g7kH4/01ooUlEIDBP9Fm62I3/d4EvMw6JCMr/unbbq4615BVCOR1Hg
Wx65vwakbTlJnfgc6TN/lSd0rZ5Qsz45qf851m9mk675Fw6cERca32EdBb9+
rnuqtIYDKn8lWipco+AB9jGqDjGsfqtFK0/6NrIvrT9NTb+6Gw6eCbmG3UD2
JWmc5N9+u/3j/H2ecEBpMRt6HO23G/8I5yH9Z0I0BXH/KSxKfH/tC48X9FHp
cCL58Er15hy1FfhU5kBHw4npY1L/AglpynU+fXOguDAZWovkredUxF6bUmew
3bXshjPChPcfKDaGncj5WhkN+vpTMk5wz+n6qJB7OLJ8kQ9Wq9ZH+y0yx/9E
DJsjSNjSNjiM/KeohhoZx3AtUG/N/C0Vye/7uKZeukxdkN/2gvPDBpKPA0k7
9YIdyfr9a1reoZgsR1j0OumdHpF/T8xTuqDrPOiY6Zc8zY7xH0ssnaeZ9MQh
/SCUtT5cAHXDE57RIkyan2zicnTHgScpt/uWpwqgKsV9RWJ/nNRnB+h/xqqQ
+ZBPr1D+audShWoBOWOzpZj/mWALhec/Yvxya+/e9nYjfOnXy0STZF/4xmHY
7fhHbP9rUmn4OCLc3bKvNDfODw753v7ggnA89cbBLeF+UNzm7e45Yj46brKL
8MQS6Hz+EETEpPmYGf2jUIf0X+GhIrbOEhtyfsuLB1xDbz5aA3q16O6LCDNt
nldbA/G3jR93jmPnu8IDv1JOIszIzGCu+dUZSoefGMWj+eOiS/UvqLeChi/X
juJ+YP5NQnhJyCix/jn8r1VXswV0b3/8kXh+SRonPf+8zDUjJjdh8vs8e8Sh
uj+EB17oWKMfXsTqIUn51y/qR9uizmP5fAv9n5oGcrIAJRU8l/AcB3YMKQ2U
ZWaBO/q0TYff4oBGq1XyO5csMP8jkC5FgwAStu5nOuKZDe/NLSbNqRHIfKWb
/UUQjjO3e9wQl/2PT4KDAJYsFlRt6V1hv2y9T4ct2v/uQuNvbvoB25t3PZNs
ML4eUv3p28ulnBLOaVBiukWmqBr5M8v+Zw4eSIMBqi7Tz+oIoMnUkbfHLxWS
rjd5OliyGGYGKc6YiYE7SH6c6bujuscYsvOaT6mj6684UoydeaILSfWXLYqe
na8coiFr4LVsfABafwM0Gcw20bDUXnHX7zAcKBH9MGliGw3tPPN53iD74MPn
q/wXrivDhl/nDvKv4cGeG+pBXNWK8CaOvSoLye+KcRsjE30FeOroY3WhXxh/
ydnCUjutNQyT7JGHBzJ+uvPLkPFWv2MnDuTLQ4/d2sNqZ9B8z+tW5t+1B0P+
O5emTyD/F/fyUE+RIijYrWGoLovlO7A9pmtySsGDv50icZdnE8n93GKOb2wt
rUyEMs+DFRPS8f/lASXCwOF3C0T/+xXDxB2CWgjoMV47FbQN4w8k+cc3LFKk
3E+cA39Xzo503cX/Z0/HAJtPhK3+j5H9d2vb/rcXrGB3ItXP9cfYeiXlv72n
u6vraW4Gq64yHWZlRfa3GqsQa7MtHP8U2dPFgp3Hk+L13H9uBRw94wUEenc9
z0LYMpzBk0XKCzwXT/DlQv7XIUmb1RfHA0BPX9FLfoTvS/TXDR4NANkxip47
0Pe91ie/UcdtB78Uppe+JGB88fohV3EXEf5t6DGpOeIGt8yWlgch7PbDXVSk
1g3e2ZAEX6mx9xEIn9e6gXC/Npdt8z47CPhvKV5F+NT7PQaBinawZ9J96TEj
Fp+UYZaXb0B4N8VAkeyaG6TPzNxOzCcJuaOrF7HqAg2oxC6fQzi34p1c7y5X
WEpXSPmGeD4q8eDlz3R3sFZ0s858Ftmzt5y8lh6pAOZLCnV+BXggtXHMIMc8
Hm5rZVJM+4zs9+u0RXyJXkCm0kk0CuGHVOY88gre4JVkga39Azw5v/Tmjg5K
WoQ138Z+3ecZAUp+e+Y2NCF952taLVseATpxu952o/Vc+2vdhTZdE1IqzjDT
rGD8gDhXlzv5W7D6uMFQOfWYrRgm1edw0Z/OabtgCGWy9AgcyF+wOM3+/OnJ
IHJ/I1fpu0bDuECQK+JpGYDwHFH6zAT+W4fIvomN3fCpU5eFWz5Z808gfbLt
15Ch1dVrmH8J5hgMr1yDtzuiTQKQ/u/qpvSu0bsEH17YLhyPMClfnlQfcfjN
VWaPi8lQ7XWI+NtrWD4NqV5vZSf3H61BrP7F635KGqfrdWjnxssSX4cDTuJX
Hv6Ovg7TCA6DjbU4oPS8537fxevw1wJln+F9AogVFIh9JXWdzI8zqXRrWlUG
47+aqOh9fZwlEuKTo5Xk0Pfrij3q5G+kAu/2+nV2ovVTTZl5jWWHAgwe+POL
k5ifQWloHvhFDr4OMXQPQuPJT85lCF1QhIzz71ptgwhAdnvGXqWbwWS+zxRv
vVX+vHPwvABnZpUvFl/t33m/OxLNX/m08eUHmp5g188llwSESf3rSPKlvnTX
4pP9Z4HxjgzDVPQ9FP4+DnIdPUuuZxROuHtkT+9ZMN7wY/0Hul5scz2dBT//
8hoR44PJRk9bs6tdAa3rbq0HFJh/sZk/h/C9r9PKTVOugC9oPMPwDx5wf9X9
6PfMFex1G/Sj2MD4RaaJ9ifCajRbtd6s/xcPOEsAgOVHckvgOdDlIBmq604A
hB1cx8sEgkHQvUUZI6QfUiwT1TdOZ4Kt7gG77Kwwfoa46Evmvej93L6w4aML
Xcjx6Ed1vvRUD6PBa8/ofVWfsfNQvc+WR0WQPUr3duWDFcEe7uxypTPH44HE
5nqxg5LlFRqmCA+uvR+WDbKDThf2VKQyY/k4Q/1r6tsQvp9+bpLWxwSOddw4
vcCE1kOSNI+IiAlUFQxII8ZbDCZPhpoxOUAVsXXa0+j5/SY3+oZb9UGZPd2Q
Onr+k814uxs46ZSK10W4OvyqVuVNN/Awg8EwEOGwDxb0VBxugLlr6c99dP+i
Pv9I9YwnKLB6uY8Y7xP40sNbX2UJ+RrYVZZosX5WTAZv9rYg/Ohn8OmtnV7Q
5bVI/FWEG7s+Btzk94YThw7xGiJ/Yae66aXcNE8orK0jwYNweMYBU5fYs7Cz
wVemDuGVcam7Z5+5Q7Oi++3pxHjUpv/oAZ3txyRvoN+TEjL53lTiBZb0lOxs
EFb8ytom5e4NamrKxgaQ/XvpFMOO/Yfcwe9tbeqlCHef3KFxXdYNnDCil39H
5KOlS3MLsHEhn/d8NXJUPdHvAt07v80Tz3u3/PMnyPx5N23m5vYkusEsAgNV
GDE+X8874pZ5llwfxnfVDLacROu52Jh1FmHS+vUrk2fAE+NX9QQF6nAXoJj1
5iFEmOSvfBNxURlH8llowXev8BcPOH8Tx/Qc4eDFKYG/XB5wRKwWT4nW9/vG
by+UiwJA3DlelVXkv1Xs0IqmPuIG3acU1B4sY/1LSP7vqGl9VvoeV3gISmiy
r2P23yb/02+kLz9Kj77Y7gL1g/d3h6L9p/PP//tX74Hef8n5yX1XOxeyfKG8
+UyZv84ZajZkeOxB8oJ0ffeZzg97kX2fTv0wL6jQHuaZ1phqovWZdfFokNvM
cfgE/+BDJJo/Unz9bJcUe9pWLF+EJM91aL/Pn10x+MdPRqxnlvxa1PfcEtgW
cBcHDyF/1fG8bMs2O1i/JlbQg/y/IvnwtOh+O2j17IPuKNJnKjcS6KNkPUHQ
H0sRIn8LSf7Q1HC8kv2EB8dk6Rwhlw/M87BjYEB4MiTt0SOcD+yvOCsoV47s
900ekXgw+ZMjkRPhJirVZx9OxYPEhODKevT9OT+FmGgxuULhrrgyovwZFpv9
a83oCg+4cPNyoP830HvuZd87T7BRnvXzF5EP9kn6p3jzs6BIbpiCyP+Pny49
bJ3vTs7PK7pjWJV60x1QHaXoJeZnhv1b34DN+PXO291IXx79/NqVzRfMG/Rc
/4Xwtc10SB9gHPk2a2sVHuT/yEi84BJHrk9ti+rSKo+NA4mf8x3CEE5Hq8/q
chxQVssaiezD+DNOmdltI/IPkM4nSP0lwpNf/nbIdYAHuj66RKHxL8/thx4X
2ENZ1SNuxP4SpPgPaX29ffbV+2iRA/KfE38R+8ua5scl8P09Cj7IhPLKI3zq
tpDo+0Xpf/KViP8EhKypSgMpybesERwYn8ZXalV7Y4SFKAm9CQIh4EvuumAo
wpE7dQhy3qGgwuaBRdBnLH+l8WVZfjjCCQbitSNiPuDe/Yb+RIS9zomEtL7y
BZGFDTwFq1g88O9+fEDJCoZJ+iiR5sP6OaBMzp9oVZu1SMtWhfYT86+oVjD/
LlfJcox1FcPG+vbMxPMSUvxQt5deAaxg2DhC97ryTgK5nvS7sPx2L4QHDMvp
NsYTwMTC91l3hO2q706o4RKA/EGRYHaExZ/ej6MZSgTxHrkdhewEcn8Jkn3K
N1vRd1P+LOyx4rYmxjtesmZPdNuehfVRU7VlaH9uq0mSn7roCZPfgYbf6Poy
RnfOd788oTV1fEoMup43iJa2usCLzG/z7s3ceiiDIxzI9XmXg/zv8e/OMZeO
eMI7wiWTrl/wQHyLiNaMmTYwyDLqe70VyxfGxR8eglsxfcT3t8V6yzyWD6R1
YvTClc9YP7nIYxemaWYxe/jPeu7qLoTtl342mR73hLbye2y3Iiw71z9u8Mgd
Tu0oC9lOjH9P9Q93ULnBFJeRCTqE88W1MhxC3SHVdn3tXrT+MmS/XuKp1ID+
tGEWgwjrJC2oiIRokPmAduXWCul8yYUX1Rzvsamh92HL1V3mzoUfvfPnKeXw
gOHQrGR/ai4k5QvmT0XMKXkGQsh8b6QxAX1Pd8neanF/cjyskaKj53GFCjSS
+UVFS6zH+289FMSVZF39P/zOL0OygxqrB/80D4oVtmCYlM++EPXnk+hDJUjJ
pvO+bgvWD52kb3ZsCd99g86dPF+JJ4cFxAtcIMl++fO1wJB50QXe2cXw2Rzh
p1yCDzynXaD1oU/7b/0h9pu98YIRupD9m8sK7fxTr1xghiJVdgcFpt+8cw7v
KUT28K9i9fykUH9QY/T39epPjM+AtD+kZKzHX8/7AwZZoadSP9D8DferVR3x
A5vp4g9woP/vQLgAtRmUcr627S6yb6Vq4pfUK81gMT0upPIhDqh5zivmi5pj
/D3UV7rr+awh+4lFi8RaPCh7oyQxo2MFeyMXBptq8CD41M5ilQ4rmK3S0DSL
5N3HgWIBm+dWcCSs0/ZQNR4kR7Q9r/pmReb7++YkwWsY4gVxVJ3X+2mx+mrS
OFfw3dbF72eBdEhz4wN6ApDTOHVc44YatL3dNvl6Fju/4TlN2HcR4WPyfCbb
p7wh489nme0IP1XRNi9g9oWKCcN5xPMvkr3zR8sowwJ9L3+zpLaIJS84N1Pb
DxBW9abP0hnyggsXDucvN+DI/l1kC3WWP9KXdko/wzvQ/7mTTWG6hMavHfYV
v3LdDlr7m0gR+1F4uNNMDJ4yBrzS/Xvzd2H5QCS+DivbhQs1WwxBqu1AJyPy
Z81eCksf9T4Jc9Mopk68x4OIi292/+6JgyHb4pnX3+H/xUWk4qG3dvNj8acY
n/Cmv40wx4z5lUsJvvB7dM2JDuL60zgWES+nCf3+HhJuQONGz/cPnTjoCWS6
dd9LPMOT9ZUS76lrOyvwQJJ+9Yrvchww9Il8/rUFy+dU1ih8+vURHqxffEtX
mxIKuo90UuShcSC8/uE1ZSjIzJi4z1lCAPaWsal2PgXw7InMaWmENZ1fF1V7
FcDZFrbY8n7se6+VSLZQSxLAL7N26qjmSNj86sQLJmo84D/58bMiLpLsbyn8
Wj94gWk/2b84GNCwO/6qEEgyLJg0/43FJ0nj9cFZVzIkDpHvr753gbn7Nh8c
imm6WreGnV9N5KpTKiN52yvjNaQLpGCHLl9+MMJG/jZyC7pH4I2tG0crifFa
0yIVgQ4x6Gauxy+Hx/xjEUvu70pf/y8//Er2Y9spPBj24202YbOA1As1O0Ld
cSCaa+eAfmk0fLz/PpMxun/85ZB47pI3KDJ0fDqNcBmV08Fv73zAaNsreRMu
LP/RfVvohbGtGP8oKR6WsbCfu+yHHez0P9ZClOck/b/IKxFH5OMn5UuRrleg
cLfYmesC7inV6tIinB0mWL9X0wWoeB6OrSSeRzRO/v1zxYMsHz1PNDSsd7nD
KgJdwWNqzF4n/V69dOOYMrs+6HTyJXxEz9tSfvRjkI0BcFejFasi5mMaDCwq
23rCu1wpAbxo/MA/+xlenelOdN6O1RtPfrxBLYiwMzQ6UhRzDpZ/3LuFowH7
nl1VetPU9Xhwuj+EIXXCHCa4CP9Z2Ip9nzcHf9nNo99f3lxP1tBwvE569QeW
3xlsbVHd44zVm//j6ySAJ9+titZ4/Mn9r/cHPZe83ZUJOc2qlczScUBm0vVz
dGwm3DQDWPBA/m5OPd3uSHiwLL3XcTceeB65VTWdGAHrXrGcebETDwydjd3Y
HkVAnzzJtIhtaH60bq2UZ0dCipMBaanIvngRJkBxv04WRtDwiIwh+fvpqanI
jbPHYOT354/nEPaQf8D7IOIUvPoXDEYR4zlrO7WyQ07BoCPvqo3RuAKTe/oK
VIBaeRIG3vJY/H3T/j9JAIuHFznnl5VBpO+0YNURLD+elN888q5KzO14JNle
5Rlq+auD9uvx3rvFHQiT9vfefaocl+uQvKM1anjlFAPoWtbNhRA2nhRzpk+M
Afu/iX5QRvY2JSVh+47HhvCJzGLm8TEsP40ULx6id9He6WwK7e8NN3WxYuuT
xPdUb927dqfEDs75mVA8XcSDUrcPOzlSQ+H5MO2q7avYfOaLzipdWsXmk2Tv
peZRxi49sIWcv62uE8+/SPq2cB/L62CEo3fXLW2fcALHohxkXyD/dy4ybSou
SYPcjyPytUKMXr4OvJ2ssTtkCrMnZfks2F7TY/ZQbPrXCGsi/82WNKuu05bw
6LZbwitoPG1mNoBqwgzqdiTGyCPcdMCF/oioMQwFRfHPRjH7iCJVI9kC4SOy
9R8aqD2hdl59awXCnOfe1c+zeUCFkSvnBNH3m0nSSPpQ4QbbfxQnsyDcVinY
tjHuBisLlQbV5pD+jwo8L+J1nMyfEyWydLldQZbs7/ZQFmacljoDNUV0KLYj
TMoffM+NZ5z9P7z+tk9ERIwAHiS9pKcpioD4A8uFH4j5q//kGdQ8+GbATZxA
5rPZDD+i66nmM3U7xiL+8ZEQkLw3ut524R6AY61Sv2TZCMB9hmWv46Q9PKam
2GqKsC3Bfd16lwMk8QEMO6V0aFZGw6iB74Jh0Tjwa29TQKFHNIyhDJxquowD
i/R1TIa/omEODff0qgQemN5itNL6mv5P3x/EA2Yma+fnohmAYeBUej+xXmjT
nrFHeoiyimi/BnhqgFITB9DlJPG9DeF7D7Nft1o6gIhPsOrBLHZew/iTx8ML
Yfkj5UI9Nk7wCtWJ2jqET9rTpW9D9vODD4OjzOj7KqWddCtNs4ei7HSBL4n1
Y+5jnMNeDpDUD+vXM3em48flYOFF8wuBQkge1UaI76hTgOyldDWZNFh8569b
mqLKKBYfvJPPpsZPPK/7b31H8CgsfaTE7BuSvx7nIHT+y/uzoGnLfgKRT3zP
LsWv6Rqc8InfAZtAhJU9KGIEjnFCgeMOeEEq7PzhxN6KwjGk33aWxNhRBLiA
qFG53US+edI4af/wZt+9fb3HGWjY7l4j1n+T+aX/m9+IxqtjwTbIXrcUPZeO
5M+Df+dL0GY6y8QU/d7+luMvOg46kPVr5m9QOuZgD1df8ZjTo/Ev/+YHnucU
x11ex74/6bziYnpjjJ2jA8wrE7H3Re//jIPfykPDgXw+4TAu/8ivCekzKe1L
dQiTfm9e5UrgAiWW30D6XqKT6g6LBQ7wlUFXZDHCaU8uRy/PO0MBr+/T97dj
9aQ0fXPPcxEWotiRPLrgCxgPva2gRvtbLv3TRBz/EXK83eT4Oy7fKmPAlJE8
/QSH8TMmr0WVxqL3vy12ypTBSQo2v/3Aw41wQ9CTiq/njsCZumNX7qHvER56
dTV+lzg8+m39VjHav9sCosV72i3I9fBGCkFZaRTmUC7GI+AbGif5Y1P2ZxUE
GQnkftO9bxYAsZ+YuXMGp75YMGxmO15hjMYlhxj2/+0PJPOzLZ8P/XzKOQ9S
h1ebVC3iQOe1SEO78Tyovp2S8cs6DthTXA68cisP6s9ocV3kwgPruFPCCm3X
/utfgQcTtQwnRI5EQEfm46/t0PNI/kLNt1dq5xHObMlV6DnhBw64fZ2QQ1gp
6yJdsY8/mc+B9+9POp1hJSBF6XL5CzH/XWQw+vSYAghXGnv1g4hpYy47bSiA
oBvfSsEOrF5xc70hfDA4JdHwuQo5f9b/6YU3c3N+MHHh8qA7sj+0zveYf1Xy
h+3Lh+hUt2D9rEj+1Q3Gvcd2pfjBzutBlwTN8eBQv9LX84ZXIYmfmzqVMzFj
NQfaR5TLnbDFA/p//SUho5xOr1sgHoydjxreuXQF7mhkHRCPxpP7Zfw7T0b2
5fTUGINrDtx+39Jg7AWy1056FTjNu0BSf/oGL2n5o31OMCFMMdjtJXY+t0r8
PoPIP5eQHVq740bm34jS5v9t7OIG5w0PVC8gTNpfIxIBl/e5IHngSqngcCsQ
GL7v1ElA+0dJKiq54K4n/NDqPZ30G5MnpP1UfIdQGeLqCd9wuqRYbmD21njF
vkOLV/Eg1OCPTuXJBEjqV7ZsIGw16J0EwyIuJ71B+uvXrYCp6pEgsEkPcA4H
Zswba6tLC+A+UybjLwE4oHSu2fN0YwGccGn2ZAnCgXNF18Vf1BbAFmfln27h
OED/91DvwcwCaGGBMyDy1/gOGvK+eEBFtgdE45gCXX+8lxd81vljzzwp/ywA
BpWdj3qEcPIKlXmQRiCsfkXRasOAxaMPPisKUvyK5Zvx3yxJTUb6Wvpbfr5Q
qh3Inu//vboPix/wvonKMduP1uM0gVBm5QGaXzxbfMtLAB3Zodo6Ae4gWPt5
Bx6tJ8YUc/Pdsx6A5L9HCd7o1PHyJK+/noAN3G97DwDfK8fJbMHO817gj4zt
bsTsHT2h9LfV9/DAzN1D+8SQAbyW5atyD2GKvuKr/dUGMELTdu4Uur5f7pt+
/klt+GqZ4gmRT02ooLVE+ZsutFD2FuZH/vr2ywV64xru5Hj7StLvocdVbuBi
2sy32kXsfJ90XtdV8sZeVMgD3G86u12UyL8ep4vndggFgaEDxXQfkb70M7mp
GeIPDwUaSFYje+LO6kUlcVVXGJn3KkIcyXeS/V1qNxO4hOTbAieNfduRYFjH
bcDUjfyRw+4cW+89DYb0nppW2sie/zy2c/uh9nMwm5NDdY4L4/+ldgkIc0P4
2/dzlGuBPnDy9pb9rch/JPUD3axfRbgUrsR//xYH+OrDbY3NCaDV6sxBDWMv
QHnH4+U6si+mWzVrDt0qIPejnH345C67UQHgfHIxkBaN+22muxSQ7dlpG++o
Ld8LyP5ejGY+pVbECbI+Es6/kRI+Lg3py9dctH9j/LQjcgerkn9h+G7Y4N/U
NQx3mIQbm61j+YCk/TX4pFle74EwGTMx0D+6v5efjMtK7nmfoN9Dxo4vtdeO
PZUk4687+e7nXhWHd9hmpKNl0H67rzt+5NJxYBxgaeiP9l/7i3Av6fBgIPGC
MfTdHow/iNRvXs//cNjtRCdA6teetnsHc/5rF7DYLKQ0txerj3Fxyes1H8Li
856tmsszCCd/iVq+XJUAAz9aCY28Rfb7jcljJUOJsPHYFFPAG9J6OA8DKt9X
+r9G10evDH2C52HvyuTgATR+ueTj9kiN8/A9q+JHOnR/n+/acHJGEhyVfj4+
jsbH99NphdckQVK89mTE/eMnP9tA1luHnKI6MPudpN9/n7h+Z8jLH0akfP5O
jD+YEJe3mT/5fItP84yVllsAYGexbaKrwYPozXzdWCDzNv3SAMLcW1dsXynF
goR8Tq/sbDyoTOl+5aniB7WDfNXmuvFg7U2WqmyTPzyi+4KRuhfL14mLextI
rD/1lYhlf9S7C/yW6BrLm8YDWcO+mYYDu8j8HSxl0e6ULaxA/Rd772Vkv00N
VVTiaumB4kmrkudoP61UuLWc9mYBShYBM17Iv2hJnqIuKdoJ9KLVlNs+48Ee
nSnujRds5P1K++fglQ5WBVjwkXNOBeHzm99DHr5lkqidXMbyOTyInw9hi7Pp
pWM75eDVuI0BfU5k/72c40s39yLzVR6xeszR+NULOvq0KF9B+P2/fECYSrTv
dhPA1jLpz5FJZ+HdWHGT0yM4kOHJtLRyMR+ezjUIjvmCI/MBkvghzopfCpkw
yifn/1S1T+J26uZBUr/fDhNR+S8fU8GD6Xe/y5qRvU117laOYwqgufuGS7KF
AA48NJz4OpYCeEs7V7RZsfX46WfZPj0WjN/likvVwkIEgdwPksRHI6Fs8yKa
IRhyfcnUckT++lUjimMMDWFQi189yGUZO5/KS+2kE7HH+CRI/rxk/rdnNvWZ
YJQgrkfLQ+LPzwWXB5hq/ZB8GhVt/Rp7JBcc1mZI3or0NX0qS43FnQswOAlP
ew1h3c2GABfg9aX451K30P5v4Chi69cj54s56HYFn4s+A3+0jHMw3MLqs2Ki
uvy+o/XikEf1/qa5EzxrgI85i+Tnx/vi+yh1beGx44mb9S4CO+8m0k/aQlHY
jKc5hPWTI9UPntC/p8wbJgPFz7e9VRIgABXCAbfUKV+wq9b4WRY/Zr/G120/
t8aN7P+eFT6/G/ZAYLfZQAsPVs+z2W8NjatGnrG+ZWT3T58gfVRuK1lWy2BI
jmdbXb1y3q3EFB4P3S78DGG+f/YtfG6WNBiI7H3xsHb50Agnsj5S33L78MMF
J/hVXLp9aBGLL1oV+xQeIvI1NNc8Gc/3I9ebBzNSbj0s6gfiUr9/1EY49l++
PfjIfsi/ZQtmX5LsN9l0gt1iux+I7LrBOIjkgXDj6SO3ov1B/KTWOXlqjM+G
pK8lLXlKGm/4k/X1wSVbcWo5f1BvfkyVBWGrf/IE2M3gJdVdcSDnxa+jz3EF
kPUXc0idGw4cp+w8GDBTAGNvv21levN/9U1KljQ4tB6ELF8KCR93gmUxFqcv
I3loVG3/xc/DEUZKP4wRRPoyJ13QOTgjiMzvUV0XN7rncDB80mlC44pwOzFc
9zgIkuKfyxuifLNUQfBOxP3xF3SY/9d5+1jqfYRty/tNfrZYwlAPZZonCMt3
RH0sDbSCsoHxE58QfvGxcSV0zhoe+sTB+RFhd5P8Hyt91uTzqXfjPePcbzNA
nImgQORupG9DKzIzizIASV5IXTq2ZXT0KmiXn2k1Q/YV29qnYOuHIVBDzoe3
BOF/6z8EkvhP76io7x7Ntwf3n5VLZCH5lm165tdIsStgmbw5tdKD5R97Huq6
U4HwjUc7DbMdvYEk78H5NoQL945HWgBvIMGs7hWC5K9MYX0nG6MXSG4OsLN/
juwnOf4e3AUXQOKnHKPVfFkk4QpSY+iffu/H/NXN/LMB5B/RbFkZqXUGAi7v
Wp5+xvJR+B9c0Q1F8mZksZ9h2U7rXz8EJG9UikVftvJogA85Gn1f2ZH8fBFg
Z8VlAN7QN5wg8ic0yhqmsIXbgJe6XT7/a+ra46FM9zghxtm1EqV7UYRTymko
0lNI1rWhGJdY13ItcimXXR3KUi7NNK3sJEm5ZkOz1JRHJcm0cm9qUO4q4x23
0Tmn2Gdy3vftz+/nnc/MvL/neX7P7/f93WwRxvMr9xzVDeiowYDN6pyoK4Kj
UMEIUN7cw4j4yg+fT+ZK/CEnf59poXI2VKPzU0yRfkn8fM01h3MZ5nYfoN9b
j+4DukJloUcmVKqIDpDOF4GCk2kq079mwrzS+5ZsK+S/SP18+iXnPBDndM13
od/H+ytyzDiMXMl8xsCJRVz3ONigUXI9QxsDI78FmGkZx0G8/1YQZZ3b9zNM
2HLt4nt1TRE4tILrr58VCkZSbZ8NVmNgSlmp/1x3HFxWH/f6ExcDezlUJ6z0
NPTST03RmETn92NniLyFH2CwfL7E8kj7Mtf6N++mHgwsOrXkdc1sCCzguTMf
9pHxo6vBVemS+w7/v4PWbilO6HxPHGayq3nxxDwNbHOWr+nFUMB85KzRDjEQ
k1ZRLx0UA/spvID/InvrlMkdD9VjcXDS+2HFWwWSv8iUc4miIvmuPF5wcHKj
Pax9/SYAQ/Lp0O55ZRTlADj3NUrqKSJwYqH+gOCjJ9ZvEarHU6Hn+7vsxUg/
bc90FFntB1Al66q5CboPy8P+0Si1JxhWJynTJXwVzhfi/p1r1c5hZmQ6ONuq
0++F9lO2RB1evgByNPVuLn2CEf548cCNaEl9Is5HOGupM/bpioh5APj78yj3
S2tyT4LJmS3SRchfx/2dWv6TuanvSf91+h1HQVqJzLepHo/xVlb6Zr7G4qwu
A+QfaB7YbuKqeIKQ7/TWnUlpXhGQ7p+3nYX0dXLVc1kthXA4V/nycyyybzev
ueuZ3BsO96cb+PQg+3OiyIaTvSoAtiumdmqpkfoXjz8xcj8kedzeR+SvX16/
qbYpkgHaf7iyU+LftLTT/ni0LRcyBdPwMLKX/qf6jMrX+R3Wn+VrzCB5Nvqo
VmycCQFSDXZDKwsxEK1dAwaSk0D22fOWnmi9D7WrdAfrBsEVRyxGhH3kfX6X
ReuvQPJ++LVP63mwqkDbPh+d94iFedtwrDmmO0xKBLhKh34dqouHNvbZNtsK
yfhASYDYuh3tR15On1vdtoPgT8qZSgOkb2+3NibOztqBZ1Z9KQ71GGjQizje
aJsBL1gK9E3kRcDyZrWMYU8Ysf6ph2muzmWpkBb63kXdCwNysaN7WcdioJWd
omkfWv+bqvrN5kURoE5tY4SVkMzHdvKlV+Uh+Q9HpoyFqu4GoxZu9nafkP1X
vDkTOwhAA5TdeslBBGw+pl+nZPqB03NlvTNov9ZEJhqYGiaAI+HcnhBkry6Z
W5qnX+QE2Ql22jlj3/gjgbv9XknqtX8eWTPu6w3q5t22dyCstux5YOslb3AJ
FNjGu4tAoQO716iSCVVWp1UJkL/mxZx9MKAYDu4nGJ+bQOtLX9i/oKu8KSEI
yVfKxbjYkMOGbsKpvWOB42Cw4vkTNoUJz0WnTssyRKC1yNYjpDFrIR6rLQLz
u77MzTaxiP7z8MORdcLodIJP0HJv0mnRi4CVZgnP2t+Q/iXO98vOte24Y78H
2p15dCztJQaW9/HooiA/0HBpQ/wvT8bBqdgb8laNTLDbQ/qnZnT+tW85OnC3
RcNX/y7PFrVg4GOwTKfc23/BTqcefQrSV3x/r+J8lQPw3Hfm3hDZ4/ziflGf
oxPA5xuY83pmOjQ3QDqvHDuD7CH7UzmlyQ0niPyQvKb+/aau5sA3X2vpDTFZ
n678o/71yVkST/YLVkvyR2bL4NXOcQBklnRfU0efb/Ix3Lk8wwTIT1xZO4yw
i73QZTjUDM7xuZN6CEcmNYQWHQIwOINdIOHvFC2MbbjcMGDjLb+nWEDyH5uj
es2oAlK/KIqvWJpaIP1mMfhL5e+p4Ku4dUTgqYJuh6AoFnp4VFqrVZP9jfF4
/breArPTI37/ry8cB86PW2bcWInE+1oknmgW30wAcU5mQj4N3S9lcrWK9ZkL
/B3an2Mbp5x6X2WAucdclh96zvpqXmcC2pYNA65NZDwdz1dQWi7meob5Enzm
g3+q1lZOHQSLZRzzPyGM9xO6W1ruWtZH5n+G+nt7qoSLQMLZD8XdJQwoFze0
NXyevF85MvvaJPE9/Hx5rskJF3xH8i/DyaMvdknmvzrc+qBvGwBHY4fkHiB5
31qoZwbaNhH+x9H7qg/9p4QS7QfaPOKsxGJJfrN1Kyb0BaW9T7tZGBlfOgxT
a2SQfmo2lZrfr0YF6uO3aSkI11W8eztxwQgsf+GoXDJLztv2VTfolmCcnzJ9
SG2LRvaoxjLaixR3Kli7j8+gSvoN7masHP3LCLy72EExliXn9+L5yIsEZW9P
/rieqIe9Hhe7V/hOk3ieNhUyT5PeQODBrtr4UPPVcAcUq+1A32e5y7CwS8qG
8JfLn9ZeuOOoR9wPgTor5M6k6EHBS2mqwReSz8DXL1/mXnYibxOBdZ0HZKIu
axD4J9qq3rbHa+FQ+UBV8Dfzc/H83AdqLVtdazYR+HHBQJNjkgaBzx8d/yub
vRb+De50Lzg=
    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwlkTlLA1EURj8RW4kimehktxEEEw0qGAlYiI0iDEJEXDAupVqGlE40PyB2
BuPyH8Q1ajHRSsEFLELQP2BstEjlCRaHk/fufd+9TEKpTWujSVIa9julvpAU
gShkvdKIT9rBMc4D0A8F+g5MKW9Ia2HpmvMqLuF1fIWzQancJfV0Sw5+xW/w
Aja1XXJy8OmXPqBK1jKzUjBF/w09t3AHSea3UjexFyap+/ASvYuw0DD1bXJs
GIY41N2SEZA8UGffWY/0zLs5fMoeZ1CAGfZ9x9PYghaIkTnImzhUqbnYw+G+
g9wHnCEnDRVqY3gchrhP4FF6EvDEuZd3zew2wQ5Jsh6ZXcNfUOG+jft2cMEv
/MA83/AQH8EWnMAxBMhbwUEcIT8K5/y+J6fG3G+45CxmXjR6ydmjVjT//9s/
SQtIpQ==
          "]]},
        "0.406`"],
       Annotation[#, 0.406, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwl0rtL1mEYxvHbUIkcRIMOnkob8hRlKOirRoMaFAS+b2bqoDQ0qBSVIO4q
/gFOOogO6uKgkCb4eqhIC6LAwcHFpTaHQCG3PtHw5frdp+t+nodf6fNXyZcZ
ETGN19ciGq9G5JdFJOiPGxHp6xGbaFBbvhKRS1dogg6URgwiWRSRwjf5Fvk2
tOLc5YhKs0GbS6Cnjm8LvY9D/sN0BLvyF/V8orlmynETFUgURtTibnFEpno7
HuPAfIpO0w66TtfQ5Rz7PCfkzsztOcsuzi5FnJf/Q5f41tt3wV1z0K8+Kh5D
iZ5xHn32jdG3akN4gyaejXRV/qPeh74f4bt4W7yFNN6Jn/J58e9sdNG+BWTa
lY0sfNX3BZ+9z6yebbkdbOFeQcQHOs8nj8eiuNxbV6ASR/pv0WpUYcq5HvAZ
1NtvJls8Jz/g+8TeU/Ty60On/DN0YZLPmlwz/1W6jvdYMLehnsYd97uNNP/f
+jdoj3w3dszVuMOxNw21n/pm7P5Fn/BJIYk555pFRtn//+wvNh9bDw==
          "]]},
        "0.392`"],
       Annotation[#, 0.392, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwl0j9IlWEUx/Ej/REdGsLSW3bNgpbSslxscXDRITOHqMXAJQ3MxUHrBqWD
OgRRdIeWFG42ZGVCkRZS4VVbQiOX/jkW1RARRRHW56Xhy+95zvmdc57ned/K
9u7W0wUR8QQ3KiK60xEd2+nOiObSiE7rQ/TM1ojPKVoZcRYZpLdFpMoieq37
8BJLOIwWTMtl9XlE03jFv4xmvT5tjjhezos9PFXYjb2oxuKWiAO0FvuwHzVo
cpZzevx0rh/4uiOin7fNvAW5GesM30Pr63xTdE5u3PxZeovm6W06T9+4U47/
LS1Huzf4K76Ki87WqP6COf04j1n9R/UdQV7uF/8ddXnvdkRNK76oW7Cfxxzu
yv/mG3Pv7zRnPyP+FM8wzN8mXiJ+gm6i15xpRa5Lv3fmFJlXjCW5Kufoc+9e
HJN7rKYDnTiJap4iPcf5CulBferwXG5Q7pL1ZVxJZttn3WnavKt0wIyjYpPq
uuyzalrsB+k3ZxkQX2Nfw1eLOnd64VvuMqsYG7AxuTdPifWf5NvTjLpl8Qa6
ilHxMTpC62m9XIG7lFo3mFVGJ+RS9J7cJJrEG5N35MuZu2jGe/FTyXcUG+J9
Tdc7VyEmnH8dXZv8B3I35R7w38dHfT6gR59cxf9//x9l23qu
          "]]},
        "0.378`"],
       Annotation[#, 0.378, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwl01tozmEcwPFnXmFlFtrBmNmGuVDMyK42mzmVQ3axqa1kF2ubtjnsdUjI
MVtiRBFm3OByCDmULDltpIRY4YacEiunl/J5c/Ht+zy/0/M8/3fLrmkub0oI
IfxCXk4IH8eF8Amf0ZIRQg1H+ZXceD7Dufw8NYSsMSE84+W5IeSIJY4KoU3+
He/ht9zKnWkhrONb9lHu5kfqb+h7yNdHh3ANscwQXsil8OrsEFbhsZo1YgX6
IvY3ndlofz7L/biL1/J0+SLnXJa/ghVid825h4jZFeYM4Eruc+fJ3vWSc8e6
O5LVp+pLwUX7C8iw3pIewlLezE3OakYEM9XPEz8tnuCMY2YHvi9eKv5E/ybO
dE5MzUhOw3DxJE7EBPS6Q6HedN9hu/qt6PfOr9jtnMM4hGq1XXJl5u+1b4vH
9TQ4sz7+Trla7z9k/gH0yvdgivqY/Xp3iMbPjvf5Bj14gB3iZ521k4fJRXmI
eS08yMw8Pa/dMWI9EPnO7PKtp/J5/sLzzRmh/ql3JOODuvc4bs4lTJJ7I/7X
+g/63emA9xxEO76ZscCMOj1z3KUMs3FVvEP9CRSasU9NTKyc58r3cqv4OetB
GIz39vt5cbyel3A71/s2R/TW68m33mDmNG6I/32J17pfn1iD2DbrYusibESB
WKPYT+sfKFa/jGdxCfqcsdC3W4xFSLa/I1+ipxTDvPeUd3biJJLsf7vTUK5G
hztU8VG9t613OS8j/vtzp+8/h+vMrMJ3c6s5Zm6N3Ay5ErFSrDS7Nf43wW1c
KVaBbr/TxJz//9//AClilAM=
          "]]},
        "0.364`"],
       Annotation[#, 0.364, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1VlsV0UUx/GBGCkKsSwtCIW24gOIgQdIoIhiBISWslQWo5HFQkRk11IM
aCkoCFZlKVHBFhKhUSkoiyyCRl/coOwKQtkhQguUnRpDrJ95+PY3Z5lzzsz/
3tv03KnPTakXQqjxp1fLECpbhVDVIoSpbUJ4yXpYWggjaXbbEAaijVjnlBDG
4X2x02KL6Bm63p5yXJbzS+sQfsWVaOMK38/sMfK+aRfCy3QUKuzdi32YJm86
XsceuTX2nLKeYH1K7kL7qvm+4luHk3y9+U7QPvT75BBuyc15JIRlD4cwhBbT
o2Y9gln2LmcX0kr2XDrTvk/4ytHE3iwMQBc1B9JsHNarlbs5FmdyB5U0hX2d
vzU9zh5KZ9Cn1TzH39B6JvLRi2+nfrswVp/b4rk0UexJHDL3hVR9aFv1q1GF
+9X8wHyXrGekh5CH43yX6F5nq8AkteayCzEHteYu02ctPscaLOC7IXc+vUl3
x98O3c1V49478LXHGDNlqLGav9T6NK1Svxo/mfNHjOabx35AXkMkYIm6/cTO
mj9d3YVmzGT3xwG5zdjV8i6jCifZD6kzSnyQvWfkTLGeitnWs9AHfbFUPNGe
JmiKcXwDzXzdndxAJrIwABfVXRHvi3bFDrld6D9qLKHtzNZM31x9bvG9rV4B
Oohtl/sYHWB/NrJwJz4D9K67SBXvKH6b70/6t31H6EX6OB0v/ipGiI+3J1mf
Cfol0cHsIRgUa4nn0KH0FfnD6Nfs4fEe1DlhtlS6VSwtPmP8i503Cz2t+9Mn
aCYdImeic7+G8ex3+bfE94D9jLl20WJ2vjpl7OaesSS8x06md8Tv479L8zAH
BXghvr9mf5EWsvfI3Y1Z7iFHn+f1GUx/wzv8RegtVkAXYT5u6lEQ3zG0kN/N
uSo8H4/ak4BGSFQ7mTbFv/JT6B51Ttj/Oy2xp87+lbS+s+7j689+FpPl1EN9
zFbnLYwwb3f2cDrdubvFmfkDraP91C+xLsUk9mRMwVb1tmGS+qvEJtLVtFz+
TTN3iu+SMzRScw09zj6GAn3y+T6Wf9d6gxlraRHfOXn71T7IXsN/iB5kH4j3
KbZA7R3OtTeu5R8WXyuvgr2T/zvc0/uq2mXiNfQKruMa0uO3wW/3IYqQzf5D
/kb7NqERtmBzfAfV3ka3ozG24tv4DshfR8vxIDZgPebILzFnKf4zUx1+kLsT
K/X6DCswUs8S2pPWyh1NS9mj6Ebn24Ras3Zip6nRmZ7HObTWI4c2EL8Xn3Fn
zGB/JG9x/L7zvRm/zfJG+C0+pV/q8QW6srtgmVkXyl2EFtbFmCevsX1v6J2H
jPi+oAfSxC/pl6RPG3uSaRX7qpp/ibVkX4u16FF2+/g+xP9Zap6P3zzrwng/
6MF3li+DTmPfwW301fuy2ZPoetqcbqDL5RZjlfyN8fvOl6DPZr4G8ZvOfirW
YV8wUwpNoPn0gG/zQezHWHmHaC7tGP/P0P8BH+Q+fw==
          "]], 
         LineBox[{6050, 7733, 7890, 7732, 9155, 9154, 5873, 8361, 8362, 7891, 
          8552, 6100, 8668, 6341, 8666, 6340, 8667, 5886, 7258, 5887, 8649, 
          6318, 8648, 6319, 8650, 6160, 8340, 7713, 7075, 6049, 8313, 6050}], 
         LineBox[{6059, 6259, 9106, 6258, 6980, 6979, 7626, 6161, 8654, 8655, 
          8651, 8653, 8652, 5900, 7261, 5899, 8721, 6456, 8720, 6457, 8998, 
          8999, 8319, 6850, 8997, 8891, 8892, 8890, 8894, 8893, 6334, 6440, 
          6439, 7317, 6083, 6851, 7509, 7508, 7507, 6020, 7562, 7563, 7061, 
          7060, 5823, 7680, 6102, 8267, 8268, 6723, 8373, 8374, 8375, 7260, 
          5898, 7252, 8985, 8986, 8984, 6835, 7578, 7577, 6059}]},
        "0.35000000000000003`"],
       Annotation[#, 0.35000000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1GtM11Ucx/FjympzreWF+P8BuVkuL02tsNkT0y6rFZCXBrYoQ/RJ0VoP
igFqaYtCvNQqNcU1LTUtK0CbyjRl6GZu6SbVzEwu4pxOUZehYr3Ogzef872e
7zk/zj/n1TemVwwIIdT7MzEZQn9OCKmJEE5kh/AnTmI+f0FuCMv5n6MraCGt
uCeE9SNCWEwT6hqsH8gKYRwOqxvB35oXwlh2gdgMTEcW/0w6C4XoQhGex9q0
EM7Q22g3HUh76JtqVtp3nb5rcYn/kj0v0gyzPCOWoOlI4gOc4+tKDWGOGZ6S
dwp/YRB+TQ/hKA5khrAf91uPxkyxDjqeXuafEOfAKj266Gx3UYJVzlSs/0tm
K43nNl9jhrPTl833Ckrxltw59F46Rb/76Fa9RtEX5S7gW4ha9pU4N03wl+mV
RsvNcgf242ccQ4o5Kumz4u1qC60LomKI9Tt4TKyaXaXf0/Qh2kcfpqniVajG
MBSbZzgtoUNpq70PoA5T9ZnrfOVY4r7fx2LMc6bz5r2Ai/F7oBd71LSg3N2s
128uTdFjoX1/ZC+i1ViAH9g1dBmarJfTsdim5xj6LX3SzPnq+6yv41/cxA1U
6/26PSvwGq7x/YOrvnmT79OMRuzEDlTK30V34ycUqSlAUv932e+hBrVYgl73
XOMsi5Bqnnx3c1jvPDW3YzDuci/D6d3ok5+k6TihPkHTUK9/Ns3BLf4j5si1
bqJX9f5Nj3b8wn7b/Flite74I3yIP8R+x3F9NslfKqcedehVsyy+TUxUd5n9
IB0U35g77Tf3DHqL7naXu/CoOf9jz+I/pf9B9knaFt8HLbLH13p8hY3xHesZ
9M+kI9XsU5tL99K8+Lb0/Fv8NMrkd9IO5Is30mY0ocfdFet9lpape0R8EnKs
W2g23UPPO+sws7TSnc4xLr6vuOarktPMt8W32EF77H0Wm9mb4pvWu8EMUzEt
vjv7jkSJ2V9gz4vnpuV0tp4lqFNXqVeLPpnWx3GnvdrYU6y749sR/469HV3s
febopOfQbs9S9jQ1k7FSzsf4Uu06dZ/E3yx6UM4htGG7fbfgG6wWWyF/DZ3k
/vMxVM4QLNWj0+w3xJJyrtMO9mr7HBG7yf6ev5+eMcsX+q3BGP9no5ErZ2/8
HYi/n9ZZYluxDU+Y63FkqMvEBjmnfZ8ONNjjqLpjSFH3qbzP8Rm6nfdKfBNi
/wPFkfxW
          "]]},
        "0.336`"],
       Annotation[#, 0.336, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1FdsjWEcx/GnhFoRIXTR9rS2itbe64LGDonVitaebUIQF6gRIjrcUDMh
0QoJImJFNHGBEuHCqD1izxskEoTPe/E9v+e//8973nNic4onF8WFEMZGH8kh
HM0MIY6OTgohISOEXJpIPyey24Xwiv2dZqaE0B5Jzono2DaETtjCtyIhhK18
K+kN9k00x1e+r3y1zufbhJBv1lB9t/Nli7U09zhfK5qjVzYapYYQjztqgtzT
8uLoI1TKfUg7i3XCFbF/et12TldTbe+YXh/SQ/iITyhmz6JptEo8nY5iN4iF
cE9dM/2+6dOU3mUfpxPs8du+U9kNcQonkWDWxLQQJmEeey6eqO2uJhfz7PdZ
XZkZX+h89kDn8XLGYYz65Xyb+Y447+Kromv5TvFtdIdZ7BL+jSjl+2HnHfQn
7SD22t5vMNwON9QNN3cTe0R0D/qJ7nO//fhjvwL2QbXxeuyjB7AfS9xxjX79
9RiAfqiTUx3thGK9ilBoTop3oIAm01J1STRLj4W0K11Au9FDtC9dRfvQ1bQf
7Ur3qltHL+uzno7kH4Eydz5kdjmtwHV5dXJqaYm8i84baE10X3e6iTLncpRg
Jyrwwz2b2bsFUtyjl7tf8x3kRDU0We/myJM7A7/kv6Cv8RjP8Qylnkmhvebg
L/uC2f/oGfqHnqO/6Vk6xYzJ2GbGAzOf6P8Uz1DfHo/0WuwZDol2N++lcy0d
K/eEO4+h+fIeytvGvs9eENXrmSGvLophevQeysnzTOLpDHY8anAZlfx7sBtd
7PVCzRL+pVikZ567VPHPpNU01aw0rLf3RfH6ZtZDHIbwDUYT/tnsAhSiiP1c
fbb578XfIZOdgSzxblGt2Fw6zNyrdujBXub8Vm4+/0Ezp5lf6XlMZU9DLPr9
Rn3oPXl30YbdOnqfMEif3WJ9PLveOBb9vvV8jyn67BKr1qcKPdk52Ot8S/yX
WI36VOc0dPd8snDJ+xCL/l9QjrLo/0RultwmdmyKxjhs1k6xCvwHMiHCvQ==

          "]]},
        "0.322`"],
       Annotation[#, 0.322, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1XtM11UYx/GDdrEV6cypCBaIoNFyzVJaIZpy03JNbMsgbJVONwRchtJf
rYtkLVuhchGjf7qQtyWXIenUUv9I2SptS9m8JWBWTstleWu9zh9vPud5zvN8
zvM7v+/3R9pLVcWVCSGELn+GjAphSnoIU9GRGkI72jBzTAgP3xvCpZEhzBob
wpfjQtiMVky0t4W26d1l3asmi96PGr3baTVdhZWYpj8Hzeq389yGLWqqUkKo
RAXGpoWQgmR8ai9D/XiMQzo2yd1qxttxG9J5let72xyr8RZOJYVQSy/aO239
0egQdvBrRxsuyWfyum5vvbp6bECduiY62BkNdBBtpC20Npk/rum7SzyJJtJ6
uTX89okzrDOxns879FX5Pvlquibu+bwn3FG7/Q3xnulme43xXulnasarycAi
ex+qWUxb7dWraUCbM0fK76Cj6H49d9j/1ZzncQ6/4zesVDNAa2gfXUX76Rv0
pL503n/G+6W/uJfD9Aw9RKffF0IuZpjlZ/69dK786fg5aK++563LcEZcSktw
PN6D+JR1XvR31gT6nHyBdSF6xLPpVvMX0RJ7pfjJeq/cUXpB/490j/gIzcMB
s+fTrz1Lu7AbC333iebrtt6JFeZ/BefkKnhWRm89S/mkyKWqL+MzDbnIwQxM
jznPw0H6tPpvee3HTXO8aN5k/Z3yj/F+HFt5neW1jSbQQehx1mG8rP6Gmnv4
XafD6Q33VyO/kqa513EYj3TM51Ejvwr/4Saq5eeYZa3eY9Yf0B5nrDDHYVru
nAXx3THjcsxHMfLVva6+gJaI11lfMcNlZ9+pZyjG6Jvscx30LD6k5jua5MxE
lKhfgH/iM0Gb+BynjbRJzwlnN9IrtEtuCn0ELRiOEdinN0fPXnrU2efVzxQ/
43MuN/8T1q85qxs78ZV8s7lmqJ2OXHSZcYm9WfY6eObRy2b9C5Pcy4N4Ek/h
Tf7l9k/wWGK9FMvEDfor4vtFj+hLtc5S06LnE2TLDXHOo/EO7W2Mv1fmq7Je
ra5JfIu4mSbQwRgU30n7i+N3zWOevUO0mK5Dp/06OiK+++box1Vxt/ps93HN
+mz8HXF3U8Vp8lPoVfFJ38MwWizON3MBCjEgLnZn81Akniy+SP9V+wKynTWa
5zfOTqIL5TaqLaPNdA/fQjPWigvobBThAp9n7f8R/dz5Mvc8kVcp797422Hd
ar5+Oie+sxjg3yc+QA/Qj2ldjNUlWOfge+fN5fUD/ZzPMHN9QdeqeR8TnJWJ
DAy1dzcaeHeIG2kn/Zvn7vis8kvGA/yy0M0zRbyI3zHf23H0xv8xajP1vGvv
PQzEuVEj9z8knQ2n
          "]]},
        "0.308`"],
       Annotation[#, 0.308, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVkDFLQnEUxU9g4ZQJ6nupSaKQTn6CPkCrS6KIILTWoDnp1CBatoQOihgE
oqCj4ewH0ETU1pacjUa1fg4/zr333P+5j+dP30VvDyRdQ86QimeS6ZcMcMGr
W4r5pCBe3ystz6UFzOHqVDrCTwakoUsK4YfhkFmHmQUNk/fL25opVaEOY7wG
WmD+yX4erZLVhze8HvpNXprapDYgS849pMiccPsDppBh5oEn6go8woqdZ/QE
tcOIrJv9t3PTRt8l9xiN8G7H7UvmCWbv7LWYD9A4/R93dxBiz8m/cEAT34r3
4JG+qMvUP2RcsLNht02/RV/InEGJfo3/D5hTN40=
          "]], LineBox[CompressedData["
1:eJwl01tMz3EYx/Fv5ipJyaGcKq7Y0kwzHdwpNjfWRqLmXGGLykbnuHCIGza6
sGHYbKFCl5TMIRfIhFpIbHKBi8jcGK//XLz3eb7P83me3/P7/v7/1G1788uj
QgijqEgKoWpeCFGpIUzA/Tkh5M9UEP9NCaGYFkXycjd5GxeEcE88xtdDm5xv
yV+aHcLdxBAanLvlh8RdtN75jviL+PD8EN7zNtF39BDdNyuEDp4KmiEXK7eU
TqF/ZoRQ4jmr1L+q59FvtHNuCLcRa+535yKzK9Vq9Q3q2SKeJI7GUf0Z6nvk
Njo3i1O8zwlaqv+UfAmdbs40lPGUy61R79Lbjd1y6fbZRePkN8v16Xkit1bP
I9qLx9hurw882WbkIAuDPCvoAL2qPqLezpvu3EYX09XyabTenXfSGlqHWmTz
93hev71f4xVynRf6bovwyTsnJIfwkV5Tu45WtOEG8njH+H7iB34jx8z1nl2A
dSjEBsR7x3i7TMUzPEWub5uHAbV6nkY0oNncI2Ydw3FEu5et5p4UTxbXqLfS
ahpDV8r32fMFzohf0v3upVRPv7iMJtj3rFql965Ai7to19uGTc4FKMRp/nHe
WPsN2eWN3TLFSfxL7PrAXaTz9dJE82Iiv3GeYn2/7DMSebaZVyJ3QYfxFi16
x+l5nmU0TU8CvYgLqJU/R+toHK3X/5Ae4Mu2x0G6nGYhEx0YtsNnnlFMNP+5
PS/z7bRfjX130GpapVaJjuT//8t/igOPxw==
          "]]},
        "0.294`"],
       Annotation[#, 0.294, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{5825, 8020, 8021, 7686, 7685, 7684, 6210, 6382, 5824, 6821, 
          8478, 8479, 8257, 6667, 8254, 8256, 8255, 7469, 6023, 7841, 7842, 
          8685, 8684, 8344, 8345, 7204, 7205, 6356, 6131}], 
         LineBox[CompressedData["
1:eJwl01tMznEcx/FvbhNzmByq54lLZmmEIafNqIfFDRcurEVZTeLGbC6ojZUL
cWczW7kRcxgj5qYetMLE5iEXZnXDbOXQM6c5vMzFe5/v6fc9/J89xdWN2/bl
RMRDfElE3C2KWFsccT0/4l1BxLTCiOnoni0+L2IdyrEGqzGs7kQyYoSOeDuM
oVkRR+ZEbBcb1OMZzmK++BS6gE6mD8ybRLMzIsq8K2WfmxmRoufpG7oCnXIH
6Vv1WfGdNPTIFc9Hn/cL7ZCmJeYus+9SZNS8nhuRxx53U7t39e7YobaBXtOz
SD6lrkXNH3a4KUl/4x77Fz0p/9zbDF6gjf/NnFb6lD/Vnbl6rf/37dAjVi6X
pvfRi2Z+m1wrjiMplsAhXPI2Y9ZlesteBexCrLTzKlzV/5O6K/Sw+Ef2bT3G
6CiqfZPddA9qMMF9Oai1ex1+sn/gOxrVXtCzxD4T5RbRPNor/lJ8CK8wYNZX
+/TTo2b22avKzf20mf9YvMM3e0Jb+APij9jH2Gm9SvWtt0sD6uy6FxvkR80a
QzvG1ZyiWXqansFiO5ZhCXaZX4v3ftdOfgeWm1NpbgUKzbmDblTYeRMGsREp
VOKGmR/0uEm38Dejy7z99qkS65HbSg/wm9Bl34tocuvnxP//xF8D74Gq
          "]]},
        "0.28`"],
       Annotation[#, 0.28, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwl0j9I1GEYwPHHxPQaDEnMK+/0TggMISgUdKkMB42GMnUs3S6koMEQBXPQ
RRwikIZSTyN1c87BhrKmpg5xDBX6o7UpNqgfaPjyfd/3+Xs/LtP/5N7jooiY
xrPaiBvJiMpsxHU+xx/rIj6hRmyyOiLJaaTQfz5iqSZiGWPy29IRH7yvoUIs
633/YsRX9wMuuH9Drz6r9RGvMhEZM+6oreXbnOIt8xrVbzjnLkSM8CPe5IKc
5pS4fj/0uMLfOWX2LlfxGRybMWjWoRlP+YV+LVzv3soL4te4Ex3ow0FVxC0z
2rFu53Gz3nh/jbPqynHa+3O1i86j/I7X7fsZw2I31QxxG49wsV476ufd79o7
z9vux/b+a95Vu65469LnPrqxJ75q3x7nZj2WzWniJS7TL4FSXFL3Uu57uXNy
85hBTt4sF/QvER9wfys/7RvmeQHzmBBLiE35bjt6zdkp533N+YH6h+jDobx/
+KJ+3O+8LL8Bf/Q/UvPLbr/xE3vYRVaPU9n//6kTMERdTA==
          "]]},
        "0.266`"],
       Annotation[#, 0.266, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwl0UtL1FEYB+BX1LzQ1ky8NaNdIGgn+AHctXLA1jqOtZsQ1EChhdhWTRQ/
gRqIjisRF3lbqjMkIm0UiUJoQKSFDUTiIy4efuf8zznve85MIv0+lS2LiI9M
NUUkGiK+tEZk5YhcYpG/POUZjbzgOf31EVeNEX8YdyavRq9xH5OPI3KJiDXm
mOWNbzPy7EnEvDyXP/nBS7WOkxEnjHHKd47V7Wi2ruZFW8QreS6bWiKKsk7W
cKP2sP4ldQfltHqdMmmeYJMFe9blBq+t/ZLffEsbXz+K6NKvR/0JPYvyNxl1
ar31IZVUU0WJZVYYJcfq3e/k7H/1tmXKvbvVqPS+Bxy45yF5Lu15p/9bhvTd
sf/IewbMO9lljw+MUOueBetFZ7fMt9nhKxVqp+Un6/2y3DwjC+5e0icv9537
bNyuzz/Z1nD/n98CxztUkw==
          "]]},
        "0.252`"],
       Annotation[#, 0.252, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwl0UsrhVEUh/FFJjqMXcPBhFKY+AgGBiSmlIEBDi9mZoxxXMoXoFBiKPkG
SuqcDIhOrkUmRi4hPxk8PXvtvfZ/r7c3PTLVP1kUEaNoqY9Yqoyo5kXeroq4
bIroTke0N0ZMqTu4E+8NEb0VEcu1ESsYtzeBMUwi82cZm+5m+ETOvjsH+KyL
SOx98DTP6J39y8c0EnzJXMOP/m98IfH+kLNd3jPjDpfyvIwSXuAUl3vzpSai
R30mY8A6Z31thlscOt9SH/ExmrHsW7r0ZjnhN3vfeu70F3vniZ9RY+YCt8l8
5Afknd9zjq/MN4dz5NEq78bsLVzgdbmrGJS9wSlcuJtVD8s89Xaf9at1Wfr/
n/wCLOJG1w==
          "]]},
        "0.23800000000000002`"],
       Annotation[#, 0.23800000000000002`, "Tooltip"]& ], {}, {}}}], 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdVX001Wccpw7ZbUK02pRi63hba46JrWOf5iYuS1KRRRnVZLIUR7gllUh1
DL/lUm6KishLXi4Lj6QYl9a0rpIbufI6Kb0sWq17e55/+p5zzz3PeZ7f5/l+
P8/3+/kY+//isWWKmppa5Nuf8v9djMHQmz/uks0nhU49UQaFj5DiUFayvWAV
8Umqf2bm8wgax2faNd1dR3iy7B32vEdoO7XPdlS+gVQaxQ2vrhxFy9oIw8TH
fsRABTAKba65ue/ZVhJqr/ziH7i3z4v6biKYtEqVMQLB4UrOLSaUmKsuGMGQ
mX+wMDaMKNHeQuJI0/TlFw5EEGU2Tj1D+Dyw3LAjLoqo4OyH0Dpt07hGwh6S
ka6MQUjWHOz72HEfeW6mBBxAwqfeN7+8Hks8VAkOYP34ohonpwO0vn6YX52S
t7HpIK2nHy9TZFy44BDZqoqHMCs+O5LeHE+uPlMC9sGrddfyWtfDZL4qwT4o
bJdqjUkTSbSqYAXKp+k+1XQ7SlRwsl6YJg7v0r1xjFirohddobcM36xKIio6
kx6gunKA+Af+SkZU5fbA7n7vealZMnF+VzD013RM8geTSY4SLrsbHxRVpvbm
pZB379UN0/6oZBKUSvmXY94Zw3y5JUf57oK+b3XDfyMc5fce2p3iPKUXf6N8
3sVXh7gJ7ZDjlL87yJ6YU7N5URrlqwOtrjprZcNplB8ZBM+1bPgXRJSPvxH7
Mvxx07Z0Wv8txE1rLYy2zKD1tkMvMtY5fCyD1ncTcybvfWJSeYLW8ycsIy3V
+6NP0vzbkO3u53rHOpPmK0WnPEQwoyOT5vcHjrbxJbl+YppPI6ZYzP3O4pqY
3t+Aux1LbutMiul9V1AcMFPh+UJM8WshMvfQLC0UU7wqSIxu7A6exb4vQ+5V
nkT4WSY9X4jzK/jGvo0n6Pk8HFC/35SWn0HP5yA/BBY6c9PpeTFObWs3GVmY
Rvc5/K6to8mv58jE3qDqsNQEDA6Le3b+mExe5T3+a6X/PljJHd1mORyj6x1I
5CcLTsfH0bUfzEhWd83rGLp2x9QKefN2z910vQy2wRu4Owt2EbX3gu1/0Wip
a28eAPa9qUCY6poQAobvVlV/2achAuz+nzmY56zYD5bfA03h3jnGR8Hy5z+5
37pZkAJWn0dMTf3+yXSw+jttjC9aDWSB8TPf9lLPKa0cMP4yE3lHluWfA+N3
2Yq+luLnuWD8+/30ckeMogDsfbgWm8s5C4rB3u9h9m1Hw+QSsPet3uB9plR6
Cez9BVVZAQWdpWD9YVb0zezhujKw/hEJIj1aosvB+mvGm+V1Qu0KsP4TyLmV
BsIKsP40kIr227RVgPVvY+hx5/U8CVh//+sXkHtriQSs/7sUPoufeEnA5oPn
296qs10CNj8iLtRBFiYBmy+XqYqx0bf7bP6SXDyHXntKwObToSB7lftiCdj8
Or/26656UQE23/nh8VWCEpa/HEZP9TJ5vhVg+uDdsXvdh6/KwfQj0cu70yCp
HExfmu8ZBzl9VA6mP81Hvq8rSi4D06cHEpndavUyMP3aWViNr7eUgunbqGmA
UFTD+FdAunR89vzpl8D00XahzkSCWwmYfmpcX1cnSigG01cjNamuSVgRmP5a
cN2BJo0XwfS5LLThByGvAEy/o62Dal2+vQCm72lXamTl/rlg+r/HK89xMOIc
mD+slOn7zt6bA+YfmRvb9MThZ8D8RUvjbGDfxiww//GtrBmtDTsJ5k/X7GL0
eXoiMP/qyt1kzUtNBfO30ojh0ybjx8D8b8LqpPH0+INg/hhvez3PShgD5p+J
CstYriMczF/VazurQz0C8Z7/4n9EbM4S
     "]]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  ImageSize->504,
  PlotRange->{{0.95, 1.05}, {-0.03, 0.03}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 313},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->140568297]
}, Open  ]],

Cell["\<\
The flow field at the trailing edge is dominated by the point vortices \
associated with the endpoints of the trailing edge panels for the lower and \
upper surfaces.  By placing the cursor over a streamline, its value is \
displayed.  This tool can be used to follow a given streamline as it \
approaches and leaves the trailing edge.  This flow does not satisfy the \
Kutta condition that the flow leaves the trailing edge smoothly.\
\>", "ExampleText",
 CellID->38752132],

Cell["\<\
Repeat the computation after altering the trailing edge by making the \
trailing-edge points coincident.\
\>", "ExampleText",
 CellID->254018838],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rTE", "=", 
   RowBox[{"Mean", "[", "ptsTE", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"ReplacePart", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"1", "\[Rule]", "rTE"}], ",", 
       RowBox[{
        RowBox[{"-", "1"}], "\[Rule]", "rTE"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gA", "=", 
   RowBox[{"PlotAirfoil", "[", "rP", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "aW", ",", "rC", ",", "vMagC"}], "}"}], "=", 
   RowBox[{"PMDoubletConstantDirichlet", "[", 
    RowBox[{"rP", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl", ",", "cm"}], "}"}], "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Doublet\>\"", ",", "a", ",", "rP", ",", "\[Alpha]", ",", "rC"}], 
   "]"}]}]}], "Input",
 CellLabel->"In[5]:=",
 CellID->230503503],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.966180550578347`", ",", 
   RowBox[{"-", "0.3249486575060325`"}]}], "}"}]], "Output",
 ImageSize->{149, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->116828970]
}, Open  ]],

Cell["\<\
These are plausible values for the lift and moment coefficients.\
\>", "ExampleText",
 CellID->88671999],

Cell["Plot the flow field near the trailing edge.", "ExampleText",
 CellID->211503654],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rAll", "=", 
   RowBox[{"Append", "[", 
    RowBox[{"rP", ",", 
     RowBox[{"{", 
      RowBox[{"xW", ",", "0."}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"aAll", "=", 
   RowBox[{"Append", "[", 
    RowBox[{"a", ",", "aW"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ic\[Psi]A", "=", 
   RowBox[{"ICPsiDoubletConstant", "[", 
    RowBox[{"rF", ",", "rAll"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Psi]", "=", 
   RowBox[{
    RowBox[{"UniformFlowPsi", "[", 
     RowBox[{"rF", ",", "\[Alpha]"}], "]"}], "+", 
    RowBox[{"ic\[Psi]A", ".", "aAll"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gStreamlines", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"Contours", "\[Rule]", "contours"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "False"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "3"}], ",", 
     RowBox[{"PerformanceGoal", "\[Rule]", "\"\<Speed\>\""}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gStreamlines", ",", "gA", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", "\[Times]", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[6]:=",
 CellID->78818121],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJxsnQV0Fcf795Hi1kIFirtDoTiFAYoUK+5OcQokJLjGCAkQSIhAghWX4lJ8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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{4546, 4927, 6556, 4499, 5345, 4731, 6583, 4732, 4659, 6437, 
          4983, 6435, 4982, 6436, 5466, 5467, 5062, 6613, 5061, 5897, 5858, 
          5859, 5660, 5661, 4412, 5339, 4411, 5659, 5056, 6107, 6108, 6109, 
          5516, 4490, 6372, 6373, 6369, 6371, 6370, 5342, 4410, 5658, 5857, 
          5856, 4843, 6592, 5690, 6106, 5895, 4919, 5894, 4918, 5341, 4724, 
          5855, 4842, 6301, 4841, 6637, 5893, 6642, 4917, 6641, 5972, 4350, 
          5973, 5892, 5581, 5689, 5688, 4489, 5423, 5687, 5686}]},
        RowBox[{"-", "1.8681`"}]],
       Annotation[#, -1.8681, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0LkuhGEUh/EzRs3sZirR2ZcJhZtASOglWhI6XIBSVLaKa6Ck1mlmzGKP
rZCInvD7iifPd877f8+cd/qW1+ZXUxGxgq9MxC/a5YiOnog06qWIBm6wkYsY
KUZM5iOWeLAQ8SD7iIZsE1Nyl5WIF9mm7wKKmM5G3MqPujfn/MKMWb7kc/0z
ZN1vybZRwon6NOk5q6jL6MGr2Tn9PK7Mm+E7/Xf9W/7ge37jI7sdYx+HOMC3
N+6a2fTdSnbHmD2Gkjdhx6615O1YQM35jzt1vsGf2Z92fzJ/T/3M13rrPOz+
uFmL3O83BjChDnOqnOJeuS75bmRQVW9xmje5k7eT/1z2HwYVPzQ=
          "]]},
        RowBox[{"-", "1.872`"}]],
       Annotation[#, -1.872, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0d1PzXEcwPFPGCH0cE4PrqpRqnMaTqO5lGkLXdjcJe10S+bOPWVkHubg
ToiN2orxPzCU9aBNKjZ3ZeYf0Hr9Lt57f7+fp+/n/E5d/vLZgaKImMCf0ohC
dcRDPMBjPMLv8oj5yohvmMEcZrGm/lRVxOZ0xE815/gXb+Er/FXNCk/xMk9z
l/qP8nt4N5bEd4qXYBcWUhHPxJ9ih/sP+S98sSIiVRNxzz738UR+BEfL7Kbn
CG9X16oux/vk9qMRDbghf13/IK4hpWeIi7jJPgfw3blY7zZsxaK3Z73VYuYM
N/O6+H/sdd8kP27GIffDGPJGgzmNyKqdU5Pheb5r9j/f67RdziCTzEW9GcPJ
dxI7zm1iORScK1Anf4vzPMLvzHmPnLlL6lawjEGxO/LtdijnMtxErb5JO7Y5
F3tjVe0xvR3uJ3Bb/UH7DnOzfAua8EHNefEe9OKqeyb57/ikvk4U9FyQ60Or
3Ce5LH/mfrE80u6VyW9W/1L9gli3c5XYc3u9Eqt2zjqP2u0FJt3f4C1eJ+/J
jXEnD3Cp+CVOu//1TTcAjVJd+w==
          "]]},
        RowBox[{"-", "1.8759000000000001`"}]],
       Annotation[#, -1.8759000000000001`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0b1L1XEUx/Fzrw+zpqYuml7NG2KF2iBqD5vmpLbcyILAhoKyQbBGLWjw
Ac1F0JYWEx30DxASQVN7QBNtcw6MtE0bfDm8+Zzv95zP53x/91Y8ftH1PBER
H9FfEHFQGPH6QsSnoog5dDtP5EekiyMKMXEx4j1acRNHJZTvUO8P6nlanR+V
RqzSB/QhenCYFzEq7xvPZb2vtIZW81ThrT3XZTa5u0Vv4w72eZr1W3Cgl3GX
w5uLAXevcGTmGAvqefSqn6DNfNjdTqd816S6zJ4ZvYQ6iR9y0vrf6RW6pLeI
GXs+4B3KeHPoOOrN1To3q1uQOf+95PyTe5//mM46Z9Rr3vLbdw/LG0Ha7Lbf
aQeD5jpkDdHPZrt5Vug9+pI2mTmhWzK61J34q5dyt05P7F6jezJH5OzTDXd9
dMeuKnOnztX0P/3JX0eXeX6Z2VQHEtg1/0XGBtaxhU288bar+teQRBYa3O2Z
T3lXO57Kn6bPaKUdKZSjApdwVy+BJIr5smU0ythW3zj/L8xMY8y+URR43xn5
41uh
          "]]},
        RowBox[{"-", "1.8798000000000001`"}]],
       Annotation[#, -1.8798000000000001`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0UtIlVEUxfEdDYRG+aycdK9QjXsMKsoelAWVpD2IcmRNUlCDCrUUmlR6
I0rJwtusUUSTQkvrVmAPGvQwrVmJQVD4BI0Kg/rdwZ919tprn3O+8yWr6yvr
5kXEM8zlRkwtjpjG2KKIcYwURDTmRVTwmuhh3hFU4aZeuoiiFQX6LbSQ7kbP
kohddDY/oty6l+6hD+ipwoiTWKW+rF5J12A1atyjTa+Wdqv79Nc7a6n6Nt1m
z07eoF4HbZad1PuLFO/Hwoiv/Bx3fsobtb7Fz1h3yj6hZfa4z9tOE+o07cZZ
/evyH/Ru0C6c4a3FOnxWH/KN8+39yNyk+5zmT9Fp5OCOupoeRZn8jOxPzOKa
M457uxrUIi2zkV+KDdiMTei3d5VsLu66Sx7dinvWW+h5c/l0xv7jsl3OnKAH
eb0yB7LfIvON16fXj1Z1Cx7zrqo7MGK+Qf4LPZF9a70r/If0mGwRdqp3oNh6
ATJyn+yfyb4lAv/4H3kpd7+Ei2hHG777H+36qew/dM4b3/0W75Awe85Z760H
sRcvzcyZeUV/eevfGMALPEedc5LmEvaqpMvsuxxJ9R/9Mfcbkrlgfpi+5tXT
EtkSmf10hfw+2qAeoOXq/2TveQ0=
          "]]},
        RowBox[{"-", "1.8837000000000002`"}]],
       Annotation[#, -1.8837000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ktsjGEUxvHXJUETyXRQrYWkVmLHioXqdCp0ioS9Sxt1SahoiZX7LRpC
UlVqo9WmkpZIBNXEpRbaaplhIyFCgikJ0tpYMMPvW/zzvM85zznfO983pbV7
NtRPCiG8RzYWwoniEE7iOF7HQzhF03SoKIRX9BnN0EFaNTeE3OwQXvDr54Tw
kub53fSp/igdoCO0sCSEOGLYOSuEJ+rP1XPm8viHjNmrdrbhLz+s/1huh/wx
9xjiL+tdwZJCz5NfTFv5P/KPZB9iJn9b74jeqPMI5tvx1Y5vyGIpP0ZX0Vr9
GmzBGc9oV+/ANfzgE3aVo9MzctEe5zFs47N0tef0mu3BT/lqmWG/M0Xb9Bep
nXNOq52ljebum4mpF9Eqme20O3oX+tV8KtqvNkOtjt6z+y7yZi7ptUR38L2a
7KxRy8ockD1vPqN2gSbVu+WK9ZrVWvkx53a5DizTm8f30BLaKLMPDfhifj/9
RC/KDup38Z/5dfYkvZuVqMRHvRaZD9FvkBmX+YU3zhO0kt6gm91/EzbiIF9n
ZiuS5hb4bv1qfXoP0I8mvV16C/UKnKdG/xu6xn37nAfsnUxvucMU2kNvohdl
5g7LLqdl8h36h/jv7v7b3HV+hfo05+koNBPHhHxCbjz6Pvxa+QpagAb5d+p3
9N/SvXzauYJWRe/HOUWTfKk7H+U7o+9gfxctpxVIoFnmNP4DJSCNSw==
          "]]},
        RowBox[{"-", "1.8876000000000002`"}]],
       Annotation[#, -1.8876000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV01dsiFEYxvEj1Kpdq1p7xIpxwZ0ZpWarolYqVmwViXFBqHlh76S1m1Ik
toRIhVKbUErChURiE70xqubvu/j3ecfzvud8p23LKfPTMiuEEJL8eFMnhORG
IbyID2FA3RCOiJNo64YhZCMH3eqF0KNxCN1pT7pVf1yDEBbGhZBgrikS0Vq/
JW2FZmiB5piAbWY+2v0Bo+xIQ2f1/fQAjqkfxyC+JPWB9Kg8H+dwFhPVfjgj
g1Z2fhWUyQ+bz8cR1Nc7Xj+EOJouH4PR2MkXY29lVLdrrLtXEseKa+Cg78zF
IaTxrtF/oL+afrLvcfQ28mHyGP1SDFf7rLdRvAmp8hN2nUQeDqOPWpZ79qUt
7B7AnxLtUhtBV5pbgSzURIK71qKJtKLzRjjvGV8v/qd0uPyxON2evfJN4n10
qN3v9O6L39N7dKQ9g3HeniE0GXniIr2U6L6oji3Rm6i9NRdLt8qXiZdiqnga
JmEKJkfqrLZoh5LoXqhtbrNdl/S30EVYjI3yf/7GKth1wXvM4vumPpbGmBtH
v8v/IfDMjN6G9qOr9PvTfHlP39ubJ1+eQWNRA8XyTs5YxdOedkQHzDSbqzdX
/FRvDn1Gy/ADVc1WwzGedfSi/lr62jnZavvkd/ja+MZh8ln2zcZatW16tdS2
89akg/yukjEQQzAYJeqn+Z/QTL4n4rn0ofyU+BGdJx9v/wxaKM9R34N4+Stv
Vo7F4iUodW659/uF39jMd9bMX/EfzHenTKTbNxov9abTJnrxWMB/Ru2nuAzP
7ZvHv8E3f7B/PT3KkyJORZreSDRCYzREE8RHmP1CS/ERN6K/Yd98kz4wex/f
orP5vtIE+p3edm5nWmDmFm+B+Kczy/HQzFU7Enmv0GsoxHJ3vmPuLnZgJ7qo
FfM3M9ccIfp/x3N769EV+gX2lMgv06a4btcNFKGr/j2+bnS3fbuwUj0Ltc38
B7DPucs=
          "]], 
         LineBox[{4662, 4502, 5225, 4553, 5533, 5115, 5164, 4357, 5268, 4358, 
          5165, 5116, 5534, 4504, 5204, 4503, 4663, 4753, 4692, 4569, 4950, 
          4951, 4948, 5438, 5733, 5439, 4949, 5567, 5191, 5192, 5508, 5507, 
          4437, 6476, 5040, 6475, 5041, 5511, 5510, 4439, 5039, 5173, 5036, 
          4438, 5509, 5035, 6474, 5038, 6472, 5037, 6473, 4386, 5446, 6412, 
          6413, 6411, 4967, 6414, 4568, 5553, 5554, 5232, 5231, 4518, 6561, 
          4669, 4752, 4662}], 
         LineBox[{6131, 6129, 4506, 6525, 4507, 5207, 4516, 5428, 4515, 5074, 
          5166, 4361, 6192, 6193, 6191, 4645, 6190, 4646, 4644, 4442, 5205, 
          4443, 5123, 5947, 5122, 6526, 5948, 5949, 5704, 5706, 5705, 4427, 
          6130, 6131}], 
         LineBox[{5237, 4573, 5558, 4572, 5556, 5557, 5236, 5235, 4522, 4671, 
          4754, 5987, 5988, 5989, 5284, 5992, 5991, 5990, 4755, 6228, 4655, 
          6119, 6120, 5701, 6156, 6157, 6158, 5729, 5582, 5946, 4550, 5224, 
          5728, 5237}]},
        RowBox[{"-", "1.8915000000000002`"}]],
       Annotation[#, -1.8915000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ltMz2EYwPHXFjI2hA5Om8PmCpNSNsxG3ZZqk0PnSDLd0FE5XnDDXMQd
0Y1jDtUF4cJ5iFzVnFejhY3NhpTx+V98932f532e531///c/p7gya+eoEMI3
vJsUwpq4ENaiLDaEcrxOCGEbv+G2ySEk2ns1JYR/HOJDGIVj00Jot5evrgB5
GC//QO6tvpPiAV4UE8InPiH+yE3cq7dD3QF7R+WirJunhpAsPoSD6Bc3ya92
7mOeaHaSfAp/c4/vSBVf07+cn6gpMrdW/VO5Oj5rxhmcwmn0yGU4PxOX1S9T
38ob7W1CFDYgF8eRjwKMQREKcUl9sr6L/MsdfiLN3G65sXItaqL5pbhXfp2z
spCNFLkr9jarycNorNJ/VS7Hfgz3qEmTz7I+bb3EOhE54iReimb5bHG6daNv
7+MGbpFfYb0y8ts5+7zfakj+HP/hCzwSeQ9nDfNA5O3MaXeHQl4gnm3dprdY
fNi8Ir4uXmxmB5eIo+VL+W/kPeRTsd46F0PyaWak473/1kJnfuBqb1KDKnw2
p5YHuZ6/ch1/4VJ3KEGFmc/NanVWF28XT+BydS/UbeNu3s7PuIyf8xbu4q3c
59x9PE7vXj6A/YgWj2AY9925wR0bUYc9qEe/3iP2b1nHmtfJcTzdfRLQ4C4P
feMjZOCG/d96bvJuVGEXalCNCufM0DcTE82diyH1t+3dwQ/fPOhud9XViu9x
p9wOnqUn3tmZPE/ffFSK/wOr748J
          "]], 
         LineBox[{5102, 5488, 5487, 6446, 6447, 6209, 4679, 6152, 6153, 5723, 
          4547, 5724, 5578, 5579, 4324, 6537, 4602, 4534, 5490, 5000, 6606, 
          5001, 4672, 5285, 4373, 4698, 6220, 4699, 4623, 6546, 4369, 4944, 
          6602, 4945, 4943, 4947, 4946, 4567, 4691, 5168, 4848, 4433, 5357, 
          4849, 4850, 4852, 5860, 4851, 5861, 4368, 5271, 5272, 4999, 4998, 
          5483, 5484, 5076, 5936, 5075, 5937, 4444, 5485, 5486, 6524, 5101, 
          5944, 5102}]},
        RowBox[{"-", "1.8954000000000002`"}]],
       Annotation[#, -1.8954000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0DtPlHEQxeHhIwgiUCCxkZIQK1fAqBAaA4HSkqgkkEhBIxAEbDTKRemQ
y67fAdxd7iRKIKEhfAC5BxFsRLxUPm9x8pszc+ad/+6Ntq7WZwURMUmzRREf
aKeMxwmqUU9jCqfwDpZciei9GnGAueKIQ+zjO65F9OA+n9Xfw+f8F/UAnvAL
6m84yHfKv8Tv/KL+KQ7x5VjnTsa9WvyId3EG++Uq7F2nNF9eElGafKPU2/iv
/C7tUXXyNjygfaoxP5NrkT/HH3Skf0xjbj5yYxRbzYupwX49PbZ3j2aSN5jf
0ktjKrlJtylVGPGa8jKv8Mh+RibN37e7qn6AmeT3qE/NX/Bb/EDyP9upVA+p
b+Igtrvzy9su6KHfu61Xpa7HWT5r56fMG7239Nk3R3GEhiln3u1tT+Tn5PP8
hfyY2Ttal//NXyZ3aN78k9xT+Swu8H/038uO04b8Ji3pN+kvJnO+0Swvv8zn
cAXnsQP/ma9hs/wq/uXX1f8Bvy1phw==
          "]], LineBox[{6645, 4387, 6646, 6644, 6643, 6645}]},
        RowBox[{"-", "1.8993000000000002`"}]],
       Annotation[#, -1.8993000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVzrdOnFEQBtDhDUhLEFBQuCFISETBM9C4slto8bK75FAQbTA0NBZPYFG5
QaIDvAQh8QAIBA9AtqGwycvZ4uj75/4z905tT//HZEFELFNbFHFaEnFUGXEm
v9Pke0n2lUd8IcnP0oi0TNHP54qIT/wojniUT/zjgf/M6n+RrzyT441V/VXm
qzlJRGRklR3W9FfLQfUAbXZoZcUebep2mmmlhU6zSwyb6Ta74ztLr/4udZ3z
1TK7kiJJhjQj/m26Y4sppql31mG/Qf+HGCDnbFiOym09v+xTY/8P+mbUs3Sp
R/WEHJEFckz+zt+Zf19vijTr6js73rNrxwt1sX3PZYm8lGPeypo999aGuzLm
5tTz7Ju51lOq90om5I0cN7Pj/y5f+UaDswP9f/z/yy33+feZ8G8vfx8LLNLo
7FB/ud0evF0hc+4vcnasLpSTet4BOapZCQ==
          "]]},
        RowBox[{"-", "1.9032000000000002`"}]],
       Annotation[#, -1.9032000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0EcyhFEUBtBLmYndQmmhaZS0FwYmqmRNy2GCFUgDC2ALLU2kvaDEFYhl
QDn/4NT33/e+2/Wqc/n1obWSiNhiIh0x0BgxKQdlo8zKjNxpiCj1PSyLmYjf
lHPzgfnR/MwTPbov8pUHmsxtes1yV7fM93ZdxJ/9w3rnvltI1UZ8yaLekfNj
+W0uyEpzFT86rc7eaiIu/M4oY4wwwThz+tW6NZTrZnnXv3SXo4N2uuhkwztO
dE6Zsrvnzbf6++Z72aPTSzf99LFp58z9ebLjTft27nQ/uHI/TZ4pZplh3m+n
vClNhb02PvWvkzcn9xRYZCGhX6tbR6VuO1/6N+6WWWGJNVaTPf163Ybkv9LN
8a0/YP4HYwQ9Ng==
          "]]},
        RowBox[{"-", "1.9071`"}]],
       Annotation[#, -1.9071, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0TcvhWEYxvHbyKaTMBh9AkzKymTV4jAqk95LYhIGn0CvR19ERHwPvXcJ
dj/DP9dz39f/PW/e8xQlOms7UiKiB9M5EaMYwwgmMI6f9IjMvIhJ59yMiELn
HHmRH1GeG1GBJd283aJckMuyRf5k6WR1tk5+myv5q/pl84pckWuyVf7q12QN
f1WmeUd9ZkSqrJPrvE37c32X3+nGkWeuzJe4wJZ+g9fA35RJ87V9D7cXx/w7
8y1usK3f4jXym3zbjrmP148BnPAfeQ+4x64+yW/ib8s985P9IHcIp/xX8wue
sa/f4TXzd+WB+c1+mDuCM/6n+QPvONTv8RL8KVlg/rKfcZ5FsbnU/1GGEszx
KvSf7qlN3/5/T74j7/+eUOUdfzaSUHA=
          "]]},
        RowBox[{"-", "1.911`"}]],
       Annotation[#, -1.911, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVz8kuw3EUxfFbojXUrIalxFTDS/AARKKtrb154Sm6NDTm2EoUC6REjA+B
p5AUS58uvjn3nPO795//4NLq/EoiIpZR1xmR7Io4H4hI0Srf0x+RQa4vIo+Q
ffMJusgXMMfPYstOwe4iFpBHDjOZiGks27n3vlKjO+JDNiS76Yko27/EBa5x
hRXdgZvbbgybb71rsPfMJ+mq7FC/w4+Y7/Qp+QvfSNdkR/pdftRc0TfJX/lm
ui471u/xY+YHfYv8jU/TDdmJvsRnzY/6Vvk730Y3Zdled/lT787Mn/7pC0+1
W7S99j3aQdN03JsX876dov2CbKN2C0VZvaxkd9K9Mj9Ff2Q/+j+66/0vrfIT
3v0DwK88JA==
          "]]},
        RowBox[{"-", "1.9149`"}]],
       Annotation[#, -1.9149, "Tooltip"]& ], {}, {}}}], 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdVX801WcYpw7ZbUK02pRi6/i11hwTW8c+jcTNklRkUUY1mSzFEW5JJVId
w10uRYqKyI/8uCy8kmJcWtO6Sm7kys9J6cei1brX9/mn55x77vc97/s+7/N8
nuf5fAx9f3HbOkVFRSX83U/xP2mjoK8Cx+4IvYLHSLIrLd6Rv5p5JdQ9N/F6
DLUTM20a761nPGnWTlveY7Se3m89ItvIKgxihtZUjKB5XZh+/BMfpqd0MAJN
YVNT7/NtLNhWceMfuLbNi/huPJC1SBQ2DP6RCqFLVDAzVT4wjEET30BBdAhT
eHvnEkcbpy+/eDCMKaJx7B7E5/5l+u0xEUzpznYQLdM2j6nF7WVpqQobgHjt
od6PHfazFyYKh/2I+9Tz1pc3opmbMsB+bBhbVO3oeJBN5tcH02tTcjc1HuLy
6cOrJKkwlH+YbVPaI5gUnRtObYpl154rHPbCo2X38hrnI2y+MsBeyK2XaoxK
4lmkMmE5yqZpP1N3OcaU7qQ9MI4f2q198zizVFoPOoNv679dncCUcCY8RFVF
P/P1/5UNK9Pths2DngsSk0TmNJkwdNe2T9gPJLJshbusLnxQWJHck5vEJqvU
BeO+iEQWkMzhL8O8s/p5MnMhh3cndL2r6v8bFnL43kebY4y75NJvHJ738NVh
4bhm0AkOv7vIGp9TvWVRCodXO1qctdZJh1I4fKTgv9Cwsr8o4vD4G9GvQp80
bk/l8r+NmGktBZHmaVy+bdAJj3YKHU3j8ruFORP3PzGqOMnl8yfMw81V+yJP
cfG3IsvVx/muZToXrwQdsiD+jPZ0Lr4/cKzVXpzjk8HF04ApZnO/M7uewb1f
j3vtS+5oTWRw711Fkd9MufvLDM5/DUSmbuolBRmcv0qIDW7uCZxF90uRc40n
FnyWzp0vwIUV9obeDSe587k4qPqgMSUvjTufjbwgmGnNTeXOZ+D09jaj4YUp
3L4Qv2tqqdvXCdn4voCqkOQ4DAxldO/6MZG9zn3y1yrf/bCQObjMsjvOrXci
3j6RfyY2hlv7wIRldlW/ieLWrphaLmva4b6HWy+DdeBG4d0Fu5nKe0b7XzSY
a9ua+oHuG/MFyc5xQSD/LpV1V7zqw0Dv/yyEafaKA6D4HqoL9s0xPAaK3/7p
g5Yt/CRQfm5R1XUHJlJB+XdYGV6y6M8E4TPf+nL3aY1sEH7p8byjy/LOg/Bd
tqK3uehFDgh/n59e7YyS54PqI2y2upK9oAhUv0dZdxz0E4tB9a3a6Hm2RHIZ
VH9+ZaZffkcJqD9MCr+ZPVRbCuofET/crTmyDNRfM94urxVoloP6jy8TrtIT
lIP6U08iOmDVWg7q34bgE04beGJQf//r45dze4kY1P+dcq/FTz3EoPngebe1
aO0Qg+ZHJAy2k4aIQfO1cqp8dOTdPs1fwkr3wTfuYtB82uVnrXZdLAbNr9Mb
n67Kl+Wg+c4Lja3kF1P8Mhg800nneZeD+MGzfc/6D1+Xgfgj3sOzQy+hDMQv
TfcNAxw/KgPxT9PR72sLE0tB/PRQLLVZo1oK4q9dBVX4emsJiN9GjP0EomrC
Xw7J0rHZ86dfBvGj9UKt8TiXYhB/qt1YXyuKKwLxq4GKRNsopBDEv2bCLn+j
hksgfi4Nrv9BwMsH8XekZUDNym8vgvg95Wq1tMw3B8T/ez1yHQbCzoP0YZVU
13v2vmyQfqRvatXJCD0L0hcNtXP+vZsyQfrjXVE9UhNyCqRP122idHk6IpB+
deZstuQlJ4P0rSRs6IzR2HGQ/o1bnDKcHnsIpI+x1jdyLQRRIP2Ml5tHC9tD
QfqqWtNRFezmj/f1938r+8dg
     "]]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  ImageSize->504,
  PlotRange->{{0.95, 1.05}, {-0.03, 0.03}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 313},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->766448454]
}, Open  ]],

Cell["\<\
Evidence of point vortices at panel edges show clearly in the two streamlines \
nearest the airfoil, but on a less magnified scale, the flow appears to leave \
the trailing edge smoothly.\
\>", "ExampleText",
 CellID->667775740]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[{
 Cell[BoxData[
  StyleBox[
   ButtonBox["PMDoubletQuadraticDirichlet",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/PMDoubletQuadraticDirichlet"],
   FontFamily->"Verdana"]], "InlineFormula"],
 " . ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["PMVortexLinearNeumann",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/PMVortexLinearNeumann"],
   FontFamily->"Verdana"]], "InlineFormula"],
 " . ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["ForceAndMoment",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/ForceAndMoment"],
   FontFamily->"Verdana"]], "InlineFormula"]
}], "SeeAlso",
 CellID->67651628],

Cell[TextData[{
 Cell[BoxData[
  StyleBox[
   ButtonBox["PlotSurfacePressureCoefficient",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/PlotSurfacePressureCoefficient"],
   FontFamily->"Verdana"]], "InlineFormula"],
 " . ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["FitAirfoil",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/FitAirfoil"],
   FontFamily->"Verdana"]], "InlineFormula"]
}], "SeeAlso",
 CellID->775123367]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["3.1 Introduction To Panel Methods",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods"]], "Tutorials",
 CellID->341631938],

Cell[TextData[ButtonBox["3.2 Validation Example",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/ValidationExample"]], "Tutorials",
 CellID->499723042],

Cell[TextData[ButtonBox["3.3 Implementation",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/Implementation"]], "Tutorials",
 CellID->336290191],

Cell[TextData[ButtonBox["3.4 Flow Visualization",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/FlowVisualization"]], "Tutorials",
 CellID->307191390],

Cell[TextData[ButtonBox["3.5 Convergence and Timing",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming"]], "Tutorials",
 CellID->351529306]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["PanelMethods",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/PanelMethods"]], "MoreAbout",
 CellID->524805846]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"PMDoubletConstantDirichlet - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 2, 22.9884000}", "context" -> 
    "Aerodynamics`PanelMethods`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Package Paclet Symbol", "language" -> "en", "paclet" -> 
    "Aerodynamics Package", "status" -> "None", "summary" -> 
    "PMDoubletConstantDirichlet[r_p, \\[Alpha]] implements a panel method \
using doublet panels of constant strength and internal Dirichlet boundary \
conditions. Panel endpoints describing the discretized airfoil are located by \
r_p, and the angle of attack of the airfoil is \\[Alpha]. The function \
returns a list of strength parameters for the doublet panels, the strength \
parameter for the wake panel, locations of collocation points and a list of \
surface speeds at collocation points.", "synonyms" -> {}, "title" -> 
    "PMDoubletConstantDirichlet", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/PMDoubletConstantDirichlet"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[12318, 377, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->686933026]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 362547, 7127}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[619, 21, 76, 1, 70, "SymbolColorBar"],
Cell[698, 24, 191, 5, 70, "LinkTrail"],
Cell[892, 31, 3882, 87, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4799, 122, 69, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4871, 125, 1111, 27, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6019, 157, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->116858593],
Cell[6602, 176, 250, 8, 70, "Notes",
 CellID->392463071],
Cell[6855, 186, 679, 25, 70, "Notes",
 CellID->46829889],
Cell[7537, 213, 770, 28, 70, "Notes",
 CellID->156357560],
Cell[8310, 243, 408, 12, 70, "Notes",
 CellID->61542297],
Cell[8721, 257, 580, 17, 70, "Notes",
 CellID->109779479],
Cell[9304, 276, 722, 23, 70, "Notes",
 CellID->73919694],
Cell[10029, 301, 444, 17, 70, "Notes",
 CellID->19186463],
Cell[10476, 320, 548, 16, 70, "Notes",
 CellID->181230849],
Cell[11027, 338, 53, 0, 70, "Notes"],
Cell[11083, 340, 1121, 29, 70, "3ColumnTableMod"],
Cell[12207, 371, 74, 1, 70, "Notes",
 CellID->918694056]
}, Closed]],
Cell[CellGroupData[{
Cell[12318, 377, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->686933026],
Cell[CellGroupData[{
Cell[12804, 394, 148, 5, 70, "ExampleSection",
 CellID->107487309],
Cell[12955, 401, 131, 6, 70, "ExampleText",
 CellID->77267041],
Cell[13089, 409, 100, 3, 70, "Input",
 CellID->326492779],
Cell[13192, 414, 157, 4, 70, "ExampleText",
 CellID->48545141],
Cell[13352, 420, 217, 8, 70, "Input",
 CellID->317289154],
Cell[13572, 430, 135, 4, 70, "ExampleText",
 CellID->136872357],
Cell[CellGroupData[{
Cell[13732, 438, 111, 3, 70, "Input",
 CellID->59487574],
Cell[13846, 443, 159, 5, 36, "Output",
 CellID->135961514]
}, Open  ]],
Cell[14020, 451, 85, 1, 70, "ExampleText",
 CellID->20057380],
Cell[14108, 454, 147, 5, 70, "Input",
 CellID->163234459],
Cell[14258, 461, 716, 23, 70, "ExampleText",
 CellID->442126641],
Cell[14977, 486, 271, 8, 70, "Input",
 CellID->53812608],
Cell[15251, 496, 677, 12, 70, "ExampleText",
 CellID->205450438],
Cell[CellGroupData[{
Cell[15953, 512, 210, 6, 70, "Input",
 CellID->544103304],
Cell[16166, 520, 163, 5, 36, "Output",
 CellID->35643753]
}, Open  ]],
Cell[16344, 528, 189, 4, 70, "ExampleText",
 CellID->33026124],
Cell[16536, 534, 363, 13, 70, "Input",
 CellID->81885161],
Cell[16902, 549, 347, 8, 70, "ExampleText",
 CellID->19233943],
Cell[17252, 559, 525, 16, 70, "Input",
 CellID->133172956],
Cell[17780, 577, 483, 8, 70, "ExampleText",
 CellID->76299467],
Cell[CellGroupData[{
Cell[18288, 589, 1205, 32, 70, "Input",
 CellID->808226483],
Cell[19496, 623, 89424, 1530, 70, "Output",
 CellID->2581982]
}, Open  ]],
Cell[108935, 2156, 623, 10, 70, "ExampleText",
 CellID->14395635],
Cell[109561, 2168, 135, 4, 70, "ExampleText",
 CellID->80454523],
Cell[CellGroupData[{
Cell[109721, 2176, 268, 9, 70, "Input",
 CellID->499718542],
Cell[109992, 2187, 260, 8, 36, "Output",
 CellID->227599051]
}, Open  ]],
Cell[110267, 2198, 71, 1, 70, "ExampleText",
 CellID->8410041],
Cell[CellGroupData[{
Cell[110363, 2203, 141, 5, 70, "Input",
 CellID->342608962],
Cell[110507, 2210, 178, 5, 36, "Output",
 CellID->203117986]
}, Open  ]],
Cell[110700, 2218, 289, 6, 70, "ExampleText",
 CellID->119738758],
Cell[CellGroupData[{
Cell[111014, 2228, 89, 3, 70, "Input",
 CellID->288057467],
Cell[111106, 2233, 194, 6, 36, "Output",
 CellID->643710237]
}, Open  ]],
Cell[111315, 2242, 121, 3, 70, "ExampleText",
 CellID->753558053],
Cell[111439, 2247, 165, 6, 70, "Input",
 CellID->435840926],
Cell[111607, 2255, 331, 6, 70, "ExampleText",
 CellID->139054009],
Cell[CellGroupData[{
Cell[111963, 2265, 1392, 39, 119, "Input",
 CellID->671472236],
Cell[113358, 2306, 6946, 121, 345, "Output",
 CellID->155318168]
}, Open  ]],
Cell[120319, 2430, 404, 16, 70, "ExampleText",
 CellID->571494656],
Cell[CellGroupData[{
Cell[120748, 2450, 619, 19, 70, "Input",
 CellID->42630738],
Cell[121370, 2471, 338, 12, 36, "Output",
 CellID->57343977]
}, Open  ]],
Cell[121723, 2486, 399, 7, 70, "ExampleText",
 CellID->24035690],
Cell[CellGroupData[{
Cell[122147, 2497, 318, 10, 70, "Input",
 CellID->34907031],
Cell[122468, 2509, 339, 9, 36, "Output",
 CellID->617452414]
}, Open  ]],
Cell[122822, 2521, 331, 6, 70, "ExampleText",
 CellID->47127959],
Cell[CellGroupData[{
Cell[123178, 2531, 236, 8, 70, "Input",
 CellID->274981366],
Cell[123417, 2541, 259, 8, 36, "Output",
 CellID->24842389]
}, Open  ]],
Cell[123691, 2552, 237, 5, 70, "ExampleText",
 CellID->328772922],
Cell[CellGroupData[{
Cell[123953, 2561, 353, 12, 70, "Input",
 CellID->762946865],
Cell[124309, 2575, 260, 8, 36, "Output",
 CellID->154679881]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[124618, 2589, 221, 7, 70, "ExampleSection",
 CellID->1293636265],
Cell[CellGroupData[{
Cell[124864, 2600, 252, 7, 70, "ExampleSubsection",
 CellID->421192202],
Cell[125119, 2609, 293, 7, 70, "ExampleText",
 CellID->117609193],
Cell[125415, 2618, 154, 5, 70, "Input",
 CellID->559405563],
Cell[125572, 2625, 159, 4, 70, "ExampleText",
 CellID->299806311],
Cell[CellGroupData[{
Cell[125756, 2633, 555, 18, 70, "Input",
 CellID->249332617],
Cell[126314, 2653, 5843, 103, 70, "Output",
 CellID->906358222]
}, Open  ]],
Cell[132172, 2759, 84, 1, 70, "ExampleText",
 CellID->27068391],
Cell[132259, 2762, 438, 13, 70, "Input",
 CellID->343427282],
Cell[132700, 2777, 120, 3, 70, "ExampleText",
 CellID->126580234],
Cell[CellGroupData[{
Cell[132845, 2784, 312, 10, 70, "Input",
 CellID->274596420],
Cell[133160, 2796, 6005, 106, 70, "Output",
 CellID->24518823]
}, Open  ]],
Cell[139180, 2905, 157, 4, 70, "ExampleText",
 CellID->33075010],
Cell[CellGroupData[{
Cell[139362, 2913, 507, 15, 70, "Input",
 CellID->252090154],
Cell[139872, 2930, 259, 8, 36, "Output",
 CellID->189895069]
}, Open  ]],
Cell[140146, 2941, 281, 6, 70, "ExampleText",
 CellID->511124957],
Cell[CellGroupData[{
Cell[140452, 2951, 1071, 32, 70, "Input",
 CellID->59705953],
Cell[141526, 2985, 4780, 92, 345, "Output",
 CellID->118437511]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[146367, 3084, 223, 7, 70, "ExampleSection",
 CellID->2061341341],
Cell[CellGroupData[{
Cell[146615, 3095, 227, 7, 70, "ExampleSubsection",
 CellID->30521753],
Cell[146845, 3104, 530, 9, 70, "ExampleText",
 CellID->120830056],
Cell[CellGroupData[{
Cell[147400, 3117, 734, 23, 70, "Input",
 CellID->210128799],
Cell[148137, 3142, 591, 11, 70, "Message",
 CellID->285829413]
}, Open  ]],
Cell[148743, 3156, 288, 7, 70, "ExampleText",
 CellID->937232338],
Cell[CellGroupData[{
Cell[149056, 3167, 307, 10, 70, "Input",
 CellID->98885038],
Cell[149366, 3179, 248, 8, 39, "Output",
 CellID->24909326]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[149663, 3193, 239, 7, 70, "ExampleSubsection",
 CellID->233233938],
Cell[149905, 3202, 611, 10, 70, "ExampleText",
 CellID->381223408],
Cell[150519, 3214, 307, 6, 70, "ExampleText",
 CellID->507899588],
Cell[CellGroupData[{
Cell[150851, 3224, 1967, 57, 70, "Input",
 CellID->49740593],
Cell[152821, 3283, 6215, 109, 70, "Output",
 CellID->9889717]
}, Open  ]],
Cell[159051, 3395, 164, 4, 70, "ExampleText",
 CellID->520516665]
}, Closed]],
Cell[CellGroupData[{
Cell[159252, 3404, 226, 7, 70, "ExampleSubsection",
 CellID->104403840],
Cell[159481, 3413, 620, 15, 70, "ExampleText",
 CellID->391229248],
Cell[CellGroupData[{
Cell[160126, 3432, 938, 30, 70, "Input",
 CellID->451142620],
Cell[161067, 3464, 250, 8, 39, "Output",
 CellID->632083273]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[161378, 3479, 231, 7, 70, "ExampleSection",
 CellID->1305812373],
Cell[161612, 3488, 412, 8, 70, "ExampleText",
 CellID->416625656],
Cell[162027, 3498, 206, 4, 70, "ExampleText",
 CellID->148664056],
Cell[CellGroupData[{
Cell[162258, 3506, 311, 10, 70, "Input",
 CellID->112529997],
Cell[162572, 3518, 5180, 92, 70, "Output",
 CellID->342316319]
}, Open  ]],
Cell[167767, 3613, 150, 4, 70, "ExampleText",
 CellID->390242666],
Cell[CellGroupData[{
Cell[167942, 3621, 212, 7, 70, "Input",
 CellID->238007975],
Cell[168157, 3630, 345, 12, 36, "Output",
 CellID->261210594]
}, Open  ]],
Cell[168517, 3645, 151, 4, 70, "ExampleText",
 CellID->25666953],
Cell[CellGroupData[{
Cell[168693, 3653, 601, 18, 70, "Input",
 CellID->592619],
Cell[169297, 3673, 263, 9, 36, "Output",
 CellID->573096007]
}, Open  ]],
Cell[169575, 3685, 119, 3, 70, "ExampleText",
 CellID->722709344],
Cell[169697, 3690, 156, 4, 70, "ExampleText",
 CellID->492689119],
Cell[CellGroupData[{
Cell[169878, 3698, 2153, 60, 70, "Input",
 CellID->266881112],
Cell[172034, 3760, 104644, 1771, 70, "Output",
 CellID->140568297]
}, Open  ]],
Cell[276693, 5534, 484, 8, 70, "ExampleText",
 CellID->38752132],
Cell[277180, 5544, 155, 4, 70, "ExampleText",
 CellID->254018838],
Cell[CellGroupData[{
Cell[277360, 5552, 1000, 31, 70, "Input",
 CellID->230503503],
Cell[278363, 5585, 258, 8, 36, "Output",
 CellID->116828970]
}, Open  ]],
Cell[278636, 5596, 114, 3, 70, "ExampleText",
 CellID->88671999],
Cell[278753, 5601, 86, 1, 70, "ExampleText",
 CellID->211503654],
Cell[CellGroupData[{
Cell[278864, 5606, 1673, 45, 70, "Input",
 CellID->78818121],
Cell[280540, 5653, 76331, 1296, 70, "Output",
 CellID->766448454]
}, Open  ]],
Cell[356886, 6952, 238, 5, 70, "ExampleText",
 CellID->667775740]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[357173, 6963, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[357593, 6977, 662, 22, 70, "SeeAlso",
 CellID->67651628],
Cell[358258, 7001, 453, 15, 70, "SeeAlso",
 CellID->775123367]
}, Open  ]],
Cell[CellGroupData[{
Cell[358748, 7021, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[359170, 7035, 190, 4, 70, "Tutorials",
 CellID->341631938],
Cell[359363, 7041, 167, 3, 70, "Tutorials",
 CellID->499723042],
Cell[359533, 7046, 160, 3, 70, "Tutorials",
 CellID->336290191],
Cell[359696, 7051, 167, 3, 70, "Tutorials",
 CellID->307191390],
Cell[359866, 7056, 177, 4, 70, "Tutorials",
 CellID->351529306]
}, Open  ]],
Cell[CellGroupData[{
Cell[360080, 7065, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[360502, 7079, 149, 3, 70, "MoreAbout",
 CellID->524805846]
}, Open  ]],
Cell[360666, 7085, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

