(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    366271,       7511]
NotebookOptionsPosition[    354066,       7108]
NotebookOutlinePosition[    356235,       7161]
CellTagsIndexPosition[    356148,       7156]
WindowTitle->PMDoubletQuadraticDirichlet - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"3.1 Introduction To Panel Methods\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods"], "\<\"3.2 \
Validation Example\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/ValidationExample"], "\<\"3.3 \
Implementation\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/Implementation"], "\<\"3.4 Flow \
Visualization\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/FlowVisualization"], "\<\"3.5 \
Convergence and Timing\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/ConvergenceAndTiming"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PMDoubletConstantDirichlet\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PMDoubletConstantDirichlet"], \
"\<\"PMVortexLinearNeumann\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PMVortexLinearNeumann"], \
"\<\"ForceAndMoment\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ForceAndMoment"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PanelMethods\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/PanelMethods"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/ref/PMDoubletQuadraticDirichlet\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/PMDoubletQuadraticDirichlet"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
PMDoubletQuadraticDirichlet.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/PMDoubletQuadraticDirichlet", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["PMDoubletQuadraticDirichlet", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"PMDoubletQuadraticDirichlet", "[", 
       RowBox[{
        SubscriptBox[
         StyleBox["r", "TI"], 
         StyleBox["p", "TI"]], ",", 
        StyleBox["\[Alpha]", "TR"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]implements a panel method using doublet panels of \
quadratically varying strength and internal Dirichlet boundary conditions.  \
Panel endpoints describing the discretized airfoil are located at ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     ", and the angle of attack of the airfoil is ",
     Cell[BoxData[
      StyleBox["\[Alpha]", "TR"]], "InlineFormula"],
     ".  The function returns three lists of strength parameters, usually \
denoted by ",
     Cell[BoxData[
      StyleBox["a", "TI"]], "InlineFormula"],
     ", ",
     Cell[BoxData[
      StyleBox["b", "TI"]], "InlineFormula"],
     " and ",
     Cell[BoxData[
      StyleBox["c", "TI"]], "InlineFormula"],
     ", for the doublet density function, the strength parameter for the \
constant-strength wake panel, locations of collocation points and a list of \
surface speeds at collocation points."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->49529734],

Cell[TextData[{
 "Collections of vectors are of the data type ",
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], "InlineFormula"],
 "."
}], "Notes",
 CellID->758974],

Cell[TextData[{
 "The discretized airfoil is specified by",
 " ",
 Cell[BoxData[
  StyleBox["N",
   FontSlant->"Italic"]], "InlineFormula"],
 " ",
 "straight-line panels, where the doublet density on each panel varies \
quadratically with distance from the beginning of the panel.\t\t\t\t\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["\[Mu]", "TR"], 
    RowBox[{"(", 
     SubscriptBox[
      StyleBox["x", "TI"], 
      StyleBox["i", "TI"]], ")"}]}], "=", 
   RowBox[{
    SubscriptBox[
     StyleBox["a", "TI"], 
     StyleBox["i", "TI"]], "+", 
    RowBox[{
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["i", "TI"]], " ", 
     SubscriptBox[
      StyleBox["x", "TI"], 
      StyleBox["i", "TI"]]}], "+", 
    RowBox[{
     SubscriptBox[
      StyleBox["c", "TI"], 
      StyleBox["i", "TI"]], " ", 
     SuperscriptBox[
      SubscriptBox[
       StyleBox["x", "TI"], 
       StyleBox["i", "TI"]], "2"], " "}]}]}]], "InlineFormula"],
 "  for  ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["i", "TI"], "=", "1"}], ",", "2", ",", "3", ",", "...", ",", 
   RowBox[{
    StyleBox[
     ButtonBox["",
      BaseStyle->"Link",
      ButtonData->"paclet:ref/"],
     FontSlant->"Italic"], " ", 
    StyleBox["N",
     FontSlant->"Italic"]}]}]], "InlineFormula"],
 "\t\t\t\tStrength parameters are denoted by ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["i", "TI"]]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["b", "TI"], 
   StyleBox["i", "TI"]]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["c", "TI"], 
   StyleBox["i", "TI"]]], "InlineFormula"],
 "; the panel-local coordinate is denoted by ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["x", "TI"], 
   StyleBox["i", "TI"]]], "InlineFormula"],
 "."
}], "Notes",
 CellID->475782397],

Cell[TextData[{
 "There are three unknown strength parameters for each airfoil panel.  \
Strength parameters for the discretized airfoil are grouped as lists.\t\t\t\t\
\t\t\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["a", "TI"], "=", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["1", "TR"]], ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["2", "TR"]], ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["3", "TR"]], ",", "...", ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["N", "TI"]]}], ")"}]}]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  RowBox[{
   StyleBox["b", "TI"], "=", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["1", "TR"]], ",", 
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["2", "TR"]], ",", 
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["3", "TR"]], ",", "...", ",", 
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["N", "TI"]]}], ")"}]}]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  RowBox[{
   StyleBox["c", "TI"], "=", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      StyleBox["c", "TI"], 
      StyleBox["1", "TR"]], ",", 
     SubscriptBox[
      StyleBox["c", "TI"], 
      StyleBox["2", "TR"]], ",", 
     SubscriptBox[
      StyleBox["c", "TI"], 
      StyleBox["3", "TR"]], ",", "...", ",", 
     SubscriptBox[
      StyleBox["c", "TI"], 
      StyleBox["N", "TI"]]}], ")"}]}]], "InlineFormula"]
}], "Notes",
 CellID->12242158],

Cell[TextData[{
 "Additionally, a constant-strength doublet panel of unknown strength ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["w", "TI"]]], "InlineFormula"],
 " extends from the trailing edge of the airfoil to a point far downstream on \
the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis.  This panel is needed to apply a Kutta condition."
}], "Notes",
 CellID->342576209],

Cell[TextData[{
 "Boundary conditions at collocation points provide",
 " ",
 Cell[BoxData[
  StyleBox["N",
   FontSlant->"Italic"]], "InlineFormula"],
 " ",
 "equations, continuity of the value and slope of the doublet density between \
panels (except at trailing edge) provide ",
 Cell[BoxData[
  RowBox[{"2", " ", 
   RowBox[{"(", 
    RowBox[{
     StyleBox["N",
      FontSlant->"Italic"], " ", "-", "1"}], ")"}]}]], "InlineFormula"],
 " equations, and the Kutta condition provides 2 equations (zero circulation \
and zero circulation density at trailing edge).  The additional equation \
needed is generated by adding a collocation point interior to the airfoil \
near the trailing edge.  The system of linear algebraic equations is solved \
by the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function LinearSolve."
}], "Notes",
 CellID->42056900],

Cell[TextData[{
 "Output of PMDoubletQuadraticDirichlet is 3 lists of strength parameters, ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  StyleBox["c", "TI"]], "InlineFormula"],
 ", strength of the wake panel, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["w", "TI"]]], "InlineFormula"],
 ", a collection of vectors locating collocation points, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["c", "TI"]]], "InlineFormula"],
 ", and a list of surface speeds just outside the airfoil near collocation \
points,  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[LeftBracketingBar]", 
    SubscriptBox["V", "c"], "\[RightBracketingBar]"}], TraditionalForm]], 
  "InlineMath"],
 "."
}], "Notes",
 CellID->452935176],

Cell[TextData[{
 "If a list of angles of attack is input instead of a single angle of attack, \
lists of ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  StyleBox["c", "TI"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["w", "TI"]]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[LeftBracketingBar]", 
    SubscriptBox["V", "c"], "\[RightBracketingBar]"}], TraditionalForm]], 
  "InlineMath"],
 " are returned."
}], "Notes",
 CellID->18491652],

Cell[TextData[{
 "The same example airfoils and computations are used in the documentation \
for ",
 ButtonBox["PMDoubletConstantDirichlet",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PMDoubletConstantDirichlet"],
 " and ",
 ButtonBox["PMVortexLinearNeumann",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PMVortexLinearNeumann"],
 ", providing a convenient way to compare results among the three \
panel-method functions in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 "."
}], "Notes",
 CellID->523298526],

Cell["The following options can be given: ", "Notes"],

Cell[BoxData[GridBox[{
   {Cell["      ", "TableRowIcon"], 
    ButtonBox["Epsilon",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/Epsilon"], 
    RowBox[{"10.", "^", 
     RowBox[{"-", "10"}]}], Cell["\<\
offset collocation points normal to panels to insure that they are inside \
discretized airfoil\
\>", "TableText"]},
   {Cell["      ", "TableRowIcon"], 
    ButtonBox["InterpolationOrder",
     BaseStyle->"Link",
     ButtonData->"paclet:ref/InterpolationOrder"], "1", Cell["\<\
specifies order of interpolation fitting velocity potential for computing \
surface speeds\
\>", "TableText"]},
   {Cell["      ", "TableRowIcon"], 
    ButtonBox["XWake",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/XWake"], 
    RowBox[{"     ", 
     RowBox[{"10.", "^", "10"}]}], Cell[
    "locates the downstream endpoint of the wake panel.", "TableText"]}
  }]], "3ColumnTableMod",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, 
   "Rows" -> {None, None, {None}}, "RowsIndexed" -> {}},
 GridBoxDividers->{"Rows" -> {{True, True, True, True}}}},
 CellID->14699597],

Cell["New in 1.1   |   Last modified in 1.2", "Notes",
 CellID->918694056]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->580140197],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->250878843],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->77267041],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->326492779],

Cell["\<\
This example uses the Eppler-205 airfoil specified by a data file located in \
the folder Airfoil Geometry.\
\>", "ExampleText",
 CellID->301361980],

Cell[BoxData[
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<Aerodynamics\\\\Airfoil Data\\\\e205.dat\>\"", "]"}]}], 
  ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->317289154],

Cell["\<\
Here is the number of points in the data file specifying the shape of the \
airfoil.\
\>", "ExampleText",
 CellID->136872357],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NumberOfVectors", "[", "rD", "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->720666783],

Cell[BoxData["61"], "Output",
 ImageSize->{18, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->132601626]
}, Open  ]],

Cell["Specify an angle of attack for the airfoil.", "ExampleText",
 CellID->20057380],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Alpha]", "=", 
   RowBox[{"6.", " ", "Degree"}]}], ";"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->233544807],

Cell[TextData[{
 "Solve the model.  Output consists of the lists of strength parameters ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  StyleBox["c", "TI"]], "InlineFormula"],
 ", the strength of the wake panel, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["w", "TI"]]], "InlineFormula"],
 ", locations of collocation points, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["c", "TI"]]], "InlineFormula"],
 " and the fluid speed associated with collocation points, but just outside \
the airfoil ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[LeftBracketingBar]", 
    SubscriptBox["V", "c"], "\[RightBracketingBar]"}], TraditionalForm]], 
  "InlineMath"],
 "."
}], "ExampleText",
 CellID->442126641],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c", ",", "aW", ",", "rC", ",", "VMagC"}], 
    "}"}], "=", 
   RowBox[{"PMDoubletQuadraticDirichlet", "[", 
    RowBox[{"rD", ",", "\[Alpha]"}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->439469742],

Cell[TextData[{
 "Flow visualization provides a plausibility check for model results.  The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function ContourPlot is used to plot streamlines for the flow field.  The \
steam function for the flow is obtained by the superposition of the stream \
function for uniform flow and the stream function due to the collection of \
doublet panels describing the discretized airfoil and wake.  Although the \
strength of the wake panel is included in the output of \
PMDoubletQuadraticDirichlet, the location of the endpoint of the wake panel \
is the value of the option XWake for the function.  The default value was \
used above to solve the model."
}], "ExampleText",
 CellID->205450438],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"xW", "=", 
  RowBox[{"XWake", "/.", 
   RowBox[{"Options", "[", 
    RowBox[{"PMDoubletQuadraticDirichlet", ",", "XWake"}], "]"}]}]}]], "Input",
 CellLabel->"In[6]:=",
 CellID->544103304],

Cell[BoxData["1.`*^10"], "Output",
 ImageSize->{54, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->11363613]
}, Open  ]],

Cell["\<\
Append the wake panel to the airfoil panels and the strength of the wake \
panel to the lists of strength parameters for the airfoil panels.\
\>", "ExampleText",
 CellID->59684772],

Cell[BoxData[{
 RowBox[{
  RowBox[{"rAll", "=", 
   RowBox[{"Append", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"{", 
      RowBox[{"xW", ",", "0."}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"aAll", ",", "bAll", ",", "cAll"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Append", "[", 
      RowBox[{"a", ",", "aW"}], "]"}], ",", 
     RowBox[{"Append", "[", 
      RowBox[{"b", ",", "0.0"}], "]"}], ",", 
     RowBox[{"Append", "[", 
      RowBox[{"c", ",", "0.0"}], "]"}]}], "}"}]}], ";"}]}], "Input",
 CellLabel->"In[7]:=",
 CellID->509204874],

Cell[TextData[{
 "Specify an arbitrary field point using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation for a single vector.  The stream function for the doublet panels \
can be written as dot product of geometric stream-function influence \
coefficients and the appropriate list of strength parameters."
}], "ExampleText",
 CellID->19233943],

Cell[BoxData[{
 RowBox[{
  RowBox[{"rF", "=", 
   RowBox[{"{", 
    RowBox[{"x", ",", "z"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Psi]A", ",", "ic\[Psi]B", ",", "ic\[Psi]C"}], "}"}], "=", 
   RowBox[{"ICPsiDoubletQuadratic", "[", 
    RowBox[{"rF", ",", "rAll"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Psi]", "=", 
   RowBox[{
    RowBox[{"UniformFlowPsi", "[", 
     RowBox[{"rF", ",", "\[Alpha]"}], "]"}], "+", 
    RowBox[{"ic\[Psi]A", ".", "aAll"}], "+", 
    RowBox[{"ic\[Psi]B", ".", "bAll"}], "+", 
    RowBox[{"ic\[Psi]C", ".", "cAll"}]}]}], ";"}]}], "Input",
 CellLabel->"In[8]:=",
 CellID->555628545],

Cell["\<\
Generate the graphics for a streamline plot for flow over the airfoil.  Some \
experimentation with option values were made to obtain the plot below.  The \
number of contours plotted was used to avoid a streamline very near the \
airfoil, PerformanceGoal->\"Speed\" was used to obtain a contour plot in \
reasonable time, and MaxRecursion was set at a minimum value to avoid \
noticeable numerical artifacts in computed streamlines.\
\>", "ExampleText",
 CellID->260053747],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"gStreamlines", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"-", ".5"}], ",", "1.5"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", 
       RowBox[{"-", "0.4"}], ",", "0.4"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"Contours", "\[Rule]", "19"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "False"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "3"}], ",", 
     RowBox[{"PerformanceGoal", "\[Rule]", "\"\<Speed\>\""}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"grD", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gStreamlines", ",", "grD", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", "\[Times]", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[9]:=",
 CellID->130177],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJx0nQd0lcX2t8UGAnaKKKgggqKoqIgI17kiFhQEUSyIikhR8apYsKGC0lRQ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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{5206, 7668, 5207, 5133, 7209, 7210, 6546, 7714, 6545, 7715, 
          4953, 6868, 4952, 5768, 6149, 6148, 5519, 6090, 5520, 7685, 5521, 
          5131, 7207, 5132, 7208, 7468, 5011, 5477, 7683, 5478, 5414, 7645, 
          4944, 7054, 4945, 7712, 7713, 7646, 6525, 6523, 5117, 6524, 5832, 
          5833, 5118, 6343, 7043}]},
        RowBox[{"-", "1.44`"}]],
       Annotation[#, -1.44, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{6169, 6170, 5822, 5823, 5112, 6318, 6319, 6320, 6544, 6543, 
          4931, 5764, 5423, 7059, 7060, 7061, 7551, 7550, 4951, 7623, 5716, 
          7622, 5717, 7624, 5705, 6143, 5422, 6054, 5887, 5888, 5400, 6765, 
          5399, 7382, 4875, 6531, 6532, 6530, 6534, 6533, 5031, 6178, 6526, 
          6527, 6529, 6528, 5834, 5835, 5120, 6344, 7044}]},
        RowBox[{"-", "1.488`"}]],
       Annotation[#, -1.488, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{5896, 5794, 4940, 6452, 6453, 6451, 6536, 6535, 7195, 5119, 
          7194, 7197, 7196, 5033, 6261, 5032, 6407, 5121, 6537, 4876, 5756, 
          5401, 6340, 6341, 6342, 5891, 6822, 6823, 6447, 5081, 6258, 6259, 
          6744, 6745, 6327, 7037, 4933, 5886, 6326, 6325, 6324, 5391, 5825, 
          5824, 6505, 5111, 6504, 6507, 6506, 5025, 6400, 6402, 6401, 6508, 
          4870, 7378, 7379, 6741, 5389, 6742, 5380, 6041, 6321}]},
        RowBox[{"-", "1.536`"}]],
       Annotation[#, -1.536, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ctL1GEUxvFjCzU1L2OaeUujCwlCILWyv0DahIucERfunfEyo+NouirB
pRC0zTa1FUQCsUWrUklBkFrpP+BlETnmws8svjzvc855zvvy+3WPZl6lyyLi
PUaaIk7bI57RE/qcntHk7YgU9jsiviUifjdHbNE9fsXMzp2IbYw0RhyrFdVS
zkMyf/nXtFc/qXbE9+kvtkbM4b7zLO2mBfrYXK25hZaIN9jFDl6qzejnkUOX
+Wn6yPy8/hzq+K/uKrpjm19qi8iq3TM7STvpFH0oU9CfRRW/IXMh85N/JzOh
1mE2Q9vpOH0gk9efQTm/JvNP5gf/VmbA+6rvRtSgElW4iWXc0jv0HVvtSsu2
0Qr7yvGZ/4Kn5v6Yy9o3jZTaMDadF+xfRJn5G2iQT+rV0wN7v/sfGflxpDGG
LrnV0j7nHCYwhUlUqO/LVcsP2lNDr+z4j1W+wH8s/R/6ie6ZL7r/Ej3yT/BC
7YO39NNc6Tvj3HzON/mltm5XwhuuAfUYVGI=
          "]]},
        RowBox[{"-", "1.584`"}]],
       Annotation[#, -1.584, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0TlPVGEUgOFDwrgEZsQBRFlM6ExsgE5JnIbgRIeGYNhFiLTYYUSwB3pU
WhiwcVRQwB8ADVsoKIh/QRoSlyAID8Wb96z3ft+99UMv2keKImIGv8ojem9F
/OY+3ub+mxEd4oO6iLN0xBNxqG/V6mMTu9jBWUVEtjqiqDKilYMf8uqNiDUk
5POel8cCjtUynjVaE7Eoz5l9jGHxc+zp3fW+vNo8Ts23qC3rLSHleZ3qY+qv
caz3yvw/HuM5rjc3Kq65HfHd+V+KN8zuO2/a/oD9ck5URRQja6fRTBNOnesE
P9Tu2E+KUyi2U3DXIzOfOCF/xN32v/A6f+Yuvmb+iv6y/DJ/5asX9+Re/RXe
4G/cw2XmS/XX5CWcQhI5+VP9AUw5bz/34aM7XddPowwVF/dBm/ln+oOYNv/B
/8u49wI/4EUu2L3P4/r3eIKbuYH/8x9+6zwzaBT/9M0m7R5yk/yd+nvM4q98
Wm8Kl3zzN/IV7+6WnwP41lVm
          "]]},
        RowBox[{"-", "1.6320000000000001`"}]],
       Annotation[#, -1.6320000000000001`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0DtPlFEUheGNiICxQkCuEqBVQqj4BZoh4SIkWDFxhsuMDNcOOrCBinBr
ycyAmqA9AQygQkdC+CtGgpJY+FC8WWfvvdY+5/ta07ODMyURsYaO6ohifcQe
dnH5NOK2MeIP2tVt6Nb7VRsx2BQxhLzeS7kCvTIrNEQs1ujTBfqXt9O8R12q
TtAH9Fq2hD8tN4oUxjGGHf2WuogR54bnEYdVEUnnJ80RK/Kr6EU/+tBl/xta
Ye8ALaeL7l24R+7OvmP7vuG1c17/FT1UH2ECL/gyNIt/99/B85H2P4v4RIvq
Jf/hxDzpjhHsu3fdd3yhv+UTNM9f7f4LmpMp0CLeevswDuRO7diQ28RXmRvZ
Sb0cWnlTPDV2vKO1NE2X3X1mviWzjTuZKfU0KryvTW6MbxyjyGACCbPvPD8w
g1lU6rXzvzefRBZTyKHH7CfPOeYwj8d6W+57iDKU4rPeI/rBu/4DElBPQg==

          "]]},
        RowBox[{"-", "1.68`"}]],
       Annotation[#, -1.68, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0TlPlFEUxvEDlFLBsCsJFUaRhGBDgmInDQgIg1oIsYDQaALfgYDOlChU
yiZhbfgGBKNYWIAiCWolPSbsDMJvin+eszzn3Pvet+rFq46XORHxDksFEZ0l
EV0YLYropkk0JyJyyyPycFkWkUMDi/xvr5vFGCYwjmX+C77/OEMG51jgb7Ov
HSP2P6YdeMh/rH+CQyzJj2h+RcQ1VFdG/Cy0Q7xDa/XS4s/FEdvyeTt+0Fn1
j+h2hyTu2tNJu/Cb/w/SfKv8G85/ciPiGZ4ijzdtNoWB0ojvfHfE9eq39G9j
1/wvpPRa7djX/4c3yOX7Zi6l/8m9tnjmeDbpurxCb973r4mLxUVIoM5cod0J
FKDdvV5721Es6Pfb3Zd9E/SgF1Pq07hv9zDfjFo1buK5eg+a1B9gUjzkTg28
x87+IH+fndW7yP4nZ+7JV3j+0oz8TP2LNxtU2zdTI7/Hn6GN9Jxu6J/6nkfO
bEMrWpC046uzSvXLUIID+RVteV9o
          "]]},
        RowBox[{"-", "1.728`"}]],
       Annotation[#, -1.728, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0jtsjmEYxvEbPRto6/t6bkksiJ2KUoN2cGgr6hADrcmgNUkw0CqDKENr
oVWJGJSFtgy0X5XBKXFKVaoWCXFYisXg8PuGf67ncF/XfT9v3iUtbU0H50TE
G2wojHhQETGBcTzCQ9QnI/qLIvqwGJnqWosj8hIRR8oijuIYfqtbpb67POIc
1jqrQYb6Sr55tIUvl+8mfVEa8RKv8Qq17u/zj+IeUhhDndxL/BdRhbnq9vHv
xQ1UVUYM0m4ZZ7GUZ48eT/V+hue4xdfB98T6MarMV4mMRd5EM2l3ScRM2i+v
Xe0H6zZ6Xv98DMq4jiv8A/in54D7P/Qy/UvL5MzQ95hGgX2v3FP8p1Fjrln9
1tEf6W9Ef9Fa+pOup53mz1DbQU+gUK9++XXu67ERpXLfyZ/CWyyw79Gni++4
uavlTKbvrD/K2M8/Zd1Kk/ZFSKBJXbU3rcaw+wn7C3IS8r7xf0exuhKMmOMO
7qJMn170IEdNim8c0wUR2Wq3WGfRrTSHbqZ5dD5y0ZD+d8xyUs8udKIDX2Rt
cldIr8kakX+V5tsvxGd8wmHeXebcjWbsxA5Mqi03XwUa7behCcP2Q/jKO6v/
Cu9r9B2znGXjtj5DWO68wfkyutJ/cMZMzeY5pKYdbUi6T+AA1vhuKWdjGMV2
tX3p/xX/AaBwecI=
          "]]},
        RowBox[{"-", "1.776`"}]],
       Annotation[#, -1.776, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV09dvzWEcx/GnVozQQYdWSykq9ooGF221EiMkbf0BXKnRWgkiiJBYiUS0
hLaIWRdOJYTQGhEjaC/EFjT2TkisC+v1u3ifz/N89/M952TPqSypiAshpPuo
TwrhfEoInTNC6ISOGN4jhB9s3zGhewj9e4YwADkoTg+hCDd7hXBG7jwxG5ND
2JUaQl5WCDvpZPk3M0O4hRtoQTNmiG2gJxDDTPdq8afVSdI3Ky2EWrVrUKzG
Uf2KaD39Jna8WRppE6biHM4iTV4qNqqZ4F6n1l7sQ736FfIe8yfxHWE7imVs
X/mW0j3yarAbdajFSLGjkeJNo+gYpDqPjd4gp1WNGJ1iD5lmfOj+APeRxzYO
uc7v8QEHcQCL5RTzLaGH3Q9hmPtQbFe7Su9q3GG/i3u4bc59dpzPvp8W0At6
zmO/bneL5BWi3H0uZvOl2WFPpKIUJViHQvlx3jGJbjPDEH0TxQ+mCbSc7ZW4
XPVe0kH0Nd0hvkXtKtpBfjVtEl/JX4HffI3uI+hwbLDv9UiWm4JjzvXIj/bN
v9K7VmA5OqhV5k1X6ED35970ll4U+4Zeou/oKprgt5CIeGSr2weHzP5ETrM5
WvBL/Z94Ln4SXxHyUYgCxLP/oX/xG0Hvf3Sh+ac5T0eWfr2j71Tdp2o+pNnu
fVAudi4qxLcT2x5xaIs2GIRuiEdXJEa/S1SKP2neU9F+7SJHrVZ1n6n/jPZz
74tSsSW4bB+fxH7Ge3zEBySafzN/L3mbaCbdQq+Kf+nd12gX39F1mqJnzK4W
0eN0gbnnY6K4tdF/wBx58h/p+xhb2V7wzYrqRnvAWfZzWMN3gu+8cxMaUabH
FzVW88X4GlDKlh7NhwwcVP8/2/ui7g==
          "]]},
        RowBox[{"-", "1.824`"}]],
       Annotation[#, -1.824, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV02lMz3EcwPFvbg/QJNKh2YQH7q0aCvEAW8Y8yV3R5Z4nPGHSQw88dE5R
zLCp0ENHNUQ5ZmsrOTs2Q64wc8Tr/+Ddu8/3+7l+v35N3Lx79a6oEMJQP1Jj
QjgyLoQ0LkkIoRhfx4RwJzGEQ2NDKMNncWp8CF84jadw+fgQpvKIuBBGomJ0
CKO4kgfrMQgNqEep3t1m9KApNoS2pBA65b3Su4u7nW13thM70CXeyttQhBIU
o1Fus7uNfm/hTfyA8/kR5/JDzuPXclPMnoxee33CR+y3y3FzTyAbx3AUT/EI
FWrPoBzp4h92/o7D6mZ7J3MwEzGefRa3m5PgndwTx3MTJ/JdbuFO+zTzGy7U
s17+BPED90nczMl8nyv1q0IFzuIMXsg/Z4/nfp+kRwen8Dk7VWGlvQY66+EB
3M2l8pP5rft2ubf17+Cb/IxvcXTkedznqsk1+6K/Zx4/UfsYDeYO4VETfCc8
GMMRLR7GJepncqx4Fk/Dcc83I/I+eXrk3fKFyP6cIa7iTD7P87nQ/ALcMLeI
i7HK+XX3dajFNVxFkz5X7F1jz1pUY4+6BWoyI9+W+z7PtlfuN2501oBF+sXZ
MYtf2vmf7yHKM4fIuxUf9E2U4hAS9Tul70mUidM5LTLP3Da9lqtdhqVoNWOf
nr/MXaDXQsx3Pg9z0aL3Tz4rr1p9JddwgRmX9Czky7yF89TmI15+Apr1bNX7
gPzFWIIs9Mvtc/4VdfbJxgrkiHPs+1tdovgPrxWvwXsz3uG0+g/ci3XOM8z5
Ky9Jfj9vcLY+8n/i/iPK5f8HqvCkOA==
          "]]},
        RowBox[{"-", "1.872`"}]],
       Annotation[#, -1.872, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1FdszmEUx/FHSIzgQgetESS4log9orVCzCChhLSJmdSFETNxgZDUqhnq
SkWHKhUSW1EiEUVLeVUV0RqJNiVGzc//4tfvc875nXOe/v/v+/ZKXzEjs0UI
YaI/OZ1CKE4MoWf3EOYkhJCVHMJc3NnZme52DeEOlVFvnl50oEsI+2lMUgip
dEhPOQ2IC6FavUH/Z2qUG2f2ZxyLDTjQzEE0Ub3YzDN0lgbLTdF7UX4yXsKp
2GTmKmwUL+VbRstpN/8xd8ihr+Z+iQ9hj9wkvlpxkn3v1Up4z9Ertatq08y6
htMxXs8V53Y9QriMNTxr7XtLjepv8LD8IXqgdhCrzQ5mP8cXFKOufM/Vx+gZ
SyVyp3lXONdhJr7DVfpW0gXxdFzULYTFtMw9Wrrrckw3J4Pm8aRFz0FuLs88
nIPzMQ03q/W096+5Hd3/H/6iP9RB/Bsf2RHzfqt4r4tfm1tL6/Qlq1XaWUF1
4lZmDvYu3zsn0lb5en2nnDvId6Q+cnfMG+Jchjf9n7eolF6afxur8YMdO3i3
U424LHo/1EQFZk6yOz+aTYX0jWc9bqQNNNPOTTgL93p3e+iT+32kTXof8ndW
e2JPnnM+dRcXYgE9le8ifhTF+AxfmNfNOaZWLG7jGZ3B1XJ55q/BGvfNdp7t
fI9vpJ5R0T3dexdvEdar3fQOYvJJzpl8peJj7nYLj2AOHaUKzyjmvvt4+2I2
9sMveppoi/5aapB/hQlqce6ViM32vZFfENV5F2JfM9vZ0Qfb4g/8SfnuXECF
dIq3iCbob1YbH30usNJdas19TSVybe05Gz2X6F2ZXUSV9t3neyrfXr0Kc6m1
83F8zJvAd5DKeU/wPpYfbW8KpVIdTz1VyqfzVGAGJuupkk+NnqdcChZhvPt0
Mj8Oy81bFH0Hou8FZuBobOLbQsP1bMUR+F3+Fw71TprxuDiLPxfrfT5TMNu+
H3gDF5u/hH6Kh+npb+dJ55168vBD9FuG+3j307bo94b/vDuViv8DFsXdjQ==

          "]]},
        RowBox[{"-", "1.92`"}]],
       Annotation[#, -1.92, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0k1Mz3EcwPFvHDyOraInDjabqWRjhVM2G0KLsDng0MJBMQ7c6uThJPJU
UhtWDthCKclw8FBbyVIXWaj+PWmT2bgwr9/hvff38/D9fL+/7/+/rOh44bG4
EMIITi8KYXtSCA2JIUynhrAnOYS9SJNbnxbCOuwW16uPpYRwSn+Mb4t3yu9C
IVL15+jNRoH4lvqIvg36+8ztR598jXpRQghZ+g/q2b+EcQAl8j/kS3l4aQgL
F4dwEh/iQxjgHj7Bm1CBC6jDqNnpHOMMHuN65zZEZ5vTZd8n+W7uF9+V/+ic
Xhxx1ky1ezyDD3OPfK2eOixwx5tcg/fyh9Tj9F3hmFlX+Qx+qZ3lHLWf0Xfy
+Wim9Tm+IV5uXcbx4r/6E7hcvEK+3brP/VKidzO3UvwMU9ZtfEftEY+LH/NF
NGNC3MQv1Vv5u/gpX0ILJsVP+Ll6I4+JH3IFhvwWw/iCb/iKTn2v3aGDe/UW
Wa911zVIjM4Rv0CpuElPsfvno0R8FLlqGZyJffLpvBJtet/6Xd7hDTrRgUHv
OqGe7+wd0V3Exc7daJ2LAWf+41b5z/paeJb3qjT7MpKwJXoT+c3RW/BWnrZv
W/Tt4jxu5gH7H5h1H0NmJNs76A55atlqJe4+xV1y3ViNSbla/+MC/88s8YR4
FY/zK7lGM+bwXMzGfMxDtfN/O6OK/3C1GVWY9g7t6jH7M80Z5XJnluG6vmv4
DxvKl5A=
          "]]},
        RowBox[{"-", "1.968`"}]],
       Annotation[#, -1.968, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0UtsjFEUwPFjJxGvdvqYPiSMLgg7VNHEou1OE4OEdlfqEUwTwopF1aOP
FVqJRkosa2dDZ9BF67FSWqFsCFOPpNqkxdpvFv/87z33nHPPd7+1bR3pzJKI
yKKrJOJoaUQ7Rssi/iUirvNuzicj7nFfVcRd7uFevMQLZCoj6ioi/jqr5T+8
nZPqm8sj9uCLHoPi3fKvYUjsgpyP1p8K/fToQ3NxxH11PdbdmHaWMtsd65PO
2u1P6HOEj/MWuU3OGtGAcbEhZ+d8x3kM2F+xv8VX+QZf5n6+xDe5iyfcMV4U
MVltTuvbyJovh0cYwWPk5eyQvxN1qMcuPDPbjPufcs2aiHLrJJ7InzZHjsvs
U777gT7DOODun2p/YQY/8L3wvWZodf836038lTdznvepOcj7OaFfCUrx3h0f
kBZfVL+Xj6lJm+ez/VL7TutXchc5occp69OotV9nv159wuw1nMIbZ28xiTnv
fFafrfK24bXYBJr813n1jVwsXoSHaqfwDofRhln1g/JWOV+N5ViJFVjwNlnO
m3Gk8BacK7wD/1ab8R393quDB/i5PnPunkeVeyuxUeyiOzZwi/ghzMo/Y+Yx
sWF9lnn7/x9Zc7E=
          "]]},
        RowBox[{"-", "2.016`"}]],
       Annotation[#, -2.016, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0TtPVGEQgOExJJoAmiByUxbdgCQCDeEWuRggIlctpKE2oaFACdBRAi0l
u4JG/AEqWIuEEIFSG6VEG0BdSYwS0IKH4s17ZuabOfOdk3z0+OHouYhII5Mf
0V4S0YkOrIubiiOaPbegDa04uBzxpjRiGa+xcyVihT/zVFHE0LWIV+Iv4rqy
iO/iDbN+8E8UmVGIarn9woikc/3O38cAPsjXJyLGCvTzE844V+Hcmvoev+P3
WMVv+3Sbd2LXfzjGghn/edj7UpwxZ1Mu4Vy7uAPnzSnXf4EreFG9VL7HmVv2
rkYVDuVruFe+krfEh+Zun30fcZn8nHhQ/FH/JyTQJ1+i/tZ+eer94gFcRxI3
UKvnl7vdtMMDO1Ryq/uGvhZ+oTdHPU+cy9mYdZdLPM0XeYZHzPmrv0vPH77H
R3yXm9QnuIHHuZGXzH3Jt/UvcTOf+HfHmLdDN7dxL6fPvjenuIez3OWr3RfE
u7zI3/gO96k/51V+xumrEU+RwqR3nwLMB1fV
          "]]},
        RowBox[{"-", "2.064`"}]],
       Annotation[#, -2.064, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0DtMlFEQxfGBQiJ2PNwVhIAk1MBuT6KIihRI6yuhoFRCJyKPWAny2MZk
Vy2FWFAQsIFOKA0JoOCjsARZhAaBwhB+W/xzvpl75sz9bn3P0+4nRRGRQaIi
InU1IknTdK88oqs64h4afNfrZWoiJjGFCdRdjpimO5UR+UTELp1T/6F99Ius
D+ZmMYNa9bsrEReTEaX4aD5bFvG/KmJFf8T5ZzpKL8krRd75gtmfeo18v+iu
uR08cLf7yMi6wFOCv+7aKmNCb1H9CQ95HuEx9szlcU3Omjvm5M+r39MB3ud4
hhcYxBjfurzXdEg9TptqI4pkFWOzsEtvg07ScncuQ5v9N3AdU/pfnfd7l290
Wn1TP2X+yN4K/n/0ll47tnm28Mb/fqc/cFs/zX/MV8l/QvcL71X4Zzqm9wot
/j/Lm8MddOCUN+GsWV4LmpBGqgB/r9y3fMOyRjCK33be1etE0uw4Duw5o4d0
lWcFy95wCS+94TmXW1qh
          "]]},
        RowBox[{"-", "2.112`"}]],
       Annotation[#, -2.112, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0TtI1mEUgPHj4BC4qHnXTxTM8rpoaoZTNigoOFSE4aVGS7HLmpkp6Neg
LU6uDgaiQYaXKRuCdHA071ugQ5pz/Roenvdc3nP+7/eV9A92PUuJiFmk5Ubc
yYmYvhqxxuu4i1bcltvPizjALvbwE3MZEecFERf4jUv8QW1mxLh7NfxE/Bhn
WREfsyMW8E5t2d0+51H5MfFbNNuzZe42fqAyP6ICXwojTtR69fdgVe+xeJRv
8Xe9E/xK3zi/5LpExJHzinOv/TfMORSX8wFf53T3m7y7EZtmvND7jT+IZ1Dl
+0f0vMG+O5/V9/jMt6fJnXK12VV4rjaMmqKIAW9qMefU/SlzkiiWfyD/Ve8m
lty/L07KF3vTFCd4HkNqi2Y/dH6tZ0W8Ze8g1/MVrjRz2Y6ndn7iSXGhNydQ
hBK7y8ws5U77OvDLOVV/hxnnap3cLXfPnkdcZ1cD/qo1cp9cmJuC93pumpvk
Hd92LHeCIxziGhL//3eewIbfdx3t9rbhH5fcW9c=
          "]]},
        RowBox[{"-", "2.16`"}]],
       Annotation[#, -2.16, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwNz7kuhGEUh/HjCsjY10QjIcGYgsRacAVDNCpTaYwgllIsQUVIuAVboUOp
QUhwDQYFgsTSWX7Fk+d9z/c/5z1fbWYsnc2LiElsJSK+KiJGiiNyJRF3GKiM
6Ecau1URO2goi6jHaWFEojxiXc8a7uUfcFMUsSd3zfv8aO623KeeL3xgW+83
D5p7IHMrm6qJyDpPOY/xNB/pa+Xx0ogJzNltyBuHZk667zg/qY1XRzzzjNo0
6swv11uBMvza7wfhvT8+V5s394wX+ILbuFfvEh/zIifNbUEzRs2/lFtRb7d7
k1nNyJftcu9EB1Z9v5Lb9M4Gbpxv0e1bUv7V7gV63jjl3oJZ/zuDlHde/FOj
/TPqw8iZ16P3Xb5E3zJW0Cd3It/PR5zmY07q/Qf85Egv
          "]]},
        RowBox[{"-", "2.208`"}]],
       Annotation[#, -2.208, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVz7kuBGAUhuHjAijs+z5CLINEQoVWKREFo9BpCEbMmIS4AUulUNONRIKY
SlyCtbNViNrUHsWb7yzfOf9/2hZXppdLImIOx+UR+eqIM7RWRkw1RrTQrqaI
VFVEd0XEq9obXvCBd/TqZXiy2MSvPFcTcW7fgl278i15FmP23dZF7NBVs9s0
2RyRqI94Fqf451HgfZJn6Cgd4SkTD9dG3JjPqRW9dSCfbYjoNP+o1k4faAct
5f/2hyGeQexjDwl3pPU2cM+b9487+sU7qfZJe+xM8ha8dY1D+5bccSVO8/bp
9WMA42YmsK5+ob9GL2nRzJFdJ+45xY93Z/7/Sv8A8rE25g==
          "]]},
        RowBox[{"-", "2.2560000000000002`"}]],
       Annotation[#, -2.2560000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{5476, 5472, 7682, 5473, 5471, 7464, 5475, 7463, 5474, 4995, 
          6882, 5665, 6881, 5666, 6088, 6087, 5516, 7684, 5517, 5418, 5741, 
          5740, 6139, 5417, 6053, 5200, 5928, 6352, 6351, 5416, 6782, 5415, 
          6592, 5245, 6196, 6197, 6198, 5954, 6955, 6954, 6195, 5244, 7469, 
          5483, 6905, 6907, 6906, 7726, 5241, 7280, 5242, 6590, 5239, 6589, 
          5240, 6586, 6588, 6587, 4760, 6903, 6904, 6419, 6421, 6420, 4903, 
          5862, 6068, 5482}]},
        RowBox[{"-", "2.3040000000000003`"}]],
       Annotation[#, -2.3040000000000003`, "Tooltip"]& ], {}, {}}}], 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdk30wG3YYx5lT61Adxm3OMK509dbSpdrVd5ptZah5K/Uy0izGKMtWi3dV
VFALahKltqxa6qWaeK3L/NRwipY2Y7KezVuuWpWE9Mpym0629Z89d889z5/P
53Pfx+pkUgDjJQ0NjcNbrZ7/lgIvNhUtTxG7IEecebDvzuTnvUwz/VmHUjkM
pp67jXEpxEYmeMrXlSPd+PrRubQgsr3Igi4Lk8H3ARHer/uUNJ7197p0ZgWD
nL0sB6sk0npkxK8x8wkiSTQlIfw0KTCR3F0PXMZ0p12bx65Ucqln7UStzmM0
t0xHfe2XRSoklhfO1yxhqOgNn58UOcSe8fp8qOlDmDjnhLOUeeTOIyFvOkOK
Ff1e56q32WQgI1Xz2b0FVAr1mM3FxWT5+LiX3GweEc0Nt1z1OGShP6fUPnoW
y8V5nTXeZWSHUqo9fXQGtASrYY/tF4iiaDZ7j0SCxLj9W2gVxIJ0PRrPmoKp
aGmPp0slKZQPNJtSxVg9I9wdksIlB60/Ss91moBdd6h/bhyP3AspjbG/OAIp
jT/lKucRup3tr8GegwjIClgRr/IIoySN/SGrD1X6+24Pn+CR1d0aQy6x3Vg0
2FabbsAlV2kbbJ6hAIbroaoJ7W9JtiikxWfjCjT3dyS0OJUTo2pOPHWQh3Nh
YtGCcQlZzK290ziej/c+N7JdP5RHclWVbpEmdHT2esaO3jxNVmMX2B0bT92d
Ei9rirSPoPXlza5y83hoHBzjmkZnw/47aeoXrHPwYQZLWB+UvuCC6v1am0iX
KpgUuJCLWnX4R58FH94WPd/3Mq7jK+8mKybnCn4xXPMYj+mA5f2kKJ2717C4
/s5fEo4ITEsbXV5fC7Sox7vSjvWDflLJPNVwA6ydhwKXREPo2xX/M9VViDUd
rfEfC0aRXcfWa0ppA4X5JaH6T8CTZJof+KwdiT5+ofOaYoS4jypnZe1w+SOm
2kw8Ca98Qb6lTgeSFPm3g25Nw7zCWfystR2r9LFI7+EHKI/YpKpW2sDOV4v6
DZEf/76vflSInr2HaYU6cxAMaI1cjRCAEnReSeHPQ2/4E/lmSitozAPKJrdF
DAgct4w1o61DLVIKo+5kil9yA9THO/o/xLvrbzq6yerA72Uc4w4uoSGgTScs
hw/2q/2XS6wfQ53OoIIa5M7cOOUfswxfh/CtyFVCUT/kTSl8gg21JuMy1Pxg
ffPPghWUVc9ty8hiYzh9ciQvSobkbzQlyzPZKHuFruuxQw7thmuM+Ewm5riq
1MAyOerfem3I5GzEf3xySG0n3c05bvj///4NQNjiEA==
     "]]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  ImageSize->504,
  PlotRange->{{-0.5, 1.5}, {-0.4, 0.4}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 224},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->11642941]
}, Open  ]],

Cell["\<\
Note that the flow appears to leave the trailing edge smoothly (Kutta \
condition).  There are no streamlines inside the airfoil indicating that the \
computed velocity there is zero, as it should be.\
\>", "ExampleText",
 CellID->259606597],

Cell["\<\
The function ForceAndMoment computes airfoil lift and pitching-moment \
coefficients.\
\>", "ExampleText",
 CellID->80454523],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl", ",", "cm"}], "}"}], "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Doublet\>\"", ",", "a", ",", "b", ",", "c", ",", "rD", ",", 
    "\[Alpha]", ",", "rC"}], "]"}]}]], "Input",
 CellLabel->"In[10]:=",
 CellID->180266037],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.9894268143354401`", ",", 
   RowBox[{"-", "0.3104568232285334`"}]}], "}"}]], "Output",
 ImageSize->{149, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[10]=",
 CellID->843333845]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.9894268143354401`", ",", 
   RowBox[{"-", "0.3104568232285334`"}]}], "}"}]], "Input",
 CellLabel->"In[11]:=",
 CellID->719599677],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.9894268143354401`", ",", 
   RowBox[{"-", "0.3104568232285334`"}]}], "}"}]], "Output",
 ImageSize->{149, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->133391181]
}, Open  ]],

Cell["Locate the center of pressure.", "ExampleText",
 CellID->8410041],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"xCenP", "=", 
  RowBox[{
   RowBox[{"-", "cm"}], "/", "cl"}]}]], "Input",
 CellLabel->"In[12]:=",
 CellID->342608962],

Cell[BoxData["0.3137744184111842`"], "Output",
 ImageSize->{60, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->55149086]
}, Open  ]],

Cell["\<\
Note that the wake panel represents the starting vortex which should be equal \
and opposite to the circulation around the airfoil.  For the dimensionless \
formulation of the problem, the lift coefficient is just twice the \
circulation.\
\>", "ExampleText",
 CellID->527976917],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"2", "aW"}]], "Input",
 CellLabel->"In[13]:=",
 CellID->123809147],

Cell[BoxData[
 RowBox[{"-", "0.9895115971008345`"}]], "Output",
 ImageSize->{68, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[13]=",
 CellID->53295971]
}, Open  ]],

Cell["\<\
Surface pressure distribution is computed using Bernouilli's equation.\
\>", "ExampleText",
 CellID->753558053],

Cell[BoxData[
 RowBox[{
  RowBox[{"cp", "=", 
   RowBox[{"1", "-", 
    RowBox[{"VMagC", "^", "2"}]}]}], ";"}]], "Input",
 CellLabel->"In[14]:=",
 CellID->435840926],

Cell["\<\
The function PlotSurfacePressureCoefficient provides the conventional plot of \
surface pressure coefficient versus the chordwise locations of collocation \
points on the airfoil.  An inset showing the shape of the airfoil and the \
location of the center of pressure is included.\
\>", "ExampleText",
 CellID->139054009],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"xC", "=", 
   RowBox[{
    RowBox[{"Components", "[", "rC", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"PlotSurfacePressureCoefficient", "[", 
  RowBox[{"xC", ",", "cp", ",", 
   RowBox[{"Epilog", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Inset", "[", 
      RowBox[{
       RowBox[{"PlotAirfoil", "[", 
        RowBox[{"rD", ",", 
         RowBox[{"Epilog", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"EdgeForm", "[", "Black", "]"}], ",", "Yellow", ",", 
            RowBox[{"Disk", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"xCenP", ",", "0"}], "}"}], ",", ".012"}], "]"}]}], 
           "}"}]}], ",", 
         RowBox[{"Frame", "\[Rule]", "False"}], ",", 
         RowBox[{"PlotRange", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"0", ",", "1"}], "}"}], ",", "Automatic"}], "}"}]}]}], 
        "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", "1"}], "]"}], "}"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "1.1"}], ",", "2.2"}], "}"}]}]}], "]"}]}], "Input",
 CellLabel->"In[15]:=",
 CellID->671472236],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3V2IHddhB/BtWtNAU1tvdfIkPzTVUyvahzoLTQRpioUfHBVCDEKt6sJF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    "], {{0, 324}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 324},
  PlotRange->{{0, 504}, {0, 324}}]], "Output",
 ImageSize->{508, 324},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[15]=",
 CellID->7098188]
}, Open  ]],

Cell[TextData[{
 "If a list of angles of attack is specified, lists of ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  StyleBox["c", "TI"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["w", "TI"]]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["v", "TI"], 
   StyleBox["c", "TI"]]], "InlineFormula"],
 " are returned."
}], "ExampleText",
 CellID->571494656],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Alpha]List", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "5."}], ",", "0.", ",", "5."}], "}"}], "Degree"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "aList", ",", "bList", ",", "cList", ",", "aWList", ",", "rC", ",", 
     "VMagCList"}], "}"}], "=", 
   RowBox[{"PMDoubletQuadraticDirichlet", "[", 
    RowBox[{"rD", ",", "\[Alpha]List"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Dimensions", "/@", 
  RowBox[{"{", 
   RowBox[{
   "aList", ",", "bList", ",", "cList", ",", "aWList", ",", "VMagCList"}], 
   "}"}]}], "\[IndentingNewLine]"}], "Input",
 CellLabel->"In[16]:=",
 CellID->42630738],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"3", ",", "60"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "60"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "60"}], "}"}], ",", 
   RowBox[{"{", "3", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "60"}], "}"}]}], "}"}]], "Output",
 ImageSize->{267, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[16]=",
 CellID->532539161]
}, Open  ]],

Cell["\<\
These results can be input to the function ForceAndMoment to obtain \
aerodynamic properties of the airfoil assuming the angles of attack are in \
the linear range of the lift curve.  Listed below are values for lift-curve \
slope, angle of attack at zero lift, location of aerodynamic center and \
pitching-moment coefficient at zero lift (couple).\
\>", "ExampleText",
 CellID->24035690],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl\[Alpha]", ",", "\[Alpha]0", ",", "xac", ",", "cm0"}], "}"}], 
  "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Doublet\>\"", ",", "aList", ",", "bList", ",", "cList", ",", "rD", 
    ",", "\[Alpha]List", ",", "rC"}], "]"}]}]], "Input",
 CellLabel->"In[17]:=",
 CellID->34907031],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"6.78513432708499`", ",", 
   RowBox[{"-", "0.04128321013931751`"}], ",", "0.25983834312313764`", ",", 
   RowBox[{"-", "0.05341961850742165`"}]}], "}"}]], "Output",
 ImageSize->{298, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[17]=",
 CellID->209153046]
}, Open  ]],

Cell["\<\
Note that the location of the aerodynamic center is close to the thin-airfoil \
theory result of the quarter-chord point.  Display the ratio of the lift \
curve slope to the thin airfoil result of 2\[Pi], and the angle of attack at \
zero lift expressed in degrees.\
\>", "ExampleText",
 CellID->47127959],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"cl\[Alpha]", "/", 
    RowBox[{"(", 
     RowBox[{"2", "\[Pi]"}], ")"}]}], ",", 
   RowBox[{"\[Alpha]0", "/", "Degree"}]}], "}"}]], "Input",
 CellLabel->"In[18]:=",
 CellID->274981366],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1.0798876676980773`", ",", 
   RowBox[{"-", "2.365353705734581`"}]}], "}"}]], "Output",
 ImageSize->{135, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[18]=",
 CellID->396978572]
}, Open  ]],

Cell["\<\
Consistency between these two approaches of computing aerodynamics properties \
can be checked by computing the lift and moment coefficients at 6\[Degree] \
using the linear properties. \
\>", "ExampleText",
 CellID->328772922],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"cl\[Alpha]", 
    RowBox[{"(", 
     RowBox[{"\[Alpha]", "-", "\[Alpha]0"}], ")"}]}], ",", 
   RowBox[{"cm0", "-", 
    RowBox[{"xac", " ", "cl\[Alpha]", 
     RowBox[{"(", 
      RowBox[{"\[Alpha]", "-", "\[Alpha]0"}], ")"}]}]}]}], " ", 
  "}"}]], "Input",
 CellLabel->"In[19]:=",
 CellID->762946865],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.9906497314348841`", ",", 
   RowBox[{"-", "0.3108284033388432`"}]}], "}"}]], "Output",
 ImageSize->{142, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[19]=",
 CellID->219137947]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Scope",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->1293636265],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Alternitives for discretization",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection",
 CellID->421192202],

Cell[TextData[{
 "Fitting a B-spline through the points in a data file provides a means for \
alternate discretizations.  In the current version of ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ", this is accomplished by the function FitAirfoil."
}], "ExampleText",
 CellID->5610347],

Cell[BoxData[
 RowBox[{
  RowBox[{"spline", "=", 
   RowBox[{"FitAirfoil", "[", "rD", "]"}]}], ";"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->242241179],

Cell["\<\
Graph the data and spline fit to check that the spline provides an adequate \
representation of the airfoil.\
\>", "ExampleText",
 CellID->299806311],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"grD", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gSpline", "=", 
   RowBox[{"ParametricPlot", "[", 
    RowBox[{
     RowBox[{"spline", "[", "t", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "0", ",", "1"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"grD", ",", "gSpline"}], "]"}]}], "Input",
 CellLabel->"In[2]:=",
 CellID->278559890],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3f+PFPUdx/Fr+pf037hEueMOFAk9D4vYWmrhZAtt1Wo0MagRalsIQrrx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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->221681679]
}, Open  ]],

Cell["Rediscretize the airfoil using the spline.", "ExampleText",
 CellID->176584194],

Cell[BoxData[{
 RowBox[{
  RowBox[{"n", "=", "60"}], ";", 
  RowBox[{"layout", "=", "ConstantSpacing"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"spline", ",", 
     RowBox[{"NDiscretizeUnitSegment", "[", 
      RowBox[{"n", ",", 
       RowBox[{"Layout", "\[Rule]", "layout"}]}], "]"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->664892474],

Cell["\<\
Graph the endpoints of panels for this discretization and the spline.\
\>", "ExampleText",
 CellID->829463257],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"grP", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rP", ",", 
     RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"grP", ",", "gSpline"}], "]"}]}], "Input",
 CellLabel->"In[4]:=",
 CellID->702755110],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3f9vFHUex/Fe7i+5f4NEaSmoXA9r9biaYEWo7omgnp7kLkUjmlOMStzA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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->103725458]
}, Open  ]],

Cell["\<\
Solve the model using this discretization, and compute the airfoil lift and \
pitching-moment coefficients.\
\>", "ExampleText",
 CellID->33075010],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "aP", ",", "bP", ",", "cP", ",", "aWP", ",", "rCP", ",", "VMagCP"}], 
    "}"}], "=", 
   RowBox[{"PMDoubletQuadraticDirichlet", "[", 
    RowBox[{"rP", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"clP", ",", "cmP"}], "}"}], "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Doublet\>\"", ",", "aP", ",", "bP", ",", "cP", ",", "rP", ",", 
    "\[Alpha]", ",", "rCP"}], "]"}]}]}], "Input",
 CellLabel->"In[5]:=",
 CellID->539184285],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.9668643275529123`", ",", 
   RowBox[{"-", "0.30600577065474477`"}]}], "}"}]], "Output",
 ImageSize->{149, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->198210789]
}, Open  ]],

Cell["\<\
Compute the surface pressure coefficients for the user specified \
discretization and plot in red.  For comparison, also plot in larger blue \
symbols, the results previously computed using data from the file for \
discretization.\
\>", "ExampleText",
 CellID->511124957],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"xCP", "=", 
   RowBox[{
    RowBox[{"Components", "[", "rCP", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cpP", "=", 
   RowBox[{"1", "-", 
    RowBox[{"VMagCP", "^", "2"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gCpP", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xCP", ",", "cpP", ",", 
     RowBox[{"PlotStyle", "\[Rule]", "Red"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gCpD", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xC", ",", "cp", ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Blue", ",", 
        RowBox[{"PointSize", "[", "Medium", "]"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gCpP", ",", "gCpD", ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "1.1"}], ",", "2.2"}], "}"}]}]}], "]"}]}], "Input",
 CellLabel->"In[6]:=",
 CellID->567786021],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], PointBox[CompressedData["
1:eJxFlXk8FOgfx8ekrZ92bZLokLPDmYoo1Vft2NpI0yRUxCSqJWlzLWVT8Rrj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     "]]}, 
   {RGBColor[0, 0, 1], PointSize[Medium], PointBox[CompressedData["
1:eJwB0QMu/CFib1JlAgAAADwAAAACAAAAvxJCeLTx7z/QhbTam/zRv35MSGjL
ue8/0IW02pv80b+Mw17q503vP/jEpiE3uc+/Se3nK0iz7j8sgmWYM+PJv47d
N9bF7e0/OEOVPaPyxL+IYI3MI//sP7hzAlsg+8K/YztRD9Ho6z/kfiVSRzPD
v1l7ZRTLreo/WPD9/cv2w78uXol2FVLpP8DAmfLvr8S/kLKcuvLZ5z88bGQn
FXvFv025Zb0YSuY/7OngdXZPxr/uwxVNZ6fkPzzG0M7eM8e/YfmjpfL24j/8
Q38y2y/Iv5gcKO0NPuE/kIL4vqk9yb/u33OTGATfP2gKzKFVZ8q/iGriwoGQ
2z/ERDaVg7nLv6pfy7T9K9g/vI3ob+01zb+pqj3L8+DUP5TTkPHC8M6/uwIH
lE250T8uCCLPD4LQv+Ovka6ZfM0/PNbKS5vM0b+3vFsgQfHHPyS72PvQcdO/
ybNRMc7fwj+eJ3i+JZfVv6XKfmtgq7w/ToFivHZ22L8y5IMnZr20P/qucTwr
bty/WbG9374OrD8E3lGrDPrgvzPauDF3LaE/zM5PEu3f5L82HbVpjNaRP8GX
NkHHMOq/9x92H0KVej83MxG10JTvv0+ZE75OQFM//n7LSgm/5L+iHAI8hEdb
P9Bd3ZUwsPg/TuHsLNU+fT+SeCMpYqr/Pz7nVbQHWpE/Ri/dHEv5/j+bkN9U
xr+fP0BG0xJfW/0/qERNsKw0qT8sfmPUUI77PwQpS0nXTLI/3j9Ti1ne+T9R
vbGfVPu4Pyzj4k3UYPg/ecAE2zNLwD8ei+8aZQ/3P5YcUoWxhcQ/niHgRSHr
9T/dQCYRUyLJPzCgoRIG7PQ/b/T4piIVzj+aDUJCYwz0P/g44guTqdE/lssP
Qkoq8z/1Xdv5fmrUP1Z8ypisA/I/g6h1MjhK1z90Sy1+GY/wP69ypxPQRNo/
VGiYDFnK7T9O9ZAYBFbdP3irB3yMKeo/4q5z5bM84D8otmF9YFvmP8YHY6pg
1OE/TN/LWFCn4j9oiWDzcW3jP3ANTwI9zt4/O6f5HYoC5T8cZFDKRjfZP8XW
u4dLjuY/4L0b17hl1D82U48KYwvoP/gxcHtQL9A/XuQReZJ06T9wD5e24crI
Pw1xt4jvxOo/eMhog4EMwj9iurlcxPfrP3Dge/nasbc/VRVzApoI7T/AnmRg
Q8GoPyFn+sHA8+0/AGHoG/B9cz/fBxrr4rbuPwArlFXSYKq/s6usQspP7z9o
496WeIO/v766qCaIuu8/sKpy76guyb+Mt6tp3vHvP2RS2WT0JNC/3EgCpA==

     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox[
    "\"\\!\\(\\*SubscriptBox[\\(C\\), \\(p\\)]\\)\"", TraditionalForm]},
  FrameTicks->{Automatic, {{-1, 
      FormBox["\"1\"", TraditionalForm]}, {0, 
      FormBox["\"0\"", TraditionalForm]}, {1, 
      FormBox["\"-1\"", TraditionalForm]}, {2, 
      FormBox["\"-2\"", TraditionalForm]}}, None, None},
  ImageSize->504,
  PlotRange->{-1.1, 2.2},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 324},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->526203215]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Options",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(3)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->2061341341],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Epsilon",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection",
 CellID->30521753],

Cell["\<\
Collocation points are displaced a distance Epsilon from panels.  When \
Epsilon is positive, the displacement is opposite the direction of outward \
unit normal vectors to panels.  In the following example, the model is mapped \
to three values for the option Epsilon.  The three values place the \
collocation points inside the discretized airfoil by the default magnitude \
for Epsilon, on the panels, and outside the discretized airfoil by the \
default magnitude of Epsilon.\
\>", "ExampleText",
 CellID->120830056],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"aMinus", ",", "a0", ",", "aPlus"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"bMinus", ",", "b0", ",", "bPlus"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"cMinus", ",", "c0", ",", "cPlus"}], "}"}], ",", "aW", ",", 
     "rC", ",", "VMagC"}], "}"}], "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"PMDoubletQuadraticDirichlet", "[", 
        RowBox[{"rD", ",", "\[Alpha]", ",", 
         RowBox[{"Epsilon", "\[Rule]", "#"}]}], "]"}], "&"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"10.", "^", 
          RowBox[{"-", "10"}]}]}], ",", "0.", ",", 
        RowBox[{"10.", "^", 
         RowBox[{"-", "10"}]}]}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Max", "[", 
      RowBox[{"Abs", "[", 
       RowBox[{"a0", "-", "aMinus"}], "]"}], "]"}], ",", 
     RowBox[{"Max", "[", 
      RowBox[{"Abs", "[", 
       RowBox[{"aPlus", "-", "aMinus"}], "]"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Max", "[", 
      RowBox[{"Abs", "[", 
       RowBox[{"b0", "-", "bMinus"}], "]"}], "]"}], ",", 
     RowBox[{"Max", "[", 
      RowBox[{"Abs", "[", 
       RowBox[{"bPlus", "-", "bMinus"}], "]"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Max", "[", 
      RowBox[{"Abs", "[", 
       RowBox[{"c0", "-", "cMinus"}], "]"}], "]"}], ",", 
     RowBox[{"Max", "[", 
      RowBox[{"Abs", "[", 
       RowBox[{"cPlus", "-", "cMinus"}], "]"}], "]"}]}], "}"}]}], 
  "}"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->210128799],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1555.4586133703438`", ",", "1569.4934556024098`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.1980217881029109`*^6", ",", "1.2096805685867167`*^6"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"1.749464214960134`*^8", ",", "1.7680268386477545`*^8"}], "}"}]}],
   "}"}]], "Output",
 ImageSize->{533, 18},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->628929861]
}, Open  ]],

Cell[TextData[{
 "There is a discontinuity in velocity potential across a doublet panel (see ",
 ButtonBox["ICDoubletQuadratic",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICDoubletQuadratic#61408839"],
 ") so large errors may occur."
}], "ExampleText",
 CellID->937232338]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "InterpolationOrder",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection",
 CellID->233233938],

Cell["\<\
The list of external surface speeds associated with collocation points is \
computed by using the jump conditions for the velocity potential to determine \
the velocity potential just outside the airfoil.  An interpolation function \
is fit to these values of velocity potential, and the velocity is computed \
from the gradient of the velocity potential.  The default value for \
InterpolationOrder is 1 giving linear interpolation.  Higher-order \
interpolation may give better results, but risks excursions of the \
interpolation function fitting the data. \
\>", "ExampleText",
 CellID->381223408],

Cell["\<\
To prepare the graphic below, the model is mapped to values of the option \
InterpolationOrder of 1, 2 and 3.  Flow speed just outside the airfoil at \
locations associated with collocation points is plotted versus distance along \
the discretized airfoil.\
\>", "ExampleText",
 CellID->507899588],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c", ",", "aW", ",", "rC", ",", "VMagC"}], 
    "}"}], "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"PMDoubletQuadraticDirichlet", "[", 
        RowBox[{"rP", ",", "\[Alpha]", ",", 
         RowBox[{"InterpolationOrder", "\[Rule]", "#"}]}], "]"}], "&"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"s", "=", 
   RowBox[{
    RowBox[{"Accumulate", "[", 
     RowBox[{"PanelLengths", "[", "rP", "]"}], "]"}], "-", 
    RowBox[{"0.5", 
     RowBox[{"PanelLengths", "[", "rP", "]"}]}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"data", "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Transpose", "[", 
       RowBox[{"{", 
        RowBox[{"s", ",", "#"}], "}"}], "]"}], "&"}], ",", "VMagC"}], "]"}]}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"data", ",", 
   RowBox[{"AspectRatio", "\[Rule]", ".5"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<s\>\"", ",", "\"\<\!\(\*SubscriptBox[\(v\), \(c\)]\)\>\""}],
      "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7.3", " ", "72"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"Green", ",", 
        RowBox[{"AbsolutePointSize", "[", "6", "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"Red", ",", 
        RowBox[{"AbsolutePointSize", "[", "4", "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"Blue", ",", 
        RowBox[{"AbsolutePointSize", "[", "2", "]"}]}], "}"}]}], "}"}]}], ",", 
   RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->49740593],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3U9oK3diB3C1JcXQsvWxLXRDe2h9CT3sxRh2s4HyePDoHspCeZdHWah6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    "], {{0, 281}, {526, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{526, 281},
  PlotRange->{{0, 526}, {0, 281}}]], "Output",
 ImageSize->{530, 281},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->149977732]
}, Open  ]],

Cell["\<\
Results for Interpolation orders of 1, 2 and 3 are displayed in large green, \
medium red and small blue symbols.\
\>", "ExampleText",
 CellID->520516665]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "XWake",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection",
 CellID->104403840],

Cell[TextData[{
 "Results should be insensitive to values of XWake if they are sufficiently \
large.  The model is mapped to three values for the option XWake, ",
 Cell[BoxData[
  SuperscriptBox["10", "10"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  SuperscriptBox["10", "5"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  SuperscriptBox["10", "3"]], "InlineFormula"],
 ", the first value being the default.  Strength parameters for the latter \
two values of XWake are compared to results for the default value and the \
maximums of absolute values of differences are displayed."
}], "ExampleText",
 CellID->391229248],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"aD", ",", "a10", ",", "a1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"bD", ",", "b10", ",", "b1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"cD", ",", "c10", ",", "c1"}], "}"}], ",", "aW", ",", "rC", ",",
      "VMagC"}], "}"}], "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"PMDoubletQuadraticDirichlet", "[", 
        RowBox[{"rD", ",", "\[Alpha]", ",", 
         RowBox[{"XWake", "\[Rule]", "#"}]}], "]"}], "&"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"10.", "^", "10"}], ",", 
        RowBox[{"10.", "^", "5"}], ",", 
        RowBox[{"10.", "^", "3"}]}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Max", "[", 
      RowBox[{"Abs", "[", 
       RowBox[{"a10", "-", "aD"}], "]"}], "]"}], ",", 
     RowBox[{"Max", "[", 
      RowBox[{"Abs", "[", 
       RowBox[{"a1", "-", "aD"}], "]"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Max", "[", 
      RowBox[{"Abs", "[", 
       RowBox[{"b10", "-", "bD"}], "]"}], "]"}], ",", 
     RowBox[{"Max", "[", 
      RowBox[{"Abs", "[", 
       RowBox[{"b1", "-", "bD"}], "]"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Max", "[", 
      RowBox[{"Abs", "[", 
       RowBox[{"c10", "-", "cD"}], "]"}], "]"}], ",", 
     RowBox[{"Max", "[", 
      RowBox[{"Abs", "[", 
       RowBox[{"c1", "-", "cD"}], "]"}], "]"}]}], "}"}]}], "}"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->451142620],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1.3497484079305622`*^-6", ",", "0.0001350052031705573`"}], "}"}],
    ",", 
   RowBox[{"{", 
    RowBox[{"0.000014278409139478754`", ",", "0.0014277772050945714`"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.00033470310775385315`", ",", "0.03346879928771784`"}], "}"}]}],
   "}"}]], "Output",
 ImageSize->{571, 18},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->534793203]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Possible Issues",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->1305812373],

Cell[TextData[{
 "In the specification of some airfoils, the trailing-edge points for the \
lower and upper surfaces are not coincident.  This causes a problem for the \
doublet models with internal Dirichlet boundary conditions implemented in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ", but not for the vortex model with external Neumann boundary conditions."
}], "ExampleText",
 CellID->416625656],

Cell["\<\
This example uses the Clark-Y airfoil specified by a data file located in the \
folder Airfoil Geometry.  Import the file and plot the discretized airfoil.\
\>", "ExampleText",
 CellID->728813289],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<Aerodynamics\\\\Airfoil Data\\\\clarky.dat\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"gD", "=", 
  RowBox[{"PlotAirfoil", "[", "rD", "]"}]}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->112529997],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3c1rFOkWx/Fc7l9y/43AMAtxIS7ECyGLkI0XGt3MbHSjm2SjmwQa3RiQ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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->281554140]
}, Open  ]],

Cell["\<\
Note that the trailing-edge points for the lower and upper surfaces are \
close, but not coincident\
\>", "ExampleText",
 CellID->390242666],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ptsTE", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"First", "[", "rD", "]"}], ",", 
    RowBox[{"Last", "[", "rD", "]"}]}], "}"}]}]], "Input",
 CellLabel->"In[2]:=",
 CellID->238007975],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1.`", ",", 
     RowBox[{"-", "0.0005993`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.`", ",", "0.0005993`"}], "}"}]}], "}"}]], "Output",
 ImageSize->{241, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->170433896]
}, Open  ]],

Cell["\<\
Specify an angle of attack, solve the model, and compute the lift and \
pitching-moment coefficients.\
\>", "ExampleText",
 CellID->25666953],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Alpha]", "=", 
   RowBox[{"5.", " ", "Degree"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c", ",", "aW", ",", "rC", ",", "VMagC"}], 
    "}"}], "=", 
   RowBox[{"PMDoubletQuadraticDirichlet", "[", 
    RowBox[{"rD", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl", ",", "cm"}], "}"}], "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Doublet\>\"", ",", "a", ",", "b", ",", "c", ",", "rD", ",", 
    "\[Alpha]", ",", "rC"}], "]"}]}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->592619],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.3720853534761064`", ",", "0.11508089251303488`"}], 
  "}"}]], "Output",
 ImageSize->{141, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->91869447]
}, Open  ]],

Cell["These coefficients are not close to expected values.", "ExampleText",
 CellID->722709344],

Cell["\<\
Plot streamlines near the trailing edge, using a scale that makes the gap at \
the trailing edge visible.\
\>", "ExampleText",
 CellID->492689119],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"xW", "=", 
   RowBox[{"XWake", "/.", 
    RowBox[{"Options", "[", 
     RowBox[{"PMDoubletQuadraticDirichlet", ",", "XWake"}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rAll", "=", 
   RowBox[{"Append", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"{", 
      RowBox[{"xW", ",", "0."}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"aAll", ",", "bAll", ",", "cAll"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Append", "[", 
      RowBox[{"a", ",", "aW"}], "]"}], ",", 
     RowBox[{"Append", "[", 
      RowBox[{"b", ",", "0."}], "]"}], ",", 
     RowBox[{"Append", "[", 
      RowBox[{"c", ",", "0."}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Psi]A", ",", "ic\[Psi]B", ",", "ic\[Psi]C"}], "}"}], "=", 
   RowBox[{"ICPsiDoubletQuadratic", "[", 
    RowBox[{"rF", ",", "rAll"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Psi]", "=", 
   RowBox[{
    RowBox[{"UniformFlowPsi", "[", 
     RowBox[{"rF", ",", "\[Alpha]"}], "]"}], "+", 
    RowBox[{"ic\[Psi]A", ".", "aAll"}], "+", 
    RowBox[{"ic\[Psi]B", ".", "bAll"}], "+", 
    RowBox[{"ic\[Psi]C", ".", "cAll"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xMin", ",", "xMax", ",", "zMin", ",", "zMax", ",", "contours"}], 
    "}"}], "=", 
   RowBox[{"{", 
    RowBox[{"0.95", ",", "1.05", ",", 
     RowBox[{"-", "0.03"}], ",", "0.03", ",", "13"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gStreamlines", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"Contours", "\[Rule]", "contours"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "False"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "3"}], ",", 
     RowBox[{"PerformanceGoal", "\[Rule]", "\"\<Speed\>\""}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gStreamlines", ",", "gD", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", "\[Times]", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[4]:=",
 CellID->266881112],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJx03QfcV+P/P3BRiPhaZZNNJb5khmNXdiiRUUoDpVLKqITsIrKy955fI+Pr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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwl0bsrB1AUwPFjIZHB+/1KWTxWk0IZ9Cvlh2JRxGCgmLD6B2xkIFYGKY+S
R3n8B+Rt85ZsBuUjw7fvOfeec+693arB8a6xlIgYx0d+xHVBxEB1xCXnyNe4
X77On/LhkogRPMiH+J5nOMteoiIikzvLIjqsJQvFnMCGGRu8iS1xpZpNrtWz
y83FEU0YK7Wmb0ndIhbM+7KeURmRjj218868El+jT82FvFdNN1L1p+FOPOuM
Gx41o6E8ohE1qEcdfnCME6zgDKdocX67Gc88x31mvIjPzWmVt+FNXuTur1zM
y/Z6qyK25T28w0fudogp9ft88Pd26132u5HEqr4JnkS+OYUowKO6XM5DNp7k
Ofyd5z/c59ab3/VOl/z/2y9wb0Wc
          "]]},
        RowBox[{"-", "1.3199999999999998`"}]],
       Annotation[#, -1.3199999999999998`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwl0s9L1GEQx/HxVkFg4a9sdXeFtmP3QNywIg2WEhJFszCjg+hmHUQIOkT3
PBkbBJqGBOkSBPlXiJZgkoKZqAhqRFh5qJd0ePN5Zj4z88zz3U1351v6SyKi
gMtVEfMYqIuYo+vJiG9Yw3QqYgpH0hFH8dZ5sDJilfcV1+Suo0K+EmMVEYlE
xCgtF5dhl7+Dgr6m2ogXtJk28BbUZdVfwJq7Sw/n0hP0Z3lE1+mIGl4t2k9F
pGgSWTUTZrzGKMbxCn/1TOst4oz4LDLodc+SWU9rIp5guTrimRk36J6eIV4r
2rAjN2LHH/SYHb/T5/oLvJfI6Ouzw33U82b1X3VfGhkkzX/vW75TW8Rn8SJy
zsPmbtMWM5poTv4KPW7+OdqNSTWP6Bt6j+7yVp0fekdR/ZRzm3OHGXm6JN6Q
38QWVsQP5Afwxf6L4hW6jC573cQd5z9q93lD6m7b+xZavalU7i6/B9W++WO/
3QfvnMEv3m/5g8N+nBQ3mnMRl/DJTp3qOlAi/ig+n/j/H/sH4Apirg==
          "]]},
        RowBox[{"-", "1.331`"}]],
       Annotation[#, -1.331, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwl0k1sTGEUgOEzNpY2fjqYakcmkUgpm0YqWsGGVEoqEn8VsdZJ2ZW2Vrrr
WAq1ICmVqKRGojMlUQssNKwaWkkldNGaKRK1YOG5sXjz3nO+c84938ytP9d1
9EIqIkr4nI74tDbizcaIWZ4TD6+LKKM1GzEpl5ZbJT5bH/G6JqKTX3HJWYO+
cS5nIjLrI7aLt2EAh/Vd4zZ+wUXnT/BS3LcpohclfZNmpTxPyOdr7eFd8/IL
WMSyOCXfxCv4Sl3EZfTgm/MOe3Z4PoaKuIpHKKjNyQ1yw4aIMXvu4D3qd/KI
eee5jxu5Xc8hHhQvcIHbeFq+yE12rLPrVi6aUSufS3I4iKk1Ebvdr9E9utw3
p7asbzX/tsOQ5xt6fqlJOz9l/gm5k6jIHedFXjJnpecqt6if0vsWzebMJDXe
tUX8Hu/Qq2eXeXnz7rpvjZrTdpjjfXqyzobtuzn5r+QfmJlVl09yPCIe9dvc
53v4a+Yf3DFviG/jJlrMusVf+Lm6Z5jAU3Vn5NvlH/IRHuVx+U4uJd8Ot3rX
Xuy3xwHMq+u2w1dedtcZuw2IZ/m73E9cEv/gaftV+aJ4iSv4mHxf4g9cQL/n
6/zY7DFctVt/5v83/g/SaHxp
          "]]},
        RowBox[{"-", "1.3419999999999999`"}]],
       Annotation[#, -1.3419999999999999`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{6157, 8803, 6158, 6700, 5568, 6313, 8515, 8517, 8516, 6257, 
          8133, 8134, 8135, 7063, 7185, 7186, 6571, 8189, 8190, 5770, 7055, 
          7263, 7162, 7163, 6684, 8087, 7516, 7517, 7161, 7520, 7519, 7518, 
          6258, 7866, 8025, 8024, 7264, 7265, 6572, 8680, 8682, 8681, 5690, 
          7064, 8519, 8520, 8518, 6314, 8521, 5771, 8191, 8866, 6259, 8487, 
          8136, 7065, 8879, 6160, 7252, 6411, 7272, 7273, 6701, 5569, 8566, 
          8013, 8014, 7750, 7752, 7751, 5772, 7266, 7267}], 
         LineBox[CompressedData["
1:eJwl011ozXEcx/GfhGFcmM3GHkxyoYU5M+LGldmNxEoetrK05sJGKQ/lISSm
XHjaA9sULpgoNytcGNuZmXMOW260WbljU8wxF6O8Ti7efX7f5+/v9+9fWF2/
rW5KCOEvnmSHkFwYQlNeCLcWhTCRH0IGX6fzU6zMCWGDeCwzhPKCEAqwFHny
V9DlKZ/8UvqLflGfpC1qmlGWG8KVBSFs1auCf4P4afZ59Rewg93Bjog/SvWk
J9n99AQtoiX8w+bnOvcvDqGbv0ZdDx1n/6I/aJKupaVYgyg7RjvskWa/mYjq
1Yt4lt52i9Gz/OdwBgPue53/GqrU3aCNmOvciS75fXImC0P4g/vowAPcM++n
2JYlIUx3jqubQdvs0I5W1LLb5d7GHvOO4xiS7Kdq75pxB7PsWMvelzqb+V3d
V/VjGEUau1jOakRw1dt+8CZD3mmTmiykmz8bRWKZ+mdhHuYjA+Nyn8l7jt/y
JvDJrFepN8YLvEQXvmGv958tvsz99tutTX4rRvg+s2/aowWnxC+z42YkMOx+
lXIuym3AJQyKv5UbS30LNPNVy5t0/oM36Mc0vUbklpm/GeXolfsa4/wN4j9o
nzt8dJ98dsTbFOC93AHE8Q4JPLbH7tTbYxeqUImoNzpID+EA6lGHqfqtMqsY
+WZE6Gp0y19DSzHHnuno4SthPzSjwre4Y4ft9C49SsfoETpKB/QqpoWpfnSQ
3a1HVO1avnV4p990b5KgNXbZKZZQ34gmHJO/XqzZea8eab7PRr4xvYadh7Cb
P4ce5svO/f+//wMCTaLF
          "]]},
        RowBox[{"-", "1.353`"}]],
       Annotation[#, -1.353, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV02tM1mUYx/FbGs6iRTXxEAcP1SZSxlJLXJZJMrdKZk1BXyRZQqYCbdme
eDyglUSKNQ6uLFe4iopaUY0lZqtcCogUoBum6zyVWuLmTEqzPv8XX37X+b7u
+/kzYVnZA6XDQgg7/cm8LoQNY0OYMTGEnFEhvJvGprfz14tPki8fwxc7nxpC
KXuIltF/1F5Aipoa9SNpW0oIl+t7in8FvWl0CKvGh5BFd4hVZoTQSIP+dPEm
c1/mbxB/Pd35/Azxd+hf/POYpPZ5+dnm3Y2dZuXSOVjn/PfUfsZOEK+kOebd
KbZebi4/zs+jS8RqaK49j8s18xvZy50xh73fvQ4gy3lTxG5BNg47uw8DaleN
s4sZK2k1/cicD5Gvf43aNjVH2efE+mmrmlnOf5Od6l5V/I4JIXSiz4z54hvV
bkKX2Bb5g3QrPWKXw2jk78JkxO3Wo68X7fzN0W9EM/GFHSvlp7H3skvVrMYh
/khzOsQ60Y6t6g7SLXSYsx+N3ob9UHQne7bghN6TKJbr4i+nhWYtRgH2qL9R
7w34RK7F/T9GNV5AudhKZyRiOKrEjuF7/CQ3S9/P9Dh/gf2GzHvV+93hzHrx
bHpW7JTe6eypyOBn0puRpu/6KIZ56PLtzdQ/xR12m9eKa9Wf0/83fkAL/yzN
M38yXaP+Cd/NGTqIa9zrFXfdgUNmdmMXu0xNzHkViCPR7ES54bTbzMtoAo7a
IU28n6b73VOjXdXHzMg240GxIX4RfwB/ir0htpTdJ96Lx+21AjG9NdiGb+3W
jST5VPd7SewifZFeoGujO5oVpxVoVtfEH0SR+afpUvsVoUR9q3sW09f8Bse8
xVzn56HZvE30ffoBRqtZQMfQUYiZlUJ/NOdLfTPMHSc24L7P2O8RsdNmJrGf
w7NYFH0bYgtpBX+Jnt/pET1j9d5j3pPesMM5nbhL3UXzZ9N6uR67NtBeWks/
NWu1XB27lA7yz6DKzM0oEPuNP5+dj3X8++l9+EX8V6xlx5Erd4C2Ixn78Q02
On9i9P2yR6AH30XfptjD3nYZFmJR9A2KJditzrvVoxbb0YBLco+50wqUoE3d
brxlh3/lFqspxB/uMsJbJNLtzk72HleZPd6ZDfx6bEO1nhNiJ3G1fDIKvOeV
6pOwD3vV9Kq9lV3nW5hKc9TNxClvnhD9X9O66P8MtdHva96laCdMV9+nfxrN
t8+9cv+JB/0ltBhZYv30NmeVR28uV4i3xZrwtL497hmjn9Ov8RX24X9R/f49

          "]]},
        RowBox[{"-", "1.3639999999999999`"}]],
       Annotation[#, -1.3639999999999999`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1WdsllUYxvHD0gAuZJUltAKKLEEwIksZUmhrZBa/yBKiiCDLAW1ZykoE
DZTdmhgpQ1BWGSJBCSYKNUBoQcCCgIB+UCkQ2jL9nQ//XPd9zr3OeZ7nfRNH
ThgwvkIIoUHFEHbVDqFGgxAex8J6IRQ2DqEIr9UJ4b61/vT3xBCKcQbncBar
xK7BrKQQVtNL6sxrEsJ8zMVCLECTuiH0xGBxPWihuD40nZ9Mk9APQ/l96WX7
vekQ/is0EXcRzNHMWi921YQQusU9/i3xt9m3aRK/O7ud/ZdpU/5d63fYd+iT
/JfYPeqHcLqh82AyexLK7A95IoRD7MN4DL+iAAP1XkfX42HkYS0Giy+VN479
LorUK0Rz9iB9WutXN94jO4G24Q9mDzPfInezGLvd9R7Ut1/BvdajY+QfdKff
sw+ot5dmyBlAj3kmq9irkYNTZlhD+8npixx5XfT5UGy+3I/oLfUzaCmdRsvo
dFpOP6CPiHsUK9R5W+5yWlOf486WwO+iZmdkm28H/Yfm6JUtfwR/FE1tFEIK
juEo5lmboeZMVBU7EZ312sgfhxn8mbjMvoR09izMxgA1B6KJ3iVmaKffDDM9
S59De2TxV9FkcStpH5pDc7EafeP7SauolxjfQzRGbnznrL8hf6cZd2G2s5ZY
vxbfLVSWU43eUGOTs2wx934zbqVN1dhsba+96XKzxda0VytiLSPOhkzcFL9A
7VKa6ix/2b+CArHN1O+PVtbrxGevzltq17M2UUyW9ffol2LL7Kfx/+anxvnY
Z9k9xPdEL6wW96a1HfqOpvn0sNjtdBR/Kx1Jt8VvV+wJZyhCIYqt/2TOg6iM
zfpV815VR1WsUH+4Wmv0Xe/MG7ARY8W+g6HyM+2l08/U3i+/O/9nfhV5i+P7
wx9m/VP7g+I9xW+BTo7flpnW4yvkYS0WiRtu76w+55CHdfjT+7DOPBdpobgi
HMdJnMByNU/TlfQ3uoKeomlqfaNmanwH9d6Er7EZ5e6gujk/V782vxb2eT+a
i98nP1j/kb9HXAY7M/rWt3tm2/CDuBT+J7E+nUvn4GMc4Jfpfwvl8RnoXzm+
z84wE1noqvcsWmIvnVaUs0e/Ceb4jk6N36Xe4zEBv7jzQ8i0fk6PB83QG2fc
SSu1W6Ml2qINOqo/Rl4Heo+29ExboLd+JfQailGGG6ihbr76u9HaLJ3ib2L8
3XbuRkhCA3Va0afjmvj2tAOuyG9H22KJ+i/SzngI3dAVp8R3oqX0BXqTTjFn
JfHD9CrnF6uf6i7SkOyMF+O7YK2heWrF/w9rNemreN09pdEK8kawK9Lr/H/l
XsV/KFT3Dr2gTgrthwIxx+kGexvjGdWeI7+AHorPnV0h/oeY7T7u4bx7CHpf
oMlyzrP/wCw9j7qvI5hkzom4Kj5fnYT4XaM+poldHt9LLENX+0fcwdF4p+pN
U6eK3Afi8+Y/Zd6x8Z7pt/wtuKHGdey09owztIj/keKb0lya5RksNfda+w31
zKPL4u86spFqPyX+b4jvrn+3+FumXga+UD839sUJ/kksZS9BB3UqmXmymlMw
Ce9jKubZy1TnefU7skfL28Cfzd9I51v7HxnlSkI=
          "]]},
        RowBox[{"-", "1.375`"}]],
       Annotation[#, -1.375, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV02lMyHEcx/FfSiWyYaRSZGLaXLNh5onZXDlDNtcDD2TOMB3PWIebMY/c
G1seMPdNbMQciXmgmU0NiWJyjAc5Xv8H7z6/7/39/quspWvz1sSEEEb6ca1n
CKPSQzjaN4QjOIzs1BAW9wlhEf71ktMvhOu4iqT+IXRCq5zbavd4b1fTIyOE
bXQrcqJYVgh3UI0PaMJJ/mo1OWkhxOMe7mKXGUXqzolvpOdpMT1Lv6n7jjZc
YR+Tf0uP7uZlew/AC3sO1KPYuwSn2aeQy1ehZnzkZx9lX2AXqp3sXevupxje
2zfgG40xuJkZwg00mLPZHjVqNtH7tEafe6hW/4B9ObqBnZwSwkzf4idfPd9L
/PK9fiMPszELdfLmmTdYbjc91plfy5fpvvn8GTSfZvM3izfYpxFx9tmhPpZO
ZCfZ55W8h95j+cfZoZ6dzP+adqbL9SrAPn0+qWtBE76iFX3kldFKpOkzCI/c
PCG6lY52w0j3zNB7KnrLb5DbiDfI1fsU3wm3PFYzyZwn0Xyx5Xol0gTsRzK6
YiX/KqyONLqbLwMr2EvZXbxz9KzQu5/Zy9xWgBSxL77JYrG+YllRHG/534g3
ogHf7ZBn1290ir1io79ldzTRqewf/HPEf9Jcdhz/PvHO3p+9F+ixEIuwRN58
9wzkz6eD6BYzk+xwiH0QB5DAF49E/pNYpVcVrTTjmZoK9nNaizqUs5/SGPFU
s3ez02isb9ABpWr/urWI/qEltN0uxTRF3qQoF7PVZbj/uVgmbbbzR/qBNtMi
33Ij5urZblYb3WDucfnraSK7E/bqVSFezb8AC7HL3ATzWrxLxZOi/1l2K/sz
dorvwHYMUDvd/RfFp9FLNF1OGg6KnYm+gX2u2KuSDkWe3YfQjnKHyRnBzmeX
eRfKW4fVWIs16KjvULF39H30e0eVvrfMuIlysf/8Sbl+
          "]]},
        RowBox[{"-", "1.386`"}]],
       Annotation[#, -1.386, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1HtM11UYx/FjMq2mAZqoXESgEMVWYVFaSLW2ZltpWM3bVgqEIMFSlInp
ltbyAl4qUxMVU6vNdTUvK1AzyWqrtbYwoiuiaOnESy5pZq/vH28+z/Oc53zO
c87vxy9tRmVBRY8QwiF/dg0IYX1qCBtwPiWEC1iTHsKreAVvJITQZ2gIfbFw
iD61zIEhzJIPo9cODmGuWm+6m1e/xBBq5f1pPC4nhzBzUAgXk0IooWUosl5K
M+yZL06nmWk81T43x2GMdO7f9txCd1j/w/pbtJ2+SWfr2U6fo9voX+qncYO5
YtGl1tN81c7PFd+FDfKVekaLa+ndtI42OmeTe9ej2x13meOs+AneqXoS7Duj
Z5p6pXyEmW801xK1C866iDoU6KulE+lVGnhfod3y/+gP3qMFr/M4F703LqDB
jMXqW2mFtd7OrpQ32V8jX0qz6DL6trOHi5eLV6DB/k+9+x4csOcgMpCO7/nd
bM5V4tXog5WoQ4EzHkOT+ISef/R20v3y39R7RZ8pHjX3eBy39rWeDnrVevBO
Nerzket+V8w0gG7RUyaehW/ls71RnD1ZvJ5VK0dHdBf5MXrKeidOYq33vced
XqP30gXR/ZGiL5bHMHvG88yI3hNZzm7Gx+JN0buId+sbQbNx2tucQblaO//p
4n7m7o857lAVIW63v0v8pD3x8mJ5LI1DIt9kJOGoN97OoydisFV/A0r1X8//
OlTzmYc4cZF6Ky2kfWle9P3yvqPdby3vp9RL1NrEFfx3YBt2Wn/Yvd+lHdYG
0nHe9RHkeItcHs1mecAMX9FMHoPxkt7FSIq+v/R9epy+R9fbk2D/OppP29Ty
zTQWxzEc2fjSTMv0HKGV8mPiObTSG06JPNUn0w+iz0E9Pfp8zbrRO9SjxUxH
8Xj0/yu/hJ/krfgRJfYl6n/I+R/KP8Jmd1qkf4+59qJB/rueLTRJfaEzFmGm
WUrwp57L0XeKV4L32ucNGhHvDQ5ZD+rf6OkR/R/yWKEnqKeJJ6kPir4/0feV
R7a1CfJ/xbeKh9JOeSo9YfafzTdN3Ean0unmTqPnzHMem+Uxzi3lX4ax5s7D
ED0H7TmAjXwn8VzHbzKNsWeGWk+ap6/Q/uX2VolPma9Q/KL5xugZp7ZTbbH8
CN8l9DD9As04qTfOeov7Pyj+VfwLNkS/peY7hPt5FEW/b9brxa3W88X34QV+
I61lY6pZx8hrnDtFvIr/M7RarYjOo8U0hccxdznhLu3RbwM6kCz/jncSncDj
Gv4T+Sbrf9kcSzGXx16+VXQf3a4/UX+p/WVYg9X4JPpdQy99+2kTGvG0e3Xx
OYtuvp9F3wc9C5DjzNvU3uF5Ox0lf149TU+5fXfIc9Qv2TuK3hn9JqnfpOd/
PuwQ4g==
          "]]},
        RowBox[{"-", "1.397`"}]],
       Annotation[#, -1.397, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVkLlywVEUh396W2OJCGMr0CioSd4gOtIJCmYoURGF2vIAsT+BsVY8gbQe
gkIZhe9ffPOd5Z57z9xAoZ6tmSR9wp9DagYk24vUwHbcwmZsNWpByYI7r9KZ
sy7yFHnS6BPfsBlfsQV3ndLeKx3A75F2eAtFiNP7epPyMIQB/FBbuqUqsws8
4p0YtQR5mbxDPMcV8hke0B9CiTjCnkX8jif44pOm+IN8jNP4F2dwn5kod+V4
c8Ku/37pAW2jxm49zoQgDEfqJ7gzU2B+x73feI23sIEVf5Gi34YnSWAweg==

          "]], LineBox[CompressedData["
1:eJwl01tMz3EYx/GHMafU2DooVOYwoRZKTZqwscYyV25yCEOt2JALXLDQ0VxY
XIrFTaWNS3OobJ2MC7IZuaALbWzcuZHX5uKz93P4PM/3+/3998+uOrWvblpE
fKIPaRGvlkRsyIrYSDmLI0ZTItaL86mYiuiN2nve7csihsWb+UZwh3xM/V56
RH9qxDb5kPqkeBDL5H3iCfpG9zN5qYPy5Oft/si3xx2OL424yT9bfkI8B1vs
7VFrxd/O/EXn1Nv0b1ArJWRHzKPBDPfRO2t396KIM9iDNWaqaY14LWXwpVNf
ckShuevuX4CNuAmvYTE2YxE24XR7a83U0SE7HqkfxF68Y/db/UFvmFDb5b6T
8hH175jIk0TzaZ1do3as8u6V9NjMU7Uu73vCOyo+yvcaj+EQHsERPIzDWIXv
nFONn+UncRxr8Ar2+yZX8SU2YB+W213vjA7f/K4zO+kBJbtLk/5qvmaca0cL
vtB7TgPmUnny3DWX+tSe2XHJrv16M7xxJo3xFNqRhEWYiCW8CfjDdy4V/zHT
bnel+WL1A1jrW02p39J/qPZVP8X8bjv/Yj3me2sBz4A9ZX7TQVzBn0YN/Jcp
nfcndtlxW30n9qolm283W4ptztuqXq6egxflp7FRv8TsFmox20bL1S/oV5ib
4s+WZ9EXZ6fgOHa6SwFvuftVinP59uJCeQUuwB67u2lW5v//2j/K/oIN
          "]]},
        RowBox[{"-", "1.408`"}]],
       Annotation[#, -1.408, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwl0l9ojXEYwPHnlDtyo+PYzjam1Uhys5o6SMqGGWfD7cqyqZFZzR2Fc+TC
aYXsws0w+TcpkTMl21IbtQtstOspyZ9L48b4vLn49v09z+/5877vObWdve0n
UhHxHCsqIg6viejEskzEoaqIjtURizURf/GsOmKju9GVEevdjYrLGFQzvipi
g9xLM+oTi9fxQm3EmNxPHudfSezusnmb+Ao/NeMT5vHN7Ly+NqTUN5kdfEpf
n7s59XdxDyX1u5I92Yhm53rew7uRcy4l78ADnDGvyXnQ3Qj/0F+h9yLXcUa+
DlPpiO12vuKGSnv19eOPmhZO87yeO2ZcVX8Ni+qW+C4puUt2/RZvdl7g7+Ys
df7KObUD+qf1N/Kc/Be7suJj+tLcw0N8AxPqd6jLJ7H8LfEwbmNC3CV/BDfF
Q+JW526+ztuSd/LNtnLX2oij6EZB3GvnyeQ3UzeGvXL3uYUf8BN3xz3bC7Oq
zNgnn+VW/iz/zu+Q875bUEQBj/R8SH4vO2bcT/pWs9ws/ij/3nmn8xsz95sz
Ys9DnPbsZ9Dh/q0d53AeZ1Gjr8BFtJt/EAfQ7xmKuIA2NcPu81yyc7n5U57n
NSZRFpfNflz5///9D50xctU=
          "]]},
        RowBox[{"-", "1.4189999999999998`"}]],
       Annotation[#, -1.4189999999999998`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwl0j9IlHEYwPFHKqiW7vpjd3pnWeDglDQIokTmQS2KF4h4NtRSQyokRZvo
oKCBnRpOKkQ2RCYZjUdCRDgFEVkEQRAUNBaakeHnaPjyfZ/n9/z5vbxv7ZWB
fH9FRMzhc3VEcypiXyaiyJPclY7oror4eSRiv/gXrx2PeHE04nJNxCpf0teL
V5URnXJ5NKAD7finJ8VpHEYGVbimfl3fVR7JRnyyZ9Lei/xDz01nt9CIkp1j
dm06G+eQ2yW3LS7qvy9exF/9g+55Sv402o5FnEEO9XacRGXZ6pbVP8Ks/nO8
17wc3zO/hfNowh7ndbybz+tdd/f3uGDGV/kNs1vFI/pWzBguW/wMy+hW11uO
5Z/rv+65cCJiAfN4J/9B30f0O+vDitxZtU+5IC7xk1pzfI/HXOAl3uAJ+zf5
DvfwN+//HdOYwkv3HnXPhF1JHMCWuoP8m9+oSZr/x/MhubS5OXdOcRt/0fva
jIfuN6juAd/gRX7rO2Wd3xZPq5/BXUyhiJKzOb1JtQl0yLVjKPv/f9sB9y5f
vg==
          "]]},
        RowBox[{"-", "1.43`"}]],
       Annotation[#, -1.43, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwlkT1LglEYhu8sqsnAMCut92NzKqzJhEr6A0qLgYG2KESBRB9Lm0JLRYVj
EDToEgYFQUNDfyAkUFoyyiWiGmqqoevQcHGf8zznvs9z3tfJriZXOiRtwfaI
9A1f0BWQ0ra0CD7oh+MBadmRWoOS15Ke0D70ES2hzSHJ40oNtIJWoQRFekX8
cXLb8AJvZFlB6XJUSuG/QBfQPLUc+OjXqJ3DNMzCDKxTf6a/T8aeyRqWyvgy
aDe9DXqbMA8N7jzkHTHWR+gc2mPmQMucraJn4OL3k+VCGK6YOULeGBRC0g/9
U86dmHvx/VL3kCO4hh2yb9A6eo/eoUnOJKATf5T9A+tXvkMev8M7XAiADRak
IQITMA5reCbNmrxbZlpinYUpU2eOGNoic5degcxe7vkk/wOi/KMDZs7xjnf2
dXwV+////gEAFEfy
          "]]},
        RowBox[{"-", "1.4409999999999998`"}]],
       Annotation[#, -1.4409999999999998`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwl0L8rxHEcx/H3KUw2HTmOYzFIGfw45UqYLGI1OLuTgWJlMDExK4ufJe5O
IX+ARRJl8qMMysjC4nEZnj3f7+/n9X5/P99vZnZ+spCIiDxWGiIeMd0e8cCv
TRFvuFHftkQMtUXkkE1HDKIPA+hHbTKi2BpRQq/+SG7JXJnX+IxLKOK5spPz
Zl7kD9UHWJTb5y6cuENJP4ekfQXu4A/5J/NZfVVzxIT6Tn0qP2XuGuv6K77w
LMEb3vMrt8k7qYhd/DRGLJjv9rwH57KdvjGD+oqdH8vtYcvcMNfYVY1L2W3v
eFA/YsrZvX7WnhnU2Tuuf1d/ue83Rt19DDmsOhvhMqfNLvNn6v///wEcNT30

          "]]},
        RowBox[{"-", "1.452`"}]],
       Annotation[#, -1.452, "Tooltip"]& ], {}, {}}}], 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdVX001Wccpw7ZbUK02pRi63hba46JrWOf5iYuS1KRRRnVZLIUR7gllUh1
DL/lUm6KishLXi4Lj6QYl9a0rpIbufI6Kb0sWq17e55/+p5zzz3PeZ7f5/l+
P8/3+/kY+//isWWKmppa5Nuf8v9djMHQmz/uks0nhU49UQaFj5DiUFayvWAV
8Umqf2bm8wgax2faNd1dR3iy7B32vEdoO7XPdlS+gVQaxQ2vrhxFy9oIw8TH
fsRABTAKba65ue/ZVhJqr/ziH7i3z4v6biKYtEqVMQLB4UrOLSaUmKsuGMGQ
mX+wMDaMKNHeQuJI0/TlFw5EEGU2Tj1D+Dyw3LAjLoqo4OyH0Dpt07hGwh6S
ka6MQUjWHOz72HEfeW6mBBxAwqfeN7+8Hks8VAkOYP34ohonpwO0vn6YX52S
t7HpIK2nHy9TZFy44BDZqoqHMCs+O5LeHE+uPlMC9sGrddfyWtfDZL4qwT4o
bJdqjUkTSbSqYAXKp+k+1XQ7SlRwsl6YJg7v0r1xjFirohddobcM36xKIio6
kx6gunKA+Af+SkZU5fbA7n7vealZMnF+VzD013RM8geTSY4SLrsbHxRVpvbm
pZB379UN0/6oZBKUSvmXY94Zw3y5JUf57oK+b3XDfyMc5fce2p3iPKUXf6N8
3sVXh7gJ7ZDjlL87yJ6YU7N5URrlqwOtrjprZcNplB8ZBM+1bPgXRJSPvxH7
Mvxx07Z0Wv8txE1rLYy2zKD1tkMvMtY5fCyD1ncTcybvfWJSeYLW8ycsIy3V
+6NP0vzbkO3u53rHOpPmK0WnPEQwoyOT5vcHjrbxJbl+YppPI6ZYzP3O4pqY
3t+Aux1LbutMiul9V1AcMFPh+UJM8WshMvfQLC0UU7wqSIxu7A6exb4vQ+5V
nkT4WSY9X4jzK/jGvo0n6Pk8HFC/35SWn0HP5yA/BBY6c9PpeTFObWs3GVmY
Rvc5/K6to8mv58jE3qDqsNQEDA6Le3b+mExe5T3+a6X/PljJHd1mORyj6x1I
5CcLTsfH0bUfzEhWd83rGLp2x9QKefN2z910vQy2wRu4Owt2EbX3gu1/0Wip
a28eAPa9qUCY6poQAobvVlV/2achAuz+nzmY56zYD5bfA03h3jnGR8Hy5z+5
37pZkAJWn0dMTf3+yXSw+jttjC9aDWSB8TPf9lLPKa0cMP4yE3lHluWfA+N3
2Yq+luLnuWD8+/30ckeMogDsfbgWm8s5C4rB3u9h9m1Hw+QSsPet3uB9plR6
Cez9BVVZAQWdpWD9YVb0zezhujKw/hEJIj1aosvB+mvGm+V1Qu0KsP4TyLmV
BsIKsP40kIr227RVgPVvY+hx5/U8CVh//+sXkHtriQSs/7sUPoufeEnA5oPn
296qs10CNj8iLtRBFiYBmy+XqYqx0bf7bP6SXDyHXntKwObToSB7lftiCdj8
Or/26656UQE23/nh8VWCEpa/HEZP9TJ5vhVg+uDdsXvdh6/KwfQj0cu70yCp
HExfmu8ZBzl9VA6mP81Hvq8rSi4D06cHEpndavUyMP3aWViNr7eUgunbqGmA
UFTD+FdAunR89vzpl8D00XahzkSCWwmYfmpcX1cnSigG01cjNamuSVgRmP5a
cN2BJo0XwfS5LLThByGvAEy/o62Dal2+vQCm72lXamTl/rlg+r/HK89xMOIc
mD+slOn7zt6bA+YfmRvb9MThZ8D8RUvjbGDfxiww//GtrBmtDTsJ5k/X7GL0
eXoiMP/qyt1kzUtNBfO30ojh0ybjx8D8b8LqpPH0+INg/hhvez3PShgD5p+J
CstYriMczF/VazurQz0C8Z7/4n9EbM4S
     "]]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  ImageSize->504,
  PlotRange->{{0.95, 1.05}, {-0.03, 0.03}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 313},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->239248625]
}, Open  ]],

Cell["\<\
The flow field at the trailing edge is dominated by the point vortices \
associated with the endpoints of the trailing edge panels for the lower and \
upper surfaces.  By placing the cursor over a streamline, its value is \
displayed.  This tool can be used to follow a given streamline as it \
approaches and leaves the trailing edge.\
\>", "ExampleText",
 CellID->38752132],

Cell["\<\
Repeat the computation after altering the trailing edge by making the \
trailing-edge points coincident.\
\>", "ExampleText",
 CellID->254018838],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rTE", "=", 
   RowBox[{"Mean", "[", "ptsTE", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"ReplacePart", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"1", "\[Rule]", "rTE"}], ",", 
       RowBox[{
        RowBox[{"-", "1"}], "\[Rule]", "rTE"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gA", "=", 
   RowBox[{"PlotAirfoil", "[", "rP", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c", ",", "aW", ",", "rC", ",", "VMagC"}], 
    "}"}], "=", 
   RowBox[{"PMDoubletQuadraticDirichlet", "[", 
    RowBox[{"rP", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl", ",", "cm"}], "}"}], "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Doublet\>\"", ",", "a", ",", "b", ",", "c", ",", "rP", ",", 
    "\[Alpha]", ",", "rC"}], "]"}]}]}], "Input",
 CellLabel->"In[5]:=",
 CellID->230503503],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1.0158461494132385`", ",", 
   RowBox[{"-", "0.35365269030517843`"}]}], "}"}]], "Output",
 ImageSize->{142, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->371932400]
}, Open  ]],

Cell["\<\
These are plausible values for the lift and moment coefficients.\
\>", "ExampleText",
 CellID->88671999],

Cell["Plot the flow field near the trailing edge.", "ExampleText",
 CellID->112543031],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rAll", "=", 
   RowBox[{"Append", "[", 
    RowBox[{"rP", ",", 
     RowBox[{"{", 
      RowBox[{"xW", ",", "0."}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"aAll", ",", "bAll", ",", "cAll"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Append", "[", 
      RowBox[{"a", ",", "aW"}], "]"}], ",", 
     RowBox[{"Append", "[", 
      RowBox[{"b", ",", "0."}], "]"}], ",", 
     RowBox[{"Append", "[", 
      RowBox[{"c", ",", "0."}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Psi]A", ",", "ic\[Psi]B", ",", "ic\[Psi]C"}], "}"}], "=", 
   RowBox[{"ICPsiDoubletQuadratic", "[", 
    RowBox[{"rF", ",", "rAll"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Psi]", "=", 
   RowBox[{
    RowBox[{"UniformFlowPsi", "[", 
     RowBox[{"rF", ",", "\[Alpha]"}], "]"}], "+", 
    RowBox[{"ic\[Psi]A", ".", "aAll"}], "+", 
    RowBox[{"ic\[Psi]B", ".", "bAll"}], "+", 
    RowBox[{"ic\[Psi]C", ".", "cAll"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gStreamlines", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"Contours", "\[Rule]", "contours"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "False"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "3"}], ",", 
     RowBox[{"PerformanceGoal", "\[Rule]", "\"\<Speed\>\""}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gStreamlines", ",", "gA", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", "\[Times]", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[6]:=",
 CellID->240459914],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJx0nQV0VLn793H3xR0Wd3cIvrjT4l4c2uJWL1WgQo22uC+LuywXd5dlWdx9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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVzrkuhGEYhuHnl6AxZmwHoLUVjgAtUYhmLAlqZpyBToLSUltLlWX01oOx
TTKiUbimuHN/7/q9wxv1hVqRZB6tUvJeSX74sS95wiI2+5MH7Mg3uGcomSon
Jd4SF/zVm/wNJN+8b8ce6mrT+mo8w9s8omcUz/a+4BRnuLP7wMwHPnGtf0zf
OCZQlYu+G75FU/1cfG9uV3zlfYkLNN0w6685PYd8hFf5N3SqVXhpMFlGFatY
wVr7dn+tc9nMMVry3eIT7y6ebN9nb4cdBX69/wGVOi5W
          "]]},
        RowBox[{"-", "1.9617000000000002`"}]],
       Annotation[#, -1.9617000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwNzkkrBlAYhuHXEplZIpTyD0yxM22UlY35MxdlWGKB+uwUG/MeG3uElR+C
+AwZyli4Fnd3532f85xT1j/ZMZEWEUN4yYroL4gYQA/60IskVrCeF7GGjPyI
WecZTOPUrJJHzetyIoa5lkd4KTdiGTfZEe8y1xxF3rPL5Gq5DB50TucEd8l1
40TvMZ7960rHNWrkiwsjSlGCZrMp+0/5L3zgB9948Fabvidu4Udu5SO0613g
edyb7zjvYg+LOv/Mf7Ht3MxNmEOn3Zv3qvRf2B36z71ZCklsmW1iAym9Ff7Y
KFPODTyuYwwJ9+vtL83PZM/xqvdWxx0OZFfl9vkfay5ACg==
          "]]},
        RowBox[{"-", "1.9656000000000002`"}]],
       Annotation[#, -1.9656000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwNzzlPlGEUhuFDgViAIkrFgJBoIHSyFmCQYemg0OBYWEEvDiQaAkGhoZ4x
LoV/AGgYMBZQwK/QwigKhpEom0vCGryKO/d7lu85MzWDw3cfFUREBgOXIiZL
Ip5ejXhfFlHN17Gtt14aUeVdiQRuXY5I6nUibf5b7w8O8A9/sSkvKacLd/Bd
3ckLyPnum7r4SsQGT6jHcWJ2jKz+V/0O7ydu7ctbVx+qj/DOzTH7S5y4FtFi
p4JbeYRHMYw0HmPP3hz/NJ/lXzzPn7jNrWbelDeFaXxxa8/Nz3xu72J5RBEa
7V3gIb+hkAf5ob17MlawjF23NmTs8E2zG2jyXa9eD/rt9GHR+4f8Eu9tfq5+
hjOZp3ij38PdmEDKrE5WLVbNZmXm9bYwg9d6r/ASeXk1fvdtO9Xczil+gAFM
qu9zjjOyFznLL3z3Vn6Wj8xX9dYw4780qPvtpdWjGEGVvXr3PuID/gNXYli5

          "]]},
        RowBox[{"-", "1.9695`"}]],
       Annotation[#, -1.9695, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0MtLFVAUxeEtqRXlNR+F2kjNqaZTX4E60SBQEaGJ3nk2UuLqDUV652vs
xPL6mNgLg6Js2v+Q18hJipWQVjb0c/Bjnb33OutsTmXydvdQTkRkMFwY0VIc
sZmIeFIU8RRTuHMh4kj/Hx6rm2kTOksjfpVE7OOP+i/eFESM8L+mj3ivyiw3
r8BlDLlzC7vmS3QZGaxiBXv8X93fQgoPZTxAo+wv9sryvFOXyRrUq+MfoLV0
gT7DRVzCcyxiCRm8lPcKB7wv6Bo6ZH1XZ5038ZZvHd36ozSFe94tpqUoQR/f
jv3POJ9FPm7wnKbDGJT32Z61eknnOjpm3zRG0a8+z3cOc97Jo7koQq/sn7Kr
+d6brbq/rfcNdzGrN4NpZOWUe6cCZWjg7aVp2kPHaNqd+3w16nb1FdpGf3hj
yht7tIpWoo+vnb8N1+V3oZ6/i7/RvAmneJppLj0wO8RvJOxegHGzm2ZHeh/U
HzGvN6f3X6/FuRXXkJBfiEn5G3yfTv7aPuN8EzgGxXlaAw==
          "]]},
        RowBox[{"-", "1.9734`"}]],
       Annotation[#, -1.9734, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0TtszWEYx/Gn2k2cnvacnuO2GCQurZRBopYmolSFatIUA4mQdFJDhTbC
4LIQejmt3izKJjGIWgxuodKJtunFZGgpEsRlFJ//8M3vufye533/73/dibam
00UR8QxnVkT8LY0Yw+5MRB324Gc6IpWKuCQ+Si/SI/Q6vYF9aMB+tKvPZCPO
0q2oLIuopu/sTNvT64w34u8847QfJ8V99BTN8qf5J+hrtVdq6+UZeUHejOdq
ZWqjzhumd+kQHbC/lO6V16MPBTwsjyhCqiLigH6Ii7EMi+7TbO423wD6cYWn
3hntzmqiJfoz4gnxLr2n4hfivPiX3kraYa4T59Eiv0lb6UHaiG70JO+WvCGm
7NimX+W73ot77ZukXbQH3Rh2t1t0hL5113EM8aw1swbTmMJOd6iya7s4Jz7M
89LcarX7zrpD79EROqi/ik7y/ha38NaK/9BC8h+wwLuIT6jWm+N/wDtLv6p9
w2d8wRLa3K9WP2N2oz3ldBOdTd4NC/qP7RnDIRzzHT94Lqg/Sv6F/7KBv4Qe
l/9L3sJZH5N3ojvULtM5uzbzLbejWJzlq5Tn+PJ4ov5BbZq3xsxV+TWc451P
3oi3Rp7nyaECS2qjdIv+f6ina5A=
          "]]},
        RowBox[{"-", "1.9773`"}]],
       Annotation[#, -1.9773, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0UtIlHEUxuEztfGa3UA3RYLQMhC67IKIbhTktKtdi26QYXY12pgZRKWb
KKYctWDGbqNBEETWpiLIKWtRKyNSqBZmZVFRZs+3+PGe97znnP83Wrt9X7ox
FRGPsH52REtlxF7cmhtxE01VEVv00xjV/zcnYhp/MIW/aNTPy/tQxpfi1ayI
a3wrXeHOS7qMDtPltICc/DlfYv4FPcA344tsAm36Rf2l6v2+4928iCF+nP+M
Ae82me+nJ8wuNnOHnpGdRQfOYUx+eH7EERzCgLmjtEDrzNe7e5Dv5xvUadTY
u+itatrL9+CCtx67dVKvPfk+ZO0sdGMBnsjqzGxWd8t+8kv4rfw2ZNSXkHU3
514f8tjh7YzZLO1GFz6620M/0FV2K+lDu+X0AR02f09ewW+Qj+At8sjhrrzT
XIe3Mt7opN8xYmeStslvq1eqK3DebLtbn/RO0U38Rpw2N2b/tZlR+obe0L+O
Sb/xG97rf6WltCT5H9kZlBdpOV+GGW6m3K7Wr0n+DnyrnV16O/Hb3Slzv2gD
3yI/br+gPqZuxhr1OqzFtBtd8j3qK7Ld9Cqt9dsX4Zmsl68yd1mWUgeN5DuS
78FM3Jc/NRvqony193/QIfwHjwJ4Cg==
          "]]},
        RowBox[{"-", "1.9812`"}]],
       Annotation[#, -1.9812, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ldsjmEYxvGHWFHdrRUixKoKB62DWokREkmtAwlFjUO0QhAxW6OtT4dd
MQ9IHCDBiVaMBKVDWxINQiSOHYjECDV+z8H/u+5x3ffzfO/7jlxXurSkRwhh
mJ/NqSHkZ4TQmhJCWXoI5biZHEJJWgg36H75F56r8iv4i0r+KlQggSMo5ZnE
Oxl/eLrRhW24bs8+9b24g9v4pJaWFUI6kpGKFGzh36O/G3n2trjXO/Vb8uzM
EFapjXVWER1DV8azzWxGM+9Hnuf0j5kVej2zQxjNd0beRU/T17SevqeL6U66
ntbRdbSW1rlfA/0sb6Q1WCOupmtphvs081yjl9V6O7cP+mKEWoteL/F7cb77
FInz6CXeZ3rp6glxsV1HaZO7PkON3oZ4B3o0nocEOuIdaDvNNTvRrs4Y0yb1
UnpWPkRvKF6IL9jXRi/SGZ7PdroDr/RfohPL9Jtpf/PHeU6gSC1PbYA4GSvl
3+N7d6ckek/eFs9AO76qZZnviO9MPFA8Pb4vmiXPRL58CnLFE3DXrgPu0kCH
yOfrzYu7zHyTL3fGLHkf8X6+MryRv8UHvRWo8r+f0MO0EhXIND+OJ4Mu9MwL
0cpzRG+82lT7djnzhnyafDjG8+fgAd9B9fv0EF3Mm+D9Z8dvvr+0mzbE++OR
3i/5Qf5j8n7iQnG2+JTeFvlWzMZj93+Cp1gi325XNU8K7xozOWo/47OhJehy
/iZ6Mn4r6ovi8zAzG0mxrraa57C8AgWYigV8D9V78tTyVMfvQr0m3tvZjThg
ZwFPOZ1OZ8RngdVmZ9JV9IfZcWaKxWPpU/WN9Ix8oGc1KH7j4nP2NdPzNFtt
VPwm7B1jfq54Dur1B8d3ovYfyPmaLg==
          "]]},
        RowBox[{"-", "1.9851`"}]],
       Annotation[#, -1.9851, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0jlMVFEUh/GDCjPghkLlNg4oaEEixI1CkWihcQ1GgdYSXBoLrNyiicZS
RVRMRC1IrFwTcIngaEJjIpoI6DBTW4uNqL9XfPnO/9xz73tz32SPnWo7WRIR
ORQWRiyrjBjixiURKXUar+QKLkeT/rC8XL1jUcRYdUQrv9Z7rveGf8mXFkf0
V0XcwwY0YsbaafP7sB97cRAHcMiefnvemRlyzgjP6l3V+8fv5WH9Uf4rl6rv
Wnsrv1C/xG/9vLNm+Ir+db6Bs+gx+9haj/5usw/ln/IjnotSTMvz+KaZci7K
FVyGNApyii9bb3PmYXxSd7iTTtxSD3Ifr/OM9Zg0X8916NJfwSvxTT/Dq9Cr
X8O1mNDP8mpc09/MW5DX3+7srGduTc5Vb0rm5G3qDG+U96ibuUZuUddyN756
9y5u1zuKI1gjT3HO2b1LfT8ek3f6VruwVl2HZ/oX7H/KF7nbe93mgtzHRb7D
0/zAswfw0dk/nJHHJL5jCsftbUn+S8ndIo2MfV/0xlG0XmWuGk+8U5n1M3op
bpc7MG7tM0Y8o0E+l/y+5C6ck/K+Jeo5yMqd1v/YmzMzYc8HbtI7zyeSezLT
nNwTLzA3HwO4jwbr/wHlyGjL
          "]]},
        RowBox[{"-", "1.989`"}]],
       Annotation[#, -1.989, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVzrkvH3AYx/HHYlPnxtJJ1fUT52BVuqBYmyYGiTRYmiiSNrQSWpqoOBe1
sjGQOIr2H2ji/AuKEkZHW16Gd97P5zm++T5u7WrqTIqICbSnROxlRuxjF4c4
wI9HEYmMiBIU4adczIv4lhaxJSenR2zzvPyOC8y+81Pe5HxeQJ29Hn6LDf0e
uRd9eO32TP/Pw1xeN89Td6ZG/PKPNflYPsEp2u0v+vNvdbf9J5yLN5jBirtp
XuVZ/srjWM5Su2twP8ZL8pj+jPer1C3qCm7mSq62V8Yv5FJu4nKetD/nX1O8
7f0d1Nttk4ftnNoZ4km9FL0tuVHOUT/nTTlJ/cX8Mx/JI/wKA+bHcj+Pmofe
hlwnZ6treV2+8/8R82G9Q/kTv8QH/xrER/y3c2nnH6/Zeea2Azd6tzhXl/CN
+XvzW650V4GEXIyEeSEXoUy/HNf2LvSvuFQuMMtHjfeq5L/697d9WbA=
          "]]},
        RowBox[{"-", "1.9929000000000001`"}]],
       Annotation[#, -1.9929000000000001`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVzzkvhFEUh/Gjxxi0JCJRaa29CrErFRJGROxLzVfQU1lja/kKdq1lbGNf
SzR+Uzx5zjn/e9733rLekfbhnIiYwXxRRD9SuCqMSGMpL2JWf52MuEEad7jF
uSxXlpfNnb3BstmcPiN/wD2e8IgLWb7sqDiiMT9iQH1r5w49+hdnqviZq/mV
E85s8LGdJrMf9TBG0Kd/5xp+41r+4AI7m3xip9nsVz2KMaT0X1zHn1zP35y0
s8X7fIBd8wr3GjebwEBBRKu+JBHRwqXcxpfeVOj8n3/t2TnMvg/3shXZoL0O
dSfa0Y0uXMm2fXcHk5jCqtmQ86c4w5B/1GTfhUp9Wl6u/lEvYBGfzkybrclS
+nV+8P0MTtzjGA3u9g8OoE/A
          "]]},
        RowBox[{"-", "1.9968000000000001`"}]],
       Annotation[#, -1.9968000000000001`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwNzUcyRmEQBdCWhrIRJtI2FMaoYgkG+OWcLQcl51Al5xymDIU9yJzBqVv9
vtuvixo66zuSIqKbxvSImdyIiYyIaTnHLEPmk6yIU54zI97p1U3Oi1jwvsg8
yywxrH+me86L7gd9+in6q97XWGGDdUb0L3QvedX9pF//ylzOpLnN7pS8MV/z
6L3NbjsJWmmhyfcjecwhp5yw5calvOKMC85p1u+UXXTQQzdPvte5U+luFRUk
3P9iwFuqeUdvl2322WPUnVt7d7zpfjOon6ZfnRNRQzGllDCmf6+br5eaHfEj
fymg1vunvQ2dA//elGW+jcsHO4U6aXb+ZJgH5T+CAkrs
          "]]},
        RowBox[{"-", "2.0007`"}]],
       Annotation[#, -2.0007, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0Ecy5WEUhvGjzOWJltoKLMEALQy56Kou2ZXjBLtgAUzllpWwBDm20GxC
HvrdwVPP977nfPUPpZ2j9SNpETGFREbEa3ZEFb/wL37jstyId66Rk87V/JEX
sZATsYh5dGVGlOvmzVrtHumO8ScrokNuRxsWzPP13fZ/pO7LnfoT51O02E/K
3ejCknmBPmm/kJflHv2Z8zna7F/xJS6wYr7nHXvs7/OqfK2/wS067N/zHf7h
r/mBvV77h7wmP+gf8T/1Xfb7Pa8PvVg3L9L32S/mDXlA/+T8nHpP+8PyEAax
aV6iH7Bf4f8MciVv6X/qt7mUJ3nO3V1Od2+HZ+Uls0+5NvXPuY6/OEs/433H
5SZuxm9MyI2cQAPG5Gl+89xvKQFJ5g==
          "]]},
        RowBox[{"-", "2.0046`"}]],
       Annotation[#, -2.0046, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0LkyA2AUhuGjVMhCbKXGdglmuAA6pQljIpMZDb01Kq5C1oqCwloJqZQR
W22taNyAJ8U733aSzJ+R3Pr8WldEFJBPROTwmIlYoTO456fpKrrxILfRQlae
S0Xs9UScysVkxC62+R1sYAubeHVz57aJPze3tIEC6rbFtO/DMY7woht29+2z
QzTvrqa7lut+95kf1H/JAzRnr+qu5Jr9ie/Xf8oZumyv6C7laueNfJ/+Q+6l
S/ay7kKu2Nt8Wv8up2jWXtKdy+XO/8An9W9ygi7YD3Vncsne6rxX/ys36U3n
zfiRG3TWfdHNiXzgfh8T/CRGMY4xTOEfK9c/NA==
          "]]},
        RowBox[{"-", "2.0085`"}]],
       Annotation[#, -2.0085, "Tooltip"]& ], {}, {}}}], 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdVX801WcYpw7ZbUK02pRi6/i11hwTW8c+jcTNklRkUUY1mSzFEW5JJVId
w10uRYqKyI/8uCy8kmJcWtO6Sm7kys9J6cei1brX9/mn55x77vc97/s+7/N8
nuf5fAx9f3HbOkVFRSX83U/xP2mjoK8Cx+4IvYLHSLIrLd6Rv5p5JdQ9N/F6
DLUTM20a761nPGnWTlveY7Se3m89ItvIKgxihtZUjKB5XZh+/BMfpqd0MAJN
YVNT7/NtLNhWceMfuLbNi/huPJC1SBQ2DP6RCqFLVDAzVT4wjEET30BBdAhT
eHvnEkcbpy+/eDCMKaJx7B7E5/5l+u0xEUzpznYQLdM2j6nF7WVpqQobgHjt
od6PHfazFyYKh/2I+9Tz1pc3opmbMsB+bBhbVO3oeJBN5tcH02tTcjc1HuLy
6cOrJKkwlH+YbVPaI5gUnRtObYpl154rHPbCo2X38hrnI2y+MsBeyK2XaoxK
4lmkMmE5yqZpP1N3OcaU7qQ9MI4f2q198zizVFoPOoNv679dncCUcCY8RFVF
P/P1/5UNK9Pths2DngsSk0TmNJkwdNe2T9gPJLJshbusLnxQWJHck5vEJqvU
BeO+iEQWkMzhL8O8s/p5MnMhh3cndL2r6v8bFnL43kebY4y75NJvHJ738NVh
4bhm0AkOv7vIGp9TvWVRCodXO1qctdZJh1I4fKTgv9Cwsr8o4vD4G9GvQp80
bk/l8r+NmGktBZHmaVy+bdAJj3YKHU3j8ruFORP3PzGqOMnl8yfMw81V+yJP
cfG3IsvVx/muZToXrwQdsiD+jPZ0Lr4/cKzVXpzjk8HF04ApZnO/M7uewb1f
j3vtS+5oTWRw711Fkd9MufvLDM5/DUSmbuolBRmcv0qIDW7uCZxF90uRc40n
FnyWzp0vwIUV9obeDSe587k4qPqgMSUvjTufjbwgmGnNTeXOZ+D09jaj4YUp
3L4Qv2tqqdvXCdn4voCqkOQ4DAxldO/6MZG9zn3y1yrf/bCQObjMsjvOrXci
3j6RfyY2hlv7wIRldlW/ieLWrphaLmva4b6HWy+DdeBG4d0Fu5nKe0b7XzSY
a9ua+oHuG/MFyc5xQSD/LpV1V7zqw0Dv/yyEafaKA6D4HqoL9s0xPAaK3/7p
g5Yt/CRQfm5R1XUHJlJB+XdYGV6y6M8E4TPf+nL3aY1sEH7p8byjy/LOg/Bd
tqK3uehFDgh/n59e7YyS54PqI2y2upK9oAhUv0dZdxz0E4tB9a3a6Hm2RHIZ
VH9+ZaZffkcJqD9MCr+ZPVRbCuofET/crTmyDNRfM94urxVoloP6jy8TrtIT
lIP6U08iOmDVWg7q34bgE04beGJQf//r45dze4kY1P+dcq/FTz3EoPngebe1
aO0Qg+ZHJAy2k4aIQfO1cqp8dOTdPs1fwkr3wTfuYtB82uVnrXZdLAbNr9Mb
n67Kl+Wg+c4Lja3kF1P8Mhg800nneZeD+MGzfc/6D1+Xgfgj3sOzQy+hDMQv
TfcNAxw/KgPxT9PR72sLE0tB/PRQLLVZo1oK4q9dBVX4emsJiN9GjP0EomrC
Xw7J0rHZ86dfBvGj9UKt8TiXYhB/qt1YXyuKKwLxq4GKRNsopBDEv2bCLn+j
hksgfi4Nrv9BwMsH8XekZUDNym8vgvg95Wq1tMw3B8T/ez1yHQbCzoP0YZVU
13v2vmyQfqRvatXJCD0L0hcNtXP+vZsyQfrjXVE9UhNyCqRP122idHk6IpB+
deZstuQlJ4P0rSRs6IzR2HGQ/o1bnDKcHnsIpI+x1jdyLQRRIP2Ml5tHC9tD
QfqqWtNRFezmj/f1938r+8dg
     "]]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  ImageSize->504,
  PlotRange->{{0.95, 1.05}, {-0.03, 0.03}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 313},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->36047808]
}, Open  ]],

Cell["\<\
Streamlines near the airfoil surface show no waviness, there are no \
streamlines interior to the airfoil and the flow leaves the trailing edge \
smoothly, satisfying the Kutta condition.\
\>", "ExampleText",
 CellID->667775740]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[{
 Cell[BoxData[
  StyleBox[
   ButtonBox["PMDoubletConstantDirichlet",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/PMDoubletConstantDirichlet"],
   FontFamily->"Verdana"]], "InlineFormula"],
 " . ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["PMVortexLinearNeumann",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/PMVortexLinearNeumann"],
   FontFamily->"Verdana"]], "InlineFormula"],
 " . ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["ForceAndMoment",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/ForceAndMoment"],
   FontFamily->"Verdana"]], "InlineFormula"]
}], "SeeAlso",
 CellID->169242850],

Cell[TextData[{
 Cell[BoxData[
  StyleBox[
   ButtonBox["PlotSurfacePressureCoefficient",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/PlotSurfacePressureCoefficient"],
   FontFamily->"Verdana"]], "InlineFormula"],
 " . ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["FitAirfoil",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/FitAirfoil"],
   FontFamily->"Verdana"]], "InlineFormula"]
}], "SeeAlso",
 CellID->163224971]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["3.1 Introduction To Panel Methods",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods"]], "Tutorials",
 CellID->341631938],

Cell[TextData[ButtonBox["3.2 Validation Example",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/ValidationExample"]], "Tutorials",
 CellID->499723042],

Cell[TextData[ButtonBox["3.3 Implementation",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/Implementation"]], "Tutorials",
 CellID->336290191],

Cell[TextData[ButtonBox["3.4 Flow Visualization",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/FlowVisualization"]], "Tutorials",
 CellID->307191390],

Cell[TextData[ButtonBox["3.5 Convergence and Timing",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming"]], "Tutorials",
 CellID->351529306]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["PanelMethods",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/PanelMethods"]], "MoreAbout",
 CellID->16906196]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"PMDoubletQuadraticDirichlet - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 2, 31.6152000}", "context" -> 
    "Aerodynamics`PanelMethods`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Package Paclet Symbol", "language" -> "en", "paclet" -> 
    "Aerodynamics Package", "status" -> "None", "summary" -> 
    "PMDoubletQuadraticDirichlet[r_p, \\[Alpha]] implements a panel method \
using doublet panels of quadratically varying strength and internal Dirichlet \
boundary conditions. Panel endpoints describing the discretized airfoil are \
located at r_p, and the angle of attack of the airfoil is \\[Alpha]. The \
function returns three lists of strength parameters, usually denoted by a, b \
and c, for the doublet density function, the strength parameter for the \
constant-strength wake panel, locations of collocation points and a list of \
surface speeds at collocation points.", "synonyms" -> {}, "title" -> 
    "PMDoubletQuadraticDirichlet", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/PMDoubletQuadraticDirichlet"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[15201, 484, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->580140197]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 356004, 7149}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[620, 21, 76, 1, 70, "SymbolColorBar"],
Cell[699, 24, 191, 5, 70, "LinkTrail"],
Cell[893, 31, 3884, 87, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4802, 122, 70, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4875, 125, 1412, 37, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6324, 167, 579, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->49529734],
Cell[6906, 186, 247, 8, 70, "Notes",
 CellID->758974],
Cell[7156, 196, 1848, 71, 70, "Notes",
 CellID->475782397],
Cell[9007, 269, 1565, 58, 70, "Notes",
 CellID->12242158],
Cell[10575, 329, 425, 12, 70, "Notes",
 CellID->342576209],
Cell[11003, 343, 863, 24, 70, "Notes",
 CellID->42056900],
Cell[11869, 369, 854, 29, 70, "Notes",
 CellID->452935176],
Cell[12726, 400, 613, 24, 70, "Notes",
 CellID->18491652],
Cell[13342, 426, 546, 16, 70, "Notes",
 CellID->523298526],
Cell[13891, 444, 53, 0, 70, "Notes"],
Cell[13947, 446, 1140, 30, 70, "3ColumnTableMod",
 CellID->14699597],
Cell[15090, 478, 74, 1, 70, "Notes",
 CellID->918694056]
}, Closed]],
Cell[CellGroupData[{
Cell[15201, 484, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->580140197],
Cell[CellGroupData[{
Cell[15687, 501, 148, 5, 70, "ExampleSection",
 CellID->250878843],
Cell[15838, 508, 131, 6, 70, "ExampleText",
 CellID->77267041],
Cell[15972, 516, 100, 3, 70, "Input",
 CellID->326492779],
Cell[16075, 521, 158, 4, 70, "ExampleText",
 CellID->301361980],
Cell[16236, 527, 217, 8, 70, "Input",
 CellID->317289154],
Cell[16456, 537, 135, 4, 70, "ExampleText",
 CellID->136872357],
Cell[CellGroupData[{
Cell[16616, 545, 112, 3, 70, "Input",
 CellID->720666783],
Cell[16731, 550, 159, 5, 36, "Output",
 CellID->132601626]
}, Open  ]],
Cell[16905, 558, 85, 1, 70, "ExampleText",
 CellID->20057380],
Cell[16993, 561, 147, 5, 70, "Input",
 CellID->233544807],
Cell[17143, 568, 845, 29, 70, "ExampleText",
 CellID->442126641],
Cell[17991, 599, 298, 9, 70, "Input",
 CellID->439469742],
Cell[18292, 610, 734, 13, 70, "ExampleText",
 CellID->205450438],
Cell[CellGroupData[{
Cell[19051, 627, 211, 6, 70, "Input",
 CellID->544103304],
Cell[19265, 635, 163, 5, 36, "Output",
 CellID->11363613]
}, Open  ]],
Cell[19443, 643, 190, 4, 70, "ExampleText",
 CellID->59684772],
Cell[19636, 649, 622, 21, 70, "Input",
 CellID->509204874],
Cell[20261, 672, 359, 8, 70, "ExampleText",
 CellID->19233943],
Cell[20623, 682, 693, 20, 70, "Input",
 CellID->555628545],
Cell[21319, 704, 484, 8, 70, "ExampleText",
 CellID->260053747],
Cell[CellGroupData[{
Cell[21828, 716, 1202, 32, 70, "Input",
 CellID->130177],
Cell[23033, 750, 92266, 1579, 70, "Output",
 CellID->11642941]
}, Open  ]],
Cell[115314, 2332, 251, 5, 70, "ExampleText",
 CellID->259606597],
Cell[115568, 2339, 135, 4, 70, "ExampleText",
 CellID->80454523],
Cell[CellGroupData[{
Cell[115728, 2347, 289, 9, 70, "Input",
 CellID->180266037],
Cell[116020, 2358, 260, 8, 36, "Output",
 CellID->843333845]
}, Open  ]],
Cell[CellGroupData[{
Cell[116317, 2371, 171, 5, 70, "Input",
 CellID->719599677],
Cell[116491, 2378, 260, 8, 36, "Output",
 CellID->133391181]
}, Open  ]],
Cell[116766, 2389, 71, 1, 70, "ExampleText",
 CellID->8410041],
Cell[CellGroupData[{
Cell[116862, 2394, 141, 5, 70, "Input",
 CellID->342608962],
Cell[117006, 2401, 176, 5, 36, "Output",
 CellID->55149086]
}, Open  ]],
Cell[117197, 2409, 289, 6, 70, "ExampleText",
 CellID->527976917],
Cell[CellGroupData[{
Cell[117511, 2419, 89, 3, 70, "Input",
 CellID->123809147],
Cell[117603, 2424, 193, 6, 36, "Output",
 CellID->53295971]
}, Open  ]],
Cell[117811, 2433, 121, 3, 70, "ExampleText",
 CellID->753558053],
Cell[117935, 2438, 165, 6, 70, "Input",
 CellID->435840926],
Cell[118103, 2446, 331, 6, 70, "ExampleText",
 CellID->139054009],
Cell[CellGroupData[{
Cell[118459, 2456, 1369, 39, 101, "Input",
 CellID->671472236],
Cell[119831, 2497, 6957, 122, 345, "Output",
 CellID->7098188]
}, Open  ]],
Cell[126803, 2622, 532, 22, 70, "ExampleText",
 CellID->571494656],
Cell[CellGroupData[{
Cell[127360, 2648, 718, 23, 70, "Input",
 CellID->42630738],
Cell[128081, 2673, 459, 16, 36, "Output",
 CellID->532539161]
}, Open  ]],
Cell[128555, 2692, 399, 7, 70, "ExampleText",
 CellID->24035690],
Cell[CellGroupData[{
Cell[128979, 2703, 346, 10, 70, "Input",
 CellID->34907031],
Cell[129328, 2715, 336, 9, 36, "Output",
 CellID->209153046]
}, Open  ]],
Cell[129679, 2727, 315, 6, 70, "ExampleText",
 CellID->47127959],
Cell[CellGroupData[{
Cell[130019, 2737, 236, 8, 70, "Input",
 CellID->274981366],
Cell[130258, 2747, 259, 8, 36, "Output",
 CellID->396978572]
}, Open  ]],
Cell[130532, 2758, 237, 5, 70, "ExampleText",
 CellID->328772922],
Cell[CellGroupData[{
Cell[130794, 2767, 353, 12, 70, "Input",
 CellID->762946865],
Cell[131150, 2781, 260, 8, 36, "Output",
 CellID->219137947]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[131459, 2795, 221, 7, 70, "ExampleSection",
 CellID->1293636265],
Cell[CellGroupData[{
Cell[131705, 2806, 252, 7, 70, "ExampleSubsection",
 CellID->421192202],
Cell[131960, 2815, 291, 7, 70, "ExampleText",
 CellID->5610347],
Cell[132254, 2824, 154, 5, 70, "Input",
 CellID->242241179],
Cell[132411, 2831, 159, 4, 70, "ExampleText",
 CellID->299806311],
Cell[CellGroupData[{
Cell[132595, 2839, 555, 18, 70, "Input",
 CellID->278559890],
Cell[133153, 2859, 5843, 103, 70, "Output",
 CellID->221681679]
}, Open  ]],
Cell[139011, 2965, 85, 1, 70, "ExampleText",
 CellID->176584194],
Cell[139099, 2968, 438, 13, 70, "Input",
 CellID->664892474],
Cell[139540, 2983, 120, 3, 70, "ExampleText",
 CellID->829463257],
Cell[CellGroupData[{
Cell[139685, 2990, 312, 10, 70, "Input",
 CellID->702755110],
Cell[140000, 3002, 6006, 106, 70, "Output",
 CellID->103725458]
}, Open  ]],
Cell[146021, 3111, 157, 4, 70, "ExampleText",
 CellID->33075010],
Cell[CellGroupData[{
Cell[146203, 3119, 563, 17, 70, "Input",
 CellID->539184285],
Cell[146769, 3138, 260, 8, 36, "Output",
 CellID->198210789]
}, Open  ]],
Cell[147044, 3149, 281, 6, 70, "ExampleText",
 CellID->511124957],
Cell[CellGroupData[{
Cell[147350, 3159, 1072, 32, 70, "Input",
 CellID->567786021],
Cell[148425, 3193, 4771, 91, 345, "Output",
 CellID->526203215]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[153257, 3291, 223, 7, 70, "ExampleSection",
 CellID->2061341341],
Cell[CellGroupData[{
Cell[153505, 3302, 227, 7, 70, "ExampleSubsection",
 CellID->30521753],
Cell[153735, 3311, 530, 9, 70, "ExampleText",
 CellID->120830056],
Cell[CellGroupData[{
Cell[154290, 3324, 1754, 55, 70, "Input",
 CellID->210128799],
Cell[156047, 3381, 487, 15, 39, "Output",
 CellID->628929861]
}, Open  ]],
Cell[156549, 3399, 290, 7, 70, "ExampleText",
 CellID->937232338]
}, Closed]],
Cell[CellGroupData[{
Cell[156876, 3411, 239, 7, 70, "ExampleSubsection",
 CellID->233233938],
Cell[157118, 3420, 611, 10, 70, "ExampleText",
 CellID->381223408],
Cell[157732, 3432, 307, 6, 70, "ExampleText",
 CellID->507899588],
Cell[CellGroupData[{
Cell[158064, 3442, 1993, 58, 70, "Input",
 CellID->49740593],
Cell[160060, 3502, 6384, 112, 70, "Output",
 CellID->149977732]
}, Open  ]],
Cell[166459, 3617, 164, 4, 70, "ExampleText",
 CellID->520516665]
}, Closed]],
Cell[CellGroupData[{
Cell[166660, 3626, 226, 7, 70, "ExampleSubsection",
 CellID->104403840],
Cell[166889, 3635, 620, 15, 70, "ExampleText",
 CellID->391229248],
Cell[CellGroupData[{
Cell[167534, 3654, 1654, 52, 70, "Input",
 CellID->451142620],
Cell[169191, 3708, 501, 16, 39, "Output",
 CellID->534793203]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[169753, 3731, 231, 7, 70, "ExampleSection",
 CellID->1305812373],
Cell[169987, 3740, 412, 8, 70, "ExampleText",
 CellID->416625656],
Cell[170402, 3750, 206, 4, 70, "ExampleText",
 CellID->728813289],
Cell[CellGroupData[{
Cell[170633, 3758, 311, 10, 70, "Input",
 CellID->112529997],
Cell[170947, 3770, 5180, 92, 70, "Output",
 CellID->281554140]
}, Open  ]],
Cell[176142, 3865, 150, 4, 70, "ExampleText",
 CellID->390242666],
Cell[CellGroupData[{
Cell[176317, 3873, 212, 7, 70, "Input",
 CellID->238007975],
Cell[176532, 3882, 345, 12, 36, "Output",
 CellID->170433896]
}, Open  ]],
Cell[176892, 3897, 151, 4, 70, "ExampleText",
 CellID->25666953],
Cell[CellGroupData[{
Cell[177068, 3905, 648, 19, 70, "Input",
 CellID->592619],
Cell[177719, 3926, 243, 8, 36, "Output",
 CellID->91869447]
}, Open  ]],
Cell[177977, 3937, 95, 1, 70, "ExampleText",
 CellID->722709344],
Cell[178075, 3940, 156, 4, 70, "ExampleText",
 CellID->492689119],
Cell[CellGroupData[{
Cell[178256, 3948, 2581, 73, 70, "Input",
 CellID->266881112],
Cell[180840, 4023, 102296, 1730, 70, "Output",
 CellID->239248625]
}, Open  ]],
Cell[283151, 5756, 385, 7, 70, "ExampleText",
 CellID->38752132],
Cell[283539, 5765, 155, 4, 70, "ExampleText",
 CellID->254018838],
Cell[CellGroupData[{
Cell[283719, 5773, 1047, 32, 70, "Input",
 CellID->230503503],
Cell[284769, 5807, 260, 8, 36, "Output",
 CellID->371932400]
}, Open  ]],
Cell[285044, 5818, 114, 3, 70, "ExampleText",
 CellID->88671999],
Cell[285161, 5823, 86, 1, 70, "ExampleText",
 CellID->112543031],
Cell[CellGroupData[{
Cell[285272, 5828, 2101, 58, 70, "Input",
 CellID->240459914],
Cell[287376, 5888, 62870, 1082, 70, "Output",
 CellID->36047808]
}, Open  ]],
Cell[350261, 6973, 238, 5, 70, "ExampleText",
 CellID->667775740]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[350548, 6984, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[350968, 6998, 661, 22, 70, "SeeAlso",
 CellID->169242850],
Cell[351632, 7022, 453, 15, 70, "SeeAlso",
 CellID->163224971]
}, Open  ]],
Cell[CellGroupData[{
Cell[352122, 7042, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[352544, 7056, 190, 4, 70, "Tutorials",
 CellID->341631938],
Cell[352737, 7062, 167, 3, 70, "Tutorials",
 CellID->499723042],
Cell[352907, 7067, 160, 3, 70, "Tutorials",
 CellID->336290191],
Cell[353070, 7072, 167, 3, 70, "Tutorials",
 CellID->307191390],
Cell[353240, 7077, 177, 4, 70, "Tutorials",
 CellID->351529306]
}, Open  ]],
Cell[CellGroupData[{
Cell[353454, 7086, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[353876, 7100, 148, 3, 70, "MoreAbout",
 CellID->16906196]
}, Open  ]],
Cell[354039, 7106, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

