(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    284288,       5930]
NotebookOptionsPosition[    273439,       5572]
NotebookOutlinePosition[    275470,       5623]
CellTagsIndexPosition[    275383,       5618]
WindowTitle->PMVortexLinearNeumann - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"3.1 Introduction To Panel Methods\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods"], "\<\"3.2 \
Validation Example\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/ValidationExample"], "\<\"3.3 \
Implementation\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/Implementation"], "\<\"3.5 Convergence \
and Timing\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/ConvergenceAndTiming"], "\<\"3.6 \
Aerodynamic Coefficients\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicCoefficients"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PMDoubletConstantDirichlet\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PMDoubletConstantDirichlet"], \
"\<\"PMDoubletQuadraticDirichlet\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PMDoubletQuadraticDirichlet"], \
"\<\"ForceAndMoment\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ForceAndMoment"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PanelMethods\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/PanelMethods"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/ref/PMVortexLinearNeumann\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/PMVortexLinearNeumann"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
PMVortexLinearNeumann.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/PMVortexLinearNeumann", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["PMVortexLinearNeumann", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"PMVortexLinearNeumann", "[", 
       RowBox[{
        SubscriptBox[
         StyleBox["r", "TI"], 
         StyleBox["p", "TI"]], ",", " ", 
        StyleBox["\[Alpha]", "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]implements a panel method using vortex panels of \
linearly varying strength and external Neumann boundary conditions. Panel \
endpoints describing the discretized airfoil are located at ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     " and the angle of attack of the airfoil is ",
     Cell[BoxData[
      StyleBox["\[Alpha]", "TR"]], "InlineFormula"],
     ". The function returns two lists of strength parameters for the \
circulation density function, locations of collocation points and surface \
speeds at collocation points."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->3053013],

Cell[TextData[{
 "Collections of vectors are of the data type ",
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], "InlineFormula"],
 "."
}], "Notes",
 CellID->308979890],

Cell[TextData[{
 "The discretized airfoil is specified by ",
 Cell[BoxData[
  StyleBox["N",
   FontSlant->"Italic"]], "InlineFormula"],
 " straight-line panels, where the circulation density for each panel varies \
linearly with distance from the beginning of the panel using the clockwise \
numbering convention.\t\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["\[Gamma]", "TR"], 
    RowBox[{"(", 
     SubscriptBox[
      StyleBox["x", "TI"], 
      StyleBox["i", "TI"]], ")"}]}], "=", 
   RowBox[{
    SubscriptBox[
     StyleBox["a", "TI"], 
     StyleBox["i", "TI"]], "+", 
    RowBox[{
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["i", "TI"]], " ", 
     SubscriptBox[
      StyleBox["x", "TI"], 
      StyleBox["i", "TI"]], " "}]}]}]], "InlineFormula"],
 "  for  ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["i", "TI"], "=", "1"}], ",", "2", ",", "3", ",", "...", ",", " ", 
   StyleBox["N",
    FontSlant->"Italic"]}]], "InlineFormula"],
 "\t\t\t\t\tStrength parameters are denoted by ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["i", "TI"]]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["b", "TI"], 
   StyleBox["i", "TI"]]], "InlineFormula"],
 ", and the panel-local coordinate is denoted by ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["x", "TI"], 
   StyleBox["i", "TI"]]], "InlineFormula"],
 "."
}], "Notes",
 CellID->312136876],

Cell[TextData[{
 "There are two unknown strength parameters for each airfoil panel resulting \
in 2 ",
 Cell[BoxData[
  StyleBox["N",
   FontSlant->"Italic"]], "InlineFormula"],
 " unknowns.  Strength parameters for the discretized airfoil are grouped as \
lists.\t\t\t\t\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["a", "TI"], "=", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["1", "TR"]], ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["2", "TR"]], ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["3", "TR"]], ",", "...", ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["N", "TI"]]}], ")"}]}]], "InlineFormula"],
 ",  ",
 Cell[BoxData[
  RowBox[{
   StyleBox["b", "TI"], "=", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["1", "TR"]], ",", 
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["2", "TR"]], ",", 
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["3", "TR"]], ",", "...", ",", 
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["N", "TI"]]}], ")"}]}]], "InlineFormula"]
}], "Notes",
 CellID->342196229],

Cell[TextData[{
 "Boundary conditions at collocation points provide ",
 Cell[BoxData[
  StyleBox["N",
   FontSlant->"Italic"]], "InlineFormula"],
 " equations, continuity of circulation density between panels (except at \
trailing edge) provide ",
 Cell[BoxData[
  RowBox[{
   StyleBox["N",
    FontSlant->"Italic"], "-", "1"}]], "InlineFormula"],
 " equations, and the Kutta condition (zero circulation density at trailing \
edge) provides the final equation for a system of 2 ",
 Cell[BoxData[
  StyleBox["N",
   FontSlant->"Italic"]], "InlineFormula"],
 " linear algebraic equations.  The system of equations is solved using the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function LinearSolve."
}], "Notes",
 CellID->400035615],

Cell[TextData[{
 "Output of PMVortexLinearNeumann is two lists of strength parameters, ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 ", a collection of vectors locating collocation points, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["c", "TI"]]], "InlineFormula"],
 ", and a list of surface speeds at collocation points, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[LeftBracketingBar]", 
    SubscriptBox["V", "c"], "\[RightBracketingBar]"}], TraditionalForm]], 
  "InlineMath"],
 "."
}], "Notes",
 CellID->299200057],

Cell[TextData[{
 "The same example airfoils and computations are used in the documentation \
for ",
 Cell[BoxData[
  ButtonBox["PMDoubletConstantDirichlet",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/PMDoubletConstantDirichlet"]], 
  "InlineFormula"],
 " and ",
 ButtonBox["PMDoubletQuadraticDirichlet",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PMDoubletQuadraticDirichlet"],
 ", providing a convenient way to compare results among the three \
panel-method functions in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 "."
}], "Notes",
 CellID->181230849],

Cell[TextData[{
 "If a list of angles of attack is input instead of a single angle of attack, \
nested lists of ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[LeftBracketingBar]", 
    SubscriptBox["V", "c"], "\[RightBracketingBar]"}], TraditionalForm]], 
  "InlineMath"],
 " are returned."
}], "Notes",
 CellID->71671648],

Cell["The following option can be given: ", "Notes"],

Cell[BoxData[GridBox[{
   {Cell["      ", "TableRowIcon"], 
    ButtonBox["Epsilon",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/Epsilon"], 
    RowBox[{"10.", "^", 
     RowBox[{"-", "10"}]}], Cell["\<\
offset collocation points normal to panels to insure that they are outside \
discretized airfoil\
\>", "TableText"]}
  }]], "3ColumnTableMod",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}},
 GridBoxDividers->{"Rows" -> {{True, True}}}}],

Cell["New in 1.1   |   Last modified in 1.2", "Notes",
 CellID->918694056]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->657232508],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->6004980],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->395972740],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->76286106],

Cell["\<\
This example uses the Eppler-205 airfoil specified by a data file located in \
the folder Airfoil Geometry.\
\>", "ExampleText",
 CellID->48545141],

Cell[BoxData[
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<Aerodynamics\\\\Airfoil Data\\\\e205.dat\>\"", "]"}]}], 
  ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->43842758],

Cell["\<\
Here is the number of points in the data file specifying the shape of the \
airfoil.\
\>", "ExampleText",
 CellID->136872357],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NumberOfVectors", "[", "rD", "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->59487574],

Cell[BoxData["61"], "Output",
 ImageSize->{18, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->79539540]
}, Open  ]],

Cell["Specify an angle of attack for the airfoil.", "ExampleText",
 CellID->711853039],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Alpha]", "=", 
   RowBox[{"6.", " ", "Degree"}]}], ";"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->847786885],

Cell[TextData[{
 "Solve the model.  Output consists of the lists of strength parameters ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 ", locations of collocation points ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["c", "TI"]]], "InlineFormula"],
 " and the fluid speed at collocation points ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[LeftBracketingBar]", 
    SubscriptBox["V", "c"], "\[RightBracketingBar]"}], TraditionalForm]], 
  "InlineMath"],
 "."
}], "ExampleText",
 CellID->20624687],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "rC", ",", "VMagC"}], "}"}], "=", 
   RowBox[{"PMVortexLinearNeumann", "[", 
    RowBox[{"rD", ",", "\[Alpha]"}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->53812608],

Cell[TextData[{
 "Flow visualization provides a plausibility check for model results.  The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function ContourPlot is used to plot streamlines for the flow field.  \
Specify an arbitrary field point using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation for a single vector.  Compute the steam function at the field \
point as the superposition of the stream function for uniform flow and the \
stream function due to the collection of vortex panels describing the \
discretized airfoil.  The stream function for the vortex panels can be \
written as dot products of geometric stream-function influence coefficients \
and appropriate lists of strength parameters."
}], "ExampleText",
 CellID->10578989],

Cell[BoxData[{
 RowBox[{
  RowBox[{"rF", "=", 
   RowBox[{"{", 
    RowBox[{"x", ",", "z"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Psi]A", ",", "ic\[Psi]B"}], "}"}], "=", 
   RowBox[{"ICPsiVortexLinear", "[", 
    RowBox[{"rF", ",", "rD"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Psi]", "=", 
   RowBox[{
    RowBox[{"UniformFlowPsi", "[", 
     RowBox[{"rF", ",", "\[Alpha]"}], "]"}], "+", 
    RowBox[{"ic\[Psi]A", ".", "a"}], "+", 
    RowBox[{"ic\[Psi]B", ".", "b"}]}]}], ";"}]}], "Input",
 CellLabel->"In[6]:=",
 CellID->133172956],

Cell["\<\
Generate the graphics for a streamline plot for flow over the airfoil.  Some \
experimentation with option values were made to obtain the plot below.  The \
number of contours plotted was used to avoid streamlines very near the \
airfoil, PerformanceGoal->\"Speed\" was used to obtain a contour plot in \
reasonable time, and MaxRecursion was set at a value to avoid noticeable \
numerical errors in computed streamlines.\
\>", "ExampleText",
 CellID->76299467],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"gStreamlines", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"-", ".5"}], ",", "1.5"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", 
       RowBox[{"-", "0.4"}], ",", "0.4"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"Contours", "\[Rule]", "22"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "False"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "2"}], ",", 
     RowBox[{"PerformanceGoal", "\[Rule]", "\"\<Speed\>\""}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"grD", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gStreamlines", ",", "grD", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7.", "\[Times]", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[7]:=",
 CellID->808226483],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJx0fQVUlM/3PjZ2YYLd3RgoV+zu7g7sbsVuMbBbUVAUaSSH7o7dBekuJQXb
Hx/hmf2/s9//Hs/xPMy8M3fuzNya6rBmx5z1VdXU1NY3UFP77/+KXxJ7cP+/
XwzbreW7S8s3nOP2/8Fd7zieN/+/nzcBD7uWWv5PwXFqxR84fvvvlyPkL+D4
X3W7SjiuoOebgJOYkW72Ld3sSE4fMOgDBn3AoA8Y9AGDPmn+Ao5BHzDok+Kk
ynxBnD5g0AcM+oBBHzDoAwZ90vwFHIM+YNAnxUmsUeP/fu6cPmDQBwz6gEEf
MOgDBn3S/AUcgz5g0CfFSazJVvfmW91fc/qAQR8w6AMGfcCgDxj0SfMXcAz6
gEGfFJf3bwUm3r+VmPdvJeb9W4l5/1Zi3r+VmPevJH8Bx7x/KzHvXwku518F
Pzl9wJx/lZjzrxJz/lVizr9KzPknyV/AMedfJeb8k+Ak1rZiPHL6gEEfMOgD
Bn3AoA8Y9EnzF3AM+oBBnxQnsdsV85nTBwz6gEEfMOgDBn3AoE+av4Bj0AcM
+qSYy2cS5DMJ8pkE+czpAxbkM6dPmp/LZ04fsCCfqWVF/ZV/PzPS5z+olcnx
8Ir6WYt/41XG5xPwv+RhpSRN/8Yxytf4h+04xvzcVcEPng6M+v/++yXydGCk
I3+rinRCOjDaA4z2tKygj/cPMNojYvAL30O/CemcfrQP9IjtxfxHOjDoBRbq
4/0H+jA+0L6K8RNNGv/mr2Vl/8cxxak+//1j/6BWGQEjPaYCE9KBMR7Bb4w/
Kf+TqcO/+u9wfqJ80Cd+D3rxPehF/eCnZuU8r19RPujl9QHj+wp603k6sMgP
kZ9or5DOx7PYHoxH8LtBRfkcg15gaf8nq8h/kf/i/AS/gPH91X8M/cL5C3kA
XEFvFucHxl/F+PHhGPJ+QUV/M3wP+QRc8b+BM9qH8Qp+AEv1fbpK/0n5483b
h/5Gf2F+gF4+nwR9iPrxPfoDWGovxJFU36er8BP0gJ/A4Cfoa/2vPGfOX2Ck
I784HtpW5sN4AMb4QnuRv0KfeHL6pPZFHq8P6eAHsLT9eST2H+rD/JSOj2Re
P+9fCf/TeTq+R/3gn4hRP/gj2l9oDy+vEvPyhHTUj/Er6h9gjHdgjCdgtA8Y
/AOuqL+IY8jnf+Q0DmeoHxj1A6N+YNQPjPqBUT8w6geG/gEGPc0r+o/TAwx6
gEEPMOgBBj3AoAcY9ACDHmDQg/7H/MD8RP8DozzkR3nAKE8q74q4vEN+0X4R
5y/mG75HOr7/3/aCDx9fwOAnMPgDjPKBpfaFDy+/Yj6F8/KBub1UiaXyOlyQ
h+HC/Azn9ACDHmDQI9YP+0LaPpmgP2SCfJaRVJ7IBH7IBH7IKO1fh5TRCImd
YOAs6neMD8gjyA/Id/Qn8gNDPknlbTgvD/lFew3lc/uzUt5wfSeR7wqV+jF+
pP4O0tOoYyVfK/iTwTHsDfAD5YPelH9/iKQWkrhEHtc/kI+ivkf94C/mB/iD
8tE+EYMfFeMlger9S3/O+S+dX3505588tub8gHwGP0EPMNIh/6XywY+398+/
/KEco35gtAf8xHyX2pvZnL6K+jN5fvBTas+lc30Hfoj9GdXtWES3Yx7cvxP1
gzQeFVcpl6O5fBbjdfge6fge81GM54n6ToyfIR3jUdR3YrxPTBfja0jXqvQv
gdF+UR9V5FPGs5CO9or6E/nRPlGfIh3tQzraJ+pX5Ef7kI72AKM9wJiPwGif
qN/EeBjS0T5RH0v9B2+uL9E+MX6HdLRP1NdiPA/paB8w2geM9gGjfaK+FeNp
SEf7gLn+qswPfV/5P5e3wKAP34M+YNAHDPpE/S76U0ivnKcM8gv0AoNeUV5L
7dMCbk+APqk/9I1j0CfaD2I8DemgB1iq/90F/a+M3yE/6AEGPcCgR7Q3xPgZ
0kGPaO+I8T3R/hHjfZDnGI/ID/qBQT8w6AeWxkPDOf1ifA3p0nip0p4R43+i
fSXGA0V7S4wPSu2pHI7RPtH+EuOFSEd7RXtMjGfCXpL2j4wE/SH0j9J+E+OJ
oj0nxhel9lqOEG8q4FjafzIev6jUbyTVc9Hcnoc+qdAvckG/5wv2XjY1r7A3
COnS+E40nx+wD4Gl9q+7YA8FCfZIkKDfgzj9iM8Ai/EzUd9L25vADh747ydn
//48/yuPJ4EfwEjH9wI/OUZ9PN4gkZcGzuAf+l/Kz3CuL2A/QR6Dn6K8lsZn
k3k67EHYU0iX2oNJPF0YD3w+i/E3pANjfAn84eNNjNciHflRPtKBUT/4XzHe
S/nfK3Asgz0OLE0vJWl6Cm8v8kvttWg+3kG/6P+I9jr8DR5Pr8SYD6I/BPkL
/PBf/f6CPFPGS6A/oS8xvyBvgJEOfdf5H37Eqv1LP8ftRel6ZLqg77OE8ZYu
xKfSuH8B/wgY413q33lzjHSpfozj7Qe/pPHgPCEekMPxzn8Z3Hl/gn6pv5jJ
6UP/on74a4/+1e9AkHfASH9eMvlpyWTG5wswygP9kN/AKA/+B8oDhvxD//H4
oNB+0It06fpvHu8v8A8Y+SFfgKXx3jxhfTdX4NdnjsFfqT+YLozfZI6hr8R0
afwlhfMDuHLcc34jHeNXrF9qnySp+G/i+rt0vCs4xvwB/6H/Ia8EzPUjxgPo
B0Z/SOPHWZwfkD92q+qW/1Ou30EfQT8hHVga7ywl5Ee8RlqegTPaB/kEPPdf
O2zpbsV45/6KqD+QDnor6xP5TaDHvqJ+Lt+l+1++cvkNeS/qZdg7fhX85fId
+YDBD1EfgB/QTyI/pfo8nnX/b3hFyDk/IE/AP8jfNpV+quifAFe0z53LW+TH
/Mb8gnwBRn2Yb1J9nc3nH+arNH6TRrH/2uPHMeYP5DMw0sX5g3Tou4h/8y2C
y8uK8RzK5y9wRf5Ezj+0Ryrf0/j8RPnS/UTR3J5D+aL/g/5FfRf+/RScH9L9
H7k8XgSM+YzxBXsE4wvjGRj2B+Y7+AF5hvkBfSTKD3H/CfqP++OV9KA8yDd8
DzohL4Axv7/+x/6nUTzf04r+4PUDI/1WRX/zdGDoY2D0BzDqB4Y8E8sT9d0y
ixpLLWoEcnkIDHoq2hvB7QFg1A+M+oAxfoBRP/oL5UnXx6NJ7E/IR8QrMR9h
f8H+g77BeAJGf6J/pfFIb8G+zOEY/Y35BvohjyAvwCfIH6SjfcBIF8cL0qXz
W+nnYTxDPorjB+mgB+moD+mgBxjpyA/7Dhj0IT/0AzDou18xnjh94nhDOugT
xzPSQQ/SYW8Dgz7kh/wR6hfmR7rwfZ5YHujg7RH8NSasbwj6JJoJ6/FMWG9h
ojwT909I4/eZfHwjXepfRHOMdMxn8Bvp4DfSwV+kg//ASEd+8AvpmN8iPdAv
oAfzHfUjHfQBI12UL0gHPUjHeEA6MNLR/0hH/wNjPEvpzePfY3yL8gr2EuiV
yic+nzn9UvvTwBn5QS/SIe+AkS713/N4OvgPDH8R8ZxKe4rLQ2CMJ9gXsHdh
P0r9MQX/HvoY5cPekdaXwzHkMepDvA3jBPwU12dE+1+Yh9wehT0MeQkMeQR5
j+9RP+QZ7BJg1C/Vv94cgz+QLyhPlB/wV1A+0oEhT1CfKB+RjvEl6ifkx3hC
fWi/VP8rOEb/iPRAXsFeE+uHfwl/XfAnOEZ+xC/4/gSJP5bE60N+yDtp/NmT
55d+b+AsjccbOIvxfJQnjT8rywMG/0V5KMbnkQ4syhsxng55gXGH+LNUfkfy
9iK/4Efx+Iso78T4OtKB8T3mH9KBpfG/PB4PR3wB+YFF/xnp8P+A0X7IK/AX
GOMb9jv6H/4i+CvujxTtQ+n+UgNncT+fKF+RjvYjHfY4/F0hvsn92//tTzHB
3vUQ/CvGsRgvwviUrl94cvrEeCLGG+QP0oGRDnknteczeXpFeyO5vwR/W4xH
wD8BvzG/QQ/kB7AYz5bmT+b6BONbrL+0oj4eD0B/wF8A1qjQb9yeh76Txr+9
ub218B+2Zb0q/HmCfw/9BP6gPcBi/BT+AuQf9Bn4I8YvxXilGI+EP4X+R/3A
Un7mCfG7JB4vAJb6Z95cn6D94v4X8BMY/Q35iHgA+P24YjxX6v9PDH42+gv5
K+I6fgz0VvAzlxZV0FFZX56A0yrpwfqlEsPeQP4KfijTMT5AP/gNfmD8AYOf
6H/Mc2k8TIH2cPkE+wLyTPTnRPtEGh9JV/HnYL+gfKn/laRij0jlTzqXR2gf
8gNrVZaL8qX+VBIT7QfRf5LO72Qer0Z5on8E+4TrO2E/A9Ih/0Usnm8S/SnM
Z2BxP4I43zH+KujP5vIC9Iv+V1uhH//3+Ry+LqKCkR/9D4z6RP9Iak+gfmW8
DOnS8zWeQv3K/QKi/4bvpes5yvNAyA9+S/2rJG6PYHyI/proD4nr/8gv9SeV
6/9SfyxaZf1fGp8s4+lCfIUEf4HXDwz9Dv0CfYX5Cf0JDPkrPf9YxDG+x/zi
50kqMd8/XonxPTDsCTEd8wvp0v1GRXx9B+nS8w7Zwnm5Io55fZUY8T3pfpcs
4TxbEce8PiE/xpPUngni3wPje3E/APpTuh4Uyb8HRn7Yb8J+Ol6/9LxWEcfo
f9E+hLz4N0zK/XvYP8gvrscgnZ+fkOwvi+YY+h32D+S1dP0vg9sD0KNSfZzE
xPUX8TwSvkd8VhofSuO4Yh6UUc4/gR7D7SekA0MfAmO8IL4KewP6HfYU7C9g
yF+snwDDH0R+YH5+qJJ/4M+NCnuK708V4l/Cfu8ETh/8Uel6Sp7QX5+F8rOE
8y+IF3hz/qI80Ad7Hvlh70vtbaX9D30H+SbutwZ9oEe6XzZNZb0I+gvySDzP
JbU3ojkW4k1MmF/cfkd+0Cfsx+HfC/OP26vIj/iU9Hsev2LCfK2sr5Dnr+BX
sUBPIf8e7RH9M2Cki/Fs0X4S7Stp/jyejvZI4+95gv1VKKw3FQv1FfL8mN/A
oBf2GNojxekcY7wAgx5g0Cu17wo5Bn1iOuw/0Id06GNgpIvrTdLzpJl8vkFf
SXEh93dADzDSMV9QH9JBDzDSpfOrkO+XQPnASBf1AdJRPjDSkR/tk55/KBTO
TxUL87WQ50d54vlMYKRL98PnCfGBQuF8WLEQTyjk+VEe0qX2lSdPR37p/r9w
7r9a/udOLQ3j8li6/pcuxN/zuD2H8Shdjy7kGPSL6WI8XrQPhfUGIf6VLMS7
4oT1Rb6+xu0bqT2ZrhJ/BxbkEadfWn8hiesD0vh4snD/QpLK+Q/YX9DH0C9o
j9ReSxDiU+mCvRDL05Ef6cDi/i7R3sB4rPAfVO0L2APQV5jv0I/A0D/S+Iaf
cD40lutb0AeMeMGrf+35wOejeP5EPJ8mXc/N5fRIx69yvxLaD/9Dej4jjvvn
iG+J67PwH/A9MOI1Uv3jrRJPEM+fIF1YL+LxfmBh/HP7Q/p9usp5FOl+zRyV
/NL9xZkq+lFKT4FQf7HSj//XnhKOwZ/GFfKG8w/6BvwFhr0s8hvfozwtid/o
zb8Hf6F/wX9gpKM8zF+Uh/4Q9bV0vVwhlJ+ucn5GXF8HBv+BwV9p/gKOwV+p
/VLCMeQlMPiD8yngp9SeiBPSy+VXJcZ4EuNFSEf7pPZBAcegV7p+VsIx6AVG
vEKMN8He4PGdSozxgvyIF4jxJ/F8DdIRL5bKpwLBni8WzjuWcAz6peWFq8Sz
xPUyMZ4F+wP0gh5g8XyL9PxkAUn3rxUL9zuUcAx6gaX30SjjY6gP/BbP4yC/
9P4aZ94eyF9gUb6K9yVI6/fm8TC0D+lon9ReKuEY7ZPqr3CVeBz8QdAjPU8T
x+NlaC++B5bGt7xV4nHS/a7Yjx7E4xPID4zyIA/E/dbS/csFwv7dYpLu3yjh
GPyQrn8mqezfEM//iPE/6XkbrMeGC/HPSN5/0vVRvr7P4xvS9cpkjiFvgCEv
gcEfaXqByn400Af+SO3VEhV7U2yPGI+R2oPJHINeYNALDHqBQZ80f4FQXrFg
j5aorGeK54Og3xFvwv45rAdK99PlC/eN5HP7qKK+Ao6RH/IL+aX2XwHHhv8a
IOPrK5h/sDeE/cOcXsyriv6IV/k76Ac/pPfJlPB4HvYvS/cTK/dfo7+l9+l9
5etLmC/Iz8d7pf0O+xf7ZWDPIh6I+Kt0/0QKX18T9sfw9VHEAzEfpbhE8F9K
+Poe6BfXt/8/+7t5fnG/Nv6vSI9hlfY+55/0fE6pcB6ulPsHmA+Q/ygP/Y34
IPwT8BP5IQ+BkS7qO+hLPt8r+wfjU/Q3YO/Dv5L6F348Hii9DyGI58f6Irc3
K+OniJcingd7Qly/Fs//IH4KeQu8+B8dr1j+lv/wfm5PSs+TpPHxhHglMOQP
8sM/Qjr2z0vPy/ipYOhP5BfXw8V4rfS+uTgeT4U/B/6h/8TzV9L1i3yV+wSk
9+V8FuKv2RzD35Omh3OM+AbKq+jvqMp4hxtfjxD3f0Cf43u0D/NdjE9L26sQ
9o+EC/5nPkn3K+QL9luecP9kLu9P8EPa/wUq543wPcqXjp8vXN4Di+urmB/S
+0CCeLxcuv6eprI+L+VPihDPSKE9FfOGp4PfFfSm8v0A6B/Rfwd9mJ9oL+wP
6X4Pby6f0T/AFe2R8fEA+pLi//u58PEqvR8vX5gPBRxj/oEfiKds+PeT8XgQ
6of+k+qzeDbrh/HMH8ZyLj+l+z0MnKU4jMsTyCPp/j83Vusff24w5Mf4w/6N
6H/8teIY8hj6Ef4t7BHoNcwHxA/R/9Lz3ik8HfYJ9BXsA2DYAbAnkF96PimB
SfeHeKvcNyddb0hViX9I8xeo2BdS/RiuEs8BRn3ieRRg1CfNXyCsN5RwjPrg
r0NfSOMR3sJ6gUKI/6dyjPql+QuE9YUSYX/+N5X9sNL9SwVcnuB7pON7YIxf
8byN9LxGCpc/oB+4or/L6f83Hl/y+SY9f1VC0vH+jaTnv8KF/fgFgr4pIfF8
IbD0flTcxyPn8gv8lO7XLxH2p34jcX+q6P9J968XCPHvEmG//DcBh/H9dhgP
0vg1jw/y+qT2JY/n8fJRHvShdL9EMseQj9L7potVzpNpVcpp2CvA3B6oxPge
GPSI8S4xv3jfshhvQjrsGYwPvr9Hct62WOV+ZDH+I+YX7ysW4y/i/cjS/fly
rn+k44Fx/1C8b1iMf0jva8jm+kd63jVIiKco7wcW4wfi+T/pfp8sYX9PMcfI
L/UXsjgGPdL7M4pV7tuFfwH9KfVnYhj0KfwT+IvA0nh+KdcrsG/hz/H7Disx
7H/x/K3U30vk+xehz6H/gaXrLUk8P/w/xBvQnkr6uL8l3U9bKtwfUkqbKtrP
MfgFejFeYY+jfcBS/ymRDCvkK5e/4voQ5CUw5CPsJdgPsA9hPwAjf8V+23S+
voPxBvrRP+BfRX5Lzj/MV8wn5Ad96E8pvRHC/Irg8hvyGPEM6Fe0H/uDUR4w
2vP7H78Zbz/6X3rfQjK3JzH/RX8C9iyw9P5SGQ0e9N/Pk/sjyA8svb8iiufn
909W2nvoD3H/i/T8f4awfzdJ5fy/9LxyslBfhnD/W6pgX6ZxDHvSosI+5PMX
ehfyQ7o+msX5ifFY0Z/hnD6pvam6ngn5XO8fvkJdK/8O+SWu10rvE4hVub9Q
Sl+aCpbeN5BGy/812Il/j/lZUX+M0L4MPl9g/2O87qpst9S/DuXzD1i8Lxjp
Ff3jK8inAI4x3qX3BXqr3B8I+qB/pOd3s4X7QnN4+RXfZ6vsB5fquzTB3koT
/MdY7g8A/xuOgyK5vyBdn4zj+YFFewPjEfIG9gPkDexfyBtg+J8YPxXxoo+s
SaUdDHtYvJ9AXD8R1uuZdD9mKbfPID+BsT/w9cz/HMgo7l9J7xWM4/HWiu8T
VTD80Yr6Erg/Cv8I/EN7pOMhmfdnhT75xKCv8L0YX8T8xHyGfAC/xfiRGO8T
z0tAXiO+Bv8X41Hqv6QJ94OmCfZfGtePGH/AkHf4nsu/yu8xf5Ef+g9Yu2J8
cH4BV8jnj1zeSNd74oR4SwzikuA3x+CXeP+CuP9Iut8xl8ufCv3iyfsD8Q20
FxjtrcifydsHeQ0MfYXxDizdHxLB2w99Bizdr+TG7StRv4I/sK+hn1A+MOI5
UnvliyAf81XGnzh+pPur0nn9sKeAoQ8hv0A/6MH4gP5FeVJ9rGrPwt/m46lC
vzDkx/xDOvoT81d6n/hnPl6ApfF4Zbyogt7PHGM8ifYg6K/Qx4lcngjnBXl8
CvJK8A/4/j7p/UGFvD/gP4F+6Jun/75/w/dvSe/bLxTin4XC/eqFgj0ZK/hb
Mbw/pef3cd9iCZfP/5L5fr1ILg9F/11637ycpOev5CrnBaT36X7l/hCw9P2R
BB7/BZbuV07i6cL6Gi9f6r98Ffw3pTzHOJX2r0JYb0rg/grsFen5hDA+34Gl
9kCSyn45qf2ZyL8HFueftLws4b2ObEJ8FVh6H0SByn2LwBXyKp/bl5gfKB/y
GOUDS98/+MzTgaX7X1MFfyKVDv3rL19u/yJ+C3mB/uX70SrlOfSv9H50ucp9
4Kgf8lh6PtJbuA/Wj6ejf0Us9UczVfbjgp/AmJ+wf7A/CRjjGRj8Boa+AYb/
I5yn5evpmJ/A0v2nBcL4iKFf/4m3mQFcPkj1d4HKfVLiexbS+9Plgj6SC/GF
Ah6fgLyB/IT9Lo2v5grrb7k8vgp6xPP36E/4T2gvMPiL8QSM8QZ/BeUBg/+Q
V9BX4nqJeJ8m/AvYm7APkF+6Xp8k3JeYxO0R6Ffped80wZ9PU7nPXnqfUIag
/5K4vMF8ldrHn0l6/8JnTg/sDel9R/mCPflFsK/yeH+A35CnwvknLg+k97OW
0aWK+7u4/4v+gzxA+cDS+/7l3P5CvA7lAUv3HyBeGcPjy6K/Ld7nJN3PnMP7
G+2plDu8vZBvwBhvwCgPGPIT81t63yy/j5b7Y8Dwb8T4nXQ/TynnB7B4/6l0
fTGV+wPA0vNCBSr6RxqPy+H9Af6K9rb0/s4IId4QIcQvlOvLmE/S9eck4Xx7
Erc/Yf8hXgT7tkKeenGM+BLsBak/G81Efxf8F+xhbp9J41UKjjFeUT/yS9eT
4jiusNfihfkWJ5zvTOD8AJauh+Vx+Qr5C4z2Q35AHknlSZzwXksCn4+I90r9
kSSKq+hPfp+L9PxIDG8/xod4PzTqw3hBfZifyA+M/ND3UnslWuW8qmhPS+mP
E+4nyOXzFxj1Q3/BvkL5qB/jU4oVwvdyXj74KfIX+cF/5AcW7UWkQ34Bg17N
ingE11di/cAV5SaxfcPktVNbR/P10yElCctHL1Vwe+v23G6tp8+JYdPM7ao0
L9dX4vsUGgHT2r9Pw/10SayDxpCd/Voo98fX26jQUuhFsiUJq68vH6j6HqvJ
wTCPe4OV71moO2U36XJFuR5+dI2ihqdGgHL9Wz+536gVyu9t+hYX31EL5N+n
uW4/Vj3dny3/lDqodK0dk66npdL6FonW3T+68e8LJk4yKGqn3O9yuq6lw+n5
Lmz2kSPmsw87MOl9k6l06zdt+6RtzCZtf6fdfJDqe6OJJ213bOn8kK8fd3Xc
Fd15yn3O34uLm72tc8WC3MZNf//RxJrvX65uutCkfRsPlfcFTLSMqn38bc7X
a/VXjP7TbZc5X8+K2d8p+7iDhXJ/8lj2oZO2J7nMsQowSIxUuS+Aqn0//qOr
8r3Q0b/dfGqrefD1iZs7HWvOsnWn5QlVm7SVu6q8F/o7MmltWncPTt+v6nGG
Fw96cPr6BvVwfTDeg9M3Vy/4puF35XssG1bFPe6/J5jTc1DH7fJZ3XK9oTMp
72hTR5X7AW70aDcj7WgwH+9rCjNy9rYI4fW/kNnk6WQp3ye1nuo2PShcuf64
WufajSfngklXw66W91jl/QBof/HWaU2+GUXx+jJdslv2r67cX2znMd9No08U
b7+P55+7DadFcXr2FM3+Oen/eW/Uqol83/CqUTQidu/cI/eiyehcswWHRkQz
16yTN47tSKUTU7vuiKoSxU4Na7SjdmoxLXLOHfM8OIpNuTDsQd3cMjqzrvtj
7zpRzNlL8ajOaRlN23Q46M6iEHbEroHxBI9isl8zaeialyFs8Krjm384lJHe
rUZ/FLODWfWaPSekW9ixa5YmVc08PNjXvdrpXsfkdGjRJsep/TzYhK0mxbbP
iml+cKCPhrknu9RvgsfXc+X2VJOA2uuNPFjm5iuWZqc96KplqFazcWYs8H1Z
/W1bFVS71smaL3VNmfO9dVodZ5fR4X7tD9qf+MB6dby3zPxyDi0InbKgzc9X
1KnjzqmNI7IpruaDztW3MdrYTPG+n7qCGqmFTpjkEUjdRwxPCAhKoUW/tCZO
GRlIVa20J5h/jGDdN19UPL4QSSXr1frNtyygZQPSZqXei6Thu+/sMBxbQtdP
u6RPfRpJl6NudvSKt2UzQjwvZUcryGlDzU3WezzpaDWXahOGx9D+paemv30R
TZltY57vmlZuH7pFykqfpFBshL7hgFgFDfbZYDzOJJtefPD31QhQ0Bnfzbp5
08vtGw+Zev+3CmLmjZ13fC6mmjF/78V6Kyh+8grZdN0YVrvD8l3tHsvY0YCq
RuHfv9KjHy1M5neVM8OeQw3KNpeSwVvXDUUN5Ty+Py2y/apBLjKWUjajyZXc
MpXzWFZlTZtN7/KOdWj4PnWmehj1mdH2x7QrDkzz8YljB/tFUtdttsv+VnFg
+Rb1LtbLSKQfs5KeF+bYMacBf/ecr5NEw+oWXWyTasfsc+Lr/hwYTm+cNs/d
bm5Pgw02G8ZfS6QRw/X7bapnRzXtbmbMXZlEEYvmPFx53ZaW/N26c9qBDIrd
c+VmavhHqpYpN+pnGcOqluY/f9dUThNnLvv97MdXejW1z5qQ5TLyLvyg13xH
KXXrqH3DRl+5H1gz5/qydb+V+31fNI1LSlqpfG+1ZWrsKKNPb7h+29jkwxLX
vsr3pW1/ai49VWBPLiceteypl079/nazPRdmQTNLSnO7H0+nwveKn1uuWXB7
oWnzvBETa71hddzbG472tqDW79c1P+xsS4rdlzoN6vWSy1/w9/KYfbdsEt9S
0fzVAxvMCae0OhOaN27nSymhMU9ypvqxHvEzX34pPUU7Cwu7u53NoGpbhtsN
SrTh8YHJ6Wdjnk95wXaNP75qa8NEWliltF/KSV9uj6xgl42jDN+yqmvOr93v
lk0TQ5pkn854xqb6dorPv5dM/nWaf7tQQxkvdmg8usdSZ18yp4Ks3o+iabpd
ZJubFp70JGSo/t1yOaO/9Ynxa1NPSqw/f2Qrlksnmw1oecbiKXvW5serm7U+
0z7mfT1I9wkL2N8hrXkN5XuQj3+M/pm8T1wPzaSxGxqO9w9x4/Jq1h3H2J0+
tlzfXfjY5049M+V9Bu3qkYu92Ruef7/ZhIvrbluR3LVVrxHPvdndgg1zdK1k
LOtD1fEJDR9QjPHoVLtzcrZK/eXVbv7lfusOq+av/7u3e7yauo5XCHk81509
PCia9bz9cGgNzQTa2v+0mv5pOasxo1lxa5Mk0hr3lDQnytnKk79azPLNoBsH
9TrPPiZj5970yWyv/pkSdn7s+Tg4mll8uayf8LmQ9g68YnEiX876hYYZ79Et
prOvB8QmvZezI+PetvHtHcN6+pVo+haHsaftxxlabPBhjzTNh62PDWN/lnT1
2Z9pRu9q+x352SKczYs30/hu/o5+rDi8vL1GONuZf/rlt5hyOX5v2eXQqREs
rJNLk6E9g6m1QUgVp1uhrO2U0BW33yTQk66uZ9QfhDOfgOp2u44kUeq0Izsi
9oSzPh1eN5hhl0HrHqenLmFh7HbZ7QdNG5b74x2+tg7vGMZ6bd24bri8kEwb
pGauXBvB/JO2nhy/oJj2DTbf9KJTBOsQ/VV3b9hXcnNPMezxMZxNDDgV0elK
KdWaed5y3J1wVu3S+EnejWOY3sjXo/Y88mH9oxpdTp/sy9jTNxsHfvdmE3vd
mZO86AXZNPS2/eLqzc4Ua5z9lRhEsvOXR57Z5s/U1Nf3f9Oq3M/5ujul4WM/
lr5+vNO7CYn0zaCTjfl8X7Z+VPCbAxOSqO1D2ZVeA32Z/Shb86KnGfRnqtaC
ET+82S+tn/F1Gn8mo9W91jbO8mL2pdYetXcW056VTQKqDvdnkS0nXTwcVUgh
f8+Or6HhxepPGHI54dVXsvoddC7lrh+r/j3pUEPLUipN3fvim54f29LpU/9+
2Qo2wN3g5bYAO7Yjvv2MGikKVnDupnuSpx3bfenSjPlXi6lxePHt5ludWdrB
CScCMgupnW9M8NAUS/aj/vqn1Q99JS/DPWPWX3ViJtk/59eLLaWWt54fkfs5
sgZdt2xe3SGWzfLrbjLb7wSbPb/r5OgbChb2w7io36JVrErdmZNHVC2ibl7L
nGsrrtMizSYRobXK6GOfdr+GWxqxlG+Zar4HvpL87RQnk5m36HD01MtmVeLY
1D42K3d6WdKS7WHVLrVQsNbDa3/9c9iWVtxobnNhTBxzqtO0Yf+uzpTR+V73
kr5FtL5s6MJ+MQ6krSnbG272lVrqRM6ZMMmRfhtNHrhpm5xlyRMPtL7jTWdy
Hzg5XvnEOrSZd2JZih+Vpq0LiS33q7oltJrdcKAXeQYM0Xr81ZuNDbEJulDF
l7o3CNM3dnxOD2ICwjU/eFOZXV7ZcGMnCtXTD1nSxIumvK9buKqbOTWNPn7y
zW0fUt92Yt39facoWTvVc18nf1J/OPf1s2afaV7T3847b/lRTfXUlBvbi6gw
NmaNU7l82xvRl7pnfqXhtSe9L/XxpY9NNxXq3JKxhHeJSzp1DqNhD+59bGQZ
yw5+uPLWzD+cpqwsNVr32pO9eXc3J8kxjL4t7i4rGxzIghKiegaOiqCSTn57
att9JJ3O2ROuVg+jsTVDN5fUfEVtqmxQq7oznEyOren0+lUY3Zk1/HfMmXB6
O7GBw9Jl8bTAt/nS0KphtDt3yYf3c+KpZUa7PwM6RVDm78v5Lztk0LdnZ1xs
Y8Lpc4ed361qfaHvZpYdtgwLp/dt8s5uflhEabcnyEy/hdEXo0t+VR2LaPSD
AxpnyvHgjTVb+XYvpRe7fg5U9AynjaOPtE2McWMnq/jX/fIpmvo5edj3e+nH
9hvU8bqrL6fbj8et+O00kjYYmn0ZvCiajDs7LsoeYU3+LWfMHKYno/YbPwxy
fneNlrW90GJJoJxMBk86tHVVMNUbZlCl0dloOrza7MIBy0jSyBloMvF3NFnQ
6pv2r0PIST+g+KyznJYM6my0dU8i1c4MbTvgoozeJXjtnzcmidzPudgvvSKj
Vo1SqZZ/BnWbveqAxXEZ6ex6eezroUxyenph9qP9Mgrr/LC+6/XPZBAZ9yFz
hIz+mF7Y37bwC7lsnOFZu6+MDro/eDJ7QhGtWd7gcXx7Gf2KNw/XqFdMqw3O
1zXsIqOdQY52n9eXkZvOI4OanWPI+EDPr8NvbiGHy9XWzu94j1q1dZy0SC2Q
HGyqf59z34XK6p4+Mq04hKoGWj38s9uVVlbJ+/s+OZ7yXR46bXF15fEEnQGf
Dm3tYcIGfz+vde5GvMr9ANFX6s8839ae7C4faXD7dgZd6ZudU81T6c9sNk9N
qzIihkJ2uE6rZR5BxmPp2VLjN6zx4vnd/hpGUWbva2Mzhr1h/e9+L67R7zP1
ndU7OlnzGfU+b9ys3Zsglf0ooWu+Lhpjb0eLA+uvWdMojm5+Grh+3kNPavf2
1wq/7Dj6fiHpxqiLntTslFXLi7m+NH56J62itLesieEazx2f/cnJO0Z3hd9b
lvwnuNZ95zCK/z0ubGzzt+zWw/wbZuYhNLzYJaLegrfMdcbwOpomWSr3BxzQ
if9c47wdZZ4t3KYo9xc/LZM7DhhvS/1zd/64kJVJpXqNez/4ZEYatTU7nxqQ
RYPcvB7mDTajet8GaHx0yCVTk7Hf6q9Q3jcw4lmDo7on3elWxIVFY9uk0cpt
6ve/fHUnS8M5d4/Fx/D7AmCfNdTzy79lH8ly1nV6tuVnPBva/NPcJPso9mJn
82FNxqi+dxmyY8ZKg8XRLLZKiwU9Z/iq3D9w2WqhxhXbSO6fz321d6/uDKV9
m+n6t19c3yi2TLNZs6MP0lTuC9htMrFO0Cxl/GBl7cufetRXpg9pXC3LyUgZ
D1gQvP5Eu0XRzK/B4bItd3BvUBBvX78vaSG/uyjfs7z01fWu/pwINt96dSOt
ff48P+hf/6PHokOHAjn97b/t3GnXK4xZ1fp+yaD4IY9P8PuAPQakOUQGsrpb
vF6N9lZ9z9LB63iTfV2CeXt8V3tX/b4kjNNvn9+w1ZQToWz6zx4DDHvFqNz3
vFhf8SGowI3T/zE2sUb41XK6U+6Gymcc4efpXULG5sU0YSz2es6aK+ucWBOz
6RveNfFjaVd9Xyy+bsPjGxY6HjFVr/iweRM0jjw5kqJyvn7N5jn9wpco36fs
Mn21Wf/J3pzed2ub9Wo82Ut5Hv6No/6f4yZ8fGjOmdlj7jtbdtR5Rr5/D0t+
Hj6o6JNH25bK+5/f2ebMbLCP0aHBDXTGjXLi5ZssbtpjThMr5fn9lkOjJvS0
VI5X8q8W8f0lPSofAotmvOb7mT7M7njg1mNfturRvT9hRh+5/5/3Ne9SXMgN
7v8fvLmh0Y6MGzzekHfsS+KQH07K+5P1Ek6Mn8i4/3/k8P2A5gcdac++tCpD
uu/j+6MmTTU0mlrip3K+3Eh2fJh5Fw/qzfbnnNK/p7KeFdwxqnU3U+V90j3s
33nuWaqMlxTaWw84NdOR01sa18l3QrYDl3/DnrYo0l3ixOmvW3WMy7plfqT3
vObTx3+iVN6PHLajyP180wDlfdBzO2RsHBFKOVklJR7bzFTej4xxOD7tfjvl
fZAHiksLf+5Tnh9vbb/qOHsXoPSnxjfz1S4I4fGOHkk7jjoMVp4fP72/34nF
9r68PSfmjFukY+jL23PfcJes7hg/3p5Dad8GB+qGk1tOvat5VyNU3pMc+8qg
QM8uiseHdv5qP8XjbRg13vrI9XCLBNXz5S2/Xh01Xnk+vGjGqJlG+RGk/37s
ItNV/ir3X3c4ZfPibrjyfsnL3Tq/SqivvO+6ff2Idl/tlfdLTttes+FKdeV5
8ccZzXSGHAmnjLPbjZ9uV31fcvzL7DVFtpEUktha21FNeR4c/Bj73Gnjh7XK
9yXVQluvuegfSVP+jnCd6BSi8r7k96Wh/s0yZHT4SkTakWILnj5Xe03n67kB
Ku9NNkrf7+3UW056f0YHtt7BVN6f7Ci3bLlzrIIWvJTn/NofxfKrfjkx3lx5
f7Vb20keg1ZGsfk556fOKLdbmh+90zM/1pQdN910rO69fHrjsbSVw3IrtlLu
vP9a3wKqW7//yFo6VmzERLez9xLyacaaXn1imt2mspbTR+9qVUAXzik+tdQ2
4v58uym2VWVWjPOruIEic8tg5X3WBTnfDmV0DuR43TXbfI8eyvus3evlB06x
COT+bonLdN3Z/QI4Xjewzw3txBDyq/88/Gn5fKk1yEoWWUdB96JsY7N3RLHO
p5/tmHhRwZzSRv2Y9zaBXWrctLbCWM7X8y33x7t8y5WxU84F/von1rDppgud
EqMtmPqMPReiwi+QjdoxDbfXH3i8w+OyT++p/h4sN+IQ3b3sT8XaB4fVau/I
1xtuqtts2XXYnq8v16npukB/jg3d61A6IatfMgV86FxyI96GVu/acOz+o6hy
v1OzyYin0RT98km1sAmfmHR/fhxb/ETtZ+oZ5X3XcZfjqxSdkNFVax12M7uE
prYrWP05RkHVn/lPfH6rlMY+nVpHXz2GFi16727qXUptjG2WuzaIofcmzteD
4lPILTkl48l4a1pddVpxs1bK9xFH1dE4pBZ+kxUYuS09PNeVOi9wDLdccZYW
ZHxd3OjvBxrheCD6Z48C+rHL8cXgkgdsXvqI/uav8mmde9WJp8c/ZL+uvDk5
JDaEnnesWeNdkQddfDRuaz15AO04cHBC2qgAmrP7Z9jSeaE0pfHQSeruAWRY
q1XbhIOBPN5+YdR77dGxbky+KHRA395PSVESeVSLVN8/nLxCJ8Pk5iE2bG7g
O2unR2zJmqY7J507xyZ1Dtb01H/MztmNav7ownluXwzt8uTz4XZu3J7S9EuI
cNN9yVY5LNFMiEyjsrPHdsrPvGR9atR8d+9ABqnb/qm6/sUL5hhVGPTiSzo5
jNRtNTnvBbPrqbE3Ps+NArsZrhtY7t8l9qzSILONHWk/3TLf8bsrdZzYuoFa
vI/Ke4g5z6bNtmnkQAN0n7x62KV8/pye2X5VewsasnbNtolB+eRxMl3HcIsF
yaxrTWzWNI+Cqt/8EfTMhWYvKMuo9z6PfvVso7unnwuXR90+XemQ4OJKueY9
XWf08CC1tRsKPjxQ3i+eNfBMyYolz9lI37NdWiVmUtvJ7ea9TH3OzpxYfl2u
mUXescGH9DyV94+f32/Q4Kz2RzJ7Yd/cblcUlVy8pPGkz0dKvK89PqWlgj6x
HZutanyky5stLtfTkdErlvesrP9HqqMd1K7P9kha1Xv6qxHHGPVXr7ukVrkc
zI0Z51n3gnK+j13UrK/FLUsy2JL+vH6i8v1DpB9+sGze5Zau7M5O78tzbHKp
b8MNuephrqx0cPXg0w0/k/rVQ7Mv6ruycU1fuDQslz86DU+MvnnPhZ1ucvjW
qDv5VJedGTfEy4WfP3425dKHJ++s2F0r818x63Jp4anDU6eqP2MX8w6mhFh/
pmjvq00/bXrMCvp31bfdkE35YxxTmjt5UdU5G2d+uZZGc2pYjKxbPm47/16+
NTIjnWbE1F+09lQA389g4LN9aEQTXyqad9wk7kMK9U27Lpfv9SBf1z8dZpik
0JAjYWU+Bzyo6vdmTiYGSbQi9eam9ZrK99geXVKXDWnqQ6adlrmV6AWqnDce
UmK9f7Hcldur4f37/Lhx2ZEVZ+mnjo0un/ehA1/JnRglRO1I2P80n9qN2tUi
fqQT/RzxwKjnmAJ6Or3/wFkBjrSIkuf2DM+ih2naOjXt/Ume6jXq10UjOnag
xsVbJ1xZ0yuD/j5uZ8XXl9s8/DJ9U2wcNb+Svz5qkz3bPbBVmxvzbyvfO0t/
MTVunye3z0qaTV430c2Nde2nue3K3Kssen/2kqyzudR9vHdKTDUb6tCuT2rY
zVwq+5juXVhmTRf0P1WLK86jkTPvfJ1raUUnJ7+flhOSTa8LZga2TXzG9598
y5n4OLibNe1Tn1R/RXw6Nc+sfckr05vC9v0IioiOpqGmG9czi1DqvfBb7+7N
UujenIEDonxCadmgnxdTxycyh34TRme7433ANMoxN/s1YbHyPvjZ3cpOHxmk
vA/+7oL799YZKc+bN/1x/8Wrbcr74Gf8vp+X4vycbIwf9Wm2UM5GFcXXzLsm
5/rdyPnUedksfg8S07D2LGgf85LPV00PTY3nW56xBaODDp6695afHx177fGZ
BUe9WNPG1T7XuCNjEfXnzZj0yZ4lZBZu9guIZtuNV4UVX82kG2MLTR9vkLE2
lxq0bpGWTfXnNH5p2FHGsme6mruOL6O16g7D5/aKYoMaWL1tXypnDRoH7Gpp
FcZSRrj0uHcmiq156bm0qCCUralia/LGxJ1NUCs87zQvjB1o0GnS9nmObEnG
lTvD7ULZJO2YW490FGw+m9O7sY4Pa1di/XHYpSjGiqolOF32YlHb7Tpr73dh
L+warOxu68nOLDiy40bRR5pyofWsyY+8WPh5l1PvZ7jSxz83Pj2d5MW22M02
6mkaSYou1r2zb3iwDY7GW7LsMumD1/S4iQHerFNOmxELo7NJd0tT/1Zbvdml
QyPH97+fT9bNOnZxverFji3P8d3Yq4CuaMsndN7rxS4t7SivctCdDa1Wu3m3
6X50UN9uX/wRJ5Y01yRBf5M/HV8ZU6dOghdb/afe+oZjgihoQ7JWtbky6uxQ
xSL6dSB992+p8DiRQmZpTgcXTAmkO/Wu9enVKIJ9HvWj+5S+4WS79nCOQTMZ
+3GrS4OaARHl/oDuhjF1HEhR5WCxWtcIGnjd2XL7Wxc6m9Q9PnV5BPX9cC2x
o+V7MgrqFjAhJpKSM/K9i05F0m6/83/ve4STaYHm9x4fZHTCro9XRFG5vdjz
sJ1Wvwi6FBGVYMwiafOR7a8vrv1ChpeiL/UbFE66g4cFjzn2hdT/aP28bBVJ
Wn1vl0ycWkqGTsYnj5fTt3pB6fuJYWX0Zu7nPluWh1NJQObjNeXphevDD78O
jKQhW4Mvvl1kztiDp2O8YmRk8rP0tN1LV2b9/Pwam/UKiminnTI30JP+vvHt
M7yRnF50XjQ019ierN5NfrHeVEH7U7x9jdtF05/UDW9/jVFw+6plj5HBvx+8
p26lw02bGGcK+7m9qYpufcPJvjaUfjj16tiqnirv4w3bG5JvGGnD7evbN/p2
DX7oQ8nRQ3XeJduxXjOsv+Q6KZh2luLg9O2hrK3t6DfbnKPYi99af/cFnGNJ
Rf43jNLkrO2PZvHVxntRj3nFmR3XxrAWRvtWOl28zqLXFe0+lhvFAtSq/2IX
XpKV0xenqUujmMmfubae5fPl7htHnYk/Fcw26PaBFeczybRNxO6G42LYJk29
rGfGqTRwycF1vX5GsulpGg3fH4tisg4BHbZERrBN32P3/0qXs/rVWjUNPx3J
btd4ckwRFM+MPRa3nT44hNUfptZjkpoD+3xKve2M6AjW8VDXAV6TPNmCxEQ2
zyCSdVyzeEx6XCjTP93+elZuMNuvZtRnt80dpt2guKciN5xZHu76KTnMg/a8
e2e+ZFMk09zrYx3Z7gl7omM9crlfMBu27p5RH6NYyqr7aGBSYDALaq1Tq25Z
AjVpGTfZdXww66Vzx2NwSjb1tXo1OmlSJLvf7lZwzzuZdNa6wD3+fCR7u9VB
u3ZRSbmdePr5xIRwlnMh8eP6i1Gs46gVFmxkAGuwPPHIgkYKdmvr9o/BfoEs
fazXEfmNeFZUf69GYFVPtubglHrNY5xYl/3PN3f66M+8pvbcu8KsXC9ldjY7
UzeAVd+f8/dk+zC2yb9kWNe1HmxMzfZXkkzdqJlej78DbgWwHMONqTXTrWik
7hDFlVOBrL1a74zn1d4wz13H7yRvcmfJuj9CXAsC6Oh2Bz2dmZ5s4JqD+zod
CKGCDdayaa09WZVWzX1rT4qlHBM7p4cJvuy9wZgTa0+n0q3IXh92rQpi5s32
9GxdGkPqLeyH2eS5s/sRxTvldxJpzZCTpwesd2eOQ/Z1jrVOpYXGWinaZ33Y
23gn3yMfculo65iTbdp7s+6nP2wY91vB2kyPmnPilxs7U6/PuOyt8Wyva2Jc
aWK5XJTHpsWXy8+JPR9Vm8tcmI/X6SSZgYLZjLzxu4WHGwt1yfO5qxfPJr2+
p9b+4gdWp256luOOGGbR16f5z5OmTFevWUBMg3hW9dsB1i3qJjuYtbNZ1RVR
bMjltx+mFzxiVepqZXpExjGTttWWnhl8nxpGBK+d/SeW6fhEFq+u8Yz6zF+g
bWQYx253S3/GRltS+z1LnnSoFsU0NIPqpma+p9fDBlpHtopnYxJG9cpY50SG
V1s3u3Mgkx6vcNrfNN2e0j7/DZ41LoF9PXvgabVy+8R2Vb9R2/M+sTd5Bho3
jntSP/o7z3RzJIvrvWtV5koPWjBN30fbM5H5P77VoateEP1qm+nR3CCRabV/
MdGtmy/1n3O9YHWQBRvYzblzmZE7LXv86PKfiRYsf8LhKWqLfEiRMq1MviaY
TdMtdKn5w58Cki72/hbsy/K7e/Z22BtI46Y9uVZUZkzXwqjdnjB3+n6hm8Mi
s/1sasHWyf4DvKh7yr13EVXfsftOvV7NtfOh/kUjeyfretG8iNZOuw3c6WOb
HkfPKlxphl3sngHrvIhCusVfTXrFonxn+Zv4l8vzyPf9Xy/Oo4Xtmp986eFJ
n4cV//K5k0dNO7a+Vdfcky7PiDAZuKGAtu8/oqH9woOC61UvjF+bT4cd8o9s
z/egDXqtdm67EcE0phqcfN40mBQzKf1gRgI7uyK044uDkdR5/NUbF7KDWPjS
Vy/rtw6lm/e7Ns1bEMTCJt18qE2hFEC3ysn4yLzq2c/rbhhCJX8XB5w4E85+
/e105n5IJJ3/MrlkpWUY29lp7NgeBuFU85jTZ7MTnjTKvtkh4z7B1H9v7LZ1
PzyobtqKo4fvhZXbxaGtnsZeY4ZdtTenJkbQ3obWJ05eMKUHWU2m9bWPpATT
/YsPjPenjqfSc2rVCKGdpe9reGmF0LQjgQ16PA2hTYcvmWxdUG5n1n6m9jo+
jJzmp6bt6ulPNrs2vK3RJZIGOfaJOtAhnD4cG+/063UkbXSMcjS58Yn+Dlj9
efTvUFrl1LhGwxnplHn3be4s3VBydczuVmtlOpXletjsGhFK6yyfLPx9OJtK
5ueOeLMwhPaG9ddI3ZNFG3utHDD5bgiNKXawPd85ly5bnra+tjuclrRZP3H3
78/0dPhrdwuHcv02cc/NK49K6Jex9c5Q40gqbvfJIbRTKT2p0X70Wd9IitOY
8OjUDSs263qO+clFUZQeo+jadnYYW5Meaa5WPYa6LtMYNbdRZ7bkztSuCxZE
k12bmgfHznWnGanei/fZRdHFb8/67tpgwg55ZBmoL46hBc8Oh+0NfM3U+5ao
udSW04d2XcY65wTS1kaBBgt3RtPmeNsoxxkKymp7e75P5yja9aCT4vdLOd2u
88n5np6MdAxYdy1dH3rac/aOultiaGbVuXtbrPajNPerdiuXy2l5/MRCu3J9
t/CB2ob/9gfdWNDyVOmwXPJpFhZ2e6aMstK+jZp5soACSyefHdFZRpqOXrtS
jj9n1oOORoxd9YJqZmh8Mvpixh6dU7RY3fMu7Um7UjVmYgyFTEvctLSqKfM+
k23SYUYceb42vXxskAkLXvV0hva1GHq2YF7NQxH2tFzniO+gPnFkOdXqSed+
9mSt3vzu058ZtDi05o2rccr7qdo5zXJrnWjL42HXPmlPndfWlL1tXsu8bXwu
NdC2+Fy1zkvS/lg7T9vkM1mvnRVUv84Trs9vxRS1WGbzisdr1sj29q0ls+fx
voWn/P1ad4lmusdu+RbYK999hX2sG9mr3dZvCmbY5WEttfdmKu85mgYtr/3x
rYxdmBiWubCcbvE9x1wXuXqdkXKOZ3Vc8W5gs3Du7/csWD9o7R4Zv6/lUb72
x13OISy6w778QvVI1fcd9RxGHE0L4/Hue/nmMwdvDWGzqyUu2DrfTvW9x0ke
Oy84hfH4+nLLDsX99Xx5ffpLr34bN8ST/bnwIXewTqTK+4+jrui9v3fOg9c/
paTW0cLzPsr1jY7R2nO97Tg9c9v8Lfjx0J3pOkw3ihwUxnIaHvr0JdCM+4dH
hptb9nHwZCMGmrS7Md6T31fyO9+vvtbzl0x6n4mBc/jX9w5XryrPG88zDVGb
M9Wb79fZlWcwr15tJX2jo7uvmamvz9crFC/39XJo6cLGLb3tkaubqPIe5HGr
Ts1HDzRhVms9HXVHetL6p3e2aXo5slr1Xxx79SyGvmkX2Zfl27OYxOHvLhZ8
oqmNZwQMvm7PWj40zdP/Hcjvu0V5qyb7+hW0m6t8B35Q9SNTLGx4vLg0ZMG7
TuMfswtRq/rY9Uzg/un6B2+6sQahzLX2sKLiC6483jyxced7tsletGKspm9W
tU8q70ku7nFWr8fyd6TRyetZT+84lfclC5eun9J+pj3Hd2uPcJbVdKOc0y30
Bg01U3lfsub8TRrzDvlQb5fX6p4bHFXem/SvcfL9OEvl/REZV4dEPZ8RSHF5
5L2wi6PK+5Opr+dOKgn2U753W3vgGb+NXjwerqepE969jy/Hdc/vcMw5GUbn
9xXXN+ytfF+336Pqds9SQ/j+In5eYpt7yKdzyvcqV79TG/vTNZLvnx/ZzadF
0y1hFBEe8uaPs/K9SozvxEytKSe/Kd+vHDGp5jCbXhHUXNtkjstuOb/f9WGI
XcKqb/zdau5PaB47b+eWGkHD7jnqTlrupvK+5Xf7cbcfjlfG99vd1Wvn8FyJ
x2bZ5JbVknH5tKyzlmWvF+EcH/K41tJWFk0r/+5Zu9Qsh6T055Fmh4O3G4Yo
37s8Sob5M4co4/0RL3Ubpw9S7o9b7Lhx79re4ZxfQ4b9mVbHVkFh/pfP9VgU
pfLeZeClzo86Do/h8cwInRNdm3RWvneZt8Gx9ZRdyvcup1y2DO3nKadZr3Jn
bt7grvL+peZ+meamZTE83rh2eryvm7GMGtYMq5Y18j5fL3AimUUrM1N+nqT6
3m/99LX9Vd6/XOWedbHgl5y+pWxafrt1FM/vfiU6aENWGU2fPqXX24cy6ubc
b5/emDJKd4++n3dGyY8/vfVk21+F8/jKm8CNrx3fe/L4SmLhwKXXU5057rWj
q3ov5kHWZHD00J0UlfcwX/dufWRQE2e6Wzoh5L3dZ5XzgyktSl/uGVRun47Z
d9XgYwS/zwryOaVXtF7eTl+OW5i6s2k/Qski68iRbj9jhPctY1no4aWTW6fK
2MXq1qE2h0upuF7TtVfnRDGbLI/0Z5mlNNXg4C82Svn+ZcyiayULtzvx9YEF
KydVDSYHVmvR3Coa2ikq71tGzjDSvTNKTrWmbnYL9Jfz8z/n46s0mjc4lp28
0b1NyY4o+hK7ZfmrjnFML6zt+3T1KOqb5HP04ZYEduPm20lv28nIcofnsfXP
41kjw20Xv0+U0cJueiYhK0xocxP1d1X9TWlrx1MTvFs8ZjmHz/yRf7cmzenm
4+5tf86O9Bg2MnOc8j3MwaP91obevUHdhyRodTX35PH2hmeuTzvR0oaZVbH6
cTnwFI8H3NRfr3U2zZs13rh74xMdL7LTcS7beN2Jjz+TF0aN68da0gnyzZ+7
I5uff3F/H9HmbM9IGufwcOWe+YFUzf/R0K3No+htp4Shr3cr37vMC9bsdd/Z
i8cDE9xbaC7Me8LuGm16bbwmiepeHTJv9Epf8jy+8sPjK8n0s8pgm2rtfOmC
93n9M2eSqNHle3XdHgfStBcepeqDkylVu7rr3nPK+xzmpg5/prbXmxm1bluo
pca4fi651GRQSBsbtm3RyMHXDprx+N4I+0VhRU+U72EWzzyWfaum8j1M0wcX
Vl5Ok7PF/SYVFT9J5O9dJtWNN5uwzZRYux4bDpMLSzZIr+fy6hPz73DLrnZT
OXuf26fb26mfWKtzhpMiqijfyxwe0i/2jqGcbcnbUHp+ih9bcFiRnFnjBeuT
GvxlUWoOqV8t3vb0iQ0NbUDudc1Syav56Mzpt3yoZICfS32dNKIp6S3jN/uQ
5ytZ+vQfadRv7vW7Pv18yKSwUc3ZHmn0zN1jm+Fo5fuZyxQ/PNr+sqb5cX9y
opnq+5chVm1OvP8mp201t59ZODKU3Y7z7XH/iZydNXXdZfn6JT/fVjNA3Unv
kSfpzEovG7xZxjpPPGW4c5QFdVozamKGVhRrojFN07+xHb/PDeNFfWTHWist
ItnKsL+vHgZnUDOrU+nnl8SwJbkr/a9apdLY1Y9CjibI+PqN89S7mw5dimHD
wztcMP7jzTawrgWrfSNZjCxqnY1xKDtUWNVrRUw4t5980zfl/5wewb70D7Nc
UGrJ71801VLLP8cy6OiPYa2zn0Qy66W5hSlGqTT71tLs9c3D2fSoyJPVysfb
ehuvs/cbBPP1vJPtx6ZtY0FseqPbTf1kPuzcB1ft+/JANuvv8YgdtcNYYLZv
v9Iufnx8fDuluBtb24tZxb8YU9s6kY+3QYPHXJ7W3pRqRDjEjqgWxCKLPr4Z
d/Uls8+MdGryypedvG5ycfrOVEq6cbr91Rs+7O3ybVldeiTRvPpuszPnuLOV
D0ZdKLitfH8T9M2+/0a/v7o7i7Zcuu+gvS+L/6hXvVobN+a/KUqu3j+Mdery
7rXrNAdWveUNo8/7o9g3Oj4+Z68VW3D61C9jVwXT8vAyvji03L+YMXFFi9MK
vr8D4yH21kmzuKPPePvWDZoc/aa3FTs7N336CJcE/j7CoEsbl5u1+cAajb8u
Mx5mx3rv0L5df6wb+7LvioVZgjn3B6K3hCWn+plx+3n3yZ/vHeU3WOTTSwtK
Pkdwe27gh125pUOjWILzZ/N2uob058Kk+9uWKJiP+9Cpj+u8ou8uPya1MJDz
9QrQ+3rajPY7NMy4vfWxSYL1DbfL1HVevVlpp+O5vHsUf/Hmg61WbHudzHcx
614y76Nji4NGZFPUmm6LA/JNqaxLusz9VSj7umxbTL3n1vSnSZv1h1wj2QrP
Fo+CMxxo5tm2Zvf95MxFFhNh3deNTA45+9z4Hi3sJ4lme/Ztmh61Svl+6PDM
ggM/wj9SjtGiK0nRjNW23ni0erALtw8KJ3brbJztyuVrYreJ1Q9NcSPHyXld
s9N8WNTu6YHpFu60euGAv8+mhbBjZlXn9zPxot4OXW5VqxHJDmXH3HVlvqS1
at3ntjXlbMSLV4eiigM4PYYO1u09nQI5PXlDryqOF/hwe8Uq2FMr9Z0PzR46
//gy0wvcXo663j/qVh8vft4I9I40eDagcb0Qbp9YhA0o0b/oTSFfW8o2vHGj
GTdjjgY+9aMGWb3qrY+0puq3TDYnzQnk7fPSD52+SEf5HunVrC7dkqZ5kcbq
/g8uJudy+3ng4l8n2w+5Tys6BdybViuEWrXLqPOm8RN2vMPsBVvWh9G1yMK2
7YNs2fhlBWd/R4TTO7WeQ9dM92B7fkwoqT+Dn1tlRTP3dWnrozx/9OvTe996
QWE09uDV47N1LLh9CPo21vgpG31Aeb/G36mJPUpzQ2nIsUY3jOdEkfQ8UzRt
qalf3OK3cn/JghrtnGoZhtK+gOiXM+unq7xXFWiWcLlmWiSxLWHnDcv82YE3
IfrHZFHUrFZnM6v8MNb19zRfvS0yclw01WeqbCS3n/UmvM9UTOL9Si26vFr8
tLCMdmlrPCE15fum9RoXuTW9Hs3tlf4rVmzKuhLB/dki12FT2u6O4nhdYMaL
mafD+fqqe9e/15p/UL7v1zHVs/eVONxPW0TFSU9OHayhfL+vybWv9ZaOjuD1
tdGd+fuaeijHYSURp37m+XH/vUb+YPcUmzCO/z7SGq5uGsjr8z2l+bt6uZwF
ztf1OvF8UwCvb9HNwi+FpSEcX4y6trZomz/f7xJ+Z1/S2phg5f6XxPPtWun7
8fxx6hrr+md4cjzsxzn9dDdnnn+wv8PCY789OG5b4++2v92ceH7DnrFpf50s
Of7kMDCsf82HPP8ui5zv3ZdZcByyY2e//avv8PtAtIur9bhsYcLfKz2z5ejN
ya+vc/zEjIVNqmLO7e2+R9O/jbhkxHGLugVlUSYf+HnUVX0eaL1zZnx/wPe3
/n1Kmrvx8hSzzT4/cnPgONKg57ZVgz14eQ/Oq38bfMSR46GHey9tesWDl9fv
jFtcbyt/jouHvmO2R0K4PedlS4uazvLjuMNdz5HBq4N5fd5hQSdbHvPl+Ofc
JqVnnZTvxe6e9Sd14rcwXv9v82rPF0f7clyQrbb1VKtgbt9fCJ//YVn3CI5X
DQvqcWhhFMe9N7c+XnpZxu3Z3ImD9jfwC+f4ytkRthodojhe8WN3Yc8jMk6P
3v1I9uGa8j3aBTr6lsEdZLz8P+1Chvf2Ua6fDRo//uk1mYK3f1Hq2A4fPij4
90d/vXjaxkrB7/M5X3WF68cJclZyTa+t2uAy6l2l/wutTOX7sznOr3PsopTv
zd77bnF+dbwD97fyBnwd38jkI+vfsv+yiCpRlBnsX9c46SNbkdy3yvoWGfTk
5qHaW/o6UJ1uliG3r0az6vuTzkZvieby/1fhsYGjQqMpsYfFT+vwBJX3Zr+4
rjVPmyajE6ybptHMVGr6pnTj7DdOVOenp2/8r1SSLV65z76hEx1+c3PqK70y
6mLzLKZpuxjqe/r11Ber75Oa42HzalkO9PXoouA1g27TaYcPnwtPv+D3KZ6z
+nbo1zZjeqnePjCh7TX25FmrkVYtD1KH5jvzBs1+xzoe7OqyzOQOTQz4W7Xo
iiOrbjwtMXbUJna94cgNA8s+8vgR7r+b9aXO+VCNIzQ+xPHkvZ5GKu/NzoxY
Mz9Y/Ry3LwuOpfX+OsKWHbDqnJD8KZ1efuphFN/Elg08uNHvsrs3DTfz7zpE
U3lf49AVi7fPeedLzyzO9iw6G0+n3IbOX/rEl5LaNV/ySiuWIpq2+ZPm5Es5
Xsvf3dT7RHpRQ0f52/jyeEP0t11jAntYs9ee2To16n2mNVvLTI3/WLF6W5+q
HXv3md543B65OMJK+T6Dd8J1rQwvyn+SUm+hZhbJO82OvdPDm8LndVxv+CWT
6sV3Kds23Jt0wqYZ3ClOJsNFd31vq/vS/BSTqwuvpVNyl9vhE7e6k3xIRO8F
88rHt/z885f2btTBINbx2sYscuzz6LBZuX2R03DbsF0TUqmnVudih0+23L4M
Gdfna5v5dnx/xQ6H65rf3irfm73Qwtzh09Vyf8C6hsbbcb4Uf+7N6wvaMWz8
juOK4rWhpKG79u4iLwXrTrqXjmyLJt8JnR17DVWwUp86q971jaZAJ8tn30cq
2M/G9fvO2+ZHhoc259boJOP6aIq/0cXYFXL2fVxW17ozP9EA2x5vhjrI2RXf
tvrbl6ZS6XeDxg1yyv2Fv+0P57f4RE+HbdL4GxDFTBv/TQldmUBtw7WGfzSM
4vqj+GK96W4GCvbEds2JVpa5tLNn5HwTXwXLqvd9XOdqn6lZB7WVwc8VbMez
ZzNymhZQlzZ+HVdNV7AhV+LzGwXl06HEZukNFiqYxbMDdrbvc2mCyaz6/YuU
+u/OtrLGdgFy9uLAuxOGpoW0xDx03vNfcrZo4oB7S4uLKedJd3vLy3JmcWvM
lKGWhfQlqXXz6QOjWdqds51XpxTSU6+FQ216RLOWu2ZVS/Twp+Fb2+y6WDOS
dT+dpR9Vbvdsn/7TfHlIBFM7vnSndh0ZLUl77xs8OIJt2msyzGxsIGVOqmIU
5BHKJvgeP/4kJpYsSjRXpZX7R0fC9MPHnk2ldQN+nNVpGc7Sax5yjzLLpQFL
D8w4rRnJbJ78KOxUPh6nRbWZ8upbBLu1rKrutbYFdGV+5qa65hHs3JTWdetZ
59P9OQ/97R0j2Kg+Vm0Pl38f8kk+/cnYMLbZ+MXULIdCWlll/+1amyLYYtN7
J6wjiinlU4TiTsMIVmpj+dvMo5CSGz79xnRCmXodw1abggqpg//ac1WHh7IW
013WulX5Srvr/95wXDeE/bmwtHil01fS0tKmrdVCWLU+PXWyugSRR3eL3aOP
+LNVN4acaaYmpx7VrOsPGufHriYnunR9lEoxczqyY0uDWLWrJf2czTJo6pXA
5btcAlmbWWargmOz6fkjivm/ss47nuovfvwRkhElZaWSmYZUZL7aKMpoiEKS
hhKRlK0kW/bMJpW955Fr73Fd99p775mQr9/j+/C+v++nP8/jfc657/d5v+85
r/08xVeN5h9eHSR+HwaphBMTU27VaPicknbiz3HI24d/uICvQu5sz70qmCdh
lFPjDmVAFTpZ+4knV2AG6nCstLlsVciilgqawqahUfn3E8vTVYj1U11nx69Z
uLc3g+Hx90r0gX/lh331LHz9ROOYEFeJih48polPmQfDmQycqGAlcj47HOqd
NQtmh7leLnCUIFUG+d039izCYhLtPCeuAjWfsj6BC1+G14HHM07MlaN1DSPa
niOLYHEg9vmfazh0tbCIss5jEYRua9XznCDzdu+F+4UlHU9HqStj/L6Nw5Cn
upJcbpWObM8L37DjHwHfWJOFN3rpSLXBXHh2U/5YvfZZcXg8DbmHPWjjKhsF
fQpDc6OJNPSZ10IlJ2Hz/egUfrywtwClFk1KlXfOAk8S5TKRsgA9/XL1sbjH
PJTjIi8/N8hHYtwOvzOjZmH63EeboPkUdOSWd3+82CKcd/3baZWQh1T7NJ+9
ebEMX3HtbUnGuSgqrNOw99oiBE/suSFcl4zO7irfqfxwsz+xSUC/JBkti2VW
FQfOQs0XetFH14KRp37Euafzs+DQUqGtfzIYLQocOyeQPA/Wjq78u/iDUNKs
8t5rz+Zhb6GF2PDrICSS+fZxnNssPBMVf/OR8guAW0aWvfwiMEjxMCbtDEBI
IaLYWWcR2uL5DR70+KOSPyVUpM3z1+MgR06qsg+qUpZffnNiEZiETVkVurzg
sJMtmFvNwvpRCupPJokgr/a7Z5B1Drad+qkdfyoRWCfPnb3UMQ/mE49jxaUS
4W097vmA9DwIXtrOxrU7ETb+9HN5Pp2FPbOej37szQU9ne0VQ5vv29by1eW6
+kTYt+vh9XbPReBJ5ZOSFkyCq59ad4ZJLkOWxvrj6ZvJkBCIXu1dWQKpzsLE
vBPJsJCq/MZvcQGSPubaUzDmAnevGS/d+jhQeRrGNnZVgOLR0yNKSpvrRXXp
jdmjYmi0sX6IV9yUt/S/PlSXKYZnu5u737MsAMcMQ3OjbDGU/A5jG6Oeh8Do
PBmms8XQISYskHBsFvimSN+f/iiH57epqRUJCxBDn6mdq1oMvLLbGMIqFiHg
dlmKqnkxfKPh35f+ehmO6vqLiLcUw+VPhVwXs5dgVvEOQ+y3YtiH5C60pS9A
uAMeMQWUY/oS82NvhYnoeqx92VhVqKe/EdRw1Yu75vFg+Ixb1TugHqi7Urzy
Hdrhr8j6mQLuJnjv++LcrcQBmBNYuuW23AhTA8Vfo33HYbag+lXBaC1QmjJL
X/achLVutmYvm1rY7zB7cUf+DKgN7K68NFMDAnsetMR2zoDwgATx645GWGU8
KiWIm4JnxkVVFDdroYk+nF5yxyy08m2bm+ivgSrz7/QXvedgPDDGJim/BoyN
HKZZxmZg1MT0UwTNpp5Fzb7f/dYCTGz7pXOtvgZyQjjixQwWYFeQg7rn5nXJ
oqsa07/m4MPrADvpzfG79J3sgX0WjNlYuO94N8PBctY3D17PAfeLh8ZCHs0Q
qzLKfeTLAsjenCk4RKgBjerSfVd2LEGpHCuFB3MtLHZdPDL8dgG4ue9LXN68
/1flzm/mkpYhj5XLeM64Ftp8n7tdM16C3U+/D13ir4Vtbk/sAkJJwCarsj/+
NB5ucVrna610QezpA33XN/VYJnj5IambBIuFM7+/3CfAcWa3HlPmfhDQPvCo
XQkPKVLNrT23+mFmhzajqzcB8q+u0Dc594BXZ/92JXU8RLHTHdx/vx2MVu/Q
+YUQgcH6Q19WSSdIbARp7Msgwmvb319PUg+A4W76B8/l8RCaOJdU3DQEyqat
DKd48TCiv0o/WdUPe/c+X2r3JICJTn55yPcxcGdpT1LY1K9lf+VR1h7bbA8E
43I370/AKc5RSmsY+scK/3QfwEOEhtVDHcsBqE85H8zSTgRz63snm48PgSCd
nlp6GxFKGyPuQ9E4cMtJ0jvlEUGS+rVBEtck3LX7kyydSQS6Hqnjsgqz8JDr
zFI2NwGsqq8phrEvQBrDBz/tMwR4LOOuH/VuFnKGpzLfxhPhc3eQ86v9c1Dl
mFazI5EI0l5jKnLHv8B/ec7CbWnRkqIxaMBd8qifWy/cYzUeuqQSg8nzl530
E89nFAON2hKnw50G8JPmZ/iEyPmW9nc6UrN8cPA+rS2IJ6sLfpkm8C7ZFGLx
xaMe87zG3TkQUnohfCSCzHfWLl0qz/VpA5tH2+l2vUSA76RWUy5og5R3b95V
PUFA4AooW4/sgE736nkh5iysHlF7/xxJxzkTQjVNKZnU+/5T76sPXtE4N46e
zIfGsGdHrxcuw4hoxyVzERL0MpuyWtQtg4QJTqFSlAQiLYcyi77owsUTsYz5
zb6QX2ck2zXQD8L8OUUCPzNQWmxmXvngAMRHlZaY3M/A9P9r2jK91anpWP1H
twB95j3tGSBWfvSxItfmvJeTo0h5OIjGBRP6LnYDe/tjhgUVHHAK35ZOahkB
Hl3vKVrfTKzeoz3dt792OlnwVjmot+jZODTdMdNwMSXzp1+li06+PZcN5/uu
Za5CGeYf2vLPRY/TnClRb0VBnsKZqwOdqNmo5uHPETKPOLaUgd3QAI/1f0HJ
YFn7Eo/s5ZV5PfnJvGaM9zm3cLOpkIBarobkLgi2wEyG8X7fS2Re83uX4Y7x
RTxa/opzd/DqAXdnVQkzLTyW/7eHykxvdL0Z019UqfV3rcUTUCzj3tf+0gP/
4acMggBSymavaEbUIUyaY4XToBzDGqvv1IL+TBQK6bLPwIjYpN6cVQta8DhQ
kDtH5jPHf8UFnbg2D9rCu/eaj+CxfD7ioYUWl7TN+7FpNzmiV/EPX9kj9VC+
vG0j4lWxOcB9mIS8v2iKOp6vQa6/Bfa+TelEsfRS4aTAClSybrCG8kn/8J1t
1c01pb6S59t1XTzQ53AtEtht2VRC+gW0oYNLRVoN6GCzTE2qdgKWr7m1vviL
jFGcfg1Y/IMuGlzYTluLZrsm6lNs8GB/EveEPqwOW+/7yfZeElR12Hqp0V55
PzXQgMDAibsjbHN/Sfp0ajquAYl9/nyDs3sUhjU9/ogbNqDLFhp7Ar9Ng6/u
hF1NSz1qcmmJ0T4yA9+UFlP48+qRGlwPLfaexfgzdPZd0gTdeeC8knLnIKrD
1jNIwj+S5mUdukbv8Hvv1yX49jPAKCCjFt1SW9G8b1SF5YdurceatHjQ9gvl
KJ2L9yE8IWHXt9bPm6vp1tyeCuz5e5nyfdU9yzB9ji3hvkfWQBnaF3nfjt16
Hs56XJuPzSDzlr9WLJWh9zh0Su/qJ8rSJRjubjlq2liM2etZcncW5CkkIZYg
kyRu3mbM/k8yn3ln1U38Dz+5BWkzbxs5cycN4WRTfIVbqv/hM/eO3iTKHchC
lV1m/FJVxv/wmNU/7Wpu+JCLxeMY69o79BFS0b3CdMnr6cVYf1EbgvL3VyXo
Bjsv3/3wQPQl3bSoyXIe7FeUXAxsU5BjQJdgbeg8zF/o2Sg1TUEJdcq53Alk
PvOs+34KhZklaFpXoB2cSMKel9QVF/v6eBC8ShKVm/FqwvwHf1qrHrgwETH7
wtbzFhKOscoUOWD2ZlzLYIhbUjpkH/lLmOLFYfEl8THiIkK35kFlaFCDcd4T
FEjrLe0l8+C0WiWbe+0LGKzxdlnuXwZvS8q/jhf9wEb7knncoWVQNzE7Fnfb
D3ZciGvK1SLzmG+9e1uQW0/4D/+5BT2bPh/jKb65vxm/ULtlUfUPr1nVNbxM
xSoDXripbrefbcfm8w/kBgvGKKxddHftV+BYDpqPKeVzK8wAI79X94y7UxFp
yKguOjQbm8+vyM6v/C/C/K+XTSYNqilywerzgcWI7l/YfFqRLhu34xHmX1jG
xQ/N8szDjl0866dxOSCQMUBhuzQPz6/Ia+gTcqAlVMb+IWkJzOXeiXiV5EJD
yt9B7jvLMFhWeSd2Wx58rrZzbMLX/cNvLm7cGHXxLYMdbMY3SISyf3jO1GdU
lN35SmH61M1S+yPtqH/fKnKWrQMqkZ9H/NS7UU8BzwR7XQMo+piGmi+QsPk9
nbq3u/8i851zcbM0b++WYfV6q+kIcatuyVi8zdb+ZK0tPXD4aAnW/hRSeqVQ
oAKSbzbrapf7/6eem20+s8g35gLJWsiMefuC75s/Nn9WmyHTgn8K5k/woL8q
oiE7il3nb1X6vY99GkxdLIt4BSuggE8xM8VjBqQdQbW8thw8IzgU/Qfn4EGG
wMgfq3Jo0P3VNyu6Kd9fvV9v6VwO/YHx2wJ8l0BA6cE4y1g5Fu8jhSgocp42
wleLAe702WgsvudZGJv9zZkiLH936/vfxiVgJf6kES4/fZlyrtsD/suH9tLz
VNVtbYCpLs7+L7sasfiirfcTIGffkuGKB14jLtcD3JX/8KInhovlVTVaQGBi
OF/BnsyPfnDIXEb7Vi+ULMtcfHC4CQaIO4fxV/rgx9FE0/L9TaDK63LYYXIY
btI0ILXsRvDMv8s7vX8Miign+0w+NWL5zwStzDhN32asPrcso8K4MUcjTP99
cPd+3xzMjz1S5PFsxvKjaT32DUdENGP201ev3u48sJ/Mi371eUOUblsr5k86
fHh7qr8tEa5H+lUf3OiBLwOTQra1ZF40n+TLQO8OIuzukhbiZxsBYTNFa+kG
Mi86Qt+TrreYCKsM+JdIdxouXhljV/5B5kXvZ1+suvZtU741sOcoEZuHMzWx
Q0Y5ZF604XwRu1w9xn1DQTo1NCOkFnT39Wltuc5WNK3TO2evTkBpq0+PmZAI
iA83MX+Mn8yHLtnwO5Irn4qOL8V0vEmeAsa0HQypHqloqN+WpyVnBj7k5NGw
caVi/b83REmxO/rAnf5UlsyMGejJ21daJ+wFvG7GDn8eE5CaBM9vk6JW0GF+
f8D+KAFZLqr65NeQ+dC+safFYQcRzaTJJ6y6N2HxCNbXONso8K3og7UC4+ox
EtINq3i5p42AfFmpcRx/icgq4IjP07YOpFLgJxeqROZFJ/ql/HZ3IfOi42vH
6yhNWtGqRa1gPn4B9gncWi7SbkWCUv27Xi4twLLv7sNX77Wiv8oXSt+FLcK9
AHwlwzkyP3qQxfYRjp3Mj65m1GVodsKjG8M/zJQGF6HcQO/TqiEeBQu2Zokk
LUKop+rf7e/w2HmtL1fMN+mdjJKFxxJfvA2H+f5xD/+7CeiNPIebJ20F1Ijj
p6vkc5GUkcbA95VWiD+7/Wu0eDaK+XtbMewJAXR7BVvkzbNR1JVWL8r7/XB5
zkEot6gIFT+9su12bS8s+vWscO34hZ1v3P3BhVpSZH50kQncS5QgwKHjnS8C
ixfguy+rH+d5Augphp+8RLsI654v2jivEsCN4eP9icBFQCyd0Qlq5Hiwc78u
81jqkf0HcsfXClUGiHBKOGK1uH4RzjG0WTyeJYKakoGMD1rc1BfM7gVPEyEy
iusm6QoJyZTYP5B/TUIbbUXGa2tL0DN40b2flcxv+6JE6XbXmYT8eJs0KLIi
4eFfka/ntpN50OPj4nyFnekgrZi01PMujpyfdPYC/UUaHywfdcv/wGyIf36S
PhVIafVLOnFT8GanjUqKXAiaarr0+ux4OWTGR3E1sheDSPs6w8W7lSArzTDQ
faMYjJbvPlAcagCf3KVRvbM4sHO52EVwKQeJI2vdjV2loHfS/oBmYDVQVvpY
tT0nx1sqjDmeqHtdhbXfeDvWJprWQe73Dkev0w1w3D3vlmFHFbZfJGuYNXi8
jYHhm/fG7JgrENX6n0b2Qz9ArilSJg5KUOSum6cmRJNhqrnsr+oHhPFcOx4y
DZ4bDUH/t57uIHiL8v8Ne16EGKZ8rO/W9MNTnubBnwlFyMhUx5VreQAIF53q
b74vQiLW1ur3vYdAzUXOO+9sEfJZNTGTrxwEs5dXmVzvFmH2Rv3Qx2ZSCQj1
XHfedqFkGNjTy294URUh1bC+jgyREXg9N34CP4jQRu3GrveSyWBTKPBBDLbq
51WAwB3hIxR7CuBMUa0Pe2vhP7zoT0dGZFmbqmGwf/vRMK5ukL7WTteTVw3b
u86IFF3ohJr0FtCvJudvRic8/VyXg4N3fCE0RzNG4XM0DU73AwKn+SIZCtlx
yNiX43pRoRDzXygdPXTJ5yiCnmY7jtvq5PrVHDMfw/X2t8DfNRENpgSE+fve
Not/P8pTiNmrSDfvtXb5lwOhXSKlxKgJnp870M+nUA4FJrU77+vjQTi3fCLQ
thyc7pQMPEpvhegARo2FtHKIOKPkWPmGAPncU8Ynv5VDy+/6VdUkEoTL3lDC
1VXDzzGaU2E/20HY6qY3I57Mh5ao+fyZp5BcT8G/kpnTWx0HY/whXJN2U1At
Ml6n+DQIqHa8cfj5aPo/9YqnwZtVVZAh0x9br9y4pP0l/oXo89SKobvhOCTS
7FwOGyhEIim8vJrfJsEsePCV/PlClP+0/+/NtClIal35+HmFnH/N1VDiOFAa
gkTldGZe3h+Dz9/LnF/ol2D2CYaI2yFZ0vXwV/XHSMnPIfjmf6NKTrEKCL83
3sfeH4CDp0dd+vqrMPvCo2GBqiizYqxd/NF1YopUBsyZJscU3/eDzpmqWecI
Mt/Zz0+LvUkKh/lfBV0L5d7ZlMLIcZnWzqB/ec/ZBzkaDj76hR767P369wbG
9cT0QUfui1eZmLPRVKbqldKAFqjy3DPRloGg/P7wiYH+XvA7wdDTXZuPyQP6
vg4uL6jzoCPI5Ms9rSm4pcC1Z3glD9iqN5hJfjPAkXZPmfQpF+RnGvRmXgxD
o4jjST2OKvDmfxwYv74M9fGVMYkFBKQhi69yzibznP93/+xEIyjFtUqvFU3p
H2JrYepC3Gbpsf2OrUhb1N6fIsQZObemvw57Hg8v9vQmBz34gQpy3FbxpGRM
vlLr0FY4vj0B8vzEhNyWdTC+lQ2Ll9sV/gYkC4zPXFo/oqMJKrdcp2qRo221
XsdOE4wv5qITeKa07QsKeSx3+AJ1A6I94mHRezgCRthYmo9n1qOom4jvhGME
6F07J3bw0gRYJvmzz6mlAXvB4rBl5Di2//7v/UyAbt3MQ8MvZD50RilfeYdF
I/K4vM8kp2kZynY6ztnda0S/K0cqedWXwYtKvc/XqBFRMS7M6woRoVowaudA
Vz1mX5v2ZNfVbCC37djCl20PNEFsKs6wV6sfQv66FaX51kNn5VXXWzLNaLh2
Spt6vAWTd6bPuSYmvW3B6se/J0b8GA7alD+0TrjzD3ehfRYUmn8H8Nj3EbEQ
psQy3or8Bnq+Hutq+YcXnWKTQjrVT8DqId21+1gQeYSINhKp6oxSybzorfM3
XriDrfNCE4qd+fFhrbcJ/V/eMxHWBV8n2q83YvMft79cKrS/Edlphux3lp79
hwdd8Z7n0LeFRmz+wlCOve5Lm+fG4/YduoLN//ChayhMrmpyViDPgmsUYvhc
7PqSpt+Lhbqqf3jRdMwybc9pK7DfmzK5/e7b3Ja94TdEKxnjRhrI+TEuWqS/
aCUX2V3WrKmUwuJAUbuqamp+wQzQXWLpaXhUgM3nbP0qfJqezINOOBt7QHtv
LiKMM0UxWHxDfXYdb89MkuttK2gX0KODX9GrhuC9B48nY/y8cUvYZiHSD1cn
Tga/exaNDpZRf7lA3wdjfypvXGeMQVeerNX3ZM1A/VjXcc+DwdjvsWU2hIpa
eWH3/+PW4BszzRToNjRJbWZowvR30wfU7RI/ZmAxJczogEoSPH/pJdVn14D+
y2tGV/IkayfqoW+HVhOJLw39l9fMxsQ+R72jEZO/znObXyDSNMLzLy9G/azq
EPH0SD6FKjn+zIie+qnPpry19T1co1LMzvEiYPqA/LkQq7t3CNj8yp+O7ZU8
RMDOq+Z1+6gKaMHsQRrc+GNmYnhMPk930Az1TGvC+Fl6v9s5ojrI9RM4CTt5
XGzrsPM/ZtCLx1W1Ght/Umb26nveKmw8VY3XbHVqHba+Q2cttx07V4vV4+np
Ytwwcq7F+l+LHXx0KxWH9VeK9NwQeFGM9RdQXnjCmFWM2fuYujlyLhj/wMZT
JsnsOasShI330Q+3ZE5IJNf/Wd54NVmQhPX3mOTA93CS6xNGfXqbaX8pACq6
LJilVJb/4THv+1B3xwWSsffh2jbx/lsUAiNdV94C3VbY31F55mUugk7udg3x
eCKIMs4+VYxD4KjjqmaxPvoPnzm0pt+a7WwxNr/UmSdik4XF0FQskVMxMwKl
k07bShIrsfMGZZ2wIHLWYuMNGHjRB1SDjVeXoZY4qFiL7YdmzgEavXfwWLyW
vKGtRxQ1Hhu/SzT0vqIvmb98Oy4gqaSFiPU3evr80dNNfXTr9wWaVt0PfSdi
4+MiBeccC8k8Rg9VubBVylZE+vokrd5iCWqul9HqMbQiXnnimQn1dFB3antK
ivuBRgzEmBhVM4Emo8AxaLO9yPrgLL9dPMav4XG8c+3uXD1wvDCX+ulO5ilz
5n/IDh3KRK7sAd3Fj7tgRP7+w/B3W/UYeyD9+zOqr4cy4IC6dV2VZQ8IzfVr
+piT+clD87ZCQz0EmEgT7Iila0PrFB4hLynI/JWl25JtyJgA17sGdg+aLkF6
oQrPIwMCaCX+0NW8tgRT1E/57TaIYHNDjjfYowPslWkedR+KQ3NiPSrmE50Y
b2fLn0J1+rz7x5+x6MBShGRRSzcA+4kwCclYdMfuWCPN+S74DVxtPt/J/IJn
21ESy+s8SDC6cqjTow+oHIgx77zJPGS0m/XZe5ckcN92xS9xfBAGXjYepeBI
xt6/fozr2k4OEuz95qJoypiO1avc4vvIcJmovBXKgtGTOkQ++k29yLp+KOdq
IjzcK1WmbJGM1XvA8iPXcxkK3/uBq1KD1RPZtH94xzvwtBq4TXlnixcV6UbL
QvcmDSJvZUQXOBWB+rldR+6NpwJH1KXvdUkIKtl13aVCybxjW0Oh9nDnAjjY
HMXwUrgXGggxmri7BVg846NSGp72AhwYFVzRthggwJgcnem7fDLPOCriHaWB
Kw7O5yeYlSw3Q7/U9vE6HzK/OOFY/4cRp2CICJ7nqH4zBMxJ7A3lmaVweMfa
uM7qECxv3OaQcSbziZUPrJTfpMBBil/Iz1GhXlivk+XLnSwGW7eRnmifXvjM
kHhouLYYex+HHtkuUwr9Akkj8fKfMv2QMC0hYT2cCoMblR/VxvrBRiA0sUY8
FQ443fVoPDoA3TIBeJ6FFJB4dVd2vWEJ3CxsHKqdGlFVRRutA3U/uJ5xPFLN
1AS3v0i7Vr/shdoXC6YdxHqI9HYU8XzWA/R776i1b/4vtuIDmcfWlMKFSBCQ
QfuNKkAX26+qDGtS+tY+wmlY0XOfDsX0zeqO2KNvFwsw/arx5Pa79qwI5B6P
fmPVbP+HPxwVOfF140cuKAxFkHZkDMCK3+mj9Ua54NzdUsUkPgh3u/0Pm//J
gXrOGDdGwhBkq0m6yp9JxOTPvlPPr/4Q8oVwOj2B64KfsO9H/MWDfG2rGnh2
w0CELzsT4w3LesnaJ+tkwavPuIX9HOUgfulZldjdDLgk42PnfXhTj9IaUTa6
lwHnkyrWKHVqIHSOxv9sWiZUCp8SiSioBIYpro9LezKBL+dCZwvDL6BePOub
L0fOn11DXk7K+39g+rBDVkOIR24yxi87S5GJn/uZsfldd7KpXxwDw8kfgSzi
GVATQ+GVtz4CsYcelsW9zISY8NAvzR9HQfMqIenmJJk/zF16JaWG8P/4GrMJ
ok390DXOvKjHXAzhIZbrnisDEHiuqWEg6hfYCz+nYiMMABfbuY9eKb/A92JK
ELV/O9I7L3YCncej52ZTsWJiJJTeO/DkRAWZR1y4q543CYdHgVBvRrTs/IdH
vI3/7medmFqU+6PDIlUiEXYIOp5cqsnH8n/fCn/qfTaWj/lTbi/oFZbyhaBb
QhL+Qj7lmPxEq9jnseFahNr9A64sTKaCzRzF8TMPszD/wVa85VisjLywRxa8
H/yx47RKN2a/3rqfTobX3XLfazF5+7tHUpRbOhGp8bg+y1Mi/MMXXoj1OFUu
0YqSW3IOKLS2/8MXVh3QUjuW24oy0zTjK793oeueJaOT0a2I6pC7ysX+Bdi5
zcq+4SseCYpcMJzcbF9IkA5L2WxXRdvskwrsha4jFNlMlunA+dD8gxJjB7I2
O2oiFUiA3VqzXE0dC6BrWhZTTiSfZxFHskzl6cl84Y+c0YkN5mnQ/Py6mHZQ
DmZP2tpfd+w6yhtenkS2FwNNwwxlMLbefWwj+3Y2J0K8/Ha6GrZyTB/DO8a1
0vJFoZiRpxdvNHohSSenp9z1o/DpQLadzyOEqBMy8OoFo1Ayl657+hmZH1tZ
4l6rlhuFfvt4Reut94N2NjF+ciUK6Qh54kQoB2Cp58PbS8tRSDJNEse1NADp
Kp+5LV6T+cOub7+VnyyJRL2no5eXhoZg5Vn3+TtPItG4e2rE95hBkOGsWRFl
iUKmLDF9KRPZIDj880+bUDaYpfTY9CdnQ9GDY+6y7NnY81L5TcTJyCVj+WbS
F7/xn9xZiNkbsttwu67cK4BR0UpVnsExiDw/wt3hXoj56y83nsezNkUgJwnq
lTiFYXBg2jdmIBqJxgQ6CphbRyDoxfnZbQ8jMH4kj0IvG+uBbIjIs//916kJ
9KluTUmvZIH27yWnqNImyJawZdCkzIbjxR+8ScZ4WH5v9Sj8/+MBz4yIrYzZ
I+CQE2u5a9QDpE4H76ukAljrVPOgNO2B9w/ppHIIZN5DqrSgUVp1Nbx3emdw
L3FTP3vpM51QiIPPAxc4nqm2QlmfnsqezfaCdnAurV4TlEkxFKUb4OBymX9i
E20zWFo6P/tkvXm+mIuV0GpMgeNhQsFhixTMPtMrorZS3J2MrWcWZexa8Y8C
7H1zD5htD+6qwuwrp6OzPuNiqzB9wPzR5aK+PdVwl+YJ96EKMj84Surn98OP
+uGldk/n2XPFkIF7xXXySQ88KF/bAB4cvKW5stc4exCKjQwPjcyXwraU67GH
DMn8YGEt1cXV9SW4rP41UHCOgIJLSyjo65fgtL9KS9gqAeMH3j3xyNyBKR+t
ScwER97OBVPNaPvswDwUD2oTJd6Z8CXk9dUnc3moJjzw7faZTXlK+cip2+8J
4Mf8Lm6EuARUlcw3X70lwPIDt3yNi3mQj28KfeWcgvGGO+PhlI5CNMZH3COg
01OnngpJ5n58UWUT4CLRFp9CSgX8QFLhY4opqCIdj48qToHbqYHjwzGTYL3j
IN/QHjJfOPdKTKF2VSnG9+XPvfXrM30BcrK5/VeOehZyKa9DmEIBil0tnf7d
Nwo7n0zuzDeJw/rztZ2ut5QLRgysNtReTLMgazeCTgsGYzzgUuEWE5FLifB7
kW7smcAsfFHHvXMtSMT4Zaq5lB8rniNsf3x4Wt3rwXwTZj9/N3yVTjqtGZOH
rhtuO9Po8w3j65qJOWbeEmkl83CLpAyuiGShbH8rb9lvvSDkKc0Vq5WJ9Y/6
UKl1Wp3M371kpcdlMEMEB41xIAgtgbzYin3UKhGzhxj2JTnZ0JEQu3JeH56J
iHpMin7JniNh+SuNIyarh/rD0U+Wx8rawtPw8mNlkY2tP/b/X55jKbUaDMP4
lU1d9xxt+0PRWIWNa0dWH9iEKKR9JpRh8kzFwQtUDvqNZN6rX3iJr+R3tLU/
SBfRHT4e8wOTd6k3KGmND1Zg+p3yTnuiVFQu1r4ssva6zdEHa3/ZW/HUVJGs
z2W73IsQ6c8l63eNj9vRjgpQ0rpu7bujCQ3XCTdnsxAxniuRqdxMndSCvQ/q
b/ZxPnzZQDrLGW4wPga8hc5uGzwx0CBmefbemXG4UDo8LWkfjckXlGtDSmpv
EIQWx7JGmfT8w09+8GivjksouS2lMbj0KTkG0V8Wd2di74HSca/PtIdj0QC9
39TV/B7wxIlL0ZXFIGvbOc0TfgOgqMsS7+6QAreit88zUA3+h789AMdDhED7
dArG15zbgy5nnyJBHt83qfa1ZbDcnirvKkECLXqNPzcbluG8GVup9/k8MNCT
npvxXIaSo9vM6nkr4PV5FUZal2XAP3m+rHiEzL+spYju5tQior1OBsT1rjbU
8WWXkAZhy7+0BO8p2L0kqUgYnzLbVL6en5bMp4zYl53IvRqE8SmjbVzqP8Qk
QfCrw4+JjmMQI74y/6mtEExOGna+JzaCOMH43tETCONPBk8cPzKjSeZPdtZm
lIpaF6BdWT5EzuJeYIic52Te+Qtt8RO5roizsE2XQm0awxJdTyv6vR4g6TiJ
h1YWk6oO0S7wpemTOZK4xX/tgH03Gp8rJP9C2irHrcrkJ6DxbPlri/2R2Pc+
Jvlq7UV2FPzZ7z5sbU7mNW7932uthHYPJ+HgfwD3NYpA
    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{2248, 2630, 2520, 2626, 2627, 2266, 2703, 3224, 3223, 3359, 
          2368, 2661, 2660, 1996, 3454, 3453, 3668, 2278, 3550, 2279, 2665, 
          2664, 2863, 2862}]},
        "0.451`"],
       Annotation[#, 0.451, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{1940, 2495, 2609, 2610, 2663, 2662, 2229, 3679, 2613, 2861, 
          3358, 2277, 3649, 3357, 3246, 2792, 2245, 3205, 2109, 2631, 2632, 
          2411, 1975, 2628, 2523, 2635}]},
        "0.41000000000000003`"],
       Annotation[#, 0.41000000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{2230, 3168, 2231, 3185, 2095, 2611, 2612, 2398, 3623, 1972, 
          2607, 2499, 3191, 3400, 3192, 2791, 3195, 3194, 2108, 2629, 2247, 
          2410, 2246, 2524, 2112, 2636, 2251, 2413, 1976, 2633, 2528, 2637}]},
        "0.369`"],
       Annotation[#, 0.369, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVzz8vg3EUxfFrKqregz9d0dgMDDaRNpE0sYiks7TYjBa0DzabN8BgUaL+
bBYWQyUiWtRWNt6AT4eT7zn33Ofm94yUKkvlvohYpiQVcTYQ0cxEPNPqUMRJ
f0QJJ3BqMKKOK3bO8R5zuKVvYEvfpiN3XnTH2JVf+Vn9DD+HC/iIa+YP2MF5
zJsXqCw3fLvjbjPtLTRKu3LFu651e/ybWYvGqCqv6250Nb5t9k7jlMgbulvd
Pt8x+6IsHcibujvdIf/Bd72lzl/2/ns44gIn8cnOtr7If9v55H/wSp/w0+4t
yqf8L5+394cFLJr/Azp4OHA=
          "]]},
        "0.328`"],
       Annotation[#, 0.328, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{2687, 2640, 2641, 2280, 3606, 3607, 3455, 3456, 1998, 3251, 
          1997, 3629, 3250, 2864, 3249, 3248, 3247, 3040, 2123, 2849, 2122, 
          3039, 3404, 2532, 2121, 2624, 2120, 2531, 3403, 3033, 2119, 2847, 
          2118, 3031, 3402, 3025, 2117, 2844, 2116, 3023, 3401, 2527, 2114, 
          2620, 2113, 2525, 2794, 1949, 2522, 2784, 2783, 2401, 2235, 2504, 
          2503, 3186, 2232, 2779, 3188, 3187, 2096, 2603, 2604, 2497, 2496, 
          1939, 2494, 1930, 2765, 2764, 2216, 3169, 2084, 2479, 2766}]},
        "0.28700000000000003`"],
       Annotation[#, 0.28700000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{1953, 2795, 1947, 2790, 2409, 2244, 2518, 3624, 2517, 3203, 
          1974, 3438, 3202, 2848, 3015, 3014, 3200, 1973, 2406, 2241, 2516, 
          2622, 2621, 2530, 1952, 3032, 3437, 3034, 2846, 3030, 3029, 1951, 
          3024, 3436, 3026, 2843, 3022, 3021, 1950, 2526, 2787, 2786, 2404, 
          2237, 2509, 2508, 2614, 2615, 2400, 2399, 2617, 2099, 3189, 3190, 
          2778, 3396, 3397, 2498, 2770, 2769, 2394, 2219, 2483, 2482, 3163, 
          2214, 2763, 2215, 3164, 2083, 2588, 2589, 2476, 2475, 2597}]},
        "0.246`"],
       Annotation[#, 0.246, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{2243, 2408, 2407, 2625, 2107, 3204, 2242, 3018, 3399, 3012, 
          2106, 2839, 2105, 3010, 3398, 3006, 2240, 3201, 2405, 2623, 2104, 
          2515, 2239, 3199, 3198, 2845, 2103, 3002, 2238, 3495, 3197, 3355, 
          3196, 2842, 2102, 3001, 2236, 2403, 2999, 2402, 2619, 2100, 2616, 
          2234, 2507, 2782, 2502, 2098, 3178, 2222, 2777, 3176, 3175, 2489, 
          2598, 2599, 2393, 2392, 2602, 2086, 3165, 3166, 2762, 3395, 1929, 
          2477, 2756, 2755, 2754, 2203, 3151, 2080, 2473, 2757}]},
        "0.20500000000000002`"],
       Annotation[#, 0.20500000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVzcsqxHEYxvHHynkjySCThZK94hawEGUhK7uhuBNLh1mwdCgLm0khJo38
kUGZslTcgQvwsfj2fd73+R3G1rYWN9uSLKDanjx1Jxu9yTOXUOAB+7o7Xtc1
eAB13GJPd80V3Q334wqX2NW9dSZ98jt/diSv/NuTNPmnK/nGiLsz5iE+xglW
7aecf7H/kkt203LNWxdYsWvpH+3uvVfwjv8avM0t/QfOnKnoJ3nCfMo1Z5ry
OY+bl/WHXNgd8JJ5WJ7lqrkuz8tzKJsHzaN85H75P+MPXTgwoA==
          "]]},
        "0.164`"],
       Annotation[#, 0.164, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVzz0vQ2EchvF/B6EvJCISk0YM0uAjePsAJG10waSNSNAaWFiNVvEJiITE
UengLbHQ1UCUL8DuZfYzXLme+z73eU7O0HK9VEtFxAIaXRHFnogSprCZixjI
Rozrm3zBdV0N/fKYnHDCG7p19Mmj8imf85putTui979PR3Q4t/U/+ryc8Z1B
TvOvLsdZXHZGLOknsZuJ2MGr995s3tHGJz6Qsm/YL9pOYNt2Cy/2Lc+/+YG/
uMJn/Oz5inMVBXlYHuECN93xxPvuTJz3uCU/omw/j0PbWZ7DvX7af83gxP5G
vsYVDuyOdUe4k2/xB1hKMeg=
          "]]},
        "0.123`"],
       Annotation[#, 0.123, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVz7krB2Acx/Gvyf1zbBRRv8HCyGDh96McCzYpg8IiFgN/gmPAjMTqyDHI
MTFgchVRBudmkSIGL8O79+d7PM/TU9k30jWcERFDmMuM2M6NGMyP2OFCbGET
s2brPGC2wQmsYRUzZgfZEYe4zorY5/u8iD2+zYm4wYe6TN1ivxVt+NX7waI7
F1CeiKiz08sFKERSPeX8NOrlSZ7AqPzt7Ju3a+19ye3urNAfNx9DDx7MU/qN
aEIaj3ov9l/xiGc8odv+nVk11+BY7whX8q57++Uq/0uql3lHb4kr1e/cYT7P
XXzpPxfolJtwLjfzGV7kV5zKn/ZPuFhdhAa5EbvuTHMKpXYK1CW84s3S/4w/
zN0+7Q==
          "]]},
        "0.082`"],
       Annotation[#, 0.082, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVzTkvBGAQh/Fxn7vb0dpVUasVQocIcaskQuL6ABKNKNE6IqyjUbgKWnd8
ARUqIo5OHKHyUzx5Zub9z7zpwcn2iZyImMJ2cURbMqKvPKKeezlVFpEx3+Et
7jHrRkKf1m/zJucnImpLInK5hvO4u1QeRfpPfSF/8L39Ly7W9/pnSH1QFNGk
vvNW7mYrWvBi/ib3zBtyzWbD/rtUX+EcI/oLXpBJuJGW/3Ynw13+70TW+zrW
UG2elPuV+XVvwP4z9/OQ+c//v95y1XlYtLOEB/c73HrkFf0yCrzn41X+DO9u
PNk/VZ9gTvbYf7N8bTdHfWOelcvo62SreFU/ymNolClze8bOtH6aG8y+ZCb0
47g1q+BKpGSTOFIfY0t2n/ewi3l7G2ZZHOoP8Ad93kK4
          "]]},
        "0.041`"],
       Annotation[#, 0.041, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVz01LlGEUh/FjmU6jhjrTwqyFoNFCyBklLDfSBygJF1EgOLlWxO+QLhLU
hYKLIBVfAntbmpW2FXQh6C51odKL+NIiISl/s7i47v85537u89Tkeh51F0TE
OAaKI2ZLIjrLIuY4iRlMo19vkjv0pjiBCdReiXjFdfzczH39x86FlyN+O1/i
YjnNRZzi63wD19AkV/MdTpm/ijQa5cFkRJYHuI+X1J8lIr7wVywjJ//N78xn
/NrcP16xSz3/xA98MPse75DN/w9n+BhHKJDbvbPj3pjzKMJe/3mnNOK7mTZv
vJHf4kC+qf+Lc3JXfjeUyLfsEM7r5rfdrczvJFfwU3e21J7wN943/9DcvP4D
3lC7YK5VfUSt104hb6pf5GO1EwziBTbt+1FvEQv4jE849K1S/TIkcdvdKm/u
8h7+6Dd7Y0hvGD3eaZHv4S5O9Vd8J2O2XL+BW+RV5zX0mx82U8kJZPSymHdu
4kak9YvkFL90v8K5HOdKOE9l
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0r1LlWEYgPE7kPSUH42CNPg31DmhJYFFJeqQRmSLorY0pDmaQ1CDOZyG
VBTUAm3SIfCcsgJdNVpKgxoaKkGsoyU4lB/Q7x0urvvrvZ/nPe+p7uxpuXMk
IqYwfSxiBtlURBE/KImYFD/kZ/wU6bKI05jTz5TKMahfw7XIqefRXxyxyGfN
1qIGo+Y2eI/fm/3HdccjzuEM8no57IpnuVl/jn/I08jgs7237f3Cj/WySR/f
UVke0ac3Ih7GE6TcuQ93sSB/jb/2vOU3uOmMl9zGr3jZ3h2z++KD5K4olx9y
GZ9AvXMqeNXsJv9Eq+e37W3h39xvfkW/oPeOB+T3sK7/Nbkzd7nroPgbCvLr
9k6Lt8zP8HNctatRvRlNWEt+V/VPPMSPcM2z89zjrF50J9/D7C0umFtyzlF7
qtXuy1/gj7kL8ovoRhe21Tr5PGc9cyN5F/GE3b98ry2c0i/wB70R/sijPMYT
zqjiYr6kdhnjyX9HfpJTfEWtzo4G7rC3Hf8B1GpfGg==
          "]]},
        RowBox[{"-", "0.041`"}]],
       Annotation[#, -0.041, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0LsvnmEYgPFbRevw6WaqBSHBJqQGh8GCgTCQEIf4A2oQZROHj1FtDsFI
Iixd23SwSxhsDoNT8KEWJKV+33Dluk/v/TzvUzQ03PktIyLGkcyOKEhEVKIC
H+VZuRFtORHtyBR/wIL4Vb+Ut+Rl/CafM1/LP/in+jx/lR/zBp/wJo/lR+x+
ihjl7+jBL2yr/eY+c/04kQ+md36OGOB9eRd341C8x+V6O+KUO1ziCh3qU/ZM
Y1B/hmeRxIHZFrVWNOM8L+IM9Xp1mNC/4Vtc49WuFP/jB/7Pd/zG93xk/pnz
nPvECX7hXB7xBqf6Vc6pkTfhC6rT56qP6V9wjbxQ/dHb/MFfu8/seEj/q16D
eNJciXjRXYt5iTOxgl79ZZ4ys8pryEKh7+vMrosbzbwDRRFGcA==
          "]]},
        RowBox[{"-", "0.082`"}]],
       Annotation[#, -0.082, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVzr0ug2EYgOGHSD60LP1qsJmoto6AmrSjxsygg5GYkJBIjFqStmG30cFJ
+BkQDsDCyIxJuDrcud/n/51qbK1sDkREA9tJxOFIxMNYxM5wRJqN2OWWXBNl
+RIuRiMKajNI1PfFB6jrubQj8e56ZziLkvcRP5l9RFmcG4/II8W6WtueHE/3
d9j7Ji5k3Nd/Y2dD7ZaLcrPo6bnCH1K1PN7NzJkv2nmnd8/sPVfFNZRQN7uM
D/kv/kYFi3iV++EmWjjGKU7wqfbLG24scRU1TLrX8Ydn+Ta/cJc73FGf4EGe
l1tAV1zhM7lzDPVvi9f89dqNVf4HIsAxHA==
          "]]},
        RowBox[{"-", "0.123`"}]],
       Annotation[#, -0.123, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0D0vQ2EUwPHTyW253dpeE3apDYlvwKRiY+ADMFOdJBaDalTCZhEaPoqX
4gN0ZPGySUr43eGf/7nn5TnPc6c2txtbhYhYxU4SURiLqKGK75GI32LEIpbw
gyH2kanv6r8oRTS5LXeEmTSijkv5A/lpfQmnvpfVGyiJi+iKTzFQX+GefWX5
M/FGOeKcD9Wu5K6xLtfjW2feoKn/nh9whz4eMTTzzC94wpv7vGJP/0f+Nue+
5+/kT67wF49zYkfGazxrpmVmUtwejZjgY/5zh4odHfGcnhOu5f+OM17Q1xXP
q/0DrJMt9Q==
          "]]},
        RowBox[{"-", "0.164`"}]],
       Annotation[#, -0.164, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{1900, 3360, 3529, 2155, 3530, 3074, 3527, 2153, 3528, 2154, 
          2431, 2430, 3367, 3366, 2714, 3091, 3090, 2041, 2560, 2380, 2381, 
          2559, 2558, 2438, 3100, 2173, 2725, 2174, 2451, 2450, 1910, 3376, 
          2937, 2057, 2805, 2058, 2938, 3377, 2455, 2059, 2561, 2060, 3117, 
          2186, 2727, 2185, 2459, 2061, 2672, 2388, 2389, 2188, 2460, 2062, 
          2678, 2390, 2391, 2189, 2950, 2064, 2814, 3118, 3415, 1963, 3119, 
          2953, 2954, 2816, 3120, 3417, 1964, 3416}]},
        RowBox[{"-", "0.20500000000000002`"}]],
       Annotation[#, -0.20500000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0D1OQmEQheExFoC/wU4NJmxBO3YB2mphawKFiUoE5eJF7JRO42/FMliB
vWJYhZoYJCQ+Fm/OzDkz893c4n6tUp2JiB0kcxFtlHIRw8WID4yyEZe8nDqD
LB4XIt7oO1JZB3vYxSAT8SS/QQ/XeHXjy81PHGGWV+Xdma/RNi/Bg/4eqfrX
nTGd0DXz61hFw/zBfMSU/222wCv7jr5+Q921c4WWuWf5Cw5lt7xjemq3gR/+
Ga3YbdIT2TltYfz/H+i2LKF12QVNMZEtu91Rb8q7NK+f8ldoacn7vC3ZH86L
MLw=
          "]]},
        RowBox[{"-", "0.246`"}]],
       Annotation[#, -0.246, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{2285, 3257, 3255, 2000, 3633, 3227, 3632, 3228, 3542, 3541, 
          2267, 3667, 3226, 3648, 3225, 2704, 3072, 3071, 2030, 2550, 2373, 
          2374, 2157, 2549, 2425, 2426, 2161, 2707, 2708, 2709, 2437, 1906, 
          2920, 2921, 2797, 2044, 2922, 3368, 2924, 2046, 2799, 2047, 2925, 
          3369, 2718, 2175, 3103, 2048, 2806, 3104, 3105, 2177, 2930, 2049, 
          2808, 3106, 3107, 2178, 3108, 2933, 2934, 2810, 3109, 3412, 1959, 
          3110, 2935, 2936, 2812, 3111, 3414, 1960, 3413}]},
        RowBox[{"-", "0.28700000000000003`"}]],
       Annotation[#, -0.28700000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{2156, 2548, 2421, 1901, 2705, 2423, 2031, 2552, 2032, 3076, 
          3077, 2706, 2160, 2903, 2033, 2796, 3078, 3405, 1954, 3079, 2907, 
          2908, 2798, 3080, 3406, 1955, 3081, 2429, 3083, 3084, 2717, 2165, 
          2442, 2441, 1908, 2926, 2927, 2801, 2929, 3407, 2928, 1909, 3370, 
          2931, 2050, 2803, 2051, 2932, 3371, 2443, 2052, 2642, 2053, 3494, 
          3350, 3352, 3639, 3351, 2369}]},
        RowBox[{"-", "0.328`"}]],
       Annotation[#, -0.328, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{2551, 2424, 1902, 2899, 2900, 2898, 2902, 2901, 3362, 3361, 
          2905, 2034, 2904, 2035, 2906, 3363, 2711, 2162, 3082, 2036, 2554, 
          2375, 2376, 2164, 2914, 2037, 2800, 3085, 3086, 2166, 3087, 2918, 
          2919, 2802, 3088, 3408, 1956, 3089, 3547, 3548, 3236, 3238, 3237, 
          2253, 2655, 2656, 2364, 2377, 1957, 2688}]},
        RowBox[{"-", "0.369`"}]],
       Annotation[#, -0.369, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{1903, 2712, 2710, 2163, 2428, 2427, 1904, 2910, 2911, 2909, 
          2913, 2912, 3365, 3364, 2916, 2038, 2915, 2039, 2917, 3231, 3232, 
          3233, 2854, 3234, 3626, 1983, 3235, 1984, 3444, 3443, 3602, 3601, 
          2269, 2855, 2856, 1986, 3334, 1985}]},
        RowBox[{"-", "0.41000000000000003`"}]],
       Annotation[#, -0.41000000000000003`, "Tooltip"]& ], {}, {}}}], 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdk30wG3YYx5lT61Adxm3OMK509dbSpdrVd5ptZah5K/Uy0izGKMtWi3dV
VFALahKltqxa6qWaeK3L/NRwipY2Y7KezVuuWpWE9Mpym0629Z89d889z5/P
53Pfx+pkUgDjJQ0NjcNbrZ7/lgIvNhUtTxG7IEecebDvzuTnvUwz/VmHUjkM
pp67jXEpxEYmeMrXlSPd+PrRubQgsr3Igi4Lk8H3ARHer/uUNJ7197p0ZgWD
nL0sB6sk0npkxK8x8wkiSTQlIfw0KTCR3F0PXMZ0p12bx65Ucqln7UStzmM0
t0xHfe2XRSoklhfO1yxhqOgNn58UOcSe8fp8qOlDmDjnhLOUeeTOIyFvOkOK
Ff1e56q32WQgI1Xz2b0FVAr1mM3FxWT5+LiX3GweEc0Nt1z1OGShP6fUPnoW
y8V5nTXeZWSHUqo9fXQGtASrYY/tF4iiaDZ7j0SCxLj9W2gVxIJ0PRrPmoKp
aGmPp0slKZQPNJtSxVg9I9wdksIlB60/Ss91moBdd6h/bhyP3AspjbG/OAIp
jT/lKucRup3tr8GegwjIClgRr/IIoySN/SGrD1X6+24Pn+CR1d0aQy6x3Vg0
2FabbsAlV2kbbJ6hAIbroaoJ7W9JtiikxWfjCjT3dyS0OJUTo2pOPHWQh3Nh
YtGCcQlZzK290ziej/c+N7JdP5RHclWVbpEmdHT2esaO3jxNVmMX2B0bT92d
Ei9rirSPoPXlza5y83hoHBzjmkZnw/47aeoXrHPwYQZLWB+UvuCC6v1am0iX
KpgUuJCLWnX4R58FH94WPd/3Mq7jK+8mKybnCn4xXPMYj+mA5f2kKJ2717C4
/s5fEo4ITEsbXV5fC7Sox7vSjvWDflLJPNVwA6ydhwKXREPo2xX/M9VViDUd
rfEfC0aRXcfWa0ppA4X5JaH6T8CTZJof+KwdiT5+ofOaYoS4jypnZe1w+SOm
2kw8Ca98Qb6lTgeSFPm3g25Nw7zCWfystR2r9LFI7+EHKI/YpKpW2sDOV4v6
DZEf/76vflSInr2HaYU6cxAMaI1cjRCAEnReSeHPQ2/4E/lmSitozAPKJrdF
DAgct4w1o61DLVIKo+5kil9yA9THO/o/xLvrbzq6yerA72Uc4w4uoSGgTScs
hw/2q/2XS6wfQ53OoIIa5M7cOOUfswxfh/CtyFVCUT/kTSl8gg21JuMy1Pxg
ffPPghWUVc9ty8hiYzh9ciQvSobkbzQlyzPZKHuFruuxQw7thmuM+Ewm5riq
1MAyOerfem3I5GzEf3xySG0n3c05bvj///4NQNjiEA==
     "]]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  ImageSize->504.,
  PlotRange->{{-0.5, 1.5}, {-0.4, 0.4}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 224},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->789492561]
}, Open  ]],

Cell["\<\
Note in the streamline plot that the flow appears to leave the trailing edge \
smoothly (Kutta condition).  There are no streamlines inside the airfoil \
indicating that the computed velocity there is zero, as it should be.\
\>", "ExampleText",
 CellID->14395635],

Cell["\<\
The function ForceAndMoment computes airfoil lift and pitching-moment \
coefficients.\
\>", "ExampleText",
 CellID->80454523],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl", ",", "cm"}], "}"}], "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Vortex\>\"", ",", "a", ",", "b", ",", "rD", ",", "\[Alpha]", ",", 
    "rC"}], "]"}]}]], "Input",
 CellLabel->"In[8]:=",
 CellID->499718542],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.989320272998931`", ",", 
   RowBox[{"-", "0.30314844461786716`"}]}], "}"}]], "Output",
 ImageSize->{142, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->97872131]
}, Open  ]],

Cell["Locate the center of pressure.", "ExampleText",
 CellID->8410041],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"xCenP", "=", 
  RowBox[{
   RowBox[{"-", "cm"}], "/", "cl"}]}]], "Input",
 CellLabel->"In[9]:=",
 CellID->342608962],

Cell[BoxData["0.30642093656782343`"], "Output",
 ImageSize->{60, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->119607110]
}, Open  ]],

Cell["\<\
Surface pressure distribution is computed using Bernouilli's equation.\
\>", "ExampleText",
 CellID->753558053],

Cell[BoxData[
 RowBox[{
  RowBox[{"cp", "=", 
   RowBox[{"1", "-", 
    RowBox[{"VMagC", "^", "2"}]}]}], ";"}]], "Input",
 CellLabel->"In[10]:=",
 CellID->435840926],

Cell["\<\
The function PlotSurfacePressureCoefficient provides the conventional plot of \
surface pressure coefficient versus the chordwise locations of collocation \
points on the airfoil.  An inset showing the shape of the airfoil and the \
location of the center of pressure is included.\
\>", "ExampleText",
 CellID->139054009],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"xC", "=", 
   RowBox[{
    RowBox[{"Components", "[", "rC", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"PlotSurfacePressureCoefficient", "[", 
  RowBox[{"xC", ",", "cp", ",", 
   RowBox[{"Epilog", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Inset", "[", 
      RowBox[{
       RowBox[{"PlotAirfoil", "[", 
        RowBox[{"rD", ",", 
         RowBox[{"Epilog", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"EdgeForm", "[", "Black", "]"}], ",", "Yellow", ",", 
            RowBox[{"Disk", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"xCenP", ",", "0"}], "}"}], ",", ".012"}], "]"}]}], 
           "}"}]}], ",", 
         RowBox[{"Frame", "\[Rule]", "False"}], ",", 
         RowBox[{"PlotRange", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"0", ",", "1"}], "}"}], ",", "Automatic"}], "}"}]}]}], 
        "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", "1"}], "]"}], "}"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "1.1"}], ",", "2.2"}], "}"}]}]}], "]"}]}], "Input",
 CellLabel->"In[11]:=",
 CellID->671472236],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3W+oHWV+B/DbbaUL3Wre1e6r+KLbvGqlfVH3QrspqaBrwU2LrBBCUwuH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    "], {{0, 324}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 324},
  PlotRange->{{0, 504}, {0, 324}}]], "Output",
 ImageSize->{508, 324},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->156290816]
}, Open  ]],

Cell[TextData[{
 "If a list of angles of attack is specified, nested lists of ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[LeftBracketingBar]", 
    SubscriptBox["V", "c"], "\[RightBracketingBar]"}], TraditionalForm]], 
  "InlineMath"],
 " are returned."
}], "ExampleText",
 CellID->571494656],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Alpha]List", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "5."}], ",", "0.", ",", "5."}], "}"}], "Degree"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"aList", ",", "bList", ",", "rC", ",", "VMagCList"}], "}"}], "=", 
   RowBox[{"PMVortexLinearNeumann", "[", 
    RowBox[{"rD", ",", "\[Alpha]List"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Dimensions", "/@", 
  RowBox[{"{", 
   RowBox[{"aList", ",", "bList", ",", "VMagCList"}], "}"}]}]}], "Input",
 CellLabel->"In[12]:=",
 CellID->42630738],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"3", ",", "60"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "60"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "60"}], "}"}]}], "}"}]], "Output",
 ImageSize->{178, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->65565644]
}, Open  ]],

Cell["\<\
These results can be input to the function ForceAndMoment to obtain \
aerodynamic properties of the airfoil assuming the angles of attack are in \
the linear range of the lift curve.  Listed below are values for lift-curve \
slope, angle of attack at zero lift, location of aerodynamic center and \
pitching-moment coefficient at zero lift (couple).\
\>", "ExampleText",
 CellID->24035690],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl\[Alpha]", ",", "\[Alpha]0", ",", "xac", ",", "cm0"}], "}"}], 
  "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Vortex\>\"", ",", "aList", ",", "bList", ",", "rD", ",", 
    "\[Alpha]List", ",", "rC"}], "]"}]}]], "Input",
 CellLabel->"In[13]:=",
 CellID->34907031],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"6.784515153124075`", ",", 
   RowBox[{"-", "0.04128080764467379`"}], ",", "0.25938923204817554`", ",", 
   RowBox[{"-", "0.046593438540551174`"}]}], "}"}]], "Output",
 ImageSize->{298, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[13]=",
 CellID->474611700]
}, Open  ]],

Cell["\<\
Note that the location of the aerodynamic center displayed above is close to \
the thin-airfoil theory result of the quarter-chord point.  Display the ratio \
of the lift curve slope to the thin airfoil result of 2\[Pi], and the angle \
of attack at zero lift expressed in degrees.\
\>", "ExampleText",
 CellID->4111782],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"cl\[Alpha]", "/", 
    RowBox[{"(", 
     RowBox[{"2", "\[Pi]"}], ")"}]}], ",", 
   RowBox[{"\[Alpha]0", "/", "Degree"}]}], "}"}]], "Input",
 CellLabel->"In[14]:=",
 CellID->274981366],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1.0797891231015637`", ",", 
   RowBox[{"-", "2.3652160529311925`"}]}], "}"}]], "Output",
 ImageSize->{135, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[14]=",
 CellID->96230897]
}, Open  ]],

Cell["\<\
Consistency between these two approaches of computing aerodynamic properties \
can be checked by computing the lift and moment coefficients at 6\[Degree] \
using the linear properties. \
\>", "ExampleText",
 CellID->328772922],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"cl\[Alpha]", 
    RowBox[{"{", 
     RowBox[{"\[Alpha]", "-", "\[Alpha]0"}], "}"}]}], ",", 
   RowBox[{"cm0", "-", 
    RowBox[{"xac", " ", "cl\[Alpha]", 
     RowBox[{"{", 
      RowBox[{"\[Alpha]", "-", "\[Alpha]0"}], "}"}]}]}]}], " ", 
  "}"}]], "Input",
 CellLabel->"In[15]:=",
 CellID->762946865],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "0.9905430304392637`", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"-", "0.3035296345168643`"}], "}"}]}], "}"}]], "Output",
 ImageSize->{170, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[15]=",
 CellID->393770335]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Scope",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->1293636265],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Alternitive for discretization",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection",
 CellID->421192202],

Cell[TextData[{
 "Fitting a B-spline through the points in a data file provides a means for \
alternate discretizations.  In the current version of ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ", this is accomplished by the function FitAirfoil."
}], "ExampleText",
 CellID->5610347],

Cell[BoxData[
 RowBox[{
  RowBox[{"spline", "=", 
   RowBox[{"FitAirfoil", "[", "rD", "]"}]}], ";"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->559405563],

Cell["\<\
Graph the data and spline fit to check that the spline provides an adequate \
representation of the airfoil.\
\>", "ExampleText",
 CellID->299806311],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"grD", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gSpline", "=", 
   RowBox[{"ParametricPlot", "[", 
    RowBox[{
     RowBox[{"spline", "[", "p", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"p", ",", "0", ",", "1"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"grD", ",", "gSpline"}], "]"}]}], "Input",
 CellLabel->"In[2]:=",
 CellID->249332617],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3f+PFPUdx/Fr+pf037hEueMOFAk9D4vYWmrhZAtt1Wo0MagRalsIQrrx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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->87644956]
}, Open  ]],

Cell["Rediscretize the airfoil using the spline.", "ExampleText",
 CellID->176584194],

Cell[BoxData[{
 RowBox[{
  RowBox[{"n", "=", "60"}], ";", 
  RowBox[{"layout", "=", "ConstantSpacing"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"spline", ",", 
     RowBox[{"NDiscretizeUnitSegment", "[", 
      RowBox[{"n", ",", 
       RowBox[{"Layout", "\[Rule]", "layout"}]}], "]"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->343427282],

Cell["\<\
Graph the endpoints of panels for this user specified discretization and the \
spline.\
\>", "ExampleText",
 CellID->126580234],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"gP", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rP", ",", 
     RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gP", ",", "gSpline"}], "]"}]}], "Input",
 CellLabel->"In[4]:=",
 CellID->274596420],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3f9vFHUex/Fe7i+5f4NEaSmoXA9r9biaYEWo7omgnp7kLkUjmlOMStzA
D9LWinxPadVWvFU8OLFVAkQhITR3Z43YSwz8YEMiITkSk967+24/ne7Mbru7
s7uf98zzkVHrtJ19T3fnNZ/5zGdmfrP56fs3/7qpqelj+ed/v2pqmv16BgAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAb+QyTWJFdjJqdng+AKAq9U5dWXAm
l//vomVPZrO5uW+T9AAQn7qn7mR2Rf4FQy+5UBE5DwCxqX/qzu9Yii06l5n7
9pwmAEBFKkvd6pXes8wfRyzK+XgLAIA0cOFZburGoEhPkRaTn5XLBr5BzgNA
BRbCs8zUjcXiM79zFcyd980LHkOQ8wBQgWB4lpW69UfOA0AFDIWnoVIBwB+G
wtNQqQDgD0PhaahUAPCHofA0VCoA+MNQeBoqFQD8YSg8DZUKAP4wFJ6GSgUA
fxgKT0OlAoA/DIWnoVIBwB+GwtNQqQDgD0PhaahUAPCHofA0VCoA+MNQeBoq
FQD8YSg8DZUKAP4wFJ6GSgUAfxgKT0OlAoA/DIWnoVIBwB+GwtNQqQDgD0Ph
aahUAPCHofA0VCoA+MNQeBoqFQD8YSg8DZUKAP4wFJ6GSgUAfxgKT0OlAoA/
DIWnoVIBwB+GwtNQqQDgD0PhaahUAPCHofA0VCoA+MNQeBoqFQD8YSg8DZUK
AP4wFJ6GSgUAfxgKT0OlAoA/GhueuYy8ftOK7OTi2ZPZFTI7k1s8l5wHgAoE
w7Os1I2BvF5+qblM6DVnX5ScB4AYLIRnmalbvYWFRrwkOQ8A8XDhWW7qVm9+
x0LOA0ANufAsN3WrV0F73om7FgBImnBm1r89T/88ANRBA/vnZwrP/AYPKZrC
p4TJeQCoQPHxNkukbv2R8wBQAUPhaahUAPCHofA0VCoA+MNQeBoqNSXOn/vv
00+d+CT375rOXP7vAohkKDwNlWpdOEWnpm4cO3Kpe9Nw79vnZP7+d7+WqW3t
eyvv6F3V3C8/7KY1rQMys3llX/7rd+TrCib99fACd716JvjSrS19ly7+6KYT
J/7FHgEIMxSehkr1U2TiffXlDwXpvaPnpIvWtrX7NXi7Ngy6mS5sn33mY5mT
eeyDYNi++foXMrPnr5/K17dv/6KvcuTwJZm5+83x4EuXmKm/Hl6gK1JfemPX
UHAX09rc7/YU69oOPLntI7ebkJ2R/uLpU9/pDmug/8KSfxwgGQyFp6FSfRAM
LmmNS0jet+5gsPnd0X5oNrfv7C1I7/Gxqy5ab9687RYYGcteCVY4PX0ruJuQ
nZGu3Us7T7kdlu4R5KCjYHcwOjJR4gABMMdQeBoqtc5cpEv7WdJJmqwSVnev
GcjnWJ+2xuUHNnYdDza/JQlnLKR37ApWWf9owd2BfKvgAEH/V+bPfveNMT0C
Ci6TwwH4zFB4Giq1doJ5MnHl+uJIn2uaSpNVZr6887TMefWVzxtdsmG6R3ht
1xe6Z5R2vvxh1969P3g4oPvQu9bMHQ7oT8pxEMkPfxgKT0OlxqIgKCTVx8eu
6vlH7WzZ8vhIMNIljhpbcEoUHA5on9grf/unHg5oy9+d13AdYtoX9OH7Vwh/
1J+h8DRUavWkQdjRPtud3rKyr3vzsKb6jp6Tev5R4r3RBWIJ7oSynjvWvqDW
VYs6gvSUwcED33RvGib8UTuGwtNQqRWQ1rts7C++8JmEeef6o9IgfLDzmATC
s8/8ffLbnxpdHeKh4f/G62PavaM5f9fqATdMSMJ/+3M5dxbgwP6v3e/SEYSK
GQpPQ6WW5jbYa9d+Pn3qu717zkpbXTfzlua+8bGrMr/RNaJ+gh1BEv7y8Qic
BZj9VHS0H5IPjJ6F0UsG9Bz6DOGP5TEUnoZKLUEa5+va3tORMNJuf2nnqaHj
lyeuXE/huBeUEB4jqmdhHsqf9tUxsd2bh3U4aGv+/K877iP8UcBQeBoq1dEt
7vChi6MjEzt6Trat3S/b5tYnRhkJg+pJsOtw0O7N78vHTM/jyC4gH/59q5r7
5djQXa1G+KdZXOGpDwuv6e2LbeW8tMFkm9Lhds13zjbMZKMLXnYE1IJ88PLh
vy/z2AfStNC7T+SnuZGfp099p+FP8qdHbDk/OZvuk9lM7W5S73nO61Yz0H+h
d995aVlJs2rXq2d0uB29MWgsSXV3IdhLO09p+Ovo3JaVfXo7CMI/weIMz6gn
UcXI55yXbeSe+ctnjhy+xAgZ+O/ggYs6oEtvBxEOf0l7rvlKhvjCM5fJP35q
MpdLfL+N+9jLViBtdR0O8XzPSZrusC4Y/nJAGrzmi/C3K67wdN3zTTV5kOws
f3L+gfsO64AZ+ZyPjky4QW5AIungn+1/zhH+RvkTnktqeKnyMR46frlrw+D9
HYcYMIOUW2b4f/XlDyR/wzU8PJevUaVKEyXz6Afbto52tB/au+fs1NSNhpQB
+C8c/u7G13ovJj3hS/jXGTlfmnwUV+cfitTa0ueemgFgmdx9fvTeqnrCt+Cp
B+NjVyeuXKfPp3bI+UgS6fL51LGRb+0e5wQrEKOCm7zt6Dnpbv0h4a/3+dcr
fAn/WJDzQfKh2rb1I/nUScL37jvPCVagblz4633+3RW+Gv6yPQ4dvyzh/+kn
/yH5y0XOB+mdZ1Y193PhKuADF/7yxd49ZyXhV+Uf8iXJL+0xzvYuEzmvpCXf
uf6o3nmGLhrAW5r8z/ecHB+7Gj7bqx3+sjnT5xNEzk9P39r+XE4m7gYMGFXQ
4S+bc2Sfj2zs6Qz/1Oa8vt0vvvBZ14ZB+TrGJQPwQbjPR+/nvPKOvrTd1Se1
Ob/+gcP6XA+64oH0yIf/Ph3hH7qrT39Sx/mkM+enpm6s//0RuuIBCIl3SYMn
t42WGOdz7drPdsM/hTkvO+stj4/QGw+gmHCfT+f6o9rn02zwlj5py3l5a+RN
oa8GQLm0z+cv2z8J3tKnoM9H74viW/inKuflLZCWPCEPIC56J+c/PXVC+3y6
NgzO3iZlVb97etfEleszjU5+P3M+l4l4AmGVpU5P36K7BkAd7Ml+5Z7epbd0
0Mu75N+jIxOa/DN1DP9geEamawNIHfmb2Bc8n6rKnJe/pxxVVVkaAFRg39vn
NPl3vzmuyd+5/qg+tLe1pS8YTbUI/4XwLJKu9TeZXTH3rJLFpVSc8/J3e+Th
oUe734+jOgCIwbVrP+tDex/aMCjBrskvX+gTSFe39Afvr3XwwDdtv33v9dfO
VPZaLjyLpWv9ze9wInLeKWuBeqJcdpqxlgkAyxXZSi+YOfbF90ODlx95+Ljk
1ZrVA/fc9a58ce/vDnT+4WjznX061LOsFw1nZrF0rb/Y2/M6RGrTxuE4qgOQ
fMuJ5RIzjx251L1puPftc5cu/qj34WlbO3ePxN595+XnddJ+GzdteXzEzZSf
lN+SKfvWl7te+bz93gMyc2PXUGWr42F7vhb983IE1LVhkJE2gHVVJnB45sSV
68FY1um+dQeDYavT3WsG9OIpF9Tu/pkFk2a1/qTeV/PZZz7W/nlpdrpX0X6b
gqs1tV0a7yWcHvbPz9RmvI38VWV3OfntT9WVBqAM1ceybLbBBO5oP6jd13rL
Mp1ca1mfWhJsLbtYXpN/MFx4kuUX/KRMG7uOaywHc/7lnaf14ilXjEx6grUO
WV0NH8fbFFH9EFDZcXdvHpa3dXRk4tQ/Jr26kAGos+oT+KknPwomsEyuYSxB
F05gncIJrFPLyoiGsbv/gPuxBzuPyZz7Ow7rLYh1cq1lfQphsLXsYlmf+xmZ
wL7Fcuz8HD8fKa5SJe3lDc1/qPpaW/oYUQ8PhaO1sliWA9iCBNb/laZOOIGL
dU0UdCMXS+BggPfuOx9OYJ3CCaxTf/+FFCZwfaQw55Vewrxp43Dn+qPSyN+7
56w0A858/j2NfFSg4liemroR7hyWqT3UP6yxLO2TyFydv/tK4SSf7chODEnO
cAIX65pYfjcyseyn1OZ8kLR5ho5f3tFz0j2VRmJfT5fwGHrrqm8YD/RfiOya
kO9G9g/rcy7CXRNta/eHc7hrw2BkJ8bmTUMFIayxvPWJkXACSywHD0tJYBQg
54PcTeok9nX4U/Ax9HreXH5Gt3TZ/IO/y+6ghGq6gkv82OFDFyMTWL4u1j8s
e/DIXF1d5CRdia4JCfPI/mF9bl24a+LmzdskMBqCnF+SC38dByv577b0+TFU
77jNv6W5z237bo+gyVP6phZx9cc2cIHT07fC/cDBMRIFXcHhNnDkGTq9JVR0
At/ZG5nA8nWx/mHZg0d2Dr+1e5yuCSQVOV+Bgi399u1f3BH0lj9+6HIsuEcI
HhqEp8h8i2x2VjOzgt8tVnDk1NF+qNgCdYxEQVdwuA0ceYZObwlFAgMV8yc8
l2SoVGeZcRSZb5HNzmpmVvC7pcsmbAETDIWnoVIBwB+GwtNQqQDgD0PhaahU
APCHofA0VCoA+MNQeBoqFQD8YSg8DZUKAP4wFJ6GSgUAfxgKT0OlAoA/DIWn
oVIBwB+GwtNQqQDgD0PhaahUAPCHofA0VCoA+MNQeBoqFQD8YSg8DZUKAP4w
FJ6GSgUAfxgKT0OlAoA/DIWnoVIBwB+GwtNQqQDgD0PhaahUAPCHofA0VCoA
+MNQeBoqFQD8YSg8DZUKAP4wFJ6GSgUAfxgKT0OlAoA/DIWnoVIBwB+NDc9c
Rl6/aUV2cvHsyewKmZ3JLZ5LzgNABZYMz7KiuDyy6PwCcpnQ4meXH2fOp3Mf
kcK1ZpXTIIWrPFPTACwzisuy8PsRSyfnY5DCtWaV0yCFqzxTywAsN4rLMr8P
IedrJYVrzSqnQQpXeaaWAVhuFJdaUFB+WRW05wEAFYgxistTy04hAMCy1DiK
F5/kDR49NAW/AQCoHaIYAAAAQKMUGf+fQEXWNMkHXqXe3ISe4SmxynpNS/Le
6JKX8CRxhRfEcZlSGpQ6v5AsRdZ0MpvNzX07cX+BUm9uPhuSt30UX2VJhMS9
wXnFVnl+P77QlZxMCW2vxKrkeKFEWXJNk/cHKLHKuUwml8Tto+gqz7dtk7dv
K/4uzwX87HvdmNLqI4mf47iVHP+fKEutaQK3hqKrrOuaxO2j2Cq7xnzyDttK
fbATetC2WBI/x3GjPT//3WzyPipFVnmhaZu8vtti73KCP+el2vPanE/am1yA
nF+G1PfPzyw09nLZhP0JSr+5idw+iq3yfNolcKVLrHJ+fmJPTMxJ4FtaC2kd
b+N6Lxcat8n7uBS7uGNWQrePJa5nSdEquwO3JK7zvMWj5ZJ+zhkAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAgISYu2tiJjv7X24mCABJNBv1kvMJfJQL
AGBO4p9qBABppk9/4BkQAJBIk3P98tpPT6MeAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAICA/wNm6pxh
    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->14906467]
}, Open  ]],

Cell["\<\
Solve the model using this discretization, and compute the airfoil lift and \
pitching-moment coefficients.\
\>", "ExampleText",
 CellID->33075010],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"aP", ",", "bP", ",", "rCP", ",", "VMagCP"}], "}"}], "=", 
   RowBox[{"PMVortexLinearNeumann", "[", 
    RowBox[{"rP", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"clP", ",", "cmP"}], "}"}], "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Vortex\>\"", ",", "aP", ",", "bP", ",", "rP", ",", "\[Alpha]", ",", 
    "rCP"}], "]"}]}]}], "Input",
 CellLabel->"In[5]:=",
 CellID->252090154],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.9855019890583476`", ",", 
   RowBox[{"-", "0.30147237282340256`"}]}], "}"}]], "Output",
 ImageSize->{149, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->192730869]
}, Open  ]],

Cell["\<\
Compute the surface pressure coefficients for the user specified \
discretization and plot in red.  For comparison, also plot in larger blue \
symbols, the results previously computed using data from the file for \
discretization.\
\>", "ExampleText",
 CellID->511124957],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"xCP", "=", 
   RowBox[{
    RowBox[{"Components", "[", "rCP", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cpP", "=", 
   RowBox[{"1", "-", 
    RowBox[{"VMagCP", "^", "2"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gCpP", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xCP", ",", "cpP", ",", 
     RowBox[{"PlotStyle", "\[Rule]", "Red"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gCpD", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xC", ",", "cp", ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Blue", ",", 
        RowBox[{"PointSize", "[", "Medium", "]"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gCpP", ",", "gCpD", ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "1.1"}], ",", "2.2"}], "}"}]}]}], "]"}]}], "Input",
 CellLabel->"In[6]:=",
 CellID->59705953],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], PointBox[CompressedData["
1:eJwVlXk8FVgbx294pzImSyFSlil80CLG3CQPubyjV0aypQgJiavFnnLFy7VE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     "]]}, 
   {RGBColor[0, 0, 1], PointSize[Medium], PointBox[CompressedData["
1:eJwB0QMu/CFib1JlAgAAADwAAAACAAAAO09CeLTx7z/y+G/JXGbYv+4wSWjL
ue8/osd0C0rC0L8IRmDq503vP1halXx7rcy/44rpK0iz7j8oFQpI7l3HvyYv
OdbF7e0/+GHxL4vpw7/QZI7MI//sP9g/ZF3YFsO/0x1SD9Ho6z88ImM6mpvD
v8tVZhTLreo/BJ7d+R1VxL8CNop2FVLpP+iB6qo1EsW/MIyduvLZ5z+AP6gl
eeDFv6uVZr0YSuY/SPDhat+7xr+iohZNZ6fkP7weaDV1rMe/T9qkpfL24j/8
kA3eRrLIvzb9KO0NPuE/OH6NubDOyb/mmHWTGATfPwBdX+FlDsu/dhPkwoGQ
2z+AbckLX3jMv2bqzLT9K9g/UL7ol2kXzr//Az/L8+DUP24xX7qCAdC/CRAI
lE250T9UWM73bS7Rv2Hukq6ZfM0/7JMmcLmp0r99w1sgQfHHP0hV/e6iktS/
/wFQMc7fwj+EIHlcMhjXvxd6dmtgq7w/2p01hwiH2r9mp3QnZr20P3I2tXmQ
R9+/c86K374OrD/sd2Pi7Pjiv9WvZjF3LaE/62FA5p+f57/Ov6lojNaRP8F1
1CttZu2/E2WSGEKVej9eRjJf79Duv1vpPYtOQFM/CLGoYwSqxj9ggh0KhEdb
PzYrSnEG//o/YmHdItU+fT/08PbJIyf9P0ppT7IHWpE/EKwP/Bq9/D+B/jlT
xr+fP3gTKxoUcfs/uGGgr6w0qT/IMLL/mPP5P3j5A0nXTLI/sJiyqJqH+D9r
TXefVPu4P5rF/uGZPPc/e/7s2jNLwD9kijd38RT2P+Q2P4WxhcQ/fIQzPRwQ
9T+L1xcRUyLJP0Sy/duvKfQ/8fXupiIVzj9kjqtJm1HzP6yE3wuTqdE/VgqU
XgxV8j+r59r5fmrUP3YBgldWDfE/H0F3MjhK1z/q//bsEgrvP7HhqhPQRNo/
oOlNp66g6z8q9JUYBFbdP84mK1mb+ec/nMV25bM84D+GUAoVokXkPyp+Zqpg
1OE/oFlTwJPY4D8wPGTzcW3jPxAQ2+Yuvts/2X/9HYoC5T+Y19IO7pvWP0HH
v4dLjuY/wJL1QQsl0j++U5MKYwvoP6jckQlhWcw/yusVeZJ06T84GCRryzrF
P/99u4jvxOo/UDqizguOvT8Sy71cxPfrP2A/VD0dsrE/NzV3ApoI7T9AAGQi
2SaaP0O3/sHA8+0/ANum1QHFmL95fh7r4rbuP4D3dUoooLa/v/awQspP7z8Y
aftaFpPEv6pirCaIuu8/jHDl8B3gzL/8za5p3vHvP+4fwMEWVte/b5j7kw==

     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox[
    "\"\\!\\(\\*SubscriptBox[\\(C\\), \\(p\\)]\\)\"", TraditionalForm]},
  FrameTicks->{Automatic, {{-1, 
      FormBox["\"1\"", TraditionalForm]}, {0, 
      FormBox["\"0\"", TraditionalForm]}, {1, 
      FormBox["\"-1\"", TraditionalForm]}, {2, 
      FormBox["\"-2\"", TraditionalForm]}}, None, None},
  ImageSize->504,
  PlotRange->{-1.1, 2.2},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 324},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->92827430]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Options",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->2061341341],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Epsilon",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection",
 CellID->1295379749],

Cell["\<\
Collocation points are displaced a distance Epsilon from panels.  When \
Epsilon is positive, the displacement is in the direction of outward unit \
normal vectors to panels.  In the following example, the model is mapped to \
three values for the option Epsilon.  The three values place the collocation \
points outside the discretized airfoil by the default magnitude for Epsilon, \
on the panels, and inside the discretized airfoil by the default magnitude of \
Epsilon.\
\>", "ExampleText",
 CellID->120830056],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "rC", ",", "VMagC"}], "}"}], "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"PMVortexLinearNeumann", "[", 
        RowBox[{"rP", ",", "\[Alpha]", ",", 
         RowBox[{"Epsilon", "\[Rule]", "#"}]}], "]"}], "&"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"10.", "^", 
         RowBox[{"-", "10"}]}], ",", "0.", ",", 
        RowBox[{"-", 
         RowBox[{"10.", "^", 
          RowBox[{"-", "10"}]}]}]}], "}"}]}], "]"}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->210128799],

Cell["\<\
Note that there are only slight differences in the results for the strength \
parameters.\
\>", "ExampleText",
 CellID->112154987],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"aPlus", ",", "a0", ",", "aMinus"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"bPlus", ",", "b0", ",", "bMinus"}], "}"}]}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Max", "[", 
    RowBox[{"Abs", "[", 
     RowBox[{"a0", "-", "aPlus"}], "]"}], "]"}], ",", 
   RowBox[{"Max", "[", 
    RowBox[{"Abs", "[", 
     RowBox[{"aMinus", "-", "aPlus"}], "]"}], "]"}], ",", 
   RowBox[{"Max", "[", 
    RowBox[{"Abs", "[", 
     RowBox[{"b0", "-", "bPlus"}], "]"}], "]"}], ",", 
   RowBox[{"Max", "[", 
    RowBox[{"Abs", "[", 
     RowBox[{"bMinus", "-", "bPlus"}], "]"}], "]"}]}], "}"}]}], "Input",
 CellLabel->"In[2]:=",
 CellID->295212498],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "6.167638826326538`*^-7", ",", "0.000012961748730333511`", ",", 
   "0.00003452063276654371`", ",", "0.0007851037697719221`"}], 
  "}"}]], "Output",
 ImageSize->{381, 18},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->622054557]
}, Open  ]],

Cell[TextData[{
 "The tangent-flow boundary condition imposed by this model is that the \
component of velocity normal to the panel be zero at collocation points, and \
this component of velocity is continuous across a vortex panel (see ",
 ButtonBox["ICVortexLinear",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ICVortexLinear#62898067"],
 ").  The small changes in strength parameters are consistent with the \
normal-component velocity being continuous across a vortex panel."
}], "ExampleText",
 CellID->3193481],

Cell["\<\
Next, plot the speed computed at collocation points versus panel number, for \
collocation points specified just outside the airfoil (green), on the airfoil \
(red) and just inside the airfoil (blue).  \
\>", "ExampleText",
 CellID->79242491],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"VMagCPlus", ",", "VMagC0", ",", "VMagCMinus"}], "}"}], "=", 
   "VMagC"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gVMagCPlus", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{"VMagCPlus", ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", "Green"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gVMagC0", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{"VMagC0", ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", "Red"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gVMagCMinus", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{"VMagCMinus", ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", "Blue"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gVMagCPlus", ",", "gVMagC0", ",", "gVMagCMinus", ",", 
   RowBox[{"AspectRatio", "\[Rule]", ".3"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"7", " ", "72"}], "}"}]}]}], "]"}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->23699121],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 0], LineBox[CompressedData["
1:eJw9zA1QzHkcx/H/uu60Fac7um5EZEvFJoqsaD9L2qJU27bbPhTHuMKt3Lkj
5PonU8rcdK1Tjuk8Vq4h5WGGDO2lrpPmWJ0s9jzEbUl6QCWu6e7G93e/mf/8
5vV/z+c3eWWqYvUIjuPS/v3+u9+dHqkm6xOBtNMqpR8QD6XWNQmfk+2x9436
vJZjHgNupbjRrGIeC49ppy6sSGd2hc1pYHllJrMb+LvJv1jTmCfB1nOxouQL
Zg9ox1Qf7lvBLMJeTqbZ9f/7Xvj5vWZXeRSzN7reX1n9xyJmX+Bo48Y6CfN0
eFxwEq2ZweyHD+XJd0I8mf2RE6gQKsczz4Sm/8ZPKc7MszAnefRj/UjmAJiN
v2VMHOokB0Lsv8n/wAvm2ch2skjr25jn4E59686jVuYgjI7wq4g2M8/FNuHm
g0/qmSUIFXyfr61mnoffHR13GiuYgzF2uqvX9iPM81HctDZzZCHzAmQWZIeN
y2UOwebIwp7cbcxSDEj/alUamIGa1M5LaUlkHtB8bScQRrMuw315mLuTlHUZ
RHnHolV+rC/EBMkx4xk31hci9MDlshEOrC/C1bb0r8b2P6O+CDmXS1ddekjm
QrG9StT8+CrroWhyyKqcXcX6YpR+uTAxpJD1xTjxYN/DkjTWw6CWTLnWrmY9
DFceHczfEsC6HI03SnW3HViXw2I3cd0Dawf1cDQYRnwz9TiZD4flenhh4DrW
I+Dp0iBYImI9ArUulv4j5qfUl+DT8NWthQYyvwSe97KK47vaqS9FS5/M7BBJ
5pdig/3BTtmONuqRsMzoPTNhjY16JCb6lotun35CPQpBnV6Ccd6t1KNwcrNL
9zjBXerLMHPXHqd8Ywv1ZbgvLusT176mHo0CaYDm6eM31KNhistdf3YqmYtB
h++L6qGGwXdGDJa+qrvSuJHMx8Cvsul5yQyyKQap8zPm5L1k78eibeiHFrca
MmIhGqXzcSgg87GYvLstJTmZbIpFbpKi9oiM7RVwTuCjN7qzvQJ9BqtseHiA
9gqEhP99K6iVbFIg6c3D0t11ZC4OowaWySYcIyMO5655J3yWwfZxkPg2nU1T
sX0c9E9LZD/6sL0Si89w8fMG+2mvxP5d4o/n/0rmlbB2pSf7FJBNShRlXds6
W0vm4lHudvP5hslsHw/jvrXpb219tI+HOm/QMFxONsWjPso/NNNA5lTo9H6U
WO9HhgoBZ0+4Krtf0V4Fa9l+j+xTZJMKeYbrEYINZE6NsAbx/XszyVDjzsVA
u4j+l7RXo8xzIDP9EtmkRlVKcaUtm8wl4JBzTkeKgowEzLUEB/3pzvYJWNH0
ZIFT9wvaJyBfUhvaW0PmNDiqPh651kiGBgrZ+C3yz8m8Bsun+e7ZGsz2GnS/
rnK/7Mz2WtzM+MBmbO+lvRaeHs2vD9WQeS3MLR1bBwvJJi0yHE7vGF5P5nS4
YV/kfEXO9jp4SizN5yexvQ7Ly+c6Bg/20F6HIk5aZG8mc3q4rFqwb9ZxMvS4
sOlib8W3ZF4PR+1HDc+UbK/HSdutvuk+bJ8Ir7dGYUZ7N+0T4f+dcNR2PZlP
REv46Vi74i7aJ2LLnuHDUw6QuSScEwq9vXRW6T/ZbU5G
     "]]}, 
   {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJw1lQ1QjHkcx5/uYi3dueMMSpRMF5PSlU6iviltu1vb1m7vL4Q94orBseOk
3aRFSoo9d116HemcExfXpa3lrLznZeSiDqVFkVZ1Vzqcm37/ndnZ+ezn93me
3Wd+8zy2y9aGKj7gOE75/v3/5/Crx3t96H3FoZ5xoC/Q2Hj4/FX+C+9hHAWr
Ull1NMf4E0z7/e7b8TZ64s9QELRnrfhpNfEkTHR6tKRSzeanIF4oP9uiZGwD
t0aXzOy3vxBPx/7BiuL+pczPgFN8RvX6V0eI7ZFqb5gkCGLeAXbL0r5SPCgj
ngV3c4miX1JK7AipjdmMRGc274Q3P+vUx64WEc/B5e8EfLkV8y4496B4hF9t
AfEXsNpm3h7LY94Vfd11qVPfPCd2g4GX0tek+J54LkqarnsbnjDvjtqa5vTS
FsZfwqvj1hyz+VrieSh0Ty58bGDeA6cXaPZG1zCej7ldvhckt/KIPbFU4+nW
Yc94ATpHLlPztGx+IZbrUvwn7GLsBQuTnpeu3EfsjVavljZ5EvPAh5J2nTKe
WAWoZw1x/GDmffDu2OknToa95H1gYWulsHck5hahkicOKDucTX4Rfhsn3JHv
Ssz54toPgm/5t7PI+6KuLH+57mEXeT88OnHIZCdg3g+twSLbS5bE3GIobrls
0ZsxvxgnO6TZiqE95P1xecy9pMV85v0xc1Ja8SlH1gvQsVkbc3c0nU8lwKV4
izUPWjrJB6DXynyhypr9/gCom1y1bmuYF2JT33Ez0QxilRDZNy3Hz/s4h7wI
JWcEbdqkZ+RFiAub+nnabHa9xbgx6PCXYkIueTEsU7p9BrqIuUC8ul/1q3Wi
kXwgivYpmz1nsf0IQm7pGW6CQxv5IOTkrL9yUs3+nwRXb6qy9rpcJC/BnO6A
/tnnBsgHQ7NxdNSz9tfkg/GwMJkvM+0mL0VvRWvNm4bBYYYU+kf+gWcdyKuk
2N4wZLS/smuY9VKES9e57+5lxw+BkK9pmlJPjBDkJnR6jGzeSX0IBp/cW7Vy
JXl9CBLm+Z8r8WF9KPp2bgzeMI31oRBdE+9Iz9FQH4rWuhttP84n1odiKOvO
4czz/1Avg1mRr491GTFkmFhuHZmQSqySoZqbIi69n0G9DO+uO4qu2dLxODkm
a14me+wghhzlG+3GL7jwN/Vy2FptWDlzH7FeDokooUs5hq4HF4ba+osv1tmS
RxiObkrYOmTspz4MM8fVyjtmZ1Ifhgvlie2FY9n+hkNT82ecwYnmEY68j1b/
tG0q28dwiE/nTs843kd9OJJ0DUKzdcRcBNSOo4ptBmgfEYFeS6exp8pp/1QR
WGJ8qd6q66U+AkY3baUxg5iLhPOBgOWuj+l+gkicKAjf1qjbT30kJnokpRdV
HKA+EoaeGj9T/Svqo3B+oChwdS4xomDde8i5+/lB6qOQmDk9b4sneX0U7nh4
8DOP5FMfjfiUtx25T03UR+NK1+WBonpiVTSMJuGK5jy6X+ujoQqrSHuXTJ6L
QdVA1qd/CFgfg8L667erbVgfg/11TmM8B3uoj4HXNKmP8jh7nsTitbNFlWTp
UepjUVkQ+O/dM/Q8UsWCp1ux9uT2E9THYlG7jcx/VRX1cXA9uJmf+vQl9XEw
55ksUmKJVXGYnDdC2/lNA/Xv572Pltjld1Mfj8KvXT6wj2nx/g9ymkEu
     "]]}, 
   {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJw11QtMk1cUB/CKjmgxKgyNk7coOsKjgGhBkD9QkIdiKQXaUhgTqk6mTNSt
+MBvgqg8FRQR5DEhiDJ0ICIWH5/RqazIdDqFGeKm4hiTCRIngsNt4dwmTfPr
P7e9370n59isTpZpDAQCgfa/9/+f468BnwKDO50VAyagL7BHLHP3L//ZZ5yT
kbTNt19UeZM8A/M+Dxz50Jonm2K0TbA2tLeFPBu6DWU+3QebyOZIbepriKxp
IFvj6nrHtLyx0+S5MHZ4XMBV15HnwV2kqU95VUu2w8GUYOfsCzXkhRCcuKfW
PK4m26NZuUT9OqyK7ICXkoaWQ2++ITthRFSTWt9eSRZBZP9aa5RTQXaBOmXa
G/+LZWRXvK/KwyrrY2Q3ZJcbnvH5roS8CJZH9b8/0Bwlu0N6d16sjVcxeTF0
zp39HU5HyEuw41Xe/AmeRWQx4m3MIsJVh8kekEqMtuTkHiJ7QnbD7GLYT4Xk
pTguTbDvsWP2Qp9S/3RsfwHZGzMGHPWXRw6Sl8FUeHYsXcvsgy11WVvOGDAD
yZzUT1t8YNwcMKjstr8uJgt84bbT/KHT9/mU+8JzR5vczoEs8IPewENcXZNH
uR8ShjZsLXUjC/wxbF2SNOVeLuX+EA8+q+zIIAskWFDk+sR2OcsluDL9lXHb
HJYHoNf02Xp+AssDMJQyaZdmNIfyQHjnZ8QFTGF5IIQRHxWcc2Drl2NC3IDV
rgSWL4djkkJh3MjyIPy6y8yRs2D7D0J6+abK5lq2/2AIY58e3hfNnj8Y7VkV
QvE0dj4hqFq4qPCzDHZ+IQjSG1rudmTnG4qSktL7mpl0P1wo3KZ2Lxn+k93X
Ciza0ZG98wXdJ7cCOXUJt5fas/pYic3eRTeE/VQ/3EoE/XGuvPFrtv8wrDk5
MTPf5RblYejtG3yv6FFSvgpz+9+NHN+4kfJVsH27flQ2mEW5FKqmW6fsz+8f
N6TQnLrjfXUh5ZwUpjHXOu30lPNSPJ/UbJObQxaEY2PrVkuBJ1sfDv1gooNh
1z5aH47tRS7u8ZlkPhy9K7968U5MFshglqbbbvJ8L62XoRR7tRkHyJwMnIHH
g2OeZF4GE139fp8nmbQ+AmHFjTfTG8iIgKfkqY2+icxFIKXi/LKqR2Q+AjPb
TnrftqHfE8jxPAmrPfaw/5fjh86EobNv2f/L8aClR9Oxje1fDudHI91aI/b8
kUi6cFe4+zR7/kiIhB8rXRLZ+UVitU4a2OOYTesjMXFtwf2K6ax+o3DhCy64
2YjuE1H4tra4JM2S1WMUUl1zrYxCqf74KLibHbluEcLqLxojfR7F1sNkRKP8
3VTDcyeo/rhonN0kKlGqqN74aHgN5//9cjbrHwrIguYo3Z6RoUBmyLXNP16i
/sMpUJa4IbXyJPUnXoG++FHf5FpWn0pUd/1WF3eJ6hNKjOXPt/3rBfU/Ton2
+iepyR7UL3kldGPZo1m1pbRehdbs9sBJEuq3UMH8wPQu51nUjzkVEvND1F2F
1K95FSQLJod2XmH9PAZz/lGfMppF/R4xuLr7y7WlVTQPuBg4GVt94LeN5gUf
A1dxwGLtGTZP1DB/Y1EbFk/zBmpcTlG8fNhK84hTo8FCvq4xneYVr0br3U9D
A9exeRaLsom+za3NNO8Qi9aEXz5Z8/YKrY/Fpanywr6tNC/5WOjueRm+t2Lz
NA5qactQ2nUT/AvDuCZD
     "]]}},
  AspectRatio->0.3,
  Axes->True,
  AxesOrigin->{0, Automatic},
  ImageSize->{504},
  PlotRange->Automatic,
  PlotRangeClipping->True]], "Output",
 ImageSize->{508, 156},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->522204541]
}, Open  ]],

Cell["\<\
The effect of small changes in the value of Epsilon is dramatic.  A vortex \
panel has a discontinuity in tangent-component velocity across the panel.  \
Numerical computation using machine numbers causes collocation points \
specified as being on the panels to be numerically on one side or the other \
of the discontinuity, causing results to switch between internal flow (blue \
line) to external flow (green line).  Recall that Aerodynamics uses a \
clockwise numbering convention so that the leading edge of the airfoil is \
near panel 60 and the trailing edge of the airfoil is near panels 1 and 120.\
\>", "ExampleText",
 CellID->189149207]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Possible Issues",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->1305812373],

Cell["\<\
In the specification of some airfoils, the trailing-edge points for the lower \
and upper surfaces are not coincident.  This appears to cause a problem for \
the doublet models with internal Dirichlet boundary condition, but not for \
the vortex model with external Neumann boundary conditions.\
\>", "ExampleText",
 CellID->416625656],

Cell["\<\
This example uses the Clark-Y airfoil specified by a data file located in the \
folder Airfoil Geometry.  Import the file and plot the discretized airfoil.\
\>", "ExampleText",
 CellID->32738591],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<Aerodynamics\\\\Airfoil Data\\\\clarky.dat\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"grD", "=", 
  RowBox[{"PlotAirfoil", "[", "rD", "]"}]}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->112529997],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3c1rFOkWx/Fc7l9y/43AMAtxIS7ECyGLkI0XGt3MbHSjm2SjmwQa3RiQ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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->112729628]
}, Open  ]],

Cell["\<\
Note that the trailing-edge points for the lower and upper surfaces are \
close, but not coincident\
\>", "ExampleText",
 CellID->390242666],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ptsTE", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"First", "[", "rD", "]"}], ",", 
    RowBox[{"Last", "[", "rD", "]"}]}], "}"}]}]], "Input",
 CellLabel->"In[2]:=",
 CellID->238007975],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1.`", ",", 
     RowBox[{"-", "0.0005993`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.`", ",", "0.0005993`"}], "}"}]}], "}"}]], "Output",
 ImageSize->{241, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->266256017]
}, Open  ]],

Cell["\<\
Specify an angle of attack, solve the model, and compute the lift and \
pitching-moment coefficients.\
\>", "ExampleText",
 CellID->25666953],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Alpha]", "=", 
   RowBox[{"5.", " ", "Degree"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "rC", ",", "VMagC"}], "}"}], "=", 
   RowBox[{"PMVortexLinearNeumann", "[", 
    RowBox[{"rD", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl", ",", "cm"}], "}"}], "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Vortex\>\"", ",", "a", ",", "b", ",", "rD", ",", "\[Alpha]", ",", 
    "rC"}], "]"}]}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->592619],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1.0118375549372982`", ",", 
   RowBox[{"-", "0.34690749648915653`"}]}], "}"}]], "Output",
 ImageSize->{142, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->93647693]
}, Open  ]],

Cell["\<\
These are plausible values for the lift and moment coefficients.\
\>", "ExampleText",
 CellID->46735897],

Cell["\<\
Plot streamlines near the trailing edge, using a scale that makes the gap at \
the trailing edge visible.\
\>", "ExampleText",
 CellID->492689119],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Psi]A", ",", "ic\[Psi]B"}], "}"}], "=", 
   RowBox[{"ICPsiVortexLinear", "[", 
    RowBox[{"rF", ",", "rD"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Psi]", "=", 
   RowBox[{
    RowBox[{"UniformFlowPsi", "[", 
     RowBox[{"rF", ",", "\[Alpha]"}], "]"}], "+", 
    RowBox[{"ic\[Psi]A", ".", "a"}], "+", 
    RowBox[{"ic\[Psi]B", ".", "b"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xMin", ",", "xMax", ",", "zMin", ",", "zMax", ",", "contours"}], 
    "}"}], "=", 
   RowBox[{"{", 
    RowBox[{"0.95", ",", "1.05", ",", 
     RowBox[{"-", "0.03"}], ",", "0.03", ",", "14"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gStreamlines", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"Contours", "\[Rule]", "contours"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "False"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "3"}], ",", 
     RowBox[{"PerformanceGoal", "\[Rule]", "\"\<Speed\>\""}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gStreamlines", ",", "grD", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", "\[Times]", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[4]:=",
 CellID->266881112],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJx0nQV0VMf794HgxbVQpFAI7q6DFrcChSLFIVhwJ0SQhGCRJQIheHErrhd3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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{4795, 4488, 3882, 4779, 3883, 4489, 5328, 5327, 3977, 5400, 
          5644, 5643, 4726, 3881, 5330, 5329, 4375, 3853, 4035, 5747, 4034, 
          4600, 3852, 5723, 4237, 5275, 5274, 5581, 5580, 4452, 4717, 4216, 
          4679, 4020, 5052, 4140, 5539, 4139, 5756, 5102, 5152, 5151, 3726, 
          5153, 5101, 4794, 4897, 4896, 3836, 4678, 4895, 4894}]},
        RowBox[{"-", "0.018500000000000003`"}]],
       Annotation[#, -0.018500000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVzzlOgmEUheGLC1AEpaMwcS5xCcQNmLgBaI10akWMpWhtUBOF2koEHBpp
YSdCgo36Y+LwUJy837nDyf2WSpWd/VRE7NLVXMTyYsQ1lucjzmcj+vyA3rIR
jZx+OuKGX8G2Wsvs2OwBP+QPcYT1hYg8HtlZVet4/2Qifqkv993OtveWfqJf
wAn21F8pMd+lR/ozV5Y3sDfUm1FLUdCanSe8xw93tfAT1/FZfQNfpn135uRs
8nvYNNOg2+lfZBbpRH7CP5j5wjZO8BTPqOrGGn6bq+Mxf4kXsjPuy1JHdlW/
Rncy/wFoNDkq
          "]]},
        RowBox[{"-", "0.0222`"}]],
       Annotation[#, -0.0222, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVz7cyBFAUh/HjBWQKVi7NCDWFVErjEbTy4AH0ajkVmNlKbNTyYwhrMSik
ZVbht8U33z33/O+599YNTwyN50XELJYLIhrKIlZ4uChiPj/iVH2Gp5KIrfKI
6sKIDXUNv8m8oqc4Ylqd5Eu9C5yjXW8OWXOanc3INHErWjCrvpKb4Wu+kkvL
X/OfWVnU6h26+4DfZT7xgTr1kf16Ps71vblMvkE9wqdcyiWYMy9pbkpuzP49
j3OnXgcy+hX2KnGivimF9bY33vIOj8rv8p16kxPqVe+o4mezn5DWm1A/8CQ/
8oJZCV6TXcee9ac7vnJvdO+Lc13W3WizHjB7EL3oRx9+5Br5l5fMW8SU+d/O
nNlLOXfOP7m/4MAd+8jgH5grT6M=
          "]]},
        RowBox[{"-", "0.0259`"}]],
       Annotation[#, -0.0259, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0TtPlEEYxfFHDJdKRdlVwBCgURAbECvUxMoa7Sy4NKIkgqChMMZtoVDu
oh8AxNtSCCFRQTdRv4QKISZQCQJLogX6e4t/zpxnzpyZfbems6f19oGImEXm
UMTl0oi5IxGFxyKKUIx1/mBZxJr90XREgfV589ThiM/WZbSZn7F3ji4djWii
jdjHAL8gN4+3KHHHPbO76McZmXrUJffZ6zO7gx70osis2LlCWkL3veOK+T96
2plTyHjjJn0ok0Geb/CePXqW9tGPqYgcPuGLs9flv9K/uv6gze9oNZui1TK1
qEE7f9W8g16jo94whvfu+YCs7gq+Evf1DeGdcyuyq8l3wRh/XO659Zyei+7r
5m/RDh1L9pcxrmMC5XLj/ARmrX/JbSKne0P+knW937Wjo47u0uWkB7vyL/AS
W0m/vk78tLfDb+M3TjiTl9nDPH9S5rV1hfkrWknf0HL61DueYRJTeIIH3jKt
cwY3ZB45/xjDaNGXt59yPo1F/hsd8dZB8yqZ7/yP5HvqGkFaxzSf9X0uyHfx
N5P/RX87cvx/wiJjxA==
          "]]},
        RowBox[{"-", "0.0296`"}]],
       Annotation[#, -0.0296, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0d8vlnEYx/FLVHJAz4NSytaB/gf8Ef04KJpkoi2hE+sontnYysnDSW2t
kFLmwIZRqSb/Qx2gNUrSrwOxDlRbr/vgvfd9fa/r+nzv+3lONF0/25ETEU8w
WBhRWRJRcCjiABaLIibSEb2piD7kYZ/+3+KIf7iMG2Yq+SQKSyOKMGVmGjO4
czAiI+su38Zb/Vv8jjPuG5d5wW67nDru4HK9YziCfvVR/m2222zG7pDcYYxg
w9muvT+oRZv5wzyoV8Y5dvcgF6fUp/HDNy3Kq/d8z/5rvatyX3Erv3Q2j/bk
fXAGDXJ/2mvyXGCuU33fHQ9wXF2BJc/LWEneTU6P737Iw1jVH+I1vunuMfnn
ZbXKOcfXeABtdrK8babLfJe6mz+aX0t+/+R/Ue/lfOxHs3pW7jPMIaVe966f
sYFP9mq4xflTHke1s6zdKt5x15Zev/oXX8FzOfN4gQX9JXNv+KJvq8eA8yy2
MOr9H+GL3Aa9S3hv/pv6KzaRlvnYzCSPcTF/d95o9oPZSd+cVpcm93MJp3gG
/wF9NGRq
          "]]},
        RowBox[{"-", "0.0333`"}]],
       Annotation[#, -0.0333, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ElsjWEUh/FzsWvpnVqusbWstL210ZawYIHEQg2rokOathK1EglWpqCG
oBXTlQhLl003NrQkhkqwKtpUGxbstNY1/Lp48rzn/M973i9fVfvh5t5ERAxj
ZlHEulTE5YqIBq5GUzqiERPO3/kb7mciFiYjjprLOl/IRoxzh94XbucxHnNn
HKXq8+URJVxw57OszXm/bB867WxVN3CxLOKuvIA7+IRRDLp/zve1mC+a/aV+
xNP8VG9a9s93hHqW/+IPNnlvI1qQNvdK7zWWe2eH91Zwo/0Z2Rv9txjBFVml
bJt7lbKtXMW39G+iSbZKfZUPya7xBtyWrefVsr1c4DN80MxZ3oUO52a+gZWo
8M2LkcVpdTkX3XuMD+oa9XtO2J3jn75vD//g3TzfvgVIYMg/mceT7k7hhPqj
+8f5t3+UdE7hkrsX0YceOx6afYBjZga9U+L8TPYcQ9iulzSX969KZcN6L/By
bo8sLauXLZG1er9OfV2/lvtwQK+Gl8nbnPPO/fIBLNXbrJfjLXyP074xg7X2
1yOPne4MoNdMP6fm3nDu8W43Tqm7+CQ/seMIr7HjHVfzCIf3yrjWXB0m7Z3C
V+TwH8ZdZjo=
          "]]},
        RowBox[{"-", "0.037000000000000005`"}]],
       Annotation[#, -0.037000000000000005`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV07lPVGEUxuGDglsxw6igAQFNiNHStUBN9A+woXEhxmjAXRDQAitlm3GB
whi3Ak2kwSFRK40asVVhoDJEEvdSsVQ0oM8tfnm/873vOefOvZk1h1vqm4si
Yhx70hFDZREP0Lw8ogVnULU0ohrbSiMe0jq6IRNxCqcxru4pj5igE2a0L4sY
kXuFeRhMRczytsrM0WPujiJt9lr9KZpBKV7yQ26Ezpgz6Fk2Og/Kt8q2ocu5
E6PmDvO38/N0B13tvgYfeefMO4sB89K8yzINyR5+St2vLqVXaB+u4qa+x3bc
ovX63vDn1LN4ofc5nuEGb7ff+sW5FhXqShx3PoGT6LanBx/MWMWrRhUKZq9w
P0ab5BoxJVPG22nPLjSZXex5jtASWkEv6LmISdlNejZjCx7J3zerl2bRL9OH
Sj0Zda33eAjt6jr6NnnvztdkCs5jGEW57Hz+lFlZXg7fku/B+0qL6HveP7l1
nmk9WuVfm9VGa+gBv+G6bAcK/PN0nB6kOf2X8NOMp2ZMu+9EF/a635d8f947
3qfkGybvDsV6b5ufp3doCf3sfr/8pOywvWn1QveNdAFN0ZX8u/KL1QN0Cb1H
F9GsnTl0oxc96LD7t54Z/MJf/MG0+yf2/JAZ0vs9+Y/QhuR98/L0PzOMdYk=

          "]]},
        RowBox[{"-", "0.0407`"}]],
       Annotation[#, -0.0407, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV01dsj2EUx/Gn1AXSQSlNlBgJiZYY1YpaTYyo2CMxelGi0iatkRgtxZVI
jBotonbQGzFKrDZx4cKFEbpCWyHGjYhxQaj1eS++/985v3Oe85y87/sfmF+y
oDgmhJDqpyYxhGNJIXSmbT1DaEd9fAgNPUKoo0t7hbAEu3qHsBMfUY4dWMRf
jHv6nuu/S6/za3ENeWZ+wZ7kED7TT+oZ+vbJx9GxyHTfOLSI76i3io/apwqd
nNmkN0m82z1NdCWvga6gjbTRuSbEyLe7M9BqZ87jHJbL//Hr6R9aR//SDtrF
zN80Q9/jhBDS+I/oKXvEmXmSLlR7pqfUXluj58OLde8H3nrnN2Aj9qi9Upuk
NhntvDbkmPc+erZqW/Q8FieY+YSOUB+JE2oDedV0Op2BaUi1w1FnqjDenP7y
LLofLc6X83egH7+rOd3RDdXyb3boqy+LZuKw/q80Den4pacDJ/hnzCgUF2Gu
eA4q7TLRDvG8bJpAZ/NzkWzuIeeao2cu7yNPjN6DfJv8I421xzDnhqJdnkzb
aB/61Oy+NAVr9RVgDdLVX3oHrchVm4UCc4/x88TD3DPT7nPoW/3vUCQvxANx
Er9S3wT5EZodfR/8BoxWy0V+NJf2NncIrbDvQfSST1HLwVTUyo/b86JdLuEC
MuxRQ8fQFGea1ceKH+Kn/vjo3YjXic+oHZYP0D+XfrDDPLoaBeavoum8NLzH
cDRH3zgy1crM76GvJxLRaO5Nc2/gtNnF9ADvlv7buBN9y7y46J0hzr27zLnC
v4rdepfxu/O+6/uB++JBaoNRYeYLZw7QdlodfbP0JJ1vl3kYpX8fiszZSxvU
nuOyvknRf9WczXgTfc/6X9NS95RhKM7ySuh/VPugvg==
          "]]},
        RowBox[{"-", "0.0444`"}]],
       Annotation[#, -0.0444, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0slvzHEYx/HH3dZaIpagtSVoSaeLg7b+BC6WmxGiCSqxzEzLBZ02iq5K
CDJjSdQScbHFjYOkSRcEJ8vJUv9AqXr9Du+8v5/neb7Pb34zszzdvO3QtIgY
wq+ZEUdKIkb4RmnEfucmvJWL8k0ck49iTC0vj/LBuREHsGx+xFLs1f88O+Kd
3gIz77lCfoPmORGHUYZyTOql3JnQ+zMv4ilP8DP+y+lZEd+cP5hbaNdHrpGH
sAmf5EXqi1Ent9hVy73qXzgvf+VKbERO3sAD+q/sfo27eK62Va9NfdDnH5c7
5N/cgC1olRu5YKbdnQ6sxjXPvo597vXpDXA/l5tfgTKswkp8d/8n2p1/cNFc
1nwOa5DB8cTqZ7kTa3Ee57DO87q5JzmjCxdwxfxLn6HfudrMA+cavihv9j33
cUq+r17FvfJj50vcID90buTLcj1n0eQzZvi02inUycG3/Na3cQedyf+Ciyhg
yjvVmqtP9qIieQe5ktdjhvnpmLK31e97Alm0IJecvUfBXBHb7TpjZxvyqHJv
XL/E+5TiiTzM3fbn1ZeYGZFH0YWepIdBz0qZ3WHf7sR270yQx/Svuj9pJuP5
/5LvTO0eV5vdZWZPYvNpfiE/4pPyfzYjdr4=
          "]]},
        RowBox[{"-", "0.048100000000000004`"}]],
       Annotation[#, -0.048100000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVz8lK1mEYhvEn0PQzcCxblziegXYEQoMaVK4UwZamgi2UKNLUBmc9ATdK
aWDgVFY7DUWccqgzqJU4NBnUz8XF9d7Pc//f73sv1d2rajgTEX1IyYiYPB/x
Gi/xCklm5dkRyTwh382JaHOu51aeQ2NuxCwfZEWc5Vvy48yIy87tfFtOdT60
X5Q75CV+YPYWTfI8H9kn+I7cYZ/n/AjVchqPmr3jZnmBH6LgQsRHzucPfOyO
97wld+pv8BPe5BHe5X7e4QHe40X9f96zIi94a4F3FuKNu0q4GEX4pFeov8RX
Tv+TfSknuIz33dGJcecu3nTXVd9Np0cs68zwZ7MtPLXvxiGe4Tm2za/pz+qt
6M/xDfk67vvdFnzDd7zQP/I/dnxTYV+JVd98kfewi2P7Hr0x7+rlH/JX8yrd
m1jTX8cv8xrzn5zgi3ZD+r/lQf7Dw6fv4nP2f7mWTziNp/Afcipciw==
          "]]},
        RowBox[{"-", "0.0518`"}]],
       Annotation[#, -0.0518, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0TcvxXEYxfHHgkUvu9GkvQFcXD1RLok2iBYRFgail80uykxMJKxWvb4M
DNob8LnDyfc5z/+c5zf8y0bnumczImKJPnIjqgsidksjarCcqgsjTvkqfOMv
ZdqKI1ppizbp3e6pKOKZ7umRHtJe/k3vlXb4v7yIBbfyzRt6d9iXH3GDKbxN
d3Ve5J9pm//VmdfJM6/rrNF5ScQZ9cuOyPXqVuKJ7JbcN077/oUHutd2PTJ9
8ikalu3mK/BYJhs/8Qg/MAu7fO+V7aFDvsV+33zl/U73hvkf8xDueSNlN2nO
9O5U+gY2Y5N9EpO4ik9uPVIHP8hP6Oamv7nViE12CWzEBlzGB/l7aucH+HGd
HPOKTj022NViAutwEcfwwn+507ulNr6VZugfzL9FsQ==
          "]]},
        RowBox[{"-", "0.0555`"}]],
       Annotation[#, -0.0555, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0DczRFEYgOFPQSnTKxWMfyGnlXNOI+4uhhmZP6GiQKn0e2RrhpXVHsUz
75xzvrn33FsxvpxYyomIFMn8iJOiiFOyhREbpRGb3NmvLI641zfrd17I8krK
fnNZRAtNtNHKdUFEp3bRTgcJ0uZ7tJdu+unjxvywjjDAEIOsmh/TcUa5Mzeh
t7pVEvGgk9b3elweUepOZew4e7S3rU+6q2ea51uedUAzmqvl5vedv1rvaVYP
9EVnPXuOKWaYZs2dFnSReZZZ4sN8+v/OrJAiybr5L+/45pNffsjYv/K/M2bO
/fMnvdB2e4fOLq3rqKeWRhr+Z72nyr2rqTGb4Mj8oPUfHGpGpw==
          "]]},
        RowBox[{"-", "0.0592`"}]],
       Annotation[#, -0.0592, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ccuRGEUB/AjLLTo7G2x1Y2Z0e2EDQslYk1ipzyB19B7eQN1eAJlYRI1
eAm/u/jlfOd/vnvvSW7j/NL4YkFErJCtjHitj8jzWBfxREo2XhPxXBVR2xDx
otapy/K52ohZcuZT+lHnLfM1Na+/ld9xURFxXx1xro565whDvjHIF/0MMCQf
5sy9nPun6qB+gD7zLJ+kydAn7+fEvTv3j9Ub30s5F6mFTLDBt9m6fMd5m017
tqjNPMiPzK+dryi3fxmfsib9V/Ks76wx5rlL2aHs1nMHalaeoddOKT7opode
eZr9ZDf399SUvocu807eaaeDLnk3u8k+yb5qp76DNvNW3vjV/yUZ7Wy7Vywv
Ycbu0/zIJ/2HBf+rVL+q/wcrGUOV
          "]]},
        RowBox[{"-", "0.0629`"}]],
       Annotation[#, -0.0629, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0MkupGEchfF/sSGIoem9sa3MV2CeiXSZllZWtMS8Ee0mJGJe4AKwMpcL
ME/d7sEd+NXi5PnOe573y1dVOjE9NJWIiBUpzY9YLYj4/VPx/FUYcYzT+kma
Mun5D1bKtf2E/9fZKTbL2Y+IO1tNScQ9zuqbWFYU0ZJ+N7a61yat+rn9wV7L
f8Q5fQvLeW32JLZzO6Rdv7A/2ev4zzivb2MFr8M+jJ3cLunUL+0v9nr+Ky7o
O+nv53XZR7Cb2yPd+pX9zd7Af8dFfRereD32Uezl9kmvfm3/sDfy/+GSvoe/
eH32MeznDki/fmNf93/95zS584nLzvaxmjvAGcdBfsr5rWRJtiScJd1NcKc4
Ke6afoszznK8L1cOiiMOJcPZEWZiprsZkqdXYC6W40b6d9i/AYSwQKY=
          "]]},
        RowBox[{"-", "0.0666`"}]],
       Annotation[#, -0.0666, "Tooltip"]& ], {}, {}}}], 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdVX001Wccpw7ZbUK02pRi63hba46JrWOf5iYuS1KRRRnVZLIUR7gllUh1
DL/lUm6KishLXi4Lj6QYl9a0rpIbufI6Kb0sWq17e55/+p5zzz3PeZ7f5/l+
P8/3+/kY+//isWWKmppa5Nuf8v9djMHQmz/uks0nhU49UQaFj5DiUFayvWAV
8Umqf2bm8wgax2faNd1dR3iy7B32vEdoO7XPdlS+gVQaxQ2vrhxFy9oIw8TH
fsRABTAKba65ue/ZVhJqr/ziH7i3z4v6biKYtEqVMQLB4UrOLSaUmKsuGMGQ
mX+wMDaMKNHeQuJI0/TlFw5EEGU2Tj1D+Dyw3LAjLoqo4OyH0Dpt07hGwh6S
ka6MQUjWHOz72HEfeW6mBBxAwqfeN7+8Hks8VAkOYP34ohonpwO0vn6YX52S
t7HpIK2nHy9TZFy44BDZqoqHMCs+O5LeHE+uPlMC9sGrddfyWtfDZL4qwT4o
bJdqjUkTSbSqYAXKp+k+1XQ7SlRwsl6YJg7v0r1xjFirohddobcM36xKIio6
kx6gunKA+Af+SkZU5fbA7n7vealZMnF+VzD013RM8geTSY4SLrsbHxRVpvbm
pZB379UN0/6oZBKUSvmXY94Zw3y5JUf57oK+b3XDfyMc5fce2p3iPKUXf6N8
3sVXh7gJ7ZDjlL87yJ6YU7N5URrlqwOtrjprZcNplB8ZBM+1bPgXRJSPvxH7
Mvxx07Z0Wv8txE1rLYy2zKD1tkMvMtY5fCyD1ncTcybvfWJSeYLW8ycsIy3V
+6NP0vzbkO3u53rHOpPmK0WnPEQwoyOT5vcHjrbxJbl+YppPI6ZYzP3O4pqY
3t+Aux1LbutMiul9V1AcMFPh+UJM8WshMvfQLC0UU7wqSIxu7A6exb4vQ+5V
nkT4WSY9X4jzK/jGvo0n6Pk8HFC/35SWn0HP5yA/BBY6c9PpeTFObWs3GVmY
Rvc5/K6to8mv58jE3qDqsNQEDA6Le3b+mExe5T3+a6X/PljJHd1mORyj6x1I
5CcLTsfH0bUfzEhWd83rGLp2x9QKefN2z910vQy2wRu4Owt2EbX3gu1/0Wip
a28eAPa9qUCY6poQAobvVlV/2achAuz+nzmY56zYD5bfA03h3jnGR8Hy5z+5
37pZkAJWn0dMTf3+yXSw+jttjC9aDWSB8TPf9lLPKa0cMP4yE3lHluWfA+N3
2Yq+luLnuWD8+/30ckeMogDsfbgWm8s5C4rB3u9h9m1Hw+QSsPet3uB9plR6
Cez9BVVZAQWdpWD9YVb0zezhujKw/hEJIj1aosvB+mvGm+V1Qu0KsP4TyLmV
BsIKsP40kIr227RVgPVvY+hx5/U8CVh//+sXkHtriQSs/7sUPoufeEnA5oPn
296qs10CNj8iLtRBFiYBmy+XqYqx0bf7bP6SXDyHXntKwObToSB7lftiCdj8
Or/26656UQE23/nh8VWCEpa/HEZP9TJ5vhVg+uDdsXvdh6/KwfQj0cu70yCp
HExfmu8ZBzl9VA6mP81Hvq8rSi4D06cHEpndavUyMP3aWViNr7eUgunbqGmA
UFTD+FdAunR89vzpl8D00XahzkSCWwmYfmpcX1cnSigG01cjNamuSVgRmP5a
cN2BJo0XwfS5LLThByGvAEy/o62Dal2+vQCm72lXamTl/rlg+r/HK89xMOIc
mD+slOn7zt6bA+YfmRvb9MThZ8D8RUvjbGDfxiww//GtrBmtDTsJ5k/X7GL0
eXoiMP/qyt1kzUtNBfO30ojh0ybjx8D8b8LqpPH0+INg/hhvez3PShgD5p+J
CstYriMczF/VazurQz0C8Z7/4n9EbM4S
     "]]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  ImageSize->504,
  PlotRange->{{0.95, 1.05}, {-0.03, 0.03}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 313},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->920434690]
}, Open  ]],

Cell["\<\
Streamlines near the airfoil surface show no waviness, there are no \
streamlines interior to the airfoil and the flow leaves the trailing edge \
smoothly, satisfying the Kutta condition.\
\>", "ExampleText",
 CellID->482308682],

Cell["\<\
Repeat the computation after altering the trailing edge by making the \
trailing-edge points coincident.\
\>", "ExampleText",
 CellID->254018838],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rTE", "=", 
   RowBox[{"Mean", "[", "ptsTE", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"ReplacePart", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"1", "\[Rule]", "rTE"}], ",", 
       RowBox[{
        RowBox[{"-", "1"}], "\[Rule]", "rTE"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"grP", "=", 
   RowBox[{"PlotAirfoil", "[", "rP", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "rC", ",", "VMagC"}], "}"}], "=", 
   RowBox[{"PMVortexLinearNeumann", "[", 
    RowBox[{"rP", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl", ",", "cm"}], "}"}], "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Vortex\>\"", ",", "a", ",", "b", ",", "rP", ",", "\[Alpha]", ",", 
    "rC"}], "]"}]}]}], "Input",
 CellLabel->"In[5]:=",
 CellID->230503503],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1.0151125092347042`", ",", 
   RowBox[{"-", "0.34849519417223074`"}]}], "}"}]], "Output",
 ImageSize->{142, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->156850911]
}, Open  ]],

Cell["\<\
These values for the lift and moment coefficients agree reasonably well with \
the results for the open trailing edge.\
\>", "ExampleText",
 CellID->88671999],

Cell["Plot the flow field near the trailing edge.", "ExampleText",
 CellID->437823294],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Psi]A", ",", "ic\[Psi]B"}], "}"}], "=", 
   RowBox[{"ICPsiVortexLinear", "[", 
    RowBox[{"rF", ",", "rP"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Psi]", "=", 
   RowBox[{
    RowBox[{"UniformFlowPsi", "[", 
     RowBox[{"rF", ",", "\[Alpha]"}], "]"}], "+", 
    RowBox[{"ic\[Psi]A", ".", "a"}], "+", 
    RowBox[{"ic\[Psi]B", ".", "b"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gStreamlines", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"Contours", "\[Rule]", "contours"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "False"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "3"}], ",", 
     RowBox[{"PerformanceGoal", "\[Rule]", "\"\<Speed\>\""}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gStreamlines", ",", "grP", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", "\[Times]", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[6]:=",
 CellID->240459914],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJx0nQWUVbfX9nF39wLF3SkUCFLcHUqB4j64ywgDMzDCGIwwgzvFihQ/uEPR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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{4309, 5060, 4310, 4644, 4643, 3807, 4847, 4848, 4751, 5037, 
          5093, 3698, 5092, 5771, 4183, 5772, 5038, 5764, 4101, 5474, 4102, 
          4993, 3985, 4558, 4184, 5039, 4185, 5040, 5042, 4202, 5691, 3822, 
          4567, 3997, 5715, 3998, 3823, 5692, 4203, 3850, 4712, 3851, 5696, 
          4346, 4873, 4458, 3853, 4738, 3852, 4457, 4752}]},
        RowBox[{"-", "0.018500000000000003`"}]],
       Annotation[#, -0.018500000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVzrsuBEAchfGzVmG3cKdHyTMoeANBZxMKOhuVYt0SGhQktuMJkOh5AYV1
ictWFg9Aq+S3xZdvzsyZ/8zIUnVmtZBkFnt9SbU7OehP9tEcSNZ7kp3epDKc
bPO7809ekb/aGWXrD+7iFpd4WW+MS2aOcpmnzZxCp86vfNSe7Y2ivMUdvM0t
HOrV5RvrAj/Ij3jCrbsTg8k47uVNnQ1c49TZGV7c6fDuMxd5VK9h/8q/GtZ3
+ME3KuYs4NL9ef0LnuM7/WN/rMmRF4eSP3nXvHN7dfkEa2ZO6tacvZrTxBv+
ASMaM1k=
          "]]},
        RowBox[{"-", "0.0222`"}]],
       Annotation[#, -0.0222, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVzzkvpWEYxvHbF7CbznKUzFfwESxBhYRItJaEsTSGxlJYOyGxOyTUppzg
nDinUUxH1LbOEDs/xT//932u676f90109jR250REH4bzIv4VR4xwTUFEb27E
bEnEHHaLIsZ/RLzIOry/clYng0RhRGN+xCj/RqWzat0E/+RLez7MfOIdOc4C
zWbmvTfxAu/rHZs54Kw9Gby5Z8jdCzoZnUXOfu9xPuz8g0d4hh/MPmJV5z/f
o8uuMd7U2cIG6uRldpfiXH7F11jx78tY05nwfes8ybX6UzyNMTy5q903PHPa
7hR2dOv1trmBk9xiVytu7L7FEY6RdOepmXLPFTjTv+M23QvZkuzQfD8GcCKr
ku3K/sr2OO0shUH5L/zBnD1fFZ5N0w==
          "]]},
        RowBox[{"-", "0.0259`"}]],
       Annotation[#, -0.0259, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0jtPlEEUh/EDagsqaiLaINFA4jfQQvECCI0WCIiXEKyUZQELOxtvBZcY
E4UgYEehjY2UIIZbRQAbozasxhZQWBALflM8ed7znzlnJrNb1tpxLVMQEWNo
K4o4dSDixf6I5YMRXzAjmyqJmOXOIxF7fWd5Hz85FPEYeWt71L3yQn6pr4AD
k6hXr9rzwN5u/DK/TnYFtdjGFvL4aa2Gq3EJl5GTzZmzwvPJZp1MOW/yBm5b
+2Ctzf3vYlG9czhiif/zVe7hPvTind7T9r/nef1zqCqOKJM95EX3XMYSLqhP
yC9yOV+XNeKVc16j3+xv8u+4Y14Gz50xrB7B/dSjXtffnt6JP+ur0zea3off
cpOZv92j0d4mZNL7qT9hzMxpPcfTe1n7hy0s6B2UD+Cj/ePowB9nTNlbYWYl
JqxPYkN+Tv6X19U/7O1EXp3lbnRhU33T/FtoRgtuoNU9Hul75rva2U+5hkvN
PIqv1nPpd0QfhtJ9/C/a5Su+c3iDZtmqMxr0r/E9WQ8fS/8J887zGeecxah6
F/7faqI=
          "]]},
        RowBox[{"-", "0.0296`"}]],
       Annotation[#, -0.0296, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVz01LlGEUh/HjFygti1apUFpJK1vqQkwXYhlRQVC+zNioJVoSbRTRXimh
VeXGXPXyCRwddRRdOBroNrM2LRQqLcM+QL9ZXFzPOfc5//t+yhJ9l3sLIuIj
7h+I+HHYd2HEtyMR4wcjRg5FpIoiOpFztox5LOAcrpvZN/MPi8URS7iLe+jH
gKwbRyMGuRtv5XbxBCfdNyy3Uk6znDN8iVftfUIOHeoV3jSbNNtit0N9C9/1
lvSy9k7hNLa845qzbZ7kNEr0s+p5fLBzVt1v74U3jOI5GuW+1nuFaucP3FvK
tbxgr8r3jKxZtKINa+hz/obH8FRGrX89rpeR+UQ9xY95mtvdPSS/QtYFM+V8
kXvQZO8OfzaTMHPTTrvMBJLI6n2RMaG/wV+xiTr1CJ/nh/xTRtp7pzCd/2d7
x7je+RAXqUvR474S3nDfmrOUep0b8Cj/9vy78c552tx7PukdJ3BV/wqWkfIP
ncjILndWgYz5OfUsZvDLm7rMvJSds7OKFezod+vv8m0e5b92/3CNvd+8p37m
+z8wJ2ew
          "]]},
        RowBox[{"-", "0.0333`"}]],
       Annotation[#, -0.0333, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0k9Mz3Ecx/F35M+pP/QLlT+dbFxRmQmbk+oQNlumlWqjlMW4SLGp5NfJ
UDanjAMHfzKGm7IZx8qWOfhtjBP640L9PH6H556fz/v1fn8+n76/Shs7atuz
IuIdJnMi1udHtBZGbOASrFkVUYiX1iPyZ9bNqyOaUJOIqEYVUuq1BRG31ffl
RQzxXh7m52Zf4FtuxAG9X7nbHbdke/TslFVglzMq7RM8oKdenuL9Zo5Z38QN
9Ni3eUu5mUH9Y/ZJHufrahOyp94yikd4gsfIdWcOduOn/l61PvRjizvT5lY6
f1Z2TS2JbPc3yZbyRnObMt8Gy91TytncIS+SL7M+xdXq7VyC07JiXiEr4/PO
rpQvsd4qO2efbz/Defybs2THZcED3nAVv9QHnZHEFfTiofyz7AGn7P96/137
HfYjvJ2nfJOPmMAdZ0zyK72vcTTz+9jX8ZTZN2bGcBiHcBBVzujSewH1evq9
67v+LtlFdGOz2qwszT9kPWqXcBkNavOydf62BdlaXuST6idQ4Px/9s38Xn8L
f+AEt8rbMt/B3UXm0vqK+az9uJ63+OP3mscctpk5gxo9nTzn3k7rClk5GtXK
uIH7nHGEPznjPk/zPf6iPmNuwb2LGM78T2MI02b/AzKGc4U=
          "]]},
        RowBox[{"-", "0.037000000000000005`"}]],
       Annotation[#, -0.037000000000000005`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ktI1FEUx/HbqkXQQ8dxm5XLjN5FzBgUqUH5iKzsQY7Ry7CWlS7cBFlY
EGmCraI2QS+nKKhIiJYtcip0ow4G2UKllSOk+bmLL797zvmdc+6d/5RlLje0
LQkhfMWOFSFcKAnhIrYnxNiJ+VUhLKB4ZQg3ikIooklxFTqXh/Bc3JIM4QV9
Zsbu4hB6+HoxwXOLZ1htKc8IrZRP4zvfMvUc/Ykf6FfP2f+QfhC3OJ9BO38H
DvC/NO+I8yt6SS3B20pLaJD/z/NZLe3uKXTaO+letTxrxTPqv8WHxH9oHW1A
fdyjr0u9g27U99TMMfEo/sql5K7IVehLut8GWko3mbsZR+0/hkGeKfNmMI2s
3oIZA7SCbz0+8kyqrZMvR7VZb+Vr6Dv6j/ewWY14zztrTwFzuMnfpX5f7bzz
vHPGeSH+Nvpr3LORzurrNusO7uIszznMxW/IU6CldFBfk/w2nq3Ygi9qQ3Kf
1MbtGJF7bGal/U/oNfXjeq/SXu9ooj20Dw+QFtfpqUfOjHvxu8jvwV6sli/D
I7WsWtaugfhdMWr+CXtum3GSjolfy6/hfxPfLp6mebpLfpxOxTvyNuOX+HT8
/yCDifib2Lkf+1CNKpyy+7q+dqTsOmh+Lb7JJ8wdim+neXSj3Kw2b26Vzzsv
Ag3Se1M=
          "]]},
        RowBox[{"-", "0.0407`"}]],
       Annotation[#, -0.0407, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ldsz2EUxvG3xEjUaqlNuDBic0MrUSQkthJBSURSRbSNGEGiTRUXLuwb
JNq6IpSq1rqwV0KsxG7t2KNWrRuf9+Lb55zzPuec9/f233VeXkZuQgihuz8b
W4SQmRxCTfMQDiWFcBjlkZYhlDYLoUurEMrks3hm4nzrEEZgJDo564wSvkP8
xbRAPc3MfDqMXkdWSgjX6C3n7fkWyTvQdkg2OwmV4j3OD4oPYIZd1e40nfep
fII9O9RSzdlGh9Lt9Ii+CjzmTbfzEc3XswHrMYTvjvquuJ/upLfpfXpJf0Oe
Btivr9qOfbTQPd47X6D+gZbjqZ7neIZJ7nOe56v4G2pxhWe2eiaGmZOGVFxW
n8Pbz6xuZp8TT+CZiDJnX93hG77jhznfUcDTib+e7+pI69Nc/hy0M2MRxqkv
pKfMOIlpzk7EbzUngf8mvY1bWCqvon/NbRS/Gct5K2NN3z88im+IQvXl8hUY
LV+lp494JW1q50fnifQT7aXeE7+dLZMfx1T5H/lncYZ4Cn76pqvyC/b9Etfh
jPwdPU3f00o9H+hHDPRuA9AfjZwfUatAT3t7IN29RmBNfFt5Ph3oLfbwF8e3
j78TFInr4t14msR3oIl0q/oWtNHTB6PUe9M6cxrTLHf+IW6t/pMm0VZIxqa4
18z57pONLDSLvxWaSP86PxXfSrwPD+QPkSMu5llMS+h6niV23XCPQXQvHUzH
ROwZTWvd9QtKnJViO7ahrbPJ7vjSnFd4gWPmbrFnM9aZvVctV62U5tEa9WrU
yJ/grDlzzcmlecjmSY3/V713+e5ht/PX9r/BW5TrO4ylPJV0rZ6jtIj2c5++
SNGzAGPNyqZV8XeGHL4UOtyudFwyI4H/Is20azZWy1dhvPg/XaOwGg==
          "]]},
        RowBox[{"-", "0.0444`"}]],
       Annotation[#, -0.0444, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0UlsjGEcx/FH4lDFrLW7NGhCIsTSRmmboAmVWKaYDkotF11mbO20pQ4q
bQ+i5cKNq0TiZIslsRyQVg9iO5Ce9MCFm9TB5z188/v//us875QfK2Ty00II
o3gfC2FnMoT79GwqhM3iLXjI19OtOCf/gF8dx9wQOsUvEiE8on/VH9MS/g5q
0iHU4reZP5hQm25mbI5bGMUNfeO03r7X4id6pux5SuP8XSTxnP8n/4wm+Dpx
l/il+BVmYCay9pfSXrVbdt7GAG7KrVI7KX/J7Fv+BP+OzsMCNPHz6YCeRTTH
L6YLcdpvPIM8TqGAor5+u5pxOMIbm/FG/gjfgjg+yX3EkHyOP4AYmpBFTn4X
3Y1ZyGAPBuX30f2Yjb1oxEX56/Z1iBPedlmcpHm+1Pduj+7y/fIx2hbdNLPS
e8r5FIpq2+S34zi/gzagWz7NF8RT0Tc1M0m7y0LoQS9a9BRpFzrxQ32J/oNu
LNOfFi+lZbSCpugv/MRXu2t9uzpsRA024ZDZHvXz4jXR/0fX0oRbcYyrf6Pf
cQVXvbPD7lb5CfEwRpDBBrMf3GkUD9Kk/nV2VdNKN6qwnr+m3mfHZz3VZr7Q
rNwQvaenUk+fmapI1S9ghf7lOKr+HxGYZeo=
          "]]},
        RowBox[{"-", "0.048100000000000004`"}]],
       Annotation[#, -0.048100000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0MtO00EYhvGPjexEVBDBjRsvQCMtFyGHRonBGENVEhU57lxySBRXJFi1
lgWWllLQBLgCtqgICIQrKFCQa+DH4snTd753Zv7T2+nB1EBNRGTw+HLEUX3E
MdquRiSxXxdRlSe5+1rEHj/kXX7EB6iYb8sJ/R0+lCd4mW9djyhb35K7dD9c
iejkv/KC9RIWUdZt0W1GUd4079B7r7/e4Gwk0WR+E7O+tWRPjm/IBXva9R9g
Db/wGxvYxB+sOmsN9xojxu37aP8Y35VXrJfd+cMdP1HGMpYwpLfAJRQxqrvI
3/UP3J3nU+f/xxmm9E+4imnftXTxBr/n9Ar2jch5HuZ5fsff+CVnuY9z/EV/
11lfOeUbs/wMRfO0XOB/5qfe0Ca/kGvNn3NSPrG+Y565+O/lz/wU8+a9cp63
zat6CTktXzLv5Vb5yPqW+Yy1TvkT9+C+tzzhVn7j/Yd6d7yzwq/lV3iLfhxb
OwcFQmCl
          "]]},
        RowBox[{"-", "0.0518`"}]],
       Annotation[#, -0.0518, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0TtL22EcxfGfL8Ak5qagLUi9VKFWQafepOAtcRK7SQe3LpUU1MTo5KS2
3VvBO4hodHbRuSVCV23H+i4c/GQ4fJ9znnOePySd8wszn5siYo1uEhGtLRGf
8hFtmKJEOuI734w1flsnn43I0TQVqS4r5yIqtEjLtESn+j/dHdlOZ3TpJhnx
wXv/ZAXbb7KhVMQWDuJXPLE7dH/Q6PB1m1mbv/yUzabspe4GDuCx/rrOPf7G
L3q/8D//wv0IDlNJ3i8vOa9656n9KH/h/A4XfaNKK1Tky1ihZerS3dbrk7/n
H7xV5Z/I38rOnd9gDZfcF5wvnScbv5XOcz6DvZjFj7jvjT3q4F/z43Z3/IRN
WtYjS2E3tuAc7rrfoXb+FT9mc8uP2yRlXbJmfIYJvJZf0Q//QxYzdEan9Ice
AeJ7PXo=
          "]]},
        RowBox[{"-", "0.0555`"}]],
       Annotation[#, -0.0555, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwN0EcyRFEUgOEj041uoeXUcs4sQJjL5iaMUCWO7cAmpD3ImYElWInwDb76
69x37ntVL7uxs7SdExG7LJRHnFdHXHDGFZe8Oz+tiNhPRXyZD/TQfMCiZ3mZ
iHxyKaSAazsJTVJECcUs2S/TckpJk+LGfrVmqKCKSpbt12odNTRQz639Zr3X
Rr3TJl2x36qP5hZ90jZ90A590aw+a7uu2u/SN3On9tDNq3lAB+mlnz7W7A/r
CEOMMcqn/SmdZpxJJli3f+Q/HTOYjpjx/2b5cJ6odE/nzPN8V0XsubPpbF9/
nf3xQ445OPGdpHulZLxzgi370+Z/TJ4uYg==
          "]]},
        RowBox[{"-", "0.0592`"}]],
       Annotation[#, -0.0592, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0VkyXHEcBeCfkBhinkpUqUo2kOCNN1Wq8ErZgAXwYg95MCaabmOCYgc8
0IZl2IB5jFkTfP3w1enzv6du3Xv7W99Ad39ORAzytSxirSZinUR1xCQNzn5U
RqTLI/adH3JAr/Pmqogm5l1v1ev9/mnXJdP6jPNZ5litiEiVRlS657TeadPB
rns9O3uhnAqSdsv2U7JMLyXDEzv2D/KRYkqyz2m3ZJ+Qn/Ui7rlj2/5G3lJA
IRN2i/a/Zb7+iWv+sWV/Ka/I4yO/7P7aj8tc/QMXnJO2P5VnBDmM2f2xH5Vv
3vGVE+fHbGa/oTzKvrPz/4zYLdgPy2c9w4Hr+2zY1+lfSPpuKYbsGvWe7H8j
v7Ni26K310bs2bTp7zCSSlE=
          "]]},
        RowBox[{"-", "0.0629`"}]],
       Annotation[#, -0.0629, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0TkzX2EcxfGfnQYJoxNbbH+78AIsMTrD6BkFqSK1QWJXM3QKhta+hMRr
UJohY8/78LnFme89zzm/5977PBVjk0Pf0yJiirIKIkYLIzpKIt7yI24+RKxZ
G+DXcZD6PQ9hHm3JN/RHrG1iKc0XRezLHoojDrCLn8b0jxGf5K1Ya66OyvgF
+aH8Uf8Iu/kZzNArl7dhvW6KKvhF+bH8Sf8Ee/hZzNSrlLdjg24jVfFL8lP5
s/4Z9vI/MUvvs/wLNuk2UzW/LD+Xv+hf4Ff+F2br1SRngy26rVTLr8h/y1/1
L7GPn8McvTp5J7bptlM9v5qch/O60nkz8wf3ZLuUq5tKviH5B8zDW7pL9jHz
35308tv8KL+Dw+b/2ueaxmmCvtEP6//k98n9mCnjc72jCd8BP/M72A==
          "]]},
        RowBox[{"-", "0.0666`"}]],
       Annotation[#, -0.0666, "Tooltip"]& ], {}, {}}}], 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdVX801WcYpw7ZbUK02pRi6/i11hwTW8c+jcTNklRkUUY1mSzFEW5JJVId
w10uRYqKyI/8uCy8kmJcWtO6Sm7kys9J6cei1brX9/mn55x77vc97/s+7/N8
nuf5fAx9f3HbOkVFRSX83U/xP2mjoK8Cx+4IvYLHSLIrLd6Rv5p5JdQ9N/F6
DLUTM20a761nPGnWTlveY7Se3m89ItvIKgxihtZUjKB5XZh+/BMfpqd0MAJN
YVNT7/NtLNhWceMfuLbNi/huPJC1SBQ2DP6RCqFLVDAzVT4wjEET30BBdAhT
eHvnEkcbpy+/eDCMKaJx7B7E5/5l+u0xEUzpznYQLdM2j6nF7WVpqQobgHjt
od6PHfazFyYKh/2I+9Tz1pc3opmbMsB+bBhbVO3oeJBN5tcH02tTcjc1HuLy
6cOrJKkwlH+YbVPaI5gUnRtObYpl154rHPbCo2X38hrnI2y+MsBeyK2XaoxK
4lmkMmE5yqZpP1N3OcaU7qQ9MI4f2q198zizVFoPOoNv679dncCUcCY8RFVF
P/P1/5UNK9Pths2DngsSk0TmNJkwdNe2T9gPJLJshbusLnxQWJHck5vEJqvU
BeO+iEQWkMzhL8O8s/p5MnMhh3cndL2r6v8bFnL43kebY4y75NJvHJ738NVh
4bhm0AkOv7vIGp9TvWVRCodXO1qctdZJh1I4fKTgv9Cwsr8o4vD4G9GvQp80
bk/l8r+NmGktBZHmaVy+bdAJj3YKHU3j8ruFORP3PzGqOMnl8yfMw81V+yJP
cfG3IsvVx/muZToXrwQdsiD+jPZ0Lr4/cKzVXpzjk8HF04ApZnO/M7uewb1f
j3vtS+5oTWRw711Fkd9MufvLDM5/DUSmbuolBRmcv0qIDW7uCZxF90uRc40n
FnyWzp0vwIUV9obeDSe587k4qPqgMSUvjTufjbwgmGnNTeXOZ+D09jaj4YUp
3L4Qv2tqqdvXCdn4voCqkOQ4DAxldO/6MZG9zn3y1yrf/bCQObjMsjvOrXci
3j6RfyY2hlv7wIRldlW/ieLWrphaLmva4b6HWy+DdeBG4d0Fu5nKe0b7XzSY
a9ua+oHuG/MFyc5xQSD/LpV1V7zqw0Dv/yyEafaKA6D4HqoL9s0xPAaK3/7p
g5Yt/CRQfm5R1XUHJlJB+XdYGV6y6M8E4TPf+nL3aY1sEH7p8byjy/LOg/Bd
tqK3uehFDgh/n59e7YyS54PqI2y2upK9oAhUv0dZdxz0E4tB9a3a6Hm2RHIZ
VH9+ZaZffkcJqD9MCr+ZPVRbCuofET/crTmyDNRfM94urxVoloP6jy8TrtIT
lIP6U08iOmDVWg7q34bgE04beGJQf//r45dze4kY1P+dcq/FTz3EoPngebe1
aO0Qg+ZHJAy2k4aIQfO1cqp8dOTdPs1fwkr3wTfuYtB82uVnrXZdLAbNr9Mb
n67Kl+Wg+c4Lja3kF1P8Mhg800nneZeD+MGzfc/6D1+Xgfgj3sOzQy+hDMQv
TfcNAxw/KgPxT9PR72sLE0tB/PRQLLVZo1oK4q9dBVX4emsJiN9GjP0EomrC
Xw7J0rHZ86dfBvGj9UKt8TiXYhB/qt1YXyuKKwLxq4GKRNsopBDEv2bCLn+j
hksgfi4Nrv9BwMsH8XekZUDNym8vgvg95Wq1tMw3B8T/ez1yHQbCzoP0YZVU
13v2vmyQfqRvatXJCD0L0hcNtXP+vZsyQfrjXVE9UhNyCqRP122idHk6IpB+
deZstuQlJ4P0rSRs6IzR2HGQ/o1bnDKcHnsIpI+x1jdyLQRRIP2Ml5tHC9tD
QfqqWtNRFezmj/f1938r+8dg
     "]]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  ImageSize->504,
  PlotRange->{{0.95, 1.05}, {-0.03, 0.03}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 313},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->140390379]
}, Open  ]],

Cell["\<\
Streamlines near the airfoil surface show no waviness, there are no \
streamlines interior to the airfoil and the flow leaves the trailing edge \
smoothly, satisfying the Kutta condition.\
\>", "ExampleText",
 CellID->667775740]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[{
 Cell[BoxData[
  StyleBox[
   ButtonBox["PMDoubletConstantDirichlet",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/PMDoubletConstantDirichlet"],
   FontFamily->"Verdana"]], "InlineFormula"],
 " . ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["PMDoubletQuadraticDirichlet",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/PMDoubletQuadraticDirichlet"],
   FontFamily->"Verdana"]], "InlineFormula"],
 " . ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["ForceAndMoment",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/ForceAndMoment"],
   FontFamily->"Verdana"]], "InlineFormula"]
}], "SeeAlso",
 CellID->286240319],

Cell[TextData[{
 Cell[BoxData[
  StyleBox[
   ButtonBox["PlotSurfacePressureCoefficient",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/PlotSurfacePressureCoefficient"],
   FontFamily->"Verdana"]], "InlineFormula"],
 " . ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["FitAirfoil",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/FitAirfoil"],
   FontFamily->"Verdana"]], "InlineFormula"]
}], "SeeAlso",
 CellID->143678339]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["3.1 Introduction To Panel Methods",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods"]], "Tutorials",
 CellID->341631938],

Cell[TextData[ButtonBox["3.2 Validation Example",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/ValidationExample"]], "Tutorials",
 CellID->499723042],

Cell[TextData[ButtonBox["3.3 Implementation",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/Implementation"]], "Tutorials",
 CellID->336290191],

Cell[TextData[ButtonBox["3.5 Convergence and Timing",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming"]], "Tutorials",
 CellID->351529306],

Cell[TextData[ButtonBox["3.6 Aerodynamic Coefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicCoefficients"]], "Tutorials",
 CellID->72778609]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["PanelMethods",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/PanelMethods"]], "MoreAbout",
 CellID->347491419]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"PMVortexLinearNeumann - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 2, 39.3684000}", "context" -> 
    "Aerodynamics`PanelMethods`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Package Paclet Symbol", "language" -> "en", "paclet" -> 
    "Aerodynamics Package", "status" -> "None", "summary" -> 
    "PMVortexLinearNeumann[r_p, \\[Alpha]] implements a panel method using \
vortex panels of linearly varying strength and external Neumann boundary \
conditions. Panel endpoints describing the discretized airfoil are located at \
r_p and the angle of attack of the airfoil is \\[Alpha]. The function returns \
two lists of strength parameters for the circulation density function, \
locations of collocation points and surface speeds at collocation points.", 
    "synonyms" -> {}, "title" -> "PMVortexLinearNeumann", "type" -> "Symbol", 
    "uri" -> "Aerodynamics/ref/PMVortexLinearNeumann"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[12594, 392, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->657232508]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 275239, 5611}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[614, 21, 76, 1, 70, "SymbolColorBar"],
Cell[693, 24, 191, 5, 70, "LinkTrail"],
Cell[887, 31, 3872, 86, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4784, 121, 64, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4851, 124, 1078, 27, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5966, 156, 578, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->3053013],
Cell[6547, 175, 250, 8, 70, "Notes",
 CellID->308979890],
Cell[6800, 185, 1430, 51, 70, "Notes",
 CellID->312136876],
Cell[8233, 238, 1213, 44, 70, "Notes",
 CellID->342196229],
Cell[9449, 284, 742, 21, 70, "Notes",
 CellID->400035615],
Cell[10194, 307, 624, 20, 70, "Notes",
 CellID->299200057],
Cell[10821, 329, 598, 18, 70, "Notes",
 CellID->181230849],
Cell[11422, 349, 449, 16, 70, "Notes",
 CellID->71671648],
Cell[11874, 367, 52, 0, 70, "Notes"],
Cell[11929, 369, 551, 15, 70, "3ColumnTableMod"],
Cell[12483, 386, 74, 1, 70, "Notes",
 CellID->918694056]
}, Closed]],
Cell[CellGroupData[{
Cell[12594, 392, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->657232508],
Cell[CellGroupData[{
Cell[13080, 409, 146, 5, 70, "ExampleSection",
 CellID->6004980],
Cell[13229, 416, 132, 6, 70, "ExampleText",
 CellID->395972740],
Cell[13364, 424, 99, 3, 70, "Input",
 CellID->76286106],
Cell[13466, 429, 157, 4, 70, "ExampleText",
 CellID->48545141],
Cell[13626, 435, 216, 8, 70, "Input",
 CellID->43842758],
Cell[13845, 445, 135, 4, 70, "ExampleText",
 CellID->136872357],
Cell[CellGroupData[{
Cell[14005, 453, 111, 3, 70, "Input",
 CellID->59487574],
Cell[14119, 458, 158, 5, 36, "Output",
 CellID->79539540]
}, Open  ]],
Cell[14292, 466, 86, 1, 70, "ExampleText",
 CellID->711853039],
Cell[14381, 469, 147, 5, 70, "Input",
 CellID->847786885],
Cell[14531, 476, 598, 20, 70, "ExampleText",
 CellID->20624687],
Cell[15132, 498, 265, 8, 70, "Input",
 CellID->53812608],
Cell[15400, 508, 769, 15, 70, "ExampleText",
 CellID->10578989],
Cell[16172, 525, 617, 19, 70, "Input",
 CellID->133172956],
Cell[16792, 546, 471, 8, 70, "ExampleText",
 CellID->76299467],
Cell[CellGroupData[{
Cell[17288, 558, 1206, 32, 70, "Input",
 CellID->808226483],
Cell[18497, 592, 54505, 957, 70, "Output",
 CellID->789492561]
}, Open  ]],
Cell[73017, 1552, 273, 5, 70, "ExampleText",
 CellID->14395635],
Cell[73293, 1559, 135, 4, 70, "ExampleText",
 CellID->80454523],
Cell[CellGroupData[{
Cell[73453, 1567, 277, 9, 70, "Input",
 CellID->499718542],
Cell[73733, 1578, 258, 8, 36, "Output",
 CellID->97872131]
}, Open  ]],
Cell[74006, 1589, 71, 1, 70, "ExampleText",
 CellID->8410041],
Cell[CellGroupData[{
Cell[74102, 1594, 140, 5, 70, "Input",
 CellID->342608962],
Cell[74245, 1601, 177, 5, 36, "Output",
 CellID->119607110]
}, Open  ]],
Cell[74437, 1609, 121, 3, 70, "ExampleText",
 CellID->753558053],
Cell[74561, 1614, 165, 6, 70, "Input",
 CellID->435840926],
Cell[74729, 1622, 331, 6, 70, "ExampleText",
 CellID->139054009],
Cell[CellGroupData[{
Cell[75085, 1632, 1369, 39, 101, "Input",
 CellID->671472236],
Cell[76457, 1673, 6959, 122, 345, "Output",
 CellID->156290816]
}, Open  ]],
Cell[83431, 1798, 422, 15, 70, "ExampleText",
 CellID->571494656],
Cell[CellGroupData[{
Cell[83878, 1817, 612, 19, 70, "Input",
 CellID->42630738],
Cell[84493, 1838, 364, 13, 36, "Output",
 CellID->65565644]
}, Open  ]],
Cell[84872, 1854, 399, 7, 70, "ExampleText",
 CellID->24035690],
Cell[CellGroupData[{
Cell[85296, 1865, 331, 10, 70, "Input",
 CellID->34907031],
Cell[85630, 1877, 338, 9, 36, "Output",
 CellID->474611700]
}, Open  ]],
Cell[85983, 1889, 330, 6, 70, "ExampleText",
 CellID->4111782],
Cell[CellGroupData[{
Cell[86338, 1899, 236, 8, 70, "Input",
 CellID->274981366],
Cell[86577, 1909, 259, 8, 36, "Output",
 CellID->96230897]
}, Open  ]],
Cell[86851, 1920, 236, 5, 70, "ExampleText",
 CellID->328772922],
Cell[CellGroupData[{
Cell[87112, 1929, 353, 12, 70, "Input",
 CellID->762946865],
Cell[87468, 1943, 309, 10, 36, "Output",
 CellID->393770335]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[87826, 1959, 221, 7, 70, "ExampleSection",
 CellID->1293636265],
Cell[CellGroupData[{
Cell[88072, 1970, 251, 7, 70, "ExampleSubsection",
 CellID->421192202],
Cell[88326, 1979, 291, 7, 70, "ExampleText",
 CellID->5610347],
Cell[88620, 1988, 154, 5, 70, "Input",
 CellID->559405563],
Cell[88777, 1995, 159, 4, 70, "ExampleText",
 CellID->299806311],
Cell[CellGroupData[{
Cell[88961, 2003, 555, 18, 70, "Input",
 CellID->249332617],
Cell[89519, 2023, 5842, 103, 70, "Output",
 CellID->87644956]
}, Open  ]],
Cell[95376, 2129, 85, 1, 70, "ExampleText",
 CellID->176584194],
Cell[95464, 2132, 438, 13, 70, "Input",
 CellID->343427282],
Cell[95905, 2147, 137, 4, 70, "ExampleText",
 CellID->126580234],
Cell[CellGroupData[{
Cell[96067, 2155, 310, 10, 70, "Input",
 CellID->274596420],
Cell[96380, 2167, 6005, 106, 70, "Output",
 CellID->14906467]
}, Open  ]],
Cell[102400, 2276, 157, 4, 70, "ExampleText",
 CellID->33075010],
Cell[CellGroupData[{
Cell[102582, 2284, 512, 15, 70, "Input",
 CellID->252090154],
Cell[103097, 2301, 260, 8, 36, "Output",
 CellID->192730869]
}, Open  ]],
Cell[103372, 2312, 281, 6, 70, "ExampleText",
 CellID->511124957],
Cell[CellGroupData[{
Cell[103678, 2322, 1071, 32, 70, "Input",
 CellID->59705953],
Cell[104752, 2356, 4774, 91, 345, "Output",
 CellID->92827430]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[109587, 2454, 223, 7, 70, "ExampleSection",
 CellID->2061341341],
Cell[CellGroupData[{
Cell[109835, 2465, 229, 7, 70, "ExampleSubsection",
 CellID->1295379749],
Cell[110067, 2474, 524, 9, 70, "ExampleText",
 CellID->120830056],
Cell[110594, 2485, 649, 20, 70, "Input",
 CellID->210128799],
Cell[111246, 2507, 140, 4, 70, "ExampleText",
 CellID->112154987],
Cell[CellGroupData[{
Cell[111411, 2515, 833, 26, 70, "Input",
 CellID->295212498],
Cell[112247, 2543, 321, 10, 39, "Output",
 CellID->622054557]
}, Open  ]],
Cell[112583, 2556, 532, 10, 70, "ExampleText",
 CellID->3193481],
Cell[113118, 2568, 252, 5, 70, "ExampleText",
 CellID->79242491],
Cell[CellGroupData[{
Cell[113395, 2577, 1189, 34, 70, "Input",
 CellID->23699121],
Cell[114587, 2613, 5557, 102, 177, "Output",
 CellID->522204541]
}, Open  ]],
Cell[120159, 2718, 657, 10, 70, "ExampleText",
 CellID->189149207]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[120865, 2734, 231, 7, 70, "ExampleSection",
 CellID->1305812373],
Cell[121099, 2743, 345, 6, 70, "ExampleText",
 CellID->416625656],
Cell[121447, 2751, 205, 4, 70, "ExampleText",
 CellID->32738591],
Cell[CellGroupData[{
Cell[121677, 2759, 312, 10, 70, "Input",
 CellID->112529997],
Cell[121992, 2771, 5180, 92, 70, "Output",
 CellID->112729628]
}, Open  ]],
Cell[127187, 2866, 150, 4, 70, "ExampleText",
 CellID->390242666],
Cell[CellGroupData[{
Cell[127362, 2874, 212, 7, 70, "Input",
 CellID->238007975],
Cell[127577, 2883, 345, 12, 36, "Output",
 CellID->266256017]
}, Open  ]],
Cell[127937, 2898, 151, 4, 70, "ExampleText",
 CellID->25666953],
Cell[CellGroupData[{
Cell[128113, 2906, 605, 18, 70, "Input",
 CellID->592619],
Cell[128721, 2926, 259, 8, 36, "Output",
 CellID->93647693]
}, Open  ]],
Cell[128995, 2937, 114, 3, 70, "ExampleText",
 CellID->46735897],
Cell[129112, 2942, 156, 4, 70, "ExampleText",
 CellID->492689119],
Cell[CellGroupData[{
Cell[129293, 2950, 1734, 46, 70, "Input",
 CellID->266881112],
Cell[131030, 2998, 67542, 1161, 70, "Output",
 CellID->920434690]
}, Open  ]],
Cell[198587, 4162, 238, 5, 70, "ExampleText",
 CellID->482308682],
Cell[198828, 4169, 155, 4, 70, "ExampleText",
 CellID->254018838],
Cell[CellGroupData[{
Cell[199008, 4177, 1005, 31, 70, "Input",
 CellID->230503503],
Cell[200016, 4210, 260, 8, 36, "Output",
 CellID->156850911]
}, Open  ]],
Cell[200291, 4221, 168, 4, 70, "ExampleText",
 CellID->88671999],
Cell[200462, 4227, 86, 1, 70, "ExampleText",
 CellID->437823294],
Cell[CellGroupData[{
Cell[200573, 4232, 1447, 37, 70, "Input",
 CellID->240459914],
Cell[202023, 4271, 67569, 1162, 70, "Output",
 CellID->140390379]
}, Open  ]],
Cell[269607, 5436, 238, 5, 70, "ExampleText",
 CellID->667775740]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[269894, 5447, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[270314, 5461, 673, 22, 70, "SeeAlso",
 CellID->286240319],
Cell[270990, 5485, 453, 15, 70, "SeeAlso",
 CellID->143678339]
}, Open  ]],
Cell[CellGroupData[{
Cell[271480, 5505, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[271902, 5519, 190, 4, 70, "Tutorials",
 CellID->341631938],
Cell[272095, 5525, 167, 3, 70, "Tutorials",
 CellID->499723042],
Cell[272265, 5530, 160, 3, 70, "Tutorials",
 CellID->336290191],
Cell[272428, 5535, 177, 4, 70, "Tutorials",
 CellID->351529306],
Cell[272608, 5541, 181, 4, 70, "Tutorials",
 CellID->72778609]
}, Open  ]],
Cell[CellGroupData[{
Cell[272826, 5550, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[273248, 5564, 149, 3, 70, "MoreAbout",
 CellID->347491419]
}, Open  ]],
Cell[273412, 5570, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

