(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     23674,        662]
NotebookOptionsPosition[     20161,        547]
NotebookOutlinePosition[     21774,        593]
CellTagsIndexPosition[     21689,        588]
WindowTitle->PlotAirfoil - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Graphics\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/Graphics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PlotSurfacePressureCoefficient\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PlotSurfacePressureCoefficient"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Graphics\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/Graphics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/PlotAirfoil\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/PlotAirfoil"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
PlotAirfoil.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/PlotAirfoil", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["PlotAirfoil", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"PlotAirfoil", "[", 
       SubscriptBox[
        StyleBox["r", "TI"], 
        StyleBox["p", "TI"]], "]"}]], "InlineFormula"],
     "\[LineSeparator]plots the airfoil defined by the collection of vectors ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->136827898],

Cell["\<\
Collections of vectors are of the data type CartesianVectors.\
\>", "Notes",
 CellID->40345418],

Cell["\<\
PlotAirfoil uses ListPlot and alters some of its options to provide a plot of \
a discretized airfoil; all options of ListPlot are accepted by PlotAirfoil.\
\>", "Notes",
 CellID->330500012],

Cell["New in 1.1", "Notes",
 CellID->638380636]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->672773266],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(2)", "ExampleCount"]
}], "ExampleSection",
 CellID->175754600],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->53294582],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->806014801],

Cell["\<\
Compute the collection of vectors specifying panel endpoints for a NACA-4412 \
airfoil with 50 panels using half-cosine spacing to concentrate points near \
the leading edge.  Show the discretized airfoil using PlotAirfoil.\
\>", "ExampleText",
 CellID->517542368],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"NACA4DigitAirfoil", "[", 
     RowBox[{"4412", ",", 
      RowBox[{"NDiscretizeUnitSegment", "[", 
       RowBox[{"25", ",", 
        RowBox[{"Layout", "\[Rule]", "HalfCosineSpacing"}]}], "]"}]}], "]"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"PlotAirfoil", "[", "rP", "]"}]}], "Input",
 CellLabel->"In[2]:=",
 CellID->139180420],

Cell[BoxData[
 GraphicsBox[
  {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwBQQO+/CFib1JlAgAAADMAAAACAAAAi1jhxKL+7z+Vxc/LdnZUv402Lv0o
9O0/CBkXwYV+XL8qYviAKO7rP+RiLSjCqmS/oa2Zimfu6T8gyY+E8MltvzWx
rFqr9uc/xP149EbTdL+cWiBCtAjmP+SvFgucBHy/ErAjqzkm5D8Y6iIOoh+C
v30XcITmUOI/bGox5t2Xhr+YY2ZdVorgP6AfWe0VN4u/lNANyiao3T90mJOM
YcKPv7bFbLsqX9o/JLomNp79kb9T90dxl0bXP47/En1AApS/NH4v1N9Z1D8w
njSusjeWvzn1/Tqml9E/kFqAf7NtmL+RUDugagPOP3Sb6o12cpq/pfvlf6wz
yT/RLAZE2Bicv07atYauw8Q/4iqj4fw4nb/tq0tgHbfAP7woHFaVsJ2/325M
wmgjuj+ibWVW1WKdvyx71eBorrM/Rt3zCio4nL/nsKa/US2sP2zFbQ7DHZq/
4VrqTLPGoj8El6VhAAWXv65CUmUEbZY/O680qNfikr/iOdzVWiSGP++hbliD
Xou/YFMyCAOUbD+cQKgV/JJ9vwAAAAAAAAAAAAAAAAAAAAAAhS166QM+Pw3D
0PFLZoA/PIlu0/VPdD+HoMaz8OGQPyR71iIhs4s/X/6AOwn7mT8+kiMiEsqa
P/FYM3T2sKE/Oa/O1+vwpT8SQJqe8HemP4TWNEDtRbA/dRBc4qI6qz9xOPJv
GZe2P+2x0iua3a8/l6JMV4DmvT85tfgiVyGyP4KxKH5xFsM/OvXwBQEltD8b
X1qjlrDHPz043veE6bU/H1DpRDq8zD9ovK8axl63P28qt7WBGdE/k5gOOEd1
uD+AAYzX4gbUP5QZcWSSHrk/wQHkk0wi1z8KH/ixjE25P9LQvTJJZNo/s9wh
kxz4uD8sgpFF4MHdPwqu1rUzN7g/GPPZRZyg4D/hJvmfIha3P+dmarIhb+I/
3ir7XtGTtT/SpjMMbErkP45USofOr7M/FLyvhlww5j8xG5lBDmqxP/1KRajL
Hug/DsKkMFaFrT8hleRCihPqP+5CrGpec6c/5u820GEM7D9nxMh12J2gP5Ok
2XsUB+4/GAdfbZsHkj+5U4+drgDwPwfGz8t2dlQ/Qw6SAw==
    "]]},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2000000000000004`, 
      FormBox["1.2000000000000004`", TraditionalForm]}, {1.4000000000000001`, 
      FormBox["1.4000000000000001`", TraditionalForm]}, {1.6, 
      FormBox["1.6`", TraditionalForm]}}, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}, None, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}},
  ImageSize->{504., 180.},
  PlotRange->{{-0.05, 1.05}, {-0.1, 0.201}},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->199722988]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell["\t", "ExampleDelimiter"],
  $Line = 0; Null]], "ExampleDelimiter",
 CellID->65036767],

Cell["\<\
For airfoils specified by data files, you may want to display the data points \
without connecting them.  Any option of ListPlot may be used to customize the \
graph.\
\>", "ExampleText",
 CellID->74092207],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rE205", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<Aerodynamics\\\\Airfoil Data\\\\e205.dat\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"PlotAirfoil", "[", 
  RowBox[{"rE205", ",", 
   RowBox[{"Joined", "\[Rule]", "False"}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Blue", ",", 
      RowBox[{"PointSize", "[", "Medium", "]"}]}], "}"}]}]}], "]"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->317289154],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3b3O49QWBuAcTg8UiBJBS0MLXwMSxdCBRkKiQhqKTIVo6SgoKMldcANH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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->92872497]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[StyleBox[ButtonBox["PlotSurfacePressureCoefficient",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/ref/PlotSurfacePressureCoefficient"],
 FontFamily->"Verdana"]], "SeeAlso",
 CellID->929782353]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["Graphics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/Graphics"]], "Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["Graphics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Graphics"]], "MoreAbout",
 CellID->119295073]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"PlotAirfoil - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 2, 6.7800000}", "context" -> 
    "Aerodynamics`Graphics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Package Paclet Symbol", "language" -> "en", "paclet" -> 
    "Aerodynamics Package", "status" -> "None", "summary" -> 
    "PlotAirfoil[r_p] plots the airfoil defined by the collection of vectors \
r_p.", "synonyms" -> {}, "title" -> "PlotAirfoil", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/PlotAirfoil"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[5579, 168, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->672773266]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 21546, 581}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[604, 21, 76, 1, 70, "SymbolColorBar"],
Cell[683, 24, 191, 5, 70, "LinkTrail"],
Cell[877, 31, 3031, 71, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3933, 106, 54, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[3990, 109, 574, 18, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4601, 132, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->136827898],
Cell[5184, 151, 105, 3, 70, "Notes",
 CellID->40345418],
Cell[5292, 156, 200, 4, 70, "Notes",
 CellID->330500012],
Cell[5495, 162, 47, 1, 70, "Notes",
 CellID->638380636]
}, Closed]],
Cell[CellGroupData[{
Cell[5579, 168, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->672773266],
Cell[CellGroupData[{
Cell[6065, 185, 148, 5, 70, "ExampleSection",
 CellID->175754600],
Cell[6216, 192, 131, 6, 70, "ExampleText",
 CellID->53294582],
Cell[6350, 200, 100, 3, 70, "Input",
 CellID->806014801],
Cell[6453, 205, 274, 5, 70, "ExampleText",
 CellID->517542368],
Cell[CellGroupData[{
Cell[6752, 214, 445, 12, 70, "Input",
 CellID->139180420],
Cell[7200, 228, 6100, 135, 201, "Output",
 CellID->199722988]
}, Open  ]],
Cell[CellGroupData[{
Cell[13337, 368, 124, 3, 70, "ExampleDelimiter",
 CellID->65036767],
Cell[13464, 373, 216, 5, 70, "ExampleText",
 CellID->74092207],
Cell[CellGroupData[{
Cell[13705, 382, 500, 15, 47, "Input",
 CellID->317289154],
Cell[14208, 399, 4000, 73, 201, "Output",
 CellID->92872497]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[18281, 480, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[18701, 494, 215, 4, 70, "SeeAlso",
 CellID->929782353]
}, Open  ]],
Cell[CellGroupData[{
Cell[18953, 503, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[19375, 517, 144, 3, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[CellGroupData[{
Cell[19556, 525, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[19978, 539, 141, 3, 70, "MoreAbout",
 CellID->119295073]
}, Open  ]],
Cell[20134, 545, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

