(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     50912,       1274]
NotebookOptionsPosition[     45512,       1108]
NotebookOutlinePosition[     47833,       1163]
CellTagsIndexPosition[     47747,       1158]
WindowTitle->PlotConvergence - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"3.4 Convergence and Timing\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/ConvergenceAndTiming"], "\<\"A2 Graphics\
\"\>":>Documentation`HelpLookup["paclet:Aerodynamics/tutorial/Graphics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Graphics\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/Graphics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/PlotConvergence\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/PlotConvergence"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
PlotConvergence.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/PlotConvergence", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["PlotConvergence", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"PlotConvergence", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["n", "p"], "TI"], ",", 
        StyleBox["c", "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]plots a graph showing convergence properties for an \
aerodynamic coefficient.  The list of numbers of panel describing the \
discretized airfoil is denoted by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["n", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     " and the list of coefficients is denoted by ",
     Cell[BoxData[
      StyleBox["c", "TI"]], "InlineFormula"],
     ".  The coefficient is plotted versus the reciprocal of the number of \
panels, and a straight line is fit to the resulting points.  A reference \
value for the coefficient is determined by extrapolating the straight line to \
zero, and the function returns a list of the reference coefficient and the \
graphic."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"PlotConvergence", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["n", "p"], "TI"], ",", " ", 
        StyleBox["c", "TI"], ",", 
        StyleBox["m", "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]plots a graph showing the convergence properties of an \
aerodynamic coefficient ",
     Cell[BoxData[
      StyleBox["c", "TI"]], "InlineFormula"],
     ".  Only the subset of data consisting of the ",
     Cell[BoxData[
      StyleBox["m", "TI"]], "InlineFormula"],
     " largest numbers of panels is used to determine the straight - line fit."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {None, {None}}, 
   "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->522824268],

Cell["\<\
The straight line fit to data is shown as a solid line within the range of \
data used to determine the line and as a dashed line where extrapolated.\
\>", "Notes",
 CellID->1067943069],

Cell["\<\
Data points used to determine the fit are shown in blue, those not used, in \
red.\
\>", "Notes",
 CellID->151000646],

Cell["New in 1.1   |   Last modified in 1.2", "Notes",
 CellID->246109562]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->152687988],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->109762551],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->337266530],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->207768003],

Cell["\<\
For this example use the Eppler-205 airfoil, which is specified by a file in \
the folder AirfoilData.  User-specified discretizations are based on the \
B-spline fit to the data.  \
\>", "ExampleText",
 CellID->99847148],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<Aerodynamics\\\\Airfoil Data\\\\e205.dat\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"spline", "=", 
   RowBox[{"FitAirfoil", "[", "rD", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gD", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gSpline", "=", 
   RowBox[{"ParametricPlot", "[", 
    RowBox[{
     RowBox[{"spline", "[", "t", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "0", ",", "1"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gD", ",", "gSpline"}], "]"}]}], "Input",
 CellLabel->"In[2]:=",
 CellID->524210768],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3f+PFPUdx/Fr+pf037hEueMOFAk9D4vYWmrhZAtt1Wo0MagRalsIQrrx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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->912945269]
}, Open  ]],

Cell["\<\
Specify an angle of attack for the airfoil and a list of discretization \
numbers.\
\>", "ExampleText",
 CellID->53729757],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Alpha]", "=", 
   RowBox[{"6.", " ", "Degree"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"n", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"Range", "[", 
      RowBox[{"4", ",", "20", ",", "4"}], "]"}], ",", 
     RowBox[{"Range", "[", 
      RowBox[{"25", ",", "125", ",", "25"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"layout", "\[Rule]", "ConstantSpacing"}], ";"}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->222327051],

Cell["\<\
Compute collections of vectors locating panel endpoints for each \
discretization number in the list, and display the list of number-of-panels \
for each discretized airfoil.\
\>", "ExampleText",
 CellID->3599668],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"AirfoilSurfacePoints", "[", 
       RowBox[{"spline", ",", 
        RowBox[{"NDiscretizeUnitSegment", "[", 
         RowBox[{"#", ",", 
          RowBox[{"Layout", "\[Rule]", "ConstantSpacing"}]}], "]"}]}], "]"}], 
      "&"}], ",", "n"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"nP", "=", 
  RowBox[{"Length", "/@", 
   RowBox[{"PanelLengths", "/@", "rP"}]}]}]}], "Input",
 CellLabel->"In[4]:=",
 CellID->146492676],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "8", ",", "16", ",", "24", ",", "32", ",", "40", ",", "50", ",", "100", ",",
    "150", ",", "200", ",", "250"}], "}"}]], "Output",
 ImageSize->{278, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->525069565]
}, Open  ]],

Cell["\<\
Solve PMDoubletQuadraticDirichlet for each discretized airfoil, and compute \
lift and moment coefficients, grouping like coefficients together.\
\>", "ExampleText",
 CellID->775435343],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c", ",", "aW", ",", "rC", ",", "vMagC"}], 
    "}"}], "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"PMDoubletQuadraticDirichlet", "[", 
        RowBox[{"#", ",", "\[Alpha]"}], "]"}], "&"}], ",", "rP"}], "]"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl", ",", "cm"}], "}"}], "=", 
  RowBox[{"Transpose", "[", 
   RowBox[{"Apply", "[", 
    RowBox[{"ForceAndMoment", ",", 
     RowBox[{"Thread", "[", 
      RowBox[{"{", 
       RowBox[{
       "\"\<Doublet\>\"", ",", "a", ",", "b", ",", "c", ",", "rP", ",", 
        "\[Alpha]", ",", "rC"}], "}"}], "]"}], ",", 
     RowBox[{"{", "1", "}"}]}], "]"}], "]"}]}]}], "Input",
 CellLabel->"In[5]:=",
 CellID->53143172],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "0.6999476879013269`", ",", "0.8426318336738858`", ",", 
     "0.8900821246057484`", ",", "0.9137266434257205`", ",", 
     "0.927992857331362`", ",", "0.9394680831796303`", ",", 
     "0.9628313052383546`", ",", "0.9709763188930984`", ",", 
     "0.9751254189177794`", ",", "0.9776577475372024`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.31241801515135503`"}], ",", 
     RowBox[{"-", "0.31635477336392803`"}], ",", 
     RowBox[{"-", "0.31404311985534317`"}], ",", 
     RowBox[{"-", "0.3120918433999881`"}], ",", 
     RowBox[{"-", "0.3106881727576358`"}], ",", 
     RowBox[{"-", "0.3094659044525401`"}], ",", 
     RowBox[{"-", "0.3065313076045568`"}], ",", 
     RowBox[{"-", "0.30549395798071477`"}], ",", 
     RowBox[{"-", "0.3049914675560532`"}], ",", 
     RowBox[{"-", "0.3046868087631953`"}]}], "}"}]}], "}"}]], "Output",
 ImageSize->{530, 50},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->291300565]
}, Open  ]],

Cell["\<\
Note that the lengths of lists of numbers of panels, and coefficients are the \
same.\
\>", "ExampleText",
 CellID->2060996],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "/@", 
  RowBox[{"{", 
   RowBox[{"nP", ",", "cl", ",", "cm"}], "}"}]}]], "Input",
 CellLabel->"In[6]:=",
 CellID->57188429],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"10", ",", "10", ",", "10"}], "}"}]], "Output",
 ImageSize->{82, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->32249589]
}, Open  ]],

Cell["\<\
Use the last five discretization numbers to compute reference values for lift \
and pitching-moment coefficients and plot the convergence graphs.  \
Extrapolation beyond the range of data determining the line is shown as \
dashed.\
\>", "ExampleText",
 CellID->388479374],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"clRef", ",", "gCl"}], "}"}], "=", 
  RowBox[{"PlotConvergence", "[", 
   RowBox[{
    RowBox[{"Take", "[", 
     RowBox[{"nP", ",", 
      RowBox[{"-", "5"}]}], "]"}], ",", 
    RowBox[{"Take", "[", 
     RowBox[{"cl", ",", 
      RowBox[{"-", "5"}]}], "]"}], ",", 
    RowBox[{"FrameLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
      "\"\<reciprocal of number of panels\>\"", ",", 
       "\"\<\!\(\*Cell[TextData[Cell[BoxData[
FormBox[
SubscriptBox[\"C\", \"\[ScriptL]\"], TraditionalForm]], \"InlineFormula\"]], \
\"InlineFormula\"]\)\>\""}], "}"}]}]}], "]"}]}]], "Input",
 CellLabel->"In[7]:=",
 CellID->640039878],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.9869604951178604`", ",", 
   GraphicsBox[{
     {RGBColor[0, 0, 1], AbsolutePointSize[5], 
      PointBox[{{0.004, 0.9776577475372024}, {0.005, 0.9751254189177794}, {
       0.006666666666666667, 0.9709763188930984}, {0.01, 
       0.9628313052383546}, {0.02, 0.9394680831796303}}]}, {{}, {}, 
      {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwV0n00lHkUB3C2k8KaUuNlJRrEZKPSKEz5opIk1ehF49hTGppkM5qMl7QW
lfGSGTNPp2lZRbS1ldShRC+mUmaJqcU6q7J2lN3o9zhtL1tT7LN/3HPP5697
7/leTlyyIP4LExOTdUz93zlWcX+vi5Ngz5Zl3e6raXT7PvWU7pLgyiq9gMVY
/k10kiZJgg880aO3oTTIpUj6T5kE8pnFnfcYNwn4b9OKJKjR97fFMRZo2KZV
9RIMhB+4WbGKRq7rA/t3ZikQoOmM7UoaQ/7zwyrOp8Cf65dhFkxDoeQPVk6X
ot5ytU9cAI1U++q+T7H74T5movHm0ZjbZf7wYXUqZAsXuzh407h5qd64cUoa
9uWmt+R40FBqG+VegnTE94gthRwaEZ9NN9xozMDkgKTEWgcaj3eFsbX2B5D3
cfgVh01DNemvJbN3ZOHfZpaTGYsGX9ehnt96EA6KsLiEKTScWyzWq62z4bPT
K2qpKY0Xr6smNX79PbjqQrGHkaDOwfAp+8n3aO/+WKp+Q2Dfv75GmpODeDtb
sYQQ7F848DKIl4sLe1YKdMMEvrNfeesMudDPGtkePEigrvJwOZifh9GOZY6e
/QQvbhy6YuF/CPWKypGSboLeZNbru88OQRp1eZeok6DpWlPnFtVhPON2v294
QDDp+PNz1SFHwBUmX3DXErxkLec5DR+BuLjBamozQeusazEpJ/KxWc/Ki68n
WK6N9RmDHP42M2sW1xI4jOdO8TTIYWxQjMw5SzDS9UvGcqoAiWLn0MJKAmEA
yrL8C9E1S7hOVEZw079327HRQgz+8ZO4iSI4rB+x3V1ehEeme88uKGHmT3jl
iYOK0eg6QNnICVbeUxUHvy5GxNadx9JyCPgct1WhmqPoLVKFhB8gOG7D+zSZ
XwI4BrCXpRIMVyRsODlSgk233nw4t5egp6U1SqRUYPsOYbFcTNDf7JqZw1Hi
K9tffQw7CMoMHytf7Feirf3d4doYAtZW6zXn7iux5anqs2YzgZF9/Va0cymU
JiEFE5EES5zfWzrtK4XWrVzbG8bcK73voW8vxUmhOcszhCCK3XNnwFUF4Sj5
rpxPYGGa6TtZpoJVdnZ5ti+zv1+dzK5LhSRX185n3kwezVkKuKhxpy3lQi2X
YLRQ7tEqVeOdzu/Sjy4Emuq5JUs71PjtyYz7Ux2ZvC22ddx2olA7drxgyIYg
miwQLJxDgbstcLaOseNjs75THApV2qG6i4yrT9QbctwoHFMt6pMxvsqdblzh
SSGL1+5uzvj30Afz2nwprEkfv+vFJnDJ883XR1AwTIjGZTOYf0r40io4kkKs
2FIVw/h8uEF1eT2FXv3luUGMeTNKK1QCCrrK8QhzxqGVow2boinUrdCUlVkT
JN4+PdS3k8LB/Ha/lukE3qczE9fEU3hPp3RUM/7nyMax6wkUJNH22wsYZ0VO
GH/YTUE0Lz5fwLjkqXBmbDKF8PbxnqFpBALtohMPJRTuLa7ZrWNsVzPVOXAf
hcDytZ8vMj6VdNXTOZXCom81bmmMRRuO1ilkFH7uCWyMYTyPJ1pikk7BLfD5
2iDGxI5/Q5JBoeJM0YAb4ytG65DBTAp203yk5oz/AxnMTMQ=
        "]]}}, {{}, {}, 
      {Hue[0.67, 0.6, 0.6], Dashing[{0.01, 0.01}], LineBox[CompressedData["
1:eJwV0n0w1HkcB3Ch8lTtKipJT9IDu0nrulP2HWk4YfOY2LXazXpa68eSp7ZD
HWI0ZHfyUI3JhBRd8nCnS61SI0pXpuNqr+6oTMm37lIdOfe9Pz7zmdfM54/P
Z96fVZLEgCh9PT09X1r/d7FBt62UeezqrjLkcDUEEG9qEmta+Isyzputp261
iL3npLnFf5niN7aK+t272SJrzQD/mLz8ggV1ccSALVszwv8l3GHjtJpAqLla
YKr5wBe7BK3ro77peKnDSGOIQ59qVsVRp/1bHzhXswhtSTsta8sIyniC5Ork
tVBvzDecLiXQKss46bpNuFwzsZwpIahWR47OGDuje0uu3pvjBLMWm9Ww1C7w
KFg46lFM4Bth/evh68Cs0/2THUUEp2tXTj696I4NB0vMWYUEdVmNuS/lu+Al
S/xbXkCw7srYnTPTnjhnmTHzIo8gnd+Wc63bG4qifBb/ewK+g9PnwghfpNy4
4dN8hKAirT7G47UASWZLAri5BJKiAbvmEH/8Zp0iPJZNsMWsKa2iOgA5bydL
9b4jGGvr0+fNCcKJ3Ly+EBVBYslooUdUMKq4jg9asgg4VaKkixdD8Gel4Qw3
k+BR/+ITdxaEQt2rcypIJ2hxtme7hO/D0Z//KJ0+SKDqPnJX3BWGZ0mvJ/xT
CVhHh7KnVguxmc9e06Qk+F1ZoBmPF0F7V5Jll0ygKT40on4QgWvLfhpUMQQ7
x0a8euZH4qODy/73CoKVG0xsDJojYRc81CxPIHg/6V9iKd2PLjeV8Z14gsio
25eDjSSoGt9d5BpH8zsgYhRXJXguXmFeFkNQNJXMcY+XwnqUJXkno/+wsbXC
bcEBaBw5utgoAm+ytMZKewCZrn6iLimBm8+F8BXRUThtUmm0VULz2XHV/Jmx
DO3jBqorkQRve97K0jtk2H4y+eN8McGTNz5xruJozMyYVuWJCK6dvW/ylB0D
LqfBQxdOsP2vQ2G1P8agwkq5dE8YvWfofL29MBbTNmHtd0MJPgf9U1Y4Lw69
r3aF2e4lUMSmci2uxGGQEa6vDCYYNF8ky5DFQ9CZNjUeSP8jcPOpcks5Oldf
Prk7gGAhJytnuEOOeE8L8a09BMqqVwZGCQnIdTxsby0gSDqb7Ry6UIHBIfaa
El+CgR3lUdI2BdZW3hw29SGwT3no5BWdCJ1N5nmZN82r+tRehzkMHknDK3Ve
dP6IUndqLoP30x7XH1NXRHtL5xkzYJVzR/qpV2/6rBg3ZeDbN4vbRc3rDMz7
gc3gNq9eW0u976lJK285g47ZE6OJ1GeXZJi78hhU1x3fqk8t+yIob3RmcN0t
XTjlSfd5bmdjs5WB7sn+nA/ULXUDG2a+YWDFcu59SX37K0c3LRiUpT+J6KV+
HfQqcdduBvnfrs9TUzd93TnR6sPg3DD7QjG10lqTZefH4JZqqj+P+suwe/5c
fwZ6zfeXZlDPV5450xPCINMqtVFE/TAk1dYllEF5S8TDEOqTLj4NDfsYtPt5
fRJQr9SfbC0SMviQu8zNnfrFiwfbvogYmC+fLdtG3dBTp5WLGTi2jxfyqBWN
hz11kQwE/oOXONRbSoPv+UoYJLzRDthR/wfzj0XQ
        "]]}}},
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->None,
    Frame->True,
    FrameLabel->{
      FormBox["\"reciprocal of number of panels\"", TraditionalForm], 
      FormBox[
      "\"\\!\\(\\*Cell[TextData[Cell[BoxData[\\nFormBox[\\nSubscriptBox[\\\"C\
\\\", \\\"\[ScriptL]\\\"], TraditionalForm]], \\\"InlineFormula\\\"]], \
\\\"InlineFormula\\\"]\\)\"", TraditionalForm]},
    ImageSize->504,
    PlotRange->All,
    PlotRangeClipping->True,
    RotateLabel->False]}], "}"}]], "Output",
 ImageSize->{582, 321},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->55683818]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cmRef", ",", "gCm"}], "}"}], "=", 
  RowBox[{"PlotConvergence", "[", 
   RowBox[{
    RowBox[{"Take", "[", 
     RowBox[{"nP", ",", 
      RowBox[{"-", "5"}]}], "]"}], ",", 
    RowBox[{"Take", "[", 
     RowBox[{"cm", ",", 
      RowBox[{"-", "5"}]}], "]"}], ",", 
    RowBox[{"FrameLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
      "\"\<reciprocal of number of panels\>\"", ",", 
       "\"\<\!\(\*Cell[TextData[Cell[BoxData[
FormBox[
SubscriptBox[\"C\", 
StyleBox[\"m\",
FontSlant->\"Italic\"]], TraditionalForm]], \"InlineFormula\"]], \
\"InlineFormula\"]\)\>\""}], "}"}]}]}], "]"}]}]], "Input",
 CellLabel->"In[8]:=",
 CellID->153900572],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "0.303506121924393`"}], ",", 
   GraphicsBox[{
     {RGBColor[0, 0, 1], AbsolutePointSize[5], 
      PointBox[{{0.004, -0.3046868087631953}, {0.005, -0.3049914675560532}, {
       0.006666666666666667, -0.30549395798071477`}, {
       0.01, -0.3065313076045568}, {0.02, -0.3094659044525401}}]}, {{}, {}, 
      {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwV0Xs01GkYwHGXk0pMCrGsJuSQTWxr5HL0pIQsg8kp7NEFuW27RuM6Tdu4
NmmMMfNTpuSutpVItij1GnKLYeota5Nta7qsS7ZDq62h9t0/3vOezz/v5ftY
RiWxDmlpaGgEkvX/bqkfNREYxQbUq6zewsfoIWPcnhPHhnm79D5PYsG+sMOl
h9mwqcBiZgfxTCPz7+dpbCgLSHQLJr7J8vgn/RQbMu9rK+OJWaVGmtUtbPhm
zGVRRpxt3Ws6r5MMF2fK9i4Qv3Db6FdenwxFxgl68iyMisQez6oMOFD5iVf4
fQ5Gqaa1owuRKVDCpA18yMXIZni5QlGbCs/X9Lry8zG63diiDlmaDmyu+rat
ACOxvFXgwMoAlXfD5+aTGAUsaga3t2bC6V8eh+0/hdGDOD8juelRqBY4Ptcu
xEii/ZeLxUEeSBSadkIRRh79g9KN3cegqyEojyHGiN6hGyRddRxG35lVdBZj
9Gq2Wrv1Kz54t+hP+EoxajJTLRx/woc5Gs9/hsLIdCyojpOVBXnKuPiM0xil
OD2d3OacDWEOw+etSzFiWLzZ1K/Khlytfdq9Moyk1bZWx07kgHRrog3zHDm/
PbdZ1y0XXhjP7psvw2gkiTbb9UcufBH6PjS7nPS9cXNojyQPihwyohwrMdI+
8/JS7fZ8iDy+rvJhFUaTNE/nta/zYaWrz9nIGoy6zW98lyw7AVrC1/VL6zDy
lEdufgsC+N1Hh1tyASOzT9lL7VUCWMYNyDD/GaOp4XuZntRJyPSjiTovYRTh
Dud4bgWgOBZ4ILCe9HYbCS+ZLoBo5jL/D5cxylNOrUkoOwUhZyajz18h9392
yInfJgRrX393myaMvO9KhF6zQnCR2NgNXSV9Ldfv9CktBAfvnx5EXMPojLHz
whIPEfB+/HT5aQtGr8tjgyumRKDn/LE87zpGjzq6d8eIi4B/gBO4shWjsVvW
3CxLMbjYxRvebMPonOpj1asUMQzveVzidwsj2t5Vuy71iEG+yPEbbsdIbdR2
J4xeDO/g5Or0Oxi50N+vWHukGHQnzGVaHeS/nB5b5UAxRDnYb2mUY7Tb6FHn
U2sJGE7uXxLahZGuJpexJE0CF/UNxU/ukve7NqWZDEuAN+i1Ob+HzOMWrwis
pMCYfSM27MNoukBg282Rgo9YTpf1Y1RaayPaMigFWb9BDwyQeeuGD6K1FCgT
xu3uDWIUNuPIclpHAV345/L7xF8+0BmttKQg6Ypq8jfiWlmLKms9BbS5icsv
ia/bGah32FMQyJt30lRg9Nind0Mfg4IB0Uo3V2KrHMYJZQAFvde8dl0gfhWr
p+/FpMBkxNu+gbjeXyW5GkRB3L++K1qInVcXl0tYFOh4MhVyYp+q6V9DwyjY
2RMRMk6ciGpejEZT0DXKCTcawmhTDTdx1yEKDNVp7ubEc/khb9tiKYi24Jpb
EfOYn9VnEyjQiuKPOxKLxiMMI5Mo8JoSHvyWmCX/WqZgU1CsL97OIjapW0bf
eoSCZ45S63DiysPX7empFPBTZC9jiWOCC5uK0kiv02U9PxBvcI5x0cigYF1b
xcUU4hkTj3Z2Juk1Vi04StysXrX9GZcCtFiXkE38H8q1gq4=
        "]]}}, {{}, {}, 
      {Hue[0.67, 0.6, 0.6], Dashing[{0.01, 0.01}], LineBox[CompressedData["
1:eJwV0ns41XccB3Ad6bilqNj2SGZqC2fEcV2dj0zqETP3u3NyctyO48vJEJrS
VrKe5XLi5LLzHOWWlMKa8GORnYrkd3JZMs/jElFsbUup7LM/3s/7ef39fn8c
megTxVBRUfHE/N9c1R5TPhnaY7L8eumikKaAa9nAlTRxyDn+0Sp085bYPmtJ
N6fjsz7VGvTSklq4oUTJCQmW6V9Gn41QmupKpjj5rW67G9FhkluntSR/c1Yz
C0+1o29bXW1Vl6yFsTUsoyF06vsaX6ZkM5zX4h1kJtBUIdsrWZa8HR6rXZpP
QneJC1lpTyxhJFgj4xFaVsSbXdWwhfm8VH17EU2tMdCu3FjkBA8udr+SoT0j
DIePUQDxBrt+U02kqfIq4zdj9S4wQjLHRejqjCsnZoT7ILTmocoI+tMbC70V
7/ZDsaObnhOhqTROy/H2HneobJTGXkRzLKyXz0R4QkyO9jVmEk1JU2tiXJ95
gatNLiMWHZmn3HE9wBsWFVMXBtE22g2pUpkP+LVyTWyTaWqh5T6Dvc4PcvN/
WfwJnXhu9oxrlD8o3bYq1MU0xSoNT6qvD4Bhf1l+HJp+YFDQuyEI6g8yZEp0
k625rlNoMDiW5pTbH6GprJ6cu9xfQ+D8w8kJOXrjydHsFZMw0JzxtNJOoalx
8WnJi/hwsPMbFonQkrOZU0UDEWBTCy9H0F8uTB1Q6PBgLrBXsPsbmjLeqWmk
ep0HzVNBM+XoP994n9PnHwLvIMVlZipN8aLuNPqrR0LV9mBeHPr24XAiuhUJ
90Z7Qx6h81aSWS7xfGAKnQ9x0vAPZs3SvRsOw0rgSIAM7b74YeVHXYeh08yr
QDOdpvZ6XA7dFh0FnMDptgR0ufMtvT80BPDw0Hcv+9HPFc8Faa0CsPNdSxyO
0tTjeY+4Pdxo8K2QP76Abpf3a47pxoDzwLazahk0tfuvzJCqmzEwE9rGikUb
j9bWmIfFQmVP5O930ct+rwvPrI8DS/3Z++xMmhLFpny+5UYcpHT+0FCMHtHb
LEgXxEOe98YhRhb+w3dXWYm+EH7uLNcQojexMo5PtgqhLNfdoh8tLn2qqp6Q
AD6OQ0UOx2gqSZ5tG7RJBJ0uxxdK0Urnkih+iwhczey+foc2PzJofSA6EaTL
iteCb3EvWVmgxToCVcZZBQq0Mkf8pIxJwGmu3f0eWhrtzl+vQaC/8T2jD21i
uSx6oUXgX5ds8QCa3eH7/TVdAm5ROX7D6OAxzWb2VgLTtbn602j5B+l6e9gE
TG3OX1DJpinBW6+SK7YEbq4M+TDQ5hM7jIzsCXh0G2itRTdVK3euOhI44i/N
YKLv2Fnt7QIC3allYTroZ35PE/cdJMBvkxsZohscOv5p9iDw6uTk0Fa02FCS
seMrAnmepj9uQ7+ddDnF9CZwY/zSqglaR1xRoQggoLqmdmInejAgxdQpiECx
Yq7EHF3s5FFXF0zAvMDMm4U2Zrxpzgsj4PtJfZcVenp64Iu34QSezi+kW6Pr
FNVdQi6Bo00sazZadOXY/ic8AjpZome2aJt8/z7PSALyfVfl9uj/AEZMa2A=

        "]]}}},
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->None,
    Frame->True,
    FrameLabel->{
      FormBox["\"reciprocal of number of panels\"", TraditionalForm], 
      FormBox[
      "\"\\!\\(\\*Cell[TextData[Cell[BoxData[\\nFormBox[\\n\\n  SubscriptBox[\
\\\"C\\\", \\nStyleBox[\\\"m\\\",\\nFontSlant->\\\"Italic\\\"]], \\n  \
TraditionalForm]], \\\"InlineFormula\\\"]], \\\"InlineFormula\\\"]\\)\"", 
       TraditionalForm]},
    ImageSize->504,
    PlotRange->All,
    PlotRangeClipping->True,
    RotateLabel->False]}], "}"}]], "Output",
 ImageSize->{523, 341},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->112701198]
}, Open  ]],

Cell["\<\
Sometimes, it is instructive to display the results of some coarser \
discretizations, but not use these results when determining the equation for \
the line.\
\>", "ExampleText",
 CellID->248309304],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"clRef", ",", "gCl"}], "}"}], "=", 
  RowBox[{"PlotConvergence", "[", 
   RowBox[{"nP", ",", "cl", ",", "5", ",", 
    RowBox[{"FrameLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
      "\"\<reciprocal of number of panels\>\"", ",", 
       "\"\<\!\(\*Cell[TextData[Cell[BoxData[
FormBox[
SubscriptBox[\"C\", \"\[ScriptL]\"], TraditionalForm]], \"InlineFormula\"]], \
\"InlineFormula\"]\)\>\""}], "}"}]}]}], "]"}]}]], "Input",
 CellLabel->"In[9]:=",
 CellID->181676159],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.9869604951178604`", ",", 
   GraphicsBox[{
     {RGBColor[0, 0, 1], AbsolutePointSize[5], 
      PointBox[{{0.004, 0.9776577475372024}, {0.005, 0.9751254189177794}, {
       0.006666666666666667, 0.9709763188930984}, {0.01, 
       0.9628313052383546}, {0.02, 0.9394680831796303}}]}, 
     {RGBColor[1, 0, 0], AbsolutePointSize[5], 
      PointBox[{{0.025, 0.927992857331362}, {0.03125, 0.9137266434257205}, {
       0.041666666666666664`, 0.8900821246057484}, {0.0625, 
       0.8426318336738858}, {0.125, 0.6999476879013269}}]}, {{}, {}, 
      {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwV0n00lHkUB3C2k8KaUuNlJRrEZKPSKEz5opIk1ehF49hTGppkM5qMl7QW
lfGSGTNPp2lZRbS1ldShRC+mUmaJqcU6q7J2lN3o9zhtL1tT7LN/3HPP5697
7/leTlyyIP4LExOTdUz93zlWcX+vi5Ngz5Zl3e6raXT7PvWU7pLgyiq9gMVY
/k10kiZJgg880aO3oTTIpUj6T5kE8pnFnfcYNwn4b9OKJKjR97fFMRZo2KZV
9RIMhB+4WbGKRq7rA/t3ZikQoOmM7UoaQ/7zwyrOp8Cf65dhFkxDoeQPVk6X
ot5ytU9cAI1U++q+T7H74T5movHm0ZjbZf7wYXUqZAsXuzh407h5qd64cUoa
9uWmt+R40FBqG+VegnTE94gthRwaEZ9NN9xozMDkgKTEWgcaj3eFsbX2B5D3
cfgVh01DNemvJbN3ZOHfZpaTGYsGX9ehnt96EA6KsLiEKTScWyzWq62z4bPT
K2qpKY0Xr6smNX79PbjqQrGHkaDOwfAp+8n3aO/+WKp+Q2Dfv75GmpODeDtb
sYQQ7F848DKIl4sLe1YKdMMEvrNfeesMudDPGtkePEigrvJwOZifh9GOZY6e
/QQvbhy6YuF/CPWKypGSboLeZNbru88OQRp1eZeok6DpWlPnFtVhPON2v294
QDDp+PNz1SFHwBUmX3DXErxkLec5DR+BuLjBamozQeusazEpJ/KxWc/Ki68n
WK6N9RmDHP42M2sW1xI4jOdO8TTIYWxQjMw5SzDS9UvGcqoAiWLn0MJKAmEA
yrL8C9E1S7hOVEZw079327HRQgz+8ZO4iSI4rB+x3V1ehEeme88uKGHmT3jl
iYOK0eg6QNnICVbeUxUHvy5GxNadx9JyCPgct1WhmqPoLVKFhB8gOG7D+zSZ
XwI4BrCXpRIMVyRsODlSgk233nw4t5egp6U1SqRUYPsOYbFcTNDf7JqZw1Hi
K9tffQw7CMoMHytf7Feirf3d4doYAtZW6zXn7iux5anqs2YzgZF9/Va0cymU
JiEFE5EES5zfWzrtK4XWrVzbG8bcK73voW8vxUmhOcszhCCK3XNnwFUF4Sj5
rpxPYGGa6TtZpoJVdnZ5ti+zv1+dzK5LhSRX185n3kwezVkKuKhxpy3lQi2X
YLRQ7tEqVeOdzu/Sjy4Emuq5JUs71PjtyYz7Ux2ZvC22ddx2olA7drxgyIYg
miwQLJxDgbstcLaOseNjs75THApV2qG6i4yrT9QbctwoHFMt6pMxvsqdblzh
SSGL1+5uzvj30Afz2nwprEkfv+vFJnDJ883XR1AwTIjGZTOYf0r40io4kkKs
2FIVw/h8uEF1eT2FXv3luUGMeTNKK1QCCrrK8QhzxqGVow2boinUrdCUlVkT
JN4+PdS3k8LB/Ha/lukE3qczE9fEU3hPp3RUM/7nyMax6wkUJNH22wsYZ0VO
GH/YTUE0Lz5fwLjkqXBmbDKF8PbxnqFpBALtohMPJRTuLa7ZrWNsVzPVOXAf
hcDytZ8vMj6VdNXTOZXCom81bmmMRRuO1ilkFH7uCWyMYTyPJ1pikk7BLfD5
2iDGxI5/Q5JBoeJM0YAb4ytG65DBTAp203yk5oz/AxnMTMQ=
        "]]}}, {{}, {}, 
      {Hue[0.67, 0.6, 0.6], Dashing[{0.01, 0.01}], LineBox[CompressedData["
1:eJwV0n0w1HkcB3Ch8lTtKipJT9IDu0nrulP2HWk4YfOY2LXazXpa68eSp7ZD
HWI0ZHfyUI3JhBRd8nCnS61SI0pXpuNqr+6oTMm37lIdOfe9Pz7zmdfM54/P
Z96fVZLEgCh9PT09X1r/d7FBt62UeezqrjLkcDUEEG9qEmta+Isyzputp261
iL3npLnFf5niN7aK+t272SJrzQD/mLz8ggV1ccSALVszwv8l3GHjtJpAqLla
YKr5wBe7BK3ro77peKnDSGOIQ59qVsVRp/1bHzhXswhtSTsta8sIyniC5Ork
tVBvzDecLiXQKss46bpNuFwzsZwpIahWR47OGDuje0uu3pvjBLMWm9Ww1C7w
KFg46lFM4Bth/evh68Cs0/2THUUEp2tXTj696I4NB0vMWYUEdVmNuS/lu+Al
S/xbXkCw7srYnTPTnjhnmTHzIo8gnd+Wc63bG4qifBb/ewK+g9PnwghfpNy4
4dN8hKAirT7G47UASWZLAri5BJKiAbvmEH/8Zp0iPJZNsMWsKa2iOgA5bydL
9b4jGGvr0+fNCcKJ3Ly+EBVBYslooUdUMKq4jg9asgg4VaKkixdD8Gel4Qw3
k+BR/+ITdxaEQt2rcypIJ2hxtme7hO/D0Z//KJ0+SKDqPnJX3BWGZ0mvJ/xT
CVhHh7KnVguxmc9e06Qk+F1ZoBmPF0F7V5Jll0ygKT40on4QgWvLfhpUMQQ7
x0a8euZH4qODy/73CoKVG0xsDJojYRc81CxPIHg/6V9iKd2PLjeV8Z14gsio
25eDjSSoGt9d5BpH8zsgYhRXJXguXmFeFkNQNJXMcY+XwnqUJXkno/+wsbXC
bcEBaBw5utgoAm+ytMZKewCZrn6iLimBm8+F8BXRUThtUmm0VULz2XHV/Jmx
DO3jBqorkQRve97K0jtk2H4y+eN8McGTNz5xruJozMyYVuWJCK6dvW/ylB0D
LqfBQxdOsP2vQ2G1P8agwkq5dE8YvWfofL29MBbTNmHtd0MJPgf9U1Y4Lw69
r3aF2e4lUMSmci2uxGGQEa6vDCYYNF8ky5DFQ9CZNjUeSP8jcPOpcks5Oldf
Prk7gGAhJytnuEOOeE8L8a09BMqqVwZGCQnIdTxsby0gSDqb7Ry6UIHBIfaa
El+CgR3lUdI2BdZW3hw29SGwT3no5BWdCJ1N5nmZN82r+tRehzkMHknDK3Ve
dP6IUndqLoP30x7XH1NXRHtL5xkzYJVzR/qpV2/6rBg3ZeDbN4vbRc3rDMz7
gc3gNq9eW0u976lJK285g47ZE6OJ1GeXZJi78hhU1x3fqk8t+yIob3RmcN0t
XTjlSfd5bmdjs5WB7sn+nA/ULXUDG2a+YWDFcu59SX37K0c3LRiUpT+J6KV+
HfQqcdduBvnfrs9TUzd93TnR6sPg3DD7QjG10lqTZefH4JZqqj+P+suwe/5c
fwZ6zfeXZlDPV5450xPCINMqtVFE/TAk1dYllEF5S8TDEOqTLj4NDfsYtPt5
fRJQr9SfbC0SMviQu8zNnfrFiwfbvogYmC+fLdtG3dBTp5WLGTi2jxfyqBWN
hz11kQwE/oOXONRbSoPv+UoYJLzRDthR/wfzj0XQ
        "]]}}, {{}, {}, 
      {Hue[0.67, 0.6, 0.6], Dashing[{0.01, 0.01}], LineBox[CompressedData["
1:eJwVz2s0lAkAxvFZih2ULkLE1Ewy3bVOk2w8JYbIaFpp4xwp1jqWlJRsLntE
hCSGFIOojDTFZAy5vG2mdd1WiRBdVt65T0crW8mx9sNznvP7+F91NGr/TzoU
CsV7bv8/IzYx4u2vPNiNUY9TjbWQp6oz95/j4aS4riRsoRZ3ef7VbZk8SNIO
/9m+QAtHEUt5s5gH5w3ideeNtOBqtD+Ht/LgdSpoXMdAi6Tgw8H/fJOPUH2J
/9d5Wrz02ek//0I++OtD3N5/0SCPqeOx9koBqDGtFv2kBtL9tM1jbwrxhmlH
RZMGD57qRfboFsHARBqSlavBydoZp2lLPkyHhbRb4RpoqNkWxUdKQZP6WgdC
g4KAyTWsh2U4+84q47OpBvnHjrEPktfRz7yx3V6lxtSW45st9SuwyOM3j+0P
1WikvWg7bXIDJfxEKb1QjdFOg8BMh5tYZ+aYcjtCjeCSiopte27BoN2sdpmr
Gl4Mqei8byW6uF5O3ivUyAupbc8+LUA6ZccCxoQKzr266ZtSq3BtnOM50qHC
nk90Ttrt26jSWnfb81Wo7+0cTe6qxtVnrtUHo1VI7es3N31xByPqrE+BniqI
c+8eOvpFCOv1uRfm0VRwj1e7sQ3uQVTamHZxSgmry9NFJ8xr4OpQ0lnXpYR7
szRucVUNJNWD4YJSJcrjd26c2VyLhANb45JildhUVB7s9qgWuxg+3ZNeSqg4
kqghTxF+tM3M8GYoMTR86U1rrwhcPauq6GkF/pVynlOO3oc7NcKQ1asA2/Uh
P1txHzkuo8MjlQo8XmWuPBNfh8FSQ+2qeAVOvbAvqjMSgzJ4Kcieq4AfXYe+
I0+MeDZrJ3OtAmHd9etX2tRjcvBqcNOMHNVUa8N9wnr8fp02uaVfDtrKPT5P
v5PgmNv0WIxQjjaev+m9NgksFvDM/JLlKCyxvj/o2QBzQ58GC385+HqdzwJH
G2Cysb04YZMcfbprOuxCGxGSekTbNl8OxyE23VfdCF4Dt3RgVIZnRpzZvoQH
eGQ8m5cmkqGoX19UOr8JrvwnHy0zZOB9rZS1FzbhESegiBMoQ+u0YfKutc1g
vHQq87aXYXkO/YNeTTMG+C5L7AxluJFMG2e6tCA9ZV5HzWsSQQZ/CQu6WpB1
Xa40kZDwDnd5eWBfK6Kv0iPdLpEIZxZnnnjbCj/xcnuTEBKNZwWLh8IIZOnb
h/7xPYkwqpNw6y8E1i2zjVoyZ9MrTz1yIwl00C3PBDqSiBZ9SfKKJqDnrJsx
5UBig2KvtuUsgcSYPuEaFokyv4mu8mwCx9+e/HjejkT6FseUiDoCvk11Ke42
JFjEE1pnPYEPHYKLeatJjO8NbrJpJJAzUFzwmkHCJSzrw6sWAj0TKYJYOomv
Ja+CuO0E2MwDPZU0ElFG55xYwwQc8j8u/daChPU1s6HcEQID5YoVvstJ9Nje
iXn/ikBMzahNmTkJ5u7+asEYgdrux9sczEj8HWdrYamZ69MpCAgzIZGj3yyO
fT/XZ5wRUreUBPL3cZ9PEAi1SoykzLm4Ji794hSBcofQpMLFJDydF65WfiKw
i+2f/m4Ric/d5QR7msDrHziX7eYsOLQtoGKGQMIRl2vxxiT8ZN1Ts7MEVkSx
KjoWkvgP9oNccw==
        "]]}}},
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->None,
    Frame->True,
    FrameLabel->{
      FormBox["\"reciprocal of number of panels\"", TraditionalForm], 
      FormBox[
      "\"\\!\\(\\*Cell[TextData[Cell[BoxData[\\nFormBox[\\nSubscriptBox[\\\"C\
\\\", \\\"\[ScriptL]\\\"], TraditionalForm]], \\\"InlineFormula\\\"]], \
\\\"InlineFormula\\\"]\\)\"", TraditionalForm]},
    ImageSize->504,
    PlotRange->All,
    PlotRangeClipping->True,
    RotateLabel->False]}], "}"}]], "Output",
 ImageSize->{582, 322},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->660809883]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cmRef", ",", "gCm"}], "}"}], "=", 
  RowBox[{"PlotConvergence", "[", 
   RowBox[{"nP", ",", "cm", ",", "5", ",", 
    RowBox[{"FrameLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
      "\"\<reciprocal of number of panels\>\"", ",", 
       "\"\<\!\(\*Cell[TextData[Cell[BoxData[
FormBox[
SubscriptBox[\"C\", 
StyleBox[\"m\",
FontSlant->\"Italic\"]], TraditionalForm]], \"InlineFormula\"]], \
\"InlineFormula\"]\)\>\""}], "}"}]}]}], "]"}]}]], "Input",
 CellLabel->"In[10]:=",
 CellID->70174203],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "0.303506121924393`"}], ",", 
   GraphicsBox[{
     {RGBColor[0, 0, 1], AbsolutePointSize[5], 
      PointBox[{{0.004, -0.3046868087631953}, {0.005, -0.3049914675560532}, {
       0.006666666666666667, -0.30549395798071477`}, {
       0.01, -0.3065313076045568}, {0.02, -0.3094659044525401}}]}, 
     {RGBColor[1, 0, 0], AbsolutePointSize[5], 
      PointBox[{{0.025, -0.3106881727576358}, {
       0.03125, -0.3120918433999881}, {
       0.041666666666666664`, -0.31404311985534317`}, {
       0.0625, -0.31635477336392803`}, {
       0.125, -0.31241801515135503`}}]}, {{}, {}, 
      {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwV0Xs01GkYwHGXk0pMCrGsJuSQTWxr5HL0pIQsg8kp7NEFuW27RuM6Tdu4
NmmMMfNTpuSutpVItij1GnKLYeota5Nta7qsS7ZDq62h9t0/3vOezz/v5ftY
RiWxDmlpaGgEkvX/bqkfNREYxQbUq6zewsfoIWPcnhPHhnm79D5PYsG+sMOl
h9mwqcBiZgfxTCPz7+dpbCgLSHQLJr7J8vgn/RQbMu9rK+OJWaVGmtUtbPhm
zGVRRpxt3Ws6r5MMF2fK9i4Qv3Db6FdenwxFxgl68iyMisQez6oMOFD5iVf4
fQ5Gqaa1owuRKVDCpA18yMXIZni5QlGbCs/X9Lry8zG63diiDlmaDmyu+rat
ACOxvFXgwMoAlXfD5+aTGAUsaga3t2bC6V8eh+0/hdGDOD8juelRqBY4Ptcu
xEii/ZeLxUEeSBSadkIRRh79g9KN3cegqyEojyHGiN6hGyRddRxG35lVdBZj
9Gq2Wrv1Kz54t+hP+EoxajJTLRx/woc5Gs9/hsLIdCyojpOVBXnKuPiM0xil
OD2d3OacDWEOw+etSzFiWLzZ1K/Khlytfdq9Moyk1bZWx07kgHRrog3zHDm/
PbdZ1y0XXhjP7psvw2gkiTbb9UcufBH6PjS7nPS9cXNojyQPihwyohwrMdI+
8/JS7fZ8iDy+rvJhFUaTNE/nta/zYaWrz9nIGoy6zW98lyw7AVrC1/VL6zDy
lEdufgsC+N1Hh1tyASOzT9lL7VUCWMYNyDD/GaOp4XuZntRJyPSjiTovYRTh
Dud4bgWgOBZ4ILCe9HYbCS+ZLoBo5jL/D5cxylNOrUkoOwUhZyajz18h9392
yInfJgRrX393myaMvO9KhF6zQnCR2NgNXSV9Ldfv9CktBAfvnx5EXMPojLHz
whIPEfB+/HT5aQtGr8tjgyumRKDn/LE87zpGjzq6d8eIi4B/gBO4shWjsVvW
3CxLMbjYxRvebMPonOpj1asUMQzveVzidwsj2t5Vuy71iEG+yPEbbsdIbdR2
J4xeDO/g5Or0Oxi50N+vWHukGHQnzGVaHeS/nB5b5UAxRDnYb2mUY7Tb6FHn
U2sJGE7uXxLahZGuJpexJE0CF/UNxU/ukve7NqWZDEuAN+i1Ob+HzOMWrwis
pMCYfSM27MNoukBg282Rgo9YTpf1Y1RaayPaMigFWb9BDwyQeeuGD6K1FCgT
xu3uDWIUNuPIclpHAV345/L7xF8+0BmttKQg6Ypq8jfiWlmLKms9BbS5icsv
ia/bGah32FMQyJt30lRg9Nind0Mfg4IB0Uo3V2KrHMYJZQAFvde8dl0gfhWr
p+/FpMBkxNu+gbjeXyW5GkRB3L++K1qInVcXl0tYFOh4MhVyYp+q6V9DwyjY
2RMRMk6ciGpejEZT0DXKCTcawmhTDTdx1yEKDNVp7ubEc/khb9tiKYi24Jpb
EfOYn9VnEyjQiuKPOxKLxiMMI5Mo8JoSHvyWmCX/WqZgU1CsL97OIjapW0bf
eoSCZ45S63DiysPX7empFPBTZC9jiWOCC5uK0kiv02U9PxBvcI5x0cigYF1b
xcUU4hkTj3Z2Juk1Vi04StysXrX9GZcCtFiXkE38H8q1gq4=
        "]]}}, {{}, {}, 
      {Hue[0.67, 0.6, 0.6], Dashing[{0.01, 0.01}], LineBox[CompressedData["
1:eJwV0ns41XccB3Ad6bilqNj2SGZqC2fEcV2dj0zqETP3u3NyctyO48vJEJrS
VrKe5XLi5LLzHOWWlMKa8GORnYrkd3JZMs/jElFsbUup7LM/3s/7ef39fn8c
megTxVBRUfHE/N9c1R5TPhnaY7L8eumikKaAa9nAlTRxyDn+0Sp085bYPmtJ
N6fjsz7VGvTSklq4oUTJCQmW6V9Gn41QmupKpjj5rW67G9FhkluntSR/c1Yz
C0+1o29bXW1Vl6yFsTUsoyF06vsaX6ZkM5zX4h1kJtBUIdsrWZa8HR6rXZpP
QneJC1lpTyxhJFgj4xFaVsSbXdWwhfm8VH17EU2tMdCu3FjkBA8udr+SoT0j
DIePUQDxBrt+U02kqfIq4zdj9S4wQjLHRejqjCsnZoT7ILTmocoI+tMbC70V
7/ZDsaObnhOhqTROy/H2HneobJTGXkRzLKyXz0R4QkyO9jVmEk1JU2tiXJ95
gatNLiMWHZmn3HE9wBsWFVMXBtE22g2pUpkP+LVyTWyTaWqh5T6Dvc4PcvN/
WfwJnXhu9oxrlD8o3bYq1MU0xSoNT6qvD4Bhf1l+HJp+YFDQuyEI6g8yZEp0
k625rlNoMDiW5pTbH6GprJ6cu9xfQ+D8w8kJOXrjydHsFZMw0JzxtNJOoalx
8WnJi/hwsPMbFonQkrOZU0UDEWBTCy9H0F8uTB1Q6PBgLrBXsPsbmjLeqWmk
ep0HzVNBM+XoP994n9PnHwLvIMVlZipN8aLuNPqrR0LV9mBeHPr24XAiuhUJ
90Z7Qx6h81aSWS7xfGAKnQ9x0vAPZs3SvRsOw0rgSIAM7b74YeVHXYeh08yr
QDOdpvZ6XA7dFh0FnMDptgR0ufMtvT80BPDw0Hcv+9HPFc8Faa0CsPNdSxyO
0tTjeY+4Pdxo8K2QP76Abpf3a47pxoDzwLazahk0tfuvzJCqmzEwE9rGikUb
j9bWmIfFQmVP5O930ct+rwvPrI8DS/3Z++xMmhLFpny+5UYcpHT+0FCMHtHb
LEgXxEOe98YhRhb+w3dXWYm+EH7uLNcQojexMo5PtgqhLNfdoh8tLn2qqp6Q
AD6OQ0UOx2gqSZ5tG7RJBJ0uxxdK0Urnkih+iwhczey+foc2PzJofSA6EaTL
iteCb3EvWVmgxToCVcZZBQq0Mkf8pIxJwGmu3f0eWhrtzl+vQaC/8T2jD21i
uSx6oUXgX5ds8QCa3eH7/TVdAm5ROX7D6OAxzWb2VgLTtbn602j5B+l6e9gE
TG3OX1DJpinBW6+SK7YEbq4M+TDQ5hM7jIzsCXh0G2itRTdVK3euOhI44i/N
YKLv2Fnt7QIC3allYTroZ35PE/cdJMBvkxsZohscOv5p9iDw6uTk0Fa02FCS
seMrAnmepj9uQ7+ddDnF9CZwY/zSqglaR1xRoQggoLqmdmInejAgxdQpiECx
Yq7EHF3s5FFXF0zAvMDMm4U2Zrxpzgsj4PtJfZcVenp64Iu34QSezi+kW6Pr
FNVdQi6Bo00sazZadOXY/ic8AjpZome2aJt8/z7PSALyfVfl9uj/AEZMa2A=

        "]]}}, {{}, {}, 
      {Hue[0.67, 0.6, 0.6], Dashing[{0.01, 0.01}], LineBox[CompressedData["
1:eJwV0Hs01HkYBnBxIpdTjS2UbSYzi+kU0i62Vt6K3CmSLbU1y245Gkm1iix7
JopcVgxmG2RJRMqlceclowld6Pf9Juluu7DU6XJULtP+9o/nPOdznv8es+AI
/181NTQ0fNj834IjceKnMVIIqD8QJrnF4KvEsRT/41LwzU7oKGd9SRpU0Zki
BY9DMhOG9Zoa+9GSPCk42bRfE9xm0G/89d6wNikIS+cJuljHh+wOeT8rG6Zz
Lg3p9DM4tGld0OzkbCiJGvNOYxjMEmq6L8vNgQm7UBvZPQaV/jyb4ScyOPo4
hntymMGmfu3wG1pysI5NyGwZZ/BQ9czaKdN8KOWevtr2kcFx3fTFeT+fBY2x
hq/PaBLM2fHBwr69EFIvDkw1GxDM3r/f9ccXf8OnXM7semOCE7YHbEx1isEy
2bcnjk+wkTfQGbXgHJwKlyxIW0HwYbferpTvS+BmbH/SYQeCIQXFxQ4e58El
3lhStJ6gl0BZcyKgFOJOihzyvQhm/VKtSo8qg9wmpVFkIEGnPq0k68QL8M8T
3rEUEUGPT3zfk+XlwFmVoxu/j2BdX/dDSU8FzEqeFwBRBBMZamI0cBHMmXiB
6x8EFZmXtgdPVoLQXX+JRQpBt9ixja56l0FUlSb3zia45PSUPNKkCo44L3fx
LmT3FmU050IV+D3oLTOpIFgUu85qxqYadoeGZC5XELSWF4VsvFoNPjyj88bt
BP/1rY8Y9KyBiG508uwhOHj/zydtfTUQEC3O8iAEPyp9iUZwLZgEfGfIf0zQ
1aU9P32kFubz3y9cP0Kwy8xk9GjsFXi1+Kbz6g8Efxv4Vn7FQAHHF5VbqdUE
A/mafMcsBfjrx+Xp6lIM7a1bvtS8DjSE4Xavv6JYocvV31xZB8MOW7Ys4lLk
LfXY1L+qHpJ9HK04Qoqd0iCjy531MDd9ne2zVRRlBdzae54NoOr4oemzI8V8
7e47ux42gGipf/ioG0VGy+L6yj2NYHVwd4mZP8U1g678gLFGaO2O+Mt8J8U7
Br5fmN+bYIW3vF5zL0U51ak5O7sZ+jpagywiKUqnS1+qZM2wJXjKUhBLsW1K
X7J+WQvI5jhWj52guCiD/067qgUMiw7hdAbFcxLec+GGVhDvufbxuZyiSO92
ZU5PK0wuma9YeJ6iT9iGoa2b28BzaKe1YRXFMGFeSuTTNojpVPa/bKLYeKyM
MxiK8LhAMKnVxf6ju7bSbh+C3srrXFvWRrn97pnhCHYdYuddrA/WTMZ7HURI
HVakNrBeMeL9uvUYwmqhOy/8GsXCwLc9RekI0mqxy10VxSTbNQniKwgeSkV6
WQ9Fe7zF665DOLw1qJayfu4d0mzeiFD4Qj2g2UtxQ2jqu0etCBNz3M1+Yj1d
8Ejkp0I453O/lnODYoTB8bX29xHUd9X3Ym5S5J4xHsx8gLAstHimlPUNy4uH
3zxC2PrZjU9ZC51pRdkwQoXp6X02tyg+i7ZcbDqOsE0kUA+zztBpURx5g5Dw
VsXn3KYI2Zv9yFuEyxKxmxPrvKropLQJBO0SRYaMtafT3G9GPyHY2gcpulh/
7i1C1ymEnSr14DvWZdsddhTPICRtK1bz+igGvuyd+PIFoXbETeDD+j/RSJIn

        "]]}}},
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->None,
    Frame->True,
    FrameLabel->{
      FormBox["\"reciprocal of number of panels\"", TraditionalForm], 
      FormBox[
      "\"\\!\\(\\*Cell[TextData[Cell[BoxData[\\nFormBox[\\n\\n  SubscriptBox[\
\\\"C\\\", \\nStyleBox[\\\"m\\\",\\nFontSlant->\\\"Italic\\\"]], \\n  \
TraditionalForm]], \\\"InlineFormula\\\"]], \\\"InlineFormula\\\"]\\)\"", 
       TraditionalForm]},
    ImageSize->504,
    PlotRange->All,
    PlotRangeClipping->True,
    RotateLabel->False]}], "}"}]], "Output",
 ImageSize->{523, 343},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[10]=",
 CellID->480062185]
}, Open  ]],

Cell["\<\
Coarser discretizations, not used to determine the equation for the line are \
shown in red.  Results from extrapolations should always be viewed with \
skepticism, but the graph provides a useful visualization of the convergence \
properties of the model for the example used.\
\>", "ExampleText",
 CellID->119128964]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["3.4 Convergence and Timing",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming"]], "Tutorials",
 CellID->111584306],

Cell[TextData[ButtonBox["A2 Graphics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/Graphics"]], "Tutorials",
 CellID->424464613]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["Graphics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Graphics"]], "MoreAbout",
 CellID->748534520]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"PlotConvergence - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 2, 11.1948000}", "context" -> 
    "Aerodynamics`Graphics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Package Paclet Symbol", "language" -> "en", "paclet" -> 
    "Aerodynamics Package", "status" -> "None", "summary" -> 
    "PlotConvergence[n_p, c] plots a graph showing convergence properties for \
an aerodynamic coefficient. The list of numbers of panel describing the \
discretized airfoil is denoted by n_p and the list of coefficients is denoted \
by c. The coefficient is plotted versus the reciprocal of the number of \
panels, and a straight line is fit to the resulting points. A reference value \
for the coefficient is determined by extrapolating the straight line to zero, \
and the function returns a list of the reference coefficient and the \
graphic.PlotConvergence[n_p, c, m] plots a graph showing the convergence \
properties of an aerodynamic coefficient c. Only the subset of data \
consisting of the m largest numbers of panels is used to determine the \
straight - line fit.", "synonyms" -> {}, "title" -> "PlotConvergence", "type" -> 
    "Symbol", "uri" -> "Aerodynamics/ref/PlotConvergence"}, "LinkTrails" -> 
  "", "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[6541, 187, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->152687988]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 47604, 1151}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[608, 21, 76, 1, 70, "SymbolColorBar"],
Cell[687, 24, 191, 5, 70, "LinkTrail"],
Cell[881, 31, 2729, 62, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3635, 97, 58, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[3696, 100, 1786, 45, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5519, 150, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->522824268],
Cell[6102, 169, 195, 4, 70, "Notes",
 CellID->1067943069],
Cell[6300, 175, 127, 4, 70, "Notes",
 CellID->151000646],
Cell[6430, 181, 74, 1, 70, "Notes",
 CellID->246109562]
}, Closed]],
Cell[CellGroupData[{
Cell[6541, 187, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->152687988],
Cell[CellGroupData[{
Cell[7027, 204, 148, 5, 70, "ExampleSection",
 CellID->109762551],
Cell[7178, 211, 132, 6, 70, "ExampleText",
 CellID->337266530],
Cell[7313, 219, 100, 3, 70, "Input",
 CellID->207768003],
Cell[7416, 224, 231, 5, 70, "ExampleText",
 CellID->99847148],
Cell[CellGroupData[{
Cell[7672, 233, 840, 27, 101, "Input",
 CellID->524210768],
Cell[8515, 262, 5843, 103, 201, "Output",
 CellID->912945269]
}, Open  ]],
Cell[14373, 368, 132, 4, 70, "ExampleText",
 CellID->53729757],
Cell[14508, 374, 519, 16, 70, "Input",
 CellID->222327051],
Cell[15030, 392, 223, 5, 70, "ExampleText",
 CellID->3599668],
Cell[CellGroupData[{
Cell[15278, 401, 539, 16, 70, "Input",
 CellID->146492676],
Cell[15820, 419, 304, 9, 36, "Output",
 CellID->525069565]
}, Open  ]],
Cell[16139, 431, 195, 4, 70, "ExampleText",
 CellID->775435343],
Cell[CellGroupData[{
Cell[16359, 439, 852, 26, 70, "Input",
 CellID->53143172],
Cell[17214, 467, 1064, 26, 71, "Output",
 CellID->291300565]
}, Open  ]],
Cell[18293, 496, 134, 4, 70, "ExampleText",
 CellID->2060996],
Cell[CellGroupData[{
Cell[18452, 504, 157, 5, 70, "Input",
 CellID->57188429],
Cell[18612, 511, 215, 7, 36, "Output",
 CellID->32249589]
}, Open  ]],
Cell[18842, 521, 281, 6, 70, "ExampleText",
 CellID->388479374],
Cell[CellGroupData[{
Cell[19148, 531, 679, 21, 70, "Input",
 CellID->640039878],
Cell[19830, 554, 4417, 83, 342, "Output",
 CellID->55683818]
}, Open  ]],
Cell[CellGroupData[{
Cell[24284, 642, 704, 23, 70, "Input",
 CellID->153900572],
Cell[24991, 667, 4454, 85, 362, "Output",
 CellID->112701198]
}, Open  ]],
Cell[29460, 755, 209, 5, 70, "ExampleText",
 CellID->248309304],
Cell[CellGroupData[{
Cell[29694, 764, 527, 15, 70, "Input",
 CellID->181676159],
Cell[30224, 781, 6432, 117, 343, "Output",
 CellID->660809883]
}, Open  ]],
Cell[CellGroupData[{
Cell[36693, 903, 552, 17, 70, "Input",
 CellID->70174203],
Cell[37248, 922, 6481, 121, 364, "Output",
 CellID->480062185]
}, Open  ]],
Cell[43744, 1046, 328, 6, 70, "ExampleText",
 CellID->119128964]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[44121, 1058, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[44543, 1072, 177, 4, 70, "Tutorials",
 CellID->111584306],
Cell[44723, 1078, 147, 3, 70, "Tutorials",
 CellID->424464613]
}, Open  ]],
Cell[CellGroupData[{
Cell[44907, 1086, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[45329, 1100, 141, 3, 70, "MoreAbout",
 CellID->748534520]
}, Open  ]],
Cell[45485, 1106, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

