(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     16698,        502]
NotebookOptionsPosition[     13247,        397]
NotebookOutlinePosition[     15173,        447]
CellTagsIndexPosition[     15088,        442]
WindowTitle->SurfaceSpeed - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"1.4 Airfoils From Conformal Mapping\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AirfoilsFromConformalMapping"], "\<\"3.2 \
Validation Example\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/ValidationExample"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"AirfoilSurfacePoints\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/AirfoilSurfacePoints"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PanelMethods\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/PanelMethods"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/SurfaceSpeed\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/SurfaceSpeed"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
SurfaceSpeed.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/SurfaceSpeed", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["SurfaceSpeed", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"SurfaceSpeed", "[", 
       RowBox[{
        StyleBox["\[Alpha]", "TI"], ",", 
        StyleBox["t", "TI"], ",", 
        StyleBox["\[Tau]", "TI"], ",", 
        StyleBox["\[Theta]", "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes surface speeds from an exact solution for \
potential flow over a member of the family of van de Vooren airfoils. The \
angle of attack is ",
     Cell[BoxData[
      StyleBox["\[Alpha]", "TR"]], "InlineFormula"],
     ", the shape parameters for the airfoil are ",
     Cell[BoxData[
      StyleBox["t", "TI"]], "InlineFormula"],
     " and ",
     Cell[BoxData[
      StyleBox["\[Tau]", "TR"]], "InlineFormula"],
     ", and a list of ",
     Cell[BoxData[
      StyleBox["\[Theta]", "TR"]], "InlineFormula"],
     "-values specifies the locations on the airfoil surface where surface \
speeds are computed."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->49813731],

Cell["New in 1.0", "Notes",
 CellID->918694056]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->664107551],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->551332486],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->58415392],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->606767683],

Cell["\<\
Use a van de Vooren airfoil with maximum thickness of 0.12 chord, \
trailing-edge angle of 9.81\[Degree] at an angle of attack of 6\[Degree].  \
Compute surface speeds from the exact solution using SurfaceSpeed.\
\>", "ExampleText",
 CellID->771933209],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"t", ",", "\[Tau]", ",", "\[Alpha]"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{"0.12", ",", 
     RowBox[{"9.81", "Degree"}], ",", 
     RowBox[{"6.", "Degree"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"n\[Theta]", "=", "50"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Theta]", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"0", ",", 
     RowBox[{"2", "\[Pi]"}], ",", 
     RowBox[{"\[Pi]", "/", "n\[Theta]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rE", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"t", ",", "\[Tau]", ",", 
     RowBox[{"Take", "[", 
      RowBox[{"\[Theta]", ",", 
       RowBox[{"n\[Theta]", "+", "1"}]}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"x", "=", 
   RowBox[{
    RowBox[{"Components", "[", "rE", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vMagE", "=", 
   RowBox[{"SurfaceSpeed", "[", 
    RowBox[{"\[Alpha]", ",", "t", ",", "\[Tau]", ",", "\[Theta]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"ListPlot", "[", 
  RowBox[{
   RowBox[{"Transpose", "[", 
    RowBox[{"{", 
     RowBox[{"x", ",", "vMagE"}], "}"}], "]"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<x\>\"", ",", "\"\<V\>\""}], "}"}]}], ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", " ", "72"}]}], ",", 
   RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellLabel->"In[2]:=",
 CellID->663191981],

Cell[BoxData[
 GraphicsBox[
  {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdlXtQE1cUhyOkvESBAWNKGoQSFCi+UamtHiy1VBQfrakwYhQRFYyggoji
CMjDRkEhoIiGRBM0AyiEFhARBFMxItUCVWLkIQEJEJLdLAlojDC10/H+0Z3Z
2Z3Z3bv3nvP7vuu2O/anSDMSiRT58fz3+t+hg0931hGA0cZxcFEyybJ4Newt
SEymd+Kw1P69KGzfKARwIMemGAeHg6R9asMoRLkpDxXsxCEyoiL3dagG9mvL
FWxrHJrKXlpfKtHA54tDE1cIMaBa7ohr0GiAiD6fedsbg+q8M0pinhZeMGTd
OSItcEzlXwzu0ALb8l50xQwtCGyHBGtytBD++3R5f5QGuG83L6M0aYG+gim0
uzsKf1d1JszEtHBC7EZifFDDq/Rf27nOGAgDeekUXzV0K3Iau9ZiYGvKvPcm
fARa74i7JmMxYBQN1uRmDoNLb1AtcRmDBNmwgCEagvn1Do6FTRgsyOvdIq5V
QXZGRrBUhcHCrJdkj8eDcMCKUu09HYfolba2JR1v4BjrmCZ5Pg7n1hSUrlIM
QBX3M/GuYBxabRaRRrv7gcMn2sKicfiziDWzplcJF2dfWTiehoPhDrVT8LoP
yBERx68U4nDyD0vndQ9ew6kanqKsFAebkpCOo809wPv6XUt8DQ4zmSHm4qdd
cKDokIt3PQ7sCLsMSZcCVKk8e1UdDq5VrQPbCDlYW8WZiSpxOLslg/OzfSdY
xJbk+Qo+vj9x2ixo5XP4q8Th5OrTOLS5V4W2H+oAHou/2zMUh+Xd8ZyY6jZQ
1u3kzvgSh5ekBJOX4zMg1FLtzVcYJC4zD9rQ8wTkIZb1PikYeAauovW3ysCq
XVjvY4eBIc6etkf5EApVqumb0rSQr5UNMlyl4EovYk2Ta8D+7ALdNadGmHZn
j7vJTANNC8e2yQPqIP/Adln6pBo2C0Wp4vZqCCzP82HdHQHJ9cNdSQ8qITIh
td+SMQyuA5WGLp9bUEDww63cVdBEdZadu3kDvtXdECVfGICmrUv5gyMCSCun
mIgFfdD3tPGbcr9LQN2aSa/eJAf/s3jLrtQs6HMWROofNUNKxC9Hau1SYCr7
TN+Q5C6QguyYWZfYwH8RKxz27wWSU1mv/OoGUJj5M04sxhAXKRQP+pPaCfR8
IpAZLzSa0PcW7j1Go8sUGn/9enJby8Yp9H/za8+OJ5VPovmlnfqBM6vjA5q/
o0d8p6PehNZXvHOIRmWY0Pqr83R+LlHvUX0mk4KOOT0wovqNfL9kRYKXEdW3
fZuk4bLgHap/8H1GDtXtHerPrXUpMfqyt6h/VX6hkvTVb1F/aeZSYaJ8AvV/
ap7D1ftHJ1A+yvizdp+kTqD8yH/kFJs3jqN8Df0WFdayfxzl7yJnrYY1axzl
M44+cJX90IDyez40cP7towaUb++l+4PVngaU/zmeoxXGHj3igzvtRkVdvh7x
89xCwXQK1iO+cpl8id5Cj/gT3C+ODJKOIT6lSsqRGcljiF91YLON76oxxDfv
+Zrcmg8E4p/h55WX3UAgP8y1SHosSSaQP2ghLDIlgEB+EZ8QWEksCeQfT3bb
3INPdchPnlnKnGX5OuQv9ZyGr0a265DfHgUs4YUxdMh/lWzrRaWvcORHL/84
FvkgjvzZSo9ZPjyFIb9ONjtzZ1/AkH9F3J6bDTQM+TmsxTmRLNQif3+318HO
l65FfjeQXYzZ6Rrk//DDpzdKH47Cp/3BnhsjK72uhv/vH/8AHpwjMA==
    "]]},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"V\"", TraditionalForm]},
  ImageSize->504,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 327},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->321355287]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[StyleBox[ButtonBox["AirfoilSurfacePoints",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/ref/AirfoilSurfacePoints"],
 FontFamily->"Verdana"]], "SeeAlso",
 CellID->929782353]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["1.4 Airfoils From Conformal Mapping",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromConformalMapping"]], "Tutorials",
 CellID->341631938],

Cell[TextData[ButtonBox["3.2 Validation Example",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/ValidationExample"]], "Tutorials",
 CellID->818994628]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["PanelMethods",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/PanelMethods"]], "MoreAbout",
 CellID->7031534]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"SurfaceSpeed - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 2, 46.7628000}", "context" -> 
    "Aerodynamics`PanelMethods`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Package Paclet Symbol", "language" -> "en", "paclet" -> 
    "Aerodynamics Package", "status" -> "None", "summary" -> 
    "SurfaceSpeed[\\[Alpha], t, \\[Tau], \\[Theta]] computes surface speeds \
from an exact solution for potential flow over a member of the family of van \
de Vooren airfoils. The angle of attack is \\[Alpha], the shape parameters \
for the airfoil are t and \\[Tau], and a list of \\[Theta]-values specifies \
the locations on the airfoil surface where surface speeds are computed.", 
    "synonyms" -> {}, "title" -> "SurfaceSpeed", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/SurfaceSpeed"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[5964, 172, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->664107551]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 14945, 435}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[605, 21, 76, 1, 70, "SymbolColorBar"],
Cell[684, 24, 191, 5, 70, "LinkTrail"],
Cell[878, 31, 3198, 74, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4101, 109, 55, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4159, 112, 1102, 30, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5298, 147, 579, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->49813731],
Cell[5880, 166, 47, 1, 70, "Notes",
 CellID->918694056]
}, Closed]],
Cell[CellGroupData[{
Cell[5964, 172, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->664107551],
Cell[CellGroupData[{
Cell[6450, 189, 148, 5, 70, "ExampleSection",
 CellID->551332486],
Cell[6601, 196, 131, 6, 70, "ExampleText",
 CellID->58415392],
Cell[6735, 204, 100, 3, 70, "Input",
 CellID->606767683],
Cell[6838, 209, 262, 5, 70, "ExampleText",
 CellID->771933209],
Cell[CellGroupData[{
Cell[7125, 218, 1698, 50, 70, "Input",
 CellID->663191981],
Cell[8826, 270, 2274, 47, 348, "Output",
 CellID->321355287]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[11161, 324, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[11581, 338, 195, 4, 70, "SeeAlso",
 CellID->929782353]
}, Open  ]],
Cell[CellGroupData[{
Cell[11813, 347, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[12235, 361, 194, 4, 70, "Tutorials",
 CellID->341631938],
Cell[12432, 367, 167, 3, 70, "Tutorials",
 CellID->818994628]
}, Open  ]],
Cell[CellGroupData[{
Cell[12636, 375, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[13058, 389, 147, 3, 70, "MoreAbout",
 CellID->7031534]
}, Open  ]],
Cell[13220, 395, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

