(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     12171,        411]
NotebookOptionsPosition[      8951,        307]
NotebookOutlinePosition[     10657,        354]
CellTagsIndexPosition[     10572,        349]
WindowTitle->TestArguments - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Influence Coefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/InfluenceCoefficients"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/TestArguments\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/TestArguments"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
TestArguments.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/TestArguments", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["TestArguments", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"TestArguments", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", 
        StyleBox[
         SubscriptBox["r", "p"], "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]tests arguments for an influence-coefficient function, \
returning True if arguments are valid and a message if they are not."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->27629836],

Cell["New in 1.0", "Notes",
 CellID->918694056]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->205354781],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->117848655],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->114915405],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->197263501],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rF", "=", 
  RowBox[{"MakeCartesianVectors", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", "x", "}"}], ",", 
     RowBox[{"{", "z", "}"}]}], "}"}], "]"}]}]], "Input",
 CellLabel->"In[2]:=",
 CellID->366701358],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", "x", "}"}], "\[UpArrow]", 
   RowBox[{"{", "z", "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{$CellContext`x}, \
{$CellContext`z}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{55, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->164739454]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rP", "=", 
  RowBox[{"MakeCartesianVectors", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "\[Delta]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}]}], "}"}], "]"}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->148936341],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "\[Delta]"}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
    0, $CellContext`\[Delta]}, {0, 0}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{91, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->43973505]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TestArguments", "[", 
  RowBox[{"rF", ",", "rP"}], "]"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->442502330],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->32096363]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TestArguments", "[", 
  RowBox[{"rF", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "\[Delta]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->245314237],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"InfluenceCoefficients", "::", "dataType"}], "MessageName"], ":", 
  " ", "\<\"Input arguments specifying locations of field points and \
singularities must be collections of two-dimensional vectors of the data type \
CartesianVectors. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:Aerodynamics/ref/InfluenceCoefficients\\\", ButtonNote \
-> \\\"Aerodynamics`InfluenceCoefficients`InfluenceCoefficients::dataType\\\"]\
\\)\"\>"}]], "Message", "MSG",
 CellLabel->"During evaluation of In[5]:=",
 CellID->228112989]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["Influence Coefficients",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/InfluenceCoefficients"]], "MoreAbout",
 CellID->1665078683]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"TestArguments - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 2, 49.9608000}", "context" -> 
    "Aerodynamics`InfluenceCoefficients`", "keywords" -> {}, "index" -> True, 
    "label" -> "Aerodynamics Package Paclet Symbol", "language" -> "en", 
    "paclet" -> "Aerodynamics Package", "status" -> "None", "summary" -> 
    "TestArguments[r_f, r_p] tests arguments for an influence-coefficient \
function, returning True if arguments are valid and a message if they are \
not.", "synonyms" -> {}, "title" -> "TestArguments", "type" -> "Symbol", 
    "uri" -> "Aerodynamics/ref/TestArguments"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[4632, 139, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->205354781]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 10429, 342}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[606, 21, 76, 1, 70, "SymbolColorBar"],
Cell[685, 24, 191, 5, 70, "LinkTrail"],
Cell[879, 31, 2387, 55, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3291, 90, 56, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[3350, 93, 579, 16, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[3966, 114, 579, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->27629836],
Cell[4548, 133, 47, 1, 70, "Notes",
 CellID->918694056]
}, Closed]],
Cell[CellGroupData[{
Cell[4632, 139, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->205354781],
Cell[CellGroupData[{
Cell[5118, 156, 148, 5, 70, "ExampleSection",
 CellID->117848655],
Cell[5269, 163, 132, 6, 70, "ExampleText",
 CellID->114915405],
Cell[5404, 171, 100, 3, 70, "Input",
 CellID->197263501],
Cell[CellGroupData[{
Cell[5529, 178, 241, 8, 70, "Input",
 CellID->366701358],
Cell[5773, 188, 391, 13, 36, "Output",
 CellID->164739454]
}, Open  ]],
Cell[CellGroupData[{
Cell[6201, 206, 302, 10, 70, "Input",
 CellID->148936341],
Cell[6506, 218, 450, 15, 36, "Output",
 CellID->43973505]
}, Open  ]],
Cell[CellGroupData[{
Cell[6993, 238, 134, 4, 70, "Input",
 CellID->442502330],
Cell[7130, 244, 160, 5, 36, "Output",
 CellID->32096363]
}, Open  ]],
Cell[CellGroupData[{
Cell[7327, 254, 295, 10, 70, "Input",
 CellID->245314237],
Cell[7625, 266, 632, 12, 70, "Message",
 CellID->228112989]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[8318, 285, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[8740, 299, 169, 3, 70, "MoreAbout",
 CellID->1665078683]
}, Open  ]],
Cell[8924, 305, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

