(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     24296,        671]
NotebookOptionsPosition[     20385,        543]
NotebookOutlinePosition[     22066,        590]
CellTagsIndexPosition[     21981,        585]
WindowTitle->ThickAirfoilQ - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"1.1 Introduction To Airfoil Geometry\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToAirfoilGeometry"], \
"\<\"1.5 Panel Geometry\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/PanelGeometry"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"TrailingEdgeAngle\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/ref/TrailingEdgeAngle"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"AirfoilGeometry\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/AirfoilGeometry"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/ThickAirfoilQ\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/ThickAirfoilQ"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
ThickAirfoilQ.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/ThickAirfoilQ", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["ThickAirfoilQ", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ThickAirfoilQ", "[", 
       StyleBox[
        SubscriptBox["r", "p"], "TI"], "]"}]], "InlineFormula"],
     "\[LineSeparator]returns True if the airfoil specified by the collection \
of vectors, ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     " has thickness, False if it does not."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->262112450],

Cell[TextData[{
 "Collections of vectors are of the data type ",
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], "InlineFormula"],
 "."
}], "Notes",
 CellID->1067943069],

Cell["New in 1.0", "Notes",
 CellID->918694056]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->3745808],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->212942371],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->52103464],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->253181148],

Cell["\<\
Specify points on the surface of of a NACA 4412 airfoil.  The last two digits \
of the airfoil identification number give the maximum thickness of the \
airfoil in percent chord.\
\>", "ExampleText",
 CellID->10415831],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rP12", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"NACA4DigitAirfoil", "[", 
     RowBox[{"4412", ",", 
      RowBox[{"NDiscretizeUnitSegment", "[", 
       RowBox[{"50", ",", 
        RowBox[{"Layout", "\[Rule]", "HalfCosineSpacing"}]}], "]"}]}], "]"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"PlotAirfoil", "[", 
  RowBox[{"rP12", ",", 
   RowBox[{"PlotJoined", "\[Rule]", "True"}]}], "]"}]}], "Input",
 CellLabel->"In[2]:=",
 CellID->508498224],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3c9rFHkax/Fe9i/ZfyMgcxAP4iFkQTyIF4VGL+NFL3qJl92LQpNcEpDJ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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->279805911]
}, Open  ]],

Cell["Test the airfoil for thickness.", "ExampleText",
 CellID->211468637],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ThickAirfoilQ", "[", "rP12", "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->13355241],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->18825137]
}, Open  ]],

Cell["\<\
Setting the last two digits of the identification number to zero gives the \
camber line for the airfoil.\
\>", "ExampleText",
 CellID->33415932],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rP00", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"NACA4DigitAirfoil", "[", 
     RowBox[{"4400", ",", 
      RowBox[{"NDiscretizeUnitSegment", "[", 
       RowBox[{"50", ",", 
        RowBox[{"Layout", "\[Rule]", "HalfCosineSpacing"}]}], "]"}]}], "]"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"PlotAirfoil", "[", 
  RowBox[{"rP00", ",", 
   RowBox[{"PlotJoined", "\[Rule]", "True"}]}], "]"}]}], "Input",
 CellLabel->"In[4]:=",
 CellID->631172468],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3c+La2cZB/CIf4n/xoC4EBfiQhTEhbi5QtCN3eim3bQb3SgE3fSC2IU/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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->38916188]
}, Open  ]],

Cell["This is a reference airfoil without thickness.", "ExampleText",
 CellID->250529750],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ThickAirfoilQ", "[", "rP00", "]"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->491720718],

Cell[BoxData["False"], "Output",
 ImageSize->{39, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->283861450]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[StyleBox[ButtonBox["TrailingEdgeAngle",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/ref/TrailingEdgeAngle"],
 FontFamily->"Verdana"]], "SeeAlso",
 CellID->929782353]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["1.1 Introduction To Airfoil Geometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToAirfoilGeometry"]], "Tutorials",
 CellID->341631938],

Cell[TextData[ButtonBox["1.5 Panel Geometry",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/PanelGeometry"]], "Tutorials",
 CellID->223241011]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["AirfoilGeometry",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/AirfoilGeometry"]], "MoreAbout",
 CellID->98815533]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"ThickAirfoilQ - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 2, 53.5488000}", "context" -> 
    "Aerodynamics`AirfoilGeometry`", "keywords" -> {}, "index" -> True, 
    "label" -> "Aerodynamics Package Paclet Symbol", "language" -> "en", 
    "paclet" -> "Aerodynamics Package", "status" -> "None", "summary" -> 
    "ThickAirfoilQ[r_p] returns True if the airfoil specified by the \
collection of vectors, r_p has thickness, False if it does not.", 
    "synonyms" -> {}, "title" -> "ThickAirfoilQ", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/ThickAirfoilQ"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[5711, 169, 459, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->3745808]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 21840, 578}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[606, 21, 76, 1, 70, "SymbolColorBar"],
Cell[685, 24, 191, 5, 70, "LinkTrail"],
Cell[879, 31, 3188, 73, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4092, 108, 56, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4151, 111, 602, 18, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4790, 134, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->262112450],
Cell[5373, 153, 251, 8, 70, "Notes",
 CellID->1067943069],
Cell[5627, 163, 47, 1, 70, "Notes",
 CellID->918694056]
}, Closed]],
Cell[CellGroupData[{
Cell[5711, 169, 459, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->3745808],
Cell[CellGroupData[{
Cell[6195, 186, 148, 5, 70, "ExampleSection",
 CellID->212942371],
Cell[6346, 193, 131, 6, 70, "ExampleText",
 CellID->52103464],
Cell[6480, 201, 100, 3, 70, "Input",
 CellID->253181148],
Cell[6583, 206, 228, 5, 70, "ExampleText",
 CellID->10415831],
Cell[CellGroupData[{
Cell[6836, 215, 514, 14, 65, "Input",
 CellID->508498224],
Cell[7353, 231, 5376, 96, 201, "Output",
 CellID->279805911]
}, Open  ]],
Cell[12744, 330, 74, 1, 70, "ExampleText",
 CellID->211468637],
Cell[CellGroupData[{
Cell[12843, 335, 111, 3, 70, "Input",
 CellID->13355241],
Cell[12957, 340, 160, 5, 36, "Output",
 CellID->18825137]
}, Open  ]],
Cell[13132, 348, 155, 4, 70, "ExampleText",
 CellID->33415932],
Cell[CellGroupData[{
Cell[13312, 356, 514, 14, 65, "Input",
 CellID->631172468],
Cell[13829, 372, 4008, 73, 201, "Output",
 CellID->38916188]
}, Open  ]],
Cell[17852, 448, 89, 1, 70, "ExampleText",
 CellID->250529750],
Cell[CellGroupData[{
Cell[17966, 453, 112, 3, 70, "Input",
 CellID->491720718],
Cell[18081, 458, 162, 5, 36, "Output",
 CellID->283861450]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[18304, 470, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[18724, 484, 189, 4, 70, "SeeAlso",
 CellID->929782353]
}, Open  ]],
Cell[CellGroupData[{
Cell[18950, 493, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[19372, 507, 196, 4, 70, "Tutorials",
 CellID->341631938],
Cell[19571, 513, 159, 3, 70, "Tutorials",
 CellID->223241011]
}, Open  ]],
Cell[CellGroupData[{
Cell[19767, 521, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[20189, 535, 154, 3, 70, "MoreAbout",
 CellID->98815533]
}, Open  ]],
Cell[20358, 541, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

