(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    135403,       2931]
NotebookOptionsPosition[    130545,       2777]
NotebookOutlinePosition[    132532,       2828]
CellTagsIndexPosition[    132445,       2823]
WindowTitle->UniformFlow - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"2.2 Point Singularities\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/PointSingularities"], "\<\"3.3 \
Implementation\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/Implementation"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"InfluenceCoefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/InfluenceCoefficients"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/UniformFlow\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/UniformFlow"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
UniformFlow.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/UniformFlow", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["UniformFlow", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"UniformFlow", "[", 
       RowBox[{
        StyleBox["n", "TI"], ",", " ", 
        StyleBox["\[Alpha]", "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes a collection of ",
     Cell[BoxData[
      StyleBox["n", "TI"]], "InlineFormula"],
     " unit velocity vectors inclined at an angle ",
     Cell[BoxData[
      StyleBox["\[Alpha]", "TR"]], "InlineFormula"],
     " to the ",
     Cell[BoxData[
      StyleBox["x", "TI"]], "InlineFormula"],
     "-axis."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"UniformFlowPhi", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox["\[Alpha]", "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes a list of velocity potentials at ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     " for dimensionless uniform flow inclined at an angle ",
     Cell[BoxData[
      StyleBox["\[Alpha]", "TR"]], "InlineFormula"],
     " to the ",
     Cell[BoxData[
      StyleBox["x", "TI"]], "InlineFormula"],
     "-axis."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"UniformFlowPsi", "[", 
       RowBox[{
        StyleBox[
         SubscriptBox["r", "f"], "TI"], ",", " ", 
        StyleBox["\[Alpha]", "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]computes a list of stream functions at ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["f", "TI"]]], "InlineFormula"],
     " for Dimensionless uniform flow inclined at an angle ",
     Cell[BoxData[
      StyleBox["\[Alpha]", "TR"]], "InlineFormula"],
     " to the ",
     Cell[BoxData[
      StyleBox["x", "TI"]], "InlineFormula"],
     "-axis."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, 
   "Rows" -> {None, None, {None}}, "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->488563757],

Cell[TextData[{
 "The input argument ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["f", "TI"]]], "InlineFormula"],
 " is a collection of vectors of the data type ",
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], "InlineFormula"],
 ".    ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation may be used for a single vector."
}], "Notes",
 CellID->678462984],

Cell[TextData[{
 "The characteristic speed for nondimensionalizing the problem of uniform \
flow over an airfoil is the speed of the uniform onset flow.  The output of \
UniformFlow is a collection of unit vectors of the data type \
CartesianVectors.  Velocity potential and stream function are lists.  For the \
special case of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation being used to specify a single field point, output will be given \
in the simplest ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation."
}], "Notes",
 CellID->54053493],

Cell["New in 1.0   |   Last modified in 1.2", "Notes",
 CellID->918694056]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->76469764],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(2)", "ExampleCount"]
}], "ExampleSection",
 CellID->165292882],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->513831186],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->66474307],

Cell[TextData[{
 "Specify a collection of ",
 Cell[BoxData[
  StyleBox["n", "TI"]], "InlineFormula"],
 " field points."
}], "ExampleText",
 CellID->41115992],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"n", "=", "4"}], ";", 
  RowBox[{"rFN", "=", 
   RowBox[{"MakeCartesianVectors", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Array", "[", 
       RowBox[{"x", ",", "n"}], "]"}], ",", 
      RowBox[{"Array", "[", 
       RowBox[{"z", ",", "n"}], "]"}]}], "}"}], "]"}]}]}]], "Input",
 CellLabel->"In[2]:=",
 CellID->103550962],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}], ",", 
     RowBox[{"x", "[", "4", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"z", "[", "1", "]"}], ",", 
     RowBox[{"z", "[", "2", "]"}], ",", 
     RowBox[{"z", "[", "3", "]"}], ",", 
     RowBox[{"z", "[", "4", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`x[1], 
     $CellContext`x[2], 
     $CellContext`x[3], 
     $CellContext`x[4]}, {
     $CellContext`z[1], 
     $CellContext`z[2], 
     $CellContext`z[3], 
     $CellContext`z[4]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{331, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->81162724]
}, Open  ]],

Cell[TextData[{
 "Compute the dimensionless uniform velocity inclined an an angle ",
 Cell[BoxData[
  StyleBox["\[Alpha]", "TR"]], "InlineFormula"],
 " to the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis at the field points."
}], "ExampleText",
 CellID->35964272],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"UniformFlow", "[", 
  RowBox[{"n", ",", "\[Alpha]"}], "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->122330855],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Cos", "[", "\[Alpha]", "]"}], ",", 
     RowBox[{"Cos", "[", "\[Alpha]", "]"}], ",", 
     RowBox[{"Cos", "[", "\[Alpha]", "]"}], ",", 
     RowBox[{"Cos", "[", "\[Alpha]", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Sin", "[", "\[Alpha]", "]"}], ",", 
     RowBox[{"Sin", "[", "\[Alpha]", "]"}], ",", 
     RowBox[{"Sin", "[", "\[Alpha]", "]"}], ",", 
     RowBox[{"Sin", "[", "\[Alpha]", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     Cos[$CellContext`\[Alpha]], 
     Cos[$CellContext`\[Alpha]], 
     Cos[$CellContext`\[Alpha]], 
     Cos[$CellContext`\[Alpha]]}, {
     Sin[$CellContext`\[Alpha]], 
     Sin[$CellContext`\[Alpha]], 
     Sin[$CellContext`\[Alpha]], 
     Sin[$CellContext`\[Alpha]]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{443, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->7777265]
}, Open  ]],

Cell["Compute the velocity potential at field points.", "ExampleText",
 CellID->118717855],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"UniformFlowPhi", "[", 
  RowBox[{"rFN", ",", "\[Alpha]"}], "]"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->731712474],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Cos", "[", "\[Alpha]", "]"}], " ", 
     RowBox[{"x", "[", "1", "]"}]}], "+", 
    RowBox[{
     RowBox[{"Sin", "[", "\[Alpha]", "]"}], " ", 
     RowBox[{"z", "[", "1", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"Cos", "[", "\[Alpha]", "]"}], " ", 
     RowBox[{"x", "[", "2", "]"}]}], "+", 
    RowBox[{
     RowBox[{"Sin", "[", "\[Alpha]", "]"}], " ", 
     RowBox[{"z", "[", "2", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"Cos", "[", "\[Alpha]", "]"}], " ", 
     RowBox[{"x", "[", "3", "]"}]}], "+", 
    RowBox[{
     RowBox[{"Sin", "[", "\[Alpha]", "]"}], " ", 
     RowBox[{"z", "[", "3", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"Cos", "[", "\[Alpha]", "]"}], " ", 
     RowBox[{"x", "[", "4", "]"}]}], "+", 
    RowBox[{
     RowBox[{"Sin", "[", "\[Alpha]", "]"}], " ", 
     RowBox[{"z", "[", "4", "]"}]}]}]}], "}"}]], "Output",
 ImageSize->{348, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->138165972]
}, Open  ]],

Cell["Compute the stream function at field points.", "ExampleText",
 CellID->286345785],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"UniformFlowPsi", "[", 
  RowBox[{"rFN", ",", "\[Alpha]"}], "]"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->366527698],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{"Sin", "[", "\[Alpha]", "]"}]}], " ", 
     RowBox[{"x", "[", "1", "]"}]}], "+", 
    RowBox[{
     RowBox[{"Cos", "[", "\[Alpha]", "]"}], " ", 
     RowBox[{"z", "[", "1", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{"Sin", "[", "\[Alpha]", "]"}]}], " ", 
     RowBox[{"x", "[", "2", "]"}]}], "+", 
    RowBox[{
     RowBox[{"Cos", "[", "\[Alpha]", "]"}], " ", 
     RowBox[{"z", "[", "2", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{"Sin", "[", "\[Alpha]", "]"}]}], " ", 
     RowBox[{"x", "[", "3", "]"}]}], "+", 
    RowBox[{
     RowBox[{"Cos", "[", "\[Alpha]", "]"}], " ", 
     RowBox[{"z", "[", "3", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{"Sin", "[", "\[Alpha]", "]"}]}], " ", 
     RowBox[{"x", "[", "4", "]"}]}], "+", 
    RowBox[{
     RowBox[{"Cos", "[", "\[Alpha]", "]"}], " ", 
     RowBox[{"z", "[", "4", "]"}]}]}]}], "}"}]], "Output",
 ImageSize->{364, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->1744437]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell["\t", "ExampleDelimiter"],
  $Line = 0; Null]], "ExampleDelimiter",
 CellID->427059611],

Cell[TextData[{
 "For visualizing the flow, specify an arbitrary field point in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation."
}], "ExampleText",
 CellID->659456463],

Cell[BoxData[
 RowBox[{
  RowBox[{"rField", "=", 
   RowBox[{"{", 
    RowBox[{"x", ",", "z"}], "}"}]}], ";"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->64953589],

Cell[TextData[{
 "Compute the velocity, velocity potential and stream function for uniform \
flow inclined at an angle ",
 Cell[BoxData[
  StyleBox["\[Alpha]", "TR"]], "InlineFormula"],
 " to the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis."
}], "ExampleText",
 CellID->344772720],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"u\[Infinity]", ",", "w\[Infinity]"}], "}"}], "=", 
   RowBox[{"UniformFlow", "[", 
    RowBox[{"1", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Phi]\[Infinity]", "=", 
   RowBox[{"UniformFlowPhi", "[", 
    RowBox[{"rField", ",", "\[Alpha]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Psi]\[Infinity]", "=", 
   RowBox[{"UniformFlowPsi", "[", 
    RowBox[{"rField", ",", "\[Alpha]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
  "u\[Infinity]", ",", "w\[Infinity]", ",", "\[Phi]\[Infinity]", ",", 
   "\[Psi]\[Infinity]"}], "}"}]}], "Input",
 CellLabel->"In[2]:=",
 CellID->374015860],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Cos", "[", "\[Alpha]", "]"}], ",", 
   RowBox[{"Sin", "[", "\[Alpha]", "]"}], ",", 
   RowBox[{
    RowBox[{"x", " ", 
     RowBox[{"Cos", "[", "\[Alpha]", "]"}]}], "+", 
    RowBox[{"z", " ", 
     RowBox[{"Sin", "[", "\[Alpha]", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"z", " ", 
     RowBox[{"Cos", "[", "\[Alpha]", "]"}]}], "-", 
    RowBox[{"x", " ", 
     RowBox[{"Sin", "[", "\[Alpha]", "]"}]}]}]}], "}"}]], "Output",
 ImageSize->{369, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->121815841]
}, Open  ]],

Cell[TextData[{
 "Flow visualization is shown below for uniform flow inclined at an angle of \
15\[Degree] to the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis."
}], "ExampleText",
 CellID->382023890],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xMin", ",", "xMax", ",", "zMin", ",", "zMax"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "1.5"}], ",", "1.5", ",", 
     RowBox[{"-", "1.2"}], ",", "1.2"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"uPlot", ",", "wPlot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"u\[Infinity]", ",", "w\[Infinity]"}], "}"}], "/.", 
    RowBox[{"\[Alpha]", "\[Rule]", 
     RowBox[{"15", " ", "Degree"}]}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Phi]Plot", ",", "\[Psi]Plot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\[Phi]\[Infinity]", ",", "\[Psi]\[Infinity]"}], "}"}], "/.", 
    RowBox[{"\[Alpha]", "\[Rule]", 
     RowBox[{"15", " ", "Degree"}]}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gV", "=", 
   RowBox[{"VectorPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"uPlot", ",", "wPlot"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"VectorColorFunction", "->", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"RGBColor", "[", 
         RowBox[{"0", ",", "1", ",", "0"}], "]"}], "&"}], ")"}]}], ",", 
     RowBox[{"VectorPoints", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"14", ",", "14"}], "}"}]}], ",", 
     RowBox[{"VectorScale", "\[Rule]", "Small"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Phi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Phi]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "7"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Red"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Psi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "13"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Blue"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gV", ",", "g\[Phi]", ",", "g\[Psi]", ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
   RowBox[{"FrameTicks", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.5"}], ",", "1.5", ",", "0.25"}], "]"}], "]"}], ",", 
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.2"}], ",", "1.2", ",", "0.2"}], "]"}], "]"}], ",", 
      "None", ",", "None"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", " ", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->1746043248],

Cell[BoxData[
 GraphicsBox[{{{}, 
    {Hue[0.67, 0.6, 0.6], 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.61132897525773, -1.2298305090144932`}, \
{-1.3886710247422702`, -1.170169490985507}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.61132897525773, -1.0452151243991086`}, \
{-1.3886710247422702`, -0.9855541063701221}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.61132897525773, -0.8605997397837238}, \
{-1.3886710247422702`, -0.8009387217547375}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.61132897525773, -0.6759843551683392}, \
{-1.3886710247422702`, -0.6163233371393528}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.61132897525773, -0.49136897055295464`}, \
{-1.3886710247422702`, -0.4317079525239683}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.61132897525773, -0.30675358593757}, \
{-1.3886710247422702`, -0.24709256790858364`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.61132897525773, -0.1221382013221854}, \
{-1.3886710247422702`, -0.062477183293199054`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.61132897525773, 
        0.06247718329319919}, {-1.3886710247422702`, 
        0.12213820132218554`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.61132897525773, 
        0.24709256790858378`}, {-1.3886710247422702`, 0.3067535859375701}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.61132897525773, 
        0.4317079525239684}, {-1.3886710247422702`, 0.49136897055295475`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.61132897525773, 
        0.6163233371393531}, {-1.3886710247422702`, 0.6759843551683394}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.61132897525773, 
        0.8009387217547377}, {-1.3886710247422702`, 0.8605997397837241}}]}}, 
     {Arrowheads[{{0.024974169292722184`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.61132897525773, 
        0.9855541063701222}, {-1.3886710247422702`, 1.0452151243991084`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.61132897525773, 
        1.1701694909855067`}, {-1.3886710247422702`, 1.229830509014493}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.3805597444884992`, -1.2298305090144932`}, \
{-1.1579017939730394`, -1.170169490985507}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.3805597444884992`, -1.0452151243991086`}, \
{-1.1579017939730394`, -0.9855541063701221}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.3805597444884992`, -0.8605997397837238}, \
{-1.1579017939730394`, -0.8009387217547375}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.3805597444884992`, -0.6759843551683392}, \
{-1.1579017939730394`, -0.6163233371393528}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.3805597444884992`, -0.49136897055295464`}, \
{-1.1579017939730394`, -0.4317079525239683}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.3805597444884992`, -0.30675358593757}, \
{-1.1579017939730394`, -0.24709256790858364`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.3805597444884992`, -0.1221382013221854}, \
{-1.1579017939730394`, -0.062477183293199054`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.3805597444884992`, 
        0.06247718329319919}, {-1.1579017939730394`, 
        0.12213820132218554`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.3805597444884992`, 
        0.24709256790858378`}, {-1.1579017939730394`, 0.3067535859375701}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.3805597444884992`, 
        0.4317079525239684}, {-1.1579017939730394`, 0.49136897055295475`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.3805597444884992`, 
        0.6163233371393531}, {-1.1579017939730394`, 0.6759843551683394}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.3805597444884992`, 
        0.8009387217547377}, {-1.1579017939730394`, 0.8605997397837241}}]}}, 
     {Arrowheads[{{0.024974169292722184`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.3805597444884992`, 
        0.9855541063701222}, {-1.1579017939730394`, 1.0452151243991084`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.3805597444884992`, 
        1.1701694909855067`}, {-1.1579017939730394`, 1.229830509014493}}]}}, 
     {Arrowheads[{{0.02497416929272221, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.1497905137192683`, -1.2298305090144932`}, \
{-0.9271325632038083, -1.170169490985507}}]}}, 
     {Arrowheads[{{0.02497416929272222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.1497905137192683`, -1.0452151243991086`}, \
{-0.9271325632038083, -0.9855541063701221}}]}}, 
     {Arrowheads[{{0.024974169292722215`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.1497905137192683`, -0.8605997397837238}, \
{-0.9271325632038083, -0.8009387217547375}}]}}, 
     {Arrowheads[{{0.024974169292722215`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.1497905137192683`, -0.6759843551683392}, \
{-0.9271325632038083, -0.6163233371393528}}]}}, 
     {Arrowheads[{{0.02497416929272221, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.1497905137192683`, -0.49136897055295464`}, \
{-0.9271325632038083, -0.4317079525239683}}]}}, 
     {Arrowheads[{{0.024974169292722215`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.1497905137192683`, -0.30675358593757}, \
{-0.9271325632038083, -0.24709256790858364`}}]}}, 
     {Arrowheads[{{0.02497416929272221, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.1497905137192683`, -0.1221382013221854}, \
{-0.9271325632038083, -0.062477183293199054`}}]}}, 
     {Arrowheads[{{0.02497416929272221, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.1497905137192683`, 
        0.06247718329319919}, {-0.9271325632038083, 0.12213820132218554`}}]}}, 
     {Arrowheads[{{0.02497416929272221, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.1497905137192683`, 
        0.24709256790858378`}, {-0.9271325632038083, 0.3067535859375701}}]}}, 
     {Arrowheads[{{0.024974169292722215`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.1497905137192683`, 
        0.4317079525239684}, {-0.9271325632038083, 0.49136897055295475`}}]}}, 
     {Arrowheads[{{0.024974169292722215`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.1497905137192683`, 
        0.6163233371393531}, {-0.9271325632038083, 0.6759843551683394}}]}}, 
     {Arrowheads[{{0.024974169292722215`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.1497905137192683`, 
        0.8009387217547377}, {-0.9271325632038083, 0.8605997397837241}}]}}, 
     {Arrowheads[{{0.02497416929272221, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.1497905137192683`, 
        0.9855541063701222}, {-0.9271325632038083, 1.0452151243991084`}}]}}, 
     {Arrowheads[{{0.02497416929272221, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.1497905137192683`, 
        1.1701694909855067`}, {-0.9271325632038083, 1.229830509014493}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9190212829500374, -1.2298305090144932`}, \
{-0.6963633324345775, -1.170169490985507}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9190212829500374, -1.0452151243991086`}, \
{-0.6963633324345775, -0.9855541063701221}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9190212829500374, -0.8605997397837238}, \
{-0.6963633324345775, -0.8009387217547375}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9190212829500374, -0.6759843551683392}, \
{-0.6963633324345775, -0.6163233371393528}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9190212829500374, -0.49136897055295464`}, \
{-0.6963633324345775, -0.4317079525239683}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9190212829500374, -0.30675358593757}, \
{-0.6963633324345775, -0.24709256790858364`}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9190212829500374, -0.1221382013221854}, \
{-0.6963633324345775, -0.062477183293199054`}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9190212829500374, 
        0.06247718329319919}, {-0.6963633324345775, 0.12213820132218554`}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9190212829500374, 
        0.24709256790858378`}, {-0.6963633324345775, 0.3067535859375701}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9190212829500374, 
        0.4317079525239684}, {-0.6963633324345775, 0.49136897055295475`}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9190212829500374, 
        0.6163233371393531}, {-0.6963633324345775, 0.6759843551683394}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9190212829500374, 
        0.8009387217547377}, {-0.6963633324345775, 0.8605997397837241}}]}}, 
     {Arrowheads[{{0.02497416929272219, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9190212829500374, 
        0.9855541063701222}, {-0.6963633324345775, 1.0452151243991084`}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9190212829500374, 
        1.1701694909855067`}, {-0.6963633324345775, 1.229830509014493}}]}}, 
     {Arrowheads[{{0.02497416929272219, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6882520521808065, -1.2298305090144932`}, \
{-0.4655941016653467, -1.170169490985507}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6882520521808065, -1.0452151243991086`}, \
{-0.4655941016653467, -0.9855541063701221}}]}}, 
     {Arrowheads[{{0.024974169292722198`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6882520521808065, -0.8605997397837238}, \
{-0.4655941016653467, -0.8009387217547375}}]}}, 
     {Arrowheads[{{0.024974169292722198`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6882520521808065, -0.6759843551683392}, \
{-0.4655941016653467, -0.6163233371393528}}]}}, 
     {Arrowheads[{{0.02497416929272219, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6882520521808065, -0.49136897055295464`}, \
{-0.4655941016653467, -0.4317079525239683}}]}}, 
     {Arrowheads[{{0.024974169292722198`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6882520521808065, -0.30675358593757}, \
{-0.4655941016653467, -0.24709256790858364`}}]}}, 
     {Arrowheads[{{0.02497416929272219, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6882520521808065, -0.1221382013221854}, \
{-0.4655941016653467, -0.062477183293199054`}}]}}, 
     {Arrowheads[{{0.02497416929272219, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6882520521808065, 
        0.06247718329319919}, {-0.4655941016653467, 0.12213820132218554`}}]}}, 
     {Arrowheads[{{0.02497416929272219, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6882520521808065, 
        0.24709256790858378`}, {-0.4655941016653467, 0.3067535859375701}}]}}, 
     {Arrowheads[{{0.024974169292722198`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6882520521808065, 
        0.4317079525239684}, {-0.4655941016653467, 0.49136897055295475`}}]}}, 
     {Arrowheads[{{0.024974169292722198`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6882520521808065, 
        0.6163233371393531}, {-0.4655941016653467, 0.6759843551683394}}]}}, 
     {Arrowheads[{{0.024974169292722198`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6882520521808065, 
        0.8009387217547377}, {-0.4655941016653467, 0.8605997397837241}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6882520521808065, 
        0.9855541063701222}, {-0.4655941016653467, 1.0452151243991084`}}]}}, 
     {Arrowheads[{{0.02497416929272219, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6882520521808065, 
        1.1701694909855067`}, {-0.4655941016653467, 1.229830509014493}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4574828214115758, -1.2298305090144932`}, \
{-0.23482487089611592`, -1.170169490985507}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4574828214115758, -1.0452151243991086`}, \
{-0.23482487089611592`, -0.9855541063701221}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4574828214115758, -0.8605997397837238}, \
{-0.23482487089611592`, -0.8009387217547375}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4574828214115758, -0.6759843551683392}, \
{-0.23482487089611592`, -0.6163233371393528}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4574828214115758, -0.49136897055295464`}, \
{-0.23482487089611592`, -0.4317079525239683}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4574828214115758, -0.30675358593757}, \
{-0.23482487089611592`, -0.24709256790858364`}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4574828214115758, -0.1221382013221854}, \
{-0.23482487089611592`, -0.062477183293199054`}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4574828214115758, 
        0.06247718329319919}, {-0.23482487089611592`, 
        0.12213820132218554`}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4574828214115758, 
        0.24709256790858378`}, {-0.23482487089611592`, 
        0.3067535859375701}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4574828214115758, 
        0.4317079525239684}, {-0.23482487089611592`, 
        0.49136897055295475`}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4574828214115758, 
        0.6163233371393531}, {-0.23482487089611592`, 0.6759843551683394}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4574828214115758, 
        0.8009387217547377}, {-0.23482487089611592`, 0.8605997397837241}}]}}, 
     {Arrowheads[{{0.02497416929272219, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4574828214115758, 
        0.9855541063701222}, {-0.23482487089611592`, 1.0452151243991084`}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4574828214115758, 
        1.1701694909855067`}, {-0.23482487089611592`, 1.229830509014493}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.22671359064234503`, -1.2298305090144932`}, \
{-0.004055640126885154, -1.170169490985507}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.22671359064234503`, -1.0452151243991086`}, \
{-0.004055640126885154, -0.9855541063701221}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.22671359064234503`, -0.8605997397837238}, \
{-0.004055640126885154, -0.8009387217547375}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.22671359064234503`, -0.6759843551683392}, \
{-0.004055640126885154, -0.6163233371393528}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.22671359064234503`, -0.49136897055295464`}, \
{-0.004055640126885154, -0.4317079525239683}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.22671359064234503`, -0.30675358593757}, \
{-0.004055640126885154, -0.24709256790858364`}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.22671359064234503`, -0.1221382013221854}, \
{-0.004055640126885154, -0.062477183293199054`}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.22671359064234503`, 
        0.06247718329319919}, {-0.004055640126885154, 
        0.12213820132218554`}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.22671359064234503`, 
        0.24709256790858378`}, {-0.004055640126885154, 
        0.3067535859375701}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.22671359064234503`, 
        0.4317079525239684}, {-0.004055640126885154, 
        0.49136897055295475`}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.22671359064234503`, 
        0.6163233371393531}, {-0.004055640126885154, 0.6759843551683394}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.22671359064234503`, 
        0.8009387217547377}, {-0.004055640126885154, 0.8605997397837241}}]}}, 
     {Arrowheads[{{0.02497416929272219, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.22671359064234503`, 
        0.9855541063701222}, {-0.004055640126885154, 1.0452151243991084`}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.22671359064234503`, 
        1.1701694909855067`}, {-0.004055640126885154, 1.229830509014493}}]}}, 
     {Arrowheads[{{0.02497416929272219, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.004055640126885765, -1.2298305090144932`}, {
        0.22671359064234559`, -1.170169490985507}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.004055640126885765, -1.0452151243991086`}, {
        0.22671359064234559`, -0.9855541063701221}}]}}, 
     {Arrowheads[{{0.024974169292722198`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.004055640126885765, -0.8605997397837238}, {
        0.22671359064234559`, -0.8009387217547375}}]}}, 
     {Arrowheads[{{0.024974169292722198`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.004055640126885765, -0.6759843551683392}, {
        0.22671359064234559`, -0.6163233371393528}}]}}, 
     {Arrowheads[{{0.02497416929272219, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.004055640126885765, -0.49136897055295464`}, {
        0.22671359064234559`, -0.4317079525239683}}]}}, 
     {Arrowheads[{{0.024974169292722198`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.004055640126885765, -0.30675358593757}, {
        0.22671359064234559`, -0.24709256790858364`}}]}}, 
     {Arrowheads[{{0.02497416929272219, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.004055640126885765, -0.1221382013221854}, {
        0.22671359064234559`, -0.062477183293199054`}}]}}, 
     {Arrowheads[{{0.02497416929272219, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.004055640126885765, 0.06247718329319919}, {
        0.22671359064234559`, 0.12213820132218554`}}]}}, 
     {Arrowheads[{{0.02497416929272219, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.004055640126885765, 0.24709256790858378`}, {
        0.22671359064234559`, 0.3067535859375701}}]}}, 
     {Arrowheads[{{0.024974169292722198`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.004055640126885765, 0.4317079525239684}, {
        0.22671359064234559`, 0.49136897055295475`}}]}}, 
     {Arrowheads[{{0.024974169292722198`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.004055640126885765, 0.6163233371393531}, {
        0.22671359064234559`, 0.6759843551683394}}]}}, 
     {Arrowheads[{{0.024974169292722198`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.004055640126885765, 0.8009387217547377}, {
        0.22671359064234559`, 0.8605997397837241}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.004055640126885765, 0.9855541063701222}, {
        0.22671359064234559`, 1.0452151243991084`}}]}}, 
     {Arrowheads[{{0.02497416929272219, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.004055640126885765, 1.1701694909855067`}, {
        0.22671359064234559`, 1.229830509014493}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.23482487089611653`, -1.2298305090144932`}, {
        0.4574828214115764, -1.170169490985507}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.23482487089611653`, -1.0452151243991086`}, {
        0.4574828214115764, -0.9855541063701221}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.23482487089611653`, -0.8605997397837238}, {
        0.4574828214115764, -0.8009387217547375}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.23482487089611653`, -0.6759843551683392}, {
        0.4574828214115764, -0.6163233371393528}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.23482487089611653`, -0.49136897055295464`}, {
        0.4574828214115764, -0.4317079525239683}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.23482487089611653`, -0.30675358593757}, {
        0.4574828214115764, -0.24709256790858364`}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.23482487089611653`, -0.1221382013221854}, {
        0.4574828214115764, -0.062477183293199054`}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.23482487089611653`, 0.06247718329319919}, {
        0.4574828214115764, 0.12213820132218554`}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.23482487089611653`, 0.24709256790858378`}, {
        0.4574828214115764, 0.3067535859375701}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.23482487089611653`, 0.4317079525239684}, {
        0.4574828214115764, 0.49136897055295475`}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.23482487089611653`, 0.6163233371393531}, {
        0.4574828214115764, 0.6759843551683394}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.23482487089611653`, 0.8009387217547377}, {
        0.4574828214115764, 0.8605997397837241}}]}}, 
     {Arrowheads[{{0.02497416929272219, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.23482487089611653`, 0.9855541063701222}, {
        0.4574828214115764, 1.0452151243991084`}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.23482487089611653`, 1.1701694909855067`}, {
        0.4574828214115764, 1.229830509014493}}]}}, 
     {Arrowheads[{{0.024974169292722205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.46559410166534737`, -1.2298305090144932`}, {
        0.6882520521808073, -1.170169490985507}}]}}, 
     {Arrowheads[{{0.02497416929272221, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.46559410166534737`, -1.0452151243991086`}, {
        0.6882520521808073, -0.9855541063701221}}]}}, 
     {Arrowheads[{{0.02497416929272221, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.46559410166534737`, -0.8605997397837238}, {
        0.6882520521808073, -0.8009387217547375}}]}}, 
     {Arrowheads[{{0.02497416929272221, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.46559410166534737`, -0.6759843551683392}, {
        0.6882520521808073, -0.6163233371393528}}]}}, 
     {Arrowheads[{{0.024974169292722205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.46559410166534737`, -0.49136897055295464`}, {
        0.6882520521808073, -0.4317079525239683}}]}}, 
     {Arrowheads[{{0.02497416929272221, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.46559410166534737`, -0.30675358593757}, {
        0.6882520521808073, -0.24709256790858364`}}]}}, 
     {Arrowheads[{{0.024974169292722205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.46559410166534737`, -0.1221382013221854}, {
        0.6882520521808073, -0.062477183293199054`}}]}}, 
     {Arrowheads[{{0.024974169292722205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.46559410166534737`, 0.06247718329319919}, {
        0.6882520521808073, 0.12213820132218554`}}]}}, 
     {Arrowheads[{{0.024974169292722205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.46559410166534737`, 0.24709256790858378`}, {
        0.6882520521808073, 0.3067535859375701}}]}}, 
     {Arrowheads[{{0.02497416929272221, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.46559410166534737`, 0.4317079525239684}, {
        0.6882520521808073, 0.49136897055295475`}}]}}, 
     {Arrowheads[{{0.02497416929272221, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.46559410166534737`, 0.6163233371393531}, {
        0.6882520521808073, 0.6759843551683394}}]}}, 
     {Arrowheads[{{0.02497416929272221, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.46559410166534737`, 0.8009387217547377}, {
        0.6882520521808073, 0.8605997397837241}}]}}, 
     {Arrowheads[{{0.0249741692927222, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.46559410166534737`, 0.9855541063701222}, {
        0.6882520521808073, 1.0452151243991084`}}]}}, 
     {Arrowheads[{{0.024974169292722205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.46559410166534737`, 1.1701694909855067`}, {
        0.6882520521808073, 1.229830509014493}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6963633324345783, -1.2298305090144932`}, {
        0.919021282950038, -1.170169490985507}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6963633324345783, -1.0452151243991086`}, {
        0.919021282950038, -0.9855541063701221}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6963633324345783, -0.8605997397837238}, {
        0.919021282950038, -0.8009387217547375}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6963633324345783, -0.6759843551683392}, {
        0.919021282950038, -0.6163233371393528}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6963633324345783, -0.49136897055295464`}, {
        0.919021282950038, -0.4317079525239683}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6963633324345783, -0.30675358593757}, {
        0.919021282950038, -0.24709256790858364`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6963633324345783, -0.1221382013221854}, {
        0.919021282950038, -0.062477183293199054`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6963633324345783, 0.06247718329319919}, {
        0.919021282950038, 0.12213820132218554`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6963633324345783, 0.24709256790858378`}, {
        0.919021282950038, 0.3067535859375701}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6963633324345783, 0.4317079525239684}, {0.919021282950038,
         0.49136897055295475`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6963633324345783, 0.6163233371393531}, {0.919021282950038,
         0.6759843551683394}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6963633324345783, 0.8009387217547377}, {0.919021282950038,
         0.8605997397837241}}]}}, 
     {Arrowheads[{{0.024974169292722184`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6963633324345783, 0.9855541063701222}, {0.919021282950038,
         1.0452151243991084`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6963633324345783, 1.1701694909855067`}, {
        0.919021282950038, 1.229830509014493}}]}}, 
     {Arrowheads[{{0.024974169292722174`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9271325632038091, -1.2298305090144932`}, {
        1.1497905137192688`, -1.170169490985507}}]}}, 
     {Arrowheads[{{0.02497416929272218, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9271325632038091, -1.0452151243991086`}, {
        1.1497905137192688`, -0.9855541063701221}}]}}, 
     {Arrowheads[{{0.02497416929272218, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9271325632038091, -0.8605997397837238}, {
        1.1497905137192688`, -0.8009387217547375}}]}}, 
     {Arrowheads[{{0.02497416929272218, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9271325632038091, -0.6759843551683392}, {
        1.1497905137192688`, -0.6163233371393528}}]}}, 
     {Arrowheads[{{0.024974169292722174`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9271325632038091, -0.49136897055295464`}, {
        1.1497905137192688`, -0.4317079525239683}}]}}, 
     {Arrowheads[{{0.02497416929272218, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9271325632038091, -0.30675358593757}, {
        1.1497905137192688`, -0.24709256790858364`}}]}}, 
     {Arrowheads[{{0.024974169292722174`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9271325632038091, -0.1221382013221854}, {
        1.1497905137192688`, -0.062477183293199054`}}]}}, 
     {Arrowheads[{{0.024974169292722174`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9271325632038091, 0.06247718329319919}, {
        1.1497905137192688`, 0.12213820132218554`}}]}}, 
     {Arrowheads[{{0.024974169292722174`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9271325632038091, 0.24709256790858378`}, {
        1.1497905137192688`, 0.3067535859375701}}]}}, 
     {Arrowheads[{{0.02497416929272218, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9271325632038091, 0.4317079525239684}, {
        1.1497905137192688`, 0.49136897055295475`}}]}}, 
     {Arrowheads[{{0.02497416929272218, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9271325632038091, 0.6163233371393531}, {
        1.1497905137192688`, 0.6759843551683394}}]}}, 
     {Arrowheads[{{0.02497416929272218, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9271325632038091, 0.8009387217547377}, {
        1.1497905137192688`, 0.8605997397837241}}]}}, 
     {Arrowheads[{{0.02497416929272217, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9271325632038091, 0.9855541063701222}, {
        1.1497905137192688`, 1.0452151243991084`}}]}}, 
     {Arrowheads[{{0.024974169292722174`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9271325632038091, 1.1701694909855067`}, {
        1.1497905137192688`, 1.229830509014493}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1579017939730398`, -1.2298305090144932`}, {
        1.3805597444884996`, -1.170169490985507}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1579017939730398`, -1.0452151243991086`}, {
        1.3805597444884996`, -0.9855541063701221}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1579017939730398`, -0.8605997397837238}, {
        1.3805597444884996`, -0.8009387217547375}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1579017939730398`, -0.6759843551683392}, {
        1.3805597444884996`, -0.6163233371393528}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1579017939730398`, -0.49136897055295464`}, {
        1.3805597444884996`, -0.4317079525239683}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1579017939730398`, -0.30675358593757}, {
        1.3805597444884996`, -0.24709256790858364`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1579017939730398`, -0.1221382013221854}, {
        1.3805597444884996`, -0.062477183293199054`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1579017939730398`, 0.06247718329319919}, {
        1.3805597444884996`, 0.12213820132218554`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1579017939730398`, 0.24709256790858378`}, {
        1.3805597444884996`, 0.3067535859375701}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1579017939730398`, 0.4317079525239684}, {
        1.3805597444884996`, 0.49136897055295475`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1579017939730398`, 0.6163233371393531}, {
        1.3805597444884996`, 0.6759843551683394}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1579017939730398`, 0.8009387217547377}, {
        1.3805597444884996`, 0.8605997397837241}}]}}, 
     {Arrowheads[{{0.024974169292722184`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1579017939730398`, 0.9855541063701222}, {
        1.3805597444884996`, 1.0452151243991084`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1579017939730398`, 1.1701694909855067`}, {
        1.3805597444884996`, 1.229830509014493}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3886710247422707`, -1.2298305090144932`}, {
        1.6113289752577304`, -1.170169490985507}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3886710247422707`, -1.0452151243991086`}, {
        1.6113289752577304`, -0.9855541063701221}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3886710247422707`, -0.8605997397837238}, {
        1.6113289752577304`, -0.8009387217547375}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3886710247422707`, -0.6759843551683392}, {
        1.6113289752577304`, -0.6163233371393528}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3886710247422707`, -0.49136897055295464`}, {
        1.6113289752577304`, -0.4317079525239683}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3886710247422707`, -0.30675358593757}, {
        1.6113289752577304`, -0.24709256790858364`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3886710247422707`, -0.1221382013221854}, {
        1.6113289752577304`, -0.062477183293199054`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3886710247422707`, 0.06247718329319919}, {
        1.6113289752577304`, 0.12213820132218554`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3886710247422707`, 0.24709256790858378`}, {
        1.6113289752577304`, 0.3067535859375701}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3886710247422707`, 0.4317079525239684}, {
        1.6113289752577304`, 0.49136897055295475`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3886710247422707`, 0.6163233371393531}, {
        1.6113289752577304`, 0.6759843551683394}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3886710247422707`, 0.8009387217547377}, {
        1.6113289752577304`, 0.8605997397837241}}]}}, 
     {Arrowheads[{{0.024974169292722184`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3886710247422707`, 0.9855541063701222}, {
        1.6113289752577304`, 1.0452151243991084`}}]}}, 
     {Arrowheads[{{0.024974169292722188`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3886710247422707`, 1.1701694909855067`}, {
        1.6113289752577304`, 1.229830509014493}}]}}}}, 
   GraphicsComplexBox[CompressedData["
1:eJx1nXucjOX//0dKOtFJckxRPqXSQQeV3kkHOSURIhGSCB0QoglFJOWcHJZC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    "], {{{}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwVzrsrxXEYx/Gv+704WOSSwWBTBgqLQWaDS5zBQRl0lIEwuAwsBh2Kc2Jm
kCKD0YTk8jcwKWWQy0B4/YZ37+f5PM/3+f0aE9N9yawQwjx+i0I4KgxhCHnF
IeQjIAvHsjhi6gK8qbM5B7moQiUm3XguCeHTvIq/uJqn5HX8o6/lBtTjFY9m
T5jDm36Wz+0l0aHvxLDbI4hjRt4rK7M3qk9gVd1dGsIKD5hVcD/HeJDX+ZsT
GMMoJjCOP7yYr7jb4kZKfYkL/SqWzJcx7zsLWMSafEPWFL2Lvo99dY/3e7xr
1sw7nEE6+l+zd9kHDlGuP+BrtzZxYucUKbe2cCPbxrmsNbopSyMTWd7m/am8
LfpfO+18xWd8x118yw+4R439f+HcQAU=
          "]]},
        "1.29`"],
       Annotation[#, 1.29, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV0jtI1WEYx/HXEDJtUiLDEJwc8tBgkJdoUhGvkLoplNDFM+iQx6GbLhre
BgdFsCCjc5xMxVGsxUyNGgIFbfQSDeHi4KXAz3/48n2f3/O8z3vO4RS0d93r
TAshvMW7jBBmMkNIIYk7l0L4lhXCG/kPvJdN44tsBZOy75g3d+VyCAt8ihPk
qOfMzGMWN2UfOcnVeiketmsIMecPeiN2vVYPoB8TsnEMyVewZm4Kxe4Pqg+d
M5xfmnmFZ3iB50iXj+knnK/zU87j7ugdrtAf5S7vdOKq8/1oh71xdQeeoEXW
jB75EnrNHav7+AD7OFKXogy3cVFWwgmO8QUu4rToc6DQ23G76u2vQy3O1Hlm
HvIiqmSVyJFl44FsAY/c/61+zL+wgz31P7P/cYq/5s64Ve+ut9r4lroYf6Lf
jZs5xkW4gT3sojH6D2DZnSbku9+g/hm95Z11MxtYxRq+Ykteo//ZeSe6y9v8
KfoO8nI7qviaOhebenNcITsHap5Y+Q==
          "]]},
        "0.86`"],
       Annotation[#, 0.86, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwVzj1IlXEUx/F/i5gK6lRNerXJNlHRQAPFm07qtUjTKXRoULlCWRDi6hV8
SdFoScpNqS1JI6kIcTAEQSrIt82XhqhARcPPM3z5nvM753+eJ3avJ9F9IYQw
jXh6CKsXQ/iKGqxnhlDNL+W1WMwI4T3ykI9Xsjp8UW9xjAvQ6E1lVggN/F2/
Y/aNd/kHJ+Q/3d5Sb2MzmkV70R35hvkduwVutKmXMKtvwZGdY5SjAtfRLk/z
7jdX6qvQ6c0N7zs4x+yvWTb/41w+xEH0D+bb6MUvfZLf2rmPIv013HLvNuZl
XSiTpdlrkbXiLpLRvu89kWcibieLb3I/N3EON3IzEjjBnmwfQ/ivT/EHt/qR
1PfigfsPsSQbwGNZzF6f7BGeqWt9e4pTZoU8yFeju/ycM8zH1E8xigmMR/8o
/2M+7G6Jeka9gmX9CGbtzGHEd0YxhnH5vKzY3qR+Cm/U9d6/5k9mpfyRy/gz
Xza7hFP9GRZwRf+O19x7gXPib1pV
          "]]},
        "0.43`"],
       Annotation[#, 0.43, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwVzz9I1HEYx/GvNKjlHxBycDKQ7tQcLG3IwSuDSLmCXKShJLUm7xxsaIiC
rGxqUgchO/9cJRhtpy4d3SYNif1zcBCLWjwU7Q8t+brhzft5Ps/zfH/8jt1I
XkkUhRCm0F4awpmyEGK8VhLCwuEQPvJrPitb5K/6DC9jCZ+OhPDBbBWXsK6P
85y9Lny28wXHEUFaFseG+jtHuR4N6NbX+n6P+x/qbdlPzvNV2aa399W/sIc/
+I0t+br5dbt17vvUWbzR96LcvALn7HbgPPrlNbJ/fFHfiSE3MfdJjpr9N4tw
0NfzLnZwEqfQjFa0YA/f7A27afTGXXUOy/rbuGxeqR/wnZu4hTvyJruP5Edx
zU514T/4CSe4hgd5CEkcsr8ty2Mcxfoxfueth3hsZxT3vH8fOdkonsqi9h7I
RjCtvuA2xdNmjZziEzzDM1xp/kL9EmnM4xWq5H8L3/fuafWCegXv9RN4ayeL
Z74zheeYlK/I2grv62exqO50n+EDw4BaMw==
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV0D1IlWEYh/FHSFCPQ4IEkmYf6GgqpLgIphJ+BZ3gODTlYoMYaKCU4qCG
4QcSrepxcDBdPUN+gGQtCkEqLnpoijDBdDDExd87XFz3/X/u57lf3nsdr+Pd
GSGEJBqyQtjJDuEx5+WEcBO1+lXMy+qRkh1wIRfhDp7ob+eG0GTuSP1VluYt
bpXt8i/9T97HHg5j7Gwf7UjrE7xoLo6/Zk5QgUp8liVwHuX8iKtQjRf6B/a/
dP9UfcNb/ziTO2S/ORs5yEIuYjhG2nmn2VL3u9WbWNG/QonzUjyzI47n6JKX
yzLNJfTteKeuc/8tN0S7uD7ax438HxdoQSua8RRtuMQfcwPeLfPGmPob1vWD
6HJeoO+xpxdvMCR/aHZaXow+M3e5nz/xCN/nYX6P0eibzJ/JzpFETD8X7fLW
OObMJDHm/Q/4LpvEgqzM3IRsElP4GP1/95fk5dH/MlPBKV7mfGer6jV8wQbW
cUt+5XzG/Rp1Sr2NH/pZXANo7lUB
          "]]},
        RowBox[{"-", "0.43`"}]],
       Annotation[#, -0.43, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwVzj9I1GEcx/EnHfpjCFFDgQ12XlKb0eA/0NPJajFwKb07OqFS/ANOgQqp
0ZQg6lDdNbVoitidg+CSi0VDKUGCDZFRFkFDYAYZvX7Dm/f3+3k+z/P7ld/o
u9p7IISQQ+JQCM+OhLDOs7zBc5FLZHhuzmMRSyjgnfzN4RBadMuOhnDZvIKn
9kt4G72HGCoQxxX5ad1WvW3zluwzf+A22Q5/sX/l7/iGj77z3tkm0vhkT/G8
3jUU2YtRp1uPBVkSB2W/uEHWiAQy9rjvd7u/Zz6p84dPcY/sDP+zl3MFYviJ
bWd98nPu3jGvYtnej4TzJiS9n0IaA/IWWalext6Je+Zm98e43dlxvs4nuIP3
8Re3cBs30Y0uFLn3Q2/Uu1XmSfMaXtjHcFdnBIO+M4Rh3Jdf0H2odxYPnFfy
OD/hLJ/nx9GOXPS/+ruy35jFMfsMv/LWBPI6BUx5fxqvZdNYll2M3pJlkcMj
ebX7eXkNXurURm/xEv8Hs05Y2w==
          "]]},
        RowBox[{"-", "0.86`"}]],
       Annotation[#, -0.86, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwVzzkvhFEUBuA7sWSGQcUIsUynkOEHWKajsoWCQjHFJIw1EUuUev4GnYhS
BNFZEwkSoxyCREahtDxTPHnvOfec++VLZhbHFiIhhA3+YiEUqasKoYpqYtRS
Q0U8hEqizq3mVqtDaJFx9bYM7nZkRC7rLdGozprbjIYwr54jx5TeJOv6R5yb
26LT7pr61fnbfT9peonq9ZV2ZJcslylZJlfosJuzO+L9YYb4UbebycpDBvUG
SOg1kNE7YMb+m3pWPpOnoA5E+LXzae5PTrv7kh/qonyXk3pJ359w7tbrIsUL
BUb1dzk2N06b2RH1nfOT96/N3HDBFZc86D9y5pw3eyKf5GnpH+z1eGNQNqub
uHe3L9MyIRuoZ49dbr31D3W6PRA=
          "]]},
        RowBox[{"-", "1.29`"}]],
       Annotation[#, -1.29, "Tooltip"]& ], {}, {}}}], 
   GraphicsComplexBox[CompressedData["
1:eJx0vXdgU9X//19cICAIDoaCCggOlL1EfblABQFRUJDhAMWByvstDpwVwQEK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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], 
         LineBox[{3250, 4415, 3498, 4162, 4691, 3309, 4692, 4022, 3783, 4693, 
          3784, 4024, 4023, 3258, 4428, 3506, 4427, 3507, 3643, 4838, 3642, 
          4429, 3508, 4166, 4704, 3311, 4705, 4034, 3790, 4706, 3791, 4036, 
          4035, 3266, 4441, 3516, 4440, 3517, 4457, 4458, 4456, 3648, 4071, 
          3518, 4055, 4074, 4170, 3313, 4075, 4043, 4044}]},
        "1.32`"],
       Annotation[#, 1.32, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwVz8kuwwEUxeFrb34DS1Zi2qih7UKQmBaIYVNseR3bllUnMfwlbQlRJZUQ
YuMh8Aw+i19Ozsk5N7kDuaO1w46IyKHVGVHvj8h0R7z1RLziF5/8D/3GF+67
ItKymu6jzW5fxJ1si+5gG035s+4tvdb70H/iWzjg3/l9uoe5Xh37lCzhb2xW
3UhkS3QFy2jIG/YJvdB70a/zNWzybX6DriPrZtl+QnbOX9lk3SjJZmkGaVzK
z+zLtKrX1K/yFSzyD/wCnceMmwX7YVmFL9mMu5GXjdAxjKIoP7E/pcX/f/QL
fB7TfIOfoilMunlsPyRr84P0D9syPcw=
          "]]},
        "1.1`"],
       Annotation[#, 1.1, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwVzz1P01EcxfHLzmN3RRESsYuokZJoBKnigJoUHAQC6GBhkZeBg5LAgu2A
JG2BBGmrEkoCLUgMoAwYNl4F6ivw8x++OTkn5/xu7tXXM6m3dSGEBRzXh7Ab
CyHZGMLvphBO8Rf/8AcXOMdBQwj9Oju6RzavWkL4Lhunk5jAofyX7j7d1jvT
/8kfY4qfRhpv8KRZx/6+ToXfs3nhRkWWosMYQk1es9+mm3on+lV+F6P8GEbw
MvqDm0X7bp1vfMVmwI0NWZI+xiNsyb/al2hZ74f+F76MZ/xzPMUget3M23fp
lPiiTY8bOdldmkB39IZ8xX6Vftar6hf4PPr4h9EtPMA9N7P2cZ11fsUm7kZG
dp3eQCcK8qz9Ml3T29LP8B9xi7+NLtxEws0F+w6dVf6TzRU35mWXaCsuY0k+
F92geb2y/gf+Pdr5DlxDG+64+c6+VSfHL9rE3JiVNdEWNCON/7dUUmE=
          "]]},
        "0.88`"],
       Annotation[#, 0.88, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV0LtL1XEcxvFPQxdI7aJJ1pIplqCCtzK7ESlokznUZEKrlpNZU6E1qIO3
ljqeQsmj0pCiURqUDjWYrv0JRSHiljn1+g1vnu/n+TzPh985hXe7Wu/viYgM
+rIiOg5FdCKdE5E6GtHLu2fupzuHI/7hL7qzI3bpAP+VXJ78A14576A5CznI
Ro/+Q7yReW0e1HlkHqH7jkTsx1481j9AR/kTcifkn/Aq3TxuLsDJxEef/lO8
lZk0j+k8M7+geW4cQy769fPpS/6U3Cn5AV6tmyXmMyjFWQzpD2NeJmNO6YyY
J+hpN4pQiDH9YjrJn5UrkX/Ou+BmjbkW53EuuaH/wb7O+yNNm8cxo1vuRkqv
jI4n/539mlwZreCleZfdvM77I99Ap3U/2zd6f6GzyS3M2V/UmdKppxlabf9d
rope4k3zrrnXwtuUv0kXdL/at3p/o+/Ni1iyb9Z5p9NE55LfZ78uV0dv8OZ5
je618bbk7yTfpLtu3+69QVfNK1i1v62zpHOLLtOr9r/krtCfMp94P+hvbPC3
k29w/z/TTFmP
          "]]},
        "0.66`"],
       Annotation[#, 0.66, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV0DtI1WEYx/EnbdCszlEoizQw8IKGlVSQoWM2N2TQ5mXsHMGh0pZKLEgx
yrIhzlGiyLKhmnLKQWt3FhWVEFFnU8HPf3j4Pu/v8v4vVR3ZW5lDETFheo9G
zKcj7h6LOIebpRELtC28nYoYOR6xbX+Jd5zbTT9/SbZTZxG7sJg/WxZRhMu0
bloN7uguyv/HDt0x/q79HXY5d5qB5Hmy93Q2MYNp/pz7UsnzaVlaPRbQluUL
MaOb4x+257HHOWuG+Huy93V28QGe5P+RO4H7tIe0RkzRVuTT2Kf7kV9q/4SP
km81o/wj3vlx8p34BCv4f+XOYAntKa3JfRW0NflKHNSd4p+1f8Pnzs/Me/4p
nRc65TiU/Pvkv8lV4WnaMO2q++pp/+Qb8JXuD/55+08cdX5tPvBrdN7oVONb
rOOvyNViLW2M1uy+a7R1+WbM6f7iX7dP47hz3kzyL+rkdC5gPiF/Va4RL9HG
aa3uu0HbkG/DL7q/+TftMzjl/NV857fqfNZpwUm8wl+Tu4wHMVdYeA==
          "]]},
        "0.44`"],
       Annotation[#, 0.44, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwVzTtLFnAUx/EjaIGp5aOglJWi0BDp4OBuLiZoaA2VNQRqEJSXJjUj0dDC
S4JFglAtNeViUL0A6wVo4F2exGyzNrW0zzMcvuf8Lv9/ye37jffSIuKt2c+O
+HwiohY/4SXcM4uJiF38QqvDIlzKNVkRy1h9POJxTsSKvR9r3BfNXf5X2Sad
ObyCafxl7wV+o12lFeNP3RX5bazXHeL/sg/jZXeDecBfkG3WmcebmJH613vp
+J12i1aKO7pr8r/xmu5z/h/7BN5wXze9/B+yrTpJbMNj/FXvZeIm7Q7tHP7V
3ZD/hy26r/gH9ilsc7eawdR/su06O9iBCf6a93JT/9M6aecxg5aUP4Iduq/5
R+1vsMvdaUb4h7LdOgfYg4X8dbkCDP/30ip4CdqmfB4+1H3Hz7e/x0fuPjPJ
z9YZ0MnCQTzD35A7jTm0J7RK752lbckX47DuB36JfQafuZ+aaf4pnVGdkziG
ZfykXCkW0cZpVd67QNuWL8dJ3Vl+hf0jvnS/MAtm3vwHNc1deQ==
          "]]},
        "0.22`"],
       Annotation[#, 0.22, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV0jtI1XEYxvH3aESJ3dFwaDAdGsoavWxltWiQRC1mIQ0ppoN6XLx1QUOj
C1okGqUNUlCtQTlpQ03lcrSoJsutoHOiwMjPf3j4/t/n9zzv/3JOaXNHQ3sq
ImaprjCidFtE+9aIlzsiOrDMvJdOOXu0PaJyS8RDrMKflNkZ8QOnedVYhPO6
GfkFPKDbbc8b12k8aK6gRufPZQ/rPMMj+JuW7MvhC14tluB73WX5D1il22vP
ous+rDFX0wXnr2XrdF5hPf6jj/at4RzvBO7Bz7qf5L/gUd2r9nx1fQ2Pm49R
m/N3sqd13uIZHHW+at8InpTJwxt0h3dJviF5V1ySX6YMnUueAdP8cbmN8ud5
5bxf7pmlMV4OG/XP0rj5rmyPTpO5D7/Jf6cVatFfxX7+fblC+VbePt5/e4I3
yUvhRf0WmjJPmAd0Ws1DmJXPJc9Bncn3x2H+lNwu+S7eft4m82aa4RVgV/Ke
9Nj8wHxdJ22+iXnun0+p5PfR34C3kv+OXIl8P++QnUXmYnrC242D+pfpqfkP
/qXbelfwHq4DrkJgnw==
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV0TtIFXAcxfFfOLR4r+UQ5SAI2mNL18xJJQM1LCuHHhoqZahNmpCp4DUF
HSqv4NhjcPHVVIGPpQKtQWtKo8JFr0tLOkUfhy+HczjnB3/+BU0dde2HImIK
yWREy9GIZpw5EnE/EZGQTeZG9GdH7MraZNs0gx30yRtzInrohF5a/zZ/C/vu
jPN79C+K9JvsD8vSug9tfsgaZd/pJjbQLa+376TP9Mb0r/CXsevOKJ+hOyjQ
b7DPkj3V7bD5IrsuW6GfsYp2+UX7Vjqml9Kv4i/gtzuD/C/6E/n6l+z/YVS3
xWZZVssv0CUsolledvBWOqL32I3zfCm+udPLf6XryNOvtN/DsO5NmzeyCn6W
zmMON+Ql9tdoSq/bjWL+LD6508V/pB9wTL/U/g8Gda/avJad41/QV3iJevlJ
+1o6oPfAjSK+EO/d6eTf0bfI1S+xz6Bft+bgX2XF/HOaxjiq5Xn2lbRX754b
J/jjmHXnLj9Dp5HUP22/hUe6FTbDslN8ij7BEMrlSfsy2qN3x40En411rOE/
bclfhA==
          "]]},
        RowBox[{"-", "0.22`"}]],
       Annotation[#, -0.22, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV0jtI1XEYxvE3MhDKjhfIwcKayiZBrZZSp4wM8kzl4gUHNcg6QRFZCgpG
51iNeRm95FhZLWlhOngBL4Hl1KA2NbVoU33O8OX5vw/P8ww//qdau5K3D0TE
FFJHIjbyIxrzIlbpOtZwl38tEdFJXxZGDByNqHdfxW5BxC/sYBul8kn9f3gh
26GzwGtwz9F5fEU7v1a/lWbkem3WuKvxw84WvmMTJfJ1+vtIy7bofORddk/T
D3iPZn6VfiN9KvfQZqW7Ast2VrCERRTLX9L/gwHZmzpTvIvuCfoak7jBL9NP
0n65lM0z7tOYtfMZM/iEIvlK/d/ok23QGeFVuF/RYQzhOv+E/hXaK3fL5nF3
Cd7ZmcZbvEFC/qz+Lnpk63QGeWXuZzSDdPYt+AX6tbRbrs1mvjuBCTuTGMcY
Dsuf1P+JR7I1Oj28Unc3fYLHqOYf0r9AH8g12cxxH8SInVEMYwi58sX6W7gv
e14nxTvm7sr+Q7iDc/y/dsrpvey7+97HHgbtPEcGaeTI5+l/wxfZDfofS+Zd
BQ==
          "]]},
        RowBox[{"-", "0.44`"}]],
       Annotation[#, -0.44, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwN0DtIlnEYhvFHSCswNXOwpVIn/WyoyFOD5Si0qJCHJRStIZXy+0xtSWtW
a87DEuRhVdc8gJMFIiJOppYumZCTJvQbLq7/87z3/QxvQWt3XVdaRHxGW2ZE
eXZEBd5kRXy/GjHAZeZv3lu+r3F+TsSjKxGz/JdXciNO+CGm7ap5hlvkE7ql
SLqzpNvDJeZF703fv3KebLnOOP/mZfeOuAyf7O7zGNfL39S9hU53FnRf8A3z
vPeG73OcI3tbZ5R/8ZJ7B1yKYbsEj3Ct/DXdPHS4M63bzrnmKe91379wpmyR
zhDv8KJ7P7gQb+0KeJBr5C/pPuB+maduXTRnYMKdSYxjDJflr+tto0+2SqfX
Lt/cwykkUWn/z517nJJr9j7DKT648xGjGEG6fLb+BpKyd3We22WZ2/kZOnDH
/lg/wa/kGrz/4Ajv3HmPIQwiTT5dfw0vZUt0mu0umJ9wExpRbL+vX8Rdco+9
97CLXndeI4Ukzv2PM7lVNzplC71/2p3yLu9jD4f4D+q8Wsk=
          "]]},
        RowBox[{"-", "0.66`"}]],
       Annotation[#, -0.66, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV0D1Pk3EUhvGjAypapE2UBJGEpDoZdWBkM8pApIkJLJRBQoJaX0ZbVy0s
LIh+AGBEP4IywCAh2pD6FANMEHxho3UytvpjuHKd/znnPsnz9E08u/f0RETM
4vK5iEeZiNsdEQm+olevcT5iB3X1Njd4LRXxwe4qD3VG/Nb7Y1ZIR6yrH3NN
/g6eqAf5G7aQNR+W+ct3ucnr7nx07xPn9Fp6LfeKshvqEh/IDuOFOsc/8B3X
zPMybbJjfIorWHHvC4/rneaTXJb9bH+aj2RHMKMe5Qbq6D/+D3Y7ZR5ymmvY
dC/hgl6G23hetmL/DTdlx/BWned/aGHAvGi3W+Y5X+JdVN3b4ZJeD7fzguym
/UU+4/vvyy+pJ/isdztumb+ym5V5yVd4H4l7e1zWu8opXpat2n/HF2Sn3Hmv
fsBd3hcxZP7a7nWZOb7Bv46/2b2fPK93k9M8KZvYP/T+D2qdVBQ=
          "]]},
        RowBox[{"-", "0.88`"}]],
       Annotation[#, -0.88, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwVzzszA3EUhvGzFMb9XmIUSqVSj1EkaWgso8VGLfT4APhCFGFiGTNuWTXq
JApF/BTPPOd/znnP7M7vVitZEhEp5oYi9iYiVkYi3vCK9mhECwXa5k3u8M1w
RMPuNa+PRfzo/Zpl4xF1dZXf5VdxqF7jAk0smJdlulzicOMW9+7VuaKXcNe9
muydvWP+lC3hRF3mb3xh0Xxbpl8m5QF+QO5ezjt6g9zLp7IN+2fckt3AuXqT
O///iyXzA7uTMvs8xS94dO+ZM71p7uML2dz+JYfv3ZK/Uqfc451g2bxmd0bm
iGf5A0/uFfwH+ns5yQ==
          "]]},
        RowBox[{"-", "1.1`"}]],
       Annotation[#, -1.1, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], 
         LineBox[{3119, 4196, 3340, 4193, 4195, 4194, 3118, 3821, 3112, 4720, 
          3667, 3820, 3819, 4480, 3666, 4479, 4478, 4096, 3331, 3532, 4719, 
          3111, 4182, 3330, 4179, 4181, 4180, 3110, 3809, 3105, 4716, 3660, 
          3808, 3807, 4467, 3659, 4466, 4465, 4092, 3321, 4450, 4451, 4086, 
          4087, 4065, 3320, 4062, 4064, 4063, 3104, 3806, 4049}]},
        RowBox[{"-", "1.32`"}]],
       Annotation[#, -1.32, "Tooltip"]& ], {}, {}}}]},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1.5, 
      FormBox[
       RowBox[{"-", "1.5`"}], TraditionalForm]}, {-1.25, 
      FormBox[
       RowBox[{"-", "1.25`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.75, 
      FormBox[
       RowBox[{"-", "0.75`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.25, 
      FormBox[
       RowBox[{"-", "0.25`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.25, 
      FormBox["0.25`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.75, 
      FormBox["0.75`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}, {1.25, 
      FormBox["1.25`", TraditionalForm]}, {1.5, 
      FormBox["1.5`", TraditionalForm]}}, {{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2, 
      FormBox["1.2`", TraditionalForm]}}, None, None},
  ImageSize->504,
  Method->{"TransparentPolygonMesh" -> True},
  PlotRange->{{-1.6152562362737912`, 
   1.6152562362737912`}, {-1.3152562362737912`, 1.3152562362737912`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 ImageSize->{508, 410},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->247304580]
}, Open  ]],

Cell["\<\
Arrows for velocity field are green, equipotential lines are red, and \
streamlines are blue.  Note orthogonality of equipotential lines and \
streamlines, streamlines everywhere parallel to the velocity field.\
\>", "ExampleText",
 CellID->62944834]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["2.2 Point Singularities",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/PointSingularities"]], "Tutorials",
 CellID->341631938],

Cell[TextData[ButtonBox["3.3 Implementation",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/Implementation"]], "Tutorials",
 CellID->74225746]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["InfluenceCoefficients",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/InfluenceCoefficients"]], "MoreAbout",
 CellID->28935711]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"UniformFlow - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 3, 7.8072000}", "context" -> 
    "Aerodynamics`InfluenceCoefficients`", "keywords" -> {}, "index" -> True, 
    "label" -> "Aerodynamics Package Paclet Symbol", "language" -> "en", 
    "paclet" -> "Aerodynamics Package", "status" -> "None", "summary" -> 
    "UniformFlow[n, \\[Alpha]] computes a collection of n unit velocity \
vectors inclined at an angle \\[Alpha] to the x-axis.UniformFlowPhi[r_f, \
\\[Alpha]] computes a list of velocity potentials at r_f for dimensionless \
uniform flow inclined at an angle \\[Alpha] to the x-axis.UniformFlowPsi[r_f, \
\\[Alpha]] computes a list of stream functions at r_f for Dimensionless \
uniform flow inclined at an angle \\[Alpha] to the x-axis.", "synonyms" -> {},
     "title" -> "UniformFlow", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/UniformFlow"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[7557, 229, 460, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->76469764]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 132303, 2816}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[604, 21, 76, 1, 70, "SymbolColorBar"],
Cell[683, 24, 191, 5, 70, "LinkTrail"],
Cell[877, 31, 2780, 65, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3682, 100, 54, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[3739, 103, 2033, 62, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5809, 170, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->488563757],
Cell[6392, 189, 475, 16, 70, "Notes",
 CellID->678462984],
Cell[6870, 207, 573, 14, 70, "Notes",
 CellID->54053493],
Cell[7446, 223, 74, 1, 70, "Notes",
 CellID->918694056]
}, Closed]],
Cell[CellGroupData[{
Cell[7557, 229, 460, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->76469764],
Cell[CellGroupData[{
Cell[8042, 246, 148, 5, 70, "ExampleSection",
 CellID->165292882],
Cell[8193, 253, 132, 6, 70, "ExampleText",
 CellID->513831186],
Cell[8328, 261, 99, 3, 70, "Input",
 CellID->66474307],
Cell[8430, 266, 157, 6, 70, "ExampleText",
 CellID->41115992],
Cell[CellGroupData[{
Cell[8612, 276, 367, 12, 70, "Input",
 CellID->103550962],
Cell[8982, 290, 895, 30, 36, "Output",
 CellID->81162724]
}, Open  ]],
Cell[9892, 323, 286, 9, 70, "ExampleText",
 CellID->35964272],
Cell[CellGroupData[{
Cell[10203, 336, 137, 4, 70, "Input",
 CellID->122330855],
Cell[10343, 342, 1038, 30, 36, "Output",
 CellID->7777265]
}, Open  ]],
Cell[11396, 375, 90, 1, 70, "ExampleText",
 CellID->118717855],
Cell[CellGroupData[{
Cell[11511, 380, 142, 4, 70, "Input",
 CellID->731712474],
Cell[11656, 386, 1088, 35, 54, "Output",
 CellID->138165972]
}, Open  ]],
Cell[12759, 424, 87, 1, 70, "ExampleText",
 CellID->286345785],
Cell[CellGroupData[{
Cell[12871, 429, 142, 4, 70, "Input",
 CellID->366527698],
Cell[13016, 435, 1174, 39, 54, "Output",
 CellID->1744437]
}, Open  ]],
Cell[CellGroupData[{
Cell[14227, 479, 125, 3, 70, "ExampleDelimiter",
 CellID->427059611],
Cell[14355, 484, 184, 6, 70, "ExampleText",
 CellID->659456463],
Cell[14542, 492, 163, 6, 70, "Input",
 CellID->64953589],
Cell[14708, 500, 304, 10, 70, "ExampleText",
 CellID->344772720],
Cell[CellGroupData[{
Cell[15037, 514, 740, 22, 70, "Input",
 CellID->374015860],
Cell[15780, 538, 608, 19, 36, "Output",
 CellID->121815841]
}, Open  ]],
Cell[16403, 560, 222, 7, 70, "ExampleText",
 CellID->382023890],
Cell[CellGroupData[{
Cell[16650, 571, 3463, 96, 70, "Input",
 CellID->1746043248],
Cell[20116, 669, 108673, 2044, 70, "Output",
 CellID->247304580]
}, Open  ]],
Cell[128804, 2716, 260, 5, 70, "ExampleText",
 CellID->62944834]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[129125, 2728, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[129547, 2742, 169, 3, 70, "Tutorials",
 CellID->341631938],
Cell[129719, 2747, 159, 3, 70, "Tutorials",
 CellID->74225746]
}, Open  ]],
Cell[CellGroupData[{
Cell[129915, 2755, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[130337, 2769, 166, 3, 70, "MoreAbout",
 CellID->28935711]
}, Open  ]],
Cell[130518, 2775, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

