(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     16487,        503]
NotebookOptionsPosition[     12929,        389]
NotebookOutlinePosition[     14642,        436]
CellTagsIndexPosition[     14557,        431]
WindowTitle->XToTheta - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"3.2 Validation Example\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/ValidationExample"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"AirfoilGeometry\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/AirfoilGeometry"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/XToTheta\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/XToTheta"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
XToTheta.html"], StandardForm]], "Input", TextClipboardType -> "PlainText"]}, 
            Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/XToTheta", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["XToTheta", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"XToTheta", "[", 
       RowBox[{
        StyleBox["t", "TI"], ",", " ", 
        StyleBox["\[Tau]", "TR"], ",", " ", 
        StyleBox["x", "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]converts a list of ",
     Cell[BoxData[
      StyleBox["x", "TI"]], "InlineFormula"],
     "-values to a list of theta-values using the van de Vooren \
transformation for an airfoil of thickness t and trailing-edge angle tau."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->41424115],

Cell["This function is specific to the van de Vooren airfoil.", "Notes",
 CellID->1067943069],

Cell["New in 1.0", "Notes",
 CellID->918694056]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->696288930],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->198842657],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->311150377],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->85203508],

Cell[TextData[{
 "Specify the shape parameters for a van de Vooren airfoil and a list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values to be used for discretization."
}], "ExampleText",
 CellID->450653109],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"t", "=", "0.12"}], ";", 
  RowBox[{"\[Tau]", "=", 
   RowBox[{"9.81", " ", "Degree"}]}], ";", 
  RowBox[{"x", "=", 
   RowBox[{"NDiscretizeUnitSegment", "[", "10", "]"}]}]}]], "Input",
 CellLabel->"In[2]:=",
 CellID->475849417],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.`", ",", "0.1`", ",", "0.2`", ",", "0.30000000000000004`", ",", "0.4`", 
   ",", "0.5`", ",", "0.6000000000000001`", ",", "0.7000000000000001`", ",", 
   "0.8`", ",", "0.9`", ",", "1.`"}], "}"}]], "Output",
 ImageSize->{345, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->392205299]
}, Open  ]],

Cell[TextData[{
 "Compute a list of \[Theta] values corresponding the the list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values."
}], "ExampleText",
 CellID->105439063],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Theta]x", "=", 
  RowBox[{"XToTheta", "[", 
   RowBox[{"t", ",", "\[Tau]", ",", "x"}], "]"}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->75917125],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.`", ",", "0.5910009410620323`", ",", "0.8632693526618288`", ",", 
   "1.089793647075696`", ",", "1.2980488921524616`", ",", 
   "1.5002280402036907`", ",", "1.7048671489501925`", ",", 
   "1.9208649027663336`", ",", "2.1618151247310746`", ",", 
   "2.459512923783499`", ",", "3.141592653589793`"}], "}"}]], "Output",
 ImageSize->{368, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->88992834]
}, Open  ]],

Cell["\<\
Using this list of \[Theta] values, compute points on the surface of the van \
de Vooren airfoil.\
\>", "ExampleText",
 CellID->109577907],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotAirfoil", "[", 
  RowBox[{
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"t", ",", "\[Tau]", ",", "\[Theta]x"}], "]"}], ",", 
   RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->261648493],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3b+O61odBeAgnoTXGAlxBaKBDglRIaRbpOQ28A4IgZQWQQHoQkMJMi0S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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->115009892]
}, Open  ]],

Cell[TextData[{
 "Note from the plot that the points on the surface of the airfoil appear to \
have ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values corresponding to the list specified for discretization."
}], "ExampleText",
 CellID->318374758]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["3.2 Validation Example",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/ValidationExample"]], "Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["AirfoilGeometry",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/AirfoilGeometry"]], "MoreAbout",
 CellID->534541978]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"XToTheta - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 3, 17.5728000}", "context" -> 
    "Aerodynamics`AirfoilGeometry`", "keywords" -> {}, "index" -> True, 
    "label" -> "Aerodynamics Package Paclet Symbol", "language" -> "en", 
    "paclet" -> "Aerodynamics Package", "status" -> "None", "summary" -> 
    "XToTheta[t, \\[Tau], x] converts a list of x-values to a list of \
theta-values using the van de Vooren transformation for an airfoil of \
thickness t and trailing-edge angle tau.", "synonyms" -> {}, "title" -> 
    "XToTheta", "type" -> "Symbol", "uri" -> "Aerodynamics/ref/XToTheta"}, 
  "LinkTrails" -> "", "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[5033, 150, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->696288930]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 14414, 424}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[601, 21, 76, 1, 70, "SymbolColorBar"],
Cell[680, 24, 191, 5, 70, "LinkTrail"],
Cell[874, 31, 2614, 61, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3513, 96, 51, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[3567, 99, 667, 18, 70, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4271, 122, 579, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->41424115],
Cell[4853, 141, 93, 1, 70, "Notes",
 CellID->1067943069],
Cell[4949, 144, 47, 1, 70, "Notes",
 CellID->918694056]
}, Closed]],
Cell[CellGroupData[{
Cell[5033, 150, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->696288930],
Cell[CellGroupData[{
Cell[5519, 167, 148, 5, 70, "ExampleSection",
 CellID->198842657],
Cell[5670, 174, 132, 6, 70, "ExampleText",
 CellID->311150377],
Cell[5805, 182, 99, 3, 70, "Input",
 CellID->85203508],
Cell[5907, 187, 229, 6, 70, "ExampleText",
 CellID->450653109],
Cell[CellGroupData[{
Cell[6161, 197, 262, 8, 70, "Input",
 CellID->475849417],
Cell[6426, 207, 382, 10, 36, "Output",
 CellID->392205299]
}, Open  ]],
Cell[6823, 220, 192, 6, 70, "ExampleText",
 CellID->105439063],
Cell[CellGroupData[{
Cell[7040, 230, 173, 5, 70, "Input",
 CellID->75917125],
Cell[7216, 237, 491, 12, 54, "Output",
 CellID->88992834]
}, Open  ]],
Cell[7722, 252, 148, 4, 70, "ExampleText",
 CellID->109577907],
Cell[CellGroupData[{
Cell[7895, 260, 265, 7, 28, "Input",
 CellID->261648493],
Cell[8163, 269, 3192, 60, 201, "Output",
 CellID->115009892]
}, Open  ]],
Cell[11370, 332, 265, 7, 70, "ExampleText",
 CellID->318374758]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[11684, 345, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[12106, 359, 167, 3, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[CellGroupData[{
Cell[12310, 367, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[12732, 381, 155, 3, 70, "MoreAbout",
 CellID->534541978]
}, Open  ]],
Cell[12902, 387, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

