(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     48063,       1362]
NotebookOptionsPosition[     41261,       1156]
NotebookOutlinePosition[     44200,       1220]
CellTagsIndexPosition[     44157,       1217]
WindowTitle->3.6  Aerodynamic Coefficients - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"3.5 Convergence and Timing\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/ConvergenceAndTiming"], "\<\"3.7 \
Worksheet for Airfoil Aerodynamics\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/WorksheetForAirfoilAerodynamics"], \
"\<\"Overview\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PanelMethods\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/PanelMethods"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/tutorial/AerodynamicCoefficients\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/AerodynamicCoefficients"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/AerodynamicCoefficients.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/AerodynamicCoefficients", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["3.6  Aerodynamic Coefficients", "Title",
 CellID->509267359],

Cell[TextData[{
 "Computation of aerodynamic characteristics for airfoils has played a \
supporting role in the discussions of validation and convergence.  This \
section focuses on how surface pressure coefficient, lift- and \
pitching-moment coefficients are obtained using ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " functions.  Choose ",
 Cell[BoxData[
  ButtonBox["PMVortexLinearNeumann",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/PMVortexLinearNeumann"]], 
  "InlineFormula"],
 " as the panel method and the NACA-4412 airfoil for the example in this \
section.  "
}], "Text",
 CellID->131238241],

Cell[TextData[{
 "The reason for choosing the NACA-4412 airfoil is because its geometry is \
available in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " both as a data file and as equations for thickness and camber properties.  \
A secondary topic for this section is to evaluate the effect on aerodynamic \
characteristics of discretization of the airfoil based a B-spline fit to the \
data in a file.  Three discretized airfoils are specified: first, a \
user-specified discretization based on equations for thickness and camber \
properties, second, a discretization by the data in the file, and finally, a \
discretization based on a B-spline fit to the data in the file and using the \
same user-specified discretization parameters as the first case.  Comparison \
of surface pressure distribution for the three discretizations provides a \
measure consistency among the three discretizations."
}], "Text",
 CellID->92798100],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "Text",
 CellID->836781195],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->2058623809],

Cell[CellGroupData[{

Cell["3.6.1 NACA-4412 Airfoil", "Section",
 CellID->259819136],

Cell[TextData[{
 "Specify angle of attack and discretization parameters.  Generate the list \
of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values on which the user-specified discretization is based."
}], "Text",
 CellID->267612329],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Alpha]", "=", 
   RowBox[{"4.", " ", "Degree"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"n", "=", "50"}], ";", 
  RowBox[{"layout", "=", "HalfCosineSpacing"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"x", "=", 
   RowBox[{"NDiscretizeUnitSegment", "[", 
    RowBox[{"n", ",", 
     RowBox[{"Layout", "\[Rule]", "layout"}]}], "]"}]}], ";"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->17065800],

Cell[TextData[{
 "Discretize the airfoil using with thickness and camber properties.  Note \
that the trailing edge is not closed, but that the mean value of the upper- \
and lower-surface trailing-edge locations is ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{"1.", ",", "0."}], "}"}]], "InlineFormula"],
 " as expected."
}], "Text",
 CellID->96382008],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rP1", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"NACA4DigitAirfoil", "[", 
     RowBox[{"4412", ",", "x"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"First", "[", "rP1", "]"}], ",", 
   RowBox[{"Last", "[", "rP1", "]"}], ",", 
   RowBox[{"Mean", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"First", "[", "rP1", "]"}], ",", 
      RowBox[{"Last", "[", "rP1", "]"}]}], "}"}], "]"}]}], "}"}]}], "Input",
 CellLabel->"In[4]:=",
 CellID->238618617],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0.9998334737126852`", ",", 
     RowBox[{"-", "0.0012489471548601018`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.0001665262873145`", ",", "0.0012489471548601265`"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"0.9999999999999998`", ",", "1.235990476633475`*^-17"}], "}"}]}], 
  "}"}]], "Output",
 ImageSize->{472, 18},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->60974542]
}, Open  ]],

Cell["\<\
For an airfoil with thickness, the number of panels is twice the \
discretization number.\
\>", "Text",
 CellID->12592807],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"nP1", "=", 
  RowBox[{"Length", "[", 
   RowBox[{"PanelLengths", "[", "rP1", "]"}], "]"}]}]], "Input",
 CellLabel->"In[6]:=",
 CellID->54667804],

Cell[BoxData["100"], "Output",
 ImageSize->{25, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->120576118]
}, Open  ]],

Cell[TextData[{
 "Next, use the data file for the NACA-4412 airfoil to specify a discretized \
airfoil.  Note that the trailing-edges are not exactly the same as those \
defined by thickness and camber properties, but that the mean value of the \
two locations is ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{"1.", ",", " ", "0."}], "}"}]], "InlineFormula"],
 " as expected."
}], "Text",
 CellID->583936543],

Cell[CellGroupData[{

Cell["\<\
rD = AirfoilFromDataFile[\"Aerodynamics\\\\Airfoil Data\\\\naca4412.dat\"];
{First[rD],Last[rD],Mean[{First[rD],Last[rD]}]}\
\>", "Input",
 CellLabel->"In[7]:=",
 CellID->108778828],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1.`", ",", 
     RowBox[{"-", "0.0013`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.`", ",", "0.0013`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.`", ",", "0.`"}], "}"}]}], "}"}]], "Output",
 ImageSize->{263, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->44418299]
}, Open  ]],

Cell["\<\
This is a relatively coarse discretization compared to the user-specified \
discretization.\
\>", "Text",
 CellID->113191753],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", 
  RowBox[{"PanelLengths", "[", "rD", "]"}], "]"}]], "Input",
 CellLabel->"In[9]:=",
 CellID->92323877],

Cell[BoxData["34"], "Output",
 ImageSize->{18, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->170157662]
}, Open  ]],

Cell["\<\
Fit a B-spline to the data for the NACA-4412 airfoil and rediscretize the \
airfoil using the same user-specified discretization as before.  Once again, \
note the coordinates of the trailing-edge.\
\>", "Text",
 CellID->11042490],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"bSpline", "=", 
   RowBox[{"FitAirfoil", "[", "rD", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rP2", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"bSpline", ",", "x"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"First", "[", "rP2", "]"}], ",", 
   RowBox[{"Last", "[", "rP2", "]"}], ",", 
   RowBox[{"Mean", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"First", "[", "rP2", "]"}], ",", 
      RowBox[{"Last", "[", "rP2", "]"}]}], "}"}], "]"}]}], "}"}]}], "Input",
 CellLabel->"In[10]:=",
 CellID->601516212],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1.`", ",", 
     RowBox[{"-", "0.0013000000000000166`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.0000000000000009`", ",", "0.0013000000000000212`"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"1.0000000000000004`", ",", "2.2768245622195593`*^-18"}], "}"}]}],
   "}"}]], "Output",
 ImageSize->{339, 18},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->77404532]
}, Open  ]],

Cell["\<\
The next two sections look at how the computed aerodynamic coefficients \
depend on the details of discretization for the NACA-4412 airfoil.  \
\>", "Text",
 CellID->158968050]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.6.2 Surface Pressure Coefficient", "Section",
 CellID->6814950],

Cell["\<\
In addition to strength parameters, the model solution provides locations of \
collocation points and surface speeds associated with collocation points.  \
Solve the panel method PMVortexLinearNeumann for each of the three \
discretized airfoils.\
\>", "Text",
 CellID->115793861],

Cell["", "Text",
 CellID->42676348],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a1", ",", "b1", ",", "rC1", ",", "VMagC1"}], "}"}], "=", 
   RowBox[{"PMVortexLinearNeumann", "[", 
    RowBox[{"rP1", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"aD", ",", "bD", ",", "rCD", ",", "VMagCD"}], "}"}], "=", 
   RowBox[{"PMVortexLinearNeumann", "[", 
    RowBox[{"rD", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a2", ",", "b2", ",", "rC2", ",", "VMagC2"}], "}"}], "=", 
   RowBox[{"PMVortexLinearNeumann", "[", 
    RowBox[{"rP2", ",", "\[Alpha]"}], "]"}]}], ";"}]}], "Input",
 CellLabel->"In[13]:=",
 CellID->821293751],

Cell[TextData[{
 "Bernoulli's equation is used to obtain surface pressure coefficient from \
surface speed.  It is conventional in the aerodynamics community to plot \
surface pressure coefficient as a function of fraction chord.  The collection \
of vectors locating collocation points is denoted by ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["c", "TI"]]], "InlineFormula"],
 ", and the list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values for the plot is obtained using the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["Components",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/Components"]], "InlineFormula"],
 ", which is the selector for the data type CartesianVectors."
}], "Text",
 CellID->250599266],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xC1", ",", "cp1"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Components", "[", "rC1", "]"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], ",", 
     RowBox[{"1", "-", 
      RowBox[{"VMagC1", "^", "2"}]}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xCD", ",", "cpD"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Components", "[", "rCD", "]"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], ",", 
     RowBox[{"1", "-", 
      RowBox[{"VMagCD", "^", "2"}]}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xC2", ",", "cp2"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Components", "[", "rC2", "]"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], ",", 
     RowBox[{"1", "-", 
      RowBox[{"VMagC2", "^", "2"}]}]}], "}"}]}], ";"}]}], "Input",
 CellLabel->"In[16]:=",
 CellID->47000734],

Cell[TextData[{
 "Superimpose the plots of surface pressure coefficients for the three \
discretized airfoils.  The surface pressure distribution for the \
user-specified discretized airfoil based on thickness and camber is used as \
the reference case, and is shown as a line joining values of ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["C",
    FontSlant->"Italic"], 
   StyleBox["p", "TI"]]], "InlineFormula"],
 " associated with collocation points.  Surface pressure coefficients \
computed from the discretized airfoil specified by the data file are shown as \
medium-sized red points.  Results obtained for the user-specified \
discretization based the fit of a B-spline to the data in the file are shown \
as small red points.  Additionally, the shape of the airfoil is shown."
}], "Text",
 CellID->139613108],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"g1", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xC1", ",", "cp1", ",", 
     RowBox[{"Epilog", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Inset", "[", 
        RowBox[{
         RowBox[{"PlotAirfoil", "[", 
          RowBox[{"rP1", ",", 
           RowBox[{"Frame", "\[Rule]", "False"}], ",", 
           RowBox[{"PlotRange", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0", ",", "1"}], "}"}], ",", "Automatic"}], "}"}]}]}], 
          "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], ",", "1"}], "]"}], "}"}]}], ",", 
     RowBox[{"ImageSize", "\[Rule]", 
      RowBox[{"7.8", " ", "72"}]}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gD", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xCD", ",", "cpD", ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Red", ",", 
        RowBox[{"PointSize", "[", "Medium", "]"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g2", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xC2", ",", "cp2", ",", 
     RowBox[{"PlotStyle", "\[Rule]", "Red"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"g1", ",", "gD", ",", "g2"}], "]"}]}], "Input",
 CellLabel->"In[19]:=",
 CellID->314038555],

Cell[BoxData[
 GraphicsBox[{
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwVkHk41PkDx0k5ovqJcqR7yqh29bTqJ8XbzxFWj+x0ucqxUXmigyySNdKm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     "]]}, 
   {RGBColor[1, 0, 0], PointSize[Medium], PointBox[CompressedData["
1:eJxTTMoPSmViYGBQAmIQ7e9hDATv7fW8t3L+cb+w/9LhmUDw1v5OkyGT3qrT
+5nyQfKv7ddsfZFRfvvU/uJooCaGF/YKuvPYWued2n/P/eyZM2ee2O9QnTXR
qOrUfiu1WUD9D+09jmjw37A4tZ/3GUj+jr1MnO0Gh6un9h9cmZ6WlnbN3kZ5
Sc67Y6f3/3kFUn/RviLb7k4T25n9E/LPA9WfsV/j/kE2yeX0fv8PIPXH7N+o
2HZnqp/en9n/Hwj227sYHLdZYX1u/32FRKD8NnseQ8Nnt60u7i+QewWUX2//
4rjmx1z5q/unH2UBun+xvVpeuOaRx/f2Z7t1GRkbT7b/XSPxNvjss/0RIrlT
Zs6stI+onsSrveH9/u8PqyeD+Gf2lVTZaVyzfySiCVaf8s1qkvXNb/a1AaeN
QOYJZWZ7unR/sbewXgO2r+7AbPu4eV/sCw86g91jJClusLj6i71y6BOwe2M0
m5ifTfhsH98XA/aPnUvCW/W4z/aFKscg/vV6PFP+2Cd7y3XTweHx6+cX+0Ce
T/YOXmng8Mp4aWNUVv/e3rHyHDg8bXz/HfHJemXPyjkbHN4itcsi3duf2m88
eg4cHzYWm+QqmB/a++YzgeNrgX9gdtjZG/bzFczA8SnwwW1brcYZ+9YXc8Dx
HaAnP+lZ+kZ7hkNm4PSwR1FJmYHt8H4ACLcRlg==
     "]]}, 
   {RGBColor[1, 0, 0], PointBox[CompressedData["
1:eJwV1Xk0lGscB3CtFI0k3EyjTUK66dyEjuanVJaQO3fau4UsU9puo5puJMlU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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  Epilog->{
    InsetBox[
     GraphicsBox[{
       Hue[0.67, 0.6, 0.6], 
       LineBox[CompressedData["
1:eJxdlfs7FPgCh4mQynoUSzcNmiQt4uRSfOREzyrVSqqp1bKUI5TCtrmkGpeI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        "]]}, {FrameLabel -> {
         FormBox["\"x\"", TraditionalForm], 
         FormBox["\"z\"", TraditionalForm]}, FrameTicks -> {{{-1.2, 
           FormBox[
            RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
           FormBox[
            RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
           FormBox[
            RowBox[{"-", "0.7999999999999999`"}], 
            TraditionalForm]}, {-0.5999999999999999, 
           FormBox[
            RowBox[{"-", "0.5999999999999999`"}], 
            TraditionalForm]}, {-0.3999999999999999, 
           FormBox[
            RowBox[{"-", "0.3999999999999999`"}], 
            TraditionalForm]}, {-0.19999999999999996`, 
           FormBox[
            RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
           FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
           FormBox["0.20000000000000018`", TraditionalForm]}, {
          0.40000000000000013`, 
           FormBox["0.40000000000000013`", TraditionalForm]}, {
          0.6000000000000001, 
           FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
           FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
           FormBox["1.0000000000000002`", TraditionalForm]}, {
          1.2000000000000004`, 
           FormBox["1.2000000000000004`", TraditionalForm]}, {
          1.4000000000000001`, 
           FormBox["1.4000000000000001`", TraditionalForm]}, {1.6, 
           FormBox["1.6`", TraditionalForm]}}, {{-1., 
           FormBox[
            RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
           FormBox[
            RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
           FormBox[
            RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
           FormBox[
            RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
           FormBox[
            RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
           FormBox[
            RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
           FormBox[
            RowBox[{"-", "0.3999999999999999`"}], 
            TraditionalForm]}, {-0.29999999999999993`, 
           FormBox[
            RowBox[{"-", "0.29999999999999993`"}], 
            TraditionalForm]}, {-0.19999999999999996`, 
           FormBox[
            RowBox[{"-", "0.19999999999999996`"}], 
            TraditionalForm]}, {-0.09999999999999998, 
           FormBox[
            RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
           FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
           FormBox["0.10000000000000009`", TraditionalForm]}, {
          0.20000000000000018`, 
           FormBox["0.20000000000000018`", TraditionalForm]}, {
          0.30000000000000004`, 
           FormBox["0.30000000000000004`", TraditionalForm]}, {
          0.40000000000000013`, 
           FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
           FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
           FormBox["0.6000000000000001`", TraditionalForm]}, {
          0.7000000000000002, 
           FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
           FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
           FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
           FormBox["1.`", TraditionalForm]}}, None, {{-1., 
           FormBox[
            RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
           FormBox[
            RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
           FormBox[
            RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
           FormBox[
            RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
           FormBox[
            RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
           FormBox[
            RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
           FormBox[
            RowBox[{"-", "0.3999999999999999`"}], 
            TraditionalForm]}, {-0.29999999999999993`, 
           FormBox[
            RowBox[{"-", "0.29999999999999993`"}], 
            TraditionalForm]}, {-0.19999999999999996`, 
           FormBox[
            RowBox[{"-", "0.19999999999999996`"}], 
            TraditionalForm]}, {-0.09999999999999998, 
           FormBox[
            RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
           FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
           FormBox["0.10000000000000009`", TraditionalForm]}, {
          0.20000000000000018`, 
           FormBox["0.20000000000000018`", TraditionalForm]}, {
          0.30000000000000004`, 
           FormBox["0.30000000000000004`", TraditionalForm]}, {
          0.40000000000000013`, 
           FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
           FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
           FormBox["0.6000000000000001`", TraditionalForm]}, {
          0.7000000000000002, 
           FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
           FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
           FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
           FormBox["1.`", TraditionalForm]}}}, ImageSize -> {504., 180.}, 
       PlotRange -> {{0, 1}, Automatic}, PlotRangeClipping -> True, 
       RotateLabel -> False}], {0, 0}, {0, 0}, 1]},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox[
    "\"\\!\\(\\*SubscriptBox[\\(C\\), \\(p\\)]\\)\"", TraditionalForm]},
  FrameTicks->{Automatic, {{-1, 
      FormBox["\"1\"", TraditionalForm]}, {0, 
      FormBox["\"0\"", TraditionalForm]}, {1, 
      FormBox["\"-1\"", TraditionalForm]}, {2, 
      FormBox["\"-2\"", TraditionalForm]}}, None, None},
  ImageSize->561.6,
  PlotRange->{{-0.05, 1.05}, {-1.2, 2}},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{566, 365},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[22]=",
 CellID->95671361]
}, Open  ]],

Cell[TextData[{
 "There are noticeable, but not dramatic, differences among the results for \
the three discretizations of the same airfoil. A small-magnitude excursion of \
",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["C", "TI"], 
   StyleBox["p", "TI"]]], "InlineFormula"],
 " on the upper surface near the pressure peak for discretization based on \
the B-spline may suggest the need for a smoother fit to the data, although it \
is consistent with the coarser discretization provided by the data file."
}], "Text",
 CellID->20411779]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
3.6.3 Lift and moment coefficients from Kutta-Joukowski theorem\
\>", "Section",
 CellID->190767290],

Cell[TextData[{
 "The ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["ForceAndMoment",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/ForceAndMoment"]], "InlineFormula"],
 " computes the lift coefficient and pitching-moment coefficient from the \
Kutta-Joukowski theorem, ",
 Cell[BoxData[
  RowBox[{"\[ScriptL]", "=", 
   RowBox[{
    StyleBox["\[Rho]", "TR"], " ", 
    StyleBox["U", "TI"], " ", 
    StyleBox["\[CapitalGamma]", "TR"]}]}]], "InlineFormula"],
 ", where ",
 Cell[BoxData["\[ScriptL]"], "InlineFormula"],
 " is the lift per unit span on the airfoil, ",
 Cell[BoxData[
  StyleBox["\[Rho]", "TR"]], "InlineFormula"],
 " is the density of the fluid, ",
 Cell[BoxData[
  StyleBox["U", "TI"]], "InlineFormula"],
 " is the free stream speed and ",
 Cell[BoxData[
  StyleBox["\[CapitalGamma]", "TR"]], "InlineFormula"],
 " is circulation.  This computation is made for each panel, summed over the \
panels and expressed as dimensionless coefficients.  For panel methods using \
doublet panels, the basic relationship between doublet density and \
circulation density is ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["\[Gamma]", "TR"], 
    RowBox[{"(", 
     StyleBox["x", "TI"], ")"}]}], "=", 
   RowBox[{
    RowBox[{"-", 
     StyleBox["d\[Mu]", "TI"]}], 
    RowBox[{
     RowBox[{"(", 
      StyleBox["x", "TI"], ")"}], "/", 
     StyleBox["dx", "TI"]}]}]}]], "InlineFormula"],
 ".  This relationship is discussed in more detail in ",
 ButtonBox["Section 2.1.2",
  BaseStyle->"Hyperlink",
  ButtonData->{"IntroductionToInfluenceCoefficients.nb", "relation"}],
 ".  Pitching moment with reference to the origin of the airfoil-fixed \
coordinate system is calculated using ",
 Cell[BoxData[
  RowBox[{
   OverscriptBox[
    StyleBox["m", "TI"], "\[RightVector]"], "=", 
   RowBox[{
    StyleBox["r", "TI"], "\[Cross]", 
    OverscriptBox["\[ScriptL]", "\[RightVector]"]}]}]], "InlineFormula"],
 "."
}], "Text",
 CellID->43157970],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"cl", ",", "cm"}], "}"}], "=", 
   RowBox[{"ForceAndMoment", "[", 
    RowBox[{
    "\"\<Vortex\>\"", ",", "a1", ",", "b1", ",", "rP1", ",", "\[Alpha]", ",", 
     "rC1"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"NumberForm", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"cl", ",", "cm"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "3"}], "}"}]}], "]"}]}], "Input",
 CellLabel->"In[23]:=",
 CellID->239099752],

Cell[BoxData[
 TagBox[
  RowBox[{"{", 
   RowBox[{
    InterpretationBox["\<\"0.993\"\>",
     0.9930218161292717,
     AutoDelete->True], ",", 
    InterpretationBox["\<\"-0.363\"\>",
     -0.36345414032918644`,
     AutoDelete->True]}], "}"}],
  NumberForm[#, {4, 3}]& ]], "Output",
 ImageSize->{106, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[24]//NumberForm=",
 CellID->270171605]
}, Open  ]],

Cell[TextData[{
 "This is the standard procedure for computing airfoil lift and \
pitching-moment coefficients in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellID->100233885]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.6.4 Lift and pitching moment from pressure coefficient", "Section",
 CellID->252223901],

Cell[TextData[{
 "Lift and pitching-moment coefficients can be also computed from the surface \
pressure distribution on the airfoil.  The lift coefficient can be written as \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["C", "\[ScriptL]"], "=", 
    RowBox[{"-", 
     RowBox[{"\[ContourIntegral]", 
      RowBox[{
       SubscriptBox["C", "p"], " ", 
       RowBox[{
        OverscriptBox["n", "^"], "\[CenterDot]", 
        OverscriptBox["\[ScriptL]", "^"]}], 
       RowBox[{"\[DifferentialD]", "\[ScriptS]"}]}]}]}]}], TraditionalForm]]],
 ", where the integral is over the airfoil contour, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "p"], TraditionalForm]]],
 " is the pressure coefficient, ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["n", "^"], TraditionalForm]]],
 " is the outward unit normal to the airfoil surface, and ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["\[ScriptL]", "^"], TraditionalForm]]],
 " is a unit vector perpendicular to the free-stream velocity in the \
direction of positive lift.  The moment coefficient can be thought of as the ",
 Cell[BoxData[
  StyleBox["y", "TI"]], "InlineFormula"],
 "-component of the vector ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", 
    RowBox[{"\[ContourIntegral]", 
     RowBox[{
      OverscriptBox["r", "\[RightVector]"], "\[Times]", 
      OverscriptBox["n", "^"], 
      SubscriptBox["C", "p"], " ", 
      RowBox[{"\[DifferentialD]", "\[ScriptS]"}]}]}]}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["r", "\[RightVector]"], TraditionalForm]]],
 " locates a point on the airfoil surface.  The integrals are approximated by \
considering the pressure coefficient constant over each panel, computing the \
contribution to lift and moment for each panel, and summing the results.  The \
user-specified discretization of the airfoil based on thickness and camber \
properties is used in the following example."
}], "Text",
 CellID->126296105],

Cell[TextData[{
 "There are ",
 StyleBox["Aerodynamic",
  FontSlant->"Italic"],
 " functions to compute panel lengths and unit vectors to panels."
}], "Text",
 CellID->195800744],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Delta]", "=", 
   RowBox[{"PanelLengths", "[", "rP1", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"uvN", "=", 
   RowBox[{"PanelNormals", "[", "rP1", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"nP", "=", 
   RowBox[{"Length", "[", "\[Delta]", "]"}]}], ";"}]}], "Input",
 CellLabel->"In[25]:=",
 CellID->41528148],

Cell["\<\
Unit vectors in the direction of lift are perpendicular to the free-stream \
velocity.\
\>", "Text",
 CellID->126606522],

Cell[BoxData[
 RowBox[{
  RowBox[{"uvL", "=", 
   RowBox[{"Cross", "[", 
    RowBox[{"UniformFlow", "[", 
     RowBox[{"nP", ",", "\[Alpha]"}], "]"}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[28]:=",
 CellID->35209663],

Cell[TextData[{
 "Compute lift coefficient and pitching-moment coefficient for each panel and \
sum over panels to obtain airfoil lift and moment coefficients.  When \
computing panel contributions to moment coefficient, the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["Components",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/Components"]], "InlineFormula"],
 " is used to select the list of components, and  the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function Part is used to select the list of ",
 Cell[BoxData[
  StyleBox["y", "TI"]], "InlineFormula"],
 "-components of the collection of moment vectors.  Note that the pitching \
moment is about the origin of the coordinate system, which in this example is \
the leading edge of the airfoil."
}], "Text",
 CellID->25066311],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[CapitalDelta]clCp", "=", 
   RowBox[{
    RowBox[{"-", "cp1"}], " ", 
    RowBox[{"(", 
     RowBox[{"uvL", ".", "uvN"}], ")"}], " ", "\[Delta]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[CapitalDelta]cmCp", "=", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{
      RowBox[{"Components", "[", 
       RowBox[{
        RowBox[{"ExtendXzToXyz", "[", "rC1", "]"}], "\[Cross]", 
        RowBox[{"ExtendXzToXyz", "[", "uvN", "]"}]}], "]"}], "[", 
      RowBox[{"[", "2", "]"}], "]"}]}], " ", "cp1", " ", "\[Delta]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"clCp", ",", "cmCp"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Apply", "[", 
      RowBox[{"Plus", ",", "\[CapitalDelta]clCp"}], "]"}], ",", 
     RowBox[{"Apply", "[", 
      RowBox[{"Plus", ",", "\[CapitalDelta]cmCp"}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"NumberForm", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"clCp", ",", "cmCp"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "3"}], "}"}]}], "]"}]}], "Input",
 CellLabel->"In[29]:=",
 CellID->391129333],

Cell[BoxData[
 TagBox[
  RowBox[{"{", 
   RowBox[{
    InterpretationBox["\<\"0.985\"\>",
     0.9851560455583452,
     AutoDelete->True], ",", 
    InterpretationBox["\<\"-0.361\"\>",
     -0.3610112413389047,
     AutoDelete->True]}], "}"}],
  NumberForm[#, {4, 3}]& ]], "Output",
 ImageSize->{106, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[32]//NumberForm=",
 CellID->986524656]
}, Open  ]],

Cell["\<\
These results seem to be in reasonable agreement with those computed in the \
previous section using the Kutta-Joukowski theorem.\
\>", "Text",
 CellID->665816348]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["PanelMethods",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/PanelMethods"]], "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["3.5 Convergence and Timing",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming"]], "RelatedTutorials",
 CellID->101392790],

Cell[TextData[ButtonBox["3.7 Worksheet for Airfoil Aerodynamics",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/WorksheetForAirfoilAerodynamics"]], \
"RelatedTutorials",
 CellID->14999323],

Cell[TextData[ButtonBox["Overview",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]], "RelatedTutorials",
 CellID->280152894]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"3.6  Aerodynamic Coefficients - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 3, 25.4352000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "None", "summary" -> 
    "Computation of aerodynamic characteristics for airfoils has played a \
supporting role in the discussions of validation and convergence. This \
section focuses on how surface pressure coefficient, lift- and \
pitching-moment coefficients are obtained using Aerodynamics functions. \
Choose PMVortexLinearNeumann as the panel method and the NACA-4412 airfoil \
for the example in this section. The reason for choosing the NACA-4412 \
airfoil is because its geometry is available in Aerodynamics both as a data \
file and as equations for thickness and camber properties. A secondary topic \
for this section is to evaluate the effect on aerodynamic characteristics of \
discretization of the airfoil based a B-spline fit to the data in a file. \
Three discretized airfoils are specified: first, a user-specified \
discretization based on equations for thickness and camber properties, \
second, a discretization by the data in the file, and finally, a \
discretization based on a B-spline fit to the data in the file and using the \
same user-specified discretization parameters as the first case. Comparison \
of surface pressure distribution for the three discretizations provides a \
measure consistency among the three discretizations. Load the Aerodynamics \
packages.", "synonyms" -> {}, "title" -> "3.6  Aerodynamic Coefficients", 
    "type" -> "Tutorial", "uri" -> 
    "Aerodynamics/tutorial/AerodynamicCoefficients"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[622, 21, 78, 1, 70, "TutorialColorBar"],
Cell[703, 24, 191, 5, 70, "LinkTrail"],
Cell[897, 31, 2982, 67, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3904, 102, 66, 1, 70, "Title",
 CellID->509267359],
Cell[3973, 105, 633, 16, 70, "Text",
 CellID->131238241],
Cell[4609, 123, 939, 16, 70, "Text",
 CellID->92798100],
Cell[5551, 141, 125, 6, 70, "Text",
 CellID->836781195],
Cell[5679, 149, 101, 3, 70, "Input",
 CellID->2058623809],
Cell[CellGroupData[{
Cell[5805, 156, 62, 1, 70, "Section",
 CellID->259819136],
Cell[5870, 159, 252, 7, 70, "Text",
 CellID->267612329],
Cell[6125, 168, 456, 13, 70, "Input",
 CellID->17065800],
Cell[6584, 183, 353, 9, 70, "Text",
 CellID->96382008],
Cell[CellGroupData[{
Cell[6962, 196, 547, 16, 70, "Input",
 CellID->238618617],
Cell[7512, 214, 506, 16, 39, "Output",
 CellID->60974542]
}, Open  ]],
Cell[8033, 233, 132, 4, 70, "Text",
 CellID->12592807],
Cell[CellGroupData[{
Cell[8190, 241, 168, 5, 70, "Input",
 CellID->54667804],
Cell[8361, 248, 160, 5, 36, "Output",
 CellID->120576118]
}, Open  ]],
Cell[8536, 256, 407, 10, 70, "Text",
 CellID->583936543],
Cell[CellGroupData[{
Cell[8968, 270, 191, 5, 70, "Input",
 CellID->108778828],
Cell[9162, 277, 401, 14, 36, "Output",
 CellID->44418299]
}, Open  ]],
Cell[9578, 294, 135, 4, 70, "Text",
 CellID->113191753],
Cell[CellGroupData[{
Cell[9738, 302, 141, 4, 70, "Input",
 CellID->92323877],
Cell[9882, 308, 159, 5, 36, "Output",
 CellID->170157662]
}, Open  ]],
Cell[10056, 316, 240, 5, 70, "Text",
 CellID->11042490],
Cell[CellGroupData[{
Cell[10321, 325, 617, 18, 70, "Input",
 CellID->601516212],
Cell[10941, 345, 492, 16, 39, "Output",
 CellID->77404532]
}, Open  ]],
Cell[11448, 364, 186, 4, 70, "Text",
 CellID->158968050]
}, Open  ]],
Cell[CellGroupData[{
Cell[11671, 373, 71, 1, 70, "Section",
 CellID->6814950],
Cell[11745, 376, 290, 6, 70, "Text",
 CellID->115793861],
Cell[12038, 384, 35, 1, 70, "Text",
 CellID->42676348],
Cell[12076, 387, 731, 20, 70, "Input",
 CellID->821293751],
Cell[12810, 409, 827, 22, 70, "Text",
 CellID->250599266],
Cell[13640, 433, 1031, 35, 70, "Input",
 CellID->47000734],
Cell[14674, 470, 819, 16, 70, "Text",
 CellID->139613108],
Cell[CellGroupData[{
Cell[15518, 490, 1571, 44, 70, "Input",
 CellID->314038555],
Cell[17092, 536, 13600, 262, 386, "Output",
 CellID->95671361]
}, Open  ]],
Cell[30707, 801, 538, 12, 70, "Text",
 CellID->20411779]
}, Open  ]],
Cell[CellGroupData[{
Cell[31282, 818, 110, 3, 70, "Section",
 CellID->190767290],
Cell[31395, 823, 2012, 60, 70, "Text",
 CellID->43157970],
Cell[CellGroupData[{
Cell[33432, 887, 495, 16, 70, "Input",
 CellID->239099752],
Cell[33930, 905, 428, 15, 50, "Output",
 CellID->270171605]
}, Open  ]],
Cell[34373, 923, 203, 7, 70, "Text",
 CellID->100233885]
}, Open  ]],
Cell[CellGroupData[{
Cell[34613, 935, 95, 1, 70, "Section",
 CellID->252223901],
Cell[34711, 938, 1961, 52, 70, "Text",
 CellID->126296105],
Cell[36675, 992, 178, 6, 70, "Text",
 CellID->195800744],
Cell[36856, 1000, 383, 11, 70, "Input",
 CellID->41528148],
Cell[37242, 1013, 130, 4, 70, "Text",
 CellID->126606522],
Cell[37375, 1019, 219, 7, 70, "Input",
 CellID->35209663],
Cell[37597, 1028, 863, 21, 70, "Text",
 CellID->25066311],
Cell[CellGroupData[{
Cell[38485, 1053, 1166, 37, 70, "Input",
 CellID->391129333],
Cell[39654, 1092, 426, 15, 50, "Output",
 CellID->986524656]
}, Open  ]],
Cell[40095, 1110, 173, 4, 70, "Text",
 CellID->665816348]
}, Open  ]],
Cell[CellGroupData[{
Cell[40305, 1119, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[40373, 1122, 158, 3, 70, "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[CellGroupData[{
Cell[40568, 1130, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[40643, 1133, 184, 4, 70, "RelatedTutorials",
 CellID->101392790],
Cell[40830, 1139, 208, 5, 70, "RelatedTutorials",
 CellID->14999323],
Cell[41041, 1146, 166, 4, 70, "RelatedTutorials",
 CellID->280152894]
}, Open  ]]
}, Open  ]],
Cell[41234, 1154, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

