(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     38979,       1258]
NotebookOptionsPosition[     29249,        980]
NotebookOutlinePosition[     30966,       1027]
CellTagsIndexPosition[     30923,       1024]
WindowTitle->Aerodynamics Overview - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACKAGE OVERVIEW", "PacletNameCell"], Cell[TextData[
    Cell[BoxData[
     ActionMenuBox[
      FrameBox["\<\"URL \[RightGuillemet]\"\>",
       StripOnInput->
        False], {"\<\"Aerodynamics/tutorial/AerodynamicsOverview\"\>":>
      None, "\<\"Copy Mathematica url\"\>":>
      Module[{DocumentationSearch`Private`nb$}, 
       DocumentationSearch`Private`nb$ = NotebookPut[
          Notebook[{
            Cell["Aerodynamics/tutorial/AerodynamicsOverview"]}, Visible -> 
           False]]; SelectionMove[
        DocumentationSearch`Private`nb$, All, Notebook]; 
       FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
       NotebookClose[DocumentationSearch`Private`nb$]; Null], 
      Delimiter, "\<\"Copy web url\"\>":>
      Module[{DocumentationSearch`Private`nb$}, 
       DocumentationSearch`Private`nb$ = NotebookPut[
          Notebook[{
            Cell[
             BoxData[
              MakeBoxes[
               Hyperlink[
               "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/AerodynamicsOverview.html"], StandardForm]], "Input", 
             TextClipboardType -> "PlainText"]}, Visible -> False]]; 
       SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
       FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
       NotebookClose[DocumentationSearch`Private`nb$]; 
       Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
        NotebookLocate[{
          URL[
           StringJoin[
           "http://reference.wolfram.com/mathematica/", 
            "Aerodynamics/tutorial/AerodynamicsOverview", ".html"]], None}]}]},
      Appearance->None,
      MenuAppearance->Automatic]],
     LineSpacing->{1.4, 0}]], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["Aerodynamics Overview", "TOCDocumentTitle"],

Cell[TextData[ButtonBox["Preface",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PrefaceToAerodynamics#509267359"]], \
"TOCChapter"],

Cell["Chapter 1  Airfoil Geometry", "Section",
 CellID->361902836],

Cell[TextData[ButtonBox["1.1  Introduction",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToAirfoilGeometry#509267359"]], \
"TOCChapter"],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["1.2  Airfoils from Data Files",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromDataFiles#509267359"]], \
"TOCChapter"],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["1.2.1 Structure of data files",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromDataFiles#339343814"]], \
"TOCSection"],

Cell[TextData[ButtonBox["Counterclockwise numbering convention",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromDataFiles#397338901"]], \
"TOCSubsection"],

Cell[TextData[ButtonBox["Fore-to-aft numbering convention",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromDataFiles#9113640"]], \
"TOCSubsection"]
}, Open  ]],

Cell[TextData[ButtonBox["1.2.2 Discretization of airfoil",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromDataFiles#636677227"]], \
"TOCSection"],

Cell[TextData[ButtonBox["1.2.3 Data from the UIUC Airfoil Coordinate Database",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromDataFiles#114536833"]], \
"TOCSection"],

Cell[TextData[ButtonBox["1.2.4 Surprises",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromDataFiles#577231104"]], \
"TOCSection"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["1.3  Airfoils from Thickness and Camber Properties",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromThicknessAndCamberProperties#\
509267359"]], "TOCChapter"],

Cell[TextData[ButtonBox["1.3.1 NACA 4412 airfoil",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromThicknessAndCamberProperties#\
780127955"]], "TOCSection"],

Cell[TextData[ButtonBox["1.3.2 NACA four-digit family of airfoils",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromThicknessAndCamberProperties#\
257533491"]], "TOCSection"],

Cell[TextData[ButtonBox["",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromThicknessAndCamberProperties#\
794950467"]], "TOCSection"],

Cell[TextData[ButtonBox["1.3.3 Trailing-edge of NACA four-digit airfoils",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromThicknessAndCamberProperties#\
140735501"]], "TOCSection"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["1.4  Airfoils from Conformal Mapping",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromConformalMapping#509267359"]], \
"TOCChapter"],

Cell[TextData[ButtonBox["1.4.1 van de Vooren airfoil of 12% thickness",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromConformalMapping#320326959"]], \
"TOCSection"],

Cell[TextData[ButtonBox["1.4.2 van de Vooren family of airfoils",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromConformalMapping#36431957"]], \
"TOCSection"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["1.5  Panel Geometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PanelGeometry#509267359"]], "TOCChapter"],

Cell[TextData[ButtonBox["1.5.1 Airfoil With Thickness",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PanelGeometry#282162397"]], "TOCSection"],

Cell[TextData[ButtonBox["1.5.2 Airfoil Without Thickness (Camber Line)",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PanelGeometry#848281288"]], "TOCSection"]
}, Closed]],

Cell["Chapter 2  Influence Coefficients", "Section",
 CellID->818141499],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["2.1  Introduction",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients#\
509267359"]], "TOCChapter"],

Cell[TextData[ButtonBox["2.1.1 Geometric influence coefficients and strength \
parameters",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients#\
320914423"]], "TOCSection"],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["2.1.2 Basic relations between two-dimensional vortex \
and doublet panels",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients#\
133089311"]], "TOCSection"],

Cell[TextData[ButtonBox["Constant-strength doublet panel",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients#\
299286692"]], "TOCSubsection"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["2.2  Point Singularities",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PointSingularities#509267359"]], "TOCChapter"],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["2.2.1  Uniform Flow Over a Circular Cylinder Without \
Circulation",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PointSingularities#100587181"]], "TOCSection"],

Cell[TextData[ButtonBox["Point doublet",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PointSingularities#215672006"]], \
"TOCSubsection"],

Cell[TextData[ButtonBox["Superposition with uniform flow",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PointSingularities#89762736"]], \
"TOCSubsection"],

Cell[TextData[ButtonBox["Boundary condition",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PointSingularities#440858664"]], \
"TOCSubsection"],

Cell[TextData[ButtonBox["Functions for velocity potential and stream function",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PointSingularities#344238109"]], \
"TOCSubsection"],

Cell[TextData[ButtonBox["Pressure coefficient",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PointSingularities#668703906"]], \
"TOCSubsection"],

Cell[TextData[ButtonBox["Flow visualization",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PointSingularities#22002075"]], \
"TOCSubsection"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["2.2.2  Flat Plate at Angle of Attack",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PointSingularities#588183864"]], "TOCSection"],

Cell[TextData[ButtonBox["Discretize the airfoil",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PointSingularities#238497841"]], \
"TOCSubsection"],

Cell[TextData[ButtonBox["Choose singularity",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PointSingularities#377269594"]], \
"TOCSubsection"],

Cell[TextData[ButtonBox["Apply boundary conditions",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PointSingularities#81322843"]], \
"TOCSubsection"],

Cell[TextData[ButtonBox["Aerodynamics",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PointSingularities#562395899"]], \
"TOCSubsection"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["2.3  Isolated Panel",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IsolatedPanel#509267359"]], "TOCChapter"],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["2.3.1  Algebraic expressions for influence \
coefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IsolatedPanel#67289592"]], "TOCSection"],

Cell[TextData[ButtonBox["Constant-strength source panel",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IsolatedPanel#22443959"]], "TOCSubsection"],

Cell[TextData[ButtonBox["Constant-strength vortex panel",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IsolatedPanel#177267806"]], "TOCSubsection"],

Cell[TextData[ButtonBox["Doublet panel of linearly varying strength",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IsolatedPanel#72624876"]], "TOCSubsection"]
}, Open  ]],

Cell[TextData[ButtonBox["2.3.2  Velocity from velocity potential or stream \
function",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IsolatedPanel#29918537"]], "TOCSection"],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["2.3.3  Flow visualization",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IsolatedPanel#34893618"]], "TOCSection"],

Cell[TextData[ButtonBox["Constant-strength source panel",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IsolatedPanel#147316578"]], "TOCSubsection"],

Cell[TextData[ButtonBox["Constant-strength vortex panel",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IsolatedPanel#671459404"]], "TOCSubsection"],

Cell[TextData[ButtonBox["Doublet panel of linearly varying strength",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IsolatedPanel#485419430"]], "TOCSubsection"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["2.3.4 Jump conditions",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IsolatedPanel#229580123"]], "TOCSection"],

Cell[TextData[ButtonBox["Constant-strength source panel",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IsolatedPanel#164768312"]], "TOCSubsection"],

Cell[TextData[ButtonBox["Constant-strength vortex panel",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IsolatedPanel#486795743"]], "TOCSubsection"],

Cell[TextData[ButtonBox["Doublet panel of linearly varying strength",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IsolatedPanel#22834914"]], "TOCSubsection"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["2.3.5 Branch cuts",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IsolatedPanel#455404028"]], "TOCSection"],

Cell[TextData[ButtonBox["Constant-strength source panel",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IsolatedPanel#181897434"]], "TOCSubsection"],

Cell[TextData[ButtonBox["Constant-strength vortex panel",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IsolatedPanel#651722584"]], "TOCSubsection"],

Cell[TextData[ButtonBox["Doublet panel of linearly varying strength",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IsolatedPanel#209793762"]], "TOCSubsection"]
}, Open  ]],

Cell[TextData[ButtonBox["2.3.6 Equivalent flow fields for vortex and doublet \
panels",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IsolatedPanel#497897766"]], "TOCSection"]
}, Closed]],

Cell["Chapter 3  Panel Methods", "Section",
 CellID->66319223],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["3.1  Introduction",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods#509267359"]], \
"TOCChapter"],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["3.1.1  Steps in implementation",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods#432056375"]], \
"TOCSection"],

Cell[TextData[ButtonBox["Airfoil discretization",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods#220682699"]], \
"TOCSubsection"],

Cell[TextData[ButtonBox["Singularity distribution",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods#64226099"]], \
"TOCSubsection"],

Cell[TextData[ButtonBox["Boundary conditions",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods#178726099"]], \
"TOCSubsection"],

Cell[TextData[ButtonBox["Kutta condition",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods#446287461"]], \
"TOCSubsection"],

Cell[TextData[ButtonBox["Smoothness conditions",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods#252269107"]], \
"TOCSubsection"],

Cell[TextData[ButtonBox["System of equations",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods#95542403"]], \
"TOCSubsection"]
}, Open  ]],

Cell[TextData[ButtonBox["3.1.2  Example",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods#275406678"]], \
"TOCSection"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["3.2  Validation Example",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ValidationExample#509267359"]], "TOCChapter",
 CellID->490140180],

Cell[TextData[ButtonBox["3.2.1  van de Vooren Airfoil",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ValidationExample#631983245"]], "TOCSection",
 CellID->95216419],

Cell[TextData[ButtonBox["3.2.2  PMVortexLinearNeumann",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ValidationExample#234765418"]], "TOCSection",
 CellID->470836979],

Cell[TextData[ButtonBox["3.2.3  PMDoubletConstantDirichlet",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ValidationExample#617689357"]], "TOCSection",
 CellID->55463762],

Cell[TextData[ButtonBox["3.2.4  PMDoubletQuadraticDirichlet",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ValidationExample#246523140"]], "TOCSection",
 CellID->452240740],

Cell[TextData[ButtonBox["3.2.5  Summary",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ValidationExample#135714564"]], "TOCSection",
 CellID->231693941]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["3.3  Implementation",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#509267359"]], "TOCChapter"],

Cell[TextData[ButtonBox["3.3.1  Eppler-205 Airfoil",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#491421075"]], "TOCSection"],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["3.3.2  Panel method using vortex panels of linearly \
varying strength and exterior Neumann boundary conditions",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#336729780"]], "TOCSection"],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["Step-by-step implementation",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#725608851"]], "TOCSubsection"],

Cell[TextData[ButtonBox["Airfoil geometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#225787759"]], \
"TOCSubsubsection"],

Cell[TextData[ButtonBox["Boundary conditions at collocation points",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#15365561"]], "TOCSubsubsection"],

Cell[TextData[ButtonBox["Panel-to-panel continuity of circulation density and \
the Kutta condition",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#103764251"]], \
"TOCSubsubsection"],

Cell[TextData[ButtonBox["Solution procedure",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#298127402"]], \
"TOCSubsubsection"]
}, Open  ]],

Cell[TextData[ButtonBox["Compare with results using function \
PMVortexLinearNeumann",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#611074881"]], "TOCSubsection"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["3.3.3 Panel method using doublet panels of constant \
strength and interior Dirichlet boundary conditions",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#577981106"]], "TOCSection"],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["Step-by-step implementation",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#104432088"]], "TOCSubsection"],

Cell[TextData[ButtonBox["Airfoil geometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#80757491"]], "TOCSubsubsection"],

Cell[TextData[ButtonBox["Boundary conditions at collocation points, and Kutta \
condition",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#292614508"]], \
"TOCSubsubsection"]
}, Open  ]],

Cell[TextData[ButtonBox["Compare with results using function \
PMDoubletConstantDirichlet",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#37612957"]], "TOCSubsection"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["3.3.4 Panel method using doublet panels of \
quadratically varying strength and interior Dirichlet boundary conditions",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#106827055"]], "TOCSection"],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["Step-by-step implementation",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#97066759"]], "TOCSubsection"],

Cell[TextData[ButtonBox["Airfoil geometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#135427747"]], \
"TOCSubsubsection"],

Cell[TextData[ButtonBox["\nBoundary conditions at collocation points",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#306855904"]], \
"TOCSubsubsection"],

Cell[TextData[ButtonBox["Kutta conditions",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#94585002"]], "TOCSubsubsection"],

Cell[TextData[ButtonBox["Panel-to-panel continuity of value of doublet \
density, and a Kutta condition",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#102303559"]], \
"TOCSubsubsection"],

Cell[TextData[ButtonBox["Panel-to-panel continuity of slope of doublet \
density function, and a Kutta condition",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#89512504"]], "TOCSubsubsection"],

Cell[TextData[ButtonBox["Solution procedure",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#165742176"]], \
"TOCSubsubsection"]
}, Open  ]],

Cell[TextData[ButtonBox["Results using function PMDoubletQuadraticDirichlet",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#913638886"]], "TOCSubsection"]
}, Open  ]],

Cell[TextData[ButtonBox["3.3.5 Comments",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation#273428036"]], "TOCSection"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["3.4  Flow Visualization",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/FlowVisualization#509267359"]], "TOCChapter"],

Cell[TextData[ButtonBox["3.4.1 Solution of the model",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/FlowVisualization#314388262"]], "TOCSection"],

Cell[TextData[ButtonBox["3.4.2 Stream function",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/FlowVisualization#1708000"]], "TOCSection"],

Cell[TextData[ButtonBox["3.4.3 Velocity field",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/FlowVisualization#56297006"]], "TOCSection"],

Cell[TextData[ButtonBox["3.4.4 StreamPlot",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/FlowVisualization#256023531"]], "TOCSection"],

Cell[TextData[ButtonBox["3.4.5 StreamDensityPlot",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/FlowVisualization#2809397"]], "TOCSection"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["3.5  Convergence and Timing",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming#509267359"]], \
"TOCChapter"],

Cell[TextData[ButtonBox["3.5.1 Eppler-205 airfoil",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming#627174914"]], \
"TOCSection"],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["3.5.2  Solve models",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming#186435948"]], \
"TOCSection"],

Cell[TextData[ButtonBox["PMVortexLinearNeumann",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming#276154793"]], \
"TOCSubsection"],

Cell[TextData[ButtonBox["PMDoubletConstantDirichlet",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming#502776918"]], \
"TOCSubsection"],

Cell[TextData[ButtonBox["PMDoubletQuadraticDirichlet",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming#118691814"]], \
"TOCSubsection"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["3.5.3  Convergence properties",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming#227129737"]], \
"TOCSection"],

Cell[TextData[ButtonBox["Lift Coefficient",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming#869225247"]], \
"TOCSubsection"],

Cell[TextData[ButtonBox["Pitching-moment coefficient",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming#870013475"]], \
"TOCSubsection"]
}, Open  ]],

Cell[TextData[ButtonBox["3.5.4 Timing properties",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming#32786706"]], "TOCSection"],

Cell[TextData[ButtonBox["3.5.5  PMDoubletConstantDirichlet revisited",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming#154697233"]], \
"TOCSection"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["3.6  Aerodynamic Coefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicCoefficients#509267359"]], \
"TOCChapter"],

Cell[TextData[ButtonBox["3.6.1 NACA-4412 Airfoil",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicCoefficients#259819136"]], \
"TOCSection"],

Cell[TextData[ButtonBox["3.6.2 Surface Pressure Coefficient",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicCoefficients#6814950"]], \
"TOCSection"],

Cell[TextData[ButtonBox["3.6.3 Built-in function to compute lift and moment \
coefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicCoefficients#190767290"]], \
"TOCSection"],

Cell[TextData[ButtonBox["3.6.4 Lift and pitching moment from pressure \
coefficient",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicCoefficients#252223901"]], \
"TOCSection",
 CellID->393157910]
}, Closed]],

Cell[TextData[ButtonBox["3.7  Worksheet for Airfoil Aerodynamics",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/WorksheetForAirfoilAerodynamics"]], \
"TOCChapter",
 CellID->357436865],

Cell[TextData[ButtonBox["References",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/References#509267359"]], "Section",
 CellID->479048299],

Cell["Appendices", "Section",
 CellID->17255043],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["A1  Data Type CartesianVectors",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/CartesianVectors#509267359"]], "TOCChapter"],

Cell[TextData[ButtonBox["A1.1 Constructor and Selector",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/CartesianVectors#152704095"]], "TOCSection"],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["A1.1 Vector algebra",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/CartesianVectors#98413141"]], "TOCSection"],

Cell[TextData[ButtonBox["Basic addition, subtraction and multiplication by a \
constant",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/CartesianVectors#578648157"]], "TOCSubsection"],

Cell[TextData[ButtonBox["Multiplication by a list",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/CartesianVectors#87368199"]], "TOCSubsection"],

Cell[TextData[ButtonBox["Dot and cross products",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/CartesianVectors#130895041"]], "TOCSubsection"],

Cell[TextData[ButtonBox["Outer",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/CartesianVectors#337431051"]], "TOCSubsection"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["A1.2 List Operations",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/CartesianVectors#23446292"]], "TOCSection"],

Cell[TextData[ButtonBox["Get pieces from a collection of vectors.",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/CartesianVectors#69497392"]], "TOCSubsection"],

Cell[TextData[ButtonBox["Join two collections of vectors.",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/CartesianVectors#292283586"]], "TOCSubsection"],

Cell[TextData[ButtonBox["Prepend, append or insert a vector to a collection.",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/CartesianVectors#234066801"]], "TOCSubsection"],

Cell[TextData[ButtonBox["Rotate position right or left.",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/CartesianVectors#70600885"]], "TOCSubsection"]
}, Open  ]],

Cell[TextData[ButtonBox["A1.3 Summary",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/CartesianVectors#23564291"]], "TOCSection"]
}, Closed]],

Cell[TextData[ButtonBox["A2  Graphics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/Graphics#509267359"]], "TOCChapter"],

Cell[TextData[ButtonBox["Acknowledgments",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Acknowledgments#509267359"]], "Section"],

Cell[CellGroupData[{

Cell["Guide Pages", "Section",
 CellID->346595432],

Cell[TextData[ButtonBox["CartesianVectors",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/CartesianVectors"]], "TOCSection"],

Cell[TextData[ButtonBox["AirfoilGeometry",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/AirfoilGeometry"]], "TOCSection"],

Cell[TextData[ButtonBox["InfluenceCoefficients",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/InfluenceCoefficients"]], "TOCSection"],

Cell[TextData[ButtonBox["PanelMethods",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/PanelMethods"]], "TOCSection"],

Cell[TextData[ButtonBox["Graphics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Graphics"]], "TOCSection"]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Aerodynamics Overview - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 3, 27.5256000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Package Overview", "language" -> "en", "paclet" -> 
    "Aerodynamics Package", "status" -> "None", "summary" -> 
    "Preface 1.1 Introduction 1.2 Airfoils from Data Files", "synonyms" -> {},
     "title" -> "Aerodynamics Overview", "type" -> "Overview", "uri" -> 
    "Aerodynamics/tutorial/AerodynamicsOverview"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[614, 21, 76, 1, 70, "SymbolColorBar"],
Cell[693, 24, 191, 5, 70, "LinkTrail"],
Cell[887, 31, 1971, 44, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[2883, 79, 49, 0, 70, "TOCDocumentTitle"],
Cell[2935, 81, 152, 4, 70, "TOCChapter"],
Cell[3090, 87, 66, 1, 70, "Section",
 CellID->361902836],
Cell[3159, 90, 170, 4, 70, "TOCChapter"],
Cell[CellGroupData[{
Cell[3354, 98, 174, 4, 70, "TOCChapter"],
Cell[CellGroupData[{
Cell[3553, 106, 174, 4, 70, "TOCSection"],
Cell[3730, 112, 185, 4, 70, "TOCSubsection"],
Cell[3918, 118, 178, 4, 70, "TOCSubsection"]
}, Open  ]],
Cell[4111, 125, 176, 4, 70, "TOCSection"],
Cell[4290, 131, 197, 4, 70, "TOCSection"],
Cell[4490, 137, 160, 4, 70, "TOCSection"]
}, Closed]],
Cell[CellGroupData[{
Cell[4687, 146, 214, 4, 70, "TOCChapter"],
Cell[4904, 152, 187, 4, 70, "TOCSection"],
Cell[5094, 158, 204, 4, 70, "TOCSection"],
Cell[5301, 164, 164, 4, 70, "TOCSection"],
Cell[5468, 170, 211, 4, 70, "TOCSection"]
}, Closed]],
Cell[CellGroupData[{
Cell[5716, 179, 188, 4, 70, "TOCChapter"],
Cell[5907, 185, 196, 4, 70, "TOCSection"],
Cell[6106, 191, 189, 4, 70, "TOCSection"]
}, Closed]],
Cell[CellGroupData[{
Cell[6332, 200, 154, 3, 70, "TOCChapter"],
Cell[6489, 205, 163, 3, 70, "TOCSection"],
Cell[6655, 210, 180, 3, 70, "TOCSection"]
}, Closed]],
Cell[6850, 216, 72, 1, 70, "Section",
 CellID->818141499],
Cell[CellGroupData[{
Cell[6947, 221, 176, 4, 70, "TOCChapter"],
Cell[7126, 227, 223, 5, 70, "TOCSection"],
Cell[CellGroupData[{
Cell[7374, 236, 232, 5, 70, "TOCSection"],
Cell[7609, 243, 193, 4, 70, "TOCSubsection"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[7851, 253, 164, 3, 70, "TOCChapter"],
Cell[CellGroupData[{
Cell[8040, 260, 206, 4, 70, "TOCSection"],
Cell[8249, 266, 158, 4, 70, "TOCSubsection"],
Cell[8410, 272, 175, 4, 70, "TOCSubsection"],
Cell[8588, 278, 163, 4, 70, "TOCSubsection"],
Cell[8754, 284, 197, 4, 70, "TOCSubsection"],
Cell[8954, 290, 165, 4, 70, "TOCSubsection"],
Cell[9122, 296, 162, 4, 70, "TOCSubsection"]
}, Open  ]],
Cell[CellGroupData[{
Cell[9321, 305, 176, 3, 70, "TOCSection"],
Cell[9500, 310, 167, 4, 70, "TOCSubsection"],
Cell[9670, 316, 163, 4, 70, "TOCSubsection"],
Cell[9836, 322, 169, 4, 70, "TOCSubsection"],
Cell[10008, 328, 157, 4, 70, "TOCSubsection"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[10214, 338, 154, 3, 70, "TOCChapter"],
Cell[CellGroupData[{
Cell[10393, 345, 191, 4, 70, "TOCSection"],
Cell[10587, 351, 167, 3, 70, "TOCSubsection"],
Cell[10757, 356, 168, 3, 70, "TOCSubsection"],
Cell[10928, 361, 179, 3, 70, "TOCSubsection"]
}, Open  ]],
Cell[11122, 367, 194, 4, 70, "TOCSection"],
Cell[CellGroupData[{
Cell[11341, 375, 159, 3, 70, "TOCSection"],
Cell[11503, 380, 168, 3, 70, "TOCSubsection"],
Cell[11674, 385, 168, 3, 70, "TOCSubsection"],
Cell[11845, 390, 180, 3, 70, "TOCSubsection"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12062, 398, 156, 3, 70, "TOCSection"],
Cell[12221, 403, 168, 3, 70, "TOCSubsection"],
Cell[12392, 408, 168, 3, 70, "TOCSubsection"],
Cell[12563, 413, 179, 3, 70, "TOCSubsection"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12779, 421, 152, 3, 70, "TOCSection"],
Cell[12934, 426, 168, 3, 70, "TOCSubsection"],
Cell[13105, 431, 168, 3, 70, "TOCSubsection"],
Cell[13276, 436, 180, 3, 70, "TOCSubsection"]
}, Open  ]],
Cell[13471, 442, 195, 4, 70, "TOCSection"]
}, Closed]],
Cell[13681, 449, 62, 1, 70, "Section",
 CellID->66319223],
Cell[CellGroupData[{
Cell[13768, 454, 167, 4, 70, "TOCChapter"],
Cell[CellGroupData[{
Cell[13960, 462, 180, 4, 70, "TOCSection"],
Cell[14143, 468, 175, 4, 70, "TOCSubsection"],
Cell[14321, 474, 176, 4, 70, "TOCSubsection"],
Cell[14500, 480, 172, 4, 70, "TOCSubsection"],
Cell[14675, 486, 168, 4, 70, "TOCSubsection"],
Cell[14846, 492, 174, 4, 70, "TOCSubsection"],
Cell[15023, 498, 171, 4, 70, "TOCSubsection"]
}, Open  ]],
Cell[15209, 505, 164, 4, 70, "TOCSection"]
}, Closed]],
Cell[CellGroupData[{
Cell[15410, 514, 182, 4, 70, "TOCChapter",
 CellID->490140180],
Cell[15595, 520, 186, 4, 70, "TOCSection",
 CellID->95216419],
Cell[15784, 526, 187, 4, 70, "TOCSection",
 CellID->470836979],
Cell[15974, 532, 191, 4, 70, "TOCSection",
 CellID->55463762],
Cell[16168, 538, 193, 4, 70, "TOCSection",
 CellID->452240740],
Cell[16364, 544, 173, 4, 70, "TOCSection",
 CellID->231693941]
}, Closed]],
Cell[CellGroupData[{
Cell[16574, 553, 155, 3, 70, "TOCChapter"],
Cell[16732, 558, 161, 3, 70, "TOCSection"],
Cell[CellGroupData[{
Cell[16918, 565, 247, 4, 70, "TOCSection"],
Cell[CellGroupData[{
Cell[17190, 573, 166, 3, 70, "TOCSubsection"],
Cell[17359, 578, 160, 4, 70, "TOCSubsubsection"],
Cell[17522, 584, 182, 3, 70, "TOCSubsubsection"],
Cell[17707, 589, 218, 5, 70, "TOCSubsubsection"],
Cell[17928, 596, 162, 4, 70, "TOCSubsubsection"]
}, Open  ]],
Cell[18105, 603, 198, 4, 70, "TOCSubsection"]
}, Open  ]],
Cell[CellGroupData[{
Cell[18340, 612, 241, 4, 70, "TOCSection"],
Cell[CellGroupData[{
Cell[18606, 620, 166, 3, 70, "TOCSubsection"],
Cell[18775, 625, 157, 3, 70, "TOCSubsubsection"],
Cell[18935, 630, 208, 5, 70, "TOCSubsubsection"]
}, Open  ]],
Cell[19158, 638, 202, 4, 70, "TOCSubsection"]
}, Open  ]],
Cell[CellGroupData[{
Cell[19397, 647, 254, 4, 70, "TOCSection"],
Cell[CellGroupData[{
Cell[19676, 655, 165, 3, 70, "TOCSubsection"],
Cell[19844, 660, 160, 4, 70, "TOCSubsubsection"],
Cell[20007, 666, 187, 4, 70, "TOCSubsubsection"],
Cell[20197, 672, 157, 3, 70, "TOCSubsubsection"],
Cell[20357, 677, 222, 5, 70, "TOCSubsubsection"],
Cell[20582, 684, 228, 4, 70, "TOCSubsubsection"],
Cell[20813, 690, 162, 4, 70, "TOCSubsubsection"]
}, Open  ]],
Cell[20990, 697, 189, 3, 70, "TOCSubsection"]
}, Open  ]],
Cell[21194, 703, 150, 3, 70, "TOCSection"]
}, Closed]],
Cell[CellGroupData[{
Cell[21381, 711, 162, 3, 70, "TOCChapter"],
Cell[21546, 716, 166, 3, 70, "TOCSection"],
Cell[21715, 721, 158, 3, 70, "TOCSection"],
Cell[21876, 726, 158, 3, 70, "TOCSection"],
Cell[22037, 731, 155, 3, 70, "TOCSection"],
Cell[22195, 736, 160, 3, 70, "TOCSection"]
}, Closed]],
Cell[CellGroupData[{
Cell[22392, 744, 171, 4, 70, "TOCChapter"],
Cell[22566, 750, 168, 4, 70, "TOCSection"],
Cell[CellGroupData[{
Cell[22759, 758, 163, 4, 70, "TOCSection"],
Cell[22925, 764, 168, 4, 70, "TOCSubsection"],
Cell[23096, 770, 173, 4, 70, "TOCSubsection"],
Cell[23272, 776, 174, 4, 70, "TOCSubsection"]
}, Open  ]],
Cell[CellGroupData[{
Cell[23483, 785, 173, 4, 70, "TOCSection"],
Cell[23659, 791, 163, 4, 70, "TOCSubsection"],
Cell[23825, 797, 174, 4, 70, "TOCSubsection"]
}, Open  ]],
Cell[24014, 804, 164, 3, 70, "TOCSection"],
Cell[24181, 809, 187, 4, 70, "TOCSection"]
}, Closed]],
Cell[CellGroupData[{
Cell[24405, 818, 176, 4, 70, "TOCChapter"],
Cell[24584, 824, 170, 4, 70, "TOCSection"],
Cell[24757, 830, 179, 4, 70, "TOCSection"],
Cell[24939, 836, 212, 5, 70, "TOCSection"],
Cell[25154, 843, 225, 6, 70, "TOCSection",
 CellID->393157910]
}, Closed]],
Cell[25394, 852, 204, 5, 70, "TOCChapter",
 CellID->357436865],
Cell[25601, 859, 156, 3, 70, "Section",
 CellID->479048299],
Cell[25760, 864, 48, 1, 70, "Section",
 CellID->17255043],
Cell[CellGroupData[{
Cell[25833, 869, 168, 3, 70, "TOCChapter"],
Cell[26004, 874, 167, 3, 70, "TOCSection"],
Cell[CellGroupData[{
Cell[26196, 881, 156, 3, 70, "TOCSection"],
Cell[26355, 886, 203, 4, 70, "TOCSubsection"],
Cell[26561, 892, 164, 3, 70, "TOCSubsection"],
Cell[26728, 897, 163, 3, 70, "TOCSubsection"],
Cell[26894, 902, 146, 3, 70, "TOCSubsection"]
}, Open  ]],
Cell[CellGroupData[{
Cell[27077, 910, 157, 3, 70, "TOCSection"],
Cell[27237, 915, 180, 3, 70, "TOCSubsection"],
Cell[27420, 920, 173, 3, 70, "TOCSubsection"],
Cell[27596, 925, 192, 3, 70, "TOCSubsection"],
Cell[27791, 930, 170, 3, 70, "TOCSubsection"]
}, Open  ]],
Cell[27976, 936, 149, 3, 70, "TOCSection"]
}, Closed]],
Cell[28140, 942, 139, 2, 70, "TOCChapter"],
Cell[28282, 946, 149, 3, 70, "Section"],
Cell[CellGroupData[{
Cell[28456, 953, 50, 1, 70, "Section",
 CellID->346595432],
Cell[28509, 956, 138, 2, 70, "TOCSection"],
Cell[28650, 960, 136, 2, 70, "TOCSection"],
Cell[28789, 964, 148, 2, 70, "TOCSection"],
Cell[28940, 968, 130, 2, 70, "TOCSection"],
Cell[29073, 972, 122, 2, 70, "TOCSection"]
}, Open  ]]
}, Open  ]],
Cell[29222, 978, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

