(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     39023,        930]
NotebookOptionsPosition[     34398,        796]
NotebookOutlinePosition[     36921,        854]
CellTagsIndexPosition[     36878,        851]
WindowTitle->1.4 Airfoils from Conformal Mapping - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"1.3 Airfoils From Thickness And Camber Properties\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/\
AirfoilsFromThicknessAndCamberProperties"], "\<\"1.5 Panel Geometry\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/PanelGeometry"], "\<\"Overview\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"AirfoilGeometry\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/AirfoilGeometry"], "\<\"Aerodynamics\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/Aerodynamics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/tutorial/AirfoilsFromConformalMapping\"\>\
":>None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/AirfoilsFromConformalMapping"]}, 
            Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/AirfoilsFromConformalMapping.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/AirfoilsFromConformalMapping", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["1.4 Airfoils from Conformal Mapping", "Title",
 CellID->509267359],

Cell[TextData[{
 "The example used in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " for airfoils from conformal mapping is the van de Vooren family of \
symmetric airfoils.  The van de Vooren airfoils provide a collection of \
airfoils with finite trailing-edge angles for which exact solutions for \
potential flow are known.  See Section 6.6 of ",
 ButtonBox["Katz and Plotkin (2001)",
  BaseStyle->"Hyperlink",
  ButtonData->{"References.nb", "katz2001"}],
 " for details.  Such airfoils are useful for validating numerical models."
}], "Text",
 CellID->1534169418],

Cell[TextData[{
 "The function ",
 Cell[BoxData[
  ButtonBox["AirfoilSurfacePoints",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilSurfacePoints"]], 
  "InlineFormula"],
 " computes a collection of vectors on the surface of the airfoil from two \
shape parameters and a list of ",
 Cell[BoxData[
  StyleBox["\[Theta]", "TR"]], "InlineFormula"],
 "-values locating points on the circle that maps to the airfoil under the \
van de Vooren transformation.  The range of ",
 Cell[BoxData[
  StyleBox["\[Theta]", "TR"]], "InlineFormula"],
 " is from ",
 Cell[BoxData["0"], "InlineFormula"],
 " to ",
 Cell[BoxData[
  StyleBox["\[Pi]", "TR"]], "InlineFormula"],
 ", and points on the lower surface of the airfoil are obtained from the \
mirror symmetry of the symmetric airfoil."
}], "Text",
 CellID->455105011],

Cell[TextData[{
 "Load ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "Text",
 CellID->23403085],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->71496216],

Cell[CellGroupData[{

Cell["1.4.1 van de Vooren airfoil of 12% thickness", "Section",
 CellID->320326959],

Cell[TextData[{
 "A van de Vooren airfoil of 12% maximum thickness is used to validate \
results of numerical models in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  User-specified discretizations need to be based on a list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values to be consistent with the methods used for airfoils from data files \
and airfoils specified by thickness and camber properties.  The function ",
 Cell[BoxData[
  ButtonBox["XToTheta",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/XToTheta"]], "InlineFormula"],
 " provides the tool for  converting ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values to ",
 Cell[BoxData[
  StyleBox["\[Theta]", "TR"]], "InlineFormula"],
 "-values."
}], "Text",
 CellID->26627051],

Cell[TextData[{
 "Specify shape parameters for a 12% maximum thickness airfoil and a \
relatively finely spaced list of ",
 Cell[BoxData[
  StyleBox["\[Theta]", "TR"]], "InlineFormula"],
 " values.  Compute a collection vectors locating points on the surface of \
the airfoil.  This will serve to specify the shape of the airfoil."
}], "Text",
 CellID->310923134],

Cell[BoxData[{
 RowBox[{
  RowBox[{"t", "=", "0.12"}], ";", 
  RowBox[{"\[Tau]", "=", 
   RowBox[{"9.81", " ", "Degree"}]}], ";", 
  RowBox[{"\[Theta]", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"0", ",", "\[Pi]", ",", 
     RowBox[{"\[Pi]", "/", "100"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"r", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"t", ",", "\[Tau]", ",", "\[Theta]"}], "]"}]}], ";"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->18936054],

Cell[TextData[{
 "Compute the collection of vectors for a user-specified discretization of \
the airfoil.  Note that this discretization is specified in terms of a list \
of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values."
}], "Text",
 CellID->985219100],

Cell[BoxData[
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"t", ",", "\[Tau]", ",", 
     RowBox[{"XToTheta", "[", 
      RowBox[{"t", ",", "\[Tau]", ",", 
       RowBox[{"NDiscretizeUnitSegment", "[", 
        RowBox[{"4", ",", 
         RowBox[{"Layout", "\[Rule]", "CosineSpacing"}]}], "]"}]}], "]"}]}], 
    "]"}]}], ";"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->130944639],

Cell["\<\
Generate and show superimposed plots of the airfoil and its user-specified \
coarse discretization, shown in red.\
\>", "Text",
 CellID->367777498],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"gr", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"r", ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "0.15"}], ",", "0.15"}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"grP", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rP", ",", 
     RowBox[{"PlotStyle", "\[Rule]", "Red"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gr", ",", "grP"}], "]"}]}], "Input",
 CellLabel->"In[4]:=",
 CellID->8862207],

Cell[BoxData[
 GraphicsBox[{
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdl300Vfkax880eiNlZXKV0jFUk6lUmjRKX5KUmJLE6E0p1SHVZMqUWzIU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     "]]}, 
   {RGBColor[1, 0, 0], 
    LineBox[{{1., 0.}, {0.8535533879177307, -0.015469287303172513`}, {
     0.500000000029821, -0.050919001767322294`}, {
     0.1464466265038108, -0.05376892827817453}, {0., 
     1.1102230246251565`*^-16}, {0.1464466265038108, 0.05376892827817453}, {
     0.500000000029821, 0.050919001767322294`}, {0.8535533879177307, 
     0.015469287303172513`}, {1., 0.}}]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2000000000000004`, 
      FormBox["1.2000000000000004`", TraditionalForm]}, {1.4000000000000001`, 
      FormBox["1.4000000000000001`", TraditionalForm]}, {1.6, 
      FormBox["1.6`", TraditionalForm]}}, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}, None, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}},
  ImageSize->{504., 180.},
  PlotRange->{-0.15, 0.15},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->380449380]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1.4.2 van de Vooren family of airfoils", "Section",
 CellID->36431957],

Cell["\<\
The effect of the two parameters on the shape of the airfoil can be seen \
using the following dynamic example.  Use the sliders to change the parameter \
values and the shape of the airfoil.  Note that the maximum thickness of the \
airfoil is included as part of the plot label.\
\>", "Text",
 CellID->302187830],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Manipulate", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"r", "=", 
     RowBox[{"AirfoilSurfacePoints", "[", 
      RowBox[{"t", ",", "\[Tau]", ",", 
       RowBox[{"Range", "[", 
        RowBox[{"0", ",", "\[Pi]", ",", 
         RowBox[{"\[Pi]", "/", "200"}]}], "]"}]}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"thickness", "=", 
     RowBox[{"200", 
      RowBox[{"Max", "[", 
       RowBox[{
        RowBox[{"Components", "[", "r", "]"}], "[", 
        RowBox[{"[", "2", "]"}], "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"PlotAirfoil", "[", 
     RowBox[{"r", ",", 
      RowBox[{"ImageSize", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"7", " ", "72"}], "}"}]}], ",", 
      RowBox[{"PlotLabel", "\[Rule]", 
       RowBox[{"StringForm", "[", 
        RowBox[{"\"\<`` % Thick van de Vooren Airfoil\>\"", ",", 
         RowBox[{"NumberForm", "[", 
          RowBox[{"thickness", ",", 
           RowBox[{"{", 
            RowBox[{"4", ",", "2"}], "}"}]}], "]"}]}], "]"}]}], ",", 
      RowBox[{"PlotRange", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "0.4"}], ",", "0.4"}], "}"}]}]}], "]"}]}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"t", ",", "0.12", ",", "\"\<t\>\""}], "}"}], ",", "0", ",", 
     "0.25"}], "}"}], ",", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"\[Tau]", ",", "0.171217", ",", "\"\<\[Tau]\>\""}], "}"}], ",", 
     "0", ",", "1.5708"}], "}"}], ",", 
   RowBox[{"Initialization", "\[RuleDelayed]", 
    RowBox[{"Get", "[", "\"\<Aerodynamics`\>\"", "]"}]}], ",", 
   RowBox[{"SaveDefinitions", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellLabel->"In[7]:=",
 CellID->186362284],

Cell[BoxData[
 TagBox[
  StyleBox[
   DynamicModuleBox[{$CellContext`t$$ = 0.12, $CellContext`\[Tau]$$ = 
    0.17121679962064373`, Typeset`show$$ = True, Typeset`bookmarkList$$ = {}, 
    Typeset`bookmarkMode$$ = "Menu", Typeset`animator$$, Typeset`animvar$$ = 
    1, Typeset`name$$ = "\"untitled\"", Typeset`specs$$ = {{{
       Hold[$CellContext`t$$], 0.12, "t"}, 0, 0.25}, {{
       Hold[$CellContext`\[Tau]$$], 0.171217, "\[Tau]"}, 0, 1.5708}}, 
    Typeset`size$$ = {504., {194., 198.}}, Typeset`update$$ = 0, 
    Typeset`initDone$$, Typeset`skipInitDone$$ = False, $CellContext`t$336$$ =
     0, $CellContext`\[Tau]$341$$ = 0}, 
    DynamicBox[Manipulate`ManipulateBoxes[
     1, StandardForm, 
      "Variables" :> {$CellContext`t$$ = 0.12, $CellContext`\[Tau]$$ = 
        0.171217}, "ControllerVariables" :> {
        Hold[$CellContext`t$$, $CellContext`t$336$$, 0], 
        Hold[$CellContext`\[Tau]$$, $CellContext`\[Tau]$341$$, 0]}, 
      "OtherVariables" :> {
       Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
        Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
        Typeset`specs$$, Typeset`size$$, Typeset`update$$, Typeset`initDone$$,
         Typeset`skipInitDone$$}, 
      "Body" :> ($CellContext`r = 
        Aerodynamics`AirfoilGeometry`AirfoilSurfacePoints[$CellContext`t$$, \
$CellContext`\[Tau]$$, 
          Range[0, Pi, Pi/200]]; $CellContext`thickness = 200 Max[
           Part[
            Aerodynamics`CartesianVectors`Components[$CellContext`r], 2]]; 
       Aerodynamics`Graphics`PlotAirfoil[$CellContext`r, ImageSize -> {7 72}, 
         PlotLabel -> StringForm["`` % Thick van de Vooren Airfoil", 
           NumberForm[$CellContext`thickness, {4, 2}]], 
         PlotRange -> {-0.4, 0.4}]), 
      "Specifications" :> {{{$CellContext`t$$, 0.12, "t"}, 0, 
         0.25}, {{$CellContext`\[Tau]$$, 0.171217, "\[Tau]"}, 0, 1.5708}}, 
      "Options" :> {}, "DefaultOptions" :> {}],
     ImageSizeCache->{553., {254., 259.}},
     SingleEvaluation->True],
    Deinitialization:>None,
    DynamicModuleValues:>{},
    Initialization:>({{$CellContext`r = 
         Aerodynamics`CartesianVectors`CartesianVectors[CompressedData["
1:eJw9lXtQVGUYxo+3MblPCJmEQq5ToQGFCUPag+OQI+koIIoluUaRrkAQBowy
ExKgIjFgBl0gaFFRRC5yMxVZpQRciPsud5eFxV0uu1DATAraPzyc/86cc77z
fu/7/H6f/Wdf+XyxSBCEeoHXBMqtlik3zBmwIhB6m2kDsotC9MZaA4Iyor+1
VRjg5ONcJak2YPs5pBpdMiCluS/q5XgDjtkPhGUcNuBF1Jbows0GHB0v7Ape
YcAbUWqxVqnHq+8cjHaV6vGhe8yW8KN6TEpSEm846CG52z+5VDeODlFtb2ru
OHI/OBN0/ONxBC+/IykyHYdT6NOT0VVjOFJqrFQfG4NH+tNAY8sx2Lr6Sc3/
GMXa12tyTQ+N4mSevSCaHUFQ/M5H4T+PQLojM9560wjcAiKs3m3QweRZ4p2h
IzpkmooX75nWQpSlqUhL1MKl3bK30UqLyFpttij3CS7uTttb6fgEjj/0e+fd
GoZ/prGl9bZhOCV3Ll1fp0GOt5A6vVsDibuJybXWIUh0ZyrE+4dwfltG/tau
QWxaZOgUfzIIuZGzMNqrxuz1S5dnA9RoyPrUrKJ/ANb/lrq5Hh7AVOUqRfZj
FXylPp7mYhViapav3nn/MeJKLG/MPuiH0TX/1m/+6sNsgMtHE3W9MPPzX5LX
2AP/s7qX2tu6ERxonlDc04X2ONPyE5pO2JXJBw9MKpHfJg9MfaFAknfCOV8L
BWYOrb3Ssa4DwTNxi73c2yEKvdet9m1D87qygy1hragrMPvRM60Fm3tPnAst
b0aMMPn21O0mdAqRz96y/BudDxPLOyIaEP3eEq9dfY9QkDGzT7WnHm/u2Gqj
ltciZkNYQrb3Q0xFWNh8PvAnzqbo56JP1eDieK1GZPcAE7JXXitoksEiyXEi
Z2U1wn7qKJfm3IXM6Z8Dyu23USco8kufV2KvNPd0Xks5nNcP59lWlaL49/Ce
U/dLIJNbf+95rwh2gyVTPRsLUHzToe3LpquQrVpde/7KZcRekK1R3ZRCts/l
N40uGx6q54qU7l+gaqx+v9AtHTlzJrBfdgEeSYZ68elkCKX/hazpTEBs4P6v
b5nHQrhqFfedTyQEL3O/5PRgCMkhZcetxRBWXu9X/roLgjhB7pC1EQuI8J7P
+T6/n19vYf35//H/rIf1sV7Wv7Cf+f1xv9w/+8H+sF/sH/vJ/rLf7D/nwflw
Xpwf58n5ct6cP/PAfDAvzA/zxHwxb8wf88h8Mq/ML/PMfDPvzD95IB/khfyQ
J/JF3sgfeSSf5JX8kmfyTd7JP31AP9AX9Ad9Qr/QN/QPfUQ/0Vf0F31Gv9F3
9B99SD/Sl/QnfUq/0rf0L31MP9PX9Dd9Tr/T9/Q/zwOeDzwveH78D1StJuE=

           "], {
           0, -0.000029995190636323425`, -0.00011687323292118567`, \
-0.00026009404253466226`, -0.00046008728097634953`, -0.0007175032038973551, \
-0.0010330284037593346`, -0.0014073028223394674`, -0.0018408718312102955`, \
-0.0023341541748097315`, -0.002887418467516138, -0.0035007649802909387`, \
-0.004174111093773584, -0.004907179545885801, -0.0056994889765039115`, \
-0.006550346470271057, -0.007458841908328746, -0.008423844001819284, \
-0.009443997915252543, -0.01051772440737763, -0.011643220427300913`, \
-0.012818461107917185`, -0.014041203099505177`, -0.015308989184976238`, \
-0.016619154115686027`, -0.017968831603507374`, -0.019354962401444714`, \
-0.02077430340171746, -0.02222343767714466, -0.02369878538896847, \
-0.025196615482023108`, -0.026713058086472258`, -0.028244117544201128`, \
-0.029785685977395296`, -0.03133355731683066, -0.032883441707964436`, \
-0.0344309802139752, -0.03597175973646333, -0.03750132807653486, \
-0.03901520906139554, -0.040508917664365235`, -0.041977975049312805`, \
-0.043417923473860906`, -0.04482434098930453, -0.04619285587893722, \
-0.04751916078036812, -0.04879902644140573, -0.05002831506311319, \
-0.05120299318769443, -0.052319144092914785`, -0.05337297965873372, \
-0.05436085167577157, -0.05527926256902645, -0.05612487551398129, \
-0.05689452392579436, -0.05758522030570571, -0.058194164432032146`, \
-0.05871875088623772, -0.059156575907445635`, -0.05950544357154912, \
-0.059763371293568235`, -0.05992859465432676, -0.059999571554679526`, \
-0.059974985702556965`, -0.05985374943991381, -0.05963500591836979, \
-0.059318130633811944`, -0.05890273233161358, -0.058388653295309345`, \
-0.05777596903265758, -0.0570649873739516, -0.05625624699824894, \
-0.05535051540391511, -0.05434878634045237, -0.05325227671908811, \
-0.05206242302002584, -0.050780877214567866`, -0.049409502220587265`, \
-0.04795036691002563, -0.04640574068719966, -0.04477808765680902, \
-0.04307006040056982, -0.041284493381351484`, -0.03942439599372749, \
-0.03749294527973285, -0.03549347832853622, -0.03342948437865362, \
-0.031304596641185733`, -0.029122583862432716`, -0.026887341644104557`, \
-0.02460288353918616, -0.02227333194141068, -0.019902908786097295`, \
-0.01749592608001227, -0.015056776277764305`, -0.012589922522128205`, \
-0.010099888765554843`, -0.007591249790065924, -0.005068621142583468, \
-0.0025366490027270983`, 1.1102230246251565`*^-16, 0.0025366490027270983`, 
            0.005068621142583468, 0.007591249790065924, 0.010099888765554843`,
             0.012589922522128205`, 0.015056776277764305`, 
            0.01749592608001227, 0.019902908786097295`, 0.02227333194141068, 
            0.02460288353918616, 0.026887341644104557`, 0.029122583862432716`,
             0.031304596641185733`, 0.03342948437865362, 0.03549347832853622, 
            0.03749294527973285, 0.03942439599372749, 0.041284493381351484`, 
            0.04307006040056982, 0.04477808765680902, 0.04640574068719966, 
            0.04795036691002563, 0.049409502220587265`, 0.050780877214567866`,
             0.05206242302002584, 0.05325227671908811, 0.05434878634045237, 
            0.05535051540391511, 0.05625624699824894, 0.0570649873739516, 
            0.05777596903265758, 0.058388653295309345`, 0.05890273233161358, 
            0.059318130633811944`, 0.05963500591836979, 0.05985374943991381, 
            0.059974985702556965`, 0.059999571554679526`, 0.05992859465432676,
             0.059763371293568235`, 0.05950544357154912, 
            0.059156575907445635`, 0.05871875088623772, 0.058194164432032146`,
             0.05758522030570571, 0.05689452392579436, 0.05612487551398129, 
            0.05527926256902645, 0.05436085167577157, 0.05337297965873372, 
            0.052319144092914785`, 0.05120299318769443, 0.05002831506311319, 
            0.04879902644140573, 0.04751916078036812, 0.04619285587893722, 
            0.04482434098930453, 0.043417923473860906`, 0.041977975049312805`,
             0.040508917664365235`, 0.03901520906139554, 0.03750132807653486, 
            0.03597175973646333, 0.0344309802139752, 0.032883441707964436`, 
            0.03133355731683066, 0.029785685977395296`, 0.028244117544201128`,
             0.026713058086472258`, 0.025196615482023108`, 
            0.02369878538896847, 0.02222343767714466, 0.02077430340171746, 
            0.019354962401444714`, 0.017968831603507374`, 
            0.016619154115686027`, 0.015308989184976238`, 
            0.014041203099505177`, 0.012818461107917185`, 
            0.011643220427300913`, 0.01051772440737763, 0.009443997915252543, 
            0.008423844001819284, 0.007458841908328746, 0.006550346470271057, 
            0.0056994889765039115`, 0.004907179545885801, 
            0.004174111093773584, 0.0035007649802909387`, 
            0.002887418467516138, 0.0023341541748097315`, 
            0.0018408718312102955`, 0.0014073028223394674`, 
            0.0010330284037593346`, 0.0007175032038973551, 
            0.00046008728097634953`, 0.00026009404253466226`, 
            0.00011687323292118567`, 0.000029995190636323425`, 
            0}], $CellContext`t$$ = 0.12, $CellContext`\[Tau]$$ = 
         0.17121679962064373`, $CellContext`thickness = 200 Part[
            Aerodynamics`CartesianVectors`Components[
             Aerodynamics`AirfoilGeometry`AirfoilSurfacePoints[
             0.12, 0.17121679962064373`, {
              0, Pi/200, Pi/100, (3 Pi)/200, Pi/50, Pi/40, (3 Pi)/100, (7 Pi)/
               200, Pi/25, (9 Pi)/200, Pi/20, (11 Pi)/200, (3 Pi)/50, (13 Pi)/
               200, (7 Pi)/100, (3 Pi)/40, (2 Pi)/25, (17 Pi)/200, (9 Pi)/
               100, (19 Pi)/200, Pi/10, (21 Pi)/200, (11 Pi)/100, (23 Pi)/
               200, (3 Pi)/25, Pi/8, (13 Pi)/100, (27 Pi)/200, (7 Pi)/
               50, (29 Pi)/200, (3 Pi)/20, (31 Pi)/200, (4 Pi)/25, (33 Pi)/
               200, (17 Pi)/100, (7 Pi)/40, (9 Pi)/50, (37 Pi)/200, (19 Pi)/
               100, (39 Pi)/200, Pi/5, (41 Pi)/200, (21 Pi)/100, (43 Pi)/
               200, (11 Pi)/50, (9 Pi)/40, (23 Pi)/100, (47 Pi)/200, (6 Pi)/
               25, (49 Pi)/200, Pi/4, (51 Pi)/200, (13 Pi)/50, (53 Pi)/
               200, (27 Pi)/100, (11 Pi)/40, (7 Pi)/25, (57 Pi)/200, (29 Pi)/
               100, (59 Pi)/200, (3 Pi)/10, (61 Pi)/200, (31 Pi)/100, (63 Pi)/
               200, (8 Pi)/25, (13 Pi)/40, (33 Pi)/100, (67 Pi)/200, (17 Pi)/
               50, (69 Pi)/200, (7 Pi)/20, (71 Pi)/200, (9 Pi)/25, (73 Pi)/
               200, (37 Pi)/100, (3 Pi)/8, (19 Pi)/50, (77 Pi)/200, (39 Pi)/
               100, (79 Pi)/200, (2 Pi)/5, (81 Pi)/200, (41 Pi)/100, (83 Pi)/
               200, (21 Pi)/50, (17 Pi)/40, (43 Pi)/100, (87 Pi)/200, (11 Pi)/
               25, (89 Pi)/200, (9 Pi)/20, (91 Pi)/200, (23 Pi)/50, (93 Pi)/
               200, (47 Pi)/100, (19 Pi)/40, (12 Pi)/25, (97 Pi)/200, (49 Pi)/
               100, (99 Pi)/200, Pi/2, (101 Pi)/200, (51 Pi)/100, (103 Pi)/
               200, (13 Pi)/25, (21 Pi)/40, (53 Pi)/100, (107 Pi)/
               200, (27 Pi)/50, (109 Pi)/200, (11 Pi)/20, (111 Pi)/
               200, (14 Pi)/25, (113 Pi)/200, (57 Pi)/100, (23 Pi)/
               40, (29 Pi)/50, (117 Pi)/200, (59 Pi)/100, (119 Pi)/
               200, (3 Pi)/5, (121 Pi)/200, (61 Pi)/100, (123 Pi)/
               200, (31 Pi)/50, (5 Pi)/8, (63 Pi)/100, (127 Pi)/200, (16 Pi)/
               25, (129 Pi)/200, (13 Pi)/20, (131 Pi)/200, (33 Pi)/
               50, (133 Pi)/200, (67 Pi)/100, (27 Pi)/40, (17 Pi)/
               25, (137 Pi)/200, (69 Pi)/100, (139 Pi)/200, (7 Pi)/
               10, (141 Pi)/200, (71 Pi)/100, (143 Pi)/200, (18 Pi)/
               25, (29 Pi)/40, (73 Pi)/100, (147 Pi)/200, (37 Pi)/
               50, (149 Pi)/200, (3 Pi)/4, (151 Pi)/200, (19 Pi)/25, (153 Pi)/
               200, (77 Pi)/100, (31 Pi)/40, (39 Pi)/50, (157 Pi)/
               200, (79 Pi)/100, (159 Pi)/200, (4 Pi)/5, (161 Pi)/
               200, (81 Pi)/100, (163 Pi)/200, (41 Pi)/50, (33 Pi)/
               40, (83 Pi)/100, (167 Pi)/200, (21 Pi)/25, (169 Pi)/
               200, (17 Pi)/20, (171 Pi)/200, (43 Pi)/50, (173 Pi)/
               200, (87 Pi)/100, (7 Pi)/8, (22 Pi)/25, (177 Pi)/200, (89 Pi)/
               100, (179 Pi)/200, (9 Pi)/10, (181 Pi)/200, (91 Pi)/
               100, (183 Pi)/200, (23 Pi)/25, (37 Pi)/40, (93 Pi)/
               100, (187 Pi)/200, (47 Pi)/50, (189 Pi)/200, (19 Pi)/
               20, (191 Pi)/200, (24 Pi)/25, (193 Pi)/200, (97 Pi)/
               100, (39 Pi)/40, (49 Pi)/50, (197 Pi)/200, (99 Pi)/
               100, (199 Pi)/200, Pi}]], 2]}; Get["Aerodynamics`"]}; 
     Typeset`initDone$$ = True),
    SynchronousInitialization->True,
    UnsavedVariables:>{Typeset`initDone$$},
    UntrackedVariables:>{Typeset`size$$}], "Manipulate",
   Deployed->True,
   StripOnInput->False],
  Manipulate`InterpretManipulate[1]]], "Output",
 ImageSize->{557, 515},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->39351414]
}, Open  ]],

Cell[TextData[{
 "Note that this interactive example can be used to obtain the shape \
parameters for an airfoil of desired maximum thickness.  Suppose that shape \
parameters for a 15% thick van de Vooren airfoil are needed.  Click on the \
\"+\" box to the right of the slider for the ",
 Cell[BoxData[
  StyleBox["t", "TI"]], "InlineFormula"],
 " parameter to display additional controls including a box listing the value \
of ",
 Cell[BoxData[
  StyleBox["t", "TI"]], "InlineFormula"],
 ".  Change the value in the box to 0.15 and enter.  Note from the plot label \
that this produces an airfoil of 13.88% thickness.  Use the ",
 Cell[BoxData[
  StyleBox["\[Tau]", "TR"]], "InlineFormula"],
 "-slider to obtain a thickness value close to 15%.  Click the \"+\" box for \
the ",
 Cell[BoxData[
  StyleBox["\[Tau]", "TR"]], "InlineFormula"],
 " slider and insert a value for the trailing-edge angle (in radians).  Trial \
and error quickly yields a value of 0.2162 for a 15% thick airfoil.  "
}], "Text",
 CellID->860335157],

Cell["\<\
Convert the trailing-edge angle of 0.216  radians to degrees.\
\>", "Text",
 CellID->311424989],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{".2162", "/", "Degree"}]], "Input",
 CellLabel->"In[8]:=",
 CellID->58885892],

Cell[BoxData["12.387347530728398`"], "Output",
 ImageSize->{53, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->50175984]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["AirfoilGeometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/guide/AirfoilGeometry"]], "TutorialMoreAbout",
 CellID->1567025153],

Cell[TextData[ButtonBox["Aerodynamics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"]], "TutorialMoreAbout",
 CellID->529874125]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["1.3 Airfoils From Thickness And Camber Properties",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromThicknessAndCamberProperties"]], \
"RelatedTutorials",
 CellID->806871991],

Cell[TextData[ButtonBox["1.5 Panel Geometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PanelGeometry"]], "RelatedTutorials",
 CellID->242123264],

Cell[TextData[ButtonBox["Overview",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]], "RelatedTutorials",
 CellID->860107027]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"1.4 Airfoils from Conformal Mapping - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 3, 29.6784000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "None", "summary" -> 
    "The example used in Aerodynamics for airfoils from conformal mapping is \
the van de Vooren family of symmetric airfoils. The van de Vooren airfoils \
provide a collection of airfoils with finite trailing-edge angles for which \
exact solutions for potential flow are known. See Section 6.6 of Katz and \
Plotkin (2001) for details. Such airfoils are useful for validating numerical \
models. The function AirfoilSurfacePoints computes a collection of vectors on \
the surface of the airfoil from two shape parameters and a list of \
\\[Theta]-values locating points on the circle that maps to the airfoil under \
the van de Vooren transformation. The range of \\[Theta] is from 0 to \\[Pi], \
and points on the lower surface of the airfoil are obtained from the mirror \
symmetry of the symmetric airfoil. Load Aerodynamics packages.", 
    "synonyms" -> {}, "title" -> "1.4 Airfoils from Conformal Mapping", 
    "type" -> "Tutorial", "uri" -> 
    "Aerodynamics/tutorial/AirfoilsFromConformalMapping"}, "LinkTrails" -> "",
   "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[628, 21, 78, 1, 70, "TutorialColorBar"],
Cell[709, 24, 191, 5, 70, "LinkTrail"],
Cell[903, 31, 3138, 71, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4066, 106, 72, 1, 70, "Title",
 CellID->509267359],
Cell[4141, 109, 578, 13, 70, "Text",
 CellID->1534169418],
Cell[4722, 124, 827, 23, 70, "Text",
 CellID->455105011],
Cell[5552, 149, 120, 6, 70, "Text",
 CellID->23403085],
Cell[5675, 157, 99, 3, 70, "Input",
 CellID->71496216],
Cell[CellGroupData[{
Cell[5799, 164, 83, 1, 70, "Section",
 CellID->320326959],
Cell[5885, 167, 806, 22, 70, "Text",
 CellID->26627051],
Cell[6694, 191, 363, 8, 70, "Text",
 CellID->310923134],
Cell[7060, 201, 491, 14, 70, "Input",
 CellID->18936054],
Cell[7554, 217, 277, 8, 70, "Text",
 CellID->985219100],
Cell[7834, 227, 420, 12, 70, "Input",
 CellID->130944639],
Cell[8257, 241, 157, 4, 70, "Text",
 CellID->367777498],
Cell[CellGroupData[{
Cell[8439, 249, 567, 19, 70, "Input",
 CellID->8862207],
Cell[9009, 270, 7901, 166, 201, "Output",
 CellID->380449380]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[16959, 442, 76, 1, 70, "Section",
 CellID->36431957],
Cell[17038, 445, 324, 6, 70, "Text",
 CellID->302187830],
Cell[CellGroupData[{
Cell[17387, 455, 1805, 48, 70, "Input",
 CellID->186362284],
Cell[19195, 505, 12572, 198, 70, "Output",
 CellID->39351414]
}, Open  ]],
Cell[31782, 706, 1025, 22, 70, "Text",
 CellID->860335157],
Cell[32810, 730, 105, 3, 70, "Text",
 CellID->311424989],
Cell[CellGroupData[{
Cell[32940, 737, 100, 3, 70, "Input",
 CellID->58885892],
Cell[33043, 742, 175, 5, 36, "Output",
 CellID->50175984]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[33267, 753, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[33335, 756, 167, 4, 70, "TutorialMoreAbout",
 CellID->1567025153],
Cell[33505, 762, 157, 3, 70, "TutorialMoreAbout",
 CellID->529874125]
}, Open  ]],
Cell[CellGroupData[{
Cell[33699, 770, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[33774, 773, 229, 5, 70, "RelatedTutorials",
 CellID->806871991],
Cell[34006, 780, 169, 4, 70, "RelatedTutorials",
 CellID->242123264],
Cell[34178, 786, 166, 4, 70, "RelatedTutorials",
 CellID->860107027]
}, Open  ]]
}, Open  ]],
Cell[34371, 794, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

