(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     80096,       2075]
NotebookOptionsPosition[     72816,       1850]
NotebookOutlinePosition[     75734,       1913]
CellTagsIndexPosition[     75691,       1910]
WindowTitle->1.2 Airfoils from Data Files - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"1.1 Introduction To Airfoil Geometry\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToAirfoilGeometry"], \
"\<\"1.3 Airfoils From Thickness And Camber Properties\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/\
AirfoilsFromThicknessAndCamberProperties"], "\<\"Overview\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"AirfoilGeometry\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/AirfoilGeometry"], "\<\"Aerodynamics\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/Aerodynamics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/tutorial/AirfoilsFromDataFiles\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/AirfoilsFromDataFiles"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/AirfoilsFromDataFiles.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/AirfoilsFromDataFiles", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["1.2 Airfoils from Data Files", "Title",
 CellID->509267359],

Cell[TextData[{
 "Michael Selig of the Aerospace Engineering Department at the University of \
Illinois at Urbana-Champaign maintains the ",
 ButtonBox["UIUC Airfoil Data Site",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.ae.uiuc.edu/m-selig/ads.html"], None}],
 " that provides links to useful information about airfoils and to data files \
for a collection of over 1500 airfoils.  This site serves as the primary \
source for specifying airfoil geometry in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellID->1534169418],

Cell[CellGroupData[{

Cell["1.2.1 Structure of data files", "Section",
 CellID->339343814],

Cell["\<\
Several data files from the UIUC Airfoil Coordinates Database have been \
downloaded and are located in the folder Airfoil Data for use as examples.  \
In the listing below, repetitive parts have been stripped away from path \
names.\
\>", "Text",
 CellID->100647037],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"StringTake", "[", 
     RowBox[{"#", ",", 
      RowBox[{
       RowBox[{
        RowBox[{"StringPosition", "[", 
         RowBox[{"#", ",", "\"\<Airfoil Data\\\\\>\""}], "]"}], "[", 
        RowBox[{"[", 
         RowBox[{"1", ",", "2"}], "]"}], "]"}], "-", 
       RowBox[{"StringLength", "[", "#", "]"}]}]}], "]"}], "&"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"FileNames", "[", 
    RowBox[{"\"\<Applications\\\\Aerodynamics\\\\Airfoil Data\\\\*\>\"", ",", 
     RowBox[{"{", 
      RowBox[{"$BaseDirectory", ",", "$UserBaseDirectory"}], "}"}]}], "]"}]}],
   "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->82954203],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"clarky.dat\"\>", ",", "\<\"e205.dat\"\>", 
   ",", "\<\"e374.dat\"\>", ",", "\<\"l7769.dat\"\>", 
   ",", "\<\"naca23012.dat\"\>", ",", "\<\"naca4412.dat\"\>", 
   ",", "\<\"naca4412m.dat\"\>", ",", "\<\"naca642415.dat\"\>", 
   ",", "\<\"naca747a315.dat\"\>", ",", "\<\"s1223.dat\"\>", 
   ",", "\<\"sd7003.dat\"\>", ",", "\<\"usa35b.dat\"\>"}], "}"}]], "Output",
 ImageSize->{572, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[0]=",
 CellID->34036477]
}, Open  ]],

Cell[TextData[{
 "There are three numbering conventions used to order the points specifying \
an airfoil: clockwise, counterclockwise and fore-to-aft.  Clockwise and \
counterclockwise conventions start and end at the trailing edge, traversing \
the airfoil in the specified direction.  The fore-to-aft convention traverses \
the upper surface of the airfoil first and then the lower surface in the \
specified direction.  ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " uses the clockwise convention because it is consistent with the convention \
used in American textbooks for clockwise circulation producing positive lift. \
 Data files in the UIUC Airfoil Database use both counterclockwise and \
fore-to-aft conventions."
}], "Text",
 CellID->448529733],

Cell[CellGroupData[{

Cell["Counterclockwise numbering convention", "Subsection",
 CellID->397338901],

Cell["\<\
Look at the data in the file for the Eppler-205 airfoil as an example of the \
counterclockwise numbering convention.\
\>", "Text",
 CellID->16776698],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Import", "[", "\"\<Aerodynamics\\\\Airfoil Data\\\\e205.dat\>\"", 
  "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->464615637],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"E205\"\>", ",", "\<\"(10.48%)\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.`", ",", "0.`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.99655`", ",", "0.00039`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.98649`", ",", "0.00174`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.97049`", ",", "0.00427`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.94916`", ",", "0.00778`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.92285`", ",", "0.01196`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.89175`", ",", "0.01668`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.85624`", ",", "0.02199`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.81684`", ",", "0.02786`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.77412`", ",", "0.03419`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.72866`", ",", "0.04088`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.68108`", ",", "0.04777`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.63204`", ",", "0.0547`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.58218`", ",", "0.06147`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.53217`", ",", "0.06782`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.48265`", ",", "0.07342`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.4341`", ",", "0.07785`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.3868`", ",", "0.08081`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.34101`", ",", "0.08214`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.29699`", ",", "0.08177`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.25496`", ",", "0.0797`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.21508`", ",", "0.07606`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.17764`", ",", "0.07111`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.14302`", ",", "0.06507`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.11157`", ",", "0.05811`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.0836`", ",", "0.0504`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.05937`", ",", "0.04211`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.03909`", ",", "0.03344`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.02292`", ",", "0.02461`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.01097`", ",", "0.01589`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.00331`", ",", "0.00766`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.00002`", ",", "0.00055`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.00233`", ",", 
     RowBox[{"-", "0.00506`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.01065`", ",", 
     RowBox[{"-", "0.00988`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.02419`", ",", 
     RowBox[{"-", "0.0142`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.04291`", ",", 
     RowBox[{"-", "0.01776`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.06669`", ",", 
     RowBox[{"-", "0.02053`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.09534`", ",", 
     RowBox[{"-", "0.02252`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.12864`", ",", 
     RowBox[{"-", "0.02378`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.16627`", ",", 
     RowBox[{"-", "0.02436`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.20783`", ",", 
     RowBox[{"-", "0.02435`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.2529`", ",", 
     RowBox[{"-", "0.02384`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.30097`", ",", 
     RowBox[{"-", "0.02292`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.35149`", ",", 
     RowBox[{"-", "0.02168`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.40388`", ",", 
     RowBox[{"-", "0.02021`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.45751`", ",", 
     RowBox[{"-", "0.01859`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.51174`", ",", 
     RowBox[{"-", "0.01689`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.56591`", ",", 
     RowBox[{"-", "0.01516`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.61938`", ",", 
     RowBox[{"-", "0.01345`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.67149`", ",", 
     RowBox[{"-", "0.0118`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.7216`", ",", 
     RowBox[{"-", "0.01023`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.76911`", ",", 
     RowBox[{"-", "0.00876`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.81343`", ",", 
     RowBox[{"-", "0.0074`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.854`", ",", 
     RowBox[{"-", "0.00614`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.89034`", ",", 
     RowBox[{"-", "0.00497`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.92195`", ",", 
     RowBox[{"-", "0.0038`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.9486`", ",", 
     RowBox[{"-", "0.00252`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.97017`", ",", 
     RowBox[{"-", "0.00125`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.98635`", ",", 
     RowBox[{"-", "0.00036`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.99651`", ",", 
     RowBox[{"-", "0.00003`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.`", ",", "0.`"}], "}"}]}], "}"}]], "Output",
 ImageSize->{576, 271},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->35660729]
}, Open  ]],

Cell[TextData[{
 "Note that after an initial descriptive list, the file consists of ordered \
pairs of numbers locating points on the surface of the airfoil.  It is \
conventional to plot the airfoil in a normalized airfoil-fixed coordinate \
system with the leading edge of the airfoil at the origin and  the trailing \
edge at ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     StyleBox["x", "TI"], ",", " ", 
     StyleBox["z", "TI"]}], ")"}], "=", 
   RowBox[{"(", 
    RowBox[{"1", ",", " ", "0"}], ")"}]}]], "InlineFormula"],
 ".  Observe from the data that the first and last data points are coincident \
and locate the trailing edge of the airfoil.  By looking at the ",
 Cell[BoxData[
  StyleBox["z", "TI"]], "InlineFormula"],
 "-values, observe that the data points are ordered such that they traverse \
the airfoil in a counterclockwise manner."
}], "Text",
 CellID->496848717],

Cell[TextData[{
 "The function ",
 Cell[BoxData[
  ButtonBox["AirfoilFromDataFile",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilFromDataFile"]], 
  "InlineFormula"],
 " imports the data file and casts the data in the form used in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "Text",
 CellID->271030335],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->26732741],

Cell["\<\
Import the data file and compute the collection of vectors locating data \
points.\
\>", "Text",
 CellID->521166357],

Cell[BoxData[
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<Aerodynamics\\\\Airfoil Data\\\\e205.dat\>\"", "]"}]}], 
  ";"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->613288026],

Cell["\<\
Count the number if vectors in the collection, and verify that the collection \
is of the data type CartesianVectors.\
\>", "Text",
 CellID->88025627],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"NumberOfVectors", "[", "rD", "]"}], ",", 
   RowBox[{"Head", "[", "rD", "]"}]}], "}"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->546386324],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"61", ",", "CartesianVectors"}], "}"}]], "Output",
 ImageSize->{155, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->254243351]
}, Open  ]],

Cell[TextData[{
 "For the data type CartesianVectors, all the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-components are listed together and all the ",
 Cell[BoxData[
  StyleBox["z", "TI"]], "InlineFormula"],
 "-components are listed together.  An infix arrow separates the two lists of \
coordinates.  Observe that a clockwise numbering convention is used in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellID->273275556],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rD", "=", 
  RowBox[{
  "AirfoilFromDataFile", "[", 
   "\"\<Aerodynamics\\\\Airfoil Data\\\\e205.dat\>\"", "]"}]}]], "Input",
 CellLabel->"In[5]:=",
 CellID->58269839],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "1.`", ",", "0.99651`", ",", "0.98635`", ",", "0.97017`", ",", "0.9486`", 
     ",", "0.92195`", ",", "0.89034`", ",", "0.854`", ",", "0.81343`", ",", 
     "0.76911`", ",", "0.7216`", ",", "0.67149`", ",", "0.61938`", ",", 
     "0.56591`", ",", "0.51174`", ",", "0.45751`", ",", "0.40388`", ",", 
     "0.35149`", ",", "0.30097`", ",", "0.2529`", ",", "0.20783`", ",", 
     "0.16627`", ",", "0.12864`", ",", "0.09534`", ",", "0.06669`", ",", 
     "0.04291`", ",", "0.02419`", ",", "0.01065`", ",", "0.00233`", ",", 
     "0.00002`", ",", "0.00331`", ",", "0.01097`", ",", "0.02292`", ",", 
     "0.03909`", ",", "0.05937`", ",", "0.0836`", ",", "0.11157`", ",", 
     "0.14302`", ",", "0.17764`", ",", "0.21508`", ",", "0.25496`", ",", 
     "0.29699`", ",", "0.34101`", ",", "0.3868`", ",", "0.4341`", ",", 
     "0.48265`", ",", "0.53217`", ",", "0.58218`", ",", "0.63204`", ",", 
     "0.68108`", ",", "0.72866`", ",", "0.77412`", ",", "0.81684`", ",", 
     "0.85624`", ",", "0.89175`", ",", "0.92285`", ",", "0.94916`", ",", 
     "0.97049`", ",", "0.98649`", ",", "0.99655`", ",", "1.`"}], "}"}], 
   "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{"0.`", ",", 
     RowBox[{"-", "0.00003`"}], ",", 
     RowBox[{"-", "0.00036`"}], ",", 
     RowBox[{"-", "0.00125`"}], ",", 
     RowBox[{"-", "0.00252`"}], ",", 
     RowBox[{"-", "0.0038`"}], ",", 
     RowBox[{"-", "0.00497`"}], ",", 
     RowBox[{"-", "0.00614`"}], ",", 
     RowBox[{"-", "0.0074`"}], ",", 
     RowBox[{"-", "0.00876`"}], ",", 
     RowBox[{"-", "0.01023`"}], ",", 
     RowBox[{"-", "0.0118`"}], ",", 
     RowBox[{"-", "0.01345`"}], ",", 
     RowBox[{"-", "0.01516`"}], ",", 
     RowBox[{"-", "0.01689`"}], ",", 
     RowBox[{"-", "0.01859`"}], ",", 
     RowBox[{"-", "0.02021`"}], ",", 
     RowBox[{"-", "0.02168`"}], ",", 
     RowBox[{"-", "0.02292`"}], ",", 
     RowBox[{"-", "0.02384`"}], ",", 
     RowBox[{"-", "0.02435`"}], ",", 
     RowBox[{"-", "0.02436`"}], ",", 
     RowBox[{"-", "0.02378`"}], ",", 
     RowBox[{"-", "0.02252`"}], ",", 
     RowBox[{"-", "0.02053`"}], ",", 
     RowBox[{"-", "0.01776`"}], ",", 
     RowBox[{"-", "0.0142`"}], ",", 
     RowBox[{"-", "0.00988`"}], ",", 
     RowBox[{"-", "0.00506`"}], ",", "0.00055`", ",", "0.00766`", ",", 
     "0.01589`", ",", "0.02461`", ",", "0.03344`", ",", "0.04211`", ",", 
     "0.0504`", ",", "0.05811`", ",", "0.06507`", ",", "0.07111`", ",", 
     "0.07606`", ",", "0.0797`", ",", "0.08177`", ",", "0.08214`", ",", 
     "0.08081`", ",", "0.07785`", ",", "0.07342`", ",", "0.06782`", ",", 
     "0.06147`", ",", "0.0547`", ",", "0.04777`", ",", "0.04088`", ",", 
     "0.03419`", ",", "0.02786`", ",", "0.02199`", ",", "0.01668`", ",", 
     "0.01196`", ",", "0.00778`", ",", "0.00427`", ",", "0.00174`", ",", 
     "0.00039`", ",", "0.`"}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[CompressedData["
1:eJwB9QEK/iFib1JlAQAAAD0AAAAAAAAAAADwP/phhPBo4+8/cRsN4C2Q7z8k
7rH0oQvvPwmKH2PuWu4/qoJRSZ2A7T+vQspPqn3sP4cW2c73U+s/nbryWZ4H
6j+U2SCTjJzoPyxlGeJYF+c/zemymNh85T/EfHkB9tHjP+tWz0nvG+I/48KB
kCxg4D8O8+UF2EfdP/CK4H8r2dk/H7+36c9+1j+J78SsF0PTPzojSnuDL9A/
0VeQZiyayj9jKCfaVUjFP2WNeohGd8A/8SkAxjNouD+mYfiImBKxP3+8V61M
+KU/E5uPa0PFmD/kg57Nqs+FP4P6ljldFmM/8WjjiLX49D6vCP63kh1rPyyf
5Xlwd4Y/8YCyKVd4lz8WhzO/mgOkP0sfuqC+Za4/1xLyQc9mtT/k9zb92Y+8
PwNDVrd6TsI/dxA7U+i8xj/yBwPPvYfLPzdxcr9DUdA/bkxPWOIB0z8z+Wab
G9PVP2/whclUwdg/8WPMXUvI2z+IhVrTvOPeP7oxPWGJB+E/N1SM8zeh4j9h
cTjzqznkP7O1vkhoy+U/U+i8xi5R5z+hvmVOl8XoP4gRwqONI+o/g92wbVFm
6z/wp8ZLN4nsP5yiI7n8h+0/yHvVyoRf7j+RCmMLQQ7vP+GX+nlTke8/iIVa
07zj7z8AAAAAAADwP7Ee+1o=
    "], CompressedData["
1:eJwB9QEK/iFib1JlAQAAAD0AAAAAAAAAAAAAAGkdVU0Qdf++D9b/OcyXN797
FK5H4XpUv03bv7LSpGS/xY8xdy0hb79dv2A3bFt0v9i2KLNBJnm/rK3YX3ZP
fr/GihpMw/CBvxYwgVt384S/7Q2+MJkqiL+WsgxxrIuLv1ysqME0DI+/64uE
tpxLkb9hbCHIQQmTv25pNSTusZS/F7zoK0gzlr/xgLIpV3iXvyi4WFGDaZi/
5WGh1jTvmL9SflLt0/GYv5kNMsnIWZi/5A8GnnsPl78S91j60AWVvwE1tWyt
L5K/hlrTvOMUjb9AahMn9zuEv7a+SGjLuXS/L26jAbwFQj8AOsyXF2B/P0xx
Vdl3RZA/+kSeJF0zmT+sqME0DB+hP3NLqyFxj6U/INJvXwfOqT9xICQLmMCt
P2Ni83FtqLA/wCZr1EM0sj9/pIgMq3izP0i/fR04Z7Q/Vz7L8+DutD9JhbGF
IAe1Px2UMNP2r7Q/klz+Q/rtsz9dUN8yp8uyP7HEA8qmXLE/DMhe7/54rz/E
sS5uowGsPxO4dTdPdag/PPceLjnupD+oUrMHWoGhP4Fbd/NUh5w/TS1b64uE
lj/4Nv3ZjxSRP5Gb4QZ8fog/dY4B2evdfz8FqKlla31xP6ciFcYWglw/5SfV
Ph2POT8AAAAAAAAAAPf66Iw=
    "]], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{551, 254},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->182504930]
}, Open  ]],

Cell[TextData[{
 "Plot the points specifying the airfoil using the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["PlotAirfoil",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/PlotAirfoil"]], "InlineFormula"],
 "."
}], "Text",
 CellID->681250186],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotAirfoil", "[", 
  RowBox[{"rD", ",", 
   RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellLabel->"In[6]:=",
 CellID->13906675],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3c+KK2kZwOGIVzK30SCKooiuHBBXgzBIljMu9BocRCFbUWRGRje6Gym3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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->408425645]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Fore-to-aft numbering convention", "Subsection",
 CellID->9113640],

Cell["\<\
The Clark-Y airfoil is as an example of the fore-to-aft numbering convention.\
\>", "Text",
 CellID->120129970],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Import", "[", "\"\<Aerodynamics\\\\Airfoil Data\\\\clarky.dat\>\"", 
  "]"}]], "Input",
 CellLabel->"In[7]:=",
 CellID->15342918],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"CLARK\"\>", ",", "\<\"Y\"\>", ",", "\<\"AIRFOIL\"\>"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"61.`", ",", "61.`"}], "}"}], ",", 
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.`", ",", "0.`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.0005`", ",", "0.002339`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.001`", ",", "0.0037271`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.002`", ",", "0.0058025`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.004`", ",", "0.0089238`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.008`", ",", "0.013735`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.012`", ",", "0.0178581`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.02`", ",", "0.0253735`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.03`", ",", "0.0330215`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.04`", ",", "0.0391283`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.05`", ",", "0.0442753`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.06`", ",", "0.0487571`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.08`", ",", "0.0564308`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.1`", ",", "0.0629981`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.12`", ",", "0.0686204`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.14`", ",", "0.073436`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.16`", ",", "0.0775707`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.18`", ",", "0.0810687`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.2`", ",", "0.0839202`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.22`", ",", "0.0861433`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.24`", ",", "0.0878308`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.26`", ",", "0.089084`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.28`", ",", "0.0900016`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.3`", ",", "0.0906804`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.32`", ",", "0.0911857`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.34`", ",", "0.0915079`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.36`", ",", "0.0916266`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.38`", ",", "0.0915212`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.4`", ",", "0.0911712`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.42`", ",", "0.0905657`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.44`", ",", "0.0897175`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.46`", ",", "0.0886427`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.48`", ",", "0.0873572`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.5`", ",", "0.0858772`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.52`", ",", "0.0842145`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.54`", ",", "0.0823712`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.56`", ",", "0.080348`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.58`", ",", "0.0781451`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.6`", ",", "0.0757633`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.62`", ",", "0.0732055`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.64`", ",", "0.0704822`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.66`", ",", "0.0676046`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.68`", ",", "0.0645843`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.7`", ",", "0.0614329`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.72`", ",", "0.0581599`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.74`", ",", "0.0547675`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.76`", ",", "0.0512565`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.78`", ",", "0.0476281`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.8`", ",", "0.0438836`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.82`", ",", "0.0400245`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.84`", ",", "0.0360536`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.86`", ",", "0.031974`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.88`", ",", "0.0277891`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.9`", ",", "0.0235025`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.92`", ",", "0.0191156`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.94`", ",", "0.0146239`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.96`", ",", "0.0100232`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.97`", ",", "0.0076868`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.98`", ",", "0.0053335`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.99`", ",", "0.002969`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.`", ",", "0.0005993`"}], "}"}], ",", 
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.`", ",", "0.`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.0005`", ",", 
     RowBox[{"-", "0.00467`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.001`", ",", 
     RowBox[{"-", "0.0059418`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.002`", ",", 
     RowBox[{"-", "0.0078113`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.004`", ",", 
     RowBox[{"-", "0.0105126`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.008`", ",", 
     RowBox[{"-", "0.0142862`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.012`", ",", 
     RowBox[{"-", "0.0169733`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.02`", ",", 
     RowBox[{"-", "0.0202723`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.03`", ",", 
     RowBox[{"-", "0.0226056`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.04`", ",", 
     RowBox[{"-", "0.0245211`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.05`", ",", 
     RowBox[{"-", "0.0260452`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.06`", ",", 
     RowBox[{"-", "0.0271277`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.08`", ",", 
     RowBox[{"-", "0.0284595`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.1`", ",", 
     RowBox[{"-", "0.0293786`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.12`", ",", 
     RowBox[{"-", "0.0299633`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.14`", ",", 
     RowBox[{"-", "0.0302404`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.16`", ",", 
     RowBox[{"-", "0.0302546`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.18`", ",", 
     RowBox[{"-", "0.030049`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.2`", ",", 
     RowBox[{"-", "0.0296656`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.22`", ",", 
     RowBox[{"-", "0.0291445`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.24`", ",", 
     RowBox[{"-", "0.0285181`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.26`", ",", 
     RowBox[{"-", "0.0278164`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.28`", ",", 
     RowBox[{"-", "0.0270696`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.3`", ",", 
     RowBox[{"-", "0.0263079`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.32`", ",", 
     RowBox[{"-", "0.0255565`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.34`", ",", 
     RowBox[{"-", "0.0248176`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.36`", ",", 
     RowBox[{"-", "0.024087`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.38`", ",", 
     RowBox[{"-", "0.0233606`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.4`", ",", 
     RowBox[{"-", "0.0226341`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.42`", ",", 
     RowBox[{"-", "0.0219042`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.44`", ",", 
     RowBox[{"-", "0.0211708`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.46`", ",", 
     RowBox[{"-", "0.0204353`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.48`", ",", 
     RowBox[{"-", "0.0196986`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.5`", ",", 
     RowBox[{"-", "0.0189619`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.52`", ",", 
     RowBox[{"-", "0.0182262`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.54`", ",", 
     RowBox[{"-", "0.0174914`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.56`", ",", 
     RowBox[{"-", "0.0167572`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.58`", ",", 
     RowBox[{"-", "0.0160232`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.6`", ",", 
     RowBox[{"-", "0.0152893`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.62`", ",", 
     RowBox[{"-", "0.0145551`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.64`", ",", 
     RowBox[{"-", "0.0138207`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.66`", ",", 
     RowBox[{"-", "0.0130862`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.68`", ",", 
     RowBox[{"-", "0.0123515`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.7`", ",", 
     RowBox[{"-", "0.0116169`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.72`", ",", 
     RowBox[{"-", "0.0108823`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.74`", ",", 
     RowBox[{"-", "0.0101478`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.76`", ",", 
     RowBox[{"-", "0.0094133`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.78`", ",", 
     RowBox[{"-", "0.0086788`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.8`", ",", 
     RowBox[{"-", "0.0079443`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.82`", ",", 
     RowBox[{"-", "0.0072098`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.84`", ",", 
     RowBox[{"-", "0.0064753`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.86`", ",", 
     RowBox[{"-", "0.0057408`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.88`", ",", 
     RowBox[{"-", "0.0050063`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.9`", ",", 
     RowBox[{"-", "0.0042718`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.92`", ",", 
     RowBox[{"-", "0.0035373`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.94`", ",", 
     RowBox[{"-", "0.0028028`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.96`", ",", 
     RowBox[{"-", "0.0020683`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.97`", ",", 
     RowBox[{"-", "0.0017011`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.98`", ",", 
     RowBox[{"-", "0.0013339`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.99`", ",", 
     RowBox[{"-", "0.0009666`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.`", ",", 
     RowBox[{"-", "0.0005993`"}]}], "}"}]}], "}"}]], "Output",
 ImageSize->{569, 526},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->206055685]
}, Open  ]],

Cell["\<\
The format for the information in the file differs from that used for the \
Eppler-205 airfoil.  After identifying the airfoil, the number of points on \
upper and lower surfaces are given and then an empty list before beginning \
the ordered pairs of numbers.\
\>", "Text",
 CellID->398895014],

Cell[TextData[{
 "The function AirfoilFromDataFile determines the numbering convention and \
rearranges the data to the form used by ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  Plotting the data points provides information on the distribution of \
points on the airfoil, and joining the data points can reveal surprises in \
data files that require attention.  It is a good idea to look at both plots."
}], "Text",
 CellID->583917234],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rDCY", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<Aerodynamics\\\\Airfoil Data\\\\clarky.dat\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gData", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rDCY", ",", 
     RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gProfile", "=", 
   RowBox[{"PlotAirfoil", "[", "rDCY", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"GraphicsGrid", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", "gData", "}"}], ",", 
    RowBox[{"{", "gProfile", "}"}]}], "}"}], 
  "]"}], "\[IndentingNewLine]"}], "Input",
 CellLabel->"In[8]:=",
 CellID->294412045],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3d+rJNWdAPC7u3/HPuy/cUHc7OBujCHGgPggw4JCoy/JsuguJCybvGhY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    "], {{0, 396}, {541, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{541, 396},
  PlotRange->{{0, 541}, {0, 396}}]], "Output",
 ImageSize->{545, 396},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->136981492]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1.2.2 Discretization of airfoil", "Section",
 CellID->636677227],

Cell[TextData[{
 "Data in the file can be used to specify panel endpoints for a discretized \
airfoil.  More flexibility in discretization is obtained by fitting the data \
in the file with a B-spline and basing a user-specified discretization of the \
airfoil on this spline function.  The ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["FitAirfoil",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/FitAirfoil"]], "InlineFormula"],
 " contains code furnished by the Wolfram Development Team to accomplish this \
task.  The spline function obtained from FitAirfoil is used together with a \
user-specified list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values in the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["AirfoilSurfacePoints",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilSurfacePoints"]], 
  "InlineFormula"],
 " to rediscretize the airfoil. "
}], "Text",
 CellID->20949403],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"spline", "=", 
  RowBox[{"FitAirfoil", "[", "rD", "]"}]}]], "Input",
 CellLabel->"In[12]:=",
 CellID->47856896],

Cell[BoxData[
 TagBox[
  RowBox[{"BSplineFunction", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"{", 
      RowBox[{"0.`", ",", "1.0000000000000004`"}], "}"}], "}"}], 
    ",", "\<\"<>\"\>"}], "]"}],
  False,
  Editable->False]], "Output",
 ImageSize->{215, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->772196878]
}, Open  ]],

Cell[TextData[{
 "Use the function ",
 Cell[BoxData[
  ButtonBox["NDiscretizeUnitSegment",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/NDiscretizeUnitSegment"]], 
  "InlineFormula"],
 " to generate a user-specified discretization list."
}], "Text",
 CellID->284760479],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"list", "=", 
  RowBox[{"NDiscretizeUnitSegment", "[", 
   RowBox[{"5", ",", 
    RowBox[{"Layout", "\[Rule]", "HalfCosineSpacing"}]}], "]"}]}]], "Input",
 CellLabel->"In[13]:=",
 CellID->11317880],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.`", ",", "0.04894348370484647`", ",", "0.19098300562505255`", ",", 
   "0.412214747707527`", ",", "0.6909830056250525`", ",", 
   "0.9999999999999999`"}], "}"}]], "Output",
 ImageSize->{332, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[13]=",
 CellID->18194836]
}, Open  ]],

Cell["\<\
Compute panel endpoints for the rediscretized airfoil; display these points \
and the spline.\
\>", "Text",
 CellID->269235534],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"spline", ",", "list"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gP", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rP", ",", 
     RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gFit", "=", 
   RowBox[{"ParametricPlot", "[", 
    RowBox[{
     RowBox[{"spline", "[", "p", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"p", ",", "0", ",", "1"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gP", ",", "gFit"}], "]"}]}], "Input",
 CellLabel->"In[14]:=",
 CellID->354096070],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3f+LFPcdx/Er/Uv6bxyYO3ux1F6qGLD+INc0miz6SyVBISiU9IfGEM52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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[17]=",
 CellID->176495906]
}, Open  ]],

Cell["Here is the user-specified discretized airfoil.", "Text",
 CellID->319617898],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotAirfoil", "[", "rP", "]"}]], "Input",
 CellLabel->"In[18]:=",
 CellID->9442322],

Cell[BoxData[
 GraphicsBox[
  {Hue[0.67, 0.6, 0.6], 
   LineBox[{{1.0000000000000004`, 7.153015998859038*^-18}, {
    0.6909887433900884, -0.011187015789188594`}, {
    0.41222571434109023`, -0.01996389793711672}, {
    0.19099811681454748`, -0.024417860421906704`}, {
    0.048967138552224114`, -0.01855333624731288}, {0.000018587349363911095`, 
    0.0006856384489956231}, {0.04894337324092631, 0.03790927266294925}, {
    0.19099241379597795`, 0.07305252935944304}, {0.4122289818566037, 
    0.07940274931287293}, {0.6909892337764048, 0.046346760822275956`}, {
    0.9999999999999991, 9.30142885511711*^-17}}]},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2000000000000004`, 
      FormBox["1.2000000000000004`", TraditionalForm]}, {1.4000000000000001`, 
      FormBox["1.4000000000000001`", TraditionalForm]}, {1.6, 
      FormBox["1.6`", TraditionalForm]}}, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}, None, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}},
  ImageSize->{504., 180.},
  ImageSizeRaw->{504, 180},
  PlotRange->{{-0.05, 1.05}, {-0.1, 0.201}},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[18]=",
 CellID->534850014]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1.2.3 Data from the UIUC Airfoil Coordinate Database", "Section",
 CellID->114536833],

Cell[TextData[{
 "Data files for airfoil coordinates can be imported directly from the ",
 ButtonBox["UIUC Airfoil Coordinates Database",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.ae.uiuc.edu/m-selig/ads/coord_database.html"], None}],
 ".  As an example, locate the data file for the Eppler-550 general aviation \
airfoil.  Place the cursor over the link to the airfoil data file, and note \
that the URL (address) is displayed at the bottom left of the screen.  Look \
at both the data points and the discretized airfoil specified by the data \
file."
}], "Text",
 CellID->342063873],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<http://www.ae.uiuc.edu/m-selig/ads/coord/e550.dat\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gData", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gProfile", "=", 
   RowBox[{"PlotAirfoil", "[", "rD", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"GraphicsGrid", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", "gData", "}"}], ",", 
    RowBox[{"{", "gProfile", "}"}]}], "}"}], "]"}]}], "Input",
 CellLabel->"In[19]:=",
 CellID->458862603],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3V2LHFd6AOBJ8jtykb8h2Hizija7LMRxQIhgTECCwb7Zvdm9sW+kGzsE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    "], {{0, 396}, {541, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{541, 396},
  PlotRange->{{0, 541}, {0, 396}}]], "Output",
 ImageSize->{545, 396},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[22]=",
 CellID->380391698]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1.2.4 Surprises", "Section",
 CellID->577231104],

Cell[TextData[{
 "If you are importing airfoil data from a file prepared by someone else, \
review it carefully before using.  Data files in the UIUC collection are \
acquired from different sources, and conventions used may not be what you \
expect.  I had several surprises using the limited number of airfoils \
selected to test functions in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  Some of these surprises are listed in the ",
 ButtonBox["Possible Issues",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/AirfoilFromDataFile#1305812373"],
 " section of the documentation page for the function AirfoilFromDataFile."
}], "Text",
 CellID->9676428]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["AirfoilGeometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/guide/AirfoilGeometry"]], "TutorialMoreAbout",
 CellID->1567025153],

Cell[TextData[ButtonBox["Aerodynamics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"]], "TutorialMoreAbout",
 CellID->80129204]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["1.1 Introduction To Airfoil Geometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToAirfoilGeometry"]], \
"RelatedTutorials",
 CellID->574311372],

Cell[TextData[ButtonBox["1.3 Airfoils From Thickness And Camber Properties",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromThicknessAndCamberProperties"]], \
"RelatedTutorials",
 CellID->895221539],

Cell[TextData[ButtonBox["Overview",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]], "RelatedTutorials",
 CellID->6988088]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"1.2 Airfoils from Data Files - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 3, 34.6080000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "None", "summary" -> 
    "Michael Selig of the Aerospace Engineering Department at the University \
of Illinois at Urbana-Champaign maintains the UIUC Airfoil Data Site that \
provides links to useful information about airfoils and to data files for a \
collection of over 1500 airfoils. This site serves as the primary source for \
specifying airfoil geometry in Aerodynamics. Several data files from the UIUC \
Airfoil Coordinates Database have been downloaded and are located in the \
folder Airfoil Data for use as examples. In the listing below, repetitive \
parts have been stripped away from path names. There are three numbering \
conventions used to order the points specifying an airfoil: clockwise, \
counterclockwise and fore-to-aft. Clockwise and counterclockwise conventions \
start and end at the trailing edge, traversing the airfoil in the specified \
direction. The fore-to-aft convention traverses the upper surface of the \
airfoil first and then the lower surface in the specified direction. \
Aerodynamics uses the clockwise convention because it is consistent with the \
convention used in American textbooks for clockwise circulation producing \
positive lift. Data files in the UIUC Airfoil Database use both \
counterclockwise and fore-to-aft conventions.", "synonyms" -> {}, "title" -> 
    "1.2 Airfoils from Data Files", "type" -> "Tutorial", "uri" -> 
    "Aerodynamics/tutorial/AirfoilsFromDataFiles"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[621, 21, 78, 1, 70, "TutorialColorBar"],
Cell[702, 24, 191, 5, 70, "LinkTrail"],
Cell[896, 31, 3143, 71, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4064, 106, 65, 1, 70, "Title",
 CellID->509267359],
Cell[4132, 109, 567, 14, 70, "Text",
 CellID->1534169418],
Cell[CellGroupData[{
Cell[4724, 127, 68, 1, 70, "Section",
 CellID->339343814],
Cell[4795, 130, 277, 6, 70, "Text",
 CellID->100647037],
Cell[CellGroupData[{
Cell[5097, 140, 699, 20, 70, "Input",
 CellID->82954203],
Cell[5799, 162, 538, 12, 54, "Output",
 CellID->34036477]
}, Open  ]],
Cell[6352, 177, 766, 14, 70, "Text",
 CellID->448529733],
Cell[CellGroupData[{
Cell[7143, 195, 79, 1, 70, "Subsection",
 CellID->397338901],
Cell[7225, 198, 160, 4, 70, "Text",
 CellID->16776698],
Cell[CellGroupData[{
Cell[7410, 206, 152, 4, 70, "Input",
 CellID->464615637],
Cell[7565, 212, 5275, 159, 292, "Output",
 CellID->35660729]
}, Open  ]],
Cell[12855, 374, 901, 21, 70, "Text",
 CellID->496848717],
Cell[13759, 397, 420, 15, 70, "Text",
 CellID->271030335],
Cell[14182, 414, 99, 3, 70, "Input",
 CellID->26732741],
Cell[14284, 419, 126, 4, 70, "Text",
 CellID->521166357],
Cell[14413, 425, 217, 8, 70, "Input",
 CellID->613288026],
Cell[14633, 435, 160, 4, 70, "Text",
 CellID->88025627],
Cell[CellGroupData[{
Cell[14818, 443, 192, 6, 70, "Input",
 CellID->546386324],
Cell[15013, 451, 220, 7, 36, "Output",
 CellID->254243351]
}, Open  ]],
Cell[15248, 461, 468, 13, 70, "Text",
 CellID->273275556],
Cell[CellGroupData[{
Cell[15741, 478, 192, 6, 70, "Input",
 CellID->58269839],
Cell[15936, 486, 4579, 92, 275, "Output",
 CellID->182504930]
}, Open  ]],
Cell[20530, 581, 309, 11, 70, "Text",
 CellID->681250186],
Cell[CellGroupData[{
Cell[20864, 596, 169, 5, 28, "Input",
 CellID->13906675],
Cell[21036, 603, 3537, 65, 201, "Output",
 CellID->408425645]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[24622, 674, 72, 1, 70, "Subsection",
 CellID->9113640],
Cell[24697, 677, 121, 3, 70, "Text",
 CellID->120129970],
Cell[CellGroupData[{
Cell[24843, 684, 153, 4, 70, "Input",
 CellID->15342918],
Cell[24999, 690, 10423, 318, 547, "Output",
 CellID->206055685]
}, Open  ]],
Cell[35437, 1011, 304, 6, 70, "Text",
 CellID->398895014],
Cell[35744, 1019, 448, 9, 70, "Text",
 CellID->583917234],
Cell[CellGroupData[{
Cell[36217, 1032, 723, 23, 101, "Input",
 CellID->294412045],
Cell[36943, 1057, 8492, 147, 400, "Output",
 CellID->136981492]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[45496, 1211, 70, 1, 70, "Section",
 CellID->636677227],
Cell[45569, 1214, 1044, 28, 70, "Text",
 CellID->20949403],
Cell[CellGroupData[{
Cell[46638, 1246, 135, 4, 70, "Input",
 CellID->47856896],
Cell[46776, 1252, 376, 14, 36, "Output",
 CellID->772196878]
}, Open  ]],
Cell[47167, 1269, 286, 9, 70, "Text",
 CellID->284760479],
Cell[CellGroupData[{
Cell[47478, 1282, 220, 6, 70, "Input",
 CellID->11317880],
Cell[47701, 1290, 348, 10, 36, "Output",
 CellID->18194836]
}, Open  ]],
Cell[48064, 1303, 137, 4, 70, "Text",
 CellID->269235534],
Cell[CellGroupData[{
Cell[48226, 1311, 698, 22, 83, "Input",
 CellID->354096070],
Cell[48927, 1335, 5373, 96, 201, "Output",
 CellID->176495906]
}, Open  ]],
Cell[54315, 1434, 83, 1, 70, "Text",
 CellID->319617898],
Cell[CellGroupData[{
Cell[54423, 1439, 107, 3, 70, "Input",
 CellID->9442322],
Cell[54533, 1444, 5508, 125, 201, "Output",
 CellID->534850014]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[60090, 1575, 91, 1, 70, "Section",
 CellID->114536833],
Cell[60184, 1578, 604, 12, 70, "Text",
 CellID->342063873],
Cell[CellGroupData[{
Cell[60813, 1594, 699, 22, 83, "Input",
 CellID->458862603],
Cell[61515, 1618, 9317, 160, 404, "Output",
 CellID->380391698]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[70881, 1784, 54, 1, 70, "Section",
 CellID->577231104],
Cell[70938, 1787, 677, 14, 70, "Text",
 CellID->9676428]
}, Open  ]],
Cell[CellGroupData[{
Cell[71652, 1806, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[71720, 1809, 167, 4, 70, "TutorialMoreAbout",
 CellID->1567025153],
Cell[71890, 1815, 156, 3, 70, "TutorialMoreAbout",
 CellID->80129204]
}, Open  ]],
Cell[CellGroupData[{
Cell[72083, 1823, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[72158, 1826, 205, 5, 70, "RelatedTutorials",
 CellID->574311372],
Cell[72366, 1833, 229, 5, 70, "RelatedTutorials",
 CellID->895221539],
Cell[72598, 1840, 164, 4, 70, "RelatedTutorials",
 CellID->6988088]
}, Open  ]]
}, Open  ]],
Cell[72789, 1848, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

