(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     42282,       1127]
NotebookOptionsPosition[     36078,        947]
NotebookOutlinePosition[     39238,       1015]
CellTagsIndexPosition[     39195,       1012]
WindowTitle->1.3 Airfoils from Thickness And Camber Properties - Wolfram \
Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"1.2 Airfoils From Data Files\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AirfoilsFromDataFiles"], "\<\"1.4 \
Airfoils From Conformal Mapping\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AirfoilsFromConformalMapping"], \
"\<\"Overview\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"AirfoilGeometry\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/AirfoilGeometry"], "\<\"Aerodynamics\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/Aerodynamics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/tutorial/\
AirfoilsFromThicknessAndCamberProperties\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
             "Aerodynamics/tutorial/AirfoilsFromThicknessAndCamberProperties"]\
}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/AirfoilsFromThicknessAndCamberProperties.html"], StandardForm]], 
              "Input", TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/AirfoilsFromThicknessAndCamberProperties",
              ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[BoxData[" "], "Input",
 CellLabel->"In[1]:=",
 CellID->376078495],

Cell[CellGroupData[{

Cell["1.3 Airfoils from Thickness And Camber Properties", "Title",
 CellID->509267359],

Cell[TextData[{
 "During the 1930s and 1940s, the National Advisory Committee for Aeronautics \
(NACA) developed and tested families of airfoils based on systematic \
variations of thickness and camber.  Algebraic equations were used to \
prescribe the thickness and camber of an airfoil, and the upper and lower \
surfaces of the airfoil were established by wrapping a thickness envelope \
around a camber line.  See Section 4.3 of ",
 ButtonBox["Anderson (1984)",
  BaseStyle->"Hyperlink",
  ButtonData->{"References.nb", "anderson1984"}],
 " or Chapter 6 of ",
 ButtonBox["Abbott and von Doenhoff (1959)",
  BaseStyle->"Hyperlink",
  ButtonData->{"References.nb", "abbott1959"}],
 " for more details."
}], "Text",
 CellID->237225321],

Cell[TextData[{
 "The example used in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " for airfoils specified by thickness and camber properties is the NACA \
four-digit family of airfoils.  If the four-digit identification is written \
as ",
 Cell[BoxData[
  RowBox[{
   StyleBox["id", "TI"], "=", 
   StyleBox["ijkl", "TI"]}]], "InlineFormula"],
 ", then ",
 Cell[BoxData[
  StyleBox["i", "TI"]], "InlineFormula"],
 " specifies maximum camber in percent chord, ",
 Cell[BoxData[
  StyleBox["j", "TI"]], "InlineFormula"],
 " specifies location of maximum camber in tenths chord and ",
 Cell[BoxData[
  StyleBox["kl", "TI"]], "InlineFormula"],
 " specifies maximum thickness in percent chord. "
}], "Text",
 CellID->3719437],

Cell[TextData[{
 "The input arguments for the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["NACA4DigitAirfoil",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/NACA4DigitAirfoil"]], "InlineFormula"],
 " are the airfoil identification number and a list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values at which thickness and camber properties are to be computed.  The \
output of the function is the nested list ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{
    StyleBox["x", "TI"], ",", 
    SubscriptBox[
     StyleBox["z", "TI"], 
     StyleBox["t", "TI"]], ",", 
    SubscriptBox[
     StyleBox["\[Theta]", "TR"], 
     StyleBox["t", "TI"]], ",", " ", 
    SubscriptBox[
     StyleBox["z", "TI"], 
     StyleBox["c", "TI"]], ",", 
    SubscriptBox[
     StyleBox["\[Theta]", "TR"], 
     StyleBox["c", "TI"]]}], "}"}]], "InlineFormula"],
 ", where ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 " is the list of values input to the function, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["z", "TI"], 
   StyleBox["t", "TI"]]], "InlineFormula"],
 " is the list of thickness values, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["\[Theta]", "TR"], 
   StyleBox["t", "TI"]]], "InlineFormula"],
 " is the list of angles that the thickness line makes with the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["z", "TI"], 
   StyleBox["c", "TI"]]], "InlineFormula"],
 " is the list of camber values and ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["\[Theta]", "TR"], 
   StyleBox["c", "TI"]]], "InlineFormula"],
 " is the list of angles that the camber line makes with the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis."
}], "Text",
 CellID->193521988],

Cell[TextData[{
 "The list of thickness and camber values are input to ",
 Cell[BoxData[
  ButtonBox["AirfoilSurfacePoints",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilSurfacePoints"]], 
  "InlineFormula"],
 " to compute a collection of vectors locating points on the airfoil surface. \
 Note that this function is not limited to the NACA four-digit airfoil, but \
could be used for any airfoil defined by a list of thickness and camber \
proprieties of the same form."
}], "Text",
 CellID->119120002],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "Text",
 CellID->278144368],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->2058623809],

Cell[CellGroupData[{

Cell["1.3.1 NACA 4412 airfoil", "Section",
 CellID->780127955],

Cell[TextData[{
 "The input list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values for the function NACA4DigitAirfoil controls the number and spacing \
of points computed on the airfoil surface.  In this example, a relatively \
small discretization number is used.  The ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["NDiscretizeUnitSegment",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/NDiscretizeUnitSegment"]], 
  "InlineFormula"],
 " is used as a convenience to generate the list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values.  The value HalfCosineSpacing for the option Layout concentrates \
points near the leading edge."
}], "Text",
 CellID->78760192],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"xList", "=", 
  RowBox[{"NDiscretizeUnitSegment", "[", 
   RowBox[{"4", ",", 
    RowBox[{"Layout", "\[Rule]", "HalfCosineSpacing"}]}], "]"}]}]], "Input",
 CellLabel->"In[2]:=",
 CellID->36949950],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.`", ",", "0.07612046748871326`", ",", "0.2928932188134524`", ",", 
   "0.6173165676349102`", ",", "0.9999999999999999`"}], "}"}]], "Output",
 ImageSize->{265, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->335560225]
}, Open  ]],

Cell[TextData[{
 "Use the list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values and the identification number for the airfoil to compute lists of \
thickness and camber properties."
}], "Text",
 CellID->158135470],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"properties", "=", 
  RowBox[{"NACA4DigitAirfoil", "[", 
   RowBox[{"4412", ",", "xList"}], "]"}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->136038124],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "0.`", ",", "0.07612046748871326`", ",", "0.2928932188134524`", ",", 
     "0.6173165676349102`", ",", "0.9999999999999999`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "0.`", ",", "0.04224482966144003`", ",", "0.060006295329734644`", ",", 
     "0.044186384497731004`", ",", "0.0012599999999999944`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "1.5707963267948966`", ",", "0.21461900569014944`", ",", 
     "0.0031819755314542327`", ",", 
     RowBox[{"-", "0.08478225618956567`"}], ",", 
     RowBox[{"-", "0.13939996933185558`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "0.`", ",", "0.013775512105067588`", ",", "0.037132034355964245`", ",", 
     "0.034752612159042386`", ",", "1.2335811384723961`*^-17"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "0.19739555984988075`", ",", "0.16054603550811242`", ",", 
     "0.05350228208284681`", ",", 
     RowBox[{"-", "0.04825508083145211`"}], ",", 
     RowBox[{"-", "0.132551532296674`"}]}], "}"}]}], "}"}]], "Output",
 ImageSize->{387, 90},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->477733320]
}, Open  ]],

Cell[TextData[{
 "Use the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["AirfoilSurfacePoints",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilSurfacePoints"]], 
  "InlineFormula"],
 " to compute a collection of vectors locating points on the surface of the \
airfoil from these lists of thickness and camber properties."
}], "Text",
 CellID->89603494],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rP", "=", 
  RowBox[{"AirfoilSurfacePoints", "[", "properties", "]"}]}]], "Input",
 CellLabel->"In[4]:=",
 CellID->175130362],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "0.9998334737126852`", ",", "0.6151851774798535`", ",", 
     "0.29610216111173066`", ",", "0.08287360953113432`", ",", "0.`", ",", 
     "0.0693673254462922`", ",", "0.2896842765151742`", ",", 
     "0.6194479577899669`", ",", "1.0001665262873145`"}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.0012489471548601018`"}], ",", 
     RowBox[{"-", "0.009382337155398324`"}], ",", 
     RowBox[{"-", "0.02278839762283289`"}], ",", 
     RowBox[{"-", "0.027926055079859263`"}], ",", "0.`", ",", 
     "0.055477079289994444`", ",", "0.09705246633476139`", ",", 
     "0.0788875614734831`", ",", "0.0012489471548601265`"}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{0.9998334737126852, 
    0.6151851774798535, 0.29610216111173066`, 0.08287360953113432, 0., 
    0.0693673254462922, 0.2896842765151742, 0.6194479577899669, 
    1.0001665262873145`}, {-0.0012489471548601018`, -0.009382337155398324, \
-0.02278839762283289, -0.027926055079859263`, 0., 0.055477079289994444`, 
    0.09705246633476139, 0.0788875614734831, 0.0012489471548601265`}], 
   "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{583, 50},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->2644115]
}, Open  ]],

Cell[TextData[{
 "To approximate the geometry of the airfoil, use a relatively large list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values, and note that the functions can be nested."
}], "Text",
 CellID->49184558],

Cell[BoxData[
 RowBox[{
  RowBox[{"r", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"NACA4DigitAirfoil", "[", 
     RowBox[{"4412", ",", 
      RowBox[{"NDiscretizeUnitSegment", "[", 
       RowBox[{"100", ",", 
        RowBox[{"Layout", "\[Rule]", "HalfCosineSpacing"}]}], "]"}]}], "]"}], 
    "]"}]}], ";"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->100398123],

Cell["\<\
Show the coarsely discretized airfoil in red and the airfoil shape \
approximated by a large discretization number in blue.\
\>", "Text",
 CellID->154464187],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"gr", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"r", ",", 
     RowBox[{"PlotStyle", "\[Rule]", "Blue"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"grP", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rP", ",", 
     RowBox[{"PlotStyle", "\[Rule]", " ", "Red"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gr", ",", "grP"}], "]"}]}], "Input",
 CellLabel->"In[6]:=",
 CellID->11328373],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3X9oJOd9x/Ftc72I1HXF4bpyMO6WGiOMSUVwjThEbumBT+DgqD+ggtKe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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->425432927]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1.3.2 NACA four-digit family of airfoils", "Section",
 CellID->257533491],

Cell["\<\
The effect of the three parameters making up the identification number for a \
NACA four-digit airfoil on the shape of the airfoil can be seen using the \
following dynamic example.  Use the sliders to control the three components \
of the identification number.\
\>", "Text",
 CellID->205773740],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Manipulate", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"idS", "=", 
     RowBox[{
      RowBox[{"ToString", "[", "c1", "]"}], "<>", 
      RowBox[{"ToString", "[", "c2", "]"}], "<>", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"c3", ">", "9"}], ",", 
        RowBox[{"ToString", "[", "c3", "]"}], ",", 
        RowBox[{"\"\<0\>\"", "<>", 
         RowBox[{"ToString", "[", "c3", "]"}]}]}], "]"}]}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"id", "=", 
     RowBox[{"ToExpression", "[", "idS", "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"rP", "=", 
     RowBox[{"AirfoilSurfacePoints", "[", 
      RowBox[{"NACA4DigitAirfoil", "[", 
       RowBox[{"id", ",", 
        RowBox[{"NDiscretizeUnitSegment", "[", 
         RowBox[{"100", ",", 
          RowBox[{"Layout", "\[Rule]", "HalfCosineSpacing"}]}], "]"}]}], 
       "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"PlotAirfoil", "[", 
     RowBox[{"rP", ",", 
      RowBox[{"ImageSize", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"7", " ", "72"}], "}"}]}], ",", 
      RowBox[{"PlotLabel", "\[Rule]", 
       RowBox[{"\"\<NACA \>\"", "<>", "idS", "<>", "\"\< Airfoil\>\""}]}], 
      ",", 
      RowBox[{"PlotRange", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "0.05"}], ",", "1.05"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "0.2"}], ",", "0.301"}], "}"}]}], "}"}]}]}], "]"}]}], 
   ",", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"c1", ",", "3", ",", 
       RowBox[{"Dynamic", "[", "c1", "]"}]}], "}"}], ",", "0", ",", "9", ",", 
     "1"}], "}"}], ",", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"c2", ",", "4", ",", 
       RowBox[{"Dynamic", "[", "c2", "]"}]}], "}"}], ",", "0", ",", "9", ",", 
     "1"}], "}"}], ",", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"c3", ",", "15", ",", 
       RowBox[{"Dynamic", "[", "c3", "]"}]}], "}"}], ",", "0", ",", "50", ",",
      "1"}], "}"}], ",", 
   RowBox[{"Initialization", "\[RuleDelayed]", 
    RowBox[{"Get", "[", "\"\<Aerodynamics`\>\"", "]"}]}], ",", 
   RowBox[{"SaveDefinitions", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellLabel->"In[9]:=",
 CellID->26946],

Cell[BoxData[
 TagBox[
  StyleBox[
   DynamicModuleBox[{$CellContext`c1$$ = 3, $CellContext`c2$$ = 
    4, $CellContext`c3$$ = 15, Typeset`show$$ = True, 
    Typeset`bookmarkList$$ = {}, Typeset`bookmarkMode$$ = "Menu", 
    Typeset`animator$$, Typeset`animvar$$ = 1, Typeset`name$$ = 
    "\"untitled\"", Typeset`specs$$ = {{{
       Hold[$CellContext`c1$$], 3, 
       Dynamic[$CellContext`c1$$]}, 0, 9, 1}, {{
       Hold[$CellContext`c2$$], 4, 
       Dynamic[$CellContext`c2$$]}, 0, 9, 1}, {{
       Hold[$CellContext`c3$$], 15, 
       Dynamic[$CellContext`c3$$]}, 0, 50, 1}}, Typeset`size$$ = {
    504., {127., 131.}}, Typeset`update$$ = 0, Typeset`initDone$$, 
    Typeset`skipInitDone$$ = False, $CellContext`c1$330$$ = 
    0, $CellContext`c2$335$$ = 0, $CellContext`c3$336$$ = 0}, 
    DynamicBox[Manipulate`ManipulateBoxes[
     1, StandardForm, 
      "Variables" :> {$CellContext`c1$$ = 3, $CellContext`c2$$ = 
        4, $CellContext`c3$$ = 15}, "ControllerVariables" :> {
        Hold[$CellContext`c1$$, $CellContext`c1$330$$, 0], 
        Hold[$CellContext`c2$$, $CellContext`c2$335$$, 0], 
        Hold[$CellContext`c3$$, $CellContext`c3$336$$, 0]}, 
      "OtherVariables" :> {
       Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
        Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
        Typeset`specs$$, Typeset`size$$, Typeset`update$$, Typeset`initDone$$,
         Typeset`skipInitDone$$}, "Body" :> ($CellContext`idS = StringJoin[
          ToString[$CellContext`c1$$], 
          ToString[$CellContext`c2$$], 
          If[$CellContext`c3$$ > 9, 
           ToString[$CellContext`c3$$], 
           StringJoin["0", 
            ToString[$CellContext`c3$$]]]]; $CellContext`id = 
        ToExpression[$CellContext`idS]; $CellContext`rP = 
        Aerodynamics`AirfoilGeometry`AirfoilSurfacePoints[
          Aerodynamics`AirfoilGeometry`NACA4DigitAirfoil[$CellContext`id, 
           Aerodynamics`AirfoilGeometry`NDiscretizeUnitSegment[
           100, Aerodynamics`AirfoilGeometry`Layout -> 
            Aerodynamics`AirfoilGeometry`HalfCosineSpacing]]]; 
       Aerodynamics`Graphics`PlotAirfoil[$CellContext`rP, ImageSize -> {7 72},
          PlotLabel -> StringJoin["NACA ", $CellContext`idS, " Airfoil"], 
         PlotRange -> {{-0.05, 1.05}, {-0.2, 0.301}}]), 
      "Specifications" :> {{{$CellContext`c1$$, 3, 
          Dynamic[$CellContext`c1$$]}, 0, 9, 1}, {{$CellContext`c2$$, 4, 
          Dynamic[$CellContext`c2$$]}, 0, 9, 1}, {{$CellContext`c3$$, 15, 
          Dynamic[$CellContext`c3$$]}, 0, 50, 1}}, "Options" :> {}, 
      "DefaultOptions" :> {}],
     ImageSizeCache->{553., {200., 205.}},
     SingleEvaluation->True],
    Deinitialization:>None,
    DynamicModuleValues:>{},
    Initialization:>({{$CellContext`rP = 
         Aerodynamics`CartesianVectors`CartesianVectors[{0.9998334737126852, 
           0.6151851774798535, 0.29610216111173066`, 0.08287360953113432, 0., 
           0.0693673254462922, 0.2896842765151742, 0.6194479577899669, 
           1.0001665262873145`}, {-0.0012489471548601018`, \
-0.009382337155398324, -0.02278839762283289, -0.027926055079859263`, 0., 
           0.055477079289994444`, 0.09705246633476139, 0.0788875614734831, 
           0.0012489471548601265`}]}; Get["Aerodynamics`"]}; 
     Typeset`initDone$$ = True),
    SynchronousInitialization->True,
    UnsavedVariables:>{Typeset`initDone$$},
    UntrackedVariables:>{Typeset`size$$}], "Manipulate",
   Deployed->True,
   StripOnInput->False],
  Manipulate`InterpretManipulate[1]]], "Output",
 ImageSize->{557, 407},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->218461760]
}, Open  ]]
}, Open  ]],

Cell["", "Section",
 CellID->794950467],

Cell[CellGroupData[{

Cell["1.3.3 Trailing-edge of NACA four-digit airfoils", "Section",
 CellID->140735501],

Cell[TextData[{
 "A characteristic of the NACA four-digit airfoils is that there is a small \
gap at the trailing edge.  This can cause problems for some panel methods.  \
Specify a discretized airfoil with 100 panels and cosine spacing for the \
layout to provide relatively fine discretization near the trailing edge.  \
Note the first and last vectors in the collection do not coincide, but that \
the mean of the two vectors is ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{"1", ",", "0"}], "}"}]], "InlineFormula"],
 " as expected for a normalized airfoil."
}], "Text",
 CellID->48076094],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"NACA4DigitAirfoil", "[", 
     RowBox[{"4412", ",", 
      RowBox[{"NDiscretizeUnitSegment", "[", 
       RowBox[{"50", ",", 
        RowBox[{"Layout", "\[Rule]", "CosineSpacing"}]}], "]"}]}], "]"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"First", "[", "rP", "]"}], ",", 
   RowBox[{"Last", "[", "rP", "]"}], ",", 
   RowBox[{"Chop", "[", 
    RowBox[{"Mean", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"First", "[", "rP", "]"}], ",", 
       RowBox[{"Last", "[", "rP", "]"}]}], "}"}], "]"}], "]"}]}], 
  "}"}]}], "Input",
 CellLabel->"In[10]:=",
 CellID->508498224],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0.9998334737126853`", ",", 
     RowBox[{"-", "0.001248947154860081`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.0001665262873147`", ",", "0.001248947154860081`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.`", ",", "0"}], "}"}]}], "}"}]], "Output",
 ImageSize->{389, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->101949095]
}, Open  ]],

Cell["\<\
By magnifying the region near the trailing edge, the small gap at the \
trailing edge is  noticeable.\
\>", "Text",
 CellID->440842507],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotAirfoil", "[", 
  RowBox[{"rP", ",", 
   RowBox[{"PlotJoined", "\[Rule]", "True"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0.75", ",", "1.02"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "0.01"}], ",", "0.06"}], "}"}]}], "}"}]}]}], 
  "]"}]], "Input",
 CellLabel->"In[12]:=",
 CellID->62089316],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3c1rVFkaB+AM85f0vyGIC3EhLpoeEBfiJkKY2ehGN7pJNrpRCLppQTqL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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->759264625]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["AirfoilGeometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/guide/AirfoilGeometry"]], "TutorialMoreAbout",
 CellID->1567025153],

Cell[TextData[ButtonBox["Aerodynamics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"]], "TutorialMoreAbout",
 CellID->325103336]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["1.2 Airfoils From Data Files",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromDataFiles"]], "RelatedTutorials",
 CellID->806871991],

Cell[TextData[ButtonBox["1.4 Airfoils From Conformal Mapping",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromConformalMapping"]], \
"RelatedTutorials",
 CellID->610993246],

Cell[TextData[ButtonBox["Overview",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]], "RelatedTutorials",
 CellID->554434694]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"1.3 Airfoils from Thickness And Camber Properties - Wolfram \
Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 3, 37.4316000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "None", "summary" -> 
    "During the 1930s and 1940s, the National Advisory Committee for \
Aeronautics (NACA) developed and tested families of airfoils based on \
systematic variations of thickness and camber. Algebraic equations were used \
to prescribe the thickness and camber of an airfoil, and the upper and lower \
surfaces of the airfoil were established by wrapping a thickness envelope \
around a camber line. See Section 4.3 of Anderson (1984) or Chapter 6 of \
Abbott and von Doenhoff (1959) for more details. The example used in \
Aerodynamics for airfoils specified by thickness and camber properties is the \
NACA four-digit family of airfoils. If the four-digit identification is \
written as id=ijkl, then i specifies maximum camber in percent chord, j \
specifies location of maximum camber in tenths chord and kl specifies maximum \
thickness in percent chord. The input arguments for the Aerodynamics function \
NACA4DigitAirfoil are the airfoil identification number and a list of \
x-values at which thickness and camber properties are to be computed. The \
output of the function is the nested list {x,z_t,\\[Theta]_t, \
z_c,\\[Theta]_c}, where x is the list of values input to the function, z_t is \
the list of thickness values, \\[Theta]_t is the list of angles that the \
thickness line makes with the x-axis, z_c is the list of camber values and \
\\[Theta]_c is the list of angles that the camber line makes with the \
x-axis.", "synonyms" -> {}, "title" -> 
    "1.3 Airfoils from Thickness And Camber Properties", "type" -> "Tutorial",
     "uri" -> 
    "Aerodynamics/tutorial/AirfoilsFromThicknessAndCamberProperties"}, 
  "LinkTrails" -> "", "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[644, 22, 78, 1, 70, "TutorialColorBar"],
Cell[725, 25, 191, 5, 70, "LinkTrail"],
Cell[919, 32, 3167, 71, 70, "AnchorBarGrid",
 CellID->1],
Cell[4089, 105, 70, 2, 70, "Input",
 CellID->376078495],
Cell[CellGroupData[{
Cell[4184, 111, 86, 1, 70, "Title",
 CellID->509267359],
Cell[4273, 114, 736, 16, 70, "Text",
 CellID->237225321],
Cell[5012, 132, 731, 22, 70, "Text",
 CellID->3719437],
Cell[5746, 156, 1840, 61, 70, "Text",
 CellID->193521988],
Cell[7589, 219, 527, 12, 70, "Text",
 CellID->119120002],
Cell[8119, 233, 125, 6, 70, "Text",
 CellID->278144368],
Cell[8247, 241, 101, 3, 70, "Input",
 CellID->2058623809],
Cell[CellGroupData[{
Cell[8373, 248, 62, 1, 70, "Section",
 CellID->780127955],
Cell[8438, 251, 774, 21, 70, "Text",
 CellID->78760192],
Cell[CellGroupData[{
Cell[9237, 276, 220, 6, 70, "Input",
 CellID->36949950],
Cell[9460, 284, 316, 9, 36, "Output",
 CellID->335560225]
}, Open  ]],
Cell[9791, 296, 237, 7, 70, "Text",
 CellID->158135470],
Cell[CellGroupData[{
Cell[10053, 307, 176, 5, 70, "Input",
 CellID->136038124],
Cell[10232, 314, 1199, 31, 111, "Output",
 CellID->477733320]
}, Open  ]],
Cell[11446, 348, 423, 13, 70, "Text",
 CellID->89603494],
Cell[CellGroupData[{
Cell[11894, 365, 149, 4, 70, "Input",
 CellID->175130362],
Cell[12046, 371, 1353, 30, 71, "Output",
 CellID->2644115]
}, Open  ]],
Cell[13414, 404, 238, 6, 70, "Text",
 CellID->49184558],
Cell[13655, 412, 377, 11, 70, "Input",
 CellID->100398123],
Cell[14035, 425, 167, 4, 70, "Text",
 CellID->154464187],
Cell[CellGroupData[{
Cell[14227, 433, 491, 16, 65, "Input",
 CellID->11328373],
Cell[14721, 451, 7092, 124, 201, "Output",
 CellID->425432927]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[21862, 581, 79, 1, 70, "Section",
 CellID->257533491],
Cell[21944, 584, 306, 6, 70, "Text",
 CellID->205773740],
Cell[CellGroupData[{
Cell[22275, 594, 2378, 65, 70, "Input",
 CellID->26946],
Cell[24656, 661, 3697, 72, 70, "Output",
 CellID->218461760]
}, Open  ]]
}, Open  ]],
Cell[28380, 737, 39, 1, 70, "Section",
 CellID->794950467],
Cell[CellGroupData[{
Cell[28444, 742, 86, 1, 70, "Section",
 CellID->140735501],
Cell[28533, 745, 592, 12, 70, "Text",
 CellID->48076094],
Cell[CellGroupData[{
Cell[29150, 761, 722, 22, 70, "Input",
 CellID->508498224],
Cell[29875, 785, 461, 14, 36, "Output",
 CellID->101949095]
}, Open  ]],
Cell[30351, 802, 145, 4, 70, "Text",
 CellID->440842507],
Cell[CellGroupData[{
Cell[30521, 810, 421, 14, 28, "Input",
 CellID->62089316],
Cell[30945, 826, 3961, 72, 201, "Output",
 CellID->759264625]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[34955, 904, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[35023, 907, 167, 4, 70, "TutorialMoreAbout",
 CellID->1567025153],
Cell[35193, 913, 157, 3, 70, "TutorialMoreAbout",
 CellID->325103336]
}, Open  ]],
Cell[CellGroupData[{
Cell[35387, 921, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[35462, 924, 187, 4, 70, "RelatedTutorials",
 CellID->806871991],
Cell[35652, 930, 203, 5, 70, "RelatedTutorials",
 CellID->610993246],
Cell[35858, 937, 166, 4, 70, "RelatedTutorials",
 CellID->554434694]
}, Open  ]]
}, Open  ]],
Cell[36051, 945, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

