(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     85754,       2620]
NotebookOptionsPosition[     75419,       2268]
NotebookOutlinePosition[     78090,       2328]
CellTagsIndexPosition[     78047,       2325]
WindowTitle->A1 Data Type CartesianVectors - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"References\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/References"], "\<\"A2 Graphics\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/Graphics"], "\<\"Overview\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"CartesianVectors\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/CartesianVectors"], "\<\"Aerodynamics\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/Aerodynamics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/tutorial/CartesianVectors\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/CartesianVectors"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/CartesianVectors.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/CartesianVectors", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["A1 Data Type CartesianVectors", "Title",
 CellID->509267359],

Cell[TextData[{
 "Collections of vectors play an important role in implementing panel \
methods.  Panel endpoints, collocation points, unit normal vectors to panels \
and velocity influence coefficients are commonly occurring collections of \
vectors.  Computation of influence coefficients involves vector algebra \
between collections of vectors representing panel endpoints and collocation \
points.  Defining a data type simplifies pattern matching and programming for \
the packages comprising the software ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  My original intent was to leave the data type \"hidden\" in the \
packages, but the need to use the constructor or selector for the data type \
occurs occasionally in the documentation.  Additionally, this introduction to \
the data type may be useful to those wanting to modify the packages or to \
those working on entirely different problems which involve the manipulation \
of large collections of vectors."
}], "Text",
 CellID->65021125],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "Text",
 CellID->280757486],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->107795920],

Cell[CellGroupData[{

Cell["A1.1 Constructor and Selector", "Section",
 CellID->152704095],

Cell[BoxData[GridBox[{
   {"MakeCartesianVectors", Cell[
    "constructs the data type CartesianVectors", "TableText"]},
   {"Components", Cell[
    "selects lists of components from CartesianVectors", "TableText"]}
  }]], "DefinitionBox",
 CellID->801716347],

Cell["Constructor and selector for the data type CartesianVectors.", "Caption",
 CellID->104944398],

Cell["\<\
Components of collections of vectors may be simple lists or matrices.  \
Vectors may be of any dimension.  Specify several collections of two- and \
three-dimensional vectors.  The heads of all these collections of vectors are \
CartesianVectors.\
\>", "Text",
 CellID->63591597],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"uS2D", "=", 
   RowBox[{"MakeCartesianVectors", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Array", "[", 
       RowBox[{"ux", ",", "5"}], "]"}], ",", 
      RowBox[{"Array", "[", 
       RowBox[{"uy", ",", "5"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vS2D", "=", 
   RowBox[{"MakeCartesianVectors", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Array", "[", 
       RowBox[{"vx", ",", "5"}], "]"}], ",", 
      RowBox[{"Array", "[", 
       RowBox[{"vy", ",", "5"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"uS3D", "=", 
   RowBox[{"MakeCartesianVectors", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Array", "[", 
       RowBox[{"ux", ",", "5"}], "]"}], ",", 
      RowBox[{"Array", "[", 
       RowBox[{"uy", ",", "5"}], "]"}], ",", 
      RowBox[{"Array", "[", 
       RowBox[{"uz", ",", "5"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vS3D", "=", 
   RowBox[{"MakeCartesianVectors", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Array", "[", 
       RowBox[{"vx", ",", "5"}], "]"}], ",", 
      RowBox[{"Array", "[", 
       RowBox[{"vy", ",", "5"}], "]"}], ",", 
      RowBox[{"Array", "[", 
       RowBox[{"vz", ",", "5"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"uM3D", "=", 
   RowBox[{"MakeCartesianVectors", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Array", "[", 
       RowBox[{"ux", ",", 
        RowBox[{"{", 
         RowBox[{"4", ",", "5"}], "}"}]}], "]"}], ",", 
      RowBox[{"Array", "[", 
       RowBox[{"uy", ",", 
        RowBox[{"{", 
         RowBox[{"4", ",", "5"}], "}"}]}], "]"}], ",", 
      RowBox[{"Array", "[", 
       RowBox[{"uz", ",", 
        RowBox[{"{", 
         RowBox[{"4", ",", "5"}], "}"}]}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Head", "/@", 
  RowBox[{"{", 
   RowBox[{"uS2D", ",", "vS2D", ",", "uS3D", ",", "vS3D", ",", "uM3D"}], 
   "}"}]}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->101936636],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "CartesianVectors", ",", "CartesianVectors", ",", "CartesianVectors", ",", 
   "CartesianVectors", ",", "CartesianVectors"}], "}"}]], "Output",
 ImageSize->{377, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->49642076]
}, Open  ]],

Cell["\<\
Display a couple of these collections of vectors, noting that an infix arrow \
is used to denote the data type CartesianVectors when displayed.\
\>", "Text",
 CellID->139029545],

Cell[CellGroupData[{

Cell["uS2D", "Input",
 CellLabel->"In[7]:=",
 CellID->203056159],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}], ",", 
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "1", "]"}], ",", 
     RowBox[{"uy", "[", "2", "]"}], ",", 
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}], ",", 
     RowBox[{"uy", "[", "5", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[1], 
     $CellContext`ux[2], 
     $CellContext`ux[3], 
     $CellContext`ux[4], 
     $CellContext`ux[5]}, {
     $CellContext`uy[1], 
     $CellContext`uy[2], 
     $CellContext`uy[3], 
     $CellContext`uy[4], 
     $CellContext`uy[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{479, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->452522401]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uM3D", "//", "MatrixForm"}]], "Input",
 CellLabel->"In[8]:=",
 CellID->186875196],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnbFS40oaRtnaJ9nX4B022SeYusHUZEvo7IYOISMkJKQ2GUJCh34Aggkd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    "], {{0, 177}, {333, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{333, 177},
  PlotRange->{{0, 333}, {0, 177}}]], "Output",
 ImageSize->{337, 177},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->228860420]
}, Open  ]],

Cell["\<\
The function Components selects the list of components from the collection of \
vectors of the data type CartesianVectors.\
\>", "Text",
 CellID->133331613],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Components", "[", "uS2D", "]"}]], "Input",
 CellLabel->"In[9]:=",
 CellID->228036416],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}], ",", 
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "1", "]"}], ",", 
     RowBox[{"uy", "[", "2", "]"}], ",", 
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}], ",", 
     RowBox[{"uy", "[", "5", "]"}]}], "}"}]}], "}"}]], "Output",
 ImageSize->{495, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->517559482]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A1.1 Vector algebra", "Section",
 CellID->98413141],

Cell[CellGroupData[{

Cell["Basic addition, subtraction and multiplication by a constant", \
"Subsection",
 CellID->578648157],

Cell[TextData[{
 "By overloading the ",
 StyleBox["Mathematica ",
  FontSlant->"Italic"],
 "functions, basic element-by-element arithmetic operations are defined for \
collections of vectors of the data type CartesianVectors.  Collections of \
vectors can be added, subtracted or multiplied by a constant."
}], "Text",
 CellID->110468743],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uS2D", "+", "vS2D"}]], "Input",
 CellLabel->"In[10]:=",
 CellID->337721988],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"ux", "[", "1", "]"}], "+", 
      RowBox[{"vx", "[", "1", "]"}]}], ",", 
     RowBox[{
      RowBox[{"ux", "[", "2", "]"}], "+", 
      RowBox[{"vx", "[", "2", "]"}]}], ",", 
     RowBox[{
      RowBox[{"ux", "[", "3", "]"}], "+", 
      RowBox[{"vx", "[", "3", "]"}]}], ",", 
     RowBox[{
      RowBox[{"ux", "[", "4", "]"}], "+", 
      RowBox[{"vx", "[", "4", "]"}]}], ",", 
     RowBox[{
      RowBox[{"ux", "[", "5", "]"}], "+", 
      RowBox[{"vx", "[", "5", "]"}]}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"uy", "[", "1", "]"}], "+", 
      RowBox[{"vy", "[", "1", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uy", "[", "2", "]"}], "+", 
      RowBox[{"vy", "[", "2", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uy", "[", "3", "]"}], "+", 
      RowBox[{"vy", "[", "3", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uy", "[", "4", "]"}], "+", 
      RowBox[{"vy", "[", "4", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uy", "[", "5", "]"}], "+", 
      RowBox[{"vy", "[", "5", "]"}]}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{$CellContext`ux[
      1] + $CellContext`vx[1], $CellContext`ux[2] + $CellContext`vx[
      2], $CellContext`ux[3] + $CellContext`vx[3], $CellContext`ux[
      4] + $CellContext`vx[4], $CellContext`ux[5] + $CellContext`vx[
      5]}, {$CellContext`uy[1] + $CellContext`vy[1], $CellContext`uy[
      2] + $CellContext`vy[2], $CellContext`uy[3] + $CellContext`vy[
      3], $CellContext`uy[4] + $CellContext`vy[4], $CellContext`uy[
      5] + $CellContext`vy[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{485, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[10]=",
 CellID->42784967]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uS3D", "-", "vS3D"}]], "Input",
 CellLabel->"In[11]:=",
 CellID->716287734],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"ux", "[", "1", "]"}], "-", 
      RowBox[{"vx", "[", "1", "]"}]}], ",", 
     RowBox[{
      RowBox[{"ux", "[", "2", "]"}], "-", 
      RowBox[{"vx", "[", "2", "]"}]}], ",", 
     RowBox[{
      RowBox[{"ux", "[", "3", "]"}], "-", 
      RowBox[{"vx", "[", "3", "]"}]}], ",", 
     RowBox[{
      RowBox[{"ux", "[", "4", "]"}], "-", 
      RowBox[{"vx", "[", "4", "]"}]}], ",", 
     RowBox[{
      RowBox[{"ux", "[", "5", "]"}], "-", 
      RowBox[{"vx", "[", "5", "]"}]}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"uy", "[", "1", "]"}], "-", 
      RowBox[{"vy", "[", "1", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uy", "[", "2", "]"}], "-", 
      RowBox[{"vy", "[", "2", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uy", "[", "3", "]"}], "-", 
      RowBox[{"vy", "[", "3", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uy", "[", "4", "]"}], "-", 
      RowBox[{"vy", "[", "4", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uy", "[", "5", "]"}], "-", 
      RowBox[{"vy", "[", "5", "]"}]}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"uz", "[", "1", "]"}], "-", 
      RowBox[{"vz", "[", "1", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uz", "[", "2", "]"}], "-", 
      RowBox[{"vz", "[", "2", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uz", "[", "3", "]"}], "-", 
      RowBox[{"vz", "[", "3", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uz", "[", "4", "]"}], "-", 
      RowBox[{"vz", "[", "4", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uz", "[", "5", "]"}], "-", 
      RowBox[{"vz", "[", "5", "]"}]}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{$CellContext`ux[
      1] - $CellContext`vx[1], $CellContext`ux[2] - $CellContext`vx[
     2], $CellContext`ux[3] - $CellContext`vx[
     3], $CellContext`ux[4] - $CellContext`vx[
     4], $CellContext`ux[5] - $CellContext`vx[
     5]}, {$CellContext`uy[1] - $CellContext`vy[
     1], $CellContext`uy[2] - $CellContext`vy[
     2], $CellContext`uy[3] - $CellContext`vy[
     3], $CellContext`uy[4] - $CellContext`vy[
     4], $CellContext`uy[5] - $CellContext`vy[
     5]}, {$CellContext`uz[1] - $CellContext`vz[
     1], $CellContext`uz[2] - $CellContext`vz[
     2], $CellContext`uz[3] - $CellContext`vz[
     3], $CellContext`uz[4] - $CellContext`vz[
     4], $CellContext`uz[5] - $CellContext`vz[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{493, 50},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->50318537]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a", "*", "uS2D"}]], "Input",
 CellLabel->"In[12]:=",
 CellID->67522338],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", " ", 
      RowBox[{"ux", "[", "1", "]"}]}], ",", 
     RowBox[{"a", " ", 
      RowBox[{"ux", "[", "2", "]"}]}], ",", 
     RowBox[{"a", " ", 
      RowBox[{"ux", "[", "3", "]"}]}], ",", 
     RowBox[{"a", " ", 
      RowBox[{"ux", "[", "4", "]"}]}], ",", 
     RowBox[{"a", " ", 
      RowBox[{"ux", "[", "5", "]"}]}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", " ", 
      RowBox[{"uy", "[", "1", "]"}]}], ",", 
     RowBox[{"a", " ", 
      RowBox[{"uy", "[", "2", "]"}]}], ",", 
     RowBox[{"a", " ", 
      RowBox[{"uy", "[", "3", "]"}]}], ",", 
     RowBox[{"a", " ", 
      RowBox[{"uy", "[", "4", "]"}]}], ",", 
     RowBox[{"a", " ", 
      RowBox[{"uy", "[", "5", "]"}]}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{$CellContext`a \
$CellContext`ux[1], $CellContext`a $CellContext`ux[
      2], $CellContext`a $CellContext`ux[3], $CellContext`a $CellContext`ux[
      4], $CellContext`a $CellContext`ux[5]}, {$CellContext`a $CellContext`uy[
      1], $CellContext`a $CellContext`uy[2], $CellContext`a $CellContext`uy[
      3], $CellContext`a $CellContext`uy[4], $CellContext`a $CellContext`uy[
      5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{295, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->685986808]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Multiplication by a list", "Subsection",
 CellID->87368199],

Cell[TextData[{
 "I was unable to define the multiplication of a collection of vectors by a \
list by overloading ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions, so an exported function MultiplyByList is defined to accomplish \
this task.  First, each component of a collection of vectors is scaled by a \
different factor.  A second definition provides multiplication of each vector \
in the collection be a different factor."
}], "Text",
 CellID->91427808],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MultiplyByList", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"sx", ",", "sy", ",", "sz"}], "}"}], ",", "uS3D"}], 
  "]"}]], "Input",
 CellLabel->"In[13]:=",
 CellID->87964621],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"sx", " ", 
      RowBox[{"ux", "[", "1", "]"}]}], ",", 
     RowBox[{"sx", " ", 
      RowBox[{"ux", "[", "2", "]"}]}], ",", 
     RowBox[{"sx", " ", 
      RowBox[{"ux", "[", "3", "]"}]}], ",", 
     RowBox[{"sx", " ", 
      RowBox[{"ux", "[", "4", "]"}]}], ",", 
     RowBox[{"sx", " ", 
      RowBox[{"ux", "[", "5", "]"}]}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"sy", " ", 
      RowBox[{"uy", "[", "1", "]"}]}], ",", 
     RowBox[{"sy", " ", 
      RowBox[{"uy", "[", "2", "]"}]}], ",", 
     RowBox[{"sy", " ", 
      RowBox[{"uy", "[", "3", "]"}]}], ",", 
     RowBox[{"sy", " ", 
      RowBox[{"uy", "[", "4", "]"}]}], ",", 
     RowBox[{"sy", " ", 
      RowBox[{"uy", "[", "5", "]"}]}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"sz", " ", 
      RowBox[{"uz", "[", "1", "]"}]}], ",", 
     RowBox[{"sz", " ", 
      RowBox[{"uz", "[", "2", "]"}]}], ",", 
     RowBox[{"sz", " ", 
      RowBox[{"uz", "[", "3", "]"}]}], ",", 
     RowBox[{"sz", " ", 
      RowBox[{"uz", "[", "4", "]"}]}], ",", 
     RowBox[{"sz", " ", 
      RowBox[{"uz", "[", "5", "]"}]}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{$CellContext`sx \
$CellContext`ux[1], $CellContext`sx $CellContext`ux[
      2], $CellContext`sx $CellContext`ux[3], $CellContext`sx $CellContext`ux[
      4], $CellContext`sx $CellContext`ux[
      5]}, {$CellContext`sy $CellContext`uy[
      1], $CellContext`sy $CellContext`uy[2], $CellContext`sy $CellContext`uy[
      3], $CellContext`sy $CellContext`uy[4], $CellContext`sy $CellContext`uy[
      5]}, {$CellContext`sz $CellContext`uz[
      1], $CellContext`sz $CellContext`uz[2], $CellContext`sz $CellContext`uz[
      3], $CellContext`sz $CellContext`uz[4], $CellContext`sz $CellContext`uz[
      5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{338, 50},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[13]=",
 CellID->215268296]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"MultiplyByList", "[", 
    RowBox[{
     RowBox[{"Array", "[", 
      RowBox[{"a", ",", 
       RowBox[{"{", 
        RowBox[{"4", ",", "5"}], "}"}]}], "]"}], ",", "uM3D"}], "]"}], ")"}], 
  "//", "MatrixForm"}]], "Input",
 CellLabel->"In[14]:=",
 CellID->100168776],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnb1yG7uWhV13nmRew+E8wjyB6gYuZ6NQ2QkViplChw5dN5FChQr1AB04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    "], {{0, 177}, {578, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{578, 177},
  PlotRange->{{0, 578}, {0, 177}}]], "Output",
 ImageSize->{582, 177},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[14]=",
 CellID->33820876]
}, Open  ]],

Cell["\<\
For most applications, the number of vectors in a collection is large, so a \
conflict between the two definitions should be rare.  If the number of \
vectors is equal to the dimensionality of the space, then multiplication of \
components by elements of the list takes precedence.\
\>", "Text",
 CellID->256903099]
}, Open  ]],

Cell[CellGroupData[{

Cell["Dot and cross products", "Subsection",
 CellID->130895041],

Cell["\<\
Dot and cross product are overloaded to provide element-by-element products \
for collections of vectors.\
\>", "Text",
 CellID->564988326],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uS2D", ".", "vS2D"}]], "Input",
 CellLabel->"In[15]:=",
 CellID->1023740835],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"ux", "[", "1", "]"}], " ", 
     RowBox[{"vx", "[", "1", "]"}]}], "+", 
    RowBox[{
     RowBox[{"uy", "[", "1", "]"}], " ", 
     RowBox[{"vy", "[", "1", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"ux", "[", "2", "]"}], " ", 
     RowBox[{"vx", "[", "2", "]"}]}], "+", 
    RowBox[{
     RowBox[{"uy", "[", "2", "]"}], " ", 
     RowBox[{"vy", "[", "2", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"ux", "[", "3", "]"}], " ", 
     RowBox[{"vx", "[", "3", "]"}]}], "+", 
    RowBox[{
     RowBox[{"uy", "[", "3", "]"}], " ", 
     RowBox[{"vy", "[", "3", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"ux", "[", "4", "]"}], " ", 
     RowBox[{"vx", "[", "4", "]"}]}], "+", 
    RowBox[{
     RowBox[{"uy", "[", "4", "]"}], " ", 
     RowBox[{"vy", "[", "4", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"ux", "[", "5", "]"}], " ", 
     RowBox[{"vx", "[", "5", "]"}]}], "+", 
    RowBox[{
     RowBox[{"uy", "[", "5", "]"}], " ", 
     RowBox[{"vy", "[", "5", "]"}]}]}]}], "}"}]], "Output",
 ImageSize->{518, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[15]=",
 CellID->408364341]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"wS3D", "=", 
  RowBox[{"uS3D", "\[Cross]", "vS3D"}]}]], "Input",
 CellLabel->"In[16]:=",
 CellID->285324683],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"uz", "[", "1", "]"}]}], " ", 
       RowBox[{"vy", "[", "1", "]"}]}], "+", 
      RowBox[{
       RowBox[{"uy", "[", "1", "]"}], " ", 
       RowBox[{"vz", "[", "1", "]"}]}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"uz", "[", "2", "]"}]}], " ", 
       RowBox[{"vy", "[", "2", "]"}]}], "+", 
      RowBox[{
       RowBox[{"uy", "[", "2", "]"}], " ", 
       RowBox[{"vz", "[", "2", "]"}]}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"uz", "[", "3", "]"}]}], " ", 
       RowBox[{"vy", "[", "3", "]"}]}], "+", 
      RowBox[{
       RowBox[{"uy", "[", "3", "]"}], " ", 
       RowBox[{"vz", "[", "3", "]"}]}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"uz", "[", "4", "]"}]}], " ", 
       RowBox[{"vy", "[", "4", "]"}]}], "+", 
      RowBox[{
       RowBox[{"uy", "[", "4", "]"}], " ", 
       RowBox[{"vz", "[", "4", "]"}]}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"uz", "[", "5", "]"}]}], " ", 
       RowBox[{"vy", "[", "5", "]"}]}], "+", 
      RowBox[{
       RowBox[{"uy", "[", "5", "]"}], " ", 
       RowBox[{"vz", "[", "5", "]"}]}]}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"uz", "[", "1", "]"}], " ", 
       RowBox[{"vx", "[", "1", "]"}]}], "-", 
      RowBox[{
       RowBox[{"ux", "[", "1", "]"}], " ", 
       RowBox[{"vz", "[", "1", "]"}]}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"uz", "[", "2", "]"}], " ", 
       RowBox[{"vx", "[", "2", "]"}]}], "-", 
      RowBox[{
       RowBox[{"ux", "[", "2", "]"}], " ", 
       RowBox[{"vz", "[", "2", "]"}]}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"uz", "[", "3", "]"}], " ", 
       RowBox[{"vx", "[", "3", "]"}]}], "-", 
      RowBox[{
       RowBox[{"ux", "[", "3", "]"}], " ", 
       RowBox[{"vz", "[", "3", "]"}]}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"uz", "[", "4", "]"}], " ", 
       RowBox[{"vx", "[", "4", "]"}]}], "-", 
      RowBox[{
       RowBox[{"ux", "[", "4", "]"}], " ", 
       RowBox[{"vz", "[", "4", "]"}]}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"uz", "[", "5", "]"}], " ", 
       RowBox[{"vx", "[", "5", "]"}]}], "-", 
      RowBox[{
       RowBox[{"ux", "[", "5", "]"}], " ", 
       RowBox[{"vz", "[", "5", "]"}]}]}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"uy", "[", "1", "]"}]}], " ", 
       RowBox[{"vx", "[", "1", "]"}]}], "+", 
      RowBox[{
       RowBox[{"ux", "[", "1", "]"}], " ", 
       RowBox[{"vy", "[", "1", "]"}]}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"uy", "[", "2", "]"}]}], " ", 
       RowBox[{"vx", "[", "2", "]"}]}], "+", 
      RowBox[{
       RowBox[{"ux", "[", "2", "]"}], " ", 
       RowBox[{"vy", "[", "2", "]"}]}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"uy", "[", "3", "]"}]}], " ", 
       RowBox[{"vx", "[", "3", "]"}]}], "+", 
      RowBox[{
       RowBox[{"ux", "[", "3", "]"}], " ", 
       RowBox[{"vy", "[", "3", "]"}]}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"uy", "[", "4", "]"}]}], " ", 
       RowBox[{"vx", "[", "4", "]"}]}], "+", 
      RowBox[{
       RowBox[{"ux", "[", "4", "]"}], " ", 
       RowBox[{"vy", "[", "4", "]"}]}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"uy", "[", "5", "]"}]}], " ", 
       RowBox[{"vx", "[", "5", "]"}]}], "+", 
      RowBox[{
       RowBox[{"ux", "[", "5", "]"}], " ", 
       RowBox[{"vy", "[", "5", "]"}]}]}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{-$CellContext`uz[
       1] $CellContext`vy[1] + $CellContext`uy[1] $CellContext`vz[
       1], -$CellContext`uz[2] $CellContext`vy[2] + $CellContext`uy[
       2] $CellContext`vz[2], -$CellContext`uz[3] $CellContext`vy[
       3] + $CellContext`uy[3] $CellContext`vz[3], -$CellContext`uz[
       4] $CellContext`vy[4] + $CellContext`uy[4] $CellContext`vz[
       4], -$CellContext`uz[5] $CellContext`vy[5] + $CellContext`uy[
       5] $CellContext`vz[5]}, {$CellContext`uz[1] $CellContext`vx[
       1] - $CellContext`ux[1] $CellContext`vz[
      1], $CellContext`uz[2] $CellContext`vx[2] - $CellContext`ux[
      2] $CellContext`vz[
      2], $CellContext`uz[3] $CellContext`vx[3] - $CellContext`ux[
      3] $CellContext`vz[
      3], $CellContext`uz[4] $CellContext`vx[4] - $CellContext`ux[
      4] $CellContext`vz[
      4], $CellContext`uz[5] $CellContext`vx[5] - $CellContext`ux[
      5] $CellContext`vz[
      5]}, {-$CellContext`uy[1] $CellContext`vx[1] + $CellContext`ux[
       1] $CellContext`vy[1], -$CellContext`uy[2] $CellContext`vx[
       2] + $CellContext`ux[2] $CellContext`vy[2], -$CellContext`uy[
       3] $CellContext`vx[3] + $CellContext`ux[3] $CellContext`vy[
       3], -$CellContext`uy[4] $CellContext`vx[4] + $CellContext`ux[
       4] $CellContext`vy[4], -$CellContext`uy[5] $CellContext`vx[
       5] + $CellContext`ux[5] $CellContext`vy[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{557, 101},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[16]=",
 CellID->432006061]
}, Open  ]],

Cell["\<\
The cross product is perpendicular to the original vectors element-by-element \
in the collections, and there were five vectors each collection.\
\>", "Text",
 CellID->378189741],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "/@", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"wS3D", ".", "uS3D"}], ",", 
    RowBox[{"wS3D", ".", "vS3D"}]}], "}"}]}]], "Input",
 CellLabel->"In[17]:=",
 CellID->110286018],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}]}], 
  "}"}]], "Output",
 ImageSize->{215, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[17]=",
 CellID->413161789]
}, Open  ]],

Cell["\<\
Generalization of the cross product  is convenient for computing \
element-by-element normal vectors for a collection of two-dimensional vectors.\
\>", "Text",
 CellID->113463184],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"wS2D", "=", 
  RowBox[{"Cross", "[", "uS2D", "]"}]}]], "Input",
 CellLabel->"In[18]:=",
 CellID->221821086],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"uy", "[", "1", "]"}]}], ",", 
     RowBox[{"-", 
      RowBox[{"uy", "[", "2", "]"}]}], ",", 
     RowBox[{"-", 
      RowBox[{"uy", "[", "3", "]"}]}], ",", 
     RowBox[{"-", 
      RowBox[{"uy", "[", "4", "]"}]}], ",", 
     RowBox[{"-", 
      RowBox[{"uy", "[", "5", "]"}]}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}], ",", 
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{-$CellContext`uy[
      1], -$CellContext`uy[2], -$CellContext`uy[3], -$CellContext`uy[
      4], -$CellContext`uy[5]}, {
     $CellContext`ux[1], 
     $CellContext`ux[2], 
     $CellContext`ux[3], 
     $CellContext`ux[4], 
     $CellContext`ux[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{519, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[18]=",
 CellID->776302981]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"wS2D", ".", "uS2D"}]], "Input",
 CellLabel->"In[19]:=",
 CellID->317318721],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}]], "Output",
 ImageSize->{97, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[19]=",
 CellID->118480808]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Outer", "Subsection",
 CellID->337431051],

Cell["\<\
In the following example, two collections of vectors locate red and blue \
points.  Outer is used to compute displacement vectors from one collection of \
points to another, and yields a collections of vectors whose components are \
matrices.\
\>", "Text",
 CellID->18165939],

Cell[TextData[{
 "Define two collections of vectors, where ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["a", "TI"]]], "InlineFormula"],
 " locates the red points and ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["b", "TI"]]], "InlineFormula"],
 " locates the blue points in the graphic below.  Use Outer to compute \
displacement vectors locating every blue point with respect to each red point."
}], "Text",
 CellID->435648623],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rA", "=", 
   RowBox[{"MakeCartesianVectors", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0.", ",", "0.", ",", "0."}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0.", ",", "1.", ",", "2."}], "}"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rB", "=", 
   RowBox[{"MakeCartesianVectors", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1.", ",", "1.", ",", "1."}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0.", ",", "1.", ",", "2."}], "}"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"(", 
   RowBox[{"rBA", "=", 
    RowBox[{"Outer", "[", 
     RowBox[{"Plus", ",", "rB", ",", 
      RowBox[{"-", "rA"}]}], "]"}]}], ")"}], "//", "MatrixForm"}]}], "Input",
 CellLabel->"In[20]:=",
 CellID->256028157],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1.`", "1.`", "1.`"},
       {"1.`", "1.`", "1.`"},
       {"1.`", "1.`", "1.`"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]], "\[UpArrow]", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"0.`", 
        RowBox[{"-", "1.`"}], 
        RowBox[{"-", "2.`"}]},
       {"1.`", "0.`", 
        RowBox[{"-", "1.`"}]},
       {"2.`", "1.`", "0.`"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[
    MatrixForm[{{1., 1., 1.}, {1., 1., 1.}, {1., 1., 1.}}], 
    MatrixForm[{{0., -1., -2.}, {1., 0., -1.}, {2., 1., 0.}}]], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{185, 42},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[22]=",
 CellID->403421165]
}, Open  ]],

Cell[TextData[{
 "Use ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["r", "TI"], 
    StyleBox["b", "TI"]], "=", 
   RowBox[{
    SubscriptBox[
     StyleBox["r", "TI"], 
     StyleBox["a", "TI"]], "+", 
    SubscriptBox[
     StyleBox["r", "TI"], 
     StyleBox["ba", "TI"]]}]}]], "InlineFormula"],
 " to locate the blue points, noting that arrows are drawn to all blue points \
from each red point."
}], "Text",
 CellID->732586039],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"gA", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{"rA", ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Red", ",", 
        RowBox[{"PointSize", "[", "Large", "]"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gB", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{"rB", ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Blue", ",", 
        RowBox[{"PointSize", "[", "Large", "]"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gArrows", "=", 
   RowBox[{"Graphics", "[", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Arrow", "[", 
        RowBox[{"rA", ",", 
         RowBox[{"rA", "+", 
          RowBox[{"rBA", "[", 
           RowBox[{"[", "#", "]"}], "]"}]}]}], "]"}], "&"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gA", ",", "gB", ",", "gArrows", ",", 
   RowBox[{"Axes", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellLabel->"In[23]:=",
 CellID->275335666],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], PointSize[Large], 
    PointBox[{{0., 0.}, {0., 1.}, {0., 2.}}]}, 
   {RGBColor[0, 0, 1], PointSize[Large], 
    PointBox[{{1., 0.}, {1., 1.}, {1., 2.}}]}, {{
     ArrowBox[{{0., 0.}, {1., 0.}}], ArrowBox[{{0., 1.}, {1., 0.}}], 
     ArrowBox[{{0., 2.}, {1., 0.}}]}, {ArrowBox[{{0., 0.}, {1., 1.}}], 
     ArrowBox[{{0., 1.}, {1., 1.}}], ArrowBox[{{0., 2.}, {1., 1.}}]}, {
     ArrowBox[{{0., 0.}, {1., 2.}}], ArrowBox[{{0., 1.}, {1., 2.}}], 
     ArrowBox[{{0., 2.}, {1., 2.}}]}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->False,
  Frame->False,
  PlotRange->Automatic,
  PlotRangeClipping->True]], "Output",
 ImageSize->{184, 112},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[26]=",
 CellID->85635655]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A1.2 List Operations", "Section",
 CellID->23446292],

Cell[TextData[{
 "Since the data type CartesianVectors is constructed from a list of a \
particular structure, the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions for manipulating lists can be overloaded for CartesianVectors.  \
This has been done as needed for the application packages, so not all list \
operations have been overloaded, and those that have been, may not be as \
general as the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " definitions.  The following examples illustrate how overloaded ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions are used for list-type operations."
}], "Text",
 CellID->4393084],

Cell[CellGroupData[{

Cell["Get pieces from a collection of vectors.", "Subsection",
 CellID->69497392],

Cell[TextData[{
 "Note that single vectors are represented in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation."
}], "Text",
 CellID->83473493],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"First", "[", "uS3D", "]"}]], "Input",
 CellLabel->"In[27]:=",
 CellID->191503734],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"ux", "[", "1", "]"}], ",", 
   RowBox[{"uy", "[", "1", "]"}], ",", 
   RowBox[{"uz", "[", "1", "]"}]}], "}"}]], "Output",
 ImageSize->{145, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[27]=",
 CellID->167751407]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uS3D", "[", 
  RowBox[{"[", "3", "]"}], "]"}]], "Input",
 CellLabel->"In[28]:=",
 CellID->703666199],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"ux", "[", "3", "]"}], ",", 
   RowBox[{"uy", "[", "3", "]"}], ",", 
   RowBox[{"uz", "[", "3", "]"}]}], "}"}]], "Output",
 ImageSize->{145, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[28]=",
 CellID->324481438]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"ux", "[", "3", "]"}], ",", 
   RowBox[{"uy", "[", "3", "]"}], ",", 
   RowBox[{"uz", "[", "3", "]"}]}], "}"}]], "Input",
 CellLabel->"In[29]:=",
 CellID->6136457],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"ux", "[", "3", "]"}], ",", 
   RowBox[{"uy", "[", "3", "]"}], ",", 
   RowBox[{"uz", "[", "3", "]"}]}], "}"}]], "Output",
 ImageSize->{145, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[29]=",
 CellID->118209414]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Rest", "[", "uS3D", "]"}]], "Input",
 CellLabel->"In[30]:=",
 CellID->87307636],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "2", "]"}], ",", 
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "2", "]"}], ",", 
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}], ",", 
     RowBox[{"uy", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uz", "[", "2", "]"}], ",", 
     RowBox[{"uz", "[", "3", "]"}], ",", 
     RowBox[{"uz", "[", "4", "]"}], ",", 
     RowBox[{"uz", "[", "5", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[2], 
     $CellContext`ux[3], 
     $CellContext`ux[4], 
     $CellContext`ux[5]}, {
     $CellContext`uy[2], 
     $CellContext`uy[3], 
     $CellContext`uy[4], 
     $CellContext`uy[5]}, {
     $CellContext`uz[2], 
     $CellContext`uz[3], 
     $CellContext`uz[4], 
     $CellContext`uz[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{395, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[30]=",
 CellID->503396216]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uM3D", "[", 
  RowBox[{"[", "3", "]"}], "]"}]], "Input",
 CellLabel->"In[31]:=",
 CellID->442110006],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", 
      RowBox[{"3", ",", "1"}], "]"}], ",", 
     RowBox[{"ux", "[", 
      RowBox[{"3", ",", "2"}], "]"}], ",", 
     RowBox[{"ux", "[", 
      RowBox[{"3", ",", "3"}], "]"}], ",", 
     RowBox[{"ux", "[", 
      RowBox[{"3", ",", "4"}], "]"}], ",", 
     RowBox[{"ux", "[", 
      RowBox[{"3", ",", "5"}], "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", 
      RowBox[{"3", ",", "1"}], "]"}], ",", 
     RowBox[{"uy", "[", 
      RowBox[{"3", ",", "2"}], "]"}], ",", 
     RowBox[{"uy", "[", 
      RowBox[{"3", ",", "3"}], "]"}], ",", 
     RowBox[{"uy", "[", 
      RowBox[{"3", ",", "4"}], "]"}], ",", 
     RowBox[{"uy", "[", 
      RowBox[{"3", ",", "5"}], "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uz", "[", 
      RowBox[{"3", ",", "1"}], "]"}], ",", 
     RowBox[{"uz", "[", 
      RowBox[{"3", ",", "2"}], "]"}], ",", 
     RowBox[{"uz", "[", 
      RowBox[{"3", ",", "3"}], "]"}], ",", 
     RowBox[{"uz", "[", 
      RowBox[{"3", ",", "4"}], "]"}], ",", 
     RowBox[{"uz", "[", 
      RowBox[{"3", ",", "5"}], "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[3, 1], 
     $CellContext`ux[3, 2], 
     $CellContext`ux[3, 3], 
     $CellContext`ux[3, 4], 
     $CellContext`ux[3, 5]}, {
     $CellContext`uy[3, 1], 
     $CellContext`uy[3, 2], 
     $CellContext`uy[3, 3], 
     $CellContext`uy[3, 4], 
     $CellContext`uy[3, 5]}, {
     $CellContext`uz[3, 1], 
     $CellContext`uz[3, 2], 
     $CellContext`uz[3, 3], 
     $CellContext`uz[3, 4], 
     $CellContext`uz[3, 5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{343, 50},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[31]=",
 CellID->133434270]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uM3D", "[", 
  RowBox[{"[", 
   RowBox[{"3", ",", "2"}], "]"}], "]"}]], "Input",
 CellLabel->"In[32]:=",
 CellID->59296511],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"ux", "[", 
    RowBox[{"3", ",", "2"}], "]"}], ",", 
   RowBox[{"uy", "[", 
    RowBox[{"3", ",", "2"}], "]"}], ",", 
   RowBox[{"uz", "[", 
    RowBox[{"3", ",", "2"}], "]"}]}], "}"}]], "Output",
 ImageSize->{199, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[32]=",
 CellID->218334605]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Take", "[", 
  RowBox[{"uS2D", ",", "2"}], "]"}]], "Input",
 CellLabel->"In[33]:=",
 CellID->542072997],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "1", "]"}], ",", 
     RowBox[{"uy", "[", "2", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[1], 
     $CellContext`ux[2]}, {
     $CellContext`uy[1], 
     $CellContext`uy[2]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{203, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[33]=",
 CellID->160345444]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Drop", "[", 
  RowBox[{"uS2D", ",", "2"}], "]"}]], "Input",
 CellLabel->"In[34]:=",
 CellID->15858325],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}], ",", 
     RowBox[{"uy", "[", "5", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[3], 
     $CellContext`ux[4], 
     $CellContext`ux[5]}, {
     $CellContext`uy[3], 
     $CellContext`uy[4], 
     $CellContext`uy[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{295, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[34]=",
 CellID->798170530]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Join two collections of vectors.", "Subsection",
 CellID->292283586],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Join", "[", 
  RowBox[{"uS3D", ",", "vS3D"}], "]"}]], "Input",
 CellLabel->"In[35]:=",
 CellID->94031565],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}], ",", 
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}], ",", 
     RowBox[{"vx", "[", "1", "]"}], ",", 
     RowBox[{"vx", "[", "2", "]"}], ",", 
     RowBox[{"vx", "[", "3", "]"}], ",", 
     RowBox[{"vx", "[", "4", "]"}], ",", 
     RowBox[{"vx", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "1", "]"}], ",", 
     RowBox[{"uy", "[", "2", "]"}], ",", 
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}], ",", 
     RowBox[{"uy", "[", "5", "]"}], ",", 
     RowBox[{"vy", "[", "1", "]"}], ",", 
     RowBox[{"vy", "[", "2", "]"}], ",", 
     RowBox[{"vy", "[", "3", "]"}], ",", 
     RowBox[{"vy", "[", "4", "]"}], ",", 
     RowBox[{"vy", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uz", "[", "1", "]"}], ",", 
     RowBox[{"uz", "[", "2", "]"}], ",", 
     RowBox[{"uz", "[", "3", "]"}], ",", 
     RowBox[{"uz", "[", "4", "]"}], ",", 
     RowBox[{"uz", "[", "5", "]"}], ",", 
     RowBox[{"vz", "[", "1", "]"}], ",", 
     RowBox[{"vz", "[", "2", "]"}], ",", 
     RowBox[{"vz", "[", "3", "]"}], ",", 
     RowBox[{"vz", "[", "4", "]"}], ",", 
     RowBox[{"vz", "[", "5", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[1], 
     $CellContext`ux[2], 
     $CellContext`ux[3], 
     $CellContext`ux[4], 
     $CellContext`ux[5], 
     $CellContext`vx[1], 
     $CellContext`vx[2], 
     $CellContext`vx[3], 
     $CellContext`vx[4], 
     $CellContext`vx[5]}, {
     $CellContext`uy[1], 
     $CellContext`uy[2], 
     $CellContext`uy[3], 
     $CellContext`uy[4], 
     $CellContext`uy[5], 
     $CellContext`vy[1], 
     $CellContext`vy[2], 
     $CellContext`vy[3], 
     $CellContext`vy[4], 
     $CellContext`vy[5]}, {
     $CellContext`uz[1], 
     $CellContext`uz[2], 
     $CellContext`uz[3], 
     $CellContext`uz[4], 
     $CellContext`uz[5], 
     $CellContext`vz[1], 
     $CellContext`vz[2], 
     $CellContext`vz[3], 
     $CellContext`vz[4], 
     $CellContext`vz[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{483, 50},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[35]=",
 CellID->176309563]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Prepend, append or insert a vector to a collection.", "Subsection",
 CellID->234066801],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Prepend", "[", 
  RowBox[{"uS3D", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[36]:=",
 CellID->59182926],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}], ",", 
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"uy", "[", "1", "]"}], ",", 
     RowBox[{"uy", "[", "2", "]"}], ",", 
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}], ",", 
     RowBox[{"uy", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{"z", ",", 
     RowBox[{"uz", "[", "1", "]"}], ",", 
     RowBox[{"uz", "[", "2", "]"}], ",", 
     RowBox[{"uz", "[", "3", "]"}], ",", 
     RowBox[{"uz", "[", "4", "]"}], ",", 
     RowBox[{"uz", "[", "5", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{$CellContext`x, 
     $CellContext`ux[1], 
     $CellContext`ux[2], 
     $CellContext`ux[3], 
     $CellContext`ux[4], 
     $CellContext`ux[5]}, {$CellContext`y, 
     $CellContext`uy[1], 
     $CellContext`uy[2], 
     $CellContext`uy[3], 
     $CellContext`uy[4], 
     $CellContext`uy[5]}, {$CellContext`z, 
     $CellContext`uz[1], 
     $CellContext`uz[2], 
     $CellContext`uz[3], 
     $CellContext`uz[4], 
     $CellContext`uz[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{523, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[36]=",
 CellID->41966192]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Append", "[", 
  RowBox[{"uS3D", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[37]:=",
 CellID->473326778],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}], ",", 
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}], ",", "x"}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "1", "]"}], ",", 
     RowBox[{"uy", "[", "2", "]"}], ",", 
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}], ",", 
     RowBox[{"uy", "[", "5", "]"}], ",", "y"}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uz", "[", "1", "]"}], ",", 
     RowBox[{"uz", "[", "2", "]"}], ",", 
     RowBox[{"uz", "[", "3", "]"}], ",", 
     RowBox[{"uz", "[", "4", "]"}], ",", 
     RowBox[{"uz", "[", "5", "]"}], ",", "z"}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[1], 
     $CellContext`ux[2], 
     $CellContext`ux[3], 
     $CellContext`ux[4], 
     $CellContext`ux[5], $CellContext`x}, {
     $CellContext`uy[1], 
     $CellContext`uy[2], 
     $CellContext`uy[3], 
     $CellContext`uy[4], 
     $CellContext`uy[5], $CellContext`y}, {
     $CellContext`uz[1], 
     $CellContext`uz[2], 
     $CellContext`uz[3], 
     $CellContext`uz[4], 
     $CellContext`uz[5], $CellContext`z}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{523, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[37]=",
 CellID->240180580]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Insert", "[", 
  RowBox[{"uS3D", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}], ",", "3"}], "]"}]], "Input",
 CellLabel->"In[38]:=",
 CellID->209839352],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}], ",", "x", ",", 
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "1", "]"}], ",", 
     RowBox[{"uy", "[", "2", "]"}], ",", "y", ",", 
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}], ",", 
     RowBox[{"uy", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uz", "[", "1", "]"}], ",", 
     RowBox[{"uz", "[", "2", "]"}], ",", "z", ",", 
     RowBox[{"uz", "[", "3", "]"}], ",", 
     RowBox[{"uz", "[", "4", "]"}], ",", 
     RowBox[{"uz", "[", "5", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[1], 
     $CellContext`ux[2], $CellContext`x, 
     $CellContext`ux[3], 
     $CellContext`ux[4], 
     $CellContext`ux[5]}, {
     $CellContext`uy[1], 
     $CellContext`uy[2], $CellContext`y, 
     $CellContext`uy[3], 
     $CellContext`uy[4], 
     $CellContext`uy[5]}, {
     $CellContext`uz[1], 
     $CellContext`uz[2], $CellContext`z, 
     $CellContext`uz[3], 
     $CellContext`uz[4], 
     $CellContext`uz[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{523, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[38]=",
 CellID->476777024]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Rotate position right or left.", "Subsection",
 CellID->70600885],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RotateRight", "[", "uS3D", "]"}]], "Input",
 CellLabel->"In[39]:=",
 CellID->257261906],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "5", "]"}], ",", 
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}], ",", 
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "5", "]"}], ",", 
     RowBox[{"uy", "[", "1", "]"}], ",", 
     RowBox[{"uy", "[", "2", "]"}], ",", 
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uz", "[", "5", "]"}], ",", 
     RowBox[{"uz", "[", "1", "]"}], ",", 
     RowBox[{"uz", "[", "2", "]"}], ",", 
     RowBox[{"uz", "[", "3", "]"}], ",", 
     RowBox[{"uz", "[", "4", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[5], 
     $CellContext`ux[1], 
     $CellContext`ux[2], 
     $CellContext`ux[3], 
     $CellContext`ux[4]}, {
     $CellContext`uy[5], 
     $CellContext`uy[1], 
     $CellContext`uy[2], 
     $CellContext`uy[3], 
     $CellContext`uy[4]}, {
     $CellContext`uz[5], 
     $CellContext`uz[1], 
     $CellContext`uz[2], 
     $CellContext`uz[3], 
     $CellContext`uz[4]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{487, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[39]=",
 CellID->471881325]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RotateLeft", "[", 
  RowBox[{"uS3D", ",", "2"}], "]"}]], "Input",
 CellLabel->"In[40]:=",
 CellID->59964444],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}], ",", 
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}], ",", 
     RowBox[{"uy", "[", "5", "]"}], ",", 
     RowBox[{"uy", "[", "1", "]"}], ",", 
     RowBox[{"uy", "[", "2", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uz", "[", "3", "]"}], ",", 
     RowBox[{"uz", "[", "4", "]"}], ",", 
     RowBox[{"uz", "[", "5", "]"}], ",", 
     RowBox[{"uz", "[", "1", "]"}], ",", 
     RowBox[{"uz", "[", "2", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[3], 
     $CellContext`ux[4], 
     $CellContext`ux[5], 
     $CellContext`ux[1], 
     $CellContext`ux[2]}, {
     $CellContext`uy[3], 
     $CellContext`uy[4], 
     $CellContext`uy[5], 
     $CellContext`uy[1], 
     $CellContext`uy[2]}, {
     $CellContext`uz[3], 
     $CellContext`uz[4], 
     $CellContext`uz[5], 
     $CellContext`uz[1], 
     $CellContext`uz[2]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{487, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[40]=",
 CellID->366786247]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A1.3 Summary", "Section",
 CellID->23564291],

Cell["\<\
The discussion and examples in this section provide an introduction to \
working with the data type CartesianVectors.  Refer to CartesianVectors.nb \
for details of implementation.\
\>", "Text",
 CellID->379619690]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["CartesianVectors",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/guide/CartesianVectors"]], "TutorialMoreAbout",
 CellID->1567025153],

Cell[TextData[ButtonBox["Aerodynamics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"]], "TutorialMoreAbout",
 CellID->829829444]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["References",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/References"]], "RelatedTutorials",
 CellID->383413863],

Cell[TextData[ButtonBox["A2 Graphics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/Graphics"]], "RelatedTutorials",
 CellID->806871991],

Cell[TextData[ButtonBox["Overview",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]], "RelatedTutorials",
 CellID->352542221]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"A1 Data Type CartesianVectors - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 3, 41.8308000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "None", "summary" -> 
    "Collections of vectors play an important role in implementing panel \
methods. Panel endpoints, collocation points, unit normal vectors to panels \
and velocity influence coefficients are commonly occurring collections of \
vectors. Computation of influence coefficients involves vector algebra \
between collections of vectors representing panel endpoints and collocation \
points. Defining a data type simplifies pattern matching and programming for \
the packages comprising the software Aerodynamics. My original intent was to \
leave the data type \"hidden\" in the packages, but the need to use the \
constructor or selector for the data type occurs occasionally in the \
documentation. Additionally, this introduction to the data type may be useful \
to those wanting to modify the packages or to those working on entirely \
different problems which involve the manipulation of large collections of \
vectors. Load the Aerodynamics packages. Constructor and selector for the \
data type CartesianVectors.", "synonyms" -> {}, "title" -> 
    "A1 Data Type CartesianVectors", "type" -> "Tutorial", "uri" -> 
    "Aerodynamics/tutorial/CartesianVectors"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[622, 21, 78, 1, 70, "TutorialColorBar"],
Cell[703, 24, 191, 5, 70, "LinkTrail"],
Cell[897, 31, 2986, 67, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3908, 102, 66, 1, 70, "Title",
 CellID->509267359],
Cell[3977, 105, 1012, 17, 70, "Text",
 CellID->65021125],
Cell[4992, 124, 125, 6, 70, "Text",
 CellID->280757486],
Cell[5120, 132, 100, 3, 70, "Input",
 CellID->107795920],
Cell[CellGroupData[{
Cell[5245, 139, 68, 1, 70, "Section",
 CellID->152704095],
Cell[5316, 142, 259, 6, 70, "DefinitionBox",
 CellID->801716347],
Cell[5578, 150, 99, 1, 70, "Caption",
 CellID->104944398],
Cell[5680, 153, 289, 6, 70, "Text",
 CellID->63591597],
Cell[CellGroupData[{
Cell[5994, 163, 2111, 68, 70, "Input",
 CellID->101936636],
Cell[8108, 233, 315, 9, 54, "Output",
 CellID->49642076]
}, Open  ]],
Cell[8438, 245, 187, 4, 70, "Text",
 CellID->139029545],
Cell[CellGroupData[{
Cell[8650, 253, 64, 2, 70, "Input",
 CellID->203056159],
Cell[8717, 257, 1048, 34, 36, "Output",
 CellID->452522401]
}, Open  ]],
Cell[CellGroupData[{
Cell[9802, 296, 105, 3, 28, "Input",
 CellID->186875196],
Cell[9910, 301, 6461, 113, 198, "Output",
 CellID->228860420]
}, Open  ]],
Cell[16386, 417, 166, 4, 70, "Text",
 CellID->133331613],
Cell[CellGroupData[{
Cell[16577, 425, 109, 3, 70, "Input",
 CellID->228036416],
Cell[16689, 430, 682, 21, 36, "Output",
 CellID->517559482]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[17420, 457, 57, 1, 70, "Section",
 CellID->98413141],
Cell[CellGroupData[{
Cell[17502, 462, 104, 2, 70, "Subsection",
 CellID->578648157],
Cell[17609, 466, 338, 8, 70, "Text",
 CellID->110468743],
Cell[CellGroupData[{
Cell[17972, 478, 99, 3, 70, "Input",
 CellID->337721988],
Cell[18074, 483, 1847, 51, 54, "Output",
 CellID->42784967]
}, Open  ]],
Cell[CellGroupData[{
Cell[19958, 539, 99, 3, 70, "Input",
 CellID->716287734],
Cell[20060, 544, 2648, 75, 71, "Output",
 CellID->50318537]
}, Open  ]],
Cell[CellGroupData[{
Cell[22745, 624, 95, 3, 70, "Input",
 CellID->67522338],
Cell[22843, 629, 1446, 40, 54, "Output",
 CellID->685986808]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[24338, 675, 65, 1, 70, "Subsection",
 CellID->87368199],
Cell[24406, 678, 476, 10, 70, "Text",
 CellID->91427808],
Cell[CellGroupData[{
Cell[24907, 692, 201, 7, 70, "Input",
 CellID->87964621],
Cell[25111, 701, 2074, 56, 71, "Output",
 CellID->215268296]
}, Open  ]],
Cell[CellGroupData[{
Cell[27222, 762, 318, 11, 28, "Input",
 CellID->100168776],
Cell[27543, 775, 10443, 179, 179, "Output",
 CellID->33820876]
}, Open  ]],
Cell[38001, 957, 325, 6, 70, "Text",
 CellID->256903099]
}, Open  ]],
Cell[CellGroupData[{
Cell[38363, 968, 64, 1, 70, "Subsection",
 CellID->130895041],
Cell[38430, 971, 149, 4, 70, "Text",
 CellID->564988326],
Cell[CellGroupData[{
Cell[38604, 979, 100, 3, 70, "Input",
 CellID->1023740835],
Cell[38707, 984, 1245, 42, 54, "Output",
 CellID->408364341]
}, Open  ]],
Cell[CellGroupData[{
Cell[39989, 1031, 132, 4, 70, "Input",
 CellID->285324683],
Cell[40124, 1037, 5420, 154, 122, "Output",
 CellID->432006061]
}, Open  ]],
Cell[45559, 1194, 188, 4, 70, "Text",
 CellID->378189741],
Cell[CellGroupData[{
Cell[45772, 1202, 210, 7, 70, "Input",
 CellID->110286018],
Cell[45985, 1211, 366, 12, 36, "Output",
 CellID->413161789]
}, Open  ]],
Cell[46366, 1226, 189, 4, 70, "Text",
 CellID->113463184],
Cell[CellGroupData[{
Cell[46580, 1234, 131, 4, 70, "Input",
 CellID->221821086],
Cell[46714, 1240, 1148, 36, 36, "Output",
 CellID->776302981]
}, Open  ]],
Cell[CellGroupData[{
Cell[47899, 1281, 99, 3, 70, "Input",
 CellID->317318721],
Cell[48001, 1286, 234, 7, 36, "Output",
 CellID->118480808]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[48284, 1299, 47, 1, 70, "Subsection",
 CellID->337431051],
Cell[48334, 1302, 285, 6, 70, "Text",
 CellID->18165939],
Cell[48622, 1310, 479, 14, 70, "Text",
 CellID->435648623],
Cell[CellGroupData[{
Cell[49126, 1328, 865, 28, 70, "Input",
 CellID->256028157],
Cell[49994, 1358, 1790, 51, 63, "Output",
 CellID->403421165]
}, Open  ]],
Cell[51799, 1412, 445, 17, 70, "Text",
 CellID->732586039],
Cell[CellGroupData[{
Cell[52269, 1433, 1220, 38, 70, "Input",
 CellID->275335666],
Cell[53492, 1473, 821, 20, 133, "Output",
 CellID->85635655]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[54374, 1500, 58, 1, 70, "Section",
 CellID->23446292],
Cell[54435, 1503, 661, 16, 70, "Text",
 CellID->4393084],
Cell[CellGroupData[{
Cell[55121, 1523, 81, 1, 70, "Subsection",
 CellID->69497392],
Cell[55205, 1526, 158, 6, 70, "Text",
 CellID->83473493],
Cell[CellGroupData[{
Cell[55388, 1536, 105, 3, 70, "Input",
 CellID->191503734],
Cell[55496, 1541, 305, 10, 36, "Output",
 CellID->167751407]
}, Open  ]],
Cell[CellGroupData[{
Cell[55838, 1556, 124, 4, 70, "Input",
 CellID->703666199],
Cell[55965, 1562, 305, 10, 36, "Output",
 CellID->324481438]
}, Open  ]],
Cell[CellGroupData[{
Cell[56307, 1577, 214, 7, 70, "Input",
 CellID->6136457],
Cell[56524, 1586, 305, 10, 36, "Output",
 CellID->118209414]
}, Open  ]],
Cell[CellGroupData[{
Cell[56866, 1601, 103, 3, 70, "Input",
 CellID->87307636],
Cell[56972, 1606, 1235, 40, 54, "Output",
 CellID->503396216]
}, Open  ]],
Cell[CellGroupData[{
Cell[58244, 1651, 124, 4, 70, "Input",
 CellID->442110006],
Cell[58371, 1657, 1889, 61, 71, "Output",
 CellID->133434270]
}, Open  ]],
Cell[CellGroupData[{
Cell[60297, 1723, 147, 5, 70, "Input",
 CellID->59296511],
Cell[60447, 1730, 380, 13, 36, "Output",
 CellID->218334605]
}, Open  ]],
Cell[CellGroupData[{
Cell[60864, 1748, 127, 4, 70, "Input",
 CellID->542072997],
Cell[60994, 1754, 641, 22, 36, "Output",
 CellID->160345444]
}, Open  ]],
Cell[CellGroupData[{
Cell[61672, 1781, 126, 4, 70, "Input",
 CellID->15858325],
Cell[61801, 1787, 777, 26, 36, "Output",
 CellID->798170530]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[62627, 1819, 74, 1, 70, "Subsection",
 CellID->292283586],
Cell[CellGroupData[{
Cell[62726, 1824, 129, 4, 70, "Input",
 CellID->94031565],
Cell[62858, 1830, 2459, 76, 71, "Output",
 CellID->176309563]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[65366, 1912, 93, 1, 70, "Subsection",
 CellID->234066801],
Cell[CellGroupData[{
Cell[65484, 1917, 188, 6, 70, "Input",
 CellID->59182926],
Cell[65675, 1925, 1516, 46, 54, "Output",
 CellID->41966192]
}, Open  ]],
Cell[CellGroupData[{
Cell[67228, 1976, 188, 6, 70, "Input",
 CellID->473326778],
Cell[67419, 1984, 1517, 46, 54, "Output",
 CellID->240180580]
}, Open  ]],
Cell[CellGroupData[{
Cell[68973, 2035, 198, 6, 70, "Input",
 CellID->209839352],
Cell[69174, 2043, 1517, 46, 54, "Output",
 CellID->476777024]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[70740, 2095, 71, 1, 70, "Subsection",
 CellID->70600885],
Cell[CellGroupData[{
Cell[70836, 2100, 111, 3, 70, "Input",
 CellID->257261906],
Cell[70950, 2105, 1439, 46, 54, "Output",
 CellID->471881325]
}, Open  ]],
Cell[CellGroupData[{
Cell[72426, 2156, 132, 4, 70, "Input",
 CellID->59964444],
Cell[72561, 2162, 1439, 46, 54, "Output",
 CellID->366786247]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[74061, 2215, 50, 1, 70, "Section",
 CellID->23564291],
Cell[74114, 2218, 224, 5, 70, "Text",
 CellID->379619690]
}, Open  ]],
Cell[CellGroupData[{
Cell[74375, 2228, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[74443, 2231, 169, 4, 70, "TutorialMoreAbout",
 CellID->1567025153],
Cell[74615, 2237, 157, 3, 70, "TutorialMoreAbout",
 CellID->829829444]
}, Open  ]],
Cell[CellGroupData[{
Cell[74809, 2245, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[74884, 2248, 155, 3, 70, "RelatedTutorials",
 CellID->383413863],
Cell[75042, 2253, 154, 3, 70, "RelatedTutorials",
 CellID->806871991],
Cell[75199, 2258, 166, 4, 70, "RelatedTutorials",
 CellID->352542221]
}, Open  ]]
}, Open  ]],
Cell[75392, 2266, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

