(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    106798,       2603]
NotebookOptionsPosition[     97971,       2312]
NotebookOutlinePosition[    100435,       2369]
CellTagsIndexPosition[    100392,       2366]
WindowTitle->3.5  Convergence And Timing - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"3.4 Flow Visualization\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/FlowVisualization"], "\<\"3.6 \
Aerodynamic Coefficients\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicCoefficients"], "\<\"Overview\
\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PanelMethods\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/PanelMethods"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/tutorial/ConvergenceAndTiming\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/ConvergenceAndTiming"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/ConvergenceAndTiming.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/ConvergenceAndTiming", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["3.5  Convergence And Timing", "Title",
 CellID->509267359],

Cell["\<\
The example used in this chapter is the Eppler-205 airfoil at an angle of \
attack of 6 degrees.  Lift coefficient and pitching-moment coefficient for \
the airfoil are used as measures of convergence.  These coefficients are \
plotted versus the reciprocal of the number of panels describing the \
discretized airfoil for constant spacing.  If this curve has a linear region \
for large discretization numbers, it is tempting to extrapolate the line to \
the coefficient axis to obtain a reference value for the coefficient.\
\>", "Text",
 CellID->163863731],

Cell["\<\
Computational times for models are plotted versus number of panels describing \
the discretized airfoil.  Since implementation of models result in a full \
system of linear algebraic equations to be solved, this curve is expected to \
be quadratic.\
\>", "Text",
 CellID->133626429],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "Text",
 CellID->836781195],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->2058623809],

Cell[CellGroupData[{

Cell["3.5.1 Eppler-205 airfoil", "Section",
 CellID->627174914],

Cell[TextData[{
 "Use ",
 Cell[BoxData[
  ButtonBox["AirfoilFromDataFile",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilFromDataFile"]], 
  "InlineFormula"],
 " to import data from the file for the Eppler-205 airfoil which is located \
in the folder Airfoil Data. "
}], "Text",
 CellID->24014087],

Cell[BoxData[
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<Aerodynamics\\\\Airfoil Data\\\\e205.dat\>\"", "]"}]}], 
  ";"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->543466123],

Cell[TextData[{
 "Use ",
 Cell[BoxData[
  ButtonBox["FitAirfoil",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/FitAirfoil"]], "InlineFormula"],
 " to fit a B-spline to the data."
}], "Text",
 CellID->193395042],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"spline", "=", 
  RowBox[{"FitAirfoil", "[", "rD", "]"}]}]], "Input",
 CellLabel->"In[2]:=",
 CellID->199940495],

Cell[BoxData[
 TagBox[
  RowBox[{"BSplineFunction", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"{", 
      RowBox[{"0.`", ",", "1.0000000000000004`"}], "}"}], "}"}], 
    ",", "\<\"<>\"\>"}], "]"}],
  False,
  Editable->False]], "Output",
 ImageSize->{215, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->369970053]
}, Open  ]],

Cell["Visually check that the spline fits the data. ", "Text",
 CellID->107920476],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"gD", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gSpline", "=", 
   RowBox[{"ParametricPlot", "[", 
    RowBox[{
     RowBox[{"spline", "[", "t", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "0", ",", "1"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gD", ",", "gSpline"}], "]"}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->88849844],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3f+PFPUdx/Fr+pf037hEueMOFAk9D4vYWmrhZAtt1Wo0MagRalsIQrrx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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->450629884]
}, Open  ]],

Cell["\<\
 Specify a value for the Layout option and a list of discretization numbers.\
\>", "Text",
 CellID->797728430],

Cell[BoxData[
 RowBox[{
  RowBox[{"layout", "=", "ConstantSpacing"}], ";", 
  RowBox[{"n", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"Range", "[", 
      RowBox[{"10", ",", "20", ",", "5"}], "]"}], ",", 
     RowBox[{"Range", "[", 
      RowBox[{"25", ",", "250", ",", "25"}], "]"}]}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[6]:=",
 CellID->124450942],

Cell["\<\
Compute a list of collections of vectors locating panel endpoints for the \
list of discretization numbers.  Display the list of numbers of panels for \
discretized airfoils.\
\>", "Text",
 CellID->728508869],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"AirfoilSurfacePoints", "[", 
       RowBox[{"spline", ",", 
        RowBox[{"NDiscretizeUnitSegment", "[", 
         RowBox[{"#", ",", 
          RowBox[{"Layout", "\[Rule]", "layout"}]}], "]"}]}], "]"}], "&"}], 
     ",", "n"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"nP", "=", 
  RowBox[{"Length", "/@", 
   RowBox[{"PanelLengths", "/@", "rP"}]}]}]}], "Input",
 CellLabel->"In[7]:=",
 CellID->207596273],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "20", ",", "30", ",", "40", ",", "50", ",", "100", ",", "150", ",", "200", 
   ",", "250", ",", "300", ",", "350", ",", "400", ",", "450", ",", "500"}], 
  "}"}]], "Output",
 ImageSize->{395, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->72248229]
}, Open  ]],

Cell[TextData[{
 "Specify the last ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["n", "TI"], 
   StyleBox["f", "TI"]]], "InlineFormula"],
 " values in the list of panel numbers to be used to determine a \
straight-line fit to the convergence data."
}], "Text",
 CellID->329689094],

Cell[BoxData[
 RowBox[{
  RowBox[{"nF", "=", "5"}], ";"}]], "Input",
 CellLabel->"In[9]:=",
 CellID->3218154]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.5.2  Solve models", "Section",
 CellID->186435948],

Cell[TextData[{
 "A given model is solved and aerodynamics coefficients computed for the list \
of discretized airfoils.  The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function Timing is used to return the computational time in seconds for \
the solution for each discretization.  Results for aerodynamic coefficients \
are grouped by type rather than by iteration number."
}], "Text",
 CellID->109828650],

Cell["Specify an angle of attack.", "Text",
 CellID->64575450],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Alpha]", "=", 
   RowBox[{"6.", " ", "Degree"}]}], ";"}]], "Input",
 CellLabel->"In[10]:=",
 CellID->63735315],

Cell[CellGroupData[{

Cell["PMVortexLinearNeumann", "Subsection",
 CellID->276154793],

Cell[TextData[{
 "Compute information for generating convergence and timing plots for the \
model using vortex panels of linearly varying strength and external Neumann \
boundary conditions.  Map the model ",
 Cell[BoxData[
  ButtonBox["PMVortexLinearNeumann",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/PMVortexLinearNeumann"]], 
  "InlineFormula"],
 " to each collection of vectors describing a discretized airfoil, arranging \
the result to collect like terms together.  Maximum computational time is \
labeled for later use.  The function ",
 Cell[BoxData[
  ButtonBox["ForceAndMoment",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/ForceAndMoment"]], "InlineFormula"],
 " is then used to determine lift a and pitching-moment coefficients used in \
convergence plots. "
}], "Text",
 CellID->114624859],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"tV2", ",", "a", ",", "b", ",", "rC", ",", "VMagC"}], "}"}], "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Flatten", "[", 
        RowBox[{
         RowBox[{"Timing", "[", 
          RowBox[{"PMVortexLinearNeumann", "[", 
           RowBox[{"#", ",", "\[Alpha]"}], "]"}], "]"}], ",", "1"}], "]"}], 
       "&"}], ",", "rP"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"clV2", ",", "cmV2"}], "}"}], "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Apply", "[", 
     RowBox[{"ForceAndMoment", ",", 
      RowBox[{"Thread", "[", 
       RowBox[{"{", 
        RowBox[{
        "\"\<Vortex\>\"", ",", "a", ",", "b", ",", "rP", ",", "\[Alpha]", ",",
          "rC"}], "}"}], "]"}], ",", 
      RowBox[{"{", "1", "}"}]}], "]"}], "]"}]}], ";"}]}], "Input",
 CellLabel->"In[11]:=",
 CellID->261387464]
}, Open  ]],

Cell[CellGroupData[{

Cell["PMDoubletConstantDirichlet", "Subsection",
 CellID->502776918],

Cell[TextData[{
 "Compute information for generating convergence and timing plots for the \
model using doublet panels of constant strength and internal Dirichlet \
boundary conditions.  This is accomplished by repeating the procedure of the \
previous section using the function ",
 Cell[BoxData[
  ButtonBox["PMDoubletConstantDirichlet",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/PMDoubletConstantDirichlet"]], 
  "InlineFormula"],
 "."
}], "Text",
 CellID->826207730],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"tD1", ",", "a", ",", "aW", ",", "rC", ",", "VMagC"}], "}"}], "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Flatten", "[", 
        RowBox[{
         RowBox[{"Timing", "[", 
          RowBox[{"PMDoubletConstantDirichlet", "[", 
           RowBox[{"#", ",", "\[Alpha]"}], "]"}], "]"}], ",", "1"}], "]"}], 
       "&"}], ",", "rP"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"clD1", ",", "cmD1"}], "}"}], "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Apply", "[", 
     RowBox[{"ForceAndMoment", ",", 
      RowBox[{"Thread", "[", 
       RowBox[{"{", 
        RowBox[{
        "\"\<Doublet\>\"", ",", "a", ",", "rP", ",", "\[Alpha]", ",", "rC"}], 
        "}"}], "]"}], ",", 
      RowBox[{"{", "1", "}"}]}], "]"}], "]"}]}], ";"}]}], "Input",
 CellLabel->"In[13]:=",
 CellID->30890011]
}, Open  ]],

Cell[CellGroupData[{

Cell["PMDoubletQuadraticDirichlet", "Subsection",
 CellID->118691814],

Cell[TextData[{
 "Compute information for generating convergence and timing plots for the \
model using doublet panels of quadratically varying strength and internal \
Dirichlet boundary conditions.  This is accomplished by repeating the \
procedure of the previous section using the function ",
 Cell[BoxData[
  ButtonBox["PMDoubletQuadraticDirichlet",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/PMDoubletQuadraticDirichlet"]], 
  "InlineFormula"],
 "."
}], "Text",
 CellID->60685572],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "tD3", ",", "a", ",", "b", ",", "c", ",", "aW", ",", "rC", ",", "VMagC"}],
     "}"}], "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Flatten", "[", 
        RowBox[{
         RowBox[{"Timing", "[", 
          RowBox[{"PMDoubletQuadraticDirichlet", "[", 
           RowBox[{"#", ",", "\[Alpha]"}], "]"}], "]"}], ",", "1"}], "]"}], 
       "&"}], ",", "rP"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"clD3", ",", "cmD3"}], "}"}], "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Apply", "[", 
     RowBox[{"ForceAndMoment", ",", 
      RowBox[{"Thread", "[", 
       RowBox[{"{", 
        RowBox[{
        "\"\<Doublet\>\"", ",", "a", ",", "b", ",", "c", ",", "rP", ",", 
         "\[Alpha]", ",", "rC"}], "}"}], "]"}], ",", 
      RowBox[{"{", "1", "}"}]}], "]"}], "]"}]}], ";"}]}], "Input",
 CellLabel->"In[15]:=",
 CellID->135262774]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.5.3  Convergence properties", "Section",
 CellID->227129737],

Cell[TextData[{
 "Graphs showing convergence properties are presented in this section.  The \
function ",
 Cell[BoxData[
  ButtonBox["PlotConvergence",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/PlotConvergence"]], "InlineFormula"],
 " is used to generate plots of lift or pitching-moment coefficient versus \
reciprocal of number of panels representing the discretized airfoil."
}], "Text",
 CellID->74633868],

Cell["\<\
In each graph, a straight line is fit to data points for relatively fine \
discretization (as specified by the user), and both data and line are shown \
in blue.  Data points for coarser discretization, not used to determine the \
line, are shown in red.  Extrapolation of the line beyond the range of data \
used to determine the equation for the line is shown as dashed.\
\>", "Text",
 CellID->98070734],

Cell[CellGroupData[{

Cell["Lift Coefficient", "Subsection",
 CellID->869225247],

Cell["\<\
Determine the minimum and maximum values of lift coefficient to be used for \
plotting convergence graphs for lift coefficient.\
\>", "Text",
 CellID->39606871],

Cell[BoxData[{
 RowBox[{
  RowBox[{"cl", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"clV2", ",", "clD1", ",", "clD3"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"clMin", ",", "clMax"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Floor", "[", 
       RowBox[{"10", 
        RowBox[{"Min", "[", "cl", "]"}]}], "]"}], ",", 
      RowBox[{"Ceiling", "[", 
       RowBox[{"10", 
        RowBox[{"Max", "[", "cl", "]"}]}], "]"}]}], "}"}], "/", "10."}]}], 
  ";"}]}], "Input",
 CellLabel->"In[17]:=",
 CellID->102708401],

Cell["\<\
The following series of graphs presents convergence properties of the lift \
coefficient for the three models.\
\>", "Text",
 CellID->2960992],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"clV2Ref", ",", "gClV2"}], "}"}], "=", 
   RowBox[{"PlotConvergence", "[", 
    RowBox[{"nP", ",", "clV2", ",", "nF", ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<reciprocal of number of panels\>\"", ",", 
        "\"\<\!\(\*Cell[TextData[Cell[BoxData[
FormBox[
SubscriptBox[\"C\", \"\[ScriptL]\"], TraditionalForm]], \"InlineFormula\"]], \
\"InlineFormula\"]\)\>\"", ",", "\"\<PMVortexLinearNeumann\>\""}], "}"}]}], 
     ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"clMin", ",", "clMax"}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"clD1Ref", ",", "gClD1"}], "}"}], "=", 
   RowBox[{"PlotConvergence", "[", 
    RowBox[{"nP", ",", "clD1", ",", "nF", ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<reciprocal of number of panels\>\"", ",", 
        "\"\<\!\(\*Cell[TextData[Cell[BoxData[
FormBox[
SubscriptBox[\"C\", \"\[ScriptL]\"], TraditionalForm]], \"InlineFormula\"]], \
\"InlineFormula\"]\)\>\"", ",", "\"\<PMDoubletConstantDirichlet\>\""}], 
       "}"}]}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"clMin", ",", "clMax"}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"clD3Ref", ",", "gClD3"}], "}"}], "=", 
   RowBox[{"PlotConvergence", "[", 
    RowBox[{"nP", ",", "clD3", ",", "nF", ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<reciprocal of number of panels\>\"", ",", 
        "\"\<\!\(\*Cell[TextData[Cell[BoxData[
FormBox[
SubscriptBox[\"C\", \"\[ScriptL]\"], TraditionalForm]], \"InlineFormula\"]], \
\"InlineFormula\"]\)\>\"", ",", "\"\<PMDoubletQuadraticDirichlet\>\""}], 
       "}"}]}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"clMin", ",", "clMax"}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"GraphicsColumn", "[", 
    RowBox[{"{", 
     RowBox[{"gClV2", ",", "gClD1", ",", "gClD3"}], "}"}], "]"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", " ", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[19]:=",
 CellID->147688655],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3c+LI916H/DOm5eXy03s+9qxL+ObXGfsJDD+ncQYkw4m8ycYr2ZlxlmI
2eVfUZaB7AIBQxYBgwaDd4GAIWAImFnIEDBcxovAgMGQCxeUKql+S+oudbee
p0r1+TDMtEqnSuc86vnq9OlS6Zf+/X/4g3//9+/u7v5j8df/+3t3d+XXOwAA
AACA89b3d63VZrfbHraUXzYO2+5Xq/u7zn2b1VHDC1WPdXe/3rYbi8Nefsx9
X9rDHLrWOyzAYhVhe8jVMnX3X23WRaJ3QnK7aW72Q3izfjBJt+v1o4FdHPql
0rgM96ZzYx4bYBmanC+/2oduEd+bdRv0xc11k8bb9o4mSvvT8iK59xP/1aqZ
7zfT6/qHhXX57+E15Sjn65eS8t99uyq8O3P0+gEPx64ebtPrdt25Zq/yi/K+
/YbqB5fq+OW+7c8m/YP3+jDsEsAsnJrPH/Kwytd1GZJNGvdb7TrJX+xxX7wi
1GFcZm4VnKv97VWzLlSEZT3bHub8tnoFqJaKivsO/esepN5y3324ukPNw6w3
g4euH2q/e+/4bfBvdp2Db/p9GHYJYCa27RJ9LzCr/N5sNoM03uf55pCgu+5S
Tj9OmzQ8zJW7x1+1S+fH8/l6t/6/g4O0k/v2APULT/VasN4MHrqX853Xt8H2
7sEHfRl2DWAWjkOrCswy6Fer9fHqSjMDbm51A/Mo57fNS8Kh0eHF4K4fsm27
0znfPchh5/p16Cjnd90peuehOwe8a34q2A1zvndwOQ/MXz2Z763LdGbNnal0
b0rfzbnu+ny7uLFrl8LrIzTr8oMTeI5m6tV6evff9iD1Asr9ffVX06zz6lP3
sN2rO7tvRlUfrN7QLNbcN79haPuwHnYJAAAAAAAAAOCk7bk3AA4uIuKaIgCz
dep84fbt6Z1T8F70eiUARDmR8+2mfbIPbkZ3EIBnOZHz3Xe7H946cjLn7wDy
hKXk/D19Pn+uzhfVf3zjKx32eo2n0IeLGk+hDxc1nkIfrtd4Cn24qHFwH+T8
JZ6+Pi/np9+HixpPoQ8XNZ5CH67XeAp9uKixnJ+s6rIgR580MeZ8mxfJ+dm5
4dHd8NB2Rjdncj6RnL8xNzy0ndHNmZxPpM5AFvkTQ52BLPInhjoDWeRPDHUG
ssifGOoMZJE/MZ5Z5y9fdn/0R7sf/GD3/e/vfv/3d58/v1S/gNsn52M8s87v
3xdHaP98++3uD/5g94d/uHv7tv3i3Tv5D5wg52M88yoTr17tvvnm737j1zev
/+n/Kv58/+f/6tvv/ei73/3SDf/iT5H25dT/wwe5D+yenTxc5Jl1LmK7nMZ/
70eHnP+VN3/2W7/5J7/z23/8737vP/3ev/3PxdfFn3/4D/5v+SDdqX+Z+zX5
D0sl52M8s86fPu1++Zd3g9n74c9XX/2kyP/iz9df//j16/3Uv3t341z+d3kt
gFsk52O8VJ2LwH/zpozqn/u53Q9/2PuiCPiPH+up/+FPmfu1c/nfNea1AJgb
OR8jrs7NK0GV+7Vz+d815rVgJD8awGTI+Rj5dT6X/11jXgtG8qMBTEZ+/izD
POo85rVgpBf80QB4nnnkz/wtrs4v+KMB8DyLy58ki6vzC/5oADzP4vIniToD
WeRPDHUGssifGOp8O5wyytzInxiuMnE7nDLKfEieSOp8O5wyytzInxjqfDuC
Txm1TMSzyZ8Y6nw7gk8ZtUzEs8mfGOrME1km4tnkTwx15om8s5hnkz8x1Jkn
8s5ink3+xFBnFsdvkCdD/sRQZxbHb5AnQ/7EUGcWx2+QJ0P+xFBnFsdvkCdD
/sRQZxbHb5AnQ/7EcJUJIIXkCaPOQBb5E0OdgSzyJ4Y6A1nkTwx1Bq7r/BvT
5E8MdQau6/wb0+RPDHUGruv8G9PkTwx1Bq7r/BvT5E8MdQau6/wb0+RPDHUG
ssifGOoMZJE/MVz3AEghecKoM5BF/sRQZyCL/ImhzkAW+RNDnYEs8ieGOgNZ
5E8MdQayyJ8Y6gxkkT8x1BnIIn/G26zKtxvcr7f9zdv1/f59CKtNf0OvoToD
WeTPWEXK75N8s+onfX27vn+3WQ9fCXbqDOSRPyMVk/Rqwt4P+ibeq+3VZL6Z
3FfUGcgif0Ya5nmtyPXqZnd7mfa9qL/ri+o1sFwy51Ln5vP1sv1wQX67Xlmf
B6ZA/ox1bn3+YBDrZbPhfP7K/QM4Tf6M1z/fpsr9wck19ck3wwV6dQayyJ8Y
6gxkkT8x1BnIIn9iqDOQRf7EUGcgi/yJoc5AFvkTQ52BLPInhjoDWeRPDHUG
ssifGK4yAcSTPJHUGcgif2KoM5BF/sRQZyCL/ImhzkAW+RNDnYEs8ieGOgNZ
5E8MdQayyJ8Y6gxkkT8x1BnIIn9iqDOQRf7EUGcgi/yJ4SoTQDzJE0mdgSzy
J4Y6A1nkTwx1BrLInxjqDGSRPzHUGcgif2KoM5BF/sRQZyCL/ImhzkAW+RND
nYEs8ieGOgNZ5E8MdQayyJ8Ydx3ZfQGWQuxEUmcgi/yJoc5AFvkTQ52BLPIn
hjoDWeRPDHUGssifGOoMZJE/MdQZyCJ/YqgzkEX+xFBnIIv8iaHOQBb5E0Od
gSzyJ4arTADxJE8kdQayyJ8Y6gxkkT8x1BnIIn9iqDOQRf7EUGcgi/yJoc5A
FvkTQ52BLIvNn82qOq30frW6X23G73K/3vY3b9f3+wPVBznZbLF1BtItM3/K
KK5juUjpo+g+s89+l82q37y+Xd1/ptky6wxMwRLzp47iixSz9mqnfoK3B9tv
P9dsiXUGpmGB+TOckI/d6+5kgBfBXt3cbz/XbIF1BiZigfmzPr0cXyT0Q/l/
bqLeXekvNj8wn+96iXEAPGTJmXMy57fr9abwwG7n1ufr/Vf1rN76PDApC8yf
8vyY7rkx5ddFzK9Wjy3a90+kqQK9OtumE+rOtwEmZZn5U50J2ZwMWcb8g5P5
Z1tmnYEpkD9PPP/mQuoMZJE/pc36CWfgXESdgSzyJ4Y6A1nkTwx1BrLInxjq
DGSRPzHUGcgif2KoM5BF/sRQZyCL/Imx2KtMALkkTxh1BrLInxjqDGSRPzHU
Gcgif2KoM5BF/sRQZyCL/ImhzkAW+RNDnYEs8ieGOgNZ5E8MdQayyJ8Y6gxk
kT8xXPcASCF5wqgzkEX+xFBnIIv8iaHOQBb5E0OdgSzyJ4Y6A1nkTwx1BrLI
nxjqDGSRPzHUGcgif2KoM5BF/sRQZyCL/ImhzkAW+RPD9W2AFJInjDoDWeRP
DHUGssifGOoMZJE/MdQZyCJ/YqgzkEX+xFBnIIv8iaHOQBb5E0OdgSzyJ4Y6
A1nkTwx1BrLInxjqDGSRPzFc3waIJ3kiqTOQRf7EUGcgi/yJoc5AFvkTQ52B
LPInhjoDWeRPDHUGssifGOoMZJE/MdQZyCJ/xtusyrcb3K+3/c3b9f1d745q
Q6+hOgNZ5M9YRcqvNvt/+0lfpHq1/XD/brMevhLs1BnII39GquP8OOirgN+s
9vdXk/mqbUOdgSzyZ6Rmun6U89WCTi/Zy7Tvbbjru35/gaWTOZd6aD5/mM73
F+S365X1eWAK5M9Y59bn6+3FC0H3jmoZp6bOQBb5M17/fJtmIac+32Z/s3ej
Q52BLPInhjoDWeRPDHUGssifGOoMZJE/MdQZyCJ/YqgzkEX+xFBnIIv8iaHO
QBb5E0OdgSzyJ4YLTQDxxE4kdQayyJ8Y6gxkkT8x1BnIIn9iqDOQRf7EUGcg
i/yJoc5AFvkTQ52BLPInhjoDWeRPDHUGssifGOoMZJE/MdQZyCJ/Yri+DRBP
7ERSZyCL/ImhzkAW+RNDnYEs8ieGOgNZ5E8MdQayyJ8Y6gxkkT8x1BnIIn9i
qDOQRf7EUGcgi/yJoc5AFvkTQ52BLPInhqtMAPEkTyR1BrLInxjqDGSRPzHU
Gcgif2KoM5BF/sRQZyCL/ImhzkAW+RNDnYEs8ieGOgNZ5E8MdQayyJ8Y6gxk
kT8x1BnIIn9iuMoEEE/yRFJnIIv8iaHOQBb5E0OdgSzyJ4Y6A1nkTwx1BrLI
nxjqDGSRPzHUGcgif8bbrMrTUO/X2/7m7fr+rnvHyWbn6nzb9b/h0d3w0HZG
N2cnR3fbQ35JRXyvNvt/+xFexHy1fX//mWYvkvNXerKu14cb7vAU+nBp4yn0
QYev3Qc5/xx1nB8HfRPw5d/nmgXn/ES+7XV4Oo2n0IfrNZ5CHy5qLOenqZ6n
H+d8tVJzuPdcMzk//T5c1HgKfbio8RT6cL3GU+jDRY3l/DQ9NJ+v1mvKzeeb
tReaAAgWFpXzdm59vt5eJHx5x7lmAExe/0SaZoWmPt+mXbDpNgMAAABgus6t
5Ay2z3TBZ9zoDqtb1dLWjIwb3eHWjT53wzcCzsbI/3el9vyJeRg7tOq5m91T
N0OP/QK32j7TX+COHF1pbv+XdmNHt12v65Nqb/G52263uxOtpu6C78zOidGz
MHpom/WsnrI5O3em5WD7AydkTtnI0Q22zcUlozu+NXWXja5+MZuL8aMrT4ye
1ffm2KFVk/n5DGzOzr1zarD9ofdhTdjI0e2/mtX/pb1LRrer30kxG+NHN8fA
GDu64fvYZ+Cyb8vyyZvP2GbLfH6+OX/RjHduE95Lf1ppU2QWxo1uU58UPadV
7Et/zNyuVzMZ2ZxZn7/19fld8y65eS2IXvDc7eaXFheNbl7fm5c9cXP7OXO2
zrzB6ibPt3lkdLMb3pjRVUPrvlluJi4Z3dyeudHfmaV55fy4oQ3fwAkAAAAA
AAAAAAAAAAAAAAAAJHupqzI+fpwXu+DPs69893KXHprZNS0Bnmx/CakHE+/F
LpxVhvSzEvrlLuH1+KgBrq2Y+q5W+zA6cUXGo8tDr1fN1f7aHQefltrZt75n
v8NxenZ2PHclwfLB1/UH6dYB3nyoXXnIdf2JH5te37pd7V6zsO32xT3pHbM7
tF4nd0eXSOy0XxXdfLGnDmCEfSIdPvL0cC31fah3E7mekbYpuv80nlVnx/vm
taH36TB1xtUNhjk/2PHU1dwP4Vi9YhwStZOevXvb4D8Eb9vVTWdobbd3l/Vk
MPxNZ2ibQSe7lWx61HTM5B6I1n4kaPsZQuuTn8nT/QSRffA3O7ZX+i5n1YMP
dLjrvCD0Qm6w4+lP7ehMh49yvnPIwfZ+V7tD255auh/Vk8Hwu0Prd7L3cG0X
t50JP0CgNiHbbOsncifX6jQ7LJ60n93QCdvuvnVqnp3Pd3cclfO9VZDODx5n
cv7w0L0Dn8r5UT3pD783tH4n+3s3HV+fXrsCuK526aOd0dcLH3ftoni7Kt5f
ee9/Sk9/fb5ZuL+/b5f/+xnb3bH6ehDC9U7HneiuJW27C/hN884ae9Oldafb
x3V4qCf94e+6Q1ud62SnR+WLzalVf4AJmdnH9L60hQ8fuH3bMzPhhVj48AEA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAB61Xd/ftVab
Zkv55aDV/X23bXF7vR3xCJtVv91m1T/2iR2aw29WDzd9hvpRhmM57t6pDg+3
PToogERFiB8yqkzz/Veb9Wp130nnIm/rm53ULpPy0agfNtq/YhxH4na93jTt
q7vPNH1kLOtHd+g81mVj6Ryh37PHBgWQq8n58qt92G2KgFq3QV/cXK+Oc74N
5fqHgvKuOjH38+XyznKP9ar9EaF5uMOM+vCA1f3NK81RDzuT7nLuvN5v2Tc9
3FlM/DftcZo9Dh3otW/anB5LPTMv7ixe7MptzVy9+hGg3KkcROeYDw7qpZ8v
gEudms8fgrqKrnW5gHIq5w871K8O5R6HhD/crI7bfTGo/j38s6rCsEjS4gh1
Zh/PqvvH3xyWkKqs3bSPUvzdPc7hruP2TZtzY9n3dp/R+3ubuXr7cli1bo/5
8KAAsnWW6OswX1cJt4/tTXfqO5wD7xfR75rgPGTkIOerPZosbOK/edBOXD60
Fn74apCt9YS+u2nXnVgfZfHJnK9epJq72zuH49j1H7v778lBAWQ7zqNDzu+D
frVa91LvxJp2G86HO3v5Wy57NLHfDdJq6jvsQmd9vt7lgePvU3/dm8HXazP1
fH5czrfr82dzfjifP5XzZwYFkKiZzDeRV225bxYg2hNgzp1v018/78xqe3Pc
zgJ2Z/2606hdPRo8QO/49QGqfzfbw+J/Z/G9Wn459He16revf3OwOn2+TX3U
ZrXnZIcHfdg+OigAAAAAAAAAYOk6Zyz3dd4NeOImAPNx6uTgzlvG25Pyji9k
BcAMnMj5wRtQzrzvEIBZOJHz3fenD94OP8j5O4AkMRF5E54+nz9X54vqP77x
lQ57vcZT6MNFjafQh4saT6EP12s8hT5c1Di4D3L+Ek9fn5fz0+/DRY2n0IeL
Gk+hD9drPIU+XNRYzk9W50rcu8EFBe8eO9/mRXJ+Cq73/XklV+rD9YY2u6JN
ocMXue0Oy/lEN5PzF7nh0d3w0HZGN2dyPpE6A1nkTwx1BrLInxjqDGSRPzHU
Gcgif2KoM5BF/sRQZyCL/ImhzkAW+RPDVSaAeJInkjoDWeRPDHUGssifGOoM
ZJE/MdQZyCJ/YqgzkEX+xFBnIIv8iaHOQBb5E0OdgSzyJ4Y6A1nkTwx1BrLI
nxjqDGSRPzHuOrL7AiyF2ImkzkAW+RNDnYEs8ieGOgNZ5E8MdQayyJ8Y6gxk
kT8x1BnIIn9iqDOQRf7EUGcgi/yJoc5AFvkTQ52BLPInhjoDWeRPDFeZAOJJ
nkjqDGSRPzHUGcgif2KoM5BF/sRQZyCL/ImhzkAW+RNDnYEs8ieGOgNZ5E8M
dQayyJ8Y6gxkkT8x1BnIIn9iqDOQRf7EcJUJIJ7kiaTOQBb5E0OdgSzyJ4Y6
A1nkTwx1BrLInxjqDGSRPzHUGcgif2KoM5BF/oy3WZWnod6vt/3N2/X9/vzU
1aa/odfwmXX+8mX34cPu7dvdu3e7z5+fcyRgceT8WEXK75N8s+onfX27vn+3
WQ9fCXbPrvP798URqj9F2gOMJ+dHKibp1YS9H/RNvFfbq8l8M7mvPLPOr16V
Cf+d7/ztIepN6YHx5PxIwzyvFble3exuL9O+F/XPrHMxh//mm7/7V//yv//c
P/o/pvTAReT8SOfm8/Wy/XBBfrteDdbnuy599E+fdl9/Xfz58a/96p/+s1/+
n1999RML9cDDnpM5C3Vuff5gEOtls5ecz+/2U/rDos0/+cf/+7d+80+K6b1Z
PTCGnB+vf75NlfuDk2vqk2+GC/TPr3MxpX/zpor6n/6pv/md3/7jn/2Zv3YG
DvAoOR/jperczOq//vrHv/Hrm196/eeW64GHyfkYL1XnZlb/1Vfl37/4w79o
1nBM7IGT5HyMF69zM7H/9ns/+u1//d9++qf+xsQeOEnOx3jxOncn9odTLn/2
Z/7akwkck/MxrlfnZmL/1Vc/ef3aJRKAITkf43p1bib2r17tPn50iQRgSM7H
iKvz4RIJzR9g8eR8jLg6N+s4xZ9yHadmPQeWSs7HiHv38XAdp2Y9BxbJdQ/C
5NfZeg4sVX7+LEN+nc+t5wC3Lj9/liG/zufWc7qs4cMtys+fZZhHna3hwy2a
R/7M3zzqbA0fbtE88mf+5lFna/hwi+aRP/M3jzqPWcMH5mYe+TN/6gxkkT8x
1BnIIn9iqDOQRf7EWFydnYoPk7G4/EkSd32biXAqPkzDspIn1eLq7FR8mIzF
5U+SxdXZqfgwGYvLnySLq7NT8WEyFpc/SdQZyCJ/YqgzkEX+xFBnIIv8iaHO
QBb5E0OdgSzyJ4Y6A1nkTwx1BrLInxjqDGSRPzEWd30bYAIkTyR1BrLInxjq
DGSRPzHUGcgif2KoM5BF/sRQZyCL/ImhzkAW+RNDnYEs8ieGOgNZ5E8MdQay
yJ8Y6gxkkT8x1BnIIn9iuMrE7fjyZffhw+7t2927d7vPn7N7Aw+RPJHU+Xa8
f188ndWfIu1h8uRPDHW+Ha9etTnvaWUO5E8Mdb4dxRy+CfnXr7N7A4+TPzHU
+XZ8+rR786YM+WJi//Fjdm/gcfInhjoDWeRPDHUGssifGOoMZFls/mxW1Wml
96vV/Wozfpf79ba/ebu+3x+oPsjJZoutM5BumflTRnEdy0VKH0X3mX32u2xW
/eb17er+M82WWWdgCpaYP3UUX6SYtVc79RO8Pdh++7lmS6wzMA0LzJ/hhHzs
XncnA7wI9urmfvu5ZgusMzARC8yf9enl+CKhH8r/cxP17kp/sfmB+XzXS4wD
4KyFZ87JnN+u15vCA7udW5+v91/Vs3rr88CkLDB/yvNjuufGlF8XMb9aPbZo
3z+Rpgr06mybTqg734aX5PKYPNsy86c6E7I5GbKM+Qcn88+2zDrzAlwek2eT
P088/+ZC6swTuTwmzyZ/Spv1E87AuYg680Quj8mzyZ8Y6swTuTwmzyZ/Yqgz
kEX+xFBnIIv8iaHOQBb5E0OdgSzyJ4Y6A1nkT4zFXmUCSCR5IqkzkEX+xFBn
IIv8iaHOQBb5E0OdgSzyJ4Y6A1nkTwx1BrLInxjqDGSRPzHUGcgif2KoM5BF
/sRQZyCL/ImhzkAW+RPDVSZYnC9fdh8+lJ97+O7d7vPn7N4slOSJpM4szvv3
7SfbFmlPHvkTQ51ZnFev2pz3/Z9K/sRQZxanmMM3If/6dXZvFk3+xFBnFufT
p92bN2XIFxP7jx+ze7No8ieGOgNZ5E8MdQayyJ8Y6gxkkT8x1BnIIn9iqDOQ
Rf7EUGcgi/yJoc5AFvkTw1UmgBSSJ4w6A1nkTwx1BrLInxjqDGSRPzHUGcgi
f2KoM5BF/sRQZyCL/ImhzkAW+RNDnYEs8ieGOgNZ5E8MdQayyJ8YrnsApJA8
YdQZyCJ/YqgzkEX+xFBnIIv8iaHOQBb5E0OdgSzyJ4Y6A1nkTwx1BrLInxjq
DGSRP+NtVuXbDe7X2/7m7fr+rndHtaHXUJ2BLPJnrCLlV5v9v/2kL1K92n64
f7dZD18JduoM5JE/I9Vxfhz0VcBvVvv7q8l81bahzkAW+TNSM10/yvlqQaeX
7GXa9zbc9V2/v8DSyZxLPTSfP0zn+wvy2/XK+jwwBfJnrHPr8/X24oWge0e1
jFNTZyCL/Bmvf75Ns5BTn2+zv9m70aHOQBb5E0Odgev68mX34cPu7dvdu3e7
z5+798ifGOoMXNf790XQVH+KtO+QPzHUGbiuV6/anO8HjvyJoc7AdRVz+Cbk
X7/u3iN/YqgzcF2fPu3evClDvpjYf/zYvUf+xFBnIIv8iaHOQBb5E0OdgSzy
J4arTADxJE8kdQayyJ8Y6gxkkT8x1BnIIn9iqDOQRf7EUGcgi/yJoc5AFvkT
Q52BLPInhjoDWeRPDHUGssifGOoMZJE/MdQZyCJ/YrjKBBBP8kRSZyCL/Imh
zkAW+RNDnYEs8ieGOgNZ5E8MdQayyJ8Y6gxkkT8x1BnIIn9iqDOQRf7EUGcg
i/yJoc5AFvkTQ52BLPInxl1Hdl+ApRA7kdQZyCJ/YqgzkEX+xFBnIIv8iaHO
QBb5E0OdgSzyJ4Y6A1nkTwx1BrLInxjqDGSRPzHUGcgif2KoM5BF/sRQZyCL
/InhKhNAPMkTSZ2BLPInhjoDWeRPDHUGssifGOoMZJE/MdQZyCJ/YqgzkEX+
xFBnIIv8GW+zKk9DvV9v+5u36/u77h0nm52r823X/4ZHd8ND2xndnJ0c3W0P
+SUV8b3a7P/tR3gR89X2/f1nmt1Mzl/U4SmM7kp9uN7QZle0KXT4IrfdYTn/
HHWcHwd9E/Dl3+eavUjOj298vf+nOjydPlzUeAp9uF7jKfThosbBfZDzI9Xz
9OOcr1ZqDveeaybnp9+HixpPoQ8XNZ5CH67XeAp9uKixnJ+mh+bz1XpNufl8
s/ZCEwDBwqJy3s6tz9fbi4Qv7zjXDIDJ659I06zQ1OfbtAs23WYAAAAATNe5
lZzB9pku+Iwb3WF1q1rampFxozvcutHnbvhGwNkY+f+u1J4/MQ9jh1Y9d7N7
6mbosV/gVttn+gvckaMrze3/0m7s6LbrdX1S7S0+d9vtdnei1dRd8J3ZOTF6
FkYPbbOe1VM2Z+fOtBxsf+CEzCkbObrBtrm4ZHTHt6bustHVL2ZzMX505YnR
s/reHDu0ajI/n4HN2bl3Tg22P/Q+rAkbObr9V7P6v7R3yeh29TspZmP86OYY
GGNHN3wf+wxc9m1ZPnnzGdtsmc/PN+cvmvHObcJ76U8rbYrMwrjRbeqToue0
in3pj5nb9WomI5sz6/O3vj6/a94lN68F0Queu9380uKi0c3re/OyJ25uP2fO
1pk3WN3k+TaPjG52wxszumpo3TfLzcQlo5vbMzf6O7M0r5wfN7ThGzgBAAAA
AAAAAAAAAAAAAAAAgGQvdVXGx4/zYhf8efaV717u0kMzu6YlwJPtLyH1YOK9
2IWzypB+VkK/3CW8Hh81wLUVU9/Vah9GJ67IeHR56PWqudpfu+Pg01I7+9b3
7Hc4Ts/OjueuJFg++Lr+IN06wJsPtSsPua4/8WPT61u3q91rFrbdvrgnvWN2
h9br5O7oEomd9quimy/21AGMsE+kw0eeHq6lvg/1biLXM9I2RfefxrPq7Hjf
vDb0Ph2mzri6wTDnBzueupr7IRyrV4xDonbSs3dvG/yH4G27uukMre327rKe
DIa/6QxtM+hkt5JNj5qOmdwD0dqPBG0/Q2h98jN5up8gsg/+Zsf2St/lrHrw
gQ53nReEXsgNdjz9qR2d6fBRzncOOdje72p3aNtTS/ejejIYfndo/U72Hq7t
4rYz4QcI1CZkm239RO7kWp1mh8WT9rMbOmHb3bdOzbPz+e6Oo3K+twrS+cHj
TM4fHrp34FM5P6on/eH3htbvZH/vpuPr02tXANfVLn20M/p64eOuXRRvV8X7
K+/9T+npr883C/f39+3yfz9juztWXw9CuN7puBPdtaRtdwG/ad5ZY2+6tO50
+7gOD/WkP/xdd2irc53s9Kh8sTm16g8wITP7mN6XtvDhA7dve2YmvBALHz4A
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAMDDtuv7u9Zq
02wpvxy0ur/vti1ur7cjHmGz6rfbrPrHPt+jccevd3rwoOceuthyckQnWx49
wnDbY0MDyNFkZJmw+68269XqvpOy201zs5PaZUY+GsXDRvsUPw7D7XrddKFu
3v360RGcOujw4Od6eNmIzj7qI0MDSNPOhYuv9jG3KaJp3QZ9cXO9Os75Qy52
fgTYZ2SdlfuZcnlnucd61f6I0DzcYS59eMDq/u36fvCicNi3d8D60fZHaWf/
1WMVr1DF1+tOm82q81X3oevHPjGiumVzwM2pfcuhHB7p8aFd69kDeNyp+fwh
V6vQKm5tTub8YYcmnIs9DoF8uFkdt/tiUP17+GdVxWCRocURqlg9mvwX2/sH
bP4uNnYfeh/FVbh22zQHb+bb/UWe0yO6Ozxw54An9q1/4qg3PjQ0gDydJfo6
zNdVtu1je7PphOFw9lsGabMqffjqKOerPZoUbOK/edBOUA5WQk68cHSmy8Nj
D9Zg7nopfKrV6RG1D9XeeWrfQb4/ODSAPMdJdMj5fcyuVute3p1YzW7D+XBn
N7T3s+wmpbsRWk16B104uT7fP+Bq/2JSzdXr+f/gsTadNsc5f3Y+367Pn835
4Xz+VM6fHhpAjmYy34RdtaVeXK6XLx4636a7Pr/rzmd7s9vOXLyzct1p1Fv/
7q5q9w7YOfen07e7ermleekZtOktwzQPvTp9vk3VctMc8GS3623Df88ODYDK
tvPLTQBulDNVAAAAAABOOPubss5bAU/cBGA+Tp0Z3Hm/eHte3vG1rACYgRM5
327aJ/vgZnQHAXiWEznffQv84B3xg5y/A0gSE5E34enz+XN1vqj+4xtf6bDX
azyFPlzUeAp9uKjxFPpwvcZT6MNFjYP7IOcv8fT1eTk//T5c1HgKfbio8RT6
cL3GU+jDRY3l/GR1L+HdX7DpbD5zvk1wzl9kCt/2F5lCh6fQh0sbT6EPOnzt
Psj5RC+S87Nzw6O74aHtjG7O5HwidQayyJ8Y6gxkkT8x1BnIIn9iqDOQRf7E
UGcgi/yJoc5AFvkTQ52BLPInhqtMAPEkTyR1BrLInxjqDGSRPzHUGcgif2Ko
M5BF/sRQZyCL/InxzDp/+bL78GH39u3u3bvd588v1SlgEeR8jGfW+f374gjV
n+9/X9QDF5DzMZ5Z51evdl999ZOf/Zm/PkT9t9+a2ANjyfkYz6zz27e7r7/+
8a/96p/+7u/81x/8wl8WmV8c7wc/sJIDPE7Ox3hmnT992n33u+VM/jvf+dt/
8c//x7/53f/yS6///Jtv/u4wvS/SHuAcOR/j+XUuov7Nm2qJvpjb/+IP/6KY
2xeZXyT/IepN7IGT5HyMl7rKRJP233xTrtj/4Bf+skj7X3nzZ99+70cm9sCA
69tEevE6N4H/1Ve77//8X/3Wb/7JIeqdggkMyPkY16tzkejNKZevX/dPwTS/
B+R8lOvVuZnYv3q1+/hxfwpmk/OeXEDOR4mr83B+X7OeA0sl52PE1Xk4v69Z
z4GlkvMx8utsPQeWKj9/liG/zufWc4Bbl58/y5Bf53PrOcCty8+fZZhHnf2u
Fm7RPPJn/uZRZ7+rhVs0j/yZv3nU2e9q4RbNI3/m764juy/n+V0t3JYZxM4N
mUed/a4WbtE88mf+1BnIIn9iqDOQRf7EUGcgi/yJoc5AFvkTQ52BLPInxuLq
7K21MBmLy58ki6uzt9bCZCwuf5Isrs7eWguTsbj8SbK4OntrLUzG4vInyeLq
7K21MBmLy58k87i+DXBbxE4kdQayyJ8Y6gxkkT8x1BnIIn9iqDOQRf7EUGcg
i/yJoc5AFvkTQ52BLPJnvM2qPA31fr3tb96u7/fnp642/Q29huoMZJE/YxUp
v0/yzaqf9PXt+v7dZj18JdipM5BH/oxUTNKrCXs/6Jt4r7ZXk/lmcl9RZyCL
/BlpmOe1Iterm93tZdr3ol6dgSzyZ6Rz8/l62X64IL9drwbr8y5xA0SSORc7
tz5/MIj1spn5PDAJ8me8/vk2Ve4PTq6pT74ZLtCrM5BF/sRQZyCL/ImhzkAW
+RNDnYEs8ieGOgNZ5E8MdQayyJ8Y6gxkkT8x1BnIIn9iqPPt+PJl9+HD7u3b
3bt3u8+fs3sDj5M/MdT5drx/Xzyd1Z8i7WHy5E8MV5m4Ha9etTnvCWXaJE8k
db4dxRy+CfnXr7N7A4+TPzHU+XZ8+rR786YM+WJi//Fjdm/gcfInhjoDWeRP
DHUGssifGOoMZJE/MdQZyCJ/YqgzkEX+xFBnIIv8iaHOQBb5E0OdgSzyJ4Y6
A1nkTwxXmeCJXB6T55E8YdSZJ3J5TJ5N/sRQZ57I5TF5NvkTQ515IpfH5Nnk
Twx15olcHpNnkz8x1BnIIn9iqDOQRf7EUGcgi/yJoc5AFvkTQ52BLPInhjoD
WeRPDNc9AFJInjDqDGSRPzHUGcgif2KoM5BF/sRQZyCL/ImhzkAW+RNDnYEs
8ieGOgNZ5E8MdQayyJ8Y6gxkkT8x1BnIIn9iqDOQRf7EcH0bFufLl92HD+Xn
2757t/v8Obs3CyV5Iqkzi/P+ffsJ5kXak0f+xFBnFufVqzbnff+nkj8x1JnF
KebwTci/fp3dm0WTPzHUmcX59Gn35k0Z8sXE/uPH7N4smvyJoc5AFvkTQ52B
LPInhjoDWRabP5tVdVrp/Wp1v9pc++EWW2cg3TLzpwz5OtvX93f36+3Yve6O
G2+LI5TqA55stsw6A1OwxPwpgvgJE/h6r82qH+H17er+M82WWGdgGhaYP8Og
HqeYtVcvDv392xeN/fZzzRZYZ2AiFpg/69PL8UVgP5T/gzxvthfBXt3cbz/X
7K7vJcYB8JAlZ87JnN+u15vC+b3OTdS7v9EtNpvPA1OzwPwpf2/a/Z1p+XUR
86vVw4v259bn64Ou6lm99XlgUpaZP9UZMs1JMmXMPzSZP+ifSFMFenW2TSfU
nW8DTIr8eer5N5dRZyCL/Clt1k84A+ci6gxkkT8x1BnIIn9iqDOQRf7EUGcg
i/yJoc5AFvkTQ52BLPInhjoDWeRPjCVfaALIInYiqTOQRf7EUGcgi/yJoc5A
FvkTQ52BLPInhjoDWeRPDHUGssifGOoMZJE/MdQZyCJ/YqgzkEX+xFBnIIv8
iaHOQBb5E8P1bYB4YieSOgNZ5E8MdQayyJ8Y6gxkkT8x1BnIIn9iqDOQRf7E
UGcgi/yJoc5AFvkTQ52B6/ryZffhw+7t2927d7vPn7v3yJ8Y6gxc1/v3RdBU
f4q075A/MdQZuK5Xr9qc7weO/ImhzsB1FXP4JuRfv+7eI39iuMoEcF2fPu3e
vClDvpjYf/x42CZ5IqkzkEX+xFBnIIv8iaHOQBb5E0OdgSzyJ4Y6A1nkTwx1
BrLInxjqDGSRPzHUGcgif2KoM5BF/sRQZyCL/ImhzkAW+RPDVSaAeJInkjoD
WeRPDHUGssifGOoMZJE/MdQZyCJ/YqgzkEX+xFBnIIv8iaHOQBb5M95mVZ6G
er/e9jdv1/d3vTuqDb2G6gxkkT9jFSm/2uz/7Sd9kerV9sP9u816+EqwU2cg
j/wZqY7z46CvAn6z2t9fTeartg11BrLIn5Ga6fpRzlcLOr1kL9O+t0GdgSzy
Z6SH5vOH6Xx/QX67Xg3W57ti+gwsmcy52Ln1+Xp78ULQvaNaxqmpM5BF/ozX
P9+mWcipz7fZ3+zd6FBnIIv8iaHOQBb5E0OdgSzyJ4Y6A1nkTwx1BrLInxjq
DGSRPzHUGcgif2KoM5BF/sRQZyCL/Inh3cdACskTRp2BLPInhjoDWeRPDHUG
ssifGOoMZJE/MdQZyCJ/YqgzkEX+xFBnIIv8iaHOQBb5E0OdgSzyJ4Y6A1nk
Twx1BrLInxiubwPEkzyR1BnIIn9iqDOQRf7EUGcgi/yJoc5AFvkTQ52BLPIn
hjoDWeRPDHUGssifGOoMZJE/MdQZyCJ/YqgzkEX+xFBnIIv8ieEqE0A8yRNJ
nYEs8ieGOgNZ5E8MdQayyJ8Y6gxkkT8x1BnIIn9iqDOQRf7EUGcgi/yJoc5A
FvkTQ52BLPInhjoDWeRPDHUGssifGHcd2X0BlkLsRFJnIIv8iaHOQBb5E0Od
gSzyJ4Y6A1nkTwx1BrLInxjqDGSRPzHUGcgif8bbrMrTUO/X2/7m7fr+rnvH
yWbn6nzb9b/h0d3w0HZGN2cnR3fbQ35JRXyvNvt/+xFexHy1fX//mWZy/sbc
8NB2Rjdncv456jg/Dvom4Mu/zzV7kZwf3/hKh71e4yn04aLGU+jDRY2n0Ifr
NZ5CHy5qHNwHOT9SPU8/zvlqpeZw77lmcn76fbio8RT6cFHjKfTheo2n0IeL
Gsv5aXpoPl+t15SbH5jPA2SJTMsZO7c+X28vEr6841wzACavfyJNs0JTn2/T
Lth0mwEAAAAwXedWcgbbZ7rgM250h9WtamlrRsaN7nDrRp+74RsBZ2Pk/7tS
e/7EPIwdWvXcze6pm6HHfoFbbZ/pL3BHjq40t/9Lu7Gj267X9Um1t/jcbbfb
3YlWU3fBd2bnxOhZGD20zXpWT9mcnTvTcrD9/HmbkzZydINtc3HJ6I5vTd1l
o6tfzOZi/OjKE6Nn9b05dmjVZH4+A5uzc++cGmx/6H1YEzZydPuvZvV/ae+S
0e3qd1LMxvjRzTEwxo5u+D72Gbjs27J88uYzttkyn59vzl80453bhPfSn1ba
FJmFcaPb1CdFz2kV+9IfM7fr1UxGNmfW5299fX7XvEtuXguiFzx3u/mlxUWj
m9f35mVP3Nx+zpytM2+wusnzbR4Z3eyGN2Z01dC6b5abiUtGN7dnbvR3Zmle
OT9uaMM3cAIAAAAAAAAAAAAAAAAAAAAAyV7qqoyPH+fFLvjz7Cvfvdylh2Z2
TUuAJ9tfQurBxHuxC2eVIf2shH65S3g9PmqAayumvqvVPoxOXJHx6PLQ61Vz
tb92x8GnpXb2re/Z73Ccnp0dz11JsHzwdf1BunWANx9qVx5yXX/ix6bXt25X
u9csbLt9cU96x+wOrdfJ3dElEjvtV0U3X+ypAxhhn0iHjzw9XEt9H+rdRK5n
pG2K7j+NZ9XZ8b55beh9OkydcXWDYc4Pdjx1NfdDOFavGIdE7aRn7942+A/B
23Z10xla2+3dZT0ZDH/TGdpm0MluJZseNR0zuQeitR8J2n6G0PrkZ/J0P0Fk
H/zNju2VvstZ9eADHe46Lwi9kBvsePpTOzrT4aOc7xxysL3f1e7QtqeW7kf1
ZDD87tD6new9XNvFbWfCDxCoTcg22/qJ3Mm1Os0OiyftZzd0wra7b52aZ+fz
3R1H5XxvFaTzg8eZnD88dO/Ap3J+VE/6w+8Nrd/J/t5Nx9en164Arqtd+mhn
9PXCx127KN6uivdX3vuf0tNfn28W7u/v2+X/fsZ2d6y+HoRwvdNxJ7prSdvu
An7TvLPG3nRp3en2cR0e6kl/+Lvu0FbnOtnpUflic2rVH2BCZvYxvS9t4cMH
bt/2zEx4IRY+fAAAAABYhP8PGS1VOQ==
    "], {{0, 913}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 913},
  PlotRange->{{0, 504}, {0, 913}}]], "Output",
 ImageSize->{508, 913},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[22]=",
 CellID->160810047]
}, Open  ]],

Cell["\<\
Display the reference values obtained by extrapolating results of the linear \
fit for each model to an infinite number of panels.\
\>", "Text",
 CellID->477679825],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NumberForm", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"clV2Ref", ",", "clD1Ref", ",", "clD3Ref"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "3"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[23]:=",
 CellID->206162233],

Cell[BoxData[
 TagBox[
  RowBox[{"{", 
   RowBox[{
    InterpretationBox["\<\"0.991\"\>",
     0.9911654719692023,
     AutoDelete->True], ",", 
    InterpretationBox["\<\"0.960\"\>",
     0.960371085306418,
     AutoDelete->True], ",", 
    InterpretationBox["\<\"0.989\"\>",
     0.9888211492477125,
     AutoDelete->True]}], "}"}],
  NumberForm[#, {4, 3}]& ]], "Output",
 ImageSize->{145, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[23]//NumberForm=",
 CellID->636775393]
}, Open  ]],

Cell["\<\
For the Eppler-205 airfoil, the model PMVortexLinearNeumann provides the most \
linear and flattest convergence properties.  There is relatively good \
agreement between the two higher order models for extrapolated value of lift \
coefficient.\
\>", "Text",
 CellID->699005627]
}, Open  ]],

Cell[CellGroupData[{

Cell["Pitching-moment coefficient", "Subsection",
 CellID->870013475],

Cell["\<\
Determine the minimum and maximum values of pitching-moment coefficient to be \
used for plotting convergence graphs for moment coefficient.\
\>", "Text",
 CellID->1827792],

Cell[BoxData[{
 RowBox[{
  RowBox[{"cm", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"cmV2", ",", "cmD1", ",", "cmD3"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"cmMin", ",", "cmMax"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Floor", "[", 
       RowBox[{"10", 
        RowBox[{"Min", "[", "cm", "]"}]}], "]"}], ",", 
      RowBox[{"Ceiling", "[", 
       RowBox[{"10", 
        RowBox[{"Max", "[", "cm", "]"}]}], "]"}]}], "}"}], "/", "10."}]}], 
  ";"}]}], "Input",
 CellLabel->"In[24]:=",
 CellID->768607065],

Cell["\<\
The following series of graphs presents convergence properties of the \
pitching-moment coefficient for the three models.\
\>", "Text",
 CellID->334610627],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"cmV2Ref", ",", "gCmV2"}], "}"}], "=", 
   RowBox[{"PlotConvergence", "[", 
    RowBox[{"nP", ",", "cmV2", ",", "nF", ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<reciprocal of number of panels\>\"", ",", 
        "\"\<\!\(\*Cell[TextData[Cell[BoxData[
FormBox[
SubscriptBox[\"C\", \"m\"], TraditionalForm]], \"InlineFormula\"]], \
\"InlineFormula\"]\)\>\"", ",", "\"\<PMVortexLinearNeumann\>\""}], "}"}]}], 
     ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"cmMin", ",", "cmMax"}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"cmD1Ref", ",", "gCmD1"}], "}"}], "=", 
   RowBox[{"PlotConvergence", "[", 
    RowBox[{"nP", ",", "cmD1", ",", "nF", ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<reciprocal of number of panels\>\"", ",", 
        "\"\<\!\(\*Cell[TextData[Cell[BoxData[
FormBox[
SubscriptBox[\"C\", \"m\"], TraditionalForm]], \"InlineFormula\"]], \
\"InlineFormula\"]\)\>\"", ",", "\"\<PMDoubletConstantDirichlet\>\""}], 
       "}"}]}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"cmMin", ",", "cmMax"}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"cmD3Ref", ",", "gCmD3"}], "}"}], "=", 
   RowBox[{"PlotConvergence", "[", 
    RowBox[{"nP", ",", "cmD3", ",", "nF", ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<reciprocal of number of panels\>\"", ",", 
        "\"\<\!\(\*Cell[TextData[Cell[BoxData[
FormBox[
SubscriptBox[\"C\", \"m\"], TraditionalForm]], \"InlineFormula\"]], \
\"InlineFormula\"]\)\>\"", ",", "\"\<PMDoubletQuadraticDirichlet\>\""}], 
       "}"}]}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"cmMin", ",", "cmMax"}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"GraphicsColumn", "[", 
    RowBox[{"{", 
     RowBox[{"gCmV2", ",", "gCmD1", ",", "gCmD3"}], "}"}], "]"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", " ", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[26]:=",
 CellID->738949569],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3U2LNN15H/CJI4Q2tgx2rAdM4scg7CdvcrKITBot9BmysZbCi0K7fJXK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    "], {{0, 913}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 913},
  PlotRange->{{0, 504}, {0, 913}}]], "Output",
 ImageSize->{508, 913},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[29]=",
 CellID->957810550]
}, Open  ]],

Cell["\<\
Display the reference values obtained by extrapolating results of the linear \
fit for each model to an infinite number of panels.\
\>", "Text",
 CellID->612014715],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NumberForm", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"cmV2Ref", ",", "cmD1Ref", ",", "cmD3Ref"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "3"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[30]:=",
 CellID->271918122],

Cell[BoxData[
 TagBox[
  RowBox[{"{", 
   RowBox[{
    InterpretationBox["\<\"-0.304\"\>",
     -0.3036073698362213,
     AutoDelete->True], ",", 
    InterpretationBox["\<\"-0.289\"\>",
     -0.28875203917087106`,
     AutoDelete->True], ",", 
    InterpretationBox["\<\"-0.303\"\>",
     -0.303430877088969,
     AutoDelete->True]}], "}"}],
  NumberForm[#, {4, 3}]& ]], "Output",
 ImageSize->{166, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[30]//NumberForm=",
 CellID->539123902]
}, Open  ]],

Cell["\<\
As with the lift coefficient, the higher order models provide the more linear \
and flatter convergence properties than the low-order model.  Again, there is \
relatively good agreement between the two higher order models for \
extrapolated value of pitching-moment coefficient.\
\>", "Text",
 CellID->116669446]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.5.4 Timing properties", "Section",
 CellID->32786706],

Cell["\<\
Determine the maximum value of computational time to be used for plotting \
timing properties for the three models.\
\>", "Text",
 CellID->23262114],

Cell[BoxData[{
 RowBox[{
  RowBox[{"t", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"tV2", ",", "tD1", ",", "tD3"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tMax", "=", 
   RowBox[{
    RowBox[{"Ceiling", "[", 
     RowBox[{"10", 
      RowBox[{"Max", "[", "t", "]"}]}], "]"}], "/", "10."}]}], 
  ";"}]}], "Input",
 CellLabel->"In[31]:=",
 CellID->120331341],

Cell["\<\
Define a function for plotting timing data and fitting a quadratic to the \
data.\
\>", "Text",
 CellID->677306114],

Cell[BoxData[
 RowBox[{
  RowBox[{"plotTiming", "[", 
   RowBox[{"nP_List", ",", "t_List"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "data", ",", "fit", ",", "gData", ",", "gExtrapolate", ",", "gFit"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"data", "=", 
      RowBox[{"Transpose", "[", 
       RowBox[{"{", 
        RowBox[{"nP", ",", "t"}], "}"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"fit", "=", 
      RowBox[{"LinearModelFit", "[", 
       RowBox[{"data", ",", 
        RowBox[{"{", 
         RowBox[{"x", ",", 
          RowBox[{"x", "^", "2"}]}], "}"}], ",", "x"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"gData", "=", 
      RowBox[{"ListPlot", "[", 
       RowBox[{"data", ",", 
        RowBox[{"PlotRange", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "0.01"}], ",", "tMax"}], "}"}]}], ",", 
        RowBox[{"PlotStyle", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"AbsolutePointSize", "[", "5", "]"}], "}"}]}]}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"gFit", "=", 
      RowBox[{"Plot", "[", 
       RowBox[{
        RowBox[{"fit", "[", "x", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"x", ",", 
          RowBox[{"First", "[", "nP", "]"}], ",", 
          RowBox[{"Last", "[", "nP", "]"}]}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Show", "[", 
      RowBox[{"gData", ",", "gFit", ",", 
       RowBox[{"Frame", "\[Rule]", "True"}]}], "]"}]}]}], "]"}]}]], "Input",
 CellLabel->"In[33]:=",
 CellID->53565918],

Cell["\<\
The following series of graphs presents timing properties for the three \
models.\
\>", "Text",
 CellID->374596295],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"gTV2", ",", "gTD1", ",", "gTD3"}], "}"}], "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"plotTiming", "[", 
       RowBox[{"nP", ",", "#"}], "]"}], "&"}], ",", 
     RowBox[{"{", 
      RowBox[{"tV2", ",", "tD1", ",", "tD3"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"GraphicsColumn", "[", 
  RowBox[{"{", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Show", "[", 
     RowBox[{"gTV2", ",", 
      RowBox[{"FrameLabel", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"\"\<time\>\"", ",", "None"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
          "\"\<number of panels\>\"", ",", "\"\<PMVortexLinearNeumann\>\""}], 
          "}"}]}], "}"}]}]}], "]"}], ",", 
    RowBox[{"Show", "[", 
     RowBox[{"gTD1", ",", 
      RowBox[{"FrameLabel", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"\"\<time\>\"", ",", "None"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
          "\"\<number of panels\>\"", ",", 
           "\"\<PMDoubletConstantDirichlet\>\""}], "}"}]}], "}"}]}]}], "]"}], 
    ",", 
    RowBox[{"Show", "[", 
     RowBox[{"gTD3", ",", 
      RowBox[{"FrameLabel", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"\"\<time\>\"", ",", "None"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
          "\"\<number of panels\>\"", ",", 
           "\"\<PMDoubletQuadraticDirichlet\>\""}], "}"}]}], "}"}]}], ",", 
      RowBox[{"ImageSize", "\[Rule]", 
       RowBox[{"7", " ", "72"}]}]}], "]"}]}], "}"}], "]"}]}], "Input",
 CellLabel->"In[34]:=",
 CellID->94940072],

Cell[BoxData[
 GraphicsBox[{{}, {InsetBox[
     GraphicsBox[{
       {Hue[0.67, 0.6, 0.6], AbsolutePointSize[5], 
        PointBox[{{20., 0.}, {30., 0.014999999999999902`}, {40., 0.}, {50., 
         0.}, {100., 0.03200000000000003}, {150., 0.030999999999999917`}, {
         200., 0.062000000000000055`}, {250., 0.10899999999999999`}, {300., 
         0.1100000000000001}, {350., 0.15599999999999992`}, {400., 
         0.21799999999999997`}, {450., 0.2340000000000002}, {500., 
         0.34299999999999975`}}]}, {{}, {}, 
        {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwVz2tUzAkYBvCRrFBJDY12Iq2yShQj1NT71iQmQle6oMuUWdtVUzO6zD9t
dJ04stFsYURX3eYItboNUjJU84/Ukcus9iQqUhFq2w/PeT485/zOeVYFhLsG
qVAoFK/Z/N/Gxl5XKRQGHss9W6DzKQpsLT4lVRkxcJThS7GeiQIPZoa/3y4G
lrqV7Y1Q50Gya/2KphwG9un13lddw4PXwlW5hNlm1O84rtXty4O8Z4OiaW9L
tHjavEKnnQdL0mL532u2omgfb+U8aTR8H/rL6cvvNmg4HVDdVMIHn+WqVYpz
NhhmzShs/5sPdY4hSyvrbVBdIpgZkPNBIGG+Clpsiw5vvV/u+cSHcc8X0Qqp
LV50OemutBbAcPMKScVXQJW6SOrKLgGMtnfESVl2GNdW/O71wlh4I3Hr4nay
cKvy9Oi58nhY/uDZ57h/WLhDw12JsnjY995H9/QkC102PTT49jQe6i05vjX6
Dugc9ShOTEmAC+28gTlHHfC97FdpsnsCOI//OSVW2Y4dl9Fg/kwC3N7ZYyjf
5Iir93+Yc+8QAadHfKMscnbi3P35H7JWnwBqyOHS+Sxn5A4RI0y3ZGijFRws
MnHB9CPqqlasVAjZTXslSHXDYHpwfy4nEzKuEk9UlntiT5KM211xBnTPKdOy
8w/gvt7BaZpfNqS4s227JryRMmJIJNJzIGqb5HBb0kHU1HbJv+6aCw4jU7Fh
ysO47WFHOFWUB3ix2FyD7o9TVO5Y8PAloD/b7d14OAC9F72gpA1J4OiXj4mt
okDsS5xhBfYVwKSpVr9ZNwfjs5a4CKmFYFV2/XLOsmAMK3rpYx5YDCPaTn6e
9kdQq3/BumuFpfAm08/zWAoX93R2DOyyKocdI3z225u/IW2O1l1WYyW8C1SL
ujFzFNuPb7FTda4Gh9W6Kin2IbhoMFNVeVcKWnLjrKrwUDQ725s4z/wG6KVW
7rO8GYYqwu2T6Q9rIMd0m5bOv+H46gjTM/fALeCILou/pkbg4g6KoOL9baBr
+rfqmkZicAlTRyStA425V3j+skjcsvZQViH3DuifWMn0Cj6GEy4NFA+3BlAz
8r3pOj8Kuyl/NIkMm8DMZnklPz8K386YWL5Y0gyR/dp3nZCH0ulqC4a+DIbH
Mj3YPTw8oeN4QW/FXXDc0CJmh0Yjt+1lhMLgHtSPWX+b1IxB5ZlpcrvdfQhd
t7G381oMNqU6dUo8WoA5zCG92HwMU5PR6GkPZv/TnDXe8LH0hHT9lcpWWJOY
LpDxBSj5lHHG/VkbrA3a1mikdxzlHFd4vrIdOA3c5BvVxzFiD/9bs9cjaLhM
cj56xOLAa1N/ZY0cxopnJjdMxqJ10PnPtTpPIMCobIFYFIe9FUv3Ssw7YNQk
21JqHo83T3X2+w12wK2r316Pt8bjMtUiZXpuJ3g5/ILWhxJw15MANym7C/RC
U2rXU4S4quqDf4W6AqgLQ6qdzwsx6UJDvl2LAq6wEmJHc4UYrSNjnGpTgHlC
Fis7T4gZqgVL2h8pYPdoVXePRIj67B95HgoFnHw6/jXwuhD7fHSmQl8pYOIK
YR/bLMTaovvysikFPLfOVhQOCVH1uX2fzwYSuNFX89jDQrSmjZuVbCRhoqIm
6P2oELPlyS0Tm0nQNuyZtJgQYqmY2XaWSYKTmj69nkJgRIPG0sdsEu6QhRzF
UgID1lm2uHBmd83bZjE0AmvljTEFR0jo2dE2QfuZwI1vjUPHj5LwuW4o9ZAB
gZkxIlNxJAlmEvPyQRMCewzvDA8IZ71eu5hMMwLtoi0eWyXNelQ32GBOYLtN
8ljWSRKCU6I7eZsJdLCQLd6aOes1nRLrbiWQ4Vv+k+g0CUlT5wPrrAjc8SOa
9eYsCVqMknUHbQiMo9JbtuSQcDG0bnwGCOyUXMoS5ZLwH949nbc=
          "]]}}},
      AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
      Axes->True,
      Frame->True,
      FrameLabel->{{
         FormBox["\"time\"", TraditionalForm], None}, {
         FormBox["\"number of panels\"", TraditionalForm], 
         FormBox["\"PMVortexLinearNeumann\"", TraditionalForm]}},
      PlotRange->{-0.01, 0.5},
      PlotRangeClipping->True], {264.6, -167.4254075523465}, 
     ImageScaled[{0.5, 0.5}], {504., 311.48913032994705`}], InsetBox[
     GraphicsBox[{
       {Hue[0.67, 0.6, 0.6], AbsolutePointSize[5], 
        PointBox[{{20., 0.}, {30., 0.015000000000000124`}, {40., 0.}, {50., 
         0.}, {100., 0.016000000000000014`}, {150., 0.03100000000000014}, {
         200., 0.030999999999999694`}, {250., 0.03200000000000003}, {300., 
         0.06200000000000028}, {350., 0.07799999999999985}, {400., 
         0.07799999999999985}, {450., 0.0940000000000003}, {500., 
         0.12400000000000011`}}]}, {{}, {}, 
        {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwVj3s0lAkYxmflbNqz2klqRsvaLrRLbjVlXN8XnTSkLSIzh4Yh96nkTr7P
ztYai6mNnQojIxWR27rnm667JdoNhS4rw7GO7I4pMW7V2j+e8/vnOb/nPOsF
R70Pa9FoNK+l/E9TU24ZjcZCjxuO/z5yiwRnm7eiWhMWLnrSiDPhkeDrmB0c
5MnC7AkDBjc7Ek56U1/dkrJw98HvBIs9kaAk1l8gLbYjW3knIFsQBUX947kf
eDtQpehSPhRFw6qs1KTFRjY2yqO3PL4vhMWJQo/ZaCfkak0o52xiYVju0xPR
7YbikfGsHl4inJ4MiLOR7ka59udp/vw00I/hX1vu5oU17Qs9fk4Z0MG8FHjV
bD92VRi/cTf6AWL2MIeSxT5oKx8P9xs9Bdll5J9aBn7IMl57TqSdBYz8kaw8
mT+O9SubJq1zIPMAx7lnhoffDDEkMjgNcXZyfocoEC03XzzQ7Psz7JycTz0y
wsfu1/wBl4WzgMXl1rqGwei9JnjFsbJ8MOzfw7vJFyD9MfusMSmFqNk3GQ9y
Q3DzRSpNuus8aMzpgxZPQ5HZGFA461IA9pVVJdK1YZiRJ4x87VIEk3oeQX6u
4She0PJvNi+G4Zwgv+OZEZjTJF3FNi0B98kkzmhTJN6sMg68ayaH1yE6cQ0f
o3BM5Dsx71kKOzcxtDJdY/BeXzt//MgloD8yldQeFaJMlKaSnCyDdeKafTua
juCH0esPBS2XQWpuR189dhT3DRnZWw5egdDckoI58TGEvuzvbbTLwXBl8AOG
eSzq6+GFF5wK0F1WGh98Jxa1t27y+yLjGhh9b+zIDTuOgtZRnnF/JeiYBDR5
L4/DWQiM+cvyOlg4GdQkyeJQoirlVydUQ+yg3l0PjMerv/39y8vBGlBN5fhy
BuJRs+naQavCWthl9XsBR5iAo9GMGPGuOqCmHBY0KxNRe1nBi85l9SDcsvV5
9+VEnJ243yFqrgdHVegTLicJdQ1igxTpvy79Z3rpDifhDUpXrGvfAJszfkq+
k5SM7+Z4z6umG+Dbw3Y3TdalIC+WiMq91wihioiTDXUpeKZzzPWhuAkUJU9C
3/imYm++BXckqBmmyj9qrDSp2Fdy+o9MdgsITCpXFOSmocPMq4Eu7VZQm+Xt
qLc+gcxOydvq0VZoLltQTj84gbb11Zrutjbg7tyIDofScUCo1rktuQHrhJmt
ljQCqXym7YrkdtD/LKbO6xyB93LX57w4QEGpW3qq+gKBOr3M4+3+FFinS9zy
iggUDpH5sgAK9qhrnw7ICTzvI5MEhVJwqm96LqSKwOTCNf7jcRTMlJKuqbcJ
3Fu3AJ/mUfDMIa/3ygSBgmlZiLCbgoiEsiKOisAWZ0m/79OlfnXj4X/UBFru
U2U6P6NAb8OAxmaGwBOd72V0JQUeOkaGFI3E4o28i81qCtqfXAntXUPiB66O
hk5XgMfKFotEJokBJtM271crYMC9Y4b5JYkvW/ZXjDMU8K5tQnzoaxLt2Dyv
u8YKsJBbXx83IzHjfHlUipUC2p+7JOZYkDhctjorfNuST98HrKxJDEtRjfja
KiAsM6E7fjuJXalTe7fBku/WjwUMNon1V434G9wUIJo/F9JmT2JTUXPlKncF
0FkVWwKdlva5t1mfeCqgWNg2/RFI3PDKfl69VwH/ATrASW4=
          "]]}}},
      AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
      Axes->True,
      Frame->True,
      FrameLabel->{{
         FormBox["\"time\"", TraditionalForm], None}, {
         FormBox["\"number of panels\"", TraditionalForm], 
         FormBox["\"PMDoubletConstantDirichlet\"", TraditionalForm]}},
      PlotRange->{-0.01, 0.5},
      PlotRangeClipping->True], {264.6, -502.2762226570395}, 
     ImageScaled[{0.5, 0.5}], {504., 311.48913032994705`}], InsetBox[
     GraphicsBox[{
       {Hue[0.67, 0.6, 0.6], AbsolutePointSize[5], 
        PointBox[{{20., 0.016000000000000014`}, {30., 0.}, {40., 
         0.01499999999999968}, {50., 0.016000000000000014`}, {100., 
         0.015000000000000568`}, {150., 0.04699999999999971}, {200., 
         0.06299999999999972}, {250., 0.09299999999999997}, {300., 
         0.14100000000000001`}, {350., 0.18700000000000028`}, {400., 
         0.2030000000000003}, {450., 0.32699999999999996`}, {500., 
         0.4209999999999994}}]}, {{}, {}, 
        {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwVlnk41Ov7x4X2SE5CliOiBaXIkuW+MYMZjH1CEhpCqGyhmHSU+MwMWUqo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          "]]}}},
      AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
      Axes->True,
      Frame->True,
      FrameLabel->{{
         FormBox["\"time\"", TraditionalForm], None}, {
         FormBox["\"number of panels\"", TraditionalForm], 
         FormBox["\"PMDoubletQuadraticDirichlet\"", TraditionalForm]}},
      ImageSize->504,
      PlotRange->{-0.01, 0.5},
      PlotRangeClipping->True], {264.6, -837.1270377617325}, 
     ImageScaled[{0.5, 0.5}], {504, 311.489130329947}]}, {}},
  ContentSelectable->True,
  ImageSize->{541.2, 1016.552445314079},
  PlotRangePadding->{6, 5}]], "Output",
 ImageSize->{545, 1017},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[35]=",
 CellID->185161783]
}, Open  ]],

Cell["\<\
As expected, the low-order model is faster than the two higher-order models \
and computational time seems to follow a quadratic behavior.\
\>", "Text",
 CellID->487272008]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.5.5  PMDoubletConstantDirichlet revisited", "Section",
 CellID->154697233],

Cell["\<\
Because the lower-order model is faster, a finer discretization is \
computationally more affordable compared to the higher-order models.  Modify \
the list of discretization numbers deleting the coarser discretizations and \
adding finer discretizations.\
\>", "Text",
 CellID->150179865],

Cell[BoxData[
 RowBox[{
  RowBox[{"n", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"50", ",", "450", ",", "50"}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[36]:=",
 CellID->2905783],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"AirfoilSurfacePoints", "[", 
       RowBox[{"spline", ",", 
        RowBox[{"NDiscretizeUnitSegment", "[", 
         RowBox[{"#", ",", 
          RowBox[{"Layout", "\[Rule]", "layout"}]}], "]"}]}], "]"}], "&"}], 
     ",", "n"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"nP", "=", 
  RowBox[{"Length", "/@", 
   RowBox[{"PanelLengths", "/@", "rP"}]}]}]}], "Input",
 CellLabel->"In[37]:=",
 CellID->359900049],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "100", ",", "200", ",", "300", ",", "400", ",", "500", ",", "600", ",", 
   "700", ",", "800", ",", "900"}], "}"}]], "Output",
 ImageSize->{295, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[38]=",
 CellID->126581085]
}, Open  ]],

Cell["\<\
Repeat the convergence computations for this revised list of discretized \
airfoils.\
\>", "Text",
 CellID->35109281],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"tD1", ",", "a", ",", "aW", ",", "rC", ",", "VMagC"}], "}"}], "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Flatten", "[", 
        RowBox[{
         RowBox[{"Timing", "[", 
          RowBox[{"PMDoubletConstantDirichlet", "[", 
           RowBox[{"#", ",", "\[Alpha]"}], "]"}], "]"}], ",", "1"}], "]"}], 
       "&"}], ",", "rP"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"clD1", ",", "cmD1"}], "}"}], "=", 
  RowBox[{"Transpose", "[", 
   RowBox[{"Apply", "[", 
    RowBox[{"ForceAndMoment", ",", 
     RowBox[{"Thread", "[", 
      RowBox[{"{", 
       RowBox[{
       "\"\<Doublet\>\"", ",", "a", ",", "rP", ",", "\[Alpha]", ",", "rC"}], 
       "}"}], "]"}], ",", 
     RowBox[{"{", "1", "}"}]}], "]"}], "]"}]}]}], "Input",
 CellLabel->"In[39]:=",
 CellID->683256555],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "0.9291009298844342`", ",", "0.939640457150619`", ",", 
     "0.9447479123593601`", ",", "0.9483278612066632`", ",", 
     "0.9511004233434739`", ",", "0.9533549208968961`", ",", 
     "0.9552445058881058`", ",", "0.9568616869643614`", ",", 
     "0.9582686994774832`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.2740449052767552`"}], ",", 
     RowBox[{"-", "0.2790390455530284`"}], ",", 
     RowBox[{"-", "0.2814346983635986`"}], ",", 
     RowBox[{"-", "0.2831095035971053`"}], ",", 
     RowBox[{"-", "0.28441073112510395`"}], ",", 
     RowBox[{"-", "0.2854721734397061`"}], ",", 
     RowBox[{"-", "0.2863644870680705`"}], ",", 
     RowBox[{"-", "0.2871308951002228`"}], ",", 
     RowBox[{"-", "0.2877995148811161`"}]}], "}"}]}], "}"}]], "Output",
 ImageSize->{455, 50},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[40]=",
 CellID->673738716]
}, Open  ]],

Cell["\<\
The computational time for the finest discretization is now comparable to \
that for the higher-order models in the previous section.\
\>", "Text",
 CellID->104667073],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"plotTiming", "[", 
    RowBox[{"nP", ",", "tD1"}], "]"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", " ", "72"}]}]}], "]"}]], "Input",
 CellLabel->"In[41]:=",
 CellID->787732813],

Cell[BoxData[
 GraphicsBox[{
   {Hue[0.67, 0.6, 0.6], AbsolutePointSize[5], 
    PointBox[{{100., 0.016000000000000014`}, {200., 0.031000000000000583`}, {
     300., 0.04699999999999971}, {400., 0.07800000000000029}, {500., 
     0.13999999999999968`}, {600., 0.18700000000000028`}, {700., 
     0.3120000000000003}, {800., 0.3439999999999994}, {900., 
     0.43599999999999994`}}]}, {{}, {}, 
    {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwVznlYjAkcwPFpPJ6yucp2TFuirGPRsWzRlt9P1FrHTJK1k43U1HawOkZp
TOdkpnpfZTs0T5MOFbOhMpkuIpnJdpFSq5lXmCxyPEKKJs3aP77P59/v4oDD
3kF0Go2240v/e5LH20yjsdE3+YfXxt1CSM3Ncro8j435nCmxuUoIcVXly0Os
2bju2BvHZc+EEKC5PbtnAxuL3H1X7tcTwZqttv3F8Ww0cHF3dXESwQCjKwi0
bNzWuXegsFQElo1WooRxXxTHyl8uSEyDvyZa2qde/YabJ+iSdp8MiBbsoG/M
98fQ3OwTD9ozoSfQaHQkJwAVM8Teh5V/gszss1As5CBrhjpo1oFcOJcrO0qU
BeOIX9nr66n54PviWhOnMAQ1D9+I+c0FYD0rb0/K9TBUMQd0UrIIQrXZpyvu
H0S7CLZqDbMUItueranu/gMt3HLaRvaUgedqQXZCTASmDGRIWuQVIBt2qzXn
RqLVhGo33UgKjwSkVH4wCg3tcUguqASxAXZyedH4K6tw1GXXRaj+tNB0WQwX
7Rj7gynDGmDdCcjUCo+gnKvwMy25BJcyXdS92TFoSQdu8MZaiGpy8zpNxqIi
Vtz88P5laO6oeJcuOYrHN/RGKs/Wwe8/rW8OFcdhW5fItCSkAQqkjvLwSh7+
YtdIxHOaIOt9c3JE/TFU5sXbKcKvwpPKIzEhMj4eCeYVnQu4Bq2e2eKU1niM
7T3zHX17C+iYKk1WSwIWT19NDfS7AdNYp1fSmIjMfXcr3KNaQUFol44EJmHB
LgOCkN6EdRcdH2sfJ2EUI3w8tE8Bb9FCVhyWjFULDYz5i9uAaN/+WjGajGOj
igdN0bdgt0He1eFDKThF6FdwGv6GwMasMzkfUrA6sl55zKYDSheVWLpxBbjj
3Oim4fhOaOFP0kJGBejZ4OXkru6CJdv0p2xjU7HaYXi+i/9tqL+zghgcS8Wx
4cg9wffuQFbbtMOi+OOYtaXDZLKhB05Fj4T5Tx3HRG7Spu6Eu5BosnCeTZwQ
bfOn7zl69MKFFTM7uDQRvnurWr9vcR/MGtNj3EgQIeuax0XR+z6odLiUUKSf
hivs57P2UfdgVUAl8/OJNKwUKt9YtfXD0M+Nst2G6bjW88kjRfkA9JfxVkNO
OrZc+X6nk+Qf0NbudLUxzsDzz4WpesL7QG+MkgszM/BtwbSkIGIQmA6eHnwT
AnlWEPM1VwUdfvah4TkEbrj1qnQwTA2mgSFXTucRmK4xqXU8rIYDIaVzevIJ
TFNesMqIVsPHqAWytYUE1mpsdT/y1bA0bWLyczmB5+Oc35edUINAdp08WUfg
coux4tQaNbjqe9XIBwlkfXvTMH1cDWmz0+nP1QSWOFdsfTWphj6jVh+LIQKl
Rh80LJ0awizXfkrQEKixHLI2N6BA7GjuvuUlgZ+y+jsuMCj4sPdRn2qKwLrn
7H+fulKABxhL5+i+/MaYX2FtpIAM9j4KeiSadm1+0eBBwZJIhWX5TBJ9bz/1
IZkUeAulQYfmkphs77XX2Z+CIuJxffF8Es21/KkzHApGTlp81WtMYnCm2cTc
UAqSJGSVkxmJK2ef+vgskoKuEiUtlEFi2XiczieGArOz096Sb0jslXbub+VR
EHDeuaLbisQGG4LhkEhBVU3EhM6aRDanYVWRgIL/AEVQYBs=
      "]]}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  Frame->True,
  ImageSize->504,
  PlotRange->{-0.01, 0.5},
  PlotRangeClipping->True]], "Output",
 ImageSize->{508, 323},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[41]=",
 CellID->446543148]
}, Open  ]],

Cell["Plot the convergence of lift coefficient.", "Text",
 CellID->788915259],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"clD1TRef", ",", "gClD1T"}], "}"}], "=", 
   RowBox[{"PlotConvergence", "[", 
    RowBox[{"nP", ",", "clD1", ",", "nF", ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<reciprocal of number of panels\>\"", ",", 
        "\"\<\!\(\*Cell[TextData[Cell[BoxData[
FormBox[
SubscriptBox[\"C\", \"\[ScriptL]\"], TraditionalForm]], \"InlineFormula\"]], \
\"InlineFormula\"]\)\>\"", ",", "\"\<PMDoubletConstantDirichlet\>\""}], 
       "}"}]}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"clMin", ",", "clMax"}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gClD1T", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", " ", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[42]:=",
 CellID->112447334],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 0, 1], AbsolutePointSize[5], 
    PointBox[{{0.0011111111111111111`, 0.9582686994774832}, {0.00125, 
     0.9568616869643614}, {0.0014285714285714286`, 0.9552445058881058}, {
     0.0016666666666666668`, 0.9533549208968961}, {0.002, 
     0.9511004233434739}}]}, 
   {RGBColor[1, 0, 0], AbsolutePointSize[5], 
    PointBox[{{0.0025, 0.9483278612066632}, {0.0033333333333333335`, 
     0.9447479123593601}, {0.005, 0.939640457150619}, {0.01, 
     0.9291009298844342}}]}, {{}, {}, 
    {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwV0ns01GkYB/Bha2blJI3ZqbbdFhH2pI5DWBnfDoo9NVbjNjfjZ2Q7tV3k
EuPOksKQbtNsa82GlKyE0m4ltELZrhvRqmzFUbwZLWVz2Xf/eM5zPud5/njO
c74Wyt2iSEMWiyWk9X+v9takBDuJMMd91W6T0wRdvDvOSWtEiLSJ7jGinnnB
Hi1xFqGVe9FrLrUwe69yyFWEnCHB4qkKguHWIJ9kgQhGGmHzEPWXPryFJzeI
MH9sh1krdcXXRWUjYhH4p882JFHr/PJuZaWIYMOzYw2dInBpKBjp7RRB9Wz/
35XlBLz0A1Zv+AHgxJb1ZJQRGM48khaGBaCzT1GmKCU42usSGVIdgKa1qU2W
Jwl2LSrI6nsXgFFGov+go/fUvo3P8wmEf6I25koJQdJH1dtbjwZicFtCR+ZP
BP71YdFXnwTiV3MzJ0Uxwc3r5n599kG4fIs7av4jQa5jj8fl+CC8F9xuev8D
gUw8sMq6KQg79tlGdmkJ7j02fHvRJBim1welJ44TqK/xO9niYLy5oYyM0xAQ
AzdVakUwZho9m52PETCT560zx4OxXt08On2EYPFmi0EvjxDcsJxy6j1MEMfR
JdurQxBfdeJm8SE6X8a3edgVgi3TE7F7iwjCD58oL1shRrpnQ5TzQYLfrsYt
Z8WJ8SCUv26qgEDi63NxSaMYynUOVt1qgmIH289l8yWwWHReos0nsEliaaUh
Enx6Ybg0Ko+gRWXPGyiRYKNdW7Mwl/63e1uElkhQVcR24B4g2H+kuG2eqxS+
zQYXXucQ1Nc8my3NkcLsZXhu1T6C0YwEl46HUvCfMo4x2QTRdZysqmUyBOqW
6v2yCLRep8oadsrQErSWxf2egO3k3pveIMN39/lWrzIIsnpWLOialWGTddHS
9nSajz7zxQF+csh3RG1RpxGYenKdCzRyVCQat2xNJfhirpmtfkAOyxTjPXYp
BLG2xqu5DqHo2dDmxEom6GGJ11xTheLGgEXnX4kEz13qFxxpD8XzxP56nYrA
8Q5ndgVXAfcuea0qgcCudXtLf7gCv1sLDNziCVZHWWbb1iiQtfHKctZempek
e3L7SQXSXN9JH8cS7HxZ+LHUNwxnlmieF8cQlJ9b3+F9KAwLWsbS46IJRlb5
H7jcG4aqdSN60R66z75g8guPQULdpvpFUQQ55YII2VcM0ke25ul30Xwlj62c
pbZKHt42TJ0fUDFx0o1B+7w9voPUBw0W5r5ay8DUNnFuH/Xx0Bc1Kg8GJcr8
tHbqyk/yZjReDBq7a2JKqG9nP9L+KWQw1TwpE1LflasjEvwY6PxT3Xyp7zt6
2n/2DQPvp4ZLvKi7+882RfgzyJ8y7nKlfiZIGxwTMVjqsszfilo/br1moZiB
W5Wn94edBP909s7UUz9xa7OcoJ4oLWwXSxhkdmw00FP/u3lSppMyuDkQ2DhA
bVjdmbk6lIHUYqvrfeo52RmbHlBPn3vN/4OaLXfmxysY/OwRNd5GbTxPd6Yx
jMGQTFV7ldqkPzBGyTBQv5o+eIna9JKRgB3OwEGVubuOmlvYyK6kfsjh+FVT
876NuStUMlAdy1t5hvo/a5JONg==
      "]]}}, {{}, {}, 
    {Hue[0.67, 0.6, 0.6], Dashing[{0.01, 0.01}], LineBox[CompressedData["
1:eJwV0HtUzGkYB/AoSxjRDaMkjaNpjEStEvOdmtFUTGZ+QkS0m93prK0VbTtp
nWUXSSJsTu3oRJra2aRBbW4VRto6ldJFmaapbUS1r9tscuvsu3885zmff57L
1/WLeGbneAsLCymt//uSebFTC0nFquwj6vGsVwRfSnsc9YYsQcu3+yQTqTO9
M/00BrVgcrgsfRy1u/tgaKahXJAy773DyEuCN+ezHv1iqBFEXZdye6g7DcNn
9xvaBZwh8zot9dpPM5P3GZ4KLoeJzm2gPpTOH1IaRgQ6B+Py3BcE751dgq60
TgSvrePUdEIgExbG/XN1BlTSG5JzwwTDjt3Fu3fMRIvcbbXXEMFeJijFNtUZ
4iOhCvVzghtHDyQF5buiOnGryecZQalK+W5GNQdJvy5KVT8lCL1bvXdD40KY
hT/vmWsiyNNo70wo8kByga6i9G+CXatXbhJX8bHKX+fG6SNQyn173sqWwDqt
5JnWSJBhnXzC+qYXcmJcRoJ6CAw5k2L6RMvw9Xdh3NvdBOmsXed3X/GGwEdR
LtUTJG52sqzF55DzhLFVXQTaxXuju+4vB/dSboxvJ73HlOee7+WHKc8Grtd3
EMxNZNsvu7UC7uEvvIXt9P+nCoVy0UrIlL3WTa0Et5dY9R6+uQpXmzKt8Igg
xUqyUSYC9NpaSXkzQcNWT7YiQohp0sF+yUOaZ370b8fmBiBldl3hjUaC9sgx
WVpHAMy5Z3SSBoLIx8RqID0QXqUmrr6e4Mc7E9YUykXQvDZfk9cRhM05cj/Q
Wgw+Idm9tXR/7+KWgIdiDLELLzIPCNp2eXTnnl6N+Eemt3X3CVSKk04dIUH4
JLYp2KYjsGh2Igk2EtQMjCQ13yWw09iZnz+QIFv018XtdwiaJFGjlSnBWMqb
z/63ioDLNGW8FYbgM7Ff2TeVBEsbNJunjIag38Ff8/EWweNrxeUHK0MxNuuD
Lu4mAZ81O7gtaQ3mtxX4fbhOwPPo7DjEX4spzh0jGRUED65Wvct+vRaeWTWd
ln8S2JZMlx3VSBFVm+d4uozmY5/qq98dhlqen9LyGsEx4XqX4z7rMMqO4Oy7
QlCvT10xe3gdvO5Fc1lagtaE13usNsugVnHDf7pMkHO5rT+2TIYFnvrHrBKC
ssa+8DEbOWKlDuoLxQQuHMsMcYIcz1VTs2f9QWDURkSPNsixp4CYin4n2Fmv
6sxskqN6k33KBerBfqOc1ywHa/IKWxW1eWZsYFSrHIVxhwQnqCftV7rpuuTo
Wu50NpHaMzTHdGJAjoC6kOBA6v19T2IXjmNg8+Ki5kkRgeVHl77q8Qy2nq8T
tlGnOsRs2WLFoGj9y/ZG6lPBw6HHJzIIrPC3vEutLvnIe8NikHigJbKIuiHZ
mVSyGejtLFiJ1MyZ6K8inBhwaxbkx1G3XyowvHJm8P0Pa/wU1EYjv4njymB6
d9bOSGpzkKA0bSEDkZpfGUCt3HHQncNlcDJifbg/9ZiyJu+2B4PuycpBb+pJ
l8IyX/IZJMXrHN2pj9ecsk7zZHDPdbDYldrW2H7AzYvOb7URzaE++479/tZS
BtsO+3TaUzvZbU/Y6M1A4xsZP436PzOWYrA=
      "]]}}, {{}, {}, 
    {Hue[0.67, 0.6, 0.6], Dashing[{0.01, 0.01}], LineBox[CompressedData["
1:eJwVzn841AccB3C01CiPW1es5TeJdFNh3W7Xm/SDPeXHZe1Kcg75kXJ+3JXr
Kse3GBVDw5Kk06bakySOlPxK4rqmeDST1Rnlvt9vqa32zG3tuz8+z/t5/fN+
fxzEyYJYEyMjoy3M/Z8O88UvtohFiKzpXF67l8Yj71H3tDgRuvRLnx9hnLtL
mFSWJILbqjzVNsb0laCXz2QivLkVYvMB4xYB78/9+SIcGx61iEyiIShjG1c3
iPCT+V/T7D00spx6rN+aRmEmxUOdGU9jnOsRUHkpCqVrT60XRtMoKOQ9PWcZ
DcXeN1w3IQ2ptWrYEBED2Rn9i6YtNFy0H2o0qliIv8owO+hP4+aVhpnQOXEw
VBfPEa2hUdiuzl0hiMfF6U1tJhwam/8xDmlVJ8B0KHdrjyONgbgAdrv1HoSt
ixTetqJRNOu5j01UEhqvaK32z6fB6+0v9ujeizv1/deCTWjY3TYLLmYlQ98Y
bTbxjsLE6+pZ6uUSmPgX/lxHUahbrDMc+VWCnphA6ZlnFKxHgmvSlCmoyql4
uGmYQrrn2JSvVypcfRUWbA0FbxuK06tLRVXmdDO7g0JxtavjoZw0cDqM7B82
Mf2tRL0ZNx0J5RXia5cpDCVbvO58kg51ykBkaDWFlqaW+9uKpGgcK+GySinM
Kv29VrVOhguDZPbs4xSmLPhetpMyXOx/ktukpND9SVN4Svl+7NwekZgvo8Bv
j1j1CgfgOCk1tUmisPjfrDnuugMYMTgt0EVR0GvvZfBLMuD34y7DvW0UdnyO
0wquHK1hq6skmync5A5tP0XKMf5dif0aPwpHH+gXJVQcxHtRPsvLh9l/vyI7
3lcBjseSm9RyCuu7io77vVbA/AR3Q689BZ6D84aNZYdgoF/liBdRKF3oZZjN
O4zfhvk8z3kUJit3h5zVH0aK0KrPzpjC4O3urTGFR2D1OG+h5i2JkRtOcqVD
JhJf5n1RoSdxWvf3uYn0TAyYuti6PSVh8TUrsPZOJjwuhEmmhkjMsJtvCe2U
8BqxPKvpI+Fj987cNlWJEFZc7r52Evq0O64P+pRYPBoYtLKRxFb2YMeYUxaG
PDuIZZdImBnLvWfLsvDD0UfS0SoSvDV1MittFlwkaf4Np0i03FAUwDEb6vDL
fQH5JMi8XNfutGzwtfIulpJEmcrl5Gf92ZAMD5YbpCSKzbb3t9kSUD+751az
h4SQ/lTgaU9APpFmkDFeMmA6XOVAgD+1RBvAWFXeoFM6E+ic3pdOJpJoXGY5
4+9OQGu0oG0141829rjd9SYwaRse1hFPMq945zzYTGBRuP7wWCyJid3z5vsF
EXi8qyT0KuNLX+qKrgYTqBDznbMZe330bWWRgIBjYkHvUsYbz5HXw4QEOBmr
2ftiSCS2nR8fjiawofRgrUFMgnNenhgYS2DuaWfFfcZvjoW+at5NoK9SE1TF
WBH0fub7BAKhF+z+8Gd8cnTHgohkAhHXO9fmR5EQtK8s10gI2DcnsSIYW9XM
tVubSkDXunCcw7gqqdHdTkogoSvumwERiZiQE3UFMgIedy13qhi7ecX4GB0g
8LKvmSNjTFvxWiUZBOq1YuMAxvUzrHVP5QSkD80ffcz4P3k/VeA=
      "]]}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->None,
  Frame->True,
  FrameLabel->{
    FormBox["\"reciprocal of number of panels\"", TraditionalForm], 
    FormBox[
    "\"\\!\\(\\*Cell[TextData[Cell[BoxData[\\nFormBox[\\nSubscriptBox[\\\"C\\\
\", \\\"\[ScriptL]\\\"], TraditionalForm]], \\\"InlineFormula\\\"]], \
\\\"InlineFormula\\\"]\\)\"", TraditionalForm], 
    FormBox["\"PMDoubletConstantDirichlet\"", TraditionalForm]},
  ImageSize->504,
  PlotRange->{0.8, 1.},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 337},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[43]=",
 CellID->42484612]
}, Open  ]],

Cell["\<\
Include the result of the revised convergence study for \
PMDoubletConstantDirichlet in the list of reference lift coefficients.\
\>", "Text",
 CellID->347645104],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NumberForm", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"clV2Ref", ",", "clD1Ref", ",", "clD1TRef", ",", "clD3Ref"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "3"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[44]:=",
 CellID->132745640],

Cell[BoxData[
 TagBox[
  RowBox[{"{", 
   RowBox[{
    InterpretationBox["\<\"0.991\"\>",
     0.9911654719692023,
     AutoDelete->True], ",", 
    InterpretationBox["\<\"0.960\"\>",
     0.960371085306418,
     AutoDelete->True], ",", 
    InterpretationBox["\<\"0.967\"\>",
     0.9669251493143558,
     AutoDelete->True], ",", 
    InterpretationBox["\<\"0.989\"\>",
     0.9888211492477125,
     AutoDelete->True]}], "}"}],
  NumberForm[#, {4, 3}]& ]], "Output",
 ImageSize->{191, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[44]//NumberForm=",
 CellID->72521381]
}, Open  ]],

Cell["\<\
Although the revised reference lift coefficient increased slightly, it is \
still noticeably different from the values for the higher-order models for \
this example.\
\>", "Text",
 CellID->34094847]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["PanelMethods",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/PanelMethods"]], "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["3.4 Flow Visualization",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/FlowVisualization"]], "RelatedTutorials",
 CellID->806871991],

Cell[TextData[ButtonBox["3.6 Aerodynamic Coefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicCoefficients"]], "RelatedTutorials",
 CellID->334470709],

Cell[TextData[ButtonBox["Overview",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]], "RelatedTutorials",
 CellID->2248688]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"3.5  Convergence And Timing - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 3, 47.3532000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "None", "summary" -> 
    "The example used in this chapter is the Eppler-205 airfoil at an angle \
of attack of 6 degrees. Lift coefficient and pitching-moment coefficient for \
the airfoil are used as measures of convergence. These coefficients are \
plotted versus the reciprocal of the number of panels describing the \
discretized airfoil for constant spacing. If this curve has a linear region \
for large discretization numbers, it is tempting to extrapolate the line to \
the coefficient axis to obtain a reference value for the coefficient. \
Computational times for models are plotted versus number of panels describing \
the discretized airfoil. Since implementation of models result in a full \
system of linear algebraic equations to be solved, this curve is expected to \
be quadratic. Load the Aerodynamics packages.", "synonyms" -> {}, "title" -> 
    "3.5  Convergence And Timing", "type" -> "Tutorial", "uri" -> 
    "Aerodynamics/tutorial/ConvergenceAndTiming"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[620, 21, 78, 1, 70, "TutorialColorBar"],
Cell[701, 24, 191, 5, 70, "LinkTrail"],
Cell[895, 31, 2945, 67, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3865, 102, 64, 1, 70, "Title",
 CellID->509267359],
Cell[3932, 105, 569, 9, 70, "Text",
 CellID->163863731],
Cell[4504, 116, 292, 6, 70, "Text",
 CellID->133626429],
Cell[4799, 124, 125, 6, 70, "Text",
 CellID->836781195],
Cell[4927, 132, 101, 3, 70, "Input",
 CellID->2058623809],
Cell[CellGroupData[{
Cell[5053, 139, 63, 1, 70, "Section",
 CellID->627174914],
Cell[5119, 142, 320, 10, 70, "Text",
 CellID->24014087],
Cell[5442, 154, 217, 8, 70, "Input",
 CellID->543466123],
Cell[5662, 164, 227, 8, 70, "Text",
 CellID->193395042],
Cell[CellGroupData[{
Cell[5914, 176, 135, 4, 70, "Input",
 CellID->199940495],
Cell[6052, 182, 375, 14, 36, "Output",
 CellID->369970053]
}, Open  ]],
Cell[6442, 199, 82, 1, 70, "Text",
 CellID->107920476],
Cell[CellGroupData[{
Cell[6549, 204, 552, 18, 65, "Input",
 CellID->88849844],
Cell[7104, 224, 5843, 103, 201, "Output",
 CellID->450629884]
}, Open  ]],
Cell[12962, 330, 120, 3, 70, "Text",
 CellID->797728430],
Cell[13085, 335, 369, 11, 70, "Input",
 CellID->124450942],
Cell[13457, 348, 218, 5, 70, "Text",
 CellID->728508869],
Cell[CellGroupData[{
Cell[13700, 357, 529, 16, 70, "Input",
 CellID->207596273],
Cell[14232, 375, 345, 10, 36, "Output",
 CellID->72248229]
}, Open  ]],
Cell[14592, 388, 280, 9, 70, "Text",
 CellID->329689094],
Cell[14875, 399, 109, 4, 70, "Input",
 CellID->3218154]
}, Open  ]],
Cell[CellGroupData[{
Cell[15021, 408, 58, 1, 70, "Section",
 CellID->186435948],
Cell[15082, 411, 418, 9, 70, "Text",
 CellID->109828650],
Cell[15503, 422, 62, 1, 70, "Text",
 CellID->64575450],
Cell[15568, 425, 147, 5, 70, "Input",
 CellID->63735315],
Cell[CellGroupData[{
Cell[15740, 434, 63, 1, 70, "Subsection",
 CellID->276154793],
Cell[15806, 437, 840, 19, 70, "Text",
 CellID->114624859],
Cell[16649, 458, 973, 29, 70, "Input",
 CellID->261387464]
}, Open  ]],
Cell[CellGroupData[{
Cell[17659, 492, 68, 1, 70, "Subsection",
 CellID->502776918],
Cell[17730, 495, 490, 12, 70, "Text",
 CellID->826207730],
Cell[18223, 509, 968, 29, 70, "Input",
 CellID->30890011]
}, Open  ]],
Cell[CellGroupData[{
Cell[19228, 543, 69, 1, 70, "Subsection",
 CellID->118691814],
Cell[19300, 546, 504, 12, 70, "Text",
 CellID->60685572],
Cell[19807, 560, 1021, 31, 70, "Input",
 CellID->135262774]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[20877, 597, 68, 1, 70, "Section",
 CellID->227129737],
Cell[20948, 600, 429, 10, 70, "Text",
 CellID->74633868],
Cell[21380, 612, 415, 7, 70, "Text",
 CellID->98070734],
Cell[CellGroupData[{
Cell[21820, 623, 58, 1, 70, "Subsection",
 CellID->869225247],
Cell[21881, 626, 170, 4, 70, "Text",
 CellID->39606871],
Cell[22054, 632, 594, 21, 70, "Input",
 CellID->102708401],
Cell[22651, 655, 152, 4, 70, "Text",
 CellID->2960992],
Cell[CellGroupData[{
Cell[22828, 663, 2357, 66, 200, "Input",
 CellID->147688655],
Cell[25188, 731, 17300, 291, 934, "Output",
 CellID->160810047]
}, Open  ]],
Cell[42503, 1025, 174, 4, 70, "Text",
 CellID->477679825],
Cell[CellGroupData[{
Cell[42702, 1033, 256, 8, 70, "Input",
 CellID->206162233],
Cell[42961, 1043, 517, 18, 50, "Output",
 CellID->636775393]
}, Open  ]],
Cell[43493, 1064, 287, 6, 70, "Text",
 CellID->699005627]
}, Open  ]],
Cell[CellGroupData[{
Cell[43817, 1075, 69, 1, 70, "Subsection",
 CellID->870013475],
Cell[43889, 1078, 182, 4, 70, "Text",
 CellID->1827792],
Cell[44074, 1084, 594, 21, 70, "Input",
 CellID->768607065],
Cell[44671, 1107, 165, 4, 70, "Text",
 CellID->334610627],
Cell[CellGroupData[{
Cell[44861, 1115, 2330, 66, 200, "Input",
 CellID->738949569],
Cell[47194, 1183, 16767, 282, 934, "Output",
 CellID->957810550]
}, Open  ]],
Cell[63976, 1468, 174, 4, 70, "Text",
 CellID->612014715],
Cell[CellGroupData[{
Cell[64175, 1476, 256, 8, 70, "Input",
 CellID->271918122],
Cell[64434, 1486, 525, 18, 50, "Output",
 CellID->539123902]
}, Open  ]],
Cell[64974, 1507, 322, 6, 70, "Text",
 CellID->116669446]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[65345, 1519, 61, 1, 70, "Section",
 CellID->32786706],
Cell[65409, 1522, 158, 4, 70, "Text",
 CellID->23262114],
Cell[65570, 1528, 381, 14, 70, "Input",
 CellID->120331341],
Cell[65954, 1544, 125, 4, 70, "Text",
 CellID->677306114],
Cell[66082, 1550, 1626, 46, 70, "Input",
 CellID->53565918],
Cell[67711, 1598, 125, 4, 70, "Text",
 CellID->374596295],
Cell[CellGroupData[{
Cell[67861, 1606, 1755, 53, 70, "Input",
 CellID->94940072],
Cell[69619, 1661, 12011, 213, 1038, "Output",
 CellID->185161783]
}, Open  ]],
Cell[81645, 1877, 182, 4, 70, "Text",
 CellID->487272008]
}, Open  ]],
Cell[CellGroupData[{
Cell[81864, 1886, 82, 1, 70, "Section",
 CellID->154697233],
Cell[81949, 1889, 299, 6, 70, "Text",
 CellID->150179865],
Cell[82251, 1897, 181, 6, 70, "Input",
 CellID->2905783],
Cell[CellGroupData[{
Cell[82457, 1907, 530, 16, 70, "Input",
 CellID->359900049],
Cell[82990, 1925, 300, 9, 36, "Output",
 CellID->126581085]
}, Open  ]],
Cell[83305, 1937, 127, 4, 70, "Text",
 CellID->35109281],
Cell[CellGroupData[{
Cell[83457, 1945, 940, 28, 70, "Input",
 CellID->683256555],
Cell[84400, 1975, 985, 25, 71, "Output",
 CellID->673738716]
}, Open  ]],
Cell[85400, 2003, 177, 4, 70, "Text",
 CellID->104667073],
Cell[CellGroupData[{
Cell[85602, 2011, 254, 8, 70, "Input",
 CellID->787732813],
Cell[85859, 2021, 2463, 48, 344, "Output",
 CellID->446543148]
}, Open  ]],
Cell[88337, 2072, 77, 1, 70, "Text",
 CellID->788915259],
Cell[CellGroupData[{
Cell[88439, 2077, 883, 25, 70, "Input",
 CellID->112447334],
Cell[89325, 2104, 6354, 116, 358, "Output",
 CellID->42484612]
}, Open  ]],
Cell[95694, 2223, 172, 4, 70, "Text",
 CellID->347645104],
Cell[CellGroupData[{
Cell[95891, 2231, 278, 9, 70, "Input",
 CellID->132745640],
Cell[96172, 2242, 610, 21, 50, "Output",
 CellID->72521381]
}, Open  ]],
Cell[96797, 2266, 209, 5, 70, "Text",
 CellID->34094847]
}, Open  ]],
Cell[CellGroupData[{
Cell[97043, 2276, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[97111, 2279, 158, 3, 70, "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[CellGroupData[{
Cell[97306, 2287, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[97381, 2290, 177, 4, 70, "RelatedTutorials",
 CellID->806871991],
Cell[97561, 2296, 189, 4, 70, "RelatedTutorials",
 CellID->334470709],
Cell[97753, 2302, 164, 4, 70, "RelatedTutorials",
 CellID->2248688]
}, Open  ]]
}, Open  ]],
Cell[97944, 2310, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

