(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    361370,       6529]
NotebookOptionsPosition[    355517,       6339]
NotebookOutlinePosition[    357555,       6391]
CellTagsIndexPosition[    357512,       6388]
WindowTitle->3.4  Flow Visualization - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"3.3 Implementation\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/Implementation"], "\<\"3.5 Convergence \
and Timing\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/ConvergenceAndTiming"], \
"\<\"Overview\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Panel Methods\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/PanelMethods"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/tutorial/FlowVisualization\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/FlowVisualization"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/FlowVisualization.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/FlowVisualization", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["3.4  Flow Visualization", "Title",
 CellID->509267359],

Cell[TextData[{
 "Flow visualization provides a useful plausibility check for numerical \
models in fluid dynamics.  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s ability to work seamlessly with symbols and numbers is used to write \
functions describing the flow field for uniform potential flow over an \
airfoil."
}], "Text",
 CellID->823686949],

Cell["Load the Aerodynamics packages.", "Text",
 CellID->34146804],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->35199140],

Cell[TextData[{
 " Specify an arbitrary field point.  A single vector may be written in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation."
}], "Text",
 CellID->249040680],

Cell[BoxData[
 RowBox[{
  RowBox[{"rF", "=", 
   RowBox[{"{", 
    RowBox[{"x", ",", "z"}], "}"}]}], ";"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->48493306],

Cell["\<\
A member of the NACA four-digit family of airfoils is selected as the example \
for this section for two reasons.  First, by changing the identification \
number of the airfoil, calculations can be made for many airfoils including \
reference airfoils without thickness (camber lines).  Second, discretization \
is based directly on the equations defining the airfoil instead of a spline \
fit to data in a file.\
\>", "Text",
 CellID->972502991],

Cell[TextData[{
 "Any of the three panel methods in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " can be used to obtain a solution for a member of the family of NACA \
four-digit airfoils at an angle of attack.  The model using vortex panels of \
linearly varying strength, and Neumann boundary conditions is chosen because \
these airfoils have an open trailing edge.  Using ",
 Cell[BoxData[
  ButtonBox["PMVortexLinearNeumann",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/PMVortexLinearNeumann"]], 
  "InlineFormula"],
 " avoids the minor inconvenience of altering the airfoil geometry to close \
the trailing edge that is required when using the doublet models."
}], "Text",
 CellID->70657603],

Cell[CellGroupData[{

Cell["3.4.1 Solution of the model", "Section",
 CellID->314388262],

Cell["\<\
Specify a four-digit identification number for the airfoil, angle of attack \
and discretization parameters.\
\>", "Text",
 CellID->75137487],

Cell[BoxData[{
 RowBox[{
  RowBox[{"id", "=", "4412"}], ";", 
  RowBox[{"\[Alpha]", "=", 
   RowBox[{"5.", "Degree"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"n", "=", "50"}], ";", " ", 
  RowBox[{"layout", "=", "HalfCosineSpacing"}], ";"}]}], "Input",
 CellLabel->"In[2]:=",
 CellID->108183604],

Cell["\<\
Compute the location of panel endpoints for the discretized airfoil and plot \
the discretized airfoil.\
\>", "Text",
 CellID->731882235],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"NACA4DigitAirfoil", "[", 
     RowBox[{"id", ",", 
      RowBox[{"NDiscretizeUnitSegment", "[", 
       RowBox[{"n", ",", 
        RowBox[{"Layout", "\[Rule]", "layout"}]}], "]"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"gA", "=", 
  RowBox[{"PlotAirfoil", "[", "rP", "]"}]}], "\[IndentingNewLine]"}], "Input",
 CellLabel->"In[4]:=",
 CellID->401362744],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3c9rFHkax/Fe9i/ZfyMgcxAP4iFkQTyIF4VGL+NFL3qJl92LQpNcEpDJ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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->943918884]
}, Open  ]],

Cell["Solve PMVortexLinearNeumann for the example specified.", "Text",
 CellID->19157149],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "rC", ",", "VMagC"}], "}"}], "=", 
   RowBox[{"PMVortexLinearNeumann", "[", 
    RowBox[{"rP", ",", "\[Alpha]"}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[6]:=",
 CellID->450780366]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.4.2 Stream function", "Section",
 CellID->1708000],

Cell["\<\
If the stream function is single valued, it can be used in a straightforward \
manner to plot streamlines of the flow.  Such plots can be useful for viewing \
the overall flow field, flow near the airfoil or flow in the vicinity of the \
trailing edge.  Contours of constant value for the stream function are \
computed and plotted for the computed flow field including the interior of \
the airfoil.  The absence of streamlines interior to the airfoil is \
indicative of a plausible solution for a model using only vortex panels.\
\>", "Text",
 CellID->35129489],

Cell["\<\
Use the stream-function influence coefficients and the strength parameters \
from the model solution to compute the stream function for the flow at a \
field point.\
\>", "Text",
 CellID->48905279],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Psi]\[Infinity]", "=", 
   RowBox[{"UniformFlowPsi", "[", 
    RowBox[{"rF", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Psi]A", ",", "ic\[Psi]B"}], "}"}], "=", 
   RowBox[{"ICPsiVortexLinear", "[", 
    RowBox[{"rF", ",", "rP"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Psi]A", ",", "\[Psi]B"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ic\[Psi]A", ".", "a"}], ",", 
     RowBox[{"ic\[Psi]B", ".", "b"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Psi]", "=", 
   RowBox[{"\[Psi]\[Infinity]", "+", "\[Psi]A", "+", "\[Psi]B"}]}], 
  ";"}]}], "Input",
 CellLabel->"In[7]:=",
 CellID->17324249],

Cell["Display streamlines describing flow over the airfoil.", "Text",
 CellID->275199393],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xMin", ",", "xMax", ",", "zMin", ",", "zMax"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.5"}], ",", "1.5", ",", 
     RowBox[{"-", "0.5"}], ",", "0.5"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Psi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"Contours", "\[Rule]", "16"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "3"}], ",", 
     RowBox[{"PerformanceGoal", "\[Rule]", "\"\<Speed\>\""}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"g\[Psi]", ",", "gA", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7.9", " ", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[11]:=",
 CellID->666894011],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJx0XQd0VkXTpokISBNBqqCoIEXp3UUERBCwUQUB6b0IUqRXkQ5SpYTQOwHp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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}}, {{}, 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{4457, 5494, 6205, 6204, 6203, 5030, 5691, 4456, 5326, 6582, 
          4939, 6581, 4940, 5253, 5252, 4303, 4941, 4857, 5681, 5682, 5683, 
          5423, 4443, 6187, 5680, 6735, 4770, 5882, 5636, 5322, 6808, 4673, 
          5424, 4858, 5469, 5470, 5146, 4291, 5425, 5239, 5254, 4600}]},
        "0.528`"],
       Annotation[#, 0.528, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0DlP1FEUhvEDAwXSwCAzww4VmJBAkB1ZBkQ2ZbE0YAIWNgYaGjR8AggV
lBBJNEALWKgtgp/Azp6CBEqMYPxN8eSdc85zz//ObVhaebmcFxE7aEpGrKYi
fpdGXJdH3KA7HdGDFv2mTMQj7D6M2EMj/6PZPk5KIl5VRHyR9/If7nBZFrFp
zwb2+dfO9ckzmbAzH39wi7f2fNLv9LsD7fjLbdVLudO2egdL+KZelN/lG9ns
Xl3yjj+W2yMzvv9Z3usPyzZ3eIx+d6qy88xdpznj/AnkowAJjPDbuR0Y4Ffz
f/Bn+JPmUyhCMR7gKb+T24VBfg3/nD/Lf27+AiVIohSj/G5uD4b4tfwL/hx/
2nwGKWSQxjN+L7cPWX4d/yf/ifqXMwd6a7xD+T73H+SAWYHckq85C5jHlV6i
MqLejnfcWXU290YYwZF62HtmMYRC9aJzSTlu/sGbH9t5glOsq1P6tbJc1sm0
/Gr3f6WpRwg=
          "]]},
        "0.462`"],
       Annotation[#, 0.462, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0LtOlFEUxfE9JAIKCHNjhlEKCsbEB6CwAEtjAWGogI4KNMojEBJ6EpiS
ylETnkCNVBgvFICvIDDcsYEOAvxO8c/aZ+219/m+MzAzX3ufiYgl1LMR4+WI
1UpESz5iui9i2HkEfwsRtVzEtnqC7tAW+k1mTPYLHaVf6a7sy15zGEaGXyu5
gBbpZ3M5fh4FbMmf88+QTR7N4y0eozvB70G/b/wj/5Sepj6vi3biDdrQns7J
xxO53/IVesLv4LXSB5hL34QMHvIfoU/ul3yZHqddvDvcYtb5mt6kHWhDSe6n
/JHeYdrLa/i/hrruHYf887o3+eT8EQu8AzPL3q9JF+Wfq6fUH+xp4Mr8iOwl
fW32HW+wGFFFTp3FlPq//g/1ZppxvsQr+Qt+1c5nmLR3TX/GvgN3NXEud5Hm
8V1vAytm/untYw8v5O8BEGNHSw==
          "]]},
        "0.396`"],
       Annotation[#, 0.396, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0TtPk3EUx/HDK6BQ6MXESHHgouhOLyC0TYiw4OCACXKZHGQqu+gmDFwm
6oCvQsqkvhAGDEXBBNqAdeTT4ZvfOaff8/yff5/c6sbi+56I+IDFvoiHgxGv
5FAm4hHO+yNms9K8icZAxAnOzF/re9UJDKtzeKcu85t+f56OuJB1s0M89rxh
5FDlVNC208KpM2upiHU7azjif8WN/bz+Ws7xe3m7vD28xTf9sjyWJV6bV5At
WZSfH0Tsq1N2D2STW5AjzhzFZDIi6ZzviYiXnGk7L7rvzfuD3yjyx7jjyPMH
+D/48/wZ7iwueX9xhRL/CfcpCvxB/k/+Ar/MrXTvw2vhBlP8Ce6dXJIpfhr/
9FXutjvU1Wn7X+StnWn5jNuRb2SGn8V/fa37ffwfO+a/5Ka+2L038p7VMFvx
Dfb1W/pP+IiO+T2saEl1
          "]]},
        "0.33`"],
       Annotation[#, 0.33, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0D1Pk2EUh/HTD0DSVyyEQZwUExdwkNHZiTjAgIOJdTEYoiIqioOJg0Vd
sK0JbSO06uKGqx9DNjWWgWAZEIqAb79nuHL9z7nPufv0Hr56c2ImFRFP8SEd
UcpHXEctF3GyP2IYpzBUiLiU4WLErHpDLsufuTUQ0cYqRu2tcTW5A51sxLkT
Ed/5on6f+Wf2y7iCdfU0f+SG+5pYwZ66zgX33bJ3B7ex5Z7T5s9gBA2/cdns
WfmG/FV+4lu/8HP7a8mcndVkh+f4m7N9M0tycTAinczKGT7Q72FefRct3MN9
/NJ/wQN2smZfyjk+0j/EgvoB2niIRzjWr/CgnbzZqlzgP/q/8Vi9iGveqISu
vOw7/zrrqDfxyf9K6Qf+6b9yviOPeZu33uQ8v+OK/q5+lX/yBf5h9zVP2qvx
tvqNO3vOV9T7XOcDHk/udd7kKfMN7qrfy/8B8rVWwQ==
          "]]},
        "0.264`"],
       Annotation[#, 0.264, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0s1LVGEUgPHTKgMza9RMwYiZ2allaSpU4tbUIBq1LxctUitn1KLamYWC
q9q5EUwXqRSY4Cot6z+wZVHR0rRVUps+6DeLh+ecc88573vvzJFrufPZXRGx
iJ79EX8TEYMVEf/4D1KlEdPyJL8riSjSsy3eQm1ZRA261U4ejGjECTSgHm/1
f9O3ia/6Sg9FlCCBanm6POK4vu8HIup4m9+YaeKPnj/hvXofc8Ydjunp4i35
S/Vm91viJn5trlF9Qn5YPs5V3Otug+J94lucQxZF8mEewRA+4xPW1NeLIx45
64V7P8cC0nbfccZtdNk5zCP4oX+Ic1gxl+UlnjfzDClzN+y9iQEU6s+Y7xMX
iK/zHu7n3dzt/Za52Z4Z8+XiVu70Pc5hXDyB9+qbZjb4qn3LvuUVPsq1+C1+
aMcY1rzLKl5hzP6fns3q+cX37LiPuyiz66kzku48w3OYxQd9o858gAH7+tGp
NsUdnNaTyv8X7P9i9oIz2uVtzjjLrfIdfdPyyvw91VbkF/kyLmESBWqn3eMM
TuV/E7ta8r8LVzk7qec/ydhdbg==
          "]]},
        "0.198`"],
       Annotation[#, 0.198, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0t9PzXEcx/GPOzfox6lEG2FxY37ccHliSlwxhSTTtNmybCgLYyoXIlPM
xvy4MKmYTX6lYrIxv65sknNOp3NuMJsNf4HH9+K51/v3+/357lvacHBr84wQ
QhoPckOomxtCPXbjPL+iIIRuunpeCPPzQ1gTC2Ezv4R9qjiE0rwQ3qhdRBvl
9uFGUQjX8Vb8jv5NhSFUoV1fSr5GLl9/NY3RWroL27ETO/ArJ4R1atfjg5oz
Yp9oJ/1Iy8XjeM9e6bZOs0/b1UFX8N+Jf2dfdEMvenCbP6Wn3q3T/CwymMl/
7S0tesbpEZpWt0f8BL+I30aP4xjO4YL8bzcW2nVU/oDdTThrxyqxz2JLzK5U
+9Oc3uj7sfvZJXSAFkdvpn/M+hvN07OYfjH3H52g1WIj6mtogibRrq/S7Aoc
tq81ugEtuOY7X8UTNY+RMSOpLoUE0phCrb2T9BuG1D1CQm2HvQNig+jHfdxD
qz1f5SaR45ZL3noZy/hZfXF2Hfu52hEMYwyj6NNbYN8G98VoG/+VGfv54zSD
Lvun6Zj37aWjtIG+oLl0WP4Znkb/ihkT0XdClfkbo+/szbPUbZMvkz/ET9jz
w11X2H1q7mIte3n03WhKvpudpCfdclPvLTz0npT8FJKYRho95ua6Jw+zkYM5
GFLf5JZydkZNnGbpQrsXYIv5ZfqXYpDfRZvte6mvz77/1WaJjQ==
          "]]},
        "0.132`"],
       Annotation[#, 0.132, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV00lsjWEUgOGvFhJKYqop0fTeJqLG0BhWVsaVthIbQ7SICFrEUDOtEC3V
ECtDjImgU0LRqlqZF0JiWBh6W+lCtahqKqSef/H2/c/wnfPf797G8gpy8pNC
CH39WToshH2jQhg1NIS9PJrnpoQwB6WDQ7gzMoTkISHUcn9OcDMWjg5hAVbK
vXYm03OZ/o3OFYp3YoRcx6AQyu1o5y/O5epv4TweY18qnuIJrst16n2FCeL7
4vHcwJ/lPqHR82JzH/JM++bJzUeHfRP5uz0n+AdPimL5hbwAy0aEcHN4CBWo
85716NaXUPvN98Sn9KebP93sdeJN4jq74nKZcn/1TuOY+KRaXDzOO2bgp/md
+IV/8t38G12oN2O7/hTn2uy67A56zLnEV3EFp8VL7Ax6ktBr5ldn+nhuVdtl
Zqn4mfwxfs615p5z5jymqsei78C+OG9ybjMKUOwzF6FKrRJvvUM110X3pr41
mm/GNp4krpefbd5Hnim+KH9U/QLPEH+Q70UrssWV3qNEvYp/OVfBXVzD5fLV
nKOvR/9Y+QaeLC40b7f6HjTKrZV7wDvkE7xe3MxNaPd5O6I7wTe0odrniHM6
YrhlTxrf5j/29CDDrDLnT2CLPZuju5JLM/u4XI34ffT7cOai5+WczwV4bNYj
7Ne3ivtyHq/Bamwwp1DfRh5n3g31WWatFR9XL8MK93xYLcvcF77nbL7mu/ij
54jeAeIz4oF8lq/IFzt3CAdQhINYJP+SszjVnDH45MwUM/rJNUT/o9G92deE
UnEJLjnzxty3SHjXZrVkfXej3dwS5aI7UH+HXPX/YLelyw==
          "]]},
        "0.066`"],
       Annotation[#, 0.066, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV01dsiFEYxvHjAhElZtVeHUIiLmomQmxFQq0apWm1qK1EYtZqpalNQiLh
gohuLcFFS6wgQYiiXKL2ukG48fsu/n3e93nHOedr2ztzTerqJiGEYX5MbBtC
644hxMaF0BEd0NAuhIEdQmiqViNvIa+ml3EJNfwzZlrqicF2bEMxmut93zmE
h+1DmKJ3Kvbq36W/gE6LDeGZWnKXEA7JC/kHabp8ERZinXoufwg/Xn5Mvlpc
TUvtn84ro2m4YN88WqG/ElWokye4SzwOyk+bbcYba64pHcDvj+NqRWrjxWPF
Y/BK/Bo7nbULXc300nPeXGfv6oJ6b3qB50hWq9SXo6+KlqEC2fJyusWuz218
V3mefD0KO4XwmBbQR3SCN4zHOHyw76i9BeJP4r74pm+z+TpxploWLvIa7D0u
fkkT3CsR8bjqrkn0Cv3n/EHmrolznVVibylGYLodG+zNVJ8hPs/bg5O8vTQO
efw95rrx4vUt8Y3S5fOi3xcdwMug8+WLo3erJ/C6RzvkteJ9tKe8vzhHfYl8
gTybbubdpInyLPHGqE6T5Gf5X51/zBs/8kbxvssTxb+8K4n+lP9Atff19OYc
MzXiHnr7qf/W9weF/AxeAX3qe31WGy3/Qldgk3NzaYn6HSxXu00b9Tbaex0r
5HV0Jb1BJ9E0ffl0cnQuL4Veolvt2mbndtziveG9w9vojmoj+dl0hLcNxzk9
9WpN3L3IfZ85N8buOfJ8fUv1L8Mqfe3479XvijeYXYf1WIsHevNoCR2mL9b8
PfH96H8Nu+3YR8c5Ywc9Ed0x+rbycvETe19G/39mG+gLDBYfiX6P6kfpYDub
82r17jCXj0Z9qc6diVlI0TuXzsFs9FUvjr5B9K3U+tBTUY/ZNPxT+4tSlKEE
FSjHK+cdct5hFGG/PIZ/gNb7Gx1Cu9r1ms6mrexrjaHO+A/+QLIV
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV00lQj2EcwPHHlSkt9pNOJvuQA4qDYkaYoTIqM6mQLBUnRlkORgtDHdrc
YobEONhn7LINM6TsRMkhB4PKdvL5H77zfX7L83ue933//4SC0lUlw0II/9Ay
OoT8USF0xIbwdHwIjfEhRI0LIRqpclHqaVwyJoRSVMofRJdc+YQQ6seG0IB2
uYlxIRyQWxTZizR0mt2F4ebWmlWHSeLumBAy7PvIyeIUXNazX+4q7+MrPF9+
Hi5ZTza7zLm7zCjlRPFF+e3WC+V28BS5EXKF6DV7pDvFoEEtT886LFZrNf80
2rDL/Cd6f3j+n/iOQQygT+8f/osh/MYvtJiXZd9qDOnJ5Ax0mrPBc2/EemxC
EU7qv4Vk97vJZ/Xd5hTxHT6FLfq2ohh3xZv5i74Svi/exg+4lFv5HhbY387P
9CU4v8pdpovLPOcMrhZftJ6tNk1cbz2VZ4mPW6/gRnGTdRPHeVepZsZyFjLR
oadZLcGsa/yej3GhPRmRZ0e8vlW8EvnyBXhnXen9vuUqzjbntXd3XvyKL/Ab
zuYcRDv3veeo0fch8u3Ea+Vzscf+vdiNCpQjR99H+z+hG73owSb9xYi1v8+c
w/o+c4y4KPLbcNfr9t/AkHyP+iAn8Rx805PI/XI7eSnSMSC/nJdhv+d/6Ds8
inwTLDHzsXye/EzxXGedi/zmrJucc8S9jmKNOBttaune0RnOFeegTr0WefrL
/NeazXipXqX2givEzWrH0Kle5cznXM1dnMRf3eFQ5LuZXcP94hPW/wFNzJph

          "]]},
        RowBox[{"-", "0.066`"}]],
       Annotation[#, -0.066, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0DtPlFEQgOGBQiMsEUUFNiAhwR8gAv4AC64q4bZLBUJLdhHQGG+NDXQu
JISLUKiVgWiltASNiQQTYKmMl5JW7TSCzxZv3jMzZ2bO99WPZHsyRRHxEh1n
Iv5URIwlI/7yTyTPRyyLq/nduYiT7vyojPiOZJU82uVSFyLS6MMA+rGpdkLt
m7tfUSWuRJvcitmj5q7yEp5hRLzMH3FYHlEhzjpncFd/jqf4Kd/h3+68r454
JN7ix/yB7/NDDOl/wE/wxd0ycdo5hdv6J5DBOLL45c6Kd69io/C9eItt7715
NqJU/2d3ZvyHHS4Rt5p1Qy1t7yAGcCxOca1aH/8T9/IR93MXb/sP13nN3nVM
m3lsT6dcq7kd3KD/ltwlcYhzhXfo60YLrmJYbkfvJ+fXeIPLaMS+XXlccW5C
MyZ9zxRemL2r9pz3eM33LHjHK57jWdTZuaXnIs/bUe58urDX/RbU+E/TWJJb
xALqxKf4mncl7Ek4l6IEeXMO5IrV1s3bt2MPebl7ak1mNhfeiwPxf3FPXTU=

          "]]},
        RowBox[{"-", "0.132`"}]],
       Annotation[#, -0.132, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwN0D1Pk2EUh/GDrdDiQF/oi2xu8mIiurSyg4uo9APQuCuziQvvcQBGB4J8
AEgQJlBkImITYQBa/AowAQtMhN9w5X+fc1/3yXmeJx8mxj61RcQX1AoRu5mI
zu6I23xEWV3Cy5ze44hH6EAaKQzhFz/Nv+EXuQW84D90144HSCKBV/jJT/Ev
+N3cPAb5h8WII/zDQCmiH1X+Dr+Df87PcbN4zm/wpmQbZ1J+7Yno82ZJ/lcv
yl71gnwq6+WIrawd1FVvK+qP8kRvzvlYzspruSzzqLnfVL+XP+SYvJMt85qY
5swghyyG3XXZL4MzO1Ts1pJJ/Q3fkJCn3rXcf9bfN++Z71rhreI7LjnfzLqS
8+7fOI9ixJvXaPHPeU15IcfNbdff9p+Szu+c36Jh/rp6DX/VB/iDPTN/o26P
exrEQl4=
          "]]},
        RowBox[{"-", "0.198`"}]],
       Annotation[#, -0.198, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0Esv3GEUx/HTBWVHi7rUoouuyKC7bto3YMM7QGw1alExzLjWJF0yYSMh
EhuJhYitS6Jl0KnVjEsvVvoSmqB8ZvHN9znn/J7z/Gde9X7oHngSERNYrYp4
/TxiriFi71lEK+/zCZ9i+EVET2NEVuaf89vqiCH1J+f+uogl57xesj7iOy+6
0yq7wC28IzNo31Et3Lnz3lpNxD3n9BNyx1xh/hTv7Ai7mtzrtPuj2a5zzmzD
vQW9hF6l+oS7uEy/HO12tSHt/jhSmMQE3tublXnpzr66mfvUvVj3LXd697jF
A/5jRObA26Mc9iY54/15vyOLb2Zjel85xYfcxjd2jnO3b0vzSum7/BfLnOCc
3JR+wbxgTxGb7lyoz1HEtPmxXLt8nmfUl6W57AW25H+pf+IKn81/yHXIn/Gs
+rf+pewVtuWv1X/wRZ0x/6v3Rv4R0QZWPA==
          "]]},
        RowBox[{"-", "0.264`"}]],
       Annotation[#, -0.264, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVz7tP1FEQxfFZ2EWjCUEQERIQBKGg0sJQUCB2BuTRbzTGYAcJECmAhA61
EQuWsKCdkZhoACMVDVIpy7LA3+GjoJTH51d8c+aee2bm3pZnY8OjqYjIo7I6
4uvNiP/XIr7QU5rm9TZElNPB6xEfeCu1ESc1EcPOQ1jl5Xnv6YDz4I2InLqg
3qzXL7tBH9Kn9Al+VkWs1UV8wpz5v+Vzdv6hr92/wTya9byiXfxxuax8vfmP
1M+T96q37f5o1yKvk3em3qV9/H6E85Z95zSLnJlFM5domfsUus0fMesFZr1j
BiXetJkLdr7DW+T1LWPC/D19v3h7WDO36LyPAibd75i1KruCv+b9Q0F2H2fe
81nPOT1I+uTL3afxwN4rvB6aci5DUc8Bgreur0l9C41oQTN63WVkK5K3Ox8i
xduQb1UX7bidzKFt9JJcozf2++NL3g91yd5DfNPTLlPi30lm0Q56WU+T3GM9
U7xd9ZH8Mb7ruUqP+EuJR+/5+11k5NO4L3MB+rVaGA==
          "]]},
        RowBox[{"-", "0.33`"}]],
       Annotation[#, -0.33, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVz7kuBFAUh/HDoLbPGBoNCl6AaDUShScQITHIqKzPoLE+AcYUoqMyg9gK
nkJh6ez78pviy3fPuf9zl9bhqcFsWURksVEdsZiMaKuLWK6PWMKn+gMjjRFX
NRHTqYhLXmuK6JBb5XbetT9aG3HQEFGUP8S1+XdnFvRO7P3o/aLHfOh185f6
G0c4Lt0lnzPXbH1qJs1n3MJ/SLunPx2R0duzLjonL7+u16mXUJ/zAJfpl+NC
fevdWb7he9yhS10hV4lXf0jI7ru/y59m7c9hBguYR6/3rsislt5ntsqdW/JD
6hw/yjzhAS94xrTcB3/iFe94w7Z8xvvHMYZJzDlvgud5011NZvus07yjPjSX
si5wkoucd84/8ndIdA==
          "]]},
        RowBox[{"-", "0.396`"}]],
       Annotation[#, -0.396, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{4591, 5244, 5214, 4261, 5809, 5810, 5808, 5812, 5811, 4710, 
          5813, 4712, 6374, 4711, 6791, 6029, 6030, 5816, 4715, 5815, 4716, 
          5817, 4714, 5348, 5569, 5568, 4717, 5818, 4905, 5819, 4719, 5570, 
          6083, 6084, 5349, 5573, 5572, 5571, 4720, 5820, 4906, 6550, 6379, 
          6380, 6085, 6086, 4346, 5350, 4347, 6088, 6087, 6382, 6381, 6482, 
          5023, 6756, 5024, 4932, 4933, 6664, 4298, 5025, 4299, 6665, 4681, 
          4680, 6354, 6355, 6351, 6353, 6352, 6356}]},
        RowBox[{"-", "0.462`"}]],
       Annotation[#, -0.462, "Tooltip"]& ], {}, {}}}], 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdlfs7FPgCh4mQynoUSzcNmiQt4uRSfOREzyrVSqqp1bKUI5TCtrmkGpeI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     "]]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  ImageSize->568.8000000000001,
  PlotRange->{{-0.5, 1.5}, {-0.5, 0.5}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{573, 301},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[13]=",
 CellID->212764180]
}, Open  ]],

Cell["\<\
Placing the cursor over streamlines near the airfoil in the overall flow \
field provides information needed for focusing on streamlines near the \
airfoil.  Smoothness of streamlines near the airfoil and absence of \
streamlines interior to the airfoil suggest that the discretization is fine \
enough.\
\>", "Text",
 CellID->130290240],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"g\[Psi]N", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"-", "0.02"}], ",", "1.02"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", 
       RowBox[{"-", ".1"}], ",", ".14"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"Contours", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "0.02"}], ",", 
        RowBox[{"-", ".03"}], ",", 
        RowBox[{"-", "0.04"}], ",", 
        RowBox[{"-", "0.05"}], ",", 
        RowBox[{"-", "0.06"}], ",", 
        RowBox[{"-", "0.07"}]}], "}"}]}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "3"}], ",", 
     RowBox[{"PerformanceGoal", "\[Rule]", "\"\<Speed\>\""}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"g\[Psi]N", ",", "gA", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7.9", " ", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[14]:=",
 CellID->176178541],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJxs3QXYVVX6Pn67xpoxx+4YO8f5qrNGsXPs7sTWUdExAcUWkRbpFqQ7PIA4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    "], {{{}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1GtM1WUcwPGnklZbdgOK+0RuQmUKol2mNHVtiRW4lhZslesy7wasZVt5
aDiwTTJOYWm5NrsIrV4V0zZflFacNsOMBYQWJU7TDG12XbY+z4sv3+fyu/3P
/xwKl69bsvaiEMJOf/qvDyGFxqwQGjA3I4R9eSH8nRlCT04I/3Av/8kp91tz
QziWHcJf9rudt14XwinnCW533oEc+5/VqbU+xbn269xfkPsfLjgbzA/hXx53
dxzPmeEY/4T3kal2BtJxp9wC8Ws5n9dwv/M6/oLrOcUHzfSa2bejQZ9G5OjR
pF42rxKzGiuREPs8Sgo8A88UU4l8TIh92uxp1o+IXe/+Mr4cndbpctJ4vb5v
8zT7d7hFfDMG08V5nmedHTbDNxjEHeaqQb/6V4m7Gs/oswEpZ1faT+Y/PGNK
biUy3GXiA/Xy4lxicnnALCv0/yo+D7/lfBf2yXkB15jpF3F5zs5wgxqN6FJn
1Hz1zlfpsxpPYFyNx/k4V4grR4O8Mb5dziHrfjkDvEb9XXwvB32WWN9nPRX5
9kWccDZDfCtPyK/nFrXqeJmYQWcPxefgRfEduavlLHeHnH1sf5hz7O923my/
mL92dsT8JXpc67Msje+DL+Ukut0n9S22ruKP5PWhx7oXnzpvtU/K6cJLeCrm
4Ki67Z7/Fp/DbbgVp8Xnid/jfi8KrPNRZpZf5d4s/gzP4AmezkP69LgvEp/m
vc/zju5SZy4v4hrui98HXmxfKq+WZzmrxiw1euUPq/MeV9ufV+t3VFqfFV/F
53gmv2rOYnev8DZ040vP8pn8z3EA59TfbO7feH/8HNCo9gl5J1GsbxGa4/sS
d1b+bv7E3Y3O6q2XYhkedfYAP8YJ3OMdbeQxfX/EHGwyV7P5WtAZn0PfjXIS
aBFbEntxKR/Uq8tnkcRW9y+iE2/G32j8DsXvsJi97nPU3cO5PAez0S1uG+ow
391CLMD95j4v9zvrUYzgKI7gYXdb9P4+/qbVmGfmLXzSfoxHzDaMMnO8bL/D
esIMC8R12c/nJC/kNq7m2ahUswpDYpfbT9W/CE3xmeP3EpPM+SEvdb9f7g96
HuC++H2N/0tRjor4G5XXjgH1msRNQznKcAMqsAE3YTq2m3cHvhX/Ou/EG9jk
rlffJ72PNutCM07BiBmGMIxCTMHFYiZhsvhM7/YS6yusOzDuc9vMWcjGCfsH
zT7MbeYdif9r7NPdvStvlFfosxI17jrEnMb/YI7yTA==
          "]]},
        RowBox[{"-", "0.02`"}]],
       Annotation[#, -0.02, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1nmQz3Ucx/Gvm10dJq3KOlKqKaRyteTMWiG7i1035ehypJgKEWbMuKIm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          "]]},
        RowBox[{"-", "0.03`"}]],
       Annotation[#, -0.03, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1nl8FtUVxvEhAbR1ISKL4tJWLWIJBCu20lLQigTC1gYFUVlEQhKoKCik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          "]]},
        RowBox[{"-", "0.04`"}]],
       Annotation[#, -0.04, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1GtsD1YYx/HTko1ESxWlV3+3ICEu07G66zZvvJnKkJghYYy2b1pL1khW
rWtH6h7ENVGRBRv2xgu3hAQVfyKuGaHmLlhsyjY+58W3v/M8z+88zznn//83
MbPsq9KUEMJ3/kzsEEKvLiGU0Jq8EGrxOjuEFlkhnO8cwjm8Un+JdWoJ+cM5
IRxBgz0j5PfQj+3J5L1gPVtuH98S/qXYwrsVlWqFahW0MXrxA98p9JMrUOtP
S+0pwwI86xRCOX1C+6nPVH9qPa4jf24IxbQrTaUZ6pN4j6mX0ON0oRmVyLCv
WPyO93P6Lx1Lb6mdVmuTr6c4DdvVkvpto5foDrqZJnlbqU80pzbeD63FlRhq
fwWt4tnC2816iFyCfqZ/rfwS7FZbZ/96JDN5vdtFmuJ9UhHQEi2Qpf8a9+hC
62lnukKvv2kaavX4h75BOv7Xa2F8J97+uCRO4qg9Y3jHYgDfHrW9aMB2tRXO
NdB6EAbgEwzGPGfdiz7u0UBX8tWhjxm3vUlveofOoFn839NC3vm0Wd/LclPN
uxjPgmScKz/SOdKdKw1t0I1nOh6pP/QWvcx4RDfKzcY0/hf25csn+BP0nXiR
/FvaVTzK+RJmj6YDMdn7TcFU1Iu/ds48mk7boW38/O0t06Mc58w5i1ZyGWa8
sH6JcvyFV1ig9iOdYE4VrbFvMW5Y38Q9915sXjV+wj3z6uUeu8t96z/RhF/N
/g37cRAHcIFnvHuc16cRPfXtgWIzm8T3URpn4C6u8E/3Xjf1/4beot/S67S7
fePsey++Ia7hz8YGuSK1jTRHvJ6muMtcZ6rme2r+JrlUuVz1rPjZ05biQ/LN
6m+xI3r1bnaG+eI39En8LslvQ767FyAHecjFA75ss3NwJP4PQbX1Sv46PLS/
SZ+e8u3wpZkZtC1Wq2XSn2l7uopOcp6r1h1QZz0s9qJ9Y2+6S8+d8fdu7izx
c+vf4+8VVeL2/Gfkxrhjkfi/+FujJfLLrJfjI/W14jWoxy7eRb4bO+k19HXG
q/QL3nyMFxfQZfYVOeNwFLr7p/jD3eY4y206TDw0/j/0uVfgFz1O6nsKx3AC
x9GIDzYs0X0=
          "]]},
        RowBox[{"-", "0.05`"}]],
       Annotation[#, -0.05, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV00lQz2Ecx/HHAaMhSztZopixjHUoClnGWA7GTDMiW3KxLwflqDGTFC6M
a3FxEY4ozJRSifrHcMBEu3FxIMvB63d493m+z/fzXX5PlVF8atfJUSGEBD9e
JoQwZ3oImZiNucjCguQQhtJDGEY/BjGAO4khfKFf0YM+9GI+f/3UEBrwCE/w
GC3656eGsAE3kkIo4o3TfxyK1FzSb5m75ViKwzzJ7tqdX6ENr9GB0fy/9BzB
T1zWowKr9V40LYSFWCA+y7dqRgilPCXqKmidvqXO9+h5msubhzWo5M9T90Vu
nvh9Wggf8EzfC+IylOIzXwlfNl9xtCsO4BAO4kf0DnZ/mhLCO55/vCvEDfos
p8fkj+MoalGDvdE7oFvvQhqje2iG+Xf1qFZb7C7N/KmY5VuG6Tf9vsvFxKdx
BjORKtesvgUDzoMYwg79+mkfelFgx/Xq16Hd76iPv8ycUqTI75RPpMlIQo95
W+27DafMKfC+0+w3QZxOr0dv5+4aPSe/CtlYibXucugo3kyawpfsbqFzlb2q
cUi8W1xMC+lXMz9FM+U+Rn9z0Q7Ybq96ux7ha7Nrvl6t0fvwvaQJPJt5JtEp
mIxy9d2+8w/Pb4wgnreRb7HcE7kl9K++Xe47MYunw4xHcoXiifps4k+z2yfn
t/ypPD3iEzgZvT3i5TbyTbRXN88WnhiNF29wX44m751Ht7pbS3Px0Ox1tI4+
wH1cVBcz/7aetajBeJ48d7lIle/0pl14g27EMNa+V9RXoRJxanL428ztjf63
9dnPtw+3fOM1eh1X8Vwcp2+T8ws0ogXNCPq26nNT/dzoW3mz6JjofxP/ASsU
oVM=
          "]]},
        RowBox[{"-", "0.06`"}]],
       Annotation[#, -0.06, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0DtP03EUxvHjpFAMcpO2SrkstDiBAQIvwSgYg2zEoIwGwuBSUkgMgTiQ
6K6RgIkuLqIygIoiMEKY2CT0JXhBbgmfDt885/L8zjn/f/PDsXujFyJiATO1
EeXXIxK4iDJcQhdu6B03ROzURWzjytWI9kxEJZ3Wq6U35XW0CjXokFfT3np1
nv/eb8o3cJqOOMMRTnCMZp5O2kJ/piLW0WPXjNosFuVvUKyJeJKMOKC919yu
9szMPZ73/Gl7RuRJ+oimaCt/FkPyovmT4pe8rzCLCfUe9+ZpoTSbp1q96O2W
PUm1X2oFu/bpA96UWoOdGfT5xltq/fQ2vnt3h36la/hW+nZzUv5lGm3uziGL
Snvuqw1iAHPyjB2fzU245xOtoMu0nO7aO6WfML+bt83scfVWmkMWeZ4cz7D5
h26eFP+jBfqFruIP5vVf46/4B4bETSVP6XY8xipW8Fu/Xm+FLskbxXfFT8UV
9l3Gmnueu/MFlvQ+ooxvVK/f7D58UFt27zu1t9jQ7/Ku0f/rpOddaVow
          "]]},
        RowBox[{"-", "0.07`"}]],
       Annotation[#, -0.07, "Tooltip"]& ], {}, {}}}], 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdlfs7FPgCh4mQynoUSzcNmiQt4uRSfOREzyrVSqqp1bKUI5TCtrmkGpeI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     "]]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  ImageSize->568.8000000000001,
  PlotRange->{{-0.02, 1.02}, {-0.1, 0.14}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{573, 156},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[15]=",
 CellID->49625382]
}, Open  ]],

Cell["\<\
Magnify the area in the vicinity of the trailing edge to verify that the flow \
leaves the trailing edge smoothly, satisfying the Kutta condition.\
\>", "Text",
 CellID->11571494],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"g\[Psi]TE", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0.8", ",", "1.05"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", 
       RowBox[{"-", ".05"}], ",", ".08"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"Contours", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "0.02"}], ",", 
        RowBox[{"-", ".03"}], ",", 
        RowBox[{"-", "0.04"}], ",", 
        RowBox[{"-", "0.05"}], ",", 
        RowBox[{"-", "0.06"}], ",", 
        RowBox[{"-", "0.07"}]}], "}"}]}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "2"}], ",", 
     RowBox[{"PerformanceGoal", "\[Rule]", "\"\<Speed\>\""}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"g\[Psi]TE", ",", "gA", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7.9", " ", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[16]:=",
 CellID->144480356],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJxlXQVYVcsWtrsDu1sM7HaZWJhwBQMDL7YiKhYqHQKKygEl5NApCnag20BR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    "], {{{}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVzT1LwlEUB+BjaUH1EXJwdQnsjYZAt3L0I7iKulVDSw0tUU1BDiItQvbe
akKtUZOfo60XsYKe//DwO/eee+7JVZuVRioiKgzSEd+ZiBEffPHJo/v1qYhT
9QQdUiyxyKVejw322eOCF3N98tMRBe/SnDFI/mSVG+82mVQfyGuueDW3YK7o
foUuy7Rp8az/JHf44Zc/zskwZpdbf91xz4PzWrKT7aTHFj1qZO1ryHlZl4e8
JXsomS/yrj6Sc3ozclYeyxPKDPX/Aa5HLow=
          "]]},
        RowBox[{"-", "0.02`"}]],
       Annotation[#, -0.02, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVz7krx2EcwPFHuf8Icp85chYxWVzlmCiKHatrYLZRFslisjCi2Nz3qFxF
uQdnFry+w6v35zl+z7dfWt9g+0BMCGGM6rgQEuNDSNIDDjmLDWFKf7jmhg02
SXI3WY/IN1dSxbB1tU7oJIOkJIQwpKnar3XO69nw/j5v5mxyeOCeJvcyonu6
QAfLrDBKjrdGNFdvvVHh7l1U63LzubNOxjm1/2yv1fykr7zQYp2pzfqom+7N
mucoYYlS0p3V6S5l5kzfyKLH3Mui/QbrNv22btcP/eKTY+9m6Un0f53t0cg6
a/amdYYiVtlmJ/pvfpNHt/lXg2/8aZf1BZdcMU+B/ULSKI7epcb8pn++saWB
d2rt/wOAqEQH
          "]]},
        RowBox[{"-", "0.03`"}]],
       Annotation[#, -0.03, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ctLlFEYwOFjeVmU1xnvLRwlI0i7rZISJPdeSksz0FHzQijd9kEF2SpL
rbQUNKgg1ymtXbRpk5pCpW1aSKX9AUrPt3j4nfec7zs6M4nkcPNQSgjhI4m0
EF6lh1CiL3WGaWLmBm2kiXHGWEsNYcXZKvfND7hHLk+J22/WC2yzw0VaaGXW
+Rx1nMgI4bP7Jq0neO78Bc8oMk/qhvMrWmye0t/mfh1kgE3zYWc/9bT7Dlnf
5as57rzMOqYF5PPL/jt7eZ6NkUvSHNcBLdJB7aOfa9FdrLEe/c/uOM66e2o9
m219TnM0izzPPNHRqBTSZm7nh3cm9JG9Cu/c0nK9rfO8p8p5NQfINn9hmVnz
HJ10Uem9O/aT1t28te6ki6POFvytT/SaC8yF5NNjTuhNLdMbesr7J8mMfkfz
Qc3Rbfbc8Vf/scM382v9rpc9d4mG6Fyva7E7Sykh0/w4+l50VLe889B6hCOc
8UylLtrPsN7HflK4aj9VO/QNvd7vId1eGmftfYi+e13QY/qHXXctRZ/NXo3n
6jlPlb3/JbtTdA==
          "]]},
        RowBox[{"-", "0.04`"}]],
       Annotation[#, -0.04, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0LcvRmEUB+BX+5gleosyaQkRBInOoETMLAY2dgOD8ml/BImya7tuU4IB
EwPCZhU8d3hyfuec97259xaPTQ1PJoQQpilPCSGDmtQQHpNDyJOf1Fp9pjzD
eiyEDUYY5cX+W91jn10OOaDA+RLq3X93rkz+UBv0pfIsRdTpX82H5Caa9c1q
Ie1yume1qTF9XE5Vf+mX+2i1+3G/Ua2gkjn7Jbs09Y8EOXDn3IA6SAdVzp6Y
zTuzrF8hkWSSuLe7srvmiE6zU7MFeVVeIyV6J2I82N3Y3XJMl9mZ2aK8KW+R
RQ7ZvNk92T1zQrfZuVlc3pZ3yCWfPD7txtUJeqj2/hdmhdHOnR79l3oa/Vuz
Fv2lfW/0vdH/Mf8HrM46Og==
          "]]},
        RowBox[{"-", "0.05`"}]],
       Annotation[#, -0.05, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0DdOA0EAhtFBIucr0MERoCTTIoFxY0s0dHASQs8BoARsbDIm55xzhisQ
JZ6Lp0+z82ul3aruvvbenBBCDyO5IfTnhTBANQmSjOaHMKlTpBmzm/Fslmnm
maPe3ZIus0CGRcbtV3WNFTZYp8F+R3fZZJstJuz39YA9jjik0f5MzznmlBMS
9pd6xQU3XNNk/6CP3HLPHUn7Z33hiRa7V23WD23TN23V9+y3239pp/OnRvRb
O/RPo/qjXfqrKfuighDizoVaQjEx5wqtpJRyykjbD7obooYUtd5Tl/2X7oad
/wGP5ET8
          "]]},
        RowBox[{"-", "0.06`"}]],
       Annotation[#, -0.06, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{1199, 1513, 1514, 1511, 1275, 1512, 1198, 1664, 1274, 1662, 
          1273, 1663, 1197, 1661, 1272, 1659, 1271, 1660, 1196, 1658, 1270, 
          1656, 1269, 1657, 1195, 1655, 1268, 1653, 1267, 1654, 1194, 1652, 
          1266, 1650, 1265, 1651, 1193, 1649, 1264, 1647, 1263, 1648, 1192, 
          1646, 1262, 1643, 1645, 1644, 1190, 1509, 1510, 1506, 1508, 1507, 
          1565}]},
        RowBox[{"-", "0.07`"}]],
       Annotation[#, -0.07, "Tooltip"]& ], {}, {}}}], 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdlfs7FPgCh4mQynoUSzcNmiQt4uRSfOREzyrVSqqp1bKUI5TCtrmkGpeI
kfswY4XcmxnGtbRf9yW0rpFNMk3jFga57CP27J4fz+d53uf9D94PxeWavdsa
CQkJt7/5x1HnhhsyV6eR2Ph7q7+/AxmfU1w/tzSFjF2uHyynz5LoA9+saH+e
hJza17WhwRfJr4ERxkHiT+hPIXYNFGei47IUoj01AaHLPu26AleywTya++Pk
OPpte3gVw+7k8VNm5JWpMexsYdDFLV5Egy5Z6DkzCuH85ibrYh/yr+IiWuHC
CBpWbn226fYlPaKVmxdWRUhpFKTcs7lFWLSdVqVyIgi5KutZ0oFkxmLa5ZPy
RwQyVhSr1oQQJZ5moSlVCNVxDYVM9XvErPHCxk7TD5C+0fbO6zSdBH19nS46
JYCnh4FoMDmM3DjjJfn06jC2ib6X3SOKICk/Xv7eMfI9MtTPTyobRxFi5JpO
pQ1hVI+hpXI/miR0bHxJzRuEb0r8Q+c6BpF/EyMl/9cfKOLSeyMWYkl5o+dz
nUsD0HpGPZC+8oi09B54cKqtHz0LzG2OS3Hk9OJRn2SbPsT8pRRkuSaBsA63
98u19kIYaM/PVEskRsF6PUPne6CfbsQpMU4i3V8Ytx3numBWo3S770wyMZ1f
MctJ7kQsLeQO3yuFrNvP2it3tAPNkt6W+35mkkenDmZ4SL2Cb9p4tP+NVMKU
yi9oZ7TCZv8tEyWHNPJk+eOdIsMWdOrKHOlXZRE1seqWgOkmNDqt5S49Z5ET
A2VhnPoGCHSyhr+YsEmgZvRYNa8OqY2r736LYpPJwmM/bK+oQbX2NsdEHpvU
nKTHMDgvkHbOn7+YxSZD3sfr3DWfIdPLzbHXhU3M82Oby9vLYdux4yu3cRbR
Deh9787hw2ZwTl7HhEXiwheGrzbxkN9c4DD9bRoZ4eWQ7/YVwbPRS6F+eyrx
fs7zl7XMhZGM8MFMcQoZpo0f5zdlQjr5ibPE2mRSt2U2aTWUDb09nmX5yomE
Y2XvJu2VhINco7w+QRzxm6XQfraOBVdR2jzjaiwRmL7tpWmFYfqx97n7TlFk
rtJuecwxAKV+wbEWtFDyw+n9clIJnmBZ5il/iQsiN7o216oUOMBfkplrzvEm
Ev+35vwLvUeWDYjeRb3uz7uuQyJ03+0xKQtsrO+YOXY5BFtem1tly7lAJnL7
SvtiBA498O6eP+mL8Iwmvng4FiqTLYb1grvY/UCp5plLIrQCXmtQ+FG4tBpy
cN0yExa5wpGYtngs5wlj5nLSYRGnqaH0MhUz5wx9F3iPYZ+krfer+2MIVDKU
r2zKhin3Vd+E+AmULFPTxT/lYHm+pHD0SwGofONm/z/yQH9tZDlpzYOf4kVB
plkh5vrs1tFO8EG50rh14tFT+JjMXlNLLsdkcdfe1EEuro/lsb3Vn2FgLDhr
YkMxkjOPnw0RvYBp2ZLGWUoJsiN1vymar4GRqc7ceh0+7hVrB/uo1MNsXrxW
clcpfgk/5Lvo0IhFpurnvYpl2HqJlpXE+w3mJu8W6WNl6PP0kGZqv0TIErtW
nlsO9VNjR8yq2+Bezd3S5FQBBa/zJnkev8M0gnqsfaEC13Qqyu6qdSI+RcHk
qF8l/tx0c0Ivsgs89xF97/5KhEg+7BPI9CBBzcc1bkcVSmf1qGqMXrz09VvY
alOFWklh/HGNPsirLxU/tKuC7G37m7OkH6MOe87cNaxCV8eL/d+6DoD/lhK/
bakSJzSLB1kKb7E7TfOn+KxK6N57ZP2+dhDynNdlhsaVKAMlKCFoCBvDExyb
SiqgOvfGmpXxHsPUP3/RUKlATJtdt1bY332JWDqs61yOkcseJZRrArzTWXbq
jC+DzHe7F05e+IBPrUMhcpxSdOUYbvC0FSLGwTb8FZcPA52qNoNDH3EkVMzS
YpZgczU37KK+CAZbmYcVPIqxY/2UXC51BAPDlv4MDR5WbK3zWneMQqEuW98x
9Cmonsr/pquOIcdeS1yXUABKotAqctM4pqyKPJxu5sKEmc5K+moCHKWiflVK
NkTTBzqcN3zClYZmv352BmyCbo1mrZtE/pqauNkuJuKz3wRslp2CquwlrzTZ
OMhR9Lke0tPIGd35nzsl91F1msHmSIgh2/S//8B/Ae53K0E=
     "]]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  ImageSize->568.8000000000001,
  PlotRange->{{0.8, 1.05}, {-0.05, 0.08}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{573, 310},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[17]=",
 CellID->317263802]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.4.3 Velocity field", "Section",
 CellID->56297006],

Cell["\<\
Plotting arrows for the velocity on a grid provides a relatively fast direct \
visualization of the velocity field.  Use the velocity influence coefficients \
and the strength parameters from the model solution to compute the velocity \
at an arbitrary field point induced by the panels representing the \
discretized airfoil.\
\>", "Text",
 CellID->253577469],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icA", ",", "icB"}], "}"}], "=", 
   RowBox[{"ICVortexLinear", "[", 
    RowBox[{"rF", ",", "rP"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"VA", ",", "VB"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"icA", ".", "a"}], ",", 
     RowBox[{"icB", ".", "b"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"V\[Infinity]", "=", 
   RowBox[{"UniformFlow", "[", 
    RowBox[{"1", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"V", "=", 
   RowBox[{"V\[Infinity]", "+", "VA", "+", "VB"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gV", "=", 
   RowBox[{"VectorPlot", "[", 
    RowBox[{"V", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gV", ",", "gA", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7.9", " ", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[18]:=",
 CellID->20343141],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnXuvbVWZ5unuT9Ifo0/S2X/YGo1d7ZVGQEBAt6BUoRZ4AUvBC9jdoDto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    "], {{0, 326}, {569, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{569, 326},
  PlotRange->{{0, 569}, {0, 326}}]], "Output",
 ImageSize->{573, 326},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[23]=",
 CellID->649391956]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.4.4 StreamPlot", "Section",
 CellID->256023531],

Cell[TextData[{
 "With suitable values for options, the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function StreamPlot can be used to plot streamlines directly from the \
velocity field.  This would be particularly useful for panel methods using \
source panels, where the stream function is not single valued.  In the \
following plot, streamlines with direction of flow indicated by arrows are \
forced though points specified by the symbol tuples."
}], "Text",
 CellID->117098232],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[CapitalDelta]", "=", "0.07"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"z0", "=", 
   RowBox[{"Range", "[", 
    RowBox[{
     RowBox[{"zMin", "+", "\[CapitalDelta]"}], ",", 
     RowBox[{"zMax", "-", "\[CapitalDelta]"}], ",", "\[CapitalDelta]"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"x0", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"xMin", "+", "\[CapitalDelta]"}], ",", 
     RowBox[{"{", 
      RowBox[{"Length", "[", "z0", "]"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tuples", "=", 
   RowBox[{"Append", "[", 
    RowBox[{
     RowBox[{"Transpose", "[", 
      RowBox[{"{", 
       RowBox[{"x0", ",", "z0"}], "}"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"xMax", "-", "\[CapitalDelta]"}], ",", "0"}], "}"}]}], "]"}]}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gSP", "=", "\[IndentingNewLine]", 
   RowBox[{"StreamPlot", "[", 
    RowBox[{"V", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"StreamPoints", "\[Rule]", 
      RowBox[{"{", "tuples", "}"}]}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}], ",", 
     RowBox[{"StreamScale", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Full", ",", "All", ",", "0.015"}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gSP", ",", "gA", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7.9", " ", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[24]:=",
 CellID->826808409],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnf2PVde532n7l/SfqFSkq8QvYGAwmDjESShxDDcTSOKQ3JrmXrsvdpvA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    "], {{0, 310}, {569, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{569, 310},
  PlotRange->{{0, 569}, {0, 310}}]], "Output",
 ImageSize->{573, 310},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[29]=",
 CellID->559253511]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.4.5 StreamDensityPlot", "Section",
 CellID->2809397],

Cell[TextData[{
 "The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function StreamDensityPlot is used to plot the scalar pressure-coefficient \
field as well as streamlines.  Compute the list of surface pressure \
coefficients using Bernoulli's equation and record the minimum pressure \
coefficient for use in color coding.  In the color function, Hue is used to \
specify color, saturation and brightness.  Positive pressure coefficients are \
shown in orange and negative pressure coefficients shown in blue.  Minimum \
and maximum values of pressure coefficient have maximum saturation and a \
pressure coefficient of 0 has a saturation value of 0."
}], "Text",
 CellID->176900651],

Cell[BoxData[{
 RowBox[{
  RowBox[{"cpC", "=", 
   RowBox[{"1", "-", 
    RowBox[{"VMagC", "^", "2"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cpCMin", "=", 
   RowBox[{"Min", "[", "cpC", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"colorFunction", "[", 
    RowBox[{"c_", ",", "cMin_"}], "]"}], ":=", 
   RowBox[{"Hue", "[", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"c", "\[GreaterEqual]", "0"}], ",", ".0483667", ",", 
       "0.633338"}], "]"}], ",", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"c", "\[GreaterEqual]", "0"}], ",", "c", ",", 
       RowBox[{"c", "/", "cMin"}]}], "]"}], ",", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{"cMin", "\[LessEqual]", "c", "<", 
        RowBox[{"-", "1"}]}], ",", "0.6472", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "\[LessEqual]", "c", "<", "0"}], ",", 
       RowBox[{"1", "+", 
        RowBox[{".3528", "c"}]}], ",", 
       RowBox[{"c", "\[GreaterEqual]", "0"}], ",", "1"}], "]"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellLabel->"In[30]:=",
 CellID->460900760],

Cell["\<\
Compute the scalar field for pressure coefficient and prepare the graphic.  \
Since the flow interior to the airfoil is being computed, there are abrupt \
changes in pressure coefficient across the surface of the airfoil at most \
locations.  Although this graphic provides the most information on the flow \
field, it is also the most expensive in terms of computational time because \
the value of the option MaxRecursion must be set to a relatively large value \
to reduce numerical artifacts.\
\>", "Text",
 CellID->76766512],

Cell[BoxData[{
 RowBox[{
  RowBox[{"cp", "=", 
   RowBox[{"1", "-", 
    RowBox[{"V", ".", "V"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gSDP", "=", "\[IndentingNewLine]", 
   RowBox[{"StreamDensityPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"V", ",", "cp"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"ColorFunction", "\[Rule]", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"colorFunction", "[", 
         RowBox[{"#", ",", "cpCMin"}], "]"}], "&"}], ")"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"ColorFunctionScaling", "\[Rule]", "False"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "6"}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}], ",", 
     RowBox[{"StreamPoints", "\[Rule]", 
      RowBox[{"{", "tuples", "}"}]}], ",", 
     RowBox[{"StreamScale", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Full", ",", "All", ",", "0.015"}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gField", "=", 
   RowBox[{"Show", "[", 
    RowBox[{"gSDP", ",", "gA"}], "]"}]}], ";"}]}], "Input",
 CellLabel->"In[33]:=",
 CellID->146745562],

Cell["\<\
Prepare graphic for surface pressure coefficient, color coded by the same \
color function used in the field plot.\
\>", "Text",
 CellID->614303007],

Cell[BoxData[{
 RowBox[{
  RowBox[{"xC", "=", 
   RowBox[{
    RowBox[{"Components", "[", "rC", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gCpSurface", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xC", ",", "cpC", ",", 
     RowBox[{"ColorFunction", "\[Rule]", 
      RowBox[{"Function", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"xC", ",", "cpC"}], "}"}], ",", 
        RowBox[{"colorFunction", "[", 
         RowBox[{
          RowBox[{"-", "cpC"}], ",", "cpCMin"}], "]"}]}], "]"}]}], ",", 
     RowBox[{"ColorFunctionScaling", "\[Rule]", "False"}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"Thickness", "[", "0.015", "]"}]}]}], "]"}]}], ";"}]}], "Input",
 CellLabel->"In[36]:=",
 CellID->318266012],

Cell["\<\
Display the graphics for a combined flow field and pressure field, with a \
plot of surface pressure coefficient serving as a key to color coding used in \
the pressure field.\
\>", "Text",
 CellID->42228577],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphicsColumn", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"gField", ",", "gCpSurface"}], "}"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7.9", " ", "72"}]}]}], "]"}]], "Input",
 CellLabel->"In[38]:=",
 CellID->448653773],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzsvQeUpcWR56s1772z782e3Tdzxu7OzM68nd0xGiONNDOySEIOEEIYCSFA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    "], {{0, 676}, {569, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{569, 676},
  PlotRange->{{0, 569}, {0, 676}}]], "Output",
 ImageSize->{573, 676},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[38]=",
 CellID->72059547]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["Panel Methods",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/PanelMethods"]], "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["3.3 Implementation",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation"]], "RelatedTutorials",
 CellID->806871991],

Cell[TextData[ButtonBox["3.5 Convergence and Timing",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming"]], "RelatedTutorials",
 CellID->334470709],

Cell[TextData[ButtonBox["Overview",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]], "RelatedTutorials",
 CellID->2248688]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"3.4  Flow Visualization - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 4, 3.4836000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "None", "summary" -> 
    "Flow visualization provides a useful plausibility check for numerical \
models in fluid dynamics. Mathematica's ability to work seamlessly with \
symbols and numbers is used to write functions describing the flow field for \
uniform potential flow over an airfoil. Load the Aerodynamics packages. \
Specify an arbitrary field point. A single vector may be written in \
Mathematica notation.", "synonyms" -> {}, "title" -> 
    "3.4  Flow Visualization", "type" -> "Tutorial", "uri" -> 
    "Aerodynamics/tutorial/FlowVisualization"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[616, 21, 78, 1, 70, "TutorialColorBar"],
Cell[697, 24, 191, 5, 70, "LinkTrail"],
Cell[891, 31, 2910, 66, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3826, 101, 60, 1, 70, "Title",
 CellID->509267359],
Cell[3889, 104, 359, 9, 70, "Text",
 CellID->823686949],
Cell[4251, 115, 66, 1, 70, "Text",
 CellID->34146804],
Cell[4320, 118, 99, 3, 70, "Input",
 CellID->35199140],
Cell[4422, 123, 185, 6, 70, "Text",
 CellID->249040680],
Cell[4610, 131, 159, 6, 70, "Input",
 CellID->48493306],
Cell[4772, 139, 456, 8, 70, "Text",
 CellID->972502991],
Cell[5231, 149, 725, 16, 70, "Text",
 CellID->70657603],
Cell[CellGroupData[{
Cell[5981, 169, 66, 1, 70, "Section",
 CellID->314388262],
Cell[6050, 172, 151, 4, 70, "Text",
 CellID->75137487],
Cell[6204, 178, 309, 9, 70, "Input",
 CellID->108183604],
Cell[6516, 189, 147, 4, 70, "Text",
 CellID->731882235],
Cell[CellGroupData[{
Cell[6688, 197, 476, 13, 83, "Input",
 CellID->401362744],
Cell[7167, 212, 5376, 96, 201, "Output",
 CellID->943918884]
}, Open  ]],
Cell[12558, 311, 89, 1, 70, "Text",
 CellID->19157149],
Cell[12650, 314, 266, 8, 70, "Input",
 CellID->450780366]
}, Open  ]],
Cell[CellGroupData[{
Cell[12953, 327, 58, 1, 70, "Section",
 CellID->1708000],
Cell[13014, 330, 573, 9, 70, "Text",
 CellID->35129489],
Cell[13590, 341, 207, 5, 70, "Text",
 CellID->48905279],
Cell[13800, 348, 791, 24, 70, "Input",
 CellID->17324249],
Cell[14594, 374, 89, 1, 70, "Text",
 CellID->275199393],
Cell[CellGroupData[{
Cell[14708, 379, 1241, 33, 70, "Input",
 CellID->666894011],
Cell[15952, 414, 80266, 1379, 70, "Output",
 CellID->212764180]
}, Open  ]],
Cell[96233, 1796, 347, 7, 70, "Text",
 CellID->130290240],
Cell[CellGroupData[{
Cell[96605, 1807, 1270, 34, 70, "Input",
 CellID->176178541],
Cell[97878, 1843, 86615, 1450, 70, "Output",
 CellID->49625382]
}, Open  ]],
Cell[184508, 3296, 189, 4, 70, "Text",
 CellID->11571494],
Cell[CellGroupData[{
Cell[184722, 3304, 1249, 33, 70, "Input",
 CellID->144480356],
Cell[185974, 3339, 24366, 430, 70, "Output",
 CellID->317263802]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[210389, 3775, 58, 1, 70, "Section",
 CellID->56297006],
Cell[210450, 3778, 370, 7, 70, "Text",
 CellID->253577469],
Cell[CellGroupData[{
Cell[210845, 3789, 1406, 42, 137, "Input",
 CellID->20343141],
Cell[212254, 3833, 27333, 456, 321, "Output",
 CellID->649391956]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[239636, 4295, 55, 1, 70, "Section",
 CellID->256023531],
Cell[239694, 4298, 495, 10, 70, "Text",
 CellID->117098232],
Cell[CellGroupData[{
Cell[240214, 4312, 1854, 53, 173, "Input",
 CellID->826808409],
Cell[242071, 4367, 18882, 317, 306, "Output",
 CellID->559253511]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[261002, 4690, 60, 1, 70, "Section",
 CellID->2809397],
Cell[261065, 4693, 695, 13, 70, "Text",
 CellID->176900651],
Cell[261763, 4708, 1112, 33, 70, "Input",
 CellID->460900760],
Cell[262878, 4743, 539, 9, 70, "Text",
 CellID->76766512],
Cell[263420, 4754, 1447, 39, 70, "Input",
 CellID->146745562],
Cell[264870, 4795, 158, 4, 70, "Text",
 CellID->614303007],
Cell[265031, 4801, 872, 23, 70, "Input",
 CellID->318266012],
Cell[265906, 4826, 218, 5, 70, "Text",
 CellID->42228577],
Cell[CellGroupData[{
Cell[266149, 4835, 263, 8, 28, "Input",
 CellID->448653773],
Cell[266415, 4845, 88136, 1452, 643, "Output",
 CellID->72059547]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[354600, 6303, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[354668, 6306, 159, 3, 70, "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[CellGroupData[{
Cell[354864, 6314, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[354939, 6317, 170, 4, 70, "RelatedTutorials",
 CellID->806871991],
Cell[355112, 6323, 184, 4, 70, "RelatedTutorials",
 CellID->334470709],
Cell[355299, 6329, 164, 4, 70, "RelatedTutorials",
 CellID->2248688]
}, Open  ]]
}, Open  ]],
Cell[355490, 6337, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

