(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     35600,        878]
NotebookOptionsPosition[     31720,        755]
NotebookOutlinePosition[     33546,        804]
CellTagsIndexPosition[     33503,        801]
WindowTitle->A2 Graphics - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"A1 CartesianVectors\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/CartesianVectors"], "\<\"Acknowledgments\
\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/Acknowledgments"], "\<\"Overview\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Graphics\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/Graphics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Functions \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PlotAirfoil\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PlotAirfoil"], "\<\"PlotConvergence\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PlotConvergence"], \
"\<\"PlotSurfacePressureCoefficient\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PlotSurfacePressureCoefficient"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/tutorial/Graphics\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/Graphics"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/Graphics.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/Graphics", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["A2 Graphics", "Title",
 CellID->509267359],

Cell[TextData[{
 "Some graphics are used so often or are so specialized that it is convenient \
to define ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " functions to generate them."
}], "Text",
 CellID->1534169418],

Cell[BoxData[GridBox[{
   {
    ButtonBox["PlotAirfoil",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/PlotAirfoil"], Cell[
    "plots an airfoil in an airfoil-fixed coordinated system", "TableText"]},
   {
    ButtonBox["PlotConvergence",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/PlotConvergence"], Cell[
    "plots convergence properties of model", "TableText"]},
   {
    ButtonBox["PlotSurfacePressureCoefficient",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/PlotSurfacePressureCoefficient"], 
    Cell["plots pressure coefficient on airfoil surface", "TableText"]}
  }]], "DefinitionBox",
 CellID->2096742444],

Cell[TextData[{
 "Special graphics functions used in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 "."
}], "Caption",
 CellID->1891092685],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "Text",
 CellID->435059],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->273674889],

Cell[TextData[{
 "PlotAirfoil is a straightforward application of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function ListPlot, but it is used so often that it is a named function in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  In the following example, a discretized NACA-4412 airfoil is plotted in \
an airfoil-fixed coordinate system.  All options of ListPlot are accepted by \
PlotAirfoil."
}], "Text",
 CellID->64486792],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"NACA4DigitAirfoil", "[", 
     RowBox[{"4412", ",", 
      RowBox[{"NDiscretizeUnitSegment", "[", 
       RowBox[{"50", ",", 
        RowBox[{"Layout", "\[Rule]", "HalfCosineSpacing"}]}], "]"}]}], "]"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"PlotAirfoil", "[", "rP", "]"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->2058623809],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3c9rFHkax/Fe9i/ZfyMgcxAP4iFkQTyIF4VGL+NFL3qJl92LQpNcEpDJ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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->190888429]
}, Open  ]],

Cell[TextData[{
 "PlotConvergence is a specialized function used only in one section of the \
tutorial, but it is used several times, and it seems appropriate to define a \
function to accomplish this task.  See ",
 ButtonBox["3.4 Convergence and Timing",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/ConvergenceAndTiming"],
 " for usage."
}], "Text",
 CellID->76441082],

Cell[TextData[{
 "The conventions used in the aerodynamics community for plotting surface \
pressure coefficient are sufficiently different from traditional mathematics \
conventions to warrant a special function.  Surface pressure coefficients \
associated with collocation points are plotted versus chordwise locations on \
the airfoil (",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values).  For a lifting airfoil at a cruise angle of attack, the upper \
surface of the airfoil has primarily negative pressure coefficients and the \
lower surface, primarily positive pressure coefficients.  For this reason \
negative pressure coefficients are plotted up and positive pressure \
coefficients, down, so that the upper curve corresponds to the upper surface \
of the airfoil.  In the following example, PMVortexLinearNeumann is used to \
solve for potential flow over a NACA-4412 airfoil at an angle of attack of \
five degrees, and pressure coefficient is obtained from Bernouilli's equation \
using fluid speeds at collocation points."
}], "Text",
 CellID->410626370],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Alpha]", "=", 
   RowBox[{"5.", "Degree"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "rC", ",", "vC"}], "}"}], "=", 
   RowBox[{"PMVortexLinearNeumann", "[", 
    RowBox[{"rP", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xC", ",", "cp"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Components", "[", "rC", "]"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], ",", 
     RowBox[{"1", "-", 
      RowBox[{"vC", "^", "2"}]}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"PlotSurfacePressureCoefficient", "[", 
  RowBox[{"xC", ",", "cp"}], "]"}], "\[IndentingNewLine]"}], "Input",
 CellLabel->"In[3]:=",
 CellID->552331023],

Cell[BoxData[
 GraphicsBox[
  {Hue[0.67, 0.6, 0.6], PointBox[CompressedData["
1:eJwVxX08FHgCB2BRJC8llCOsEFtdai+tVXydFcVuK7bNW8coKpeP3cxqUqso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    "]]},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox[
    "\"\\!\\(\\*SubscriptBox[\\(C\\), \\(p\\)]\\)\"", TraditionalForm]},
  FrameTicks->{Automatic, {{-1, 
      FormBox["\"1\"", TraditionalForm]}, {0, 
      FormBox["\"0\"", TraditionalForm]}, {1, 
      FormBox["\"-1\"", TraditionalForm]}, {2, 
      FormBox["\"-2\"", TraditionalForm]}}, None, None},
  ImageSize->504,
  ImageSizeRaw->{504, 330},
  PlotRange->{{-0.05, 1.05}, {-1.2, 2}},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 330},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->355694088]
}, Open  ]],

Cell["\<\
This plot can be enhanced using the option Epilog to provide a concise \
summary of the geometric and aerodynamic properties of the airfoil.  Compute \
the lift and pitching-moment coefficients.\
\>", "Text",
 CellID->26314046],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl", ",", "cm"}], "}"}], "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Vortex\>\"", ",", "a", ",", "b", ",", "rP", ",", "\[Alpha]", ",", 
    "rC"}], "]"}]}]], "Input",
 CellLabel->"In[7]:=",
 CellID->490095263],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1.1123843349886422`", ",", 
   RowBox[{"-", "0.3944569767556705`"}]}], "}"}]], "Output",
 ImageSize->{142, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->793734776]
}, Open  ]],

Cell["Locate the center of pressure.", "Text",
 CellID->134219308],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"xCenP", "=", 
  RowBox[{
   RowBox[{"-", "cm"}], "/", "cl"}]}]], "Input",
 CellLabel->"In[8]:=",
 CellID->379056241],

Cell[BoxData["0.3546049367547935`"], "Output",
 ImageSize->{60, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->677287257]
}, Open  ]],

Cell["\<\
Generate a plot summarizing the geometric and aerodynamic properties of the \
airfoil.\
\>", "Text",
 CellID->12305289],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotSurfacePressureCoefficient", "[", 
  RowBox[{"xC", ",", "cp", ",", "\[IndentingNewLine]", 
   RowBox[{"Epilog", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Inset", "[", 
       RowBox[{
        RowBox[{"Panel", "[", 
         RowBox[{"Grid", "[", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"Row", "[", 
              RowBox[{"{", 
               RowBox[{"\"\<\[Alpha] = \>\"", ",", 
                RowBox[{"NumberForm", "[", 
                 RowBox[{
                  RowBox[{"\[Alpha]", "/", "Degree"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"3", ",", "2"}], "}"}]}], "]"}], ",", 
                "\"\<\[Degree]\>\""}], "}"}], "]"}], "}"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"{", 
             RowBox[{"Row", "[", 
              RowBox[{"{", 
               RowBox[{"\"\<\!\(\*SubscriptBox[
StyleBox[\"C\",
FontSlant->\"Italic\"], \"\[ScriptL]\"]\) = \>\"", ",", 
                RowBox[{"NumberForm", "[", 
                 RowBox[{"cl", ",", 
                  RowBox[{"{", 
                   RowBox[{"3", ",", "2"}], "}"}]}], "]"}]}], "}"}], "]"}], 
             "}"}], ",", "\[IndentingNewLine]", 
            RowBox[{"{", 
             RowBox[{"Row", "[", 
              RowBox[{"{", 
               RowBox[{"\"\<\!\(\*
StyleBox[SubscriptBox[\"C\", \"m\"],
FontSlant->\"Italic\"]\) = \>\"", ",", 
                RowBox[{"NumberForm", "[", 
                 RowBox[{"cm", ",", 
                  RowBox[{"{", 
                   RowBox[{"3", ",", "2"}], "}"}]}], "]"}]}], "}"}], "]"}], 
             "}"}], ",", "\[IndentingNewLine]", 
            RowBox[{"{", 
             RowBox[{"Row", "[", 
              RowBox[{"{", 
               RowBox[{"\"\<\!\(\*SubscriptBox[
StyleBox[\"x\",
FontSlant->\"Italic\"], \"cp\"]\) = \>\"", ",", " ", 
                RowBox[{"NumberForm", "[", 
                 RowBox[{"xCenP", ",", 
                  RowBox[{"{", 
                   RowBox[{"3", ",", "2"}], "}"}]}], "]"}]}], "}"}], "]"}], 
             "}"}]}], "}"}], "\[IndentingNewLine]", "]"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"Right", ",", "Top"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"Right", ",", "Top"}], "}"}]}], "]"}], ",", 
      RowBox[{"Inset", "[", 
       RowBox[{
        RowBox[{"PlotAirfoil", "[", 
         RowBox[{"rP", ",", 
          RowBox[{"Epilog", "\[Rule]", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", "Black", "]"}], ",", "Yellow", ",", 
             RowBox[{"Disk", "[", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"xCenP", ",", "0"}], "}"}], ",", ".012"}], "]"}]}], 
            "}"}]}], ",", 
          RowBox[{"Frame", "\[Rule]", "False"}], ",", 
          RowBox[{"PlotRange", "\[Rule]", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"0", ",", "1"}], "}"}], ",", "Automatic"}], "}"}]}]}], 
         "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", "1"}], "]"}]}], "}"}]}]}], 
  "]"}]], "Input",
 CellLabel->"In[9]:=",
 CellID->68442233],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3U+IHOeZx/FeVrA6bbTZwJqcVpdEkD3oqB1YVkYrEq0OisIaH4TAyEuT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    "], {{0, 330}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 330},
  PlotRange->{{0, 504}, {0, 330}}]], "Output",
 ImageSize->{508, 330},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->116706209]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["Graphics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Graphics"]], "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["A1 CartesianVectors",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/CartesianVectors"]], "RelatedTutorials",
 CellID->147844826],

Cell[TextData[ButtonBox["Acknowledgments",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Acknowledgments"]], "RelatedTutorials",
 CellID->237435115],

Cell[TextData[ButtonBox["Overview",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]], "RelatedTutorials",
 CellID->324670292]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"A2 Graphics - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 4, 9.9888000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "None", "summary" -> 
    "Some graphics are used so often or are so specialized that it is \
convenient to define Aerodynamics functions to generate them. Special \
graphics functions used in Aerodynamics. Load the Aerodynamics packages.", 
    "synonyms" -> {}, "title" -> "A2 Graphics", "type" -> "Tutorial", "uri" -> 
    "Aerodynamics/tutorial/Graphics"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[604, 21, 78, 1, 70, "TutorialColorBar"],
Cell[685, 24, 191, 5, 70, "LinkTrail"],
Cell[879, 31, 3493, 79, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4397, 114, 48, 1, 70, "Title",
 CellID->509267359],
Cell[4448, 117, 223, 7, 70, "Text",
 CellID->1534169418],
Cell[4674, 126, 681, 17, 70, "DefinitionBox",
 CellID->2096742444],
Cell[5358, 145, 146, 6, 70, "Caption",
 CellID->1891092685],
Cell[5507, 153, 122, 6, 70, "Text",
 CellID->435059],
Cell[5632, 161, 100, 3, 70, "Input",
 CellID->273674889],
Cell[5735, 166, 450, 11, 70, "Text",
 CellID->64486792],
Cell[CellGroupData[{
Cell[6210, 181, 446, 12, 83, "Input",
 CellID->2058623809],
Cell[6659, 195, 5376, 96, 201, "Output",
 CellID->190888429]
}, Open  ]],
Cell[12050, 294, 390, 9, 70, "Text",
 CellID->76441082],
Cell[12443, 305, 1087, 18, 70, "Text",
 CellID->410626370],
Cell[CellGroupData[{
Cell[13555, 327, 819, 24, 70, "Input",
 CellID->552331023],
Cell[14377, 353, 3001, 61, 351, "Output",
 CellID->355694088]
}, Open  ]],
Cell[17393, 417, 237, 5, 70, "Text",
 CellID->26314046],
Cell[CellGroupData[{
Cell[17655, 426, 277, 9, 70, "Input",
 CellID->490095263],
Cell[17935, 437, 259, 8, 36, "Output",
 CellID->793734776]
}, Open  ]],
Cell[18209, 448, 66, 1, 70, "Text",
 CellID->134219308],
Cell[CellGroupData[{
Cell[18300, 453, 140, 5, 70, "Input",
 CellID->379056241],
Cell[18443, 460, 176, 5, 36, "Output",
 CellID->677287257]
}, Open  ]],
Cell[18634, 468, 129, 4, 70, "Text",
 CellID->12305289],
Cell[CellGroupData[{
Cell[18788, 476, 3305, 86, 173, "Input",
 CellID->68442233],
Cell[22096, 564, 8690, 150, 351, "Output",
 CellID->116706209]
}, Open  ]],
Cell[CellGroupData[{
Cell[30823, 719, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[30891, 722, 150, 3, 70, "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[CellGroupData[{
Cell[31078, 730, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[31153, 733, 173, 4, 70, "RelatedTutorials",
 CellID->147844826],
Cell[31329, 739, 168, 4, 70, "RelatedTutorials",
 CellID->237435115],
Cell[31500, 745, 166, 4, 70, "RelatedTutorials",
 CellID->324670292]
}, Open  ]]
}, Open  ]],
Cell[31693, 753, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

