(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    152520,       5338]
NotebookOptionsPosition[    130789,       4571]
NotebookOutlinePosition[    133119,       4626]
CellTagsIndexPosition[    133076,       4623]
WindowTitle->3.3  Implementation - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"3.2 Validation Example\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/ValidationExample"], "\<\"3.4 Flow \
Visualization\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/FlowVisualization"], "\<\"Overview\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PanelMethods\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/PanelMethods"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/tutorial/Implementation\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/Implementation"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/Implementation.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/Implementation", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["3.3  Implementation", "Title",
 CellID->509267359],

Cell[TextData[{
 "This section provides details for the implementation of the three \
panel-method functions implemented in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " for a coarsely discretized Eppler-205 airfoil.  In addition to \
illustrating the common steps used in constructing a model, some of the \
choices that must be made are pointed out.  Also, the display of intermediate \
results may aid those constructing their own panel methods."
}], "Text",
 CellID->1534169418],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "Text",
 CellID->585195885],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->20779588],

Cell[CellGroupData[{

Cell["3.3.1  Eppler-205 Airfoil", "Section",
 CellID->491421075],

Cell[TextData[{
 "The data file for the Eppler-205 airfoil has been downloaded from the UIUC \
Airfoil Data Base to the Airfoil Data folder in the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " software.  Use ",
 Cell[BoxData[
  ButtonBox["AirfoilFromDataFile",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilFromDataFile"]], 
  "InlineFormula"],
 " to import data from the file and rearrange the data to the form used in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellID->323137],

Cell[BoxData[
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<Aerodynamics\\\\Airfoil Data\\\\e205.dat\>\"", "]"}]}], 
  ";"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->543466123],

Cell["Count the number of vectors in the file.", "Text",
 CellID->173149258],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NumberOfVectors", "[", "rD", "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->416922631],

Cell[BoxData["61"], "Output",
 ImageSize->{18, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->721774173]
}, Open  ]],

Cell[TextData[{
 "Use ",
 Cell[BoxData[
  ButtonBox["FitAirfoil",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/FitAirfoil"]], "InlineFormula"],
 " to fit a B-spline to the data and check visually that the spline fits the \
data."
}], "Text",
 CellID->193395042],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"spline", "=", 
   RowBox[{"FitAirfoil", "[", "rD", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gD", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gSpline", "=", 
   RowBox[{"ParametricPlot", "[", 
    RowBox[{
     RowBox[{"spline", "[", "t", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "0", ",", "1"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gD", ",", "gSpline"}], "]"}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->88849844],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3f+PFPUdx/Fr+pf037hEueMOFAk9D4vYWmrhZAtt1Wo0MagRalsIQrrx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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->633905778]
}, Open  ]],

Cell[TextData[{
 "Specify  discretization parameters, compute panel endpoints using ",
 Cell[BoxData[
  ButtonBox["AirfoilSurfacePoints",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilSurfacePoints"]], 
  "InlineFormula"],
 ", and plot the discretized airfoil."
}], "Text",
 CellID->135072324],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"n", "=", "3"}], ";", 
  RowBox[{"layout", "=", "HalfCosineSpacing"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"spline", ",", 
     RowBox[{"NDiscretizeUnitSegment", "[", 
      RowBox[{"n", ",", 
       RowBox[{"Layout", "\[Rule]", "layout"}]}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"PlotAirfoil", "[", "rP", "]"}]}], "Input",
 CellLabel->"In[7]:=",
 CellID->25651006],

Cell[BoxData[
 GraphicsBox[
  {Hue[0.67, 0.6, 0.6], 
   LineBox[{{1.0000000000000004`, 6.564562253346912*^-18}, {
    0.5000093318158763, -0.017261350440290774`}, {
    0.13399064407280628`, -0.023909975303061548`}, {0.00001858734936400401, 
    0.0006856384489956171}, {0.1339804611392721, 0.0632280031927559}, {
    0.5000132846042157, 0.07155895753461129}, {0.9999999999999993, 
    9.539953333063768*^-17}}]},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2000000000000004`, 
      FormBox["1.2000000000000004`", TraditionalForm]}, {1.4000000000000001`, 
      FormBox["1.4000000000000001`", TraditionalForm]}, {1.6, 
      FormBox["1.6`", TraditionalForm]}}, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}, None, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}},
  ImageSize->{504., 180.},
  ImageSizeRaw->{504, 180},
  PlotRange->{{-0.05, 1.05}, {-0.1, 0.201}},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->191119674]
}, Open  ]],

Cell["Specify an angle of attack.", "Text",
 CellID->203229668],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Alpha]", "=", 
   RowBox[{"6.", "Degree"}]}], ";"}]], "Input",
 CellLabel->"In[10]:=",
 CellID->254593671]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
3.3.2  Panel method using vortex panels of linearly varying strength and \
exterior Neumann boundary conditions\
\>", "Section",
 CellID->336729780],

Cell[TextData[{
 "The discretized airfoil is specified by ",
 Cell[BoxData[
  StyleBox["N",
   FontFamily->"Times New Roman",
   FontSize->12,
   FontSlant->"Italic"]], "InlineFormula"],
 " straight-line panels, where the circulation density for each panel varies \
linearly with distance from the beginning of the panel.  The clockwise \
numbering convention is used to determine the beginning of a panel."
}], "Text",
 CellID->479438142],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["\[Gamma]", "TR"], 
    RowBox[{"(", 
     SubscriptBox[
      StyleBox["x", "TI"], 
      StyleBox["j", "TI"]], ")"}]}], 
   StyleBox["=",
    FontFamily->"Courier New",
    FontSize->12], 
   RowBox[{
    SubscriptBox[
     StyleBox["a", "TI"], 
     StyleBox["j", "TI"]], "+", 
    RowBox[{
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["j", "TI"]], " ", 
     SubscriptBox[
      StyleBox["x", "TI"], 
      StyleBox["j", "TI"]], " "}]}]}]], "InlineFormula"],
 "  for  ",
 Cell[BoxData[
  StyleBox[
   RowBox[{
    RowBox[{
     StyleBox["j", "TI"], "=", "1"}], ",", "2", ",", "3", ",", 
    StyleBox["\[Ellipsis]", "TR"], ",", " ", 
    StyleBox["N",
     FontSlant->"Italic"], " "}],
   FontFamily->"Times New Roman",
   FontSize->12]], "InlineFormula"]
}], "Text",
 CellID->111507038],

Cell[TextData[{
 "Strength parameters are denoted by ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["j", "TI"]]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["b", "TI"], 
   StyleBox["j", "TI"]]], "InlineFormula"],
 " and the panel-local coordinate is denoted by ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["x", "TI"], 
   StyleBox["j", "TI"]]], "InlineFormula"],
 ".  Strength parameters for the discretized airfoil are grouped as lists."
}], "Text",
 CellID->101991089],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["a", "TI"], "=", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["1", "TR"]], ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["2", "TR"]], ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["3", "TR",
       FontFamily->"Times New Roman",
       FontSize->9]], ",", 
     StyleBox["\[Ellipsis]", "TR"], ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["N", "TI"]]}], ")"}]}]], "InlineFormula"],
 ",  ",
 Cell[BoxData[
  RowBox[{
   StyleBox["b", "TI"], "=", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["1", "TR"]], ",", 
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["2", "TR"]], ",", 
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["3", "TR"]], ",", 
     StyleBox["\[Ellipsis]", "TR"], ",", 
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["N", "TI"]]}], ")"}]}]], "InlineFormula"]
}], "Text",
 CellID->104238284],

Cell[TextData[{
 "There are ",
 Cell[BoxData[
  StyleBox[
   RowBox[{"2", 
    StyleBox["N",
     FontSlant->"Italic"]}],
   FontFamily->"Times New Roman",
   FontSize->12]], "InlineFormula"],
 " unknown strength parameters in the formulation of this model."
}], "Text",
 CellID->547624004],

Cell[TextData[{
 "Boundary conditions at collocation points provide ",
 Cell[BoxData[
  StyleBox["N",
   FontFamily->"Times New Roman",
   FontSize->12,
   FontSlant->"Italic"]], "InlineFormula"],
 " equations, continuity of circulation density between panels (except at \
trailing edge) provide ",
 Cell[BoxData[
  RowBox[{
   StyleBox["N",
    FontFamily->"Times New Roman",
    FontSize->12,
    FontSlant->"Italic"], "-", "1"}]], "InlineFormula"],
 " equations, and a Kutta condition (zero circulation density at trailing \
edge) provides the final equation for a system of ",
 Cell[BoxData[
  StyleBox[
   RowBox[{"2", 
    StyleBox["N",
     FontSlant->"Italic"]}],
   FontFamily->"Times New Roman",
   FontSize->12]], "InlineFormula"],
 " linear algebraic equations involving the unknown strength parameters.  The \
system of equations is solved using the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function LinearSolve."
}], "Text",
 CellID->375367223],

Cell[CellGroupData[{

Cell["Step-by-step implementation", "Subsection",
 CellID->725608851],

Cell[CellGroupData[{

Cell["Airfoil geometry", "Subsubsection",
 CellID->225787759],

Cell["\<\
Geometric quantities for the discretized airfoil are determined from panel \
endpoints.\
\>", "Text",
 CellID->472269960],

Cell["Compute panel lengths.", "Text",
 CellID->74064168],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Delta]", "=", 
  RowBox[{"PanelLengths", "[", "rP", "]"}]}]], "Input",
 CellLabel->"In[11]:=",
 CellID->106567359],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.5002885392353391`", ",", "0.36607906794806616`", ",", 
   "0.13621107223184767`", ",", "0.14784225044636196`", ",", 
   "0.36612761798854326`", ",", "0.5050815775453552`"}], "}"}]], "Output",
 ImageSize->{409, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->494467118]
}, Open  ]],

Cell["Count the panels.", "Text",
 CellID->10851564],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"nP", "=", 
  RowBox[{"Length", "[", "\[Delta]", "]"}]}]], "Input",
 CellLabel->"In[12]:=",
 CellID->152897124],

Cell[BoxData["6"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->436402049]
}, Open  ]],

Cell["Compute outward unit-normal vectors to panels.", "Text",
 CellID->162262132],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uvn", "=", 
  RowBox[{"PanelNormals", "[", "rP", "]"}]}]], "Input",
 CellLabel->"In[13]:=",
 CellID->691393489],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0.03450279006325772`", ",", "0.018161718177544046`", ",", 
     RowBox[{"-", "0.1805698563931166`"}], ",", 
     RowBox[{"-", "0.4230344475610578`"}], ",", 
     RowBox[{"-", "0.02275423631690108`"}], ",", "0.14167801938526525`"}], 
    "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.999404601489232`"}], ",", 
     RowBox[{"-", "0.9998350623942129`"}], ",", 
     RowBox[{"-", "0.9835621622257381`"}], ",", "0.9061136000395925`", ",", 
     "0.999741088847325`", ",", "0.9899127935444962`"}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{0.03450279006325772, 
    0.018161718177544046`, -0.1805698563931166, -0.4230344475610578, \
-0.02275423631690108, 
    0.14167801938526525`}, {-0.999404601489232, -0.9998350623942129, \
-0.9835621622257381, 0.9061136000395925, 0.999741088847325, 
    0.9899127935444962}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{455, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[13]=",
 CellID->292213793]
}, Open  ]],

Cell["\<\
Compute locations of collocations points, placing them just outside the \
discretized airfoil from panel centers.\
\>", "Text",
 CellID->11573225],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rC", "=", 
  RowBox[{
   RowBox[{"PanelPoints", "[", "rP", "]"}], "+", 
   RowBox[{
    RowBox[{"10.", "^", 
     RowBox[{"-", "10"}]}], " ", "uvn"}]}]}]], "Input",
 CellLabel->"In[14]:=",
 CellID->92036532],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "0.7500046659113886`", ",", "0.3169999879461574`", ",", 
     "0.06700461569302815`", ",", "0.06699952420201462`", ",", 
     "0.3169968728694685`", ",", "0.7500066423162752`"}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.008630675320085844`"}], ",", 
     RowBox[{"-", "0.020585662971659667`"}], ",", 
     RowBox[{"-", "0.011612168525389181`"}], ",", "0.03195682091148711`", ",",
      "0.0673934804636577`", ",", "0.035779478866296975`"}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{0.7500046659113886, 
    0.3169999879461574, 0.06700461569302815, 0.06699952420201462, 
    0.3169968728694685, 
    0.7500066423162752}, {-0.008630675320085844, -0.020585662971659667`, \
-0.011612168525389181`, 0.03195682091148711, 0.0673934804636577, 
    0.035779478866296975`}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{490, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[14]=",
 CellID->305679991]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Boundary conditions at collocation points", "Subsubsection",
 CellID->15365561],

Cell["\<\
The equations applying tangent-flow boundary condition at each collocation \
point can be written in matrix form.\
\>", "Text",
 CellID->412123985],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      StyleBox[
       SubscriptBox[
        StyleBox["C",
         FontWeight->"Bold"], "a"],
       FontWeight->"Bold"], ".", "a"}], "+", 
     RowBox[{
      StyleBox[
       SubscriptBox[
        StyleBox["C",
         FontWeight->"Bold"], "b"],
       FontWeight->"Bold"], ".", "b"}]}], "=", 
    RowBox[{"-", 
     SubscriptBox[
      StyleBox["V", "TI"], 
      RowBox[{
       StyleBox["n", "TI"], 
       StyleBox[",", "TI"], 
       StyleBox["\[Infinity]",
        FontFamily->"Times New Roman",
        FontSlant->"Plain"]}]]}]}], TraditionalForm]], "InlineMath"]
}], "Text",
 CellID->89575051],

Cell[TextData[{
 "The elements of matrices ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    SubscriptBox["C", "a"],
    FontWeight->"Bold"], TraditionalForm]], "InlineMath"],
 " and ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    SubscriptBox["C", "b"],
    FontWeight->"Bold"], TraditionalForm]], "InlineMath"],
 " consist of the components of velocity influence coefficients normal to \
panels of the discretized airfoil.  Each matrix has ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]], "InlineMath"],
 " rows and ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]], "InlineMath"],
 " columns; each row consists of influence coefficients at a given \
collocation point due to all panels.  The list of components of free-stream \
velocities normal to panels is denoted by ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["V", "TI"], 
   RowBox[{
    StyleBox["n", "TI"], 
    StyleBox[",", "TI"], "\[Infinity]"}]]]],
 ".  Each row of the system generates the equation for the tangent-flow \
boundary condition at one of the collocation points.  This matrix equation \
represents a system of ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]], "InlineMath"],
 " linear algebraic equations and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"2", " ", "N"}], TraditionalForm]], "InlineMath"],
 "  unknowns."
}], "Text",
 CellID->173755935],

Cell["\<\
Compute the velocity influence coefficients at collocation points due to \
vortex panels representing the discretized airfoil.  The output of this \
computation is suppressed because of its length, but its dimensions are \
displayed.  The collection of two-dimensional vectors associated with each \
list of strength parameters is arranged as a matrix with six rows and six \
columns. \
\>", "Text",
 CellID->23730492],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ica", ",", "icb"}], "}"}], "=", 
   RowBox[{"ICVortexLinear", "[", 
    RowBox[{"rC", ",", "rP"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Dimensions", "/@", 
  RowBox[{"{", 
   RowBox[{"ica", ",", "icb"}], "}"}]}]}], "Input",
 CellLabel->"In[15]:=",
 CellID->42090444],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", "6", ",", "6"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "6", ",", "6"}], "}"}]}], "}"}]], "Output",
 ImageSize->{143, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[16]=",
 CellID->304408583]
}, Open  ]],

Cell["\<\
Compute the normal-component velocity influence coefficients and display them \
in matrix form.  Chop is used to replace real numbers that are close to zero \
by the integer 0 to yield a compact display of the result.\
\>", "Text",
 CellID->407869361],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ican", "=", 
   RowBox[{"ica", ".", "uvn"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Chop", "[", "ican", "]"}], "//", "MatrixForm"}]}], "Input",
 CellLabel->"In[17]:=",
 CellID->151960291],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0.1434905470570162`", "0.03182522556147925`", 
      "0.0343002571511607`", "0.13644453301425113`", 
      RowBox[{"-", "0.07065016751806055`"}]},
     {
      RowBox[{"-", "0.20964945269279409`"}], "0", "0.08870083531125184`", 
      "0.08980076407306767`", 
      RowBox[{"-", "0.0011818443025962263`"}], 
      RowBox[{"-", "0.20031626344354467`"}]},
     {
      RowBox[{"-", "0.1202031631867313`"}], 
      RowBox[{"-", "0.2938696544822235`"}], "0", "0.14515006194726193`", 
      RowBox[{"-", "0.20952926880841838`"}], 
      RowBox[{"-", "0.11809884336227766`"}]},
     {"0.10664821495105584`", "0.19626605611761305`", 
      RowBox[{"-", "0.16395081748793716`"}], "0", "0.28031136151015845`", 
      "0.11271041332979466`"},
     {"0.19555295724804853`", 
      RowBox[{"-", "0.003894615082285717`"}], 
      RowBox[{"-", "0.07991041060288698`"}], 
      RowBox[{"-", "0.09497420342903377`"}], "0", "0.21063553658538386`"},
     {"0.07618679295765188`", 
      RowBox[{"-", "0.13610671456333492`"}], 
      RowBox[{"-", "0.031038220884357468`"}], 
      RowBox[{"-", "0.03417602802635723`"}], 
      RowBox[{"-", "0.1427021458090046`"}], "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 ImageSize->{497, 87},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[18]//MatrixForm=",
 CellID->288973517]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"icbn", "=", 
    RowBox[{"icb", ".", "uvn"}]}], ")"}], "//", "MatrixForm"}]], "Input",
 CellLabel->"In[19]:=",
 CellID->49864735],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0.0796233939415278`", "0.02237023762479765`", "0.0020961219911013167`",
       "0.0026115732494333796`", "0.02833386897983471`", 
      RowBox[{"-", "0.07809862871203926`"}]},
     {
      RowBox[{"-", "0.06363691885856688`"}], "0.058263293176408555`", 
      "0.005484528292589153`", "0.007041887107460467`", 
      RowBox[{"-", "0.02704648870819166`"}], 
      RowBox[{"-", "0.04123424465704047`"}]},
     {
      RowBox[{"-", "0.033900875335845404`"}], 
      RowBox[{"-", "0.06965671067115599`"}], "0.021678665399545743`", 
      "0.0011475893060461564`", 
      RowBox[{"-", "0.03173402713324316`"}], 
      RowBox[{"-", "0.026366376770058715`"}]},
     {"0.029844549372037812`", "0.042333098169608716`", 
      RowBox[{"-", "0.01829823993166699`"}], "0.023529824906368468`", 
      "0.03625384290775614`", "0.02461573302416074`"},
     {"0.05804854844289716`", 
      RowBox[{"-", "0.027532620694403546`"}], 
      RowBox[{"-", "0.005096977581238297`"}], 
      RowBox[{"-", "0.007707425120822556`"}], "0.058271020155337805`", 
      "0.041725267081807654`"},
     {
      RowBox[{"-", "0.04002446233017008`"}], 
      RowBox[{"-", "0.021417769470758116`"}], 
      RowBox[{"-", "0.0020496309513599615`"}], 
      RowBox[{"-", "0.0026147979127629736`"}], 
      RowBox[{"-", "0.029984371658085367`"}], "0.08038622968099575`"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 ImageSize->{504, 87},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[19]//MatrixForm=",
 CellID->47187414]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Panel-to-panel continuity of circulation density and the Kutta condition\
\>", "Subsubsection",
 CellID->103764251],

Cell["\<\
Continuity of circulation density from one panel to the next can be written \
in terms of the the strength parameters and panel lengths.  At the trailing \
edge, a Kutta condition is applied that the circulation density is zero.\
\>", "Text",
 CellID->386586113],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubscriptBox[
     StyleBox["a", "TI"], 
     StyleBox["j", "TI"]], "+", 
    RowBox[{
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["j", "TI"]], " ", 
     SubscriptBox[
      StyleBox["\[Delta]", "TR"], 
      StyleBox["j", "TI"]]}]}], "=", 
   SubscriptBox[
    StyleBox["a", "TI"], 
    StyleBox[
     RowBox[{"j", 
      StyleBox["+",
       FontSlant->"Plain"], 
      StyleBox["1",
       FontSlant->"Plain"]}], "TI"]]}]], "InlineFormula"],
 "  for  ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["j", "TI"], "=", 
    StyleBox["1",
     FontFamily->"Courier New"]}], 
   StyleBox[",",
    FontFamily->"Courier New"], 
   StyleBox["2",
    FontFamily->"Courier New"], 
   StyleBox[",",
    FontFamily->"Courier New"], 
   StyleBox["3",
    FontFamily->"Courier New"], 
   StyleBox[",",
    FontFamily->"Courier New"], "...", ",", " ", 
   RowBox[{
    StyleBox["N",
     FontSlant->"Italic"], 
    StyleBox["-",
     FontFamily->"Courier New"], "1"}]}]], "InlineFormula",
  FontFamily->"Times New Roman",
  FontSize->12]
}], "Text",
 CellID->750004683],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubscriptBox[
     StyleBox["a", "TI"], 
     StyleBox["N", "TI"]], "+", 
    RowBox[{
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["N", "TI"]], " ", 
     SubscriptBox[
      StyleBox["\[Delta]", "TR"], 
      StyleBox["N", "TI",
       FontFamily->"Times New Roman",
       FontSize->9,
       FontSlant->"Italic"]]}], "+", 
    SubscriptBox[
     StyleBox["a", "TI"], 
     StyleBox["1", "TR"]]}], "=", 
   StyleBox["0",
    FontFamily->"Times New Roman",
    FontSize->12]}]], "InlineFormula"]
}], "Text",
 CellID->123989520],

Cell["These equations can be written in matrix form.", "Text",
 CellID->503495065],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      StyleBox[
       SubscriptBox["M", "a"],
       FontWeight->"Bold"], ".", "a"}], "+", 
     RowBox[{
      StyleBox[
       SubscriptBox["M", "b"],
       FontWeight->"Bold"], ".", "b"}]}], "=", "0"}], TraditionalForm]], 
  "InlineMath"]
}], "Text",
 CellID->446965378],

Cell[TextData[{
 "Compute the matrix of coefficients for ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " in the continuity of value equation, replacing the last row by the Kutta \
condition."
}], "Text",
 CellID->531359183],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"matI", "=", 
   RowBox[{"IdentityMatrix", "[", "nP", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"(", 
   RowBox[{"mata", "=", 
    RowBox[{"ReplacePart", "[", 
     RowBox[{
      RowBox[{"matI", "-", 
       RowBox[{"RotateLeft", "[", "matI", "]"}]}], ",", "1", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}]}], ")"}], "//", 
  "MatrixForm"}]}], "Input",
 CellLabel->"In[20]:=",
 CellID->23909254],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", 
      RowBox[{"-", "1"}], "0", "0", "0", "0"},
     {"0", "1", 
      RowBox[{"-", "1"}], "0", "0", "0"},
     {"0", "0", "1", 
      RowBox[{"-", "1"}], "0", "0"},
     {"0", "0", "0", "1", 
      RowBox[{"-", "1"}], "0"},
     {"0", "0", "0", "0", "1", 
      RowBox[{"-", "1"}]},
     {"1", "0", "0", "0", "0", "1"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 ImageSize->{146, 87},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[21]//MatrixForm=",
 CellID->62951937]
}, Open  ]],

Cell[TextData[{
 "Compute the matrix of coefficients for ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 " in the continuity of value equation."
}], "Text",
 CellID->126776796],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"matb", "=", 
    RowBox[{"DiagonalMatrix", "[", "\[Delta]", "]"}]}], ")"}], "//", 
  "MatrixForm"}]], "Input",
 CellLabel->"In[22]:=",
 CellID->31347865],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0.5002885392353391`", "0.`", "0.`", "0.`", "0.`", "0.`"},
     {"0.`", "0.36607906794806616`", "0.`", "0.`", "0.`", "0.`"},
     {"0.`", "0.`", "0.13621107223184767`", "0.`", "0.`", "0.`"},
     {"0.`", "0.`", "0.`", "0.14784225044636196`", "0.`", "0.`"},
     {"0.`", "0.`", "0.`", "0.`", "0.36612761798854326`", "0.`"},
     {"0.`", "0.`", "0.`", "0.`", "0.`", "0.5050815775453552`"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 ImageSize->{400, 87},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[22]//MatrixForm=",
 CellID->450398653]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solution procedure", "Subsubsection",
 CellID->298127402],

Cell[TextData[{
 "Solve the continuity of value equations for ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 " in terms of ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 "."
}], "Text",
 CellID->122106569],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "=", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SuperscriptBox[
        StyleBox[
         SubscriptBox["M", "b"],
         FontWeight->"Bold"], 
        RowBox[{"-", "1"}]], ".", 
       StyleBox[
        SubscriptBox["M", "a"],
        FontWeight->"Bold"], ".", "a"}]}], "=", 
     RowBox[{
      StyleBox[
       SubscriptBox["M", 
        StyleBox["ba",
         FontSlant->"Italic"]],
       FontWeight->"Bold"], ".", "a"}]}]}], TraditionalForm]], "InlineMath"]
}], "Text",
 CellID->416517119],

Cell[TextData[{
 "Substitute into the system of equations for boundary conditions to \
eliminate ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 "."
}], "Text",
 CellID->27432560],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      StyleBox["(",
       FontWeight->"Bold"], 
      RowBox[{
       StyleBox[
        SubscriptBox[
         StyleBox["C",
          FontWeight->"Bold"], "a"],
        FontWeight->"Bold"], "+", 
       RowBox[{
        StyleBox[
         SubscriptBox[
          StyleBox["C",
           FontWeight->"Bold"], "b"],
         FontWeight->"Bold"], ".", 
        StyleBox[
         SubscriptBox["M", 
          StyleBox["ba",
           FontSlant->"Italic"]],
         FontWeight->"Bold"]}]}], 
      StyleBox[")",
       FontWeight->"Bold"]}], ".", "a"}], "=", 
    RowBox[{"-", 
     SubscriptBox[
      StyleBox["V", "TI"], 
      RowBox[{
       StyleBox["n", "TI"], 
       StyleBox[",", "TI"], "\[Infinity]"}]]}]}], TraditionalForm]], 
  "InlineMath"]
}], "Text",
 CellID->94688220],

Cell[TextData[{
 "Solve this system of ",
 Cell[BoxData[
  StyleBox["N",
   FontFamily->"Times New Roman",
   FontSize->12,
   FontSlant->"Italic"]], "InlineFormula"],
 " equations and ",
 Cell[BoxData[
  StyleBox["N",
   FontFamily->"Times New Roman",
   FontSize->12,
   FontSlant->"Italic"]], "InlineFormula"],
 " unknowns for ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ", then, solve for ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 " using ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "=", 
    RowBox[{
     StyleBox[
      SubscriptBox["M", 
       StyleBox["ba",
        FontSlant->"Italic"]],
      FontWeight->"Bold"], ".", "a"}]}], TraditionalForm]], "InlineMath"],
 "."
}], "Text",
 CellID->44848813],

Cell[TextData[{
 "Compute ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    StyleBox["M",
     FontWeight->"Bold"], 
    StyleBox["ba",
     FontSlant->"Italic"]], TraditionalForm]], "InlineMath"],
 ", noting that ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    SubscriptBox["M", "b"],
    FontWeight->"Bold"], TraditionalForm]], "InlineMath"],
 " is a diagonal matrix so its inverse is simple."
}], "Text",
 CellID->517009306],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"matba", "=", 
    RowBox[{"-", 
     RowBox[{
      RowBox[{"DiagonalMatrix", "[", 
       RowBox[{"1", "/", "\[Delta]"}], "]"}], ".", "mata"}]}]}], ")"}], "//", 
  "MatrixForm"}]], "Input",
 CellLabel->"In[23]:=",
 CellID->319384963],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", "1.9988465087136311`"}], "1.9988465087136311`", "0.`", 
      "0.`", "0.`", "0.`"},
     {"0.`", 
      RowBox[{"-", "2.7316503115164865`"}], "2.7316503115164865`", "0.`", 
      "0.`", "0.`"},
     {"0.`", "0.`", 
      RowBox[{"-", "7.341547082882363`"}], "7.341547082882363`", "0.`", "0.`"},
     {"0.`", "0.`", "0.`", 
      RowBox[{"-", "6.763966301790068`"}], "6.763966301790068`", "0.`"},
     {"0.`", "0.`", "0.`", "0.`", 
      RowBox[{"-", "2.7312880833570214`"}], "2.7312880833570214`"},
     {
      RowBox[{"-", "1.9798781908853174`"}], "0.`", "0.`", "0.`", "0.`", 
      RowBox[{"-", "1.9798781908853174`"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 ImageSize->{406, 87},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[23]//MatrixForm=",
 CellID->36808920]
}, Open  ]],

Cell["Compute the uniform flow normal to panels.", "Text",
 CellID->31359196],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Vn\[Infinity]", "=", 
  RowBox[{
   RowBox[{"UniformFlow", "[", 
    RowBox[{"nP", ",", "\[Alpha]"}], "]"}], ".", "uvn"}]}]], "Input",
 CellLabel->"In[24]:=",
 CellID->211953321],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "0.07015244700708635`"}], ",", 
   RowBox[{"-", "0.08644899620810997`"}], ",", 
   RowBox[{"-", "0.2823909171721261`"}], ",", 
   RowBox[{"-", "0.3260023584364338`"}], ",", "0.08187181345319944`", ",", 
   "0.24437595544925306`"}], "}"}]], "Output",
 ImageSize->{455, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[24]=",
 CellID->108384008]
}, Open  ]],

Cell[TextData[{
 "Solve the boundary-conditions equation for ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 "."
}], "Text",
 CellID->388832003],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a", "=", 
  RowBox[{"LinearSolve", "[", 
   RowBox[{
    RowBox[{"ican", "+", 
     RowBox[{"icbn", ".", "matba"}]}], ",", 
    RowBox[{"-", "Vn\[Infinity]"}]}], "]"}]}]], "Input",
 CellLabel->"In[25]:=",
 CellID->367664505],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "1.2435494755986152`"}], ",", 
   RowBox[{"-", "0.9088440472222551`"}], ",", 
   RowBox[{"-", "0.8222846153481762`"}], ",", "0.7132996637720803`", ",", 
   "1.53325933084721`", ",", "1.284845658670146`"}], "}"}]], "Output",
 ImageSize->{398, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[25]=",
 CellID->153732725]
}, Open  ]],

Cell[TextData[{
 "Compute ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 " from ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 "."
}], "Text",
 CellID->535259082],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"b", "=", 
  RowBox[{"matba", ".", "a"}]}]], "Input",
 CellLabel->"In[26]:=",
 CellID->28035240],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.6690247769575874`", ",", "0.23645009904351744`", ",", 
   "11.273564284895336`", ",", "5.54617955692318`", ",", 
   RowBox[{"-", "0.6784893025601724`"}], ",", 
   RowBox[{"-", "0.08176141223013111`"}]}], "}"}]], "Output",
 ImageSize->{411, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[26]=",
 CellID->161396544]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Compare with results using function PMVortexLinearNeumann", "Subsection",
 CellID->611074881],

Cell[TextData[{
 "Use the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["PMVortexLinearNeumann",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/PMVortexLinearNeumann"]], 
  "InlineFormula"],
 " to solve the model for this example.  Use the suffix VLN to denote results \
computed from the programmed model function."
}], "Text",
 CellID->19519575],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"aVLN", ",", "bVLN", ",", "rCVLN", ",", "vMagCVLN"}], "}"}], "=", 
   RowBox[{"PMVortexLinearNeumann", "[", 
    RowBox[{"rP", ",", "\[Alpha]"}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[27]:=",
 CellID->50303841],

Cell["\<\
Verify that the results of the step-by-step implementation are the same as \
those from the model PMVortexLinearNeumann.\
\>", "Text",
 CellID->68346340],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", "==", "aVLN"}], ",", 
   RowBox[{"b", "==", "bVLN"}]}], "}"}]], "Input",
 CellLabel->"In[28]:=",
 CellID->514967147],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[28]=",
 CellID->44462280]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
3.3.3 Panel method using doublet panels of constant strength and interior \
Dirichlet boundary conditions\
\>", "Section",
 CellID->577981106],

Cell[TextData[{
 "The discretized airfoil is specified by ",
 Cell[BoxData[
  StyleBox[
   StyleBox["N", "TI"],
   FontSlant->"Italic"]], "InlineFormula"],
 " straight-line panels, where the doublet density for each panel is constant."
}], "Text",
 CellID->3302604],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["\[Mu]", "TR"], 
    RowBox[{"(", 
     SubscriptBox[
      StyleBox["x", "TI"], 
      StyleBox["j", "TI"]], ")"}]}], "=", 
   SubscriptBox[
    StyleBox["a", "TI"], 
    StyleBox["j", "TI"]]}]], "InlineFormula"],
 " for ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["j", "TI"], 
    StyleBox[
     RowBox[{"=", "1"}],
     FontFamily->"Times New Roman",
     FontSize->12]}], 
   StyleBox[",",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox["2",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox[",",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox["3",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox[",",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox["...",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox[",",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox["N", "TI"]}]], "InlineFormula"]
}], "Text",
 CellID->181147249],

Cell[TextData[{
 "Strength parameters are denoted by ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["j", "TI"]]], "InlineFormula"],
 ", and the collection of strength parameters for the discretized airfoil are \
grouped as a list."
}], "Text",
 CellID->260469098],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["a", "TI"], "=", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["1", "TR"]], ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["2", "TR"]], ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["3", "TR"]], ",", 
     StyleBox["\[Ellipsis]", "TR"], ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["N", "TI",
       FontFamily->"Times New Roman",
       FontSize->9,
       FontSlant->"Italic"]]}], ")"}]}]], "InlineFormula"]
}], "Text",
 CellID->295014137],

Cell[TextData[{
 "A wake panel of strength ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["w", "TI"]]], "InlineFormula"],
 " that extends from the trailing edge of the airfoil far downstream along \
the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis is appended to ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ".  There are thus ",
 Cell[BoxData[
  RowBox[{
   StyleBox[
    StyleBox["N", "TI"],
    FontSlant->"Italic"], "+", 
   StyleBox["1",
    FontFamily->"Times New Roman",
    FontSize->12]}]], "InlineFormula"],
 " unknown strength parameters in the formulation of this model."
}], "Text",
 CellID->409613708],

Cell[TextData[{
 "Boundary conditions at collocation points provide ",
 Cell[BoxData[
  StyleBox[
   StyleBox["N", "TI"],
   FontSlant->"Italic"]], "InlineFormula"],
 " equations, and the Kutta condition (zero circulation at trailing edge) \
provides the final equation for a system of ",
 Cell[BoxData[
  RowBox[{
   StyleBox[
    StyleBox["N", "TI"],
    FontSlant->"Italic"], 
   StyleBox["+",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox["1",
    FontFamily->"Times New Roman",
    FontSize->12]}]], "InlineFormula"],
 " linear algebraic equations involving the unknown strength parameters.  The \
system of equations is solved using the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function LinearSolve."
}], "Text",
 CellID->191057130],

Cell[CellGroupData[{

Cell["Step-by-step implementation", "Subsection",
 CellID->104432088],

Cell[CellGroupData[{

Cell["Airfoil geometry", "Subsubsection",
 CellID->80757491],

Cell["\<\
The example discretized airfoil for this model is the same as that used in \
section 3.3.2.  Note however, that this model uses interior Dirichlet \
boundary conditions so collocation points are placed just inside the airfoil.\
\>", "Text",
 CellID->217356065],

Cell["Panel lengths.", "Text",
 CellID->176816722],

Cell[CellGroupData[{

Cell[BoxData["\[Delta]"], "Input",
 CellLabel->"In[29]:=",
 CellID->100724537],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.5002885392353391`", ",", "0.36607906794806616`", ",", 
   "0.13621107223184767`", ",", "0.14784225044636196`", ",", 
   "0.36612761798854326`", ",", "0.5050815775453552`"}], "}"}]], "Output",
 ImageSize->{409, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[29]=",
 CellID->88793639]
}, Open  ]],

Cell["Number of panels.", "Text",
 CellID->25165121],

Cell[CellGroupData[{

Cell[BoxData["nP"], "Input",
 CellLabel->"In[30]:=",
 CellID->603369164],

Cell[BoxData["6"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[30]=",
 CellID->427119333]
}, Open  ]],

Cell["Outward unit-normal vectors to panels.", "Text",
 CellID->513602593],

Cell[CellGroupData[{

Cell[BoxData["uvn"], "Input",
 CellLabel->"In[31]:=",
 CellID->630817148],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0.03450279006325772`", ",", "0.018161718177544046`", ",", 
     RowBox[{"-", "0.1805698563931166`"}], ",", 
     RowBox[{"-", "0.4230344475610578`"}], ",", 
     RowBox[{"-", "0.02275423631690108`"}], ",", "0.14167801938526525`"}], 
    "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.999404601489232`"}], ",", 
     RowBox[{"-", "0.9998350623942129`"}], ",", 
     RowBox[{"-", "0.9835621622257381`"}], ",", "0.9061136000395925`", ",", 
     "0.999741088847325`", ",", "0.9899127935444962`"}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{0.03450279006325772, 
    0.018161718177544046`, -0.1805698563931166, -0.4230344475610578, \
-0.02275423631690108, 
    0.14167801938526525`}, {-0.999404601489232, -0.9998350623942129, \
-0.9835621622257381, 0.9061136000395925, 0.999741088847325, 
    0.9899127935444962}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{455, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[31]=",
 CellID->369335311]
}, Open  ]],

Cell["\<\
Compute locations of collocation points, placing them just inside the \
discretized airfoil from panel centers.\
\>", "Text",
 CellID->360726760],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rC", "=", 
  RowBox[{
   RowBox[{"PanelPoints", "[", "rP", "]"}], "-", 
   RowBox[{
    RowBox[{"10.", "^", 
     RowBox[{"-", "10"}]}], " ", "uvn"}]}]}]], "Input",
 CellLabel->"In[32]:=",
 CellID->5240558],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "0.7500046659044881`", ",", "0.3169999879425251`", ",", 
     "0.06700461572914213`", ",", "0.0669995242866215`", ",", 
     "0.3169968728740193`", ",", "0.7500066422879397`"}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.008630675120204923`"}], ",", 
     RowBox[{"-", "0.020585662771692652`"}], ",", 
     RowBox[{"-", "0.01161216832867675`"}], ",", "0.031956820730264394`", ",",
      "0.06739348026370948`", ",", "0.035779478668314414`"}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{0.7500046659044881, 
    0.3169999879425251, 0.06700461572914213, 0.0669995242866215, 
    0.3169968728740193, 
    0.7500066422879397}, {-0.008630675120204923, -0.020585662771692652`, \
-0.01161216832867675, 0.031956820730264394`, 0.06739348026370948, 
    0.035779478668314414`}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{490, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[32]=",
 CellID->85891681]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Boundary conditions at collocation points, and Kutta condition\
\>", "Subsubsection",
 CellID->292614508],

Cell[TextData[{
 "Apply the boundary condition that the velocity potential is zero at each \
collocation point; the system of panels includes the wake panel.  An \
additional equation is added for the Kutta condition.  This system of \
equations can be written in matrix form.  In the following equation, the \
strength parameter for the wake panel is appended to the list ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " and renamed as the double-struck character ",
 Cell[BoxData[
  StyleBox["\[DoubleStruckA]",
   FontFamily->"Times New Roman",
   FontSize->12]], "InlineFormula"],
 "."
}], "Text",
 CellID->104611666],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     StyleBox[
      SubscriptBox["C", 
       StyleBox["\[DoubleStruckA]",
        FontFamily->"Times New Roman",
        FontSize->9,
        FontSlant->"Plain"]],
      FontWeight->"Bold"], ".", 
     StyleBox["\[DoubleStruckA]",
      FontFamily->"Times New Roman",
      FontSize->12,
      FontSlant->"Plain"]}], "=", 
    StyleBox["rhs",
     FontFamily->"Times New Roman",
     FontSize->12,
     FontSlant->"Italic"]}], TraditionalForm]], "InlineMath"]
}], "Text",
 CellID->80393232],

Cell[TextData[{
 "The ",
 Cell[BoxData[
  StyleBox[
   RowBox[{
    StyleBox["N",
     FontSlant->"Italic"], "+", "1"}],
   FontFamily->"Times New Roman",
   FontSize->12]], "InlineFormula"],
 " by ",
 Cell[BoxData[
  RowBox[{
   StyleBox["N",
    FontFamily->"Times New Roman",
    FontSize->12,
    FontSlant->"Italic"], "+", 
   StyleBox["1",
    FontFamily->"Times New Roman",
    FontSize->12]}]], "InlineFormula"],
 " matrix ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    SubscriptBox["C", 
     StyleBox["\[DoubleStruckA]",
      FontFamily->"Times New Roman",
      FontSize->9,
      FontSlant->"Plain"]],
    FontWeight->"Bold"], TraditionalForm]], "InlineMath"],
 " is the matrix of velocity-potential influence coefficients to which a row \
has been appended for the Kutta condition.  ",
 "The Kutta condition imposed is based on the equivalence of a \
constant-strength doublet panel to a pair of contra-rotating point vortices \
at panel endpoints.  The relationship between circulation density and doublet \
density is discussed in more detail in ",
 ButtonBox["Section 2.1",
  BaseStyle->"Hyperlink",
  ButtonData->{"IntroductionToInfluenceCoefficients.nb", "relation"}],
 "."
}], "Text",
 CellID->177985028],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["a", "N"], "-", 
     SubscriptBox["a", 
      StyleBox["w",
       FontFamily->"Times New Roman",
       FontSize->9]], "-", 
     SubscriptBox["a", "1"]}], "=", "0"}], TraditionalForm]], "InlineMath"]
}], "Text",
 CellID->196488211],

Cell[TextData[{
 "The right-hand-side of the matrix equation (",
 Cell[BoxData[
  StyleBox["rhs",
   FontFamily->"Times New Roman",
   FontSize->12,
   FontSlant->"Italic"]], "InlineFormula"],
 ") is the negative of the list of free-stream velocity potentials at \
collocation points to which a zero has been appended."
}], "Text",
 CellID->120707751],

Cell["\<\
Compute the velocity-potential influence coefficients at collocation points \
due to the doublet panels representing the discretized airfoil and the wake \
panel.  Display the dimensions of the resulting matrix.\
\>", "Text",
 CellID->128200992],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ic\[Phi]", "=", 
   RowBox[{"ICPhiDoubletConstant", "[", 
    RowBox[{"rC", ",", 
     RowBox[{"Append", "[", 
      RowBox[{"rP", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"10.", "^", "10"}], ",", "0.0"}], "}"}]}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Dimensions", "[", "ic\[Phi]", "]"}]}], "Input",
 CellLabel->"In[33]:=",
 CellID->9956353],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"6", ",", "7"}], "}"}]], "Output",
 ImageSize->{43, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[34]=",
 CellID->273744321]
}, Open  ]],

Cell["Compute the coefficients for the Kutta condition.", "Text",
 CellID->790473042],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kutta", "=", 
  RowBox[{"Flatten", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "1."}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{"0.", ",", 
       RowBox[{"{", 
        RowBox[{"nP", "-", "2"}], "}"}]}], "]"}], ",", "1.", ",", 
     RowBox[{"-", "1."}]}], "}"}], "]"}]}]], "Input",
 CellLabel->"In[35]:=",
 CellID->167614302],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "1.`"}], ",", "0.`", ",", "0.`", ",", "0.`", ",", "0.`", ",", 
   "1.`", ",", 
   RowBox[{"-", "1.`"}]}], "}"}]], "Output",
 ImageSize->{198, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[35]=",
 CellID->695483810]
}, Open  ]],

Cell["\<\
Append the row expressing the Kutta condition to the matrix of influence \
coefficients, and display the result in matrix form.\
\>", "Text",
 CellID->54865923],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"mat", "=", 
    RowBox[{"Append", "[", 
     RowBox[{"ic\[Phi]", ",", "kutta"}], "]"}]}], ")"}], "//", 
  "MatrixForm"}]], "Input",
 CellLabel->"In[36]:=",
 CellID->87445909],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0.49999999987274946`", "0.0015455902883346497`", 
      "0.005923707618673915`", "0.016504839930439844`", 
      "0.030920078183353603`", "0.4451057841064485`", "0.005492379620102115`"},
     {"0.0019048020419853296`", "0.4999999998260977`", 
      "0.013554922888940883`", "0.057684038193349794`", 
      "0.35741669280338856`", "0.0694395442462377`", "0.004795488107070001`"},
     {"0.004057050586950521`", "0.026820802845580367`", 
      "0.49999999953262264`", "0.3372860896294975`", "0.10361460625757093`", 
      "0.028221451147778076`", "0.0019807585337394337`"},
     {"0.012563886285040913`", "0.09261522905682606`", "0.3198541553449522`", 
      "0.49999999956939256`", "0.055002086470351534`", 
      "0.019964643273436763`", 
      RowBox[{"-", "0.005449191673949344`"}]},
     {"0.053300724196137995`", "0.35739292598749606`", "0.04064053368295851`",
       "0.029390543089923914`", "0.49999999982612076`", 
      "0.019275273217362856`", 
      RowBox[{"-", "0.01565351136633488`"}]},
     {"0.44410132300664973`", "0.017900318746378263`", 
      "0.007931641394620098`", "0.014528677413945854`", 
      "0.015538039564449095`", "0.49999999987395705`", 
      RowBox[{"-", "0.022624882283626357`"}]},
     {
      RowBox[{"-", "1.`"}], "0.`", "0.`", "0.`", "0.`", "1.`", 
      RowBox[{"-", "1.`"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 ImageSize->{549, 102},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[36]//MatrixForm=",
 CellID->34198324]
}, Open  ]],

Cell["\<\
Compute the free stream velocity potential at collocation points and append a \
zero for the Kutta condition equation.\
\>", "Text",
 CellID->134373867],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rhs", "=", 
  RowBox[{"-", 
   RowBox[{"Append", "[", 
    RowBox[{
     RowBox[{"UniformFlowPhi", "[", 
      RowBox[{"rC", ",", "\[Alpha]"}], "]"}], ",", "0.0"}], 
    "]"}]}]}]], "Input",
 CellLabel->"In[37]:=",
 CellID->134984038],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "0.7449939106631026`"}], ",", 
   RowBox[{"-", "0.31311164114544865`"}], ",", 
   RowBox[{"-", "0.06542375532276734`"}], ",", 
   RowBox[{"-", "0.0699728912441579`"}], ",", 
   RowBox[{"-", "0.3223048677627097`"}], ",", 
   RowBox[{"-", "0.749638001348713`"}], ",", "0.`"}], "}"}]], "Output",
 ImageSize->{496, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[37]=",
 CellID->199300039]
}, Open  ]],

Cell["Solve the system of equations.", "Text",
 CellID->6313767],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"soln", "=", 
  RowBox[{"LinearSolve", "[", 
   RowBox[{"mat", ",", "rhs"}], "]"}]}]], "Input",
 CellLabel->"In[38]:=",
 CellID->173842100],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "0.6108118179607713`"}], ",", 
   RowBox[{"-", "0.17853903916694905`"}], ",", "0.060338529923754866`", ",", 
   RowBox[{"-", "0.04875947697624153`"}], ",", 
   RowBox[{"-", "0.42789853089855373`"}], ",", 
   RowBox[{"-", "0.9520430904670479`"}], ",", 
   RowBox[{"-", "0.34123127250627666`"}]}], "}"}]], "Output",
 ImageSize->{538, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[38]=",
 CellID->86588103]
}, Open  ]],

Cell["\<\
Separate the strength parameters for the discretized airfoil from the \
strength parameter for the wake panel.\
\>", "Text",
 CellID->201664465],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"a", ",", "aW"}], "}"}], "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Most", "[", "soln", "]"}], ",", 
    RowBox[{"Last", "[", "soln", "]"}]}], "}"}]}]], "Input",
 CellLabel->"In[39]:=",
 CellID->51826491],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.6108118179607713`"}], ",", 
     RowBox[{"-", "0.17853903916694905`"}], ",", "0.060338529923754866`", ",", 
     RowBox[{"-", "0.04875947697624153`"}], ",", 
     RowBox[{"-", "0.42789853089855373`"}], ",", 
     RowBox[{"-", "0.9520430904670479`"}]}], "}"}], ",", 
   RowBox[{"-", "0.34123127250627666`"}]}], "}"}]], "Output",
 ImageSize->{552, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[39]=",
 CellID->34079921]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Compare with results using function PMDoubletConstantDirichlet\
\>", "Subsection",
 CellID->37612957],

Cell[TextData[{
 "Use the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["PMDoubletConstantDirichlet",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/PMDoubletConstantDirichlet"]], 
  "InlineFormula"],
 " to solve the model for this example.  Use the suffix DCD to denote results \
computed from the programmed model function.  "
}], "Text",
 CellID->756079987],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"aDCD", ",", "aWDCD", ",", "rCDCD", ",", "vMagCDCD"}], "}"}], "=", 
   RowBox[{"PMDoubletConstantDirichlet", "[", 
    RowBox[{"rP", ",", "\[Alpha]"}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[40]:=",
 CellID->178842356],

Cell["\<\
Verify that the results of the step-by-step implementation are the same as \
those from the model PMDoubletConstantDirichlet.\
\>", "Text",
 CellID->289996779],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", "==", "aDCD"}], ",", 
   RowBox[{"aW", "==", "aWDCD"}]}], "}"}]], "Input",
 CellLabel->"In[41]:=",
 CellID->229212092],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True"}], "}"}]], "Output",
 ImageSize->{85, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[41]=",
 CellID->692095846]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
3.3.4 Panel method using doublet panels of quadratically varying strength and \
interior Dirichlet boundary conditions\
\>", "Section",
 CellID->106827055],

Cell[TextData[{
 "The discretized airfoil is specified by ",
 Cell[BoxData[
  StyleBox["N",
   FontFamily->"Times New Roman",
   FontSize->12,
   FontSlant->"Italic"]], "InlineFormula"],
 " straight-line panels, where the doublet density for each panel varies \
quadratically with distance from the beginning of the panel.  The clockwise \
numbering convention is used to determine the beginning of a panel."
}], "Text",
 CellID->784000936],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{"\[Mu]", 
    RowBox[{"(", 
     SubscriptBox[
      StyleBox["x", "TI"], 
      StyleBox["j", "TI"]], ")"}]}], "=", 
   RowBox[{
    SubscriptBox[
     StyleBox["a", "TI"], 
     StyleBox["j", "TI"]], "+", 
    RowBox[{
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["j", "TI"]], " ", 
     SubscriptBox[
      StyleBox["x", "TI"], 
      StyleBox["j", "TI"]]}], "+", 
    RowBox[{
     SubscriptBox[
      StyleBox["c", "TI"], 
      StyleBox["j", "TI"]], " ", 
     SuperscriptBox[
      SubscriptBox[
       StyleBox["x", "TI"], 
       StyleBox["j", "TI"]], "2"], " "}]}]}]], "InlineFormula"],
 "  for  ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["j", "TI"], "=", 
    StyleBox["1",
     FontFamily->"Times New Roman",
     FontSize->12]}], 
   StyleBox[",",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox["2",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox[",",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox["3",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox[",",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox["\[Ellipsis]", "TR",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox[",",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox["N",
    FontFamily->"Times New Roman",
    FontSize->12,
    FontSlant->"Italic"]}]], "InlineFormula"]
}], "Text",
 CellID->20572744],

Cell[TextData[{
 "Strength parameters are denoted by ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["j", "TI"]]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["b", "TI"], 
   StyleBox["j", "TI"]]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["c", "TI"], 
   StyleBox["j", "TI"]]], "InlineFormula"],
 " and the panel-local coordinate is denoted by ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["x", "TI"], 
   StyleBox["j", "TI"]]], "InlineFormula"],
 ".  Strength parameters for the discretized airfoil are grouped as lists."
}], "Text",
 CellID->321938409],

Cell[TextData[{
 "\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["a", "TI"], "=", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["1", "TR"]], ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["2", "TR"]], ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["3", "TR"]], ",", 
     StyleBox["\[Ellipsis]", "TR"], ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["N", "TI"]]}], "}"}]}]], "InlineFormula"],
 ",  ",
 Cell[BoxData[
  RowBox[{
   StyleBox["b", "TI"], "=", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["1", "TR"]], ",", 
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["2", "TR"]], ",", 
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["3", "TR"]], ",", 
     StyleBox["\[Ellipsis]", "TR"], ",", 
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["N", "TI"]]}], "}"}]}]], "InlineFormula"],
 ",  ",
 Cell[BoxData[
  RowBox[{
   StyleBox["c", "TI"], "=", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox[
      StyleBox["c", "TI"], 
      StyleBox["1", "TR"]], ",", 
     SubscriptBox[
      StyleBox["c", "TI"], 
      StyleBox["2", "TR"]], ",", 
     SubscriptBox[
      StyleBox["c", "TI"], 
      StyleBox["3", "TR"]], ",", 
     StyleBox["\[Ellipsis]", "TR"], ",", 
     SubscriptBox[
      StyleBox["c", "TI"], 
      StyleBox["N", "TI"]]}], "}"}]}]], "InlineFormula"]
}], "Text",
 CellID->206580932],

Cell[TextData[{
 "A wake panel of constant unkown strength ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["w", "TI"]]], "InlineFormula"],
 " extends from the trailing edge of the airfoil far downstream along the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis.  There are thus ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["3",
     FontFamily->"Times New Roman",
     FontSize->12], 
    StyleBox["N", "TI"]}], "+", 
   StyleBox["1",
    FontFamily->"Times New Roman",
    FontSize->12]}]], "InlineFormula"],
 " unknown strength parameters in the formulation of this model."
}], "Text",
 CellID->63853891],

Cell[TextData[{
 "Boundary conditions at collocation points provide ",
 Cell[BoxData[
  StyleBox["N", "TI"]], "InlineFormula"],
 " equations, continuity of value and slope of the doublet density between \
panels (except at trailing edge) provide ",
 Cell[BoxData[
  RowBox[{
   StyleBox["2",
    FontFamily->"Times New Roman",
    FontSize->12], 
   RowBox[{"(", 
    RowBox[{
     StyleBox["N", "TI"], "-", 
     StyleBox["1",
      FontFamily->"Times New Roman",
      FontSize->12]}], ")"}]}]], "InlineFormula"],
 " equations, and Kutta conditions provide ",
 StyleBox["2",
  FontFamily->"Times New Roman",
  FontSize->12],
 " equations (zero circulation and zero circulation density at trailing \
edge).  This makes a total of ",
 Cell[BoxData[
  RowBox[{
   StyleBox["3",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox["N", "TI"]}]], "InlineFormula"],
 " equations.  An additional equation is generated by adding a collocation \
point interior to the airfoil near the trailing edge.  The resulting system \
of linear algebraic equations is solved using the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function LinearSolve."
}], "Text",
 CellID->24556244],

Cell[CellGroupData[{

Cell["Step-by-step implementation", "Subsection",
 CellID->97066759],

Cell[CellGroupData[{

Cell["Airfoil geometry", "Subsubsection",
 CellID->135427747],

Cell["\<\
The discretized airfoil used as example for this model is the same as that \
used in section 3.3.3, including location of collocation points\
\>", "Text",
 CellID->189252988],

Cell["Panel lengths.", "Text",
 CellID->211654584],

Cell[CellGroupData[{

Cell[BoxData["\[Delta]"], "Input",
 CellLabel->"In[42]:=",
 CellID->391861234],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.5002885392353391`", ",", "0.36607906794806616`", ",", 
   "0.13621107223184767`", ",", "0.14784225044636196`", ",", 
   "0.36612761798854326`", ",", "0.5050815775453552`"}], "}"}]], "Output",
 ImageSize->{409, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[42]=",
 CellID->32025243]
}, Open  ]],

Cell["Number of panels.", "Text",
 CellID->653071787],

Cell[CellGroupData[{

Cell[BoxData["nP"], "Input",
 CellLabel->"In[43]:=",
 CellID->307414976],

Cell[BoxData["6"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[43]=",
 CellID->3898044]
}, Open  ]],

Cell["Outward unit-normal vectors to panels.", "Text",
 CellID->80868268],

Cell[CellGroupData[{

Cell[BoxData["uvn"], "Input",
 CellLabel->"In[44]:=",
 CellID->427036223],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0.03450279006325772`", ",", "0.018161718177544046`", ",", 
     RowBox[{"-", "0.1805698563931166`"}], ",", 
     RowBox[{"-", "0.4230344475610578`"}], ",", 
     RowBox[{"-", "0.02275423631690108`"}], ",", "0.14167801938526525`"}], 
    "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.999404601489232`"}], ",", 
     RowBox[{"-", "0.9998350623942129`"}], ",", 
     RowBox[{"-", "0.9835621622257381`"}], ",", "0.9061136000395925`", ",", 
     "0.999741088847325`", ",", "0.9899127935444962`"}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{0.03450279006325772, 
    0.018161718177544046`, -0.1805698563931166, -0.4230344475610578, \
-0.02275423631690108, 
    0.14167801938526525`}, {-0.999404601489232, -0.9998350623942129, \
-0.9835621622257381, 0.9061136000395925, 0.999741088847325, 
    0.9899127935444962}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{455, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[44]=",
 CellID->369764010]
}, Open  ]],

Cell["Locations of collocation points.", "Text",
 CellID->421281807],

Cell[CellGroupData[{

Cell[BoxData["rC"], "Input",
 CellLabel->"In[45]:=",
 CellID->342567280],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "0.7500046659044881`", ",", "0.3169999879425251`", ",", 
     "0.06700461572914213`", ",", "0.0669995242866215`", ",", 
     "0.3169968728740193`", ",", "0.7500066422879397`"}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.008630675120204923`"}], ",", 
     RowBox[{"-", "0.020585662771692652`"}], ",", 
     RowBox[{"-", "0.01161216832867675`"}], ",", "0.031956820730264394`", ",",
      "0.06739348026370948`", ",", "0.035779478668314414`"}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{0.7500046659044881, 
    0.3169999879425251, 0.06700461572914213, 0.0669995242866215, 
    0.3169968728740193, 
    0.7500066422879397}, {-0.008630675120204923, -0.020585662771692652`, \
-0.01161216832867675, 0.031956820730264394`, 0.06739348026370948, 
    0.035779478668314414`}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{490, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[45]=",
 CellID->255597634]
}, Open  ]],

Cell["\<\
Locate an extra collocation point at the mean location of the collocation \
points for the first and last panels.  Append it to the list of collocation \
points for the discretized airfoil.\
\>", "Text",
 CellID->18288899],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xCExtra", ",", "zCExtra"}], "}"}], "=", 
   RowBox[{"Mean", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"First", "[", "rC", "]"}], ",", 
      RowBox[{"Last", "[", "rC", "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"rCAll", "=", 
  RowBox[{"Append", "[", 
   RowBox[{"rC", ",", 
    RowBox[{"{", 
     RowBox[{"xCExtra", ",", "zCExtra"}], "}"}]}], "]"}]}]}], "Input",
 CellLabel->"In[46]:=",
 CellID->164764330],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "0.7500046659044881`", ",", "0.3169999879425251`", ",", 
     "0.06700461572914213`", ",", "0.0669995242866215`", ",", 
     "0.3169968728740193`", ",", "0.7500066422879397`", ",", 
     "0.7500056540962139`"}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.008630675120204923`"}], ",", 
     RowBox[{"-", "0.020585662771692652`"}], ",", 
     RowBox[{"-", "0.01161216832867675`"}], ",", "0.031956820730264394`", ",",
      "0.06739348026370948`", ",", "0.035779478668314414`", ",", 
     "0.013574401774054745`"}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{0.7500046659044881, 
    0.3169999879425251, 0.06700461572914213, 0.0669995242866215, 
    0.3169968728740193, 0.7500066422879397, 
    0.7500056540962139}, {-0.008630675120204923, -0.020585662771692652`, \
-0.01161216832867675, 0.031956820730264394`, 0.06739348026370948, 
    0.035779478668314414`, 0.013574401774054745`}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{564, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[47]=",
 CellID->406643534]
}, Open  ]],

Cell["\<\
Compute the locations of the endpoints for the constant-strength wake panel.\
\>", "Text",
 CellID->102672742],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rPWake", "=", 
  RowBox[{"Take", "[", 
   RowBox[{
    RowBox[{"Append", "[", 
     RowBox[{"rP", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"10.", "^", "10"}], ",", "0."}], "}"}]}], "]"}], ",", 
    RowBox[{"-", "2"}]}], "]"}]}]], "Input",
 CellLabel->"In[48]:=",
 CellID->240976878],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0.9999999999999993`", ",", "1.`*^10"}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{"9.539953333063768`*^-17", ",", "0.`"}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{0.9999999999999993, 
    1.*^10}, {9.539953333063768*^-17, 0.}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{231, 19},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[48]=",
 CellID->31033561]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\

Boundary conditions at collocation points\
\>", "Subsubsection",
 CellID->306855904],

Cell[TextData[{
 "Apply the boundary condition that the velocity potential is zero at each \
collocation point due to all doublet panels.  This collection of equations \
can be written in matrix form.  In the following equation, the strength \
parameter for the wake panel is appended to to the list ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " and renamed as the double-struck character ",
 Cell[BoxData[
  StyleBox["\[DoubleStruckA]",
   FontSize->14]], "InlineFormula"],
 "."
}], "Text",
 CellID->1539757],

Cell[TextData[{
 "\t\t",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      StyleBox[
       SubscriptBox["C", "\[DoubleStruckA]"],
       FontWeight->"Bold"], ".", "\[DoubleStruckA]"}], "+", 
     RowBox[{
      StyleBox[
       SubscriptBox[
        StyleBox["C",
         FontWeight->"Bold"], "b"],
       FontWeight->"Bold"], ".", "b"}], "+", 
     RowBox[{
      StyleBox[
       SubscriptBox[
        StyleBox["C",
         FontWeight->"Bold"], "c"],
       FontWeight->"Bold"], ".", "c"}]}], "=", 
    RowBox[{
     StyleBox["-",
      FontSlant->"Italic"], 
     SubscriptBox["\[Phi]", "\[Infinity]"]}]}], TraditionalForm]], 
  "InlineMath"]
}], "Text",
 CellID->131892371],

Cell[TextData[{
 "The  ",
 Cell[BoxData[
  RowBox[{
   StyleBox["N",
    FontFamily->"Times New Roman",
    FontSize->12,
    FontSlant->"Italic"], "+", "1"}]], "InlineFormula"],
 " by ",
 Cell[BoxData[
  RowBox[{
   StyleBox["N", "TI"], "+", "1"}]], "InlineFormula"],
 " matrix ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    SubscriptBox["C", "\[DoubleStruckA]"],
    FontWeight->"Bold"], TraditionalForm]], "InlineMath"],
 " is the matrix of velocity-potential influence coefficients associated with \
the list of parameters ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ", to which a column has been appended for the constant-strength wake \
vortex, ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    SubscriptBox["C", "b"],
    FontWeight->"Bold"], TraditionalForm]], "InlineMath"],
 " and ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    SubscriptBox["C", "c"],
    FontWeight->"Bold"], TraditionalForm]], "InlineMath"],
 " are ",
 Cell[BoxData[
  RowBox[{
   StyleBox["N", "TI"], "+", "1"}]], "InlineFormula"],
 " by ",
 Cell[BoxData[
  StyleBox["N", "TI"]], "InlineFormula"],
 " matrices of velocity-potential influence coefficients associated with the \
lists of unknowns ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]], "InlineMath"],
 " and ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]], "InlineMath"],
 " respectively, and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Phi]", "\[Infinity]"], TraditionalForm]], "InlineMath"],
 " denotes the list of free-stream velocity potentials at  collocation points."
}], "Text",
 CellID->29559755],

Cell["\<\
Compute the velocity-potential influence coefficients at all collocation \
points due to panels forming the discretized airfoil, and display their \
dimensions.\
\>", "Text",
 CellID->48064062],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Phi]a", ",", "ic\[Phi]b", ",", "ic\[Phi]c"}], "}"}], "=", 
   RowBox[{"ICPhiDoubletQuadratic", "[", 
    RowBox[{"rCAll", ",", "rP"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Dimensions", "/@", 
  RowBox[{"{", 
   RowBox[{"ic\[Phi]a", ",", "ic\[Phi]b", ",", "ic\[Phi]c"}], 
   "}"}]}]}], "Input",
 CellLabel->"In[49]:=",
 CellID->189238130],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"7", ",", "6"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"7", ",", "6"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"7", ",", "6"}], "}"}]}], "}"}]], "Output",
 ImageSize->{157, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[50]=",
 CellID->125891941]
}, Open  ]],

Cell["\<\
Append the influence coefficients at collocation points due to the \
constant-strength wake panel.\
\>", "Text",
 CellID->65240430],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ic\[Phi]\[DoubleStruckA]", "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Append", "[", 
     RowBox[{
      RowBox[{"Transpose", "[", "ic\[Phi]a", "]"}], ",", 
      RowBox[{"Flatten", "[", 
       RowBox[{"ICPhiDoubletConstant", "[", 
        RowBox[{"rCAll", ",", "rPWake"}], "]"}], "]"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Dimensions", "[", "ic\[Phi]\[DoubleStruckA]", "]"}]}], "Input",
 CellLabel->"In[51]:=",
 CellID->24538656],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"7", ",", "7"}], "}"}]], "Output",
 ImageSize->{43, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[52]=",
 CellID->162933089]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Kutta conditions", "Subsubsection",
 CellID->94585002],

Cell[TextData[{
 "The relationship between circulation density and doublet density is \
discussed in more detail in ",
 ButtonBox["Section 2.1",
  BaseStyle->"Hyperlink",
  ButtonData->{"IntroductionToInfluenceCoefficients.nb", "relation"}],
 ".  Write this relationship for a quadratically varying doublet panel."
}], "Text",
 CellID->349121839],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["\[Gamma]", "TR"], 
    RowBox[{"(", 
     SubscriptBox[
      StyleBox["x", "TI"], 
      StyleBox["j", "TI"]], ")"}]}], "=", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{
       RowBox[{"\[DifferentialD]", 
        StyleBox["\[Mu]", "TR"]}], 
       RowBox[{"(", 
        SubscriptBox[
         StyleBox["x", "TI"], 
         StyleBox["j", "TI"]], ")"}]}], 
      RowBox[{"\[DifferentialD]", 
       SubscriptBox[
        StyleBox["x", "TI"], 
        StyleBox["j", "TI"]]}]]}], "=", 
    RowBox[{
     RowBox[{"-", 
      SubscriptBox[
       StyleBox["b", "TI"], 
       StyleBox["j", "TI"]]}], "-", 
     RowBox[{"2", 
      SubscriptBox[
       StyleBox["c", "TI"], 
       StyleBox["j", "TI"]], " ", 
      SubscriptBox[
       StyleBox["x", "TI"], 
       StyleBox["j", "TI"]]}]}]}]}]], "InlineFormula"]
}], "Text",
 CellID->55333495],

Cell["\<\
Using the relation between the circulation density function and the doublet \
density function, write the condition that the circulation density be zero at \
the trailing edge.\
\>", "Text",
 CellID->291362026],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubscriptBox[
     StyleBox["b", "TI"], 
     StyleBox["N", "TI"]], "+", 
    RowBox[{"2", 
     SubscriptBox[
      StyleBox["c", "TI"], 
      StyleBox["N", "TI"]], " ", 
     SubscriptBox[
      StyleBox["\[Delta]", "TR"], 
      StyleBox["N", "TI"]]}], "+", 
    SubscriptBox[
     StyleBox["b", "TI"], 
     StyleBox["1", "TR"]]}], "=", "0"}]], "InlineFormula"]
}], "Text",
 CellID->404054939],

Cell["\<\
Requiring that the value of the doublet density at the trailing edge be zero \
implies that there is no point vortex singularity at the trailing edge.\
\>", "Text",
 CellID->226623022],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubscriptBox[
     StyleBox["a", "TI"], 
     StyleBox["N", "TI"]], "+", 
    RowBox[{
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["N", "TI"]], 
     SubscriptBox[
      StyleBox["\[Delta]", "TR"], 
      StyleBox["N", "TI"]]}], "+", 
    RowBox[{
     SubscriptBox[
      StyleBox["c", "TI"], 
      StyleBox["N", "TI"]], " ", 
     SuperscriptBox[
      SubscriptBox[
       StyleBox["\[Delta]", "TR"], 
       StyleBox["N", "TI"]], "2"]}], "-", 
    SubscriptBox[
     StyleBox["a", "TI"], 
     StyleBox["w", "TI"]], "-", 
    SubscriptBox[
     StyleBox["a", "TI"], 
     StyleBox["1", "TR"]]}], "=", "0"}]], "InlineFormula"]
}], "Text",
 CellID->335859064],

Cell["\<\
These Kutta conditions are implemented by modifying the equations for \
panel-to-panel continuity of doublet density and slope of doublet density in \
the following two sections.\
\>", "Text",
 CellID->880967397]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Panel-to-panel continuity of value of doublet density, and a Kutta condition\
\>", "Subsubsection",
 CellID->102303559],

Cell["\<\
Continuity of the value of the doublet density function from one panel to the \
next is required, except at the trailing edge, where a Kutta condition is \
imposed.\
\>", "Text",
 CellID->54578817],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubscriptBox[
     StyleBox["a", "TI"], 
     StyleBox["j", "TI"]], "+", 
    RowBox[{
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["j", "TI"]], " ", 
     SubscriptBox[
      StyleBox["\[Delta]", "TR"], 
      StyleBox["j", "TI"]]}], "+", 
    RowBox[{
     SubscriptBox[
      StyleBox["c", "TI"], 
      StyleBox["j", "TI"]], " ", 
     SuperscriptBox[
      SubscriptBox[
       StyleBox["\[Delta]", "TR"], 
       StyleBox["j", "TI"]], "2"]}]}], "=", 
   SubscriptBox[
    StyleBox["a", "TI"], 
    StyleBox[
     RowBox[{"j", 
      StyleBox["+",
       FontSlant->"Plain"], 
      StyleBox["1",
       FontSlant->"Plain"]}], "TI"]]}]], "InlineFormula"],
 " for ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["j", "TI"], "=", 
    StyleBox["1",
     FontFamily->"Times New Roman",
     FontSize->12]}], 
   StyleBox[",",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox[" ",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox["2",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox[",",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox[" ",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox["3",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox[",",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox[" ",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox["...",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox[",",
    FontFamily->"Times New Roman",
    FontSize->12], 
   StyleBox[" ",
    FontFamily->"Times New Roman",
    FontSize->12], 
   RowBox[{
    StyleBox["N", "TI"], "-", "1"}]}]], "InlineFormula"]
}], "Text",
 CellID->219874086],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubscriptBox[
     StyleBox["a", "TI"], 
     StyleBox["N", "TI"]], "+", 
    RowBox[{
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["N", "TI"]], 
     SubscriptBox[
      StyleBox["\[Delta]", "TR"], 
      StyleBox["N", "TI"]]}], "+", 
    RowBox[{
     SubscriptBox[
      StyleBox["c", "TI"], 
      StyleBox["N", "TI"]], " ", 
     SuperscriptBox[
      SubscriptBox[
       StyleBox["\[Delta]", "TR"], 
       StyleBox["N", "TI"]], "2"]}], "-", 
    SubscriptBox[
     StyleBox["a", "TI"], 
     StyleBox["w", "TI"]], "-", 
    SubscriptBox[
     StyleBox["a", "TI"], 
     StyleBox["1", "TR"]]}], "=", "0"}]], "InlineFormula"]
}], "Text",
 CellID->39977282],

Cell["These equations can be written in matrix form.", "Text",
 CellID->397767550],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      StyleBox[
       SubscriptBox["M", "\[DoubleStruckA]"],
       FontWeight->"Bold"], ".", "\[DoubleStruckA]"}], "+", 
     RowBox[{
      StyleBox[
       SubscriptBox["M", "b"],
       FontWeight->"Bold"], ".", "b"}], "+", 
     RowBox[{
      StyleBox[
       SubscriptBox["M", "c"],
       FontWeight->"Bold"], ".", "c"}]}], "=", "0"}], TraditionalForm]], 
  "InlineMath"]
}], "Text",
 CellID->240283937],

Cell[TextData[{
 "The dimensions of ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    SubscriptBox["M", "\[DoubleStruckA]"],
    FontWeight->"Bold"], TraditionalForm]], "InlineMath"],
 " are ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]], "InlineMath"],
 " by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", "+", "1"}], TraditionalForm]], "InlineMath"],
 ",  and the dimensions of ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    SubscriptBox["M", "b"],
    FontWeight->"Bold"], TraditionalForm]], "InlineMath"],
 " and ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    SubscriptBox["M", "c"],
    FontWeight->"Bold"], TraditionalForm]], "InlineMath"],
 " are ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]], "InlineMath"],
 " by ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]], "InlineMath"],
 "."
}], "Text",
 CellID->856713082],

Cell["\<\
Compute matrices for panel-to-panel continuity of doublet density function.  \
The first matrix includes coefficients for continuity of value from last \
panel to first, which will be replaced by a Kutta condition\
\>", "Text",
 CellID->756125371],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"matI", "=", 
   RowBox[{"IdentityMatrix", "[", "nP", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"(", 
   RowBox[{"mata", "=", 
    RowBox[{"matI", "-", 
     RowBox[{"RotateLeft", "[", "matI", "]"}]}]}], ")"}], "//", 
  "MatrixForm"}]}], "Input",
 CellLabel->"In[53]:=",
 CellID->615129277],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", 
      RowBox[{"-", "1"}], "0", "0", "0", "0"},
     {"0", "1", 
      RowBox[{"-", "1"}], "0", "0", "0"},
     {"0", "0", "1", 
      RowBox[{"-", "1"}], "0", "0"},
     {"0", "0", "0", "1", 
      RowBox[{"-", "1"}], "0"},
     {"0", "0", "0", "0", "1", 
      RowBox[{"-", "1"}]},
     {
      RowBox[{"-", "1"}], "0", "0", "0", "0", "1"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 ImageSize->{154, 87},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[54]//MatrixForm=",
 CellID->740061203]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"matb", "=", 
    RowBox[{"DiagonalMatrix", "[", "\[Delta]", "]"}]}], ")"}], "//", 
  "MatrixForm"}]], "Input",
 CellLabel->"In[55]:=",
 CellID->16413054],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0.5002885392353391`", "0.`", "0.`", "0.`", "0.`", "0.`"},
     {"0.`", "0.36607906794806616`", "0.`", "0.`", "0.`", "0.`"},
     {"0.`", "0.`", "0.13621107223184767`", "0.`", "0.`", "0.`"},
     {"0.`", "0.`", "0.`", "0.14784225044636196`", "0.`", "0.`"},
     {"0.`", "0.`", "0.`", "0.`", "0.36612761798854326`", "0.`"},
     {"0.`", "0.`", "0.`", "0.`", "0.`", "0.5050815775453552`"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 ImageSize->{400, 87},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[55]//MatrixForm=",
 CellID->30442987]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"matc", "=", 
    RowBox[{"DiagonalMatrix", "[", 
     RowBox[{"\[Delta]", "^", "2"}], "]"}]}], ")"}], "//", 
  "MatrixForm"}]], "Input",
 CellLabel->"In[56]:=",
 CellID->741221869],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0.25028862249022943`", "0.`", "0.`", "0.`", "0.`", "0.`"},
     {"0.`", "0.13401388398972483`", "0.`", "0.`", "0.`", "0.`"},
     {"0.`", "0.`", "0.018553456198549624`", "0.`", "0.`", "0.`"},
     {"0.`", "0.`", "0.`", "0.02185733101704481`", "0.`", "0.`"},
     {"0.`", "0.`", "0.`", "0.`", "0.13404943265396466`", "0.`"},
     {"0.`", "0.`", "0.`", "0.`", "0.`", "0.25510739997570464`"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 ImageSize->{414, 87},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[56]//MatrixForm=",
 CellID->64692338]
}, Open  ]],

Cell[TextData[{
 "Modify ",
 Cell[BoxData["mata"], "InlineFormula"],
 " by adding a column of zeros to accommodate the strength parameter for the \
wake panel, and modify the last row to be the Kutta condition. "
}], "Text",
 CellID->92270313],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"kutta1", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"-", "1"}], "}"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{"0", ",", 
       RowBox[{"{", 
        RowBox[{"nP", "-", "2"}], "}"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", "1"}]}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"(", 
   RowBox[{"mat\[DoubleStruckA]", "=", 
    RowBox[{"ReplacePart", "[", 
     RowBox[{
      RowBox[{"Transpose", "[", 
       RowBox[{"Append", "[", 
        RowBox[{
         RowBox[{"Transpose", "[", "mata", "]"}], ",", 
         RowBox[{"Table", "[", 
          RowBox[{"0", ",", 
           RowBox[{"{", "nP", "}"}]}], "]"}]}], "]"}], "]"}], ",", 
      RowBox[{
       RowBox[{"-", "1"}], "\[Rule]", "kutta1"}]}], "]"}]}], ")"}], "//", 
  "MatrixForm"}]}], "Input",
 CellLabel->"In[57]:=",
 CellID->599743264],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", 
      RowBox[{"-", "1"}], "0", "0", "0", "0", "0"},
     {"0", "1", 
      RowBox[{"-", "1"}], "0", "0", "0", "0"},
     {"0", "0", "1", 
      RowBox[{"-", "1"}], "0", "0", "0"},
     {"0", "0", "0", "1", 
      RowBox[{"-", "1"}], "0", "0"},
     {"0", "0", "0", "0", "1", 
      RowBox[{"-", "1"}], "0"},
     {
      RowBox[{"-", "1"}], "0", "0", "0", "0", "1", 
      RowBox[{"-", "1"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 ImageSize->{177, 87},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[58]//MatrixForm=",
 CellID->193753726]
}, Open  ]],

Cell[TextData[{
 "Display the dimensions of the matrices ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    SubscriptBox["M", "\[DoubleStruckA]"],
    FontFamily->"Times New Roman",
    FontSize->12,
    FontWeight->"Bold",
    FontSlant->"Italic"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  StyleBox[
   SubscriptBox["M", "b"],
   FontFamily->"Times New Roman",
   FontSize->12,
   FontWeight->"Bold",
   FontSlant->"Italic"]], "InlineFormula"],
 ", and ",
 Cell[BoxData[
  StyleBox[
   SubscriptBox["M", "c"],
   FontFamily->"Times New Roman",
   FontSize->12,
   FontWeight->"Bold",
   FontSlant->"Italic"]], "InlineFormula"],
 "."
}], "Text",
 CellID->526156131],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Dimensions", "/@", 
  RowBox[{"{", 
   RowBox[{"mat\[DoubleStruckA]", ",", "matb", ",", "matc"}], 
   "}"}]}]], "Input",
 CellLabel->"In[59]:=",
 CellID->92044694],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"6", ",", "7"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"6", ",", "6"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"6", ",", "6"}], "}"}]}], "}"}]], "Output",
 ImageSize->{157, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[59]=",
 CellID->20868307]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Panel-to-panel continuity of slope of doublet density function, and a Kutta \
condition\
\>", "Subsubsection",
 CellID->89512504],

Cell["\<\
Continuity of the slope of the doublet density function from one panel to the \
next is required, except at the trailing edge.  At the trailing edge, the \
slope of the doublet density is required to be zero as part of the Kutta \
condition.\
\>", "Text",
 CellID->126141448],

Cell["\<\
Continuity of slope of the doublet density function from one panel to the \
next can be written in terms of the the strength parameters and panel lengths.\
\>", "Text",
 CellID->61776658],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubscriptBox[
     StyleBox["b", "TI"], 
     StyleBox["j", "TI"]], "+", 
    RowBox[{"2", 
     SubscriptBox[
      StyleBox["c", "TI"], 
      StyleBox["j", "TI"]], " ", 
     SubscriptBox[
      StyleBox["\[Delta]", "TR"], 
      StyleBox["j", "TI"]]}]}], "=", 
   SubscriptBox[
    StyleBox["b", "TI"], 
    StyleBox[
     RowBox[{"j", 
      StyleBox["+",
       FontSlant->"Plain"], 
      StyleBox["1",
       FontSlant->"Plain"]}], "TI"]]}]], "InlineFormula"],
 "   for  ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["j", "TI"], "=", "1"}], ",", "2", ",", "3", ",", "...", ",", 
   Cell["", "InlineFormula"]}]], "InlineFormula"],
 Cell[BoxData[
  RowBox[{
   StyleBox["N",
    FontFamily->"Times New Roman",
    FontSize->12,
    FontSlant->"Italic"], "-", "1"}]], "InlineFormula"]
}], "Text",
 CellID->72666099],

Cell["\<\
Write the Kutta condition that the circulation density at the trailing edge \
be zero.\
\>", "Text",
 CellID->26705995],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubscriptBox[
     StyleBox["b", "TI"], 
     StyleBox["N", "TI"]], "+", 
    RowBox[{"2", 
     SubscriptBox[
      StyleBox["c", "TI"], 
      StyleBox["N", "TI"]], " ", 
     SubscriptBox[
      StyleBox["\[Delta]", "TR"], 
      StyleBox["N", "TI"]]}], "+", 
    SubscriptBox[
     StyleBox["b", "TI"], 
     StyleBox["1", "TR"]]}], "=", "0"}]], "InlineFormula"]
}], "Text",
 CellID->44996651],

Cell["These equations can be written in matrix form.", "Text",
 CellID->112301587],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      StyleBox[
       SubscriptBox["S", "b"],
       FontWeight->"Bold"], ".", "b"}], "+", 
     RowBox[{
      StyleBox[
       SubscriptBox["S", "c"],
       FontWeight->"Bold"], ".", "c"}]}], "=", "0"}], TraditionalForm]], 
  "InlineMath"]
}], "Text",
 CellID->47088591],

Cell[TextData[{
 "The dimensions of ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    SubscriptBox["S", "b"],
    FontWeight->"Bold"], TraditionalForm]], "InlineMath"],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "c"], TraditionalForm]], "InlineMath",
  FontWeight->"Bold"],
 " are ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]], "InlineMath"],
 " by ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]], "InlineMath"],
 "."
}], "Text",
 CellID->454274171],

Cell[TextData[{
 "Compute the matrix of coefficients for ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 ", replacing the last row of the continuity matrix with coefficients for the \
Kutta condition on slope of the doublet density function."
}], "Text",
 CellID->31540012],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"matSb", "=", 
    RowBox[{"ReplacePart", "[", 
     RowBox[{"mata", ",", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "1"}], "}"}], "\[Rule]", "1"}]}], "]"}]}], 
   ")"}], "//", "MatrixForm"}]], "Input",
 CellLabel->"In[60]:=",
 CellID->262110288],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", 
      RowBox[{"-", "1"}], "0", "0", "0", "0"},
     {"0", "1", 
      RowBox[{"-", "1"}], "0", "0", "0"},
     {"0", "0", "1", 
      RowBox[{"-", "1"}], "0", "0"},
     {"0", "0", "0", "1", 
      RowBox[{"-", "1"}], "0"},
     {"0", "0", "0", "0", "1", 
      RowBox[{"-", "1"}]},
     {"1", "0", "0", "0", "0", "1"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 ImageSize->{146, 87},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[60]//MatrixForm=",
 CellID->193955619]
}, Open  ]],

Cell[TextData[{
 "The matrix ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    SubscriptBox["S", "c"],
    FontWeight->"Bold"], TraditionalForm]], "InlineMath"],
 " is just twice ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    SubscriptBox["M", "b"],
    FontWeight->"Bold"], TraditionalForm]], "InlineMath"],
 "."
}], "Text",
 CellID->278147230],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"matSc", "=", 
    RowBox[{"2", "matb"}]}], ")"}], "//", "MatrixForm"}]], "Input",
 CellLabel->"In[61]:=",
 CellID->97230909],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1.0005770784706782`", "0.`", "0.`", "0.`", "0.`", "0.`"},
     {"0.`", "0.7321581358961323`", "0.`", "0.`", "0.`", "0.`"},
     {"0.`", "0.`", "0.27242214446369534`", "0.`", "0.`", "0.`"},
     {"0.`", "0.`", "0.`", "0.2956845008927239`", "0.`", "0.`"},
     {"0.`", "0.`", "0.`", "0.`", "0.7322552359770865`", "0.`"},
     {"0.`", "0.`", "0.`", "0.`", "0.`", "1.0101631550907104`"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 ImageSize->{386, 87},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[61]//MatrixForm=",
 CellID->404246861]
}, Open  ]],

Cell[TextData[{
 "Display the dimensions of ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    SubscriptBox["S", "b"],
    FontWeight->"Bold"], TraditionalForm]], "InlineMath"],
 " and ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    SubscriptBox["S", "c"],
    FontWeight->"Bold"], TraditionalForm]], "InlineMath"],
 "."
}], "Text",
 CellID->727079737],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Dimensions", "/@", 
  RowBox[{"{", 
   RowBox[{"matSb", ",", "matSc"}], "}"}]}]], "Input",
 CellLabel->"In[62]:=",
 CellID->19263021],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"6", ",", "6"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"6", ",", "6"}], "}"}]}], "}"}]], "Output",
 ImageSize->{107, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[62]=",
 CellID->456788288]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solution procedure", "Subsubsection",
 CellID->165742176],

Cell[TextData[{
 "Solve the equations for panel-to-panel continuity of slope of doublet \
density and a Kutta condition on circulation density for ",
 Cell[BoxData[
  StyleBox["c", "TI"]], "InlineFormula"],
 " in terms of ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 "."
}], "Text",
 CellID->12866520],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  FormBox[
   RowBox[{"c", "=", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       StyleBox[
        SuperscriptBox[
         SubscriptBox["S", "c"], 
         StyleBox[
          RowBox[{"-", "1"}],
          FontWeight->"Plain"]],
        FontWeight->"Bold"], ".", 
       StyleBox[
        SubscriptBox["S", "b"],
        FontWeight->"Bold"], ".", "b"}]}], "=", 
     RowBox[{
      StyleBox[
       SubscriptBox["M", "cb"],
       FontWeight->"Bold"], ".", "b"}]}]}], TraditionalForm]], "InlineMath"]
}], "Text",
 CellID->446515824],

Cell[TextData[{
 "Eliminate ",
 Cell[BoxData[
  StyleBox["c", "TI"]], "InlineFormula"],
 " from the system of equations."
}], "Text",
 CellID->111227654],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      StyleBox[
       SubscriptBox["C", "\[DoubleStruckA]"],
       FontWeight->"Bold"], ".", "\[DoubleStruckA]"}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        StyleBox[
         SubscriptBox["C", "b"],
         FontWeight->"Bold"], "+", 
        RowBox[{
         StyleBox[
          SubscriptBox["C", "c"],
          FontWeight->"Bold"], ".", 
         StyleBox[
          SubscriptBox["M", "cb"],
          FontWeight->"Bold"]}]}], ")"}], ".", "b"}]}], "=", 
    RowBox[{"-", 
     SubscriptBox["\[Phi]", "\[Infinity]"]}]}], TraditionalForm]], 
  "InlineMath"],
 "\t\t(boundary conditions)"
}], "Text",
 CellID->430827873],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      StyleBox[
       SubscriptBox["M", "\[DoubleStruckA]"],
       FontWeight->"Bold"], ".", "\[DoubleStruckA]"}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        StyleBox[
         SubscriptBox["M", "b"],
         FontWeight->"Bold"], "+", 
        RowBox[{
         StyleBox[
          SubscriptBox["M", "c"],
          FontWeight->"Bold"], ".", 
         StyleBox[
          SubscriptBox["M", "cb"],
          FontWeight->"Bold"]}]}], ")"}], ".", "b"}]}], "=", "0"}], 
   TraditionalForm]], "InlineMath"],
 "\t\t(continuity of ",
 Cell[BoxData[
  StyleBox["\[Mu]", "TR"]], "InlineFormula"],
 " and Kutta)"
}], "Text",
 CellID->19233526],

Cell[TextData[{
 "Solve the continuity/Kutta system of equations for ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 " in terms of ",
 Cell[BoxData["\[DoubleStruckA]"], "InlineFormula"],
 "."
}], "Text",
 CellID->358389442],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "=", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          StyleBox[
           SubscriptBox["M", "b"],
           FontWeight->"Bold"], "+", 
          RowBox[{
           StyleBox[
            SubscriptBox["M", "c"],
            FontWeight->"Bold"], ".", 
           StyleBox[
            SubscriptBox["M", "cb"],
            FontWeight->"Bold"]}]}], ")"}], 
        RowBox[{"-", "1"}]], ".", 
       StyleBox[
        SubscriptBox["M", "\[DoubleStruckA]"],
        FontWeight->"Bold"], ".", "\[DoubleStruckA]"}]}], "=", 
     RowBox[{
      StyleBox[
       SubscriptBox["M", "b\[DoubleStruckA]"],
       FontWeight->"Bold"], ".", "\[DoubleStruckA]"}]}]}], TraditionalForm]], 
  "InlineMath"]
}], "Text",
 CellID->268860931],

Cell[TextData[{
 "Eliminate ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 " from the boundary-condition equations."
}], "Text",
 CellID->187237148],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          StyleBox[
           SubscriptBox["C", "b"],
           FontWeight->"Bold"], "+", 
          RowBox[{
           StyleBox[
            SubscriptBox["C", "c"],
            FontWeight->"Bold"], ".", 
           StyleBox[
            SubscriptBox["M", "cb"],
            FontWeight->"Bold"]}]}], ")"}], ".", 
        StyleBox[
         SubscriptBox["M", "b\[DoubleStruckA]"],
         FontWeight->"Bold"]}], "+", 
       StyleBox[
        SubscriptBox["C", "\[DoubleStruckA]"],
        FontWeight->"Bold"]}], ")"}], ".", "\[DoubleStruckA]"}], "=", 
    RowBox[{"-", 
     SubscriptBox["\[Phi]", "\[Infinity]"]}]}], TraditionalForm]], 
  "InlineMath"]
}], "Text",
 CellID->710384853],

Cell["\<\
Write this  system of linear algebraic equations in the following form.\
\>", "Text",
 CellID->621887025],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     StyleBox["M",
      FontWeight->"Bold"], ".", "\[DoubleStruckA]"}], "=", 
    RowBox[{"-", 
     SubscriptBox["\[Phi]", "\[Infinity]"]}]}], TraditionalForm]], 
  "InlineMath"]
}], "Text",
 CellID->947573354],

Cell[TextData[{
 "Solve using the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function LinearSolve."
}], "Text",
 CellID->242166576],

Cell[TextData[{
 "Compute ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["m", "TI"], 
   StyleBox["cb", "TI"]]], "InlineFormula"],
 "."
}], "Text",
 CellID->159109476],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"matcb", "=", 
    RowBox[{"-", 
     RowBox[{
      RowBox[{"Inverse", "[", "matSc", "]"}], ".", "matSb"}]}]}], ")"}], "//",
   "MatrixForm"}]], "Input",
 CellLabel->"In[63]:=",
 CellID->588459847],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", "0.9994232543568156`"}], "0.9994232543568156`", "0.`", 
      "0.`", "0.`", "0.`"},
     {"0.`", 
      RowBox[{"-", "1.3658251557582433`"}], "1.3658251557582433`", "0.`", 
      "0.`", "0.`"},
     {"0.`", "0.`", 
      RowBox[{"-", "3.6707735414411813`"}], "3.6707735414411813`", "0.`", 
      "0.`"},
     {"0.`", "0.`", "0.`", 
      RowBox[{"-", "3.381983150895034`"}], "3.381983150895034`", "0.`"},
     {"0.`", "0.`", "0.`", "0.`", 
      RowBox[{"-", "1.3656440416785107`"}], "1.3656440416785107`"},
     {
      RowBox[{"-", "0.9899390954426587`"}], "0.`", "0.`", "0.`", "0.`", 
      RowBox[{"-", "0.9899390954426587`"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 ImageSize->{420, 87},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[63]//MatrixForm=",
 CellID->194489683]
}, Open  ]],

Cell[TextData[{
 "Compute ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["m", "TI"], 
   StyleBox[
    RowBox[{
     StyleBox["b", "TI"], "\[DoubleStruckA]"}]]]], "InlineFormula"],
 "."
}], "Text",
 CellID->167905182],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"matb\[DoubleStruckA]", "=", 
    RowBox[{"-", 
     RowBox[{
      RowBox[{"Inverse", "[", 
       RowBox[{"matb", "+", 
        RowBox[{"matc", ".", "matcb"}]}], "]"}], ".", 
      "mat\[DoubleStruckA]"}]}]}], ")"}], "//", "MatrixForm"}]], "Input",
 CellLabel->"In[64]:=",
 CellID->1046617244],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", "3.9787246995989483`"}], "4.730496820230118`", 
      RowBox[{"-", "10.073197394398846`"}], "14.105513384672427`", 
      RowBox[{"-", "9.495254385147087`"}], "4.711166274242338`", 
      RowBox[{"-", "1.979878190885317`"}]},
     {
      RowBox[{"-", "0.018968317828313763`"}], 
      RowBox[{"-", "0.7328038028028558`"}], "10.073197394398846`", 
      RowBox[{"-", "14.105513384672427`"}], "9.495254385147087`", 
      RowBox[{"-", "4.711166274242338`"}], "1.979878190885317`"},
     {"0.018968317828313763`", 
      RowBox[{"-", "4.730496820230117`"}], 
      RowBox[{"-", "4.609896771365879`"}], "14.105513384672435`", 
      RowBox[{"-", "9.495254385147092`"}], "4.7111662742423395`", 
      RowBox[{"-", "1.9798781908853174`"}]},
     {
      RowBox[{"-", "0.018968317828313763`"}], "4.730496820230118`", 
      RowBox[{"-", "10.073197394398846`"}], "0.5775807810922906`", 
      "9.495254385147092`", 
      RowBox[{"-", "4.71116627424234`"}], "1.9798781908853178`"},
     {"0.018968317828313763`", 
      RowBox[{"-", "4.730496820230118`"}], "10.073197394398846`", 
      RowBox[{"-", "14.105513384672427`"}], "4.032678218433045`", 
      "4.71116627424234`", 
      RowBox[{"-", "1.9798781908853178`"}]},
     {
      RowBox[{"-", "0.018968317828313763`"}], "4.730496820230118`", 
      RowBox[{"-", "10.073197394398846`"}], "14.105513384672427`", 
      RowBox[{"-", "9.495254385147087`"}], "0.7514098924717028`", 
      "1.9798781908853178`"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 ImageSize->{492, 87},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[64]//MatrixForm=",
 CellID->194048090]
}, Open  ]],

Cell[TextData[{
 "Compute ",
 Cell[BoxData[
  StyleBox["m", "TI"]], "InlineFormula"],
 "."
}], "Text",
 CellID->597170703],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"m", "=", 
    RowBox[{"ic\[Phi]\[DoubleStruckA]", "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"ic\[Phi]b", "+", 
        RowBox[{"ic\[Phi]c", ".", "matcb"}]}], ")"}], ".", 
      "matb\[DoubleStruckA]"}]}]}], ")"}], "//", "MatrixForm"}]], "Input",
 CellLabel->"In[65]:=",
 CellID->284020968],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0.25193027797083367`", "0.6556856356536204`", 
      RowBox[{"-", "1.1241958028304677`"}], "1.5882394279680556`", 
      RowBox[{"-", "1.044510128413266`"}], "0.6728505896512245`", 
      "0.10633940266660595`"},
     {"0.011079777187541742`", "0.24818644259815487`", "0.792700336662685`", 
      RowBox[{"-", "0.9000471546657872`"}], "0.7767826605227279`", 
      "0.07129793769467754`", "0.06664599713794575`"},
     {"0.007255959664391106`", 
      RowBox[{"-", "0.037190297820392286`"}], "0.2554350387526039`", 
      "0.5768020149877027`", "0.10502087614010179`", "0.09267640827559286`", 
      RowBox[{"-", "0.010606920487013506`"}]},
     {"0.006845988084618377`", "0.08620981855425783`", "0.08719288884124349`",
       "0.5516138147849909`", "0.2974304531418374`", 
      RowBox[{"-", "0.029292963406948036`"}], "0.02392446655661373`"},
     {"0.011909254820922616`", "0.06875823037712803`", "0.7978445205714123`", 
      RowBox[{"-", "0.9020905971837645`"}], "0.7702466135845383`", 
      "0.2533319778297635`", 
      RowBox[{"-", "0.056655960350650396`"}]},
     {"0.24759603559984616`", "0.6669597375176768`", 
      RowBox[{"-", "1.1181854967441696`"}], "1.5867566129591093`", 
      RowBox[{"-", "1.0483582965230085`"}], "0.6652314071905461`", 
      "0.12854753514741135`"},
     {"0.2516775279772983`", "0.6580178676487465`", 
      RowBox[{"-", "1.1123689269343084`"}], "1.5751859470452945`", 
      RowBox[{"-", "1.0380984313762147`"}], "0.6655860156391842`", 
      "0.11846783515128867`"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 ImageSize->{532, 102},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[65]//MatrixForm=",
 CellID->132190725]
}, Open  ]],

Cell["\<\
Compute the list of free stream velocity potentials at collocation points.\
\>", "Text",
 CellID->328797457],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Phi]\[Infinity]", "=", 
  RowBox[{"UniformFlowPhi", "[", 
   RowBox[{"rCAll", ",", "\[Alpha]"}], "]"}]}]], "Input",
 CellLabel->"In[66]:=",
 CellID->244118140],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.7449939106631026`", ",", "0.31311164114544865`", ",", 
   "0.06542375532276734`", ",", "0.0699728912441579`", ",", 
   "0.3223048677627097`", ",", "0.749638001348713`", ",", 
   "0.7473159560059078`"}], "}"}]], "Output",
 ImageSize->{490, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[66]=",
 CellID->162012592]
}, Open  ]],

Cell[TextData[{
 "Solve the system of equations for the list of strength parameters ",
 Cell[BoxData["\[DoubleStruckA]"], "InlineFormula"],
 "."
}], "Text",
 CellID->190811117],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[DoubleStruckA]", "=", 
  RowBox[{"LinearSolve", "[", 
   RowBox[{"m", ",", 
    RowBox[{"-", "\[Phi]\[Infinity]"}]}], "]"}]}]], "Input",
 CellLabel->"In[67]:=",
 CellID->446810411],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "0.7913951699608199`"}], ",", 
   RowBox[{"-", "0.3369762229021266`"}], ",", 
   RowBox[{"-", "0.001584393965366289`"}], ",", "0.009523536820030445`", ",", 
   RowBox[{"-", "0.16817322801969598`"}], ",", 
   RowBox[{"-", "0.6961132264612939`"}], ",", 
   RowBox[{"-", "0.4593998667899517`"}]}], "}"}]], "Output",
 ImageSize->{538, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[67]=",
 CellID->229794115]
}, Open  ]],

Cell[TextData[{
 "Compute the list of strength parameters ",
 Cell[BoxData[
  StyleBox["b", "TI"]], "InlineFormula"],
 "."
}], "Text",
 CellID->253484638],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"b", "=", 
  RowBox[{"matb\[DoubleStruckA]", ".", "\[DoubleStruckA]"}]}]], "Input",
 CellLabel->"In[68]:=",
 CellID->215094552],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.9318710505956833`", ",", "0.884756401047504`", ",", "0.947589986942867`",
    ",", 
   RowBox[{"-", "0.7844911932340897`"}], ",", 
   RowBox[{"-", "1.6193786653919577`"}], ",", 
   RowBox[{"-", "1.2645337875501643`"}]}], "}"}]], "Output",
 ImageSize->{412, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[68]=",
 CellID->27746478]
}, Open  ]],

Cell[TextData[{
 "Compute the list of strength parameters ",
 Cell[BoxData[
  StyleBox["c", "TI"]], "InlineFormula"],
 "."
}], "Text",
 CellID->203098661],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"c", "=", 
  RowBox[{"matcb", ".", "b"}]}]], "Input",
 CellLabel->"In[69]:=",
 CellID->156425214],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "0.04708747637932231`"}], ",", "0.08581969224238306`", ",", 
   RowBox[{"-", "6.358077767821788`"}], ",", 
   RowBox[{"-", "2.8235753637312566`"}], ",", "0.4845917931447843`", ",", 
   "0.3293158489081981`"}], "}"}]], "Output",
 ImageSize->{433, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[69]=",
 CellID->10251174]
}, Open  ]],

Cell["\<\
Separate the strength of the wake panel from the panels specifying the \
discretized airfoil.\
\>", "Text",
 CellID->64776907],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"a", ",", "aW"}], "}"}], "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Most", "[", "\[DoubleStruckA]", "]"}], ",", 
    RowBox[{"Last", "[", "\[DoubleStruckA]", "]"}]}], "}"}]}]], "Input",
 CellLabel->"In[70]:=",
 CellID->57059381],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.7913951699608199`"}], ",", 
     RowBox[{"-", "0.3369762229021266`"}], ",", 
     RowBox[{"-", "0.001584393965366289`"}], ",", "0.009523536820030445`", 
     ",", 
     RowBox[{"-", "0.16817322801969598`"}], ",", 
     RowBox[{"-", "0.6961132264612939`"}]}], "}"}], ",", 
   RowBox[{"-", "0.4593998667899517`"}]}], "}"}]], "Output",
 ImageSize->{552, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[70]=",
 CellID->45924774]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Results using function PMDoubletQuadraticDirichlet", "Subsection",
 CellID->913638886],

Cell[TextData[{
 "Use the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["PMDoubletQuadraticDirichlet",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/PMDoubletQuadraticDirichlet"]], 
  "InlineFormula"],
 " to solve the model for the example discretized airfoil at the specified \
angle of attack."
}], "Text",
 CellID->213089269],

Cell["\<\
Compute the strength parameters, locations of collocation points and surface \
speeds near collocation points using the suffix DQD to denote results \
computed from the programmed model function.  \
\>", "Text",
 CellID->525928298],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "aDQD", ",", "bDQD", ",", "cDQD", ",", "aWDQD", ",", "rC", ",", 
     "vMagCDQD"}], "}"}], "=", 
   RowBox[{"PMDoubletQuadraticDirichlet", "[", 
    RowBox[{"rP", ",", "\[Alpha]"}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[71]:=",
 CellID->239185265],

Cell["\<\
Verify that the results of the step-by-step implementation are the same as \
those from the model PMDoubletQuadraticDirichlet.\
\>", "Text",
 CellID->476901211],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", "==", "aDQD"}], ",", 
   RowBox[{"b", "==", "bDQD"}], ",", 
   RowBox[{"c", "==", "cDQD"}], ",", 
   RowBox[{"aW", "==", "aWDQD"}]}], "}"}]], "Input",
 CellLabel->"In[72]:=",
 CellID->1385341],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True", ",", "True", ",", "True"}], "}"}]], "Output",
 ImageSize->{163, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[72]=",
 CellID->808620393]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.3.5 Comments", "Section",
 CellID->273428036],

Cell["\<\
The model PMVortexLinearNeumann is the most straightforward to implement.  \
Both doublet models require a constant-strength wake panel to accommodate a \
Kutta condition, which complicates implementation.  Additionally, \
PMDoubletQuadraticDirichlet, has the complication that the location of an \
additional collocation point must be specified.\
\>", "Text",
 CellID->107864109]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[Cell[BoxData[
 ButtonBox["PanelMethods",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Aerodynamics/guide/PanelMethods"]], "InlineFormula"]], \
"TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["3.2 Validation Example",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ValidationExample"]], "RelatedTutorials",
 CellID->17795993],

Cell[TextData[ButtonBox["3.4 Flow Visualization",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/FlowVisualization"]], "RelatedTutorials",
 CellID->14999323],

Cell[TextData[ButtonBox["Overview",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]], "RelatedTutorials",
 CellID->280152894]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"3.3  Implementation - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 4, 19.5516000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "None", "summary" -> 
    "This section provides details for the implementation of the three \
panel-method functions implemented in Aerodynamics for a coarsely discretized \
Eppler-205 airfoil. In addition to illustrating the common steps used in \
constructing a model, some of the choices that must be made are pointed out. \
Also, the display of intermediate results may aid those constructing their \
own panel methods. Load the Aerodynamics packages. The data file for the \
Eppler-205 airfoil has been downloaded from the UIUC Airfoil Data Base to the \
Airfoil Data folder in the Aerodynamics software. Use AirfoilFromDataFile to \
import data from the file and rearrange the data to the form used in \
Aerodynamics.", "synonyms" -> {}, "title" -> "3.3  Implementation", "type" -> 
    "Tutorial", "uri" -> "Aerodynamics/tutorial/Implementation"}, 
  "LinkTrails" -> "", "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[612, 21, 78, 1, 70, "TutorialColorBar"],
Cell[693, 24, 191, 5, 70, "LinkTrail"],
Cell[887, 31, 2893, 65, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3805, 100, 56, 1, 70, "Title",
 CellID->509267359],
Cell[3864, 103, 492, 10, 70, "Text",
 CellID->1534169418],
Cell[4359, 115, 125, 6, 70, "Text",
 CellID->585195885],
Cell[4487, 123, 99, 3, 70, "Input",
 CellID->20779588],
Cell[CellGroupData[{
Cell[4611, 130, 64, 1, 70, "Section",
 CellID->491421075],
Cell[4678, 133, 539, 16, 70, "Text",
 CellID->323137],
Cell[5220, 151, 217, 8, 70, "Input",
 CellID->543466123],
Cell[5440, 161, 76, 1, 70, "Text",
 CellID->173149258],
Cell[CellGroupData[{
Cell[5541, 166, 112, 3, 70, "Input",
 CellID->416922631],
Cell[5656, 171, 159, 5, 36, "Output",
 CellID->721774173]
}, Open  ]],
Cell[5830, 179, 278, 9, 70, "Text",
 CellID->193395042],
Cell[CellGroupData[{
Cell[6133, 192, 664, 21, 83, "Input",
 CellID->88849844],
Cell[6800, 215, 5843, 103, 201, "Output",
 CellID->633905778]
}, Open  ]],
Cell[12658, 321, 316, 9, 70, "Text",
 CellID->135072324],
Cell[CellGroupData[{
Cell[12999, 334, 504, 14, 70, "Input",
 CellID->25651006],
Cell[13506, 350, 5306, 122, 201, "Output",
 CellID->191119674]
}, Open  ]],
Cell[18827, 475, 63, 1, 70, "Text",
 CellID->203229668],
Cell[18893, 478, 143, 5, 70, "Input",
 CellID->254593671]
}, Open  ]],
Cell[CellGroupData[{
Cell[19073, 488, 158, 4, 70, "Section",
 CellID->336729780],
Cell[19234, 494, 439, 11, 70, "Text",
 CellID->479438142],
Cell[19676, 507, 885, 36, 70, "Text",
 CellID->111507038],
Cell[20564, 545, 521, 18, 70, "Text",
 CellID->101991089],
Cell[21088, 565, 1075, 42, 70, "Text",
 CellID->104238284],
Cell[22166, 609, 290, 11, 70, "Text",
 CellID->547624004],
Cell[22459, 622, 972, 30, 70, "Text",
 CellID->375367223],
Cell[CellGroupData[{
Cell[23456, 656, 69, 1, 70, "Subsection",
 CellID->725608851],
Cell[CellGroupData[{
Cell[23550, 661, 61, 1, 70, "Subsubsection",
 CellID->225787759],
Cell[23614, 664, 131, 4, 70, "Text",
 CellID->472269960],
Cell[23748, 670, 57, 1, 70, "Text",
 CellID->74064168],
Cell[CellGroupData[{
Cell[23830, 675, 140, 4, 70, "Input",
 CellID->106567359],
Cell[23973, 681, 368, 10, 36, "Output",
 CellID->494467118]
}, Open  ]],
Cell[24356, 694, 52, 1, 70, "Text",
 CellID->10851564],
Cell[CellGroupData[{
Cell[24433, 699, 134, 4, 70, "Input",
 CellID->152897124],
Cell[24570, 705, 159, 5, 36, "Output",
 CellID->436402049]
}, Open  ]],
Cell[24744, 713, 82, 1, 70, "Text",
 CellID->162262132],
Cell[CellGroupData[{
Cell[24851, 718, 135, 4, 70, "Input",
 CellID->691393489],
Cell[24989, 724, 1115, 27, 54, "Output",
 CellID->292213793]
}, Open  ]],
Cell[26119, 754, 156, 4, 70, "Text",
 CellID->11573225],
Cell[CellGroupData[{
Cell[26300, 762, 231, 8, 70, "Input",
 CellID->92036532],
Cell[26534, 772, 1081, 26, 54, "Output",
 CellID->305679991]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[27664, 804, 85, 1, 70, "Subsubsection",
 CellID->15365561],
Cell[27752, 807, 157, 4, 70, "Text",
 CellID->412123985],
Cell[27912, 813, 697, 28, 70, "Text",
 CellID->89575051],
Cell[28612, 843, 1325, 40, 70, "Text",
 CellID->173755935],
Cell[29940, 885, 428, 8, 70, "Text",
 CellID->23730492],
Cell[CellGroupData[{
Cell[30393, 897, 345, 11, 70, "Input",
 CellID->42090444],
Cell[30741, 910, 323, 11, 36, "Output",
 CellID->304408583]
}, Open  ]],
Cell[31079, 924, 261, 5, 70, "Text",
 CellID->407869361],
Cell[CellGroupData[{
Cell[31365, 933, 238, 7, 70, "Input",
 CellID->151960291],
Cell[31606, 942, 1859, 45, 122, "Output",
 CellID->288973517]
}, Open  ]],
Cell[CellGroupData[{
Cell[33502, 992, 181, 6, 70, "Input",
 CellID->49864735],
Cell[33686, 1000, 2034, 48, 122, "Output",
 CellID->47187414]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[35769, 1054, 125, 3, 70, "Subsubsection",
 CellID->103764251],
Cell[35897, 1059, 272, 5, 70, "Text",
 CellID->386586113],
Cell[36172, 1066, 1151, 48, 70, "Text",
 CellID->750004683],
Cell[37326, 1116, 615, 25, 70, "Text",
 CellID->123989520],
Cell[37944, 1143, 82, 1, 70, "Text",
 CellID->503495065],
Cell[38029, 1146, 367, 16, 70, "Text",
 CellID->446965378],
Cell[38399, 1164, 238, 7, 70, "Text",
 CellID->531359183],
Cell[CellGroupData[{
Cell[38662, 1175, 501, 16, 70, "Input",
 CellID->23909254],
Cell[39166, 1193, 1026, 31, 122, "Output",
 CellID->62951937]
}, Open  ]],
Cell[40207, 1227, 189, 6, 70, "Text",
 CellID->126776796],
Cell[CellGroupData[{
Cell[40421, 1237, 205, 7, 70, "Input",
 CellID->31347865],
Cell[40629, 1246, 1089, 26, 122, "Output",
 CellID->450398653]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[41767, 1278, 63, 1, 70, "Subsubsection",
 CellID->298127402],
Cell[41833, 1281, 233, 9, 70, "Text",
 CellID->122106569],
Cell[42069, 1292, 579, 23, 70, "Text",
 CellID->416517119],
Cell[42651, 1317, 192, 7, 70, "Text",
 CellID->27432560],
Cell[42846, 1326, 878, 36, 70, "Text",
 CellID->94688220],
Cell[43727, 1364, 746, 31, 70, "Text",
 CellID->44848813],
Cell[44476, 1397, 425, 17, 70, "Text",
 CellID->517009306],
Cell[CellGroupData[{
Cell[44926, 1418, 286, 10, 70, "Input",
 CellID->319384963],
Cell[45215, 1430, 1344, 35, 122, "Output",
 CellID->36808920]
}, Open  ]],
Cell[46574, 1468, 77, 1, 70, "Text",
 CellID->31359196],
Cell[CellGroupData[{
Cell[46676, 1473, 202, 6, 70, "Input",
 CellID->211953321],
Cell[46881, 1481, 437, 12, 36, "Output",
 CellID->108384008]
}, Open  ]],
Cell[47333, 1496, 157, 6, 70, "Text",
 CellID->388832003],
Cell[CellGroupData[{
Cell[47515, 1506, 248, 8, 70, "Input",
 CellID->367664505],
Cell[47766, 1516, 411, 11, 36, "Output",
 CellID->153732725]
}, Open  ]],
Cell[48192, 1530, 190, 9, 70, "Text",
 CellID->535259082],
Cell[CellGroupData[{
Cell[48407, 1543, 119, 4, 70, "Input",
 CellID->28035240],
Cell[48529, 1549, 398, 11, 36, "Output",
 CellID->161396544]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[48988, 1567, 99, 1, 70, "Subsection",
 CellID->611074881],
Cell[49090, 1570, 411, 13, 70, "Text",
 CellID->19519575],
Cell[49504, 1585, 278, 8, 70, "Input",
 CellID->50303841],
Cell[49785, 1595, 163, 4, 70, "Text",
 CellID->68346340],
Cell[CellGroupData[{
Cell[49973, 1603, 172, 6, 70, "Input",
 CellID->514967147],
Cell[50148, 1611, 209, 7, 36, "Output",
 CellID->44462280]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[50418, 1625, 152, 4, 70, "Section",
 CellID->577981106],
Cell[50573, 1631, 265, 8, 70, "Text",
 CellID->3302604],
Cell[50841, 1641, 1046, 45, 70, "Text",
 CellID->181147249],
Cell[51890, 1688, 287, 9, 70, "Text",
 CellID->260469098],
Cell[52180, 1699, 619, 24, 70, "Text",
 CellID->295014137],
Cell[52802, 1725, 678, 24, 70, "Text",
 CellID->409613708],
Cell[53483, 1751, 773, 25, 70, "Text",
 CellID->191057130],
Cell[CellGroupData[{
Cell[54281, 1780, 69, 1, 70, "Subsection",
 CellID->104432088],
Cell[CellGroupData[{
Cell[54375, 1785, 60, 1, 70, "Subsubsection",
 CellID->80757491],
Cell[54438, 1788, 270, 5, 70, "Text",
 CellID->217356065],
Cell[54711, 1795, 50, 1, 70, "Text",
 CellID->176816722],
Cell[CellGroupData[{
Cell[54786, 1800, 78, 2, 70, "Input",
 CellID->100724537],
Cell[54867, 1804, 367, 10, 36, "Output",
 CellID->88793639]
}, Open  ]],
Cell[55249, 1817, 52, 1, 70, "Text",
 CellID->25165121],
Cell[CellGroupData[{
Cell[55326, 1822, 72, 2, 70, "Input",
 CellID->603369164],
Cell[55401, 1826, 159, 5, 36, "Output",
 CellID->427119333]
}, Open  ]],
Cell[55575, 1834, 74, 1, 70, "Text",
 CellID->513602593],
Cell[CellGroupData[{
Cell[55674, 1839, 73, 2, 70, "Input",
 CellID->630817148],
Cell[55750, 1843, 1115, 27, 54, "Output",
 CellID->369335311]
}, Open  ]],
Cell[56880, 1873, 155, 4, 70, "Text",
 CellID->360726760],
Cell[CellGroupData[{
Cell[57060, 1881, 230, 8, 70, "Input",
 CellID->5240558],
Cell[57293, 1891, 1080, 26, 54, "Output",
 CellID->85891681]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[58422, 1923, 115, 3, 70, "Subsubsection",
 CellID->292614508],
Cell[58540, 1928, 635, 15, 70, "Text",
 CellID->104611666],
Cell[59178, 1945, 570, 22, 70, "Text",
 CellID->80393232],
Cell[59751, 1969, 1223, 40, 70, "Text",
 CellID->177985028],
Cell[60977, 2011, 330, 13, 70, "Text",
 CellID->196488211],
Cell[61310, 2026, 351, 10, 70, "Text",
 CellID->120707751],
Cell[61664, 2038, 255, 5, 70, "Text",
 CellID->128200992],
Cell[CellGroupData[{
Cell[61944, 2047, 422, 13, 70, "Input",
 CellID->9956353],
Cell[62369, 2062, 204, 7, 36, "Output",
 CellID->273744321]
}, Open  ]],
Cell[62588, 2072, 85, 1, 70, "Text",
 CellID->790473042],
Cell[CellGroupData[{
Cell[62698, 2077, 366, 12, 70, "Input",
 CellID->167614302],
Cell[63067, 2091, 311, 10, 36, "Output",
 CellID->695483810]
}, Open  ]],
Cell[63393, 2104, 170, 4, 70, "Text",
 CellID->54865923],
Cell[CellGroupData[{
Cell[63588, 2112, 226, 8, 70, "Input",
 CellID->87445909],
Cell[63817, 2122, 2018, 44, 137, "Output",
 CellID->34198324]
}, Open  ]],
Cell[65850, 2169, 162, 4, 70, "Text",
 CellID->134373867],
Cell[CellGroupData[{
Cell[66037, 2177, 258, 9, 70, "Input",
 CellID->134984038],
Cell[66298, 2188, 480, 13, 36, "Output",
 CellID->199300039]
}, Open  ]],
Cell[66793, 2204, 64, 1, 70, "Text",
 CellID->6313767],
Cell[CellGroupData[{
Cell[66882, 2209, 162, 5, 70, "Input",
 CellID->173842100],
Cell[67047, 2216, 500, 13, 36, "Output",
 CellID->86588103]
}, Open  ]],
Cell[67562, 2232, 154, 4, 70, "Text",
 CellID->201664465],
Cell[CellGroupData[{
Cell[67741, 2240, 259, 9, 70, "Input",
 CellID->51826491],
Cell[68003, 2251, 549, 15, 36, "Output",
 CellID->34079921]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[68613, 2273, 111, 3, 70, "Subsection",
 CellID->37612957],
Cell[68727, 2278, 424, 13, 70, "Text",
 CellID->756079987],
Cell[69154, 2293, 285, 8, 70, "Input",
 CellID->178842356],
Cell[69442, 2303, 169, 4, 70, "Text",
 CellID->289996779],
Cell[CellGroupData[{
Cell[69636, 2311, 174, 6, 70, "Input",
 CellID->229212092],
Cell[69813, 2319, 210, 7, 36, "Output",
 CellID->692095846]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[70084, 2333, 165, 4, 70, "Section",
 CellID->106827055],
Cell[70252, 2339, 440, 11, 70, "Text",
 CellID->784000936],
Cell[70695, 2352, 1502, 62, 70, "Text",
 CellID->20572744],
Cell[72200, 2416, 628, 23, 70, "Text",
 CellID->321938409],
Cell[72831, 2441, 1497, 59, 70, "Text",
 CellID->206580932],
Cell[74331, 2502, 658, 22, 70, "Text",
 CellID->63853891],
Cell[74992, 2526, 1190, 36, 70, "Text",
 CellID->24556244],
Cell[CellGroupData[{
Cell[76207, 2566, 68, 1, 70, "Subsection",
 CellID->97066759],
Cell[CellGroupData[{
Cell[76300, 2571, 61, 1, 70, "Subsubsection",
 CellID->135427747],
Cell[76364, 2574, 184, 4, 70, "Text",
 CellID->189252988],
Cell[76551, 2580, 50, 1, 70, "Text",
 CellID->211654584],
Cell[CellGroupData[{
Cell[76626, 2585, 78, 2, 70, "Input",
 CellID->391861234],
Cell[76707, 2589, 367, 10, 36, "Output",
 CellID->32025243]
}, Open  ]],
Cell[77089, 2602, 53, 1, 70, "Text",
 CellID->653071787],
Cell[CellGroupData[{
Cell[77167, 2607, 72, 2, 70, "Input",
 CellID->307414976],
Cell[77242, 2611, 157, 5, 36, "Output",
 CellID->3898044]
}, Open  ]],
Cell[77414, 2619, 73, 1, 70, "Text",
 CellID->80868268],
Cell[CellGroupData[{
Cell[77512, 2624, 73, 2, 70, "Input",
 CellID->427036223],
Cell[77588, 2628, 1115, 27, 54, "Output",
 CellID->369764010]
}, Open  ]],
Cell[78718, 2658, 68, 1, 70, "Text",
 CellID->421281807],
Cell[CellGroupData[{
Cell[78811, 2663, 72, 2, 70, "Input",
 CellID->342567280],
Cell[78886, 2667, 1081, 26, 54, "Output",
 CellID->255597634]
}, Open  ]],
Cell[79982, 2696, 232, 5, 70, "Text",
 CellID->18288899],
Cell[CellGroupData[{
Cell[80239, 2705, 508, 17, 70, "Input",
 CellID->164764330],
Cell[80750, 2724, 1194, 28, 54, "Output",
 CellID->406643534]
}, Open  ]],
Cell[81959, 2755, 120, 3, 70, "Text",
 CellID->102672742],
Cell[CellGroupData[{
Cell[82104, 2762, 323, 11, 70, "Input",
 CellID->240976878],
Cell[82430, 2775, 517, 15, 40, "Output",
 CellID->31033561]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[82996, 2796, 95, 4, 70, "Subsubsection",
 CellID->306855904],
Cell[83094, 2802, 526, 13, 70, "Text",
 CellID->1539757],
Cell[83623, 2817, 701, 28, 70, "Text",
 CellID->131892371],
Cell[84327, 2847, 1563, 55, 70, "Text",
 CellID->29559755],
Cell[85893, 2904, 203, 5, 70, "Text",
 CellID->48064062],
Cell[CellGroupData[{
Cell[86121, 2913, 420, 12, 70, "Input",
 CellID->189238130],
Cell[86544, 2927, 362, 13, 36, "Output",
 CellID->125891941]
}, Open  ]],
Cell[86921, 2943, 141, 4, 70, "Text",
 CellID->65240430],
Cell[CellGroupData[{
Cell[87087, 2951, 499, 13, 70, "Input",
 CellID->24538656],
Cell[87589, 2966, 204, 7, 36, "Output",
 CellID->162933089]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[87842, 2979, 60, 1, 70, "Subsubsection",
 CellID->94585002],
Cell[87905, 2982, 346, 8, 70, "Text",
 CellID->349121839],
Cell[88254, 2992, 938, 37, 70, "Text",
 CellID->55333495],
Cell[89195, 3031, 220, 5, 70, "Text",
 CellID->291362026],
Cell[89418, 3038, 467, 19, 70, "Text",
 CellID->404054939],
Cell[89888, 3059, 194, 4, 70, "Text",
 CellID->226623022],
Cell[90085, 3065, 749, 30, 70, "Text",
 CellID->335859064],
Cell[90837, 3097, 222, 5, 70, "Text",
 CellID->880967397]
}, Open  ]],
Cell[CellGroupData[{
Cell[91096, 3107, 129, 3, 70, "Subsubsection",
 CellID->102303559],
Cell[91228, 3112, 207, 5, 70, "Text",
 CellID->54578817],
Cell[91438, 3119, 1797, 75, 70, "Text",
 CellID->219874086],
Cell[93238, 3196, 748, 30, 70, "Text",
 CellID->39977282],
Cell[93989, 3228, 82, 1, 70, "Text",
 CellID->397767550],
Cell[94074, 3231, 504, 20, 70, "Text",
 CellID->240283937],
Cell[94581, 3253, 826, 34, 70, "Text",
 CellID->856713082],
Cell[95410, 3289, 257, 5, 70, "Text",
 CellID->756125371],
Cell[CellGroupData[{
Cell[95692, 3298, 343, 11, 70, "Input",
 CellID->615129277],
Cell[96038, 3311, 1049, 32, 122, "Output",
 CellID->740061203]
}, Open  ]],
Cell[CellGroupData[{
Cell[97124, 3348, 205, 7, 70, "Input",
 CellID->16413054],
Cell[97332, 3357, 1088, 26, 122, "Output",
 CellID->30442987]
}, Open  ]],
Cell[CellGroupData[{
Cell[98457, 3388, 232, 8, 70, "Input",
 CellID->741221869],
Cell[98692, 3398, 1091, 26, 122, "Output",
 CellID->64692338]
}, Open  ]],
Cell[99798, 3427, 243, 6, 70, "Text",
 CellID->92270313],
Cell[CellGroupData[{
Cell[100066, 3437, 940, 30, 70, "Input",
 CellID->599743264],
Cell[101009, 3469, 1101, 33, 122, "Output",
 CellID->193753726]
}, Open  ]],
Cell[102125, 3505, 663, 28, 70, "Text",
 CellID->526156131],
Cell[CellGroupData[{
Cell[102813, 3537, 187, 6, 70, "Input",
 CellID->92044694],
Cell[103003, 3545, 361, 13, 36, "Output",
 CellID->20868307]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[103413, 3564, 139, 4, 70, "Subsubsection",
 CellID->89512504],
Cell[103555, 3570, 285, 6, 70, "Text",
 CellID->126141448],
Cell[103843, 3578, 197, 4, 70, "Text",
 CellID->61776658],
Cell[104043, 3584, 897, 36, 70, "Text",
 CellID->72666099],
Cell[104943, 3622, 129, 4, 70, "Text",
 CellID->26705995],
Cell[105075, 3628, 466, 19, 70, "Text",
 CellID->44996651],
Cell[105544, 3649, 82, 1, 70, "Text",
 CellID->112301587],
Cell[105629, 3652, 366, 16, 70, "Text",
 CellID->47088591],
Cell[105998, 3670, 467, 20, 70, "Text",
 CellID->454274171],
Cell[106468, 3692, 286, 7, 70, "Text",
 CellID->31540012],
Cell[CellGroupData[{
Cell[106779, 3703, 342, 12, 70, "Input",
 CellID->262110288],
Cell[107124, 3717, 1027, 31, 122, "Output",
 CellID->193955619]
}, Open  ]],
Cell[108166, 3751, 340, 15, 70, "Text",
 CellID->278147230],
Cell[CellGroupData[{
Cell[108531, 3770, 176, 6, 70, "Input",
 CellID->97230909],
Cell[108710, 3778, 1086, 26, 122, "Output",
 CellID->404246861]
}, Open  ]],
Cell[109811, 3807, 345, 15, 70, "Text",
 CellID->727079737],
Cell[CellGroupData[{
Cell[110181, 3826, 157, 5, 70, "Input",
 CellID->19263021],
Cell[110341, 3833, 303, 11, 36, "Output",
 CellID->456788288]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[110693, 3850, 63, 1, 70, "Subsubsection",
 CellID->165742176],
Cell[110759, 3853, 317, 10, 70, "Text",
 CellID->12866520],
Cell[111079, 3865, 582, 23, 70, "Text",
 CellID->446515824],
Cell[111664, 3890, 153, 6, 70, "Text",
 CellID->111227654],
Cell[111820, 3898, 738, 28, 70, "Text",
 CellID->430827873],
Cell[112561, 3928, 749, 29, 70, "Text",
 CellID->19233526],
Cell[113313, 3959, 236, 8, 70, "Text",
 CellID->358389442],
Cell[113552, 3969, 862, 31, 70, "Text",
 CellID->268860931],
Cell[114417, 4002, 162, 6, 70, "Text",
 CellID->187237148],
Cell[114582, 4010, 857, 31, 70, "Text",
 CellID->710384853],
Cell[115442, 4043, 115, 3, 70, "Text",
 CellID->621887025],
Cell[115560, 4048, 290, 12, 70, "Text",
 CellID->947573354],
Cell[115853, 4062, 143, 6, 70, "Text",
 CellID->242166576],
Cell[115999, 4070, 166, 8, 70, "Text",
 CellID->159109476],
Cell[CellGroupData[{
Cell[116190, 4082, 249, 9, 70, "Input",
 CellID->588459847],
Cell[116442, 4093, 1354, 36, 122, "Output",
 CellID->194489683]
}, Open  ]],
Cell[117811, 4132, 216, 10, 70, "Text",
 CellID->167905182],
Cell[CellGroupData[{
Cell[118052, 4146, 346, 11, 70, "Input",
 CellID->1046617244],
Cell[118401, 4159, 2176, 50, 122, "Output",
 CellID->194048090]
}, Open  ]],
Cell[120592, 4212, 122, 6, 70, "Text",
 CellID->597170703],
Cell[CellGroupData[{
Cell[120739, 4222, 354, 11, 70, "Input",
 CellID->284020968],
Cell[121096, 4235, 2213, 46, 137, "Output",
 CellID->132190725]
}, Open  ]],
Cell[123324, 4284, 118, 3, 70, "Text",
 CellID->328797457],
Cell[CellGroupData[{
Cell[123467, 4291, 185, 5, 70, "Input",
 CellID->244118140],
Cell[123655, 4298, 397, 11, 36, "Output",
 CellID->162012592]
}, Open  ]],
Cell[124067, 4312, 176, 5, 70, "Text",
 CellID->190811117],
Cell[CellGroupData[{
Cell[124268, 4321, 206, 6, 70, "Input",
 CellID->446810411],
Cell[124477, 4329, 500, 13, 36, "Output",
 CellID->229794115]
}, Open  ]],
Cell[124992, 4345, 154, 6, 70, "Text",
 CellID->253484638],
Cell[CellGroupData[{
Cell[125171, 4355, 150, 4, 70, "Input",
 CellID->215094552],
Cell[125324, 4361, 414, 12, 36, "Output",
 CellID->27746478]
}, Open  ]],
Cell[125753, 4376, 154, 6, 70, "Text",
 CellID->203098661],
Cell[CellGroupData[{
Cell[125932, 4386, 120, 4, 70, "Input",
 CellID->156425214],
Cell[126055, 4392, 414, 11, 36, "Output",
 CellID->10251174]
}, Open  ]],
Cell[126484, 4406, 136, 4, 70, "Text",
 CellID->64776907],
Cell[CellGroupData[{
Cell[126645, 4414, 283, 9, 70, "Input",
 CellID->57059381],
Cell[126931, 4425, 554, 16, 36, "Output",
 CellID->45924774]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[127546, 4448, 92, 1, 70, "Subsection",
 CellID->913638886],
Cell[127641, 4451, 393, 13, 70, "Text",
 CellID->213089269],
Cell[128037, 4466, 241, 5, 70, "Text",
 CellID->525928298],
Cell[128281, 4473, 320, 10, 70, "Input",
 CellID->239185265],
Cell[128604, 4485, 170, 4, 70, "Text",
 CellID->476901211],
Cell[CellGroupData[{
Cell[128799, 4493, 248, 8, 70, "Input",
 CellID->1385341],
Cell[129050, 4503, 237, 7, 36, "Output",
 CellID->808620393]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[129348, 4517, 53, 1, 70, "Section",
 CellID->273428036],
Cell[129404, 4520, 390, 7, 70, "Text",
 CellID->107864109]
}, Open  ]],
Cell[CellGroupData[{
Cell[129831, 4532, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[129899, 4535, 200, 6, 70, "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[CellGroupData[{
Cell[130136, 4546, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[130211, 4549, 176, 4, 70, "RelatedTutorials",
 CellID->17795993],
Cell[130390, 4555, 176, 4, 70, "RelatedTutorials",
 CellID->14999323],
Cell[130569, 4561, 166, 4, 70, "RelatedTutorials",
 CellID->280152894]
}, Open  ]]
}, Open  ]],
Cell[130762, 4569, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

