(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     16001,        403]
NotebookOptionsPosition[     10632,        280]
NotebookOutlinePosition[     14926,        366]
CellTagsIndexPosition[     14853,        361]
WindowTitle->1.1  Introduction to Airfoil Geometry - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Preface\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/PrefaceToAerodynamics"], "\<\"1.2 \
Airfoils From Data Files\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AirfoilsFromDataFiles"], "\<\"Overview\"\
\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"AirfoilGeometry\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/AirfoilGeometry"], "\<\"Aerodynamics\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/Aerodynamics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/tutorial/IntroductionToAirfoilGeometry\"\>\
":>None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/IntroductionToAirfoilGeometry"]}, 
            Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/IntroductionToAirfoilGeometry.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/IntroductionToAirfoilGeometry", ".html"]],
            None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["1.1  Introduction to Airfoil Geometry", "Title",
 CellID->509267359],

Cell[TextData[{
 "The term airfoil is used is several ways.  Historically, the lift of a \
classic aircraft with a high-aspect ratio wing was approximated by the lift \
of the wing alone.  For a high-aspect ratio wing, lift was estimated by \
solving a potential-flow model for two-dimensional flow over a wing section, \
and then modifying this result to account for three-dimensional effects for \
the finite wing.  This two-dimensional wing section is often referred to as \
an airfoil.  Experimental measurements for this two-dimensional problem are \
made by testing a rectangular wing of constant cross section that spans the \
test section of a wind tunnel.  Thinking of the theoretical two-dimensional \
problem, the airfoil is a contour in two-dimensional space, but , thinking of \
the wind-tunnel test, it makes sense to refer to the surface of an airfoil.  \
A summary of the state of airfoil aerodynamics in the 1930s is available in \
the Dover edition of ",
 ButtonBox["Abbott and von Doenhoff (1959)",
  BaseStyle->"Hyperlink",
  ButtonData->{"References.nb", "abbott1959"}],
 ".  By the 1970s panel methods were used in the aircraft industry on a daily \
basis to estimate lift and pitching-moment characteristics for entire \
aircraft at cruise conditions (",
 ButtonBox["Hess, 1990)",
  BaseStyle->"Hyperlink",
  ButtonData->{"References.nb", "hess1990"}],
 ".  The terminology of panel methods is three-dimensional, although the \
method can be used to solve two-dimensional problems as is done in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  For these historical reasons, the terms airfoil contour and airfoil \
surface are used interchangeably, and two-dimensional straight-line \
singularities are referred to as panels in the documentation.  "
}], "Text",
 CellTags->"abbott1959",
 CellID->171382246],

Cell[TextData[{
 "Airfoil geometry can be specified in different ways.  ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " provides tools for handling three methods for describing airfoil geometry: \
data files, equations for thickness and camber properties, and conformal \
mapping.  Regardless of the source of information, points on the surface of \
the airfoil are used as endpoints for straight-line panels describing the \
discretized airfoil used in a computational model."
}], "Text",
 CellID->68115679],

Cell[TextData[{
 "Probably, the most common method for specifying airfoil geometry is to \
provide a data file specifying coordinates of points on the airfoil contour.  \
The ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["AirfoilFromDataFile",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilFromDataFile"]], 
  "InlineFormula"],
 " imports this data and arranges it in a form used for computational models \
in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  For user-specified discretization, the function ",
 Cell[BoxData[
  ButtonBox["FitAirfoil",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/FitAirfoil"]], "InlineFormula"],
 " fits a B-spline to the data, and the resulting spline function is used as \
input to the function ",
 Cell[BoxData[
  ButtonBox["AirfoilSurfacePoints",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilSurfacePoints"]], 
  "InlineFormula"],
 " to generate the discretized airfoil."
}], "Text",
 CellID->7369620],

Cell[TextData[{
 "Early attempts to describe the aerodynamics of airfoils were based on \
families of airfoils defined by equations for thickness and camber \
properties.  These equations contained parameters that could be changed to \
alter these properties in a prescribed manner.  An example of this approach \
is the family of NACA four-digit airfoils whose properties are computed at a \
user-specified list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values by the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["NACA4DigitAirfoil",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/NACA4DigitAirfoil"]], "InlineFormula"],
 ".  The resulting thickness and camber properties are used as input to \
AirfoilSurfacePoints to generate panel endpoints for a discretized airfoil."
}], "Text",
 CellID->47466828],

Cell[TextData[{
 "Finally, transformation functions in complex variables can be used to map a \
circle to an airfoil shape.  The example used in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " is the van de Vooren airfoil, which is generated by a special definition \
for the function ",
 Cell[BoxData[
  ButtonBox["AirfoilSurfacePoints",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilSurfacePoints"]], 
  "InlineFormula"],
 "."
}], "Text",
 CellID->277569441],

Cell[TextData[{
 "Although procedures vary depending on the source of information for airfoil \
geometry, the output of AirfoilSurfacePoints is a collection of \
two-dimensional vectors locating points on the contour (surface) of an \
airfoil that specify endpoints of connected straight-line panels for the \
discretized airfoil.  Additional functions compute useful geometric \
properties of the discretized airfoil such as panel lengths, angles that \
panels make with the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis, unit normal vectors to panels and the trailing-edge angle."
}], "Text",
 CellID->535071041],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["AirfoilGeometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/guide/AirfoilGeometry"]], "TutorialMoreAbout",
 CellID->103402507],

Cell[TextData[ButtonBox["Aerodynamics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"]], "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["Preface",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PrefaceToAerodynamics"]], "RelatedTutorials",
 CellID->474440529],

Cell[TextData[ButtonBox["1.2 Airfoils From Data Files",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromDataFiles"]], "RelatedTutorials",
 CellID->134656653],

Cell[TextData[ButtonBox["Overview",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]], "RelatedTutorials",
 CellID->280152894]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"1.1  Introduction to Airfoil Geometry - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 4, 21.7668000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "None", "summary" -> 
    "The term airfoil is used is several ways. Historically, the lift of a \
classic aircraft with a high-aspect ratio wing was approximated by the lift \
of the wing alone. For a high-aspect ratio wing, lift was estimated by \
solving a potential-flow model for two-dimensional flow over a wing section, \
and then modifying this result to account for three-dimensional effects for \
the finite wing. This two-dimensional wing section is often referred to as an \
airfoil. Experimental measurements for this two-dimensional problem are made \
by testing a rectangular wing of constant cross section that spans the test \
section of a wind tunnel. Thinking of the theoretical two-dimensional \
problem, the airfoil is a contour in two-dimensional space, but , thinking of \
the wind-tunnel test, it makes sense to refer to the surface of an airfoil. A \
summary of the state of airfoil aerodynamics in the 1930s is available in the \
Dover edition of Abbott and von Doenhoff (1959). By the 1970s panel methods \
were used in the aircraft industry on a daily basis to estimate lift and \
pitching-moment characteristics for entire aircraft at cruise conditions \
(Hess, 1990). The terminology of panel methods is three-dimensional, although \
the method can be used to solve two-dimensional problems as is done in \
Aerodynamics. For these historical reasons, the terms airfoil contour and \
airfoil surface are used interchangeably, and two-dimensional straight-line \
singularities are referred to as panels in the documentation. Airfoil \
geometry can be specified in different ways. Aerodynamics provides tools for \
handling three methods for describing airfoil geometry: data files, equations \
for thickness and camber properties, and conformal mapping. Regardless of the \
source of information, points on the surface of the airfoil are used as \
endpoints for straight-line panels describing the discretized airfoil used in \
a computational model. Probably, the most common method for specifying \
airfoil geometry is to provide a data file specifying coordinates of points \
on the airfoil contour. The Aerodynamics function AirfoilFromDataFile imports \
this data and arranges it in a form used for computational models in \
Aerodynamics. For user-specified discretization, the function FitAirfoil fits \
a B-spline to the data, and the resulting spline function is used as input to \
the function AirfoilSurfacePoints to generate the discretized airfoil.", 
    "synonyms" -> {}, "title" -> "1.1  Introduction to Airfoil Geometry", 
    "type" -> "Tutorial", "uri" -> 
    "Aerodynamics/tutorial/IntroductionToAirfoilGeometry"}, "LinkTrails" -> 
  "", "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "abbott1959"->{
  Cell[4074, 106, 1837, 32, 70, "Text",
   CellTags->"abbott1959",
   CellID->171382246]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"abbott1959", 14739, 354}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[630, 21, 78, 1, 70, "TutorialColorBar"],
Cell[711, 24, 191, 5, 70, "LinkTrail"],
Cell[905, 31, 3067, 68, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3997, 103, 74, 1, 70, "Title",
 CellID->509267359],
Cell[4074, 106, 1837, 32, 70, "Text",
 CellTags->"abbott1959",
 CellID->171382246],
Cell[5914, 140, 516, 10, 70, "Text",
 CellID->68115679],
Cell[6433, 152, 1060, 30, 70, "Text",
 CellID->7369620],
Cell[7496, 184, 892, 20, 70, "Text",
 CellID->47466828],
Cell[8391, 206, 491, 14, 70, "Text",
 CellID->277569441],
Cell[8885, 222, 636, 12, 70, "Text",
 CellID->535071041],
Cell[CellGroupData[{
Cell[9546, 238, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[9614, 241, 166, 4, 70, "TutorialMoreAbout",
 CellID->103402507],
Cell[9783, 247, 158, 3, 70, "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[CellGroupData[{
Cell[9978, 255, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[10053, 258, 166, 4, 70, "RelatedTutorials",
 CellID->474440529],
Cell[10222, 264, 187, 4, 70, "RelatedTutorials",
 CellID->134656653],
Cell[10412, 270, 166, 4, 70, "RelatedTutorials",
 CellID->280152894]
}, Open  ]]
}, Open  ]],
Cell[10605, 278, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

