(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     32100,       1040]
NotebookOptionsPosition[     23606,        776]
NotebookOutlinePosition[     29039,        926]
CellTagsIndexPosition[     28945,        920]
WindowTitle->2.1  Introduction to Influence Coefficients - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"1.5 Panel Geometry\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/PanelGeometry"], "\<\"2.2 Point \
Singularities\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/PointSingularities"], "\<\"Overview\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"InfluenceCoefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/InfluenceCoefficients"], \
"\<\"Aerodynamics\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/Aerodynamics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/tutorial/\
IntroductionToInfluenceCoefficients\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
             "Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"]}, 
            Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/IntroductionToInfluenceCoefficients.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/IntroductionToInfluenceCoefficients", 
             ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["2.1  Introduction to Influence Coefficients", "Title",
 CellID->509267359],

Cell[TextData[{
 "Velocity, velocity potential and stream function at a point due to a \
singularity can be written as the product of one or more geometric influence \
coefficients and strength parameter(s) for the singularity.  The basic \
potential-flow singularities are source, vortex and doublet.  These \
singularities can be concentrated at a point or distributed over lines or \
surfaces, and can be used in combination, providing a rich variety of options \
for constructing panel methods to compute potential flow over an object.  The \
current version of ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " is restricted to models using two-dimensional point and straight-line \
singularities.  For line singularities, the strength density function is a \
polynomial in distance along the line segment."
}], "Text",
 CellID->351158547],

Cell[CellGroupData[{

Cell["\<\
2.1.1 Geometric influence coefficients and strength parameters\
\>", "Section",
 CellTags->"geom",
 CellID->320914423],

Cell["\<\
The sketch below illustrates the local coordinate system used for an isolated \
two-dimensional line singularity.  In keeping with the terminology of panel \
methods, this singularity is referred to as a panel.\
\>", "Text",
 CellTags->"geom",
 CellID->1534169418],

Cell[BoxData[
 GraphicsBox[{
   {Thickness[0.003890451449942805], 
    LineBox[{{0.09444444444444444, 0.06944444444444442}, {0.6444444444444443, 
     0.06944444444444442}}]}, 
   {Thickness[0.00041686938347033507`], 
    LineBox[{{0.09722205555555558, 0.06666666666666665}, {0.09722205555555558,
      0.9055555555555554}}]}, 
   {Thickness[0.00041686938347033507`], 
    LineBox[{{0.09444427777777778, 0.06944444444444442}, {1.1944442777777777`,
      0.06944444444444442}}]}, 
   {AbsolutePointSize[6.095368972401692], 
    PointBox[{1.1111109444444442`, 0.825}]}, 
   {AbsolutePointSize[6.095368972401692], Thickness[0.00041686938347033507`], 
    EdgeForm[{GrayLevel[0.], Opacity[1.], Thickness[
    0.00041686938347033507`]}], 
    ArrowBox[{{0.09722205555555558, 0.06944444444444442}, {
     1.1111109444444442`, 0.8222222222222222}}]}, 
   {AbsolutePointSize[6.095368972401692], Thickness[0.00041686938347033507`], 
    EdgeForm[{GrayLevel[0.], Opacity[1.], Thickness[
    0.00041686938347033507`]}], 
    ArrowBox[{{0.6444442777777777, 0.06666666666666665}, {1.1111109444444442`,
      0.825}}]}, 
   InsetBox["", {1.3055553888888887`, 0.42222222222222217`}, {Left, Baseline},
    Alignment->{Left, Top}], InsetBox[
    StyleBox[Cell["P",
      GeneratedCell->False,
      CellAutoOverwrite->False,
      CellBaseline->Baseline,
      TextAlignment->Left],
     FontSize->14,
     Background->GrayLevel[1.]], {1.152777611111111, 0.8638888888888889}, {
    Left, Baseline},
    Alignment->{Left, Top}], InsetBox[
    StyleBox[
     TagBox[
      SubscriptBox["r", "2"],
      HoldForm],
     FontSize->14,
     FontSlant->"Italic",
     Background->GrayLevel[1.]], {0.8805553888888888, 0.36111111111111105`}, {
    Left, Baseline},
    Alignment->{Left, Top}], InsetBox[
    StyleBox[
     TagBox[
      SubscriptBox["r", "1"],
      HoldForm],
     FontSize->14,
     FontSlant->"Italic",
     Background->GrayLevel[1.]], {0.5444442777777778, 0.461111111111111}, {
    Left, Baseline},
    Alignment->{Left, Top}], InsetBox[
    StyleBox[
     TagBox["",
      HoldForm],
     FontSize->14,
     FontSlant->"Italic",
     Background->GrayLevel[1.]], {0.6638887222222222, 0.49444444444444446`}, {
    Left, Baseline},
    Alignment->{Left, Top}], 
   StyleBox[InsetBox[
     StyleBox[
      TagBox["\[Delta]",
       HoldForm],
      TextAlignment->Center,
      Background->GrayLevel[
       1.]], {0.41111094444444435`, 0.0972222222222221}, {Left, Baseline},
     Alignment->{Left, Top}],
    FontSize->14], InsetBox[
    StyleBox[
     TagBox[
      SubscriptBox["\[Theta]", "1"],
      HoldForm],
     TextAlignment->Center,
     FontSize->14,
     Background->GrayLevel[1.]], {0.2249998333333333, 0.09722222222222232}, {
    Left, Baseline},
    Alignment->{Left, Top}], InsetBox[
    StyleBox[
     TagBox[
      SubscriptBox["\[Theta]", "2"],
      HoldForm],
     TextAlignment->Center,
     FontSize->14,
     Background->GrayLevel[1.]], {0.7138887222222221, 0.09444444444444433}, {
    Left, Baseline},
    Alignment->{Left, Top}], InsetBox[
    StyleBox[
     TagBox["x",
      HoldForm],
     TextAlignment->Center,
     FontSize->14,
     Background->GrayLevel[1.]], {1.2305553888888887`, 0.05833333333333335}, {
    Left, Baseline},
    Alignment->{Left, Top}], InsetBox[
    StyleBox[
     TagBox["z",
      HoldForm],
     TextAlignment->Center,
     FontSize->14,
     Background->GrayLevel[1.]], {0.08888872222222224, 0.9388888888888889}, {
    Left, Baseline},
    Alignment->{Left, Top}]},
  ContentSelectable->True,
  ImagePadding->{{0., 0.}, {0., 0.}},
  ImageSize->{480, 360},
  PlotRange->{{0., 1.333333}, {0., 1.}},
  PlotRangePadding->Automatic]], "Input",
 CellID->193927183],

Cell[TextData[{
 "The panel extends from the origin, along the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis for the length of the panel, ",
 Cell[BoxData[
  StyleBox["\[Delta]", "TR"]], "InlineFormula"],
 ".  A field point at which velocity, velocity potential or stream function \
are computed is labeled P.  Geometric influence coefficients depend on the \
displacement vectors from ends of the panel to P.  Algebraic expressions for \
influence coefficients are sometimes simpler in appearance when written in \
terms of the angles ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["\[Theta]", "TR"], 
   StyleBox["1", "TR"]]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["\[Theta]", "TR"], 
   StyleBox["2", "TR"]]], "InlineFormula"],
 "."
}], "Text",
 CellTags->"geom",
 CellID->344307243],

Cell[TextData[{
 "If the singularity density function (source, vortex or doublet) for the \
panel is a constant ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ", then the velocity components, velocity potential and stream function can \
be written as products of the appropriate influence coefficient (",
 Cell[BoxData[
  StyleBox["ic", "TI"]], "InlineFormula"],
 ") and the strength parameter. "
}], "Text",
 CellTags->"geom",
 CellID->39044744],

Cell[TextData[{
 "\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["f", "TI"], 
    RowBox[{"(", 
     StyleBox["x", "TI"], ")"}]}], "=", 
   StyleBox["a", "TI"], " "}]], "InlineFormula"]
}], "Text",
 CellTags->"geom",
 CellID->434040083],

Cell[TextData[{
 "\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      StyleBox["u", "TI"], ",", 
      StyleBox["w", "TI"]}], "}"}], "=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       SubscriptBox[
        StyleBox["ic", "TI"], "u"], " ", 
       StyleBox["a", "TI"]}], ",", 
      RowBox[{
       SubscriptBox[
        StyleBox["ic", "TI"], "w"], " ", 
       StyleBox["a", "TI"]}]}], "}"}]}], ",", " ", 
   RowBox[{
    StyleBox["\[Phi]", "TR"], "=", 
    RowBox[{
     SubscriptBox[
      StyleBox["ic", "TI"], 
      StyleBox["\[Phi]", "TR"]], " ", 
     StyleBox["a", "TI"]}]}], ",", " ", 
   RowBox[{
    StyleBox["\[Psi]", "TR"], "=", 
    RowBox[{
     SubscriptBox[
      StyleBox["ic", "TI"], 
      StyleBox["\[Psi]", "TR"]], " ", 
     StyleBox["a", "TI"]}]}]}]], "InlineFormula"]
}], "Text",
 CellTags->"geom",
 CellID->103946562],

Cell["\<\
If the singularity density varies linearly along the panel, then the velocity \
components, velocity potential and stream function can be written as products \
of appropriate influence coefficients and strength parameters as follows.\
\>", "Text",
 CellTags->"geom",
 CellID->559574336],

Cell[TextData[{
 "\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["f", "TI"], 
    RowBox[{"(", 
     StyleBox["x", "TI"], ")"}]}], "=", 
   RowBox[{
    StyleBox["a", "TI"], "+", 
    RowBox[{
     StyleBox["b", "TI"], " ", 
     StyleBox["x", "TI"]}]}]}]], "InlineFormula"]
}], "Text",
 CellTags->"geom",
 CellID->728334774],

Cell[TextData[{
 "\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      StyleBox["u", "TI"], ",", 
      StyleBox["w", "TI"]}], "}"}], "=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox[
         StyleBox["ica", "TI"], "u"], " ", 
        StyleBox["a", "TI"]}], "+", 
       RowBox[{
        SubscriptBox[
         StyleBox["icb", "TI"], "u"], " ", 
        StyleBox["b", "TI"]}]}], ",", 
      RowBox[{
       RowBox[{
        SubscriptBox[
         StyleBox["ica", "TI"], "w"], " ", 
        StyleBox["a", "TI"]}], "+", 
       RowBox[{
        SubscriptBox[
         StyleBox["icb", "TI"], "w"], " ", 
        StyleBox["b", "TI"]}]}]}], "}"}]}], ",", " ", 
   RowBox[{
    StyleBox["\[Phi]", "TR"], "=", 
    RowBox[{
     RowBox[{
      SubscriptBox[
       StyleBox["ica", "TI"], 
       StyleBox["\[Phi]", "TR"]], " ", 
      StyleBox["a", "TI"]}], "+", 
     RowBox[{
      SubscriptBox[
       StyleBox["icb", "TI"], 
       StyleBox["\[Phi]", "TR"]], " ", 
      StyleBox["b", "TI"]}]}]}], ",", " ", 
   RowBox[{
    StyleBox["\[Psi]", "TR"], "=", 
    RowBox[{
     RowBox[{
      SubscriptBox[
       StyleBox["ica", "TI"], 
       StyleBox["\[Psi]", "TR"]], " ", 
      StyleBox["a", "TI"]}], "+", 
     RowBox[{
      SubscriptBox[
       StyleBox["icb", "TI"], 
       StyleBox["\[Psi]", "TR"]], " ", 
      StyleBox["b", "TI"]}]}]}]}]], "InlineFormula"]
}], "Text",
 CellTags->"geom",
 CellID->37516354],

Cell[TextData[{
 "A straightforward generalization of these equations serves for strength \
distributions described by higher-order polynomials.  Functions are defined \
in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " to compute influence-coefficients functions for source, vortex and doublet \
panels of constant and linearly varying strength.  Additionally, \
influence-coefficients are defined for doublet panels of quadratically \
varying strength."
}], "Text",
 CellTags->"geom",
 CellID->377094181],

Cell[TextData[{
 "Names for all influence-coefficient functions defined in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " begin with IC and continue with descriptive names.  For example, velocity, \
velocity-potential and stream-function influence-coefficient functions for \
vortex panels of linearly varying circulation density are named \
ICVortexLinear, ICPhiVortexLinear and ICPsiVortexLinear.  Additionally, \
influence-coefficient functions are provided for two-dimensional point \
source, vortex and doublet singularities."
}], "Text",
 CellTags->"geom",
 CellID->52889481]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
2.1.2 Basic relations between two-dimensional vortex and doublet panels\
\>", "Section",
 CellTags->"relation",
 CellID->133089311],

Cell[TextData[{
 "The general solution of Laplace's equation, based on one of Green's \
identifies, is discussed in Chapter 3 of ",
 ButtonBox["Katz and Plotkin (2001)",
  BaseStyle->"Hyperlink",
  ButtonData->{"References.nb", "katz2001"}],
 ".  This development leads naturally to the use of source, vortex and \
doublet singularities.  There is a relationship between circulation density \
and doublet density for vortex and doublet panels which can be stated as \
follows."
}], "Text",
 CellTags->"relation",
 CellID->179165981],

Cell[TextData[{
 "\t\t\t\t ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     StyleBox["\[Gamma]", "TR"], "(", 
     StyleBox["x", "TI"], ")"}], "=", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{
       StyleBox["d\[Mu]", "TI"], "(", 
       StyleBox["x", "TI"], ")"}], 
      StyleBox["dx", "TI"]]}]}], TraditionalForm]]]
}], "Text",
 CellTags->"relation",
 CellID->60328517],

Cell[TextData[{
 "Circulation density is denoted by ",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Gamma]", "TR"], 
   RowBox[{"(", 
    StyleBox["x", "TI"], ")"}]}]], "InlineFormula"],
 " and doublet density is denoted by ",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Mu]", "TR"], 
   RowBox[{"(", 
    StyleBox["x", "TI"], ")"}]}]], "InlineFormula"],
 ", where ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 " denotes the distance along the panel in a panel-local coordinate system."
}], "Text",
 CellTags->"relation",
 CellID->22533550],

Cell[TextData[{
 "The total strength of a vortex panel extending from ",
 Cell[BoxData["0"], "InlineFormula"],
 " to ",
 Cell[BoxData["\[Delta]"], "InlineFormula"],
 " is obtained by integrating the circulation density function over the panel."
}], "Text",
 CellTags->"relation",
 CellID->154263561],

Cell[TextData[{
 "\t\t\t\t",
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[CapitalGamma]", "=", 
    RowBox[{
     SubsuperscriptBox["\[Integral]", "0", "\[Delta]"], 
     RowBox[{
      RowBox[{"\[Gamma]", "(", "x", ")"}], 
      RowBox[{"\[DifferentialD]", "x"}]}]}]}], TraditionalForm]]]
}], "Text",
 CellTags->"relation",
 CellID->52774633],

Cell["\<\
Substitute for the circulation density in terms of the doublet density.\
\>", "Text",
 CellTags->"relation",
 CellID->45577279],

Cell[TextData[{
 "\t\t\t\t ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    StyleBox["\[CapitalGamma]", "TR"], "=", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SubsuperscriptBox["\[Integral]", "0", 
        StyleBox["\[Delta]", "TR"]], 
       RowBox[{
        FractionBox[
         RowBox[{
          StyleBox["d\[Mu]", "TI"], "(", 
          StyleBox["x", "TI"], ")"}], 
         StyleBox["dx", "TI"]], 
        RowBox[{"\[DifferentialD]", 
         StyleBox["x", "TI"]}]}]}]}], "=", 
     RowBox[{
      RowBox[{
       StyleBox["\[Mu]", "TR"], "(", "0", ")"}], "-", 
      RowBox[{
       StyleBox["\[Mu]", "TR"], "(", 
       StyleBox["\[Delta]", "TR"], ")"}]}]}]}], TraditionalForm]]]
}], "Text",
 CellTags->"relation",
 CellID->482647799],

Cell[CellGroupData[{

Cell["Constant-strength doublet panel", "Subsection",
 CellID->299286692],

Cell[TextData[{
 "For example, set the doublet density to a constant, ",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Mu]", "TR"], "=", 
   StyleBox["a", "TI"]}]], "InlineFormula"],
 ".  Then the circulation density on the panel is zero and the total \
circulation for the panel is zero."
}], "Text",
 CellTags->"relation",
 CellID->30442929],

Cell[BoxData[
 FormBox[
  RowBox[{"\t\t\t\t", 
   RowBox[{
    StyleBox["\[CapitalGamma]", "TR"], "=", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SubsuperscriptBox["\[Integral]", "0", 
        StyleBox["\[Delta]", "TR"]], 
       RowBox[{
        FractionBox[
         RowBox[{
          StyleBox["d\[Mu]", "TI"], "(", 
          StyleBox["x", "TI"], ")"}], 
         StyleBox["dx", "TI"]], 
        RowBox[{"\[DifferentialD]", 
         StyleBox["x", "TI"]}]}]}]}], "=", 
     RowBox[{
      RowBox[{
       StyleBox["a", "TI"], "-", 
       StyleBox["a", "TI"]}], "=", "0"}]}]}]}], TraditionalForm]], "Text",
 CellTags->"relation",
 CellID->622982394],

Cell[TextData[{
 "To see what is happening at the end points, look at the integral from \
negative infinity to just inside the panel near ",
 Cell[BoxData[
  RowBox[{
   StyleBox["x", "TI"], "=", "0"}]], "InlineFormula"],
 ".  Since the doublet density is zero off the panel and changes abruptly to ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " on the panel, the circulation computed is the value of a point vortex \
located at ",
 Cell[BoxData[
  RowBox[{
   StyleBox["x", "TI"], "=", "0"}]], "InlineFormula"],
 "."
}], "Text",
 CellTags->"relation",
 CellID->701435519],

Cell[BoxData[
 FormBox[
  RowBox[{"\t\t\t\t", 
   RowBox[{
    StyleBox[
     RowBox[{"\[CapitalGamma]", "(", "0", ")"}], "TR"], "=", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SubsuperscriptBox["\[Integral]", 
        RowBox[{"-", "\[Infinity]"}], 
        StyleBox["0", "TR"]], 
       RowBox[{
        FractionBox[
         RowBox[{
          StyleBox["d\[Mu]", "TI"], "(", 
          StyleBox["x", "TI"], ")"}], 
         StyleBox["dx", "TI"]], 
        RowBox[{"\[DifferentialD]", 
         StyleBox["x", "TI"]}]}]}]}], "=", 
     RowBox[{
      RowBox[{
       RowBox[{
        StyleBox["\[Mu]", "TR"], "(", 
        RowBox[{"-", "\[Infinity]"}], ")"}], "-", 
       RowBox[{
        StyleBox["\[Mu]", "TR"], "(", 
        StyleBox["0", "TR"], ")"}]}], "=", 
      RowBox[{"-", 
       StyleBox["a", "TI"]}]}]}]}]}], TraditionalForm]], "Text",
 CellTags->"relation",
 CellID->249012586],

Cell[TextData[{
 "Likewise, the circulation associated with the endpoint of the panel ",
 Cell[BoxData[
  RowBox[{
   StyleBox["x", "TI"], "=", 
   StyleBox["\[Delta]", "TR"]}]], "InlineFormula"],
 " is obtained by evaluating the integral."
}], "Text",
 CellTags->"relation",
 CellID->330029328],

Cell[BoxData[
 FormBox[
  RowBox[{"\t\t\t\t", 
   RowBox[{
    StyleBox[
     RowBox[{"\[CapitalGamma]", "(", "\[Delta]", ")"}], "TR"], "=", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SubsuperscriptBox["\[Integral]", "\[Delta]", 
        StyleBox["\[Infinity]", "TR"]], 
       RowBox[{
        FractionBox[
         RowBox[{
          StyleBox["d\[Mu]", "TI"], "(", 
          StyleBox["x", "TI"], ")"}], 
         StyleBox["dx", "TI"]], 
        RowBox[{"\[DifferentialD]", 
         StyleBox["x", "TI"]}]}]}]}], "=", 
     RowBox[{
      RowBox[{
       RowBox[{
        StyleBox["\[Mu]", "TR"], "(", "\[Delta]", ")"}], "-", 
       RowBox[{
        StyleBox["\[Mu]", "TR"], "(", "\[Infinity]", ")"}]}], "=", 
      StyleBox["a", "TI"]}]}]}]}], TraditionalForm]], "Text",
 CellTags->"relation",
 CellID->710526163],

Cell[TextData[{
 "Thus, a constant-strength doublet panel gives the same flow field as a \
contra-rotating pair of point vortices of strength \[MinusPlus]",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " at panel endpoints.  In general, the flow field induced by a doublet panel \
whose doublet density function is described by a polynomial of order ",
 Cell[BoxData[
  StyleBox["n", "TI"]], "InlineFormula"],
 " is equivalent to that of a vortex panel whose circulation density is a \
polynomial of order ",
 Cell[BoxData[
  RowBox[{
   StyleBox["n", "TI"], "-", "1"}]], "InlineFormula"],
 " and point vortices at panel endpoints."
}], "Text",
 CellTags->"relation",
 CellID->4484441]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["InfluenceCoefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/guide/InfluenceCoefficients"]], "TutorialMoreAbout",
 CellID->1567025153],

Cell[TextData[ButtonBox["Aerodynamics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"]], "TutorialMoreAbout",
 CellID->80129204]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["1.5 Panel Geometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PanelGeometry"]], "RelatedTutorials",
 CellID->574311372],

Cell[TextData[ButtonBox["2.2 Point Singularities",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PointSingularities"]], "RelatedTutorials",
 CellID->895221539],

Cell[TextData[ButtonBox["Overview",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]], "RelatedTutorials",
 CellID->6988088]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"2.1  Introduction to Influence Coefficients - Wolfram \
Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 4, 24.0132000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "None", "summary" -> 
    "Velocity, velocity potential and stream function at a point due to a \
singularity can be written as the product of one or more geometric influence \
coefficients and strength parameter(s) for the singularity. The basic \
potential-flow singularities are source, vortex and doublet. These \
singularities can be concentrated at a point or distributed over lines or \
surfaces, and can be used in combination, providing a rich variety of options \
for constructing panel methods to compute potential flow over an object. The \
current version of Aerodynamics is restricted to models using two-dimensional \
point and straight-line singularities. For line singularities, the strength \
density function is a polynomial in distance along the line segment. The \
sketch below illustrates the local coordinate system used for an isolated \
two-dimensional line singularity. In keeping with the terminology of panel \
methods, this singularity is referred to as a panel. The panel extends from \
the origin, along the x-axis for the length of the panel, \\[Delta]. A field \
point at which velocity, velocity potential or stream function are computed \
is labeled P. Geometric influence coefficients depend on the displacement \
vectors from ends of the panel to P. Algebraic expressions for influence \
coefficients are sometimes simpler in appearance when written in terms of the \
angles \\[Theta]_1 and \\[Theta]_2.", "synonyms" -> {}, "title" -> 
    "2.1  Introduction to Influence Coefficients", "type" -> "Tutorial", 
    "uri" -> "Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"}, 
  "LinkTrails" -> "", "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "geom"->{
  Cell[5032, 129, 128, 4, 70, "Section",
   CellTags->"geom",
   CellID->320914423],
  Cell[5163, 135, 274, 6, 70, "Text",
   CellTags->"geom",
   CellID->1534169418],
  Cell[9153, 253, 840, 24, 70, "Text",
   CellTags->"geom",
   CellID->344307243],
  Cell[9996, 279, 460, 12, 70, "Text",
   CellTags->"geom",
   CellID->39044744],
  Cell[10459, 293, 247, 11, 70, "Text",
   CellTags->"geom",
   CellID->434040083],
  Cell[10709, 306, 883, 35, 70, "Text",
   CellTags->"geom",
   CellID->103946562],
  Cell[11595, 343, 296, 6, 70, "Text",
   CellTags->"geom",
   CellID->559574336],
  Cell[11894, 351, 336, 15, 70, "Text",
   CellTags->"geom",
   CellID->728334774],
  Cell[12233, 368, 1486, 57, 70, "Text",
   CellTags->"geom",
   CellID->37516354],
  Cell[13722, 427, 515, 12, 70, "Text",
   CellTags->"geom",
   CellID->377094181],
  Cell[14240, 441, 590, 12, 70, "Text",
   CellTags->"geom",
   CellID->52889481]},
 "relation"->{
  Cell[14867, 458, 141, 4, 70, "Section",
   CellTags->"relation",
   CellID->133089311],
  Cell[15011, 464, 532, 12, 70, "Text",
   CellTags->"relation",
   CellID->179165981],
  Cell[15546, 478, 390, 16, 70, "Text",
   CellTags->"relation",
   CellID->60328517],
  Cell[15939, 496, 548, 19, 70, "Text",
   CellTags->"relation",
   CellID->22533550],
  Cell[16490, 517, 299, 8, 70, "Text",
   CellTags->"relation",
   CellID->154263561],
  Cell[16792, 527, 344, 13, 70, "Text",
   CellTags->"relation",
   CellID->52774633],
  Cell[17139, 542, 137, 4, 70, "Text",
   CellTags->"relation",
   CellID->45577279],
  Cell[17279, 548, 752, 27, 70, "Text",
   CellTags->"relation",
   CellID->482647799],
  Cell[18132, 582, 338, 10, 70, "Text",
   CellTags->"relation",
   CellID->30442929],
  Cell[18473, 594, 665, 23, 70, "Text",
   CellTags->"relation",
   CellID->622982394],
  Cell[19141, 619, 588, 17, 70, "Text",
   CellTags->"relation",
   CellID->701435519],
  Cell[19732, 638, 904, 31, 70, "Text",
   CellTags->"relation",
   CellID->249012586],
  Cell[20639, 671, 295, 9, 70, "Text",
   CellTags->"relation",
   CellID->330029328],
  Cell[20937, 682, 829, 27, 70, "Text",
   CellTags->"relation",
   CellID->710526163],
  Cell[21769, 711, 698, 17, 70, "Text",
   CellTags->"relation",
   CellID->4484441]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"geom", 26674, 837},
 {"relation", 27609, 871}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[636, 21, 78, 1, 70, "TutorialColorBar"],
Cell[717, 24, 191, 5, 70, "LinkTrail"],
Cell[911, 31, 3138, 72, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4074, 107, 80, 1, 70, "Title",
 CellID->509267359],
Cell[4157, 110, 850, 15, 70, "Text",
 CellID->351158547],
Cell[CellGroupData[{
Cell[5032, 129, 128, 4, 70, "Section",
 CellTags->"geom",
 CellID->320914423],
Cell[5163, 135, 274, 6, 70, "Text",
 CellTags->"geom",
 CellID->1534169418],
Cell[5440, 143, 3710, 108, 70, "Input",
 CellID->193927183],
Cell[9153, 253, 840, 24, 70, "Text",
 CellTags->"geom",
 CellID->344307243],
Cell[9996, 279, 460, 12, 70, "Text",
 CellTags->"geom",
 CellID->39044744],
Cell[10459, 293, 247, 11, 70, "Text",
 CellTags->"geom",
 CellID->434040083],
Cell[10709, 306, 883, 35, 70, "Text",
 CellTags->"geom",
 CellID->103946562],
Cell[11595, 343, 296, 6, 70, "Text",
 CellTags->"geom",
 CellID->559574336],
Cell[11894, 351, 336, 15, 70, "Text",
 CellTags->"geom",
 CellID->728334774],
Cell[12233, 368, 1486, 57, 70, "Text",
 CellTags->"geom",
 CellID->37516354],
Cell[13722, 427, 515, 12, 70, "Text",
 CellTags->"geom",
 CellID->377094181],
Cell[14240, 441, 590, 12, 70, "Text",
 CellTags->"geom",
 CellID->52889481]
}, Open  ]],
Cell[CellGroupData[{
Cell[14867, 458, 141, 4, 70, "Section",
 CellTags->"relation",
 CellID->133089311],
Cell[15011, 464, 532, 12, 70, "Text",
 CellTags->"relation",
 CellID->179165981],
Cell[15546, 478, 390, 16, 70, "Text",
 CellTags->"relation",
 CellID->60328517],
Cell[15939, 496, 548, 19, 70, "Text",
 CellTags->"relation",
 CellID->22533550],
Cell[16490, 517, 299, 8, 70, "Text",
 CellTags->"relation",
 CellID->154263561],
Cell[16792, 527, 344, 13, 70, "Text",
 CellTags->"relation",
 CellID->52774633],
Cell[17139, 542, 137, 4, 70, "Text",
 CellTags->"relation",
 CellID->45577279],
Cell[17279, 548, 752, 27, 70, "Text",
 CellTags->"relation",
 CellID->482647799],
Cell[CellGroupData[{
Cell[18056, 579, 73, 1, 70, "Subsection",
 CellID->299286692],
Cell[18132, 582, 338, 10, 70, "Text",
 CellTags->"relation",
 CellID->30442929],
Cell[18473, 594, 665, 23, 70, "Text",
 CellTags->"relation",
 CellID->622982394],
Cell[19141, 619, 588, 17, 70, "Text",
 CellTags->"relation",
 CellID->701435519],
Cell[19732, 638, 904, 31, 70, "Text",
 CellTags->"relation",
 CellID->249012586],
Cell[20639, 671, 295, 9, 70, "Text",
 CellTags->"relation",
 CellID->330029328],
Cell[20937, 682, 829, 27, 70, "Text",
 CellTags->"relation",
 CellID->710526163],
Cell[21769, 711, 698, 17, 70, "Text",
 CellTags->"relation",
 CellID->4484441]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[22516, 734, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[22584, 737, 179, 4, 70, "TutorialMoreAbout",
 CellID->1567025153],
Cell[22766, 743, 156, 3, 70, "TutorialMoreAbout",
 CellID->80129204]
}, Open  ]],
Cell[CellGroupData[{
Cell[22959, 751, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[23034, 754, 169, 4, 70, "RelatedTutorials",
 CellID->574311372],
Cell[23206, 760, 179, 4, 70, "RelatedTutorials",
 CellID->895221539],
Cell[23388, 766, 164, 4, 70, "RelatedTutorials",
 CellID->6988088]
}, Open  ]]
}, Open  ]],
Cell[23579, 774, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

