(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     42435,       1129]
NotebookOptionsPosition[     36520,        945]
NotebookOutlinePosition[     38934,       1001]
CellTagsIndexPosition[     38891,        998]
WindowTitle->3.1  Introduction to Panel Methods - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"2.3 Isolated Panel\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IsolatedPanel"], "\<\"3.2 Validation \
Example\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/ValidationExample"], "\<\"Overview\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PanelMethods\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/PanelMethods"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Functions \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PMDoubletConstantDirichlet\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PMDoubletConstantDirichlet"], \
"\<\"PMDoubletQuadraticDirichlet\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PMDoubletQuadraticDirichlet"], \
"\<\"PMVortexLinearNeumann\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PMVortexLinearNeumann"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/tutorial/IntroductionToPanelMethods\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/IntroductionToPanelMethods"]}, 
            Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/IntroductionToPanelMethods.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/IntroductionToPanelMethods", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["3.1  Introduction to Panel Methods", "Title",
 CellID->509267359],

Cell[TextData[{
 "Panel methods are numerical models for solving potential flow over an \
object, usually with relatively complex geometry such as an aircraft or its \
components.  These methods are based on a boundary-integral solution to \
Laplace's equation.  Source, vortex or doublet singularities are placed on \
the boundary of the object and suitable boundary conditions are applied.  \
Refer to ",
 ButtonBox["Low-Speed Aerodynamics by Katz and Plotkin (2001)",
  BaseStyle->"Hyperlink",
  ButtonData->{"References.nb", "katz2001"}],
 " for details."
}], "Text",
 CellID->1534169418],

Cell[TextData[{
 "Currently, the panel methods in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " are restricted to steady two-dimensional flow over a discretized airfoil \
specified by straight-line panels.  Three panel methods are implemented by \
defined functions in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellID->775308575],

Cell[BoxData[GridBox[{
   {
    ButtonBox["PMVortexLinearNeumann",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/PMVortexLinearNeumann"], Cell["\<\
panel method using vortex panels of linearly varying strength and external \
Neumann boundary conditions \
\>", "TableText"]},
   {
    ButtonBox["PMDoubletConstantDirichlet",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/PMDoubletConstantDirichlet"], 
    Cell["\<\
panel method using doublet panels of constant strength and internal Dirichlet \
boundary conditions\
\>", "TableText"]},
   {
    ButtonBox["PMDoubletQuadraticDirichlet",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/PMDoubletQuadraticDirichlet"], 
    Cell["\<\
panel method using doublet panels of quadratically varying strength and \
internal Dirichlet boundary conditions\
\>", "TableText"]}
  }]], "DefinitionBox",
 CellID->2096742444],

Cell["Functions implementing two-dimensional panel methods.", "Caption",
 CellID->1891092685],

Cell[CellGroupData[{

Cell["3.1.1  Steps in implementation", "Section",
 CellID->432056375],

Cell["\<\
Although steps described below are common to the construction of any panel \
method, many choices remain.  These choices of details may have significant \
effects on model results. \
\>", "Text",
 CellID->250015064],

Cell[CellGroupData[{

Cell["Airfoil discretization", "Subsection",
 CellID->220682699],

Cell[TextData[{
 "Use points on the surface of an airfoil as endpoints for straight-line \
panels approximating the shape of the airfoil.  The resulting shape is \
referred to as the discretized airfoil.  In a commonly used airfoil-fixed \
coordinate system, the leading edge of the airfoil is located at the origin \
with the chord line coinciding with the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis.  This coordinate system is normalized such that the chord of the \
airfoil is one.  For airfoils specified by data files, there may be \
deviations from this convention in terms of the locations of leading and \
trailing edges.  Support functions constructing the discretized airfoil for \
use in panel methods are contained in the package ",
 Cell[BoxData[
  ButtonBox["AirfoilGeometry",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/guide/AirfoilGeometry"]], "InlineFormula"],
 "."
}], "Text",
 CellID->612564464]
}, Open  ]],

Cell[CellGroupData[{

Cell["Singularity distribution", "Subsection",
 CellID->64226099],

Cell[TextData[{
 "Singularities can be source, vortex or doublet, sometimes in combination.  \
Geometrically, they can be point, line or surface singularities.  For \
example, the two-dimensional panel methods in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " use two-dimensional line singularities: PMVortexLinearNeumann uses vortex \
panels of linearly varying strength, PMDoubletConstantDirichlet uses \
constant-strength doublet panels and PMDoubletQuadraticDirichlet uses doublet \
panels of quadratically varying strength.  Velocity, velocity potential and \
stream function induced at a point by a panel can be written as the product \
of geometric influence coefficients for the singularity and strength \
parameters.  Influence coefficients for commonly used two-dimensional \
singularities are contained in the package ",
 Cell[BoxData[
  ButtonBox["InfluenceCoefficients",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/guide/InfluenceCoefficients"]], 
  "InlineFormula"],
 "."
}], "Text",
 CellID->368099075]
}, Open  ]],

Cell[CellGroupData[{

Cell["Boundary conditions", "Subsection",
 CellID->178726099],

Cell[TextData[{
 "Boundary conditions are applied at a finite number of points referred to as \
collocation points.  For the models implemented in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ", collocation points are associated with the middle of each panel.  For \
example, PMVortexLinearNeumann locates collocation points just outside the \
airfoil and requires that the component of velocity normal to the panel be \
zero at these points (tangent-flow boundary condition), \
PMDoubletConstantDirichlet and PMDoubletQuadraticDirichlet locate collocation \
points just inside the airfoil and require that the velocity potential be \
zero at collocation points.  "
}], "Text",
 CellID->55517967]
}, Open  ]],

Cell[CellGroupData[{

Cell["Kutta condition", "Subsection",
 CellID->446287461],

Cell[TextData[{
 "The requirement that the flow leave the trailing edge smoothly is known as \
the Kutta condition, and it summarizes the gross effect of viscosity for \
these potential-flow models.  The Kutta condition implemented for \
PMVortexLinearNeumann requires that the circulation density be zero at the \
trailing edge.  Both doublet models implement a Kutta condition by adding a \
constant-strength doublet panel extending from the trailing edge of the \
airfoil to a point on the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis far downstream.  For PMDoubletConstantDirichlet, the circulation at \
the trailing edge is set to zero.  For PMDoubletQuadraticDirichlet, the \
circulation and circulation density at the trailing edge are set to zero."
}], "Text",
 CellID->107744017]
}, Open  ]],

Cell[CellGroupData[{

Cell["Smoothness conditions", "Subsection",
 CellID->252269107],

Cell["\<\
Models with singularities of linearly varying strength, such as \
PMVortexLinearNeumann, have additional equations requiring continuity of \
singularity strength from one panel to the next.  Models with singularities \
of quadratically varying strength, such as PMDoubletQuadraticDirichlet, \
require additional smoothness conditions on the slope of the singularity \
function from panel to panel.  These smoothness conditions are replaced by a \
Kutta condition at the trailing edge of the airfoil.\
\>", "Text",
 CellID->190335304]
}, Open  ]],

Cell[CellGroupData[{

Cell["System of equations", "Subsection",
 CellID->95542403],

Cell[TextData[{
 "All panel methods lead to a system of linear algebraic equations to be \
solved for unknown strength parameters.  Once the system of equations has \
been solved for the strength parameters, the flow field and aerodynamic \
properties can be computed.  Three fully implemented models are contained in \
the package ",
 Cell[BoxData[
  ButtonBox["PanelMethods",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/guide/PanelMethods"]], "InlineFormula"],
 "."
}], "Text",
 CellID->340388638]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.1.2  Example", "Section",
 CellID->275406678],

Cell[TextData[{
 "The following example illustrates the application of each of the three \
panel methods implemented in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " to a NACA-4412 airfoil at an angle of attack of 5 degrees.  Results are \
computed for lift coefficient, pitching-moment coefficient about the leading \
edge, and surface pressure coefficient."
}], "Text",
 CellID->137812698],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "Text",
 CellID->836781195],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->2058623809],

Cell["\<\
Specify the airfoil identification number, discretization parameters and \
angle of attack.\
\>", "Text",
 CellID->48545141],

Cell[BoxData[{
 RowBox[{
  RowBox[{"id", "=", "4412"}], ";", 
  RowBox[{"n", "=", "75"}], ";", 
  RowBox[{"layout", "=", "HalfCosineSpacing"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Alpha]", "=", 
   RowBox[{"5.", " ", "Degree"}]}], ";"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->43842758],

Cell[TextData[{
 "Compute the collection of vectors locating panel endpoints for the \
discretized airfoil.  The ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["NACA4DigitAirfoil",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/NACA4DigitAirfoil"]], "InlineFormula"],
 " computes thickness and camber properties at a list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values generated by ",
 Cell[BoxData[
  ButtonBox["NDiscretizeUnitSegment",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/NDiscretizeUnitSegment"]], 
  "InlineFormula"],
 ", and ",
 Cell[BoxData[
  ButtonBox["AirfoilSurfacePoints",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilSurfacePoints"]], 
  "InlineFormula"],
 " computes a collection of vectors on the surface of the airfoil specifying \
panel endpoints."
}], "Text",
 CellID->114570266],

Cell[BoxData[
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"NACA4DigitAirfoil", "[", 
     RowBox[{"id", ",", 
      RowBox[{"NDiscretizeUnitSegment", "[", 
       RowBox[{"n", ",", 
        RowBox[{"Layout", "\[Rule]", "layout"}]}], "]"}]}], "]"}], "]"}]}], 
  ";"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->112440851],

Cell[TextData[{
 "Note that the trailing edge of the airfoil is not closed, but that the mean \
of the two trailing-edge points is very close to the expected value of ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"x", ",", "z"}], ")"}], "=", 
   RowBox[{"(", 
    RowBox[{"1", ",", "0"}], ")"}]}]], "InlineFormula"],
 "."
}], "Text",
 CellID->253458896],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"ptTEL", ",", "ptTEU", ",", "ptTEM"}], "}"}], "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"First", "[", "rP", "]"}], ",", 
    RowBox[{"Last", "[", "rP", "]"}], ",", 
    RowBox[{"Mean", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"First", "[", "rP", "]"}], ",", 
       RowBox[{"Last", "[", "rP", "]"}]}], "}"}], "]"}]}], "}"}]}]], "Input",
 CellLabel->"In[4]:=",
 CellID->59487574],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0.9998334737126852`", ",", 
     RowBox[{"-", "0.0012489471548601018`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.0001665262873145`", ",", "0.0012489471548601265`"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"0.9999999999999998`", ",", "1.235990476633475`*^-17"}], "}"}]}], 
  "}"}]], "Output",
 ImageSize->{472, 18},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->708253272]
}, Open  ]],

Cell[TextData[{
 "For the doublet models with Dirichlet boundary conditions implemented in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ", the trailing edge needs to be closed to obtain plausible results."
}], "Text",
 CellID->486625314],

Cell[BoxData[
 RowBox[{
  RowBox[{"rPClosed", "=", 
   RowBox[{"ReplacePart", "[", 
    RowBox[{"rP", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"1", "\[Rule]", "ptTEM"}], ",", 
       RowBox[{
        RowBox[{"-", "1"}], "\[Rule]", "ptTEM"}]}], "}"}]}], "]"}]}], 
  ";"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->225120950],

Cell["\<\
Evaluate each model for the specified airfoil and angle of attack.  In \
addition to the appropriate strength parameters for each model, locations of \
collocation points and surface speeds just outside the airfoil associated \
with each collocation point are computed. \
\>", "Text",
 CellID->59805657],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"aV2", ",", "bV2", ",", "rCV2", ",", "VMagCV2"}], "}"}], "=", 
   RowBox[{"PMVortexLinearNeumann", "[", 
    RowBox[{"rP", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"aD1", ",", "aWD1", ",", "rCD1", ",", "VMagCD1"}], "}"}], "=", 
   RowBox[{"PMDoubletConstantDirichlet", "[", 
    RowBox[{"rPClosed", ",", "\[Alpha]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "aD3", ",", "bD3", ",", "cD3", ",", "aWD3", ",", "rCD3", ",", "VMagCD3"}],
     "}"}], "=", 
   RowBox[{"PMDoubletQuadraticDirichlet", "[", 
    RowBox[{"rPClosed", ",", "\[Alpha]"}], "]"}]}], ";"}]}], "Input",
 CellLabel->"In[6]:=",
 CellID->117496578],

Cell[TextData[{
 "Compute lift coefficient and pitching-moment coefficient about the leading \
edge, using the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["ForceAndMoment",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/ForceAndMoment"]], "InlineFormula"],
 ".  Lists of lift coefficient and pitching-moment coefficient for the three \
models are displayed to provide a comparison of results among the models."
}], "Text",
 CellID->474191],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"clV2", ",", "cmV2"}], "}"}], "=", 
   RowBox[{"ForceAndMoment", "[", 
    RowBox[{
    "\"\<Vortex\>\"", ",", "aV2", ",", "bV2", ",", "rP", ",", "\[Alpha]", ",",
      "rCV2"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"clD1", ",", "cmD1"}], "}"}], "=", 
   RowBox[{"ForceAndMoment", "[", 
    RowBox[{
    "\"\<Doublet\>\"", ",", "aD1", ",", "rP", ",", "\[Alpha]", ",", "rCD1"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"clD3", ",", "cmD3"}], "}"}], "=", 
   RowBox[{"ForceAndMoment", "[", 
    RowBox[{
    "\"\<Doublet\>\"", ",", "aD3", ",", "bD3", ",", "cD3", ",", "rP", ",", 
     "\[Alpha]", ",", "rCD3"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"clV2", ",", "clD1", ",", "clD3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"cmV2", ",", "cmD1", ",", "cmD3"}], "}"}]}], "}"}]}], "Input",
 CellLabel->"In[9]:=",
 CellID->499718542],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "1.1159651419350765`", ",", "1.1050041579972891`", ",", 
     "1.120852035562267`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.396151086108651`"}], ",", 
     RowBox[{"-", "0.3908225336843567`"}], ",", 
     RowBox[{"-", "0.40294081007120863`"}]}], "}"}]}], "}"}]], "Output",
 ImageSize->{426, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->329942508]
}, Open  ]],

Cell["\<\
Use Bernouilli's equation to compute surface pressure coefficients at \
collocation points.\
\>", "Text",
 CellID->21296710],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"cpV2", ",", "cpD1", ",", "cpD3"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"1", "-", 
      RowBox[{"VMagCV2", "^", "2"}]}], ",", 
     RowBox[{"1", "-", 
      RowBox[{"VMagCD1", "^", "2"}]}], ",", 
     RowBox[{"1", "-", 
      RowBox[{"VMagCD3", "^", "2"}]}]}], "}"}]}], ";"}]], "Input",
 CellLabel->"In[13]:=",
 CellID->36210806],

Cell["\<\
Plot the results of PMVortexLinearNeumann as a line, the results of \
PMDoubletConstantDirichlet as large filled circles colored red, and the \
results for PMDoubletQuadraticDirichlet as medium-sized filled circles \
colored blue.  The superposition of surface pressure coefficients for the \
three models provides a visual indication of agreement among them for this \
example.\
\>", "Text",
 CellID->281832136],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xCV2", ",", "xCD1", ",", "xCD3"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Components", "[", "rCV2", "]"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], ",", 
     RowBox[{
      RowBox[{"Components", "[", "rCD1", "]"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], ",", 
     RowBox[{
      RowBox[{"Components", "[", "rCD3", "]"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gCpV2", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xCV2", ",", "cpV2", ",", 
     RowBox[{"Epilog", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Inset", "[", 
        RowBox[{
         RowBox[{"PlotAirfoil", "[", 
          RowBox[{"rP", ",", 
           RowBox[{"Frame", "\[Rule]", "False"}], ",", 
           RowBox[{"PlotRange", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0", ",", "1"}], "}"}], ",", "Automatic"}], "}"}]}]}], 
          "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], ",", "1"}], "]"}], "}"}]}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gCpD1", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xCD1", ",", "cpD1", ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Red", ",", 
        RowBox[{"PointSize", "[", "Large", "]"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gCpD3", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xCD3", ",", "cpD3", ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Blue", ",", 
        RowBox[{"PointSize", "[", "Medium", "]"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gCpV2", ",", "gCpD1", ",", "gCpD3"}], "]"}]}], "Input",
 CellLabel->"In[14]:=",
 CellID->80931720],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3Q+YFPV9x/GJwnH+CYJEvKRpkprUKFbFhD7GS4z3hPJEpJGgImcoMYC9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    "], {{0, 330}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 330},
  PlotRange->{{0, 504}, {0, 330}}]], "Output",
 ImageSize->{508, 330},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[18]=",
 CellID->366976038]
}, Open  ]],

Cell["", "Text",
 CellID->152202905]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["PanelMethods",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/PanelMethods"]], "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["2.3 Isolated Panel",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IsolatedPanel"]], "RelatedTutorials",
 CellID->59868671],

Cell[TextData[ButtonBox["3.2 Validation Example",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ValidationExample"]], "RelatedTutorials",
 CellID->806871991],

Cell[TextData[ButtonBox["Overview",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]], "RelatedTutorials",
 CellID->280152894]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"3.1  Introduction to Panel Methods - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 4, 27.6480000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "None", "summary" -> 
    "Panel methods are numerical models for solving potential flow over an \
object, usually with relatively complex geometry such as an aircraft or its \
components. These methods are based on a boundary-integral solution to \
Laplace's equation. Source, vortex or doublet singularities are placed on the \
boundary of the object and suitable boundary conditions are applied. Refer to \
Low-Speed Aerodynamics by Katz and Plotkin (2001) for details. Currently, the \
panel methods in Aerodynamics are restricted to steady two-dimensional flow \
over a discretized airfoil specified by straight-line panels. Three panel \
methods are implemented by defined functions in Aerodynamics. Functions \
implementing two-dimensional panel methods.", "synonyms" -> {}, "title" -> 
    "3.1  Introduction to Panel Methods", "type" -> "Tutorial", "uri" -> 
    "Aerodynamics/tutorial/IntroductionToPanelMethods"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[627, 21, 78, 1, 70, "TutorialColorBar"],
Cell[708, 24, 191, 5, 70, "LinkTrail"],
Cell[902, 31, 3643, 82, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4570, 117, 71, 1, 70, "Title",
 CellID->509267359],
Cell[4644, 120, 592, 12, 70, "Text",
 CellID->1534169418],
Cell[5239, 134, 368, 11, 70, "Text",
 CellID->775308575],
Cell[5610, 147, 915, 25, 70, "DefinitionBox",
 CellID->2096742444],
Cell[6528, 174, 93, 1, 70, "Caption",
 CellID->1891092685],
Cell[CellGroupData[{
Cell[6646, 179, 69, 1, 70, "Section",
 CellID->432056375],
Cell[6718, 182, 225, 5, 70, "Text",
 CellID->250015064],
Cell[CellGroupData[{
Cell[6968, 191, 64, 1, 70, "Subsection",
 CellID->220682699],
Cell[7035, 194, 954, 19, 70, "Text",
 CellID->612564464]
}, Open  ]],
Cell[CellGroupData[{
Cell[8026, 218, 65, 1, 70, "Subsection",
 CellID->64226099],
Cell[8094, 221, 1040, 21, 70, "Text",
 CellID->368099075]
}, Open  ]],
Cell[CellGroupData[{
Cell[9171, 247, 61, 1, 70, "Subsection",
 CellID->178726099],
Cell[9235, 250, 705, 13, 70, "Text",
 CellID->55517967]
}, Open  ]],
Cell[CellGroupData[{
Cell[9977, 268, 57, 1, 70, "Subsection",
 CellID->446287461],
Cell[10037, 271, 811, 14, 70, "Text",
 CellID->107744017]
}, Open  ]],
Cell[CellGroupData[{
Cell[10885, 290, 63, 1, 70, "Subsection",
 CellID->252269107],
Cell[10951, 293, 543, 9, 70, "Text",
 CellID->190335304]
}, Open  ]],
Cell[CellGroupData[{
Cell[11531, 307, 60, 1, 70, "Subsection",
 CellID->95542403],
Cell[11594, 310, 513, 12, 70, "Text",
 CellID->340388638]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[12156, 328, 53, 1, 70, "Section",
 CellID->275406678],
Cell[12212, 331, 401, 9, 70, "Text",
 CellID->137812698],
Cell[12616, 342, 125, 6, 70, "Text",
 CellID->836781195],
Cell[12744, 350, 101, 3, 70, "Input",
 CellID->2058623809],
Cell[12848, 355, 134, 4, 70, "Text",
 CellID->48545141],
Cell[12985, 361, 308, 9, 70, "Input",
 CellID->43842758],
Cell[13296, 372, 935, 28, 70, "Text",
 CellID->114570266],
Cell[14234, 402, 361, 11, 70, "Input",
 CellID->112440851],
Cell[14598, 415, 365, 11, 70, "Text",
 CellID->253458896],
Cell[CellGroupData[{
Cell[14988, 430, 447, 14, 70, "Input",
 CellID->59487574],
Cell[15438, 446, 507, 16, 39, "Output",
 CellID->708253272]
}, Open  ]],
Cell[15960, 465, 246, 6, 70, "Text",
 CellID->486625314],
Cell[16209, 473, 338, 12, 70, "Input",
 CellID->225120950],
Cell[16550, 487, 313, 6, 70, "Text",
 CellID->59805657],
Cell[16866, 495, 802, 23, 70, "Input",
 CellID->117496578],
Cell[17671, 520, 505, 13, 70, "Text",
 CellID->474191],
Cell[CellGroupData[{
Cell[18201, 537, 1061, 32, 70, "Input",
 CellID->499718542],
Cell[19265, 571, 504, 16, 36, "Output",
 CellID->329942508]
}, Open  ]],
Cell[19784, 590, 134, 4, 70, "Text",
 CellID->21296710],
Cell[19921, 596, 414, 14, 70, "Input",
 CellID->36210806],
Cell[20338, 612, 422, 8, 70, "Text",
 CellID->281832136],
Cell[CellGroupData[{
Cell[20785, 624, 2116, 60, 191, "Input",
 CellID->80931720],
Cell[22904, 686, 12619, 214, 351, "Output",
 CellID->366976038]
}, Open  ]],
Cell[35538, 903, 36, 1, 70, "Text",
 CellID->152202905]
}, Open  ]],
Cell[CellGroupData[{
Cell[35611, 909, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[35679, 912, 158, 3, 70, "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[CellGroupData[{
Cell[35874, 920, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[35949, 923, 168, 4, 70, "RelatedTutorials",
 CellID->59868671],
Cell[36120, 929, 177, 4, 70, "RelatedTutorials",
 CellID->806871991],
Cell[36300, 935, 166, 4, 70, "RelatedTutorials",
 CellID->280152894]
}, Open  ]]
}, Open  ]],
Cell[36493, 943, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

