(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     36657,       1030]
NotebookOptionsPosition[     31282,        867]
NotebookOutlinePosition[     33765,        924]
CellTagsIndexPosition[     33722,        921]
WindowTitle->1.5 Panel Geometry - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"1.4 Airfoils From Conformal Mapping\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AirfoilsFromConformalMapping"], "\<\"2.1 \
Introduction To Influence Coefficients\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"], "\
\<\"Overview\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Airfoil Geometry\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/AirfoilGeometry"], "\<\"Aerodynamics\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/Aerodynamics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Functions \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PanelAngles\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PanelAngles"], "\<\"PanelLengths\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PanelLengths"], "\<\"PanelNormals\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PanelNormals"], "\<\"PanelPoints\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PanelPoints"], "\<\"ThickAirfoilQ\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ThickAirfoilQ"], "\<\"TrailingEdgeAngle\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/ref/TrailingEdgeAngle"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/tutorial/PanelGeometry\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/PanelGeometry"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/PanelGeometry.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/PanelGeometry", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["1.5 Panel Geometry", "Title",
 CellID->509267359],

Cell["\<\
Different sources of information about airfoil geometry require different \
procedures for determining a collection of vectors locating points on the \
surface of the airfoil that serve as panel endpoints.  Once the panel \
endpoints are established, however, computing geometric properties of the \
discretized airfoil is based only on this collection of vectors.   The \
purpose of most of these functions will be clear from the short descriptions \
given below.  The function PanelPoints is used to locate collocation points \
on the panels where boundary conditions are applied for numerical models.\
\>", "Text",
 CellID->1534169418],

Cell[BoxData[GridBox[{
   {
    ButtonBox["PanelAngles",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/PanelAngles"], Cell[TextData[{
     "angles that panels make with ",
     Cell[BoxData[
      FormBox["x", TraditionalForm]], "InlineMath"],
     "-axis"
    }], "TableText"]},
   {
    ButtonBox["PanelLengths",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/PanelLengths"], Cell[
    "list of panel lengths", "TableText"]},
   {
    ButtonBox["PanelNormals",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/PanelNormals"], Cell[
    "outward unit normals to panels", "TableText"]},
   {
    ButtonBox["PanelPoints",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/PanelPoints"], Cell[
    "points on panels", "TableText"]},
   {
    ButtonBox["TrailingEdgeAngle",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/TrailingEdgeAngle"], Cell[
    "included angle between trailing edge panels", "TableText"]},
   {
    ButtonBox["ThickAirfoilQ",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/ThickAirfoilQ"], Cell[
    "does the airfoil have thickness", "TableText"]}
  }]], "DefinitionBox",
 CellID->306953523],

Cell["Functions to compute panel geometry from panel endpoints.", "Caption",
 CellID->711473682],

Cell["\<\
The family of NACA four-digit airfoils is chosen as the example for this \
section because airfoils with and without thickness can be generated simply \
by changing the identification number.\
\>", "Text",
 CellID->157402686],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "Text",
 CellID->38213042],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->318373978],

Cell[CellGroupData[{

Cell["1.5.1 Airfoil With Thickness", "Section",
 CellID->282162397],

Cell["\<\
Once panel endpoints specifying a discretized airfoil have been established, \
additional geometric quantities for the discretized airfoil can be computed.  \
Specify the identification number for an airfoil and discretization \
parameters.  Compute the collection of vectors locating panel endpoints, and \
prepare a graphic showing the discretized airfoil.\
\>", "Text",
 CellID->311381061],

Cell[BoxData[{
 RowBox[{
  RowBox[{"id", "=", "4415"}], ";", 
  RowBox[{"n", "=", "5"}], ";", 
  RowBox[{"layout", "=", "HalfCosineSpacing"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"NACA4DigitAirfoil", "[", 
     RowBox[{"id", ",", 
      RowBox[{"NDiscretizeUnitSegment", "[", 
       RowBox[{"n", ",", 
        RowBox[{"Layout", "\[Rule]", "layout"}]}], "]"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gPanels", "=", 
   RowBox[{"PlotAirfoil", "[", "rP", "]"}]}], ";"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->32270844],

Cell[TextData[{
 "Compute outward unit normals to panels and show them on a plot of the \
discretized airfoil, with tails at mid-panel, which is the default value for \
the option PanelOffset in the function PanelPoints.  Label the normal vectors \
with panel number",
 StyleBox[".",
  FontSlant->"Italic"]
}], "Text",
 CellID->769934012],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"uN", "=", 
   RowBox[{"PanelNormals", "[", "rP", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rC", "=", 
   RowBox[{"PanelPoints", "[", "rP", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gNormals", "=", 
   RowBox[{"Graphics", "[", 
    RowBox[{"Arrow", "[", 
     RowBox[{"rC", ",", 
      RowBox[{"rC", "+", 
       RowBox[{"0.15", "uN"}]}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gPanels", ",", "gNormals", ",", 
   RowBox[{"Epilog", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Text", "[", 
        RowBox[{"i", ",", 
         RowBox[{
          RowBox[{"rC", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "+", 
          RowBox[{"0.17", " ", 
           RowBox[{"uN", "[", 
            RowBox[{"[", "i", "]"}], "]"}]}]}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{"NumberOfVectors", "[", "rC", "]"}]}], "}"}]}], "]"}], 
     "}"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "0.25"}], ",", "1.2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "0.25"}], ",", "0.3"}], "}"}]}], "}"}]}]}], 
  "]"}]}], "Input",
 CellLabel->"In[4]:=",
 CellID->66680097],

Cell[BoxData[
 GraphicsBox[{
   {Hue[0.67, 0.6, 0.6], 
    LineBox[{{0.9997918421408566, -0.0015611839435751304`}, {
     0.6879573940418148, -0.016198441614263596`}, {
     0.41201947304449976`, -0.03195715360845929}, {
     0.19836607364310915`, -0.04156765018480128}, {
     0.05655570289022391, -0.03417765488167223}, {0., 0.}, {
     0.04133126451946903, 0.052557316065027525`}, {0.18359993760699594`, 
     0.09972359821603287}, {0.4124100223705542, 0.11192399803922404`}, {
     0.6940086172082903, 0.07738263948924372}, {1.0002081578591433`, 
     0.0015611839435751551`}}]}, {
    ArrowBox[{{0.8438746180913357, -0.008879812778919364}, {
     0.850907753450712, -0.1587148387475382}}], 
    ArrowBox[{{0.5499884335431573, -0.024077797611361446`}, {
     0.558540942038854, -0.17383378112598113`}}], 
    ArrowBox[{{0.30519277334380446`, -0.036762401896630284`}, {
     0.31193321506669125`, -0.1866108801851346}}], 
    ArrowBox[{{0.12746088826666652`, -0.03787265253323675}, {
     0.11965470875219478`, -0.18766939335688698`}}], 
    ArrowBox[{{
     0.028277851445111954`, -0.017088827440836115`}, {-0.04930376944253391, \
-0.14546753323134326`}}], 
    ArrowBox[{{0.020665632259734515`, 
     0.026278658032513762`}, {-0.09724262155768046, 0.11900213770043848`}}], 
    ArrowBox[{{0.11246560106323249`, 0.0761404571405302}, {
     0.06526262638099037, 0.21851980670059667`}}], 
    ArrowBox[{{0.29800497998877507`, 0.10582379812762846`}, {
     0.2900181643989505, 0.25561101646328666`}}], 
    ArrowBox[{{0.5532093197894222, 0.09465331876423388}, {0.5714716988182416, 
     0.24353745328570578`}}], 
    ArrowBox[{{0.8471083875337169, 0.03947191171640944}, {0.8831626359547899, 
     0.18507442258346674`}}]}},
  Epilog->{{
     InsetBox[
      FormBox["1", TraditionalForm], {
      0.8518455048319622, -0.1786928422100207}], 
     InsetBox[
      FormBox["2", TraditionalForm], {
      0.5596812765049468, -0.1938012455945971}], 
     InsetBox[
      FormBox["3", TraditionalForm], {
      0.31283194062974284`, -0.2065906772902686}], 
     InsetBox[
      FormBox["4", TraditionalForm], {
      0.11861388481693187`, -0.2076422921333737}], 
     InsetBox[
      FormBox[
      "5", TraditionalForm], {-0.0596479855608867, -0.16258469400341088`}], 
     InsetBox[
      FormBox["6", TraditionalForm], {-0.11296372206666914`, 
      0.13136526832282844`}], 
     InsetBox[
      FormBox["7", TraditionalForm], {0.05896889642335807, 
      0.23750371997527223`}], 
     InsetBox[
      FormBox["8", TraditionalForm], {0.2889532556536406, 
      0.27558264557470774`}], 
     InsetBox[
      FormBox["9", TraditionalForm], {0.5739066826887508, 0.263388671221902}], 
     InsetBox[
      FormBox["10", TraditionalForm], {0.8879698690775997, 
      0.20448809069907442`}]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2000000000000004`, 
      FormBox["1.2000000000000004`", TraditionalForm]}, {1.4000000000000001`, 
      FormBox["1.4000000000000001`", TraditionalForm]}, {1.6, 
      FormBox["1.6`", TraditionalForm]}}, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}, None, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}},
  ImageSize->{504., 180.},
  PlotRange->{{-0.25, 1.2}, {-0.25, 0.3}},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->74458849]
}, Open  ]],

Cell[TextData[{
 "Compute the list of angles that the panels make with the ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]], "InlineMath"],
 "-axis."
}], "Text",
 CellID->182365288],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PanelAngles", "[", "rP", "]"}]], "Input",
 CellLabel->"In[8]:=",
 CellID->594385488],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "3.0946878875587047`"}], ",", 
   RowBox[{"-", "3.084544992329381`"}], ",", 
   RowBox[{"-", "3.096641238605829`"}], ",", "3.089527937743146`", ",", 
   "2.598003868947146`", ",", "0.9044010084478203`", ",", 
   "0.3201263540894222`", ",", "0.05327062856620143`", ",", 
   RowBox[{"-", "0.12205199635460329`"}], ",", 
   RowBox[{"-", "0.24273841274686234`"}]}], "}"}]], "Output",
 ImageSize->{366, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->56687131]
}, Open  ]],

Cell["Compute the list of panel lengths.", "Text",
 CellID->61777939],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PanelLengths", "[", "rP", "]"}]], "Input",
 CellLabel->"In[9]:=",
 CellID->497996672],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.31217778962211923`", ",", "0.2763875417742928`", ",", 
   "0.21386943849043513`", ",", "0.1420027932248202`", ",", 
   "0.06608070537318722`", ",", "0.06686213351918648`", ",", 
   "0.14988339972803474`", ",", "0.22913512311593007`", ",", 
   "0.2837091363792604`", ",", "0.31544738359329777`"}], "}"}]], "Output",
 ImageSize->{424, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->228686829]
}, Open  ]],

Cell["Compute the trailing-edge angle in degrees.", "Text",
 CellID->368516261],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"TrailingEdgeAngle", "[", "rP", "]"}], "/", "Degree"}]], "Input",
 CellLabel->"In[10]:=",
 CellID->14075685],

Cell[BoxData["16.595331708729734`"], "Output",
 ImageSize->{53, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[10]=",
 CellID->284117795]
}, Open  ]],

Cell["\<\
It can be useful to have a test for whether an airfoil has thickness.\
\>", "Text",
 CellID->814060285],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ThickAirfoilQ", "[", "rP", "]"}]], "Input",
 CellLabel->"In[11]:=",
 CellID->257551952],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->418031504]
}, Open  ]],

Cell["\<\
Note that the number of panels for an airfoil with thickness is twice the \
discretization number.\
\>", "Text",
 CellID->212162634]
}, Open  ]],

Cell[CellGroupData[{

Cell["1.5.2 Airfoil Without Thickness (Camber Line)", "Section",
 CellID->848281288],

Cell[TextData[{
 "Sometimes, it is useful to work with the camber line, which can be \
considered a reference airfoil without thickness.  Such an airfoil can be \
constructed by setting the last two digits of the identification number for a \
NACA four-digit airfoil to zero.  The fore-to-aft numbering convention is \
used in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " for the special case of reference airfoils without thickness.  The graphic \
below shows unit normals to panels and labels the vectors with panel numbers \
to illustrate this.  Note that for airfoils without thickness, the number of \
panels is equal to the discretization number."
}], "Text",
 CellID->148980166],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rPCamberLine", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"NACA4DigitAirfoil", "[", 
     RowBox[{"4400", ",", 
      RowBox[{"NDiscretizeUnitSegment", "[", 
       RowBox[{"n", ",", 
        RowBox[{"Layout", "\[Rule]", "layout"}]}], "]"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gPanelsCamberLine", "=", 
   RowBox[{"PlotAirfoil", "[", "rPCamberLine", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"uNCamberLine", "=", 
   RowBox[{"PanelNormals", "[", "rPCamberLine", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rCCamberLine", "=", 
   RowBox[{"PanelPoints", "[", "rPCamberLine", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gUNCamberLine", "=", 
   RowBox[{"Graphics", "[", 
    RowBox[{"Arrow", "[", 
     RowBox[{"rCCamberLine", ",", 
      RowBox[{"rCCamberLine", "+", 
       RowBox[{"0.15", "uNCamberLine"}]}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gPanelsCamberLine", ",", "gUNCamberLine", ",", 
   RowBox[{"Epilog", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Text", "[", 
        RowBox[{"i", ",", 
         RowBox[{
          RowBox[{"rCCamberLine", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "+", 
          RowBox[{"0.17", " ", 
           RowBox[{"uNCamberLine", "[", 
            RowBox[{"[", "i", "]"}], "]"}]}]}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{"NumberOfVectors", "[", "rCCamberLine", "]"}]}], "}"}]}], 
      "]"}], "}"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "0.25"}], ",", "1.2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "0.25"}], ",", "0.3"}], "}"}]}], "}"}]}]}], 
  "]"}]}], "Input",
 CellLabel->"In[12]:=",
 CellID->10976936],

Cell[BoxData[
 GraphicsBox[{
   {Hue[0.67, 0.6, 0.6], 
    LineBox[{{0., 0.}, {0.04894348370484647, 0.009189830591677649}, {
     0.19098300562505255`, 0.029077974015615796`}, {0.412214747707527, 
     0.039983422215382376`}, {0.6909830056250525, 0.03059209893749006}, {
     0.9999999999999999, 1.2335811384723961`*^-17}}]}, {
    ArrowBox[{{0.024471741852423234`, 
     0.0045949152958388246`}, {-0.003209153955846932, 0.15201868545266153`}}],
     ArrowBox[{{0.11996324466494951`, 0.01913390230364672}, {
     0.0991633904491181, 0.167684789425199}}], 
    ArrowBox[{{0.30159887666628976`, 0.03453069811549909}, {0.294213709529165,
      0.18434878549601777`}}], 
    ArrowBox[{{0.5515988766662898, 0.03528776057643622}, {0.5566493074392626, 
     0.18520271362985052`}}], 
    ArrowBox[{{0.8454915028125263, 0.015296049468745037`}, {
     0.8602689818614628, 0.1645663616977956}}]}},
  Epilog->{{
     InsetBox[
      FormBox["1", TraditionalForm], {-0.006899940063616286, 
      0.1716751881402379}], 
     InsetBox[
      FormBox["2", TraditionalForm], {0.0963900765536739, 
      0.1874915743747393}], 
     InsetBox[
      FormBox["3", TraditionalForm], {0.29322902057754835`, 
      0.20432453048008695`}], 
     InsetBox[
      FormBox["4", TraditionalForm], {0.5573226982089923, 
      0.20519137403697246`}], 
     InsetBox[
      FormBox["5", TraditionalForm], {0.8622393124013211, 
      0.18446906999500234`}]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2000000000000004`, 
      FormBox["1.2000000000000004`", TraditionalForm]}, {1.4000000000000001`, 
      FormBox["1.4000000000000001`", TraditionalForm]}, {1.6, 
      FormBox["1.6`", TraditionalForm]}}, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}, None, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}},
  ImageSize->{504., 180.},
  ImageSizeRaw->{504, 180},
  PlotRange->{{-0.25, 1.2}, {-0.25, 0.3}},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[17]=",
 CellID->139857911]
}, Open  ]],

Cell["Test the airfoil for thickness.", "Text",
 CellID->112816300],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ThickAirfoilQ", "[", "rPCamberLine", "]"}]], "Input",
 CellLabel->"In[18]:=",
 CellID->61466950],

Cell[BoxData["False"], "Output",
 ImageSize->{39, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[18]=",
 CellID->603880931]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["Airfoil Geometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/guide/AirfoilGeometry"]], "TutorialMoreAbout",
 CellID->8867598],

Cell[TextData[ButtonBox["Aerodynamics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"]], "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["1.4 Airfoils From Conformal Mapping",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromConformalMapping"]], \
"RelatedTutorials",
 CellID->58607457],

Cell[TextData[ButtonBox["2.1 Introduction To Influence Coefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"]], \
"RelatedTutorials",
 CellID->76715951],

Cell[TextData[ButtonBox["Overview",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]], "RelatedTutorials",
 CellID->172539578]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"1.5 Panel Geometry - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 4, 40.2372000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "None", "summary" -> 
    "Different sources of information about airfoil geometry require \
different procedures for determining a collection of vectors locating points \
on the surface of the airfoil that serve as panel endpoints. Once the panel \
endpoints are established, however, computing geometric properties of the \
discretized airfoil is based only on this collection of vectors. The purpose \
of most of these functions will be clear from the short descriptions given \
below. The function PanelPoints is used to locate collocation points on the \
panels where boundary conditions are applied for numerical models. Functions \
to compute panel geometry from panel endpoints. The family of NACA four-digit \
airfoils is chosen as the example for this section because airfoils with and \
without thickness can be generated simply by changing the identification \
number.", "synonyms" -> {}, "title" -> "1.5 Panel Geometry", "type" -> 
    "Tutorial", "uri" -> "Aerodynamics/tutorial/PanelGeometry"}, "LinkTrails" -> 
  "", "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[611, 21, 78, 1, 70, "TutorialColorBar"],
Cell[692, 24, 191, 5, 70, "LinkTrail"],
Cell[886, 31, 3989, 86, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4900, 121, 55, 1, 70, "Title",
 CellID->509267359],
Cell[4958, 124, 648, 10, 70, "Text",
 CellID->1534169418],
Cell[5609, 136, 1223, 36, 70, "DefinitionBox",
 CellID->306953523],
Cell[6835, 174, 96, 1, 70, "Caption",
 CellID->711473682],
Cell[6934, 177, 235, 5, 70, "Text",
 CellID->157402686],
Cell[7172, 184, 124, 6, 70, "Text",
 CellID->38213042],
Cell[7299, 192, 100, 3, 70, "Input",
 CellID->318373978],
Cell[CellGroupData[{
Cell[7424, 199, 67, 1, 70, "Section",
 CellID->282162397],
Cell[7494, 202, 402, 7, 70, "Text",
 CellID->311381061],
Cell[7899, 211, 635, 18, 70, "Input",
 CellID->32270844],
Cell[8537, 231, 338, 8, 70, "Text",
 CellID->769934012],
Cell[CellGroupData[{
Cell[8900, 243, 1370, 44, 70, "Input",
 CellID->66680097],
Cell[10273, 289, 7647, 175, 201, "Output",
 CellID->74458849]
}, Open  ]],
Cell[17935, 467, 183, 6, 70, "Text",
 CellID->182365288],
Cell[CellGroupData[{
Cell[18143, 477, 108, 3, 70, "Input",
 CellID->594385488],
Cell[18254, 482, 565, 14, 54, "Output",
 CellID->56687131]
}, Open  ]],
Cell[18834, 499, 69, 1, 70, "Text",
 CellID->61777939],
Cell[CellGroupData[{
Cell[18928, 504, 109, 3, 70, "Input",
 CellID->497996672],
Cell[19040, 509, 490, 12, 54, "Output",
 CellID->228686829]
}, Open  ]],
Cell[19545, 524, 79, 1, 70, "Text",
 CellID->368516261],
Cell[CellGroupData[{
Cell[19649, 529, 142, 4, 70, "Input",
 CellID->14075685],
Cell[19794, 535, 177, 5, 36, "Output",
 CellID->284117795]
}, Open  ]],
Cell[19986, 543, 113, 3, 70, "Text",
 CellID->814060285],
Cell[CellGroupData[{
Cell[20124, 550, 111, 3, 70, "Input",
 CellID->257551952],
Cell[20238, 555, 162, 5, 36, "Output",
 CellID->418031504]
}, Open  ]],
Cell[20415, 563, 142, 4, 70, "Text",
 CellID->212162634]
}, Open  ]],
Cell[CellGroupData[{
Cell[20594, 572, 84, 1, 70, "Section",
 CellID->848281288],
Cell[20681, 575, 696, 13, 70, "Text",
 CellID->148980166],
Cell[CellGroupData[{
Cell[21402, 592, 1966, 59, 70, "Input",
 CellID->10976936],
Cell[23371, 653, 6319, 146, 201, "Output",
 CellID->139857911]
}, Open  ]],
Cell[29705, 802, 67, 1, 70, "Text",
 CellID->112816300],
Cell[CellGroupData[{
Cell[29797, 807, 120, 3, 70, "Input",
 CellID->61466950],
Cell[29920, 812, 163, 5, 36, "Output",
 CellID->603880931]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[30132, 823, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[30200, 826, 165, 4, 70, "TutorialMoreAbout",
 CellID->8867598],
Cell[30368, 832, 158, 3, 70, "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[CellGroupData[{
Cell[30563, 840, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[30638, 843, 202, 5, 70, "RelatedTutorials",
 CellID->58607457],
Cell[30843, 850, 216, 5, 70, "RelatedTutorials",
 CellID->76715951],
Cell[31062, 857, 166, 4, 70, "RelatedTutorials",
 CellID->172539578]
}, Open  ]]
}, Open  ]],
Cell[31255, 865, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

