(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     15327,        410]
NotebookOptionsPosition[     10962,        301]
NotebookOutlinePosition[     14021,        366]
CellTagsIndexPosition[     13978,        363]
WindowTitle->Preface - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Overview\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/Aerodynamics"], "\<\"CartesianVectors\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/CartesianVectors"], \
"\<\"AirfoilGeometry\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/AirfoilGeometry"], \
"\<\"InfluenceCoefficients\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/InfluenceCoefficients"], \
"\<\"PanelMethods\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/PanelMethods"], "\<\"Graphics\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/Graphics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/tutorial/PrefaceToAerodynamics\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/PrefaceToAerodynamics"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/PrefaceToAerodynamics.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/PrefaceToAerodynamics", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["Preface", "Title",
 CellID->509267359],

Cell[TextData[{
 "When I taught undergraduate aerodynamics at the University of Florida, I \
often wished for a collection of relatively easy-to-use tools for computing \
more complex flows than those illustrated in textbooks.  ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " 1 is a start of such a collection.  The current version is limited to \
steady incompressible two-dimensional potential flow over an airfoil."
}], "Text",
 CellID->34163350],

Cell[TextData[{
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " contains a collection of functions for constructing panel methods \
including three fully implemented models.  The easy-to-use aspect is \
addressed by providing over 40 functions to accomplish well-defined tasks, \
and for displaying final results using conventions of the aerodynamics \
community.  These tasks range from importing data files for airfoil geometry \
to plotting model results for surface pressure coefficient.  Each function is \
fully documented and illustrated with examples."
}], "Text",
 CellID->233074666],

Cell[TextData[{
 "The needs of either a survey or an in-depth study of two-dimensional \
potential flow over an airfoil can be met by ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  The three panel methods implemented and their documentation can be used \
as a brief exposure to potential-flow aerodynamics.  To accommodate an \
in-depth study, the content of all packages is available to users for \
critique and modification.  Functions are defined for commonly used \
two-dimensional influence coefficients and for presentation graphics, so  \
emphasis can be placed on model construction rather than these details."
}], "Text",
 CellID->154329092],

Cell["\<\
The collection of tutorials is organized in the form of an interactive book.  \
Examples are usually presented with input specification grouped together to \
enable you to change input values easily and rerun the example for other \
cases.  A table of contents is given on the Overview page listed below under \
the heading Related Tutorials.\
\>", "Text",
 CellID->140730767],

Cell[TextData[{
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " consists of five packages.  The guide page for ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " provides links to guide pages for each package.  All guide pages are \
listed below under the heading More About and on the Overview page.  \
Additionally, function names are indexed in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Help, providing quick access to symbol documentation when working on a \
problem."
}], "Text",
 CellID->136038040],

Cell[TextData[{
 "A comment about notation may be useful.  Conventional aerodynamics notation \
is used in the text and conventional ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation is used in input cells.  Hopefully, the connection between a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " symbol and the corresponding symbol in aerodynamics notation is \
recognizable.  The only exception to this is that the symbol chosen to \
represent velocity in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " input statements is capital V instead of a name starting with a lower-case \
letter."
}], "Text",
 CellID->464972710],

Cell[TextData[{
 "Dimensionless variables are used throughout the packages in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  The characteristic length is the chord of the airfoil, and the \
characteristic speed is that of the uniform onset flow.  For example, \
circulation has units of length squared divided by time.  Dividing \
dimensional circulation by the product of the speed of the onset flow and the \
chord of the airfoil gives the dimensionless measure of circulation.  This \
leads to the result that the airfoil lift coefficient is just twice the \
dimensionless circulation."
}], "Text",
 CellID->94038502],

Cell[TextData[{
 "Aerodynamics 1.2 updates version 1.1 to ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " 8.0.  The function AirfoilFromDataFile has been modified to detect the \
numbering convention used in the data file and to modify some observed \
anomalies to conform to the conventions used in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  The option NumberingConvention for AirfoilFromDataFile is obsolete.  \
Names for functions computing influence coefficients for velocity potential \
and stream function have been altered to conform to ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " conventions; Phi and Psi are used instead of \[Phi] and \[Psi] in the \
function names.  The final section of the tutorial in Aerodynamics 1.1 has \
been replaced with a worksheet that enables the user to select an airfoil, \
angle of attack, discretization parameters, model and desired flow \
visualization to generate a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notebook showing results.  Other minor changes in the category of \
housekeeping have been made, but technical content is essentially the same as \
",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " 1.1."
}], "Text",
 CellID->870901264],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[Cell[BoxData[
 ButtonBox["Aerodynamics",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Aerodynamics/guide/Aerodynamics"]], "InlineFormula"]], \
"TutorialMoreAbout",
 CellID->1567025153],

Cell[TextData[Cell[BoxData[
 ButtonBox["CartesianVectors",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Aerodynamics/guide/CartesianVectors"]], "InlineFormula"]], \
"TutorialMoreAbout",
 CellID->285068269],

Cell[TextData[Cell[BoxData[
 ButtonBox["AirfoilGeometry",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Aerodynamics/guide/AirfoilGeometry"]], "InlineFormula"]], \
"TutorialMoreAbout",
 CellID->403490632],

Cell[TextData[Cell[BoxData[
 ButtonBox["InfluenceCoefficients",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Aerodynamics/guide/InfluenceCoefficients"]], "InlineFormula"]], \
"TutorialMoreAbout",
 CellID->929488745],

Cell[TextData[Cell[BoxData[
 ButtonBox["PanelMethods",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Aerodynamics/guide/PanelMethods"]], "InlineFormula"]], \
"TutorialMoreAbout",
 CellID->4762702],

Cell[TextData[ButtonBox["Graphics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Graphics"]], "TutorialMoreAbout",
 CellID->47881637]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["Overview",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]], "RelatedTutorials",
 CellID->806871991]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Preface - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 4, 47.4288000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "None", "summary" -> 
    "When I taught undergraduate aerodynamics at the University of Florida, I \
often wished for a collection of relatively easy-to-use tools for computing \
more complex flows than those illustrated in textbooks. Aerodynamics 1 is a \
start of such a collection. The current version is limited to steady \
incompressible two-dimensional potential flow over an airfoil. Aerodynamics \
contains a collection of functions for constructing panel methods including \
three fully implemented models. The easy-to-use aspect is addressed by \
providing over 40 functions to accomplish well-defined tasks, and for \
displaying final results using conventions of the aerodynamics community. \
These tasks range from importing data files for airfoil geometry to plotting \
model results for surface pressure coefficient. Each function is fully \
documented and illustrated with examples. The needs of either a survey or an \
in-depth study of two-dimensional potential flow over an airfoil can be met \
by Aerodynamics. The three panel methods implemented and their documentation \
can be used as a brief exposure to potential-flow aerodynamics. To \
accommodate an in-depth study, the content of all packages is available to \
users for critique and modification. Functions are defined for commonly used \
two-dimensional influence coefficients and for presentation graphics, so \
emphasis can be placed on model construction rather than these details.", 
    "synonyms" -> {}, "title" -> "Preface", "type" -> "Tutorial", "uri" -> 
    "Aerodynamics/tutorial/PrefaceToAerodynamics"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[600, 21, 78, 1, 70, "TutorialColorBar"],
Cell[681, 24, 191, 5, 70, "LinkTrail"],
Cell[875, 31, 3235, 72, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4135, 107, 44, 1, 70, "Title",
 CellID->509267359],
Cell[4182, 110, 457, 9, 70, "Text",
 CellID->34163350],
Cell[4642, 121, 597, 11, 70, "Text",
 CellID->233074666],
Cell[5242, 134, 661, 12, 70, "Text",
 CellID->154329092],
Cell[5906, 148, 386, 7, 70, "Text",
 CellID->140730767],
Cell[6295, 157, 527, 14, 70, "Text",
 CellID->136038040],
Cell[6825, 173, 646, 16, 70, "Text",
 CellID->464972710],
Cell[7474, 191, 631, 12, 70, "Text",
 CellID->94038502],
Cell[8108, 205, 1240, 28, 70, "Text",
 CellID->870901264],
Cell[CellGroupData[{
Cell[9373, 237, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[9441, 240, 200, 6, 70, "TutorialMoreAbout",
 CellID->1567025153],
Cell[9644, 248, 207, 6, 70, "TutorialMoreAbout",
 CellID->285068269],
Cell[9854, 256, 205, 6, 70, "TutorialMoreAbout",
 CellID->403490632],
Cell[10062, 264, 217, 6, 70, "TutorialMoreAbout",
 CellID->929488745],
Cell[10282, 272, 197, 6, 70, "TutorialMoreAbout",
 CellID->4762702],
Cell[10482, 280, 148, 3, 70, "TutorialMoreAbout",
 CellID->47881637]
}, Open  ]],
Cell[CellGroupData[{
Cell[10667, 288, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[10742, 291, 166, 4, 70, "RelatedTutorials",
 CellID->806871991]
}, Open  ]]
}, Open  ]],
Cell[10935, 299, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

