(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     60005,       1370]
NotebookOptionsPosition[     54375,       1196]
NotebookOutlinePosition[     56768,       1252]
CellTagsIndexPosition[     56725,       1249]
WindowTitle->3.2  Validation Example - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"3.1 Introduction to Panel Methods\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods"], "\<\"3.3 \
Implementation\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/Implementation"], "\<\"Overview\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PanelMethods\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/PanelMethods"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/tutorial/ValidationExample\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/ValidationExample"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/ValidationExample.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/ValidationExample", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["3.2  Validation Example", "Title",
 CellID->509267359],

Cell[TextData[{
 "An exact solution for the van de Vooren airfoil is used to validate the \
three panel methods implemented in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  The family of van de Vooren airfoil airfoils are symmetric airfoils \
obtained by mapping a circle to an airfoil shape using the van de Vooren \
transformation.  Surface pressure coefficient is chosen as the quantity to \
compare between the exact solution and numerical panel-method solutions."
}], "Text",
 CellID->25608729],

Cell[TextData[{
 "Refer to ",
 ButtonBox["Katz and Plotkin (2001)",
  BaseStyle->"Hyperlink",
  ButtonData->{"References.nb", "katz2001"}],
 " for a discussion of the van de Vooren airfoil and the exact potential-flow \
solution for this airfoil.  The functions in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " for the airfoil geometry and the exact solution are from this reference as \
I have been unable to locate a copy of the original paper."
}], "Text",
 CellID->124571872],

Cell["Load the packages.", "Text",
 CellID->836781195],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->2058623809],

Cell[CellGroupData[{

Cell["3.2.1  van de Vooren Airfoil", "Section",
 CellID->631983245],

Cell["\<\
Specify parameters for a 12% maximum thickness van de Vooren airfoil, and an \
angle of attack of 6 degrees.\
\>", "Text",
 CellID->405071093],

Cell[BoxData[
 RowBox[{
  RowBox[{"t", "=", "0.12"}], ";", " ", 
  RowBox[{"\[Tau]", "=", 
   RowBox[{"9.81", " ", "Degree"}]}], ";", 
  RowBox[{"\[Alpha]", "=", 
   RowBox[{"6.", " ", "Degree"}]}], ";"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->692274660],

Cell["Specify parameters for discretizing the airfoil. ", "Text",
 CellID->14124012],

Cell[BoxData[
 RowBox[{
  RowBox[{"n", "=", "50"}], ";", 
  RowBox[{"layout", "=", "CosineSpacing"}], ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->361350040],

Cell[TextData[{
 "Compute panel endpoints for the discretized airfoil using ",
 Cell[BoxData[
  ButtonBox["AirfoilSurfacePoints",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilSurfacePoints"]], 
  "InlineFormula"],
 ".  The discretization is specified by a list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values computed by ",
 Cell[BoxData[
  ButtonBox["NDiscretizeUnitSegment",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/NDiscretizeUnitSegment"]], 
  "InlineFormula"],
 " and converted to ",
 Cell[BoxData[
  StyleBox["\[Theta]", "TR"]], "InlineFormula"],
 "-values by ",
 Cell[BoxData[
  ButtonBox["XToTheta",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/XToTheta"]], "InlineFormula"],
 ".  Note that the number of panels is twice the specified discretization \
number ",
 Cell[BoxData[
  StyleBox["n", "TI"]], "InlineFormula"],
 "."
}], "Text",
 CellID->565396260],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"t", ",", "\[Tau]", ",", 
     RowBox[{"XToTheta", "[", 
      RowBox[{"t", ",", "\[Tau]", ",", 
       RowBox[{"NDiscretizeUnitSegment", "[", 
        RowBox[{"n", ",", 
         RowBox[{"Layout", "\[Rule]", "layout"}]}], "]"}]}], "]"}]}], "]"}]}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{"nP", "=", 
  RowBox[{
   RowBox[{"NumberOfVectors", "[", "rP", "]"}], "-", "1"}]}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->11132],

Cell[BoxData["100"], "Output",
 ImageSize->{25, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->108759205]
}, Open  ]],

Cell[TextData[{
 "Compute the surface pressure coefficient at a relatively large number of ",
 Cell[BoxData[
  StyleBox["\[Theta]", "TR"]], "InlineFormula"],
 "-values using the exact solution.  The function ",
 Cell[BoxData[
  ButtonBox["SurfaceSpeed",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/SurfaceSpeed"]], "InlineFormula"],
 " computes surface speeds from the exact solution, and Bernoulli's equation \
is used to compute surface pressure coefficient."
}], "Text",
 CellID->332686776],

Cell[BoxData[{
 RowBox[{
  RowBox[{"n\[Theta]", "=", "250"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Theta]", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"0", ",", 
     RowBox[{"2", "\[Pi]"}], ",", 
     RowBox[{"\[Pi]", "/", "n\[Theta]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"VMagE", "=", 
   RowBox[{"SurfaceSpeed", "[", 
    RowBox[{"\[Alpha]", ",", "t", ",", "\[Tau]", ",", "\[Theta]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cpE", "=", 
   RowBox[{"1", "-", 
    RowBox[{"VMagE", "^", "2"}]}]}], ";"}]}], "Input",
 CellLabel->"In[5]:=",
 CellID->663191981],

Cell[TextData[{
 "Compute ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values for a conventional plot of surface pressure coefficient using ",
 Cell[BoxData[
  ButtonBox["Components",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/Components"]], "InlineFormula"],
 ", which is the selector for the data type CartesianVectors."
}], "Text",
 CellID->472644503],

Cell[BoxData[{
 RowBox[{
  RowBox[{"rE", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"t", ",", "\[Tau]", ",", 
     RowBox[{"Take", "[", 
      RowBox[{"\[Theta]", ",", 
       RowBox[{"n\[Theta]", "+", "1"}]}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"x", "=", 
   RowBox[{
    RowBox[{"Components", "[", "rE", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], ";"}]}], "Input",
 CellLabel->"In[9]:=",
 CellID->494361716],

Cell["\<\
Generate graphic for surface pressure coefficient using the exact solution.\
\>", "Text",
 CellID->191818375],

Cell[BoxData[{
 RowBox[{
  RowBox[{"grP", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rP", ",", 
     RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gCpE", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"x", ",", "cpE", ",", 
     RowBox[{"Epilog", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Inset", "[", 
        RowBox[{
         RowBox[{"PlotAirfoil", "[", 
          RowBox[{"rP", ",", 
           RowBox[{"Frame", "\[Rule]", "False"}], ",", 
           RowBox[{"PlotRange", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0", ",", "1"}], "}"}], ",", "Automatic"}], "}"}]}]}], 
          "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], ",", "1"}], "]"}], "}"}]}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "1.1"}], ",", "2.7"}], "}"}]}]}], "]"}]}], 
  ";"}]}], "Input",
 CellLabel->"In[11]:=",
 CellID->527771544]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.2.2  PMVortexLinearNeumann", "Section",
 CellID->234765418],

Cell["\<\
In addition to strength parameters, the output of PMVortexLinearNeumann \
includes locations of collocation points and surface speeds of the fluid at \
collocation points.  Bernoulli's equation is used to compute surface pressure \
coefficient at collocation points.\
\>", "Text",
 CellID->815037782],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "rC", ",", "VMagC"}], "}"}], "=", 
   RowBox[{"PMVortexLinearNeumann", "[", 
    RowBox[{"rP", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cp", "=", 
   RowBox[{"1", "-", 
    RowBox[{"VMagC", "^", "2"}]}]}], ";"}]}], "Input",
 CellLabel->"In[13]:=",
 CellID->349582656],

Cell["\<\
Plot the surface pressure coefficient at collocation points superimposed on a \
line showing results from the exact solution.  The shape of the discretized \
airfoil is included on the graph.\
\>", "Text",
 CellID->63663458],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"xC", "=", 
   RowBox[{
    RowBox[{"Components", "[", "rC", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gCp", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xC", ",", "cp"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gCpE", ",", "gCp"}], "]"}]}], "Input",
 CellLabel->"In[15]:=",
 CellID->726630446],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3W1MXVe+33FXfd0HqX3bqvOmat+0fVtXjm2wx8Y4dnCGkMTxnQl4CCQZ
SDL2zFzozJg8QJprdxBONMZ2jB0DiYE4EDKHeAIZYu7lwkzGVAju9TW+dUGy
oJKRJSNLPZIl+oc/rGzv88DhPOyz9t7fj/Z4yAHOWWezz2+tvfbaa32vvPZQ
+T/dsmXLrn+yZcv/k/+tfL3siS0AgDzxLOe9eSEAgBM5DwDBRs4DQLCR8wAQ
bOQ8AAQbOQ8AwWZVzg8N3o5GH3lQGAAID3tyfnpqobioreHEoDflAYCQsCfn
2y5823pmvOTg5aWlqDdFAoAwsCfn6+uujVy/09Q4PBC56U2RACAM7Mn5ivKe
mVv3JOol8L0pEgCEgT05X1zUtrQUlU2+8KREABAK9uT89m2t+oU27HNfIgAI
BUtyfnHxYcnBy/r16ZbR7q5JTwoFAMFnSc7Pzt4/cviKfj0QudnUOOxJoQAg
+CzJ+Ykbd2tr+vXr+fkHZaWdnhQKAILPwpxfXr8mm/tCAUDwWZLzrr4aHUuf
8zIBQAjYmfMd7ROtZ8ZzXygACD57cv6nr/+utqZfb4Z1deMAANJmSc63Xfi2
uOji9m2tuwvPy39Go4/0CwBAhuzJ+Tde/0Jy/tTJEX2Eu6UAICvsyXnZnI9I
4Pf1Tue4UAAQfJbkvDPVx8fmamv633t3mLulACBzluS8czristLO7dtadxWc
524pAMichTnf0T6hHfXcLQUAmbMk5xtODA4N3nY9yN1SAJA5S3K+tqZ/4sZd
14PdXZOnW0ZzWSgACD6bc356aqG6qjeXhQKA4LM556PRR8VFbfJvLssFAAFn
c84neRwAkCJLcr66qnd6aiH28dj7pwAAm2JJzpeVds7PP4h9nAnNACBDluc8
XfQAkCHLc36ZLnoAyIz9OU8XPQBkwuac1wnNWn87Rhc9AKTNkpwvOXh5cfGh
60Gd0Gx34Xm66AEgbZbkvOR57INmQrPjxyLSts9Z0QAgyGzIeclwyXMzX2Ws
vt5ps84UAGBTbMh50z+T6Afm5x8wFz0ApMeGnDf9M0l+PcmAHABAEjbk/HKC
/nmn0y2j3V2TWS0RAISCX3J+fGzu+LFIVksEAKHgl5xnAgQASI8NOZ/iZVaW
EQSANPgo5xldCQBp8FHOLy4+LDl4OauFAoDg81HOLzN3JQBsXg5yPlK5ZcXW
5pkUXyj1nKfrBgA2K+s5P9PcvDr8UdL+saTPPOfHx+Zeru4t3tuWeSEBIDxy
128Tqcxyzuv0CDueOEvXDQCkLmc5H6msfOy2pi2Pc34rxZzX6RFeq+lvahzO
cmEBILgyzfm1zvh160349d6blF5oU9OUccMUAGxKLtrzM81bV/M+0uzouMlW
zgtpzyeZxBgA4JT1nHc28J09N0leaHpqobqqN/WXmLhxl5UEASBFNoyfTyO3
maYYAFLk05xvPTPe0T6RcaEAIPh8mvOsMAUAKfJpzgsWBweAVPg350eu36mv
u5ZZoQAg+Pyb89Hoo7LSzsXFh5mVCwACzr85L9oufCtbBoUCgODzb87rtGYH
9l/i3lgASMK/Oa/TmhXsONvdNZlZ0QAgyPyb8zqt2Vtvfs0ASwBIwr85b7A+
OAAkEYCcn7l1r6K8J+1fB4BgC0DOL9OkB4DEbMj5gcjNDJcOoUkPAIkEI+eX
adIDQAKByXma9AAQV2ByXjScGBwavJ358wBAkAQp5xcXH5aVdnJ7LAA4BSnn
x8fmfvB0x1sNg5k/FQAERpByXmdC2PFEK5NYAoARpJzXmRBeq+nPyrMBQDAE
KeeN6qre6amFLD4hAPhXIHOeMZYAYAQy58XpllHmKwaA5eDm/NJS9MjhK1yQ
BYCg5vxyNqZHA4AACHDOi9Yz4x3tE7l4ZgDwi2DnfDT6qKK8Z3b2fi6eHAB8
Idg5L6anFmpr+pkMAUBoBT7nx8fmni37+Oc/i+To+QHAcoHP+fXJEM5O3Lib
o5cAAJsFPud1MoTGd/5QUd7DMEsAIRT4nDekPX/8WISOegBhE56cF91dk6db
Rj14IQCwR6hyXpw6OdLXO+3NawGADcKW8+L4scj42JxnLwcA+WVDznvcnbK0
FK2t6Z+5dc+zVwSAPLIh59sufCubN8VQ8/MPJOrlXy9fFADyIpw5Pz4291Ll
Z889+7G07b18XQDwXjhzXm+e2lVwrrqql5GWAIItnDmvN0+dOjkiDfv6umtE
PYAAC2fOOxH1AILNhpx/+82hsmc6ByI3vSlJLKIeQIDZkPPFRRe3b2vdXXje
m5LERdQDCCobcv6N17/Q3nJvSpKIRj0jcAAEjA05n9/+eSeJ+tqafqa1BBAk
5LzLZ59OFe1ta798I98FAYDsIOddzLok01ML+S4LAGQBOe+iQ+vfbfqmtqY/
j0OAACBbyPlEotFHTY3DzFcPwO/I+eS6uyYZhAPA18j5DY1cv1Nd1Ts7ez/f
BQGAdJDzqZCQl6gfGryd74IAwKaR8ylaWZ3kJ58fKmnv//zv8l0WANgEcj51
ZsglfTgAfIScT50OufzVL7+qKO9hyCUAvyDn06BDLo8fizBDAgD7kfNpGx+b
O3L4ivyb74IAQDLkfCaWlqLSqpe2PQPsAViLnM/cQOSmNOxHrt/Jd0EAIA5y
PisGv5o58OSlF3/YRY89ANuQ81mhQy4Ld56Thn1H+0S+iwMA3yHns0KHXJ46
ObK0FG09M15R3jNx426+CwUAK8j5XJi5da+2pr/hxCDdOADyjpzPnaHB20cO
X+numsx3QQCEGjmfU0tL0dMto2WlnYzGAZAv5LwH5ucfvFzdW7S37cNzf8x3
WQCEDjnvDTMHWn3dtZlb9/JdHAAhQs57wwzIGbl+p6K8p+HEoDTy810oAKGQ
g5yfad4qz1oZSfmFwpDzLnqJ9nTLKANyAORabtrzEvXk/Aai0UfdXZNlpZ2k
PYCcIufzy6R9fd21l6t7mdYeQNaR8zaQtN+/7+L2ba07t5+l3x5AdmWa85HK
LU5bm2dWHo2f807Ob5Hzy+sXal+r6Ze2fcOJwemphXyXCEBA0J630NDg7eqq
3tqafu6uApC5XOT8Wht/rW2/8QuR83FN3LhbX3ftyOErA5Gb0eijfBcHgF8x
ft5ys7P3mxqHy0o7335ziAu1ANJAzvvC4uLD4qK1C7V03QPYFHLeL8yF2uqq
3oryHhr2AFJEzvvRzK17TY3DJQcvt54ZZxwmgOTIef9aXHwojXwdhzk+Npfv
4gCwFDkfAEODt48fi0jgn/jVV9VVXKsF8BhH/K7f8rS1eSbS/NiYyCy/kBs5
nxXz8w/MTbWS/AzFBKDW4ndljsn18e4rX7vvcsraC8VDzmeLXqv9+c8GGk4M
lhy83NQ4zOAcAKvxK8nuvKlpprky6815ct5rS0vRgcjN6qreI4ev9PVOMyUm
EFor8euK+dy9UALkfE7Nzt4/dXJEmvf1ddfozwFCaCV+I5U56KeJ90IJkPPe
GLl+p+HE4P59F0ueutz627F8FweAR1bjV4L+uwa9o9dm5cpsZfPK8lCZt/fJ
eUuUPt2hK9VKC/90yyiL1QKB57gO+/jUwqtWgn7lvyOVGTf4yXlLmJVqFxcf
dndNVpT36JpWs7P38100ADmRdPx8pFJDf/3/c/RC5Hx+zc8/kMA/cviKbPKH
4AZbIGCS5fxaKz473ffkvP2kSS9/CGneSyP/3abhlyo/45YrIACSxK/jvqnc
vhA5b5uZW/f27W1b7cZvbT0zTh8+4GuJ4zcLffKpvRA5byPtxm84MShfmD78
iRt3810uAJuWIH61LZ/NUfXkvK9pH35tTX/JwcunTo6woCHgI8xjhk1ZXHzY
1ztdX3etuKhNWvtnz4y/+kof3fiAzch5pGdpKTo0eHvP7gs6Gl9a+wzUAexE
ziMT2o3/l7/48nTLaFlp55HDV+QLJk8DrELOI4tmZ+9Lw766qld7dQYiN5k/
Dcg7ch65oL06urhhRXmPNPs/+3SqtqafnnzAe+Q8cm3m1r3WM+OFO8+bNVCk
Fsh3oYAQIefhDe3Jf62mv+HEYHFRW3VVrzxCTz7gAXIeeSEJL418SfvdheeP
H4t0d00ykRqQI+Q88isafTQ+Nne6ZfTI4StlpZ0/Pz7wF0e6ej+byne5gOAg
52GP+fkHTxZf2r6tVTZp6kuDX6qAfBcK8D1yHlYx0+NPTy3I18ePRXYXnq+v
u9bdNcl0akB6yHlYLhp9NHL9zumW0YrynuKiNjIf2CxyHj6ytBQ1mb9394cH
nrz0VsMgmQ8kR87Dp3Sh24IdZ2nnA8nZkPMd7ROtZ8a9KQYCw/TkLz/ezifz
ARcbcn4gcrOpcdibYiDwYjP/1F9dr/zxVaZcQGiR8wgwzfyiPW06VlMyX84C
WBULYUPOI/C0h+fdpm8k81vPjNfW9DvH50ejj/JdQCC3yHmEk3N8fkV5z6mT
I3IcfjnwD0yqieAh54GZW/f6eqflICzYcU4n1ZT/5DIuAoOcBwzt4Xnj9d9J
814a+dLUl+Z924Vvx8fmmEsZ/kXOA4lEo48mbtyVnD9+LFJc1Hbk8BU5UE1T
X8KfTh74AjkPpGh29r4cq6apL5s0/gt3nqOpD8uR80AapKl/8r1vJOdLn+6I
beoDViHngcy5mvquXn16eJBf5DyQXa5e/bLSzu/v+lB7ePJdNIQUOQ/k1Pz8
g3fe/lpy/lBJu96fdbpldGjwNuskwjPkPOCl6amF7q7JhhODRw5fMT08I9fv
SHWwTA8PcoOct5y0+iZu3HVtHe0TOplz7CZ7UoLCs01eLlFJZJNWa2zhZWOq
AWV6eOrrrpWVdpYcvGx6eBjDgywi5z02c+ueZp28a5OHp06OmOSsKO/RSbd0
k1ZfbLq2nhlPFK3ytHGjNUeb813EbtJqjVs76IhE56YtW+cm+8T5VDr/mNm0
9Rswi4sPTQ+PjuGRHahvnJoRmSDns0ibZ7LJiblE0+mWUZNaJtAkxmNbwn29
0ybBwjkwz+w6s8k+cea8zj9mNmn9OquJ6qpe8y1pG8etHaanFvL9LjdHTuXk
hEjfuNSDrtinhwepI+fTIImhp9s6psLEjmmUSsLLtyTtTcjku8gBp38R3Uau
34lbO0hdEPf0wVkvyO9aW9u6Yr9wdSqegh3nOLqwIRty/oP3R0ueumxhy0Qb
mdo1oV0QcjZtWo+aDNKsCmo3QoA5Tx+c9YJkfmzvmTkFk2NAf8zU4Hk8R/jg
/b+Vsr34ox49W5RCSmNJCua70xZ4wIacP7D/kjaxvClJIpLVeolTe8sl0rXV
px0sekmRq2MhZC6pyDGgOW965JznCLFXT7QN4E3vuhRSGiRSMC2S/CuHscT+
xx0TP3n1cwsbUfCSDTn/61/+3iz06SVp+fT1Tsvr6kejrLRTP6TaW06kY1Ni
R0Npn57zurO5jmDOIHLUTaTHtsS+vvSOJ1rlZERPQ7L7QvAFG3Les/55nWbc
BLu2eeQRTnXhDXMdweS8q5tIGxvmEo85KUj7ENWZliXk5WRET0O0k4eRPKES
+Jw3ywYVF7XpskEEO6ylnYdmyJY5KTAdRKYi0N4hc6VgcfFh6q8iDR7nJV2N
/bffHJIveronc/fukC+BzHk5jM2ScGYZUPphEACmItDeIXOloOTgZedlAnPJ
WE8HkvcLaewXF11c6eHZ1ipPJZ8d/d1NVR+wVmByXk4/R67fkea6NHikWcIS
zwgn1yVjPR0w/UJ6253GuBldoLWA9vDIJ0iyXT47+ruS+XIirBcUdE4exu37
kd9zXlrp8uv1ddek6S7/9vVOM8QRSESn0dAYN6OFTS2gI0jNuYBWGf/4j4t6
QUHn5NGf1CV0uarrFzbkvLQT5BDa7BPqb0l7Q+oIaclnVDgA6yNIzbmA89Zj
c2fZKy/3yX++8Pwndb/48scVn5qh+1zVtZkNOS+Hhxw/KT6PNNfl1FLOJXUI
AccV4AHXnB46XZ65Olz6gw7ZDh74qGDHuf37Lr76ct9PX/ui/EWu6trCRzk/
PbWg0/rJCSMXVQF7OE8E/uepkR/9sLtgdVoG7eHZX3zpyAtX3n7r6+bf/LVe
F+byrsd8kfNybEjLQTb6ZwBfMFd1//7v/+/HHRMNvx58/rlP9M734n0X9+y+
IF98f9d5+VqqAOdFYZZfyQXLc17+6MePRWRjxDsQDNr509Q4vKvgvI7krCj/
9I3Xv3ittv/VV/r0Uq+5HKC3Cej8cgyxSJu1Ob+0FD3dMip/dGnM57hoAPJA
2/y/+uVXA5GbetNWcVGbjt4/f+5PchYwNDjzbtNw2TOdRys+NVeEdZynmTuC
KiAVdub8zK171VW9UunnvlwALOIcvS/BvuOJs6vTL5+VPNcuHWkBOueO0ClG
41YBXAswLMz5vt5pCXm66QBIXEuAVx69KnmuXTq61piZk81535arCtB7xPRO
YfnXeYNY2KoA23L+dMuo/BEZLQkgLl1rzMzJluJ6KxLpzhvEYqsAs9yM1hQB
G9FnVc7rfvamPAACIHa9FddUnBtO1KBVgFluRnt+dEUhnSZCbxA2U8Z5+Oay
xp6cJ+QBZMg1FWdZaef3d3240uzfeS6NJrpOE6H3BZgp45wzBbnmiLD2LMCS
nK88elUqYm9KAiAk5ucfvPP21xLLh0rapYlu5mST1rsZopPezGzOKsB1FqBz
R7s6gvI7IigXOR+plGfdsrV5JsUXkp1w4MlL3AMFIKf0Qq2O0pE01jGcOiHz
roKsrVuqc0e7OoJc0wTpOjI6F5wHNwdlP+cl5Ssjq///WNInncdsJu+LwwII
Gx3D+cbrX0gC792z0to3E+9rD0zWJ2F2TROkS1GbaYJ0vlBzIqC3BmRlTcms
5/xM89bVmHcHfZIX+ujin6U9n91iAMCmSLaboft6x5bp2/emADpNkDkR0FsD
zKzRuriwqxZI8Vwg6zm/3pyPk/OJ/Nt/s/d7/64kyQ8AQMj983/2vX/5L/79
v/5X/+U//oej//k//fS//dffaP7rtuGvZzfn02jPd7T/r+J9bdktBryR/MTW
3Lci28cdEyvrfP12zDyiA6HNiti6vf3mUNkznfV1X7oej920W7Vgx9ks/qS8
+vPPfeIqpGszLajUT+pZgykA5O8uh+upkyNHDl/ZXXhetrSH8Wwo+XIwrkXB
5LjacLRn1nM+jf75sb+dLSzw6MwIKdI5Qk3imXEFupkxZnq0683psZu5D935
k+YRvbEx7UzWbtXKo1ez+JNra6Q+XkjXZnpTk2x6ih373psah+O+ro73dtWG
yU/JqTvyKBp9dPK9b1Ym3n+6Qw5yCV45kuVvoXfxp/KnMR31us6vaz7/JMs7
pif7Ob/58Tbz8w8k57NyuQEb+nLgHzRJTDegGRLgXE5aN5N4ZpywbuaeET3a
/3vd7zd8XTNRbVZ+LEc29epJflgi2nkKoHvpL3/xpXz24+a8jvd21YZxKxQz
YMP5k6bP1mzaeWsqjkttG9+ZQsWRNol3aepLVks+S+xrr/6ugnNxx166Bt7o
hJzaJs/dwJtc5PxmX0hy/qknP5Jj1ZuSBJh+VNsv35BjxkSKXs0xK3vqAhCS
D85wMIlkZvzIb96G1oa73bQDTd3hvHJnNv2jO6qDOCcder+/66TDeSalzUjd
TDOM6sDFGeYS2q++0lfy1OX1ZlLr/uJL5S92v/c/vnn7ra8rXlxpX+VlIL0l
OS91nJykMAXxppgePNMa1P4TObrka9NFoE07My8cAR4qSf7cer+/66TD2bul
3cK6mc7htQRbnU/MbKaHwWwfXfqznjYmaaP6qMrQIfGu6XFi50Yw/W865YL8
K78iH085NduxTSfezE/HhT05L/tE9ga9N3HJbrlw/k9lz3RWvXTVNL3M58sc
XW2rk/uR4cgdrTga3/mDs44wAWg27buQhkds71OiMwjTGeg8r/SyLnC1zGNv
cXVNd7mpK7BS8ek8PFL9yZmUNMPkhYb/8L+9eXf25Pzy6vAMOSpYOsr0upgO
czk2zFBe01oArJXkPCLRGYS5uB/vOtF3tYPeSWry1ly5TlQSV02hLXNXmGuN
E7dlnovhNFKLSXnkhczs+rlu31qV8/q1RL3sZG9KZQPtcX23afhQSftzz36s
DR7tdaHDHCFn1pwytYPeSWr6T2JHfEkLXDKk4sWeH/1Fd0X5p7sL184s9uz+
UK9KHK341BnmUlPU/OTzvHQf6bvT2b0kA6UK++zTqVy08G3LeSX1rPwFA7wK
gFTfukSmjsWVv+y+vW16jT7fRQP8QVvmOk1Z5Y+vOrtZJMwlOV99pa++7tpz
ZSttpxee/+TnPxuIHRmrH0D5Yuf2s64xrtNTC172GsnbkUwo3LnWl5XdhZbs
zPnl1bMteVB2u7VTfW6KvB1pRdTXfamHohyEUnebAbfLNNeBeOKuD5JkWsjN
xoV8ALX7SGoB1xhXZ3XgmnlGPq1J+osyqR00B6pe+kwqICnAB++PSm2VeUVj
bc4vr3ZoyC6V99vXO53LouWQtNvlDycH5/rV9rM2T1IN5MWGiz1lcgE0E6b1
5Zp5RuoCV3+R9u3r5QO9265w5zlnmzyN8JcTir3fv6C5kWHnhs05r+QNyn6W
H5CTGl8kpBRyaPB2U+OwnjzKISF/Yr3aTnMdoZVkwnY/Lt7qOgHXd6eXD/T+
60Ml7eamFdM7JImtt0Tpu9sw/PVVXqnuk13009d/Jy389Nr2NuS8BKP8uZP/
utQFUlHqpYr8ztgfl/y9Xq7ubXznaz10G04Myp/D8gMVyDq9EVhHi/l6Aabs
Mr1DlUev6hQHerZieuP1ooBOmJko+Q/sv5R2296GnBdS/lSeRPaDVJd6U5Xs
ChsOEtnnfb3T5lKOT5ePBFJk5qaLOzGLTuyjo8V8vaCqN1qa/0Z74/WigDQR
tYNXkkSXIHHGvmnbS4154fyfNtUL5K+cN+Tta8eINA/kTDCzoqUjGn0kO1mq
GymDnLtpZU23DIJBu6NdfSzaBDVz03kzMUvYaAfvT179XJcgWZ+r5Kwz86Wq
1btpUl+eyac5ryRs5VCUqPeyq0QOaali5BXlXyZqgE/poES9h9pMceBa2MjV
x0I/pPfMAHuT+YU7z0nu6eOlT3ekeMukr3PeiL30mfX7y2R/fvD+6N49F557
9mOpULgjFZZzdbBo+9Dc+6mDEmtXF8U2U5bRLLeZZvsLh69IytXXXXu5emXU
pfwrjf8NO3CCkfNOOpRR0l6O5+PHIvJ1hkevNGOk4pDPhY5xYilbWEIveprh
iCbJdRCLq4NF+3uZNCMApAbfv29tmQb9YsNQCl7OG87J4nQyAT0DTf3qrZzb
6pBOqSwk7bmVCV7STvLYGDdzv+hFTzMc0SS5DeMTkFOmP+cHh9pXpkf7YXfy
nw9wzrtoV6S2diS69dY2+RDF7XWUhG9qHJYfk88O7R/kgukh1wlbzG2Yug6F
6SSPjXH6yWGsVfpPtCbvuglPzjvJR0xvbdMBM3qbhnzQ5KP04bk/Pn2o48D+
S76YFht2MlO7myErpjVuYtz0kOsEjGZaFQtvD4G1tGGvfThJfsySnJc2dh5P
NvVTKXus6qXP1m9kPmuS37lIB0LONMKdM66bxZtMhpulmsyQFdMaJ8aRXdpF
L8GV5GcsyXn5gOT3+JcP4JHDV/SKlVlGQZJfm2F6/7J+ePUeEB02TP4Hg3Nl
JR2d4myBmzEqzka4cwUlsxgrGQ7vaZN+7562JD9DzstnXAcPbxjamgZ6T7fe
Bmjmr3CegJuWmzflR1xmVKFzqVxn29sMSN7++EqpWtc7/46MUYHl5GiXwzhJ
l0jIc14SQLI68ztqnRfUTDvQNampc81W5xJpnBQk4Yzr2GUmzLVLV7eJc1Sh
c7c7295UxAgSybEkh3Q4c16nHXvxh12SAB5cF9ABcs6GpXOJNHNS4GpYmsmu
nZvezxK7WdJjoCO6YzdzOdK5OZeZjt0VsXEdu2ycuXZp1U4AvHf8WCRJezWc
Ob8+WZx1izc5O4rNZNfJszG2KbvhpitYbbjFpu6Gm47ojt3M5cjkdRanNkB6
pMmaZIhgCHNeGoG6lFho73jSFWk33EhdwC/kPDfJstqhyvmlpWh93bXWM+O5
fiEA8JKeIyf6riU5X1Hek/WZx1ykHqmu6h25fienrwIA3vNFztfW9Od0/MP0
1IJUJXREAAikkOf8+Njciz/qOfLCFSZ3AhBUp/5qRLZE3w18zuuiikwmDCDA
3nt3+PlnP0n03WDn/EDk5o+PXg3z0BoAgffRpT/vKji/b++FRD8Q4JyXkG9q
HM7ucwKAbQp2nNMbWBL9QFBzfuT6nfq6a1l8QgCw0/FjkdUbPxP2TluS88lv
5tqs6akFqTiYewpASPiifz6LOT9z656EPKNrAIRHqHJ+cfFhdVUv81kBCJXw
5Pzf/PX/Kd538cNzf8zweQDAX8KT80V7LjBUHkAI+SLnk9+0mwqGygMILftz
fnxsruyZzvq6L9N+8tnZ+xXlPQywARBO9ue8rpFRsCPZeuVJSLwzRxmAMLM/
53W98sqjV9N75tMto91dk+mWCwB8z/6cX16N+vSW/5ieWqiu6k23UAAQBL7I
+fTmoqHHBgCWg5vz42NzpT/o+PUvf59ZuQDA94Ka84eeame0PAAsBzfnnyv7
mNHyALDsk5yfuHG3tqY/9Wdj2mEAMIKX81x+BQCn4OV8d9fk6ZbRbBQKAILA
Fzk/c+ueNNFTeR5pzJeVdi4uPsxSuQDA93yR8/PzDyS9U3ke7n4FAJcg5bw0
4+XHmK8MAJx8kfMS3amMhKcxDwCxfJHzYvu21uQ/MPjVzO7CD/s//7vsFQoA
giAwOV+0p40bYAEgll9yvqy0M8n63dHoI10WkBtgAcAlGDnPmHkASMQvOV9b
0z9x426i7yavBQAgzAKQ88xmAwBJ+CXnG04MDg3ejvutivKemVv3clAoAAgC
v+R8U+PwQORm7OOsDAgAyfkl59sufCtb7OOJ8h8AoPyS87EjasbH5l6u7j2w
/xITHQBAEn7J+aHB2w0nBp2PlJV2bt/WWrDjbC7LBQC+55ecj52CvqN9QnK+
8Z0/5LBYAOB/fsn52CkrGU4JAKnwS87HTlmZ/M4pAIDyS86Pj80VFpzv6V6b
djj1FaYAIOQ8z/mZ5q3yrJWRTb6QXnUt3HlO/7O+7trI9TvZLhsABFA+2vMS
9ZvOeb3qWnn06jKNeQDYDL/k/PL6ra/jY3P7iy998D6zUwJASuzJeadEv1lb
07+r4DzriQBA6uzJ+VR+c2kp+ptTI6wnAgCpW1hYmpycT/Rd23IeAJBduYjf
SOVq18zW5pkcvxAAYEM2jJ+nCnBibzixN5zYG07sDRcbMtaGMvgCe8MpK3vD
kl2aeTHYG9l9hmw9iSVsyNhclyEwhw3FoBgUw0fFyMqT5LoY5DzFoBgUg2Lk
90mClPMAgLzwJucBAAAAAAAAAEAe6d2yj98sm+zxYEvwruPeUhx8yY6BOHNp
BFySvaFrPITq8EiwN3RPhGxfrIi/zMeyJUEqhVgtWqTy8YIkejzYErzrmebm
yNq32Rvme3EP6uBKvDfkEx6mw2JVor2xXvuvfz9U4jV97AjS70r2eDkSPR5s
G77rMO2MZHsjUlkZCVl7PuHeWG/AhqraS3xsrOXayhGSn6LlUZyPhCVB+l21
+3gxEj0ebBu963AdvAn3hu6GkOV8or1hGvOhOtlL9kkJ36neujgfCUuClPa8
U/J3vd57ExYJ9sZ37df89zp6iE+KU7L2vDbnQ3RoGPa25+mff8zGfbCR5vDs
juTHQMja88k+KasPhGt/bJQbYbxmYXH//LL7crDzPCNcDTYVd2+sjbYJWyds
4mNjRbhybUXyT0q4joyEe8Oc7oVsd7jH5IU9SAEAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAA8IzOhVjZvPL/oZsPEQBCYWXaW8n5cK34AgDhEs61igAg
JHT5hsdXPAEABMVav7yuWUSjHgAAAAAAAAAAAJvx/wGnbtY+
    "], {{0, 324}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 324},
  PlotRange->{{0, 504}, {0, 324}}]], "Output",
 ImageSize->{508, 324},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[17]=",
 CellID->279409335]
}, Open  ]],

Cell["Clear variables for use in next section.", "Text",
 CellID->386399333],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{
  "a", ",", "b", ",", "rC", ",", "VMagC", ",", "cp", ",", "xC", ",", "gCp"}], 
  "]"}]], "Input",
 CellLabel->"In[18]:=",
 CellID->437524282]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.2.3  PMDoubletConstantDirichlet", "Section",
 CellID->617689357],

Cell["\<\
The same procedure is followed to compute data for the validation graph using \
the model PMDoubletConstantDirichlet.  Jump conditions are used in the model \
function to compute fluid speed just outside the airfoil associated with \
collocation points.\
\>", "Text",
 CellID->311897436],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "aW", ",", "rC", ",", "VMagC"}], "}"}], "=", 
   RowBox[{"PMDoubletConstantDirichlet", "[", 
    RowBox[{"rP", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cp", "=", 
   RowBox[{"1", "-", 
    RowBox[{"VMagC", "^", "2"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"xC", "=", 
   RowBox[{
    RowBox[{"Components", "[", "rC", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gCp", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xC", ",", "cp"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gCpE", ",", "gCp"}], "]"}]}], "Input",
 CellLabel->"In[19]:=",
 CellID->186135957],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3VtsHNd9x3EVfe4FaF9bNC9F+9L2tSpsybrYlihbkWtFsWX6QjoMGTtS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    "], {{0, 324}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 324},
  PlotRange->{{0, 504}, {0, 324}}]], "Output",
 ImageSize->{508, 324},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[23]=",
 CellID->93460077]
}, Open  ]],

Cell[BoxData[""], "Input",
 CellLabel->"In[24]:=",
 CellID->672251866],

Cell["Clear variables for use in next section.", "Text",
 CellID->499431756],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{
  "a", ",", "aW", ",", "rC", ",", "VMagC", ",", "cp", ",", "xC", ",", "gCp"}],
   "]"}]], "Input",
 CellLabel->"In[25]:=",
 CellID->755557078]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.2.4  PMDoubletQuadraticDirichlet", "Section",
 CellID->246523140],

Cell["\<\
Repeat the procedure for the model PMDoubletQuadraticDirichlet.\
\>", "Text",
 CellID->140642404],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c", ",", "aW", ",", "rC", ",", "VMagC"}], 
    "}"}], "=", 
   RowBox[{"PMDoubletQuadraticDirichlet", "[", 
    RowBox[{"rP", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cp", "=", 
   RowBox[{"1", "-", 
    RowBox[{"VMagC", "^", "2"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"xC", "=", 
   RowBox[{
    RowBox[{"Components", "[", "rC", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gCp", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xC", ",", "cp"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gCpE", ",", "gCp"}], "]"}]}], "Input",
 CellLabel->"In[26]:=",
 CellID->519865160],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3VtsHNd9x3EVfe4FaF9bNC9F+9L2tSpiybo4lmhblmtZtWXGNmkzZOxI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    "], {{0, 324}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 324},
  PlotRange->{{0, 504}, {0, 324}}]], "Output",
 ImageSize->{508, 324},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[30]=",
 CellID->429901368]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.2.5  Summary", "Section",
 CellID->135714564],

Cell["\<\
Although there are noticeable differences in surface pressure coefficient \
between each of the three models and the exact solution, each model can be \
characterized as providing plausible results for this  example.  Changing \
input parameters and re-evaluating the notebook enables the user to judge \
agreement of the three panel methods implemented with the results of an exact \
solution over a range of input parameters.\
\>", "Text",
 CellID->20514405]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["PanelMethods",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/PanelMethods"]], "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["3.1 Introduction to Panel Methods",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods"]], \
"RelatedTutorials",
 CellID->806871991],

Cell[TextData[ButtonBox["3.3 Implementation",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation"]], "RelatedTutorials",
 CellID->289624919],

Cell[TextData[ButtonBox["Overview",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]], "RelatedTutorials",
 CellID->280152894]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"3.2  Validation Example - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 4, 52.3428000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "None", "summary" -> 
    "An exact solution for the van de Vooren airfoil is used to validate the \
three panel methods implemented in Aerodynamics. The family of van de Vooren \
airfoil airfoils are symmetric airfoils obtained by mapping a circle to an \
airfoil shape using the van de Vooren transformation. Surface pressure \
coefficient is chosen as the quantity to compare between the exact solution \
and numerical panel-method solutions. Refer to Katz and Plotkin (2001) for a \
discussion of the van de Vooren airfoil and the exact potential-flow solution \
for this airfoil. The functions in Aerodynamics for the airfoil geometry and \
the exact solution are from this reference as I have been unable to locate a \
copy of the original paper. Load the packages.", "synonyms" -> {}, "title" -> 
    "3.2  Validation Example", "type" -> "Tutorial", "uri" -> 
    "Aerodynamics/tutorial/ValidationExample"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[616, 21, 78, 1, 70, "TutorialColorBar"],
Cell[697, 24, 191, 5, 70, "LinkTrail"],
Cell[891, 31, 2928, 66, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3844, 101, 60, 1, 70, "Title",
 CellID->509267359],
Cell[3907, 104, 511, 10, 70, "Text",
 CellID->25608729],
Cell[4421, 116, 489, 12, 70, "Text",
 CellID->124571872],
Cell[4913, 130, 54, 1, 70, "Text",
 CellID->836781195],
Cell[4970, 133, 101, 3, 70, "Input",
 CellID->2058623809],
Cell[CellGroupData[{
Cell[5096, 140, 67, 1, 70, "Section",
 CellID->631983245],
Cell[5166, 143, 152, 4, 70, "Text",
 CellID->405071093],
Cell[5321, 149, 258, 8, 70, "Input",
 CellID->692274660],
Cell[5582, 159, 84, 1, 70, "Text",
 CellID->14124012],
Cell[5669, 162, 161, 5, 70, "Input",
 CellID->361350040],
Cell[5833, 169, 950, 30, 70, "Text",
 CellID->565396260],
Cell[CellGroupData[{
Cell[6808, 203, 527, 15, 70, "Input",
 CellID->11132],
Cell[7338, 220, 160, 5, 36, "Output",
 CellID->108759205]
}, Open  ]],
Cell[7513, 228, 511, 12, 70, "Text",
 CellID->332686776],
Cell[8027, 242, 633, 20, 70, "Input",
 CellID->663191981],
Cell[8663, 264, 391, 11, 70, "Text",
 CellID->472644503],
Cell[9057, 277, 470, 15, 70, "Input",
 CellID->494361716],
Cell[9530, 294, 119, 3, 70, "Text",
 CellID->191818375],
Cell[9652, 299, 1183, 35, 70, "Input",
 CellID->527771544]
}, Open  ]],
Cell[CellGroupData[{
Cell[10872, 339, 67, 1, 70, "Section",
 CellID->234765418],
Cell[10942, 342, 310, 6, 70, "Text",
 CellID->815037782],
Cell[11255, 350, 391, 12, 70, "Input",
 CellID->349582656],
Cell[11649, 364, 234, 5, 70, "Text",
 CellID->63663458],
Cell[CellGroupData[{
Cell[11908, 373, 449, 13, 65, "Input",
 CellID->726630446],
Cell[12360, 388, 12680, 215, 345, "Output",
 CellID->279409335]
}, Open  ]],
Cell[25055, 606, 76, 1, 70, "Text",
 CellID->386399333],
Cell[25134, 609, 190, 6, 70, "Input",
 CellID->437524282]
}, Open  ]],
Cell[CellGroupData[{
Cell[25361, 620, 72, 1, 70, "Section",
 CellID->617689357],
Cell[25436, 623, 297, 6, 70, "Text",
 CellID->311897436],
Cell[CellGroupData[{
Cell[25758, 633, 801, 23, 101, "Input",
 CellID->186135957],
Cell[26562, 658, 12423, 211, 345, "Output",
 CellID->93460077]
}, Open  ]],
Cell[39000, 872, 70, 2, 70, "Input",
 CellID->672251866],
Cell[39073, 876, 76, 1, 70, "Text",
 CellID->499431756],
Cell[39152, 879, 191, 6, 70, "Input",
 CellID->755557078]
}, Open  ]],
Cell[CellGroupData[{
Cell[39380, 890, 73, 1, 70, "Section",
 CellID->246523140],
Cell[39456, 893, 107, 3, 70, "Text",
 CellID->140642404],
Cell[CellGroupData[{
Cell[39588, 900, 827, 24, 101, "Input",
 CellID->519865160],
Cell[40418, 926, 12412, 211, 345, "Output",
 CellID->429901368]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[52879, 1143, 53, 1, 70, "Section",
 CellID->135714564],
Cell[52935, 1146, 470, 8, 70, "Text",
 CellID->20514405]
}, Open  ]],
Cell[CellGroupData[{
Cell[53442, 1159, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[53510, 1162, 158, 3, 70, "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[CellGroupData[{
Cell[53705, 1170, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[53780, 1173, 199, 5, 70, "RelatedTutorials",
 CellID->806871991],
Cell[53982, 1180, 170, 4, 70, "RelatedTutorials",
 CellID->289624919],
Cell[54155, 1186, 166, 4, 70, "RelatedTutorials",
 CellID->280152894]
}, Open  ]]
}, Open  ]],
Cell[54348, 1194, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

