(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     37709,        870]
NotebookOptionsPosition[     34169,        774]
NotebookOutlinePosition[     36583,        830]
CellTagsIndexPosition[     36540,        827]
WindowTitle->3.7 Worksheet for Airfoil Aerodynamics - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"3.6 Aerodynamic Coefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicCoefficients"], \
"\<\"References\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/References"], "\<\"Overview\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PanelMethods\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/PanelMethods"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/tutorial/WorksheetForAirfoilAerodynamics\"\
\>":>None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/WorksheetForAirfoilAerodynamics"]}, 
            Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/WorksheetForAirfoilAerodynamics.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/WorksheetForAirfoilAerodynamics", 
             ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["3.7 Worksheet for Airfoil Aerodynamics", "Title",
 CellID->509267359],

Cell["", "Text",
 CellID->550749044],

Cell[TextData[{
 "The following worksheet enables you to compute the aerodynamic \
characteristics for potential flow over an airfoil of interest to you.  The \
airfoil may be one of the 1500+ airfoils specified by data files in the ",
 ButtonBox["UIUC Airfoil Coordinates Database",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.ae.uiuc.edu/m-selig/ads/coord_database.html"], None}],
 ", a data file in the folder Airfoil Data associated with ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " or a member of the NACA family of four-digit airfoils.  When you evaluate \
the worksheet for the first time in a session you are asked in a pop-up \
window if you want to evaluate initialization cells.  Choosing yes loads the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages needed for the computation."
}], "Text",
 CellID->1534169418],

Cell["\<\
Output for this worksheet is a new notebook containing graphs for flow \
visualization and surface pressure distribution.\
\>", "Text",
 CellID->8167176],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 Evaluatable->True,
 InitializationCell->True,
 CellLabel->"In[9]:=",
 CellID->236023479],

Cell[BoxData[
 TagBox[
  DynamicModuleBox[{$CellContext`\[Alpha]D$$ = 5., $CellContext`filename$$ = 
   "e205.dat", $CellContext`id$$ = 4412, $CellContext`nFile$$ = 
   "data", $CellContext`nEq$$ = 50, $CellContext`layout$$ = 
   Aerodynamics`AirfoilGeometry`HalfCosineSpacing, $CellContext`model$$ = 
   Aerodynamics`PanelMethods`PMVortexLinearNeumann, $CellContext`reset$$ = 
   False, $CellContext`source$$ = 1, $CellContext`viz$$ = $CellContext`sp}, 
   InterpretationBox[
    PanelBox[
     TagBox[GridBox[{
        {
         StyleBox["\<\"Worksheet for Airfoil Aerodynamics\"\>", "Section",
          StripOnInput->False], "\[SpanFromLeft]", "\[SpanFromLeft]", 
         "\[SpanFromLeft]", "\[SpanFromLeft]", "\[SpanFromLeft]", 
         "\[SpanFromLeft]"},
        {"\<\"\"\>", "\[SpanFromLeft]", "\[SpanFromLeft]", "\[SpanFromLeft]", 
         "\[SpanFromLeft]", "\[SpanFromLeft]", "\[SpanFromLeft]"},
        {
         StyleBox["\<\"Input:\"\>", "Subsection",
          StripOnInput->False], 
         "\[SpanFromLeft]", "\<\"angle of attack (deg)\"\>", 
         "\[SpanFromLeft]", "\[SpanFromLeft]", 
         InputFieldBox[Dynamic[$CellContext`\[Alpha]D$$], Number,
          FieldSize->3], ""},
        {"\<\"\"\>", "\<\"source of airfoil\"\>", 
         InterpretationBox[
          StyleBox[GridBox[{
             {
              
              SetterBox[Dynamic[$CellContext`source$$], {
               1}, "\<\"UIUC database\"\>"], 
              
              SetterBox[Dynamic[$CellContext`source$$], {
               2}, "\<\"file in Airfoil Data\"\>"], 
              
              SetterBox[Dynamic[$CellContext`source$$], {
               3}, "\<\"NACA 4-Digit\"\>"]}
            },
            BaselinePosition->{1, 1},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.]}, 
                Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}, "RowsIndexed" -> {}}],
           Deployed->True],
          SetterBar[
           Dynamic[$CellContext`source$$], {
           1 -> "UIUC database", 2 -> "file in Airfoil Data", 3 -> 
            "NACA 4-Digit"}]], "\[SpanFromLeft]", "\[SpanFromLeft]", 
         "\[SpanFromLeft]", "\[SpanFromLeft]"},
        {"\<\"\"\>", "\<\"file name\"\>", 
         InputFieldBox[Dynamic[$CellContext`filename$$], String,
          Enabled->Dynamic[$CellContext`source$$ != 3],
          FieldSize->10], "\[SpanFromLeft]", "\<\"\"\>", "\<\"4-digit id\"\>", 
         InputFieldBox[
          Dynamic[$CellContext`id$$, If[
           0 <= # <= 9999, $CellContext`id$$ = #, $CellContext`id$$ = 
            FromDigits[
              Take[
               IntegerDigits[#], 4]]]& ],
          ContinuousAction->True,
          Enabled->Dynamic[$CellContext`source$$ == 3],
          FieldSize->3]},
        {"\<\"\"\>", "\<\"discretization:\"\>", 
         InputFieldBox[Dynamic[$CellContext`nFile$$],
          Enabled->Dynamic[$CellContext`source$$ != 3],
          FieldSize->3], "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", 
         InputFieldBox[Dynamic[$CellContext`nEq$$],
          Enabled->Dynamic[$CellContext`source$$ == 3],
          FieldSize->3]},
        {"\<\"\"\>", "\<\"layout\"\>", 
         InterpretationBox[
          StyleBox[GridBox[{
             {
              
              SetterBox[Dynamic[$CellContext`layout$$], {
               Aerodynamics`AirfoilGeometry`ConstantSpacing}, \
"\<\"ConstantSpacing\"\>",
               Enabled->Dynamic[
                 
                 Or[$CellContext`source$$ == 3, $CellContext`nFile$$ != 
                  "data"]]], 
              
              SetterBox[Dynamic[$CellContext`layout$$], {
               Aerodynamics`AirfoilGeometry`CosineSpacing}, \
"\<\"CosineSpacing\"\>",
               Enabled->Dynamic[
                 
                 Or[$CellContext`source$$ == 3, $CellContext`nFile$$ != 
                  "data"]]], 
              SetterBox[Dynamic[$CellContext`layout$$], {
               Aerodynamics`AirfoilGeometry`HalfCosineSpacing}, \
"\<\"HalfCosineSpacing\"\>",
               Enabled->Dynamic[
                 
                 Or[$CellContext`source$$ == 3, $CellContext`nFile$$ != 
                  "data"]]]}
            },
            BaselinePosition->{1, 1},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.]}, 
                Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}, "RowsIndexed" -> {}}],
           Deployed->True],
          SetterBar[
           Dynamic[$CellContext`layout$$], {
           Aerodynamics`AirfoilGeometry`ConstantSpacing -> "ConstantSpacing", 
            Aerodynamics`AirfoilGeometry`CosineSpacing -> "CosineSpacing", 
            Aerodynamics`AirfoilGeometry`HalfCosineSpacing -> 
            "HalfCosineSpacing"}, Enabled -> Dynamic[
             
             Or[$CellContext`source$$ == 3, $CellContext`nFile$$ != 
              "data"]]]], "\[SpanFromLeft]", "\[SpanFromLeft]", 
         "\[SpanFromLeft]", "\[SpanFromLeft]"},
        {"\<\"\"\>", "\<\"model: PM\"\>", 
         InterpretationBox[
          StyleBox[GridBox[{
             {
              
              SetterBox[Dynamic[$CellContext`model$$], {
               Aerodynamics`PanelMethods`PMVortexLinearNeumann}, \
"\<\"VortexLinearNeumann\"\>"], 
              
              SetterBox[Dynamic[$CellContext`model$$], {
               Aerodynamics`PanelMethods`PMDoubletQuadraticDirichlet}, \
"\<\"DoubletQuadraticDirichlet\"\>"], 
              
              SetterBox[Dynamic[$CellContext`model$$], {
               Aerodynamics`PanelMethods`PMDoubletConstantDirichlet}, \
"\<\"DoubletConstantDirichlet\"\>"]}
            },
            BaselinePosition->{1, 1},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.]}, 
                Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}, "RowsIndexed" -> {}}],
           Deployed->True],
          SetterBar[
           Dynamic[$CellContext`model$$], {
           Aerodynamics`PanelMethods`PMVortexLinearNeumann -> 
            "VortexLinearNeumann", 
            Aerodynamics`PanelMethods`PMDoubletQuadraticDirichlet -> 
            "DoubletQuadraticDirichlet", 
            Aerodynamics`PanelMethods`PMDoubletConstantDirichlet -> 
            "DoubletConstantDirichlet"}]], "\[SpanFromLeft]", 
         "\[SpanFromLeft]", "\[SpanFromLeft]", "\[SpanFromLeft]"},
        {
         StyleBox["\<\"Output:\"\>", "Subsection",
          StripOnInput->False], "\[SpanFromLeft]", "\[SpanFromLeft]", 
         "\[SpanFromLeft]", "\[SpanFromLeft]", "\[SpanFromLeft]", 
         "\[SpanFromLeft]"},
        {"\<\"\"\>", "\<\"flow visualization\"\>", 
         InterpretationBox[
          StyleBox[GridBox[{
             {
              
              SetterBox[
               Dynamic[$CellContext`viz$$], {$CellContext`sp}, "\<\"stream \
plot\"\>"], 
              
              SetterBox[
               Dynamic[$CellContext`viz$$], {$CellContext`slp}, \
"\<\"streamline plot\"\>"], 
              SetterBox[Dynamic[$CellContext`viz$$], {$CellContext`sdp}, 
               TemplateBox[{"\"stream-\"",SubscriptBox["C", "p"],"\"plot\""},
                "Row",
                
                DisplayFunction->(
                 RowBox[{#, "\[InvisibleSpace]", #2, 
                   "\[InvisibleSpace]", #3}]& ),
                InterpretationFunction->(RowBox[{"Row", "[", 
                   RowBox[{"{", 
                    RowBox[{#, ",", #2, ",", #3}], "}"}], "]"}]& )]]}
            },
            BaselinePosition->{1, 1},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.]}, 
                Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}, "RowsIndexed" -> {}}],
           Deployed->True],
          SetterBar[
           
           Dynamic[$CellContext`viz$$], {$CellContext`sp -> 
            "stream plot", $CellContext`slp -> 
            "streamline plot", $CellContext`sdp -> Row[{"stream-", 
               Subscript[C, $CellContext`p], "plot"}]}]], "\[SpanFromLeft]", 
         "\[SpanFromLeft]", "\[SpanFromLeft]", "\[SpanFromLeft]"},
        {"\<\"\"\>", 
         TemplateBox[{
          "\"Aerodynamic properties are presented as graph of surface\"",
           InterpretationBox[
            Cell[
             BoxData[
              Cell[
               BoxData[
                FormBox[
                 SubscriptBox["C", "p"], TraditionalForm]]]]], 
            TextCell[
             RawBoxes[
              Cell[
               BoxData[
                FormBox[
                 SubscriptBox["C", "p"], TraditionalForm]]]]]],
           "\"with numerical values for\"",InterpretationBox[
            Cell[
             BoxData[
              Cell[
               BoxData[
                FormBox[
                 SubscriptBox["C", "\[ScriptL]"], TraditionalForm]]]]], 
            TextCell[
             RawBoxes[
              Cell[
               BoxData[
                FormBox[
                 SubscriptBox["C", "\[ScriptL]"], TraditionalForm]]]]]],
           "\"and\"",InterpretationBox[
            Cell[
             BoxData[
              Cell[
               BoxData[
                FormBox[
                 SubscriptBox["C", "m"], TraditionalForm]]]]], 
            TextCell[
             RawBoxes[
              Cell[
               BoxData[
                FormBox[
                 SubscriptBox["C", "m"], TraditionalForm]]]]]],"\".\""},
          "Row",
          
          DisplayFunction->(
           RowBox[{#, "\[InvisibleSpace]", #2, "\[InvisibleSpace]", #3, 
             "\[InvisibleSpace]", #4, "\[InvisibleSpace]", #5, 
             "\[InvisibleSpace]", #6, "\[InvisibleSpace]", #7}]& ),
          InterpretationFunction->(RowBox[{"Row", "[", 
             RowBox[{"{", 
               
               RowBox[{#, ",", #2, ",", #3, ",", #4, ",", #5, ",", #6, 
                 ",", #7}], "}"}], "]"}]& )], "\[SpanFromLeft]", 
         "\[SpanFromLeft]", "\[SpanFromLeft]", "\[SpanFromLeft]", 
         "\[SpanFromLeft]"},
        {"\<\"\"\>", 
         ButtonBox["\<\"reset\"\>",
          Appearance->"Palette",
          ButtonData->"paclet:ref/reset",
          
          ButtonFunction:>($CellContext`\[Alpha]D$$ = 
            5.; $CellContext`filename$$ = "e205.dat"; $CellContext`id$$ = 
            4412; $CellContext`nEq$$ = 50; $CellContext`nFile$$ = 
            "data"; $CellContext`layout$$ = 
            Aerodynamics`AirfoilGeometry`HalfCosineSpacing; \
$CellContext`model$$ = 
            Aerodynamics`PanelMethods`PMVortexLinearNeumann; \
$CellContext`reset$$ -> False; $CellContext`source$$ = 
            1; $CellContext`viz$$ = $CellContext`sp; Null),
          Evaluator->Automatic,
          Method->"Preemptive"], "", "", "", "", ""}
       },
       AutoDelete->False,
       GridBoxAlignment->{
        "Columns" -> {Left, Right, Left, Center, Right, Right, Left}, 
         "Rows" -> {{Center}}, 
         "ItemsIndexed" -> {{1, 1} -> {Center, Center}, {-2, 2} -> {
            Left, Center}, {3, 3} -> {Right, Center}, {3, 6} -> {
            Left, Center}}},
       GridBoxDividers->{"Columns" -> {{None}}, "Rows" -> {{None}}},
       GridBoxItemSize->{
        "Columns" -> {0.1, 10, 7, 4, 4, 5, 15}, "Rows" -> {{Automatic}}}],
      "Grid"],
     ImageSize->All],
    Module[{$CellContext`xMin$ = -0.5, $CellContext`xMax$ = 
      1.5, $CellContext`zMin$ = -0.5, $CellContext`zMax$ = 
      0.5, $CellContext`\[CapitalDelta]$ = 
      0.07, $CellContext`a$, $CellContext`aAll$, $CellContext`aW$, \
$CellContext`b$, $CellContext`bAll$, $CellContext`c$, $CellContext`cAll$, \
$CellContext`cl$, $CellContext`cm$, $CellContext`colorFunction$, \
$CellContext`cp$, $CellContext`cpCMin$, $CellContext`cpS$, \
$CellContext`gAirfoil$, $CellContext`gSurfaceCp$, $CellContext`gViz$, \
$CellContext`icA$, $CellContext`ic\[Psi]A$, $CellContext`icB$, \
$CellContext`ic\[Psi]B$, $CellContext`icC$, $CellContext`ic\[Psi]C$, \
$CellContext`nP$, $CellContext`rAll$, $CellContext`rC$, $CellContext`rF$, \
$CellContext`rP$, $CellContext`rPC$, $CellContext`rTE$, $CellContext`tuples$, \
$CellContext`V$, $CellContext`VMagC$, $CellContext`V\[Infinity]$, \
$CellContext`xC$, $CellContext`xCenP$, $CellContext`xW$, $CellContext`x0$, \
$CellContext`z0$, $CellContext`\[Alpha]$, $CellContext`\[Psi]$, $CellContext`\
\[Psi]\[Infinity]$}, {$CellContext`\[Alpha]$, $CellContext`rF$, \
$CellContext`V\[Infinity]$} = {$CellContext`\[Alpha]D$$ 
        Degree, {$CellContext`x, $CellContext`z}, 
        Aerodynamics`InfluenceCoefficients`UniformFlow[
        1, $CellContext`\[Alpha]$]}; $CellContext`rP$ = Which[
        And[$CellContext`source$$ == 1, $CellContext`nFile$$ == "data"], 
        Aerodynamics`AirfoilGeometry`AirfoilFromDataFile[
         StringJoin[
         "http://www.ae.uiuc.edu/m-selig/ads/coord/", \
$CellContext`filename$$]], $CellContext`source$$ == 1, 
        Aerodynamics`AirfoilGeometry`AirfoilSurfacePoints[
         Aerodynamics`AirfoilGeometry`FitAirfoil[
          Aerodynamics`AirfoilGeometry`AirfoilFromDataFile[
           StringJoin[
           "http://www.ae.uiuc.edu/m-selig/ads/coord/", \
$CellContext`filename$$]]], 
         Aerodynamics`AirfoilGeometry`NDiscretizeUnitSegment[$CellContext`\
nFile$$, Aerodynamics`AirfoilGeometry`Layout -> $CellContext`layout$$]], 
        And[$CellContext`source$$ == 2, $CellContext`nFile$$ == "data"], 
        Aerodynamics`AirfoilGeometry`AirfoilFromDataFile[
         StringJoin[
         "Aerodynamics\\Airfoil Data\\", $CellContext`filename$$]], \
$CellContext`source$$ == 2, 
        Aerodynamics`AirfoilGeometry`AirfoilSurfacePoints[
         Aerodynamics`AirfoilGeometry`FitAirfoil[
          Aerodynamics`AirfoilGeometry`AirfoilFromDataFile[
           StringJoin[
           "Aerodynamics\\Airfoil Data\\", $CellContext`filename$$]]], 
         Aerodynamics`AirfoilGeometry`NDiscretizeUnitSegment[$CellContext`\
nFile$$, Aerodynamics`AirfoilGeometry`Layout -> $CellContext`layout$$]], \
$CellContext`source$$ == 3, 
        Aerodynamics`AirfoilGeometry`AirfoilSurfacePoints[
         Aerodynamics`AirfoilGeometry`NACA4DigitAirfoil[$CellContext`id$$, 
          
          Aerodynamics`AirfoilGeometry`NDiscretizeUnitSegment[$CellContext`\
nEq$$, Aerodynamics`AirfoilGeometry`Layout -> $CellContext`layout$$]]]]; \
$CellContext`nP$ = 
      Aerodynamics`CartesianVectors`NumberOfVectors[$CellContext`rP$] - 
       1; {$CellContext`cl$, $CellContext`cm$} = 
      Which[$CellContext`model$$ === 
        Aerodynamics`PanelMethods`PMVortexLinearNeumann, {$CellContext`a$, \
$CellContext`b$, $CellContext`rC$, $CellContext`VMagC$} = \
$CellContext`model$$[$CellContext`rP$, $CellContext`\[Alpha]$]; 
        Aerodynamics`PanelMethods`ForceAndMoment[
         "Vortex", $CellContext`a$, $CellContext`b$, $CellContext`rP$, \
$CellContext`\[Alpha]$, $CellContext`rC$], $CellContext`model$$ === 
        Aerodynamics`PanelMethods`PMDoubletQuadraticDirichlet, \
$CellContext`rTE$ = Mean[{
            First[$CellContext`rP$], 
            Last[$CellContext`rP$]}]; $CellContext`rPC$ = 
         ReplacePart[$CellContext`rP$, {
           1 -> $CellContext`rTE$, -1 -> $CellContext`rTE$}]; \
{$CellContext`a$, $CellContext`b$, $CellContext`c$, $CellContext`aW$, \
$CellContext`rC$, $CellContext`VMagC$} = 
         Aerodynamics`PanelMethods`PMDoubletQuadraticDirichlet[$CellContext`\
rPC$, $CellContext`\[Alpha]$]; $CellContext`xW$ = 
         ReplaceAll[Aerodynamics`PanelMethods`XWake, 
           Options[
           Aerodynamics`PanelMethods`PMDoubletQuadraticDirichlet, 
            Aerodynamics`PanelMethods`XWake]]; $CellContext`rAll$ = 
         Append[$CellContext`rP$, {$CellContext`xW$, 
            
            0.}]; {$CellContext`aAll$, $CellContext`bAll$, \
$CellContext`cAll$} = {
           Append[$CellContext`a$, $CellContext`aW$], 
           Append[$CellContext`b$, 0.], 
           Append[$CellContext`c$, 0.]}; 
        Aerodynamics`PanelMethods`ForceAndMoment[
         "Doublet", $CellContext`a$, $CellContext`b$, $CellContext`c$, \
$CellContext`rPC$, $CellContext`\[Alpha]$, $CellContext`rC$], \
$CellContext`model$$ === 
        Aerodynamics`PanelMethods`PMDoubletConstantDirichlet, \
$CellContext`rTE$ = Mean[{
            First[$CellContext`rP$], 
            Last[$CellContext`rP$]}]; $CellContext`rPC$ = 
         ReplacePart[$CellContext`rP$, {
           1 -> $CellContext`rTE$, -1 -> $CellContext`rTE$}]; \
{$CellContext`a$, $CellContext`aW$, $CellContext`rC$, $CellContext`VMagC$} = 
         Aerodynamics`PanelMethods`PMDoubletConstantDirichlet[$CellContext`\
rPC$, $CellContext`\[Alpha]$]; $CellContext`xW$ = 
         ReplaceAll[Aerodynamics`PanelMethods`XWake, 
           Options[
           Aerodynamics`PanelMethods`PMDoubletQuadraticDirichlet, 
            Aerodynamics`PanelMethods`XWake]]; $CellContext`rAll$ = 
         Append[$CellContext`rP$, {$CellContext`xW$, 0.}]; $CellContext`aAll$ = 
         Append[$CellContext`a$, $CellContext`aW$]; 
        Aerodynamics`PanelMethods`ForceAndMoment[
         "Doublet", $CellContext`a$, $CellContext`rPC$, \
$CellContext`\[Alpha]$, $CellContext`rC$]]; $CellContext`xC$ = Part[
        Aerodynamics`CartesianVectors`Components[$CellContext`rC$], 
        1]; $CellContext`cpS$ = 
      1 - $CellContext`VMagC$^2; $CellContext`xCenP$ = \
(-$CellContext`cm$)/$CellContext`cl$; $CellContext`gSurfaceCp$ = 
      Aerodynamics`Graphics`PlotSurfacePressureCoefficient[$CellContext`xC$, \
$CellContext`cpS$, Epilog -> {
          Inset[
           Panel[
            Grid[{{
               Row[{"\[Alpha] = ", 
                 NumberForm[$CellContext`\[Alpha]$/Degree, {3, 2}], 
                 "\[Degree]"}]}, {
               
               Row[{"\!\(\*SubscriptBox[\n StyleBox[\"C\",\n\
FontSlant->\"Italic\"], \"\[ScriptL]\"]\) = ", 
                 NumberForm[$CellContext`cl$, {3, 2}]}]}, {
               
               Row[{"\!\(\*\nStyleBox[SubscriptBox[\"C\", \"m\"],\n\
FontSlant->\"Italic\"]\) = ", 
                 NumberForm[$CellContext`cm$, {3, 3}]}]}, {
               
               Row[{"\!\(\*SubscriptBox[\n StyleBox[\"x\",\n\
FontSlant->\"Italic\"], \"cp\"]\) = ", 
                 NumberForm[$CellContext`xCenP$, {3, 3}]}]}}]], {
           Right, Top}, {Right, Top}], 
          Inset[
           Aerodynamics`Graphics`PlotAirfoil[$CellContext`rP$, Epilog -> {
              EdgeForm[Black], Yellow, 
              Disk[{$CellContext`xCenP$, 0}, 0.012]}, Frame -> False, 
            PlotRange -> {{0, 1}, Automatic}], {0, 0}, {0, 0}, 1]}]; 
     If[$CellContext`viz$$ === $CellContext`slp, $CellContext`\[Psi]\
\[Infinity]$ = 
        Aerodynamics`InfluenceCoefficients`UniformFlowPsi[$CellContext`rF$, \
$CellContext`\[Alpha]$]; 
       Which[$CellContext`model$$ === 
         Aerodynamics`PanelMethods`PMVortexLinearNeumann, {$CellContext`ic\
\[Psi]A$, $CellContext`ic\[Psi]B$} = 
          Aerodynamics`InfluenceCoefficients`ICPsiVortexLinear[$CellContext`\
rF$, $CellContext`rP$]; $CellContext`\[Psi]$ = \
$CellContext`\[Psi]\[Infinity]$ + 
           Dot[$CellContext`ic\[Psi]A$, $CellContext`a$] + 
           Dot[$CellContext`ic\[Psi]B$, $CellContext`b$], \
$CellContext`model$$ === 
         Aerodynamics`PanelMethods`PMDoubletQuadraticDirichlet, \
{$CellContext`ic\[Psi]A$, $CellContext`ic\[Psi]B$, $CellContext`ic\[Psi]C$} = 
          Aerodynamics`InfluenceCoefficients`ICPsiDoubletQuadratic[$\
CellContext`rF$, $CellContext`rAll$]; $CellContext`\[Psi]$ = $CellContext`\
\[Psi]\[Infinity]$ + Dot[$CellContext`ic\[Psi]A$, $CellContext`aAll$] + 
           Dot[$CellContext`ic\[Psi]B$, $CellContext`bAll$] + 
           Dot[$CellContext`ic\[Psi]C$, $CellContext`cAll$], \
$CellContext`model$$ === 
         Aerodynamics`PanelMethods`PMDoubletConstantDirichlet, \
$CellContext`ic\[Psi]A$ = 
          Aerodynamics`InfluenceCoefficients`ICPsiDoubletLinear[$CellContext`\
rF$, $CellContext`rAll$]; $CellContext`\[Psi]$ = $CellContext`\[Psi]\
\[Infinity]$ + 
           Dot[$CellContext`ic\[Psi]A$, $CellContext`aAll$]]; \
$CellContext`gViz$ = 
        ContourPlot[$CellContext`\[Psi]$, {$CellContext`x, \
$CellContext`xMin$, $CellContext`xMax$}, {$CellContext`z, $CellContext`zMin$, \
$CellContext`zMax$}, AspectRatio -> Automatic, Contours -> 16, ContourShading -> 
          None, FrameLabel -> {"x", "z"}, MaxRecursion -> 3, PerformanceGoal -> 
          "Speed", RotateLabel -> False], $CellContext`V\[Infinity]$ = 
        Aerodynamics`InfluenceCoefficients`UniformFlow[
         1, $CellContext`\[Alpha]$]; $CellContext`z0$ = 
        Range[$CellContext`zMin$, $CellContext`zMax$, $CellContext`\
\[CapitalDelta]$]; $CellContext`x0$ = 
        Table[$CellContext`xMin$ + $CellContext`\[CapitalDelta]$, {
           Length[$CellContext`z0$]}]; $CellContext`tuples$ = 
        Transpose[{$CellContext`x0$, $CellContext`z0$}]; 
       Which[$CellContext`model$$ === 
         Aerodynamics`PanelMethods`PMVortexLinearNeumann, {$CellContext`icA$, \
$CellContext`icB$} = 
          Aerodynamics`InfluenceCoefficients`ICVortexLinear[$CellContext`rF$, \
$CellContext`rP$]; $CellContext`V$ = $CellContext`V\[Infinity]$ + 
           Dot[$CellContext`icA$, $CellContext`a$] + 
           Dot[$CellContext`icB$, $CellContext`b$], $CellContext`model$$ === 
         Aerodynamics`PanelMethods`PMDoubletQuadraticDirichlet, \
{$CellContext`icA$, $CellContext`icB$, $CellContext`icC$} = 
          Aerodynamics`InfluenceCoefficients`ICDoubletQuadratic[$CellContext`\
rF$, $CellContext`rAll$]; $CellContext`V$ = $CellContext`V\[Infinity]$ + 
           Dot[$CellContext`icA$, $CellContext`aAll$] + 
           Dot[$CellContext`icB$, $CellContext`bAll$] + 
           Dot[$CellContext`icC$, $CellContext`cAll$], $CellContext`model$$ === 
         Aerodynamics`PanelMethods`PMDoubletConstantDirichlet, \
$CellContext`icA$ = 
          Aerodynamics`InfluenceCoefficients`ICDoubletConstant[$CellContext`\
rF$, $CellContext`rAll$]; $CellContext`V$ = $CellContext`V\[Infinity]$ + 
           Dot[$CellContext`icA$, $CellContext`aAll$]]; 
       Which[$CellContext`viz$$ === $CellContext`sp, $CellContext`gViz$ = 
         StreamPlot[$CellContext`V$, {$CellContext`x, $CellContext`xMin$, \
$CellContext`xMax$}, {$CellContext`z, $CellContext`zMin$, $CellContext`zMax$},
            AspectRatio -> Automatic, FrameLabel -> {"x", "z"}, 
           StreamPoints -> {$CellContext`tuples$}, RotateLabel -> False, 
           StreamScale -> {
            Full, All, 
             0.015}], $CellContext`viz$$ === $CellContext`sdp, \
$CellContext`cpCMin$ = Min[$CellContext`cpS$]; $CellContext`colorFunction$[
            Pattern[$CellContext`c, 
             Blank[]], 
            Pattern[$CellContext`cMin, 
             Blank[]]] := Hue[
            If[$CellContext`c >= 0, 0.0483667, 0.633338], 
            
            If[$CellContext`c >= 
             0, $CellContext`c, $CellContext`c/$CellContext`cMin], 
            Which[
             
             Inequality[$CellContext`cMin, LessEqual, $CellContext`c, 
              Less, -1], 0.6472, 
             Inequality[-1, LessEqual, $CellContext`c, Less, 0], 1 + 
             0.3528 $CellContext`c, $CellContext`c >= 0, 
             1]]; $CellContext`cp$ = 
          1 - Dot[$CellContext`V$, $CellContext`V$]; $CellContext`gViz$ = 
          StreamDensityPlot[{$CellContext`V$, $CellContext`cp$}, \
{$CellContext`x, $CellContext`xMin$, $CellContext`xMax$}, {$CellContext`z, \
$CellContext`zMin$, $CellContext`zMax$}, AspectRatio -> Automatic, 
            ColorFunction -> ($CellContext`colorFunction$[#, \
$CellContext`cpCMin$]& ), ColorFunctionScaling -> False, 
            FrameLabel -> {"x", "z"}, MaxRecursion -> 6, RotateLabel -> False,
             StreamPoints -> {$CellContext`tuples$}, 
            StreamScale -> {Full, All, 0.015}]]; 
       Null]; $CellContext`gAirfoil$ = 
      Aerodynamics`Graphics`PlotAirfoil[$CellContext`rP$]; \
$CellContext`graphs = GraphicsColumn[{
         Show[$CellContext`gViz$, $CellContext`gAirfoil$], \
$CellContext`gSurfaceCp$}]; CreateDocument[{
        Which[$CellContext`source$$ == 1, 
         TextCell[
          StringJoin[
          "Potential Flow Over the Airfoil Described by the UIUC File ", \
$CellContext`filename$$], "Section"], $CellContext`source$$ == 2, 
         TextCell[
          StringJoin[
          "Potential Flow Over the Airfoil Described by the Data File ", \
$CellContext`filename$$], "Section"], $CellContext`source$$ == 3, 
         TextCell[
          StringJoin["Potential Flow Over a NACA-", 
           ToString[$CellContext`id$$], " Airfoil"], "Section"]], 
        TextCell[
         StringJoin["Model: ", 
          ToString[$CellContext`model$$]]], 
        If[
         And[
          
          Or[$CellContext`source$$ == 1, $CellContext`source$$ == 
           2], $CellContext`nFile$$ == "data"], 
         TextCell[
          StringJoin["Discretization: ", 
           ToString[$CellContext`nP$], " panels from data file."]], 
         TextCell[
          StringJoin["Discretization: ", 
           ToString[$CellContext`nP$], " panels in ", 
           ToString[$CellContext`layout$$], 
           " layout"]]], $CellContext`graphs}]; Null]],
   DynamicModuleValues:>{}],
  Setting[#, {0}]& ]], "Output",
 ImageSize->{567, 298},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->46976369]
}, Open  ]],

Cell[TextData[{
 "This worksheet is intended to be intuitive to use, but the following \
comments may be helpful.  The worksheet is evaluated by placing the blinking \
cursor anywhere in the worksheet or selecting the worksheet cell and \
evaluating (Shift+Enter).  If \"data\" is selected as the discretization \
input field for airfoils from data files, then the layout options are not \
active.  Airfoils defined by data files in the UIUC data base may be \
rediscretized by changing the default value of discretization from \"data\" \
to an integer and specifying a layout option.  For airfoils with thickness, \
the number of panels representing the airfoil is twice the discretization \
number.  If you select the stream-",
 Cell[BoxData[
  StyleBox[
   SubscriptBox["C", "p"],
   FontSlant->"Italic"]], "InlineFormula"],
 " plot for flow visualization, expect the evaluation to take at least an \
order of magnitude longer than for the other two selections.  At the bottom \
left of the worksheet is a reset button to return the worksheet to its \
default values."
}], "Text",
 CellID->626708176],

Cell[TextData[{
 "The purpose of the folder Airfoil Data in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " is to provide a place to store additional airfoil data files.  You may \
want to copy data files from the UIUC data base for use when you are not \
connected to the Internet, or you may have other sources for data files.  \
Also, data files from the UIUC site are acquired from different sources, and \
conventions may not be what is expected in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  It may be necessary to modify a data file from the UIUC site, and the \
folder Airfoil Data provides an place to store the modified file that is \
accessible to the worksheet. See the ",
 ButtonBox["Possible Issues",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/AirfoilFromDataFile#1305812373"],
 " section of the documentation page for the function AirfoilFromDataFile."
}], "Text",
 CellID->378315288],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["PanelMethods",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/PanelMethods"]], "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["3.6 Aerodynamic Coefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicCoefficients"]], "RelatedTutorials",
 CellID->334470709],

Cell[TextData[ButtonBox["References",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/References"]], "RelatedTutorials",
 CellID->383413863],

Cell[TextData[ButtonBox["Overview",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AerodynamicsOverview"]], "RelatedTutorials",
 CellID->280152894]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"3.7 Worksheet for Airfoil Aerodynamics - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 15, 10, 4, 54.4176000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "None", "summary" -> 
    "The following worksheet enables you to compute the aerodynamic \
characteristics for potential flow over an airfoil of interest to you. The \
airfoil may be one of the 1500+ airfoils specified by data files in the UIUC \
Airfoil Coordinates Database, a data file in the folder Airfoil Data \
associated with Aerodynamics or a member of the NACA family of four-digit \
airfoils. When you evaluate the worksheet for the first time in a session you \
are asked in a pop-up window if you want to evaluate initialization cells. \
Choosing yes loads the Aerodynamics packages needed for the computation. \
Output for this worksheet is a new notebook containing graphs for flow \
visualization and surface pressure distribution.", "synonyms" -> {}, "title" -> 
    "3.7 Worksheet for Airfoil Aerodynamics", "type" -> "Tutorial", "uri" -> 
    "Aerodynamics/tutorial/WorksheetForAirfoilAerodynamics"}, "LinkTrails" -> 
  "", "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[631, 21, 78, 1, 70, "TutorialColorBar"],
Cell[712, 24, 191, 5, 70, "LinkTrail"],
Cell[906, 31, 2971, 67, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3902, 102, 75, 1, 70, "Title",
 CellID->509267359],
Cell[3980, 105, 36, 1, 70, "Text",
 CellID->550749044],
Cell[4019, 108, 869, 18, 70, "Text",
 CellID->1534169418],
Cell[4891, 128, 163, 4, 70, "Text",
 CellID->8167176],
Cell[CellGroupData[{
Cell[5079, 136, 147, 5, 70, "Input",
 Evaluatable->True,
 InitializationCell->True,
 CellID->236023479],
Cell[5229, 143, 25948, 548, 70, "Output",
 CellID->46976369]
}, Open  ]],
Cell[31192, 694, 1103, 20, 70, "Text",
 CellID->626708176],
Cell[32298, 716, 938, 19, 70, "Text",
 CellID->378315288],
Cell[CellGroupData[{
Cell[33261, 739, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[33329, 742, 158, 3, 70, "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[CellGroupData[{
Cell[33524, 750, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[33599, 753, 189, 4, 70, "RelatedTutorials",
 CellID->334470709],
Cell[33791, 759, 155, 3, 70, "RelatedTutorials",
 CellID->383413863],
Cell[33949, 764, 166, 4, 70, "RelatedTutorials",
 CellID->280152894]
}, Open  ]]
}, Open  ]],
Cell[34142, 772, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

